package com.ibm.btools.blm.ie.exprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.List;


/**
 * The result of the model export: 
 * Total error count, warning count, list of file names and list of ExportMessage. 
 * relative to the export directory that have been produced in the export process.
 */
public class ExportResult {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private int errorCount = 0;
	private int warningCount = 0;
	private List outputFileNames = null;
	private boolean terminated = false;
	
	private List messages;

		/**
		 * Add message to the result
		 * @param message
		 */
		public void addMessage(final ExportMessage message) {
			getMessages().add(message);
		}
	
		/**
		 * Get all messages of the result
		 * @return the messages
		 */
		public List getMessages() {
			if (messages == null) {
				messages = new ArrayList();
			}
		
			return messages;
		}
	

	
	/**
	 * Gets total error count
	 * @return 
	 */
	public int getErrorCount() {
		
		return errorCount;
	}
	
	/**
	 * Gets total warning count
	 * @return
	 */
	public int getWarningCount() {
				
		return warningCount;
	}
	/**
	 * Sets total error count
	 * @param i
	 */
	public void setErrorCount(int i) {
		errorCount = i;
	}

	/**
	 * Gets total warning count
	 * @param i
	 */
	public void setWarningCount(int i) {
		warningCount = i;
	}
	
	/**
	 * Returns a list of file names that have been produced by export
	 * The name of file should be relative to the export directory.
	 * @return
	 */
	public List getOutputFileNames(){
		
		if(outputFileNames == null){
			outputFileNames = new ArrayList();		
		}
		return outputFileNames;
	}
	
	/**
	 * Sets the files names that have been produced by export 
	 * The name of file should be relative to the export directory.
	 * @param fileNames 
	 */
	public void setOutputFileNames(List fileNames){
		
		outputFileNames = fileNames;
	}
	
	
	/**
	 * @return Returns the terminated.
	 */
	public boolean isTerminated() {
		return terminated;
	}
	/**
	 * @param terminated The terminated to set.
	 */
	public void setTerminated(boolean terminated) {
		this.terminated = terminated;
	}
}
