package com.ibm.btools.blm.ie.exprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;

/**
 * Session for each Model Export.
 */
public class ExportSession {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ExportResult result;
	private IExportQuery query;
	private IProgressMonitor monitor;
	private ExportOptions options;
	//use additional map to kep input and output for the session
	private Map context;
	
	//AZ add in the session for the XML export 
	private NavigationProjectNode projectNode;
	
	/**
	 * Sets the project node for the export session
	 * @param theNode
	 */
	public void setProjectNode(NavigationProjectNode theNode){
		this.projectNode = theNode;
	}
	
	/**
	 * Gets the project node for the export session
	 * @return NavigationProjectNode the project node
	 */
	public NavigationProjectNode getProjectNode(){
		return this.projectNode;
	}
	
	/**
	 * Sets the progress monitor for exporting progress
	 * @param monitor
	 */
	public void setProgressMonitor(final IProgressMonitor monitor) {
		this.monitor = monitor;
	}
	
	/**
	 * Sets the export query
	 * @param query
	 */
	public void setExportQuery(final IExportQuery query) {		
		this.query = query;
	}

	/**
	 * Sets export options
	 * @param options
	 */
	public void setExportOption(final ExportOptions options) {
		this.options = options;
	}
	
	
	
	/**
	 * Sets export result
	 * @param result
	 */
	public void setExportResult(final ExportResult result) {
		this.result = result;
	}		

	/**
	 * Sets export options
	 * @return options
	 */
	public ExportOptions getExportOptions() {
		return options;
	}

	/**
	 * Gets progress monitor
	 * @return monitor
	 */
	public IProgressMonitor getProgressMonitor() {
		return monitor;
	}

	/**
	 * Gets overall export callback
	 * @return query
	 */
	public IExportQuery getOverwriteQuery() {
		return query;
	}

	
	/**
	 * Gets export result
	 * @return result
	 */
	public ExportResult getExportResult() {
		if (result == null) {
			result = new ExportResult();
		}
		return result;
	}
	
	public Map getContext(){
		if(context == null){
			context = new HashMap();
		}
		return context;
	}
}
