package com.ibm.btools.blm.ie.exprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.List;


/**
 * The export operation interface. Each model export operation should implement
 * this interface. An abstract class <code>AbstractExportOperation</code>
 * provided. The model export operation can extend this 
 * abstract class and provide the implementation of method <code>export
 * </code>
 */
public interface IExportOperation {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Sets the name of the operation
	 * @param name
	 */
	public void setName(final String name);
	
	/**
	 * Gets the name of the operation
	 * @return
	 */
	public String getName();
	
	/**
	 * Sets the output directory for export
	 * @param outputDir 
	 */
	public void setOutputDir(final String outputDir);

	/**
	 * Gets the output directory
	 */
	public String getOutputDir();
	
	
	/**
	 * Sets the selected model elements for export
	 * @param selectedModelElements list of BOM top level model elements
	 */
	public void setModelElements(final List selectedModelElements);

	/**
	 * Gets the selected model elements for export
	 * @return List - list of BOM top level model elements	
	 */
	public List getModelElements();

	/**
	 * Sets the project name
	 * @param projectName
	 */
	public void setProjectName(final String projectName);
	
	/**
	 * Gets the project name
	 * @param projectName
	 */
	public String getProjectName();	
	
	
	/**
	 * Sets the export session
	 * @param session
	 */
	public void setExportSession(final ExportSession session);	

	/**
	 * Gets the export session
	 * @return ExportSession
	 */
	public ExportSession getExportSession();	
	

	/**
	 * Executes the export operation
	 */
	public abstract void export();

	

}
