package com.ibm.btools.blm.ie.exprt.option;

import java.util.List;
import java.util.Map;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 * BmMMExportOptionsBean to store the export options for MM generation.
 * @author Grace Lo
 */
public class BmMMExportOptionsBean {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	// select PI format?
	private boolean isMMProjectInterchangeEnabled = true;
	// Monitor PI name
	private String monProjectInterchangeName;
	// append timestamp?
	private boolean isAppendTimestampToMMPIName = true;
	// Modeler project names
	private String[] mdlProjectNames;
	// Monitor project names
	private String[] monProjectNames;
	// overwrite files?
	private boolean overwriteAll = false;
	// For WID MM export: which export option
	private int mmExportOption = -1;   // default
	// for standalone MM export only, the list of model elements
	private List[] exportElements;
	
	//deployment information to sandbox
	private Map<String, Object> deploymentInformatoin;
	//private Map<String, Object> generatedInformation;
	
	public static String MONITOR_MODEL_NAME_HL = "monitorModelNameHL";
	public static String MONITOR_OUTPUT_DIR = "monitorOutputDir";
	// to do: move those constants to export Bean
	public static final String DEPLOYMENT_ID_HL = "deploymentIdHL";
	public static final String DEPLOYMENT_ID_LL = "deploymentIdLL";
	public static final String DEPLOYMENT_ID_MM = "deploymentIdMM";
	public static final String CONTAINER_MCS = "containerMcs";
	//private static final String MONITOR_OUTPUT_DIR = "monitorOutputDir";
	public static final int DTD_OPTION = 1;

	public static final String MODEL_HL_VERSION = "modelHLVersion";
	public static final String MODEL_LL_VERSION = "modelLLVersion";
	public static final String MODEL_MM_VERSION = "modelMMVersion";
 
	/**
	 * Constructor for standalone MM export (export .mm) 
	 * @param exportEles
	 * @param mdlProjectNames
	 * @param monProjectInterchangeName
	 * @param monProjectNames
	 * @param isMMProjectInterchangeEnabled
	 * @param isAppendTimestampToMMPIName
	 */
	public BmMMExportOptionsBean(List[] exportEles,  String[] mdlProjectNames, String monProjectInterchangeName, String[] monProjectNames,
			boolean isMMProjectInterchangeEnabled, boolean isAppendTimestampToMMPIName, boolean overwriteAll) {
		this.exportElements = exportEles;
		this.mdlProjectNames = mdlProjectNames;
		this.monProjectInterchangeName = monProjectInterchangeName;
		this.monProjectNames = monProjectNames;
		this.isMMProjectInterchangeEnabled = isMMProjectInterchangeEnabled;
		this.isAppendTimestampToMMPIName = isAppendTimestampToMMPIName;
		this.overwriteAll = overwriteAll;
	}
	
	
	/**
	 * Constructor for WID MM export (export WID together with .mm)
	 * @param mmExportOption
	 * @param mdlProjectNames
	 * @param monProjectInterchangeName
	 * @param monProjectNames
	 * @param isMMProjectInterchangeEnabled
	 * @param isAppendTimestampToMMPIName
	 * @param overwriteAll
	 */
	public BmMMExportOptionsBean(int mmExportOption, String[] mdlProjectNames, String monProjectInterchangeName, String[] monProjectNames,
			boolean isMMProjectInterchangeEnabled, boolean isAppendTimestampToMMPIName, boolean overwriteAll) {
		this.mmExportOption = mmExportOption;
		this.mdlProjectNames = mdlProjectNames;
		this.monProjectInterchangeName = monProjectInterchangeName;
		this.monProjectNames = monProjectNames;
		this.isMMProjectInterchangeEnabled = isMMProjectInterchangeEnabled;
		this.isAppendTimestampToMMPIName = isAppendTimestampToMMPIName;
		this.overwriteAll = overwriteAll;
	}

	/**
	 * MM Export Option
	 * @return
	 */
	public int getMmExportOption() {
		return mmExportOption;
	}

	public void setMmExportOption(int mmExportOption) {
		this.mmExportOption = mmExportOption;
	}
	
	
	/**
	 * Mon Project Interchange name
	 * @return
	 */
	public String getMonProjectInterchangeName() {
		return monProjectInterchangeName;
	}

	public void setMonProjectInterchangeName(String monProjectInterchangeName) {
		this.monProjectInterchangeName = monProjectInterchangeName;
	}

	
	/**
	 * Mon project names
	 * @return
	 */
	public String[] getMonProjectNames() {
		return monProjectNames;
	}

	public void setMonProjectNames(String[] monProjectNames) {
		this.monProjectNames = monProjectNames;
	}

	
	/**
	 * MDL project names
	 * @return
	 */
	public String[] getMdlProjectNames() {
		return mdlProjectNames;
	}

	public void setMdlProjectName(String[] mdlProjectNames) {
		this.mdlProjectNames = mdlProjectNames;
	}
	
	
	/**
	 * Is MM PI format enabled?
	 * @return
	 */
	public boolean isMMProjectInterchangeEnabled() {
		return this.isMMProjectInterchangeEnabled;
	}
	
	public void setMMProjectInterchangeEnabled(boolean isMMProjectInterchangeEnabled) {
		this.isMMProjectInterchangeEnabled = isMMProjectInterchangeEnabled;
	}
		
	
	/**
	 * Is Append time stamp to MM PI name?
	 * @return
	 */
	public boolean isAppendTimestampToMMPIName() {
		return this.isAppendTimestampToMMPIName;
	}
	
	public void setAppendTimestampToMMPIName(boolean isAppendTimestampToMMPIName) {
		this.isAppendTimestampToMMPIName = isAppendTimestampToMMPIName;
	}

	
	/**
	 * Is ovewrite file?
	 * @return
	 */
	public boolean isOverwriteAll() {
		return overwriteAll;
	}

	public void setOverwriteAll(boolean overwriteAll) {
		this.overwriteAll = overwriteAll;
	}

	
	/**
	 * MM Export elements
	 * @return List[]
	 */
	public List[] getExportElements() {
		return exportElements;
	}

	public void setExportElementsn(List[] exportElements) {
		this.exportElements = exportElements;
	}
	
	/**
	 * Gets deployment Information
	 * @return
	 */
	public Map<String, Object> getDeploymentInformatoin() {
		return deploymentInformatoin;
	}
  
	/**
	 * Sets deployment information
	 * @param deploymentInformatoin
	 */
	public void setDeploymentInformatoin(Map<String, Object> deploymentInformatoin) {
		this.deploymentInformatoin = deploymentInformatoin;
	}
}
