package com.ibm.btools.blm.ie.imprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;

/**
 * Provides an abstract implementation of the import operation. The 
 * extended class should provide the implementation of the <code>
 * readObjects</code> method.
 */
public abstract class AbstractImportOperation implements IImportOperation {

	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private List selectedFiles;
	private String projectName;
	private ImportSession session;	
	protected EList importObjects;
	protected EList predefinedTypes;
	private String name; 
	// Default is false			
	private boolean isRootModels = false;
	
	/**
	 * Set the name of the operation
	 * @param name
	 */
	public void setName(final String name) {
		this.name = name;
	}
	
	/**
	 * Get the name of the operation
	 * @return
	 */
	public String getName() {
		return name;	
	}
	
	/**
	 * Set the selected files for import. It can be a file or it can be
	 * any objects. Recommendation: java.io.File objects
	 * @param selectedFiles
	 */
	public void setInputFiles(final List selectedFiles) {
		this.selectedFiles = selectedFiles;
	}

	/**
	 * Get the selected files for import
	 * @param selectedFiles
	 */
	public List getInputFiles() {
		return selectedFiles;
	}
	
	/**
	 * Sets the project name
	 * @param projectName
	 */
	public void setProjectName(final String projectName) {
		this.projectName = projectName;
	}	

	/**
	 * Gets the project name
	 * @param projectName
	 */
	public String getProjectName() {
		return projectName;	
	}
	
	/**
	 * Sets the import session
	 * @param session
	 */
	public void setImportSession(final ImportSession session) {
		this.session = session;	
	}
	
	/**
	 * Gets the import session
	 * @param session
	 */
	public ImportSession getImportSession() {
		return session;	
	}
	
	/**
	 * Return the list of imported objects created by this operation
	 * @return
	 */
	public EList getObjects() {
		if (importObjects == null) {
			importObjects = new BasicEList();
		}
		return importObjects;
	}
	
	/**
	 * Return the predefined types
	 * @return
	 */
	public EList getPredefinedTypes() {
		if (predefinedTypes == null) {
			predefinedTypes = new BasicEList();
		}
		return predefinedTypes;
	}
	
	/**
	 * Clean up the operation
	 */
	public void finish() {
		importObjects = null;
	}
	
	/**
	 * Set true if the objects are root models.
	 * @param flag
	 */
	public void setIsRootModels(final boolean flag) {
		this.isRootModels = flag;
	}
	
	/**
	 * Return true if the objects are root models
	 * @return
	 */
	public boolean isRootModels() {
		return isRootModels;	
	}
	
	/**
	 * Return true if the operation can be executed.
	 * @return
	 */
	public boolean canExecute() {
		// Create an Empty Import Result
		getImportSession().getImportResult();
		return true;
	}
}