package com.ibm.btools.blm.ie.imprt;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 *	Implementors of this interface answer one of the prescribed return codes
 *	when asked whether to overwrite a certain path string (which could
 *	represent a resource path, a file system path, etc).
 */
public interface IImportQuery {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		
	/**
	 * Return code indicating the operation should be canceled.
	 */
	public static final int CANCEL = 0;

	/**
	 * Return code indicating the entity should not be overwritten, 
	 * but operation should not be canceled.
	 */
	public static final int IGNORE = 1;

	/**
	 * Return code indicating the entity should be overwritten.
	 */
	public static final int OVERWRITE = 2;

	/**
	 * Return code indicating the entity should be a new copy.
	 */
	public static final int NEW_COPY = 3;

	/**
	 * Return code indicating the entity and the rest of the entities 
	 * should not be overwritten, but operation should not be canceled.
	 */
	public static final int ALWAYS_IGNORE = 4;

	/**
	 * Return code indicating the entity and the rest of the entities 
	 * should be overwritten.
	 */
	public static final int ALWAYS_OVERWRITE = 5;

	/**
	 * Return code indicating the entity and the rest of the entities 
	 * should be a new copy.
	 */
	public static final int ALWAYS_NEW_COPY = 6;

	/**
	 * Returns one of the return code constants declared on this interface,
	 * indicating whether the entity represented by the passed String should be overwritten.
	 * <p>
	 * This method may be called from a non-UI thread, in which case this method must run the query
	 * in a sync exec in the UI thread, if it needs to query the user.
	 * </p>
	 * @param pathString the path representing the entity to be overwritten
	 * @return one of the return code constants declared on this interface
	 */
	public int queryImportOption(final Object displayName);
}