package com.ibm.btools.blm.ie.imprt.engine;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.cef.gef.emf.command.AddRootObjectForUpdateCefCommand;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.CloseWorkingSetCmd;
import com.ibm.btools.model.modelmanager.SaveWorkingSetCmd;

public abstract class AbstractBOMProcessor {
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	protected final boolean isMaster = false;

	protected ImportSession session;
	protected String projectName;
	protected  Map copyIDMap = new HashMap();
	/**
	 * Return the name of the project
	 * @return
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * Set the project name
	 * @param string
	 */
	public void setProjectName(final String projectName) {
		this.projectName = projectName;
	}

	/**
	 * Return the Import session
	 * @return
	 */
	public ImportSession getImportSession() {
		return session;
	}

	/**
	 * Set the Import Session
	 * @param session
	 */
	public void setImportSession(final ImportSession session) {
		this.session = session;
	}

	/**
	 * Open master copy
	 * @param viewURI
	 * @param uri
	 * @return
	 */	
	protected EObject openMasterCopy(final String viewURI, final String uri) {
		if (viewURI != null) {
			return (EObject) BOMUtil.loadRootObject(viewURI, getProjectName());
		} else if (uri != null) {
			return (EObject) BOMUtil.loadRootObject(uri, getProjectName());
		}	
		return null;	
	}
	
	/**
	 * Save master copy
	 * @param viewURI
	 * @param uri
	 */
	protected void saveMasterCopy(final String viewURI, final String uri) {
		if (viewURI != null) {
			BOMUtil.saveRootObject(viewURI, projectName);
		}
			
		if (uri != null) {
			BOMUtil.saveRootObject(uri, projectName);
		}	
	}	
	
	/**
	 * Save working copy
	 * @param viewURI
	 * @param uri
	 * @param copyID
	 */
	protected void saveWorkingCopy(final String viewURI, final String uri, String copyID) {
		String projectPath = FileMGR.getProjectPath(projectName);
		
		SaveWorkingSetCmd saveCmd = new SaveWorkingSetCmd();
		saveCmd.setProjectName(projectName);
		saveCmd.setProjectPath(projectPath);
		saveCmd.setWorkingSetID(copyID);
		if(saveCmd.canExecute()) {
			try {				
				saveCmd.execute();				
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "saveWorkingCopy", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
				LoggingUtil.traceExit(this, "saveWorkingCopy");
				return;					
			}
		} else {
			LoggingUtil.trace(this, "saveWorkingCopy", IImportResultMessages.CANNOT_EXECUTE_SAVE_COMMAND);								
			LoggingUtil.traceExit(this, "saveWorkingCopy");
			return;
		}
	}		
	
	/**
	 * Open working Copy
	 * @param viewURI
	 * @param uri
	 * @return
	 */
	protected Object[] openWorkingCopy(final String viewURI, final String uri) {
		Object[] object = (Object[]) copyIDMap.get(uri);
		if (object != null) {
			return object;
		}
		
		EObject workingCopy = null;
		OpenRootObjectForUpdateBOMCmd openCmd = new OpenRootObjectForUpdateBOMCmd();
		openCmd.setProjectName(projectName);
		openCmd.setROBLM_URI(uri);
		String copyID = null;
		if (openCmd.canExecute()) {
			try{
				openCmd.execute();
				copyID = openCmd.getCopyID();
				workingCopy = openCmd.getROCopy();
			} catch (RuntimeException e) {
				LoggingUtil.traceExit(this, "openWorkingCopy", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);	
				LoggingUtil.traceExit(this, "openWorkingCopy");						
				return null;									
			}
		} else {
			LoggingUtil.traceExit(this, "openWorkingCopy", IImportResultMessages.CANNOT_EXECUTE_OPEN_COMMAND);				
			LoggingUtil.traceExit(this, "openWorkingCopy");
			return null;
		}
		if (viewURI != null) {
			LoggingUtil.trace(this, "processElement", "This element is a view model.");
			AddRootObjectForUpdateCefCommand addCefCmd = new AddRootObjectForUpdateCefCommand();
			addCefCmd.setProjectName(projectName);
			addCefCmd.setRO_URI(viewURI);
			addCefCmd.setCopyID(copyID);
			if(addCefCmd.canExecute()) {
				try {					
					addCefCmd.execute();
					workingCopy = addCefCmd.getROCopy();
				} catch (RuntimeException e) {
					LoggingUtil.traceExit(this, "openWorkingCopy", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
					LoggingUtil.traceExit(this, "openWorkingCopy");
					return null;
				}
			} else {
				LoggingUtil.traceExit(this, "openWorkingCopy", IImportResultMessages.CANNOT_EXECUTE_ADD_CEF_COMMAND);				
				LoggingUtil.traceExit(this, "openWorkingCopy");
				return null;
			}				
		}	

		Object[] result =  new Object[] {workingCopy, copyID};
		copyIDMap.put(uri, result);

		return result;		
	}
	
	/**
	 * Close the workingcopy by id
	 * @param copyID
	 */
	protected void closeWorkingCopy(final String copyID) {
		LoggingUtil.traceEntry(this, "closeWorkingCopy");		
		if (copyID == null) {
			LoggingUtil.trace(this, "closeWorkingCopy", "The copyID should not be null.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");			
			return;
		}
		CloseWorkingSetCmd closeCmd = new CloseWorkingSetCmd();
		closeCmd.setWorkingSetID(copyID);
		if(closeCmd.canExecute()) {
			try {			
				closeCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "closeWorkingCopy", "The close command throws a RuntimeException.");
				LoggingUtil.traceExit(this, "closeWorkingCopy");
				return;
			}
		} else {
			LoggingUtil.trace(this, "closeWorkingCopy", "The close command cannot be executed.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");
			return;
		}
		LoggingUtil.traceExit(this, "closeWorkingCopy");		
	}

}
