package com.ibm.btools.blm.ie.imprt.rule.externalModel.bo;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.net.URL;
import java.util.HashMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ComplexTypeFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.externalmodels.ExternalDocument;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
* Import the business item from a information model
*/
public class ComplexTypeRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		
	private AbstractChildContainerNode parentNode;
	//Class with "SchemaType aspect"
	private Class newComplexType;
	//This node represents NavigationComplexTypeNode/NavigationInlineComplexTypeNode
	private AbstractChildLeafNode node;
		
	/**
	 * The parent can either be a NavigationOperationNode or a NavigationInlineXSDSchemaNode
	 * @param parentNode
	 */
	public void setParent(final AbstractChildContainerNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the process model for import
	 * @param newProcessModel
	 */
	public void setSource(final Object newComplexType) {
		this.newComplexType = (Class) newComplexType;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */	
	public AbstractChildLeafNode getNewCompexTypeNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}	
		
		if (newComplexType == null) {
			LoggingUtil.trace(this, "invoke", "The source should null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}		
		
		String complexTypeName = newComplexType.getName();
		if (complexTypeName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newComplexType);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newComplexType);
		}
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");
				
		try {			
			ComplexTypeFactory factory = new ComplexTypeFactory(); 
			node = factory.create(
				parentNode, 
				newComplexType, 
				description, 
				getImportSession(), 
				resolve);			
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {	
				showMessageBox(e,complexTypeName);
		}
		if(node == null){
			return;
		}
		if(BOMUtil.isFileAttachment(newComplexType)){			
			//set the aspect to fileAttachment
			Class thisObject = (Class)BOMUtil.loadRootObject(node);
//			if(thisObject == null){
//				return;
//			}
			thisObject.setAspect(BOMUtil.ASPECT_FILE_ATTACHMENT);
			//registerFileAttachment(thisObject);
			String sourceFile = null;			
			ExternalDocument newExternalSchemaModel = BOMUtil.getExternalDocument(newComplexType);
			
			String modelFileURL = newExternalSchemaModel.getLocationURL();			
			
			HashMap networkMap = (HashMap)getImportSession().getContext().get("netWorkFileTemp");			
			HashMap urlMap = (HashMap)getImportSession().getContext().get(IImportFramework.FILE_URLS_KEY);
			HashMap xmlUrlMap = (HashMap)getImportSession().getContext().get("xmlTempFiles");
			if(networkMap != null && networkMap.containsKey(modelFileURL)){
				sourceFile = (String)networkMap.get(modelFileURL);
			}
			else if(urlMap != null && urlMap.containsKey(modelFileURL) && 
					!(networkMap != null && networkMap.containsKey(modelFileURL))){
				sourceFile = (String)urlMap.get(modelFileURL);				
			}else if(xmlUrlMap != null && xmlUrlMap.containsKey(modelFileURL)){
				sourceFile = (String)xmlUrlMap.get(modelFileURL);
			}
			newExternalSchemaModel.setLocationURL(sourceFile);			
			AbstractChildContainerNode selectedBOcatalogNode = NavigatorUtil.getSelectedBOCatalog(getImportSession().getImportOptions(),getProjectName());
			if(sourceFile != null && selectedBOcatalogNode != null ){
			boolean success = BOMUtil.attachFile2BomActivity(getImportSession(), getProjectName(), node,	newExternalSchemaModel,	selectedBOcatalogNode);
				if(success){
					if(xmlUrlMap != null && xmlUrlMap.containsKey(modelFileURL)||
							networkMap != null && networkMap.containsKey(modelFileURL)){
						File tempFile = new File(sourceFile);
						if(tempFile.exists()){
							boolean deleted = tempFile.delete();
							if (!deleted) {
								// Deletion failed
								LoggingUtil.logWarning(IeMessageKeys.TEMP_FILE_DELETE_ERROR, new String[]{tempFile.toString()}, null, null);
							}
							
						}
					}
				}
			}
		}		
		else if( BOMUtil.isInlineComplexType(newComplexType)){
			Class thisObject = (Class)BOMUtil.loadRootObject(node);
			thisObject.setAspect(BOMUtil.ASPECT_INLINE_TYPE);
		}else{
			Class thisObject = (Class)BOMUtil.loadRootObject(node);
			thisObject.setAspect(newComplexType.getAspect());
		}
		
		if (node != null) {
			getImportSession().getContext().put(newComplexType, node);
		}
		
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}	
	
}
