package com.ibm.btools.blm.ie.imprt.rule.externalModel.service;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.OperationFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationOperationNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.bom.model.processes.activities.Activity;

/**
* The WSDL operation is mapped to BOM Activity, the name of the Activity is the name of the operation
*/
public class OperationRule extends AbstractImportRule {
//	 copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$		
	
	private NavigationWSDLPortTypeNode parentNode;
	//Activity with "ServiceOperation" as aspect
	private Activity newOperation;
	private NavigationOperationNode node;
		
	/**
	 * Set the parent node. It must be a type of NavigationWSDLPortTypeNode
	 * @param parentNode
	 */
	public void setParent(final NavigationWSDLPortTypeNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the Activity + SAN for import
	 * @param new Service
	 */
	public void setSource(final Object newOperation) {
		this.newOperation = (Activity)newOperation;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public NavigationOperationNode getNewOperationNode() {		
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newOperation == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		//Make aspect assertion
		if(!BOMUtil.isServiceOperation(newOperation.getImplementation())){
			LoggingUtil.trace(this, "invoke", "The source should have an aspect of 'ServiceOperation'.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String operationName = newOperation.getName();
		if (operationName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newOperation);
		
		try {				
			OperationFactory factory = new OperationFactory(); 
			node = factory.create(
				parentNode, 
				newOperation, 
				description, 
				getImportSession(), 
				false, 
				false);				
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {			
			showMessageBox(e,operationName);			
		}
		if (node != null) {
			getImportSession().getContext().put(newOperation, node);	
		}
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}

}
