package com.ibm.btools.blm.ie.imprt.rule.externalModel.service;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.navigator.CatalogRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.ServiceInterfaceFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.bom.model.externalmodels.ServiceInterface;


/**
*Service interface is mapped to the WSDL portType 
*/
public class ServiceInterfaceRule extends CatalogRule{
	//	copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$		
	
	private ServiceInterface newServiceInterfaceModel;	
	private NavigationWSDLPortTypeNode node;
		
	/**
	 * Set the process model for import
	 * @param newProcessModel
	 */
	public void setSource(final Object newServiceInterfaceModel) {
		this.newServiceInterfaceModel = (ServiceInterface)newServiceInterfaceModel;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */	
	public NavigationWSDLPortTypeNode getNewServiceInterfaceNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newServiceInterfaceModel == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String serviceInterfaceName = newServiceInterfaceModel.getName();
		if (serviceInterfaceName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newServiceInterfaceModel);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newServiceInterfaceModel);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			ServiceInterfaceFactory factory = new ServiceInterfaceFactory();
			node = factory.create(
				parentNode, serviceInterfaceName, description, getImportSession(), isDefault, resolve);			
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {				
			showMessageBox(e,serviceInterfaceName);			
		}
		if (node != null) {
			getImportSession().getContext().put(newServiceInterfaceModel, node);						
		}
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}		
		LoggingUtil.traceExit(this, "invoke");		
	}
}
