package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryNode;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateClassBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateClassifierBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdatePackageBOMCmd;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.model.resourcemanager.util.PredefUtil;

/**
 * Updates a BOM Classifier.
 */
public class UpdateClassifierRule extends AbstractImportRule {
	
	//	copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected Classifier classifier;
	protected Classifier workingCopy;
	
	/** 
	 * Invokes the rule execution.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");	
		updateClassifierProperties();
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Set Original BOM Classifier
	 * @param classifier
	 */
	public void setSource(final Object classifier) {
		this.classifier = (Classifier) classifier;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Classifier) workingCopy;
	}
	
	/**
	 * Sets the superClassifier of the BOM Classifier, if any
	 */
	private void updateClassifierProperties() {
		LoggingUtil.traceEntry(this, "updateClassifierProperties");	
		//Classifier  workingSuperClass = null;
		EList superClassifierList = classifier.getSuperClassifier();
		if(superClassifierList != null && !superClassifierList.isEmpty()){
			if(superClassifierList.size() >1){
				LoggingUtil.trace(this, "updateClassifierProperties", "Multiple superclassifier not supported.");
			}
			Type superClassifier = (Type)superClassifierList.get(0);				
			Classifier  superClass = (Classifier)superClassifier;
			
			EList workingCopySuperClassifier = workingCopy.getSuperClassifier();
			if(workingCopySuperClassifier != null && !workingCopySuperClassifier.isEmpty()){
				if(workingCopySuperClassifier.size() >1){
					LoggingUtil.trace(this, "updateClassifierProperties", "Multiple superclassifier not supported.");
				}					
				Type workingSuperClassifier = (Type)workingCopySuperClassifier.get(0);						
				Classifier  workingSuperClass = (Classifier)workingSuperClassifier;					
				if(workingSuperClass != null){
					UpdateClassifierBOMCmd removeCmd = new UpdateClassifierBOMCmd(workingCopy);
					removeCmd.removeSuperClassifier(workingSuperClass);
					if(removeCmd.canExecute()){
						try{
							removeCmd.execute();
						}catch(RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CLASSIFIER_SUPER_CLASSIFIER_EXCEPTION, new String[]{workingCopy.getName(), superClassifier.getName()}, ex);
						}
					}
				}
			}
			
			if(superClass != null && workingCopy.getSuperClassifier().size() == 0){
				UpdateClassifierBOMCmd updateCmd = new UpdateClassifierBOMCmd(workingCopy);
				//AZ d141525 update template in the context
				Classifier target = getTemplateFromContext(superClass);
				updateSuperClass(superClass,target);
				updateCmd.addSuperClassifier(target);
				if (updateCmd.canExecute()) {
					try{
						updateCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CLASSIFIER_SUPER_CLASSIFIER_EXCEPTION, new String[]{classifier.getName(), superClassifier.getName()}, ex);
					}
				} else {
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CLASSIFIER_SUPER_CLASSIFIER_EXCEPTION, new String[]{classifier.getName(), superClassifier.getName()}, null);
				}
			}else{
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Classifier", superClassifier.getName()}, null);
			}		
		}else{
			//The incoming business item doesn't contain super classifier,
			//remove the classifier for the existing business item
			EList workingCopySuperClassifier = workingCopy.getSuperClassifier();
			if(workingCopySuperClassifier != null && !workingCopySuperClassifier.isEmpty()){
				if(workingCopySuperClassifier.size() >1){
					LoggingUtil.trace(this, "updateClassifierProperties", "Multiple superclassifier not supported.");
				}					
				Type workingSuperClassifier = (Type)workingCopySuperClassifier.get(0);						
				Classifier  workingSuperClass = (Classifier)workingSuperClassifier;					
				if(workingSuperClass != null){
					UpdateClassifierBOMCmd removeCmd = new UpdateClassifierBOMCmd(workingCopy);
					removeCmd.removeSuperClassifier(workingSuperClass);
					if(removeCmd.canExecute()){
						try{
							removeCmd.execute();
						}catch(RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CLASSIFIER_SUPER_CLASSIFIER_EXCEPTION, new String[]{workingCopy.getName(), ""}, ex);
						}
					}
				}
			}
		}
		updateComments();
		LoggingUtil.traceExit(this, "updateClassifierProperties");
	}	
	
	/**
	 * Update BOM Properties comments.
	 *
	 */	
	private void updateComments(){
		LoggingUtil.traceEntry(this, "updateComments");
		EList comments = classifier.getOwnedComment();
		if(comments != null && !comments.isEmpty()){	
		//JR26020, add and update Comments for the Classifier.		
			for(int i=0;i< comments.size();i++){
				Object eachCommentObj = comments.get(i);
				if(eachCommentObj instanceof Comment){							
					Comment eachComment = (Comment)eachCommentObj;
					if(eachComment.getBody()== null || eachComment.getBody().length() == 0){
						continue;
					}
					if(workingCopy.getOwnedComment() == null || 
							workingCopy.getOwnedComment().size() < 1 ||
							!(workingCopy.getOwnedComment().get(0) instanceof Comment)){					
						return;
					}
					//we are meant to update the first element as type of comment				
					if(workingCopy.getOwnedComment()!= null && workingCopy.getOwnedComment().size() >=i+1 && workingCopy.getOwnedComment().get(i) instanceof Comment){
						AddUpdateCommentBOMCmd updateCmd = new UpdateCommentBOMCmd((Comment)workingCopy.getOwnedComment().get(i));
						updateCmd.setBody(eachComment.getBody());
						if(updateCmd.canExecute()){
							try{
								updateCmd.execute();
							}catch(RuntimeException ex){
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)classifier).getName()}, ex);
							}
						}else{
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)classifier).getName()}, null);
						}
					//If there no exitsing comment on the workingcopy, we add the comment to it
					}else{
						AddCommentToElementBOMCmd  addCmd = new AddCommentToElementBOMCmd(workingCopy);
						addCmd.setBody(eachComment.getBody());
						if(addCmd.canExecute()){
							try{
								addCmd.execute();
							}catch(RuntimeException ex){
								LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)classifier).getName()}, ex);
							}
						}else{
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)classifier).getName()}, null);
						}				
					}					
				}
			}
		}
		LoggingUtil.traceExit(this, "updateComments");
	}
	
	private Classifier getTemplateFromContext(Classifier newTemplate){
		Object registry = getImportSession().getContext().get(IImportConstants.TEMPLATE_REGISTRY_ENTRY);
		if(registry instanceof List){
			List templates = (List)registry;
			for(int i = 0; i< templates.size();i++){				
				Object eachTemp = templates.get(i);
				if(eachTemp instanceof Classifier){
					Classifier eachTempClazz = (Classifier)eachTemp;
					if(eachTempClazz.getName().equals(newTemplate.getName())){
						//AZ This is for the external schema
						if(BOMUtil.isSchemaType(newTemplate)){
							if(eachTempClazz.getOwningPackage() != null && newTemplate.getOwningPackage() != null &&
									eachTempClazz.getOwningPackage().getName()!= null &&
									eachTempClazz.getOwningPackage().getName().equals(newTemplate.getOwningPackage().getName())	){
								return eachTempClazz;
							}else if(eachTempClazz.getOwningPackage() != null && newTemplate.getOwningPackage() != null){
								if(eachTempClazz.getOwningPackage().getOwningPackage() instanceof ExternalService && 
										newTemplate.getOwningPackage().getOwningPackage() instanceof ExternalService){
									//it's inline types, the namespace has been hashCoded, so we need to compare the hash string
									String regTempPkgName = eachTempClazz.getOwningPackage().getName();
									String newTempPkgName = newTemplate.getOwningPackage().getName();
									if(regTempPkgName.equals(String.valueOf(newTempPkgName.hashCode()))){
										return eachTempClazz;
									}
								}
							}
						}else{
							//AZ FDL predefined data structure and other custom defined categories 
							if(eachTempClazz.getOwningPackage() != null && newTemplate.getOwningPackage() != null){
								if(eachTempClazz.getOwningPackage().getName() != null &&
										eachTempClazz.getOwningPackage().getName().equals(newTemplate.getOwningPackage().getName())){
									//defect 90062 retrieve the template through the registry to get the counterpart node created
									Object registeredNode = getImportSession().getContext().get(newTemplate);
									if(registeredNode instanceof NavigationCategoryNode){
										//Find the one created in the workspace in case a duplicate created such as "name_1"
										Object bomObj = BOMUtil.loadRootObject((NavigationCategoryNode)registeredNode);
										if(bomObj instanceof Class){
											eachTempClazz = (Class)bomObj;
										}
									}
									return eachTempClazz;
								}
							}else{
								return eachTempClazz;
							}
						}
					}
				}else if(eachTemp instanceof Map){
					//AZ Those are predefined types- Organizations, Resources
					Map predefinedMap = (Map)eachTemp;
					if(predefinedMap.values()!= null && predefinedMap.values().size() >0){
						Iterator predefinedIt = predefinedMap.values().iterator();
						while(predefinedIt.hasNext()){
							Object eachPredefined = predefinedIt.next();
							if(eachPredefined instanceof Classifier){
								Classifier eachClassifier = (Classifier)eachPredefined;
								// Ray - d 160951 For a predefined type, it must have a fixed UID
								if((eachClassifier.getUid()!=null && newTemplate.getUid()!=null && (eachClassifier.getUid().equals(newTemplate.getUid())))
										&& eachClassifier.getName().equals(newTemplate.getName())){
									// Ray - Fix for Cross Project reference 
									//Type type = (Classifier)BOMUtil.getPredefinedType(getProjectName(),eachClassifier.getName(),eachClassifier);
									Type type = (Classifier)BOMUtil.getPredefinedType(PredefUtil.getPredefinedProjectName(),eachClassifier.getName(),eachClassifier);
									if(type instanceof Classifier){
										return (Classifier)type;
									}
								}
							}
						}
					}					
				}				
			}
		}
		return null;
	}
	
	/**
	 * Update the template saved in the context in the case of overwrite
	 * @param newCopy
	 * @param workingCopy
	 */
	private void updateSuperClass(Classifier newCopy,Classifier workingCopy){
		if(newCopy instanceof Class && workingCopy instanceof Class){
			UpdateBusinessItemRule updateRule = new UpdateBusinessItemRule();
			updateRule.setImportSession(getImportSession());
			updateRule.setProjectName(getProjectName());
			updateRule.setSource(newCopy);
			updateRule.setWorkingCopy(workingCopy);
			updateRule.setBIRuleForSuperClassifier(true);
			updateRule.invoke();	
		}
	}
}
