package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.information.AddConstraintAndOpaqueExpressionINFCmd;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToPropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToPropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateOpaqueExpressionBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToPropertyBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;


public class UpdateExpressionRule extends AbstractImportRule {

	//copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Classifier classifier;
	private Classifier workingCopy;
	
	/**
	 * Set Original BOM Classifier
	 * @param classifier
	 */
	public void setSource(final Object classifier) {
		
		this.classifier = (Classifier) classifier;
	}

	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
	
		this.workingCopy = (Classifier) workingCopy;
		
	}

	/**
	 * Invoke the rule.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		
		if(classifier.getOwnedConstraint() != null && ! classifier.getOwnedConstraint().isEmpty()){
			updateWithConstraints(classifier.getOwnedConstraint(), workingCopy);
		}
		if(classifier instanceof Class){
			EList attributes = ((Class)classifier).getOwnedAttribute();
			updatePropertiesConstraints(attributes);
		}else if(classifier instanceof Signal){
			EList attributes =((Signal)classifier).getOwnedAttribute();
			updatePropertiesConstraints(attributes);
		}else if(classifier instanceof IndividualResourceType){
			EList attributes = ((IndividualResourceType)classifier).getOwnedAttribute();
			updatePropertiesConstraints(attributes);
		}else if(classifier instanceof BulkResourceType){
			EList attributes = ((BulkResourceType)classifier).getOwnedAttribute();
			updatePropertiesConstraints(attributes);
		}
		LoggingUtil.traceExit(this, "invoke");
	}

	/**
	 * Updates the Classifier with properties as in the attributes list.
	 * @param attributes - list of BOM Properties.
	 */
	private void updatePropertiesConstraints(List attributes){
		LoggingUtil.traceEntry(this, "updatePropertyConstraints");
		
		if(attributes != null && !attributes.isEmpty()){
			Iterator iterator = attributes.iterator();
			Property attribute = null;
			while(iterator.hasNext()){
				attribute = (Property)iterator.next();
				Property workingAttribute = getWorkingAttribute(workingCopy, attribute);
				if(attribute.getOwnedConstraint() != null && ! attribute.getOwnedConstraint().isEmpty()){
					updateWithConstraints(attribute.getOwnedConstraint(), workingAttribute);
				}
				//now update the rest of the default values.. 
				if(attribute.getDefaultValue() != null && !attribute.getDefaultValue().isEmpty()){
					EList defaultValues = attribute.getDefaultValue();
					Iterator valuesIt = defaultValues.iterator();
					while(valuesIt.hasNext()){
						ValueSpecification valueSpecification = (ValueSpecification)valuesIt.next();
						AddUpdateValueSpecificationBOMCmd addValueSpecCmd = null;
						if (valueSpecification instanceof InstanceValue){
							addValueSpecCmd = new AddInstanceValueToPropertyBOMCmd(workingAttribute);						
						}else if (valueSpecification instanceof StructuredOpaqueExpression){
							//d97575 - jie: add create StructuredOpaqueExpression
							addValueSpecCmd = new AddStructuredOpaqueExpressionToPropertyBEXCmd(workingAttribute);
						}else if (valueSpecification instanceof OpaqueExpression){
							addValueSpecCmd = new AddOpaqueExpressionToPropertyBOMCmd(workingAttribute);
						}
						if(addValueSpecCmd != null){
							if(addValueSpecCmd.canExecute()){
								try{
									addValueSpecCmd.execute();
									ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
									UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
									updateValueSpecRule.setImportSession(getImportSession());
									updateValueSpecRule.setSource(valueSpecification);
									updateValueSpecRule.setWorkingCopy(workingValueSpecification);
									updateValueSpecRule.setProjectName(getProjectName());
									updateValueSpecRule.invoke();
								}catch(RuntimeException ex){
									LoggingUtil.logError(IeMessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{attribute.getName()}, ex );
								}
							}else{
								LoggingUtil.logWarning(IeMessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{attribute.getName()}, null );
							}
						}
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "updatePropertyConstraints");
	}
	/**
	 * Retrieves the BOM Property in the <code>classifier</code> with the same name as
	 * the <code>attribute</code>
	 * @param classifier a BOM Classifier. 
	 * @param attribute a BOM Property
	 * @return Property - a Property of the <code>classifier</code> with the same name
	 * as the </code>attribute</code>
	 */
	private Property getWorkingAttribute(Classifier classifier, Property attribute){
		LoggingUtil.traceEntry(this, "getWorkingAttribute");
		Property property = null;
		EList attributes = null;
		if(classifier instanceof Class){
			attributes = ((Class)classifier).getOwnedAttribute();
		}else if (classifier instanceof Signal){
			attributes = ((Signal)classifier).getOwnedAttribute();
		}else if(classifier instanceof IndividualResourceType){
			 attributes = ((IndividualResourceType)classifier).getOwnedAttribute();
		}else if(classifier instanceof BulkResourceType){
			attributes = ((BulkResourceType)classifier).getOwnedAttribute();
		}
		if(attributes != null && !attributes.isEmpty()){
			Iterator it = attributes.iterator();
			Property workingAttribute = null;
			while(it.hasNext()){
				workingAttribute= (Property)it.next();
				if(workingAttribute.getName().equals(attribute.getName())){
					property = workingAttribute;
					break;
				}
			}
		}
		LoggingUtil.traceExit(this, "getWorkingAttribute");
		return property;
	}
	
	/**
	 * Updates the BOM <code>namedElement</code> with a <code>constraints</code>
	 * @param constraints list of BOM Constraints
	 * @param namedElement BOM NamedElement
	 */
	private void updateWithConstraints(List constraints, NamedElement namedElement){
		LoggingUtil.traceEntry(this, "updateWithConstraints");
		Iterator iterator = constraints.iterator();
		Constraint constraint = null;
		while(iterator.hasNext()){
			constraint = (Constraint)iterator.next();
			if(!(constraint.getSpecification() instanceof StructuredOpaqueExpression)){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.UNSUPPORTED_CONSTAINT_SPECIFICATION_EXCEPTION);
				//log error .. only StructuredOpaqueExpressions constraints are supported
			}else{
				BtCompoundCommand wrapperCmd = new BtCompoundCommand();
				AddConstraintAndOpaqueExpressionINFCmd addCommand = new AddConstraintAndOpaqueExpressionINFCmd();
				addCommand.setParentElement(namedElement);
				try{
					wrapperCmd.appendAndExecute(addCommand);
	
					StructuredOpaqueExpression exprWorkingCopy = (StructuredOpaqueExpression)((Constraint)namedElement.getOwnedConstraint().get(namedElement.getOwnedConstraint().size() -1)).getSpecification();
			
					UpdateOpaqueExpressionBOMCmd updateExpressionCommand = new UpdateOpaqueExpressionBOMCmd(exprWorkingCopy);
					String name = constraint.getSpecification().getName();
					updateExpressionCommand.setName(name);
					updateExpressionCommand.setDescription(((StructuredOpaqueExpression)constraint.getSpecification()).getDescription());
					wrapperCmd.appendAndExecute(updateExpressionCommand);	

					com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule expressionRule =  new com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule();
					expressionRule.setImportSession(getImportSession());
					expressionRule.setProjectName(getProjectName());
					expressionRule.setSource((StructuredOpaqueExpression)constraint.getSpecification());
					expressionRule.setWorkingCopy(exprWorkingCopy);
					expressionRule.invoke();	
				}catch(RuntimeException ex){
					LoggingUtil.logWarning(getImportSession(),IeMessageKeys.PROPERTY_PROPERTY_EXCEPTION, new String[]{namedElement.getName()}, ex );
				}
			}				
		}
		LoggingUtil.traceEntry(this, "updateWithConstraints");
	}
}
