package com.ibm.btools.blm.ie.imprt.rule.informationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.information.AddConstraintAndOpaqueExpressionINFCmd;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.PredefinedTypeUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddConstraintToNamedElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToPropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToPropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveLiteralNullBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveLiteralSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveLiteralStringBOMCmd;
import com.ibm.btools.bom.command.artifacts.RemoveValueSpecificationBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateOpaqueExpressionBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdatePropertyBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateValueSpecificationBOMCmd;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.RemoveObjectCommand;

/**
 * Updates a BOM Property
 */
public class UpdatePropertyRule extends UpdateMultiplicityRule {

	//	copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");
		super.invoke();
		Property property = (Property)super.property;
		Type type = property.getType();

		UpdatePropertyBOMCmd updateCmd = new UpdatePropertyBOMCmd((Property)workingCopy);
		//update the property name only if it is not null, otherwise the addCmd would create a 
		//default name for it
		if(property.getName()!= null){
			updateCmd.setName(property.getName());
		}
		if(type != null){
			Type masterType =(Type) BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
			if(masterType == null){
				masterType =(Type) BOMUtil.getTypeFromWorkspace(getImportSession(),getProjectName(),type);
			}
			if(masterType == null){
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Property", type.getName()}, null);
			}
			updateCmd.setType(masterType);
		}
		//update the isReadOnly
		if(property.getIsReadOnly() != null){
			updateCmd.setIsReadOnly(property.getIsReadOnly().booleanValue());
		}else{
			updateCmd.setIsReadOnly(false);
		}
		//update isStatic 
		if(property.getIsStatic() != null){
			updateCmd.setIsStatic(property.getIsStatic().booleanValue());
		}else{
			updateCmd.setIsStatic(false);
		}
		if (updateCmd.canExecute()) {
			try{
				updateCmd.execute();
			}catch(RuntimeException ex){
				LoggingUtil.logError(IeMessageKeys.PROPERTY_PROPERTY_EXCEPTION, new String[]{property.getName()}, ex );
			}
		} else {
			LoggingUtil.logError(IeMessageKeys.PROPERTY_PROPERTY_EXCEPTION, new String[]{property.getName()},null);
		}
		updateComments();	
		updateDefaultValues();
		//defect 163647
		updateOwnedConstraints();
		LoggingUtil.traceEntry(this, "invoke");
	}
	/**
	 * Update BOM Properties comments.
	 *
	 */	
	private void updateComments(){
		LoggingUtil.traceEntry(this, "updateComments");
		EList comments = property.getOwnedComment();
		//defect 92157. make sure we remove the comments before adding the new ones
		if(workingCopy.getOwnedComment() != null && !workingCopy.getOwnedComment().isEmpty()){
			EList workingComments = workingCopy.getOwnedComment();
			List removableComments = new LinkedList(workingComments);
			Comment workingComment = null; 
			RemoveCommentBOMCmd removeCmd = null;
			Iterator it2 = removableComments.iterator();
			while(it2.hasNext()){
				workingComment = (Comment)it2.next();
				removeCmd = new RemoveCommentBOMCmd(workingComment);
				if(removeCmd.canExecute()){
					try{
						removeCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, ex);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, null);
				}
			}
		}
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			Comment comment = null;
		
			AddCommentToElementBOMCmd addCmd = null;
			while(it.hasNext()){
				comment = (Comment)it.next();
				addCmd = new AddCommentToElementBOMCmd(workingCopy);
				addCmd.setBody(comment.getBody());
				if(addCmd.canExecute()){
					try{
						addCmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, ex);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, null);
				}
			}
		}
		LoggingUtil.traceExit(this, "updateComments");
	}
	/**
	 * Update the <code>workingCopy</code> BOM Property with default values.
	 * Note: this will only partially update the default values. In case the 
	 * default value is an OpaqueExpression or an InstanceSpecification, the update
	 * will be done at a next path through the Property.
	 */
	private void updateDefaultValues(){
		LoggingUtil.traceEntry(this, "updateDefaultValues");
		Property property = (Property)super.property;
		Property workingCopy = (Property)super.workingCopy;

		//AZ Fix the multiplicity of the attribute default value
		//we need to remove the default values for the working copy if there is any
		if(workingCopy.getDefaultValue() != null && !workingCopy.getDefaultValue().isEmpty()){
			List workingDefaultValues = workingCopy.getDefaultValue();
			List list = new LinkedList(workingDefaultValues);			
			Iterator workingIt = list.iterator();	
			RemoveObjectCommand removeCmd = null;
			ValueSpecification workingValueSpec = null;
			BtCompoundCommand workingWrapperCmd = new BtCompoundCommand();			 

			while(workingIt.hasNext()){
				workingValueSpec = (ValueSpecification)workingIt.next();				
				if(workingValueSpec instanceof LiteralNull){
					removeCmd = new RemoveLiteralNullBOMCmd((LiteralNull)workingValueSpec);
				}else if(workingValueSpec instanceof LiteralSpecification){					
					LiteralSpecification literalSpec = (LiteralSpecification)workingValueSpec;					
					removeCmd = new RemoveLiteralSpecificationBOMCmd(literalSpec);
				}
				if(removeCmd != null){
					workingWrapperCmd.appendAndExecute(removeCmd);
				}
			}			
		}

		if(property.getDefaultValue()!= null && !property.getDefaultValue().isEmpty()){
			List defaultValues = property.getDefaultValue();
			Iterator it = defaultValues.iterator();
			AddUpdateValueSpecificationBOMCmd addValueSpecCmd = null;
			ValueSpecification valueSpec = null;
			BtCompoundCommand wrapperCmd = new BtCompoundCommand();
			while(it.hasNext()){
				valueSpec = (ValueSpecification)it.next();
				//TODO AR 29 Mar added literal null;
				if(valueSpec instanceof LiteralNull){
					addValueSpecCmd = new AddLiteralNullToPropertyBOMCmd(workingCopy);
				}else if(valueSpec instanceof LiteralSpecification){
					//no matter what type of Literal Specification the user sends, only create the
					//LiteralSpecification as per the tool behavior
					//LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;	
					//validate the value and create empty value specification if the value
					//is invalid
					//Update just for the predefined types that are PrimitiveType
					if(valueSpec instanceof LiteralString && valueSpec.getType() instanceof PrimitiveType){
						LiteralString literalSpec = (LiteralString)valueSpec;							
						boolean isValid = PredefinedTypeUtil.isValidValueForType(valueSpec.getType().getName(), literalSpec.getValue());
						if(!isValid){
							//log warning for the invalid default value and skip updating the default value
							//for the attribute
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.INVALID_VALUE_FOR_PROPERTY_TYPE, 
									new String[]{literalSpec.getValue(),literalSpec.getType().getName(),property.getName()}, null);
							continue;
						}
					}
					addValueSpecCmd = new AddLiteralStringToPropertyBOMCmd(workingCopy);
				}
				if(addValueSpecCmd != null){
					if(wrapperCmd.appendAndExecute(addValueSpecCmd)){						
						ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
						UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
						updateValueSpecRule.setImportSession(getImportSession());
						updateValueSpecRule.setProjectName(getProjectName());
						updateValueSpecRule.setSource(valueSpec);
						updateValueSpecRule.setWorkingCopy(workingValueSpecification);
						updateValueSpecRule.setWrapperCommand(wrapperCmd);
						updateValueSpecRule.invoke();
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{property.getName()}, null);
					}
				}
			}
		}
		LoggingUtil.traceExit(this, "invoke");
	}

//	/**
//	* Update the <code>workingCopy</code> BOM Property with default values.
//	* Note: this will only partially update the default values. In case the 
//	* default value is an OpaqueExpression or an InstanceSpecification, the update
//	* will be done at a next path through the Property.
//	*/
//	private void updateDefaultValues(){
//	LoggingUtil.traceEntry(this, "updateDefaultValues");
//	Property property = (Property)super.property;
//	Property workingCopy = (Property)super.workingCopy;
//	if(property.getDefaultValue()!= null && !property.getDefaultValue().isEmpty()){
//	List defaultValues = property.getDefaultValue();
//	Iterator it = defaultValues.iterator();
//	AddUpdateValueSpecificationBOMCmd addValueSpecCmd = null;
//	ValueSpecification valueSpec = null;
//	BtCompoundCommand wrapperCmd = new BtCompoundCommand();
//	while(it.hasNext()){
//	valueSpec = (ValueSpecification)it.next();
//	if(valueSpec instanceof LiteralSpecification){
//	LiteralSpecification literalSpec = (LiteralSpecification)valueSpec;
//	if(literalSpec instanceof LiteralString){
//	addValueSpecCmd = new AddLiteralStringToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralUnlimitedNatural){
//	addValueSpecCmd = new AddLiteralUnlimitedNaturalToPropertyBOMCmd(workingCopy);
//	}else if(literalSpec instanceof LiteralNull){
//	addValueSpecCmd = new AddLiteralNullToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralInteger){
//	addValueSpecCmd = new AddLiteralIntegerToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralBoolean){
//	addValueSpecCmd = new AddLiteralBooleanToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralTime){
//	addValueSpecCmd = new AddLiteralTimeToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralDuration){
//	addValueSpecCmd = new AddLiteralDurationToPropertyBOMCmd(workingCopy);
//	}else if (literalSpec instanceof LiteralReal){
//	addValueSpecCmd = new AddLiteralRealToPropertyBOMCmd(workingCopy);
//	}
//	}
////	else if (valueSpec instanceof InstanceValue){
////	addValueSpecCmd = new AddInstanceValueToPropertyBOMCmd(workingCopy);

////	}else if (valueSpec instanceof OpaqueExpression){
////	addValueSpecCmd = new AddOpaqueExpressionToPropertyBOMCmd(workingCopy);
////	}

////	if(addValueSpecCmd != null && addValueSpecCmd.canExecute()){
////	try{
////	addValueSpecCmd.execute();
////	ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
////	UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
////	updateValueSpecRule.setImportSession(getImportSession());
////	updateValueSpecRule.setSource(valueSpec);
////	updateValueSpecRule.setWorkingCopy(workingValueSpecification);
////	updateValueSpecRule.invoke();
////	}catch(RuntimeException ex){
////	LoggingUtil.logWarning(getImportSession(), MessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{property.getName()}, ex);
////	}
////	}else{
////	LoggingUtil.logWarning(getImportSession(), MessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{property.getName()}, null);
////	}
////	}
//	}
//	if(addValueSpecCmd != null){
//	if(wrapperCmd.appendAndExecute(addValueSpecCmd)){
//	ValueSpecification workingValueSpecification = (ValueSpecification)addValueSpecCmd.getObject();
//	UpdateValueSpecificationRule updateValueSpecRule = new UpdateValueSpecificationRule();
//	updateValueSpecRule.setImportSession(getImportSession());
//	updateValueSpecRule.setSource(valueSpec);
//	updateValueSpecRule.setWorkingCopy(workingValueSpecification);
//	updateValueSpecRule.setWrapperCommand(wrapperCmd);
//	updateValueSpecRule.invoke();
//	}else{
//	LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_DEFAULT_VALUE_EXCEPTION, new String[]{property.getName()}, null);
//	}
//	}
//	}
//	LoggingUtil.traceExit(this, "invoke");
//	}
	/**
	 * Update property owned contraints
	 */
	private void updateOwnedConstraints(){
		LoggingUtil.traceEntry(this, "updateOwnedConstraints");
		if(property instanceof Property){
			Property instance = (Property)property;
			EList constraints = instance.getOwnedConstraint();
			if(constraints != null && !constraints.isEmpty()){
				Iterator cnstrntIt = constraints.iterator();
				Constraint constraint = null;				
				AddConstraintAndOpaqueExpressionINFCmd addConstraintCmd = null;
				while(cnstrntIt.hasNext()){
					constraint = (Constraint)cnstrntIt.next();
					addConstraintCmd = new AddConstraintAndOpaqueExpressionINFCmd();
					addConstraintCmd.setParentElement((NamedElement)workingCopy);
					if(addConstraintCmd.canExecute()){
						try{
							addConstraintCmd.execute();
						}catch(RuntimeException ex){
							LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, ex);
						}
					}else{
						LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, null);
					}
					Collection affectedObjs = addConstraintCmd.getAffectedObjects();
					if(affectedObjs != null && !affectedObjs.isEmpty()){
						Object[] objArray = affectedObjs.toArray();
						//Only the first rule description gets updated
						if(objArray[0] instanceof Constraint){
							Constraint newConstraint = (Constraint)objArray[0];
							if(newConstraint.getSpecification() instanceof OpaqueExpression &&
									constraint.getSpecification()instanceof StructuredOpaqueExpression	){
								StructuredOpaqueExpression expression = (StructuredOpaqueExpression)constraint.getSpecification();
								UpdateOpaqueExpressionBOMCmd updateOECmd = new UpdateOpaqueExpressionBOMCmd((OpaqueExpression)newConstraint.getSpecification());
								updateOECmd.setDescription(expression.getDescription());
								if(updateOECmd.canExecute()){
									try{
										updateOECmd.execute();
									}catch(RuntimeException ex){
										LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, ex);
									}
								}else{
									LoggingUtil.logWarning(getImportSession(), IeMessageKeys.PROPERTY_COMMENT_CREATE_EXCEPTION, new String[] {((Property)property).getName()}, null);
								}
							}
						}
					}					
					LoggingUtil.traceExit(this, "updateOwnedConstraints");
				}
			}
		}
	}
}
