package com.ibm.btools.blm.ie.imprt.rule.navigator;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.BulkResourceCategoryFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationResourceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationResourceDefinitionCategoryNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.resources.BulkResourceType;

/**
 * Import the bulk resource definition category from a resource model
 */
public class BulkResourceDefinitionCategoryRule extends AbstractImportRule {
	
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		
	private NavigationResourceCatalogNode parentNode;
	private BulkResourceType newResourceDefinitionCategory;
	private NavigationResourceDefinitionCategoryNode node;
		
	/**
	 * Set the parent node. It can be either a project node or a resource catalog node
	 * @param parentNode
	 */
	public void setParent(final NavigationResourceCatalogNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the resource definition category for import
	 * @param newResourceDefinitionCategory
	 */
	public void setSource(final Object newResourceDefinitionCategory) {
		this.newResourceDefinitionCategory = (BulkResourceType) newResourceDefinitionCategory;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public NavigationResourceDefinitionCategoryNode getNewBulkResourceDefinitionCategoryNode() {
		return node;
	}
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");	
			return;
		}
		
		if (newResourceDefinitionCategory == null) {
			LoggingUtil.trace(this, "invoke", "The source should null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String name = newResourceDefinitionCategory.getName();
		if (name == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newResourceDefinitionCategory);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof List) {
			resolve = ((List) list).contains(newResourceDefinitionCategory);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			BulkResourceCategoryFactory factory = new BulkResourceCategoryFactory(); 
			node = factory.create(
				parentNode, name, description, getImportSession(), resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CANNOT_CREATE_BULK_RESOURCE_DEFINITION_CATEGORY, new String[]{name}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		if (node != null) {
			getImportSession().getContext().put(newResourceDefinitionCategory, node);
		}	
		Object thisObject = (Object)BOMUtil.loadRootObject(node);		
		registerTemplate(thisObject);
		
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");
	}
}
