package com.ibm.btools.blm.ie.imprt.rule.navigator;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.CategoryCatalogFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationCategoryCatalogNode;
import com.ibm.btools.bom.model.models.OrganizationModel;

/**
 * Import the categoryCatalog (organization model with aspect = "categorycatalog" either from the root or from 
 * another categoryCatalog (organization model).
 */
public class CategoryCatalogRule extends CatalogRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private OrganizationModel newOrganizationModel;
	private NavigationCategoryCatalogNode node;
	
	/**
	 * Set the organization model for import
	 * @param newOrganizationModel
	 */
	public void setSource(final Object newOrganizationModel) {
		this.newOrganizationModel = (OrganizationModel) newOrganizationModel;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */	
	public NavigationCategoryCatalogNode getNewCategoryOrganizationModelNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		//TODO: this class is for support temporary solution of categories. It should be deleted in 6.0
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newOrganizationModel == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String organizationModelName = newOrganizationModel.getName();
		if (organizationModelName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		//BEGIN - TN20050725, The following can be commented out when the L003512 is implemented		
		//The following check-condition code is not needed when the nav label matches the BOM model name
		
		//special handling of default categoryCatalog label mismatch with the model name		
		// BEGIN - TN20050529, put additonal check for the null UID for the category model		
		//if (newOrganizationModel.getUid() != null) {		
		//	if(newOrganizationModel.getUid().equals(PredefTypesConst.CATEGORY_ORG_MODEL_FUID)){
		//		organizationModelName = UtilResourceBundleSingleton.INSTANCE.getMessage(
		//			CompoundCommandTranslatedMessageKeys.class, CompoundCommandTranslatedMessageKeys.CCB9671I); 
		//	}
		//}
		//	END - TN20050529, put additonal check for the null UID for the category model
		
		//END - TN20050725, The above can be commented out when the L003512 is implemented
		
		String description = BOMUtil.getDescription(newOrganizationModel);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newOrganizationModel);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			CategoryCatalogFactory factory = new CategoryCatalogFactory(); 
			node = factory.create(
				parentNode, organizationModelName, description, getImportSession(), isDefault, resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CANNOT_CREATE_ORGANIZATON_CATALOG, new String[]{organizationModelName}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		if (node != null) {
			getImportSession().getContext().put(newOrganizationModel, node);						
		}
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}	
		LoggingUtil.traceExit(this, "invoke");
	}
}
