package com.ibm.btools.blm.ie.imprt.rule.navigator;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.IndividualResourceFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationResourceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationResourceNode;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;

/**
 * Import the individual resource definition from a resource model
 */
public class IndividualResourceRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$		
	
	private NavigationResourceCatalogNode parentNode;
	private InstanceSpecification newResource;
	private NavigationResourceNode node;
		
	/**
	 * Set the parent node. It can be either a project node or a resource catalog node
	 * @param parentNode
	 */
	public void setParent(final NavigationResourceCatalogNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the resource for import
	 * @param newResource
	 */
	public void setSource(final Object newResource) {
		this.newResource = (InstanceSpecification) newResource;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public NavigationResourceNode getNewIndividualResourceNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");		
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newResource == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String name = newResource.getName();
		if (name == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newResource);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newResource);
		}		
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");		
		
		try {		
			IndividualResourceFactory factory = new IndividualResourceFactory(); 
			node = factory.create(
				parentNode, name, description, getImportSession(), resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CANNOT_CREATE_INDIVIDUAL_RESOURCE, new String[]{name}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		if (node != null) {
			getImportSession().getContext().put(newResource, node);	
		}
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}
}
