package com.ibm.btools.blm.ie.imprt.rule.navigator;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.engine.ProgressCalculator;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.navigator.factory.SignalCategoryFactory;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationDataCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationSignalCategoryNode;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Signal;

/**
 * Import the signal category from a information model
 */
public class SignalCategoryRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$		
	
	private NavigationDataCatalogNode parentNode;
	private Signal newSignalCategory;
	private NavigationSignalCategoryNode node;
		
	/**
	 * Set the parent node. It can be either a project node or a data catalog node
	 * @param parentNode
	 */
	public void setParent(final NavigationDataCatalogNode parentNode) {
		this.parentNode = parentNode;
	}
	
	/**
	 * Set the signal category for import
	 * @param newSignalCategory
	 */
	public void setSource(final Object newSignalCategory) {
		this.newSignalCategory = (Signal) newSignalCategory;
	}
	
	/**
	 * Get navigation node
	 * @return
	 */
	public NavigationSignalCategoryNode getNewSignalCategoryNode() {
		return node;
	}
	
	
	/**
	 * Invoke the rule
	 */	
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (parentNode == null) {
			LoggingUtil.trace(this, "invoke", "The parent node should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		if (newSignalCategory == null) {
			LoggingUtil.trace(this, "invoke", "The source should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String businessItemName = newSignalCategory.getName();
		if (businessItemName == null) {
			LoggingUtil.trace(this, "invoke", "The source name should not be null.");
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		
		String description = BOMUtil.getDescription(newSignalCategory);
		
		boolean resolve = false;
		Object list = getImportSession().getContext().get(IImportFramework.CONTEXT_PREDEFINED_REGISTRY_KEY);
		if (list != null && list instanceof EList) {
			resolve = ((EList) list).contains(newSignalCategory);
		}	
		LoggingUtil.trace(this, "invoke", "The resolve value: " + resolve + ".");
		
		try {		
			SignalCategoryFactory factory = new SignalCategoryFactory(); 
			node = factory.create(
				parentNode, businessItemName, description, getImportSession(), resolve);
		} catch (OperationCanceledException e) {			
			throw e;		
		} catch (RuntimeException e) {
			LoggingUtil.logWarning(getImportSession(), IeMessageKeys.CANNOT_CREATE_SIGNAL_CATEGORY, new String[]{businessItemName}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);				
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		if (node != null) {
			getImportSession().getContext().put(newSignalCategory, node);	
		}
		
		Classifier thisObject = (Classifier)BOMUtil.loadRootObject(node);		
		registerTemplate(thisObject);
		
		NavigatorUtil.save(getProjectName());
		LoggingUtil.trace(this, "invoke", "The navigatior model is saved.");		
		
		IProgressMonitor monitor = getImportSession().getProgressMonitor();
		if (monitor != null) {
			if (monitor.isCanceled()) {
				LoggingUtil.trace(this, "invoke", "The user cancels the operation.");				
				throw new OperationCanceledException();					
			}
			monitor.worked(ProgressCalculator.getSimpleTask());
		}
		LoggingUtil.traceExit(this, "invoke");		
	}

}
