package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddBOModelNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogsNode;
import com.ibm.btools.bom.model.models.ExternalModel;

public class ExternalModelBOFactory extends NavigatorFactory {
	//copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new data catalog
	 * @param navNode
	 * @param externalModel
	 * @param comment
	 * @return the newly created navigation node
	 */
	public AbstractChildContainerNode create(
			final String projectName,
			final AbstractNode navNode, 
			final ExternalModel newExternalModel, 
			final String comment, 
			ImportSession session, 
			final boolean isDefault, 
			final boolean resolve) {
		
		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;
		
		AbstractChildContainerNode parentNode = null;		
		boolean isProjectNode = false;
		AbstractChildContainerNode newNode = null;
		
		String name = newExternalModel.getName();	
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);	
		if(navNode instanceof NavigationBOCatalogNode){
			parentNode = (NavigationBOCatalogNode)navNode;			
		}else if(navNode instanceof NavigationBOCatalogsNode){
			parentNode = (NavigationBOCatalogsNode)navNode;	
		}
		else{			
			parentNode = NavigatorUtil.getDefaultCatalogForExternalSchema(projectName);
			if(parentNode == null){
				LoggingUtil.trace(this, "create", "No default navigation node available.");					
				LoggingUtil.traceExit(this, "create");
				return null;
			}
		}		
		// Check to see if the model is already existed.
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		if(BOMUtil.isInlineSchema(newExternalModel)){
			precondition.setName(newExternalModel.getName());
		}else{
			precondition.setName(name);	
		}			
		
		int flag = VALID; 
		try {
			if(!precondition.isValid(true)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, newExternalModel));
					}					
					return getNode(parentNode, newExternalModel);
				}	
//				AZ we always merge the package so we always flag as overwrite in which the existing
//				element will be reused
				flag = OVERWRITE;
				
//				if (!isDefault) {					
//						flag = getCollisionHandleRequest(session, name,true);					
//					if (flag == IGNORE) {
//						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
//						LoggingUtil.traceExit(this, "create");	
//						return null;
//					} else if (flag == OVERWRITE) {
//						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");							
//					} 
//				} else {
//					flag = OVERWRITE;
//				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");			
			return null;
		}		
		
		if (flag != OVERWRITE) {
			createNode(
					parentNode,
					session,
					projectName,					
					newExternalModel,
					comment,
					fileLengthExceeded);
		}			
		newNode = getNode(parentNode,newExternalModel);
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_EXTERNAL_MODEL, 
					new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		}
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}	
	
	/**
	 * Return the external model node by name
	 * @param parentNode
	 * @param externalModel
	 * @return
	 */
	private AbstractChildContainerNode getNode(final AbstractChildContainerNode parentNode, final ExternalModel externalModel) {
		LoggingUtil.traceEntry(this, "getNode");		
		String schemaCatlogName = externalModel.getName();	
		EList list = null;
		AbstractChildContainerNode navContainerNode = null;
		String label = null;
		//Catalog within a catalog
		if (parentNode instanceof NavigationBOCatalogNode)  {	
			list = ((NavigationBOCatalogNode)parentNode).getBoCatalogChildren();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator BOCatalogNodeIterator = list.iterator();
			
			while(BOCatalogNodeIterator.hasNext()) {
				navContainerNode = (AbstractChildContainerNode) BOCatalogNodeIterator.next();
				label = navContainerNode.getLabel();
				if (label != null && label.compareTo(schemaCatlogName) == 0) {
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");
					if(navContainerNode instanceof NavigationBOCatalogNode){
						return (NavigationBOCatalogNode) navContainerNode;
					}					
				}			
			}
			//When the catalog under the root
		}else if(parentNode instanceof NavigationBOCatalogsNode){
			list = ((NavigationBOCatalogsNode)parentNode).getBoCatalog();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator boCatalogIterator = list.iterator();			
			while(boCatalogIterator.hasNext()){
				navContainerNode = (AbstractChildContainerNode)boCatalogIterator.next();
				label = navContainerNode.getLabel();
				if(label != null && label.compareTo(schemaCatlogName) == 0){						
					LoggingUtil.traceExit(this, "getNode");
					if(navContainerNode instanceof NavigationBOCatalogNode){
						return navContainerNode;
					}
				}
			}			
		}
		return null;
	}
	
	private void createNode(
			AbstractChildContainerNode parentNode,
			ImportSession session,
			String projectName,
			ExternalModel newExternalModel,
			String description,
			boolean fileLengthExceeded){	
		
		String domainName = newExternalModel.getName(); 
		final AddBOModelNAVCmd addCmd = new AddBOModelNAVCmd();
		addCmd.setProjectName(projectName);
		addCmd.setDomainModelName(domainName);
		addCmd.setDescription(description);
		if(parentNode instanceof NavigationBOCatalogNode){				
			addCmd.setAbstractLibraryChildNode((NavigationBOCatalogNode)parentNode);			
			addCmd.setParentInformationModelURI((String)((NavigationBOCatalogNode)parentNode).getEntityReference());
		}else if(parentNode instanceof NavigationBOCatalogsNode){			
			addCmd.setAbstractLibraryChildNode((NavigationBOCatalogsNode)parentNode);
			addCmd.setParentInformationModelURI((String)((NavigationBOCatalogsNode)parentNode).getEntityReference());					
		}		
		if (addCmd.canExecute()) {
			try{
				addCmd.execute();		
			}catch(RuntimeException e){
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{domainName}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);	
						throw e;
					}					
				}
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");	
			}					
		}else{
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}	
	}
  }
