package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.CompoundcommandPlugin;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddDomainObjectNavigatorCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddInlineXSDSchemaNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.add.AddXSDFileNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveComplexTypeDefinitionNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveComplexTypeTemplateNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveDomainObjectNavigatorCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveInlineComplexTypeDefinitionNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveInlineComplexTypeTemplateNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.model.blmfilemanager.BLMFileMGR;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeTemplateNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplateNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLFileNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationXSDFileNode;
import com.ibm.btools.blm.ui.navigation.model.command.business.UpdateAbstractNodeBusCmd;
import com.ibm.btools.bom.command.compound.CloseRootObjectBOMCmd;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.bom.command.compound.SaveRootObjectBOMCmd;
import com.ibm.btools.bom.command.externalmodels.AddUpdateExternalDocumentBOMCmd;
import com.ibm.btools.bom.command.externalmodels.UpdateExternalSchemaBOMCmd;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.infrastructure.util.UIDGenerator;
import com.ibm.btools.model.refactor.refactoring.RenameCommand;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.util.logging.LogHelper;

/**
 * Factory class to create an external schema catalog
 */
public class ExternalSchemaFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new data catalog
	 * @param navNode
	 * @param externalSchemaName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public AbstractChildContainerNode create(
			final String projectName,
			final AbstractNode navNode, 
			final ExternalSchema newExternalSchemaModel, 
			final String comment, 
			ImportSession session, 
			final boolean isDefault, 
			final boolean resolve) {
		
		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;
		
		AbstractChildContainerNode parentNode = null;		
		boolean isProjectNode = false;
		AbstractChildContainerNode newNode = null;
		
		String name = newExternalSchemaModel.getName();
		
		if(!BOMUtil.isInlineSchema(newExternalSchemaModel)){
			assertModelNameLength(name);
		}
		
		//name = getProperRootElementName(session, name);	
		if(navNode instanceof NavigationBOCatalogNode){
			parentNode = (NavigationBOCatalogNode)navNode;			
		}else if(navNode instanceof NavigationWSDLFileNode) {
			//it's the inline schema, so the parent node should be the NavigationWSDLFileNode		
			parentNode = (NavigationWSDLFileNode)navNode;				
		}else if(navNode == null ||
				(!(navNode instanceof NavigationWSDLFileNode)&&
						!(navNode instanceof NavigationBOCatalogNode))){			
			parentNode = NavigatorUtil.getDefaultCatalogForExternalSchema(projectName);
			if(parentNode == null){
				LoggingUtil.trace(this, "create", "No default navigation node available.");					
				LoggingUtil.traceExit(this, "create");
				return null;
			}
		}		
		else {
			LoggingUtil.trace(this, "create", "This is a not valid navigation node.");					
			LoggingUtil.traceExit(this, "create");
			return null;
		}
		
		// Check to see if the model is already existed.
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		if(BOMUtil.isInlineSchema(newExternalSchemaModel)){
			precondition.setName(newExternalSchemaModel.getName());
		}else{
			precondition.setName(name);	
		}			
		
		int flag = VALID; 
		try {
			if(!precondition.isValid(true)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, newExternalSchemaModel,false));
					}
					if(BOMUtil.isInlineSchema(newExternalSchemaModel)){
						return getNode(parentNode, newExternalSchemaModel,false);
					}
					return getNode(parentNode, newExternalSchemaModel,false);
				}				
				if (!isDefault) {	
					//Give the user option whether they want to overwrite the existing xsd file
					//AZ Always overwrite the inline schemas
					if(BOMUtil.isInlineSchema(newExternalSchemaModel)){
						flag = OVERWRITE;//getCollisionHandleRequest(session, newExternalSchemaModel.getName(),true);;
					}else{
						flag = getCollisionHandleRequest(session, name,true);
					}			
					
					if (flag == IGNORE) {
						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
						LoggingUtil.traceExit(this, "create");	
						return null;
					} else if (flag == OVERWRITE) {
						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
						consolidateExternalSchema(getNode(parentNode,newExternalSchemaModel,false),newExternalSchemaModel,projectName);
					} 
				} else {
					flag = OVERWRITE;
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");			
			return null;
		}		
		
		if (flag != OVERWRITE) {
			createNode(
					parentNode,
					session,
					projectName,					
					newExternalSchemaModel,
					comment,
					fileLengthExceeded);
		}
		
		//update aspect for external schema	and node attribute where appropriate	
		newNode = getNode(parentNode,newExternalSchemaModel,false);
		if(newNode != null){
			updateAspect(newNode,projectName,newExternalSchemaModel);		
			BOMUtil.updateNodeAttribute(newExternalSchemaModel,newNode);
		}		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_EXTERNAL_SCHEMA, 
					new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		}
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}	
	
	/**
	 * Update inline external schema's aspect as 'inlineData'
	 * @param newNode
	 * @param projectName
	 * @param newExternalSchemaModel
	 */
	private void updateAspect(AbstractChildContainerNode newNode,
			String projectName,
			ExternalSchema newExternalSchemaModel){
		if(newNode == null || newExternalSchemaModel == null){
			return;
		}
		String blmUri = (String)newNode.getEntityReference();
		OpenRootObjectForUpdateBOMCmd roOpenCmd = new OpenRootObjectForUpdateBOMCmd();
		roOpenCmd.setProjectName(projectName);
		roOpenCmd.setROBLM_URI(blmUri);
		if(roOpenCmd.canExecute()){
			roOpenCmd.execute();
		}
		
		EObject bomActivity = roOpenCmd.getROCopy();
		//record the copy id for update later on
		String copyId = roOpenCmd.getCopyID();
		
		if(bomActivity instanceof ExternalSchema){	
			AddUpdateExternalDocumentBOMCmd updateCmd = null;				
			updateCmd = new UpdateExternalSchemaBOMCmd((ExternalSchema)bomActivity);				
			updateCmd.setAspect(newExternalSchemaModel.getAspect());					
			//Update external document attributes
			if(updateCmd.canExecute()){
				updateCmd.execute();
			}	
			//Save the opened root object model
			SaveRootObjectBOMCmd saveRootObjectBOMCmd = new SaveRootObjectBOMCmd();
			saveRootObjectBOMCmd.setProjectName(projectName);
			saveRootObjectBOMCmd.setCopyID(copyId);
			
			if(saveRootObjectBOMCmd.canExecute()){
				saveRootObjectBOMCmd.execute();
			}
		}
        if (copyId != null) {
            CloseRootObjectBOMCmd roCloseCmd = new CloseRootObjectBOMCmd();
            roCloseCmd.setCopyID(copyId);
            if(roCloseCmd.canExecute()){
                   roCloseCmd.execute();
            }
     }

	}
	
	/**
	 * Consolidate the complex types contained by the external schema
	 * @param existingNode
	 * @param importedSchema
	 * @param projectName
	 */
	private void consolidateExternalSchema(AbstractChildContainerNode existingNode,
			ExternalSchema importedSchema,
			String projectName){
		List removedNode = new ArrayList();
		EList bomComplexTypes = importedSchema.getOwnedMember();
		if(bomComplexTypes == null){
			return;
		}
		if(existingNode instanceof NavigationXSDFileNode){
			NavigationXSDFileNode theNode = (NavigationXSDFileNode)existingNode;
			NavigationComplexTypeDefinitionsNode ComplexTypeDefinitions = theNode.getComplexTypeDefinitionsNode();	
			NavigationComplexTypeTemplatesNode ComplexTypeTemplates = theNode.getComplexTypeTemplatesNode();		
			if(ComplexTypeDefinitions != null){
				EList existingComplexTypeDefinitions = ComplexTypeDefinitions.getComplexTypeDefinitionNode();
				for(int i = 0;i<existingComplexTypeDefinitions.size();i++){
					boolean matchFound = false;
					AbstractChildLeafNode existingElement = (AbstractChildLeafNode)existingComplexTypeDefinitions.get(i);
					for(int  j = 0; j< bomComplexTypes.size();j++){
						NamedElement bomElement = (NamedElement)bomComplexTypes.get(j);
						if(existingElement.getLabel()!= null && existingElement.getLabel().equals(bomElement.getName())){
							matchFound = true;
							break;
						}						
					}
					if(!matchFound){
						removedNode.add(existingElement);
					}
				}
			}
			if(ComplexTypeTemplates != null){
				EList existingComplexTypeTemplates = ComplexTypeTemplates.getComplexTypeTemplateNode();
				for(int i = 0;i<existingComplexTypeTemplates.size();i++){
					boolean matchFound = false;
					AbstractChildLeafNode existingElement = (AbstractChildLeafNode)existingComplexTypeTemplates.get(i);
					for(int  j = 0; j< bomComplexTypes.size();j++){
						NamedElement bomElement = (NamedElement)bomComplexTypes.get(j);
						if(existingElement.getLabel()!= null && existingElement.getLabel().equals(bomElement.getName())){
							matchFound = true;
							break;
						}						
					}
					if(!matchFound){
						removedNode.add(existingElement);
					}
				}
			}
			
		}else if(existingNode instanceof NavigationInlineXSDSchemaNode){
			NavigationInlineXSDSchemaNode inlineNode = (NavigationInlineXSDSchemaNode)existingNode;
			NavigationInlineComplexTypeDefinitionsNode ComplexTypeDefinitions = inlineNode.getInlineComplexTypeDefinitionsNode();	
			NavigationInlineComplexTypeTemplatesNode ComplexTypeTemplates = inlineNode.getInlineComplexTypeTemplatesNode();
			if(ComplexTypeDefinitions != null ){
				EList existingInlineComplexTypeDefinitions = ComplexTypeDefinitions.getInlineComplexTypeDefinitionNode();
				for(int i = 0;i<existingInlineComplexTypeDefinitions.size();i++){
					boolean matchFound = false;
					AbstractChildLeafNode existingElement = (AbstractChildLeafNode)existingInlineComplexTypeDefinitions.get(i);
					for(int  j = 0; j< bomComplexTypes.size();j++){
						NamedElement bomElement = (NamedElement)bomComplexTypes.get(j);
						if(existingElement.getLabel()!= null && existingElement.getLabel().equals(bomElement.getName())){
							matchFound = true;
							break;
						}						
					}
					if(!matchFound){
						removedNode.add(existingElement);
					}
				}
			}
			if(ComplexTypeTemplates != null){
				EList existingInlineComplexTypeTemplates = ComplexTypeTemplates.getInlineComplexTypeTemplateNode();
				for(int i = 0;i<existingInlineComplexTypeTemplates.size();i++){
					boolean matchFound = false;
					AbstractChildLeafNode existingElement = (AbstractChildLeafNode)existingInlineComplexTypeTemplates.get(i);
					for(int  j = 0; j< bomComplexTypes.size();j++){					
						NamedElement bomElement = (NamedElement)bomComplexTypes.get(j);
						if(existingElement.getLabel()!= null && existingElement.getLabel().equals(bomElement.getName())){
							matchFound = true;
							break;
						}						
					}
					if(!matchFound){
						removedNode.add(existingElement);
					}
				}
			}
		}		
		//Remove the nodes
		if(removedNode != null && removedNode.size() > 0){
			Iterator rmIt = removedNode.iterator();
			while(rmIt.hasNext()){
				AbstractChildLeafNode theNode = (AbstractChildLeafNode)rmIt.next();
				RemoveDomainObjectNavigatorCmd removeCmd = null;
				if(theNode instanceof NavigationComplexTypeDefinitionNode){
					removeCmd = new RemoveComplexTypeDefinitionNAVCmd();					
				}else if(theNode instanceof NavigationComplexTypeTemplateNode ){
					removeCmd = new RemoveComplexTypeTemplateNAVCmd();					
				}else if(theNode instanceof NavigationInlineComplexTypeDefinitionNode){
					removeCmd = new RemoveInlineComplexTypeDefinitionNAVCmd();					
				}else if(theNode instanceof NavigationInlineComplexTypeTemplateNode){
					removeCmd = new RemoveInlineComplexTypeTemplateNAVCmd();					
				}				
				removeCmd.setProjectName(projectName);
				removeCmd.setParentNavigatorNode(theNode);
				closeEditors(theNode,projectName);
				if(removeCmd.canExecute()){
					removeCmd.execute();
				}
			}
		}
		//The node has been modified and needs to be saved
		NavigatorUtil.saveNode(existingNode,projectName);	
	}
	
	/**
	 * Return the external schema node by name
	 * @param parentNode
	 * @param schemaFileName
	 * @return
	 */
	private AbstractChildContainerNode getNode(final AbstractChildContainerNode parentNode, final ExternalSchema externalSchema,final boolean hashNamespace) {
		LoggingUtil.traceEntry(this, "getNode");	
		String schemaFileName = null;
		if(parentNode instanceof NavigationWSDLFileNode && !BOMUtil.isFileAttachment(externalSchema) && hashNamespace){
			schemaFileName = String.valueOf(externalSchema.getTargetNamespace().hashCode());
		}else{
			schemaFileName = externalSchema.getName();
		}
		EList list = null;
		AbstractChildContainerNode navContainerNode = null;
		String label = null;
		//default schema/external schema
		if (parentNode instanceof NavigationBOCatalogNode)  {	
			list = ((NavigationBOCatalogNode)parentNode).getXsdFile();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator BOCatalogNodeIterator = list.iterator();
			
			while(BOCatalogNodeIterator.hasNext()) {
				navContainerNode = (AbstractChildContainerNode) BOCatalogNodeIterator.next();
				label = navContainerNode.getLabel();
				if (label != null && label.compareTo(schemaFileName) == 0) {
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");
					if(navContainerNode instanceof NavigationXSDFileNode){
						return (NavigationXSDFileNode) navContainerNode;
					}					
				}			
			}
			//This is the case of inline schema
		}else if(parentNode instanceof NavigationWSDLFileNode){
			list = ((NavigationWSDLFileNode)parentNode).getOnlineXSDSchema();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator inLineXSDSchemaIterator = list.iterator();			
			while(inLineXSDSchemaIterator.hasNext()){
				navContainerNode = (AbstractChildContainerNode)inLineXSDSchemaIterator.next();
				label = navContainerNode.getLabel();
				if((label != null && label.compareTo(schemaFileName) == 0) ||
						(BOMUtil.isFileAttachment(externalSchema) && BOMUtil.FILE_ATTACHMENT_NODE_NAME.compareTo(label)== 0)){
					LoggingUtil.traceExit(this, "getNode");
					if(navContainerNode instanceof NavigationInlineXSDSchemaNode){
						return navContainerNode;
					}
				}
			}			
		}
		return null;
	}
	
	private void createNode(
			AbstractChildContainerNode parentNode,
			ImportSession session,
			String projectName,
			ExternalSchema newExternalSchemaModel,
			String description,
			boolean fileLengthExceeded){	
		
		String originalName = newExternalSchemaModel.getName();
		//AZ relax the truncation for the 50 characters name length 
		String domainName = originalName;//getProperRootElementName(session, originalName); 		
		// inline schema	
		AddDomainObjectNavigatorCmd addCmd = null;
		if(parentNode instanceof NavigationWSDLFileNode){
			addCmd = new AddInlineXSDSchemaNAVCmd();			
			addCmd.setParentInformationModelURI((String)(((NavigationWSDLFileNode)parentNode).getEntityReference()));
			if(BOMUtil.isFileAttachment(newExternalSchemaModel)){
				addCmd.setDomainModelName(BOMUtil.FILE_ATTACHMENT_NODE_NAME);
				addCmd.setLabel(BOMUtil.FILE_ATTACHMENT_NODE_NAME);
			}else{
				addCmd.setDomainModelName(String.valueOf((newExternalSchemaModel.getTargetNamespace().hashCode())));
				addCmd.setLabel(String.valueOf((newExternalSchemaModel.getTargetNamespace().hashCode())));
			}
			//default or external schema
		}else if(parentNode instanceof NavigationBOCatalogNode){
			addCmd = new AddXSDFileNAVCmd();
			addCmd.setParentInformationModelURI((String)(((NavigationBOCatalogNode)parentNode).getEntityReference()));
			addCmd.setDomainModelName(domainName);	
			addCmd.setLabel(domainName);
		}
		addCmd.setProjectName(projectName);
		addCmd.setDefaultID("XSD file");
		addCmd.setAbstractLibraryChildNode(parentNode);
		//addCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV")); 
		addCmd.setDescription("External model represents the xsd file");
		
		if (addCmd.canExecute()) {
			try{
				addCmd.execute();
				//update the label name as the original name for the inline schema
				if(addCmd instanceof AddInlineXSDSchemaNAVCmd && !BOMUtil.isFileAttachment(newExternalSchemaModel)){
					AbstractNode createdNode = getNode(parentNode,newExternalSchemaModel,true);
					UpdateAbstractNodeBusCmd updateCmd = new UpdateAbstractNodeBusCmd(createdNode);
					//set the id as the BOM element name		
					updateCmd.setId(String.valueOf((newExternalSchemaModel.getTargetNamespace().hashCode())));										
					updateCmd.setLabel(originalName);					
					if (updateCmd.canExecute()) {
						updateCmd.execute();
					}
				}
			}catch(RuntimeException e){
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{originalName}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);	
						throw e;
					}					
				}
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");	
			}					
		}else{
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}	
	}
	
}

