package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddFormNAVCmd;
import com.ibm.btools.blm.compoundcommand.navigator.remove.RemoveFormNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateProcessModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationFormNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationFormsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProcessCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.util.NavigationConstants;

/**
 * Factory class to create a business item
 */
public class FormFactory extends NavigatorFactory {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new form
	 * @param navNode
	 * @param form name
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationFormNode create(final NavigationProcessCatalogNode parentNode, final String formName, 
		final String comment, ImportSession session, final boolean resolve) {


		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;

		String projectName = null;
		boolean isProjectNode = false;
		boolean canRemove = true;
		NavigationFormNode newNode = null;
					;	
		String name = getProperRootElementName(session, formName);	
		projectName = parentNode.getProjectNode().getLabel();
		
		// Check to see if the model is already existed.
		CreateProcessModelElementPrecondition precondition = new CreateProcessModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);
		precondition.setType(NavigationConstants.PROCESS);		
		 
		int flag = VALID;
		try { 
			if(!precondition.isValid()) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}
				flag = getCollisionHandleRequest(session, formName);
				if (flag == IGNORE) {
					LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
					LoggingUtil.traceExit(this, "create");	
					return null;
				} else if (flag == OVERWRITE) {
					
					LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
					NavigationFormNode formNode = getNode(parentNode, formName);
					if (isNavigatorNodeAlreadyExisted(session, formNode)) {
						// IGNORE
						return null;	
					}					
					closeEditors(formNode, projectName);				
					RemoveFormNAVCmd navCmd = new RemoveFormNAVCmd();
					navCmd.setProjectName(projectName);
					navCmd.setParentNavigatorNode(formNode);
					if (navCmd.canExecute()) {
						try {
							navCmd.execute();
						} catch (RuntimeException e) {
							LoggingUtil.trace(this, "create", "The remove command throws a RuntimeException.");							
							canRemove = false;
						}
					} else {
						LoggingUtil.trace(this, "create", "Cannot execute the remove command.");						
						canRemove = false;
					}	
				} else if (flag == NEW) {
					LoggingUtil.trace(this, "create", "The user chooses to create an element with a new name.");
					name = generateName(parentNode, name, precondition);	
					if (name == null) {											
						LoggingUtil.traceExit(this, "create");	
						return null;	
					}
				} else if (flag == CANCEL) {
					LoggingUtil.trace(this, "create", "The user chooses to cancel the import.");
					throw new OperationCanceledException();
				}
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");							
			return null;
		}
		
		if (canRemove) {		
		//	NavigatorUtil.recordNewNode(session,process);
			// Parameters:
			// 1. project name
			// 2. process name
			// 3. parent Navigator node
			AddFormNAVCmd navCmd = new AddFormNAVCmd();
			navCmd.setProjectName(projectName);
			navCmd.setDomainModelName(name);
			navCmd.setDescription(comment);
			navCmd.setAbstractLibraryChildNode(parentNode);		
			navCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
			// Ray - added a flag to control creation of default form.  in the case
			// of import engine, we want to switch the flag off.
			navCmd.setCreateDefaultForm(Boolean.FALSE);
			if (navCmd.canExecute()) {
				try {
					navCmd.execute();
				} catch (RuntimeException e) {
					if (e instanceof CCRuntimeException) {
						CCRuntimeException ccEx = (CCRuntimeException) e;
						if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
							fileLengthExceeded = true;						
							LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);							
						}
					}
					LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
				}
			} else {
				LoggingUtil.trace(this, "create", "Add command cannot be executed.");
			}			
			
			// The Navigator model is created, but need to update it.
			newNode = getNode(parentNode, name);
		}
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_FORM, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} else {
			if (name.compareTo(formName) != 0)
			LoggingUtil.logWarning(session, IeMessageKeys.CREATE_FORM_DIFF_NAME, 
				new String[]{formName, name}, null);			
		}
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;	
	}	

	/**
	 * Return the form node by name
	 * @param parentNode
	 * @param formName
	 * @return
	 */
	private NavigationFormNode getNode(final NavigationProcessCatalogNode parentNode, final String formName) {
		LoggingUtil.traceEntry(this, "getNode");
		NavigationFormsNode entitiesNode = parentNode.getFormsNode();
		if (entitiesNode == null) {
			LoggingUtil.traceExit(this, "getNode");
			return null;
		}
		EList nodes = entitiesNode.getFormNodes();		
		Iterator formNodeIterator = nodes.iterator();
		NavigationFormNode navContainerNode = null;
		String label = null;
		while(formNodeIterator.hasNext()) {
			navContainerNode = (NavigationFormNode) formNodeIterator.next();
			label = navContainerNode.getLabel();			
			if (label != null && formName.compareTo(label) == 0) {
				// This is the Navigator model
				LoggingUtil.traceExit(this, "getNode");
				return navContainerNode;
			}
		}
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}	
}
