package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddOperationNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationOperationNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.blm.ui.navigation.model.command.business.UpdateAbstractNodeBusCmd;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.infrastructure.util.UIDGenerator;
import com.ibm.btools.ui.navigation.manager.util.NavigationManagerPlugin;

/**
 * @author andyxz 
 */
public class OperationFactory extends NavigatorFactory {
//	 copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new operation node
	 * @param navNode
	 * @param operationName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationOperationNode create(
			final AbstractNode navNode, 
			final Activity operation, 
			final String comment, 
			ImportSession session, 
			final boolean isDefault, 
			final boolean resolve) {

		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;

		AbstractChildContainerNode parentNode = null;
		String projectName = null;
		boolean isProjectNode = false;
		NavigationOperationNode newNode = null;
					
		String name = operation.getName();	
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);	
		if (navNode instanceof NavigationWSDLPortTypeNode) {
			parentNode = (NavigationWSDLPortTypeNode) navNode;
			projectName = ((NavigationWSDLPortTypeNode)navNode).getProjectNode().getLabel();
		} else {
			LoggingUtil.trace(this, "create", "This is a not valid navigation node.");					
			LoggingUtil.traceExit(this, "create");
			return null;
		}
		
		// Check to see if the model is already existed.
		//Andy leave it for now - how to validate??
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);		
		 
		int flag = VALID; 
		try {
			if(!precondition.isValid(false)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					LoggingUtil.traceExit(this, "create");	
					return getNode(parentNode, name);
				}				
				if (!isDefault) {				
					flag = NavigatorUtil.getExternalServiceOption(session.getImportOptions(),operation);
					if (flag == IGNORE) {
						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
						LoggingUtil.traceExit(this, "create");	
						return null;
					} else if (flag == OVERWRITE) {
						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");								
						NavigationOperationNode operationNode = getNode(parentNode, name);
						final String blmURI = (String)operationNode.getEntityReferences().get(0);					
						//close open editors before overwrite
						closeEditors(operationNode,projectName);				
					}
				} else {
					flag = OVERWRITE;
				}				
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");		
			return null;
		}
		
		if (flag != OVERWRITE) {
			//flag to indicate the operation is anew 
			NavigatorUtil.recordNewNode(session,operation);			
			createNode(
					parentNode,
					session,
					projectName,
					name,
					comment,
					fileLengthExceeded);			
			}		
				
		// The Navigator model is created, but need to update it.
		newNode = getNode(parentNode, name);
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_OPERATION, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} 
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}

	/**
	 * look up operation node for the named operation
	 * @param parentNode
	 * @param operationName
	 * @return
	 */
	private NavigationOperationNode getNode(final AbstractChildContainerNode parentNode, final String operationName) {
		LoggingUtil.traceEntry(this, "getNode");
		if (parentNode instanceof NavigationWSDLPortTypeNode)  {
			EList list = ((NavigationWSDLPortTypeNode) parentNode).getOperation();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator operationNodesIterator = list.iterator();
			NavigationOperationNode navContainerNode = null;
			String label = null;
			while(operationNodesIterator.hasNext()) {
				navContainerNode = (NavigationOperationNode)operationNodesIterator.next();
				label = navContainerNode.getLabel();				
				if (label != null && 
					label.compareTo(operationName) == 0 &&
					navContainerNode instanceof NavigationOperationNode) {					
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");						
					return navContainerNode;
				}			
			}
		} 
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}
	
	private void createNode(
			AbstractChildContainerNode parentNode,
			ImportSession session,
			String projectName,
			String domainName,
			String description,
			boolean fileLengthExceeded){	
		//Operation represents Activity + SAN with aspect of "ServiceOperation"
		
		AddOperationNAVCmd addOperationCmd = new AddOperationNAVCmd();
		addOperationCmd.setProjectName(projectName);
		addOperationCmd.setDefaultID("WSDL operation");
		addOperationCmd.setAbstractLibraryChildNode(parentNode);
		//addOperationCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV"));
		//defect 86727. revert, allow for Operation documentation, do not overwrite
		addOperationCmd.setDescription(description);
	//	addOperationCmd.setDescription(LoggingUtil.getLocalizedString(IeMessageKeys.SERVICE_OPERATION_DESCRIPTION,null));
		addOperationCmd.setDomainModelName(domainName);
		addOperationCmd.setLabel(domainName);
		addOperationCmd.setParentInformationModelURI((String)parentNode.getEntityReference());
		
		if (addOperationCmd.canExecute()) {
			try {
				addOperationCmd.execute();
			} catch (RuntimeException e) {
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
							new String[]{domainName}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);
						throw e;
					}
				}
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
			}
		} else {
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
	}
	}
}
