package com.ibm.btools.blm.ie.imprt.rule.navigator.factory;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.navigator.add.AddWSDLPortTypeNAVCmd;
import com.ibm.btools.blm.compoundcommand.util.CCRuntimeException;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.CreateExternalModelElementPrecondition;
import com.ibm.btools.blm.ie.imprt.rule.navigator.precondition.InvalidTypeException;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLFileNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.infrastructure.util.UIDGenerator;

/**
 * @author andyxz 
 */
public class ServiceInterfaceFactory extends NavigatorFactory {
	//	 copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	/**
	 * Create a new external service catalog to mimic the external service catalog
	 * @param navNode
	 * @param serviceInterfaceName
	 * @param comment
	 * @return the newly created navigation node
	 */
	public NavigationWSDLPortTypeNode create(
			final AbstractNode navNode, 
			final String serviceInterfaceName, 
			final String comment, 
			ImportSession session, 
			final boolean isDefault, 
			final boolean resolve) {

		LoggingUtil.traceEntry(this, "create");	
		boolean fileLengthExceeded = false;

		NavigationWSDLFileNode parentNode = null;
		String projectName = null;
		boolean isProjectNode = false;
		NavigationWSDLPortTypeNode newNode = null;
					
		String name = serviceInterfaceName;	
		assertModelNameLength(name);
		//name = getProperRootElementName(session, name);	
		if (navNode instanceof NavigationWSDLFileNode) {
			parentNode = (NavigationWSDLFileNode) navNode;
			projectName = parentNode.getProjectNode().getLabel();
		} else {
			LoggingUtil.trace(this, "create", "This is a not valid navigation node.");					
			LoggingUtil.traceExit(this, "create");
			return null;
		}
		
		// Check to see if the model is already existed.
		//Andy leave it for now - how to validate??
		CreateExternalModelElementPrecondition precondition = new CreateExternalModelElementPrecondition();
		precondition.setParent(parentNode);
		precondition.setName(name);		
		 
		int flag = VALID; 
		try {
			if(!precondition.isValid(false)) {
				LoggingUtil.trace(this, "create", "The precondition is failed.");
				if (resolve) {
					LoggingUtil.trace(this, "create", "This is a predefined type.");					
					LoggingUtil.traceExit(this, "create");	
					Object list = session.getContext().get(IImportFramework.CONTEXT_RESOLVED_TYPE);
					if (list != null && list instanceof List) {
						((List)list).add(getNode(parentNode, name));
					}
					return getNode(parentNode, name);
				}				
				if (!isDefault) {	
					//we always overwrite the portType
					flag = OVERWRITE;
					if (flag == IGNORE) {
						LoggingUtil.trace(this, "create", "The user chooses to ignore the name collision.");
						LoggingUtil.traceExit(this, "create");	
						return null;
					} else if (flag == OVERWRITE) {
						LoggingUtil.trace(this, "create", "The user chooses to overwrite the name collision.");	
						// do nothing
					}
				} else {
					flag = OVERWRITE;
				}				
			}
		} catch (InvalidTypeException e) {
			// Ignore
			LoggingUtil.traceExit(this, "create");		
			return null;
		}
		
		if (flag != OVERWRITE) {
			createNode(
					parentNode,
					session,
					projectName,
					name,
					comment,
					fileLengthExceeded);		
		}
				
		// The Navigator model is created, but need to update it.
		newNode = getNode(parentNode, name);
		
		if (newNode == null && fileLengthExceeded == false) {
			LoggingUtil.logWarning(session, IeMessageKeys.CANNOT_CREATE_SERVICE_INTERFACE, 
				new String[]{name}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);		
		} 
		
		LoggingUtil.traceExit(this, "create");	
		return newNode;
	}

	/**
	 * Return the process catalog node by name
	 * @param parentNode
	 * @param processModelName
	 * @return
	 */
	private NavigationWSDLPortTypeNode getNode(final AbstractChildContainerNode parentNode, final String serviceInterfaceName) {
		LoggingUtil.traceEntry(this, "getNode");
		if (parentNode instanceof NavigationWSDLFileNode)  {	
			EList list = ((NavigationWSDLFileNode) parentNode).getWsdlPortType();
			if(list == null){
				LoggingUtil.traceExit(this, "getNode");
				return null;
			}
			Iterator catalogsNodeIterator = list.iterator();
			AbstractChildContainerNode navContainerNode = null;
			String label = null;
			while(catalogsNodeIterator.hasNext()) {
				navContainerNode = (AbstractChildContainerNode) catalogsNodeIterator.next();
				label = navContainerNode.getLabel();				
				if (label != null && label.compareTo(serviceInterfaceName) == 0) {
					// This is the Navigator model
					LoggingUtil.traceExit(this, "getNode");
					return (NavigationWSDLPortTypeNode) navContainerNode;
				}			
			}
		} 
		LoggingUtil.traceExit(this, "getNode");
		return null;
	}
	
	private void createNode(
			NavigationWSDLFileNode parentNode,
			ImportSession session,
			String projectName,
			String domainName,
			String description,
			boolean fileLengthExceeded){
		
		AddWSDLPortTypeNAVCmd addPortTypeCmd = new AddWSDLPortTypeNAVCmd();
		addPortTypeCmd.setProjectName(projectName);
		addPortTypeCmd.setDefaultID("WSDL port");
		addPortTypeCmd.setAbstractLibraryChildNode(parentNode);
		//addPortTypeCmd.setDefaultModelID(UIDGenerator.getUID("BLMNAV")); //PredefTypesConst.PREDEF_EXT_SERVICE_FUID);
		addPortTypeCmd.setDescription(description);
		addPortTypeCmd.setDomainModelName(domainName);
		addPortTypeCmd.setLabel(domainName);
		addPortTypeCmd.setParentInformationModelURI((String)parentNode.getEntityReference());	
		if (addPortTypeCmd.canExecute()) {
			try {
				addPortTypeCmd.execute();
			} catch (RuntimeException e) {					
				if (e instanceof CCRuntimeException) {
					CCRuntimeException ccEx = (CCRuntimeException) e;
					if (ccEx.getCode().equals(ImportMessage.FILE_LENGTH_EXCEEDED)) {
						fileLengthExceeded = true;						
						LoggingUtil.logWarning(session, IeMessageKeys.FILE_LENGTH_EXCEEDED_GENERIC, 
								new String[]{domainName}, null, IImportResultMessages.CANNOT_CREATE_NAVIGATION_NODE);
						throw e;
					}					
				}				
				LoggingUtil.trace(this, "create", "Add command throws a RuntimeException.");					
			}
		} else {
			LoggingUtil.trace(this, "create", "Add command cannot be executed.");
		}		
	}
}
