package com.ibm.btools.blm.ie.imprt.rule.navigator.precondition;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalServiceCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeDefinitionsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineComplexTypeTemplatesNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationInlineXSDSchemaNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLFileNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationWSDLPortTypeNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationXSDFileNode;
import com.ibm.btools.blm.ui.navigation.model.util.precondition.NameCheckConstants;
import com.ibm.btools.blm.ui.navigation.model.util.precondition.NameCheckUtil;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 * The precondition to check does it meet the precondition to create process
 * model
 */
public class CreateExternalModelElementPrecondition extends CreateModelPrecondition {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	private int resultCode = -1;
	/**
	 * We just check the collision at the external schema and external service level
	 * algorithm: if the external schema and external service name collided, the subcomponent
	 * will be replaced
	 * @return boolean indicates valid or not
	 */
	public boolean isValid(boolean isExternalSchema) {
		EList container = null;
		if(parentNode instanceof NavigationExternalServiceCatalogNode){
			container = ((NavigationExternalServiceCatalogNode)parentNode).getWsdlFile();
			int innerResult = NameCheckUtil.checkNameConflict(container, newName);
			if(innerResult == -1){
				container = ((NavigationExternalServiceCatalogNode)parentNode).getExternalServiceCatalogChildren();
			}
			
		}else if(parentNode instanceof NavigationBOCatalogNode){
			container = ((NavigationBOCatalogNode)parentNode).getXsdFile();
			int innerResult = NameCheckUtil.checkNameConflict(container, newName);
			if(innerResult == -1){
				container = ((NavigationBOCatalogNode)parentNode).getBoCatalogChildren();
			}
			
		}else if(parentNode instanceof NavigationWSDLFileNode){
			if(isExternalSchema){
				container =  ((NavigationWSDLFileNode)parentNode).getOnlineXSDSchema();
			}else{
				container = ((NavigationWSDLFileNode)parentNode).getWsdlPortType();
			}
			
		}else if(parentNode instanceof NavigationWSDLPortTypeNode){
			container = ((NavigationWSDLPortTypeNode)parentNode).getOperation();
			
		}else if(parentNode instanceof NavigationXSDFileNode){
			NavigationComplexTypeDefinitionsNode definitions = null;
			NavigationComplexTypeTemplatesNode    templates = null;
			if(parentNode instanceof NavigationXSDFileNode){
				definitions = ((NavigationXSDFileNode)parentNode).getComplexTypeDefinitionsNode();				
				if(definitions == null){
					LoggingUtil.traceExit(this,"getNode()");
					return true;
				}
				container = definitions.getComplexTypeDefinitionNode(); 	
				int innerResult = NameCheckUtil.checkNameConflict(container, newName);
				if (innerResult == -1){
					templates = ((NavigationXSDFileNode)parentNode).getComplexTypeTemplatesNode();
					if(templates == null){
						LoggingUtil.traceExit(this,"getNode()");
						return true;
					}
					container = templates.getComplexTypeTemplateNode();
				}
			}
			
		}else if(parentNode instanceof NavigationInlineXSDSchemaNode){
			NavigationInlineComplexTypeDefinitionsNode nodesDef = ((NavigationInlineXSDSchemaNode)parentNode).getInlineComplexTypeDefinitionsNode();
			NavigationInlineComplexTypeTemplatesNode nodesTemplate = ((NavigationInlineXSDSchemaNode)parentNode).getInlineComplexTypeTemplatesNode();
			if(nodesDef == null && nodesTemplate == null){
				return true;
			}
			//AZ defect 141186 Handle inline template collision test
			if(nodesDef != null){
				container = nodesDef.getInlineComplexTypeDefinitionNode();
				int innerResult = NameCheckUtil.checkNameConflict(container, newName);
				if(innerResult == -1){
					if(nodesTemplate != null){
						container = nodesTemplate.getInlineComplexTypeTemplateNode();
					}
				}
			//d76559
			}else if(nodesTemplate != null){
				container = nodesTemplate.getInlineComplexTypeTemplateNode();
			}
		}		
		resultCode = NameCheckUtil.checkNameConflict(container, newName);		
		if(resultCode == NameCheckConstants.NAME_IS_NOT_UNIQUE_DIFFERENT_CASE ||
				resultCode == NameCheckConstants.NAME_IS_NOT_UNIQUE ){
			return false;
		}
		return true;
	}
	
	public int getResult(){
		return this.resultCode;
	}
}
