package com.ibm.btools.blm.ie.imprt.rule.organizationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateInstanceSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddUpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateInstanceSpecificationBOMCmd;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Comment;


public class UpdateOrganizationUnitRule
	extends UpdateInstanceSpecificationRule {

	static final String COPYRIGHT = " Copyright IBM Corporation 2004, 2009." ;
	/* (non-Javadoc)
	 * @see com.ibm.btools.blm.ie.imprt.rule.IImportRule#invoke()
	 */
	public void invoke() {
		
		super.invoke();

	}
	
	/**
	 * Updates the <code>workingCopy</code> InstanceSpecification with the Classifier. 
	 *
	 */
	protected void updateInstanceClassifier(){
		//TODO: support temprary solution of category value,  this method should be removed in v6.0
		LoggingUtil.traceEntry(this, "updateInstanceClassifier");
		if(instanceSample.getOwningPackage() == null){
			//handle cases that first step didn't set owning package for org unit.
			super.updateInstanceClassifier();
			return;
		}
		String aspect = instanceSample.getOwningPackage().getAspect();
		if(aspect == null || !aspect.equals(PredefTypesConst.CATEGORY_ASPECT)){
			super.updateInstanceClassifier();				
		}
		
		else{			
			
			// need to update second comment, this is needed to avoid UI throw error 			
			String commentStr = "";
			if(instanceSample.getOwnedComment().size() > 1){
				commentStr = ((Comment)instanceSample.getOwnedComment().get(1)).getBody();
			}			
			AddUpdateCommentBOMCmd cmd;
			if (instanceSample.getOwnedComment().size() > 0) {
				// Overwrite first comment body
				cmd = new UpdateCommentBOMCmd((Comment)workingCopy.getOwnedComment().get(0));
				cmd.setBody(((Comment)instanceSample.getOwnedComment().get(0)).getBody());
				if(cmd.canExecute()){
					try{
						cmd.execute();
					}catch(RuntimeException ex){
						LoggingUtil.logError(IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceSample.getName()}, ex );
					}
				}
			}
			
			// Add or update body of second comment
			if (workingCopy.getOwnedComment().size() > 1) {
				cmd = new UpdateCommentBOMCmd((Comment)workingCopy.getOwnedComment().get(1));
				
			} else {
				cmd = new AddCommentToElementBOMCmd(workingCopy);
			}
			cmd.setBody(commentStr);
			if(cmd.canExecute()){
				try{
					cmd.execute();
				}catch(RuntimeException ex){
					LoggingUtil.logError(IeMessageKeys.INSTANCE_UPDATE_EXCEPTION, new String[]{instanceSample.getName()}, ex );
				}
			}
					
			// orgUnit is categoryValue
			// the superClassifier may have beed updated by the CreateNavigationCmd ? (step 1 of step 2)
			if(workingCopy.getClassifier()!= null && !workingCopy.getClassifier().isEmpty() && workingCopy.getClassifier().get(0) != null){
				newSlots = false;
				return;
			}
			
			//first set the type
			EList classifiers = instanceSample.getClassifier();
			if(classifiers != null && classifiers.size()>1){
				LoggingUtil.trace(this, "updateClassifierProperties", "Multiple superclassifier not supported.");
			}else if(classifiers.isEmpty()){
				LoggingUtil.trace(this, "updateClassifierProperties", "No classifier was found.");
				return;
			}else{
				Classifier classifier = (Classifier)instanceSample.getClassifier().get(0);
				Classifier workingClassifier = (Classifier)BOMUtil.getCategoryValueTemplate(getProjectName(), classifier, getImportSession().getContext());
				if(workingClassifier != null){
					UpdateInstanceSpecificationBOMCmd updateInstanceCmd = new UpdateInstanceSpecificationBOMCmd(workingCopy);
					if (workingCopy.getName() == null) {				
						updateInstanceCmd.setName(instanceSample.getName());
					}
					updateInstanceCmd.addClassifier(workingClassifier);
					if(updateInstanceCmd.canExecute()){
						try{
							updateInstanceCmd.execute();
						}catch(RuntimeException ex){
							LoggingUtil.logError(IeMessageKeys.INSTANCE_SUPER_CLASSIFIER_EXCEPTION, new String[]{instanceSample.getName()}, ex );
						}
					}else{
						LoggingUtil.logError(IeMessageKeys.INSTANCE_SUPER_CLASSIFIER_EXCEPTION, new String[]{instanceSample.getName()}, null);
					}
				}else{
					LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Classifier",classifier.getName()}, null);
				}
			}
		}
		LoggingUtil.traceExit(this, "updateInstanceClassifier");
	}
	

}
