package com.ibm.btools.blm.ie.imprt.rule.organizationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.gef.RemoveDomainViewObjectPEBaseCmd;
import com.ibm.btools.blm.compoundcommand.orgChart.tree.CreateNodeCommand;
import com.ibm.btools.blm.compoundcommand.resource.CompoundCommandMessageKeys;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationHierarchyStructureDefinitionNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationOrganizationCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationOrganizationCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddConstraintToNamedElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.organizationstructures.AddNodeToTreeBOMCmd;
import com.ibm.btools.bom.command.organizationstructures.UpdateTreeBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.organizationstructures.Location;
import com.ibm.btools.bom.model.organizationstructures.Node;
import com.ibm.btools.bom.model.organizationstructures.NodeType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.organizationstructures.Tree;
import com.ibm.btools.bom.model.organizationstructures.TreeStructure;
import com.ibm.btools.bom.model.resources.BulkResource;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.bom.model.resources.SkillProfile;
import com.ibm.btools.cef.gef.emf.command.AddCommonLinkModelCommand;
import com.ibm.btools.cef.gef.emf.command.AddModelPropertyCommand;
import com.ibm.btools.cef.gef.emf.command.UpdateCommonNodeModelCommand;
import com.ibm.btools.cef.gef.emf.command.UpdateVisualModelDocumentCommand;
import com.ibm.btools.cef.model.CommonModel;
import com.ibm.btools.cef.model.CommonNodeModel;
import com.ibm.btools.cef.model.Content;
import com.ibm.btools.cef.model.VisualModelDocument;
import com.ibm.btools.cef.registry.RegistryManager;
import com.ibm.btools.expression.bom.command.AddIsKindOfMetamodelTypeExpressionToStructuredOpaqueExpressionBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToConstraintBEXCmd;
import com.ibm.btools.expression.bom.model.IsKindOfMetamodelTypeExpression;
import com.ibm.btools.expression.bom.model.MetamodelType;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.util.resource.UtilResourceBundleSingleton;


public class UpdateTreeRule extends AbstractImportRule {

	static final String COPYRIGHT = " Copyright IBM Corporation 2004, 2009." ;
	private VisualModelDocument vmd;
	private Tree tree;
	private Tree workingCopy;
	private String VIRTUAL_ROOT_NODE_NAME = "#VirtualRootNode#";
	private String defaultStructName=UtilResourceBundleSingleton.INSTANCE.getMessage(CompoundCommandMessageKeys.class, CompoundCommandMessageKeys.CCB9603I);
	private boolean isDefaultTree=false;
	private TreeStructure struct=null;

	/* (non-Javadoc)
	 * @see com.ibm.btools.blm.ie.imprt.rule.IImportRule#setSource(java.lang.Object)
	 */
	public void setSource(Object tree) {
		this.tree = (Tree) tree;			
	}
	
	public void setWorkingCopy(final Object workingCopy) {
		if(workingCopy instanceof Tree){
			this.workingCopy = (Tree) workingCopy;			
		}
		else if(workingCopy instanceof VisualModelDocument){
			this.vmd = (VisualModelDocument) workingCopy;
			checkDomainContent();
			this.workingCopy=(Tree) vmd.getDomainContent().get(0);			
		}
	}
	private String getDescriptorId(PackageableElement element){
		if(element instanceof Location){
			return "com.ibm.btools.blm.gef.treeeditor.Location";
		}
		else if(element instanceof OrganizationUnit){
			return "com.ibm.btools.blm.gef.treeeditor.OrganizationUnit";
		}
		else if(element instanceof BulkResource){
			return "com.ibm.btools.blm.gef.treeeditor.BulkResource";
		}
		else if(element instanceof IndividualResource){
			return "com.ibm.btools.blm.gef.treeeditor.IndividualResource";
		}
		else if(element instanceof Role){
			return "com.ibm.btools.blm.gef.treeeditor.Role";
		}
		else if(element instanceof SkillProfile){
			return "com.ibm.btools.blm.gef.treeeditor.SkillProfile";
		}
		else if(element instanceof InstanceSpecification){
			InstanceSpecification instance=(InstanceSpecification)element;
			if(!instance.getClassifier().isEmpty()){
				if(instance.getClassifier().get(0) instanceof Class)
				return "com.ibm.btools.blm.gef.treeeditor.Class";
			}
		}
		return "com.ibm.btools.blm.gef.treeeditor.Category";
	}
	
	private void checkDomainContent(){
	  if(vmd.getCurrentContent()!=null){
		  Content content=vmd.getCurrentContent();					
		  if(!content.getContentChildren().isEmpty()){
			  CommonModel commonModel=(CommonModel) content.getContentChildren().get(0);
			  String root="com.ibm.btools.blm.gef.treeeditor.root";
			  if(root.equals(commonModel.getDescriptor().getId())){
				  CommonNodeModel container=(CommonNodeModel) commonModel;
				  workingCopy= (Tree) container.getDomainContent().get(0);
				  UpdateCommonNodeModelCommand nodeModelCommand= new UpdateCommonNodeModelCommand(container);
				  nodeModelCommand.removeDomainContent(workingCopy);
				  if(nodeModelCommand.canExecute()){
					  nodeModelCommand.execute();
				  }
				  RemoveDomainViewObjectPEBaseCmd removeNodeCmd= new RemoveDomainViewObjectPEBaseCmd();
				  removeNodeCmd.setViewObject(container);
				  if(removeNodeCmd.canExecute()){
					  removeNodeCmd.execute();
				  }				
			  }
		  }
	  }
	  if(vmd.getDomainContent().isEmpty()){
		  UpdateVisualModelDocumentCommand UpdaVMDCmd= new UpdateVisualModelDocumentCommand(vmd);
		  UpdaVMDCmd.addDomainContent(workingCopy);
		  if(UpdaVMDCmd.canExecute()){
			  UpdaVMDCmd.execute();
		  }				
	  }
   }
   
   private NodeType getNodeType(NodeType r_nodeType,NodeType c_root){
   	if(c_root.getName().equals(r_nodeType.getName())){
   		return c_root;
   	}
   	for (Iterator iter = c_root.getChildType().iterator(); iter.hasNext();) {
		NodeType nextRoot = (NodeType) iter.next();
		NodeType nodeType =getNodeType(r_nodeType,nextRoot);
		if(nodeType!=null){
			return nodeType;		
		}
	}
	return null;
   }
	
	private void createNode(Node r_child, Element c_parent,CommonNodeModel c_ParentView,int x,int y){
		Content currentContent = vmd.getCurrentContent();
		Rectangle childConstraint= new Rectangle(x,y,150,55);
		Element newDomainChild=null;
		CommonNodeModel newViewChild=null;
		String descriptorID=getDescriptorId(r_child.getNodeElement());
		NodeType nodeType=null;
		if(isDefaultTree){
			nodeType=struct.getRootType();
		}else{
			nodeType=getNodeType(r_child.getNodeType(),struct.getRootType());
		}
		CreateNodeCommand command = new CreateNodeCommand();
		command.setDomainNodeParent(c_parent);
		command.setDefaultName(r_child.getName());
		command.setDescriptorID(descriptorID);
		command.setViewParent(currentContent);
		command.setLayoutID(currentContent.getLayoutId());
		command.setChildDimension(childConstraint.getSize());
		command.setX(new Integer(childConstraint.x));
		command.setY(new Integer(childConstraint.y));
		command.setName(r_child.getName());
		command.setNodeType(nodeType);
		PackageableElement nodeElement = (PackageableElement) BOMUtil.loadRootObject((AbstractLibraryChildNode) getImportSession().getContext().get(r_child.getNodeElement()));
		command.setNodeElement(nodeElement);
		if(command.canExecute()){
			command.execute();
		}
		if("com.ibm.btools.blm.gef.treeeditor.Category".equals(descriptorID) && !isDefaultTree){
			AddCategoryConstraint(nodeType)	;
		}
		if("com.ibm.btools.blm.gef.treeeditor.Role".equals(descriptorID) && !isDefaultTree){
			AddRoleConstraint(nodeType)	;
		}

		newDomainChild=(Element) command.getNewDomainModel();
		newViewChild  =(CommonNodeModel) command.getNewViewModel();
		if(c_parent instanceof Node && !VIRTUAL_ROOT_NODE_NAME.equals(((Node)c_parent).getName())){
			AddCommonLinkModelCommand linkCmd = new AddCommonLinkModelCommand(currentContent);
			linkCmd.setSource(c_ParentView);
			linkCmd.setTarget(newViewChild);
			String linkDescriptorId="com.ibm.btools.blm.gef.treeeditor.Connection";
			linkCmd.setDescriptor(RegistryManager.instance().getRegistry("com.ibm.btools.blm.gef.treeeditor").getDescriptor(linkDescriptorId));
			linkCmd.setSourceAnchorPoint(4);
			linkCmd.setTargetAnchorPoint(1);
			if(linkCmd.canExecute()){
				linkCmd.execute();
			}	
		}
		int index=0;
		for (Iterator iter = r_child.getChild().iterator(); iter.hasNext();) {
			Node child = (Node) iter.next();
			int nextX=x;
			if(index!=0 && index%2==0){
				nextX=x-(200*(index/2));
			}
			if(index!=0 && index%2!=0){
				nextX=x+(200*((index+1)/2));
			}
			createNode(child,newDomainChild,newViewChild,nextX,y+95);	
			index++;
		}
	}
  
	public void invoke() {
		isDefaultTree=false;
		struct=null;
		AbstractLibraryChildNode node=(AbstractLibraryChildNode) getImportSession().getContext().get(tree.getStructure());
		struct = (TreeStructure) BOMUtil.loadRootObject(node);// master
		if(struct==null){
			struct=getDefaultTreeStructure();
		}
		if(struct==null) return;
		Node root=tree.getRoot();
		UpdateTreeBOMCmd cmd= new UpdateTreeBOMCmd(workingCopy);
		cmd.setStructure(struct);
		if(cmd.canExecute()){
			cmd.execute();
		}								
		if(root==null){
			return;
		}
		// addd virtual root,
		if(!VIRTUAL_ROOT_NODE_NAME.equals(root.getName())){
			AddNodeToTreeBOMCmd command = new AddNodeToTreeBOMCmd(workingCopy);
			command.setName(VIRTUAL_ROOT_NODE_NAME);
			command.setNodeType(struct.getRootType());
			if(command.canExecute()){
				command.execute();			
			}	
			Node virtualRoot = (Node) command.getObject();
			createNode(root,virtualRoot,null,200,80);
		}
		else{
			createNode(root,workingCopy,null,200,80);
			
		}
		updateComments();
		AddModelPropertyCommand addModelPropertyCommand= new AddModelPropertyCommand(vmd);
		addModelPropertyCommand.setName("doLayout");
		addModelPropertyCommand.setValue(Boolean.TRUE);
		if(addModelPropertyCommand.canExecute()){
			addModelPropertyCommand.execute();			
		}
		if(!isDefaultTree){
			String uri = NavigatorUtil.getURI(node);// save the tree structure to update the constrains to the master copy
			if (uri != null) {
				BOMUtil.saveRootObject(uri, getProjectName());							
			}
		}


	}
	
	private void updateComments(){
		EList comments = tree.getOwnedComment();
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			while(it.hasNext()){
				Comment comment = (Comment)it.next();
				AddCommentToElementBOMCmd addCmd = new AddCommentToElementBOMCmd(workingCopy);
				if(addCmd.canExecute()){
					addCmd.execute();
					Comment workingComment = (Comment)addCmd.getObject();
					UpdateCommentBOMCmd updateComment = new UpdateCommentBOMCmd(workingComment);
					updateComment.setBody(comment.getBody());
					if(updateComment.canExecute()){
						updateComment.execute();
					}else{
						//log an error, comment body can not be filled
					}
				}else{
					//log an error, the comment could not be added
				}
			}
		}
	}

	private IsKindOfMetamodelTypeExpression createOrGetIsKindExpression(NodeType nodeType){
		if(nodeType.getOwnedConstraint().isEmpty()){
			AddConstraintToNamedElementBOMCmd addConstraintToNamedElementBOMCmd = new AddConstraintToNamedElementBOMCmd(nodeType);
			if(addConstraintToNamedElementBOMCmd.canExecute()){
				addConstraintToNamedElementBOMCmd.execute();
			}
			Constraint constraint=(Constraint)nodeType.getOwnedConstraint().get(0);
			AddStructuredOpaqueExpressionToConstraintBEXCmd opaqueExpressionToConstraintBEXCmd = new AddStructuredOpaqueExpressionToConstraintBEXCmd(constraint);
			if(opaqueExpressionToConstraintBEXCmd.canExecute()){
				opaqueExpressionToConstraintBEXCmd.execute();
			}
			StructuredOpaqueExpression structuredOpaqueExpression =(StructuredOpaqueExpression)opaqueExpressionToConstraintBEXCmd.getObject();
			AddIsKindOfMetamodelTypeExpressionToStructuredOpaqueExpressionBEXCmd  isKindOfCmd = new AddIsKindOfMetamodelTypeExpressionToStructuredOpaqueExpressionBEXCmd(structuredOpaqueExpression);
			if(isKindOfCmd.canExecute()){
				isKindOfCmd.execute();
			}
			IsKindOfMetamodelTypeExpression isKindOfExpression =(IsKindOfMetamodelTypeExpression)isKindOfCmd.getObject();
			
			return isKindOfExpression;
		}
		else{
			Constraint constraint=(Constraint)nodeType.getOwnedConstraint().get(0);
			StructuredOpaqueExpression structuredOpaqueExpression =(StructuredOpaqueExpression)constraint.getSpecification();
			IsKindOfMetamodelTypeExpression isKindOfExpression=(IsKindOfMetamodelTypeExpression)structuredOpaqueExpression.getExpression();
			return isKindOfExpression;
		}
		
	}

	
	public  void AddRoleConstraint(NodeType nodeType){
		createOrGetIsKindExpression(nodeType).setType(MetamodelType.ROLE_LITERAL);
	}

	public  void AddSkillProfileConstraint(NodeType nodeType){
		createOrGetIsKindExpression(nodeType).setType(MetamodelType.SKILL_PROFILE_LITERAL);
	}
	
	public  void AddCategoryConstraint(NodeType nodeType){
		createOrGetIsKindExpression(nodeType).setType(MetamodelType.BOOLEAN_LITERAL);
	}

	public TreeStructure getDefaultTreeStructure(){
		NavigationProjectNode projectNode=NavigatorUtil.getProjectNode(getProjectName());
		if(projectNode==null) return null;
		Vector hierarchyStructureDefinitionNodesVector = new Vector();
		NavigationOrganizationCatalogsNode organizationCatalogsNode = projectNode.getLibraryNode().getOrganizationCatalogsNode();			
		retrieveHierarchyStructureDefinitionsFromOrganizationCatalogs(hierarchyStructureDefinitionNodesVector, organizationCatalogsNode, new Vector() );
		if(hierarchyStructureDefinitionNodesVector.size()==1 && hierarchyStructureDefinitionNodesVector.get(0) instanceof NavigationHierarchyStructureDefinitionNode){
			NavigationHierarchyStructureDefinitionNode navHierStruDefNode=(NavigationHierarchyStructureDefinitionNode)hierarchyStructureDefinitionNodesVector.get(0);
			String navHierStruDefNodeURI = (String) navHierStruDefNode.getEntityReferences().iterator().next();
			String projectName = navHierStruDefNode.getProjectNode().getLabel() ;
			String projectPath = FileMGR.getProjectPath(projectName);
			TreeStructure treeStructure =(TreeStructure)ResourceMGR.getResourceManger().getRootObjects(projectName,projectPath,navHierStruDefNodeURI).get(0);
			isDefaultTree=true;
			return treeStructure;
		}
		return null;
	}
	
	protected void 	retrieveHierarchyStructureDefinitionsFromOrganizationCatalogs(Vector hierarchyStructureNodesVector, 
								NavigationOrganizationCatalogsNode navigationOrganizationCatalogsNode,
								Vector organizationCatalogHierarchy) {
											  		
		Iterator organizationCatalogNodesIterator = navigationOrganizationCatalogsNode.getOrganizationCatalogNodes().iterator();
		while (organizationCatalogNodesIterator.hasNext()) {
				retrieveHierarchyStructureDefinitionsFromOrganizationCatalog(hierarchyStructureNodesVector,
											(NavigationOrganizationCatalogNode) organizationCatalogNodesIterator.next(), 
											organizationCatalogHierarchy );
		}
	}

	protected void retrieveHierarchyStructureDefinitionsFromOrganizationCatalog(Vector hierarchyStructureNodesVector,
													NavigationOrganizationCatalogNode organizationCatalogNode,
													Vector organizationCatalogHierarchy) {
		
		Vector organizationCatalogHierarchyForThisLevel = (Vector) organizationCatalogHierarchy.clone();
		organizationCatalogHierarchyForThisLevel.add( organizationCatalogNode );
		if (organizationCatalogNode.getHierarchyStructureDefinitionsNode() != null){ 
			Iterator hierarchyStructureNodes = organizationCatalogNode.getHierarchyStructureDefinitionsNode().getHierarchyStructureDefinitionNodes().iterator();
			while (hierarchyStructureNodes.hasNext()) {
				NavigationHierarchyStructureDefinitionNode tsNode=(NavigationHierarchyStructureDefinitionNode) hierarchyStructureNodes.next();
					if(defaultStructName.equals(tsNode.getLabel())){
						hierarchyStructureNodesVector.add( tsNode );
					}
				}
		}
		/* Now check for any Data Catalogs contained within this Data Catalog */
		Iterator children = organizationCatalogNode.getOrganizationCatalogNodeChildren().iterator();
		while (children.hasNext()) {
			Object obj = children.next();
			if (obj instanceof NavigationOrganizationCatalogsNode) {
				retrieveHierarchyStructureDefinitionsFromOrganizationCatalogs(hierarchyStructureNodesVector, (NavigationOrganizationCatalogsNode) obj, organizationCatalogHierarchyForThisLevel);
				}
			else
			if(obj instanceof NavigationOrganizationCatalogNode) {
				retrieveHierarchyStructureDefinitionsFromOrganizationCatalog(hierarchyStructureNodesVector, (NavigationOrganizationCatalogNode) obj, organizationCatalogHierarchyForThisLevel);
			}
		}
	}
}
