package com.ibm.btools.blm.ie.imprt.rule.organizationModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;

import com.ibm.btools.blm.compoundcommand.gef.RemoveDomainViewObjectPEBaseCmd;
import com.ibm.btools.blm.compoundcommand.orgChart.treestruct.CreateNodeTypeCommand;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.bom.command.artifacts.AddCommentToElementBOMCmd;
import com.ibm.btools.bom.command.artifacts.UpdateCommentBOMCmd;
import com.ibm.btools.bom.command.organizationstructures.AddNodeTypeToTreeStructureBOMCmd;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.NodeType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.organizationstructures.TreeStructure;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.cef.gef.emf.command.AddCommonLinkModelCommand;
import com.ibm.btools.cef.gef.emf.command.AddModelPropertyCommand;
import com.ibm.btools.cef.gef.emf.command.UpdateCommonNodeModelCommand;
import com.ibm.btools.cef.gef.emf.command.UpdateVisualModelDocumentCommand;
import com.ibm.btools.cef.model.CommonModel;
import com.ibm.btools.cef.model.CommonNodeModel;
import com.ibm.btools.cef.model.Content;
import com.ibm.btools.cef.model.VisualModelDocument;
import com.ibm.btools.cef.registry.RegistryManager;


public class UpdateTreeStructureRule extends AbstractImportRule {

	static final String COPYRIGHT = " Copyright IBM Corporation 2004, 2009." ;
	private VisualModelDocument vmd;
	private TreeStructure treeStruct;
	private TreeStructure workingCopy;
	private String VIRTUAL_ROOT_NODE_NAME = "#VirtualRootNode#";

	/* (non-Javadoc)
	 * @see com.ibm.btools.blm.ie.imprt.rule.IImportRule#setSource(java.lang.Object)
	 */
	public void setSource(Object treeStruct) {
		this.treeStruct = (TreeStructure) treeStruct;			
	}

	public void setWorkingCopy(final Object workingCopy) {
		if(workingCopy instanceof TreeStructure){
			this.workingCopy = (TreeStructure) workingCopy;			
		}
		else if(workingCopy instanceof VisualModelDocument){
			this.vmd = (VisualModelDocument) workingCopy;			
			checkDomainContent();
			this.workingCopy=(TreeStructure) vmd.getDomainContent().get(0);			
		}
	}
	private void checkDomainContent(){
	   if(vmd.getCurrentContent()!=null){
		   Content content=vmd.getCurrentContent();					
		   if(!content.getContentChildren().isEmpty()){
			   CommonModel commonModel=(CommonModel) content.getContentChildren().get(0);
			   String root="com.ibm.btools.blm.gef.treestructeditor.root";
			   if(root.equals(commonModel.getDescriptor().getId())){
				   CommonNodeModel container=(CommonNodeModel) commonModel;
				   workingCopy= (TreeStructure) container.getDomainContent().get(0);
				   UpdateCommonNodeModelCommand nodeModelCommand= new UpdateCommonNodeModelCommand(container);
				   nodeModelCommand.removeDomainContent(workingCopy);
				   if(nodeModelCommand.canExecute()){
					   nodeModelCommand.execute();
				   }
				   RemoveDomainViewObjectPEBaseCmd removeNodeCmd= new RemoveDomainViewObjectPEBaseCmd();
				   removeNodeCmd.setViewObject(container);
				   if(removeNodeCmd.canExecute()){
					   removeNodeCmd.execute();
				   }				
			   }
		   }
	   }
	   if(vmd.getDomainContent().isEmpty()){
		   UpdateVisualModelDocumentCommand UpdaVMDCmd= new UpdateVisualModelDocumentCommand(vmd);
		   UpdaVMDCmd.addDomainContent(workingCopy);
		   if(UpdaVMDCmd.canExecute()){
			   UpdaVMDCmd.execute();
		   }				
	   }
	}
	private String getDescriptorId(Type type){
		if(type instanceof LocationType){
			return "com.ibm.btools.blm.gef.treestructeditor.Location";
		}
		else if(type instanceof OrganizationUnitType){
			return "com.ibm.btools.blm.gef.treestructeditor.OrganizationUnit";
		}
		else if(type instanceof Class){
			return "com.ibm.btools.blm.gef.treestructeditor.Class";
		}
		else if(type instanceof BulkResourceType){
			return "com.ibm.btools.blm.gef.treestructeditor.BulkResource";
		}
		else if(type instanceof IndividualResourceType){
			return "com.ibm.btools.blm.gef.treestructeditor.IndividualResource";
		}
		return "com.ibm.btools.blm.gef.treestructeditor.Category";
	}
	
	private void createNodeType(NodeType r_child, Element c_parent,CommonNodeModel c_ParentView,int x,int y){
		Content currentContent = vmd.getCurrentContent();
		Rectangle childConstraint= new Rectangle(x,y,150,55);
		Element newDomainChild=null;
		CommonNodeModel newViewChild=null;
		CreateNodeTypeCommand command = new CreateNodeTypeCommand();
		command.setDomainNodeParent(c_parent);
		command.setDefaultName(r_child.getName());;
		command.setDescriptorID(getDescriptorId(r_child.getType()));
		command.setViewParent(currentContent);
		command.setLayoutID(currentContent.getLayoutId());
		command.setChildDimension(childConstraint.getSize());
		command.setX(new Integer(childConstraint.x));
		command.setY(new Integer(childConstraint.y));
		command.setName(r_child.getName());
		command.setRecursive(r_child.getIsRecursive().booleanValue());
		Type masterType =(Type) BOMUtil.getType(getProjectName(), r_child.getType(), getImportSession().getContext());
		command.setType(masterType);
		if(command.canExecute()){
			command.execute();
		}	
		newDomainChild=(Element) command.getNewDomainModel();
		newViewChild  =(CommonNodeModel) command.getNewViewModel();
		if(c_parent instanceof NodeType && !VIRTUAL_ROOT_NODE_NAME.equals(((NodeType)c_parent).getName())){
			AddCommonLinkModelCommand linkCmd = new AddCommonLinkModelCommand(currentContent);
			linkCmd.setSource(c_ParentView);
			linkCmd.setTarget(newViewChild);
			String linkDescriptorId="com.ibm.btools.blm.gef.treestructeditor.Connection";
			linkCmd.setDescriptor(RegistryManager.instance().getRegistry("com.ibm.btools.blm.gef.treestructeditor").getDescriptor(linkDescriptorId));
			linkCmd.setSourceAnchorPoint(4);
			linkCmd.setTargetAnchorPoint(1);
			if(linkCmd.canExecute()){
				linkCmd.execute();
			}	
		}
		int index=0;
		for (Iterator iter = r_child.getChildType().iterator(); iter.hasNext();) {
			NodeType child = (NodeType) iter.next();
			int nextX=x;
			if(index!=0 && index%2==0){
				nextX=x-(200*(index/2));
			}
			if(index!=0 && index%2!=0){
				nextX=x+(200*((index+1)/2));
			}
			createNodeType(child,newDomainChild,newViewChild,nextX,y+95);	
			index++;
		}
	}
  
	
	public void invoke() {
		NodeType root=treeStruct.getRootType();
		if(root==null){
			return;
		}
		// addd virtual root,
		if(!VIRTUAL_ROOT_NODE_NAME.equals(root.getName())){
			AddNodeTypeToTreeStructureBOMCmd command = new AddNodeTypeToTreeStructureBOMCmd(workingCopy);
			command.setName(VIRTUAL_ROOT_NODE_NAME);
			if(command.canExecute()){
				command.execute();			
			}	
			NodeType virtualRoot = (NodeType) command.getObject();
			createNodeType(root,virtualRoot,null,200,80);
		}
		else{
			createNodeType(root,workingCopy,null,200,80);
			
		}
		
		updateComments();
		AddModelPropertyCommand addModelPropertyCommand= new AddModelPropertyCommand(vmd);
		addModelPropertyCommand.setName("doLayout");
		addModelPropertyCommand.setValue(Boolean.TRUE);
		if(addModelPropertyCommand.canExecute()){
			addModelPropertyCommand.execute();			
		}
	}

	private void updateComments(){
		EList comments = treeStruct.getOwnedComment();
		if(comments != null && !comments.isEmpty()){
			Iterator it = comments.iterator();
			while(it.hasNext()){
				Comment comment = (Comment)it.next();
				AddCommentToElementBOMCmd addCmd = new AddCommentToElementBOMCmd(workingCopy);
				if(addCmd.canExecute()){
					addCmd.execute();
					Comment workingComment = (Comment)addCmd.getObject();
					UpdateCommentBOMCmd updateComment = new UpdateCommentBOMCmd(workingComment);
					updateComment.setBody(comment.getBody());
					if(updateComment.canExecute()){
						updateComment.execute();
					}else{
						//log an error, comment body can not be filled
					}
				}else{
					//log an error, the comment could not be added
				}
			}
		}
	}
}
