package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.expressionModel.UpdateExpressionRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.processes.activities.UpdateActivityBOMCmd;
import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.cef.model.CommonContainerModel;
import com.ibm.btools.cef.model.VisualModelDocument;
import com.ibm.btools.expression.bom.command.AddPostconditionToActivityBEXCmd;
import com.ibm.btools.expression.bom.command.AddPreconditionToActivityBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;

/**
 * Update the Activity Expression
 */
public class UpdateActivityExpressionRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected Activity activity;
	protected VisualModelDocument workingCopy;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setSource(final Object activity) {
		this.activity = (Activity) activity;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (VisualModelDocument) workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (activity == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROCESS_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		Activity workingCopyActivity = getActivity();
		if (workingCopyActivity == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);			
			LoggingUtil.traceExit(this, "invoke");			
			return;	
		}

		LoggingUtil.trace(this, "invoke", "Create the context for process");
		
		createExpression(workingCopyActivity);		
				
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Create Activity expression
	 * @param process
	 */
	protected void createExpression(Activity workingCopyActivity) {
		LoggingUtil.traceEntry(this, "createExpression");	
		UpdateActivityBOMCmd cmd = new UpdateActivityBOMCmd(workingCopyActivity);		
		boolean updateAttributes = false;
		
		// Attribute precondition
		Iterator iterator = activity.getPrecondition().iterator();
		Constraint constraint = null;
		Constraint newConstraint = null;
		ValueSpecification spec = null;
		AddPreconditionToActivityBEXCmd preCmd = null;
		while (iterator.hasNext()) {
			constraint = (Constraint) iterator.next();
			spec = constraint.getSpecification();
			if (spec instanceof StructuredOpaqueExpression) {
				preCmd = new AddPreconditionToActivityBEXCmd();
				preCmd.setActivity(workingCopyActivity);
				preCmd.setExpressionName(spec.getName());
				if (preCmd.canExecute()) {
					try {
						preCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {		
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);									
				}
				newConstraint = preCmd.getConstraint();
				
				// CallExpression
				UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
				expressionRule.setImportSession(getImportSession());
				expressionRule.setProjectName(getProjectName());
				expressionRule.setSource(spec);
				expressionRule.setWorkingCopy(newConstraint.getSpecification());
				expressionRule.setProcess(activity);
				expressionRule.invoke();	
			}
		}
		
		// Attribute postcondition		
		iterator = activity.getPostcondition().iterator();
		AddPostconditionToActivityBEXCmd postCmd = null;
		while (iterator.hasNext()) {
			constraint = (Constraint) iterator.next();
			spec = constraint.getSpecification();
			if (spec instanceof StructuredOpaqueExpression) {
				postCmd = new AddPostconditionToActivityBEXCmd();
				postCmd.setActivity(workingCopyActivity);
				postCmd.setExpressionName(constraint.getName());
				if (postCmd.canExecute()) {
					try {
						postCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {		
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);									
				}
				newConstraint = postCmd.getConstraint();
				
				// CallExpression
				UpdateExpressionRule expressionRule =  new UpdateExpressionRule();
				expressionRule.setImportSession(getImportSession());
				expressionRule.setProjectName(getProjectName());
				expressionRule.setSource(spec);
				expressionRule.setWorkingCopy(newConstraint.getSpecification());
				expressionRule.setProcess(activity);
				expressionRule.invoke();	
			}
		}
		
		updateImplementationExpression(workingCopyActivity);
		LoggingUtil.traceExit(this, "createExpression");	
	}
	
	
	
	/**
	 * Update Implementation
	 */
	protected void updateImplementationExpression(Activity workingCopyActivity) {
		
		StructuredActivityNode node = activity.getImplementation();
		UpdateStructuredActivityNodeExpressionRule rule = new UpdateStructuredActivityNodeExpressionRule();
		rule.setWorkingCopy(workingCopyActivity.getImplementation());
		rule.setImportSession(getImportSession());
		rule.setProjectName(getProjectName());
		rule.setSource(activity.getImplementation());
		rule.setProcess(activity);
		rule.invoke();		
	}
	
	/**
	 * Get Activity from VisualDocument
	 * @return
	 */
	protected Activity getActivity() {
		LoggingUtil.traceEntry(this, "getActivity");	
		if (workingCopy == null) {
			LoggingUtil.traceExit(this, "getActivity");
			return null;
		}
		Object topContainer = null;
		Object domainObject = null;
		
		try {			
			topContainer = (Object) workingCopy.getRootContent().getContentChildren().get(0);
			if (topContainer instanceof CommonContainerModel) {
				domainObject = ((CommonContainerModel)topContainer).getDomainContent().get(0);
				if (domainObject instanceof Activity) {
					LoggingUtil.traceExit(this, "getActivity");
					return (Activity) domainObject;
				}
			}
		} catch (RuntimeException e) {
			LoggingUtil.traceExit(this, "getActivity");
			return null;
		}
		
		LoggingUtil.traceExit(this, "getActivity");
		return null;
	}
}
