package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionComputedValueToRepositoryBEXCmd;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionDefaultValueToRepositoryBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;


/**
 * Update the Datastore's value
 */
public class UpdateDatastoreValueRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	private Datastore datastore;
	private Datastore workingCopy;

	/**
	 * Set Original datastore
	 * @param datastore
	 */
	public void setSource(final Object datastore) {
		this.datastore = (Datastore) datastore;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (Datastore) workingCopy;
	}	
	
	/** 
	 * Invokes the rule execution.
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");	
		updateModel();
		LoggingUtil.traceExit(this, "invoke");
	}	
	
	/**
	 * Update the datastore
	 */
	private void updateModel() {
		LoggingUtil.traceEntry(this, "updateModel");	
		
		// Attribute computedValue
		// Attribute defaultValue
		Iterator conmputedIterator = datastore.getComputedValue().iterator();
		Iterator defaultIterator = datastore.getDefaultValue().iterator();
		ValueSpecification value = null;
		AddUpdateObjectCommand cmd = null;
		InstanceSpecification spec = null;
		
		while(conmputedIterator.hasNext()) {
			value = (ValueSpecification) conmputedIterator.next();
			
			if (value instanceof LiteralNull) {
				cmd = new AddLiteralNullComputedValueToRepositoryBOMCmd((Repository)workingCopy);				
			} else if (value instanceof LiteralSpecification) {
				cmd = new AddLiteralStringComputedValueToRepositoryBOMCmd((Repository)workingCopy);				
			} else if (value instanceof StructuredOpaqueExpression) {
				// [Expression] Attrbibute constant value
				//d97575 - jie: add creation of structedOpaqueExpression
				cmd = new AddStructuredOpaqueExpressionComputedValueToRepositoryBEXCmd((Repository)workingCopy);
			}else if (value instanceof OpaqueExpression) {
				// [Expression] Attrbibute constant value
				cmd = new AddOpaqueExpressionComputedValueToRepositoryBOMCmd((Repository)workingCopy);
			}else if (value instanceof InstanceValue) {
				cmd = new AddInstanceValueComputedValueToRepositoryBOMCmd((Repository)workingCopy);	
			}
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
			
			// After creation
			Object newDomainModel = cmd.getObject();
			if (newDomainModel != null) {
				UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
				rule.setSource(value);
				rule.setWorkingCopy(newDomainModel);
				rule.setProjectName(getProjectName());
				rule.setImportSession(getImportSession());
				rule.invoke();
			}		
		}
		
		while(defaultIterator.hasNext()) {
			value = (ValueSpecification) defaultIterator.next();
			
			if (value instanceof LiteralNull) {
				cmd = new AddLiteralNullDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
			} else if (value instanceof LiteralSpecification) {
				cmd = new AddLiteralStringDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
			} else if (value instanceof OpaqueExpression) {
				// [Expression] Attrbibute constant value
				if (value instanceof StructuredOpaqueExpression) {
					cmd = new AddStructuredOpaqueExpressionDefaultValueToRepositoryBEXCmd((Repository)workingCopy);
				} else {				
					cmd = new AddOpaqueExpressionDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
				}
			} else if (value instanceof InstanceValue) {
				cmd = new AddInstanceValueDefaultValueToRepositoryBOMCmd((Repository)workingCopy);	
			}
			
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
			
			// After creation
			Object newDomainModel = cmd.getObject();
			if (newDomainModel != null) {
				UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
				rule.setSource(value);
				rule.setWorkingCopy(newDomainModel);
				rule.setProjectName(getProjectName());
				rule.setImportSession(getImportSession());
				rule.invoke();
			}
		}
		LoggingUtil.traceExit(this, "updateModel");		
	}
}
