package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.conn.add.AddActivityConnPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ConnectableNode;
import com.ibm.btools.bom.model.processes.activities.ControlFlow;
import com.ibm.btools.bom.model.processes.activities.ObjectFlow;


/**
 * Update the Edge
 */
public class UpdateEdgeRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private ActivityEdge edge;
	private Object workingCopy;
	private Object process;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}	
	
	/**
	 * Set Original edge
	 * @param edge
	 */
	public void setSource(final Object edge) {
		this.edge = (ActivityEdge) edge;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (edge == null) {
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();

		// Attribute aspect - not handled
		// Attribute name - handled
		// Attribute ownedConstraint - not handled
		// Attribute source - handled by command
		// Attribute target - handled by command
		// Attribute uid - generated
		// Attribute visibility - not handle 

		AddActivityConnPeCmd cmd = null;
		if (edge instanceof ObjectFlow) {
			cmd = cmdFactory.buildAddObjectConnPeCmd((EObject) workingCopy);
		} else if (edge instanceof ControlFlow) {
			cmd = cmdFactory.buildAddControlConnPeCmd((EObject) workingCopy);			
		}
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);		
		cmd.setName(edge.getName());

		ConnectableNode source = edge.getSource();
		ConnectableNode target = edge.getTarget();
		cmd.setViewTarget( (EObject) context.get(target));
		cmd.setViewSource( (EObject) context.get(source));
		
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
			}
		} else {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
		}		
		
		// Attribute ownedComment		
		EObject domainModel = cmd.getNewDomainModel();
		if (domainModel != null) {
			BOMUtil.updateDescription((Element)domainModel, BOMUtil.getDescription((Element)edge));			
			//	20050418 - create the UID hashmap for ADF Import
			if (edge.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(edge.getUid(), ((ActivityEdge)domainModel).getUid());
			}
			// end of 20050418 - create the UID hashmap for ADF Import
		}
		LoggingUtil.traceExit(this, "invoke");
	}
	
	/**
	 * Return the view object by node
	 * @param source
	 * @return
	 */
	private EObject getViewTarget(final ConnectableNode node) {
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);

		List list = new ArrayList();	
		Object object = null; 		
		EObject container = node.eContainer();
		
		if (container instanceof Merge) {
			
		} else if (container instanceof Decision || container instanceof Join || container instanceof Fork) {
			return getBranchFromViewAction((ControlAction) container);
		} else {
			return (EObject) context.get(node);
		}
		
		return null;
	}
	
	/**
	 * Return the view object by node
	 * @param source
	 * @return
	 */
	private EObject getViewSource(final ConnectableNode node) {
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		 		
		EObject container = node.eContainer();
		if (container instanceof Merge) {
			
		} else if (container instanceof Decision) {		
		} else if (container instanceof Join) {
			
		} else if (container instanceof Fork) {
		} else {
			return (EObject) context.get(node);
		}
		
		return null;
	}	
	
	/**
	 * Return the branch view model of this action
	 * @param model
	 * @return
	 */
	private EObject getBranchFromViewAction(final Action model) {
		if (model instanceof Decision || model instanceof Fork || model instanceof Join) {
			ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
			Object viewModel = context.get(model);		
			List list = PEDomainViewObjectHelper.getAllInBranch((EObject) viewModel);
			if (list.size()>0) {
				return (EObject) list.get(0);
			}				
		}
		
		return null;
	}
}
