package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.AbstractAddPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.pinset.add.AddOutputPinSetPeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.processes.activities.UpdateInputPinSetBOMCmd;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.activities.InputDeliveryOptionKind;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;

/**
 * Update the Input Pin Set
 */
public class UpdatePinSetRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private PinSet pinSet;
	private Object workingCopy;
	private Object process;	
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}
	
	/**
	 * Set Original pinSet
	 * @param pinSet
	 */
	public void setSource(final Object pinSet) {
		this.pinSet = (PinSet) pinSet;
	}	
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (pinSet == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PIN_SET_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();
		AbstractAddPeCmd pinSetCmd = null;
		if (pinSet instanceof InputPinSet) {
			if (pinSet.eContainer() instanceof ControlAction) {
				pinSetCmd = cmdFactory.buildAddInBranchToControlActionPeCmd((EObject) workingCopy);
			} else {
				pinSetCmd = cmdFactory.buildAddInputPinSetPeCmd((EObject) workingCopy);
			}
		} else if (pinSet instanceof OutputPinSet) {
			if (pinSet.eContainer() instanceof ControlAction) {
				pinSetCmd = cmdFactory.buildAddOutBranchToControlActionPeCmd((EObject) workingCopy);
			} else {
				pinSetCmd = cmdFactory.buildAddOutputPinSetPeCmd((EObject) workingCopy);
				// Attribute isStream
				// Attribute isException				
				((AddOutputPinSetPeCmd)pinSetCmd).setIsException(((OutputPinSet)pinSet).getIsException().booleanValue());
				((AddOutputPinSetPeCmd)pinSetCmd).setIsStream(((OutputPinSet)pinSet).getIsStream().booleanValue());		
			}
			
		} else {
			LoggingUtil.trace(this, "invoke",
				IImportResultMessages.IMPROPER_BOM_MODEL);	
			LoggingUtil.traceExit(this, "invoke");
			return;				
		}
		
		pinSetCmd.setName(pinSet.getName());
		
		
				
		if (pinSetCmd.canExecute()) {
			try {
				pinSetCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke",
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				LoggingUtil.traceExit(this, "invoke");
				return;
			}
		} else {
			LoggingUtil.trace(this, "invoke", 
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			LoggingUtil.traceExit(this, "invoke");
			return;			
		}
		
		// Attribute aspect - ignore
		// Attribute auditLog - not handled
		// Attribute uid - generated
		// Attribute visibility - Default	
		// Attribute ownedConstraint - not handled	
		// [Expression] Attribute inputSetConstraint - InputPinSet
		// [Expression] Attribute correlationPredicate - InputPinSet
		// [Expression] Attribute condition - DecisionOutputSet

			
		EObject domainModel = pinSetCmd.getNewDomainModel();
		if (domainModel != null) {
			// Attribute ownedComment
			//TODO: Jie: walk around for now. we don't update the description for control action 
			// because the domainModel returned is not pinSet but the control action, 
			// and UI doesn't show description for pin set on control action
			if (!(pinSet.eContainer() instanceof ControlAction)){	
				BOMUtil.updateDescription((Element)domainModel, BOMUtil.getDescription((Element)pinSet));
			}
	
			if (domainModel instanceof InputPinSet) {
				// Attribute multipleCorrelationMatches
				UpdateInputPinSetBOMCmd cmd = new UpdateInputPinSetBOMCmd((InputPinSet)domainModel);
				InputDeliveryOptionKind deliveryKind = ((InputPinSet)pinSet).getMultipleCorrelationMatches();				
				cmd.setMultipleCorrelationMatches(InputDeliveryOptionKind.get(deliveryKind.getValue()));
				// Attribute noCorrelationMatches
				deliveryKind = ((InputPinSet)pinSet).getNoCorrelationMatches();
				cmd.setNoCorrelationMatches(InputDeliveryOptionKind.get(deliveryKind.getValue()));				

				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						
						LoggingUtil.trace(this, "invoke", 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke", 
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);
				}				
			}

			// 20050418 - create the UID hashmap for ADF Import
			// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
			if (pinSet.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());			
				if (domainModel instanceof PinSet) {
					// 20050413 - create the UID hashmap for ADF Import
					// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
					processHashMap.put(pinSet.getUid(), ((PinSet)domainModel).getUid());				
					// end of 20050413 - create the UID hashmap for ADF Import
				} else if (domainModel instanceof ControlAction) {
					ControlAction domainModelCtrlAction = (ControlAction) domainModel;				
					if (pinSet instanceof InputPinSet) {					
						ControlAction pinSetOwner = (ControlAction)((InputPinSet) pinSet).getAction();						
						List inputPinSets = pinSetOwner.getInputPinSet();
						if (inputPinSets != null && !inputPinSets.isEmpty()) {
							int i = inputPinSets.indexOf(pinSet);						
							List domainModelPinSets = domainModelCtrlAction.getInputPinSet();
							if (domainModelPinSets != null && !domainModelPinSets.isEmpty()) {
								InputPinSet domainModelPinSet = (InputPinSet) domainModelPinSets.get(i);									
								// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
								processHashMap.put(pinSet.getUid(), domainModelPinSet.getUid());								
							} else {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.DOMAIN_MODEL_IS_NULL);
							}
						}					
					} else if (pinSet instanceof OutputPinSet) {
						ControlAction pinSetOwner = (ControlAction)((OutputPinSet) pinSet).getAction();					
						List outputPinSets = pinSetOwner.getOutputPinSet();
						if (outputPinSets != null && !outputPinSets.isEmpty()) {
							int i = outputPinSets.indexOf(pinSet);
							List domainModelPinSets = domainModelCtrlAction.getOutputPinSet();
							if (domainModelPinSets != null && !domainModelPinSets.isEmpty()) {
								OutputPinSet domainModelPinSet = (OutputPinSet) domainModelPinSets.get(i);								
								// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
								processHashMap.put(pinSet.getUid(), domainModelPinSet.getUid());														
							} else {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.DOMAIN_MODEL_IS_NULL);
							}
						}
					}				
				}
			}
		}
		
		LoggingUtil.trace(this, "invoke", "Put the object into the context");
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		context.put(pinSet, pinSetCmd.getNewViewModel());
		
		LoggingUtil.traceExit(this, "invoke");
	}
}
