package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.ParamPeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.param.remove.RemoveParamPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.remove.RemoveInputParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.remove.RemoveOutputParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.remove.RemoveParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.update.AssociateInputParamSetWithOutputParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.update.AssociateParamsWithParamSetsPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.update.DisassociateInputParamSetFromOutputParamSetPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.paramset.update.DisassociateParamsFromParamSetsPeCmd;
import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.artifacts.UpdateParameterBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateActivityBOMCmd;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Parameter;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.bom.model.services.ParameterSet;
import com.ibm.btools.cef.gef.emf.command.AddModelPropertyCommand;
import com.ibm.btools.cef.model.CommonContainerModel;
import com.ibm.btools.cef.model.CommonNodeModel;
import com.ibm.btools.cef.model.CommonVisualModel;
import com.ibm.btools.cef.model.VisualModelDocument;

/**
* Update the Reusable Process
*/
public class UpdateReusableProcessRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private final String DO_LAYOUT_PROPERTY_ID 		= "doLayout";
	private final String DO_LAYOUT_PROPERTY_NAME 	= "doLayout";
	private final Boolean DO_LAYOUT_PROPERTY_VALUE	= Boolean.TRUE;
	
	protected Activity process;
	protected VisualModelDocument workingCopy;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setSource(final Object process) {
		this.process = (Activity) process;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = (VisualModelDocument) workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (process == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROCESS_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		Activity workingCopyActivity = getActivity();
		if (workingCopyActivity == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);			
			LoggingUtil.traceExit(this, "invoke");			
			return;	
		}
		addFlagForAutoLayout();
		addStyle();
		
		createContext();
		LoggingUtil.trace(this, "invoke", "Create the context for process");
		
		createAttributes(workingCopyActivity);		
		
		// 20050413 - create the UID hashmap for ADF Import
		if (this.process.getUid() != null) {
			HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
			// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
			processHashMap.put(this.process.getUid(), workingCopyActivity.getUid());
		}
		// end of 20050413 - create the UID hashmap for ADF Import
		LoggingUtil.traceExit(this, "invoke");
	}
	
	private void addFlagForAutoLayout() {
		CommonContainerModel container = getImplementationViewModel();
		if (container != null) {
			AddModelPropertyCommand command = new AddModelPropertyCommand(container);
			command.setId(DO_LAYOUT_PROPERTY_ID);
			command.setName(DO_LAYOUT_PROPERTY_NAME);
			command.setValue(DO_LAYOUT_PROPERTY_VALUE);	
			
			if (command.canExecute()) {
				try {				
					command.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "addFlagForAutoLayout", "Fail to update the command to add model property.");	
				}
			} else {
				LoggingUtil.trace(this, "addFlagForAutoLayout", "Fail to update the command to add model property.");	
			}
		}
	}
	
	private void addStyle() {
		StructuredActivityNode san = process.getImplementation();
		HashMap<StructuredActivityNode, String> sanStyleMap =
			(HashMap<StructuredActivityNode, String>)
				getImportSession().getContext().get(IImportConstants.PROCESS_STYLE);
		boolean isBPMN = sanStyleMap != null && "BPMN".equals(sanStyleMap.get(san));
		if (isBPMN) {
			AddModelPropertyCommand command = new AddModelPropertyCommand(workingCopy);
			// command.setId("BPMN");
			command.setName("BPMN");
			command.setValue(Boolean.TRUE);	
			
			if (command.canExecute()) {
				try {				
					command.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "addStyle", "Fail to update the command to add model property.");	
				}
			} else {
				LoggingUtil.trace(this, "addStyle", "Fail to update the command to add model property.");	
			}
		}
	}
	
	/**
	 * Create Process's attributes
	 * @param process
	 */
	protected void createAttributes(Activity workingCopyActivity) {
		LoggingUtil.traceEntry(this, "createAttributes");	
		UpdateActivityBOMCmd cmd = new UpdateActivityBOMCmd(workingCopyActivity);		
		boolean updateAttributes = false;
		
		// Attribute aspect - ignore
		
		// Attribute context - ignore	
		
		// Attribute isAbstract
		Boolean isAbstract = process.getIsAbstract();
		if (isAbstract != null) {
			cmd.setIsAbstract(isAbstract.booleanValue());
			LoggingUtil.trace(this, "createAttributes", "Set isAbstract.");
			updateAttributes = true;
		}	
		
		// Attribute language - not handled
		// Attribute name - handled
		// Attribute ownedAttribute - not handled
		// ownedConstraint - not handled
		// Attribute ownedOperation - not handled
		// Attribute owningPackage - handled
		// Attribute uid - generated
		// Attribute visibility - must be public = default		
		// Attribute specification - not handled
		// Attribute superClassifier - not handled
		// Attribute body - not handled	
		// Attribute ownedComment - handled by creation command
		
		// [Expression] Attribute postcondition - Sequence 
		// [Expression] Attribute precondition - Sequence
		
		// Execute the command
		if (updateAttributes) {
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {		
				LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);									
			}	
		}
			
		// Attribute inputParameterSet 
		// Attribute ownedParameterSet			
		if(NavigatorUtil.isNewNode(getImportSession(),process)){			
			updateParameterSets();		
			//Attribute parameter		
			updateParameters();	
		}else{
			if(NavigatorUtil.isOptionIgnore(getImportSession().getImportOptions(),process)){	
				//no-op
			}		
			else if(NavigatorUtil.isOptionOverwrite(getImportSession().getImportOptions(),process)){
				consolidateParameterSets();				
			}				
		}		
		// Attribute implementation - 2nd sequence	
		updateImplementation();
		
		LoggingUtil.traceExit(this, "createAttributes");	
	}
	
	private void consolidateParameterSets(){
		
		Activity targetActivity = getActivity();
		if(workingCopy.getRootContent() == null || workingCopy.getRootContent().getContentChildren().size() == 0){
			return;
		}
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		
		Iterator bomInputParameterSetIterator = process.getInputParameterSet().iterator();		
		Iterator tgtInputParameterSetIterator = null;
		
		//1. check if there are new added parameterSet		
		List addedInputParameterSets = new ArrayList();	
		List removedInputParameterSets = new ArrayList();		
		while(bomInputParameterSetIterator.hasNext()){
			boolean addedInputParamSet = true;
			ParameterSet bomParameterSet = (ParameterSet)bomInputParameterSetIterator.next();
			tgtInputParameterSetIterator = targetActivity.getInputParameterSet().iterator();
			while(tgtInputParameterSetIterator.hasNext()){
				ParameterSet targetParameterSet = (ParameterSet)tgtInputParameterSetIterator.next();
				if(bomParameterSet.getName()!= null && 
						bomParameterSet.getName().equals(targetParameterSet.getName())){
					//we find the matching parameterset
					addedInputParamSet = false;					
					break;
				}
			}	
			if(addedInputParamSet){
				addedInputParameterSets.add(bomParameterSet);
			}
		}
		
		//Refresh the collection
		tgtInputParameterSetIterator = targetActivity.getInputParameterSet().iterator();	
		
		//2. check if there are removed parameterSet
		while(tgtInputParameterSetIterator.hasNext()){
			boolean isRemovedInputParamSet = true;
			ParameterSet targetParameterSet = (ParameterSet)tgtInputParameterSetIterator.next();
			bomInputParameterSetIterator = process.getInputParameterSet().iterator();
			while(bomInputParameterSetIterator.hasNext()){
				ParameterSet bomParameterSet = (ParameterSet)bomInputParameterSetIterator.next();
				if(targetParameterSet.getName() != null &&
						targetParameterSet.getName().equals(bomParameterSet.getName())){
					isRemovedInputParamSet = false;
					break;
				}
			}			
			if(isRemovedInputParamSet){
				removedInputParameterSets.add(targetParameterSet);
			}
		}
		
		//3. Work with the output OutputPinset
		Iterator bomOutputParameterSetIterator = process.getOutputParameterSet().iterator();
		Iterator tgtOutputParameterSetIterator = null; 
		
		//4. check if there are new added output parameter set
		List addedOutputParameterSets = new ArrayList();
		List removedOutputParameterSets = new ArrayList();		
		while(bomOutputParameterSetIterator.hasNext()){
			ParameterSet bomOutputParameterSet = (ParameterSet)bomOutputParameterSetIterator.next();
			boolean addedOutputParamSet = true;
			tgtOutputParameterSetIterator = targetActivity.getOutputParameterSet().iterator();
			while(tgtOutputParameterSetIterator.hasNext()){
				ParameterSet targetOutputParameterSet = (ParameterSet)tgtOutputParameterSetIterator.next();
				if(bomOutputParameterSet.getName() != null &&
						bomOutputParameterSet.getName().equals(targetOutputParameterSet.getName())){
					addedOutputParamSet = false;
					break;
				}
			}
			if(addedOutputParamSet){
				addedOutputParameterSets.add(bomOutputParameterSet);
			}
		}
		
		//Refresh the collection
		tgtOutputParameterSetIterator = targetActivity.getOutputParameterSet().iterator();
		
		//5. check if there are removed output parameter set
		while(tgtOutputParameterSetIterator.hasNext()){
			ParameterSet targetOutputParameterSet = (ParameterSet)tgtOutputParameterSetIterator.next();
			boolean removedOutputParamSet = true;
			bomOutputParameterSetIterator = process.getOutputParameterSet().iterator();	
			while(bomOutputParameterSetIterator.hasNext()){
				ParameterSet bomOutputParameterSet = (ParameterSet)bomOutputParameterSetIterator.next();
				if(targetOutputParameterSet.getName() != null &&
						targetOutputParameterSet.getName().equals(bomOutputParameterSet.getName())){
					removedOutputParamSet = false;
					break;
				}
			}
			if(removedOutputParamSet){
				removedOutputParameterSets.add(targetOutputParameterSet);
			}
		}
		
		//6. Process added parameter sets. we need to append on the target activity
		if(addedInputParameterSets != null && addedInputParameterSets.size() > 0){			
			for(Iterator addedIt = addedInputParameterSets.iterator();addedIt.hasNext();){
				ParameterSet addedParameterSet = (ParameterSet)addedIt.next();
				//attach this parameter set on the target activity
				UpdateParameterSetRule inputParamSetRule = new UpdateParameterSetRule();
				inputParamSetRule.setImportSession(getImportSession());
				inputParamSetRule.setProjectName(getProjectName());
				inputParamSetRule.setSource(addedParameterSet);
				inputParamSetRule.setWorkingCopy(topContainer);
				inputParamSetRule.setProcess(process);
				inputParamSetRule.invoke(); 
			}
		}
		
		//6.1 Disassociate the input parameter set with the output parameter set that are to be removed
	     disassociateInputParameterSetWithOutputParameterSet(removedInputParameterSets,removedOutputParameterSets);
		
		//7. Remove the parameter set from the target parameterset
		if(removedInputParameterSets != null && removedInputParameterSets.size() > 0){
			for(Iterator removedIt = removedInputParameterSets.iterator();removedIt.hasNext();){
				ParameterSet removeableParameterSet = (ParameterSet)removedIt.next();				
				//remove this input parameter set from the target activity					
				removeParameterSetViewModel(removeableParameterSet);					
			}
		}	
		
		
		//8. Append new output parameter set to the target activity if there is any
		if(addedOutputParameterSets != null && addedOutputParameterSets.size() >0 ){
			Iterator addedOutputParamSetsIterator = addedOutputParameterSets.iterator();
			while(addedOutputParamSetsIterator.hasNext()){
				ParameterSet addedOutputParamSet = (ParameterSet)addedOutputParamSetsIterator.next();
				UpdateParameterSetRule inputParamSetRule = new UpdateParameterSetRule();
				inputParamSetRule.setImportSession(getImportSession());
				inputParamSetRule.setProjectName(getProjectName());
				inputParamSetRule.setSource(addedOutputParamSet);
				inputParamSetRule.setWorkingCopy(topContainer);
				inputParamSetRule.setProcess(process);
				inputParamSetRule.invoke(); 		
			}			
		}
		
		//9. Remove output parameter set from the target activity if there is any
		if(removedOutputParameterSets != null && removedOutputParameterSets.size() >0 ){
			Iterator removedOutputParamSetsIterator = removedOutputParameterSets.iterator();
			while(removedOutputParamSetsIterator.hasNext()){
				ParameterSet removeableOutputParamSet = (ParameterSet)removedOutputParamSetsIterator.next();				
				//Remove the parameter set					
				removeParameterSetViewModel(removeableOutputParamSet);					
			}
		}
		
		//New created parameter view models have been put into the context
		updateParameters();
		
		//Associate the new added input parameter set with the output parameter set
		
		if(addedInputParameterSets == null || addedInputParameterSets.size() == 0){
			return;
		}
		
		AssociateInputParamSetWithOutputParamSetPeCmd cmd = null;
		AssociateParamsWithParamSetsPeCmd cmd2 = null;
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		InputParameterSet inputParameterSet = null;
		OutputParameterSet outputParameterSet = null;
		Iterator outputParameterSetIterator = null;
		EObject inputViewModel = null;
		EObject outputViewModel = null;
		
		EObject domainModel = null;
		EList contents = null;
		PeCmdFactory cmdFactory = getPeCmdFactory();
		
		Iterator parameterSetIterator = addedInputParameterSets.iterator();
		while(parameterSetIterator.hasNext()) {
			inputParameterSet = (InputParameterSet) parameterSetIterator.next();
			inputViewModel = (EObject) context.get(inputParameterSet);
			if (inputViewModel == null) {
				continue;
			}
			
			//1. add the association between the param and parameter set
			EList inputParamList = inputParameterSet.getParameter();
			if(inputParamList != null && inputParamList.size() != 0 ){
				
				//We need search the real parameters corresponding to the input parameter set
							
				Parameter zeroIndexParam = (Parameter)inputParamList.get(0);
				EObject objectpinViewModel = (EObject)context.get(zeroIndexParam);
				
				if(objectpinViewModel != null){
					
					List viewParamList = new ArrayList();
					viewParamList.add(objectpinViewModel);
					List viewParamSetList = new ArrayList();
					viewParamSetList.add(inputViewModel);
					
					cmd2 = cmdFactory.buildAssociateParamsWithParamSetsPeCmd(viewParamList,viewParamSetList);
					if(cmd2.canExecute()){
						cmd2.execute();
					}
				}
			}			
			
			outputParameterSetIterator = inputParameterSet.getOutputParameterSet().iterator();
			while(outputParameterSetIterator.hasNext()) {
				outputParameterSet = (OutputParameterSet) outputParameterSetIterator.next();
				outputViewModel = (EObject) context.get(outputParameterSet);
				if (outputViewModel == null) {
					continue;
				}	
				
				//2.Associate output param with output param set
				EList outputParamList = outputParameterSet.getParameter();
				if(outputParamList != null && outputParamList.size() != 0){
					Parameter outputObjectPinParam = (Parameter)outputParamList.get(0);
				    EObject outputObjectPinViewModel = (EObject)context.get(outputObjectPinParam);
					
					if(outputObjectPinViewModel != null){
						List outputViewParamList = new ArrayList();
						outputViewParamList.add(outputObjectPinViewModel);
						List outputViewParamSetList = new ArrayList();
						outputViewParamSetList.add(outputViewModel);
						
						cmd2 = cmdFactory.buildAssociateParamsWithParamSetsPeCmd(outputViewParamList,outputViewParamSetList);
						if(cmd2.canExecute()){
							cmd2.execute();
						}
					}
				}				
				
				//3. Associate the input parameter set with the output parameter set
				cmd = (AssociateInputParamSetWithOutputParamSetPeCmd) 
				cmdFactory.buildAssociateInputParamSetWithOutputParamSetPeCmd(inputViewModel, outputViewModel);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);							
					}
				} else {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_ASSOCIATE_CEF_COMMAND);
				}
			}
		}
	}
	
	private void disassociateInputParameterSetWithOutputParameterSet(List inputParamSet,List outputParamSet){
		if(inputParamSet == null || inputParamSet.size() == 0 ||
				outputParamSet == null || outputParamSet.size() == 0){
			return;
		}
		for(Iterator inputParameterSetIterator = inputParamSet.iterator();inputParameterSetIterator.hasNext();){
			InputParameterSet ips = (InputParameterSet)inputParameterSetIterator.next();
			EList opsList = ips.getOutputParameterSet();
			if(opsList == null || opsList.size() == 0){
				continue;
			}
			//we assume one-to-one relationship between the parameter sets
			OutputParameterSet ops = (OutputParameterSet)opsList.get(0);
			CommonNodeModel inputViewModel = getParameterViewModelByDomainContent(ips);
			CommonNodeModel outputViewModel = getParameterViewModelByDomainContent(ops);
			DisassociateInputParamSetFromOutputParamSetPeCmd cmd = getPeCmdFactory().buildDisassociateInputParamSetFromOutputParamSetPeCmd(inputViewModel,outputViewModel);
			if(cmd.canExecute()){
				cmd.execute();
			}
		}
	}
	private CommonNodeModel getParameterViewModelByDomainContent(ParameterSet domianModelToRemove){
		CommonNodeModel viewParameterSet = null;
		if(domianModelToRemove == null){
			return null;
		}
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		if (topContainer instanceof CommonContainerModel) {
			List removeList = new ArrayList(((CommonContainerModel)topContainer).getCompositionChildren());
			Iterator compositeChildrenIterator = removeList.iterator();				
			while(compositeChildrenIterator.hasNext()) {
				Object compositeChildren = compositeChildrenIterator.next();
				if (compositeChildren instanceof CommonNodeModel) {
					if (((CommonNodeModel)compositeChildren).getDescriptor() == null) {
						// Already removed.
						continue;		
					}
					LoggingUtil.trace(this, "updateParameterSets", "Common Node Model is used to represent the Parameter Set.");
					EList contentList = ((CommonNodeModel)compositeChildren).getDomainContent();					
					if (contentList.size() <= 0) {
						continue;
					}
					Object domainModel = contentList.get(0);					
					if (domainModel instanceof ParameterSet) {
						ParameterSet removeable = (ParameterSet)domainModel;
						if(removeable.getName() != null && removeable.getName().equals(domianModelToRemove.getName())){
							//we find the right view model corresponding to the domain mdoel
							viewParameterSet = (CommonNodeModel)compositeChildren;
							break;
						}						
					}
				}
			}			
		}
		return viewParameterSet;
	}
	
	private void removeParameterSetViewModel(ParameterSet removeableParamSet){
		
		PeCmdFactory cmdFactory = getPeCmdFactory();
		RemoveParamSetPeCmd removeCmd = null;
		List removeCmdList = new ArrayList();
		
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		Object compositeChildren = null;
		Object domainModel = null;
		EList contentList = null;
		if (topContainer instanceof CommonContainerModel) {
			List removeList = new ArrayList(((CommonContainerModel)topContainer).getCompositionChildren());
			
			Iterator compositeChildrenIterator = removeList.iterator();				
			while(compositeChildrenIterator.hasNext()) {
				compositeChildren = compositeChildrenIterator.next();
				if (compositeChildren instanceof CommonNodeModel) {
					if (((CommonNodeModel)compositeChildren).getDescriptor() == null) {
						// Already removed.
						continue;		
					}
					LoggingUtil.trace(this, "updateParameterSets", "Common Node Model is used to represent the Parameter Set.");
					contentList = ((CommonNodeModel)compositeChildren).getDomainContent();					
					if (contentList.size() <= 0) {
						continue;
					}
					domainModel = contentList.get(0);					
					if (domainModel instanceof ParameterSet) {
						ParameterSet removeable = (ParameterSet)domainModel;
						if(removeable.getName() != null && (removeable.getName().equals(removeableParamSet.getName()))){
							
							EList<CommonNodeModel> visualElements = ((CommonNodeModel)compositeChildren).getElements();
							if(visualElements != null && visualElements.size() <= 0){
								continue;
							}
							List<CommonNodeModel> visualParamList = new ArrayList<CommonNodeModel>(visualElements);
							for(CommonNodeModel visualParameter:visualParamList){
								//Remove it's associated parameters first
								if(visualParameter instanceof CommonVisualModel){

									ParamPeCmdFactory paramCmdFactory = new ParamPeCmdFactory(cmdFactory);

									DisassociateParamsFromParamSetsPeCmd disassociateCmd = cmdFactory.buildDisassociateParamsFromParamSetsPeCmd(visualElements,contentList);
									if(disassociateCmd.canExecute()){
										disassociateCmd.execute();
									}

									RemoveParamPeCmd removeParamCmd = paramCmdFactory.buildRemoveParamPeCmd((EObject)visualParameter);
									if(removeParamCmd.canExecute()){
										removeParamCmd.execute();
									}
								}						
							}
							removeCmd = (RemoveParamSetPeCmd) cmdFactory.buildRemovePeCmd((CommonNodeModel)compositeChildren);
							if (removeCmd.canExecute()) {
								try {
									removeCmd.execute();												

								} catch (RuntimeException e) {
									e.printStackTrace();
									return;							
								}							
							} 
							break;
						}
					}
				}
			}
			
		} else {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);
			LoggingUtil.traceExit(this, "updateParameterSets");
			return;
		}		
		
	}
	/**
	 * Update parameter sets
	 */
	protected void updateParameterSets() {
		LoggingUtil.traceEntry(this, "updateParameterSets");
		
		// Use command to remove the existing parameters and create the sets later.
		// If cannot create the set, undo the remove.
		
		LoggingUtil.trace(this, "updateParameterSets", "Remove existing parameter sets");
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);	
			LoggingUtil.traceExit(this, "updateParameterSets");											
			return;
		}
		
		PeCmdFactory cmdFactory = getPeCmdFactory();
		RemoveParamSetPeCmd removeCmd = null;
		List removeCmdList = new ArrayList();
		try {
			Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
			Object compositeChildren = null;
			Object domainModel = null;
			EList contentList = null;
			if (topContainer instanceof CommonContainerModel) {
				List removeList = new ArrayList(((CommonContainerModel)topContainer).getCompositionChildren());
				
				Iterator compositeChildrenIterator = removeList.iterator();				
				while(compositeChildrenIterator.hasNext()) {
					compositeChildren = compositeChildrenIterator.next();
					if (compositeChildren instanceof CommonNodeModel) {
						if (((CommonNodeModel)compositeChildren).getDescriptor() == null) {
							// Already removed.
							continue;		
						}
						LoggingUtil.trace(this, "updateParameterSets", "Common Node Model is used to represent the Parameter Set.");
						contentList = ((CommonNodeModel)compositeChildren).getDomainContent();
						if (contentList.size() <= 0) {
							continue;
						}
						domainModel = contentList.get(0);
						//Why should we remove the parameter set and create new???
						if (domainModel instanceof ParameterSet) {
							//if(shouldeRemove((ParameterSet)domainModel)){
							removeCmd = (RemoveParamSetPeCmd) cmdFactory.buildRemovePeCmd((CommonNodeModel)compositeChildren);
							if (removeCmd.canExecute()) {
								try {
									removeCmd.execute();
									removeCmdList.add(removeCmd);
								} catch (RuntimeException e) {
									LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
									Iterator iterator = removeCmdList.iterator();
									RemoveParamSetPeCmd alreadyRemovedCmd = null;
									while(iterator.hasNext()) {
										alreadyRemovedCmd = (RemoveParamSetPeCmd) iterator.next();
										try {
											alreadyRemovedCmd.undo();
										} catch (RuntimeException re) {
											LoggingUtil.trace(this, "updateParameterSets", "Cannot undo remove command.");
											LoggingUtil.traceExit(this, "updateParameterSets");
											return;	
										}
									}
									LoggingUtil.traceExit(this, "updateParameterSets");
									return;							
								}
								//}
							} else {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_REMOVE_CEF_COMMAND);								
								Iterator iterator = removeCmdList.iterator();
								RemoveParamSetPeCmd alreadyRemovedCmd = null;
								while(iterator.hasNext()) {
									alreadyRemovedCmd = (RemoveParamSetPeCmd) iterator.next();
									try {
										alreadyRemovedCmd.undo();
									} catch (RuntimeException re) {
										LoggingUtil.trace(this, "updateParameterSets", "Cannot undo remove command.");	
										LoggingUtil.traceExit(this, "updateParameterSets");
										return;	
									}
								}
								LoggingUtil.traceExit(this, "updateParameterSets");
								return;								
							}
						}
					}
				}
				
			} else {
				LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);
				LoggingUtil.traceExit(this, "updateParameterSets");
				return;
			}		
		} catch (RuntimeException e) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);
			Iterator iterator = removeCmdList.iterator();
			RemoveParamSetPeCmd alreadyRemovedCmd = null;
			while(iterator.hasNext()) {
				alreadyRemovedCmd = (RemoveParamSetPeCmd) iterator.next();
				try {
					alreadyRemovedCmd.undo();
				} catch (RuntimeException re) {
					LoggingUtil.trace(this, "updateParameterSets", "Cannot undo remove command.");	
					LoggingUtil.traceExit(this, "updateParameterSets");
					return;	
				}
			}
			LoggingUtil.traceExit(this, "updateParameterSets");
			return;
		}
		
		LoggingUtil.trace(this, "updateParameterSets", "Add new parameter sets");
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		ParameterSet eachParameterSet = null;
		UpdateParameterSetRule inputParamSetRule = null;
		UpdateParameterSetRule outputParamSetRule = null;
		Iterator parameterSetIterator = process.getInputParameterSet().iterator();
		LoggingUtil.trace(this, "updateParameterSets", "Create input parameter sets");
		while(parameterSetIterator.hasNext()) {
			eachParameterSet = (ParameterSet) parameterSetIterator.next();
			inputParamSetRule = new UpdateParameterSetRule();
			inputParamSetRule.setImportSession(getImportSession());
			inputParamSetRule.setProjectName(getProjectName());
			inputParamSetRule.setSource(eachParameterSet);
			inputParamSetRule.setWorkingCopy(topContainer);
			inputParamSetRule.setProcess(process);
			inputParamSetRule.invoke(); 
		}
		// Check to see how many parameterset is contained.		
		Activity updatedActivity = getActivity();
		if (updatedActivity != null && updatedActivity.getInputParameterSet().size() == 0) {
			Iterator iterator = removeCmdList.iterator();
			RemoveParamSetPeCmd alreadyRemovedCmd = null;
			while(iterator.hasNext()) {
				alreadyRemovedCmd = (RemoveParamSetPeCmd) iterator.next();
				try {
					if (alreadyRemovedCmd instanceof RemoveInputParamSetPeCmd) {
						alreadyRemovedCmd.undo();
					}
				} catch (RuntimeException re) {
					LoggingUtil.trace(this, "updateParameterSets", "Cannot undo remove command.");
				}
			}		
		}
		
		
		LoggingUtil.trace(this, "updateParameterSets", "Create output parameter sets");		
		parameterSetIterator = process.getOutputParameterSet().iterator();
		while(parameterSetIterator.hasNext()) {
			eachParameterSet = (ParameterSet) parameterSetIterator.next();
			outputParamSetRule = new UpdateParameterSetRule();
			outputParamSetRule.setImportSession(getImportSession());
			outputParamSetRule.setProjectName(getProjectName());
			outputParamSetRule.setSource(eachParameterSet);
			outputParamSetRule.setWorkingCopy(topContainer);
			outputParamSetRule.setProcess(process);
			outputParamSetRule.invoke(); 
		}
		if (updatedActivity != null && updatedActivity.getOutputParameterSet().size() == 0) {
			Iterator iterator = removeCmdList.iterator();
			RemoveParamSetPeCmd alreadyRemovedCmd = null;
			while(iterator.hasNext()) {
				alreadyRemovedCmd = (RemoveParamSetPeCmd) iterator.next();
				try {
					if (alreadyRemovedCmd instanceof RemoveOutputParamSetPeCmd) {
						alreadyRemovedCmd.undo();
					}
				} catch (RuntimeException re) {
					LoggingUtil.trace(this, "updateParameterSets", "Cannot undo remove command.");
				}
			}	
			LoggingUtil.traceExit(this, "updateParameterSets");
			return;	
		}		
		
		// All parameter sets are created. Set the reference between the parameter set.
		
		// Attribute outputParameterSet
		// Attribute inputParameterSet
		AssociateInputParamSetWithOutputParamSetPeCmd cmd = null;
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		InputParameterSet inputParameterSet = null;
		OutputParameterSet outputParameterSet = null;
		Iterator outputParameterSetIterator = null;
		EObject inputViewModel = null;
		EObject outputViewModel = null;
		
		EObject domainModel = null;
		EList contents = null;
		parameterSetIterator = process.getInputParameterSet().iterator();
		while(parameterSetIterator.hasNext()) {
			inputParameterSet = (InputParameterSet) parameterSetIterator.next();
			inputViewModel = (EObject) context.get(inputParameterSet);
			if (inputViewModel == null) {
				continue;
			}
			outputParameterSetIterator = inputParameterSet.getOutputParameterSet().iterator();
			while(outputParameterSetIterator.hasNext()) {
				outputParameterSet = (OutputParameterSet) outputParameterSetIterator.next();
				outputViewModel = (EObject) context.get(outputParameterSet);
				if (outputViewModel == null) {
					continue;
				}
				cmd = (AssociateInputParamSetWithOutputParamSetPeCmd) 
				cmdFactory.buildAssociateInputParamSetWithOutputParamSetPeCmd(inputViewModel, outputViewModel);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);							
					}
				} else {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_ASSOCIATE_CEF_COMMAND);
				}
			}
		}
		
		LoggingUtil.traceExit(this, "updateParameterSets");
	}
	
	/**
	 * Update parameters
	 */
	protected void updateParameters() {	
		LoggingUtil.traceEntry(this, "updateParameters");			
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		if (topContainer instanceof CommonContainerModel) {
			Parameter eachParameter = null;
			UpdateParameterRule rule = null;
			
			//imported one
			Iterator parameterIterator = process.getParameter().iterator();	 
			
			//Remove parameter from the visual model document if there is any
			List removableParamters = getRemovableParameter();
			if(removableParamters != null && !removableParamters.isEmpty()){
				Iterator removableIt = removableParamters.iterator();
				while(removableIt.hasNext()){
					Parameter theRemovable = (Parameter)removableIt.next();				
					ParamPeCmdFactory paramFactory = getPeCmdFactory().getParamPeCmdFactory();
					RemoveParamPeCmd removeParamCmd = paramFactory.buildRemoveParamPeCmd(getViewModelForParameter(theRemovable));
				    if(removeParamCmd != null && removeParamCmd.canExecute()){
				    	removeParamCmd.execute();
				    }
				}
			}
			while(parameterIterator.hasNext()) {
				eachParameter = (Parameter) parameterIterator.next();
				//we need add logic here to decide whether to add parameters			
				
				//Check if there is any dependencies in the workspace
				if(shouldSkipParameter(eachParameter)){
					//we won't create new object pin, copy over the values
					//look up the matched visual model
					Parameter visualParamter = getMatchedViewParameter(eachParameter);
					if(visualParamter == null){
						return;
					}				   
					//Update the values of the parameter only
					copyValueAndExecute(eachParameter,visualParamter);
					
				}else{					
						rule = new UpdateParameterRule();
						rule.setImportSession(getImportSession());
						rule.setProjectName(getProjectName());
						rule.setSource(eachParameter);
						rule.setWorkingCopy(topContainer);
						rule.setProcess(process);
						rule.invoke(); 		
				}
				
			}
		}
		LoggingUtil.traceExit(this, "updateParameters");		
	}
	
	private boolean isTargetParameterSetRemoved(){
		return true;
	}
	
	/**
	 * Match the exisiting parameters with the incoming ones, if no match found, remove it.
	 * @return
	 */
	private List getRemovableParameter(){
		List removableParameterList = new ArrayList();		
		Activity visualActivity = getActivity();
		EList paramList = visualActivity.getParameter();
		if(paramList != null && paramList.size() > 0){
			Iterator existingParamIt = paramList.iterator();
			while(existingParamIt.hasNext()){
				Object paramObj = existingParamIt.next();
				Parameter existingParam = null;				
				boolean matchFound = false;
				if(paramObj instanceof Parameter){
					existingParam = (Parameter)paramObj;	
					Iterator newParameterIterator2 = process.getParameter().iterator();
					while(newParameterIterator2.hasNext()){
						Parameter newParam = (Parameter)newParameterIterator2.next();
						if(existingParam.getName()!= null && existingParam.getName().equals(newParam.getName())&&
								existingParam.getDirection().getValue()== newParam.getDirection().getValue()){
							matchFound = true;							
							break;
						}						
					}	
					if(existingParam != null && !removableParameterList.contains(existingParam) && !matchFound){
						removableParameterList.add(existingParam);
					}
				}
//				if(existingParam != null && !removableParameterList.contains(existingParam) && !matchFound){
//					removableParameterList.add(existingParam);
//				}
			}
		}
		return removableParameterList;
	}
	
	private void copyValueAndExecute(Parameter source, Parameter target){
		int lowerBound = 1; // Default in Tool and BOM
		int upperBound = 1; // Default in Tool and BOM
		
		ValueSpecification value = null;		
		value = source.getLowerBound();
		target.setLowerBound(value);
		
		// Attribute upperBound
		value = source.getUpperBound();		
		target.setUpperBound(value);
		
		UpdateParameterBOMCmd updateParameterBOMCmd = new UpdateParameterBOMCmd(target);
		
		// Attribute type
		Type type = source.getType();
		PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
		if(thePackageableElement == null){
			thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
		}
		if (thePackageableElement != null && thePackageableElement instanceof Type) {
			//Add object pin that is business item
			updateParameterBOMCmd.setType((Type)thePackageableElement);			
		}
		LoggingUtil.trace(this, "invoke", "Set Type.");	
		
		// Attribute isOrdered
		updateParameterBOMCmd.setIsOrdered(source.getIsOrdered().booleanValue());
		// Attribute isUnique
		updateParameterBOMCmd.setIsUnique(source.getIsUnique().booleanValue());
		
		if(updateParameterBOMCmd.canExecute()){
			updateParameterBOMCmd.execute();
		}
	}	

	
	private boolean shouldSkipParameter(Parameter bomParameter){		
		Activity visualActivity = getActivity();
		EList paramList = visualActivity.getParameter();
		Parameter visualParameter = null;
		if(paramList == null || paramList.size() == 0){
			return false;
		}
		boolean shouldSkip = false;
		for(Iterator it = paramList.iterator();it.hasNext();){
			visualParameter = (Parameter)it.next();
			// d160171
			if(bomParameter.getName()!= null && 
					bomParameter.getName().equals(visualParameter.getName()) &&
					bomParameter.getDirection().equals(visualParameter.getDirection())
					){
				//Here we find a matched one in terms of the element name				
				//Element with the same name exists in the work space, which should be retained and its
				//contents shall be updated
//				if(bomParameter.getType() != null && bomParameter.getType() instanceof PrimitiveType ){
//					if(bomParameter.getType().getName().equals(visualParameter.getType().getName())){
//						//If both are primitivetype and in the same name, replace it
//						shouldSkip = true;
//						break;
//					}
//				}	
				shouldSkip = true;
				break;
			}			
		}	
		return shouldSkip;
	}
	
	private Parameter getMatchedViewParameter(Parameter bomParameter){
		Activity visualActivity = getActivity();
		EList paramList = visualActivity.getParameter();
		if(paramList == null || paramList.size() == 0){
			return null;
		}
		for(Iterator it = paramList.iterator();it.hasNext();){
			Parameter visualParameter = (Parameter)it.next();
			if(bomParameter.getName()!= null && bomParameter.getName().equals(visualParameter.getName())){
				//Here we find a matched one in terms of the element name
				//This element has dependency in the work space, which should be retained
				return visualParameter;			
			}
		}
		return null;
	}
	
	private EObject getViewModelForParameter(Parameter paramDomainModel){
		EObject parameterInView = null;
		Object topContainer = workingCopy.getRootContent().getContentChildren().get(0);
		Object compositeChildren = null;
		Object domainModel = null;
		EList contentList = null;
		EList visualParameterContentList = null;
		boolean viewModelFound = false;
		
		if (topContainer instanceof CommonContainerModel) {
			List removeList = new ArrayList(((CommonContainerModel)topContainer).getCompositionChildren());
			
			Iterator compositeChildrenIterator = removeList.iterator();				
			while(compositeChildrenIterator.hasNext()) {
				compositeChildren = compositeChildrenIterator.next();
				if (compositeChildren instanceof CommonNodeModel) {
					if (((CommonNodeModel)compositeChildren).getDescriptor() == null) {
						// Already removed.
						continue;		
					}					
					contentList = ((CommonNodeModel)compositeChildren).getDomainContent();					
					if (contentList.size() <= 0) {
						continue;
					}
					domainModel = contentList.get(0);					
					if (domainModel instanceof ParameterSet) {					
						
						EList<CommonNodeModel> visualElements = ((CommonNodeModel)compositeChildren).getElements();
						if(visualElements != null && visualElements.size() <= 0){
							continue;
						}
						List<CommonNodeModel> viasualParamList = new ArrayList<CommonNodeModel>(visualElements);
						for(CommonNodeModel visualParameter:viasualParamList){							
							//Object visualParameter = ((CommonNodeModel)compositeChildren).getElements().get(i);	
							if(visualParameter instanceof CommonVisualModel){					
								visualParameterContentList = ((CommonVisualModel)visualParameter).getDomainContent();
								if(visualParameterContentList != null && visualParameterContentList.size() <= 0){
									continue;
								}
								if(paramDomainModel.getName() != null && 
										paramDomainModel.getName().equals(((Parameter)visualParameterContentList.get(0)).getName())){
									viewModelFound = true;
									parameterInView = (EObject)visualParameter;
									return parameterInView;
									//break;
								}	
							}
						}					
					}
				}
			}
			
		}
		return parameterInView;
	}
	
	private ParameterSet getMatchParameterSetForBomParameterSet(ParameterSet bomParamSet){
		EList workingCopy = getActivity().getInputParameterSet();
		for(Iterator wcIt = workingCopy.iterator();wcIt.hasNext();){
			InputParameterSet ips = (InputParameterSet)wcIt.next();
			if(ips.getName()!= null && ips.getName().equals(bomParamSet.getName())){
				return ips;
			}
		}
		return null;
	}
	
	/**
	 * Update Implementation
	 */
	protected void updateImplementation() {		
	}
	
	/**
	 * Get Activity from VisualDocument
	 * @return
	 */
	protected Activity getActivity() {
		LoggingUtil.traceEntry(this, "getActivity");	
		if (workingCopy == null) {
			LoggingUtil.traceExit(this, "getActivity");
			return null;
		}
		Object topContainer = null;
		Object domainObject = null;
		
		try {			
			topContainer = (Object) workingCopy.getRootContent().getContentChildren().get(0);
			if (topContainer instanceof CommonContainerModel) {
				domainObject = ((CommonContainerModel)topContainer).getDomainContent().get(0);
				if (domainObject instanceof Activity) {
					LoggingUtil.traceExit(this, "getActivity");
					return (Activity) domainObject;
				}
			}
		} catch (RuntimeException e) {
			LoggingUtil.traceExit(this, "getActivity");
			return null;
		}
		
		LoggingUtil.traceExit(this, "getActivity");
		return null;
	}
	
	/**
	 * Create Subcontext
	 */
	private void createContext() {
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY);
		context.getSubContext(process);
	}
	
	/**
	 * Return Implementation View Model
	 */		
	protected CommonContainerModel getImplementationViewModel() {	
		LoggingUtil.traceEntry(this, "getImplementationViewModel");	
		
		try {
			EObject topContainer = (EObject) workingCopy.getRootContent().getContentChildren().get(0);
			if (topContainer instanceof CommonContainerModel) {
				EObject implementationContainer = (EObject) ((CommonContainerModel)topContainer).getContent().getContentChildren().get(0);
				if (implementationContainer instanceof CommonContainerModel) {					
					return (CommonContainerModel)implementationContainer;
				}
			}			
		} catch (RuntimeException e) {	
			LoggingUtil.trace(this, "invoke", IImportResultMessages.IMPROPER_VISUAL_MODEL);
			LoggingUtil.traceExit(this, "getImplementationViewModel");		
		}
		return null;
	}	
	
	private ParameterSet locateBomParameterSetForParameter(Parameter bomParameter){
		ParameterSet targetPinSet = null;
		EList inputPinSets = process.getInputParameterSet();
		for(Iterator ipsIterator = inputPinSets.iterator(); ipsIterator.hasNext();){
			InputParameterSet eachSet = (InputParameterSet)ipsIterator.next();
			if(eachSet.getParameter().contains(bomParameter)){
				targetPinSet = eachSet;
				break;
			}
		}
		return targetPinSet;
	}
}
