package com.ibm.btools.blm.ie.imprt.rule.processModel;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.node.add.AddVariablePeCmd;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.AbstractImportRule;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionComputedValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionDefaultValueToRepositoryBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateVariableBOMCmd;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionComputedValueToRepositoryBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;


/**
 * Update the Variable
 */
public class UpdateVariableRule extends AbstractImportRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private Variable variable;
	private Object workingCopy;
	private Object process;
	
	/**
	 * Set Original process
	 * @param process
	 */
	public void setProcess(final Object process) {
		this.process = process;
	}	
	
	/**
	 * Set Original variable
	 * @param variable
	 */
	public void setSource(final Object variable) {
		this.variable = (Variable) variable;
	}
	
	/**
	 * Set the working copy
	 * @param workingCopy
	 */
	public void setWorkingCopy(final Object workingCopy) {
		this.workingCopy = workingCopy;
	}
	
	/**
	 * Invoke the rule
	 */
	public void invoke() {
		LoggingUtil.traceEntry(this, "invoke");			
		if (variable == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.EDGE_IS_NULL);			
			LoggingUtil.traceExit(this, "invoke");
			return;
		}
		NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
		if (projectNode == null) {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.PROJECT_NODE_IS_NULL);												
			return;
		}
		PeCmdFactory cmdFactory = getPeCmdFactory();

		AddVariablePeCmd cmd = cmdFactory.buildAddVariablePeCmd((EObject)workingCopy);
		cmd.setName(variable.getName());
		
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
			}
		} else {
			LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
		}		
		
		// Attribute ownedComment		
		EObject domainModel = cmd.getNewDomainModel();
		if (domainModel != null && domainModel instanceof Variable) {
			BOMUtil.updateDescription((Element)domainModel, BOMUtil.getDescription((Element)variable));
			
			LoggingUtil.trace(this, "invoke", "Put the object into the context");
			ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
			context.put(variable, cmd.getNewViewModel());						
			
			// Attribute aspect - not handled
			// Attribute name - handled
			// Attribute ownedConstraint - not handled
			// Attribute uid - generated
			// Attribute visibility - not handle
		 
			UpdateVariableBOMCmd updateCmd = new UpdateVariableBOMCmd((Variable) domainModel);  
			// Attribute isOrdered 
			updateCmd.setIsOrdered(variable.getIsOrdered().booleanValue());
			// Attribute isUnique
			updateCmd.setIsUnique(variable.getIsUnique().booleanValue());
			// Attribute isReadOnly
			updateCmd.setIsReadOnly(variable.getIsReadOnly().booleanValue()); 
			
			// Attribute type
			Type type = variable.getType();
			PackageableElement thePackageableElement = BOMUtil.getType(getProjectName(), type, getImportSession().getContext());
			if(thePackageableElement == null){
				thePackageableElement = BOMUtil.getTypeFromWorkspace(getImportSession(), getProjectName(), type);
			}
			if (thePackageableElement != null && thePackageableElement instanceof Type) {
				updateCmd.setType((Type)thePackageableElement);
			}
			
			if (updateCmd.canExecute()) {
				try {
					updateCmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}				
			
			int upperBound = -1; // Default in Tool
			// Attribute lowerBound - always 0

			// Attribute upperBound
			ValueSpecification value = variable.getUpperBound();
			if (value instanceof LiteralInteger) {
				upperBound = ((LiteralInteger)value).getValue().intValue();				
			} else if (value instanceof LiteralUnlimitedNatural) {
				upperBound = -1;
			}		
			BOMUtil.updateUpperBound((Variable)domainModel, upperBound);
			
			// Attribute computedValue
			// Attribute defaultValue
			Iterator conmputedIterator = variable.getComputedValue().iterator();
			Iterator defaultIterator = variable.getDefaultValue().iterator();;
			AddUpdateObjectCommand updateValueCmd = null;
			InstanceSpecification spec = null;
		
			while(conmputedIterator.hasNext()) {
				value = (ValueSpecification) conmputedIterator.next();
					
				if (value instanceof LiteralNull) {
					updateValueCmd = new AddLiteralNullComputedValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if (value instanceof LiteralSpecification) {
					updateValueCmd = new AddLiteralStringComputedValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if(value instanceof StructuredOpaqueExpression){
					updateValueCmd = new AddStructuredOpaqueExpressionComputedValueToRepositoryBEXCmd((Repository)workingCopy);	
				} else if (value instanceof OpaqueExpression) {
					// [Expression] Attrbibute constant value
					updateValueCmd = new AddOpaqueExpressionComputedValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if (value instanceof InstanceValue) {
					updateValueCmd = new AddInstanceValueComputedValueToRepositoryBOMCmd((Repository)workingCopy);	
					/*
					spec = ((InstanceValue)value).getInstance();
					if (spec != null) {
						PackageableElement packageableElement = BOMUtil.getType(getProjectName(), spec, getImportSession().getContext());
						if (packageableElement instanceof InstanceSpecification) {						
							((AddInstanceValueComputedValueToRepositoryBOMCmd)updateValueCmd).setInstance((InstanceSpecification)packageableElement);
						}	
					}	
					*/	
				}
				if (updateValueCmd.canExecute()) {
					try {
						updateValueCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
				}
				
				// After creation
				Object newDomainModel = updateValueCmd.getObject();
				if (newDomainModel != null) {
					UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
					rule.setSource(value);
					rule.setWorkingCopy(newDomainModel);
					rule.setProjectName(getProjectName());
					rule.setImportSession(getImportSession());
					rule.invoke();
				}
			}
		
			while(defaultIterator.hasNext()) {
				value = (ValueSpecification) defaultIterator.next();
			
				if (value instanceof LiteralNull) {
					updateValueCmd = new AddLiteralNullDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if (value instanceof LiteralSpecification) {
					updateValueCmd = new AddLiteralStringDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if (value instanceof OpaqueExpression) {
					// [Expression] Attrbibute constant value
					updateValueCmd = new AddOpaqueExpressionDefaultValueToRepositoryBOMCmd((Repository)workingCopy);
				} else if (value instanceof InstanceValue) {					
					updateValueCmd = new AddInstanceValueDefaultValueToRepositoryBOMCmd((Repository)workingCopy);	
				}
			
				if (updateValueCmd.canExecute()) {
					try {
						updateValueCmd.execute();
					} catch (RuntimeException e) {
						LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
					}
				} else {
					LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
				}
				
				// After creation
				Object newDomainModel = updateValueCmd.getObject();
				if (newDomainModel != null) {
					UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
					rule.setSource(value);
					rule.setWorkingCopy(newDomainModel);
					rule.setProjectName(getProjectName());
					rule.setImportSession(getImportSession());
					rule.invoke();
				}		
			}	
			
			// 20050419 - create the UID hashmap for ADF Import
			if (variable.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(variable.getUid(), ((Variable) domainModel).getUid());
			}
			// end of 20050419 - create the UID hashmap for ADF Import
		}
		
		LoggingUtil.traceExit(this, "invoke");
	}
}
