package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.gef.DomainViewObjectHelper;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.bom.command.processes.activities.UpdatePinBOMCmd;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.cef.model.CommonContainerModel;


/**
 * Update the Control Action
 */
public class UpdateControlActionRule extends UpdateActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	protected final int DEFAULT_BRANCH = 2;		

	/**
	 * Return the first in branch view model of this action
	 * @param model
	 * @return
	 */
	protected EObject getInBranchFromViewAction() {
		if (domainModel instanceof ControlAction) {
			List list = PEDomainViewObjectHelper.getAllInBranch(viewModel);
			if (list.size()>0) {
				return (EObject) list.get(0);
			}				
		}
		
		return null;
	}	
	
	/**
	 * Return the first out branch view model of this action
	 * @param model
	 * @return
	 */
	protected EObject getOutBranchFromViewAction() {
		if (domainModel instanceof ControlAction) {
			List list = PEDomainViewObjectHelper.getAllOutBranch(viewModel);
			if (list.size()>0) {
				return (EObject) list.get(0);
			}				
		}
		
		return null;
	}

	/**
	 * Get the pin view object. 
	 * @param pin
	 * @param list branches
	 * @return
	 */
	protected EObject getViewObjectByPin(final Object pin, final List list) {
		
		/*
		if (pin instanceof OutputObjectPin || pin instanceof OutputControlPin) {
			return PEDomainViewObjectHelper.getViewOutputPinFromDomainOutputPin(domainModel, (EObject) pin);
		} else if (pin instanceof InputObjectPin || pin instanceof InputControlPin) {
			return PEDomainViewObjectHelper.getViewInputPinFromDomainInputPin(domainModel, (EObject) pin);	
		}
		return null;
		*/
		
		Iterator iterator = list.iterator();
		Iterator compositeIterator = null;
		Object branch = null;
		EObject viewObject = null;
		while (iterator.hasNext()) {
			branch = iterator.next();
			if (branch instanceof CommonContainerModel) {
				compositeIterator =((CommonContainerModel)branch).getCompositionChildren().iterator();
				while (compositeIterator.hasNext()) {
					viewObject = (EObject) compositeIterator.next();
					if (pin == DomainViewObjectHelper.getDomainObject(viewObject)) {
						return viewObject;
					}					
				} 
			}
		}
		return null;
	}
	
	/**
	 * Update the name of the pins
	 */
	protected void updatePinName() {
		int size = 0;
		
		EList newList = ((Action)domainModel).getInputObjectPin();
		EList originalList = action.getInputObjectPin();
		Pin newPin = null;
		Pin originalPin = null;
		UpdatePinBOMCmd cmd = null;		
		if (originalList.size() > newList.size()) {
			size = newList.size();			
		} else {
			size = 	originalList.size();			
		}
		
		for (int i=0; i < size; i++) {
			newPin = (Pin) newList.get(i);
			originalPin = (Pin) originalList.get(i);	
			if (originalPin.getName() == null) {
				continue;
			}
			cmd = new UpdatePinBOMCmd(newPin);
			cmd.setName(originalPin.getName());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			}	
		}
		
		newList = ((Action)domainModel).getInputControlPin();
		originalList = action.getInputControlPin();	
		if (originalList.size() > newList.size()) {
			size = newList.size();			
		} else {
			size = originalList.size();			
		}
		
		for (int i=0; i < size; i++) {
			newPin = (Pin) newList.get(i);
			originalPin = (Pin) originalList.get(i);	
			if (originalPin.getName() == null) {
				continue;
			}			
			cmd = new UpdatePinBOMCmd(newPin);
			cmd.setName(originalPin.getName());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			}	
		}		
		
		newList = ((Action)domainModel).getOutputObjectPin();
		originalList = action.getOutputObjectPin();	
		if (originalList.size() > newList.size()) {			
			size = newList.size();					
		} else {
			size = originalList.size();
		}
		
		for (int i=0; i < size; i++) {
			newPin = (Pin) newList.get(i);
			originalPin = (Pin) originalList.get(i);
			if (originalPin.getName() == null) {
				continue;
			}
			cmd = new UpdatePinBOMCmd(newPin);
			cmd.setName(originalPin.getName());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			}	
		}			
		
		newList = ((Action)domainModel).getOutputControlPin();
		originalList = action.getOutputControlPin();	
		if (originalList.size() > newList.size()) {
			size = newList.size();			
		} else {			
			size = 	originalList.size();
		}
		
		for (int i=0; i < size; i++) {
			newPin = (Pin) newList.get(i);
			originalPin = (Pin) originalList.get(i);
			if (originalPin.getName() == null) {
				continue;
			}	
			cmd = new UpdatePinBOMCmd(newPin);
			cmd.setName(originalPin.getName());
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
			}	
		}						
	}
}
