package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.compoundcommand.pe.branch.add.AddInBranchToControlActionPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.factory.PeCmdFactory;
import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToInputValuePinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.convert.ConvertToRetrieveArtifactPinPeCmd;
import com.ibm.btools.blm.compoundcommand.pe.pin.update.UpdateObjectPinPeCmd;
import com.ibm.btools.blm.compoundcommand.process.util.PEDomainViewObjectHelper;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.rule.informationModel.UpdateValueSpecificationRule;
import com.ibm.btools.blm.ie.imprt.rule.processModel.UpdatePinRule;
import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.imprt.rule.util.NavigatorUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.bom.command.artifacts.AddInstanceValueToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralNullToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddLiteralStringToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.artifacts.AddOpaqueExpressionToInputValuePinBOMCmd;
import com.ibm.btools.bom.command.processes.activities.UpdateRetrieveArtifactPinBOMCmd;
import com.ibm.btools.bom.model.artifacts.InstanceSpecification;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.expression.bom.command.AddStructuredOpaqueExpressionToInputValuePinBEXCmd;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.model.modelmanager.util.AddUpdateObjectCommand;

/**
 * Update the Join
 */
public class UpdateJoinRule extends UpdateSinglePinSetControlActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();

		updateSinglePinSets();
		
					
		LoggingUtil.traceExit(this, "updateModel");
	}

	/**
	 * Create pins
	 */
	protected void createPin() {
		LoggingUtil.traceEntry(this, "createPin");
		EObject outBranch = getOutBranchFromViewAction();

		Pin eachPin = null;
		Pin newPin = null;
		UpdatePinRule inputObjectPinRule = null;
		UpdatePinRule inputControlPinRule = null;
		
		Iterator pinIterator = action.getOutputObjectPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create output object pin");		
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputObjectPinRule = new UpdatePinRule();
			inputObjectPinRule.setImportSession(getImportSession());
			inputObjectPinRule.setProjectName(getProjectName());
			inputObjectPinRule.setSource(eachPin);
			inputObjectPinRule.setWorkingCopy(outBranch);
			inputObjectPinRule.setProcess(process);
			inputObjectPinRule.setAction(action);
			inputObjectPinRule.invoke(); 
		}	
		
		pinIterator = action.getOutputControlPin().iterator();
		LoggingUtil.trace(this, "createPin", "Create output control pin");	
		while(pinIterator.hasNext()) {
			eachPin = (Pin) pinIterator.next();
			inputControlPinRule = new UpdatePinRule();
			inputControlPinRule.setImportSession(getImportSession());
			inputControlPinRule.setProjectName(getProjectName());
			inputControlPinRule.setSource(eachPin);
			inputControlPinRule.setWorkingCopy(outBranch);
			inputControlPinRule.setProcess(process);
			inputControlPinRule.setAction(action);
			inputControlPinRule.invoke(); 
		}
		
		// Put the output pins in the context
		List inBranches = PEDomainViewObjectHelper.getAllInBranch(viewModel);
		
		ImportContext context = getImportSession().getContext().getSubContext(IImportFramework.CONTEXT_PROCESS_KEY).getSubContext(process);
		EObject pinView = null;			
		
		EList list = action.getInputObjectPin();
		EList newList = ((Action)domainModel).getInputObjectPin();	
		LoggingUtil.trace(this, "createPin", "Get input object pin");	
		if (list.size() <= newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				if (context.get(eachPin) == null) {
					newPin = (Pin) newList.get(i);
					pinView = getViewObjectByPin(newPin, inBranches);
					if (pinView != null) {
						//first update the multiplicity on the pin
						UpdateObjectPinPeCmd pinCmd = getPeCmdFactory().buildUpdateObjectPinPeCmd(pinView);
						// Attribute lowerBound
						int lowerBound = 1; // Default in Tool and BOM
						int upperBound = 1; // Default in Tool and BOM
						ValueSpecification value = null;		
						value = ((InputObjectPin)eachPin).getLowerBound();
						if (value instanceof LiteralInteger) {
							lowerBound = ((LiteralInteger)value).getValue().intValue();
						}
						LoggingUtil.trace(this, "invoke", "Set lower bound.");			
						pinCmd.setLowerbound(lowerBound);
						// Attribute upperBound
						value = ((InputObjectPin)eachPin).getUpperBound();
						if (value instanceof LiteralInteger) {
							upperBound = ((LiteralInteger)value).getValue().intValue();
							pinCmd.setUpperbound(upperBound);
						} else if (value instanceof LiteralUnlimitedNatural) {
							pinCmd.setUpperbound(-1);
						} else {
							pinCmd.setUpperbound(upperBound);
						}
						//d97947- jie: missing update these two attributes.
						pinCmd.setIsOrdered(((InputObjectPin)eachPin).getIsOrdered().booleanValue());
						pinCmd.setIsUnique(((InputObjectPin)eachPin).getIsUnique().booleanValue());					
						if (pinCmd.canExecute()) {
							try {
								pinCmd.execute();
							} catch (RuntimeException e) {
								LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							}
						} else {
							LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
						}
					

						if (eachPin instanceof InputValuePin) {
							ConvertToInputValuePinPeCmd convertCmd = getPeCmdFactory().buildConvertToInputValuePinPeCmd(pinView);
							if (convertCmd.canExecute()) {
								convertCmd.execute();
								pinView = convertCmd.getNewViewModel();
								EObject pinDomain = PEDomainViewObjectHelper.getDomainObject(pinView);
								addAttributeValue(eachPin, (InputValuePin)pinDomain);
							}
						} else if (eachPin instanceof RetrieveArtifactPin) {
							ConvertToRetrieveArtifactPinPeCmd convertCmd = getPeCmdFactory().buildConvertToRetrieveArtifactPinPeCmd(pinView);
							if (convertCmd.canExecute()) {
								convertCmd.execute();
								pinView = convertCmd.getNewViewModel();
								EObject pinDomain = PEDomainViewObjectHelper.getDomainObject(pinView);
								//update the RetrivedArtifactPin based on the original pin
								UpdateRetrieveArtifactPinBOMCmd updatePinCmd = new UpdateRetrieveArtifactPinBOMCmd((RetrieveArtifactPin)pinDomain);
								updatePinCmd.setAtBeginning(((RetrieveArtifactPin)eachPin).getAtBeginning().booleanValue());
								updatePinCmd.setIsRemove(((RetrieveArtifactPin)eachPin).getIsRemove().booleanValue());
								if (updatePinCmd.canExecute()) {
									try {
										updatePinCmd.execute();
									} catch (RuntimeException e) {
										LoggingUtil.trace(this, "invoke", IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
									}
								} else {
									LoggingUtil.trace(this, "invoke", IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
								}
							}
						}
						context.put(eachPin, pinView);
					}
				}
			}
		}		
		
		list = action.getInputControlPin();
		newList = ((Action)domainModel).getInputControlPin();	
		LoggingUtil.trace(this, "createPin", "Get input control pin");	
		if (list.size() <= newList.size()) {
			for (int i=0; i<list.size(); i++) {
				eachPin = (Pin) list.get(i);
				if (context.get(eachPin) == null) {
					newPin = (Pin) newList.get(i);
					pinView = getViewObjectByPin(newPin, inBranches);
					if (pinView != null) {
						context.put(eachPin, pinView);
					}
				}
			}			
		}
		
		//TN - March10 fixes to update the pin name after creating the pin, not update the name in the pinset
		updatePinName();
		//end of TN - March10 fixes to update the pin name after creating the pin, not update the name in the pinset
			
		LoggingUtil.traceExit(this, "createPin");		
	}
	
	/**
	 * Create the pin set
	 */
	protected void createPinSet() {
		LoggingUtil.traceEntry(this, "createPinSet");
		// one inputSet, one outputSet
		// determine #in-branch by
		// inputControlPin = ax, outputControlPin = a
		// inputObjectPin = bx, outputObjectPin = b
		
		EList inputControlPinList = action.getInputControlPin();
		EList inputObjectPinList = action.getInputObjectPin();		
		EList outputControlPinList = action.getOutputControlPin();
		EList outputObjectPinList = action.getOutputObjectPin();
		
		int inputControlPinSize = inputControlPinList.size();		
		int inputObjectPinSize = inputObjectPinList.size();
		int outputControlPinSize = outputControlPinList.size();		
		int outputObjectPinSize = outputObjectPinList.size();		
		int inBranch = 0;
		
		if (outputObjectPinSize != 0) {
			inBranch = inputObjectPinSize / outputObjectPinSize;
		} else if (outputControlPinSize != 0) {
			inBranch = inputControlPinSize / outputControlPinSize;			
		}			

		if (inBranch > DEFAULT_BRANCH) {
			NavigationProjectNode projectNode = NavigatorUtil.getProjectNode(getProjectName());
			if (projectNode == null) {
				LoggingUtil.traceExit(this, "createPinSet");											
				return;
			}
			PeCmdFactory cmdFactory = getPeCmdFactory();			
			AddInBranchToControlActionPeCmd cmd = null; 
			for (int i=0; i< inBranch - DEFAULT_BRANCH; i++) {
				cmd = cmdFactory.buildAddInBranchToControlActionPeCmd(viewModel);
				if (cmd.canExecute()) {
					try {
						cmd.execute();
					} catch (RuntimeException e) {
						logWarning(getImportSession(), 
							IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
							IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
							break;
					}
				} else {
					logWarning(getImportSession(), 
						IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
						IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);	
					break;			
				}		
			}
		}
		
		//TN - fixes on Mar10 to update the pin name for join
		//updatePinName();
		// end of TN - fixes on Mar10 to update the pin name for join
		
		LoggingUtil.traceExit(this, "createPinSet");		
	}
	
	private void addAttributeValue(Pin pin, Pin theNewPin) {
		Iterator iterator = ((InputValuePin)pin).getValue().iterator();
		ValueSpecification value = null;
		AddUpdateObjectCommand cmd = null;
		InstanceSpecification spec = null;
		while(iterator.hasNext()) {
			value = (ValueSpecification) iterator.next();	
			if (value instanceof LiteralNull) {
				cmd = new AddLiteralNullToInputValuePinBOMCmd((InputValuePin)theNewPin);			
			} else if (value instanceof LiteralSpecification) {
				cmd = new AddLiteralStringToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof StructuredOpaqueExpression){
				cmd = new AddStructuredOpaqueExpressionToInputValuePinBEXCmd((InputValuePin)theNewPin);				
			} else if (value instanceof OpaqueExpression) {
				// [Expression] Attrbibute constant valueb
				cmd = new AddOpaqueExpressionToInputValuePinBOMCmd((InputValuePin)theNewPin);
			} else if (value instanceof InstanceValue) {
				cmd = new AddInstanceValueToInputValuePinBOMCmd((InputValuePin)theNewPin);
			
			}
			
			if (cmd.canExecute()) {
				try {
					cmd.execute();
				} catch (RuntimeException e) {
					LoggingUtil.trace(this, "invoke",
						IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);
				}
			} else {
				LoggingUtil.trace(this, "invoke", 
					IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);				
			}
			
			// After creation
			Object newDomainModel = cmd.getObject();
			if (newDomainModel != null) {
				UpdateValueSpecificationRule rule = new UpdateValueSpecificationRule();
				rule.setSource(value);
				rule.setWorkingCopy(newDomainModel);
				rule.setProjectName(getProjectName());
				rule.setImportSession(getImportSession());
				rule.invoke();
			}						
		}
	}	
}
