package com.ibm.btools.blm.ie.imprt.rule.processModel.node;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//import java.util.List;

import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.blm.ie.imprt.rule.util.IImportResultMessages;
import com.ibm.btools.blm.ie.imprt.rule.util.LoggingUtil;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.command.compound.CreateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.bom.command.processes.actions.UpdateTimerActionBOMCmd;
import com.ibm.btools.bom.command.time.UpdateTimeIntervalsBOMCmd;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.processes.actions.TimerAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.emf.cf.BtCompoundCommand;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.CloseWorkingSetCmd;
import com.ibm.btools.model.modelmanager.SaveWorkingSetCmd;
import com.ibm.btools.blm.ie.imprt.rule.util.BOMUtil;

/**
 * Update the TimerAction
 */
public class UpdateTimerActionRule extends UpdateObservationActionRule {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private final String NAMESPACE_SEP = "_";
	private final String NAME_PREFIX = "$";
	private final String NAME_SUFFIX = "_recurringTime";
	
	/**
	 * Update Model
	 */
	protected void updateModel() {		
		LoggingUtil.traceEntry(this, "updateModel");
		super.updateModel();
		if (action instanceof TimerAction && domainModel instanceof TimerAction) {		
			TimerAction timerAction = (TimerAction) action;
			TimerAction dmnMdlTimerAction = (TimerAction) domainModel;
			
			TimeIntervals timeInts = timerAction.getRecurringTime();
			if (timeInts == null) return;
			
			TimeIntervals workingTimeIntervals = null;
			BtCompoundCommand wrapperCmd = new BtCompoundCommand();
			AbstractChildContainerNode resCatalogNode = (AbstractChildContainerNode)getImportSession().getContext().get(((RecurringTimeIntervals)timeInts.getRecurringTimeIntervals().get(0)).getOwningPackage());
			if(resCatalogNode == null){
				//can not find the resource model, can not create the TimeIntervals
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.OBJECT_REFERENCE_NOT_FOUND_EXCEPTION, new String[]{"Resource Catalog", timeInts.getOwningPackage().getName()}, null);
				//can not create the TimeIntervals that will contain all the RecurringTimeInterval ExemptPeriods, return 
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.EXEMPT_PERIOD_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{timeInts.getName()}, null, "Can not retrieve the resourceCatalog");
				LoggingUtil.traceExit(this, "updateExemptPeriods");
				return;
			}
			
			String intervalsURI = createTimeIntervals(resCatalogNode, wrapperCmd);
			if(intervalsURI == null){
				//could not create the TimeIntervals, return.
				LoggingUtil.logWarning(getImportSession(), IeMessageKeys.EXEMPT_PERIOD_TIME_INTERVALS_CREATION_EXCEPTION, new String[]{timeInts.getName()}, null, "IntervalsURI is null");
				LoggingUtil.traceExit(this, "updateExemptPeriods");
				return;
			}
			OpenRootObjectForUpdateBOMCmd openCmd = new OpenRootObjectForUpdateBOMCmd();
			openCmd.setProjectName(getProjectName());
			openCmd.setROBLM_URI(intervalsURI);
			if(!wrapperCmd.appendAndExecute(openCmd)){
				wrapperCmd.undo();
			}else{
				// need to get copy id for saving
				String copyID = openCmd.getCopyID();
				workingTimeIntervals = (TimeIntervals)openCmd.getROCopy();
				//now update the TimeIntervals with the exemptPeriods 
				UpdateTimeIntervalsBOMCmd updateIntervalsCmd = new UpdateTimeIntervalsBOMCmd(workingTimeIntervals);
				
				List recTIs = timeInts.getRecurringTimeIntervals();
				if (recTIs != null && !recTIs.isEmpty()) {
					AbstractLibraryChildNode recTimeIntervalsNode = (AbstractLibraryChildNode) getImportSession().getContext().get((RecurringTimeIntervals)recTIs.get(0));
					RecurringTimeIntervals workingRecInterval =(RecurringTimeIntervals)BOMUtil.loadRootObject(recTimeIntervalsNode);
					updateIntervalsCmd.addRecurringTimeIntervals(workingRecInterval);
					wrapperCmd.appendAndExecute(updateIntervalsCmd);
				}
			
				SaveWorkingSetCmd saveCmd = new SaveWorkingSetCmd();
				saveCmd.setProjectName(getProjectName());
				String projectPath = FileMGR.getProjectPath(getProjectName());
				saveCmd.setProjectPath(projectPath);
				saveCmd.setWorkingSetID(copyID);
				if(!wrapperCmd.appendAndExecute(saveCmd)){
					wrapperCmd.undo();
				}
								
				UpdateTimerActionBOMCmd cmd = new UpdateTimerActionBOMCmd(dmnMdlTimerAction);
				TimeIntervals openedTimeIntervals = (TimeIntervals)BOMUtil.loadRootObject(intervalsURI, getProjectName());
				cmd.setRecurringTime(openedTimeIntervals); 
				wrapperCmd.appendAndExecute(cmd);	
				
				if(!wrapperCmd.appendAndExecute(cmd)){
					wrapperCmd.undo();				
				}			
				closeWorkingCopy(copyID);
			}
			
			// 20050413 - create the UID hashmap for ADF Import
			if (timerAction.getUid() != null) {
				HashMap processHashMap = BOMUtil.getUIDMap(getImportSession().getContext());
				// track the source process elements UID as 'key' and its corresponding working copy process element UID as 'value'
				processHashMap.put(timerAction.getUid(), dmnMdlTimerAction.getUid());
			}
			// end of 20050413 - create the UID hashmap for ADF Import	
		}		

		// TODO: [Miltstone 5] recurringTime - not for Beta
		LoggingUtil.traceExit(this, "updateModel");
	}		
	
	/**
	 * Close the workingcopy by id
	 * @param copyID
	 */
	protected void closeWorkingCopy(final String copyID) {
		LoggingUtil.traceEntry(this, "closeWorkingCopy");		
		if (copyID == null) {
			LoggingUtil.trace(this, "closeWorkingCopy", "The copyID should not be null.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");			
			return;
		}
		CloseWorkingSetCmd closeCmd = new CloseWorkingSetCmd();
		closeCmd.setWorkingSetID(copyID);
		if(closeCmd.canExecute()) {
			try {			
				closeCmd.execute();
			} catch (RuntimeException e) {
				LoggingUtil.trace(this, "closeWorkingCopy", "The close command throws a RuntimeException.");
				LoggingUtil.traceExit(this, "closeWorkingCopy");
				return;
			}
		} else {
			LoggingUtil.trace(this, "closeWorkingCopy", "The close command cannot be executed.");
			LoggingUtil.traceExit(this, "closeWorkingCopy");
			return;
		}
		LoggingUtil.traceExit(this, "closeWorkingCopy");		
	}
		
	private String createTimeIntervals(AbstractChildContainerNode node, BtCompoundCommand wrapperCmd) {
		LoggingUtil.traceEntry(this, "createTimeIntervals");
		
		// TN - experiment ... March 11 on the timer timetable's name
		//final String timeIntervalsName = NAME_PREFIX + getQualifiedName(action) + NAME_SUFFIX;
		final String timeIntervalsName = NAME_PREFIX + getQualifiedName((TimerAction) domainModel) + NAME_SUFFIX;
		// end of TN - experiment ... March 11 on the timer timetable's name
		
		CreateTimeIntervalsBOMCmd createCmd = new CreateTimeIntervalsBOMCmd();
		createCmd.setParentModelBLM_URI((String)node.getEntityReference());
		createCmd.setProjectName(node.getProjectNode().getLabel());
		//create an unique name for the exempt period.. unique exempt period/recurringInterval...
		//TODO !!!!!!! add precondition for interval name and not execute the command otherwise !!!!!
		createCmd.setName(timeIntervalsName);
		if(!wrapperCmd.appendAndExecute(createCmd)){
			wrapperCmd.undo();
		}
	
		LoggingUtil.traceExit(this, "createTimeIntervals");
		return createCmd.getROBLM_URI();
	}
	
	private void mapRecurringTime(TimerAction timerAction, TimerAction dmnMdlTimerAction) {		
		UpdateTimerActionBOMCmd cmd = new UpdateTimerActionBOMCmd(dmnMdlTimerAction);
		cmd.setRecurringTime(timerAction.getRecurringTime()); 
		
		if (cmd.canExecute()) {
			try {
				cmd.execute();
			} catch (RuntimeException e) {
				logWarning(getImportSession(), 
					IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);								
			}
		} else {
			logWarning(getImportSession(), 
				IeMessageKeys.FAIL_TO_UPDATE_ELEMENT, new String[]{getElement(), getName(), getProcessName()}, null, 
				IImportResultMessages.CANNOT_EXECUTE_BOM_COMMAND);			
		}		
	}
	
	/**
	 * Return the BOM namespace for the given element.
	 * @param packageElement
	 * @return
	 */
	private String getQualifiedName(final Action action) {
		
		LoggingUtil.traceEntry(this, "getQualifiedName(final Action action)", "action -->, " + action);

		final StringBuffer sb = new StringBuffer();
		if (action != null) {
			NamedElement element = action;
			EObject parent = null;
			while (element != null) {
				sb.insert(0, element.getName());
				sb.insert(0, NAMESPACE_SEP);
				parent = element;
				if (element instanceof PackageableElement) {
					element = ((PackageableElement)element).getOwningPackage();
				} else {
					element = (NamedElement)element.eContainer();
				}
			}
		}
		return stripSpecialChars(sb.toString());
	}
	
	/**
	 * Replace all ":" with "_".
	 * @param name
	 * @return
	 */
	private String stripSpecialChars(final String name) {
		LoggingUtil.traceEntry(this, "stripSpecialChars", "name -->, " + name);

		String newName = null;
		if (name != null) {
			newName = name.replace(':', '_');
		}
		return newName;
	}
}