package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.text.MessageFormat;

import org.eclipse.core.runtime.Plugin;

import com.ibm.btools.blm.ie.IePlugin;
import com.ibm.btools.blm.ie.imprt.ImportMessage;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.util.logging.LogHelper;

/**
 * This utility class contains static methods used for logging information.
 */
public class LoggingUtil {
	//	copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	// use plugin name as the component name
	// This name must be registered in logging.xmi (com.ibm.btools.util) for tracing to occur.
	static final String COMPONENT_NAME = "com.ibm.btools.blm.ie";//"Modeler Import and Export Framework";
	
	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving
	 * @param plugin			Plugin
	 * @param theClass			Message key class
	 */
	public static void logError(ImportSession session, final String errorCode, final String[] param, final Throwable exception, 
		final String additionalInfo, final Plugin plugin, final Class theClass) {
		LogHelper.log(LogHelper.ERROR, plugin, theClass, errorCode, param, exception, additionalInfo);
		String message = getLocalizedString(errorCode, param, theClass);
		if (session != null) {
			ImportMessage importMessage = new ImportMessage(ImportMessage.ERROR, message);
			importMessage.setErrorCode(errorCode);
			session.getImportResult().addMessage(importMessage);
		}
	}
	
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving 
	 * @param plugin			Plugin
	 * @param theClass			Message key class 
	 */
	public static void logWarning(ImportSession session, final String errorCode, final String[] param, final Throwable exception, 
		final String additionalInfo, final Plugin plugin, final Class theClass) {
		LogHelper.log(LogHelper.WARNING, plugin, theClass, errorCode, param, exception, additionalInfo);
		String message = getLocalizedString(errorCode, param, theClass);
		if (session != null) {
			ImportMessage importMessage = new ImportMessage(ImportMessage.WARNING, message);
			importMessage.setErrorCode(errorCode);
			session.getImportResult().addMessage(importMessage);
		}
	}	
	
	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving
	 */
	public static void logError(ImportSession session, final String errorCode, final String[] param, final Throwable exception, final String additionalInfo) {
		LogHelper.log(LogHelper.ERROR, IePlugin.getDefault(), IeMessageKeys.class, errorCode, param, exception, additionalInfo);
		String message = getLocalizedString(errorCode, param);
		if (session != null) {
			ImportMessage importMessage = new ImportMessage(ImportMessage.ERROR, message);
			importMessage.setErrorCode(errorCode);
			session.getImportResult().addMessage(importMessage);
		}
	}
	
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logWarning(ImportSession session, final String errorCode, final String[] param, final Throwable exception, final String additionalInfo) {
		LogHelper.log(LogHelper.WARNING, IePlugin.getDefault(), IeMessageKeys.class, errorCode, param, exception, additionalInfo);
		String message = getLocalizedString(errorCode, param);
		if (session != null) {
			ImportMessage importMessage = new ImportMessage(ImportMessage.WARNING, message);
			importMessage.setErrorCode(errorCode);
			session.getImportResult().addMessage(importMessage);
		}
	}
	
	/**
	 * Logs information. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logInfo(ImportSession session, final String errorCode, final String[] param, final Throwable exception, final String additionalInfo) {
		LogHelper.log(LogHelper.WARNING, IePlugin.getDefault(), IeMessageKeys.class, errorCode, param, exception, additionalInfo);
		String message = getLocalizedString(errorCode, param);
		if (session != null) {
			ImportMessage importMessage = new ImportMessage(ImportMessage.INFO, message);
			importMessage.setErrorCode(errorCode);
			session.getImportResult().addMessage(importMessage);
		}
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 */
	public static void logError(ImportSession session, final String errorCode, final String[] param, final Throwable exception) {
		logError(session, errorCode, param, exception, null);
	}
	
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 */
	public static void logWarning(ImportSession session, final String errorCode, final String[] param, final Throwable exception) {
		logWarning(session, errorCode, param, exception, null);		
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logError(ImportSession session, final String errorCode, final String additionalInfo) {
		logError(session, errorCode, null, null, additionalInfo);
	}

	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logWarning(ImportSession session, final String errorCode, final String additionalInfo) {
		logWarning(session, errorCode, null, null, additionalInfo);
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session	  
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 */
	public static void logError(ImportSession session, final String errorCode) {
		logError(session, errorCode, null, null);
	}

	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param session			Import session
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 */
	public static void logWarning(ImportSession session, final String errorCode) {
		logWarning(session, errorCode, null, null);
	}	
	
	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving
	 */
	public static void logError(final String errorCode, final String[] param, final Throwable exception, final String additionalInfo) {
		logError(null, errorCode, param, exception, additionalInfo);
		
	}
	
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logWarning(final String errorCode, final String[] param, final Throwable exception, final String additionalInfo) {
		logWarning(null, errorCode, param, exception, additionalInfo);		
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 */
	public static void logError(final String errorCode, final String[] param, final Throwable exception) {
		logError(null, errorCode, param, exception, null);
	}
	
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 */
	public static void logWarning(final String errorCode, final String[] param, final Throwable exception) {
		logWarning(null, errorCode, param, exception, null);		
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logError(final String errorCode, final String additionalInfo) {
		logError(null, errorCode, null, null, additionalInfo);
	}

	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param additionalInfo	Additional information for problem solving 
	 */
	public static void logWarning(final String errorCode, final String additionalInfo) {
		logWarning(null, errorCode, null, null, additionalInfo);
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 */
	public static void logError(final String errorCode) {
		logError(null, errorCode, null, null);
	}

	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 */
	public static void logWarning(final String errorCode) {
		logWarning(null, errorCode, null, null);
	}		
	
	/**
	 * Trace Entry 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceEntry(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(IePlugin.getDefault(), thisObject, methodName, message, COMPONENT_NAME);			
		}
			
	}

	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceExit(final Object thisObject, final String methodName) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceExit(IePlugin.getDefault(), thisObject, methodName, "", COMPONENT_NAME);
		}
	}		
	
	/**
	 * Trace Entry 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceEntry(final Object thisObject, final String methodName) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(IePlugin.getDefault(), thisObject, methodName, "", COMPONENT_NAME);
		}
			
	}

	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceExit(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceExit(IePlugin.getDefault(), thisObject, methodName, message, COMPONENT_NAME);
		}
	}	
	
	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void trace(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.trace(LogHelper.INFO, IePlugin.getDefault(), thisObject, methodName, message, "", COMPONENT_NAME);
		}
	}
	
	/**
	 * Return ImportMessage
	 * @param errorCode
	 * @return
	 */
	public static ImportMessage getMessage(final String errorCode) {
		String message = LogHelper.getLocalizedString(IeMessageKeys.class, errorCode);		
		ImportMessage importMessage = new ImportMessage(ImportMessage.ERROR, message);
		importMessage.setErrorCode(errorCode);
		return importMessage;
	}
	
	/**
	 * Return Localized Message from IeMessageKeys
	 * @param errorCode
	 * @return
	 */	
	public static String getLocalizedString(final String key, final String[] param, final Class theClass) {
		String localizedMessage = null;
		
		if (param == null) { 
			localizedMessage = LogHelper.getLocalizedString(theClass, key);
		} else {
			localizedMessage = MessageFormat.format(LogHelper.getLocalizedString(theClass, key), param);
		}
		
		return localizedMessage;			
	}	
	
	/**
	 * Return Localized Message from IeMessageKeys
	 * @param errorCode
	 * @return
	 */	
	public static String getLocalizedString(final String key, final String[] param) {		
		return getLocalizedString(key, param, IeMessageKeys.class);			
	}
	
}