package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.util.precondition.NameIsLegal;
import com.ibm.btools.util.resource.CommonErrorMessageKeys;

/**
 * Util class applies restrictions and handles creation of valid name for import 
 */
public class NamingUtil extends NameIsLegal{
	
	//copyright
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	//August 08, 2006 Tom suggested that CVS have problem with "~" 
	//and "_" be the better alternative 
	private static final String STRING_UNDER_SCORE = "_";
	
	private static final char CHAR_UNDER_SCORE = '_';
	
	/**
	 * Returns a valid name for the given roote element name <code>name</code>
	 * Applies all the restrictions for root element name.
	 * <Li>If name is empty, returns "DEFAULT" as name</Li>
	 * <Li>If name is one of the restricted name of system, append "~" at end.</Li>
	 * <Li>If name contains any invalid character, replace the invalid character with "~".</Li>
	 * <Li>If name start with "." or end with ".", replace "." with "~".</Li>
	 * <Li>If name length is longer that maximum allowed, truncate it to maximum.</Li> 
	 * @param session
	 * @param name
	 * @return
	 */
	public static String getValidRootElementName(ImportSession session, String name){
		if ( name == null || name.equals("")){
			LoggingUtil.logWarning(session, CommonErrorMessageKeys.NAME_IS_EMPTY);
			return "DEFAULT";
		}
	
		String newName = name; 
		// continue check the illegal string set
		for (int i = 0; i < illegal_strings.length; i++) {
			if (name.equalsIgnoreCase(illegal_strings[i])) {
				String illegalName = illegal_strings[i];								
				LoggingUtil.logWarning(session, CommonErrorMessageKeys.NAME_CONFLICT_WITH_SYSTEM_RESERVED_NAMES,
					new String[]{illegalName}, null);
				// newName = name + "~";
				newName = name + STRING_UNDER_SCORE;
				break; 				
			}
		}	
			
   		// check the length
	   if (newName.length() > MAX_LENGTH){
	   		newName = newName.trim();
			if(newName.length()> MAX_LENGTH){			
				newName = newName.substring(0, MAX_LENGTH);
				LoggingUtil.logWarning(session, CommonErrorMessageKeys.NAME_IS_TOO_LONG,
					new String[]{name}, null);
			}
	   }		   
	       
	   // check character type
	   int nameLength = newName.length();
	   int type;
	   StringBuffer sb = new StringBuffer();
	   for (int i = 0; i < nameLength; i++) {
	   	char nextChar = newName.charAt(i);
		type = Character.getType(nextChar);
		if (type == Character.CONTROL ||
			type == Character.OTHER_SYMBOL ||
			isIllegalChar(nextChar)) {			
				LoggingUtil.logWarning(session, CommonErrorMessageKeys.NAME_CONTAINS_ILLEGAL_CHARS, new String[]{String.valueOf(nextChar)}, null);			
				//nextChar = '~';
				nextChar = CHAR_UNDER_SCORE;
		}
			sb.append(nextChar);	
	   }
	

		//check the starting and ending chars
		int lastIndex = sb.length()- 1;
		if (sb.charAt(lastIndex)=='.'){
			LoggingUtil.logWarning(
				session, CommonErrorMessageKeys.NAME_CANNOT_END_OR_START_WITH, new String[]{"."}, null);
			//sb.setCharAt(lastIndex, '~');
			sb.setCharAt(lastIndex, CHAR_UNDER_SCORE);
		} 
		if(sb.charAt(0)== '.') {
			LoggingUtil.logWarning(
				session, CommonErrorMessageKeys.NAME_CANNOT_END_OR_START_WITH, new String[]{"."}, null);
			//sb.setCharAt(0, '~');		
			sb.setCharAt(0, CHAR_UNDER_SCORE);
		}
		
		newName = sb.toString();  
		return newName;
		
	}
	
	private static boolean isIllegalChar(char aChar){
		for(int i = 0; i<illegal_chars.length; i++){
			 if(aChar == illegal_chars[i]){
			 	return true;
			 }
		}
		return false;
	}
	
/*
	public static String getValidLeafElementName(ImportSession session, String name){
		String newName = name;
		//check the length
		if (newName.length() > MAX_LENGTH){
			 newName = newName.trim();
			 if(newName.length()> MAX_LENGTH){			
				 newName = newName.substring(0, MAX_LENGTH);
				 LoggingUtil.logWarning(session, IeMessageKeys.NAME_IS_TOO_LONG,
					 new String[]{name}, null);
			 }
		}
		
		return newName;		
	}
*/
}
