package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.osgi.framework.Bundle;

import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.IImportFramework;
import com.ibm.btools.blm.ie.imprt.ImportOptions;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ie.resource.IeMessageKeys;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildContainerNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractChildLeafNode;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationBOCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalModelCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationExternalServiceCatalogsNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationProjectNode;
import com.ibm.btools.blm.ui.navigation.model.NavigationRoot;
import com.ibm.btools.blm.ui.view.BLMdialogsMessageKeys;
import com.ibm.btools.bom.command.compound.CloseRootObjectBOMCmd;
import com.ibm.btools.bom.command.compound.OpenRootObjectForUpdateBOMCmd;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.command.compound.SaveRootObjectBOMCmd;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.models.ExternalModel;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.cef.model.Content;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.modelmanager.ModelMGR;
import com.ibm.btools.model.modelmanager.dependencymanager.Dependency;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyManager;
import com.ibm.btools.model.modelmanager.dependencymanager.DependencyModel;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.ui.navigation.manager.AbstractModelAccessor;
import com.ibm.btools.ui.navigation.manager.NavigationManager;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.btools.util.resource.CommonMessageKeys;
import com.ibm.btools.util.resource.UtilResourceBundleSingleton;

/**
 * Utility class for Navigator model
 */
public class NavigatorUtil {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private static final String ACCESSOR_ID = "com.ibm.btools.blm.ui.navigation.manager.accessor.id";
	//	Skip
	public static final int IGNORE		= 1;
	//Replace
	public static final int OVERWRITE	= 2;
	//Common key for external schema collision handling option
	private final static String SCHEMA_OPTION_KEY = "external_schema_collision_option";
	//Common key for external service collision handling option
	private final static String SERVICE_OPTION_KEY = "external_service_collision_option";
	//Key for the business item catalog the user specify, use default otherwise
	public final static String SOUREC_EXTERNAL_BO_CATALOG_NODE = "source_external_bo_node";
	//Key for the service catalog the user specified, use default otherwise
	public final static String SOUREC_EXTERNAL_SERVICE_CATALOG_NODE = "source_external_service_node";

	public final static String SOUREC_FORM_PROCESS_CATALOG_NODE = "source_process_catalog_node";
	
	//Singleton model accessor
	private static AbstractModelAccessor accessor = null;
	
	
	/**
	 * Return the project node
	 * @param projectName
	 * @return
	 */
	public static NavigationProjectNode getProjectNode(final String projectName) {
		LoggingUtil.traceEntry(NavigatorUtil.class, "getProjectNode");
		// Navigation Model
		NavigationManager navManager = NavigationManager.getManager();
		
		// Get the BLM Navigator root
		
		AbstractModelAccessor accessor = null;
		NavigationRoot navRoot = null;
		
		LoggingUtil.trace(NavigatorUtil.class, "getProjectNode", "Find the root node.");
		accessor = NavigationManager.getManager().getModelAccessor(ACCESSOR_ID);
		if (accessor != null) {
			navRoot = (NavigationRoot) accessor.getRoot();
		}
		
		if (navRoot != null) {		
			EList projectNodes = navRoot.getProjectNodes();
			Iterator iterator = projectNodes.iterator();
			NavigationProjectNode navProjectNode = null;
			
			while(iterator.hasNext()) {
				navProjectNode = (NavigationProjectNode) iterator.next();
				if (projectName != null && projectName.compareTo(navProjectNode.getLabel()) == 0) {
					LoggingUtil.trace(NavigatorUtil.class, "getProjectNode", "Get the project node by name: " + projectName + ".");
					LoggingUtil.traceExit(NavigatorUtil.class, "getProjectNode");
					return navProjectNode;
				}
			}
		}
		
		LoggingUtil.traceExit(NavigatorUtil.class, "getProjectNode");
		return null;		
	}
	
	/**
	 * Save he navigation model
	 * @param projectName
	 */
	public static void save(final String projectName) {
		LoggingUtil.traceEntry(NavigatorUtil.class, "save");
		if(ModelMGR.getModelMGRInstance().isInfraFilesSavingSuspended()){
			return;
		}
		// Navigation Model
		NavigationManager navManager = NavigationManager.getManager();
		
		// Get the BLM Navigator root
		Iterator accIterator = navManager.getModelAccessors().iterator();
//		AbstractModelAccessor accessor = null;
//		NavigationRoot navRoot = null;
//		accessor = NavigationManager.getManager().getModelAccessor(ACCESSOR_ID);
		accessor = getAbstractModelAccessor();
		if (accessor != null) {
			accessor.setProjectName(projectName);
			LoggingUtil.trace(NavigatorUtil.class, "getProjectNode", "Find the navigator accessor by project name: " + projectName + ".");
			accessor.save();
			LoggingUtil.trace(NavigatorUtil.class, "getProjectNode", "Save the navigator.");
		}
		
		LoggingUtil.traceExit(NavigatorUtil.class, "save");
	}
	
	/**
	 * Get the singleton model accessor for saving navigation nodes
	 * @return
	 */
	public static AbstractModelAccessor getAbstractModelAccessor(){
		if(accessor == null){
			accessor = NavigationManager.getManager().getModelAccessor(ACCESSOR_ID);
		}
		return accessor;
	}
	
	
	/**
	 * Get URI from Navigation Node
	 * @param navNode
	 * @return
	 */
	public static String getURI(final AbstractLibraryChildNode navNode) {
		LoggingUtil.traceEntry(NavigatorUtil.class, "getURI");
		String blmURI = null;
		String projectName = null;
		Object object = null;
		if (navNode instanceof AbstractChildLeafNode) {
			LoggingUtil.trace(NavigatorUtil.class, "getURI", "The navigation node is a leaf node.");	
			List blmUriList = ((AbstractChildLeafNode)navNode).getEntityReferences();
			if (blmUriList != null) {
				if (blmUriList.size() > 1) {
					object = blmUriList.get(0);
				} else if (blmUriList.size() > 0) {	
					LoggingUtil.trace(NavigatorUtil.class, "getURI", "Get the object reference from the navigation node.");			
					object = blmUriList.get(0);
				}
			}
		} else if (navNode instanceof AbstractChildContainerNode) {
			LoggingUtil.trace(NavigatorUtil.class, "getURI", "The navigation node is a container node.");
			object = ((AbstractChildContainerNode)navNode).getEntityReference();
		}
		
		if (object instanceof String) {
			LoggingUtil.traceExit(NavigatorUtil.class, "getURI");
			return (String) object;
		}
		LoggingUtil.traceExit(NavigatorUtil.class, "getURI");
		return null;
	}
	
	/**
	 * Get View URI from Navigation Node
	 * @param navNode
	 * @return
	 */
	public static String getViewURI(final AbstractLibraryChildNode navNode) {
		LoggingUtil.traceEntry(NavigatorUtil.class, "getViewURI");
		String blmURI = null;
		String projectName = null;
		Object object = null;
		if (navNode instanceof AbstractChildLeafNode) {
			LoggingUtil.trace(NavigatorUtil.class, "getViewURI", "The navigation node is a leaf node.");
			List blmUriList = ((AbstractChildLeafNode)navNode).getEntityReferences();
			if (blmUriList != null && blmUriList.size() > 1) {
				object = blmUriList.get(1);
			}
		} 
		
		if (object instanceof String) {
			LoggingUtil.traceExit(NavigatorUtil.class, "getViewURI");
			return (String) object;
		}
		LoggingUtil.traceExit(NavigatorUtil.class, "getViewURI");
		return null;
	}	
	
	/**
	 * Get Default Process Catalog Name
	 * @return
	 */
	public static String getDefaultProcessCatalogName() {
		return UtilResourceBundleSingleton.INSTANCE.getMessage(
				BLMdialogsMessageKeys.class, BLMdialogsMessageKeys.QSDefaultProcessCatalogName);
		
	}
	
	/**
	 * Get Default Data Catalog Name
	 * @return
	 */
	public static String getDefaultDataCatalogName() {
		return UtilResourceBundleSingleton.INSTANCE.getMessage(
				BLMdialogsMessageKeys.class, BLMdialogsMessageKeys.QSDefaultDataCatalogName);
		
	}
	
	
	/**
	 * Get Default Organization Catalog Name
	 * @return
	 */
	public static String getDefaultOrganizationCatalogName() {
		return UtilResourceBundleSingleton.INSTANCE.getMessage(
				BLMdialogsMessageKeys.class, BLMdialogsMessageKeys.DefaultOrganizationCatalogName);
		
	}
	
	
	/**
	 * Get Default Resource Catalog Name
	 * @return
	 */
	public static String getDefaultResourceCatalogName() {
		return UtilResourceBundleSingleton.INSTANCE.getMessage(
				BLMdialogsMessageKeys.class, BLMdialogsMessageKeys.DefaultResourceCatalogName);
		
	}
	
	/**
	 * Get Default Process Catalog Id
	 * Note: first step imported process model that want to be treated as default process model must set its model id to this id.
	 * @return
	 */
	public static String getDefaultProcessCatalogId(){
		return PredefTypesConst.PREDEF_PROCESSES_FUID;
	}	
	
	/**
	 * Get Default Classifier Catalog Name
	 * @return
	 */
	public static String getDefaultClassifierCatalogName(){
		return UtilResourceBundleSingleton.INSTANCE.getMessage(CommonMessageKeys.class, CommonMessageKeys.UTIL_CLASSIFIER);
	}	
	
	/**
	 * Get the default catalog for external service in case of no root node supplied
	 * @param projectName
	 * @return
	 */
	
	public static AbstractChildContainerNode getDefaultCatalogForExternalService(String projectName){
		NavigationProjectNode projectNode = getProjectNode(projectName);
		if(projectNode != null){
			NavigationExternalModelCatalogsNode xternalModelCats = projectNode.getLibraryNode().getExternalModelCatalogs();
			if(xternalModelCats != null){
				NavigationExternalServiceCatalogsNode catalogs = xternalModelCats.getExternalServiceCatalogs();
				EList serviceCatalogs = catalogs.getExternalServiceCatalog();
				if(serviceCatalogs != null && serviceCatalogs.size()> 0){
					return (AbstractChildContainerNode)catalogs.getExternalServiceCatalog().get(0);
				}				
			}
		}
		return null;
	}
	
	/**
	 * Get the default catalog for external schema in case of no root node supplied
	 * @param projectName
	 * @return
	 */
	public static AbstractChildContainerNode getDefaultCatalogForExternalSchema(String projectName){
		NavigationProjectNode projectNode = getProjectNode(projectName);
		if(projectNode != null){
			NavigationExternalModelCatalogsNode xternalModelCats = projectNode.getLibraryNode().getExternalModelCatalogs();
			if(xternalModelCats != null){
				NavigationBOCatalogsNode catalogs =  xternalModelCats.getBoCatalogs();
				if(catalogs != null){
					EList boCatalogs = catalogs.getBoCatalog();
					if(boCatalogs != null && boCatalogs.size() > 0){
						return (AbstractChildContainerNode)catalogs.getBoCatalog().get(0);
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * Get the root external model node
	 * @param projectName
	 * @return
	 */
	public static AbstractChildContainerNode getRootExternalModelNode(String projectName){
		NavigationProjectNode projectNode = getProjectNode(projectName);
		if(projectNode != null){
			return  projectNode.getLibraryNode().getExternalModelCatalogs();
		}
		return null;
	}
	
	/**
	 * This method is to get all the depending objects for the node within the project
	 * @param node
	 * @param projectName
	 * @return List of dependant blm uri
	 */
	public static List getDependingObjects(AbstractChildLeafNode node,String projectName){
		List dependency = new ArrayList();
		if(node == null ||projectName == null){
			return dependency;
		}
		String projectUri = FileMGR.getProjectPath(projectName);
		String blmUri = (String)node.getEntityReferences().get(0);
		String projectPath = FileMGR.getProjectPath(projectName);
		DependencyModel dependencyModel = DependencyManager.instance().getDependencyModel(projectName, projectPath);	 	
		List d = dependencyModel.getAllDependencies(null, blmUri);
		if(d.size() > 0){
			for(int i = 0; i<d.size();i++){
				EObject o = ((Dependency)d.get(i)).getSource().getEObject();
				AbstractChildLeafNode navNode = null;
				String blmURI = null;
				if (o instanceof Content) {
					if (ResourceMGR.getResourceManger().getIDRecord((EObject)o) != null) {
						String cefUri = ResourceMGR.getResourceManger().getIDRecord((EObject)o).getId();
						navNode = getLeafNode(projectName, cefUri);						
					}							
				} else {
					//navNode = getLeafNode(projectName, o);
					//defect 85913 to handle cross project reference
					navNode = getLeafNode(ResourceMGR.getResourceManger().getProjectName(o), o);
				}
				if(navNode != null){
					blmURI = (String)navNode.getEntityReferences().get(0);
					dependency.add(blmURI);
				}				
			}
		}
		return dependency;
	}
	
	public static AbstractChildLeafNode getLeafNode(String projectName, Object element) {
		String bomUID = null;
		if (element instanceof NamedElement) {
			bomUID =  ((NamedElement)element).getUid();
		} 
		
		if (bomUID == null || projectName == null) {
			//the element was removed. nothing to do
			return null;
		}
		NavigationProjectNode project = getProjectNode(projectName);
		TreeIterator it = project.eAllContents();
		Object item;
		while (it.hasNext()) {
			item = it.next();
			if (item instanceof AbstractChildLeafNode &&
					bomUID.equals(((AbstractChildLeafNode)item).getBomUID())) {
				return (AbstractChildLeafNode)item;
			}
		}
		//try a second search by RID:
		try {
			String rid = null;
			if (ResourceMGR.getResourceManger().getIDRecord((EObject)element) != null) {
				rid = ResourceMGR.getResourceManger().getIDRecord((EObject)element).getId();
			}
			String nodeRID = null;
			it = project.eAllContents();
			while (it.hasNext()) {
				item = it.next();
				if (item instanceof AbstractChildLeafNode) {
					nodeRID = (String)((AbstractChildLeafNode)item).getEntityReferences().get(0);
					if (rid != null && rid.equals(nodeRID)) {
						return (AbstractChildLeafNode)item;
					} else if (((AbstractChildLeafNode)item).getEntityReferences().size() > 1) {
						nodeRID = (String)((AbstractChildLeafNode)item).getEntityReferences().get(1);
						if (rid != null && rid.equals(nodeRID)) {
							return (AbstractChildLeafNode)item;
						}
					}
					
				}
			}
		} catch (Exception e) {
			//nothing to do; it may be a deleted object
		}
		return null;
	}
	
	/**
	 * Record the isNew flag for the node corresponding to the imported BOM model 
	 * @param session
	 * @param element
	 */
	public static void recordNewNode(ImportSession session,Object element){
		if(session == null || element == null){
			return;
		}
		Map options = session.getImportOptions().getAdditionalOptions();
		if(options != null){
			options.put(element,Boolean.TRUE);
		}		
	}
	
	/**
	 * Retrieve the flag from the session to determine whether to create everything new 
	 * or refactor the BOM model based on some rules
	 * @param session
	 * @param element
	 * @return boolean is new or not
	 */
	public static boolean isNewNode(ImportSession session,Object element){
		if(session == null || element == null){
			throw new RuntimeException("This operation doesn't support null arguments");
		}
		Map options = session.getImportOptions().getAdditionalOptions();
		if(options != null){
			Boolean flag = (Boolean)options.get(element);
			if(flag != null){
				return flag.booleanValue();
			}
		}		
		return false;		
	}	
	
	/**
	 * Retrieve external service name collision option for the service operation
	 * @param options
	 * @param element
	 * @return
	 */
	public static int getExternalServiceOption(ImportOptions options,PackageableElement element){
		if(element instanceof Activity &&
				element.getOwningPackage()!= null){
			if(element.getOwningPackage().getOwningPackage() != null){
				String packageName = element.getOwningPackage().getOwningPackage().getName();
				Map serviceOptionMap = getServiceOptionHash(options);
				Object selection = serviceOptionMap.get(packageName);
				if(selection instanceof Integer){
					return ((Integer)selection).intValue();
				}
			}
		}
		return OVERWRITE;
	}
	
	/**
	 * Retrieve external schema name collision option for the complex type
	 * @param options
	 * @param element
	 * @return
	 */
	public static int getExternalSchemaOption(ImportOptions options,PackageableElement element){
		if( element instanceof com.ibm.btools.bom.model.artifacts.Class &&
				element.getOwningPackage()!= null){
			String packageName = element.getOwningPackage().getName();
			Map schemaOptionMap = getSchemaOptionHash(options);
			Object selection = schemaOptionMap.get(packageName);
			if(selection instanceof Integer){
				return ((Integer)selection).intValue();
			}
		}
		return OVERWRITE;
	}
	
	/**
	 * This method helps look up the user option for the element's containing element
	 * @param options
	 * @param element
	 * @return true if the user chooses Ignore, false otherwise
	 */
	public static boolean isOptionIgnore(ImportOptions options,PackageableElement element){
		int userOption;
		if(element instanceof com.ibm.btools.bom.model.artifacts.Class &&
				BOMUtil.isSchemaType(element)){
			userOption = getExternalSchemaOption(options,element);
			if(userOption == OVERWRITE){
				return false;
			}
			return true;
		}else if(element instanceof Activity ){
			userOption = getExternalServiceOption(options,element);
			if(userOption == OVERWRITE){
				return false;
			}
			return true;
		}
		return false;
	}
	
	/**
	 * This method helps look up the user option for the element's containing element
	 * @param options
	 * @param element
	 * @return true if the user chooses Overwrite, false otherwise
	 */
	public static boolean isOptionOverwrite(ImportOptions options,PackageableElement element){
		int userOption;
		if(element instanceof com.ibm.btools.bom.model.artifacts.Class
				&& BOMUtil.isSchemaType(element)){
			userOption = getExternalSchemaOption(options,element);
			if(userOption == OVERWRITE){
				return true;
			}
			return false;
		}else if(element instanceof Activity ){
			userOption = getExternalServiceOption(options,element);
			if(userOption == OVERWRITE){
				return true;
			}
			return false;
		}
		return true;
	}
	
	/**
	 * Retrieve or initialize Map for the external schema option
	 * @param options
	 * @return
	 */
	public static Map getSchemaOptionHash(ImportOptions options){
		if(options.getAdditionalOptions().get(SCHEMA_OPTION_KEY) == null){
			initSchemaOptionHash(options);
		}
		return (Map)options.getAdditionalOptions().get(SCHEMA_OPTION_KEY);
	}
	
	/**
	 * Retrieve or initialize Map for the external service option
	 * @param options
	 * @return
	 */
	public static Map getServiceOptionHash(ImportOptions options){
		if(options.getAdditionalOptions().get(SERVICE_OPTION_KEY) == null){
			initServiceOptionHash(options);
		}
		return (Map)options.getAdditionalOptions().get(SERVICE_OPTION_KEY);
	}
	
	private static void initSchemaOptionHash(ImportOptions options){
		options.getAdditionalOptions().put(SCHEMA_OPTION_KEY,new HashMap());
	}
	
	private static void initServiceOptionHash(ImportOptions options){
		options.getAdditionalOptions().put(SERVICE_OPTION_KEY,new HashMap());
	}
	
	/**
	 * This method is responsible for saving any changes after external schema and external service
	 * consolidation
	 * @param containerNode
	 * @param projectName
	 */
	public static void saveNode(AbstractChildContainerNode containerNode, String projectName){
		if(containerNode == null || projectName == null){
			return;
		}
		String blmUri = (String)containerNode.getEntityReference();
		OpenRootObjectForUpdateBOMCmd roOpenCmd = new OpenRootObjectForUpdateBOMCmd();
		roOpenCmd.setProjectName(projectName);
		roOpenCmd.setROBLM_URI(blmUri);
		if(roOpenCmd.canExecute()){
			roOpenCmd.execute();
		}
		String copyId = roOpenCmd.getCopyID();
		SaveRootObjectBOMCmd saveRootObjectBOMCmd = new SaveRootObjectBOMCmd();
		saveRootObjectBOMCmd.setProjectName(projectName);
		saveRootObjectBOMCmd.setCopyID(copyId);
		
		if(saveRootObjectBOMCmd.canExecute()){
			saveRootObjectBOMCmd.execute();
		}
        if (copyId != null) {
            CloseRootObjectBOMCmd roCloseCmd = new CloseRootObjectBOMCmd();
            roCloseCmd.setCopyID(copyId);
            if(roCloseCmd.canExecute()){
                   roCloseCmd.execute();
            }
     }

	}
	
	/**
	 * Get the user specified BO catalog, use the default otherwise
	 * @param option
	 * @param projectName
	 * @return
	 */
	public static AbstractChildContainerNode getSelectedBOCatalog(ImportOptions options,String projectName){
		AbstractChildContainerNode selectedBOcatalogNode = null;
		selectedBOcatalogNode = (AbstractChildContainerNode)options.getAdditionalOption(SOUREC_EXTERNAL_BO_CATALOG_NODE);
		if(selectedBOcatalogNode == null ||!(selectedBOcatalogNode instanceof NavigationBOCatalogNode)){
			selectedBOcatalogNode = NavigatorUtil.getDefaultCatalogForExternalSchema(projectName);
		}
		return selectedBOcatalogNode;
	}
	
	/**
	 * The external model must be designated into either a business object catalog or
	 * a service catalog, otherwise the operation behaviour will be unpredictable, therefore,
	 * we throw BTRuntimeException to abort the import session	 
	 * @return boolean true if it's BO catalog, false otherwise
	 */
	public static boolean isBOCatalogNode(ExternalModel model,ImportSession session){
		if(model == null || session == null){
			BTRuntimeException e = new BTRuntimeException("Method isBOCatalogNode(model,session) parameter(s) cannot be null");
			LoggingUtil.logError(session, IeMessageKeys.NO_CATALOG_DEFINED, new String[]{model.getName()}, e, 
					IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);	
			throw e;
		}
		Object catalogKey = session.getContext().get(IImportConstants.XML_IMPORT_CATALOG_KEY);
		if(catalogKey instanceof Map){
			Map modelCatlogMap = (Map)catalogKey;
			Object boCatalogListObj = modelCatlogMap.get(IImportConstants.XML_IMPORT_BO_CATALOG_KEY);
			if(boCatalogListObj instanceof List){
				List boCatalogList = (List)boCatalogListObj;
				if(boCatalogList.contains(model)){
					return true;
				}
			}
			Object serviceCatalogListObj = modelCatlogMap.get(IImportConstants.XML_IMPORT_SERVICE_CATALOG_KEY);
			if(serviceCatalogListObj instanceof List){
				List serviceCatalogList = (List)serviceCatalogListObj;
				if(serviceCatalogList.contains(model)){
					return false;
				}
			}
		}
		BTRuntimeException e = new BTRuntimeException("Import engine cannot determine the catalog, aborting import session");
		LoggingUtil.logError(session, IeMessageKeys.NO_CATALOG_DEFINED, new String[]{model.getName()}, e, 
				IImportResultMessages.COMMAND_RUNTIME_EXCEPTION);	
		throw e;		
	}
	
	public static boolean isRootCatalogCustomCatalog(ExternalModel model){
		if(model == null){
			return false;
		}
		if(model.getOwningPackage() != null){
			return IImportFramework.ROOT_EXTERNAL_MODEL_NAME.equals(model.getOwningPackage().getName());
		}
		return false;
	}
	
	public static AbstractChildContainerNode getRootBONode(String projectName,ImportSession session){
		NavigationProjectNode projectNode = getProjectNode(projectName);
		if(projectNode != null){			
			return projectNode.getLibraryNode().getExternalModelCatalogs().getBoCatalogs();
		}
		return null;
	}
	
	public static AbstractChildContainerNode getRootServiceNode(String projectName,ImportSession session){
		NavigationProjectNode projectNode = getProjectNode(projectName);
		if(projectNode != null){			
			return projectNode.getLibraryNode().getExternalModelCatalogs().getExternalServiceCatalogs();
		}
		return null;
	}
	
	public static AbstractChildContainerNode getContainerNode(String projectName, String bomUID, Object o) {
		try{
			// Load the plugin
			Bundle plugin = Platform.getBundle("com.ibm.btools.blm.ui.navigation");

			// Just in case it doesn't exist
			if (plugin == null) 
				return null;

			Class params[] = {String.class,String.class,Object.class};

			Class BLMManagerUtilClass = plugin.loadClass("com.ibm.btools.blm.ui.navigation.manager.util.BLMManagerUtil");

			java.lang.reflect.Method method_getNode = BLMManagerUtilClass.getMethod("getContainerNode", params);

			Object args[] = {projectName,bomUID,null};

			Object value = method_getNode.invoke(BLMManagerUtilClass, args);

			if(value instanceof AbstractChildContainerNode){
				return (AbstractChildContainerNode)value;
			}

		}catch(ClassNotFoundException cnfe){			
			return null;
		}catch(NoSuchMethodException nsme){
			return null;
		}catch(Exception e){
			return null;
		}
		return null;	
	}
}