package com.ibm.btools.blm.ie.imprt.rule.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2004, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.blm.ie.imprt.ImportContext;
import com.ibm.btools.blm.ie.imprt.ImportSession;
import com.ibm.btools.blm.ui.navigation.model.AbstractLibraryChildNode;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.ArtifactsFactory;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.models.ModelsFactory;
import com.ibm.btools.bom.model.models.OrganizationModel;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationstructuresFactory;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.ResourcesFactory;
import com.ibm.btools.ui.widgets.PrimitiveTypeValueDisplayHelper;
import com.ibm.btools.util.datatype.BTDataTypeManager;
import com.ibm.btools.util.datatype.IBTDataTypeRegistry;

/**
 * Utility class for BOM
 */
public class PredefinedTypeUtil {
	// copyright	
	static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2004, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	// [Code] Resource Type - wait for new changes

	private static boolean initialize = false;

	private static Map types;	
	private static Map resourceTypes;
	private static Map organizationTypes;
	//BEGIN: TN20050601 - additional support for classifiers
	private static Map classifiers;
	//END: TN20050601 - additional support for classifiers
	//BEGIN: TN20050601 - additional support for classifier value
	private static Map classifierValues;
	//END: TN20050601 - additional support for classifier value

	private static String[] typeNames = {
		IBTDataTypeRegistry.PREDEF_PRIMITIVE,
		IBTDataTypeRegistry.PREDEF_BOOLEAN,
		IBTDataTypeRegistry.PREDEF_NUMBER,
		IBTDataTypeRegistry.PREDEF_INTEGERNUMBER,
		IBTDataTypeRegistry.PREDEF_LONG,
		IBTDataTypeRegistry.PREDEF_INTEGER,
		IBTDataTypeRegistry.PREDEF_REALNUMBER,
		IBTDataTypeRegistry.PREDEF_DOUBLE,
		IBTDataTypeRegistry.PREDEF_DATE,		
		IBTDataTypeRegistry.PREDEF_STRING,
		IBTDataTypeRegistry.PREDEF_TIME,
		IBTDataTypeRegistry.PREDEF_DATETIME,
		IBTDataTypeRegistry.PREDEF_DURATION,
		IBTDataTypeRegistry.PREDEF_SHORT,
		IBTDataTypeRegistry.PREDEF_BYTE,
		IBTDataTypeRegistry.PREDEF_FLOAT
	};

	/**
	 * init
	 *
	 */
	private static void init() {
		if (initialize) {
			return;
		}
		initialize = true;	

		initPredefinedTypes();
		initPredefinedResourceTypes();
		initPredefinedOrganizationTypes();
		updatePredefinedTypes();

		// The classifiers has to be created before the classifier values
		//BEGIN: TN20050601 - additional support for classifiers
		initPredefinedClassifiers();
		//END: TN20050601 - additional support for classifiers

		//BEGIN: TN20050601 - additional support for classifier value
		initPredefinedClassifierValues();
		//END: TN20050601 - additional support for classifier value		
	}

	/**
	 * Create the predefined type	 
	 */
	private static void initPredefinedTypes() {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "initPredefinedTypes");	
		types = new HashMap(typeNames.length);

		PrimitiveType primitiveType = null;

		for (int count=0; count<typeNames.length; count++) {		
			primitiveType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
			primitiveType.setName(typeNames[count]);
			if (typeNames[count].compareTo(IBTDataTypeRegistry.PREDEF_PRIMITIVE) != 0) {			
				primitiveType.getSuperClassifier().add(getPredefinedType(IBTDataTypeRegistry.PREDEF_PRIMITIVE));
			}
			LoggingUtil.trace(PredefinedTypeUtil.class, "initPredefinedTypes", "create predefined type: " + typeNames[count] + ".");
			types.put(typeNames[count], primitiveType);
		}
		LoggingUtil.traceExit(PredefinedTypeUtil.class, "initPredefinedTypes");
	}

	//	BEGIN: TN20050601 - additional support for classifier value
	/**
	 * Create the predefined classifier values	 
	 */
	private static void initPredefinedClassifiers() {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "initPredefinedClassifiers()");
		classifiers = new HashMap();

		// NOTE: this predefined classifiers do not have a complete hierrarchy structure, that is: their owningPackage and parents are not set/created
		// NOTE: this predefined classifiers are used for reference only
		OrganizationModel category_ValueAdded = ModelsFactory.eINSTANCE.createOrganizationModel(); 
		category_ValueAdded .setUid   (PredefTypesConst.CATEGORY_INSTANCE_1_FUID);
		category_ValueAdded .setName  (PredefTypesConst.CATEGORY_INSTANCE_1_NAME);
		category_ValueAdded .setAspect(PredefTypesConst.CATEGORY_ASPECT);

		OrganizationModel category_QualityControl = ModelsFactory.eINSTANCE.createOrganizationModel(); 
		category_QualityControl .setUid   (PredefTypesConst.CATEGORY_INSTANCE_2_FUID);
		category_QualityControl .setName  (PredefTypesConst.CATEGORY_INSTANCE_2_NAME);
		category_QualityControl .setAspect(PredefTypesConst.CATEGORY_ASPECT);

		OrganizationModel category_Workflow = ModelsFactory.eINSTANCE.createOrganizationModel(); 
		category_Workflow.setUid	 (PredefTypesConst.CATEGORY_INSTANCE_3_FUID);
		category_Workflow.setName  (PredefTypesConst.CATEGORY_INSTANCE_3_NAME);
		category_Workflow.setAspect(PredefTypesConst.CATEGORY_ASPECT);

		classifiers.put(PredefTypesConst.CATEGORY_INSTANCE_1_NAME, category_ValueAdded);
		classifiers.put(PredefTypesConst.CATEGORY_INSTANCE_2_NAME, category_QualityControl);
		classifiers.put(PredefTypesConst.CATEGORY_INSTANCE_3_NAME, category_Workflow);

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "initPredefinedClassifiers()");	
	}
	//END: TN20050601 - additional support for classifier value		

	//BEGIN: TN20050601 - additional support for classifier value
	/**
	 * Create the predefined classifier values	 
	 */
	private static void initPredefinedClassifierValues() {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "initPredefinedClassifierValues");
		classifierValues = new HashMap();

		// NOTE: this predefined classifier values do not have a complete hierrarchy structure, that is: their owningPackage and parents are not set/created
		// NOTE: this predefined classifier values are used for reference only

		OrganizationModel category_ValueAdded = getPredefinedClassifier(PredefTypesConst.CATEGORY_INSTANCE_1_NAME);
		OrganizationModel category_QualityControl = getPredefinedClassifier(PredefTypesConst.CATEGORY_INSTANCE_2_NAME);				
		OrganizationModel category_Workflow = getPredefinedClassifier(PredefTypesConst.CATEGORY_INSTANCE_3_NAME);

		// BEGIN: Quality Control classifier value
		OrganizationUnit qc = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		qc.setName(PredefTypesConst.CATEGORY_INSTANCE_2_V2_NAME);
		qc.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		qc.setUid(PredefTypesConst.CATEGORY_INSTANCE_2_V2_FUID);
		qc.setOwningPackage(category_QualityControl);
		// END: Quality Control classifier value

		// BEGIN: Not Quality Control classifier value
		OrganizationUnit nqc = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		nqc.setName(PredefTypesConst.CATEGORY_INSTANCE_2_V1_NAME);
		nqc.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		nqc.setUid(PredefTypesConst.CATEGORY_INSTANCE_2_V1_FUID);
		nqc.setOwningPackage(category_QualityControl);
		// END: Not Quality Control classifier value

		// BEGIN: Business Value Added classifier value
		OrganizationUnit bva = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		bva.setName(PredefTypesConst.CATEGORY_INSTANCE_1_V1_NAME);
		bva.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		bva.setUid(PredefTypesConst.CATEGORY_INSTANCE_1_V1_FUID);
		bva.setOwningPackage(category_ValueAdded);
		// END: Business Value Added classifier value

		// BEGIN: Real Value Added classifier value
		OrganizationUnit rva = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		rva.setName(PredefTypesConst.CATEGORY_INSTANCE_1_V3_NAME);
		rva.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		rva.setUid(PredefTypesConst.CATEGORY_INSTANCE_1_V3_FUID);
		rva.setOwningPackage(category_ValueAdded);
		// END: Real Value Added classifier value

		// BEGIN: No value added classifier value
		OrganizationUnit nva = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		nva.setName(PredefTypesConst.CATEGORY_INSTANCE_1_V2_NAME);
		nva.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		nva.setUid(PredefTypesConst.CATEGORY_INSTANCE_1_V2_FUID);
		nva.setOwningPackage(category_ValueAdded);
		// END: No value added classifier value

		// BEGIN: Not Workflow classifier value
		OrganizationUnit nwf = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		nwf.setName(PredefTypesConst.CATEGORY_INSTANCE_3_V2_NAME);
		nwf.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		nwf.setUid(PredefTypesConst.CATEGORY_INSTANCE_3_V2_FUID);
		nwf.setOwningPackage(category_Workflow);
		// END: Not Workflow classifier value

		// BEGIN: Potential Workflow classifier value
		OrganizationUnit pwf = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		pwf.setName(PredefTypesConst.CATEGORY_INSTANCE_3_V3_NAME);
		pwf.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		pwf.setUid(PredefTypesConst.CATEGORY_INSTANCE_3_V3_FUID);
		pwf.setOwningPackage(category_Workflow);
		// END: Potential Workflow classifier value

		// BEGIN: Current Workflow classifier value
		OrganizationUnit cwf = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnit();
		cwf.setName(PredefTypesConst.CATEGORY_INSTANCE_3_V1_NAME);
		cwf.setAspect(PredefTypesConst.CATEGORY_VALUE_INSTANCE_ASPECT);
		cwf.setUid(PredefTypesConst.CATEGORY_INSTANCE_3_V1_FUID);
		cwf.setOwningPackage(category_Workflow);
		// END: Current Workflow classifier value

		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_2_V2_NAME, qc);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_2_V1_NAME,nqc);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_1_V1_NAME,bva);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_1_V3_NAME,rva);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_1_V2_NAME,nva);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_3_V2_NAME,nwf);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_3_V3_NAME,pwf);
		classifierValues.put(PredefTypesConst.CATEGORY_INSTANCE_3_V1_NAME,cwf);

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "initPredefinedResourceTypes");	
	}
	//END: TN20050601 - additional support for classifier value	



	/**
	 * Create the predefined resource type	 
	 */
	private static void initPredefinedResourceTypes() {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "initPredefinedResourceTypes");
		resourceTypes = new HashMap();

		Property property = null;

		// Person_Category
		IndividualResourceType personCategory = ResourcesFactory.eINSTANCE.createIndividualResourceType();
		personCategory.setName(PredefTypesConst.PERSON_CATEGORY_TYPE_NAME);
		personCategory.setIsAbstract(PredefTypesConst.PERSON_CATEGORY_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);
		property = createProperty((PrimitiveType)types.get(IBTDataTypeRegistry.PREDEF_STRING), 
				PredefTypesConst.PERSON_CATEGORY_STRING_ATTR[0], 1, 1);
		personCategory.getOwnedAttribute().add(property);
		personCategory.setUid(PredefTypesConst.PERSON_CATEGORY_TYPE_FUID);

		for (int i=1; i<PredefTypesConst.PERSON_CATEGORY_STRING_ATTR.length; i++) {
			property = createProperty((PrimitiveType)types.get(IBTDataTypeRegistry.PREDEF_STRING), 
					PredefTypesConst.PERSON_CATEGORY_STRING_ATTR[i], 0, 1);
			personCategory.getOwnedAttribute().add(property);
		}

		// Person
		IndividualResourceType person = ResourcesFactory.eINSTANCE.createIndividualResourceType();
		person.setName(PredefTypesConst.PERSON_TYPE_NAME);
		personCategory.setIsAbstract(PredefTypesConst.PERSON_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		person.getSuperClassifier().add(personCategory);
		person.setUid(PredefTypesConst.PERSON_TYPE_FUID);

		// Staff_Category
		IndividualResourceType staffCategory = ResourcesFactory.eINSTANCE.createIndividualResourceType();
		staffCategory.setName(PredefTypesConst.STAFF_CATEGORY_TYPE_NAME);
		staffCategory.setIsAbstract(PredefTypesConst.STAFF_CATEGORY_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		staffCategory.getSuperClassifier().add(personCategory);
		staffCategory.setUid(PredefTypesConst.STAFF_CATEGORY_TYPE_FUID);

		for (int i=0; i<PredefTypesConst.STAFF_CATEGORY_STRING_ATTR.length; i++) {
			property = createProperty((PrimitiveType)types.get(IBTDataTypeRegistry.PREDEF_STRING), 
					PredefTypesConst.STAFF_CATEGORY_STRING_ATTR[i], 0, 1);
			staffCategory.getOwnedAttribute().add(property);
		}

		//Staff
		IndividualResourceType staff = ResourcesFactory.eINSTANCE.createIndividualResourceType();
		staff.setName(PredefTypesConst.STAFF_TYPE_NAME);
		staff.setIsAbstract(PredefTypesConst.STAFF_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		staff.getSuperClassifier().add(staffCategory);	
		staff.setUid(PredefTypesConst.STAFF_TYPE_FUID);

		//Resource_Category
		BulkResourceType resourceCategory = ResourcesFactory.eINSTANCE.createBulkResourceType();
		resourceCategory.setName(PredefTypesConst.RESOURCE_CATEGORY_TYPE_NAME);
		resourceCategory.setIsAbstract(PredefTypesConst.RESOURCE_CATEGORY_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);
		//first abbreviation property
		property = createProperty((PrimitiveType)types.get(IBTDataTypeRegistry.PREDEF_STRING), 
				PredefTypesConst.RESOURCE_CATEGORY_STRING_ATTR[0], 0, 1);
		resourceCategory.getOwnedAttribute().add(property);	
		resourceCategory.setUid(PredefTypesConst.RESOURCE_CATEGORY_TYPE_FUID);

		//Equipment
		BulkResourceType equipment = ResourcesFactory.eINSTANCE.createBulkResourceType();
		equipment.setName(PredefTypesConst.EQUIPMENT_TYPE_NAME);
		equipment.setIsAbstract(PredefTypesConst.EQUIPMENT_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		equipment.getSuperClassifier().add(resourceCategory);
		equipment.setUid(PredefTypesConst.EQUIPMENT_TYPE_FUID);

		//Facility	
		BulkResourceType facility = ResourcesFactory.eINSTANCE.createBulkResourceType();
		facility.setName(PredefTypesConst.FACILITY_TYPE_NAME);
		facility.setIsAbstract(PredefTypesConst.FACILITY_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		facility.getSuperClassifier().add(resourceCategory);
		facility.setUid(PredefTypesConst.FACILITY_TYPE_FUID);

		//Machine	
		BulkResourceType machine = ResourcesFactory.eINSTANCE.createBulkResourceType();
		machine.setName(PredefTypesConst.MACHINE_TYPE_NAME);
		machine.setIsAbstract(PredefTypesConst.MACHINE_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		machine.getSuperClassifier().add(resourceCategory);
		machine.setUid(PredefTypesConst.MACHINE_TYPE_FUID);

		//Tool	
		BulkResourceType tool = ResourcesFactory.eINSTANCE.createBulkResourceType();
		tool.setName(PredefTypesConst.TOOL_TYPE_NAME);
		tool.setIsAbstract(PredefTypesConst.TOOL_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		tool.getSuperClassifier().add(resourceCategory);
		tool.setUid(PredefTypesConst.TOOL_TYPE_FUID);

		//General Service	
		BulkResourceType generalService = ResourcesFactory.eINSTANCE.createBulkResourceType();
		generalService.setName(PredefTypesConst.GENERAL_SERVICE_TYPE_NAME);
		generalService.setIsAbstract(PredefTypesConst.GENERAL_SERVICE_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		generalService.getSuperClassifier().add(resourceCategory);
		generalService.setUid(PredefTypesConst.GENERAL_SERVICE_TYPE_FUID);

		//Communication Service	
		BulkResourceType communicationService = ResourcesFactory.eINSTANCE.createBulkResourceType();
		communicationService.setName(PredefTypesConst.COMMUNICATION_SERVICE_TYPE_NAME);
		communicationService.setIsAbstract(PredefTypesConst.COMMUNICATION_SERVICE_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		communicationService.getSuperClassifier().add(resourceCategory);
		communicationService.setUid(PredefTypesConst.COMMUNICATION_SERVICE_TYPE_FUID);

		resourceTypes.put(PredefTypesConst.PERSON_CATEGORY_TYPE_NAME, personCategory);
		resourceTypes.put(PredefTypesConst.PERSON_TYPE_NAME, person);
		resourceTypes.put(PredefTypesConst.STAFF_CATEGORY_TYPE_NAME, staffCategory);
		resourceTypes.put(PredefTypesConst.STAFF_TYPE_NAME, staff);
		resourceTypes.put(PredefTypesConst.RESOURCE_CATEGORY_TYPE_NAME, resourceCategory);
		resourceTypes.put(PredefTypesConst.EQUIPMENT_TYPE_NAME, equipment);
		resourceTypes.put(PredefTypesConst.FACILITY_TYPE_NAME, facility);
		resourceTypes.put(PredefTypesConst.MACHINE_TYPE_NAME, machine);
		resourceTypes.put(PredefTypesConst.TOOL_TYPE_NAME, tool);
		resourceTypes.put(PredefTypesConst.GENERAL_SERVICE_TYPE_NAME, generalService);
		resourceTypes.put(PredefTypesConst.COMMUNICATION_SERVICE_TYPE_NAME, communicationService);

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "initPredefinedResourceTypes");		
	}

	/**
	 * Create the predefined organization type	 
	 */
	private static void initPredefinedOrganizationTypes() {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "initPredefinedOrganizationTypes");
		Property property = null;
		organizationTypes = new HashMap();

		//organization unit
		OrganizationUnitType organizationUnitCategory = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnitType();
		organizationUnitCategory.setName(PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_NAME);
		organizationUnitCategory.setIsAbstract(PredefTypesConst.ORGANIZATION_CATEGORY_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);		
		//organizationUnitType.getSuperClassifier().add(personCategory);

		for (int i=0; i<PredefTypesConst.ORGANIZATION_CATEGORY_STRING_ATTR.length; i++) {
			property = createProperty((PrimitiveType)types.get(IBTDataTypeRegistry.PREDEF_STRING), 
					PredefTypesConst.ORGANIZATION_CATEGORY_STRING_ATTR[i], 0, 1);
			organizationUnitCategory.getOwnedAttribute().add(property);
		}
		organizationUnitCategory.setUid(PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_FUID);

		//Organization
		OrganizationUnitType organizationUnit = OrganizationstructuresFactory.eINSTANCE.createOrganizationUnitType();
		organizationUnit.setName(PredefTypesConst.ORGANIZATION_TYPE_NAME);
		organizationUnitCategory.setIsAbstract(PredefTypesConst.ORGANIZATION_IS_ABSTRACT ? Boolean.TRUE : Boolean.FALSE);
		organizationUnit.getSuperClassifier().add(organizationUnitCategory);	
		organizationUnit.setUid(PredefTypesConst.ORGANIZATION_TYPE_FUID);

		organizationTypes.put(PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_NAME, organizationUnitCategory);
		organizationTypes.put(PredefTypesConst.ORGANIZATION_TYPE_NAME, organizationUnit);
		LoggingUtil.traceExit(PredefinedTypeUtil.class, "initPredefinedOrganizationTypes");
	}	

	/**
	 * Update predefined types
	 *
	 */
	private static void updatePredefinedTypes(){
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "updatePredefinedTypes");
		Property property = null;
		IndividualResourceType staffCategory = (IndividualResourceType)resourceTypes.get(PredefTypesConst.STAFF_CATEGORY_TYPE_NAME);
		OrganizationUnitType organizationUnitCategory = (OrganizationUnitType)organizationTypes.get(PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_NAME);

		for (int i=0; i<PredefTypesConst.ORGANIZATION_CATEGORY_STAFF_CAT_ATTR.length; i++) {
			property = createProperty((IndividualResourceType)resourceTypes.get(PredefTypesConst.STAFF_TYPE_NAME), 
					PredefTypesConst.ORGANIZATION_CATEGORY_STAFF_CAT_ATTR[i], 0, 1);
			organizationUnitCategory.getOwnedAttribute().add(property);
		}
		for (int i=0; i<PredefTypesConst.ORGANIZATION_CATEGORY_STAFF_CAT_MANY_ATTR.length; i++) {
			property = createProperty((IndividualResourceType)resourceTypes.get(PredefTypesConst.STAFF_TYPE_NAME), 
					PredefTypesConst.ORGANIZATION_CATEGORY_STAFF_CAT_MANY_ATTR[i], 0, "*");
			organizationUnitCategory.getOwnedAttribute().add(property);
		}

		for (int i=0; i<PredefTypesConst.ORGANIZATION_CATEGORY_ORG_CAT_ATTR.length; i++) {
			property = createProperty((OrganizationUnitType)organizationTypes.get(PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_NAME), 
					PredefTypesConst.ORGANIZATION_CATEGORY_ORG_CAT_ATTR[i], 0, 1);
			organizationUnitCategory.getOwnedAttribute().add(property);
		}		

		for (int i=0; i<PredefTypesConst.STAFF_CATEGORY_ORG_CAT_ATTR.length; i++) {
			property = createProperty((OrganizationUnitType)organizationTypes.get(PredefTypesConst.ORGANIZATION_TYPE_NAME), 
					PredefTypesConst.STAFF_CATEGORY_ORG_CAT_ATTR[i], 0, 1);
			staffCategory.getOwnedAttribute().add(property);
		}

		for (int i=0; i<PredefTypesConst.STAFF_CATEGORY_STAFF_CAT_ATTR.length; i++) {
			property = createProperty((IndividualResourceType)resourceTypes.get(PredefTypesConst.STAFF_TYPE_NAME), 
					PredefTypesConst.STAFF_CATEGORY_STAFF_CAT_ATTR[i], 0, 1);
			staffCategory.getOwnedAttribute().add(property);
		}

		BulkResourceType resourceCategory = (BulkResourceType)resourceTypes.get(PredefTypesConst.RESOURCE_CATEGORY_TYPE_NAME);
		for (int i=0; i<PredefTypesConst.RESOURCE_CATEGORY_ORG_MANY_ATTR.length; i++) {
			property = createProperty((OrganizationUnitType)organizationTypes.get(PredefTypesConst.ORGANIZATION_TYPE_NAME), 
					PredefTypesConst.RESOURCE_CATEGORY_ORG_MANY_ATTR[i], 0, "*");				
			resourceCategory.getOwnedAttribute().add(property);
		}

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "updatePredefinedTypes");	
	}

	/**
	 * Set the predefined type to the import context
	 * @param context
	 */
	public static void setPredefinedTypeToContext(final String projectName, 
			ImportContext context) {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "setPredefinedTypeToContext");

		if (projectName == null) {
			LoggingUtil.trace(PredefinedTypeUtil.class, "setPredefinedTypeToContext", "project nanme should not be null.");
			LoggingUtil.traceExit(PredefinedTypeUtil.class, "setPredefinedTypeToContext");
			return;
		}

		if (context == null) {
			LoggingUtil.trace(PredefinedTypeUtil.class, "setPredefinedTypeToContext", "context should not be null.");
			LoggingUtil.traceExit(PredefinedTypeUtil.class, "setPredefinedTypeToContext");
			return;
		}
		init();

		Iterator keyIterator = types.keySet().iterator();
		String key = null;
		AbstractLibraryChildNode node = null;
		while(keyIterator.hasNext()) {
			key = (String) keyIterator.next();
			node = BOMUtil.getPrimitiveTypeNode(projectName, key);
			if (node != null) {
				context.put(types.get(key), node);
			}
		}
		LoggingUtil.traceExit(PredefinedTypeUtil.class, "setPredefinedTypeToContext");		
	}

	/**
	 * Return the primitive type by name.
	 * @param name
	 * @return
	 */
	public static PrimitiveType getPrimitiveType(final String name) {
		init();

		return (PrimitiveType) types.get(name);
	}

	/**
	 * Return the predefined type by name.
	 * @param name
	 * @return
	 */
	public static PrimitiveType getPredefinedType(final String name) {
		init();

		return (PrimitiveType) types.get(name);
	}

	/**
	 * Return the predefined resouce type by name.
	 * @param name
	 * @return
	 */
	public static Type getPredefinedResourceType(final String name) {
		init();

		return (Type) resourceTypes.get(name);
	}

	// BEGIN: TN20050601 - additional support for classifiers
	/**
	 * Return the predefined classifier by name.
	 * @param name
	 * @return
	 */
	public static OrganizationModel getPredefinedClassifier(final String name) {
		init();

		return (OrganizationModel) classifiers.get(name);
	}	
	//END: TN20050601 - additional support for classifiers	

	//BEGIN: TN20050601 - additional support for classifier value
	/**
	 * Return the predefined classifier value by name.
	 * @param name
	 * @return
	 */
	public static OrganizationUnit getPredefinedClassifierValue(final String name) {
		init();

		return (OrganizationUnit) classifierValues.get(name);
	}	
	//END: TN20050601 - additional support for classifier value



	/**
	 * Return the predefined organization type by name.
	 * @param name
	 * @return
	 */
	public static Type getPredefinedOrganizationType(final String name) {
		init();

		return (Type) organizationTypes.get(name);
	}

	/**
	 * Create the property
	 * @param type
	 * @param name
	 * @param lowerBound
	 * @param upperBound
	 * @return
	 */
	private static Property createProperty(final Type type, final String name, final int lowerBound, final int upperBound) {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "createProperty");
		Property property = ArtifactsFactory.eINSTANCE.createProperty();
		property.setName(name);
		property.setType(type);

		LiteralInteger lowerValue = ArtifactsFactory.eINSTANCE.createLiteralInteger();
		lowerValue.setValue(new Integer(lowerBound));

		LiteralInteger upperValue = ArtifactsFactory.eINSTANCE.createLiteralInteger();
		upperValue.setValue(new Integer(upperBound));

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "createProperty");
		return property;
	}

	/**
	 * Create the property
	 * @param type
	 * @param name
	 * @param lowerBound
	 * @param upperBound
	 * @return
	 */
	private static Property createProperty(final Type type, final String name, final int lowerBound, final String upperBound) {
		LoggingUtil.traceEntry(PredefinedTypeUtil.class, "createProperty");
		Property property = ArtifactsFactory.eINSTANCE.createProperty();
		property.setName(name);
		property.setType(type);

		LiteralInteger lowerValue = ArtifactsFactory.eINSTANCE.createLiteralInteger();
		lowerValue.setValue(new Integer(lowerBound));

		LiteralUnlimitedNatural upperValue = ArtifactsFactory.eINSTANCE.createLiteralUnlimitedNatural();
		upperValue.setValue(upperBound);

		LoggingUtil.traceExit(PredefinedTypeUtil.class, "createProperty");
		return property;
	}

	public static void registerPredefinedTypes(ImportSession session){
		List registry = (List)session.getContext().get(IImportConstants.TEMPLATE_REGISTRY_ENTRY);
		if(registry == null){
			registry = new ArrayList();	
			session.getContext().put(IImportConstants.TEMPLATE_REGISTRY_ENTRY, registry);
		}
		if(resourceTypes != null && resourceTypes.size() > 0){
			registry.add(resourceTypes);
		}
		if(organizationTypes != null && organizationTypes.size() > 0){
			registry.add(organizationTypes);
		}		
	}
	
	/**
	 * Check if the default value is valid for its type
	 * @param typeName
	 * @param value
	 * @return
	 */
	public static boolean isValidValueForType(String typeName,String value) {
		if(IBTDataTypeRegistry.PREDEF_BOOLEAN.equals(typeName)) {
			if("True".equals(value)||"False".equals(value))
				return true;
		}		
		boolean valid = false;
		// Accept null and empty values
		if (value == null || value.equals("") || typeName == null || typeName.equals("")) {
			valid = true;
		}
		else {	
			if(IBTDataTypeRegistry.PREDEF_DOUBLE.equals(typeName) ||
					IBTDataTypeRegistry.PREDEF_FLOAT.equals(typeName)) {
				//float and decimal need to consider the Locale of display also locale for input
				valid = PrimitiveTypeValueDisplayHelper.isValidDoubleInput(value);
			}
			else if(IBTDataTypeRegistry.PREDEF_UNLIMITED_NATURAL.equals(typeName)) {
				//verify listener ensures this already
				valid = true;
			}
			else if(IBTDataTypeRegistry.PREDEF_REALNUMBER.equals(typeName)) {
				//using same formatting as double value
				valid = PrimitiveTypeValueDisplayHelper.isValidDoubleInput(value);
			}
			else {				
				try {	
					//the type object will validate the value			
					if (BTDataTypeManager.instance.isValidValue(typeName, value)) {
						valid = true;
					}
				} catch (Exception e) {
					// Input value is not a decimal or other exception caught during validation,
					// pass through and set error indicator and return false.					
					valid = false;
				}
			}
		}		
		return valid;
	}	
}
