package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.net.URL;
import java.net.URLDecoder;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Import;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.wst.wsdl.internal.util.WSDLResourceFactoryImpl;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDInclude;
import org.eclipse.xsd.XSDRedefine;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.util.XSDResourceFactoryImpl;
import org.eclipse.xsd.util.XSDResourceImpl;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.blm.ie.imprt.IImportConstants;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.externalmodels.ExternalDocument;
import com.ibm.btools.bom.model.models.ExternalModel;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplementation;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.common.componentdef.Property;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.TemplateInstanceRule;
import com.ibm.wbit.model.utils.NamespaceUtils;
import com.ibm.wbit.tel.util.TaskResourceImpl;
import com.ibm.ws.fabric.da.scdl.DynamicAssembler;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Implementation;

/**
 * This utility class contains all the utility methods for the <code>ExportOperation</code> 
 */

public class ExportOperationUtil implements ExportOperationConstants {

	static final String COPYRIGHT = " Copyright IBM Corporation 2003, 2009.";
	static List<Integer> uidList = null;

	/**
	 * Strip the space from the folder name.
	 * @param folderName String
	 * @return folderNameNoSpace String
	 */
	public static String stripSpace(String folderName){
		if(folderName == null){
			return "";
		}
		StringBuffer sb = new StringBuffer();
		char[] charArray = folderName.toCharArray();
		for(int i = 0; i < charArray.length; i++){
			char eachChar = charArray[i];
			if(eachChar == ' '){
				continue;
			}
			sb.append(eachChar);
		}		
		return sb.toString().toLowerCase();
	}

	/**
	 * Strip the file extension from the file name.
	 * @param filename String
	 * @param extName String
	 * @return fileNoExtension String
	 */
	public static String stripFileExtension (String filename,String extName) {
		if(filename != null){
			if(filename.lastIndexOf("/") != -1){
				filename = filename.substring(filename.lastIndexOf("/")+1);
			}
			int extInd = filename.lastIndexOf(extName);
			if (extInd != -1) {
				return filename.substring(0,extInd);
			}
		}
		return filename;
	}	

	/**
	 * Count the number of folders in the path based on the URI.
	 * @param path String
	 * @return count int
	 */
	public static int countFileSegmentURI(String path) {
		if (path == null) {
			return 0;
		}
		int count=0;
		for (int i=0; i<path.length(); i++) {
			if (path.charAt(i)==ExportOperation.FILE_SEPARATOR_CHAR) {
				count++;
			}
		}
		return count;
	}

	/**
	 * Count the number of folders in the path.
	 * @param path String
	 * @return count int
	 */
	public static int countFileSegment(String path) {
		if (path == null) {
			return 0;
		}
		int count=0;
		for (int i=0; i<path.length(); i++) {
			if (path.charAt(i)==File.separatorChar) {
				count++;
			}
		}
		return count;
	}

	/**
	 * Build the backward path from folder count.
	 * @param count int
	 * @return path String
	 */
	public static String buildBackwardPath(int count) {
		String path="";
		for (int i=0; i<=count; i++) {
			path = path+".."+ FILE_SEPARATOR;
		}
		return path;
	}

	/**
	 * Extract the file extension.
	 * @param filename String
	 * @return fileExtension String
	 */
	public static String extractFileExtension (String filename) {
		if(filename != null){
			int extInd = filename.lastIndexOf('.');
			if (extInd != -1) {
				return filename.substring(extInd,filename.length());
			}
		}
		return "";
	}

	/**
	 * Utility to determine if the WSDL resource has already been saved.
	 * @param fullPathName
	 * @param savedWSDLSet
	 * @return
	 */
	public static boolean savedWSDL(String fullPathName, Set savedWSDLSet){
		if(fullPathName == null || fullPathName.length() == 0 || savedWSDLSet == null){
			return false;
		}
		if(savedWSDLSet.size() > 0){
			Iterator it = savedWSDLSet.iterator();
			while(it.hasNext()){
				Object entry = it.next();
				if(entry instanceof String){
					String wsdlRelativePath = (String)entry;				
					if(fullPathName.indexOf(wsdlRelativePath)!= -1){
						return true;
					}
				}
			}
		}
		return false;
	}


	/**
	 * Reconcile 2 absolute file paths and return the relative location (URL).
	 * @param fullImportLocation String
	 * @param absoluteProcessPath String
	 * @return relativePath String
	 */
	public static String reconcilePaths(String fullImportLocation, String absoluteProcessPath, int commonPathLength) {
		if (fullImportLocation == null || absoluteProcessPath == null) {
			return null;
		}
		//AZ 09/11/2007 clean the protocol part
		if(fullImportLocation.startsWith(URI_FILE)){
			fullImportLocation = fullImportLocation.substring(6);
		}
		if(absoluteProcessPath.startsWith(URI_FILE)){
			absoluteProcessPath = absoluteProcessPath.substring(6);
		}

		StringBuffer location = new StringBuffer();
		//pmr41333
		String removedConvertedSpacesFullImport = ExportOperationUtil.replaceConvertedSpaces(fullImportLocation);
		String removedConvertedAbsolutePath = ExportOperationUtil.replaceConvertedSpaces(absoluteProcessPath);
		String convertedImportLocationWithProject = removedConvertedSpacesFullImport.replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR).substring(commonPathLength);
		String convertedAbsoluteProcessPathWithProject = removedConvertedAbsolutePath.replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR).substring(commonPathLength);

		String convertedImportLocation = convertedImportLocationWithProject.substring(convertedImportLocationWithProject.indexOf(ExportOperation.FILE_SEPARATOR) + 1);
		String convertedAbsoluteProcessPath = convertedAbsoluteProcessPathWithProject.substring(convertedAbsoluteProcessPathWithProject.indexOf(ExportOperation.FILE_SEPARATOR) + 1);

		if (convertedImportLocation.equals(convertedAbsoluteProcessPath)){
			if (convertedImportLocation.indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
				return convertedImportLocation.substring(convertedImportLocation.lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);
			} else {					
				return convertedImportLocation;
			}
		}

		StringTokenizer importST = new StringTokenizer(convertedImportLocation, ExportOperation.FILE_SEPARATOR);
		StringTokenizer processST = new StringTokenizer(convertedAbsoluteProcessPath, ExportOperation.FILE_SEPARATOR);
		int commonPath = 0;

		String importToken = null;
		String processToken = null;

		while (importST.hasMoreTokens() && processST.hasMoreTokens()) {
			importToken = importST.nextToken();
			processToken = processST.nextToken();
			if (importToken.equals(processToken)) {
				commonPath += importToken.length() + 1;		//+1 for delimiter
			} else {	// tokens are not the same
				break;
			}
		}

		int fileSegment = ExportOperationUtil.countFileSegmentURI(convertedAbsoluteProcessPath.substring(commonPath));
		location.append(ExportOperationUtil.buildBackwardPath(fileSegment - 1));
		location.append(convertedImportLocation.substring(commonPath));
		return location.toString();
	}

	/**
	 * Create the key for BPEL process import.
	 * @param importObj Import
	 * @return key String
	 */
	public static String createKeyForProcessImport(com.ibm.wbit.bpel.Import importObj) {
		if (importObj == null || importObj.getLocation() == null) {
			return null;
		}

		String importLocation = importObj.getLocation();
		StringBuffer key = null;
		String keyResult = null;

		if (isExternalFile(importLocation)) {	// key is the import file location
			String externalFile = replaceConvertedSpaces(importLocation);
			key = new StringBuffer(externalFile);
			keyResult = replaceConvertedSpaces(key.toString());
		} else {
			if (importObj.getNamespace() != null) {
				key = new StringBuffer(importObj.getNamespace());
			}

			if (importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) != -1) {
				if (key == null) {
					key = new StringBuffer(importLocation.substring(importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) + 1));
				} else {
					key.append(importLocation.substring(importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) + 1));
				} 
			} else {
				if (key == null) {
					key = new StringBuffer(importLocation);
				} else {
					key.append(importLocation);
				} 				
			}
			//defect 74287.1 the modeler generated artifacts keyed by encoded namespaces,
			//so the decoding is not necessary
			keyResult = key.toString();
		}		
		//String keyResult = replaceConvertedSpaces(key.toString());
		return keyResult;
	}

	/**
	 * Replace %20 with spaces.
	 * @param location String
	 * @return convertedLocation String
	 */
	public static String replaceConvertedSpaces(String fileName) {
		if (fileName == null) {
			return null;
		}
		String convertedLocation = fileName;
		int index = -1;
		while (convertedLocation.indexOf(CONVERTED_SPACES_IN_FILE) > -1) {
			index = convertedLocation.indexOf(CONVERTED_SPACES_IN_FILE);
			if (index == 0) {	// starts with "%20"
				if (convertedLocation.length() > 3) {
					convertedLocation = " ";
				} else {
					return null;
				}
			} else {
				convertedLocation = convertedLocation.substring(0, index) + " " +
				convertedLocation.substring(index + 3);
			}

		}
		return convertedLocation;
	}

	/**
	 * Check if this location is based on the imported wsdl or xsd.
	 * @param originalSchemaLocation String
	 * @return flag boolean
	 */
	public static boolean isExternalFile(String originalSchemaLocation) {
		if (originalSchemaLocation == null)
			return false;

		return (originalSchemaLocation.toLowerCase().indexOf(ROOT_EXTERNAL_MODEL.toLowerCase()) > -1);
	}

	// pre: fileLocation is lower case
	/**
	 * Extract the index for catalogue from the imported wsdl or xsd.
	 * @param fileLocation String
	 * @return index int
	 */
	public static int catalogueFromIndex (String fileLocation) {
		if (fileLocation == null || fileLocation.length() == 0) {
			return 0;
		}
		return (fileLocation.toLowerCase()).indexOf(ROOT_EXTERNAL_MODEL.toLowerCase()) + ROOT_EXTERNAL_MODEL.length() + 1;
	}

	// Note: traversing import has to be done ONLY for external resources (ones created in Modeler have their SA created).
	/**
	 * Go through XSD import/include/redifne and load registry with schemas references from this xsd.
	 * @param xsdSchema XSDSchema
	 * @param registry Map
	 * @param filePath String 
	 * @param relativePath String
	 * @return 
	 */
	public static void processXSDImportIncludeRedefine(XSDSchema xsdSchema, Map registry, String filePath, String relativePath) {
		if (xsdSchema == null || registry == null) {
			return;
		}

		Iterator it = xsdSchema.getContents().iterator();
		String key = null;
		Object object = null;
		XSDSchema xsd = null;

		//20080702 add for performance improvements
		xsdSchema.setIncrementalUpdate(false);
		
		while (it.hasNext()) {
			object = it.next();

			if (object instanceof XSDImport) {	
				XSDImport xsdImport = ((XSDImport)object);
				xsd = xsdImport.getResolvedSchema();
				//PMR 17765,999,000
				//defect 167662	
				if (xsdImport.getSchemaLocation() == null) {
					continue;
				}
				if (xsd != null) {
					key = replaceConvertedSpaces(reconcileURIForSchema(xsd.getSchemaLocation()));
					if (key == null || key.equals(xsdSchema.getSchemaLocation())) {		//this is an import to itself (inline schema in wsdl)
						continue;
					}
					xsdImport.setSchemaLocation(key);
//					xsd.setSchemaLocation(key);
					if (!registry.containsKey(key)) {
						String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 
						URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
						List value = new ArrayList(2);
						value.add(xsd);
						value.add(targetLocation.toString());
						registry.put(key, value);
						processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
					}					
				} else {

					//START =========================================================================================//
					//AZ Add support for the inline schema XSD import, xsdImport.getSchema().getSchemaLocation() returns
					//the target file path for the containing definition, which is wrong					
					String base = null;
					boolean isInlineSchemaImport = false;
					if(xsdImport.eContainer()instanceof XSDSchema){
						if(xsdImport.eContainer().eContainer() instanceof XSDSchemaExtensibilityElement){
							if(xsdImport.eContainer().eContainer().eContainer() instanceof Types){
								if(xsdImport.eContainer().eContainer().eContainer().eContainer()instanceof Definition){
									isInlineSchemaImport = true;
								}
							}
						}
					}
					if(isInlineSchemaImport){
						base = ((Definition)xsdImport.eContainer().eContainer().eContainer().eContainer()).getLocation();
					}else{
						base = xsdImport.getSchema().getSchemaLocation();
					}
					//skip the already processed import for the external service/schema
					if((isExternalFile(base) && 
							xsdImport.getSchemaLocation()!= null && 
							xsdImport.getSchemaLocation().startsWith(URI_FILE))||
							!isExternalFile(xsdImport.getSchemaLocation())){
						continue;
					}
					//END ==============================================================================================//

					key = replaceConvertedSpaces(resolveURI(base, xsdImport.getSchemaLocation()));
					if (isExternalFile(key) && !xsdImport.getSchemaLocation().startsWith(URI_FILE)) {	// import not updated by transformation
						xsd = forceLoad(key);
						if (xsd != null) {
							xsdImport.setSchemaLocation(key);
							xsd.setSchemaLocation(key);
							if (!registry.containsKey(key)) {
								String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 
								URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
								List value = new ArrayList(2);
								value.add(xsd);
								value.add(targetLocation.toString());
								registry.put(key, value);
								processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
							}
						}
					}
				}
			} else if(object instanceof XSDInclude){	// only external schemas contain XSDInclude (not used in Modeler transformation)
				XSDInclude xsdInclude = (XSDInclude)object;
				xsd = xsdInclude.getResolvedSchema();
				String sourceFilePath = null;
				if (xsd != null) {
 					key = replaceConvertedSpaces(reconcileURIForSchema(xsd.getSchemaLocation()));
					if (key.equals(xsdSchema.getSchemaLocation())) {		//this is an include to itself (inline schema in wsdl)
						continue;
					}
					xsdInclude.setSchemaLocation(key);
					//defect 91836
//					xsd.setSchemaLocation(key);
					if (!registry.containsKey(key)) {
						String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 
						URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
						List value = new ArrayList(2);
						value.add(xsd);
						value.add(targetLocation.toString());
						registry.put(key, value);
						processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
					}
				} else {
					//Add support for the inline schema XSD import, xsdImport.getSchema().getSchemaLocation() returns
					//the target file path for the containing definition, which is wrong					
					String base = null;
					boolean isInlineSchemaImport = false;
					if(xsdInclude.eContainer()instanceof XSDSchema){
						if(xsdInclude.eContainer().eContainer() instanceof XSDSchemaExtensibilityElement){
							if(xsdInclude.eContainer().eContainer().eContainer() instanceof Types){
								if(xsdInclude.eContainer().eContainer().eContainer().eContainer()instanceof Definition){
									isInlineSchemaImport = true;
								}
							}
						}
					}
					if(isInlineSchemaImport){
						base = ((Definition)xsdInclude.eContainer().eContainer().eContainer().eContainer()).getLocation();
					}else{
						base = xsdInclude.getSchema().getSchemaLocation();
					}

					//skip the already processed import for the external service/schema
					if(isExternalFile(base) && 
							xsdInclude.getSchemaLocation()!= null && 
							xsdInclude.getSchemaLocation().startsWith(URI_FILE)){
						continue;
					}
					//d78189
					if (base != null && base.indexOf("..") != -1) {
						base = reconcileURIForSchema(base);
					}
					
					key = replaceConvertedSpaces(resolveURI(base, xsdInclude.getSchemaLocation()));
					if (isExternalFile(key) && !xsdInclude.getSchemaLocation().startsWith(URI_FILE)) {	// include not updated by this method
						xsd = forceLoad(key);
						if (xsd != null) {
							xsdInclude.setSchemaLocation(key);
							xsd.setSchemaLocation(key);
							if (!registry.containsKey(key)) {
								String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 
								URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
								List value = new ArrayList(2);
								value.add(xsd);
								value.add(targetLocation.toString());
								registry.put(key, value);
								processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
							}
						}
					}
				}
			} else if (object instanceof XSDRedefine) {	// only external schemas contain XSDRedefine (not used in Modeler transformation)	
				XSDRedefine xsdRedefine = ((XSDRedefine)object);
				xsd = xsdRedefine.getResolvedSchema();				
				if (xsd != null) {
					key = replaceConvertedSpaces(reconcileURIForSchema(xsd.getSchemaLocation()));						
					if (key.equals(xsdSchema.getSchemaLocation())) {		//this is a redefine to itself (inline schema in wsdl)
						continue;
					}
					xsdRedefine.setSchemaLocation(key);
//					xsd.setSchemaLocation(key);
					if (!registry.containsKey(key)) {
						String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 						
						URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
						List value = new ArrayList(2);
						value.add(xsd);
						value.add(targetLocation.toString());
						registry.put(key, value);
						processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
					}
				} else {
					//Add support for the inline schema XSD import, xsdImport.getSchema().getSchemaLocation() returns
					//the target file path for the containing definition, which is wrong					
					String base = null;
					boolean isInlineSchemaImport = false;
					if(xsdRedefine.eContainer()instanceof XSDSchema){
						if(xsdRedefine.eContainer().eContainer() instanceof XSDSchemaExtensibilityElement){
							if(xsdRedefine.eContainer().eContainer().eContainer() instanceof Types){
								if(xsdRedefine.eContainer().eContainer().eContainer().eContainer()instanceof Definition){
									isInlineSchemaImport = true;
								}
							}
						}
					}
					if(isInlineSchemaImport){
						base = ((Definition)xsdRedefine.eContainer().eContainer().eContainer().eContainer()).getLocation();
					}else{
						base = xsdRedefine.getSchema().getSchemaLocation();
					}

					//skip the already processed import for the external service/schema
					if(isExternalFile(base) && 
							xsdRedefine.getSchemaLocation()!= null && 
							xsdRedefine.getSchemaLocation().startsWith(URI_FILE)){
						continue;
					}

					key = replaceConvertedSpaces(resolveURI(base, xsdRedefine.getSchemaLocation()));
					if (isExternalFile(key) && !xsdRedefine.getSchemaLocation().startsWith(URI_FILE)) {	// redefine not updated by this method
						xsd = forceLoad(key);
						if (xsd != null) {
							xsdRedefine.setSchemaLocation(key);
							xsd.setSchemaLocation(key);
							if (!registry.containsKey(key)) {
								String relativeFilePath = calculateRelativeTargetLocation(key, relativePath); 
								URI targetLocation = URI.createFileURI((filePath + relativeFilePath).replace(File.separatorChar, ExportOperation.FILE_SEPARATOR_CHAR));
								List value = new ArrayList(2);
								value.add(xsd);
								value.add(targetLocation.toString());
								registry.put(key, value);
								processXSDImportIncludeRedefine(xsd, registry, filePath, relativePath);		// call recursively
							}
						}
					}
				}
			}
		}
	}

	// utility method to force load a schema from its URI
	private static XSDSchema forceLoad(String fileName){
		if (fileName == null) {
			return null;
		}
		if (fileName.startsWith(URI_FILE)) {
			fileName = fileName.substring(6);
		}
		try {
			File inputFile = new File(fileName);
			URI fileURI = URI.createFileURI(inputFile.getAbsolutePath());
			final ResourceSet resourceSet = new ResourceSetImpl();
			Resource resource = resourceSet.getResource(fileURI, true);
			if (!resourceSet.getResources().isEmpty()) {
				for (Iterator it = resourceSet.getResources().iterator(); it.hasNext(); ) {
					resource = (Resource)it.next();
					if (resource instanceof XSDResourceImpl){
						XSDResourceImpl xsdResource = (XSDResourceImpl)resource;
						if (!xsdResource.getContents().isEmpty()) {
							XSDSchema xsdSchema = (XSDSchema)xsdResource.getContents().get(0);
//							if (xsdSchema.getTargetNamespace() == null) {
//								String params[] = {xsdSchema.getSchemaLocation()};
//								LoggingUtil.logError(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION, params, null);
//								BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION));
//								throw runtimeEx;
//							}

							return xsdSchema;
						}
					}
				}
			}
		} catch (Exception ex) {	// this might happen if the import or schema location have been updated already
//20090706			String params[] = {fileName};
//			LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);			
		}

		return null;
	}	

	/**
	 * Resolve the URI from the base location and relative path.
	 * @param base String 
	 * @param relativePath String
	 * @return uri String
	 */
	public static String resolveURI(String base, String relativePath) {
		if (base == null) {
			return relativePath;
		}
		if (relativePath == null) {
			return base;
		}

		URI baseURI = URI.createURI(base);
		String stringResolvedURI = base;
		while (stringResolvedURI != null && stringResolvedURI.indexOf("..") != -1) {
			String basePart = base.substring(0, base.indexOf(".."));
			String relativePart = base.substring(base.indexOf(".."));
			stringResolvedURI = resolveURI(basePart, relativePart);
			if (stringResolvedURI != null && stringResolvedURI.equals(basePart + relativePart)) {	// if URI resolution did not change the original
				break;
			}
		}
		if (stringResolvedURI != null && !base.equals(stringResolvedURI)) {
			baseURI = URI.createURI(stringResolvedURI);
		}
		URI relativePathURI = null;
		if (relativePath.indexOf(File.separatorChar) != -1) {
			relativePathURI = URI.createURI(relativePath.replace(File.separatorChar, FILE_SEPARATOR_CHAR));
		} else {
			relativePathURI = URI.createURI(relativePath);
		}		
		URI resolvedURI = relativePathURI.resolve(baseURI);

		return resolvedURI.toString();
	}

	/**
	 * Calculate relative target location from the sourceLocation location and relative path.
	 * @param base String 
	 * @param relativePath String
	 * @return uri String
	 */	
	public static String calculateRelativeTargetLocation(String sourceLocation, String relativePath) {
		if (sourceLocation == null) {
			return relativePath;
		}
		if (relativePath == null) {
			return sourceLocation;
		}

		String fileName = sourceLocation.substring(sourceLocation.lastIndexOf(FILE_SEPARATOR_CHAR) + 1);
		if (fileName == null) {
			String params[] = {sourceLocation};
			LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION));
			throw runtimeEx;
		}

		StringBuffer pathWithoutFileAttachment = new StringBuffer(sourceLocation.substring(0, sourceLocation.length() - (fileName.length() + 1)));
		int lastSectionLength = fileName.length();
		if (pathWithoutFileAttachment.toString().endsWith(ExportOperation.FILE_ATTACHMENT)) {
			pathWithoutFileAttachment.delete(pathWithoutFileAttachment.length() - (ExportOperation.FILE_ATTACHMENT.length() + 1), pathWithoutFileAttachment.length());
			lastSectionLength += ExportOperation.FILE_ATTACHMENT.length() + 1;
		}
		// for inline schemas there are 2 file attachment folders
		if (pathWithoutFileAttachment.toString().endsWith(ExportOperation.FILE_ATTACHMENT)) {
			pathWithoutFileAttachment.delete(pathWithoutFileAttachment.length() - (ExportOperation.FILE_ATTACHMENT.length() + 1), pathWithoutFileAttachment.length());
			lastSectionLength += ExportOperation.FILE_ATTACHMENT.length() + 1;
		}
//20081119		lastSectionLength += (pathWithoutFileAttachment.substring(pathWithoutFileAttachment.toString().lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1)).length() + 1;

		String catalogue = "";
		String relativeFilePath = null;
		if (sourceLocation.length() > lastSectionLength) {
			catalogue = sourceLocation.substring(ExportOperationUtil.catalogueFromIndex(sourceLocation.toLowerCase()), 	// catalogue includes last file separator
					sourceLocation.length() - lastSectionLength);
			String relFolderName = null;
			if (relativePath.indexOf(File.separatorChar) != -1) {
				relFolderName = relativePath.substring(0, relativePath.indexOf(File.separatorChar) + 1);	//module name or catalogue name
			} else {
				relFolderName = relativePath + FILE_SEPARATOR;
			}
			catalogue = ExportOperationUtil.stripSpace(catalogue); //strip space, convert to lower case
			if(skipDefaultCatalog()){
				catalogue = catalogue.substring(catalogue.indexOf(FILE_SEPARATOR_CHAR)+1);
				if(catalogue == null || catalogue.length() == 0){
					catalogue = FILE_SEPARATOR;
				}
			}
			//use JavaNameConverter for the path conversion (or when saving the resource)
			if (catalogue.length() >= relFolderName.length() && catalogue.startsWith(relFolderName.substring(0, relFolderName.length() - 1))) {	// this will happen when the module name is not specified
				relativeFilePath = convertCatalogNameToJavaNCName(catalogue) + fileName;
			} else {
//				GJD 20070212				relativeFilePath = 	convertCatalogNameToJavaNCName(relFolderName + catalogue) + fileName;
				relativeFilePath = relFolderName + convertCatalogNameToJavaNCName(catalogue) + fileName;
			}
		} else {
			relativeFilePath = 	convertCatalogNameToJavaNCName(relativePath) + fileName;
		}
		return relativeFilePath;
	}

	/**
	 * Fix schema imports for schema resources contained in the regirstry.
	 * @param sourcePathToTargetPathAbsoluteRegistry Map 
	 * @param resourceSet ResourceSet
	 * @param xsdResourceFactory XSDResourceFactoryImpl
	 * @param wsdlResourceFactory WSDLResourceFactoryImpl
	 * @return
	 */	
	protected static void fixXSDImports(Map sourcePathToTargetPathAbsoluteRegistry, ResourceSet resourceSet, XSDResourceFactoryImpl xsdResourceFactory, WSDLResourceFactoryImpl wsdlResourceFactory,int commonPathLength) {
		if (sourcePathToTargetPathAbsoluteRegistry == null || sourcePathToTargetPathAbsoluteRegistry.isEmpty() || 
				resourceSet == null || xsdResourceFactory == null || wsdlResourceFactory == null) {
			return;
		}

		Set keys = sourcePathToTargetPathAbsoluteRegistry.keySet();
		Iterator it = keys.iterator();
		String key = null;
		List val = null;
		XSDSchema xsd = null;

		while(it.hasNext()) {
			key = (String)it.next();
			val = (List)sourcePathToTargetPathAbsoluteRegistry.get(key);
			if (val == null || val.size() != 2) {
				String params[] = {key};
				LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);
				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION));
				throw runtimeEx;
			}

			if (val.get(0) instanceof XSDSchema) {
				xsd = (XSDSchema)val.get(0);
				//GJD do not udpate xsd
				xsd.setIncrementalUpdate(false);				
				updateXSDImports(xsd, sourcePathToTargetPathAbsoluteRegistry,commonPathLength);
			} else if (val.get(0) instanceof XSDResourceImpl) {
				XSDResourceImpl xsdResource = (XSDResourceImpl)val.get(0);
				xsd = xsdResource.getSchema();
				xsd.setIncrementalUpdate(false);				
				updateXSDImports(xsd, sourcePathToTargetPathAbsoluteRegistry,commonPathLength);
			}
		}
	}


	private static void updateXSDImports(XSDSchema xsd, Map sourcePathToTargetPathAbsoluteRegistry,int commonPathLength) {
		if (xsd == null || sourcePathToTargetPathAbsoluteRegistry == null) {
			return;
		}
		Iterator it = xsd.getContents().iterator();
		String schemaLocation = replaceConvertedSpaces(xsd.getSchemaLocation());

		String sourceLocation = null;
		String targetLocation = null;
		String fixedLocation = null;
		List values = null;
		Object object = null;

		while (it.hasNext()) {
			object = it.next();

			if (object instanceof XSDImport) {	
				XSDImport xsdImport = ((XSDImport)object);
				if (xsdImport.getSchemaLocation() == null) {
					continue;
				}
				// calculate the target path for the import
				sourceLocation = replaceConvertedSpaces(xsdImport.getSchemaLocation());
				if (isExternalFile(sourceLocation)) {
					values = (List)sourcePathToTargetPathAbsoluteRegistry.get(sourceLocation);
				} else { // check for internal schemas
					String key = xsdImport.getNamespace() + xsdImport.getSchemaLocation();
					values = (List)sourcePathToTargetPathAbsoluteRegistry.get(key);
				}
				if (values == null || values.size() != 2) {
					String params[] = {sourceLocation};
					LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION));
					throw runtimeEx;					
				}
				targetLocation = replaceConvertedSpaces((String)values.get(1));

				// calculate the target path for the containing schema
				String reconciledSchema = reconcileURIForSchema(schemaLocation);
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(reconciledSchema);
				String schemaTargetLocation = null;
				if (values == null) {		// already in the target path
					schemaTargetLocation = schemaLocation;
				} else {
					schemaTargetLocation = (String)values.get(1);					
				}

				fixedLocation = replaceConvertedSpaces(reconcilePaths(targetLocation, schemaTargetLocation,commonPathLength));
				//AZ 06/22/07
				
				String encodedProjectName = null; 
				String projectName = null; 
				if (xsdImport.getSchemaLocation() != null && xsdImport.getSchemaLocation().endsWith(XSD_FILE_EXT)) {
					encodedProjectName = xsdImport.getSchemaLocation().substring(0, xsdImport.getSchemaLocation().length() - 4);
				} else {
					encodedProjectName = xsdImport.getSchemaLocation();
				}
				if (encodedProjectName != null) {
					projectName = NamespaceUtils.convertUriToNamespace(encodedProjectName);
				}
				if (projectName != null) {
					if(!CrossProjectReferenceUtil.isPredefinedProject(projectName)) {		
						fixedLocation = flatenRelativePath(fixedLocation);
					}
				} else {
					fixedLocation = flatenRelativePath(fixedLocation);
				}		
				if (fixedLocation != null) {
					if (!fixedLocation.equals(xsdImport.getSchemaLocation())) {
						xsdImport.setSchemaLocation(fixedLocation);
					} else {
						return;		// xsd processed
					}
				}
			} else if(object instanceof XSDInclude){	// only external schemas contain XSDInclude (not used in Modeler transformation)
				XSDInclude xsdInclude = (XSDInclude)object;
				if (xsdInclude.getSchemaLocation() == null) {
					continue;
				}
				sourceLocation = replaceConvertedSpaces(xsdInclude.getSchemaLocation());
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(sourceLocation);
				// Ray Fix for Share Across Project - the sourceLocation should include the hashcode of the schema
				// If nothing found for the sourceLocation as the key (this is the case when all schemas are registered based on its tns only
				// and only include the hashcode in the key when there is a conflict in the registry), then try to use the targetNamespace as the key
				// and try again.
				// See ExportOperation.java createResource method
//				if (values == null) {
//				String namespace = xsd.getTargetNamespace();
//				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(namespace+ProcessUtil.createFileName(namespace)+".xsd");
//				}
				if (values == null || values.size() != 2) {
					String params[] = {sourceLocation};
					LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION));					throw runtimeEx;					
				}
				targetLocation = replaceConvertedSpaces((String)values.get(1));

				// calculate the target path for the containing schema
				String reconciledSchema = reconcileURIForSchema(schemaLocation);
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(reconciledSchema);
				String schemaTargetLocation = null;
				if (values == null) {		// already in the target path
					schemaTargetLocation = schemaLocation;
				} else {
					schemaTargetLocation = (String)values.get(1);					
				}

				fixedLocation = replaceConvertedSpaces(reconcilePaths(targetLocation, schemaTargetLocation,commonPathLength));	
				//AZ 06/22/07
				fixedLocation = flatenRelativePath(fixedLocation);
				if (fixedLocation != null) {
					if (!fixedLocation.equals(xsdInclude.getSchemaLocation())) {
						xsdInclude.setSchemaLocation(fixedLocation);
					} else {
						return;		// xsd processed
					}
				}
			} else if (object instanceof XSDRedefine) {	// only external schemas contain XSDRedefine (not used in Modeler transformation)	
				XSDRedefine xsdRedefine = ((XSDRedefine)object);
				if (xsdRedefine.getSchemaLocation() == null) {
					continue;
				}
				sourceLocation = replaceConvertedSpaces(xsdRedefine.getSchemaLocation());
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(sourceLocation);
				if (values == null || values.size() != 2) {
					String params[] = {sourceLocation};
					LoggingUtil.logError(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_COPYING_ORIGINAL_XSD_DEFINITION));
					throw runtimeEx;					
				}
				targetLocation = replaceConvertedSpaces((String)values.get(1));
				// calculate the target path for the containing schema
				String reconciledSchema = reconcileURIForSchema(schemaLocation);
				values = (List)sourcePathToTargetPathAbsoluteRegistry.get(reconciledSchema);
				String schemaTargetLocation = null;
				if (values == null) {		// already in the target path
					schemaTargetLocation = schemaLocation;
				} else {
					schemaTargetLocation = (String)values.get(1);					
				}

				fixedLocation = replaceConvertedSpaces(reconcilePaths(targetLocation, schemaTargetLocation,commonPathLength));	
				//AZ 06/22/07
				fixedLocation = flatenRelativePath(fixedLocation);
				if (fixedLocation != null) {
					if (!fixedLocation.equals(xsdRedefine.getSchemaLocation())) {
						xsdRedefine.setSchemaLocation(fixedLocation);
					} else {
						return;		// xsd processed
					} 
				}
			}
		}
	}


	private static String reconcileURIForSchema(String schemaLocation) {
		if (schemaLocation == null) {
			return null;
		}

		//d78189 replace converted spaces
		String convertedSchemaLocation = replaceConvertedSpaces(schemaLocation);
		while (convertedSchemaLocation.indexOf("..") != -1) {
			int index = convertedSchemaLocation.indexOf("..");
			convertedSchemaLocation = resolveURI(convertedSchemaLocation.substring(0, index), convertedSchemaLocation.substring(index));
		}

		return convertedSchemaLocation;
	}

	/**
	 * Caculate the target path 
	 * @param element
	 * @param relativePath
	 * @return
	 */
	public static String calculateAbsoluteTargetPath(Object element,String absolutePathPrefix,String relativePath){
		if(element instanceof Definition){
			Definition definition = (Definition)element;
			String location = definition.getLocation();
			location = normalizeImportLocation(location);
			if(location != null && location.indexOf(ROOT_EXTERNAL_MODEL)!= -1){			
				String catalogName = extractCatalogFromAbsolutePath(location);	
				if(skipDefaultCatalog()){
					catalogName = catalogName.substring(catalogName.indexOf(File.separator)+1);
					if(catalogName == null || catalogName.length() == 0){
						catalogName = FILE_SEPARATOR;
					} 
				}
				String fileName = location.substring(location.lastIndexOf(FILE_SEPARATOR)+1);
				if(relativePath != null){
					return absolutePathPrefix + 
					relativePath +
					fileName;			
				}
				return absolutePathPrefix + 
				getTopPackageName(element)+
				File.separator +
				catalogName +
				fileName;			
			}else{
				if(definition.getQName() != null){
					String fileName = definition.getQName().getLocalPart() + WSDL_FILE_EXT;					
					//The relative path should be fixed later on for the TE generated definitions
					String catalogRelativePath = 
						relativePath == null?(getTopPackageName(element)+File.separator):relativePath;					
						return absolutePathPrefix + 							
						catalogRelativePath +
						fileName;								
				}
			}					
		}
		return null;
	}

	/**
	 * Compute the WSDL file name based on the definition object
	 * @param element
	 * @return fileName
	 */
	public static String computeNameFromDefinitionObject(Object element){	
		String fileName = null;
		String location = null;
		if(element instanceof Definition){
			location = ((Definition)element).getLocation();
		}else if(element instanceof XSDSchema){
			location = ((XSDSchema)element).getSchemaLocation();
		}
		location = normalizeImportLocation(location);
		if(location != null){					
			fileName = location.substring(location.lastIndexOf(FILE_SEPARATOR)+1);						
		}else{
			if(element instanceof Definition){
				if(((Definition)element).getQName() != null){
					fileName = ((Definition)element).getQName().getLocalPart() + WSDL_FILE_EXT;					
				}
			}
//			else if(element instanceof XSDSchema){
//			if(((XSDSchema)element).getgetQName() != null){
//			fileName = ((Definition)element).getQName().getLocalPart() + WSDL_FILE_EXT;					
//			}
//			}
		}
		return fileName;
	}

	private static String convertCatalogNameToJavaNCName(String catalogName){
		if(catalogName == null){
			return null;
		}
		StringBuffer sb = new StringBuffer();
		JavaNCNameConverter converter = new JavaNCNameConverter();		
		StringTokenizer strTok = null;
		String convCatalogName = catalogName.replace(File.separatorChar, FILE_SEPARATOR_CHAR);
		strTok = new StringTokenizer(convCatalogName,"/");
		while (strTok.hasMoreTokens()) {
			String pathSegment = strTok.nextToken();
			pathSegment = converter.convertName(pathSegment);			
			sb.append(pathSegment);
			sb.append(File.separator);
		}

		return sb.toString();
	}

	/**
	 * Recursively create the registry for the handled element and its imports
	 * 1. Registry key = originalResourceAbsolutePath String
	 * 2. Registry value = {resource,targetAbsolutePath} List
	 * 3. The targetAbsolutePath might be Null, the resource is the EMF resource
	 * 4. The size of the list can be one
	 * @param registry
	 * @param element
	 * @param objectFilePathMap 
	 * @param originalPath
	 * @param targetPath
	 */
	public static void createRegistryForResource(
			ResourceFactoryImpl resourceFactory,
			ResourceSet resourceSet,
			Map registry,
			Object element,
			String absolutePathPrefix,
			String relativePath,
			Map obj2FileNameRegistry, 
			Map objectFilePathMap){		
		// recursive function to register definitions
		if(element instanceof Definition){
			Definition def = (Definition) element;
			String keyLocation = replaceConvertedSpaces(def.getLocation());
			registerImportsForDefintion(resourceFactory,
					resourceSet,
					registry,
					def,
					absolutePathPrefix,
					relativePath,
					false,
					obj2FileNameRegistry,
					objectFilePathMap);
		}
	}

	/**
	 * Recursively register the WSDL definition references
	 * @param resourceFactory
	 * @param resourceSet
	 * @param registry
	 * @param definition
	 * @param absolutePathPrefix The target WPS export project absolute path prefix
	 * @param relativePath The relative path computed by the file manager based on the target element
	 */	
	private static void registerImportsForDefintion(
			ResourceFactoryImpl resourceFactory,
			ResourceSet resourceSet,
			Map registry,
			Definition definition,
			String absolutePathPrefix,
			String relativePath,
			boolean isImport,
			Map obj2FileNameRegistry,
			Map objectFilePathMap){			
		//Register the first class definition first
		absolutePathPrefix = absolutePathPrefix.replace('/','\\');	int i=0;	
		List valueList = new ArrayList();
		String sourcePath = getAbsolutePathForSource(definition,absolutePathPrefix,relativePath,definition.getTargetNamespace());		
		String targetPath = calculateAbsoluteTargetPath(definition,absolutePathPrefix,relativePath);
		Resource resource = null;
		if(isImport){
			definition.setLocation(sourcePath);
			valueList.add(0,definition);
			valueList.add(1,targetPath);
		}else{
			//AZ 20070209 move outside			
			resource = createResource(definition,resourceFactory,resourceSet,targetPath);
			if(!registry.containsKey(sourcePath)){
				//	resource = createResource(definition,resourceFactory,resourceSet,targetPath);
				valueList.add(0,resource);
				valueList.add(1,targetPath);				
			}
		}		
		//valueList.add(1,targetPath);
		if(sourcePath != null){
			if(!registry.containsKey(sourcePath)){
				registry.put(sourcePath,valueList);
			}else{
				if(isExternalFile(sourcePath)){				
					//update the entry, this definition is associated with a SA
					Object valueListObj = (List)registry.get(sourcePath);
					if(valueListObj instanceof List){
						if(!isImport){
							List updatedValueList = (List)valueListObj;
							//refresh this contents of the list
							updatedValueList.clear();
							updatedValueList.add(0,resource);
							updatedValueList.add(1,targetPath);
							registry.put(sourcePath,updatedValueList);
						}
					}
				}
			}			
		}

		//1a. Retrieve the original location if it's loaded resource
		//1b. If it's TE generated artifacts, we need compute the target path based on target namespace and 
		//file name
		//2. Compute the target WPS directory after export
		List eImports = new LinkedList();			
		eImports.addAll(definition.getEImports());			
		if(definition.getETypes() != null){
			List schemas = definition.getETypes().getSchemas();
			if(schemas != null){
				Iterator schemasIt= schemas.iterator();
				while(schemasIt.hasNext()){
					XSDSchema schema = (XSDSchema)schemasIt.next();
					if (schema.getSchemaLocation() != null) {
						processXSDImportIncludeRedefine(schema, registry, absolutePathPrefix, getTopPackageName(definition));
					}
				}
			}
		}			
		Iterator it = eImports.iterator();	
		while(it.hasNext()){
			Object eachImport = it.next();
			if(eachImport instanceof Import){
				Import importObj = (Import)eachImport;
				String locationUri = importObj.getLocationURI();
				if(locationUri != null && locationUri.endsWith(XSD_FILE_EXT)){
					//XSD import
					XSDSchema imprtSchema = importObj.getESchema();
					//Fixme sometimes the e schema is null
					if (imprtSchema != null && imprtSchema.getSchemaLocation() != null) {
						processXSDImportIncludeRedefine(imprtSchema, registry, absolutePathPrefix, getTopPackageName(definition));
					}
				}else if(locationUri != null && locationUri.endsWith(WSDL_FILE_EXT)){
					//wsdl import					
					String absoluteImportedLocationPath = resolveAbsolutePathForImport(importObj);					
					//native created elements can be resolved
					Definition imprtDefinition = importObj.getEDefinition();	
					if(imprtDefinition == null && isExternalFile(absoluteImportedLocationPath)){
						//try to look up the definition that already been loaded if referenced during transformation
						if(obj2FileNameRegistry.containsValue(absoluteImportedLocationPath)){
							//look up the key that is the WSDL definition
							Set defKeys = obj2FileNameRegistry.keySet();
							for(Iterator keys = defKeys.iterator();keys.hasNext();){
								Object key = keys.next();
								if(obj2FileNameRegistry.get(key).equals(absoluteImportedLocationPath)){
									imprtDefinition = (Definition)key;
									break;
								}
							}
							//imprtDefinition = (Definition)resolveRegistryPathKey(registry, absoluteImportedLocationPath);

						}
						if(imprtDefinition == null){
							imprtDefinition = OriginalWsdlUtil.loadOriginalWSDLResource(absoluteImportedLocationPath);
						}						
					}					
					//Here the relative path should be null
					//we skip the registration for the TE generation SA		
					if(imprtDefinition == null || definition == null || !isExternalFile(definition.getLocation())){
						continue;
					}
					//put this original definition within the object to file name registry
					if(!obj2FileNameRegistry.containsValue(absoluteImportedLocationPath)){
						obj2FileNameRegistry.put(imprtDefinition,absoluteImportedLocationPath);	
					}
					registerImportsForDefintion(resourceFactory,resourceSet,registry,imprtDefinition,absolutePathPrefix,null,true,obj2FileNameRegistry,objectFilePathMap);
				}
			}
		}	
	}

	/**
	 * Create resource representation for the target object
	 * @param element
	 * @param resourceFactory
	 * @param resourceSet
	 * @param absoluteFilePath
	 * @return
	 */
	public static Resource createResource(
			Object element,
			ResourceFactoryImpl resourceFactory,
			ResourceSet resourceSet,
			String absoluteFilePath){       
		URI fileURI = URI.createFileURI(absoluteFilePath);		
		if (resourceFactory != null) {
			Resource resource = resourceFactory.createResource(fileURI);
			if(resource instanceof TaskResourceImpl){
				TaskResourceImpl.setUseImports(true);
			}
			resourceSet.getResources().add(resource);
			resource.getContents().add((EObject) element);	
			return resource;
		}
		return null;
	}	

	/**
	 * Extract catalog name based on the absolute path
	 * @param path
	 * @return
	 */
	private static String extractCatalogFromAbsolutePath(String path){
		if(path != null && path.indexOf(ROOT_EXTERNAL_MODEL)!= -1){			
			path = normalizeImportLocation(path);
			String fileNameLastPart = path.substring(path.lastIndexOf(FILE_SEPARATOR)+1);
			fileNameLastPart = fileNameLastPart.substring(0,fileNameLastPart.indexOf(WSDL_FILE_EXT));
			String catalogName = path.substring((path.indexOf(ROOT_EXTERNAL_MODEL)+ROOT_EXTERNAL_MODEL.length()),path.indexOf(ExportOperation.FILE_ATTACHMENT));
//			catalogName = catalogName.substring(0,catalogName.lastIndexOf(fileNameLastPart));			
			//strip space and to lower case
			catalogName = stripSpace(catalogName);
			//comply with the java nc name
			catalogName = convertCatalogNameToJavaNCName(catalogName);	
			return catalogName;
		}
		return " ";
	}

	/**
	 * Fix import for the target element
	 * @param element
	 * @param registry
	 * @param isArtifactDefinition TODO
	 */
	public static void fixImport(Object element,Map registry, boolean isArtifactDefinition,int commonPathLength){	
		List duplicateImports = null;
		Set wsdlImports = new HashSet();  
		if(element instanceof Definition){
			Definition definition = (Definition)element;
			//hack the code here for local task process implementation
			if(definition.getQName()!= null && definition.getQName().getLocalPart() != null){
				if(definition.getQName().getLocalPart().endsWith("Artifacts")){
					isArtifactDefinition = true;
				}
			}
			String enclosingingDefLocation = replaceConvertedSpaces(definition.getLocation());
			String importLocation = null;
			String reconciledLocation = null;
			List eImports = new LinkedList();			
			eImports.addAll(definition.getEImports());			
			if(definition.getETypes() != null){
				List schemas = definition.getETypes().getSchemas();
				if(schemas != null){
					Iterator schemasIt= schemas.iterator();
					while(schemasIt.hasNext()){
						XSDSchema schema = (XSDSchema)schemasIt.next();
						List contents = schema.getContents();
						Iterator importIt= contents.iterator();
						while(importIt.hasNext()){
							Object importDecl = importIt.next();
							if(importDecl instanceof XSDImport ||
									importDecl instanceof XSDRedefine||
									importDecl instanceof XSDInclude){
								eImports.add(importDecl);
							}
						}
					}
				}
			}			
			Iterator it = eImports.iterator();	
			while(it.hasNext()){
				Object eachImport = it.next();
				if(eachImport instanceof Import){					
					Import importObj = (Import)eachImport;
					importLocation = importObj.getLocationURI();
					//AZ 20070209 add to handle wsdl import for schema
					if(importLocation != null && 
							!importLocation.startsWith("file:/") && 
							importLocation.indexOf(ROOT_EXTERNAL_MODEL) != -1){
						importLocation = "file:/"+resolveAbsolutePathForImport(importObj);
					}
					reconciledLocation = reconcilePaths(getTargetPathFromRegistry(importLocation,importObj,registry, element),
							getTargetPathFromRegistry(enclosingingDefLocation,importObj,registry, element),commonPathLength);
					//AZ This will apply to the case of artifacts definition
					//defect 149073
					if(isArtifactDefinition){
						reconciledLocation = flatenRelativePath(reconciledLocation);
					}
					importObj.setLocationURI(reconciledLocation);
					//AZ 20070212 defect 146639 remove duplicate WSDL import
					if(wsdlImports.contains(importObj.getNamespaceURI() + importObj.getLocationURI())) {
						if (duplicateImports == null) {
							duplicateImports = new ArrayList();
						}
						duplicateImports.add(importObj);
					} else {
						wsdlImports.add(importObj.getNamespaceURI() + importObj.getLocationURI());
					}
				}else if(eachImport instanceof XSDImport){
					XSDImport xsdImport = (XSDImport)eachImport;	
					importLocation = xsdImport.getSchemaLocation();
					if(importLocation != null){
						reconciledLocation = reconcilePaths(getTargetPathFromRegistry(importLocation,xsdImport,registry, element),
								getTargetPathFromRegistry(enclosingingDefLocation,xsdImport,registry, element),commonPathLength);	
						
						String encodedProjectName = null; 
						String projectName = null; 
						if (importLocation.endsWith(XSD_FILE_EXT)) {
							encodedProjectName = importLocation.substring(0, importLocation.length() - 4);
						} else {
							encodedProjectName = importLocation;
						}
						if (encodedProjectName != null) {
							projectName = NamespaceUtils.convertUriToNamespace(encodedProjectName);
						}
						if (projectName != null) {
							if (!CrossProjectReferenceUtil.isPredefinedProject(projectName)) {		
								reconciledLocation = flatenRelativePath(reconciledLocation);	
							}
						} else {
							reconciledLocation = flatenRelativePath(reconciledLocation);
						}
						xsdImport.setSchemaLocation(reconciledLocation);
					}					
				}else if(eachImport instanceof XSDInclude){
					XSDInclude xsdInclude = (XSDInclude)eachImport;	
					importLocation = xsdInclude.getSchemaLocation();
					if(importLocation != null){
						reconciledLocation = reconcilePaths(getTargetPathFromRegistry(importLocation,xsdInclude,registry, element),
								getTargetPathFromRegistry(enclosingingDefLocation,xsdInclude,registry, element),commonPathLength);
						reconciledLocation = flatenRelativePath(reconciledLocation);
						xsdInclude.setSchemaLocation(reconciledLocation);
					}					
				}else if(eachImport instanceof XSDRedefine){
					XSDRedefine xsdRedefine = (XSDRedefine)eachImport;	
					importLocation = xsdRedefine.getSchemaLocation();
					if(importLocation != null){
						reconciledLocation = reconcilePaths(getTargetPathFromRegistry(importLocation,xsdRedefine,registry, element),
								getTargetPathFromRegistry(enclosingingDefLocation,xsdRedefine,registry, element),commonPathLength);
						reconciledLocation = flatenRelativePath(reconciledLocation);
						xsdRedefine.setSchemaLocation(reconciledLocation);
					}					
				}
			}
			//remove duplicate imports
			if (duplicateImports != null && !duplicateImports.isEmpty()) {
				Iterator duplicateIt = duplicateImports.iterator();
				while (duplicateIt.hasNext()) {
					Object importObj = duplicateIt.next();
					definition.getEImports().remove(importObj);
				}
			}
		}		
	}

	/**
	 * Get the source absolute path
	 * @param element
	 * @param absolutePathPrefix
	 * @return
	 */
	public static String getAbsolutePathForSource(Object element,String absolutePathPrefix,String relativePath,String namespace){
		String originalAbsolutePath = null;
		String relativeFilePath = null;
		String fileName = null;			
		if(element instanceof Definition){
			Definition definition  = (Definition)element;
			originalAbsolutePath = definition.getLocation();
			if(originalAbsolutePath != null &&
					originalAbsolutePath.indexOf(ROOT_EXTERNAL_MODEL) != -1){
				//This is the original absolute file path
				return 	normalizeImportLocation(originalAbsolutePath);	
			}
			if (definition.getQName()!=null) {
				fileName = definition.getQName().getLocalPart()+WSDL_FILE_EXT;
			}
			//This is the target of the TE created resource
			return namespace + fileName;
		}
		return null;
	}


	/**
	 * Retrieve the target path for the import object from the registry
	 * @param originalPathKey
	 * @param importObj
	 * @param registry
	 * @param enclosingElement TODO
	 * @return
	 */
	private static String getTargetPathFromRegistry(String originalPathKey,Object importObj,Map registry, Object enclosingElement){		
		Object valueList = null;		
		if(isExternalFile(originalPathKey)){
			originalPathKey = replaceConvertedSpaces(originalPathKey);
			valueList = registry.get(originalPathKey);				
		}else{
			String compoundKey = null;
			if(importObj instanceof Import){
				compoundKey = ((Import)importObj).getNamespaceURI();				
			}else if(importObj instanceof XSDImport){
				compoundKey = ((XSDImport)importObj).getNamespace();				
			}else if(importObj instanceof XSDInclude){
				Object container = ((XSDInclude)importObj).eContainer();
				if(container instanceof XSDSchema){
					compoundKey = ((XSDSchema)container).getTargetNamespace();
				}				
			}else if(importObj instanceof XSDRedefine){				
				Object container = ((XSDRedefine)importObj).eContainer();
				if(container instanceof XSDSchema){
					compoundKey = ((XSDSchema)container).getTargetNamespace();
				}			
			}
			if(originalPathKey != null){				
				compoundKey = compoundKey + originalPathKey;		
			}else{
				Object containingWSDL = ((EObject)importObj).eContainer();
				if(containingWSDL instanceof Definition){
					String fileName = ((Definition)containingWSDL).getQName().getLocalPart()+ WSDL_FILE_EXT;
					String tns = ((Definition)containingWSDL).getTargetNamespace();
					compoundKey = tns + fileName;
				}else if(containingWSDL instanceof XSDSchema){
					//This is the enclosing schema (Definition) target location	
					String targetFilePath = ((XSDSchema)containingWSDL).getSchemaLocation();
					if(targetFilePath != null && targetFilePath.startsWith(URI_FILE)){
						targetFilePath = targetFilePath.substring(6);
						return targetFilePath;
					}
					//Retrieve its definition
					XSDSchema inlineSchema = (XSDSchema)containingWSDL;	
					EObject container = inlineSchema.eContainer();
					while (container!= null && !(container instanceof Definition)){
						container = container.eContainer();
					}
					if(container instanceof Definition){
						String fileName = ((Definition)container).getQName().getLocalPart()+ WSDL_FILE_EXT;
						String tns = ((Definition)container).getTargetNamespace();
						compoundKey = tns + fileName;
					}					
				}
			}			
			valueList = registry.get(compoundKey);				
		}
		if(valueList instanceof List){
			List values = (List)valueList;
			String targetFilePath = (String)values.get(1);
			if(targetFilePath.startsWith(URI_FILE)){
				targetFilePath = targetFilePath.substring(6);
			}
			return targetFilePath.replace('\\','/');
		}
		return null;
	}

	/**
	 * Get the module name for the target source to go into
	 * Rules for best practice pattern support:
	 * 1. BPEL, Artifacts WSDL, Business Rule, Human Task (TEL), Import -> Top Level Module
	 * 2. Interface WSDL, schema XSD -> Library module
	 * 3. Component, Export -> Implementation Module
	 * @param element
	 * @return
	 */
	public static String getTopPackageName(Object element){	
		String projectName=CrossProjectReferenceUtil.getProjectNameForTargetObject(element);
		if(BpelOptionsUtil.isNoneOption()){
			return projectName;
		}
		String libName = BpelOptionsUtil.getWPSLibraryName(projectName);
		String moduleName = BpelOptionsUtil.getWPSModuleName(projectName);
		//AZ 20070314 support pattern
		String implName = BpelOptionsUtil.getImplementationModuleName(projectName);
		ExportSession exportSession = TransformationSessionUtil.getExportSession(TransformationEngine.getTransformationSession());
		if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
			if(element instanceof Definition || element instanceof XSDSchema){
				return libName;
			}else if(element instanceof DocumentRoot){
				DocumentRoot doc = (DocumentRoot)element;
				if(doc.getComponent() != null || doc.getExport() != null){					
					return implName;
				}
			}else if(element instanceof JavaImplementation ||element instanceof SACLRoot){
				return implName;
			}
			return moduleName;
		}else{
			if(moduleName == null && libName == null){
				return "";
			}
			if(element instanceof Definition || element instanceof XSDSchema){
				if(libName != null){
					return libName;
				}
				return moduleName;
			}
			return moduleName;
		}
	}	

	/**
	 * File Path Pattern:
	 * The resolved import WSDL definition or XSD schema location contains two parts:
	 * A. The containing file's absolute file directory
	 * B. The relative path of the imported file to the modeler project root
	 * C. Segment A is encoded according to URL specification.
	 * D. Segmment B is clear modeler relative path
	 * file:/D:/602AllInOnePlatform/eclipse/runtime-workspaceF/Project%201/RootExternalModel/Business%20services/import_wsdl_message/__FILE__ATTACHMENT/__FILE__ATTACHMENT/
	 * ..\..\..\..\..\RootExternalModel\Business services\inline_wraptype\__FILE__ATTACHMENT\__FILE__ATTACHMENT\inline_wraptype.wsdl
	 * @param importLocation
	 * @return
	 */
	public static String normalizeImportLocation(String importLocation){
		if(importLocation != null && 
				importLocation.indexOf(RELATIVE_PATH_TOKEN) != -1 && 
				importLocation.indexOf(ROOT_EXTERNAL_MODEL) != -1){
			int rootIndx = importLocation.indexOf(RELATIVE_PATH_TOKEN);
			String fileURLPart = importLocation.substring(0,rootIndx);
			String relativePart = importLocation.substring(rootIndx);
			String normalized = resolveURI(fileURLPart,relativePart);	
			normalized = replaceConvertedSpaces(normalized);
			return normalized;			
		}
		return replaceConvertedSpaces(importLocation);
	}

	/**
	 * Resolve the import absolute path for the import based on the container document's 
	 * target path absolute prefix
	 * @param importObj
	 * @return
	 */
	public static String resolveAbsolutePathForImport(Import importObj){
		Object container = importObj.eContainer();
		String absoluteImportedLocationPath = null;
		if(container instanceof Definition){					
			String containerLocation = ((Definition)container).getLocation();
			if(containerLocation == null){
				if(importObj.getLocationURI().startsWith(URI_FILE)){
					String imprtLocation = importObj.getLocationURI();
					return imprtLocation.substring(6);
				}
				return importObj.getLocationURI();
			}
			String importLocation = importObj.getLocationURI();	
			if(containerLocation.indexOf(ROOT_EXTERNAL_MODEL) == -1 || 
					importLocation.indexOf(ROOT_EXTERNAL_MODEL) == -1){
				return null;
			}
			String absolutePrefix = containerLocation.substring(0,containerLocation.indexOf(ROOT_EXTERNAL_MODEL));					
			absoluteImportedLocationPath = absolutePrefix + importLocation.substring(importLocation.indexOf(ROOT_EXTERNAL_MODEL));
			try{
				absoluteImportedLocationPath = URLDecoder.decode(absoluteImportedLocationPath,"UTF-8");
				URL url = new URL(absoluteImportedLocationPath);
				String urlFile = url.getFile();
				absoluteImportedLocationPath = urlFile.substring(1);
			}catch(Exception e){}	
			importObj.setLocationURI(URI_FILE + FILE_SEPARATOR + absoluteImportedLocationPath);
		}
		return absoluteImportedLocationPath;
	}	

	/**
	 * Check whether this document contains import. 
	 * @param containingWSDL Definition
	 * @param importObj Import
	 * @return flag boolean
	 */
	public static boolean wsdlContainsImport(Definition containingWSDL, Import importObj) {
		if (containingWSDL == null || importObj == null || containingWSDL.getEImports() == null) {
			return false;
		}

		Iterator it = containingWSDL.getEImports().iterator();
		Import importInWsdl = null;
		while (it.hasNext()) {
			importInWsdl = (Import)it.next();
			if (importInWsdl.getNamespaceURI() != null && importInWsdl.getNamespaceURI().equals(importObj.getNamespaceURI()) &&
					importInWsdl.getLocationURI() != null && importInWsdl.getLocationURI().equals(importObj.getLocationURI())) {
				return true;
			}
		}

		return false;
	}

	/**
	 * 
	 * @param targetList
	 * @return
	 */
	public static Process getProcessFromActivity(List targetList) {
		if (targetList == null || targetList.isEmpty()) {
			return null;
		}		
		Iterator it = targetList.iterator();
		Object obj = null;
		while (it.hasNext()){
			obj = it.next();
			if (obj instanceof Process){
				return (Process)obj;
			}
		}
		return null;
	}	

	public static com.ibm.wsspi.sca.scdl.DocumentRoot getComponentFromTargets(List targetList) {
		if (targetList == null || targetList.isEmpty()) {
			return null;
		}

		Iterator it = targetList.iterator();
		Object obj = null;
		while (it.hasNext()){
			obj = it.next();
			if (obj instanceof DocumentRoot &&
					((DocumentRoot)obj).getComponent() != null) {		// 2 Doc roots - get the component
				return (DocumentRoot)obj;
			}
		}
		return null;
	}	

	/**
	 * Calculate file name from the catalogue name.
	 * @param base String 
	 * @param relativePath String
	 * @return String fileName
	 */	
	public static String calculateFileNameFromCatalogue(String relativePath) {
		if (relativePath == null) {
			return null;
		}

		String relFolderName = null;
		//remove the file separator
		if (relativePath.endsWith(File.separator) && relativePath.length() > 1) {
			relFolderName = relativePath.substring(0, relativePath.length() - 1);
		} else {
			relFolderName = relativePath;
		}

		//get the catalogue folder name
		String catalogue = null;
		relFolderName = relFolderName.replace('/', File.separatorChar);
		if (relFolderName.indexOf(File.separatorChar) != -1) {
			catalogue = relFolderName.substring(relFolderName.lastIndexOf(File.separatorChar) + 1);	//last catalogue name
		}else {
			catalogue = relFolderName;
		}

		return catalogue;
	}

	/**
	 * This is to test if the definition is artifacts definition
	 * @param element
	 * @return
	 */
	public static boolean isArtifactsDefinition(Object element){
		if(element instanceof Definition){
			Definition def = (Definition)element;
			return def.getQName()!= null && 
			def.getQName().getLocalPart().endsWith(ExportOperationConstants.ARTIFACTS_SUFFIX);
		}
		return false;		
	}

	/**
	 * This method helps flaten the relative path in the case of library options specified with 
	 * module projects since WID resolves dependency based on class path instead of phisical file
	 * structure. Basically the direcotry is flatened without considering the lib project directory
	 * @param fixedRelativePath
	 * @return
	 */
	public static String flatenRelativePath(String fixedRelativePath){
		if(!BpelOptionsUtil.isLibraryEnabledWithModuleProject()){
			return fixedRelativePath;
		}
		if(!CrossProjectReferenceUtil.skipConsolidate(fixedRelativePath)){
			if(fixedRelativePath != null && fixedRelativePath.indexOf(RELATIVE_PATH_TOKEN) != -1){
				String pathPrefix = fixedRelativePath.substring(0,fixedRelativePath.lastIndexOf(RELATIVE_PATH_TOKEN));
				String pathSegment = fixedRelativePath.substring(fixedRelativePath.lastIndexOf(RELATIVE_PATH_TOKEN)+3);
				pathSegment = pathSegment.substring(pathSegment.indexOf(FILE_SEPARATOR)+1);
				return pathPrefix + pathSegment;
			}
		}
		return fixedRelativePath;
	}

	/**
	 * This method is used for lib only case to force saving the unreferenced XSD/WSDL imports
	 * @param options
	 * @return
	 */
	public static boolean isFileAttachment(Map options){
		if(options == null){
			return false;
		}
		if(options.containsKey(FILE_ATTACHMENT)){
			if(TRUE_LIETRAL.equals(((String)options.get(FILE_ATTACHMENT)))){
				return true;
			}
		}
		return false;
	}

	public static void persistImportResources(Map<String,Resource> importResourceMap){
		if(importResourceMap != null && !importResourceMap.isEmpty()){
			Iterator<String> filePathIt = importResourceMap.keySet().iterator();
			//this is the file path
			while(filePathIt.hasNext()){		
				String filePath = (String)filePathIt.next();
				if(existExportAndComponent(filePath)){
					continue;
				}
				Resource resource = (Resource)importResourceMap.get(filePath);
				//need to check if the export and component exist, we will
				//skip if component and export exist in the same directory
				try{
					Map<String, String> options = new HashMap<String, String>();			
					options.put(XMLResource.OPTION_ENCODING, "UTF-8");
					resource.save(options);
				}catch(Exception e){
					String params[] = {""};
					LoggingUtil.logError(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_SAVE_IMPORT_COMPONENT));
					throw runtimeEx;
				}
			}
		}		
	}

	private static boolean existExportAndComponent(String filePath) {
		if(filePath == null){
			return false;
		}
		String dirPath = null;
		String imprtName = null;
		boolean existComponent = false;
		boolean existExport = false;
		if(filePath.indexOf(File.separator) != -1){
			dirPath = filePath.substring(0,filePath.lastIndexOf(File.separator));
			imprtName = filePath.substring(filePath.lastIndexOf(File.separator)+1);
		}else{
			dirPath = filePath.substring(0,filePath.lastIndexOf(FILE_SEPARATOR));
			imprtName = filePath.substring(filePath.lastIndexOf(FILE_SEPARATOR)+1);
		}
		imprtName=imprtName.substring(0,imprtName.lastIndexOf(SCA_IMPORT_FILE_EXT));			 
		File dir = new File(dirPath);
		if(dir.exists()){
			String[] children = dir.list();
			if (children == null) {
				return true;
			} else {
				for (int i=0; i<children.length; i++) {
					// Get filename of file or directory
					String filename = children[i];	
					if(filename != null && filename.equals(imprtName+SCA_COMPONENT_FILE_EXT)){
						existComponent = true;
					}else if(filename != null  && filename.equals(imprtName+SCA_EXPORT_FILE_EXT)){
						existExport = true;
					}
				}
			}
		}        
		return (existComponent && existExport);
	}	
	/**
	 * Get the human being readable timestamp, which is the same pattern as 
	 * the delta model files
	 * @return
	 */
	public static String getTimeStamp(){
		String now;		
		Calendar cal = new GregorianCalendar();
		// Get the components of the time
		int year = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH) + 1;
		int day = cal.get(Calendar.DAY_OF_MONTH);
		int hour = cal.get(Calendar.HOUR_OF_DAY);
		int min = cal.get(Calendar.MINUTE);
		int sec = cal.get(Calendar.SECOND);
		NumberFormat format2Dec = new DecimalFormat("00");

		String smonth = format2Dec.format(month);
		String sday = format2Dec.format(day);
		String shour = format2Dec.format(hour);
		String smin = format2Dec.format(min);
		String ssec = format2Dec.format(sec);

		now = year + "-" +  smonth + "-" + sday + "T" + shour + "." + smin + "." + ssec;

		return now;
	}

	/**
	 * convert a single char to corresponding nibble
	 * @param c char to convert. must be 0-9 a-f A-F, no
	 * spaces, plus or minus signs.
	 *
	 * @return corresponding integer
	 */
	private static int charToNibble ( char c )
	{
		if ( '0' <= c && c <= '9' )
		{
			return c - '0';
		}
		else if ( 'a' <= c && c <= 'f' )
		{
			return c - 'a' + 0xa;
		}
		else if ( 'A' <= c && c <= 'F' )
		{
			return c - 'A' + 0xa;
		}
		else
		{
			throw new IllegalArgumentException ( "Invalid hex character: " + c );
		}
	}
   
	/**
	 * Convert the hex string to byte array
	 * @param s
	 * @return
	 */
	private static byte[] fromHexString ( String s )
	{
		int stringLength = s.length();
		
		if ( (stringLength & 0x1) != 0 )
		{
			throw new IllegalArgumentException ( "fromHexString requires an even number of hex characters" );
		}
		
		byte[] b = new byte[stringLength / 2];

		for ( int i=0,j=0; i<stringLength; i+=2,j++ )
		{
			int high = charToNibble( s.charAt ( i ) );
			int low = charToNibble( s.charAt ( i+1 ) );
			b[j] = (byte)(( high << 4 ) | low );
		}
		return b;
	}
    
	/**
	 * We assume the hex string is diverse evenly and we pick the every position
	 * 4 hex char to assemble a hex string with length of 8, which can be directly
	 * mapped to int
	 * @param originalHexString
	 * @return
	 */
	private static String pick(String originalHexString){
		StringBuffer sb = new StringBuffer();
		if(originalHexString != null && originalHexString.length() == 32){
			for(int i=3;i<32;){
				sb.append(originalHexString.charAt(i));
				i = i+4;
			}
			return sb.toString();
		}
		return sb.toString();
	}

	/**
	 * Convert the byte array to an int starting from the given offset.
	 * @param b The byte array
	 * @param offset The array offset
	 * @return The integer
	 */
	private static int byteArrayToInt(byte[] b, int offset) {
		int value = 0;      
		for (int i = 0; i < 4; i++) {
			int shift = (4 - 1 - i) * 8;
			value += (b[i + offset] & 0x000000FF) << shift;
		}
		return value;
	}

	/**
	 * Validate the uid to ensure the uniqueness of the id
	 * @param uid
	 * @return
	 */
	private static int validateUid(int uid){
		if(uidList == null){
			uidList = new ArrayList<Integer>();
		}
		while(uidList.contains(new Integer(uid))){
			uid++;
		}
		uidList.add(new Integer(uid));
		return uid;
	}
    
	/**
	 * Reset the uid int list to null
	 *
	 */
	public static void resetUidList(){
		uidList = null;
	}
    
	/**
	 * Generate unique id for the bpel artifacts
	 * Note: Although we can ensure the uniqueness of the id within the process context,
	 * we cannot guarantee that BPELVariable with the same and type will have the same
	 * id due to the fact the the BPEL variables are created based on the data flow
	 * and any miss-match of the order of objectpins may result in collision of the BPELVariable ID
	 * @param eObj
	 * @return
	 */
	public static Integer generateConstructUidForTarget(EObject eObj){
		int uid = 0;
		String bomUID = CrossProjectReferenceUtil.getBOMUidForTarget(eObj);		
		//This is the case that we didn't map the target to the source BOM uid		
		if(bomUID == null || bomUID.length()==0){
			uid = 1000;
			uid = validateUid(uid);
			return new Integer(uid);
		}    
		uid = byteArrayToInt(fromHexString(pick(bomUID)),0);
		uid = validateUid(uid);
		return new Integer(uid);
	}
	
	
	/**
	 * Returns a new generated name that is unique in the collection 
	 * @param duplicatedName  the duplicated name that needs to be replaced
	 * @param values Collection
	 * @return String the newly generated string name
	 */	
	public static String generateUniqueXSDPath(String duplicatedName, Collection<String> values) {
		// create path/folderForXSD2/xsdFileName.xsd
		if (duplicatedName == null || values == null) {
			return duplicatedName;
		}
		
		String fileName = null;
		String folderName = null;
		String path = "";
		
		if (duplicatedName.indexOf(File.separatorChar) != -1) {
			fileName = duplicatedName.substring(duplicatedName.lastIndexOf(File.separatorChar) + 1);
			String pathNoFile = duplicatedName.substring(0, duplicatedName.lastIndexOf(File.separatorChar));
			if (pathNoFile.indexOf(File.separatorChar) != -1) {
				folderName = pathNoFile.substring(pathNoFile.lastIndexOf(File.separatorChar) + 1);
				path = pathNoFile.substring(0, pathNoFile.lastIndexOf(File.separatorChar) + 1);   // include file separator
			} else {
				folderName = pathNoFile;
			}
		} else {
			fileName = duplicatedName;
		}
		
		int count = 2;
		String newFolderName = folderName + "_" + count++;
		String newName = path + newFolderName;
		// Keep trying to generate new name until name is unique
		while (values.contains((newName + File.separatorChar + fileName).toLowerCase())) {
			if (folderName == null) {
				newName = duplicatedName  + "_" + count++;
			} else {
				newFolderName = folderName  + "_" + count++;
				newName = path + newFolderName;
			}
		}
		return newName + File.separatorChar + fileName;
	}
	
	
	/**
	 * Create system property for the business rule group as type of user defined property
	 */
	public static void createSystemCustomProperty(EObject eObj, String name, String value){
		if(eObj == null){
			return;
		}		
		if(eObj instanceof BusinessRuleGroup){			
			BusinessRuleGroup brg = (BusinessRuleGroup)eObj ;
			Property systemProperty = com.ibm.wbiservers.common.componentdef.ComponentdefFactory.eINSTANCE.createProperty();
			systemProperty.setType(Property.PROPERTY_TYPE__SYSTEM_DEFINED);
			systemProperty.setName(name);
			systemProperty.setValue( value);
			brg.getProperties().add(systemProperty);	
		}else if(eObj instanceof RuleSet){
			RuleSet rs = (RuleSet)eObj ;
			com.ibm.wbit.br.core.model.Property ruleSetModelProperty = com.ibm.wbit.br.core.model.ModelFactory.eINSTANCE.createProperty();
			ruleSetModelProperty.setType(com.ibm.wbit.br.core.model.Property.PROPERTY_TYPE__SYSTEM_DEFINED);
			ruleSetModelProperty.setName(name);
			ruleSetModelProperty.setValue( value);
			rs.getProperty().add(ruleSetModelProperty);	
		}else if(eObj instanceof TemplateInstanceRule){
			TemplateInstanceRule tir = (TemplateInstanceRule)eObj;
			com.ibm.wbit.br.core.model.Property ruleModelProperty = com.ibm.wbit.br.core.model.ModelFactory.eINSTANCE.createProperty();
			ruleModelProperty.setType(com.ibm.wbit.br.core.model.Property.PROPERTY_TYPE__SYSTEM_DEFINED);
			ruleModelProperty.setName(name);
			ruleModelProperty.setValue( value);
			tir.getProperty().add(ruleModelProperty);
		}else if(eObj instanceof com.ibm.wbit.br.core.model.DocumentRoot){
			com.ibm.wbit.br.core.model.DocumentRoot docRoot = (com.ibm.wbit.br.core.model.DocumentRoot)eObj;
			if(docRoot.getRuleSet() != null){
				RuleSet rs = docRoot.getRuleSet() ;
				com.ibm.wbit.br.core.model.Property ruleSetModelProperty = com.ibm.wbit.br.core.model.ModelFactory.eINSTANCE.createProperty();
				ruleSetModelProperty.setType(com.ibm.wbit.br.core.model.Property.PROPERTY_TYPE__SYSTEM_DEFINED);
				ruleSetModelProperty.setName(name);
				ruleSetModelProperty.setValue( value);
				rs.getProperty().add(ruleSetModelProperty);	
			}
		}
	}		
	/**
	 * Validate the implementation type, specifically check if the user selects Dynamic Assembler
	 * implement type for non-WBSF mode
	 * @param action
	 * @param implmentation
	 */
	public static void validateImplementationType(Action action, Implementation implmentation){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || implmentation == null||action == null){
			return;
		}
		if(implmentation instanceof DynamicAssembler){
		//logwarning						
			String params[] = {action.getName()};
			LoggingUtil.logWarning(MessageKeys.NOT_SUPPORTED_IMPLEMENTATION_TYPE, params, null);
		}
	}
	
	/**
	 * Check the implementation type for the action
	 * @param avp
	 * @param action
	 * @return boolean whether the implementation type is DA or not
	 */
	public static boolean isDynamicAssembler(AttributeValueProvider avp,Action action) {
		if(avp == null || action == null){
			return false;
		}
		Object implementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
		if(implementation  instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.DynamicAssembler){
			return true;
		}
		return false;
	}
	
	/**
	 * Propagate the external document from the bom comment flag to the export context
	 * @param externalDocument
	 */
	public static void addToExtDocumentWithArtCatList(ExternalDocument externalDocument){
		if(externalDocument == null){
			return;
		}
		List<Comment> comments = externalDocument.getOwnedComment();
		if(comments == null || comments.size() == 0){
			return;
		}
		for(Comment comment:comments){
			String uid = comment.getUid();
			if(uid.startsWith(IImportConstants.ARTIFICIAL_DEFAULT_CATALOG_UID_PREFIX)){
				ExportSession exportSession = BpelOptionsUtil.getExportSession();
				if(exportSession == null){
					return;
				}
				Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
				if(additionalOptions != null){
					Object registry = additionalOptions.get(IImportConstants.ARTIFICIAL_DEFAULT_CATALOG_FOR_EXTERNAL_DOCUMENT_KEY);
					if(!(registry instanceof Boolean)){
						registry = new Boolean(true);
						additionalOptions.put(IImportConstants.ARTIFICIAL_DEFAULT_CATALOG_FOR_EXTERNAL_DOCUMENT_KEY, registry);
					}					
				}
				break;
			}
		}
	}	

	/**
	 * This method is to determine whether we need to skip the default catalog
	 * @return
	 */
	public static boolean skipDefaultCatalog(){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return false;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(IImportConstants.ARTIFICIAL_DEFAULT_CATALOG_FOR_EXTERNAL_DOCUMENT_KEY);				
			if(registry instanceof Boolean){
				Boolean flagObject = (Boolean)registry;
				return flagObject.booleanValue();
			}
		}
		return false;
	}
	
	public static boolean ignoreDefaultCatalog(ExternalModel extModel){
		if(extModel == null){
			return false;
		}
		//The default catalog
		if(extModel.getUid().startsWith("FID-")){
			return skipDefaultCatalog();
		}
		return false;
	}
	
	/**
	 * Compute the key for the TTask import WSDL absolute file location
	 * @param tImport
	 * @return
	 */
	public static String createKeyForTTaskImport(com.ibm.wbit.tel.TImport tImport) {
		if (tImport == null || tImport.getLocation() == null) {
			return null;
		}		
		String importLocation = tImport.getLocation().toFileString();
		StringBuffer key = null;
		String keyResult = null;

		if (isExternalFile(importLocation)) {	// key is the import file location
			String externalFile = replaceConvertedSpaces(importLocation);
			key = new StringBuffer(externalFile);
			keyResult = replaceConvertedSpaces(key.toString());
		} else {
			if (tImport.getNamespace() != null) {
				key = new StringBuffer(tImport.getNamespace().toString());
			}

			if (importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) != -1) {
				if (key == null) {
					key = new StringBuffer(importLocation.substring(importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) + 1));
				} else {
					key.append(importLocation.substring(importLocation.lastIndexOf(FILE_SEPARATOR.charAt(0)) + 1));
				} 
			} else {
				if (key == null) {
					key = new StringBuffer(importLocation);
				} else {
					key.append(importLocation);
				} 				
			}			
			keyResult = key.toString();
		}			
		return keyResult;
	}
    
	/**
	 * Prefix the project NCName to the regular derived schema name by the following rules:
	 * 1. If library name is available, library name is utilized
	 * 2. If module only and none option, we don't mangle the folder name or schema name
	 * 3. The schema folder name will not be mangled with library project name prefix
	 * @param xsd
	 * @param name
	 * @return
	 */
	public static String generateUniqueSchemaName(XSDSchema xsd, String name) {
		if(xsd == null || name == null){
			return null;
		}
		String projectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(xsd);
		if(projectName != null){				
			String libName = BpelOptionsUtil.getWPSLibraryName(projectName);	
			if( libName != null){
				return libName+"_"+name;
			}
		}
		return name;
	}
}
