package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.text.MessageFormat;

import com.ibm.btools.blm.ie.exprt.ExportMessage;
import com.ibm.btools.blm.ie.exprt.ExportResult;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.util.logging.LogHelper;

/**
 * This utility class contains static methods used for logging information.
 *
 */
public class LoggingUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static int errorCount=0;
	private static int warningCount=0;

	/**
	 * Return the number of errors that have been logged
	 * @return the number of errors logged
	 */	
	public static int getErrorCount() {
		return errorCount;
	}

	/**
	 * Return the number of errors that have been logged
	 * @return the number of errors logged
	 */	
	public static int getWarningCount() {
		return warningCount;
	}
	
	/**
	 * Reset all the error logs count
	 *
	 */	
	public static void resetErrorCount() {
		errorCount = 0;
	}

	/**
	 * Reset all the warning logs count
	 *
	 */	
	public static void resetWarningCount() {
		warningCount = 0;
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 */
	public static void logError(final String errorCode, final String[] param, final Throwable exception) {
		errorCount++;
		//LogHelper.log(LogHelper.ERROR, IlmPlugin.getDefault(), MessageKeys.MESSAGE_PROPERTY_FILE, errorCode, param, exception);
		LogHelper.log(LogHelper.ERROR, IlmPlugin.getDefault(), MessageKeys.class, errorCode, param, exception, null);
		recordInExportResult(ExportMessage.ERROR, errorCode, param);										
	}
	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 * @param exception			the original exception
	 **/
	public static void logWarning(final String errorCode, final String[] param, final Throwable exception) {
		warningCount++;
		//LogHelper.log(LogHelper.WARNING, IlmPlugin.getDefault(), MessageKeys.MESSAGE_PROPERTY_FILE, errorCode, param, exception);
		LogHelper.log(LogHelper.WARNING, IlmPlugin.getDefault(), MessageKeys.class, errorCode, param, exception, null);
		recordInExportResult(ExportMessage.WARNING, errorCode, param);			
	}

	/**
	 * Logs the error. Uses com.ibm.btools.util.LogHelper. 
	 * @param plugin			Used for logging. If it is null, will just write to the System.out.
	 * @param resourceBundle	ResourceBundle contains the localized message.	
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 * @param param				parameters of the message.
	 */
	public static void logError(final String errorCode) {
		logError(errorCode,null,null);
//		LogHelper.log(IStatus.ERROR, IlmPlugin.getDefault(), MessageKeys.MESSAGE_PROPERTY_FILE, errorCode, null, null);
	}

	/**
	 * Logs the warning. Uses com.ibm.btools.util.LogHelper. 
	 * @param errorCode			Used to represent a message, if the error code cannot be found, it will be just returned as it is.
	 */
	public static void logWarning(final String errorCode) {
		logWarning(errorCode,null,null);
//		LogHelper.log(IStatus.WARNING, IlmPlugin.getDefault(), MessageKeys.MESSAGE_PROPERTY_FILE, errorCode, null, null);
	}
	
	/**
	 * Trace Entry 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceEntry(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(IlmPlugin.getDefault(), thisObject, methodName, message, MessageKeys.PLUGIN_ID);
		}
			
	}

	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceExit(final Object thisObject, final String methodName) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceExit(IlmPlugin.getDefault(), thisObject, methodName, "", MessageKeys.PLUGIN_ID);
		}
	}		
	
	/**
	 * Trace Entry 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceEntry(final Object thisObject, final String methodName) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(IlmPlugin.getDefault(), thisObject, methodName, "", MessageKeys.PLUGIN_ID);
		}
	}

	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void traceExit(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceExit(IlmPlugin.getDefault(), thisObject, methodName, message, MessageKeys.PLUGIN_ID);
		}
	}	
	
	/**
	 * Trace Exit 
	 * @param thisObject	Used to determine the class name, or null if not applicable.
	 * @param methodName	a method name, , or null if not applicable.
	 * @param message		a message, or null if not applicable.
	 */
	public static void trace(final Object thisObject, final String methodName, final String message) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.trace(LogHelper.INFO, IlmPlugin.getDefault(), thisObject, methodName, message, "", MessageKeys.PLUGIN_ID);
		}
	}

	/**
	 * Records message in export result
	 * @param severity
	 * @param errorCode
	 * @param params
	 */	
	public static void recordInExportResult(int severity, String errorCode, String[] params) {
		//ExportSession session = TransformationEngine.getExportSession();
		TransformationSession context = TransformationEngine.getTransformationSession();
		ExportResult result = TransformationSessionUtil.getExportResult(context);
		if(result != null){
			String msg = LogHelper.getLocalizedString(MessageKeys.class, errorCode);
			if(params != null){
				msg = MessageFormat.format(msg, params);
			}					
			ExportMessage expMsg = new ExportMessage(severity, msg, errorCode);
			result.addMessage(expMsg);	
		}
	}		
}
