package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.List;

import com.ibm.btools.blm.ie.exprt.ExportOptions;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.ie.ui.ilm.config.BOM2ILMSettingHelper;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTESetting;
import com.ibm.btools.model.modelmanager.copyregistry.CopyRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;

/** 
 * This class is responsible for runing the validation operation.
 */
public class RunValidationOperation implements TransformationSessionKeyConstants {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private Activity ivWorkingCopy;
	private List ivValidationRecords;
	
	/**
	 * 
	 */
	public RunValidationOperation(Activity workingCopy) {
		ivWorkingCopy = workingCopy;
		
	}
	/**
	 * Runs the validation operation. The validation operates on a model BOM Process and produces
	 * a list of TEValidationRecords.
	 *
	 */
	public void execute() {
		LoggingUtil.traceEntry(this, "execute");
		
		if(ivWorkingCopy == null){
			return;
		}
	
		TransformationEngine aTransformationEngine = new TransformationEngine();
		ExportSession exportSession = new ExportSession();
		
//		ExportOptions exportOptions = new ExportOptions();
//		exportOptions.setAdditionalOption(RUN_VALIDATION, Boolean.TRUE);
//		exportSession.setExportOption(exportOptions);
		
		TransformationSession session = new TransformationSession();
		session.put(RUN_VALIDATION, Boolean.TRUE);
		
		
		Activity masterCopy = (Activity)CopyRegistry.instance().getMaster(ivWorkingCopy);
		TransformationSessionUtil.putMasterCopy(session, masterCopy);
		
		aTransformationEngine.registerTransformationSession(session);
		
		BOM2ILMTESetting teSetting = BOM2ILMSettingHelper.createAndInitializeTESetting();
		BOM2ILMSettingHelper.setWBITargetType(teSetting);
		
		List modelElements = new ArrayList();
		modelElements.add(ivWorkingCopy);
		
		//aTransformationEngine.execute(modelElements, teSetting);
		aTransformationEngine.execute(modelElements, exportSession);
		
		ivValidationRecords = TransformationSessionUtil.getValidationRecordList(session);
		
		
		aTransformationEngine.deregisterTransformationSession();	
		
		LoggingUtil.traceEntry(this, "execute");		
	}
	
	/**
	 * Returns the model working copy activity.
	 * @return BOM Activity
	 */
	public Activity getWorkingCopy(){
		return ivWorkingCopy;
	}
	
	/**
	 * Sets the working copy activity.
	 * @param workingCopy a BOM Activity
	 */
	public void setWorkingCopy(Activity workingCopy){
		ivWorkingCopy = workingCopy;
	}
	
	/**
	 * Returns a list of TEValidationRecords produced by the RunValidationOperation.
	 * @return
	 */
	public List getValidationRecordsList(){
		return ivValidationRecords;
	}
	

}
