package com.ibm.btools.te.ilm;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;

import com.ibm.btools.te.ilm.bptransformation.BOMTransformer;
import com.ibm.btools.te.ilm.Transformer;
import com.ibm.btools.te.ilm.graphs.BPGraph;
import com.ibm.btools.te.ilm.graphs.ProcessFlowGraph;
import com.ibm.btools.te.ilm.heuristics.HeuristicsTransformer;

/**
 * Creates a com.ibm.btools.te.framework.Transformer 
 */
public class TransformerFactory {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * Creates a transformer
	 * @param source - an object source
	 * @return Transformer - transformer created depending on 
	 * the type of source	 
	 */
	public static Transformer createTransformer(Object source)
	{
		Transformer transformer=null;
		if (source instanceof ProcessFlowGraph)
		{ 
			// TODO: this should take into account for User Perference,
			// e.g. we may want to tranform to a different IT model like FDL
			// instead of BPEL
//			transformer = new ProcessFlowToBpelSATransformer();	
		} else if (source instanceof BPGraph)
		{
			transformer = new HeuristicsTransformer();
//		} else if (source instanceof SolutionGraph || source instanceof SABundle)
//		{
//			transformer = new SolutionTransformer();
		} else {
			// TODO: in the future, we may take a BLM other than BOM
			transformer = new BOMTransformer();
		}
		return transformer;
	}

	/**
	 * Creates a transformer
	 * @param sources - a collection of source objects
	 * @return Transformer - transformer created depending on
	 * type of source
	 */
	public static Transformer createTransformer(Collection sources)
	{
		return createTransformer(sources.iterator().next());
	}
}
