package com.ibm.btools.te.ilm.attribute;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;

/**
* This class is responsible for providing the values for specific types of Objects. 
*/
public abstract class AttributeValueProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	//Transformation Context
	protected TransformationContext context ; 
	//Parent Context for which the attribute value is unique
	protected Object inParentContext;
	
	/**
	 * Returns the value of the technical attribute of the <code>propertyName</code> derived from the <code>fromSource</code>.
	 * @param type an Object
	 * @param propertyName the propertyName
	 * @param fromSource a BOM NamedElement
	 */	
	public abstract Object getAttributeValueForType(NamedElement fromSource, String propertyName);
	
	/**
	 * Returns the technical attribute for the <code>propertyName</code> value for the Object <code>type</code> in the specified <code>context</code>
	 * @param type an EObject
	 * @param attribute the attribute name of the type
	 * @param context a TransformationContext
	 * @return the value of the attribute for the type.
	 */
	public abstract Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry);
	
	/**   
	 * Sets the TransformationContext
	 * @param transformationContext
	 */
	public void setContext(TransformationContext transformationContext){
		context = transformationContext;
	}
	
	/** 
	 * Returns the TransformationContext
	 * @return TransformationContext
	 */
	public TransformationContext getContext(){
		return context;
	}

	/**   
	 * Set the object for which the property should be unique
	 * @param transformationContext
	 */
	public void setInParentContext(Object inParentContext){
		this.inParentContext = inParentContext;
	}
	
	/** 
	 * Returns the parentObject used to determine uniqueness.
	 * @return TransformationContext
	 */
	public Object getInParentContext(){
		return inParentContext;
	}

	
}
