package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.helper.UpdateCommandHelper;
import com.ibm.btools.te.attributes.resource.MessageKeys;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TTask;

/**
* Class responsible for providing values for BPEL type attributes. 
*/
public class TELAttributeValueProvider extends AttributeValueProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Returns the <code>propertyName</code> for <code>fromSource</code>.
	 * @param propertyName the name of the property.
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {
		return getAttributeValueForType(getObjectType(propertyName), fromSource, propertyName, null);
	}
	/**
	 * Returns the technical attribute value for the <code>propertyName<code> for the Object <code>forType</code> from the specified <code>registry</code>
	 * @param type an EObject
	 * @param attribute the attribute name of the type
	 * @param context a TransformationContext
	 * @return the value of the attribute for the type.
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		if (forType==null) {
			forType = getObjectType(propertyName);
		}
		if (propertyName.equals(AttributeNameConstants.TEL_TASK_DISPLAY_NAME) || propertyName.equals(AttributeNameConstants.TEL_ESCALATION_DISPLAY_NAME)) {
			return getDisplayName(forType,fromSource,propertyName,registry);
		} else {
			return getName(forType,fromSource,propertyName,registry);
		}
	}
	
	/**
	 * Returns the value of the name attribute for the <code>type</code> Object derived from the <code>fromSource</code>.
	 * @param type an TEL Object type
	 * @param fromSource an BOM NamedElement
	 * @param attribute attribute 
	 * @param registry a NamingRegistry
	 * @return String name of TEL element. 
	 */
	protected String getName(Object type, EObject fromSource, String attribute, NamingRegistry registry){
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		nameProvider.setContext(this.getContext());
		nameProvider.setParentContext(inParentContext);
		String	valueForType = nameProvider.getName(type, (NamedElement)fromSource, registry);
		return valueForType;
		
	}

	/**
	 * Returns the TEL display name for a TEL Element, if applicable. The name is not unique in the registry.
	 * @param type a TEL Object
	 * @param fromSource the BOM source element.
	 * @param attribute the attribute property name 
	 * @param registry a NamingRegistry
	 * @return String TEL display name.
	 */
	private String getDisplayName(Object type,  EObject fromSource, String attribute, NamingRegistry registry){
		String displayName = null;
		NamedElement displaySource = (NamedElement)fromSource;
		NameProvider nameProvider = NameProviderFactory.getNameProvider(type);
		//the display name should not be unique in the registry.. so do not pass in the registry... 
		displayName = nameProvider.getName(type, displaySource, null);
		return displayName;
	}
	
	/**
	 * Returns the object Class based on the name of the property.
	 * @param propertyName - constant for a specific property. 
	 * @return Object - object Class based on the name of the property
	 */
	private Object getObjectType(String propertyName){
		Class objectType = null;
		if(propertyName.equals(AttributeNameConstants.TEL_TASK_NAME) || propertyName.equals(AttributeNameConstants.TEL_TASK_DISPLAY_NAME)){
			objectType = TTask.class;
		}else if(propertyName.equals(AttributeNameConstants.TEL_ESCALATION_NAME)||propertyName.equals(AttributeNameConstants.TEL_ESCALATION_DISPLAY_NAME)){
			objectType = TEscalation.class;
		}else if(propertyName.equals(AttributeNameConstants.TEL_EMAIL_NAME)){
			objectType = TEmail.class;
		}
		return objectType;
	}
}
