package com.ibm.btools.te.ilm.attribute.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;

/**
 * Class responsible for providing attribute values for WSDL Elements.
 */
public class WSDLAttributeValueProvider extends AttributeValueProvider{
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String)
	 */
	public Object getAttributeValueForType(NamedElement fromSource, String propertyName) {
		return getAttributeValueForType(getObjectType(propertyName), fromSource, propertyName, null);
	}
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.attribute.AttributeValueProvider#getAttributeValueForType(com.ibm.btools.bom.model.artifacts.NamedElement, java.lang.String, com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry)
	 */
	public Object getAttributeValueForType(Object forType, NamedElement fromSource, String propertyName, NamingRegistry registry) {
		String valueForType = null;
		//AZ added to handle external service's part name, reuse it instead of generate
		boolean isExternalService = false;
		if(fromSource.eContainer() instanceof StructuredActivityNode){
			StructuredActivityNode san = (StructuredActivityNode)fromSource.eContainer();
			if("ServiceOperation".equals(san.getAspect())){
				isExternalService= true;
			}
		}
		if(isExternalService){
			if(isNameAttribute(propertyName)){
				return fromSource.getName();
			}else if(isTargetNamespaceAttribute(propertyName)){
				TargetNamespaceProvider namespaceProvider = new TargetNamespaceProvider();
				namespaceProvider.setContext(getContext());
				valueForType =  namespaceProvider.getTargetNamespace(forType, fromSource, propertyName, context, true);
			}
		}else{
			if(isNameAttribute(propertyName)){
				NameProvider nameProvider = NameProviderFactory.getNameProvider(forType);
				nameProvider.setContext(this.getContext());
				nameProvider.setParentContext(inParentContext);
				valueForType = nameProvider.getName(forType, fromSource, registry, propertyName);
			}else if(isTargetNamespaceAttribute(propertyName)){
				TargetNamespaceProvider namespaceProvider = new TargetNamespaceProvider();
				namespaceProvider.setContext(getContext());
				valueForType =  namespaceProvider.getTargetNamespace(forType, fromSource, propertyName, context, true);
			}
		}
		return valueForType;
	}
	
	
	/**
	 * Returns true if this attribute is for a <code>name</code> property of a WSDL Element.
	 * @param attribute the name of the attribute
	 * @return
	 */
	private boolean isNameAttribute(String attribute){
		return attribute.equals(AttributeNameConstants.WSDL_PORT_TYPE_NAME) || 
		attribute.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_NAME) ||
		attribute.equals(AttributeNameConstants.WSDL_OPERATION_NAME) ||
		attribute.equals(AttributeNameConstants.WSDL_MESSAGE_NAME) ||
		attribute.equals(AttributeNameConstants.WSDL_PART_NAME);
		
	}
	/**
	 * Returns true if the property is the <code>targetNamespace</code> attribute..
	 * @param attribute
	 * @return boolean
	 */
	private boolean isTargetNamespaceAttribute(String attribute){
		return attribute.equals(AttributeNameConstants.WSDL_TARGET_NAMESPACE)||attribute.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE) ; 
		
	}
	
	/**
	 * Returns the object Class based on the name of the property.
	 * @param propertyName - constant for a specific property. 
	 * @return Object - object Class based on the name of the property
	 */
	private Class getObjectType(String propertyName){
		Class objectType = null;
		if(propertyName.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_NAME) || propertyName.equals(AttributeNameConstants.WSDL_PORT_TYPE_NAME)){
			objectType = PortType.class;
		}else if(propertyName.equals(AttributeNameConstants.WSDL_MESSAGE_NAME)){
			objectType = Message.class;
		}else if(propertyName.equals(AttributeNameConstants.WSDL_PART_NAME)){
			objectType = Part.class;
		}else if(propertyName.equals(AttributeNameConstants.WSDL_OPERATION_NAME)){
			objectType = Operation.class;
		}
		return objectType;
	}
}