package com.ibm.btools.te.ilm.form;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.blm.ie.exprt.ExportOptions;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.ArtifactsFactory;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.processes.humantasks.FormData;
import com.ibm.btools.bom.model.processes.humantasks.FormDataType;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.ilm.IlmPlugin;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.TransformationSession;
import com.ibm.btools.te.ilm.heuristics.HeuristicsFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.model.utils.XSDUtil;
import com.ibm.wbit.tel.generation.forms.FormsGenerationException;
import com.ibm.wbit.tel.generation.forms.LotusFormsGenerator;

/**
 * FormGenerator provides API for generating an XFDL form from a BOM Form object
 * 
 */
public class FormGenerator {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	public static String ASPECT_FOR_TEMP_FORM_SAN="TEMP_TASK_FOR_FORM"; //$NON-NLS-1$
	public static String INPUT_PINSET_SUFFIX = "ips"; //$NON-NLS-1$
	private static FormGenerator formGenerator=new FormGenerator();
	private static TransformationRoot transformationRoot = null;
	
	private static  Map<XSDElementDeclaration, List<XSDTypeDefinition>> visitedForRecursive = new HashMap<XSDElementDeclaration, List<XSDTypeDefinition>>();
	private static  Map<XSDTypeDefinition, List<XSDElementDeclaration>> elementsForDefinition = new HashMap<XSDTypeDefinition, List<XSDElementDeclaration>>();
	private static  Map<XSDElementDeclaration, XSDTypeDefinition>  markedForRecursion = new HashMap<XSDElementDeclaration, XSDTypeDefinition>();
	
	/**
	 * Get singleton instance of the form generator
	 * @return
	 */
	public static FormGenerator getInstance() {
		return formGenerator;
	}

	/**
	 * Create a temporary StructuredActivityNode based on BOM Form object for generating WSDL/XSD required for XFDL generation
	 * @param form
	 * @return
	 */
	protected StructuredActivityNode createSANFromForm (Form form) {
		StructuredActivityNode san = ActivitiesFactory.eINSTANCE.createStructuredActivityNode();
		san.setName(form.getName());
		san.setAspect(ASPECT_FOR_TEMP_FORM_SAN);
		san.setUid(form.getUid());
		InputPinSet ips = ActivitiesFactory.eINSTANCE.createInputPinSet();
		ips.setName(form.getName()+INPUT_PINSET_SUFFIX);
		ips.setUid(form.getUid()+INPUT_PINSET_SUFFIX);
		san.getInputPinSet().add(ips);
		ips.setName(form.getName());
		for (Iterator it=form.getData().iterator();it.hasNext();) {
			FormData formData = (FormData)it.next();
			InputObjectPin iop = ActivitiesFactory.eINSTANCE.createInputObjectPin();
			iop.setUid(formData.getUid());
			iop.setName(formData.getName());
			if(formData.getOwnedComment() != null && !formData.getOwnedComment().isEmpty()){
				Iterator it1 = formData.getOwnedComment().iterator();
				while(it1.hasNext()){
					Comment formComment = (Comment)it1.next();
					Comment iopComment = ArtifactsFactory.eINSTANCE.createComment();
					iopComment.setBody(formComment.getBody());
					iop.getOwnedComment().add(iopComment);
				}
			}
			//defect90024.1 Use the original data type to make the behavior consistent with the export path
//			iop.setType(formData.getType());
			if(formData.getType() instanceof FormDataType){
				iop.setType(((FormDataType)formData.getType()).getOriginalType());
			//defect 90633, treat any unwrapped type as the original type
			}else{
				iop.setType(formData.getType());
			}
			san.getInputObjectPin().add(iop);
			ips.getInputObjectPin().add(iop);
		}
		return san;
	}
	
	/**
	 * Create a dummy transformation root
	 * @return
	 */
	private static TransformationRoot createTransformationRoot() {
		// Create dummy transformation root
		transformationRoot = HeuristicsFactory.eINSTANCE.createHeuristicsTransformation();
		ExportSession exportSession = new ExportSession();
		exportSession.setExportOption(new ExportOptions());
		// require a dummy Obj2FileNameRegistry for faking the loading of the original schemas
		exportSession.getExportOptions().setAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY,new HashMap());
		transformationRoot.getContext().put(TransformationSessionKeyConstants.EXPORT_SESSION,exportSession);
		if (TransformationEngine.getTransformationSession()==null) {
			TransformationSession session = new TransformationSession();
			TransformationSessionUtil.setExportSession(session,exportSession);
			TransformationEngine aTransformationEngine = new TransformationEngine();
			aTransformationEngine.registerTransformationSession(session);			
		}
		return transformationRoot;
	}

	
	
//	/**
//	 * Generate a WSDL Definition from a BOM Form
//	 * @param form
//	 * @return
//	 */
//	protected Definition generateWSDLDefinitionFromForm (Form form) {
//		Message message = generateWSDLMessageFromForm (form);
//		return message.getEnclosingDefinition();
//	}
	
	/**
	 * Generate a WSDL Message from a BOM Form
	 * @param form
	 * @return
	 */
	protected Message generateWSDLMessageFromForm (Form form) {
		// Create temporary SAN based on form for transformation
		StructuredActivityNode san = createSANFromForm(form);
		Message message = generateWSDLMessageFromSAN(san);
		return message;
	}
	
	/**
	 * Generate a WSDL Message from a BOM Form
	 * @param form
	 * @return
	 */
	public static Message generateWSDLMessageFromSAN (StructuredActivityNode san) {
		// Create temporary SAN based on form for transformation
//		StructuredActivityNode san = createSANFromForm(form);
		InputPinSet ips = (InputPinSet)san.getInputPinSet().get(0);
		transformationRoot = createTransformationRoot();
		ProcessInterfaceRule procIntRule = AbstractbpelFactory.eINSTANCE.createProcessInterfaceRule();
		procIntRule.getSource().add(san);
		transformationRoot.getChildRules().add(procIntRule);
		// Register ProcessInterfaceRule in the context to ensure the transformation rule works properly
		ProcessUtil.putProcessInterfaceRule(transformationRoot.getContext(),procIntRule);
		ProcessUtil.putProcessInterfaceRule(transformationRoot.getContext(),ips,procIntRule);
		procIntRule.transformSource2Target();
		Definition wsdlDefinition = (Definition)procIntRule.getTarget().get(0);
		// Create message
		MessageRule messageRule = WsdlFactory.eINSTANCE.createMessageRule();
		messageRule.getSource().add(ips);
		//wsdlDefRule.getChildRules().add(messageRule);
		procIntRule.getChildRules().add(messageRule);
		messageRule.transformSource2Target();
		//defect 91833
		//Message wsdlMessage = (Message)messageRule.getTarget().get(0);
		Message wsdlMessage = null;
		if(messageRule != null){
			if(messageRule.getTarget() != null && !messageRule.getTarget().isEmpty()) wsdlMessage = (Message)messageRule.getTarget().get(0);
			URI fileURI = URI.createFileURI(IlmPlugin.getDefault().getStateLocation().append("Temp.wsdl").toFile().getAbsolutePath());
			ResourceSet rs = new ResourceSetImpl();
			org.eclipse.emf.ecore.resource.Resource resource = rs.createResource(fileURI);
			resource.getContents().add(((Message)messageRule.getTarget().get(0)).getEnclosingDefinition());
		}
		wsdlDefinition.addMessage(wsdlMessage);
		//wsdlDefinition.getEMessages().add(messageRule.getTarget().get(0));
		//return (Message)messageRule.getTarget().get(0);
		return wsdlMessage;
	}
	
	/**
	 * Generate a xfdl (stream in String) from a BOM form
	 * @param form
	 * @return
	 * @throws BTRuntimeException
	 */
	public String generateXFDL(Form form) throws BTRuntimeException {
		return generateXFDL(form,form.getName());
	}

	/**
	 * Generate a xfdl (stream in String) from a BOM form
	 * @param form
	 * @param formTitle
	 * @return
	 * @throws BTRuntimeException
	 */
	public String generateXFDL(Form form, String formTitle) throws BTRuntimeException {
		Message wsdlMessage = generateWSDLMessageFromForm(form);
		return generateXFDL(wsdlMessage,formTitle);
	}
	
	/**
	 * Generate a xfdl (stream in String) from a wsdl message
	 * @param wsdlMessage
	 * @param formTitle
	 * @return
	 * @throws BTRuntimeException
	 */
	public static String generateXFDL(Message wsdlMessage, String formTitle) throws BTRuntimeException {
		LotusFormsGenerator formGenerator = new LotusFormsGenerator();
		//generate the map that keeps the path/displayName 
		//feature 190156, form enhancement
		final Map<String, String> friendlyMap = generateFriendlyMap(wsdlMessage);
//		printMap(friendlyMap);
		try {
			return formGenerator.generate(wsdlMessage, formTitle, friendlyMap);
		} catch (FormsGenerationException ex) {
			LoggingUtil.logError(MessageKeys.FAILED_TO_GENERATE_XFDL_FORM, null, null);
			BTRuntimeException btEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.FAILED_TO_GENERATE_XFDL_FORM));
			btEx.setChainedException(ex);
			throw btEx;
		}

	}
	
	/**
	 * Prints the content of the map
	 * @param friendlyMap
	 */
	private static void printMap(Map<String, String> friendlyMap) {
		if(friendlyMap == null) return;
		Set<String> keys = friendlyMap.keySet();
		for(String key: keys){
			System.out.println("{" + key + "," + friendlyMap.get(key) + "}");
		}
	}

	
	/**
	 * Constructs the map for the given wsdlMessage. Will iterate through the content of the wsdl message and build a Map : 
	 * key (XPath path to the element in the WSDL) = > value displayName as registered in the TransformationContext
	 * Note : this method should be called for WsdlMessaged generated from Modeler, for HumanTask and Process, so it is assumed it is always a DocLitWrappedStyle 
	 * Note: once the definition of the Type cannot be found in the Context, this is a sign we reached an imported element for which we don't have displayName anymore
	 * in this case we stop walking the Message
	 * @param wsdlMessage
	 * @return
	 */
	private static Map<String, String> generateFriendlyMap(Message wsdlMessage) {
		markedForRecursion = new HashMap<XSDElementDeclaration, XSDTypeDefinition>();
		elementsForDefinition = new HashMap<XSDTypeDefinition, List<XSDElementDeclaration>>();
		visitedForRecursive = new HashMap<XSDElementDeclaration, List<XSDTypeDefinition>>();
		Map<String, String> friendlyMap = new HashMap<String, String>();
		if(transformationRoot == null){
			return null;
		}else{
			TransformationContext context = transformationRoot.getContext();
			//first unwrap the wrapped element declaration for the only Part
			List<XSDElementDeclaration> elementDeclarations = getUnwrappedElementDeclarations((Part)wsdlMessage.getEParts().get(0));
			for(XSDElementDeclaration declaration:  elementDeclarations){
				StringBuffer currentPath = new StringBuffer("/");
				currentPath.append(declaration.getName());
				String displayName = ProcessUtil.getDisplayNameForXSDElement(context, declaration);
				if(displayName != null){
					friendlyMap.put(currentPath.toString(), displayName);
				}
				//now start navigating the wsdl message... 
				XSDTypeDefinition xsdTypeDefinition = declaration.getTypeDefinition();
				if(xsdTypeDefinition != null && ProcessUtil.getDisplayNameForXSDElement(context, xsdTypeDefinition) != null){
					//now navigate through each of the elements of this type Definition..
					List<XSDElementDeclaration> typeDeclarations = getElementDeclarationsForType(xsdTypeDefinition, new LinkedList());
					for(XSDElementDeclaration innerDeclaration:  typeDeclarations){
						currentPath = recordAndAppendDisplayName(context, friendlyMap, currentPath, innerDeclaration, true);
					}
				}
			}
		}
		return friendlyMap;
	}

	/**
	 * Records the elementDelcaration with the displayName in the Map. Then walk again the type definition.
	 * @param context TransformationContext
	 * @param friendlyMap the "path" to "displayName" Map
	 * @param currentPath = the current parent path
	 * @param innerDeclaration an XSDElementDeclaration to be processed
	 * @return
	 */
	private static StringBuffer recordAndAppendDisplayName(TransformationContext context, Map friendlyMap, StringBuffer currentPath, XSDElementDeclaration innerDeclaration, boolean recurse) {
		String displayName = ProcessUtil.getDisplayNameForXSDElement(context, innerDeclaration);
		StringBuffer thisCurrentPath = new StringBuffer(currentPath);
		if(displayName != null){
			thisCurrentPath.append("/");
			thisCurrentPath.append(innerDeclaration.getName());
			friendlyMap.put(thisCurrentPath.toString(), displayName);
		}
		if(recurse){
			boolean internalRecurse = false;
			XSDTypeDefinition xsdTypeDefinition = innerDeclaration.getTypeDefinition();
			if(xsdTypeDefinition != null && ProcessUtil.getDisplayNameForXSDElement(context, xsdTypeDefinition) != null){
				List<XSDElementDeclaration> typeDeclarations = null;
				if(markedForRecursion.get(innerDeclaration) != null){
					typeDeclarations = elementsForDefinition.get(xsdTypeDefinition);
					internalRecurse = false;
				}else{
					List<XSDTypeDefinition> visited = visitedForRecursive.get(innerDeclaration);
					typeDeclarations = getElementDeclarationsForType(xsdTypeDefinition, visited);
					internalRecurse = true;
				}
				for(XSDElementDeclaration thisInner:  typeDeclarations){
					thisCurrentPath = recordAndAppendDisplayName(context, friendlyMap, thisCurrentPath, thisInner, internalRecurse);
				}

			}
		}
		return currentPath;
	}

	/**
	 * Unwraps the elementDeclarations in this xsdTypeDefinition. 
	 * Note: if we are here, means that this typeDefinition was recorded, so it is a Modeler generated typeDefinition, we can be certain about its content.
	 * @param xsdTypeDefinition
	 * @return
	 */
	private static List<XSDElementDeclaration> getElementDeclarationsForType(XSDTypeDefinition xsdTypeDefinition, List visitedDefinitions) {

		List<XSDElementDeclaration> elementDeclarations = new LinkedList<XSDElementDeclaration>();	

		if(((XSDComplexTypeDefinition)xsdTypeDefinition).getBaseTypeDefinition() != null && !org.eclipse.xsd.util.XSDUtil.isAnyType(((XSDComplexTypeDefinition)xsdTypeDefinition).getBaseTypeDefinition())){
			visitedDefinitions.add(xsdTypeDefinition);
			elementDeclarations.addAll(getElementDeclarationsForType(((XSDComplexTypeDefinition)xsdTypeDefinition).getBaseTypeDefinition(), visitedDefinitions));
		}
		if(XSDUtil.isSchemaType(xsdTypeDefinition.getName())){
			return null;
		}
		if(org.eclipse.xsd.util.XSDUtil.isAnyType(xsdTypeDefinition)){
			return elementDeclarations;
		}
		//if this is a simpleTypeDefinition, will map to a Modeler Primitive type so return.
		if(!(xsdTypeDefinition instanceof XSDComplexTypeDefinition))return null;
		XSDParticle particle = (XSDParticle)((XSDComplexTypeDefinition)xsdTypeDefinition).getContent();
		if(particle != null){
			XSDModelGroup modelGroup = (XSDModelGroup)particle.getContent();
			if(modelGroup != null){
				List<XSDParticle> particles = modelGroup.getContents();
				//If the pin represents a single element, we don't create query string
				if(particles != null){
					for(XSDParticle thisParticle: particles){
						XSDElementDeclaration  thisElementDeclaration = (XSDElementDeclaration) thisParticle.getContent();
						XSDTypeDefinition elementDefinition = thisElementDeclaration.getTypeDefinition();
						if(visitedDefinitions.contains(elementDefinition)){
							markedForRecursion.put(thisElementDeclaration, elementDefinition);
						}
						elementDeclarations.add(thisElementDeclaration);
						List visited = new LinkedList();
						visited.add(xsdTypeDefinition);
						visited.addAll(visitedDefinitions);
						visitedForRecursive.put(thisElementDeclaration, visited);
					}
				}
			}
		}
		elementsForDefinition.put(xsdTypeDefinition, elementDeclarations);
		return elementDeclarations;
	}


	/**
	 * Check if the action is a temporarily created SAN for a form
	 * @param action
	 * @return
	 */
	public static boolean isTemporarySANForForm(Action action) {
		if (action instanceof StructuredActivityNode && action.getAspect().equals(ASPECT_FOR_TEMP_FORM_SAN)) {
			return true;
		}
		return false;
	}
	
	
	/**
	 * Generate a WSDL Message from a BOM Human Task.
	 * 
	 * @param ht  A Human Task
	 * @param forInputs  Controls if the message is generated for human task inputs or outputs.
	 * @return
	 */
	protected Message generateWSDLMessageFromHT(HumanTask ht, boolean forInputs) {
		StructuredActivityNode san = (StructuredActivityNode)ht;
		PinSet pinSet = (forInputs ? (InputPinSet)san.getInputPinSet().get(0) : (OutputPinSet)san.getOutputPinSet().get(0));
		transformationRoot = createTransformationRoot();
		ProcessInterfaceRule procIntRule = AbstractbpelFactory.eINSTANCE.createProcessInterfaceRule();
		procIntRule.getSource().add(san);
		transformationRoot.getChildRules().add(procIntRule);
		// Register ProcessInterfaceRule in the context to ensure the transformation rule works properly
		ProcessUtil.putProcessInterfaceRule(transformationRoot.getContext(),procIntRule);
		ProcessUtil.putProcessInterfaceRule(transformationRoot.getContext(), pinSet, procIntRule);
		procIntRule.transformSource2Target();
		Definition wsdlDefinition = (Definition)procIntRule.getTarget().get(0);
		// Create message
		MessageRule messageRule = WsdlFactory.eINSTANCE.createMessageRule();
		messageRule.getSource().add(pinSet);
		//wsdlDefRule.getChildRules().add(messageRule);
		procIntRule.getChildRules().add(messageRule);
		messageRule.transformSource2Target();
		//wsdlDefinition.getEMessages().add(messageRule.getTarget().get(0));
		
		return (Message)messageRule.getTarget().get(0);		
	}
	
	

	/**
	 * * Returns an updated xfdl String.
	 * 
	 * @param humanTask
	 * @param forInputs
	 * @param formsTitle
	 * @param oldXFDLStr
	 * @param instancesToRemove
	 * @return
	 */
	public String updateXFDL(
			HumanTask humanTask, 
			Boolean forInputs, 
			String formsTitle, 
			String oldXFDLStr, 
			Set<String> instancesToRemove) {
		
		String newXfdlStr = null;
		Message message = generateWSDLMessageFromHT(humanTask, forInputs.booleanValue());
		if (message != null) {
			LotusFormsGenerator generator = new LotusFormsGenerator();
			newXfdlStr = generator.update(message, formsTitle, oldXFDLStr, instancesToRemove);
		}		
		return newXfdlStr;
	}	

	
	/**
	 * Returns an updated xfdl String.
	 * 
	 * @param form
	 * @param oldXFDLStr
	 * @param instancesToRemove
	 * @return
	 */
	public String updateXFDL(
			Form form, 
			String oldXFDLStr, 
			Set<String> instancesToRemove) {
		
		String newXfdlStr = null;
		Message message = generateWSDLMessageFromForm(form);
		if (message != null) {
			LotusFormsGenerator generator = new LotusFormsGenerator();
			newXfdlStr = generator.update(message, form.getName(), oldXFDLStr, instancesToRemove);
		}		
		return newXfdlStr;
	}

	/**
	 * Returns the transformationRoot. Needed to get the TransformationContext where the Map is registered
	 * @return
	 */
	public static TransformationRoot getTransformationRoot() {
		return transformationRoot;
	}

	/**
	 * Sets the transformationRoot. This is for the case we generate the Form from the te.ilm transformation.
	 * @return
	 */
	public static void setTransformationRoot(TransformationRoot transformationRoot) {
		FormGenerator.transformationRoot = transformationRoot;
	}	
	
	public static List<XSDElementDeclaration> getUnwrappedElementDeclarations(Part part) {
		XSDModelGroup modelGroup = getUnwrappedElementModelGroup(part);
		if (modelGroup == null) {
			return Collections.emptyList();
		}
		List<XSDElementDeclaration> unwrappedElementsList = new ArrayList<XSDElementDeclaration>();
		List<XSDParticle> particleList = modelGroup.getContents();
		for (XSDParticle particle : particleList) {
			Object unwrappedElementDeclaration = particle.getContent();
			if (unwrappedElementDeclaration instanceof XSDElementDeclaration)
				unwrappedElementsList
						.add((XSDElementDeclaration) unwrappedElementDeclaration);
		}
		return unwrappedElementsList;
	}
	
	/**
	 * handy method to unwrap elements in DocLitWrap wsdl. Returns null if it is
	 * not a wrapper element.
	 * 
	 * @param part
	 * @return xsdmodelgroup of complex element that part refers to
	 */
	private static XSDModelGroup getUnwrappedElementModelGroup(Part part) {
		XSDElementDeclaration wrapperElement = part.getElementDeclaration();
		if(wrapperElement==null || wrapperElement.eIsProxy() || wrapperElement.eContainer()==null){
			return null;
		}
		XSDTypeDefinition wrapperType = wrapperElement.getAnonymousTypeDefinition();
		if(wrapperType == null){
			return null;
		}
		XSDParticle xsdParticle = wrapperType.getComplexType();
		if (xsdParticle == null){
			xsdParticle = (XSDParticle)((XSDComplexTypeDefinition)wrapperType).getContent();
			if(xsdParticle == null)
				return null;
		}
		XSDParticleContent xsdParticleContent = xsdParticle.getContent();
		if (xsdParticleContent instanceof XSDModelGroup) {
			return (XSDModelGroup) xsdParticleContent;
		}
		return null;
	}
	
	
}
