/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtserviceFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;
import com.ibm.wbit.bpel.services.partnerlinktype.impl.PartnerLinkTypeImpl;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * Parent rule of rules that involves BPEL Partner creation, e.g. CallBehaviorAction,
 * StructureActivityNode (Task/Process). It provides a set of methods for
 * creating Partner, PartnerLink, and WSDL Port type
 */
public abstract class ActionRuleImpl extends ConnectableRuleImpl implements ActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ActionRuleImpl() {
		super();
	}

	Definition wsdlDefinition = null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.ACTION_RULE;
	}

/*
	protected void setupChildRules(Action action)
	{
		ProcessInterfaceRule processIntRule = createProcessInterfaceRule(action);
		PortTypeRule portTypeRule = createPortTypeRule(processIntRule,action);
		PartnerLinkRule partnerLinkRule = 
			createPartnerLinkRule(processIntRule,action,(PortType)portTypeRule.getTarget().get(0));
		PartnerRule pr = createPartnerRule(processIntRule,action);
	}
*/	
	/**
	* Creates the interface rules. Sets the source to be the BOM Action.
	*/
	protected TransformationRule createPortTypeRule(ProcessInterfaceRule processIntRule, Action action){
		//Creates an interfaceRule and set its source to be the ActionRule source
		//(an action) and the target of the ProcessInterfaceRule
		Activity interfaceActivity = null;
		if(FabricIntegrationUtil.getInterfaceActivityIfAny((StructuredActivityNode)action) != null){
			interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny((StructuredActivityNode)action);
		}
		if(interfaceActivity != null){
			action = interfaceActivity.getImplementation();
		}
		TransformationRule portTypeRule = null;
		if(BomUtils.isServiceOperation(action)){
			portTypeRule = 
						TransformationUtil.getRuleForSource(processIntRule,
						com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule.class, 
						action, 
						PortType.class);
			if(portTypeRule == null){
				portTypeRule = ExtserviceFactory.eINSTANCE.createPortTypeRule();
			}
		}else{
		  portTypeRule = 
		  			TransformationUtil.getRuleForSource(processIntRule,PortTypeRule.class, action, PortType.class);
		 if(portTypeRule == null){
		 	portTypeRule = WsdlFactory.eINSTANCE.createPortTypeRule();
		 }
		}
//		//if(portTypeRule == null  || !portTypeRule.isComplete()){
//		if (portTypeRule == null) {
//			portTypeRule = WsdlFactory.eINSTANCE.createPortTypeRule();
			portTypeRule.getSource().add(action);
			processIntRule.getChildRules().add(portTypeRule);
			processIntRule.transformSource2Target();
//		}
		//just to make sure the portType is completed.. 
		if(!portTypeRule.isComplete()){
			portTypeRule.transformSource2Target();
		}		
		return portTypeRule;
	}
	
	/**
	* Creates the interface rules. Sets the source to be the BOM NamedElement.
	* TODO changed to NamedElement to accomodate BehavioredClass and Interface
	*/
	protected PortTypeRule createPortTypeRule(ProcessInterfaceRule processIntRule, NamedElement namedElement){
		//Creates an interfaceRule and set its source to be the ActionRule source
		//(an action) and the target of the ProcessInterfaceRule
		PortTypeRule portTypeRule = 
			(PortTypeRule)TransformationUtil.getRuleForSource(processIntRule,PortTypeRule.class, namedElement, PortType.class);
		//if(portTypeRule == null  || !portTypeRule.isComplete()){
		if (portTypeRule == null) {
			portTypeRule = WsdlFactory.eINSTANCE.createPortTypeRule();
			portTypeRule.getSource().add(namedElement);
			processIntRule.getChildRules().add(portTypeRule);
			processIntRule.transformSource2Target();
		}
		
		return portTypeRule;

	}
	
	protected PartnerLinkRule createPartnerLinkRule(ProcessInterfaceRule processIntRule, Action action, PortType portType, TransformationRule enclosingRule)
	{	
		PartnerLinkRule partnerLinkRule  = null;
		CallBehaviorAction cba = FabricIntegrationUtil.getCBAForRule(enclosingRule);
		boolean overriden = FabricIntegrationUtil.isOverriden(cba);
		//defect 75458
//		if(!overriden && cba != null){
//			//Lookup the partnerLink rule based on the portType 
//			partnerLinkRule = (PartnerLinkRule)TransformationUtil.getRuleForSource(processIntRule, PartnerLinkRule.class, portType, PartnerLinkType.class);
//		}else{
			partnerLinkRule = (PartnerLinkRule)	TransformationUtil.getRuleForSource(processIntRule, PartnerLinkRule.class, action, PartnerLinkType.class);
			//create partner link for each top level process variation in case the are referenced as a cba in another process and has been transformed 
			if(action instanceof StructuredActivityNode && BomUtils.isProcess(action)&& 
					FabricIntegrationUtil.getInterfaceActivityIfAny((StructuredActivityNode)action) != null){
				if(partnerLinkRule != null){
					partnerLinkRule = null;
				}
			}
//		}	
		if (partnerLinkRule == null || overriden){
			partnerLinkRule = WsdlFactory.eINSTANCE.createPartnerLinkRule();
			partnerLinkRule.getSource().add(action);
			partnerLinkRule.getSource().add(portType);				
			if(cba != null && overriden && BomUtils.isServiceOperation(action)){
				partnerLinkRule.getSource().add(cba);
			}
			processIntRule.getChildRules().add(partnerLinkRule);
			processIntRule.transformSource2Target();
		}
		return partnerLinkRule;
	}
	
	protected PartnerLinkRule createPartnerLinkRule_Overriden(ProcessInterfaceRule processIntRule, Action action, PortType portType, TransformationRule enclosingRule,Collection sources)
	{	
		PartnerLinkRule partnerLinkRule  = null;
		CallBehaviorAction cba = FabricIntegrationUtil.getCBAForRule(enclosingRule);
		boolean overriden = FabricIntegrationUtil.isOverriden(cba);
		boolean isCBAImplOverridenSuported = FabricIntegrationUtil.isSupportedCBA(action);
		//defect 75458
		if(!overriden){
			//Lookup the partnerLink rule based on the portType 
			if(!isCBAImplOverridenSuported){
				partnerLinkRule = (PartnerLinkRule)TransformationUtil.getRuleForSource(processIntRule, PartnerLinkRule.class, portType, PartnerLinkType.class);
				//create partner link for each process cba in case the process has already been transformed 
				if(partnerLinkRule != null && isProcessCBA(cba)){					
					partnerLinkRule = FabricIntegrationUtil.getPartnerLinkRuleForAction(getContext(),processIntRule.getParentRule(),action);
				}
			}else{
				partnerLinkRule = FabricIntegrationUtil.getPartnerLinkRuleForAction(getContext(),processIntRule.getParentRule(),action);
			}
		}else{
			partnerLinkRule = (PartnerLinkRule)	TransformationUtil.getRuleForSource(processIntRule, PartnerLinkRule.class, sources, PartnerLinkType.class);
		}	
		if (partnerLinkRule == null || overriden){
			partnerLinkRule = WsdlFactory.eINSTANCE.createPartnerLinkRule();
			partnerLinkRule.getSource().add(action);
			partnerLinkRule.getSource().add(portType);				
			if(cba != null && overriden && BomUtils.isServiceOperation(action)){
				partnerLinkRule.getSource().add(cba);
			}
			processIntRule.getChildRules().add(partnerLinkRule);
			if(!overriden && isCBAImplOverridenSuported){
				FabricIntegrationUtil.registerPartnerLinkRuleForAction(getContext(),processIntRule.getParentRule(),partnerLinkRule,action);
			}else if(isProcessCBA(cba)){
				FabricIntegrationUtil.registerPartnerLinkRuleForAction(getContext(),processIntRule.getParentRule(),partnerLinkRule,action);
			}
			processIntRule.transformSource2Target();			
		}
		return partnerLinkRule;
	}

	/**
	 * Register the self invoked process cba partner as the process' partner since the partner will be reused when transform the enclosing process'
	 * partner link. We need to optimize the partner link for the process since myRole may not be set for this case
	 * @param sources
	 * @param partnerRule
	 * @param action
	 */
	private void registerProcessPatnerIfSelfInvoked(List sources,PartnerRule partnerRule, Action action) {
		if(sources.size() == 2 && sources.get(0) instanceof Action && sources.get(1) instanceof CallBehaviorAction){
			CallBehaviorAction cba = (CallBehaviorAction)sources.get(1);
			boolean isSelfInvoke = false;
			if(cba.getBehavior() instanceof Activity){
				Activity activity = (Activity)cba.getBehavior();
				if(activity.getImplementation()== action){
					if(cba.eContainer() instanceof StructuredActivityNode && cba.eContainer() == action){
						isSelfInvoke = true;
					}
				}
			}
			if(isSelfInvoke && BomUtils.isProcess(action) && action instanceof StructuredActivityNode && !partnerRule.getTarget().isEmpty()){
				ProcessUtil.putProcessPartner(getContext(),(StructuredActivityNode)action,(PartnerLink)partnerRule.getTarget().get(0));
			}
		}
	}

	private boolean isProcessCBA(CallBehaviorAction cba) {
		return cba.getBehavior() instanceof Activity && BomUtils.isProcess(((Activity)cba.getBehavior()).getImplementation())&& 
				FabricIntegrationUtil.getInterfaceActivityIfAny(((Activity)cba.getBehavior()).getImplementation()) != null;
	}

	protected PartnerLinkRule createPartnerLinkRule(ProcessInterfaceRule processIntRule, Action action, PortType myRolePortType, PortType parnterRolePortType)
	{	
		PartnerLinkRule partnerLinkRule = (PartnerLinkRule)
			TransformationUtil.getRuleForSource(
				processIntRule, PartnerLinkRule.class, action, PartnerLinkType.class);
		if (partnerLinkRule == null)
		{
			partnerLinkRule = WsdlFactory.eINSTANCE.createPartnerLinkRule();
			partnerLinkRule.getSource().add(action);
			partnerLinkRule.getSource().add(myRolePortType);
			partnerLinkRule.getSource().add(parnterRolePortType);
			processIntRule.getChildRules().add(partnerLinkRule);
			processIntRule.transformSource2Target();
		}
		return partnerLinkRule;
	}


	protected PartnerLinkRule createPartnerLinkRule(ProcessInterfaceRule processIntRule, com.ibm.btools.bom.model.services.Operation operation, PortType portType)
		{	
			PartnerLinkRule partnerLinkRule = (PartnerLinkRule)
				TransformationUtil.getRuleForSource(
					processIntRule, PartnerLinkRule.class, operation, PartnerLinkType.class);
			if (partnerLinkRule == null)
			{
				partnerLinkRule = WsdlFactory.eINSTANCE.createPartnerLinkRule();
				partnerLinkRule.getSource().add(operation);
				partnerLinkRule.getSource().add(portType);
				processIntRule.getChildRules().add(partnerLinkRule);
				processIntRule.transformSource2Target();
			}
			
			return partnerLinkRule;
		}

	protected PartnerRule createPartnerRule(ProcessDefinitionRule processDefRule, Action action, ProcessInterfaceRule processIntRule)
	{	
		PartnerRule partnerRule = (PartnerRule)
			TransformationUtil.getRuleForSource(
				processDefRule, PartnerRule.class, action, PartnerLink.class);
		if (partnerRule == null)
		{
			partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().add(action);
			partnerRule.getSource().add(processIntRule);
			getChildRules().add(partnerRule);
			partnerRule.transformSource2Target();
		}
		return partnerRule;
	}


	protected PartnerRule createPartnerRule(ProcessDefinitionRule processDefRule, Action action)
	{	
		PartnerRule partnerRule = (PartnerRule)
			TransformationUtil.getRuleForSource(
				processDefRule, PartnerRule.class, action, PartnerLink.class);
		if (partnerRule == null)
		{
			partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().add(action);
			getChildRules().add(partnerRule);
			partnerRule.transformSource2Target();
		}
		return partnerRule;
	}

	protected PartnerRule createPartnerRule(ProcessDefinitionRule processDefRule, List sources, PartnerLinkRule partnerLinkRule, DocumentRoot component)
	{	
		PartnerRule partnerRule = null;
		boolean overriden = false;
		boolean isCBAImplOverridenSuported = false;
		Action action = null;
		if(sources.size() == 2 && sources.get(0) instanceof Action && sources.get(1) instanceof CallBehaviorAction){
			action = (Action)sources.get(0);
			CallBehaviorAction cba = (CallBehaviorAction)sources.get(1);
			overriden = FabricIntegrationUtil.isOverriden(cba);
			isCBAImplOverridenSuported = FabricIntegrationUtil.isSupportedCBA(action);
		}
		
		//	//defect 75458
		//Lookup the partner rule based on the portType 
		if(sources.size() == 2 && sources.get(0) instanceof Action && sources.get(1) instanceof CallBehaviorAction && !overriden){
			if(!isCBAImplOverridenSuported){
				partnerRule = (PartnerRule)	TransformationUtil.getRuleForSource(processDefRule, PartnerRule.class, (EObject)sources.get(0), PartnerLink.class);		
			}else{
				partnerRule = FabricIntegrationUtil.getPartnerRuleForAction(getContext(),processDefRule.getParentRule(),action);
			}
		}else{
			partnerRule = (PartnerRule)	TransformationUtil.getRuleForSource(processDefRule, PartnerRule.class, sources, PartnerLink.class);
		}
		if (partnerRule == null)
		{
			partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().addAll(sources);
			if(partnerLinkRule != null){
				partnerRule.getSource().add(partnerLinkRule);
			}
			if(component != null){
				partnerRule.getSource().add(component);
			}
			getChildRules().add(partnerRule);
			if(isCBAImplOverridenSuported && !overriden){
				FabricIntegrationUtil.registerPartnerRuleForAction(getContext(),processDefRule.getParentRule(),partnerRule,action);
			}
			partnerRule.transformSource2Target();
			//Register the partner with the process, we have to make sure the CBA is a self invoke of the process
			registerProcessPatnerIfSelfInvoked(sources, partnerRule, action);
		}
		return partnerRule;
	}
	
	protected PartnerRule createPartnerRule(ProcessDefinitionRule processDefRule, com.ibm.btools.bom.model.services.Operation operation)
	{	
		PartnerRule partnerRule = (PartnerRule)
			TransformationUtil.getRuleForSource(
				processDefRule, PartnerRule.class, operation, PartnerLink.class);
		if (partnerRule == null)
		{
			partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().add(operation);
			getChildRules().add(partnerRule);
			partnerRule.transformSource2Target();
		}
		return partnerRule;
	}

	protected Operation findOneWayOperation(ProcessInterfaceRule pfr, PinSet set){
		Vector sources = new Vector();
		// TODO: Assume one to one input/output set correlation
		// Should report if there are more than one outpset is found
		sources.add(set);
		//OperationRule opRule = (OperationRule)TransformationUtil.getRuleForSource(pfr,sources,Operation.class);
		//defect 85137, PMR 68264
		TransformationRule opRule = (TransformationRule)TransformationUtil.getRuleForSource(pfr,sources,Operation.class);
		
		// Assume the first target of the Operation Rule is always the operation		
		return  (Operation)opRule.getTarget().get(0);
	}

	
	protected Operation findOperation(ProcessInterfaceRule pfr, NamedElement set){
		Vector sources = new Vector();
		// TODO: Assume one to one input/output set correlation
		// Should report if there are more than one outpset is found
		sources.add(set);
		if(set instanceof InputPinSet){
			InputPinSet processIPS = (InputPinSet)set;
			InputPinSet interfaceIPS = null;
			if(processIPS.eContainer() instanceof StructuredActivityNode){
				StructuredActivityNode san = (StructuredActivityNode)processIPS.eContainer();
				if(BomUtils.isProcess(san) && FabricIntegrationUtil.getInterfaceActivityIfAny(san) != null){
					interfaceIPS = FabricIntegrationUtil.getInterfaceActivityInputPinSetForProcessIPS(processIPS);
				}
			}
			if(interfaceIPS != null){
				sources.remove(set);
				sources.add(interfaceIPS);
				sources.add(interfaceIPS.getOutputPinSet().get(0));
				
			}else{
				sources.add(((InputPinSet)set).getOutputPinSet().get(0));
			}
		}else if(set instanceof InputParameterSet){
			sources.add(((InputParameterSet)set).getOutputParameterSet().get(0));
		}
		
		//sources.add(opInputPinSet);
		//sources.add(opInputPinSet.getOutputPinSet().get(0));
	//	OperationRule opRule = (OperationRule)
		TransformationRule opRule = 
			TransformationUtil.getRuleForSource(
			pfr,sources,Operation.class);
		// Assume the first target of the Operation Rule is always the operation
		return  (Operation)opRule.getTarget().get(0);	
	}
	
//	protected Operation findOperation(ProcessInterfaceRule pfr,InputPinSet opInputPinSet)
//	{
//		Vector sources = new Vector();
//		// TODO: Assume one to one input/output set correlation
//		// Should report if there are more than one outpset is found
//		sources.add(opInputPinSet);
//		sources.add(opInputPinSet.getOutputPinSet().get(0));
//		OperationRule opRule = (OperationRule)
//			TransformationUtil.getRuleForSource(
//			pfr,sources,Operation.class);
//		// Assume the first target of the Operation Rule is always the operation
//		return  (Operation)opRule.getTarget().get(0);
//		
//	}
//	protected Operation findOperation(ProcessInterfaceRule pfr, InputParameterSet opInputPinSet)
//		{
//			Vector sources = new Vector();
//			// TODO: Assume one to one input/output set correlation
//			// Should report if there are more than one outpset is found
//			sources.add(opInputPinSet);
//			sources.add(opInputPinSet.getOutputParameterSet().get(0));
//			OperationRule opRule = (OperationRule)
//				TransformationUtil.getRuleForSource(
//				pfr,sources,Operation.class);
//			// Assume the first target of the Operation Rule is always the operation
//			return  (Operation)opRule.getTarget().get(0);
//		
//		}
} //ActionRuleImpl
