/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Query;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.To;

/**
 * Creates a BPEL Assign (incl Copy/From/To) and create two links that links
 * between the BPEL construct representing the output set, the BPEL Assign
 * and the BPEL construct representing the input set 
 * Source: BOM edge, Outputset (source set), Inputset (target set)
 * Target:
 * Target Propagated: BPEL constructs - Variables, Partners, Sequence, Recieve, Reply 
 */
public class AssignRuleImpl extends ConnectableRuleImpl implements AssignRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected AssignRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.ASSIGN_RULE;
	}

	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target","already completed");
			return true;
		}
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target","no source");
			return false;
		}
			
		// Initialization
		ActivityEdge edge = (ActivityEdge)getSource().get(0);
		NamedElement srcSet = (NamedElement)getSource().get(1);
		NamedElement tarSet = (NamedElement)getSource().get(2);	
		TransformationContext context = getContext();
			
		// Create containers for the source and target sets
		BPELVariable srcCon = createContainer(srcSet);
		BPELVariable tarCon = createContainer(tarSet);
		
		// Create a link from the source to the assign
		Pin srcPin = (Pin)edge.getSource();
		Link srcToAssignLink = createLink(edge,srcPin,srcSet);
		// Create a link from the assign to the target
		Pin tarPin = (Pin)edge.getTarget();
		Link assignToTarLink = createLink(edge,tarPin,tarSet);
		
		// Creates BPEL Assign and set the links
		Assign assign = createAssign(srcSet, srcCon, tarSet, tarCon);
		
		Source assignSrc = BPELFactory.eINSTANCE.createSource();
		assignSrc.setActivity(assign);
		Target assignTar = BPELFactory.eINSTANCE.createTarget();
		assignTar.setActivity(assign);
		srcToAssignLink.getTargets().add(assignTar);
		String srcToAssignLinkName="Assign";
		if (srcToAssignLink.getName() != null){		
			srcToAssignLinkName = srcToAssignLink.getName()+srcToAssignLinkName;
			//Check to see if it's a valid name by calling NameConverter
			JavaNCNameConverter converter = new JavaNCNameConverter();
			srcToAssignLinkName = converter.convertName(NamingUtil.findRegistry(this), srcToAssignLink, srcToAssignLink.getName()+srcToAssignLinkName);			
		}
		srcToAssignLink.setName(srcToAssignLinkName);
		assignToTarLink.getSources().add(assignSrc);
		String assignToTarLinkName="Assign_to_";
		if (assignToTarLink.getName() != null)
			assignToTarLinkName = assignToTarLinkName+assignToTarLink.getName();
		//JavaNCNameConverter converter = new JavaNCNameConverter();
		//String convertedName = converter.convertName(NamingUtil.findRegistry(this), assignToTarLink, assignToTarLinkName);
		//assignToTarLink.setName(convertedName);
		assignToTarLink.setName(assignToTarLinkName);
		
		// Propagate links
		getTarget().add(srcToAssignLink);
		getTarget().add(assignToTarLink);
		
		// Propagate assign
		getTarget().add(assign);
		
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}



	private Assign createAssign(NamedElement srcSet, BPELVariable srcCon, NamedElement tarSet, BPELVariable tarCon){
		
		ActivityEdge edge = (ActivityEdge)getSource().get(0);
		// Create a link from the source to the assign
		Pin srcPin = (Pin)edge.getSource();
		Pin tarPin = (Pin)edge.getTarget();
		
		// Creates BPEL Assign and set the links
		Assign assign = BPELFactory.eINSTANCE.createAssign();
	
		// Creates BPEL Copy and set to the Assign
		Part srcPart = findPart(srcSet,srcPin,srcCon);
		Part tarPart = findPart(tarSet,tarPin,tarCon);
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		from.setVariable(srcCon);
		from.setPart(srcPart);
		
//		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
//		
//		//bpelExpr.setExpression(createExpressionModel(inPinPart, k+1));
//		bpelExpr.setExpression(createExpressionModel(srcSet, srcPin, srcPart));
//		from.addExtensibilityElement(bpelExpr);
//		to.setVariable(tarCon);
//		to.setPart(tarPart);
//		bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
//		//bpelExpr.setExpression(createExpressionModel(inPinPart, k+1));
//		bpelExpr.setExpression(createExpressionModel(tarSet, tarPin, tarPart));
//		to.addExtensibilityElement(bpelExpr);
		
		
		String queryFromString = createQueryString(srcSet, srcPin, srcPart);
		if(queryFromString != null && !queryFromString.equals("")){
			Query queryFrom = BPELFactory.eINSTANCE.createQuery();
			//TODO AR migration changes.. verify namespace etc.. the default query language is XPATH 1.0
			// so there is no need to specify the ql
			//queryFrom.setQueryLanguage("XPATH"); .. do not set anything as it will be takes as default XPATH 1.0
			queryFrom.setValue(queryFromString);
			from.setQuery(queryFrom);
		}
		

		to.setVariable(tarCon);
		to.setPart(tarPart);
		String queryToString = createQueryString(tarSet, tarPin, tarPart);
		if(queryToString != null && !queryToString.equals("")){
			Query queryTo = BPELFactory.eINSTANCE.createQuery();
			//queryTo.setQueryLanguage("XPATH");  .. do not set anything as it will be takes as default XPATH 1.0
			queryTo.setValue(queryToString);
			to.setQuery(queryTo);
		}
	
		copy.setFrom(from);
		copy.setTo(to);
		assign.getCopy().add(copy);

		return assign;
		
		
	}


	private Part findPart(NamedElement set,Pin pin, BPELVariable con)
	{
		// Find the message representing the container
		// and then find the part by the index of the 
		// object pin.  This is assuming the parts
		// are in the same order as the object pins in
		// a set
		// Note: the original idea is to use getRuleForSource
		// to find the part based on pin.  But this will fail
		// when a container is reused for an input set because
		// in such as there will not be a part created for the
		// pin


		//AR WS-I The part returned should be either the only Part created for this Variable , in the case we create a
		//WS-I compliant WSDL or the Part corresponding to the Pin in the case of a non WS-I compliant WSDL 
		Part part = null;		
		// Get the WSDL message from container
		Message msg = con.getMessageType();
		//if(set.eContainer() instanceof Action && ProcessUtil.isMultiplePartWSDL((Action)set.eContainer(), getContext())){
		if(set.eContainer() instanceof Action && !isWSICompliant((Action)set.eContainer())){
			// find the pin index in the set
			List pins = getPinsFromSet(set);
			int ind = pins.indexOf(pin);
			// find the corresponding part for the pin
			part = (Part)msg.getOrderedParts(null).get(ind);		
		}else{
			part = (Part)msg.getOrderedParts(null).get(0);;
		}
		return part;
	}
	
	private List getPinsFromSet(NamedElement set)
	{
		List pins;
		if (set instanceof InputPinSet)
			pins = ((InputPinSet)set).getInputObjectPin();
		else
			pins = ((OutputPinSet)set).getOutputObjectPin();
		return pins;		
	}



	private String createQueryString( NamedElement set, Pin pin, Part part){
		if(part.getTypeDefinition() != null){      
			return "";
			//this is the case of a single Pin in the PinSet when we did not generate the wrapper complex type
		}else if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && getPinsFromSet(set) != null && getPinsFromSet(set).size()== 1){
			return "";
		}
		String pinElementDeclaration = AbstractbpelUtil.getPinElementName(set, pin, part);
		
		return  "/" + pinElementDeclaration;

	}


	/** 
	 * Create the expression model representation of the BPEL assignment expression
	 */	
	private ModelPathExpression createExpressionModel(NamedElement set, Pin pin, Part part){
	
		String pinElementDeclaration = AbstractbpelUtil.getPinElementName(set, pin, part);
		
		ModelPathExpression pathExpr = ModelFactory.eINSTANCE.createModelPathExpression();
		StaticStep step1 = ModelFactory.eINSTANCE.createStaticStep();
		step1.setStepName(pinElementDeclaration);
		StaticStep step2 = ModelFactory.eINSTANCE.createStaticStep();		
//		// TODO: Do we have to make the first character a capital letter? Check BPEL expression context definition
//		step2.setStepName("address");
//		StaticStep step3 = ModelFactory.eINSTANCE.createStaticStep();		
//		step3.setStepName("value");
		pathExpr.setIsAbsolute(Boolean.TRUE);
		pathExpr.getSteps().add(step1);
//		pathExpr.getSteps().add(step2);		
//		pathExpr.getSteps().add(step3);					
		return pathExpr;
		

	}


//	/**
//	 * Source: a BOM Pin
//	 * Target: Bpel Activity
// 	*/
//	public boolean transformSource2Target(){
//		
//		//The source of the Rule is a Pin (Input or Output ObjectPin). 
//		//Depending on the type of activities containers of source pins, we will generate different activities;
//		//1. if the sets belong to 2 activities that have similar runtime implementation, shoudl create a regular assign
//		//that does a part to part assign
//		//2. if sets belong to 2 activities that have similar runtime implementation that do not require WSDL with 
//		//XSD Parts, we should create a alternate activity.: for BPEL - create query string, for BPEL+ create Java code
//		//3. if set belong to activitoes that have different implementation.. complex assign 	 
//		
//		LoggingUtil.traceEntry(this, "transformSource2Target");
//		
//		if (isComplete()) {
//			LoggingUtil.traceExit(this, "transformSource2Target","already completed");
//			return true;
//		}
//		if (getSource() == null || getSource().isEmpty()) {
//			LoggingUtil.traceExit(this, "transformSource2Target","no source");
//			return false;
//		}
//		//	Initialization
//		ActivityEdge edge = (ActivityEdge)getSource().get(0);
//		NamedElement srcSet = (NamedElement)getSource().get(1);
//		NamedElement tarSet = (NamedElement)getSource().get(2);	
//		TransformationContext context = getContext();
//			
//		// Create containers for the source and target sets
//		BPELVariable srcCon = createContainer(srcSet);
//		BPELVariable tarCon = createContainer(tarSet);
//		
//		// Create a link from the source to the assign
//		Pin srcPin = (Pin)edge.getSource();
//		Link srcToAssignLink = createLink(edge,srcPin,srcSet);
////		// Create a link from the assign to the target
//		Pin tarPin = (Pin)edge.getTarget();
//		Link assignToTarLink = createLink(edge,tarPin,tarSet);
//		
//  
//  		
//		Activity assignActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();		
//		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//		
//		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
//		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
//			
//		((AlternativeActivity)assignActivity).getAlternative().add(bpelAlt);
//		((AlternativeActivity)assignActivity).getAlternative().add(bpelpAlt);
//			
//		/* BPEL: Create <assign>, with multiple BPEL Copies, if there is at least one object pin */
//		 //bpelAlt.setActivity(createAssign(is, isVariable, os, osVariable));
//		 //TODO AR   check casting !!1
//
//		if(ProcessUtil.doAssignForPartsWithTypes(srcSet, tarSet)){
//			//create assign for parts..
//			//set the bpel and bpelp to same activity....
//		}else{
//			bpelAlt.setActivity(createAssign(srcSet, srcCon, tarSet, tarCon));
//			/* BPEL+: Create <invoke><wpc:script><wpc:javaCode>... */
//			bpelpAlt.setActivity(createJavaInvoke(srcPin, srcSet, srcCon, tarPin, tarSet, tarCon));
//			
//		}
//
//		//now create the links...		
//		Source assignSrc = BPELFactory.eINSTANCE.createSource();
//		assignSrc.setActivity(assignActivity);
//		Target assignTar = BPELFactory.eINSTANCE.createTarget();
//		assignTar.setActivity(assignActivity);
//		srcToAssignLink.getTargets().add(assignTar);
//		String srcToAssignLinkName="Assign";
//		if (srcToAssignLink.getName() != null){		
//			srcToAssignLinkName = srcToAssignLink.getName()+srcToAssignLinkName;
//			//Check to see if it's a valid name by calling NameConverter
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			srcToAssignLinkName = converter.convertName(NamingUtil.findRegistry(this), srcToAssignLink, srcToAssignLink.getName()+srcToAssignLinkName);			
//		}
//		srcToAssignLink.setName(srcToAssignLinkName);
//		assignToTarLink.getSources().add(assignSrc);
//		String assignToTarLinkName="Assign_to_";
//		if (assignToTarLink.getName() != null)
//			assignToTarLinkName = assignToTarLinkName+assignToTarLink.getName();
//		//JavaNCNameConverter converter = new JavaNCNameConverter();
//		//String convertedName = converter.convertName(NamingUtil.findRegistry(this), assignToTarLink, assignToTarLinkName);
//		//assignToTarLink.setName(convertedName);
//		assignToTarLink.setName(assignToTarLinkName);
//		
//		getTarget().add(srcToAssignLink);
//		getTarget().add(assignToTarLink);
//			
//		
//		
//		// Creates BPEL Assign and set the links
////		Assign assign = BPELFactory.eINSTANCE.createAssign();
////		Source assignSrc = BPELFactory.eINSTANCE.createSource();
////		assignSrc.setActivity(assign);
////		Target assignTar = BPELFactory.eINSTANCE.createTarget();
////		assignTar.setActivity(assign);
////		srcToAssignLink.getTargets().add(assignTar);
////		String srcToAssignLinkName="Assign";
////		if (srcToAssignLink.getName() != null){		
////			srcToAssignLinkName = srcToAssignLink.getName()+srcToAssignLinkName;
////			//Check to see if it's a valid name by calling NameConverter
////			JavaNCNameConverter converter = new JavaNCNameConverter();
////			srcToAssignLinkName = converter.convertName(NamingUtil.findRegistry(this), srcToAssignLink, srcToAssignLink.getName()+srcToAssignLinkName);			
////		}
////		srcToAssignLink.setName(srcToAssignLinkName);
////		assignToTarLink.getSources().add(assignSrc);
////		String assignToTarLinkName="Assign_to_";
////		if (assignToTarLink.getName() != null)
////			assignToTarLinkName = assignToTarLinkName+assignToTarLink.getName();
////		//JavaNCNameConverter converter = new JavaNCNameConverter();
////		//String convertedName = converter.convertName(NamingUtil.findRegistry(this), assignToTarLink, assignToTarLinkName);
////		//assignToTarLink.setName(convertedName);
////		assignToTarLink.setName(assignToTarLinkName);
////		
////		// Creates BPEL Copy and set to the Assign
////		Part srcPart = findPart(srcSet,srcPin,srcCon);
////		Part tarPart = findPart(tarSet,tarPin,tarCon);
////		Copy copy = BPELFactory.eINSTANCE.createCopy();
////		From from = BPELFactory.eINSTANCE.createFrom();
////		To to = BPELFactory.eINSTANCE.createTo();
////		from.setVariable(srcCon);
////		from.setPart(srcPart);
////		String queryFrom = createQueryString(srcSet, srcPin, srcPart);
////		from.setQuery(queryFrom);
////	
////		to.setVariable(tarCon);
////		to.setPart(tarPart);
////		String queryTo = createQueryString(tarSet, tarPin, tarPart);
////		to.setQuery(queryTo);
////		
////
////		copy.setFrom(from);
////		copy.setTo(to);
////		assign.getCopy().add(copy);
////
//		// Propagate containers
//		//getTarget().add(srcCon);
//		//getTarget().add(tarCon);
//		
//		// Propagate links
////		getTarget().add(srcToAssignLink);
////		getTarget().add(assignToTarLink);
//		
//		// Propagate assign
//		//getTarget().add(assign);
//		getTarget().add(assignActivity);
//		
//		setComplete(true);
//		LoggingUtil.traceExit(this, "transformSource2Target");
//		return isComplete();		
//	}
//
//	private Part findPart(NamedElement set,Pin pin, BPELVariable con)
//	{
//		// Find the message representing the container
//		// and then find the part by the index of the 
//		// object pin.  This is assuming the parts
//		// are in the same order as the object pins in
//		// a set
//		// Note: the original idea is to use getRuleForSource
//		// to find the part based on pin.  But this will fail
//		// when a container is reused for an input set because
//		// in such as there will not be a part created for the
//		// pin
//
//
////		PartRule partRule = (PartRule)
////					TransformationUtil.getRuleForSource(getRoot(),
////														PartRule.class,
////														pin);
////
////		return (Part)partRule.getTarget().get(0);
//
//		// Get the WSDL message from container
//		Message msg = con.getMessageType();
//		// find the pin index in the set
//		List pins = getPinsFromSet(set);
//		int ind = pins.indexOf(pin);
//		// find the corresponding part for the pin
//	//	Part part = (Part)msg.getOrderedParts(null).get(ind);		
//		//TODO AR we will always use the first part .. there is only one part.. with one exception
//		//WIS implementation
//		Part part = (Part)msg.getOrderedParts(null).get(0);
//		return part;
//		
//		
//		
//	}
	
	

//	
//	
//	
//	private Activity createJavaInvoke(Pin srcPin, NamedElement srcSet, BPELVariable srcCon, Pin tarPin, NamedElement tarSet, BPELVariable tarCon){
//		
//		//if we are dealing with similar types of messages.. ie messages with compatibles -> similar type definitions, do a regular Assign
//		//TODO here check compatibility between ICS and WBIMB
//		if((((Part)srcCon.getMessageType().getEParts().get(0)).getElement().getTypeDefinition()).equals(((Part)tarCon.getMessageType().getEParts().get(0)).getElement().getTypeDefinition())){
//			return createAssign(srcSet, srcCon, tarSet, tarCon);
//		}else{
//		//else we will need to create a Java invoke that will copy all the 
//			Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
//			JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
//			JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
//			String javaString = new String();
//			String tarVarName = AbstractbpelUtil.capitalizeFirstChar(tarCon.getName());
//			String tarPartName = AbstractbpelUtil.capitalizeFirstChar(((Part)tarCon.getMessageType().getEParts().get(0)).getName());
//			String srcVarName = AbstractbpelUtil.capitalizeFirstChar(srcCon.getName());
//			String srcPartName = AbstractbpelUtil.capitalizeFirstChar(((Part)srcCon.getMessageType().getEParts().get(0)).getName());
////					if(!ExportOperation.bInterfaceWISCompliant){
////						//Ray Apr 13,04
////						if(obj==null || obj instanceof LiteralNull || !(obj instanceof LiteralSpecification)){
////							javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true)"
////													+ ".set" + partName + "(null);\n";
////						}else{
////							javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true)"
////										+ ".set" + partName + "(" + AbstractbpelUtil.getLiteralAssignmentString(value, destPart) + ");\n";
////						}
////					}else{
//						//TODO AR 26.07
//					//	InputPinSet is = (InputPinSet)BomUtils.getInputPinSets(sourcePin).get(0);
//			String tarPinElementName = AbstractbpelUtil.getPinElementName(tarSet, tarPin, findPart(tarSet, tarPin, tarCon));		
//			String srcPinElementName = AbstractbpelUtil.getPinElementName(srcSet, srcPin, findPart(srcSet, srcPin, srcCon));
//
//			javaString += AbstractbpelUtil.generateJavaGetterForVariable(tarVarName) + "(true)" 
//											+ "." + AbstractbpelUtil.generateJavaGetterForVariable(tarPartName) +"()" 
//													+ ".set" + tarPinElementName + "(" 
//															+ AbstractbpelUtil.generateJavaGetterForVariable(srcVarName) +"()"
//																+ "." + AbstractbpelUtil.generateJavaGetterForVariable(srcPartName) +"()" +
//																		"." + AbstractbpelUtil.generateJavaGetterForVariable(srcPinElementName) + "()"
//																 
//													 + ");\n";
//			javaCode.setValue(javaString);
//			jsActivity.setJavaCode(javaCode);
//			javaInvoke.getExtensibilityElements().add(jsActivity);
//			createDisplayName(javaInvoke,"Set value for " + srcPin.getName());		
//			//	createDisplayName(javaInvoke,"Set Constant Value");		
//			return javaInvoke;
//		
//		}
//	}
//	
//	
//	private List getPinsFromSet(NamedElement set)
//	{
//		List pins;
//		if (set instanceof InputPinSet)
//			pins = ((InputPinSet)set).getInputObjectPin();
//		else
//			pins = ((OutputPinSet)set).getOutputObjectPin();
//		return pins;		
//	}
} //AssignRuleImpl
