package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.br.BRGroupRule;
import com.ibm.btools.te.ilm.heuristics.br.BRSetRule;
import com.ibm.btools.te.ilm.heuristics.br.BrFactory;
import com.ibm.btools.te.ilm.heuristics.br.util.BRExpressionConstants;
import com.ibm.btools.te.ilm.heuristics.br.util.BRExpressionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.wbiservers.businessrule.model.brg.BrgFactory;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbiservers.common.componentdef.ComponentdefFactory;
import com.ibm.wbit.br.core.model.ModelFactory;
import com.ibm.wbit.br.core.model.Property;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>BR Reusable Task Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BRReusableTaskRuleImpl extends ActionRuleImpl implements BRReusableTaskRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private AttributeValueProvider avp = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BUSINESS_RULE_NAME);
//	private boolean expose2BusinessSpace = false;
//	private boolean firstTimeCreateSystemProperty = true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BRReusableTaskRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.BR_REUSABLE_TASK_RULE;
	}

	public AttributeValueProvider getAvp() {
		return avp;
	}


	/**
	 * Creates and executes rules to map BOM BusinessRuleAction to Business Rule constructs.
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
		
		// Need to get the PortType to create the SCDL Component/Export
		BusinessRuleAction businessRuleAction = (BusinessRuleAction)getSource().get(0);
		ProcessInterfaceRule processIntRule = createProcessInterfaceRule(businessRuleAction);
		TransformationRule portTypeRule = createPortTypeRule(processIntRule,businessRuleAction);	
		
		PortType pt = (PortType)portTypeRule.getTarget().get(0);
		
		avp.setContext(getContext());
		BusinessRuleGroup businessRuleGroup = BrgFactory.eINSTANCE.createBusinessRuleGroup();
		//feature 16509, 87176
	    createCustomProperty(businessRuleAction.getActivity(), businessRuleGroup);
	    
//		businessRuleGroup.setProperty(BRExpressionConstants.PROPERTY_IBM_SYSTEM_VERSION, BRExpressionConstants.PROPERTY_SYSTEM_DEFINED, BRExpressionConstants.PROPERTY_VALUE);
		String baseName = BrUtil.createRuleGroupName(businessRuleAction, businessRuleGroup, getAvp(), getContext());
		businessRuleGroup.setName(baseName);
		getTarget().add(0, businessRuleGroup);
//		Add system property on the rule set to be exposed to the business space LI 15004
		ExportOperationUtil.createSystemCustomProperty(businessRuleGroup, ExportOperationConstants.BRR_CUSTOM_PROPERTY_NAME,ExportOperationConstants.TRUE_LIETRAL);
		
		// reset rules for input and output pin sets
		BRExpressionSerializer.getInstance().setInputPinSetRule(null);
		BRExpressionSerializer.getInstance().setOutputPinSetRule(null);
		BRExpressionSerializer.getInstance().setProcessInterfaceRule(processIntRule);
		// create business rule sets
		createBusinessRuleSets(businessRuleAction, pt);		
 		
		BRGroupRule brGroupRule = BrFactory.eINSTANCE.createBRGroupRule();
		
		BusinessRuleGroupTable brGroupTable = null;
		if (businessRuleAction.getOwnedRuleGroup() != null && !businessRuleAction.getOwnedRuleGroup().isEmpty()) {
			brGroupRule.getSource().add(businessRuleAction.getOwnedRuleGroup().get(0));		// add BusinessRuleGroup
			brGroupRule.getSource().add(pt);		// add PortType as second source
			
			// create child rule - business rule group table rule and add brgt as child rule
			getChildRules().add(brGroupRule);
			brGroupRule.transformSource2Target();
			// only 1 target (.brgt document) - it is added to the resource targets when business rule SA is created
			brGroupTable = (BusinessRuleGroupTable)brGroupRule.getTarget().get(0); 
		} else {
			brGroupTable = BrUtil.createEmptyBRGroupTable(businessRuleAction, baseName, pt);
			getTarget().add(brGroupTable);
		}
		
		BrUtil.updateBusinessRuleDocument(businessRuleGroup, businessRuleAction, pt, baseName, brGroupTable, getTarget());

		
		// Create SCDL Component
		DocumentRoot componentRoot = createSCDLComponent(businessRuleAction, pt);
		DocumentRoot exportRoot = null;
	    // Create SCDL Export
		//feature 118131, only create SCDL Export if we have created a Component
		//AZ WMT 20070322 add pattern support
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(ScdlUtil.isComponent(componentRoot)|| (BpelOptions.isBestPracticePatternEnabed(exportSession)&& componentRoot!= null)){
			exportRoot = createSCDLExport(componentRoot, pt);
		}
		
		//Add component and export to the top level module registry
		if (exportSession != null) {
			Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
			if(docRootRegistry instanceof Set){
				((Set)docRootRegistry).add(componentRoot);
				((Set)docRootRegistry).add(exportRoot);
				
				if (getTarget() != null) {
					Iterator it = getTarget().iterator();
					Object obj = null;
					while (it.hasNext()) {
						obj = it.next();
						if (obj instanceof RuleSet) {
							((Set)docRootRegistry).add(obj);
						} else if (obj instanceof BusinessRuleGroup) {
							((Set)docRootRegistry).add(obj);
							if (((BusinessRuleGroup)obj).getBusinessRuleGroupTable() != null) {
								((Set)docRootRegistry).add(((BusinessRuleGroup)obj).getBusinessRuleGroupTable());
							}
						} else if (obj instanceof com.ibm.wbit.br.core.model.DocumentRoot) {
							((Set)docRootRegistry).add(obj);
						}
					}
				}
			}			
		}
		
	    propagateChildTargets();
				
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}

	// create Business Rule Sets for the action
	@SuppressWarnings("unchecked")
	private void createBusinessRuleSets(BusinessRuleAction businessRuleAction, PortType pt) {
		if (businessRuleAction.getOwnedRuleSet() == null ||
				businessRuleAction.getOwnedRuleSet().isEmpty()) {
			return;
		}
		
		Iterator it = businessRuleAction.getOwnedRuleSet().iterator();
		BusinessRuleSet brSet = null;
		BRSetRule brSetRule = null;
		
		while (it.hasNext()) {
			brSet = (BusinessRuleSet)it.next();
			brSetRule = BrFactory.eINSTANCE.createBRSetRule();
			brSetRule.getSource().add(brSet);		// add BusinessRuleSet
			brSetRule.getSource().add(pt);

			getChildRules().add(brSetRule);
			
			// Execute the rule set
			brSetRule.transformSource2Target();			
			
			//propagate target for a Rule Set - add .ruleset to the targets
			getTarget().add(brSetRule.getTarget().get(0));
			//Add system property on the rule set to be exposed to the business space LI 15004
			ExportOperationUtil.createSystemCustomProperty((EObject)brSetRule.getTarget().get(0), ExportOperationConstants.BRR_CUSTOM_PROPERTY_NAME,ExportOperationConstants.TRUE_LIETRAL);
		}
	}
	
	/**
	 * Creates a custom property which identifies the moduleName to be used as filter in the BusinessSpace.
	 * @param activity
	 * @param task
	 */
	public void createCustomProperty(Activity activity, BusinessRuleGroup brGroup) {
		//defect 84992 create the default BusinessRule property
		final String PROPERTY_NAME__VERSION = "IBMSystemVersion";
		com.ibm.wbiservers.common.componentdef.Property defaultProperty = ComponentdefFactory.eINSTANCE.createProperty();
		
		defaultProperty.setName(PROPERTY_NAME__VERSION);
		defaultProperty.setValue("6.1.0");
		defaultProperty.setType("SystemDefined");
		if(brGroup.getProperties() != null){
			brGroup.getProperties().add(defaultProperty);
		}
		//Feature 161512, create BusinessSpace filter.
		String moduleName = BpelOptionsUtil.getWPSModuleyNameForElement(activity.getUid());
		if(moduleName != null){
			com.ibm.wbiservers.common.componentdef.Property property = ComponentdefFactory.eINSTANCE.createProperty();
			property.setName("BUSINESS_SPACE_BIUSINESSRULE_FILTER");
			property.setValue(moduleName);
			property.setType("SystemDefined");
			if(brGroup.getProperties() != null){
				brGroup.getProperties().add(property);
			}
		}
		
	}
	
	

//	public boolean isExpose2BusinessSpace() {
//		return expose2BusinessSpace;
//	}
//
//	public void setExpose2BusinessSpace(boolean expose2BusinessSpace) {
//		this.expose2BusinessSpace = expose2BusinessSpace;
//	}

//	public boolean isFirstTimeCreateSystemProperty() {
//		return firstTimeCreateSystemProperty;
//	}
//
//	public void setFirstTimeCreateSystemProperty(
//			boolean firstTimeCreateSystemProperty) {
//		this.firstTimeCreateSystemProperty = firstTimeCreateSystemProperty;
//	}

} //BRReusableTaskRuleImpl