/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Message;

import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BpelOptimizationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeWithCallbackRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CycleLinkUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.util.StringHelper;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Condition;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.OnMessage;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Id;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Bpel Optimization Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BpelOptimizationRuleImpl extends TransformationRuleImpl implements BpelOptimizationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	

	private List linksToBePropagated=null;

	private BPELVariable emptyVar=null;
	private List structuredActivities=null;
	private HashMap caseActivityMappings = new HashMap(); // Mapping of outputPinSet and Case Activity (Flow or Empty)
	private HashMap<Activity, Case> activityToCase = new HashMap<Activity, Case>();  //mapping of onMessage to their respective activity (flow or empty)
	private HashMap onMessageActivityMappings = new HashMap(); // Mapping of PinSets and OnMessage Activity (Flow or Empty)
	//private HashMap<PinSet, Activity> onMessageActivityMappings = new HashMap<PinSet, Activity>(); // Mapping of PinSets and OnMessage Activity (Flow or Empty)
	private HashMap<Activity, OnMessage> activityToOnMessage = new HashMap<Activity, OnMessage>();  //mapping of onMessage to their respective activity (flow or empty)
	private HashMap targetActivityJoinCondition = new HashMap();
	
	//Collection to record kept gateway activity for final consolidation
	private List<Activity> keptActivityForGateway = new ArrayList<Activity>();

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BpelOptimizationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.BPEL_OPTIMIZATION_RULE;
	}

	/**
	 * Optimize the BPEL process
	 * Source = BPEL Process
	 * Target = Optimized BPEL Process
	 */
	public boolean transformSource2Target() {
		Process process = (Process)getSource().get(0);
		optimizeTopLevelProcessContainer(process);
		transformActivity(process.getActivity());
		getTarget().add(process);
		if (linksToBePropagated != null) {
			propagateLinksToParent(linksToBePropagated);
		}
		if (BpelOptimizationUtil.bRebuildExclusiveStructure) {
			rebuildExclusiveStructure(structuredActivities);
		}

		assignWpcIds(process, 1000);	
		//Final consolidate the retained Empty activity
		consolidateRedundantEmpty();
		//Assign gateway type
		CBPELUtil.optimizeActivityTypeAttribute(this, process.getActivity(), getContext());			
		//The above call will convert any flows in the onMessage into GFlows.  As such, the mappings stored in 
		//onMessageActivityMappings are no longer valid since they are still pointing to the old flows.  We'll need
		//to fix this map in order for the handlers to generate proper mappings
		//		for (Iterator it = onMessageActivityMappings.keySet().iterator(); it.hasNext(); ) {
		//			PinSet key = (PinSet) it.next();
		//			Activity activity = (Activity) onMessageActivityMappings.get(key);
		//			if (activity.eContainer() == null) {
		//				//GFlow was generated and replaced the old flow, retrieve the GFlow that was generated from the scope
		//				onMessageActivityMappings.put(key, activityToOnMessage.get(activity).getActivity());
		//			}
		//		}
		//		//Same thing goes for Case
		//		for (Iterator it = caseActivityMappings.keySet().iterator(); it.hasNext(); ) {
		//			PinSet key = (PinSet) it.next();
		//			Activity activity = (Activity) caseActivityMappings.get(key);
		//			if (activity.eContainer() == null) {
		//				//GFlow was generated and replaced the old flow, retrieve the GFlow that was generated from the scope
		//				caseActivityMappings.put(key, activityToCase.get(activity).getActivity());
		//			}
		//		}
		fixConvertedGFlows(onMessageActivityMappings, activityToOnMessage);
		fixConvertedGFlows(caseActivityMappings, activityToCase);

		executeHandlers();

		return true;
	}
    
	/**
	 * Final consolidate for the non-optimized empty activity before assign gateway type
	 */
	private void consolidateRedundantEmpty() {
		if(keptActivityForGateway != null && !keptActivityForGateway.isEmpty()){
			for(Activity eachAct:keptActivityForGateway){
				if(eachAct instanceof Empty){
					removeActivity(eachAct);
				}
			}			
		}		
	}

	/**
	 * 
	 * @param mappingToFix one of onMessageActivityMappings or caseActivityMappings
	 * @param activityToContainerMap one of activityToOnMessage or activityToCase
	 */
	private void fixConvertedGFlows(HashMap mappingToFix, HashMap activityToContainerMap)
	{
		for (Iterator it = mappingToFix.keySet().iterator(); it.hasNext(); ) {
			PinSet key = (PinSet) it.next();
			Activity activity = (Activity) mappingToFix.get(key);
			if (activity.eContainer() == null) {
				//GFlow was generated and replaced the old flow, retrieve the GFlow that was generated from the scope
				if (activityToContainerMap.get(activity) instanceof OnMessage) {
					mappingToFix.put(key, ((OnMessage)activityToContainerMap.get(activity)).getActivity());
				}
				else if (activityToContainerMap.get(activity) instanceof Case) {
					mappingToFix.put(key, ((Case)activityToContainerMap.get(activity)).getActivity());
				}
			}
		}
	}

	/**
	 * Assigns a unique value to the wpc:id attribute of each BPEL object 
	 * (either process or activity)
	 * 
	 * @param ee The extensibility element to add a wpc:id to 
	 * @param startId The id value to assign to this element
	 * @return The next available id value for assignment.
	 */
	private int assignWpcIds(EObject eObj, int startId){

		if (eObj instanceof Activity || eObj instanceof Process || eObj instanceof BPELVariable ||
				eObj instanceof Case || eObj instanceof Link){
			Id id = BPELPlusFactory.eINSTANCE.createId();
			//id.setId(new Integer(startId));  
			id.setId(ExportOperationUtil.generateConstructUidForTarget(eObj));
			((ExtensibleElement)eObj).getEExtensibilityElements().add(id);
			// startId++;	            			
		} 
		EObject o = null;
		for (Iterator it = eObj.eContents().iterator(); it.hasNext(); ){
			o = (EObject)it.next();
			startId = assignWpcIds(o, startId);
		}

		return startId;
	}	

	/**
	 * This method attempts to rebuild the structured flow by analyzing the flow follows from each
	 * structured activity and put the activity into the structure whenever possible
	 * @param acts - a list of structured activities
	 */
	private void rebuildExclusiveStructure(List acts) {
		Activity structuredActivity,parentContainer;
		Pick pick;
		Switch switchAct;
		OnMessage onMsg;
		Case caseAct;
		if (acts!=null) {
			List actsWithLinksGoingOutTheFlow = new ArrayList();
			for (Iterator it = acts.iterator();it.hasNext();) {
				structuredActivity=(Activity)it.next();
				parentContainer = (Activity)structuredActivity.eContainer();
				if (structuredActivity instanceof Pick) {
					pick = (Pick)structuredActivity;
					// Get the pinsets for the onMessage to prepare for recording of the mapping 
					List pinSets=getPinSetsForOnMessages(pick);
					int onMsgCount=0;
					List msgs=pick.getMessages();
					for (Iterator it2=msgs.iterator();it2.hasNext();) {
						onMsg = (OnMessage)it2.next();
						if (onMsg.getActivity() instanceof Empty) {
							Empty empty = (Empty)onMsg.getActivity();
							Flow flow = buildStructuredFlow(structuredActivity,empty,parentContainer,actsWithLinksGoingOutTheFlow);
							if (!flow.getActivities().isEmpty()) {
								onMsg.setActivity(flow);								
							} else {
								onMsg.setActivity(empty);
							}
						}
						// Record the mapping to be picked up by the Handler
						if (pinSets!=null) {
							onMessageActivityMappings.put(pinSets.get(onMsgCount), onMsg.getActivity());
							activityToOnMessage.put(onMsg.getActivity(), onMsg);
							onMsgCount++;								
						}
					}
				} else if (structuredActivity instanceof Switch) {
					switchAct = (Switch)structuredActivity;
					// Get the pinsets for the case to prepare for recording of the mapping 
					List pinSets=getPinSetsForCase(switchAct);
					int caseCount=0;
					List cases = switchAct.getCases();
					for (Iterator it2=cases.iterator();it2.hasNext();) {
						caseAct = (Case)it2.next();
						if (caseAct.getActivity() instanceof Empty) {
							Empty empty=(Empty)caseAct.getActivity();
							Flow flow = buildStructuredFlow(structuredActivity,empty,parentContainer,actsWithLinksGoingOutTheFlow);
							// If the flow has no content, then we should not set it as the activity for the Case
							// because WID will complain: The parallel activities must contain an activity (parallel activities 'ParallelActivities').
							// In fact, if the flow has no activity, then it is arguable that we should remove the Case from the Switch all together.
							// But this may produce a problem to Monitor where it needs to calculate the probability of each Decision Branch.  So
							// if without the explicity switch case then there will be no way to calcuate the probability (but in a binary branch
							// it may still be possible if we assume both branches probability should add up to 100%
							if (!flow.getActivities().isEmpty()) {
								caseAct.setActivity(flow);								
							} else {
								caseAct.setActivity(empty);
							}
							// Record the mapping to be picked up by the Handler
							if (pinSets!=null) {
								caseActivityMappings.put(pinSets.get(caseCount), caseAct.getActivity());
								activityToCase.put(caseAct.getActivity(), caseAct);
								caseCount++;								
							}
						}
					}
				}
			}	
			// Take care of the crossing links after restructed the flow
			fixCrossingLinksForPickOrSwitch(actsWithLinksGoingOutTheFlow,new ArrayList());
		}
	}

	/**
	 * Get the list of PinSets that mapped to the OnMessages of a Pick
	 * @param pick
	 * @return
	 */
	private List getPinSetsForOnMessages(Pick pick) {
		if (pick.getCreateInstance()==false) {
			TransformationRule rule=TransformationUtil.getRuleForTarget(this.getParentRule(),InvokeWithCallbackRule.class, pick);
			if (rule!=null) {
				InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);
				return inputPinSet.getOutputPinSet();
			}
		} else {
			com.ibm.btools.bom.model.processes.activities.Activity BOMProcess = (com.ibm.btools.bom.model.processes.activities.Activity)getSource().get(1);
			StructuredActivityNode SAN=BOMProcess.getImplementation();
			return SAN.getInputPinSet();
		}
		return null;
	}

	/**
	 * Get the list of PinSets that mapped to the Case of a Switch
	 * @param switchActivity
	 * @return
	 */
	private List getPinSetsForCase(Switch switchActivity) {
		TransformationRule rule=TransformationUtil.getRuleForTarget(this.getParentRule(),DecisionRule.class, switchActivity);
		List outputPinSets=null;
		if (rule!=null) {
			InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);
			outputPinSets = inputPinSet.getOutputPinSet();
		}
		return outputPinSets;
	}
	/**
	 * Maintain a list of activities that has crossinglinks after the restructure of the flow.
	 * Attempt to fix these crossing links.  This method maybe calling itself recursively until it
	 * can't fix the links anymore
	 * @param actsWithLinksGoingOutTheFlow
	 * @param actsToBeRevisited - for recusive calling use
	 */
	private void fixCrossingLinksForPickOrSwitch(List actsWithLinksGoingOutTheFlow, List actsToBeRevisited) {
		List prevActsWithLinksGoingOutTheFlow = null;
		// Keep fixing the crossing links until either they are all fixed or can't fix any more 
		while (!actsWithLinksGoingOutTheFlow.isEmpty() &&
				!(actsWithLinksGoingOutTheFlow != null && actsWithLinksGoingOutTheFlow.equals(prevActsWithLinksGoingOutTheFlow))) {
			prevActsWithLinksGoingOutTheFlow = actsWithLinksGoingOutTheFlow;
			actsWithLinksGoingOutTheFlow = fixCrossingLinksForPickOrSwitch(actsWithLinksGoingOutTheFlow);
			// if there are still crossing links and the list of crossing links have been updated since it was called, then we
			// should attempt to rebuild the structure flow again
			List newActsWithLinksGoingOutTheFlow = new ArrayList();
			for (Iterator it=actsWithLinksGoingOutTheFlow.iterator();it.hasNext();) {	// rebuild all flows that had problems before
				Activity act = (Activity)it.next();
				EObject parent = null;
				if (act instanceof Switch || act instanceof Pick) {
					parent = act;
					buildStructuredFlow((Activity)parent.eContainer(),(Flow)act.eContainer(),act,new ArrayList(),newActsWithLinksGoingOutTheFlow);
				}
				parent = BpelOptimizationUtil.findPickOrSwitchParent(act);
				if (parent != null) {
					if (act.eContainer() instanceof Flow) {
						// after the links are fixed, try to rebuild the structured flow
						buildStructuredFlow((Activity)parent.eContainer(),(Flow)act.eContainer(),act,new ArrayList(),newActsWithLinksGoingOutTheFlow);
					}
				}
			}

			actsWithLinksGoingOutTheFlow = newActsWithLinksGoingOutTheFlow;
		}

		// if there are still crossing links and the list of crossing links have been updated since it was called, then we
		// should attempt to rebuild the structure flow again
		if (!actsToBeRevisited.isEmpty() && (actsToBeRevisited != null && !actsToBeRevisited.equals(actsWithLinksGoingOutTheFlow))) {
			List newActsWithLinksGoingOutTheFlow = new ArrayList();
			for (Iterator it=actsWithLinksGoingOutTheFlow.iterator();it.hasNext();) {
				Activity act = (Activity)it.next();
				EObject parent = BpelOptimizationUtil.findPickOrSwitchParent(act);
				if (parent != null) {
					if (act.eContainer() instanceof Flow) {
						// after the links are fixed, try to rebuild the structured flow
						buildStructuredFlow((Activity)parent.eContainer(),(Flow)act.eContainer(),act,new ArrayList(),newActsWithLinksGoingOutTheFlow);
					}
				}
			}
			// A new list of activities of crossing link maybe created.  Try to fix them again
			if (!newActsWithLinksGoingOutTheFlow.isEmpty()) {
				fixCrossingLinksForPickOrSwitch(newActsWithLinksGoingOutTheFlow,new ArrayList(newActsWithLinksGoingOutTheFlow));				
			}
		}

	}

	/**
	 * Fix the crossing links in the Pick or Switch
	 */
	private List fixCrossingLinksForPickOrSwitch(List actsWithLinksGoingOutTheFlow) {
		List moreActsWithLinksGoingOutTheFlow = new ArrayList();
		if (!actsWithLinksGoingOutTheFlow.isEmpty()) {
			Activity act,tarAct;
			Source src;
			Link link;
			Target tar;
			List sourceToBeUpdated = new ArrayList();
			List sources;
			List visitedActivities = new ArrayList();
			// For each of the activities with crossing link, navigate to its linked target, then
			// 1) ensure the target's join condition is only OR not AND
			// 2) find all the activities linked to this target, and ensure that each branch of the Pick/Switch contains at least
			// one of these activities.
			// If both are true, then it is safe to relink the links directly to the Pick or Switch
			for (Iterator it=actsWithLinksGoingOutTheFlow.iterator();it.hasNext();) {
				act = (Activity)it.next();
				if (act.getSources()!=null) {
					sources = act.getSources().getChildren();
					for (int i=sources.size()-1;i>=0;i--) {
						src = (Source)sources.get(i);
						link = src.getLink();
						tar = (Target)link.getTargets().get(0);
						tarAct = tar.getActivity();
						if (isJoinConditionOr(BpelOptimizationUtil.getJoinCondition(tarAct), tarAct)) {
							Activity structuredActivity = (Activity)BpelOptimizationUtil.findPickOrSwitchParent(act);
							List srcActs = findSourceActivitiesInSamePickOrSwitch(tarAct,structuredActivity,new ArrayList());
							if (areActivitiesSpreadInEachStructuredBranch(srcActs,structuredActivity)) {
								src.setActivity(structuredActivity);
								if (structuredActivity.getSources()!=null) {
									BpelOptimizationUtil.fixParallelLinksForSourceActivity(structuredActivity);			
								}	
								// Update the list of remaining act with crossing links
								if (!moreActsWithLinksGoingOutTheFlow.contains(structuredActivity)) {
									moreActsWithLinksGoingOutTheFlow.add(structuredActivity);
								}
							} else if (srcActs != null && srcActs.size() == 1 && (act instanceof Switch || act instanceof Pick) && 
									act == srcActs.get(0) && structuredActivity instanceof Switch && !(tarAct instanceof Empty)) {	//promote activity to its container
								if (act.eContainer() instanceof Flow) {
									Flow flow = (Flow)act.eContainer();
									flow.getActivities().add(tarAct);
									Activity structuredActivityForTar = (Activity)BpelOptimizationUtil.findPickOrSwitchParent(tarAct);
									// fix my sources and targets - for example - moved an Invoke to parent container									
									// update join condition for target - remove the reorganized link from transition condition
									// assume that transition condition will be handled by containing activity									
									//Sources - if in the same container, add the link to flow, otherwise set source to container
									Source tarActSource = null;
									Activity tarActSourceActivity = null;
									Target tarActTarget = null;
									Activity tarActTargetActivity = null;
									Link linkFromORToTarAct = null;
									Map<Source, Activity> sourceToActivity =  new HashMap();
									if (tarAct == null || tarAct.getSources() == null) {
										continue;
									}
									List tarActAsSources = tarAct.getSources().getChildren();
									for (Object obj: tarActAsSources) {
										tarActSource = (Source)obj;		// this is the same source as before tarAct; e.g. Invoke 
										linkFromORToTarAct = tarActSource.getLink();
										tarActSourceActivity = tarActSource.getActivity();
										if (linkFromORToTarAct != null && linkFromORToTarAct.getTargets() != null && !linkFromORToTarAct.getTargets().isEmpty()) {
											tarActTarget = (Target)linkFromORToTarAct.getTargets().get(0);
										}
										if (tarActTarget != null) {
											tarActTargetActivity = tarActTarget.getActivity();
										}
										if (tarActSourceActivity.eContainer() == tarActTargetActivity.eContainer()) {
											if (flow.getLinks() == null) {
												Links links = BPELFactory.eINSTANCE.createLinks();
												flow.setLinks(links);
											}
											flow.getLinks().getChildren().add(linkFromORToTarAct);
										} else { //set source to container
											if (structuredActivityForTar != null) {
												sourceToActivity.put(tarActSource, structuredActivityForTar);
											}
										}
									}
									if (!sourceToActivity.isEmpty()) {
										Iterator <Source> sourceIt = sourceToActivity.keySet().iterator();
										Source containerSource = null;
										Activity containerActivity = null;
										while (sourceIt.hasNext()) {
											containerSource = sourceIt.next();
											containerActivity = sourceToActivity.get(containerSource);
											containerSource.setActivity(containerActivity);
											
											if (containerActivity.eContainer() instanceof Flow) {
												Flow containerOfStructActivity = (Flow)containerActivity.eContainer();
												if (containerOfStructActivity.getLinks() == null) {
													Links links = BPELFactory.eINSTANCE.createLinks();
													containerOfStructActivity.setLinks(links);
												}
												containerOfStructActivity.getLinks().getChildren().add(containerSource.getLink());													
												if (containerActivity.getSources()!= null) {
													BpelOptimizationUtil.fixParallelLinksForSourceActivity(containerActivity);			
												}	
											}
										}
									}								
									sourceToActivity.clear();
									if (tarAct.getSources()!=null) {
										BpelOptimizationUtil.fixParallelLinksForSourceActivity(tarAct);			
									}	
									if (structuredActivityForTar.getSources()!= null) {
										BpelOptimizationUtil.fixParallelLinksForSourceActivity(structuredActivityForTar);			
									}	
					
									//Targets - if in the same container, add the flow to link, otherwise??? set target to container 
									Map<Target, Activity> targetToActivity =  new HashMap();
									List tarActAsTargets = tarAct.getTargets().getChildren();
									for (Object obj: tarActAsTargets) {
										tarActTarget = (Target)obj;
										linkFromORToTarAct = tarActTarget.getLink();
										tarActTargetActivity = tarActTarget.getActivity();
										if (linkFromORToTarAct != null && linkFromORToTarAct.getSources() != null && !linkFromORToTarAct.getSources().isEmpty()) {
											tarActSource = (Source)linkFromORToTarAct.getSources().get(0);
										}
										if (tarActSource != null) {
											tarActSourceActivity = tarActSource.getActivity();
										}
										if (tarActSourceActivity.eContainer() == tarActTargetActivity.eContainer()) {
											if (flow.getLinks() == null) {
												Links links = BPELFactory.eINSTANCE.createLinks();
												flow.setLinks(links);
											}
											flow.getLinks().getChildren().add(linkFromORToTarAct);
										} else { //set target to container
											if (structuredActivityForTar != null) {
												targetToActivity.put(tarActTarget, structuredActivityForTar);
											}
										}
									}

									if (!targetToActivity.isEmpty()) {
										Iterator <Target> targetIt = targetToActivity.keySet().iterator();
										Target containerTarget = null;
										Activity containerActivity = null;
										while (targetIt.hasNext()) {
											containerTarget = targetIt.next();
											if(link != containerTarget.getLink()) {
												BpelOptimizationUtil.updateJoinCondition(containerTarget.getLink());
											}
											containerActivity = targetToActivity.get(containerTarget);
											containerTarget.setActivity(containerActivity);
											if (containerActivity.eContainer() instanceof Flow) {
												Flow containerOfStructActivity = (Flow)containerActivity.eContainer();
												if (containerOfStructActivity.getLinks() == null) {
													Links links = BPELFactory.eINSTANCE.createLinks();
													containerOfStructActivity.setLinks(links);
												}
												containerOfStructActivity.getLinks().getChildren().add(containerTarget.getLink());
												if (containerActivity.getTargets()!=null) {
													BpelOptimizationUtil.fixParallelLinksForTargetActivity(containerActivity);			
												}													
											}
										}
									}								

									if (tarAct.getTargets()!=null) {
										BpelOptimizationUtil.fixParallelLinksForTargetActivity(tarAct);			
									}	
									if (structuredActivityForTar.getSources()!= null) {
										BpelOptimizationUtil.fixParallelLinksForTargetActivity(structuredActivityForTar);			
									}	
									
									targetToActivity.clear();
									
									if (!moreActsWithLinksGoingOutTheFlow.contains(structuredActivity)) {
										moreActsWithLinksGoingOutTheFlow.add(structuredActivity);
									}
								}	
							} else if ((structuredActivity instanceof Switch || structuredActivity instanceof Pick) && 
									((act instanceof Empty || act instanceof Switch || act instanceof Pick || act instanceof Assign)) && 
									BpelOptimizationUtil.isNoSourceForLinksWithinBranchAct(act)) {
								src.setActivity(structuredActivity);
								if (structuredActivity.getSources()!=null) {
									BpelOptimizationUtil.fixParallelLinksForSourceActivity(structuredActivity);			
								}	
								if (!moreActsWithLinksGoingOutTheFlow.contains(structuredActivity)) {
									moreActsWithLinksGoingOutTheFlow.add(structuredActivity);
								}
							} else if (structuredActivity == null && tarAct.eContainer() == act.eContainer() && act.eContainer() instanceof Flow) {
								if (act instanceof Empty) {
									BpelOptimizationUtil.fixParallelLinksForSourceActivity(act);
									BpelOptimizationUtil.fixParallelLinksForTargetActivity(tarAct);
									BpelOptimizationUtil.removeLink(link);
								}
							}
						}
					}
					if(sources.isEmpty()){
						act.setSources(null);
					} else {
						if (!moreActsWithLinksGoingOutTheFlow.contains(act)) {
							moreActsWithLinksGoingOutTheFlow.add(act);
						}
					}
				}
			}
		}			

		return moreActsWithLinksGoingOutTheFlow;
	}

	/**
	 * Check if the activities in the list (acts) are located in each of the branch in the Pick or Switch
	 * @param acts
	 * @param pickOrSwitchAct
	 * @return
	 */
	private boolean areActivitiesSpreadInEachStructuredBranch (List acts, Activity pickOrSwitchAct) {
		if (pickOrSwitchAct == null) {
			 return false;
		}
		int numberOfEmptyBranches = 0;
		List belongingBranches = new ArrayList();
		for (Iterator it=acts.iterator();it.hasNext();) {
			EObject act = (EObject)it.next();
			if (act == pickOrSwitchAct) {
				return true;
			} else {
				while (act.eContainer()!=null && act.eContainer().eContainer() != null &
						(!(act instanceof OnMessage || act instanceof Case) && 
						act.eContainer() != pickOrSwitchAct)) {
					act = act.eContainer().eContainer();
				}
				if (act != null) {
					if (!belongingBranches.contains(act)) {
						belongingBranches.add(act);
					}
				}

			}
		}
		if (pickOrSwitchAct instanceof Pick) {
			return (belongingBranches.size() + numberOfEmptyBranches) == ((Pick)pickOrSwitchAct).getMessages().size();			
		} else if (pickOrSwitchAct instanceof Switch){
			return (belongingBranches.size() + numberOfEmptyBranches) == ((Switch)pickOrSwitchAct).getCases().size();						
		} else {
			return false;
		}
	}

	/**
	 * From the target activity (tarAct), trace back the source activity of each of the link.  Return all source activities that are located
	 * in any one of the branch of the same Pick or Switch activity (pickOrSwitchAct)
	 * @param tarAct
	 * @param pickOrSwitchAct
	 * @return
	 */
	private List findSourceActivitiesInSamePickOrSwitch(Activity tarAct, Activity pickOrSwitchAct, List visitedActivities) {
		if (pickOrSwitchAct == null) {
			return null;
		}
			
		List srcActs = new ArrayList();

		if (!visitedActivities.contains(tarAct)) {
			visitedActivities.add(tarAct);
			Targets tars = tarAct.getTargets();
			List tarList = tars.getChildren();
			Target tar;
			Source src;
			Activity srcAct;
			EObject parent;
			for (Iterator it=tarList.iterator();it.hasNext();) {
				tar = (Target)it.next();
				src = (Source)tar.getLink().getSources().get(0);
				srcAct=(Activity)src.eContainer().eContainer();
				if (srcAct==pickOrSwitchAct) {
					srcActs.add(srcAct);
				} else {
					parent = BpelOptimizationUtil.findPickOrSwitchParent(srcAct);
					if (parent != null) {
						if (parent == pickOrSwitchAct) {
							srcActs.add(srcAct);
						}
					}
				}
			}
		}
		return srcActs;
	}

	/**
	 * @param tarAct
	 * @param pickOrSwitchAct
	 * @return pick or switch that owns all sources in all branches
	 */
	private Object findSameParentForSourceInAllBranchesPickOrSwitch(List targets) {
		if (targets != null && !targets.isEmpty() && targets.size() < 2) {
			return null;
		}
			
		int numberOfTargets = targets.size();
		List parentActivities = new ArrayList(numberOfTargets);
//		for (int i = 0; i < numberOfActivities; i++) {
//			parentActivities.add(i, new HashSet<Activity>());
//		}
		
		EObject act = null;
		EObject parent = null;
		Target target = null;
		Source mySrc = null;
		List sources = null;
		EObject srcAct = null;
		EObject pickOrSwitch =  null;
		for (int i = 0; i < numberOfTargets; i++) {
			List containers = new ArrayList();
			target = (Target)targets.get(i);
			sources = target.getLink().getSources();
			for (Object src: sources) {
				mySrc = (Source)src;
				srcAct = mySrc.eContainer().eContainer();
				if (srcAct instanceof OnMessage || act instanceof Case) {
					containers.add(act);
				}
				while (srcAct != null) {
					pickOrSwitch = BpelOptimizationUtil.findPickOrSwitchParent(srcAct.eContainer());
					if (pickOrSwitch != null) {
						containers.add(pickOrSwitch);
					}
					if (pickOrSwitch != null) {
						srcAct = pickOrSwitch.eContainer();
					} else {
						srcAct = pickOrSwitch;
					}
				}
			}
			parentActivities.add(i, containers);
		}
		
		// find the common container for the targets - if exists, start from the last one
		List firstTargetsContainers = (List)parentActivities.get(parentActivities.size() - 1);
		Object latestCommonContainer = null;
		for(Object container: firstTargetsContainers) {
			for (int j = 0; j < parentActivities.size() - 1; j++) {
				if (((List)parentActivities.get(j)).contains(container)){
					latestCommonContainer = container;
					break;	// break inner loop
				} else {
					latestCommonContainer = null;
//					break;	// break inner loop
				}
			}
			if (latestCommonContainer != null) {
				break;	// break outer loop
			}
		}
		//find the first container of the non null target
		if (latestCommonContainer == null) {
			for (int i = 0; i < parentActivities.size(); i++) {
				if (((List)parentActivities.get(i)).size() > 0) {
					return ((List)parentActivities.get(i)).get(0); 
				}
			}
		}
		return latestCommonContainer;
	}

	/**
	 * Create the Flow activity and pull in activities that is qualified to be pulled into this branch
	 * Update the list of activities that have links going outside of the flow (actsWithLinksGoingOutTheFlow), i.e. crossing links.
	 * These activities with crossing links will need to be handled later to see if the crossing links can be reduced
	 * @param structuredActivity
	 * @param empty
	 * @param parentContainer
	 * @param actsWithLinksGoingOutTheFlow
	 * @return
	 */
	private Flow buildStructuredFlow(Activity structuredActivity,Empty empty,Activity parentContainer,List actsWithLinksGoingOutTheFlow) {
		Flow flow = BPELFactory.eINSTANCE.createFlow();
		//defect 139867. generate name for the flow
		String proposedName = structuredActivity.getName() + "_" + empty.getName()+ "_" + "Flow";
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.getRegistry(this.getContext(), (Process)getSource().get(0)), flow, proposedName);
		flow.setName(convertedName);
		createDisplayName(flow, convertedName);
		List visitedActs = new ArrayList();
		buildStructuredFlow(parentContainer,flow,empty,visitedActs,actsWithLinksGoingOutTheFlow);	
		if (flow.getActivities().isEmpty() && (empty.getSources()!=null && !empty.getSources().getChildren().isEmpty()) && !actsWithLinksGoingOutTheFlow.contains(empty)) {
			actsWithLinksGoingOutTheFlow.add(empty);
		}
		//allow empty into the flow without link sources
		if (flow.getActivities().isEmpty()
				|| isSourceOfLinkWithTransitionCondition(empty)) {
			flow.getActivities().add(empty);
		} else {
			if (BpelOptimizationUtil.getNonOptimizableElements(getContext()).contains(empty)) {
				flow.getActivities().add(empty);
			} else {
				removeActivity(empty);
			}
		}
		return flow;
	}

	/**
	 * Check if an Activity act is the source of a link that has transition condition
	 * @param act
	 * @return
	 */
	private boolean isSourceOfLinkWithTransitionCondition(Activity act) {
		if (act.getSources()!=null) {
			for (Iterator it=act.getSources().getChildren().iterator();it.hasNext();) {
				Source src = (Source)it.next();
				if (src.getTransitionCondition()!=null) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Check if the joinCondition is all OR condition
	 * @param joinCondition
	 * @return
	 */
	private boolean isJoinConditionOr(com.ibm.wbit.bpel.Condition joinCondition, Activity bpelActivity) {
		//always get the original JoinCondition
		if(targetActivityJoinCondition.get(bpelActivity) != null){
			joinCondition = BpelOptimizationUtil.getJoinCondition(bpelActivity);
			targetActivityJoinCondition.put(bpelActivity, joinCondition);
		}else{
			joinCondition = (com.ibm.wbit.bpel.Condition)targetActivityJoinCondition.get(bpelActivity);
		}
		if (joinCondition != null) {
			// only handle Java type expression for now
			if (joinCondition.getExpressionLanguage().equals(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE)) {
				if (joinCondition.getBody() instanceof String) {
					String body = (String)joinCondition.getBody();
					if ((body.indexOf("&&")!=-1) && (body.indexOf("||")==-1)) {
						//if ((body.indexOf("&&")!=-1)) {
						return false;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Reconstruct the structured flow
	 * @param parentContainer
	 * @param flow
	 * @param startAct
	 * @param visitedActs
	 * @param actsWithLinksGoingOutTheFlow
	 */
	private void buildStructuredFlow (Activity parentContainer,Flow flow,Activity startAct,List visitedActs, List actsWithLinksGoingOutTheFlow) {
		visitedActs.add(startAct);
		List acts = getNextActivities(startAct);
		if (acts == null || acts.isEmpty()) {
			return;
		}
		
		Activity act;
		Targets tars;
		// For each of the activities that the startAct leads to, check if it should be put in the flow or not
		// This is determined by checking if the activity is a target of a link that links from another activity that has not
		// been visited and qualified to be put in the flow, then the activity should not be in the flow
		Link prevLink = null;
		for (Iterator it=acts.iterator();it.hasNext();) {
			act = (Activity)it.next();
			//20090410, d?
			if (act == null) {
				continue;
			}
			tars = act.getTargets();
			// Need to make sure the act is at the same level as the structuredActivity (i.e. Pick or Switch).
			// In other words, the act and the structuredActivity must belong to the same Flow activity
			if (act.eContainer()==parentContainer && tars != null) {
				Target tar;
				Link link = null;
				Source src;
				Activity srcAct;
				boolean bInBranch=true;
				for (Iterator it2=tars.getChildren().iterator();it2.hasNext();) {
					tar = (Target)it2.next();
					link = tar.getLink();
					
					src = (Source)link.getSources().get(0);
					srcAct = src.getActivity();
					if (!visitedActs.contains(srcAct)) {
						//if (!acts.contains(srcAct)) {
						bInBranch=false;
						if (!actsWithLinksGoingOutTheFlow.contains(startAct)) {
							actsWithLinksGoingOutTheFlow.add(startAct);
						}
						//System.out.println("Acts with links going out:"+startAct.getName());
						break;
						//}
					} else {
						if (prevLink != null && flow.getLinks()==null) {
							Links links = BPELFactory.eINSTANCE.createLinks();
							flow.setLinks(links);
						}
						if (prevLink != null) {
							flow.getLinks().getChildren().add(prevLink);
						}						
					}
					prevLink = link;
				}
				if (bInBranch) {
					//System.out.print(act.getName()+" ");
					flow.getActivities().add(act);
					if (flow.getLinks()==null) {
						Links links = BPELFactory.eINSTANCE.createLinks();
						flow.setLinks(links);
					}
					if (link != null) {
						flow.getLinks().getChildren().add(link);
					}
					buildStructuredFlow(parentContainer,flow,act,visitedActs,actsWithLinksGoingOutTheFlow);
				}
			} else {
				if (!actsWithLinksGoingOutTheFlow.contains(startAct)) {
					actsWithLinksGoingOutTheFlow.add(startAct);
				}
			}

		}
	}

	/**
	 * Find a list of activities that the activity (act) leads to
	 * @param act
	 * @return
	 */
	private List getNextActivities(Activity act) {
		List acts = new ArrayList();
		Sources srcs = act.getSources();
		if (srcs != null) {
			Source src;
			Link link;
			Target tar;
			for (Iterator it=srcs.getChildren().iterator(); it.hasNext();){
				src = (Source)it.next();
				link = src.getLink();
				tar = (Target)link.getTargets().get(0); // assume single target
				acts.add(tar.getActivity());
			}			
		}
		return acts;
	}

	private void optimizeTopLevelProcessContainer(Process process) {
		if (process.getActivity() instanceof Sequence) {
			process.setActivity(convertSequenceToFlow((Sequence)process.getActivity()));
			flattenChildFlow((Flow)process.getActivity());
		} else if (process.getActivity() instanceof Flow) {
			flattenChildFlow((Flow)process.getActivity());			
		}
		//mark the gen source model is from WBI modeler
		CBPELUtil.createProcessOriginProperty(process);
	}

	//	private void transformActivity(EObject bpelObject){
	//		
	//		/* Recursive base case */
	//		if (bpelObject == null){
	//			return;
	//		}
	//	
	//		ArrayList objsToBeRemoved = new ArrayList();
	//		EObject content;
	////		for (int i=0; i<bpelObject.eContents().size(); i++){
	//		for (int i=bpelObject.eContents().size()-1; i>=0; i--){
	//			content = (EObject)bpelObject.eContents().get(i);
	//			content = applyOptimization(content);
	//			if (content != null) {
	//				/* Recursively transform this object's children */
	//				transformActivity(content);
	//			}
	//		}
	//		applyOptimization(bpelObject);
	//	}

	private void transformActivity(EObject bpelObject){

		/* Recursive base case */
		if (bpelObject == null){
			return;
		}

		ArrayList objsToBeRemoved = new ArrayList();
		EObject content;
		//		for (int i=0; i<bpelObject.eContents().size(); i++){
		for (int i=bpelObject.eContents().size()-1; i>=0; i--){
			content = (EObject)bpelObject.eContents().get(i);
			if (content instanceof Flow || content instanceof Sequence) {
				/* Recursively transform this object's children */
				transformActivity(content);
			} else if (content instanceof Scope) {
				transformActivity(((Scope)content).getActivity());
			} else if (content instanceof While) {
				transformActivity(((While)content).getActivity());
			} else if (content instanceof Pick || content instanceof Switch) {
				if (BpelOptimizationUtil.bRebuildExclusiveStructure) {
					if (structuredActivities==null) {
						structuredActivities = new ArrayList();
					}
					structuredActivities.add(content);
				}
			}
			content = applyOptimization(content);
		}
		//applyOptimization(bpelObject);

		// Perform additional cleanup
		if (bpelObject instanceof Flow) {
			// After optimized the flow, there may not be links anymore left within the flow.
			// The classic example is a subprocess with a single task where after the Java activities representing
			// the termination logic are removed, there will be no more links exist in the flow.  So we need to
			// remove the Links object from Flow.  Otherwise, we will generate an empty <links/> tag which causes
			// problem when deploying the bpel in WSADIE 5.1
			Links links = ((Flow)bpelObject).getLinks();
			if (links!=null && links.getChildren().isEmpty()) {
				((Flow)bpelObject).setLinks(null);
			}
		}
	}	

	private void reduceEmptyVariable(Activity act) {
		if (act instanceof Invoke) {
			BPELVariable var=(BPELVariable)((Invoke)act).getInputVariable();
			BPELVariable newVar = getEmptyVariable(var);
			if (var != newVar) {
				((Invoke)act).setInputVariable(newVar);
				replaceElement(var,null);
			}
			var=(BPELVariable)((Invoke)act).getOutputVariable();
			newVar = getEmptyVariable(var);
			if (var != newVar) {
				((Invoke)act).setOutputVariable(newVar);
				replaceElement(var,null);
			}
		} else if (act instanceof Receive){
			BPELVariable var=(BPELVariable)((Receive)act).getVariable();
			BPELVariable newVar = getEmptyVariable(var);
			if (var != newVar) {
				((Receive)act).setVariable(newVar);
				replaceElement(var,null);
			}
		} else if (act instanceof Reply){
			BPELVariable var=(BPELVariable)((Reply)act).getVariable();
			BPELVariable newVar = getEmptyVariable(var);
			if (var != newVar) {
				((Reply)act).setVariable(newVar);
				replaceElement(var,null);				
			}			
		} else if (act instanceof Pick) {
			List onMsgs = ((Pick)act).getMessages();
			OnMessage onMsg;
			BPELVariable var;
			BPELVariable newVar;
			for (Iterator it=onMsgs.iterator();it.hasNext();) {
				onMsg = (OnMessage)it.next();
				var = (BPELVariable)onMsg.getVariable();
				newVar = getEmptyVariable(var);
				if (var != newVar) {
					onMsg.setVariable(newVar);
					replaceElement(var,null);				
				}			
			}
		}
	}

	private BPELVariable getEmptyVariable(BPELVariable var) {
		if (var != null) {		
			Message msg = var.getMessageType();
			if (msg.getParts().isEmpty()) {
				if (emptyVar == null) {
					emptyVar = var;
				}
				if (emptyVar != var && emptyVar.getMessageType()==var.getMessageType()) {
					return emptyVar;
				}
			}
		}
		return var;
	}
	/*	
	private Flow convertSequenceToFlow(Sequence sequence) {
		Flow flowContainer = BPELFactory.eINSTANCE.createFlow();
		List activities = sequence.getActivities();
		Activity firstActivity = (Activity)activities.get(0);
		Activity lastActivity = (Activity)activities.get(activities.size()-1);
		// go through the list of sequenced activities one by one and link them together and put it in the flow
		Activity activity=null, prevActivity=null;
		for (Iterator it=activities.iterator();it.hasNext();) {
			activity = (Activity)it.next();
			//flowContainer.getActivities().add(activity);
			if (prevActivity != null) {
				List links = new ArrayList();
				if (prevActivity instanceof Flow) {
					// find all activities inside the flow that is not a source of any link
					List noSourceAct = new ArrayList();
					List acts = ((Flow)prevActivity).getActivities();
					Activity act;
					for (int i=0; i<acts.size(); i++) {
						act = (Activity)acts.get(i);
						if (act.getSources().isEmpty()) {
							noSourceAct.add(act);				
						}
					}
					Link link;
					Source src;
					Activity srcAct;
					for (int i=0; i<noSourceAct.size(); i++) {
						link = BPELFactory.eINSTANCE.createLink();
						src = BPELFactory.eINSTANCE.createSource();
						link.getSources().add(src);
						srcAct = (Activity)noSourceAct.get(i);						
						src.setActivity(srcAct);
						links.add(link);
						link.setName(getActivityName(srcAct)+"To");
					}
				} else {
					Link link = BPELFactory.eINSTANCE.createLink();
					Source src = BPELFactory.eINSTANCE.createSource();
					link.getSources().add(src);
					src.setActivity(prevActivity);
					links.add(link);
					link.setName(getActivityName(prevActivity)+"To");
				}

				if (activity instanceof Flow) {
					// find all activities inside the flow that is not a target of any link
					List noTargetAct = new ArrayList();
					List acts = ((Flow)activity).getActivities();
					Activity act;
					for (int i=0; i<acts.size(); i++) {
						act = (Activity)acts.get(i);
						if (act.getTargets().isEmpty()) {
							noTargetAct.add(act);				
						}
					}
					Link link;
					Target tar;
					Activity tarAct;
					for (int i=0; i<noTargetAct.size(); i++) {
						for (int j=0; j<links.size(); j++) {
							link = (Link)links.get(j);
							tar = BPELFactory.eINSTANCE.createTarget();
							link.getTargets().add(tar);				
							tarAct = (Activity)noTargetAct.get(i);
							tar.setActivity(tarAct);
							link.setName(link.getName()+getActivityName(tarAct));
						}
					}
				} else {
					Link link;
					Target tar;
					for (int j=0; j<links.size(); j++) {
						link = (Link)links.get(j);
						tar = BPELFactory.eINSTANCE.createTarget();
						link.getTargets().add(tar);
						tar.setActivity(activity);
						link.setName(link.getName()+getActivityName(activity));
					}
				}
				if (flowContainer.getLinks()==null) {
					flowContainer.setLinks(BPELFactory.eINSTANCE.createLinks());
				}
				flowContainer.getLinks().getChildren().addAll(links);
			}
			prevActivity = activity;
		}
		flowContainer.getActivities().addAll(sequence.getActivities());
		lastActivity.getSources().addAll(sequence.getSources());
		firstActivity.getTargets().addAll(sequence.getTargets());
		return flowContainer;
	}
	 */

	private Flow convertSequenceToFlow(Sequence sequence) {
		Flow flowContainer = BPELFactory.eINSTANCE.createFlow();
		//defect 139867. generate name for the flow
		EObject container = sequence.eContainer();
		if(!(container instanceof Activity)){
			container = container.eContainer();
		}
		if(container != null){
			String proposedName = ((Activity)container).getName() + "_" + "Flow";
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.getRegistry(getContext(), (Process)getSource().get(0)), flowContainer, proposedName);
			flowContainer.setName(convertedName);
			createDisplayName(flowContainer, convertedName);
		}
		List activities = sequence.getActivities();
		Activity firstActivity = (Activity)activities.get(0);
		Activity lastActivity = (Activity)activities.get(activities.size()-1);
		// go through the list of sequenced activities one by one and link them together and put it in the flow
		Activity activity=null, prevActivity=null;
		for (Iterator it=activities.iterator();it.hasNext();) {
			activity = (Activity)it.next();
			//flowContainer.getActivities().add(activity);
			if (prevActivity != null) {
				List srcs = new ArrayList();
				List links = new ArrayList();
				if (prevActivity instanceof Flow) {
					// find all activities inside the flow that is not a source of any link
					//List noSourceAct = new ArrayList();
					List acts = ((Flow)prevActivity).getActivities();
					Activity act;
					for (int i=0; i<acts.size(); i++) {
						act = (Activity)acts.get(i);
						//if (act.getSources().isEmpty()) {
						//11.09 AR migration
						if (act.getSources() == null || act.getSources().getChildren().isEmpty()) {
							srcs.add(act);				
						}
					}
				} else {
					srcs.add(prevActivity);
				}

				if (activity instanceof Flow) {
					// find all activities inside the flow that is not a target of any link
					List noTargetAct = new ArrayList();
					List acts = ((Flow)activity).getActivities();
					Activity act;
					for (int i=0; i<acts.size(); i++) {
						act = (Activity)acts.get(i);
						//if (act.getTargets() .isEmpty()) {
						//						11.09 AR migration
						if (act.getTargets() == null || act.getTargets().getChildren().isEmpty()) {
							noTargetAct.add(act);				
						}
					}
					Link link;
					Target tar;
					Source src;
					Activity tarAct,srcAct;
					for (int i=0; i<noTargetAct.size(); i++) {
						for (int j=0; j<srcs.size(); j++) {
							link = createLink((Activity)srcs.get(j),(Activity)noTargetAct.get(i),flowContainer);
							/*
							link = BPELFactory.eINSTANCE.createLink();
							src = BPELFactory.eINSTANCE.createSource();
							link.getSources().add(src);
							srcAct = (Activity)srcs.get(j);						
							src.setActivity(srcAct);
							tar = BPELFactory.eINSTANCE.createTarget();
							link.getTargets().add(tar);				
							tarAct = (Activity)noTargetAct.get(i);
							tar.setActivity(tarAct);
							link.setName(getUniqueName(flowContainer,Link.class,getActivityName(srcAct)+"To"+getActivityName(tarAct)));
							 */
							links.add(link);
						}
					}
				} else {
					Link link;
					Target tar;
					Source src;
					Activity srcAct;
					for (int j=0; j<srcs.size(); j++) {
						link = createLink((Activity)srcs.get(j),activity,flowContainer);
						/*
						link = BPELFactory.eINSTANCE.createLink();
						src = BPELFactory.eINSTANCE.createSource();
						link.getSources().add(src);
						srcAct = (Activity)srcs.get(j);						
						src.setActivity(srcAct);
						tar = BPELFactory.eINSTANCE.createTarget();
						link.getTargets().add(tar);
						tar.setActivity(activity);
						link.setName(getUniqueName(flowContainer,Link.class,getActivityName(srcAct)+"To"+getActivityName(activity)));
						 */
						links.add(link);
					}
				}
				if (flowContainer.getLinks()==null) {
					flowContainer.setLinks(BPELFactory.eINSTANCE.createLinks());
				}
				flowContainer.getLinks().getChildren().addAll(links);
			}
			prevActivity = activity;
		}
		flowContainer.getActivities().addAll(sequence.getActivities());
		//migration AR 11.09


		if(sequence.getSources() != null && !sequence.getSources().getChildren().isEmpty()){
			if(lastActivity.getSources()== null){
				Sources sources = BPELFactory.eINSTANCE.createSources();
				lastActivity.setSources(sources);
			}
			lastActivity.getSources().getChildren().addAll(sequence.getSources().getChildren());
		}

		if(sequence.getTargets() != null && !sequence.getTargets().getChildren().isEmpty()){
			if(firstActivity.getTargets() == null){
				Targets targets = BPELFactory.eINSTANCE.createTargets();
				firstActivity.setTargets(targets);
			}
			firstActivity.getTargets().getChildren().addAll(sequence.getTargets().getChildren());
		}

		//		lastActivity.getSources().addAll(sequence.getSources());
		//		firstActivity.getTargets().addAll(sequence.getTargets());
		return flowContainer;
	}

	private Link createLink(Activity srcAct, Activity tarAct, Activity owner) {
		Link link;
		Target tar;
		Source src;
		link = BPELFactory.eINSTANCE.createLink();
		src = BPELFactory.eINSTANCE.createSource();
		link.getSources().add(src);
		src.setActivity(srcAct);
		tar = BPELFactory.eINSTANCE.createTarget();
		link.getTargets().add(tar);				
		tar.setActivity(tarAct);
		String name = getActivityName(srcAct)+"To"+getActivityName(tarAct);
		//		if (owner != null) {
		//			link.setName(getUniqueName(owner,Link.class,name));
		//		} else {
		//			link.setName(name);
		//		}
		link.setName(getUniqueName(null,Link.class,name));
		return link;
	}

	private HashMap nameRegistry = new HashMap();

	private String getUniqueName(Object container, Class type, String name) {
		String uniqueName=name;
		if (container == null) {
			container = this;
		}
		HashMap subRegistry = (HashMap)nameRegistry.get(container);
		if (subRegistry==null) {
			subRegistry = new HashMap();
			nameRegistry.put(container,subRegistry);
		}
		List names = (List)subRegistry.get(type);
		if (names == null) {
			names = new ArrayList();
			subRegistry.put(type,names);
		}
		String regName;
		for (int i=0; i<names.size();i++) {
			regName = (String)names.get(i);
			if (name.equals(regName)) {
				uniqueName = name+i;
				return getUniqueName(container,type,uniqueName);
			}
		}
		names.add(uniqueName);
		return uniqueName;
	}

	private String getActivityName(Activity activity) {
		if (activity.getName()!=null && activity.getName().length()>0) {
			return activity.getName();
		} else if (activity instanceof Sequence) {
			return "Sequence";
		} else if (activity instanceof Flow) {
			return "Flow";
		} else if (activity instanceof Scope) {
			return "Scope";
		} else if (activity instanceof Assign) {
			return "Assign";
		} else {
			return "Default";
		}
	}

	/*
	private String getUniqueName(String activityType) {
		String uniqueName=activityType;
		Integer counter = (Integer) nameRegistry.get(activityType);
		if (counter == null) {
			counter = new Integer(0);
		}
		int counterInt = counter.intValue();
		if (counterInt > 0) {
			uniqueName = uniqueName+counterInt;
		}
		nameRegistry.put(activityType,new Integer(++counterInt));
	 	return uniqueName;
	}

	private String getActivityName(Activity activity) {
		if (activity.getName()!=null && activity.getName().length()>0) {
			return activity.getName();
		} else if (activity instanceof Sequence) {
			return getUniqueName("Sequence");
		} else if (activity instanceof Flow) {
			return getUniqueName("Flow");
		} else if (activity instanceof Scope) {
			return getUniqueName("Scope");
		} else if (activity instanceof Assign) {
			return getUniqueName("Assign");
		} else {
			return getUniqueName("Default");
		}
	}
	 */	
	//	private EObject applyOptimization(EObject activity) {
	//		if (activity instanceof Sequence) {
	//			Flow flow = convertSequenceToFlow((Sequence)activity);
	//			replaceElement(activity,flow);
	//			transformActivity(flow);
	//			if (flow.eContainer() instanceof Flow) {
	//				flattenChildFlow((Flow)flow.eContainer());				
	//			}
	//		} else if (activity instanceof Scope) {
	//			return ((Scope)activity).getActivity();
	//		} else if (activity instanceof While) {
	//			return ((While)activity).getActivity();
	//		} else if (activity instanceof Flow) {
	//			flattenChildFlow((Flow)activity);
	//		} else if (activity instanceof Invoke) {
	//			reduceEmptyVariable((Activity)activity);
	//		} else if (activity instanceof Receive) {
	//			reduceEmptyVariable((Activity)activity);
	//		} else if (activity instanceof Reply) {
	//			reduceEmptyVariable((Activity)activity);
	//		} else if (activity instanceof Empty) {
	//			removeEmptyActivity((Empty)activity);
	//		}
	//		return null;
	//	}

	private EObject applyOptimization(EObject activity) {
		if (activity instanceof Sequence) {
			Flow flow = convertSequenceToFlow((Sequence)activity);
			replaceElement(activity,flow);
			transformActivity(flow);
			if (flow.eContainer() instanceof Flow) {
				flattenChildFlow((Flow)flow.eContainer());				
			}			
		} else if (activity instanceof Scope) {
			CycleLinkUtil.removeDuplicatedSourceForScope((Scope)activity);
			return ((Scope)activity).getActivity();
		} else if (activity instanceof While) {
			return ((While)activity).getActivity();
		} else if (activity instanceof Flow) {
			flattenChildFlow((Flow)activity);			
		} else if (activity instanceof Invoke) {
			//reduceEmptyVariable((Activity)activity);
			reduceOrphanLinks((Invoke)activity);
		} else if (activity instanceof Receive) {
			//reduceEmptyVariable((Activity)activity);
			CBPELUtil.createGeneratedAttribute((Receive)activity, CBPELUtil.GEN_BPC_START_ACTIVITY);
		} else if (activity instanceof Reply) {
			//reduceEmptyVariable((Activity)activity);
			CBPELUtil.createGeneratedAttribute((Reply)activity, CBPELUtil.GEN_BPC_END_ACTIVITY);
		} else if (activity instanceof Empty) {
			removeActivity((Empty)activity);
		} else if (activity instanceof Pick) {
			//reduceEmptyVariable((Activity)activity);			
		}
		else if(activity instanceof Assign){
			reduceAssign((Assign)activity);
		}
		return null;
	}

	private void reduceOrphanLinks(Activity activity) {
		if(activity != null){
			if(activity.getSources()!= null && !activity.getSources().getChildren().isEmpty()){
				EList sourceList = activity.getSources().getChildren();
				List sourceToRemove = new ArrayList();
				for(Object eachSource:sourceList){
					if(eachSource instanceof Source){
						Source theSource = (Source)eachSource;
						Link examinedLink = theSource.getLink();
						if(examinedLink.getTargets()== null ||examinedLink.getTargets().isEmpty()){
							//This link does have a target, we remove it
							if(examinedLink.eContainer()instanceof Links){
								Links linkContainer = (Links)examinedLink.eContainer();
								linkContainer.getChildren().remove(examinedLink);
							}
							sourceToRemove.add(theSource);
						}
					}
				}	
				if(!sourceToRemove.isEmpty()){
					activity.getSources().getChildren().removeAll(sourceToRemove);
				}
			}
			if(activity.getTargets()!= null && !activity.getTargets().getChildren().isEmpty()){
				EList targetList = activity.getTargets().getChildren();
				List <Target> targetToRemove = new ArrayList<Target>();
				for(Object eachTarget:targetList){
					if(eachTarget instanceof Target){
						Target theTarget = (Target)eachTarget;
						Link examinedLink = theTarget.getLink();
						if(examinedLink.getSources()== null ||examinedLink.getSources().isEmpty()){
							//This link does have a target, we remove it
							if(examinedLink.eContainer()instanceof Links){
								Links linkContainer = (Links)examinedLink.eContainer();
								linkContainer.getChildren().remove(examinedLink);
							}
							targetToRemove.add(theTarget);
						}
					}
				}
				if(!targetToRemove.isEmpty()){
//20090521			activity.getTargets().getChildren().removeAll(targetToRemove);
					Link linkedTarget = null;
					for (Target targetRemove: targetToRemove) {
						linkedTarget = targetRemove.getLink();
						BpelOptimizationUtil.updateJoinCondition(linkedTarget);
						activity.getTargets().getChildren().remove(targetRemove);
					}					
				}
			}
		}

	}

	private boolean isSourceOfLink(Activity act) {
		return !(act.getSources()==null || (act.getSources() != null && act.getSources().getChildren().isEmpty()));
	}
	private boolean isTargetOfLink(Activity act) {
		return !(act.getTargets()==null || (act.getTargets() != null && act.getTargets().getChildren().isEmpty()));
	}

	private void reduceAssign(Assign assign){

		//the asssign should be removed only in the Copy contains similar From and To. 
		//check for now if we are in the case of the fork
		if(BpelOptimizationUtil.bOptimizeForkTransformation || BpelOptimizationUtil.bOptimizeJoinTransformation || BpelOptimizationUtil.bOptimizeMergeTransformation){
			TransformationRule rule = TransformationUtil.getRuleForTarget(ProcessUtil.getProcessDefinitionRule(getContext()), assign);
			AssignRule assignRule = null; 
			if(rule instanceof AssignRule){
				assignRule = (AssignRule)rule;
			}else{
				return;
			}
			if(BpelOptimizationUtil.getOptimizableElements(getContext()).contains(assign)){
				removeActivity(assign);
			}
			if(BpelOptimizationUtil.bOptimizeForkTransformation ){
				if(assignRule != null && (((PinSet)(assignRule.getSource().get(2))).eContainer() instanceof Fork)){
					removeActivity(assign);
				}
			}
			if(BpelOptimizationUtil.bOptimizeJoinTransformation){
				if(assignRule != null && (((PinSet)(assignRule.getSource().get(2))).eContainer() instanceof Join)){
					removeActivity(assign);
				}
			}
			if(BpelOptimizationUtil.bOptimizeMergeTransformation){
				if(assignRule != null && (((PinSet)(assignRule.getSource().get(1))).eContainer() instanceof Merge)){
					removeActivity(assign);
				}
			}
		}
	}

	private void removeActivity (Activity empty) {
		//add assertion for gateway
		//20090521		
		reduceOrphanLinks(empty);
		if (!BpelOptimizationUtil.getNonOptimizableElements(getContext()).contains(empty)) {
			/**
			 * There are 5 cases we want to remove the empty activity
			 * 1. it has no incoming or outgoing link
			 * 2. it has no incoming link but it has one or more outgoing link
			 * 3. it has no outgoing link but it has one or more incoming link
			 * 4. it has a single incoming link which has no transitionCondition
			 * 5. it has a single outgoing link which has no transitionCondition
			 */			
			//if (empty.getSources().isEmpty() && empty.getTargets().isEmpty()) {
			if (!isSourceOfLink(empty) && !isTargetOfLink(empty)) {
				replaceElement(empty,null);
				//} else if (empty.getSources().isEmpty() && !empty.getTargets().isEmpty()) {
			} else if (!isSourceOfLink(empty) && isTargetOfLink(empty)) {
				//List targets = empty.getTargets();
				Targets bpelTargets = empty.getTargets();
				List targets = bpelTargets.getChildren();
				Target tar;
				Link link;
				//TransitionCondition tc=null;
				Condition tc=null;
				boolean bTransitionConditionExist=false;
				for (int i=targets.size()-1; i>=0; i--) {
					tar = (Target)targets.get(i);
					link = tar.getLink();
					tc=BpelOptimizationUtil.getTransitionCondition(link);
					bTransitionConditionExist = bTransitionConditionExist | !(tc==null);
					if (tc==null) {
						if (!link.getSources().isEmpty()) { // it should always have target (unless the transformation has error)
							Source src = (Source)link.getSources().get(0);
							// remove link and update join condition
							BpelOptimizationUtil.updateJoinCondition(link);
							//src.getActivity().getSources().remove(src);
							Activity act = src.getActivity();
							act.getSources().getChildren().remove(src);
							if(act.getSources().getChildren().isEmpty()){
								act.setSources(null);
							}
						}
						Links links = (Links)link.eContainer();
						replaceElement(link,null);
						if (links.getChildren().isEmpty()) {
							replaceElement(links,null);
						}
						targets.remove(tar);
					}
				}
				if (!bTransitionConditionExist) {
					replaceElement(empty,null);			
				}

				if(bpelTargets.getChildren().isEmpty())
					empty.setTargets(null);
				//	} else if (!empty.getSources().isEmpty() && empty.getTargets().isEmpty()) {
			} else if (isSourceOfLink(empty) && !isTargetOfLink(empty)) {
				//List sources = empty.getSources();
				Sources bpelSources = empty.getSources();
				List sources = bpelSources.getChildren();
				Source src;
				Link link;
				boolean bTransitionConditionExist=false;
				Condition tc=null;
				for (int i=sources.size()-1; i>=0; i--) {
					src = (Source)sources.get(i);
					link = src.getLink();
					tc = BpelOptimizationUtil.getTransitionCondition(link);
					bTransitionConditionExist = bTransitionConditionExist | !(tc==null); 
					if (tc==null) {
						if (!link.getTargets().isEmpty()) { // it should always have target (unless the transformation has error)
							Target tar = (Target)link.getTargets().get(0);
							// remove link and update join condition
							BpelOptimizationUtil.updateJoinCondition(link);
							//tar.getActivity().getTargets().remove(tar);
							Activity act = tar.getActivity();
							act.getTargets().getChildren().remove(tar);
							if(act.getTargets().getChildren().isEmpty()){
								act.setTargets(null);
							}
						}
						Links links = (Links)link.eContainer();
						replaceElement(link,null);
						if (links.getChildren().isEmpty()) {
							replaceElement(links,null);
						}
						sources.remove(src);

					}
				}
				if (!bTransitionConditionExist) {
					replaceElement(empty,null);
				}
				if(bpelSources.getChildren().isEmpty())
					empty.setSources(null);
				//	} else if (empty.getTargets().size()==1 && BpelOptimizationUtil.getTransitionCondition(((Target)empty.getTargets().get(0)).getLink())==null) {
			} else if (empty.getTargets()!=null && empty.getTargets().getChildren().size()==1 && BpelOptimizationUtil.getTransitionCondition(((Target)empty.getTargets().getChildren().get(0)).getLink())==null) {
				//Target tar = (Target)empty.getTargets().get(0);
				//if the source refers to a fault link, we need to keep it so that it can be propagated to the parent flow links			
				Target tar = (Target)empty.getTargets().getChildren().get(0); 
				if(!CBPELUtil.shouldKeepTargetForGateway(empty,tar, this, getContext())){
					boolean isFaultSource = false;
					Link link = tar.getLink();
					//TransitionCondition tc = BpelOptimizationUtil.getTransitionCondition(link);
					//if (tc==null) {

					if (!link.getSources().isEmpty()) {
						Source src = (Source)link.getSources().get(0); // assume single source and single target of a link
						Activity srcAct = src.getActivity();
						//srcAct.getSources().addAll(empty.getSources());

						//check if the link is a fault link
						isFaultSource = CycleLinkUtil.isFaultSourceLink(getContext(), link);

						//consolidate the link that in parallel with the fault link
						CycleLinkUtil.consolidateLinkInParallelWithFaultLink(empty, link,srcAct, isFaultSource);

						if(!isFaultSource){
							srcAct.getSources().getChildren().addAll(empty.getSources().getChildren());
						}

						// cleanup
						//srcAct.getSources().remove(src);						


						srcAct.getSources().getChildren().remove(src);					

						//cleanup empty Sources.
						if(srcAct.getSources().getChildren().isEmpty()){
							srcAct.setSources(null);
						}
						// Cleanup if there is parallel links that have the same source and target
						if(srcAct.getSources() != null){
							BpelOptimizationUtil.fixParallelLinksForSourceActivity(srcAct);
						}
					}
					Links links = (Links)link.eContainer();
					if(!isFaultSource){
						replaceElement(link,null);
					}			
					if (links.getChildren().isEmpty()) {
						replaceElement(links,null);
					}
					replaceElement(empty,null);
					//}
					//} else if (empty.getSources().size()==1) {
				}else{
					if(keptActivityForGateway!= null && empty instanceof Empty && !keptActivityForGateway.contains(empty)){
						keptActivityForGateway.add(empty);
					}
				}
			} else if (empty.getSources()!=null && empty.getSources().getChildren().size()==1) {
				//Source src = (Source)empty.getSources().get(0);
				Source src = (Source)empty.getSources().getChildren().get(0);
				if(!CBPELUtil.shouldKeepSourceForGateway(empty,src)){
					Link link = src.getLink();
					//TransitionCondition tc = BpelOptimizationUtil.getTransitionCondition(link);
					Condition tc = BpelOptimizationUtil.getTransitionCondition(link);
					if (tc==null) {
						if (!link.getTargets().isEmpty()) {
							Target tar = (Target)link.getTargets().get(0);
							Activity tarAct = tar.getActivity();
							//List tars = empty.getTargets();
							List tars = empty.getTargets().getChildren();
							for (Iterator it=tars.iterator();it.hasNext();) {
								recordLinkToBePropagated(((Target)it.next()).getLink());
							}
							//tarAct.getTargets().getChildren().addAll(tars);
							Condition jc = BpelOptimizationUtil.getJoinCondition(empty);
							if (jc != null) {
								//	JoinCondition tarJc = BpelOptimizationUtil.getJoinCondition(tarAct);
								Condition tarJc = BpelOptimizationUtil.getJoinCondition(tarAct);
								if (tarJc == null) {
									//	tarAct.getEExtensibilityElements().add(jc);
									tarAct.getTargets().setJoinCondition(jc);
								} else {
									String toBeRemovedLinkStatus = BpelOptimizationUtil.buildGetLinkStatusJavaSnippet(link);
									//TODO migration ?? 
									//String substituteBody = jc.getJavaCode().getValue();
									//								String substituteBody = jc.getJavaCode();
									//								substituteBody = StringUtil.replaceAll(substituteBody,"return","");
									//								substituteBody = StringUtil.replaceAll(substituteBody,";","");							
									//								//String newBody = tarJc.getJavaCode().getValue();
									//								String newBody = tarJc.getJavaCode();
									//								newBody = StringUtil.replaceAll(newBody,toBeRemovedLinkStatus,substituteBody);
									//								//tarJc.getJavaCode().setValue(newBody);
									//								tarJc.setJavaCode(newBody);

									String substituteBody = (String)jc.getBody();
									//feature 109954
									//substituteBody = StringUtil.replaceAll(substituteBody,"return","");
									//substituteBody = StringUtil.replaceAll(substituteBody,";","");	
									substituteBody = StringHelper.replaceAll(substituteBody,"return","");
									substituteBody = StringHelper.replaceAll(substituteBody,";","");							
									//String newBody = tarJc.getJavaCode().getValue();
									String newBody = (String)tarJc.getBody();
									//	newBody = StringUtil.replaceAll(newBody,toBeRemovedLinkStatus,substituteBody);
									newBody = StringHelper.replaceAll(newBody,toBeRemovedLinkStatus,substituteBody);
									//tarJc.getJavaCode().setValue(newBody);
									tarJc.setBody(newBody);
								}
							} else {
								Condition tarJc = BpelOptimizationUtil.getJoinCondition(tarAct);
								if (tarJc!=null) {
									if (empty.getTargets() != null && empty.getTargets().getChildren()!=null && empty.getTargets().getChildren().size()>0) {
										String toBeRemovedLinkStatus = BpelOptimizationUtil.buildGetLinkStatusJavaSnippet(link);
										String tarJcBody = (String)tarJc.getBody();
										String substituteBody=null;
										List emptyTargets = empty.getTargets().getChildren();
										for (Iterator it=emptyTargets.iterator();it.hasNext();) {
											Target target = (Target)it.next();
											Link lnk = target.getLink();
											// Since the empty has no joinCondition, then by default the joinCondition is 'OR' of all linkStatus
											if (substituteBody==null) {
												substituteBody = BpelOptimizationUtil.buildGetLinkStatusJavaSnippet(lnk);
											} else {
												substituteBody = substituteBody+"||"+BpelOptimizationUtil.buildGetLinkStatusJavaSnippet(lnk);											
											}
										}
										if (substituteBody!=null) {
											tarJcBody = StringHelper.replaceAll(tarJcBody,toBeRemovedLinkStatus,substituteBody);
											tarJc.setBody(tarJcBody);																											
										}
									} else {
										BpelOptimizationUtil.updateJoinCondition(link);
									}
								}
							}
							// cleanup
							//tarAct.getTargets().remove(tar);
							tarAct.getTargets().getChildren().addAll(tars);
							tarAct.getTargets().getChildren().remove(tar);
							//cleanup if empty Targets
							if(tarAct.getTargets().getChildren().isEmpty()){
								tarAct.setTargets(null);
							}
							// Cleanup if there is parallel links that have the same source and target
							BpelOptimizationUtil.fixParallelLinksForTargetActivity(tarAct);
						}
						Links links = (Links)link.eContainer();
						replaceElement(link,null);
						if (links.getChildren().isEmpty()) {
							replaceElement(links,null);
						}
						replaceElement(empty,null);
					}			
				}else{
					if(keptActivityForGateway!= null && empty instanceof Empty && !keptActivityForGateway.contains(empty)){
						keptActivityForGateway.add(empty);
					}
				}
				// TODO: temp fix for empty target/source problem
				//			if (empty.eContainer()!=null) {
				//				if(empty.getSources() !=null && empty.getSources().getChildren()!=null && empty.getSources().getChildren().isEmpty()){
				//					empty.setSources(null);
				//				}
				//				if(empty.getTargets() !=null && empty.getTargets().getChildren()!=null && empty.getTargets().getChildren().isEmpty()){
				//					empty.setTargets(null);
				//				}
				//			}
			}
		}
		//This is to defend the case for subprocess cycle link, where an empty is registered as non-optimized element
		if (!isSourceOfLink(empty) && !isTargetOfLink(empty)) {
			replaceElement(empty,null);
		}
		//Fix problem for empty sources or targets 
		if (empty.eContainer()!=null) {
			if(empty.getSources() !=null && empty.getSources().getChildren()!=null && empty.getSources().getChildren().isEmpty()){
				empty.setSources(null);
			}
			if(empty.getTargets() !=null && empty.getTargets().getChildren()!=null && empty.getTargets().getChildren().isEmpty()){
				empty.setTargets(null);
			}
		}
	}

	private void propagateLinksToParent(List links) {
		Link link;
		Flow origOwnerFlow,pflow;
		for (Iterator it=links.iterator();it.hasNext();) {
			link = (Link) it.next();
			if (link.eContainer()!=null) {
				origOwnerFlow=(Flow)link.eContainer().eContainer();
				pflow = getParentFlow(origOwnerFlow);
				if (AbstractbpelUtil.isLinkToExternalActivity(link,origOwnerFlow) && pflow!=null) {
					Links plinks = pflow.getLinks();
					if (plinks==null) {
						plinks = BPELFactory.eINSTANCE.createLinks();
						pflow.setLinks(plinks);
					}
					plinks.getChildren().add(link);
				}
			}
		}
	}

	private void recordLinkToBePropagated(Link link) {
		if (linksToBePropagated == null) {
			linksToBePropagated = new ArrayList();
		}
		linksToBePropagated.add(link);
	}

	private Flow getParentFlow(Activity act) {
		EObject container = act.eContainer();
		while (container!=null && !(container instanceof Flow)) {
			container = container.eContainer();
		}
		return (Flow)container;
	}
	/**
	 * Obtain the transition condition from the source of the link
	 * @param link
	 * @return
	 */
	/*	
	private TransitionCondition getTransitionCondition(Link link) {
		List extensibilityElements = ((Source)link.getSources().get(0)).getExtensibilityElements();
		Iterator it = extensibilityElements.iterator();
		Object extElement;
		while (it.hasNext()) {
			extElement = it.next();
			if (extElement instanceof TransitionCondition) {
				return (TransitionCondition)extElement;
			}
		}
		return null;
	}

	private String buildGetLinkStatusJavaSnippet(Link link) {
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getLinkStatus");
		def.setFunctionName("bpws:getLinkStatus");

		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr1.setStringSymbol(link.getName());
		arg1.setArgumentValue(strExpr1);

		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		JavaExpressionSerializer javaExprSerializer = new JavaExpressionSerializer();
		return javaExprSerializer.serialize(functionExpr);
	}

	private JoinCondition getJoinCondition(Activity targetActivity) {

		if (targetActivity != null) {
			List exElements = targetActivity.getExtensibilityElements();
			// find the JoinCondition extensibility element.  There should only be one 
			for (int i=0; i<exElements.size(); i++) {
				if (exElements.get(i) instanceof JoinCondition) {
					return (JoinCondition)exElements.get(i);
				}
			}
		}
		return null;
	}
	 */
	/**
	 * Upon removal of the link, we need to remove the getLinkStatus for the link in the JoinCondition
	 * @param linkToBeRemoved
	 */
	/*
	private void updateJoinCondition(Link linkToBeRemoved) {
		Activity targetActivity = ((Target)linkToBeRemoved.getTargets().get(0)).getActivity();
		if (targetActivity != null) {
//			List exElements = targetActivity.getExtensibilityElements();
//			// find the JoinCondition extensibility element.  There should only be one 
//			JoinCondition joinCondition=null;
//			for (int i=0; i<exElements.size() && joinCondition==null; i++) {
//				if (exElements.get(i) instanceof JoinCondition) {
//					joinCondition = (JoinCondition)exElements.get(i);
//				}
//			}
			JoinCondition joinCondition = getJoinCondition(targetActivity);
			if (joinCondition != null) {
				List targets = targetActivity.getTargets();
				if (targets.size()==2) {
					// if there are 2 targets, then there will only be one left when the link
					// is removed.  That means we don't need any JoinCondition (i.e. only 1 link goes
					// into this activity)
					List exElements = targetActivity.getExtensibilityElements();
					exElements.remove(joinCondition);
				} else {
					// Clean up the getLinkStatus snippet of the link from the JoinCondition
					String body = joinCondition.getJavaCode().getValue();
					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(linkToBeRemoved);
					body = StringUtil.replaceAll(body,"&& ("+getLinkStatusSnippet+")","");
					body = StringUtil.replaceAll(body,"|| ("+getLinkStatusSnippet+")","");
					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") &&","");
					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") ||","");
					joinCondition.getJavaCode().setValue(body);
				}
			}
		}
	}
	 */
	private void replaceElement(EObject origElement, EObject newElement) {
		if (origElement.eContainer() != null) {
			if (origElement.eContainer().eGet(origElement.eContainmentFeature()) instanceof List){
				List list = (List)origElement.eContainer().eGet(origElement.eContainmentFeature());
				if (newElement != null) {
					int index = list.indexOf(origElement);
					list.add(index, newElement);
					/* The altActivity has shifted up one in the list -- remove it from its new position */
					list.remove(index + 1);
				} else {
					list.remove(origElement);
				}
			} else {
				/* If the container isn't a list, just set the new containment value directly */
				origElement.eContainer().eSet(origElement.eContainmentFeature(), newElement);
			}
		}		
	}
	private void flattenChildFlow(Flow parentFlow){
		Activity childActivity = null;

		/* Working from the front of the list means that we will replace all nested flows in one shot, since the 
		 * activities that replace a child flow will then be searched for flows themselves 	 */
		for (int i=0; i<parentFlow.getActivities().size(); i++){
			childActivity = (Activity)parentFlow.getActivities().get(i);
			if (childActivity instanceof Flow){
				parentFlow.getActivities().remove(childActivity);
				parentFlow.getActivities().addAll(i, ((Flow)childActivity).getActivities());
				if (parentFlow.getLinks() != null && ((Flow)childActivity).getLinks() != null) {				
					parentFlow.getLinks().getChildren().addAll(((Flow)childActivity).getLinks().getChildren());
				}
				/* Re-check the current position in the list, since we just inserted a new element */
				i--;
			}
		}			
	}	

	/**
	 * Creates a BPELP DisplayName and set the text value to <code>name</code>
	 * @param bpelAct a BPELP Activity
	 * @param name the DisplayName value.
	 */
	private void createDisplayName(com.ibm.wbit.bpel.Activity bpelAct, String name) {
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		bpelAct.getEExtensibilityElements().add(dspName);
		dspName.setText(name);		
	}

	/**
	 * Return a list of collected mappings between OutputPinSet of a decision and the case activity (Flow or Empty)
	 * @return
	 */
	public HashMap getCaseActivityMappings() {
		return caseActivityMappings;
	}

	/**
	 * Return a list of collected mappings between Input/OutputPinSet of a Process/Task and the onMessage activity (Flow or Empty)
	 * @return
	 */
	public HashMap getOnMessageActivityMappings() {
		return onMessageActivityMappings;
	}
} //BpelOptimizationRuleImpl