/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Parameter;
import com.ibm.btools.bom.model.processes.actions.CallOperationAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.bom.model.services.BehavioredClass;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallOperationActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * Triggers the creation of a BPEL invoke that maps to the BOM CallOperationAction
 * The invoke will be referencing to the WSDL operation of the remote Process
 * which is defined in the Remote Process' WSDL definition.  If such
 * WSDL definition is not found, a new one will be created.
 * 
 */
public class CallOperationActionRuleImpl extends ActionRuleImpl implements CallOperationActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private boolean ivFirstTimeExec = true;
	private BPELVariable ivInputContainer = null;
	private BPELVariable ivOutputContainer = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected CallOperationActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.CALL_OPERATION_ACTION_RULE;
	}

	/**
	 * Source: BOM InputSet
	 * Target: BPEL Invoke or BPEL Sequence containing a BPEL Assign and a BPEL Invoke
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()  
	 */
	public boolean transformSource2Target(){
		//	Executes as follows:
		//	Retrieves the BOM Operation the CallOperationAction (to which the source inputSet) 
		//	belongs. 
		//	Creates the WSDL Definition for the Interface the Operation belongs to. 
		// 		-creates the ProcessInterfaceRule for the Interface namespace
		// 	Retrieves the WSDL Operation corresponding to the referenced BOM Operation 
		// 	based on the signature. 
		// 	Creates the Invoke - from an InvokeRule
		
			//assertion
			if (getSource() == null || getSource().isEmpty())
				 return false;

			InputPinSet inputSet = (InputPinSet)getSource().get(0);
			//check if this is first time execute this rule
			if(!ivFirstTimeExec){
				reExecute(inputSet);
				return true;		  
			}
 
			ivFirstTimeExec = false;
			
			CallOperationAction coa = (CallOperationAction)inputSet.getAction();
			/* Check the attributes that are not supported by the TE */
			if (coa.getIsSynchronous() != null && coa.getIsSynchronous().booleanValue() == false){
					LoggingUtil.logWarning(MessageKeys.ASYNCH_COA_NOT_SUPPORTED);		
			}
			InputObjectPin targetPin = coa.getTarget();
			if(targetPin == null){
				BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.MISSING_TARGET_FOR_COA));
				LoggingUtil.logError(MessageKeys.MISSING_TARGET_FOR_COA);
				throw e;
			}

			List resultList = mapInputPinSet(inputSet);
			ProcessInterfaceRule myProcIntRule = createProcessInterfaceRule(coa);
			ProcessDefinitionRule myProcDefRule = (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext());
			com.ibm.btools.bom.model.services.Operation operation = coa.getOperation(); 
			//the operation refereced by the CallOperationAction belongs to an services.Interface 
			//!! TODO !! check if model changed 
			ProcessInterfaceRule processIntRule = null;
			TransformationRule portTypeRule = null;
			Behavior behavior = findReferencedBehaviorForOperation(operation, (BehavioredClass)coa.getTarget().getType());
			if(behavior instanceof com.ibm.btools.bom.model.processes.activities.Activity){
				Action action = ((com.ibm.btools.bom.model.processes.activities.Activity)behavior).getImplementation();
				processIntRule = createProcessInterfaceRule(action);
				portTypeRule = createPortTypeRule(processIntRule, action);	 	
			}else if(behavior instanceof com.ibm.btools.bom.model.services.Activity){
				processIntRule = createProcessInterfaceRule(behavior);
				portTypeRule = createPortTypeRule(processIntRule, behavior);
			}
			NamedElement set = getCorrespondingSet((NamedElement)inputSet);
			Operation op = findOperation(processIntRule, set);
			createPartnerLinkRule(myProcIntRule, operation, (PortType)portTypeRule.getTarget().get(0));
			PartnerLink partnerLink =(PartnerLink) createPartnerRule(myProcDefRule, operation).getTarget().get(0);
	
		//first create a sequence and inside add an assign and a invoke
		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
	
		doInputPinSetConnection(sequence);
	
		getTarget().add(sequence);
		
		Activity assignActivity = createAssignActivity(coa, inputSet, targetPin, partnerLink);
		
		sequence.getActivities().add(assignActivity);

	//	Assign assignActivity = createAssign(coa, inputSet, targetPin, partnerLink);
	//	sequence.getActivities().add(assignActivity);
		
		sequence.getActivities().add(createInvoke(inputSet, coa, op, partnerLink));
	
		createBranchRuleIfAny();
			
		mapOutputPinSet((OutputPinSet)inputSet.getOutputPinSet().get(0));
	
		doOutputPinSetConnection(sequence);
	
		// propagate all child target
		propagateChildTargets();

		setComplete(true);
		return isComplete();
		
	}
	/**
	 * Find the message representing the container and then find the part by the index of the 
	 * object pin.  It assumes the parts are in the same order as the object pins in a set
	 * @param set
	 * @param pin a BOM NamedElement, a BOM Pin or a BOM Parameter
	 * @param con
	 * @return
	 */		
	private Part findPart(NamedElement set, NamedElement pin, BPELVariable con){
	
		// Get the WSDL message from container
		Message msg = con.getMessageType();
		// find the pin index in the set
		List pins = getPinsFromSet(set);
		int ind = pins.indexOf(pin);
		// find the corresponding part for the pin
		Part part = (Part)msg.getOrderedParts(null).get(ind);		
		return part;
	}
	
	/**
	 * Returns the list of Pins or Parameters in the <code>set<set>. 
	 * @param set
	 * @return
	 */	
	private List getPinsFromSet(NamedElement set){
		List pins;
		if (set instanceof InputPinSet)
			pins = ((InputPinSet)set).getInputObjectPin();
		else if(set instanceof OutputPinSet)
			pins = ((OutputPinSet)set).getOutputObjectPin();
		else if (set instanceof InputParameterSet)
			pins = ((InputParameterSet)set).getParameter();
		else 
			pins = ((OutputParameterSet)set).getParameter();
		return pins;		
	}

	/**
	 * For each pin in the input set, get the link created by parent rule, 
	 * and set <code>activity</code> as the target of that link.
	 * Updates the join codition of the <code>activity</code> to depend on all links coming into the input set. 
	 * @param activity  BPEL Sequence
	 */	
	private void doInputPinSetConnection(Sequence activity){
	   
		InputPinSet inputSet = (InputPinSet)getSource().get(0);

		 /* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(inputSet);
		Activity inputSetActivity = null;
		if (isResultList.isEmpty() || !(isResultList.get(0) instanceof Activity)){
		   inputSetActivity = activity;
		} else {
		   inputSetActivity = (Activity)isResultList.get(0);
						   /* Link the new input set activity to the sequence */
		   Link link = linkActivities(inputSetActivity, 
								  activity, 
							  BomUtils.getCanonicalName(inputSet) + "-to-" + inputSet.getAction().getName());
			isResultList.add(link);
				
		 /* Propagate results from the input set mapping */
		  	getTarget().addAll(isResultList);
		}
		setTargetOfLinks(inputSet, inputSetActivity);	
	    
	}

	/**
	 * For each output pin in the Output Set correlated with the InputPinSet,
	 * finds the link created by the parent rule, sets the <code>activity</code> as the source of the link.
	 * @param invoke  BPEL Activity, can be either a Sequence or an Invoke
	 */
	private void doOutputPinSetConnection(Sequence activity){
	
		List osList = ((InputPinSet)getSource().get(0)).getOutputPinSet();
		
		OutputPinSet nextOS;		
		Pin pin;
		//although we don't support multiple outputs correlated to one input, here implement for multiple outputs
		for(int i=0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet) osList.get(i);				

			List osResultList = mapOutputPinSet(nextOS);
			Activity outputSetActivity = null;
			if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity)){
				outputSetActivity = activity;
			} else {
				outputSetActivity = (Activity)osResultList.get(0);
				
				/* Link the new input set activity to the sequence */
				Link link = linkActivities(activity, 
										   outputSetActivity, 
										   nextOS.getAction().getName() + "-to-" + BomUtils.getCanonicalName(nextOS));
				
				osResultList.add(link);
				
				/* Propagate the results from the input set mapping */
				getTarget().addAll(osResultList);
			}

			setSourceOfLinks(nextOS, outputSetActivity);
		
		}		
	}		
	/**
	 * Creates a BPEL Invoke. 
	 * @param inputSet
	 * @param action
	 * @param operation
	 * @param partner
	 * @return
	 */	
	private Invoke createInvoke(InputPinSet inputSet, CallOperationAction action, Operation operation , PartnerLink partner){
		
		createContainerRule(action);
	   	OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0); //assume only one correlated output set  
 
    	PortType portType = (PortType)operation.eContainer();
 		//create BPEL invoke
		Invoke invoke = BPELFactory.eINSTANCE.createInvoke();
		invoke.setName(BomUtils.getCanonicalName(inputSet));
		invoke.setInputVariable(ivInputContainer);
		invoke.setOutputVariable(ivOutputContainer);
		invoke.setOperation(operation);
		invoke.setPartnerLink(partner);
		invoke.setPortType(portType);
		return invoke;	   

	}

	/**
	 * Creates container rules for the input set of the CallOperationAction, 
	 * the InputPinSet and corresponding OutputPinSet of the referenced
	 * Behaviored. Will register the pins on the CallOperationAction OutputPinSet 
	 * with the container of the referenced Operation OutputParameterSet.
	 * @param coa the CallOperationAction
	 */
	private void createContainerRule(CallOperationAction coa){
		  InputPinSet inputSet = (InputPinSet)getSource().get(0);
		 
		 //first create container rule for input set
		  ContainerRule cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
		  cr.getSource().add(inputSet);
		  getChildRules().add(cr);
		  cr.transformSource2Target();
		 //create the container rule for the input parameter set on the referenced Operation	  
		  cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
		 
		  NamedElement set = getCorrespondingSet(inputSet);
		  if(set instanceof InputPinSet){
		  	InputPinSet ips =(InputPinSet)set;
			cr.getSource().add(ips);
			getChildRules().add(cr);
			cr.transformSource2Target();
			ivInputContainer = (BPELVariable)cr.getTarget().get(0);
		  	
		  }else if(set instanceof InputParameterSet){
		  	InputParameterSet ips = (InputParameterSet)set;
			cr.getSource().add(ips);
			getChildRules().add(cr);
			cr.transformSource2Target();
			ivInputContainer = (BPELVariable)cr.getTarget().get(0);
		  }
   		  //although we only support one-one correlation of input set and output set, 
		  // here implement correlation to multiple output set
		  List osList = null;
		  if(set instanceof InputParameterSet){
		  	osList = ((InputParameterSet)set).getOutputParameterSet();
		  }else if(set instanceof InputPinSet){
		  	osList = ((InputPinSet)set).getOutputPinSet();
		  }
		 
		  if(osList.size() > 1){
			//log it and continue
			//LogHelper.log(3,null, "com.ibm.btolls.te.heuristics.resources.Messages",":");
		  }	  
		  //create the container rule for the output ParameterSet of the referenced Operation
		 // OutputParameterSet nextOS = (OutputParameterSet)osList.get(0);
		  NamedElement nextOS = (NamedElement)osList.get(0);
		  cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
		  cr.getSource().add(nextOS);
		  getChildRules().add(cr);
		  cr.transformSource2Target();
		  ivOutputContainer = (BPELVariable)cr.getTarget().get(0);
		  // register the OutputPinSet on the CallOperationAction with the OutputParameterSet container
		  BomHelper.getInstance().reuseContainer(cr, (PinSet)coa.getOutputPinSet().get(0));
		  		  
	}
	/**
	 * Creates ExclusiveBranchRule or ConcurrentBranchRule. 
	 * The branch rule will map all elements in the branch region directly following the invoked Task or CallBehaviourAction.
	 */
	private void createBranchRuleIfAny(){
		/* 
		 * First check if the InputPinSet is an exclusive branch, 
		 * if true, created ExclusiveBranchRule,
		 * if not, check if the correlated OutputPinSet is an concurrent branch, 
		 * 	if true, create a ConcurrentBranchRule.
		 */  
		TransformationRule rule;
		InputPinSet inputSet = ((InputPinSet)getSource().get(0));
		//check if input set is exclusive branch
		if(BomHelper.getInstance().isEBranch(inputSet)){
			rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
			rule.getSource().add(inputSet);
			getChildRules().add(rule);
			rule.transformSource2Target();
			
		} else { //check if output is Concurrent branch
			
			//is it safe to assume there is only one output set since it is not e-branch?
			OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
			if(BomHelper.getInstance().isCBranch(outputSet)){
				rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
				rule.getSource().add(outputSet);
				getChildRules().add(rule);	
				rule.transformSource2Target();				 
			}else{
				//do nothing
			}
		}		
	}

		/**
		 * Creates a BPEL Assign that will copy all the parts in the CallOperationAction InputSet to the referenced
		 * Operation InputParameterSet parts. The "target" Pin of the CallOperationAction will be copied to the 
		 * <code>partnerLink</code>.
		 * @param coa BOM CallOperationAction
		 * @param inputSet CallOperationAction inputSet
		 * @param target  CallOperationAction inputPin target
		 * @param partnerLink for the 
		 * @return assign - BPEL Assign
		 */	
		private Assign createAssign(CallOperationAction coa, InputPinSet inputSet, InputObjectPin target, PartnerLink partnerLink){
			
			//now create the assign
			Assign assign = BPELFactory.eINSTANCE.createAssign();
			
			BPELVariable srcCon = createContainer(inputSet);
			//assume as per Jim, one inputParameterSet
			NamedElement pinSet = getCorrespondingSet((NamedElement)inputSet);
			//InputParameterSet inputParameterSet = (InputParameterSet)coa.getOperation().getInputParameterSet().get(0);
			//Variable targetCon = createContainer(inputParameterSet);
			BPELVariable targetCon = createContainer(pinSet);
			
			Part srcPart;
			Copy copy;
			From from;
			To to;
			List set = coa.getInputObjectPin();
			for(int i=0; i<set.size(); i++){
				copy = BPELFactory.eINSTANCE.createCopy();
				from= BPELFactory.eINSTANCE.createFrom();
				to = BPELFactory.eINSTANCE.createTo();
				InputObjectPin iop = (InputObjectPin)set.get(i);
				srcPart = findPart(inputSet, iop, srcCon);
				from.setVariable(srcCon);
				from.setPart(srcPart);
				copy.setFrom(from);
				if(iop == target){
					to.setPartnerLink(partnerLink);					
				}else{
					to.setVariable(targetCon);
					Part trgPart;
					if(pinSet instanceof InputPinSet){
						trgPart = findPart(pinSet, (InputObjectPin)((InputPinSet)pinSet).getInputObjectPin().get(i), targetCon);
					//this should be an InputParameterSet
					}else{
						InputParameterSet inputParameterSet = (InputParameterSet)pinSet;
						trgPart = findPart(inputParameterSet, (Parameter)inputParameterSet.getParameter().get(i), targetCon);
					}
					to.setPart(trgPart);
				}
				copy.setTo(to);
				assign.getCopy().add(copy);
			}
			return assign;
		}


	/**
	 * Creates a BPEL Assign that will copy all the parts in the CallOperationAction InputSet to the referenced
	 * Operation InputParameterSet parts. The "target" Pin of the CallOperationAction will be copied to the 
	 * <code>partnerLink</code>.
	 * @param coa BOM CallOperationAction
	 * @param inputSet CallOperationAction inputSet
	 * @param target  CallOperationAction inputPin target
	 * @param partnerLink for the 
	 * @return assign - BPEL Assign
	 */	
	private Invoke createJavaInvoke(CallOperationAction coa, InputPinSet inputSet, InputObjectPin target, PartnerLink partnerLink){
		
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaString = new String();
		
		BPELVariable srcVar = createContainer(inputSet);
		//assume as per Jim, one inputParameterSet
		NamedElement pinSet = getCorrespondingSet((NamedElement)inputSet);
		BPELVariable targetVar = createContainer(pinSet);
		InputObjectPin targetPin = coa.getTarget();
		String sourceVariableName = AbstractbpelUtil.capitalizeFirstChar(srcVar.getName());
		String targetVariableName = AbstractbpelUtil.capitalizeFirstChar(targetVar.getName());
		for(Iterator it=inputSet.getInputObjectPin().iterator(); it.hasNext();){
			InputObjectPin iop = (InputObjectPin)it.next();
			Part srcPart = findPart(inputSet, iop, srcVar);
			String srcPartName = AbstractbpelUtil.capitalizeFirstChar(srcPart.getName());
			if(!iop.equals(targetPin)){	
				Part trgPart = findPart(inputSet, iop, targetVar);
				String trgPartName = AbstractbpelUtil.capitalizeFirstChar(trgPart.getName());
				javaString +=AbstractbpelUtil.generateJavaGetterForVariable(targetVariableName) +"(true)"
					+ ".set" + trgPartName + "("+AbstractbpelUtil.generateJavaGetterForVariable(sourceVariableName) +"().get" + srcPartName+ "()):\n";
			}else{
				String partnerLinkName = AbstractbpelUtil.capitalizeFirstChar(partnerLink.getName());
				//now create the invoke for the partnerLink
				javaString +="setPartnerLink" + partnerLinkName + "("  + AbstractbpelUtil.generateJavaGetterForVariable(sourceVariableName) +"().get" + srcPartName+ "()):\n";
				
			}
		}	
		//TODO migration????
//		javaCode.setValue(javaString);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaString);
		javaInvoke.getExtensibilityElements().add(jsActivity);
		return javaInvoke;
	}
		
	private Activity createAssignActivity(CallOperationAction coa, InputPinSet inputSet, InputObjectPin targetPin, PartnerLink partnerLink){
		
		Activity assignActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();		
		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
			
		((AlternativeActivity)assignActivity).getAlternative().add(bpelAlt);
		((AlternativeActivity)assignActivity).getAlternative().add(bpelpAlt);
			
		/* BPEL: Create <assign>, with multiple BPEL Copies, one for each input value pin value*/
		bpelAlt.setActivity(createAssign(coa, inputSet, targetPin, partnerLink));	
		/* BPEL+: Create <invoke><wpc:script><wpc:javaCode>... */
		bpelpAlt.setActivity(createJavaInvoke(coa, inputSet, targetPin, partnerLink));
		
		return assignActivity;
			
	}

} //CallOperationActionRuleImpl
