/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.FlowFinalNode;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Variable;

/**
 * <!-- begin-user-doc -->
 * Source: BOM OutputPinSet
 * Target: BPEL Flow 
 * 
 * Maps all BOM elements in the concurrent region excluding the concurrent merge point 
 * and includes mapped activities in the taget BPEL flow. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ConcurrentBranchRuleImpl extends ProcessWalkingRuleImpl implements ConcurrentBranchRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* attribute indicates whether it is first time execution of this rule */
	private boolean firstTimeExec = true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ConcurrentBranchRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.CONCURRENT_BRANCH_RULE;
	}

	/**
	 * Transforms elements in the concurrent region to BPEL targets.
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target_bak() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/* 
		 * tranformation includes following steps:
		 * 1. create flow
		 * 2. create links for the concurrent branch point output set.
		 * 3. for each input set reachable by the branch point input set, 
		 *    step through it, create child rules and execute the rule, 
		 *    until reached the merge point or reached end of Structured Activity Node.
		 * 4. consume and propagate targets produced by child rules.  
		 */	
		 		
		 //assertion
		 if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;		 	
		 }
		 //first source of this rule is one output set
		 OutputPinSet outputSet = (OutputPinSet)getSource().get(0);
  	
		 //check if this is first time execute this rule
		  if(!firstTimeExec){
			 reExecute(null); 
			 LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			 return true;
		  }	   
		firstTimeExec = false;	   
	
		//create BPEL flow
		Flow flow = BPELFactory.eINSTANCE.createFlow();
		getTarget().add(flow);	
		
		createLinksForOutputPinSet(outputSet);
		
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		//find the concurrent merge point
		NamedElement mergePoint = BomHelper.getInstance().findConcurrentMergePoint(outputSet);
		Action mergePointAction = null;
		if(mergePoint instanceof InputPinSet)
			mergePointAction = ((InputPinSet)mergePoint).getAction();
		else if(mergePoint instanceof StructuredActivityNode)
			mergePointAction = (Action)mergePoint;
					
		//!!main logic of this method is here
		  InputPinSet nextIS;
		  //get a list of input set reachable from the output set. since output set is concurrent branch point, this should never step out of the branch region.
		  List inputSetList = bw.getNext(outputSet, BomWalker.INPUT_SET, false);			  
		  if(inputSetList == null || inputSetList.isEmpty()){
			//we have reached end of SAN ??? what to do??
			//need do the connection for the output set of SAN to internal activities of SAN? 
			//No, because the process rule will take case of doing output set connection with internal and external
		  }else{
			int count = inputSetList.size();
	
		  	for (int i=0; i<count; i++){ 
			    nextIS = (InputPinSet)inputSetList.get(i);
				if(nextIS.getAction() != mergePointAction){				
				    do{									 
						nextIS = stepThroughBOM(nextIS);
						if(nextIS != null){ //means we just created a child rule
						  //we must continue
						  //bw.getNext(...) return list fo inputSet. Is it safe just to use the first elment?
						  //The answer is: since we always skip branch point, we should only have one InputPinSet back
						  List nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
						  if(nextInputPinSets == null || nextInputPinSets.isEmpty() ){						  
							  nextIS = null; //reached end of SAN
							  //Is it needed to do the connection for the output set of SAN to internal activities of SAN?
							  //The answer is no. Process rule take care of doing it. 
							  //doOutputPinSetConnection(flow, true);
						  }else{						  
							  nextIS= (InputPinSet)nextInputPinSets.get(0);	
							  if(nextIS.getAction() == mergePointAction)
								nextIS = null; //we are at the merge point, so exclude the concurrent merge point
						  }
					  }else{ // stepTrhoughBOM didn't create child rule, so we should stop tracing this path
						//do nothing
					  }
			 
					 }while(nextIS!= null);	
				}
		  } 			 
	  }	    
	  
    processChildTargets(flow);	
	
	setComplete(true);
	LoggingUtil.traceExit(this, "transformSource2Target");	
	return isComplete();
	}
	
	
	/**
	 * Transforms elements in the concurrent region to BPEL targets.
	 * This rule will also be in charged of managing the Variable Registry. 
	 * Variables new created/already created should not be reused between paralel flows.
	 * 2 Registries will be created.. One variable regitry that is shared by elements of the entire ConcurrentBranchRegion, 
	 * keeping all the variables created or used in the branch region that should not be reused by other paths.
	 * A second Registry that keeps the variables that are in used by one Path of the CBR and can be reused within the path. 
	 * When finishing stepping through one path, the first registry should be populated with elements in the second registry, and the
	 * second registry should be reinitialized.
	 * The OutputSet, source of this rule should be checked if it belongs to a CBR itself (this is the case of a nested CBR). 
	 * In this case, the first registry is populated with the list of the main CBR, same the currentRegistry.
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/* 
		 * tranformation includes following steps:
		 * 1. create flow
		 * 2. create links for the concurrent branch point output set.
		 * 3. for each input set reachable by the branch point input set, 
		 *    step through it, create child rules and execute the rule, 
		 *    until reached the merge point or reached end of Structured Activity Node.
		 * 4. consume and propagate targets produced by child rules.  
		 */	
		 		
		 //assertion
		 if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;		 	
		 }
		 //first source of this rule is one output set
		 OutputPinSet outputSet = (OutputPinSet)getSource().get(0);
  	
		 //check if this is first time execute this rule
		  if(!firstTimeExec){
			 reExecute(null); 
			 LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			 return true;
		  }	   
		firstTimeExec = false;	   
	
		//create BPEL flow
		Flow flow = BPELFactory.eINSTANCE.createFlow();
		//defect 139867. generate name for the flow
		String convertedName = NamingUtil.getFlowName(flow, this);
		flow.setName(convertedName);
		createDisplayName(flow, convertedName);
		getTarget().add(flow);	
		
		createLinksForOutputPinSet(outputSet);
		
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		//find the concurrent merge point
		NamedElement mergePoint = BomHelper.getInstance().findConcurrentMergePoint(outputSet);
		Action mergePointAction = null;
		if(mergePoint instanceof InputPinSet)
			mergePointAction = ((InputPinSet)mergePoint).getAction();
		else if(mergePoint instanceof StructuredActivityNode)
			mergePointAction = (Action)mergePoint;
		
		initializePathsVariablesForBranchRegion();
		
//		//!!main logic of this method is here
		  InputPinSet nextIS;
		  //get a list of input set reachable from the output set. since output set is concurrent branch point, this should never step out of the branch region.
		  List inputSetList = bw.getNext(outputSet, BomWalker.INPUT_SET, false);			  
		  if(inputSetList == null || inputSetList.isEmpty()){
			//we have reached end of SAN ??? what to do??
			//need do the connection for the output set of SAN to internal activities of SAN? 
			//No, because the process rule will take case of doing output set connection with internal and external
		  }else{
			int count = inputSetList.size();
	
		  	for (int i=0; i<count; i++){ 
		  		
			    nextIS = (InputPinSet)inputSetList.get(i);
			    InputPinSet mergeSet = nextIS;
			    //for the CB the BCPoint Variables are already added to the variablesOnCurrentPath. We should 
			    //remove the CB variable created for the CB set that is not in focus - not linked to this inputPinSet
			    //and add them to the list of forbiddenVariables. This should not apply to the Fork and Decision for which
			    // the OutputSet carries similar variables.
			     if(!(outputSet.getAction() instanceof Fork)){
			     	updatePathRegistryAtBranchPoint(outputSet, nextIS);
			    }
			    
				if(nextIS.getAction() != mergePointAction){				
				    do{		
						nextIS = stepThroughBOM(nextIS);
						if(nextIS != null){ //means we just created a child rule
						  //we must continue
						  //bw.getNext(...) return list fo inputSet. Is it safe just to use the first elment?
						  //The answer is: since we always skip branch point, we should only have one InputPinSet back
						  List nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
						  if(nextInputPinSets == null || nextInputPinSets.isEmpty() ){						  
							  nextIS = null; //reached end of SAN
							  //Is it needed to do the connection for the output set of SAN to internal activities of SAN?
							  //The answer is no. Process rule take care of doing it. 
							  //doOutputPinSetConnection(flow, true);
						  }else{						  
							  nextIS= (InputPinSet)nextInputPinSets.get(0);	
							  if(nextIS.getAction() == mergePointAction)
								nextIS = null; //we are at the merge point, so exclude the concurrent merge point
						  }
					  }else{ // stepTrhoughBOM didn't create child rule, so we should stop tracing this path
						//do nothing
					  }
			 
					 }while(nextIS!= null);	
				}
			resetPathRegistryInBranchPoint(outputSet, mergeSet);
		  } 			 
	  }	
	//reset the forbiden variables to the values snapshot value.. 
    resetPathsRegistry();
	processChildTargets(flow);	
	
	setComplete(true);
	LoggingUtil.traceExit(this, "transformSource2Target");	
	return isComplete();
	}
	
	/**
	 * Processes targets produced by child rules.
	 * Links, Containers and Partners are propagated to parent.
	 * Activities are contained by this concurrent branch flow. 
	 * Empty flows are eliminated.
	 * @param flow  the BPEL flow created by this rule.
	 */
	private void processChildTargets(Flow flow){
		
		List childRules = getChildRules();
		TransformationRule nextRule = null;
		List targetList= null;
		Object target;
		
		for(int i=0, count=childRules.size(); i<count; i++){
			nextRule = (TransformationRule)childRules.get(i);
			targetList = nextRule.getTarget();
			for (int j=0, targetCount= targetList.size(); j< targetCount; j++){
				target = targetList.get(j);
				if(target instanceof Link || target instanceof Variable || target instanceof PartnerLink){
					getTarget().add(target); //propagate all these types to parent	
				}
				else if (target instanceof Activity){
					    if(target instanceof Flow && ((Flow)target).getActivities().isEmpty()){
					    	//do nothing since we don't want empty flow 						
					    }
					    else
							//add all other activity to this flow
							flow.getActivities().add(target);
				} else{
					getTarget().add(target); //propagate all other unknown type to parent
				}
			}
		}
	
	}	
	
	/**
	 * Creates AssignRule or LinkRule for each pin in the output set. 
	 * @param outputSet  OutputPinSet of the concurrent branch point
	 */
	private void createLinksForOutputPinSet(OutputPinSet outputSet){
		/* pin in the output set is connected to an input pin, it is responsibility of this 
		 * rule to create a BPEL representaiton of this connection.
		 * check each pin in the output set */
		Pin pin;
		Vector pinList = new Vector();			 
		pinList.addAll(outputSet.getOutputControlPin());
		pinList.addAll(outputSet.getOutputObjectPin());
	
		List nonConnectedPins = new ArrayList();
		// boolean doAssign = (BomHelper.getInstance().isNextSetCompatible(outputSet, getContext())== null? true : false);
		 boolean doAssign = false;
		 for(int i= 0, count = pinList.size(); i<count; i++){
		 //	boolean assignCallOperationActionTarget = false;
			 pin = (Pin)pinList.elementAt(i);

			/* Ignore pins that have no outgoing connection */
			if (pin.getOutgoing() == null || pin.getOutgoing().getTarget() instanceof FlowFinalNode){
				nonConnectedPins.add(pin);
			   continue;
			}
			/* If the current edge is connected to the 'target' pin of a CallOperationAction, just
			 * create a link, even though we may have been told to create an <assign> */
//			if(pin.getOutgoing().getTarget() instanceof InputObjectPin){
//				if(((InputObjectPin)pin.getOutgoing().getTarget()).getAction() instanceof CallOperationAction){
//					CallOperationAction coa = (CallOperationAction)((InputObjectPin)pin.getOutgoing().getTarget()).getAction();
//					if(coa.getTarget() != null){
//						if(coa.getTarget() == ((InputObjectPin)pin.getOutgoing().getTarget())){
//							assignCallOperationActionTarget = true;
//						}
//					}
//				} 
//			} 
			
			if (pin.getOutgoing().getTarget() instanceof TerminationNode) {
				//LinkRule linkRule = createTerminationLinkRule(pin);
				LinkRule linkRule = createTerminationLinkRule(pin,null);
				getChildRules().add(linkRule);
				linkRule.transformSource2Target();


			} else if (pin.getOutgoing().getTarget() instanceof Pin) {
				NamedElement targetSet = BomUtils.findTargetSet(pin.getOutgoing());
				//if ((pin instanceof ObjectPin) && doAssign && !assignCallOperationActionTarget) {
				if ((pin instanceof ObjectPin) && doAssign) {
					if (needAssignRule(pin, outputSet, targetSet)) {
						// create assign rule
						AssignRule ar = AbstractbpelFactory.eINSTANCE.createAssignRule();
						ar.getSource().add(pin.getOutgoing()); //edge
						ar.getSource().add(outputSet); //output set
						ar.getSource().add(targetSet);
						//input set on the target side of the edge
						//add Assign rule as child rule
						getChildRules().add(ar);
						ar.transformSource2Target();
					}
				} else { //create link rule for control pin and object pin in compatible set
					if (needLinkRule(pin, outputSet, targetSet)) {
						LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
						lr.getSource().add(pin.getOutgoing());
						//set the outgoing edge as the source
						lr.getSource().add(pin); //set the output pin as the source
						lr.getSource().add(pin.getOutgoing().getTarget());
						//input pin on the target side of the edge
						lr.getSource().add(outputSet); //set on source side of the edge
						lr.getSource().add(targetSet);
						//input set on target side of the edge
						//add link rule as child rule
						getChildRules().add(lr);
						lr.transformSource2Target();
					}
				}
			}
		}	
		// If non of the pins in the pin set is connected, then apply implicit termination
		if (nonConnectedPins.size()==pinList.size()) {
			//we need to scope out the implicit termination for the exceptional output pinset
			if(!CBPELUtil.isFaultSetNoConnections(outputSet)){
				handleImplicitTermination(outputSet);
			}
		}
	}
		
	private void handleImplicitTermination(PinSet pinSet) {
			/* Do nothing if there is no outgoing edge from pin */
			// Ray: Need to connect empty outputPinSet or outputPinSet with pins with no outgoing edge
			// to the default outputCriteria
			//OutputPinSet outPinSet=findDefaultOutputPinSet(inputSet);
			List outputPinSets = findDefaultOutputPinSet(pinSet);
			//if (outPinSet != null) {
			if (outputPinSets.size()>0) {
				OutputPinSet outPinSet;
				for (Iterator it=outputPinSets.iterator();it.hasNext();) {
					outPinSet = (OutputPinSet)it.next();
					LinkRule linkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
					linkRule.getSource().add(pinSet);
					//OutputPinSet outPinSet = (OutputPinSet)inputSet.getAction().getInStructuredNode().getOutputPinSet().get(0);
					linkRule.getSource().add(outPinSet);
					getChildRules().add(linkRule);
					linkRule.transformSource2Target();					
				}
			}
	}
	
	private List getPathAddedVariables(List currentPathVariablesSnapshot, List variablesOnCurrentPath){
		//the actual variables on CurrentPath
		//List variablesOnCurrentPath = ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext());
		
//		int initialSize = currentPathVariablesSnapshot.size();
//		int finalSize = variablesOnCurrentPath.size();
		List addedVariables = new LinkedList();
//		if(finalSize > initialSize){
//			for(int j= initialSize; j< finalSize; j++){
//				addedVariables.add(variablesOnCurrentPath.get(j));
//			}
//		}
		if(variablesOnCurrentPath != null){
			Iterator it = variablesOnCurrentPath.iterator();
			while(it.hasNext()){
				Object var = it.next();
				if(!currentPathVariablesSnapshot.contains(var) && !addedVariables.contains(var)){
					addedVariables.add(var);
				}
			}
		}
		
		return addedVariables;
		
	}
	
	/**
	 * Returns he list of variables created for the concurrent branch point, starting at <code>cbSet</code>
	 * that merges at <code>mergeSet</code>
	 * @param cbSet concurrent branch point, a PinSet
	 * @param mergeSet concurrent merge Point, a PinSet
	 * @return List of BPELVariables.
	 */
	private List getVariablesForCBPoint(PinSet cbSet, PinSet mergeSet, boolean included){
		List var = new LinkedList();
		List outVar = new LinkedList();
		//List outObjectPins = os.getOutputObjectPin();
		List outObjectPins = BomUtils.getObjectPinsForSet(cbSet);
		List inputObjectPins = BomUtils.getObjectPinsForSet(mergeSet);
		Iterator it = outObjectPins.iterator();
		while(it.hasNext()){
			ObjectPin oop = (ObjectPin)it.next();
			if(oop instanceof OutputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}else{
					outVar.add(createVariable(oop));
				}
			}else if(oop instanceof InputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}else{
					outVar.add(createVariable(oop));
				}
			}
		}
		if(included){
			return var;
		}else{
			return outVar;
		}
	}
	
} //ConcurrentBranchRuleImpl
