/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.RepositoryAction;
import com.ibm.btools.bom.model.processes.actions.RetrieveArtifactAction;
import com.ibm.btools.bom.model.processes.actions.StoreArtifactAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DatastoreAccessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DatastoreRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExclusiveBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Datastore Access Rule</b></em>'.
 * 
 * <p>This class is responsble for mapping Store/RetrieveArtifactPins and Store/RetrieveArtifactActions
 * that reference a BOM Repository that is a Datastore </p>
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DatastoreAccessRuleImpl extends ConnectableRuleImpl implements DatastoreAccessRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private static final boolean bUseEmptyJavaActivity=false;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DatastoreAccessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.DATASTORE_ACCESS_RULE;
	}

	/* 
	 * SOURCE = InputPinSet from [Store/Retrieve]Artifact[Action/Pin] that references a Datastore
	 * TARGET = BPEL Invoke that performs the requested datastore operation
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no sources");
			return false;
		}
		List results = new ArrayList();
		Datastore bomDatastore = null;
		Activity target = null; 

		if (getSource().get(0) instanceof StoreArtifactPin){
	
			StoreArtifactPin pin = (StoreArtifactPin)getSource().get(0);

			bomDatastore = (Datastore)pin.getRepository();
			
			if (BpelOptionsUtil.generateGlobalRepositoryJavaSnippet()) {
				results.add(createEmptyJavaInvoke(pin,bomDatastore));
			} else if(BpelOptionsUtil.generateGlobalRepositoryInvoke()){
				DatastoreRule ds = getDatastoreRule(bomDatastore);
				/* It is a restriction that pin be in only one set */
				OutputPinSet pinSet = (OutputPinSet)BomUtils.getOutputPinSets(pin).get(0);
				target = BPELFactory.eINSTANCE.createSequence();
				int pinIndex;
				if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && BomUtils.isSinglePinInSet(pinSet)){
					//this is the case where we do not have a wrapper complex type for the PinSet... only one Pin in the Set
					pinIndex = -1;
				}else{
					pinIndex = pinSet.getOutputObjectPin().indexOf(pin);	
				}
				Activity javaSnippet = 
					transformVariableStorage(getBPELVariable(pinSet),
							getBPELVariablePart(pin),
							pin,
							false, 	// pin.getAtBeginning().booleanValue(), // not supported
							false, 	// pin.getIsInsert().booleanValue(), // not supported
							false,			// There is no overflow value for a pin
							getBPELVariable(bomDatastore),
							getBPELVariablePart(bomDatastore),
							bomDatastore, pinIndex, 0);

				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
				javaSnippet.setName(convertedName);
				//addDisplayName(pin, bomDatastore, target);	 

				//add this java snippet to the sequence
				((Sequence)target).getActivities().add(javaSnippet);
				Invoke invoke = createInvoke(bomDatastore);
				((Sequence)target).getActivities().add(invoke);
				results.add(target);
				results.add(getBPELVariable(bomDatastore));
			}
		} else if (getSource().get(0) instanceof RetrieveArtifactPin){

			RetrieveArtifactPin pin = (RetrieveArtifactPin)getSource().get(0);
			bomDatastore = (Datastore)pin.getRepository();

			if (BpelOptionsUtil.generateGlobalRepositoryJavaSnippet()) {
				results.add(createEmptyJavaInvoke(pin,bomDatastore));
			} else if(BpelOptionsUtil.generateGlobalRepositoryInvoke()){

				target = BPELFactory.eINSTANCE.createSequence();
				DatastoreRule ds = getDatastoreRule(bomDatastore);
				/* It is a restriction that pin be in only one set */
				InputPinSet pinSet = (InputPinSet)BomUtils.getInputPinSets(pin).get(0);
				int pinIndex;
				//support the case where the destination is a PinSet with a single Pin.. the pinIndex = -1
				if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && BomUtils.isSinglePinInSet(pinSet)){
					//this is the case where we do not have a wrapper complex type for the PinSet... only one Pin in the Set
					pinIndex = -1;
				}else{
					pinIndex = pinSet.getInputObjectPin().indexOf(pin);	
					
				}
				Activity javaSnippet = 
					transformVariableRetrieval(	getBPELVariable(bomDatastore),
												getBPELVariablePart(bomDatastore),
												bomDatastore,
												false, 	// pin.getAtBeginning().booleanValue(), // not supported
												false, // pin.getIsRemove().booleanValue(), // not supported
												getBPELVariable(pinSet),
												getBPELVariablePart(pin),
												pin, 0, pinIndex
												);
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
				target.setName(convertedName);
				//addDisplayName(pin,bomVariable,target);	 
				((Sequence)target).getActivities().add(javaSnippet);
				Invoke invoke = createInvoke(bomDatastore);
				((Sequence)target).getActivities().add(invoke);

				results.add(target);
				results.add(getBPELVariable(bomDatastore));
			}
		} else if (getSource().get(0) instanceof InputPinSet){

			InputPinSet pinSet = (InputPinSet)getSource().get(0);
			RepositoryAction action = (RepositoryAction)pinSet.getAction();
			// TODO: handle the case where there is more than one output set (possibly exception output sets)
			OutputPinSet resultSet = (OutputPinSet)pinSet.getOutputPinSet().get(0);

			if (bUseEmptyJavaActivity) {
				target = createEmptyJavaInvoke(action,action.getRepository());
			} else {
				/* Create and execute the container rule that corresponds to the input pin set */
				ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
				crIS.getSource().add(pinSet);
				this.getChildRules().add(crIS);
				crIS.transformSource2Target();
				results.addAll(crIS.getTarget());
			
				/* Create and execute the container rule that corresponds to the output pin set */
				ContainerRule crOS = AbstractbpelFactory.eINSTANCE.createContainerRule();
				crOS.getSource().add(resultSet);
				this.getChildRules().add(crOS);
				crOS.transformSource2Target();
				results.addAll(crOS.getTarget());

				/* Get the BOM Variable around which this action is based */
				bomDatastore = (Datastore)action.getRepository();

				if (action instanceof StoreArtifactAction){
					StoreArtifactAction saAction = (StoreArtifactAction)action;

					String upperBound = 
						AbstractbpelUtil.getLiteralValue((LiteralSpecification)saAction.getValue().getUpperBound())
										.toString();
				
					target = 
						transformDatastoreStorage(pinSet,
												 getBPELVariable(pinSet),
												 getBPELVariablePart(saAction.getValue()),
												 saAction.getValue(),
												 upperBound,
												 saAction.getAtBeginning().booleanValue(),
												 saAction.getIsInsert().booleanValue(),
												 saAction.getOverflow().booleanValue(),
												 bomDatastore);
				
				} else if (action instanceof RetrieveArtifactAction){
					RetrieveArtifactAction raAction = (RetrieveArtifactAction)action;
					String length;
					/* Check the attributes that are not supported by the TE */
					if (   !( raAction.getLength() instanceof InputValuePin ) 
						|| ((InputValuePin)raAction.getLength()).getValue().size() <= 0 
						|| !( ((InputValuePin)raAction.getLength()).getValue().get(0) instanceof LiteralUnlimitedNatural ) ){
					
						LoggingUtil.logError(MessageKeys.RETRIEVE_ART_NOT_IVP_NOT_SUPPORTED);
						length = "1";
							
					} else {
						length = 
							AbstractbpelUtil.getLiteralValue((LiteralSpecification)
								((InputValuePin)raAction.getLength()).getValue().get(0)).toString();
//						length = ((LiteralUnlimitedNatural)((InputValuePin)raAction.getLength()).getValue().get(0)).getValue(); 
					}

					target = 
						transformDatastoreRetrieval(pinSet,
													length,
													raAction.getAtBeginning().booleanValue(),
													raAction.getIsRemove().booleanValue(),
													bomDatastore,
													getBPELVariable(resultSet),
													getBPELVariablePart((Pin)resultSet.getOutputObjectPin().get(0)),
													(OutputObjectPin)resultSet.getOutputObjectPin().get(0)
													);
				}
			}	
			
			/* Map the input sets and connect the incoming and links for this action */
			connectInputSet(pinSet, target);

			//addDisplayName(action,bomVariable,target);	 
			
			/* Add the target AFTER we have mapped and added the input set */
			results.add(target);
			
			/* Check if this action is an Exclusive Branch Point */
			if (BomHelper.getInstance().isEBranch(pinSet)){
				ExclusiveBranchRule rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
				rule.getSource().add(pinSet);
				getChildRules().add(rule);
				rule.transformSource2Target();
				results.addAll(rule.getTarget());
				
			} else if (BomHelper.getInstance().isCBranch(resultSet)){
				ConcurrentBranchRule rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
				rule.getSource().add(resultSet);
				getChildRules().add(rule);
				rule.transformSource2Target();
				results.addAll(rule.getTarget());
			}

			/* Map the output sets and connect the outgoing links for this action */
			connectOutputSet(resultSet, target);
			results.add(target);
		} 
		
		this.getTarget().addAll(results);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");				
		return true;

	}

	/**
	 * Calls the method to generate the mapped representation of the input set, and connects
	 * the result via a BPEL link to the target activity, if required
	 * @param pinSet The input set to map
	 * @param target The target activity to connect to the input set representation
	 */
	private void connectInputSet(InputPinSet pinSet, Activity target){
		/* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(pinSet);
		Activity inputSetActivity = null;
		if (isResultList.isEmpty() || !(isResultList.get(0) instanceof Activity)){
			inputSetActivity = target;
		} else {
			inputSetActivity = (Activity)isResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(inputSetActivity, 
									   target, 
									   BomUtils.getCanonicalName(pinSet) + "_to_" + pinSet.getAction().getName());
				
			isResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(isResultList);
		}

		/* Now that we have determined that activity to use for the input set, connect it up */
		setTargetOfLinks(pinSet, inputSetActivity);	
	}

	/**
	 * Calls the method to generate the mapped representation of the output set, and connects
	 * the source to the result via a BPEL link, if required.
	 * 
	 * @param pinSet The output pin set to map
	 * @param source The BPEL activity to connect the output set reprsentation to
	 */	
	private void connectOutputSet(OutputPinSet pinSet, Activity source){
		List osResultList = mapOutputPinSet(pinSet);
		Activity outputSetActivity = null;
		if (osResultList.isEmpty()|| !(osResultList.get(0) instanceof Activity)){
			outputSetActivity = source;
		} else {
			outputSetActivity = (Activity)osResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(source, 
									   outputSetActivity, 
									   pinSet.getAction().getName() + "_to_" + BomUtils.getCanonicalName(pinSet));
				
			osResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(osResultList);
		}

		setSourceOfLinks(pinSet, outputSetActivity);	
	}

	/**
	 * Generates the BPEL Activites required to retrieve information from the given BOM
	 * datastore.
	 * @param source The NamedElement that uniquely identifies this datastore access
	 * @param length The number of elements to return
	 * @param atBeginning Whether or not the returned elements should be taken from the beginning
	 * @param isRemove If true, elements that are returned should be removed from the datastore
	 * @param datastore The BOM Datastore from which the information will be retrieved
	 * @param destVariable The BPEL variable into which the information will be placed
	 * @param destVariablePart The part of the BPEL variable into which the info will be placed
	 * @return Activity - The BPEL Activity required to retrieve the information
	 */
	protected Activity transformDatastoreRetrieval(
												NamedElement source,
												String length,
												boolean atBeginning,
												boolean isRemove,
												Datastore datastore,
												BPELVariable destVariable,
												Part destVariablePart,
												MultiplicityElement pin
												){
	
		/* Create the BPEL variable that holds the parameters for input to the datastore invoke */
		BPELVariable inputVar = BPELFactory.eINSTANCE.createBPELVariable();
		
		// Check to see if it's a valid name by calling NameConverter
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), inputVar, BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_ACCESS_VAR_POSTFIX);
		inputVar.setName(convertedName);
		//inputVar.setName(AbstractbpelUtil.convertString(BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_ACCESS_VAR_POSTFIX));
		getTarget().add(inputVar);
		
		/* Create the BPEL variable that holds the result */
		BPELVariable outputVar = BPELFactory.eINSTANCE.createBPELVariable();
		
		//	Check to see if it's a valid name by calling NameConverter
		convertedName = converter.convertName(NamingUtil.findRegistry(this), outputVar, BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_RESULT_VAR_POSTFIX);
		outputVar.setName(convertedName);
		//outputVar.setName(AbstractbpelUtil.convertString(BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_RESULT_VAR_POSTFIX));
		getTarget().add(outputVar);

		/* We need to get the message type that corresponds to the input parameters of the 
		 * datastore access method so that we can set the variable to have the same message type */
		DatastoreRule dsRule = getDatastoreRule(datastore);
		
		PortType portType = null;
		Operation operation = null;
		Object obj = null;

		for (Iterator it = ((Definition)dsRule.getTarget().get(0)).getEPortTypes().iterator(); it.hasNext(); ){
			portType = (PortType)it.next();
		}

		for (Iterator it = portType.getOperations().iterator(); it.hasNext(); ){
			obj = it.next();
			if ( AbstractbpelConstants.RETRIEVE_ARTIFACT_DATASTORE_OPERATION_NAME.equals(((Operation)obj).getName()) ){
				operation = (Operation)obj;
				inputVar.setMessageType( (Message) operation.getInput().getMessage() );
				outputVar.setMessageType( (Message) operation.getOutput().getMessage());
			} 
		}
		
		/* Create the sequence to contain the <assign><invoke><assign> */
		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
		// Check to see if it's a valid name by calling NameConverter

		NameProvider nameProvider = NameProviderFactory.getNameProvider(sequence);
		

		if (source instanceof InputPinSet){
			convertedName = nameProvider.getName(sequence, ((InputPinSet)source).getAction(), NamingUtil.findRegistry(this));
	//		convertedName= converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(((InputPinSet)source).getAction()));
			sequence.setName(convertedName);
			//sequence.setName(BomUtils.getCanonicalName(((InputPinSet)source).getAction()));
		} else {
			convertedName = nameProvider.getName(sequence, source, NamingUtil.findRegistry(this));
			//convertedName= converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(source));
			sequence.setName(convertedName);
			//sequence.setName(BomUtils.getCanonicalName(source));
		}
				
		/* We need to initalize the output parameters for the webservice call.  But because
		 * BPEL+ doesn't support XPath, we need to create an AlternativeActivity so that 
		 * we have both a BPEL and a BPEL+ version of the <assign>
		 */
		AlternativeActivity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);

		bpelAlt.setActivity(createDatastoreParamAssign(inputVar, length, isRemove, atBeginning));
		bpelpAlt.setActivity(createDatastoreParamJava(inputVar, length, isRemove, atBeginning));	
		
		altActivity.getAlternative().add(bpelAlt);
		altActivity.getAlternative().add(bpelpAlt);
		
		sequence.getActivities().add(altActivity);


		/* The PartnerLinkType must be created before the PartnerLink can be created */
		PartnerLinkRule partnerLinkTypeRule =
			createPartnerLinkRule((ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext()),
								  datastore,
								  portType);
								  
		// We need to propagate these targets all the way up to the process definition rule
		getTarget().addAll(partnerLinkTypeRule.getTarget());
		
		PartnerRule partnerLinkRule = 
			createPartnerRule((ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext()),
							  datastore);
		
		// We need to propagate these targets all the way up to the process definition rule
		getTarget().addAll(partnerLinkRule.getTarget());

		Invoke invoke = BPELFactory.eINSTANCE.createInvoke();
// Set the name of the *sequence* (not the invoke) to the name of the source
//		invoke.setName(BomUtils.getCanonicalName(source));
		invoke.setInputVariable(inputVar);
		invoke.setOutputVariable(outputVar);
		invoke.setOperation(operation);
		invoke.setPartnerLink((PartnerLink)partnerLinkRule.getTarget().get(0));
		invoke.setPortType(portType);

		sequence.getActivities().add(invoke);

		/* Create the assign to copy the data to the output variable */
		AlternativeActivity assignOut = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt2 = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		Alternative bpelpAlt2 = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt2.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt2.setType(ProcessFlowType.BPELPP_LITERAL);

		Part srcPart = (Part)outputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_DATA_PART_NAME); 
		bpelAlt2.setActivity(AbstractbpelUtil.createSimpleAssign(outputVar, srcPart, destVariable, destVariablePart));
		bpelpAlt2.setActivity(
			createComplexAssignJava(
				outputVar, 
				srcPart,
				AbstractbpelUtil.isTransformedMultiplicityElementAList(datastore,srcPart),
				destVariable, 
				destVariablePart,
				AbstractbpelUtil.isTransformedMultiplicityElementAList(pin,destVariablePart)
				));
		
		assignOut.getAlternative().add(bpelAlt2);
		assignOut.getAlternative().add(bpelpAlt2);
/*
		Assign assignOut = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();

		from.setVariable(outputVar);
		from.setPart( (Part)outputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_DATA_PART_NAME));
		to.setVariable(destVariable);
		to.setPart(destVariablePart);
		
		copy.setFrom(from);
		copy.setTo(to);
		assignOut.getCopy().add(copy);
*/		
		sequence.getActivities().add(assignOut);
		
		return sequence;
	}

	protected Activity transformDatastoreStorage(NamedElement source,
											  BPELVariable srcVariable,
											  Part srcVariablePart,
											  MultiplicityElement pin,
											  String length,
											  boolean atBeginning,
											  boolean isInsert,
											  boolean overflow,
											  Datastore datastore){

		/* Create the BPEL variable that holds the parameters for input to the datastore invoke */
		BPELVariable inputVar = BPELFactory.eINSTANCE.createBPELVariable();
		
		// Check to see if it's a valid name by calling NameConverter
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), inputVar, BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_ACCESS_VAR_POSTFIX);
		inputVar.setName(convertedName);
		//inputVar.setName(AbstractbpelUtil.convertString(BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_ACCESS_VAR_POSTFIX));
		getTarget().add(inputVar);

		/* Create the BPEL variable that holds the result */
		BPELVariable outputVar = BPELFactory.eINSTANCE.createBPELVariable();
		// Check to see if it's a valid name by calling NameConverter		
		convertedName = converter.convertName(NamingUtil.findRegistry(this), outputVar, BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_RESULT_VAR_POSTFIX);
		outputVar.setName(convertedName);
		//outputVar.setName(AbstractbpelUtil.convertString(BomUtils.getCanonicalName(source) + AbstractbpelConstants.DATASTORE_RESULT_VAR_POSTFIX));
		getTarget().add(outputVar);

		/* We need to get the message type that corresponds to the input parameters of the 
		 * datastore access method so that we can set the variable to have the same message type */
		DatastoreRule dsRule = getDatastoreRule(datastore);

		PortType portType = null;
		Operation operation = null;
		Object obj = null;

		for (Iterator it = ((Definition)dsRule.getTarget().get(0)).getEPortTypes().iterator(); it.hasNext(); ){
			portType = (PortType)it.next();
		}
		
		for (Iterator it = portType.getOperations().iterator(); it.hasNext(); ){
			obj = it.next();
			if ( AbstractbpelConstants.STORE_ARTIFACT_DATASTORE_OPERATION_NAME.equals(((Operation)obj).getName()) ){
				operation = (Operation)obj;
				inputVar.setMessageType( (Message) operation.getInput().getMessage() );
				outputVar.setMessageType( (Message) operation.getOutput().getMessage());
			}
		}

		/* Create the sequence to contain the <assign><invoke><assign> */
		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
		NameProvider nameProvider = NameProviderFactory.getNameProvider(sequence);

		if (source instanceof InputPinSet){
			convertedName = converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(((InputPinSet)source).getAction()));
			sequence.setName(convertedName);
			//sequence.setName(BomUtils.getCanonicalName(((InputPinSet)source).getAction()));
		} else {
			convertedName = converter.convertName(NamingUtil.findRegistry(this), sequence, BomUtils.getCanonicalName(source));
			sequence.setName(convertedName);
			//sequence.setName(BomUtils.getCanonicalName(source));
		}
	
		/* We need to initalize the output parameters for the webservice call.  But because
		 * BPEL+ doesn't support XPath, we need to create an AlternativeActivity so that 
		 * we have both a BPEL and a BPEL+ version of the <assign> */
		AlternativeActivity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);

		bpelAlt.setActivity(createDatastoreStorageParamAssign(srcVariable, srcVariablePart, inputVar, length, isInsert, atBeginning, overflow));
		bpelpAlt.setActivity(createDatastoreStorageParamJava(srcVariable, srcVariablePart, pin, inputVar, datastore, length, isInsert, atBeginning, overflow));	
		
		altActivity.getAlternative().add(bpelAlt);
		altActivity.getAlternative().add(bpelpAlt);
		
		sequence.getActivities().add(altActivity);

		/* The PartnerLinkType must be created before the PartnerLink can be created */
		PartnerLinkRule partnerLinkTypeRule =
			createPartnerLinkRule((ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext()),
								  datastore,
								  portType);
		
		// We need to propagate these targets all the way up to the process definition rule		
		getTarget().addAll(partnerLinkTypeRule.getTarget());

		PartnerRule partnerLinkRule = 
			createPartnerRule((ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext()),
							  datastore);

		// We need to propagate these targets all the way up to the process definition rule		
		getTarget().addAll(partnerLinkRule.getTarget());

		Invoke invoke = BPELFactory.eINSTANCE.createInvoke();
// Set the name of the sequence to the name of the source instead
//		invoke.setName(BomUtils.getCanonicalName(source));
		invoke.setInputVariable(inputVar);
		invoke.setOutputVariable(outputVar);
		invoke.setOperation(operation);
		invoke.setPartnerLink((PartnerLink)partnerLinkRule.getTarget().get(0));
		invoke.setPortType(portType);

		sequence.getActivities().add(invoke);

		return sequence;

	  	
	}
	
	/**
	 * Creates a copy clause to be used to initialize the retrieve artifact operation call
	 */
	private Copy createCopy(Object value, BPELVariable var, String partName){
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();

		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
		if (value instanceof Boolean){
			BooleanLiteralExpression strExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
			strExpr.setBooleanSymbol((Boolean)value);
			bpelExpr.setExpression(strExpr);

		} else if (value instanceof Integer){
			IntegerLiteralExpression strExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
			strExpr.setIntegerSymbol((Integer)value);
			bpelExpr.setExpression(strExpr);

		} else {
			StringLiteralExpression strExpr = ModelFactory.eINSTANCE.createStringLiteralExpression();
			strExpr.setStringSymbol(value.toString());
			bpelExpr.setExpression(strExpr);

		}		

		from.addExtensibilityElement(bpelExpr);
		to.setVariable(var);
		to.setPart( (Part) var.getMessageType().getPart(partName));
		copy.setFrom(from);
		copy.setTo(to);
		return copy;
	}
		
	

	/** 
	 * Creates and executes a PartnerLinkRule for the given datastore, if one doesn't already exist.  
	 * If one does exist, it returns it.
	 * 
	 * @param processIntRule The process interface rule in which to search for a 
	 * pre-existing PartnerLinkRule
	 * @param datastore The datastore around which to build or find the PartnerLinkRule
	 * @param portType The associated PortTpe
	 * @return PartnerLinkRule - The resultant rule.
	 */
	private PartnerLinkRule createPartnerLinkRule(ProcessInterfaceRule processIntRule, 
												  Datastore datastore, 
												  PortType portType) {	
		PartnerLinkRule partnerLinkRule = (PartnerLinkRule)
			TransformationUtil.getRuleForSource(
				processIntRule, PartnerLinkRule.class, datastore);

		if (partnerLinkRule == null) {
			partnerLinkRule = WsdlFactory.eINSTANCE.createPartnerLinkRule();
			partnerLinkRule.getSource().add(datastore);
			partnerLinkRule.getSource().add(portType);
			processIntRule.getChildRules().add(partnerLinkRule);
			processIntRule.transformSource2Target();
		}
		return partnerLinkRule;
	}
	
	/**
	 * Creates and executes a PartnerRule for the given datastore, if one doesn't alrady exist.  
	 * If one does exist, it is returned.  
	 * 
	 * @param processDefRule The process definition rule in which to search
	 * @param datastore The datastore 
	 * @return PartnerRule - A new or pre-exiting PartnerRule created for the datastore
	 */
	private PartnerRule createPartnerRule(ProcessDefinitionRule processDefRule, 
											Datastore datastore) {	

		PartnerRule partnerRule = (PartnerRule)TransformationUtil.getRuleForSource(processDefRule, PartnerRule.class, datastore);
				
		if (partnerRule == null) {
			partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().add(datastore);
			
			/* HACK: set this rule to be a child of the ProcessDefinitionRule, because in the 
			 * ConnectableRule, we have to remove the DatastoreAccessRule as a child rule 
			 * or else too much gets propagated.  The problem is that there is no way to differentiate
			 * between setting something as the target of the rule, and indicating that something
			 * should be propagated.
			 * 
			 * By setting this as the child of the processDefinitionRule, we will always be able to
			 * find it later, which will mean that we will only create one partner link in the BPEL.
			 */
//			getChildRules().add(partnerRule);
			processDefRule.getChildRules().add(partnerRule);
			
			partnerRule.transformSource2Target();
		}
		
		return partnerRule;
	}
	
	private BPELVariable getBPELVariable(EObject bomObject){
		BPELVariable destVariable = null;
		if(bomObject instanceof Datastore){
			destVariable = (BPELVariable)TransformationUtil.getRuleForSource(getRoot(), DatastoreRule.class,	bomObject).getTarget().get(0);
		}else{
			destVariable = (BPELVariable)
			TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
												ContainerRule.class,
												bomObject)
							  .getTarget()
							  .get(0);
			
		}
			
		return destVariable;
	}

	private Part getBPELVariablePart(EObject bomObject){

		Part destPart = null;
		// Ray
		/* Start searching from the root, because the bomObject could be a pin on a CallBehaviourAction
		 * etc., that is within a WSDL other than the one that contains this Datastore */		
//			TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),
		
		if(bomObject instanceof Pin){
			Action action = null;
			PinSet pinSet = null;
			if(bomObject instanceof InputObjectPin){
				action = ((InputObjectPin)bomObject).getAction();
				pinSet = (InputPinSet)BomUtils.getInputPinSets((Pin)bomObject).get(0);
			}else if(bomObject instanceof OutputObjectPin){
				action = ((OutputObjectPin)bomObject).getAction();
				pinSet = (OutputPinSet)BomUtils.getOutputPinSets((Pin)bomObject).get(0);
			}
			//if(ProcessUtil.isMultiplePartWSDL(action)){
			if(!isWSICompliant(action)){
				destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, bomObject).getTarget().get(0);
			}else{
				
				destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, pinSet).getTarget().get(0);
			}
		}else{
			destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, bomObject).getTarget().get(0);
		}
			
		  
			
		return destPart;
	}

	private Assign createDatastoreParamAssign(BPELVariable inputVar, String lengthValue, boolean isRemoveValue, boolean atBeginningValue){
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		
		/* Create a Copy clause for each of the input parameters */
		assign.getCopy().add(createCopy(lengthValue, inputVar, AbstractbpelConstants.DATASTORE_LENGTH_PART_NAME));
		assign.getCopy().add(createCopy(new Boolean(isRemoveValue), inputVar, AbstractbpelConstants.DATASTORE_IS_REMOVE_PART_NAME));
		assign.getCopy().add(createCopy(new Boolean(atBeginningValue), inputVar, AbstractbpelConstants.DATASTORE_AT_BEGINNING_PART_NAME));

		return assign;		
	}
	
	private Assign createDatastoreParamAssign(BPELVariable inputVar, String lengthValue, boolean isRemoveValue, boolean atBeginningValue, boolean overflowValue){
		/* Use the other version of createDatastoreParamAssign to set the first three values */
		Assign assign = createDatastoreParamAssign(inputVar, lengthValue, isRemoveValue, atBeginningValue);
		assign.getCopy().add(createCopy(new Boolean(overflowValue), inputVar, AbstractbpelConstants.DATASTORE_OVERFLOW_PART_NAME));
		return assign;		
	}

	private Invoke createDatastoreParamJava(BPELVariable inputVar, String lengthValue, boolean isRemoveValue, boolean atBeginningValue){
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity javaScriptActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();

		String javaString = new String();
		Part lengthPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_LENGTH_PART_NAME);
		Part isRemovePart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_IS_REMOVE_PART_NAME);
		Part atBeginningPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_AT_BEGINNING_PART_NAME);
		 
		javaString += getSetPartValueJava(inputVar, lengthPart, lengthValue);
		javaString += getSetPartValueJava(inputVar, isRemovePart, new Boolean(isRemoveValue));
		javaString += getSetPartValueJava(inputVar, atBeginningPart, new Boolean(atBeginningValue));
		//TODO AR migration ???  
//		javaCode.setValue(javaString);
//		javaScriptActivity.setJavaCode(javaCode);
		javaScriptActivity.setJavaCode(javaString);
		javaInvoke.getExtensibilityElements().add(javaScriptActivity);
		return javaInvoke;
	}

	private Assign createDatastoreStorageParamAssign(BPELVariable sourceVar, Part sourceVarPart, BPELVariable inputVar, String lengthValue, boolean isInsertValue, boolean atBeginningValue, boolean overflowValue){
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		
		/* Create a Copy clause for each of the input parameters */
		assign.getCopy().add(createCopy(lengthValue, inputVar, AbstractbpelConstants.DATASTORE_LENGTH_PART_NAME));
		assign.getCopy().add(createCopy(new Boolean(isInsertValue), inputVar, AbstractbpelConstants.DATASTORE_IS_INSERT_PART_NAME));
		assign.getCopy().add(createCopy(new Boolean(atBeginningValue), inputVar, AbstractbpelConstants.DATASTORE_AT_BEGINNING_PART_NAME));
		assign.getCopy().add(createCopy(new Boolean(overflowValue), inputVar, AbstractbpelConstants.DATASTORE_OVERFLOW_PART_NAME));
		
		/* Create the copy to insert the data into the outbound variable */
		Copy c = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		from.setVariable(sourceVar);
		from.setPart(sourceVarPart);
		
		to.setVariable(inputVar);
		to.setPart((Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_DATA_PART_NAME));
		
		c.setFrom(from);
		c.setTo(to);
		assign.getCopy().add(c);

		return assign;		
	}

	private Invoke createDatastoreStorageParamJava(BPELVariable srcVariable, Part srcVariablePart, MultiplicityElement srcBomElement, BPELVariable inputVar, MultiplicityElement destBomElement, String lengthValue, boolean isInsertValue, boolean atBeginningValue, boolean overflowValue){
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity javaScriptActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();		
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();

		String javaString = new String();
		Part lengthPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_LENGTH_PART_NAME);
		Part isInsertPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_IS_INSERT_PART_NAME);
		Part atBeginningPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_AT_BEGINNING_PART_NAME);
		Part overflowPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_OVERFLOW_PART_NAME);
		Part dataPart = (Part)inputVar.getMessageType().getPart(AbstractbpelConstants.DATASTORE_DATA_PART_NAME);
		 
		javaString += getSetPartValueJava(inputVar, lengthPart, lengthValue);
		javaString += getSetPartValueJava(inputVar, isInsertPart, new Boolean(isInsertValue));
		javaString += getSetPartValueJava(inputVar, atBeginningPart, new Boolean(atBeginningValue));
		javaString += getSetPartValueJava(inputVar, overflowPart, new Boolean(overflowValue));

		/* Ray
		javaString += AbstractbpelUtil.generateJavaGetterForVariable(inputVar.getName()) + "(true)"
						   + ".set"	+ AbstractbpelUtil.capitalizeFirstChar(dataPart.getName()) 
						   + "(" + AbstractbpelUtil.generateJavaGetterForVariable(srcVariable.getName()) + "()"
						   			+ ".get" + AbstractbpelUtil.capitalizeFirstChar(srcVariablePart.getName()) + "());\n";
		*/	
		javaString += createComplexAssignJavaCode(
						srcVariable,
						srcVariablePart,
						AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart),
						inputVar,
						dataPart,
						AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,dataPart)
						);		
		//TODO AR migration ????
//		javaCode.setValue(javaString);
//		javaScriptActivity.setJavaCode(javaCode);
		javaScriptActivity.setJavaCode(javaString);
		javaInvoke.getExtensibilityElements().add(javaScriptActivity);
		
		return javaInvoke;
	}

	protected Invoke createComplexAssignJava(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr=createComplexAssignJavaCode(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList);
		//TODO AR migration ???
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}
	
	protected String createComplexAssignJavaCode(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList) {
		String javaStr = new String();
//		String outVarName = AbstractbpelUtil.capitalizeFirstChar(destVariable.getName());
//		String outPartName = AbstractbpelUtil.capitalizeFirstChar(destVariablePart.getName());
//		if (isDestAList) {
//			String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(destVariablePart.getTypeName());
//			// Create partClass instance
//			javaStr += fullPartTypeName + " " + destVariablePart.getName()+";\n";
//			javaStr += "try {\n";
//			javaStr += "\t"+destVariablePart.getName()+ " = " + AbstractbpelUtil.generateJavaGetterForVariable(outVarName)+"(true).get"+outPartName + "();\n";				
//			javaStr += "} catch (java.lang.IllegalArgumentException ex) {\n";
//			javaStr += "\t"+destVariablePart.getName()+ " = new " + fullPartTypeName + "();\n";				
//			javaStr += "}\n";
//		}
		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable,destVariablePart,true);
		}	
		javaStr +=AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaPartSetterCode(destVariable,destVariablePart,destVariablePart.getName());
		}
//		if (isDestAList) {
//			javaStr +=
//			AbstractbpelUtil.generateJavaGetterForVariable(outVarName)+ 
//			"(true).set"+outPartName+"("+destVariablePart.getName()+");\n";
//		}
		
		return javaStr;

	}

	private String getSetPartValueJava(BPELVariable var, Part part, Object value){
		String javaString = new String();

		/* TODO: Do we have to use eGet(...) and eSet(...) calls to be generic and avoid casting? */
		/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */	
		
//		javaString += AbstractbpelUtil.generateJavaGetterForVariable(var.getName()) + "(true)"
//				   + ".set"	+ AbstractbpelUtil.capitalizeFirstChar(part.getName()) 
//				   + "(" + AbstractbpelUtil.getLiteralAssignmentString(value) + ");\n";
		javaString += AbstractbpelUtil.createJavaPartSetterCode(var,part,AbstractbpelUtil.getLiteralAssignmentString(value));		
		return javaString;
		
	}

	/**
	 * Returns the datastore rule for the given datastore, or creates a new one
	 * if one doesn't already exist.
	 * @param ds The BOM Datastore
	 * @return DatastoreRule An existing or newly created DatastoreRule
	 */
	private DatastoreRule getDatastoreRule(Datastore ds){
		DatastoreRule dsRule = (DatastoreRule)
			TransformationUtil.getRuleForSource(getRoot(),
												DatastoreRule.class,
												ds);

		if (dsRule == null){
			dsRule = AbstractbpelFactory.eINSTANCE.createDatastoreRule();
			dsRule.getSource().add(ds);
			/* Add the datastore to the HeuristicsTransformation root, since its target ( the WSDL
			 * definition) will eventually become an UnclassifiedSolutionArtifact */
			getRoot().getChildRules().add(dsRule);
			dsRule.transformSource2Target();			
		}
		
		return dsRule;
		
	}

	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable
	 */
	private Invoke createEmptyJavaInvoke(Object action,Repository ds){
		Invoke genericAct = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaAssign = "// Add Implementation Codes here";

		jsActivity.setJavaCode(javaAssign);
		genericAct.getEExtensibilityElements().add(jsActivity);
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		genericAct.getEExtensibilityElements().add(dspName);
		String actionName;
		if (action instanceof RetrieveArtifactPin || action instanceof RetrieveArtifactAction) {
			actionName="Read from";
		} else {
			actionName="Write to";
		}
		dspName.setText(actionName+" "+(ds==null?"":ds.getName()));
		
		return genericAct;
	}
	
	/*
	 * Returns an Assign activity that updates the destVariable with the information provided
	 * in the srcVariable
	 */
	protected Activity transformVariableStorage(BPELVariable srcVariable,
											  Part srcVariablePart,
											  MultiplicityElement srcBomElement,
											  boolean atBeginning,
											  boolean isInsert,
											  boolean overflow,
											  BPELVariable destVariable,
											  Part destVariablePart,
											  MultiplicityElement destBomElement, int srcIndex, int destIndex
											  ){
											  	
		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
		Activity assign=null;
		if (srcVariable.getMessageType() == destVariable.getMessageType() ) {
			assign = AbstractbpelUtil.createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
		} else { 
			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
			// TODO: for BPEL should handle creation of multiple Assign
			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart, srcIndex);
			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariablePart);
			bpelAlternative.setActivity(AbstractbpelUtil.createComplexAssign(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
			destIndex = -1;
			if(srcBomElement instanceof StoreArtifactPin){
				if(!isWSICompliant(((StoreArtifactPin)srcBomElement).getAction())){
					srcIndex = -1;
				}
			}
			if(srcBomElement instanceof StoreArtifactPin){
				if(!isWSICompliant(((StoreArtifactPin)srcBomElement).getAction())){
				//	bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
				}else{
					if(!BpelOptionsUtil.isTargetDiamondCompatible()){
				//		bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
					}else{
						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliantDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, srcIndex, destIndex));
					}
				}
			}
			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
		}		
		return assign;
	
												  	
	}	
	
	
	protected Invoke createComplexAssignJavaForWSICompliantDiamond(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();
//		initialize the outputVariable if not already initialized
		javaStr += AbstractbpelUtil.initializeVariable(destVariable, destVariablePart, true);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.initializeJavaListPropertyDiamond(destVariable, destVariablePart, true, destIndex);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippetsDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex, true )
		+ ";\n";
			if(isDestAList){
				String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
				String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex));
				XSDTypeDefinition pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex);
				javaStr += AbstractbpelUtil.generateJavaSetterForVariableDiamond(destVariable.getName(), pinElementName, pinTypeDefinition, listElement) + ";\n";
			}
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}
	
	
	protected Activity transformVariableRetrieval(BPELVariable srcVariable,
			Part srcVariablePart,
			MultiplicityElement srcBomElement,
			boolean atBeginning,
			boolean isRemove,
			BPELVariable destVariable,
			Part destVariablePart,
			MultiplicityElement destBomElement, int srcIndex, int destIndex	){
		
		Activity assign = null;
		
		if (srcVariable.getMessageType() == destVariable.getMessageType() ) {
			assign = AbstractbpelUtil.createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
		} else { 
			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
			// TODO: for BPEL should handle creation of multiple Assign
			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart);
			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariablePart, destIndex);
			if(destBomElement instanceof RetrieveArtifactPin){
				if(!isWSICompliant(((RetrieveArtifactPin)destBomElement).getAction())){
					destIndex = -1;
					//bpelpAlternative.setActivity(createComplexAssignJavaForNonWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
				//	bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
				}else{
					if(!BpelOptionsUtil.isTargetDiamondCompatible()){
					//	bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
					}else{
						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliantDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
					}
				}
				
			}
			//bpelAlternative.setActivity(AbstractbpelUtil.createComplexAssign(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
			//bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));				
			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
		}		
		
		return assign;

			  	
}
	/**
	 * Creates an Invoke for the GlobalRepository. It will use the Interface created by the 
	 * @param ds
	 * @param repository
	 * @return
	 */
	private Invoke createInvoke(Datastore repository){
		//cannot use the InvokeRule as there is no InputSet to set as source, we will manually build 
		//the Invoke
		Invoke invoke = BPELFactory.eINSTANCE.createInvoke();

		Operation operation = null;
		PortType portType = (PortType)TransformationUtil.getRuleForSource(getRoot(), PortTypeRule.class, repository, PortType.class).getTarget().get(0);
		BPELVariable variable = (BPELVariable)getBPELVariable(repository);
		
//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String convertedName = converter.convertName(NamingUtil.findRegistry(this), invoke, BomUtils.getCanonicalName(repository));
		
		NameProvider nameProvider = NameProviderFactory.getNameProvider(repository);
		String convertedName = nameProvider.getName(invoke, repository, NamingUtil.findRegistry(this));

		//todo
		PartnerLink partner = (PartnerLink)TransformationUtil.getRuleForSource(getRoot(), PartnerRule.class, repository, PartnerLink.class).getTarget().get(0);
		
		if(getSource().get(0) instanceof StoreArtifactPin){
			operation =(Operation)portType.getEOperations().get(0);
			invoke.setInputVariable(variable);
		}else if(getSource().get(0) instanceof RetrieveArtifactPin){
			operation =(Operation)portType.getEOperations().get(1);
			invoke.setOutputVariable(variable);
		}
		invoke.setName(convertedName);
		invoke.setOperation(operation);
		invoke.setPartnerLink(partner);
		invoke.setPortType(portType);
		//createDisplayName(invoke,getDisplayName(repository);
		
		return invoke;
	}
	
} //DatastoreAccessRuleImpl
