/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.util.EcoreUtil;

import com.ibm.btools.blm.ie.exprt.ExportOptions;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.DecisionOutputSet;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.NotExpression;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExclusiveBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.BooleanExpression;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Documentation;

/**
 * <!-- begin-user-doc -->
 * Source: BOM Decision InputPinSet
 * Target: BPEL Switch
 * Maps all elements of exclusive branch region marked by the Input Set of 
 * Decision Action up to exclusive merge point including the merge point. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DecisionRuleImpl extends ConnectableRuleImpl implements DecisionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* attribute indicate whether it is first time execution of this rule, default to true */
	private boolean ivFirstTimeExec = true;
	/* attribute indicates whether the source's container, a decision is an inclusive decision */
	boolean isMultiplePath = true;
	boolean isAllVariablesMapped = false;
	
	/* This map keeps track of the BPEL Case created for each output pin set */
	private Map fPinSetToCaseMap = new HashMap();
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DecisionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.DECISION_RULE;
	}

	/**
	 * Maps all elements of the exclusive branch region.
	 *   
	 * SOURCE = InputSet on a Decision
	 * TARGET = Switch Activity
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/* 
		 * 1. create BPEL Switch
		 * 2. set Switch as target of the links into the inputSet.
		 * 3. create BPEL Cases, each Case contains one empty. 
		 * 4. create ExclusiveBranch rule to map the exclusive region.
		 * 5. set each case empty as the source of the outgoing links from the output set.
		 * 6. propagate all child rule's targets. 
		 */
		
		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
		  
		//source of this rule is one input set
		InputPinSet inputSet = (InputPinSet)getSource().get(0);

		//check if this is first time execute this rule
		if(!ivFirstTimeExec){
//			make sure we call again the create container rule so that all variable rules are completed.
			createContainerRule();
			reExecute(inputSet); 
			
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		}
   
		if (((Decision)inputSet.getAction()).getMultiplePaths().equals(Boolean.FALSE)){
			isMultiplePath=false;
		}
		
		ivFirstTimeExec = false;
		 
		createContainerRule();
		
		 //create switch
		Switch bpelSwitch = BPELFactory.eINSTANCE.createSwitch();

		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME_SWITCH);
		valueProvider.setContext(getContext());
		String convertedName = (String)valueProvider.getAttributeValueForType(bpelSwitch, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_NAME_SWITCH, NamingUtil.findRegistry(this));
		String displayName = (String)valueProvider.getAttributeValueForType(bpelSwitch, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_SWITCH, null);
		
//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String convertedName = converter.convertName(NamingUtil.findRegistry(this), bpelSwitch, BomUtils.getCanonicalName(inputSet.getAction()));
		
		bpelSwitch.setName(convertedName);
		// Ray: added display name support
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
//		dspName.setText(inputSet.getAction().getName());
		dspName.setText(displayName);
		bpelSwitch.getEExtensibilityElements().add(dspName);
		
		//bpelSwitch.setName(BomUtils.getCanonicalName(inputSet.getAction()));
		getTarget().add(bpelSwitch);
		
		// Map the input set and link it to the new switch, if necessary
		doInputPinSetConnection(bpelSwitch);
		
		//create cases
		createCases(bpelSwitch);
		
		//Ray: Create exclusiveBranchRule only if it is really needed
		//if (BomHelper.getInstance().isEBranch(inputSet)) {
			createExclusiveBranchRule();
		//}
			
		
		// Mapping the output sets is now done in the createCases(...) method
		connectOutputSets(bpelSwitch);

		// Ray: added support for descriptions and documentations	
		Action action = inputSet.getAction();	
		Description desc = AbstractbpelUtil.transformDescription(action);
		if (desc!=null) {
			bpelSwitch.addExtensibilityElement(desc);
		}
		Documentation doc = AbstractbpelUtil.transformDocumentation(action);
		if (doc!=null) {
			bpelSwitch.addExtensibilityElement(doc);
		}			

		//propagate all targets from children to parent
		propagateChildTargets();
		
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Creates an ExclusiveBranchRule to map all elements in the exclusive branch region.
	 */
	private void createExclusiveBranchRule(){
		InputPinSet inputSet = ((InputPinSet)getSource().get(0));
		//create an exclusiveBranch rule to represent the cases
		ExclusiveBranchRule rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
		rule.getSource().add(inputSet);
		getChildRules().add(rule);
		rule.transformSource2Target();
	}
	
	private Expression buildMutualExclusiveExpr(List osList){		
		BinaryLogicalBooleanExpression appendExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
		appendExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
		for(int i= 0; i<osList.size(); i++){
			NotExpression notExpr = ModelFactory.eINSTANCE.createNotExpression();
			Expression tempCopy = (Expression)EcoreUtil.copy(((StructuredOpaqueExpression)((DecisionOutputSet)osList.get(i)).getCondition()).getExpression());
			notExpr.setExpression(tempCopy);				
			if (appendExpr.getFirstOperand()==null){
				if ( osList.size()!=1){
					//need to create second operand
					appendExpr.setFirstOperand(notExpr);
				}else{
					//end of expression					
					return notExpr;
				}
			}else{
				if ( i!=osList.size()-1 ){
					//need to create second operand
					appendExpr.setSecondOperand(notExpr);
					Expression nextExprCopy = (Expression)EcoreUtil.copy(appendExpr);
					appendExpr.setFirstOperand(nextExprCopy);					
				}else{
					//end of expression		
					appendExpr.setSecondOperand(notExpr);										
				}					
			}				
		}
		return (Expression)appendExpr;		
	}
	
	private Expression buildMutualExclusiveExpr(List osList, int currentOS){	
		BinaryLogicalBooleanExpression expr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();		
		//non-empty expression in decision output set
		expr.setOperator(LogicalBooleanOperator.AND_LITERAL);
		Expression exprCopy = (Expression)EcoreUtil.copy(((StructuredOpaqueExpression)((DecisionOutputSet)osList.get(currentOS)).getCondition()).getExpression());
		expr.setFirstOperand(exprCopy);
		
		BinaryLogicalBooleanExpression appendExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();
		appendExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
		for(int i= 0; i<osList.size(); i++){			
			if (i!=currentOS){
				/* Ray - Support the case where a "true" boolean literal expression is used for the last branch
				 * of an exclusive decision. Skip creating a negated "true" expression in this case.
				 */
				Expression branchExpr = ((StructuredOpaqueExpression)((DecisionOutputSet)osList.get(i)).getCondition()).getExpression();
				if (branchExpr instanceof BooleanLiteralExpression && ((BooleanLiteralExpression)branchExpr).getBooleanSymbol().compareTo(Boolean.TRUE)==0 && i==osList.size()-1) {
					// Ray- Need to update the secondOperand of the appendExpr to null because it was set to a previous negated expression in the previous iteration
					// in which it is expected to be overwritten by a negated 'true' expression in this iteration.
					if (appendExpr.getSecondOperand()!=null) {
						appendExpr.setSecondOperand(null);
					}
					continue;
				}
				NotExpression notExpr = ModelFactory.eINSTANCE.createNotExpression();
				Expression tempCopy = (Expression)EcoreUtil.copy(((StructuredOpaqueExpression)((DecisionOutputSet)osList.get(i)).getCondition()).getExpression());
				notExpr.setExpression(tempCopy);				
				if (appendExpr.getFirstOperand()==null){
					if ( osList.size()!=2){
						//need to create second operand
						appendExpr.setFirstOperand(notExpr);
					}else{
						//end of expression 
						expr.setSecondOperand(notExpr);
						// Ray - we also don't want to leave a "true" check in the expression which is not useful at all
						if (expr.getFirstOperand() instanceof BooleanLiteralExpression &&
								((BooleanLiteralExpression)expr.getFirstOperand()).getBooleanSymbol().compareTo(Boolean.TRUE)==0 &&
								currentOS == osList.size()-1
							) {
							// Ray - return just the secondOperand because the first operation is a "true" check
							return expr.getSecondOperand();
						}
						return expr;
					}
				}else{
					if ((currentOS == osList.size()-1) && (i==osList.size()-2)){
						//end of expression		
						appendExpr.setSecondOperand(notExpr);
						expr.setSecondOperand(appendExpr);			
					}else if ( (i!=osList.size()-1)  ){
						//need to create second operand
						appendExpr.setSecondOperand(notExpr);
						Expression nextExprCopy = (Expression)EcoreUtil.copy(appendExpr);
						appendExpr.setFirstOperand(nextExprCopy);					
					}else{
						//end of expression		
						appendExpr.setSecondOperand(notExpr);
						expr.setSecondOperand(appendExpr);						
					}					
				}				
			}			
		}
		// Ray - Ensure we return the expression correctly
		if (expr.getSecondOperand()==null) {
			if (appendExpr.getSecondOperand()==null) {
				if (appendExpr.getFirstOperand()!=null) {
					// Ray - this is the case where we skipped the negated "true" check in the expression.
					// In this case, the secondOperand will not be set
					expr.setSecondOperand(appendExpr.getFirstOperand());
					return expr;
				}
			}
			return expr.getFirstOperand();
		}
		// Ray - we also don't want to leave a "true" check in the expression which is not useful at all
		if (expr.getFirstOperand() instanceof BooleanLiteralExpression &&
				((BooleanLiteralExpression)expr.getFirstOperand()).getBooleanSymbol().compareTo(Boolean.TRUE)==0 &&
				currentOS == osList.size()-1
			) {
			return expr.getSecondOperand();
		}
		return (Expression)expr;
		
	}

	/**
	 * Creates BPEL Cases for each output set. 
	 * Creates BPEL Empty (or equivalent) for each case as the only activity.
	 * @param bpelSwitch  the BPEL Switch for the Decision
	 */	
	private void createCases(Switch bpelSwitch){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		List osList = inputSet.getOutputPinSet();
		DecisionOutputSet nextOS;
		Case bpelCase; 
		BooleanExpression bexp;
		List expressionList = null;		
		//boolean isMultiplePath = true;		
		//if (((Decision)inputSet.getAction()).getMultiplePaths().equals(Boolean.FALSE)){
		if (!isMultiplePath){
			//isMultiplePath=false;
			expressionList = new LinkedList();
			for (int i=0; i<osList.size(); i++){
				nextOS = (DecisionOutputSet)osList.get(i);
				if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){
					expressionList.add(nextOS);
				}
			}
		}
			
	
		for(int i= 0, count=osList.size(); i<count; i++){
			nextOS = (DecisionOutputSet)osList.get(i);
			bpelCase = BPELFactory.eINSTANCE.createCase();
			fPinSetToCaseMap.put(nextOS, bpelCase);
			/* Create Expression model */ 
//			if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression){
			// Ray: Try to handle the case where there is no OpaqueExpression set for the condition
			//generate the BOM Expression only if all variables are mapped.
			if(isAllVariablesMapped){
				ExtensibilityExpression expr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
				if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){				
//d89317					if (!isMultiplePath && i!=0 && !(expressionList.isEmpty()) && expressionList.size()!=1){
//								//if (BpelOptionsUtil.generateSwitchForExclusiveDecision()) {
//								if (generateSwitchForExclusiveDecision(inputSet.getAction())) {
//									expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));						
//								} else {
//									expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList, expressionList.indexOf(nextOS))));								
//								}
//							}else{
						expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));
//							}
					bpelCase.addExtensibilityElement(expr);	
				} else {
//d89317			if (!isMultiplePath && i!=0 && !(expressionList.isEmpty())){
							//	if (!BpelOptionsUtil.generateSwitchForExclusiveDecision()) {
//						if (!generateSwitchForExclusiveDecision(inputSet)) {
//							expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList)));
//							bpelCase.addExtensibilityElement(expr);		
//						}
//					}
					// Log a warning
					String params[] = new String[1];
					params[0] = BomUtils.getCanonicalName(nextOS,false,true);
					LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);
				}
			}
			   
			/* Create the mapped representation of this output set, dealing with any complex pins */
			List osResultList = mapOutputPinSet(nextOS);
			Activity outputSetActivity = null;
			
			if (osResultList.isEmpty()|| !(osResultList.get(0) instanceof Activity)){
				/* Create an empty if there were no activities created to map the output set */
				outputSetActivity = BPELFactory.eINSTANCE.createEmpty();
				//AR set the name only if we create the Empty, else, the name should have already been set 
				NameProvider nameProvider = NameProviderFactory.getNameProvider(outputSetActivity);
				String convertedName = (String)nameProvider.getName(outputSetActivity, nextOS, NamingUtil.findRegistry(this));
				outputSetActivity.setName(convertedName);

			} else {
				//if an Actvity exists in the list, do not set the name as it was already set
				outputSetActivity = (Activity)osResultList.get(0);
					/* Propagate all other (non-activities) to the parent rule */
				getTarget().addAll(osResultList.subList(1, osResultList.size()));
			}
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), outputSetActivity, BomUtils.getCanonicalName(nextOS));
			//outputSetActivity.setName(BomUtils.getCanonicalName(nextOS));
			
			bpelCase.setActivity(outputSetActivity);
			bpelSwitch.getCases().add(bpelCase);	

			// Ray: added support for descriptions and documentations on output branches	
			if (nextOS.getCondition()!=null) {
				if (nextOS.getCondition().getDescription() != null && nextOS.getCondition().getDescription().trim().length()>0) {
					Documentation doc=BPELPlusFactory.eINSTANCE.createDocumentation();
//GJD JR24896 - limit documentation to 4096 chars and description to 254 chars					
					String documentation = nextOS.getCondition().getDescription();
					if (documentation.length() > 4096) {
						doc.setValue(documentation.substring(0, 4096));
						String[] params = new String[1];
						params[0] = outputSetActivity.getName();
						LoggingUtil.logWarning(MessageKeys.TRUNCATED_ANNOTATION_WARNING, params, null);	
					} else {
						doc.setValue(documentation);
					}
					bpelCase.addExtensibilityElement(doc);
				}
				if (nextOS.getCondition().getName() != null && nextOS.getCondition().getName().trim().length()>0) {
					Description desc=BPELPlusFactory.eINSTANCE.createDescription();
					String descrip = nextOS.getCondition().getName();
					if (descrip.length() > 254) {
						desc.setValue(descrip.substring(0, 254));
//d160626				String[] params = new String[1];
//						params[0] = outputSetActivity.getName();
//						LoggingUtil.logWarning(MessageKeys.TRUNCATED_DESCRIPTION_WARNING, params, null);	
					} else {
						desc.setValue(descrip);
					}
					bpelCase.addExtensibilityElement(desc);
				}
			}

		}

	}
	
	/**
	 * Creates BPEL Cases for each output set. 
	 * Creates BPEL Empty (or equivalent) for each case as the only activity.
	 * @param bpelSwitch  the BPEL Switch for the Decision
	 */	
	private void createCases_bak(Switch bpelSwitch){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		List osList = inputSet.getOutputPinSet();
		DecisionOutputSet nextOS;
		Case bpelCase; 
		BooleanExpression bexp;
		List expressionList = null;		
		//boolean isMultiplePath = true;		
		//if (((Decision)inputSet.getAction()).getMultiplePaths().equals(Boolean.FALSE)){
		if (!isMultiplePath){
			//isMultiplePath=false;
			expressionList = new LinkedList();
			for (int i=0; i<osList.size(); i++){
				nextOS = (DecisionOutputSet)osList.get(i);
				if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){
					expressionList.add(nextOS);
				}
			}
		}
			
	
		for(int i= 0, count=osList.size(); i<count; i++){
			nextOS = (DecisionOutputSet)osList.get(i);
			bpelCase = BPELFactory.eINSTANCE.createCase();
			fPinSetToCaseMap.put(nextOS, bpelCase);
			/* Create Expression model */ 
//			if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression){
			// Ray: Try to handle the case where there is no OpaqueExpression set for the condition
			ExtensibilityExpression expr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){				
				if (!isMultiplePath && i!=0 && !(expressionList.isEmpty()) && expressionList.size()!=1){
					//if (BpelOptionsUtil.generateSwitchForExclusiveDecision()) {
					if (generateSwitchForExclusiveDecision(inputSet.getAction())) {
						expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));						
					} else {
						expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList, expressionList.indexOf(nextOS))));								
					}
				}else{
					expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));
				}
				bpelCase.addExtensibilityElement(expr);	
			} else {
				if (!isMultiplePath && i!=0 && !(expressionList.isEmpty())){
				//	if (!BpelOptionsUtil.generateSwitchForExclusiveDecision()) {
					if (!generateSwitchForExclusiveDecision(inputSet)) {
						expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList)));	
						bpelCase.addExtensibilityElement(expr);		
					}
				}
				// Log a warning
				String params[] = new String[1];
				params[0] = BomUtils.getCanonicalName(nextOS,false,true);
				LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);
			}
			   
			/* Create the mapped representation of this output set, dealing with any complex pins */
			List osResultList = mapOutputPinSet(nextOS);
			Activity outputSetActivity = null;
			
			if (osResultList.isEmpty()|| !(osResultList.get(0) instanceof Activity)){
				/* Create an empty if there were no activities created to map the output set */
				outputSetActivity = BPELFactory.eINSTANCE.createEmpty();
				//AR set the name only if we create the Empty, else, the name should have already been set 
				NameProvider nameProvider = NameProviderFactory.getNameProvider(outputSetActivity);
				String convertedName = (String)nameProvider.getName(outputSetActivity, nextOS, NamingUtil.findRegistry(this));
				outputSetActivity.setName(convertedName);

			} else {
				//if an Actvity exists in the list, do not set the name as it was already set
				outputSetActivity = (Activity)osResultList.get(0);
					/* Propagate all other (non-activities) to the parent rule */
				getTarget().addAll(osResultList.subList(1, osResultList.size()));
			}
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), outputSetActivity, BomUtils.getCanonicalName(nextOS));
			//outputSetActivity.setName(BomUtils.getCanonicalName(nextOS));
			
			bpelCase.setActivity(outputSetActivity);
			bpelSwitch.getCases().add(bpelCase);	

			// Ray: added support for descriptions and documentations on output branches	
			if (nextOS.getCondition()!=null) {
				if (nextOS.getCondition().getDescription() != null && nextOS.getCondition().getDescription().trim().length()>0) {
					Documentation doc=BPELPlusFactory.eINSTANCE.createDocumentation();
//GJD JR24896 - limit documentation to 4096 chars and description to 254 chars
					String documentation = nextOS.getCondition().getDescription();
					if (documentation.length() > 4096) {
						doc.setValue(documentation.substring(0, 4096));
						String[] params = new String[1];
						params[0] = outputSetActivity.getName();
						LoggingUtil.logWarning(MessageKeys.TRUNCATED_ANNOTATION_WARNING, params, null);	
					} else {
						doc.setValue(documentation);
					}
					bpelCase.addExtensibilityElement(doc);
				}
				if (nextOS.getCondition().getName() != null && nextOS.getCondition().getName().trim().length()>0) {
					Description desc=BPELPlusFactory.eINSTANCE.createDescription();
					String descrip = nextOS.getCondition().getName();
					if (descrip.length() > 254) {
						desc.setValue(descrip.substring(0, 254));
//d160626				String[] params = new String[1];
//						params[0] = outputSetActivity.getName();
//						LoggingUtil.logWarning(MessageKeys.TRUNCATED_DESCRIPTION_WARNING, params, null);	
					} else {
						desc.setValue(descrip);
					}
					bpelCase.addExtensibilityElement(desc);
				}
			}

		}

	}
	/**
	 * Returns the BPEL Case that was created for the given output pin set 
	 * 
	 * @param pinSet The pin set whose matching Case is requested
	 * @return The BPEL Case created for the pin set
	 */
	public Case getCaseForPinSet(OutputPinSet pinSet){
	    return (Case)fPinSetToCaseMap.get(pinSet);
	}
	
	/**
	 * Connects the links stemming from each output set. 
	 * @param bpelSwitch  the BPEL Switch for the Decision
	 */	
	private void connectOutputSets(Switch bpelSwitch){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		List osList = inputSet.getOutputPinSet();
		DecisionOutputSet nextOS;
		for(int i= 0, count=osList.size(); i<count; i++){
			nextOS = (DecisionOutputSet)osList.get(i);
			setSourceOfLinks(nextOS, ((Case)bpelSwitch.getCases().get(i)).getActivity());		
		}
	}


	/**
	 * Converts BOM expression to BPEL boolean expression for the case condition 
	 * @param opaqueExpression
	 * @return String - the expression string suitable for BPEL logic condition
	 */
	private Expression transformBOMExpression(Expression expression){
		
		// create an ExpressionRule rule to transform the expression in a bom context
		// an expression in a BPEL context		
	    ExpressionRule exprRule = AbstractbpelFactory.eINSTANCE.createExpressionRule();
		exprRule.getSource().add(expression);
	    getChildRules().add(exprRule);
		exprRule.transformSource2Target(); 
		return (Expression)exprRule.getTarget().get(0);
	}
	
	/**
	 * Creates container rules for the Input Set and Output Sets on the Decision.
	 */
	private void createContainerRule_back(){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		//create container rule for input set
		ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
		crIS.getSource().add(inputSet);
		getChildRules().add(crIS);
		crIS.transformSource2Target();
		
		// For decision node, there will be multiple output set correlated to the input set
		// each output set is the copy of the input set, we use the same container target for the container rule just created 
		List osList = inputSet.getOutputPinSet();
		OutputPinSet nextOS;
		ContainerRule crOS;
		   
		for(int i= 0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet)osList.get(i);
			BomHelper.getInstance().reuseContainer(crIS, nextOS);
			
//			crOS = AbstractbpelFactory.eINSTANCE.createContainerRule();
//			crOS.getSource().add(nextOS);
//			getChildRules().add(crOS);
//			
//			//set the input set's container as its target
//			crOS.getTarget().add(crIS.getTarget().get(0));
//			
//			crOS.setComplete(true);
//			//do not need to do the transform since we fooled in the target, but...
//		    // cr.transformSource2Target();		
//
//			/* ... still need to add the output pins to the source of the PartRules in the Message  
//			 * on each output on which we're re-using the input set container */
//			/* TODO: This is REAL hack... the decision node should not use containerRules this way
//			 *       IMHO, container rules should be able to be used and they should know when to
//			 * 	     reuse containers and when not to. */
//			registerPartsWithPins(inputSet, (OutputPinSet)osList.get(i));
			
			//TODO AR register here, if the Decision is inclusive all the consecutive actions
			//this will serve as flag fpr not reusing the variable later... 
			// if this is Decision is found in the DecisionRegistry, it means it got there as a next action
			//from an inclusive decision.. will need to add the register the next actions also

			if((getContext().get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY) != null && ((Map)getContext().get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY)).keySet().contains(inputSet.getAction())) || isMultiplePath){
				registerConsecutiveActions(nextOS);
			}
		}
	}
	
	/**
	 * Creates container rules for the Input Set and Output Sets on the Decision.
	 */
	private void createContainerRule(){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		//create container rule for input set
		List variables = getVariablesForPinSet(inputSet);
		if(variables.size() == BomUtils.getObjectPinsForSet(inputSet).size()){
			isAllVariablesMapped = true;
		}
		// For decision node, there will be multiple output set correlated to the input set
		// each output set is the copy of the input set, we use the same container target for the container rule just created 
		List osList = inputSet.getOutputPinSet();
		OutputPinSet nextOS;
		ContainerRule crOS;
		TransformationRule defRule = ProcessUtil.getProcessDefinitionRule(getContext());   
		for(int i= 0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet)osList.get(i);
			BomHelper.getInstance().reuseContainer(defRule, nextOS, inputSet);
		
			
		if((getContext().get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY) != null && ((Map)getContext().get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY)).keySet().contains(inputSet.getAction())) || isMultiplePath){
				registerConsecutiveActions(nextOS);
			}
		}
	}
	
	
	
//	/**
//	 * Register all actions that are consecutive to this decision. 
//	 * @param nextOS
//	 */
//	private void registerConsecutiveActions(OutputPinSet nextOS){
//		List decisionList = (List)getContext().get("mydecisionKey");
//		if(decisionList == null){
//			decisionList = new LinkedList();
//			getContext().put("mydecisionKey", decisionList);
//		}
//		List inputPinSets = BomUtils.getNextInputPinSets(nextOS);
//		if(!inputPinSets.isEmpty()){
//			Iterator it = inputPinSets.iterator();
//			while(it.hasNext()){
//				InputPinSet is = (InputPinSet)it.next();
//				decisionList.add(is.getAction());
//			}
//		}
//		
//		
//	}
	
	private List getVariablesForPinSet(PinSet pinSet){
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		List variables = new LinkedList();
		if(!objectPins.isEmpty()) {
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin op = (ObjectPin)it.next();
				BPELVariable variable = createVariable(op);
				if(variable != null){
					getTarget().add(variable);
					variables.add(variable);
				}
			}
		}
		return variables;
	}
	
	/**
	 * Register all actions that are consecutive to this decision. 
	 * The registry entry is a map that contain all inputSets that should not qualify for variable reuse keyed by the
	 * action.
	 * @param nextOS
	 */
	private void registerConsecutiveActions(OutputPinSet nextOS){
		Map decisionMap = (Map)getContext().get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY);
		if(decisionMap == null){
			decisionMap = new HashMap();
			getContext().put(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY, decisionMap);
		}
		
		List inputPinSets = BomUtils.getNextInputPinSets(nextOS);
		if(!inputPinSets.isEmpty()){
			Iterator it = inputPinSets.iterator();
			List entry = null;
			while(it.hasNext()){
				InputPinSet is = (InputPinSet)it.next();
				if(decisionMap.containsKey(is.getAction())){
					entry = (List)decisionMap.get(is.getAction());
				}else{
					entry = new LinkedList();
					decisionMap.put(is.getAction(), entry);
				}
				entry.add(is);
			}
		}
		
		
	}
	
	/**
	 * Finds the PartRule associated with each pin in the input for which a container 
	 * already exists, and adds the corresponding pin in the unregistered set to the list of the
	 * sources associated with the part.
	 * 
	 * NOTE: The two sets must previously have been found to be compatible, and the 
	 * set that is the first parameter must have an associated container with its pins registered
	 * for the parts of the message that type the container.
	 *  
	 * @param setWithContainer The PinSet whose pins have already been associated with parts
	 * @param setToRegister The PinSet whose pins are to be associated with the parts
	 */
	private void registerPartsWithPins(PinSet setWithContainer, PinSet setToRegister){
		
		List registeredPins = new ArrayList();
		List unregisteredPins = new ArrayList();
		
		if (setWithContainer instanceof InputPinSet){
			registeredPins.addAll( ((InputPinSet)setWithContainer).getInputObjectPin());	
		} else if (setWithContainer instanceof OutputPinSet){
			registeredPins.addAll( ((OutputPinSet)setWithContainer).getOutputObjectPin());
		}
		
		if (setToRegister instanceof InputPinSet){
			unregisteredPins.addAll( ((InputPinSet)setToRegister).getInputObjectPin());
		} else if (setToRegister instanceof OutputPinSet){
			unregisteredPins.addAll( ((OutputPinSet)setToRegister).getOutputObjectPin());
		}
		
		/* If there are no unregistered object pins, return */
		if (unregisteredPins.isEmpty()){
			return;
		}
		
		/* Because the two sets are matching, they must have the same number of input pins, 
		 * and the object pins must be in the same order, so match them up by index	 */
		
		ObjectPin registeredPin = null;
		PartRule partRule = null;
		for (int i=0, count = registeredPins.size(); i<count; i++){
			
			partRule = (PartRule)
					   TransformationUtil.getRuleForSource(getRoot(), 
														   PartRule.class,
														   (Pin)registeredPins.get(i));

			/* Add the new corresponding pin to the PartRule so that we can find this PartRule
			 * later based on this pin */
			partRule.getSource().add(unregisteredPins.get(i));
			
		}
		
	}
		
	/**
	 * Sets the BPEL switch as the target for the links coming into the pins in the InputPinSet.
	 * @param bpelSwitch
	 */
	private void doInputPinSetConnection(Switch bpelSwitch){
  
		InputPinSet inputSet = (InputPinSet)getSource().get(0); 
		   
		/* Handle any complex pins that are in the input set */
		List isActivityList = mapInputPinSet(inputSet); 
		Activity inputSetActivity = null;
		
		if (isActivityList.isEmpty() || !(isActivityList.get(0) instanceof Activity)){
			inputSetActivity = bpelSwitch;
		} else {
			inputSetActivity = (Activity)isActivityList.get(0);

			/* Create a BPEL Link from the input set activity to the Switch */
			String linkName = BomUtils.getCanonicalName(inputSet) + "_to_" + bpelSwitch.getName();
			//linkActivities(inputSetActivity, bpelSwitch, linkName);
			Link link = linkActivities(inputSetActivity, bpelSwitch, linkName);
			
			/* Propagate the rest of the objects in the list to the parent rule */
			//getTarget().addAll(isActivityList.subList(1, isActivityList.size()));
			getTarget().addAll(isActivityList);
			getTarget().add(link);
		}

		setTargetOfLinks(inputSet, inputSetActivity);		   		 
	}
	
	private boolean generateSwitchForExclusiveDecision(NamedElement origSrcObj){
		/* We never want to generate a switch when we're invoked from the BME Export*/
		ExportSession session = TransformationSessionUtil.getExportSession(TransformationEngine.getTransformationSession());
		if(session != null){
			ExportOptions exportOption = TransformationSessionUtil.getExportSession(TransformationEngine.getTransformationSession()).getExportOptions();
			if(exportOption != null){
				if (exportOption.getAdditionalOption("CREATE_OM_HASHMAP") != null){
					return false;
				}
			}
		}
		
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.GENERATE_SWITCH);
		boolean generateSwitch = ((Boolean)valueProvider.getAttributeValueForType(origSrcObj,AttributeNameConstants.GENERATE_SWITCH)).booleanValue();

		
		return generateSwitch;
	}
	
	/**
	 * Re-executes this rule. After reExecuting the super, will attempt to rebuild the JavaSnippet that might have been created for
	 * this rule. If the input branches in this Join belong to parallel branches that were not processed at the time of processing 
	 * the flow (ex. parallel flow starting from the Process starts and/or pinSet), the Snippet created for this Join will be left
	 * as empty "". It will be created on a subsequent path (reExecute) when all the variables are known. 
	 * This is currently a workaround.. it should be handled by a ConcurrentBranchRule for the Process. 
	 */
	protected void reExecute(InputPinSet inputSet){
		super.reExecute(inputSet);
		//reExecuteSwitch();
		Switch bpelSwitch = getSwitch(getTarget());
		if(bpelSwitch != null){
			updateCases(bpelSwitch);
			
		}
		
	}
	
	
	private void updateCases(Switch bpelSwitch){
		if(!isAllVariablesMapped){
			return;
		}
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		List osList = inputSet.getOutputPinSet();
		DecisionOutputSet nextOS;
		Case bpelCase; 
		BooleanExpression bexp;
		List expressionList = null;		
		if (!isMultiplePath){
			expressionList = new LinkedList();
			for (int i=0; i<osList.size(); i++){
				nextOS = (DecisionOutputSet)osList.get(i);
				if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){
					expressionList.add(nextOS);
				}
			}
		}
		for(int i= 0, count=osList.size(); i<count; i++){
			nextOS = (DecisionOutputSet)osList.get(i);
			bpelCase = (Case)fPinSetToCaseMap.get(nextOS);
				
			/* Create Expression model */ 
			// Ray: Try to handle the case where there is no OpaqueExpression set for the condition
		
			ExtensibilityExpression expr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			if (nextOS.getCondition()!=null && nextOS.getCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression() != null){				
//d89317		if (!isMultiplePath && i!=0 && !(expressionList.isEmpty()) && expressionList.size()!=1){
//					if (generateSwitchForExclusiveDecision(inputSet.getAction())) {
//						expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));						
//					} else {
//						expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList, expressionList.indexOf(nextOS))));								
//					}
//				}else{
					expr.setExpression(transformBOMExpression( ((StructuredOpaqueExpression)nextOS.getCondition()).getExpression()  ));
//				}
				bpelCase.addExtensibilityElement(expr);	
			} else {
//89317			if (!isMultiplePath && i!=0 && !(expressionList.isEmpty())){
//					if (!generateSwitchForExclusiveDecision(inputSet)) {
//						expr.setExpression( transformBOMExpression(buildMutualExclusiveExpr(expressionList)));	
//						bpelCase.addExtensibilityElement(expr);		
//					}
//				}
				// Log a warning
				String params[] = new String[1];
				params[0] = BomUtils.getCanonicalName(nextOS,false,true);
				LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);
			}
	}

		
		
		
		
	}

	/** 
	 * Returns the Switch that was created for this Decision.
	 * @param targets
	 * @return
	 */
	private Switch getSwitch(List targets){
		if(targets == null || targets.isEmpty()){
			return null;
		}
		Switch thisSwitch = null;
		Iterator it = targets.iterator();
		while(it.hasNext()){
			Object targetActivity = it.next();
			if(targetActivity instanceof Switch){
				thisSwitch = (Switch)targetActivity;
				return thisSwitch;
			}
		}
		return thisSwitch;
	}
} //DecisionRuleImpl
