/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Part;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ForkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Query;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Fork Rule</b></em>'.
 * 
 * <p>This class is responsible for mapping a BOM fork into the appropriate BPEL Activities</p>
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ForkRuleImpl extends ConnectableRuleImpl implements ForkRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* Keeps track of whether this is the first time executing this rule or not */
	private boolean fFirstTime = true;
	
	private Activity fForkActivity;


	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ForkRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.FORK_RULE;
	}

	/**
	 * source = input set of a BOM fork node
	 * target = <BPEL Empty>
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
		
		if (!fFirstTime){
			reExecute((InputPinSet)getSource().get(0));
			
			executeHandlers();
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		}
		
		fFirstTime = false;
		
		InputPinSet is = (InputPinSet)getSource().get(0);
		OutputPinSet os = (OutputPinSet)is.getOutputPinSet().get(0);
		
		List isVariables = getVariablesForPinSet(is);
		List osVariables = getVariablesForPinSet(os);
		
		fForkActivity = null;
		fForkActivity = BPELFactory.eINSTANCE.createEmpty();
		
		NameProvider nameProvider = NameProviderFactory.getNameProvider(fForkActivity);
		String convertedName = nameProvider.getName(fForkActivity, is.getAction(), NamingUtil.findRegistry(this));
		fForkActivity.setName(convertedName);

		// Ray: added display name support
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		dspName.setText(is.getAction().getName());
		fForkActivity.getEExtensibilityElements().add(dspName);
		
		connectInputSet(is, fForkActivity);		

		/* Add this to the list of targets *after* we add the input set mapping activity */
		getTarget().add(fForkActivity);	
		
		/* Create the concurrent branch which should be necessary, given this is a fork */
		if (BomHelper.getInstance().isCBranch(os)){
			ConcurrentBranchRule cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
			cbr.getSource().add(os);
			getChildRules().add(cbr);
			cbr.transformSource2Target();
			getTarget().addAll(cbr.getTarget());
		}
			
		connectOutputSet(os, fForkActivity);	
		
		mapDescriptionAndDocumentation(is.getAction(),fForkActivity);
			
		if (BpelOptionsUtil.generateEmptyActivityForControlNode()||CBPELUtil.handleFaultLinkFork(is)) {
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(),fForkActivity);			
		}		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}
	
	/**
	 * Returns the primary activity created to represent the fork
	 * 
	 * @return The primary fork activity
	 */
	public Activity getForkActivity(){
	    return fForkActivity;
	}
	
	/**
	 * Calls the method to generate the mapped representation of the input set, and connects
	 * the result via a BPEL link to the target activity, if required
	 * @param pinSet The input set to map
	 * @param target The target activity to connect to the input set representation
	 */
	private void connectInputSet(InputPinSet pinSet, Activity target){
		/* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(pinSet);
		Activity inputSetActivity = null;
		//if the list is Empty or we have a BPEL Variable inside, do not do anything.
		if (isResultList.isEmpty()|| !(isResultList.get(0) instanceof Activity)){
			inputSetActivity = target;
		} else {
			inputSetActivity = (Activity)isResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(inputSetActivity, 
									   target, 
									   BomUtils.getCanonicalName(pinSet) + "_to_" + pinSet.getAction().getName());
				
			isResultList.add(link);
				
			/* Propagate results from the input set mapping */
			getTarget().addAll(isResultList);
		}

		setTargetOfLinks(pinSet, inputSetActivity);	
		if((BomUtils.isLocalProcess(pinSet.getAction().getInStructuredNode()) ||
				pinSet.getAction().getInStructuredNode() instanceof LoopNode) && 
				pinSet.getAction().getInStructuredNode().getInputPinSet().size() == 1 &&
				ProcessUtil.pinSetIsEntryToSubprocess(pinSet) &&
				ProcessUtil.hasLinksToDifferentActivities((OutputPinSet)pinSet.getAction().getOutputPinSet().get(0))) {	// Fork has 1 input and 1 output pin sets
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(), inputSetActivity);
		}
	}

	/**
	 * Calls the method to generate the mapped representation of the output set, and connects
	 * the source to the result via a BPEL link, if required.
	 * 
	 * @param pinSet The output pin set to map
	 * @param source The BPEL activity to connect the output set reprsentation to
	 */	
	private void connectOutputSet(OutputPinSet pinSet, Activity source){
		List osResultList = mapOutputPinSet(pinSet);
		Activity outputSetActivity = null;
//		if the list is Empty or we have a BPEL Variable inside, do not do anything.
		if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity)){
			outputSetActivity = source;
		} else {
			outputSetActivity = (Activity)osResultList.get(0);
						
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(source, 
									   outputSetActivity, 
										pinSet.getAction().getName() + "_to_" + BomUtils.getCanonicalName(pinSet));
				
			osResultList.add(link);
			//defect 91062 set the fork empty activity as the source of all the activities
			//in the concurret region
			//revert change to address 92306
			//outputSetActivity = source;
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(osResultList);
		}

		setSourceOfLinks(pinSet, outputSetActivity);	
	}
	
	/**
	 * Returns the list of BPEL Variables for this pinSet. 
	 * @param pinSet a BOM Input or OutputPinSet
	 * @return
	 */
	private List getVariablesForPinSet(PinSet pinSet){
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		List variables = new LinkedList();
		if(!objectPins.isEmpty()) {
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin op = (ObjectPin)it.next();
				BPELVariable variable = createVariable(op);
				if(variable != null){
					getTarget().add(variable);
					variables.add(variable);
				}
//				VariableRule crIS = AbstractbpelFactory.eINSTANCE.createVariableRule();
//				crIS.getSource().add(op.getType());
//				crIS.getSource().add(op);
//				getChildRules().add(crIS);
//				crIS.transformSource2Target();
//				if(crIS.getTarget()!= null && !crIS.getTarget().isEmpty()){
//					variables.add(crIS.getTarget().get(0));	
//				}
//				
//				getTarget().addAll(crIS.getTarget());
				
			}
		}
		return variables;
	}
	
	/**
	 * source = input set of a BOM fork node
	 * target = <input set mapping activity>, BPEL <assign>, <output set mapping activity>
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target_bak() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
		
		if (!fFirstTime){
			reExecute((InputPinSet)getSource().get(0));
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		}
		
		fFirstTime = false;
		
		InputPinSet is = (InputPinSet)getSource().get(0);
		OutputPinSet os = (OutputPinSet)is.getOutputPinSet().get(0);
		
		List isVariables = getVariablesForPinSet(is);
		List osVariables = getVariablesForPinSet(os);
		
//		/* Create the container rule for the input set */
		Variable isVariable = null;
		if(!is.getInputObjectPin().isEmpty()) {
			VariableRule crIS = AbstractbpelFactory.eINSTANCE.createVariableRule();
			crIS.getSource().add(is);
			getChildRules().add(crIS);
			crIS.transformSource2Target();
			getTarget().addAll(crIS.getTarget());
			isVariable = (Variable)crIS.getTarget().get(0);
		}
//		
//		/* For a fork, the input set must be correlated to a single output set */
//		
//
////		/* Create the container rule for the output set */
		Variable osVariable = null;
		if (!os.getOutputObjectPin().isEmpty()) {
			ContainerRule crOS = AbstractbpelFactory.eINSTANCE.createContainerRule();
			crOS.getSource().add(os);
			getChildRules().add(crOS);
			crOS.transformSource2Target();
			getTarget().addAll(crOS.getTarget());
			osVariable = (Variable)crOS.getTarget().get(0);
		}

		/* Create single BPEL Assign, with multiple BPEL Copies, if there is at least one object pin */
		Activity forkActivity = null;
		//always create the Empty in case we optimize the Fork transformation
		if (is.getInputObjectPin().size() > 0 && ! BpelOptimizationUtil.bOptimizeForkTransformation){
			forkActivity = BPELFactory.eINSTANCE.createAssign();
		} else {
			forkActivity = BPELFactory.eINSTANCE.createEmpty();
		}
		
		NameProvider nameProvider = NameProviderFactory.getNameProvider(forkActivity);
		String convertedName = nameProvider.getName(forkActivity, is.getAction(), NamingUtil.findRegistry(this));

		
		forkActivity.setName(convertedName);
		//forkActivity.setName(BomUtils.getCanonicalName(is.getAction()));		
		// Ray: added display name support
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		dspName.setText(is.getAction().getName());
		forkActivity.getEExtensibilityElements().add(dspName);
		
		if(!BpelOptimizationUtil.bOptimizeForkTransformation){
			createAssign(is, isVariable, os, osVariable, forkActivity);

			connectInputSet(is, forkActivity);		

			/* Add this to the list of targets *after* we add the input set mapping activity */
			getTarget().add(forkActivity);	
			
			/* Create the concurrent branch which should be necessary, given this is a fork */
			if (BomHelper.getInstance().isCBranch(os)){
				ConcurrentBranchRule cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
				cbr.getSource().add(os);
				getChildRules().add(cbr);
				cbr.transformSource2Target();
				getTarget().addAll(cbr.getTarget());
			}
			
			connectOutputSet(os, forkActivity);	
			
			mapDescriptionAndDocumentation(is.getAction(),forkActivity);
			
			if (BpelOptionsUtil.generateEmptyActivityForControlNode()) {
				BpelOptimizationUtil.registerNonOptimizableElement(getContext(),forkActivity);			
			}
			
		}else{

			connectInputSet(is, forkActivity);		

			/* Add this to the list of targets *after* we add the input set mapping activity */
			getTarget().add(forkActivity);	
			
			/* Create the concurrent branch which should be necessary, given this is a fork */
			if (BomHelper.getInstance().isCBranch(os)){
				ConcurrentBranchRule cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
				cbr.getSource().add(os);
				getChildRules().add(cbr);
				cbr.transformSource2Target();
				getTarget().addAll(cbr.getTarget());
			}
			
			connectOutputSet(os, forkActivity);	
			
			transformForkAssigns(os, is);
			
			getTarget().remove(osVariable);
			getTarget().remove(isVariable);

			mapDescriptionAndDocumentation(is.getAction(),forkActivity);
			
			if (BpelOptionsUtil.generateEmptyActivityForControlNode()) {
				BpelOptimizationUtil.registerNonOptimizableElement(getContext(),forkActivity);			
			}
			
		}

		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}
	
	/**
	 * The method modifies the Assigns created for the Fork OutputSet so that instead of copying from the 
	 * Fork's outputSet it will copy from the corresponding OutputSet pin connected to the Fork. 
	 * @param os Fork's OutputPinSet
	 * @param is Forks inputPinSet
	 */
	private void transformForkAssigns(OutputPinSet os, InputPinSet is){
		
		//first find out all Assigns that were created for the ForkOutputSet... 
		Collection collectionAssignsOut = (Collection)TransformationUtil.getRulesForSource(this, AssignRule.class, os );
			
		if(!collectionAssignsOut.isEmpty()){
		//take each of the Assign and check if its creation was necessary... 	
			Iterator it = collectionAssignsOut.iterator();
			while(it.hasNext()){
				AssignRule thisAssignRule = (AssignRule)it.next();
				Assign assign = null;
				List assignTargets = thisAssignRule.getTarget();
				Iterator it1 = assignTargets.iterator();
				boolean removeAssign = false;
				while(it1.hasNext()){
					EObject target = (EObject)it1.next();
					if(target instanceof Assign){
						assign = (Assign)target;
						if(assign.getCopy() != null){
							Copy copy = (Copy)assign.getCopy().get(0);
							List assignSources = thisAssignRule.getSource();
							//retrieves the Assign target Set which is the Set connecting to the Fork
							PinSet targetSet = (PinSet)assignSources.get(2);
							ActivityEdge edge = (ActivityEdge)assignSources.get(0);
							//retrieves the Pin in the Fork OutputSet to which this Set Connects to
							OutputObjectPin pinInForkOutput = (OutputObjectPin)edge.getSource();
							//total number of Pins in the Fork OutputSet
							int outPins = BomUtils.getPinsInSet(os).size();
//							total number of Pins in the Fork InputSet
							//int inPins = BomUtils.getPinsInSet(is).size();
							int inPins = BomUtils.getObjectPinsForSet(is).size();
							//the index of the Pin in the Fork OutputSet
							//int index = BomUtils.getPinsInSet(os).indexOf(pinInForkOutput);
							int index = BomUtils.getObjectPinsForSet(os).indexOf(pinInForkOutput);
							//int index = BomUtils.getPinsInSet(os).indexOf(pinInForkOutput);
							//now get the correspondent pin in the is of the Fork inputSet
							int inputIndex = index%inPins; 
						
							InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(inputIndex);
							
							//InputObjectPin inPin = (InputObjectPin)BomUtils.getPinsInSet(is).get(inputIndex);
							
							//the pin to which this Pin connects to
							if(inPin.getIncoming() == null){
								//we need to register this assign so that it will be cleaned in the end
								BpelOptimizationUtil.registerOptimizableElement(getContext(), assign);
								break;
							}
							OutputObjectPin  connectedPin = (OutputObjectPin)inPin.getIncoming().getSource();
							//get now the Variable and Part/ maybe query string for this Pin
							PinSet pinSet = BomUtils.findPinSetForPin(connectedPin);
							//call the containerRule to retrieve the variable as it may not be yet created
							Variable fromVariable = getVariableForPinSet(pinSet);
							Variable toVariable = copy.getTo().getVariable();
							//try to optimize more.. if the 2 variables are compatibles, then do not update the 
							// assign, rather reduce it.
							if(fromVariable == toVariable){
								BpelOptimizationUtil.registerOptimizableElement(getContext(), assign);
								break;
							}
							
							Part part = null;
							String queryFrom = null;
								if(BpelOptionsUtil.isTargetWSICompliant()){
									part = (Part)TransformationUtil.getRuleForSource(getRoot(),
											PartRule.class,
											pinSet).getTarget().get(0);
									//in this case we should also set the query.. element name
									queryFrom = createQueryString(pinSet, connectedPin, part);
								}else{
									part = (Part)((PartRule)	
											TransformationUtil.getRuleForSource(getRoot(), PartRule.class, connectedPin)).getTarget().get(0);
								}
							From from = copy.getFrom();
							from.setVariable(fromVariable);
							from.setPart(part);
							if(queryFrom != null && !queryFrom.equals("")){
								Query bpelQueryFrom = BPELFactory.eINSTANCE.createQuery();
								bpelQueryFrom.setQueryLanguage("");
								bpelQueryFrom.setValue(queryFrom);
								from.setQuery(bpelQueryFrom);
							}
						}
				}
			}
		}
	}
			
			
}
	
	

	
	private String createQueryString( NamedElement set, Pin pin, Part part){
		if(part.getTypeDefinition() != null){
			return "";
		}else if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && BomUtils.isSinglePinInSet((PinSet)set)){
			return "";
		}
		String pinElementDeclaration = AbstractbpelUtil.getPinElementName(set, pin, part);
		return  "/" + pinElementDeclaration;

	}	
	

//	/**
//	 * Register the Fork in the context. The context contains one entry for all the Forks at a predefined key.
//	 * All forks are register in a Map : key -> fork activity, object -> List of outputPins that are linked to the 
//	 * inputPinSet of the Fork.
//	 * @param is
//	 */
//	public static void registerForkEntry(TransformationContext context, InputPinSet is){
//		HashMap contextEntry = null;
//		contextEntry =(HashMap) context.get(AbstractbpelConstants.FORK_ENTRY_CONTEXT_KEY);
//		
//		if( contextEntry == null){
//			contextEntry = new HashMap();
//			context.put(AbstractbpelConstants.FORK_ENTRY_CONTEXT_KEY, contextEntry);
//		}
//		
//		//now register the fork....
//		//create the List of outputPins that are linked to the inputPins that contribute to the fork inputSet.
//		List pins = new LinkedList();
//		//pins = findPinsForPinSet();
//		if(is.getInputObjectPin() != null && !is.getInputObjectPin().isEmpty()){
//			Iterator it = is.getInputObjectPin().iterator();
//			while (it.hasNext()){
//				InputObjectPin thisPin = (InputObjectPin)it.next();
//				OutputObjectPin  connectedPin = (OutputObjectPin)thisPin.getIncoming().getSource();
//				pins.add(connectedPin);
//			}
//		}
//		contextEntry.put(is.getAction(), pins);
//	}

	
	/**
	 * Creates a BPEL Assign with multiple Copy that copies from each InputObjectPin of the Fork InputSet 
	 * to the corresponding OuttputObjectPin in the Fork outputSet
	 * @param is
	 * @param isVariable
	 * @param os
	 * @param osVariable
	 * @param forkActivity
	 */
	private void createAssign(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, Activity forkActivity ){
		
		//TODO AR modified for WSI compliant
		Part inPinPart = null;
		Part outPinPart = null;
		if(isWSICompliant(is.getAction())){
			inPinPart = (Part)TransformationUtil.getRuleForSource(getRoot(),
												PartRule.class,
												is).getTarget().get(0);
			
			outPinPart = (Part) 
				TransformationUtil.getRuleForSource(getRoot(), PartRule.class,	os).getTarget().get(0);
			

			for (int i=0, inPinCount = is.getInputObjectPin().size(); i<inPinCount; i++){
				for (int j=i, outPinCount = os.getOutputObjectPin().size(); j<outPinCount; j+=inPinCount){
					Copy copy = BPELFactory.eINSTANCE.createCopy();
					From from = BPELFactory.eINSTANCE.createFrom();
					To to = BPELFactory.eINSTANCE.createTo();
					
					from.setVariable(isVariable);
					from.setPart(inPinPart);
					String queryFrom = createQueryString(is, (Pin)is.getInputObjectPin().get(i), inPinPart);
					if(queryFrom != null && !queryFrom.equals("")){
						Query bpelQueryFrom = BPELFactory.eINSTANCE.createQuery();
						bpelQueryFrom.setQueryLanguage("");
						bpelQueryFrom.setValue(queryFrom);
						from.setQuery(bpelQueryFrom);
						
					}
					
					to.setVariable(osVariable);
					to.setPart(outPinPart);
					
					String queryTo = createQueryString(os, (Pin)os.getOutputObjectPin().get(j), outPinPart);
					if(queryTo != null && !queryTo.equals("")){
						Query bpelQueryTo = BPELFactory.eINSTANCE.createQuery();
						bpelQueryTo.setQueryLanguage("");
						bpelQueryTo.setValue(queryTo);
						to.setQuery(bpelQueryTo);
					}				
					
					copy.setFrom(from);
					copy.setTo(to);
					((Assign)forkActivity).getCopy().add(copy);
				}						 
			}
		}else{
			
			/* For each pin in the InputSet, there is one or more corresponding pins in the output set */
			for (int i=0, inPinCount = is.getInputObjectPin().size(); i<inPinCount; i++){
				 inPinPart = (Part)((PartRule)	
					TransformationUtil.getRuleForSource(getRoot(),
														PartRule.class,
														(Pin)is.getInputObjectPin().get(i))).getTarget().get(0);
														
	
				for (int j=i, outPinCount = os.getOutputObjectPin().size(); j<outPinCount; j+=inPinCount){
					outPinPart = (Part)((PartRule) 
	//					TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),
						TransformationUtil.getRuleForSource(getRoot(),
															PartRule.class,
															(Pin)os.getOutputObjectPin().get(j))).getTarget().get(0);
	
	
					Copy copy = BPELFactory.eINSTANCE.createCopy();
					From from = BPELFactory.eINSTANCE.createFrom();
					To to = BPELFactory.eINSTANCE.createTo();
					
					from.setVariable(isVariable);
					from.setPart(inPinPart);
					String queryFrom = createQueryString(is, (Pin)is.getInputObjectPin().get(i), inPinPart);
						if(queryFrom != null && !queryFrom.equals("")){
							Query bpelQueryFrom = BPELFactory.eINSTANCE.createQuery();
							bpelQueryFrom.setQueryLanguage("");
							bpelQueryFrom.setValue(queryFrom);
							from.setQuery(bpelQueryFrom);
					}
					
					to.setVariable(osVariable);
					to.setPart(outPinPart);
					
					String queryTo = createQueryString(os, (Pin)os.getOutputObjectPin().get(j), outPinPart);
					if(queryTo != null && !queryTo.equals("")){
						Query bpelQueryTo = BPELFactory.eINSTANCE.createQuery();
						bpelQueryTo.setQueryLanguage("");
						bpelQueryTo.setValue(queryTo);
						from.setQuery(bpelQueryTo);						
					}				
					
					copy.setFrom(from);
					copy.setTo(to);
					((Assign)forkActivity).getCopy().add(copy);
				}						 
			}
		}
	}
	
	private Variable getVariableForPinSet(PinSet pinSet){
		ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
		crIS.getSource().add(pinSet);
		getChildRules().add(crIS);
		crIS.transformSource2Target();
		//getTarget().addAll(crIS.getTarget());
		Variable variable = (Variable)crIS.getTarget().get(0);
		return variable;
	}
	


} //ForkRuleImpl
