/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.GenericActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;




import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Generic Action Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class GenericActionRuleImpl extends ConnectableRuleImpl implements GenericActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private boolean fFirstTimeExecuting=true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected GenericActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.GENERIC_ACTION_RULE;
	}

	/**
	 * Calls the method to generate the mapped representation of the input set, and connects
	 * the result via a BPEL link to the target activity, if required
	 * @param pinSet The input set to map
	 * @param target The target activity to connect to the input set representation
	 */
	private void connectInputSet(InputPinSet pinSet, Activity target){
		/* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(pinSet);
		Activity inputSetActivity = null;
		if (isResultList.isEmpty()|| !(isResultList.get(0) instanceof Activity)){
			inputSetActivity = target;
		} else {
			inputSetActivity = (Activity)isResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(inputSetActivity, 
									   target, 
									   BomUtils.getCanonicalName(pinSet) + "_to_" + pinSet.getAction().getName());
				
			isResultList.add(link);
				
			/* Propagate results from the input set mapping */
			getTarget().addAll(isResultList);
		}

		setTargetOfLinks(pinSet, inputSetActivity);	
	}

	/**
	 * Calls the method to generate the mapped representation of the output set, and connects
	 * the source to the result via a BPEL link, if required.
	 * 
	 * @param pinSet The output pin set to map
	 * @param source The BPEL activity to connect the output set reprsentation to
	 */	
	private void connectOutputSet(OutputPinSet pinSet, Activity source){
		List osResultList = mapOutputPinSet(pinSet);
		Activity outputSetActivity = null;
		if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity) ){
			outputSetActivity = source;
		} else {
			outputSetActivity = (Activity)osResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(source, 
									   outputSetActivity, 
									   pinSet.getAction().getName() + "_to_" + BomUtils.getCanonicalName(pinSet));
				
			osResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(osResultList);
		}

		setSourceOfLinks(pinSet, outputSetActivity);	
	}

	/**
	 * source = input set of a BOM Join node
	 * target = BPEL <sequence>
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
		
		InputPinSet is = (InputPinSet)getSource().get(0);
		if (!fFirstTimeExecuting){
			reExecute(is);
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		} else {
			fFirstTimeExecuting = false;
		}
		
		//create the list of variables for the input/outputPinSet
		List isVariables = getVariablesForSet(is);
		OutputPinSet os = null;
		/* Assume the input set correlated to a single output set */
		if(is.getOutputPinSet().size() >0 ){
			os = (OutputPinSet)is.getOutputPinSet().get(0);
		}
		List osVariables = getVariablesForSet(os);
		
		Activity genericAct = null;				
		/* Create an AlterativeActivity with two alternatives, one for BPEL and one for BPEL+ since
		 * BPEL+ doesn't support XPath expressions in <assign> activities */
		genericAct = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();		
		Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);
			
		((AlternativeActivity)genericAct).getAlternative().add(bpelAlt);
		((AlternativeActivity)genericAct).getAlternative().add(bpelpAlt);
			
		/* BPEL: Create <assign>, with multiple BPEL Copies, if there is at least one object pin */
		bpelAlt.setActivity(BPELFactory.eINSTANCE.createEmpty());
			
		/* BPEL+: Create <invoke><wpc:script><wpc:javaCode>... */
//		bpelpAlt.setActivity(createJavaInvoke(is, isVariable, os, osVariable));
		bpelpAlt.setActivity(createJavaInvoke(is, null, os, null));


		NameProvider nameProvider = NameProviderFactory.getNameProvider(genericAct);
		String convertedName = nameProvider.getName(genericAct, is.getAction(), NamingUtil.findRegistry(this));

//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String convertedName = converter.convertName(NamingUtil.findRegistry(this), genericAct, BomUtils.getCanonicalName(is.getAction()));
		genericAct.setName(convertedName);
		
		
		//genericAct.setName(BomUtils.getCanonicalName(is.getAction()));		
		// Ray: added display name support
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		dspName.setText(is.getAction().getName());
		genericAct.getEExtensibilityElements().add(dspName);
		mapDescriptionAndDocumentation(is.getAction(),genericAct);
				
		/* Connect the incoming links */
		connectInputSet(is, genericAct);

		/* Add the genericAct to the list of targets AFTER the input set is mapped */
		getTarget().add(genericAct);
		
		/* Create the concurrent branch which should be necessary, given that this is a fork */
		if (os!=null && BomHelper.getInstance().isCBranch(os)){
			ConcurrentBranchRule cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
			cbr.getSource().add(os);
			getChildRules().add(cbr);
			cbr.transformSource2Target();
			getTarget().addAll(cbr.getTarget());
		}
				
		/* Connect the outgoing links */
		if (os != null) {
			connectOutputSet(os, genericAct);	
		}
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}
	
	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable
	 */
	private Invoke createJavaInvoke(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable){
		Invoke genericAct = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaAssign = "// Add Implementation Codes here";
		jsActivity.setJavaCode(javaAssign);
		genericAct.getEExtensibilityElements().add(jsActivity);
		
		return genericAct;
	}
	
	/**
	 * Creates an empty BPEL Assign activity
	 * 
	 * @param is The InputPinSet that is providing the information to copy
	 * @param isVariable The BpEL variable that actually holds the source information
	 * @param os The OutputPinSet that the information is being copied to
	 * @param osVariable The BPEL variable that will hold the information once it is copied 
	 * @return Assign - The BPEL Assign activitiy with potentially many Copy elements
	 */
	private Assign createAssign(InputPinSet is, List isVariables, OutputPinSet os, List osVariables){
		
		return BPELFactory.eINSTANCE.createAssign();

		
	}
	
//	/**
//	 * Creates the Java version of the assignment to copy the input pin information to the 
//	 * corresponding output pin variable
//	 */
//	private Invoke createJavaInvoke(InputPinSet is, List isVariables, OutputPinSet os, List osVariables){
//		Invoke mapActivity = AbstractbpelUtil.createJavaInvoke();
//		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
//		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
//		String javaAssign = "// Add Implementation Codes here";
//		if (isVariables!=null && osVariables!=null) {
//			javaAssign += "\n// Map the contents of variables '"+  getVariablesListName(isVariables) + "' to variable '"+ getVariablesListName(osVariables) +"'";
//		}	
//
//		jsActivity.setJavaCode(javaAssign);
//		mapActivity.getEExtensibilityElements().add(jsActivity);
//		
//		return mapActivity;
//	}
//	
//	private String getVariablesListName(List variables){
//		StringBuffer buf = new StringBuffer();
//		Iterator it = variables.iterator();
//		while(it.hasNext()){
//			buf.append(((BPELVariable)it.next()).getName());
//			buf.append(" ");
//		}
//		
//		return buf.toString();
//	}
//	
//	
//	private List createVariables(PinSet set){
//		List variables = new LinkedList();
//		List pins = BomUtils.getObjectPinsForSet(set);
//		if(pins != null && !pins.isEmpty()){
//			Iterator it = pins.iterator();
//			while(it.hasNext()){
//				ObjectPin objectPin = (ObjectPin)it.next();
//				BPELVariable variable = createVariable(objectPin);
//				if(variable != null){
//					getTarget().add(variable);
//					variables.add(variable);
//				}
////				VariableRule variableRule = AbstractbpelFactory.eINSTANCE.createVariableRule();
////				variableRule.getSource().add(objectPin.getType());
////				variableRule.getSource().add(objectPin);
////				getChildRules().add(variableRule);
////				if(variableRule.transformSource2Target()){
////					BPELVariable variable = (BPELVariable)getTarget().get(0);
////					variables.add(variable);
////					//TODO AR new check 
////					getTarget().add(variable);
////				}
//			}
//		}
//		return variables;
//	}

} //GenericActionRuleImpl
