/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.humantasks.EmailEscalationAction;
import com.ibm.btools.bom.model.processes.humantasks.EmailMessage;
import com.ibm.btools.bom.model.processes.humantasks.Escalation;
import com.ibm.btools.bom.model.processes.humantasks.EscalationAction;
import com.ibm.btools.bom.model.processes.humantasks.EscalationChain;
import com.ibm.btools.bom.model.processes.humantasks.Form;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.bom.model.processes.humantasks.WorkItemState;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.TransformationSession;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.FormRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HTBusinessCalendarRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.StaffActivityRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpFactory;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.util.datatype.BTDataTypeException;
import com.ibm.btools.util.datatype.BTDataTypeManager;
import com.ibm.btools.util.datatype.BTDuration;
import com.ibm.btools.util.datatype.IBTDataTypeRegistry;
import com.ibm.btools.util.localeservice.LocaleRegistry;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.CustomProperty;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.tel.DocumentRoot;
import com.ibm.wbit.tel.TActivationStates;
import com.ibm.wbit.tel.TAtLeastExpectedStates;
import com.ibm.wbit.tel.TCustomClientSettings;
import com.ibm.wbit.tel.TCustomSetting;
import com.ibm.wbit.tel.TDisplayName;
import com.ibm.wbit.tel.TDocumentation;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TEscalationActions;
import com.ibm.wbit.tel.TEscalationChain;
import com.ibm.wbit.tel.TEscalationReceiver;
import com.ibm.wbit.tel.TEscalationSettings;
import com.ibm.wbit.tel.TLocalizedEmail;
import com.ibm.wbit.tel.TPotentialOwner;
import com.ibm.wbit.tel.TStaffSettings;
import com.ibm.wbit.tel.TTask;
import com.ibm.wbit.tel.TUISettings;
import com.ibm.wbit.tel.TVerb;
import com.ibm.wbit.tel.TaskFactory;
import com.ibm.wbit.tel.util.CustomTaskFactory;
import com.ibm.wbit.tel.util.TaskResourceImpl;
import com.ibm.wsspi.sca.scdl.Component;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Human Task Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class HumanTaskRuleImpl extends AbstractProcDefRuleImpl implements HumanTaskRule {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private static final String WSDL_NAMESPACE_PREFIX = "wsdl"; 
	private static final String NAME_SEPARATOR="_";
	private HashMap emailMessageMap;
	private String emailPrefix="<html>\n<head>\n</head>\n<body>\n<p>";  //TODO: should this be escaped xml tags
	private String emailSuffix="</p>\n</body>\n</html>"; //TODO: should this be escaped xml tags
	private HashMap <Escalation,TEscalation> escalationsForPostTransform= new HashMap<Escalation,TEscalation>();
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected HumanTaskRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.HUMAN_TASK_RULE;
	}

	/**
	 * Maps a BOM Action to a DocumentRoot HumanTask .
	 * 
	 * SOURCE = [Action, PortType] 
	 * TARGET = tel DocumentRoot 
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");


	  //assertion
	  if (getSource() == null || getSource().isEmpty()) {
		LoggingUtil.traceExit(this, "transformSource2Target", "no source");
		return false;	  	
	  }
	  
	  	Action action = (Action)getSource().get(0);
		PortType portType = (PortType)getSource().get(1);
			
		//we assume using the first Operation of the PortType
		Operation operation = (Operation)portType.getOperations().get(0);
		DocumentRoot documentRoot;
		if (action instanceof HumanTask) {
			HumanTask ht = (HumanTask)action;	
		
			documentRoot=createTELFromHumanTask(ht,portType,operation);
			// process Business Calendar
			processBusinessCalendar(ht, documentRoot);
		} else {
			documentRoot=createTELFromResourceRequirement(action,portType,operation);
		}
//GJD add doc root as the first element		getTarget().add(documentRoot);
		getTarget().add(0, documentRoot);
		
		TransformationSession session = TransformationEngine.getTransformationSession();
		TransformationSessionUtil.setHasHumanTask(session);
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
	  return isComplete();
	}
	
	private DocumentRoot createTELFromHumanTask(HumanTask action, PortType portType, Operation operation) {
		String name = action.getName();
		String namespace = createTargetNamespace(action);
		URI uri = URI.createURI(namespace);
		URI relativeWSDLLocation = URI.createURI(portType.getEnclosingDefinition().getQName().getLocalPart() + ExportOperation.WSDL_FILE_EXT);
		//set the TaskResource flag to true. Even if a little early as we do not have a Resource until now, 
		//we have to set the TaskResource flag to true in order to avoid attempting to use the ArtifactLoader.
		TaskResourceImpl.setUseImports(true);
		TTask task = CustomTaskFactory.getInstance().createDefaultPTask(name,uri, portType, operation, relativeWSDLLocation, portType.getQName().getNamespaceURI(), AbstractbpelConstants.WSDL_IMPORT_TYPE, false);

		// Ray - d 156354
		//task.setJndiNameStaffPluginProvider(HumanTaskUtil.JNDI_NAME_STAFF_PLUGIN_PROVIDER_USERREGISTRY);
		//LI16509 for direct to deploy use the runtime people directory as per the mapping file is exists, else, use the default
		String peopleDirectoryName = HumanTaskUtil.getRuntimePeopleDirectoryName();
		//defect 83690
		if(peopleDirectoryName != null && !"".equals(peopleDirectoryName)){
			task.setJndiNameStaffPluginProvider(peopleDirectoryName);
		}else{
			task.setJndiNameStaffPluginProvider(HumanTaskUtil.JNDI_NAME_STAFF_PLUGIN_PROVIDER_VMM);
		}
		//task.setSupportsSubTask(TBoolean.YES_LITERAL);
		DocumentRoot documentRoot = TaskFactory.eINSTANCE.createDocumentRoot();
		documentRoot.getXMLNSPrefixMap().put(WSDL_NAMESPACE_PREFIX, portType.getQName().getNamespaceURI());
		documentRoot.setTask(task);
		// get unique name
		name = HumanTaskUtil.getTelName(getContext(),action,task,AttributeNameConstants.TEL_TASK_NAME);
		task.setName(name);
		//defect 75442.. consistently set the duration until deleted to null.
		if (task.getDurationUntilDeleted() != null) {
			task.setDurationUntilDeleted(null);
		}
		// Set display name
//		TDisplayName tDisplayName=TaskFactory.eINSTANCE.createTDisplayName();
//		String displayName = getTelName(action,task,AttributeNameConstants.TEL_TASK_DISPLAY_NAME);
//		tDisplayName.setValue(displayName);
//		task.getDisplayName().add(tDisplayName);
		
		// Transform Staff Settings + Potential Owner
//		StaffActivityRule staffActivityRule = AbstractbpelFactory.eINSTANCE.createStaffActivityRule();
//		getChildRules().add(staffActivityRule);
//		staffActivityRule.getSource().add(action.getPotentialOwner());
//		boolean completed = staffActivityRule.transformSource2Target();
//		if(completed){
		if (action.getPotentialOwner()!=null) {
			
			TVerb verb = transformResourceRequirement(action.getPotentialOwner());
//			if(staffActivityRule.getTarget() != null && !staffActivityRule.getTarget().isEmpty() && staffActivityRule.getTarget().get(0) instanceof TVerb){
				TStaffSettings staffSettings = TaskFactory.eINSTANCE.createTStaffSettings();
			  	//List resourceRequirements = getSource();
			  	//TVerb verb = (TVerb)staffActivityRule.getTarget();
			  	TPotentialOwner potOwner = TaskFactory.eINSTANCE.createTPotentialOwner();
				potOwner.setVerb(verb);
				staffSettings.setPotentialOwner(potOwner);
				task.setStaffSettings(staffSettings);
//			}
		}
		
		// Transform Email Message
		EmailMessage emailMessage;
		int i=0;
		for (Iterator it=action.getEmailMessages().iterator();it.hasNext();i++) {
			emailMessage = (EmailMessage)it.next();
			TEmail tEmailMessage = transformEmailMessage(emailMessage);
			// The first email message should always be the default email message.
			// We need to set the message name to the WID default email message name
			// to allow the WID UI to recognize it is the default email message.
			if (i==0) {
				tEmailMessage.setName(HumanTaskUtil.DEFAULT_EMAIL_NAME);
			}
			task.getEmail().add(tEmailMessage);
			getEmailMessageMap().put(emailMessage, tEmailMessage);
		}
		// Transform Escalation
		// when HT using business calendar set duration in xsd:duration format
		boolean hasBusinessCalendar = false;
		if (action.getRecurringTime() != null) {		// do not check the content
			hasBusinessCalendar = true;
		}
		
		if (!action.getEscalationChains().isEmpty()) {
			TEscalationSettings escalationSettings = transformEscalations(action.getEscalationChains(), hasBusinessCalendar);
			boolean isEmailAction = postTransformEscalations(escalationsForPostTransform);
			task.setEscalationSettings(escalationSettings);
			// d165010
			if (isEmailAction) {
				task.setJndiNameStaffPluginProvider(HumanTaskUtil.JNDI_NAME_STAFF_PLUGIN_PROVIDER_VMM);
			}
		}
		//Feature 16512, for the case we generate for Deploy, we should generate the default forms 
//		if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
		//defect 81481, should enable the generation of default forms for all HumanTasks
		boolean enableGenerationOfDefaultForms = true;
		if(enableGenerationOfDefaultForms){
			InputPinSet inputSet = (InputPinSet)action.getInputPinSet().get(0);
			TUISettings uiSettings = HumanTaskUtil.createUISetting(inputSet, documentRoot, this);
			task.setUiSettings(uiSettings);
		}else{
			// Transform UI Setting
			if (action.getOutputForm()!=null || action.getInputForm()!=null) {
				TUISettings uiSettings = createUISetting(action);
				task.setUiSettings(uiSettings);
			}
		}				
	
		return documentRoot;
		
	}
	
	private TUISettings createUISetting(HumanTask action) {
		TUISettings uiSettings = TaskFactory.eINSTANCE.createTUISettings();
		TCustomClientSettings customClientSettings = TaskFactory.eINSTANCE.createTCustomClientSettings();
		customClientSettings.setClientType(HumanTaskUtil.CLIENT_SETTING_TYPE_LOTUS_FORMS);
		uiSettings.getCustomClientSettings().add(customClientSettings);
		TCustomSetting customSetting;
		if (action.getInputForm()!=null) {
			customSetting = createCustomSetting(action.getInputForm(),HumanTaskUtil.CUSTOM_SETTING_INPUT_FORM);
			customClientSettings.getCustomSetting().add(customSetting);
		}
		if (action.getOutputForm()!=null) {
			customSetting = createCustomSetting(action.getOutputForm(),HumanTaskUtil.CUSTOM_SETTING_OUTPUT_FORM);
			customClientSettings.getCustomSetting().add(customSetting);
		}
		return uiSettings;	
	}
	
	private TCustomSetting createCustomSetting(Form form, String formName) {
		// triggers form transformation if not already done so 
		if (TransformationUtil.getRuleForSource(this.getRoot(), form) == null) {
			FormRule formRule = AbstractbpelFactory.eINSTANCE.createFormRule();
			this.getRoot().getChildRules().add(formRule);
			formRule.getSource().add(form);
			formRule.transformSource2Target();
		}
		TCustomSetting customSetting = TaskFactory.eINSTANCE.createTCustomSetting();
		customSetting.setName(formName);
		String formFilename = HumanTaskUtil.getAttachedFormFilename(form);
		// set the form uid at this stage and replace it with the actual file location when the xfdl is being persisted 
		customSetting.setValue(form.getUid()+"/"+formFilename);     
		return customSetting;
	}
	
	private DocumentRoot createTELFromResourceRequirement(Action action, PortType portType, Operation operation) {
		String name = getUniqueName(action);
		String namespace = createTargetNamespace(action);
		URI uri = URI.createURI(namespace);
		URI relativeWSDLLocation = URI.createURI(portType.getEnclosingDefinition().getQName().getLocalPart() + ExportOperation.WSDL_FILE_EXT);
		//set the TaskResource flag to true. Even if a little early as we do not have a Resource until now, 
		//we have to set the TaskResource flag to true in order to avoid attempting to use the ArtifactLoader.
		TaskResourceImpl.setUseImports(true);
		TTask task = CustomTaskFactory.getInstance().createDefaultPTask(name,uri, portType, operation, relativeWSDLLocation, portType.getQName().getNamespaceURI(), AbstractbpelConstants.WSDL_IMPORT_TYPE, false);
		//task.setSupportsSubTask(TBoolean.YES_LITERAL);
		DocumentRoot documentRoot = TaskFactory.eINSTANCE.createDocumentRoot();
		documentRoot.getXMLNSPrefixMap().put(WSDL_NAMESPACE_PREFIX, portType.getQName().getNamespaceURI());
		documentRoot.setTask(task);
		//if(action.getResourceRequirement() != null && !action.getResourceRequirement().isEmpty()){
		//defect 118064
		if(!HumanTaskUtil.isUse61HumanTaskSupport() && StaffUtil.isStaffRequired(action)){
			StaffActivityRule staffActivityRule = AbstractbpelFactory.eINSTANCE.createStaffActivityRule();
			getChildRules().add(staffActivityRule);
			staffActivityRule.getSource().addAll(action.getResourceRequirement());
			boolean completed = staffActivityRule.transformSource2Target();
			if(completed){
				if(staffActivityRule.getTarget() != null && !staffActivityRule.getTarget().isEmpty() && staffActivityRule.getTarget().get(0) instanceof TStaffSettings){	
					task.setStaffSettings((TStaffSettings)staffActivityRule.getTarget().get(0));
				}
			}
		}
		return documentRoot;
	}
	/**
	 * Builds the TargetNamespace for the input action.
	 * @param action a BOM Action
	 * @return String targetNamespace
	 */
	private String createTargetNamespace(Action action){
		// RK D130847
		//String encodedNamespace = ProcessUtil.createNamespace(action);
		String encodedNamespace = ProcessUtil.createNamespace(action,true);
		return encodedNamespace;
	}
	


	/**
	 * Returns an unique name of the source BOM NamedElement. The HumanTask will get the name of the component, if one was already created for it, this is the 
	 * case of a Component with implementation HumanTask. If the component was not yet created (this is the case of an Invoke) - an inline HumanTask, the name of
	 * the HumanTask will be set from the name of the action.  
	 * 
	 * @param fromSource
	 * @return
	 */
	private String getUniqueName(NamedElement fromSource) {
		String fullName = BomUtils.getFullName(fromSource);
		JavaNCNameConverter np = new JavaNCNameConverter();
		String name = null;
		ComponentRule compRule = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(), ComponentRule.class, getSource(), com.ibm.wsspi.sca.scdl.DocumentRoot.class);
		if(compRule != null){
			com.ibm.wsspi.sca.scdl.DocumentRoot root = (com.ibm.wsspi.sca.scdl.DocumentRoot )compRule.getTarget().get(0);
			Component component = root.getComponent();
			if (component != null) {
				//get name of the component, this will be either the default name or the name from the TA
				AttributeValueProvider avp = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
				avp.setContext(this.getContext());
				name = (String)avp.getAttributeValueForType(component,fromSource,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));
			} else if (root.getImport() != null) {
				//get name of the component, this will be either the default name or the name from the TA
				AttributeValueProvider avp = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
				avp.setContext(this.getContext());
				name = (String)avp.getAttributeValueForType(root.getImport(),fromSource,AttributeNameConstants.COMPONENT_NAME,NamingUtil.getDefaultRegistry(getContext(),NamingUtil.DEFAULT_REGISTRY_SCDL));				
			}
			if (name.indexOf('/')!=-1) {
				name = name.substring(name.lastIndexOf('/')+1);
			}
		}else{
			name = np.convertName(fromSource.getName() + NAME_SEPARATOR + NamingUtil.getKeyFromString(fullName));
		}
		return name;

	}

	private HashMap getEmailMessageMap() {
		if (emailMessageMap == null) {
			emailMessageMap = new HashMap();
		}
		return emailMessageMap;
	}
	
	private TEmail transformEmailMessage (EmailMessage emailMessage) {
		TEmail tEmail = TaskFactory.eINSTANCE.createTEmail();
		// create email message
		String emailName = HumanTaskUtil.getTelName(getContext(),emailMessage,tEmail,AttributeNameConstants.TEL_EMAIL_NAME);
		tEmail.setName(emailName); 
		TLocalizedEmail tLocalizedEmail = TaskFactory.eINSTANCE.createTLocalizedEmail();
		tLocalizedEmail.setSubject(emailMessage.getSubject());
		tLocalizedEmail.setLocale(LocaleRegistry.instance().getStartupLocale().toString());
		StringBuffer stringBuffer = new StringBuffer();
		stringBuffer.append(emailPrefix);
		stringBuffer.append(emailMessage.getBody());
		stringBuffer.append(emailSuffix);
		tLocalizedEmail.setBody(stringBuffer.toString());
		tEmail.getLocalizedEmail().add(tLocalizedEmail);
		return tEmail;
	}
	
	/**
	 * Transform the escalation settings in Human Task to TEL escalation settings
	 * @param escalationChains
	 * @return
	 */
	private TEscalationSettings transformEscalations(List escalationChains, boolean hasBusinessCalendar) {
		TEscalationSettings tEscalationSettings = TaskFactory.eINSTANCE.createTEscalationSettings();
		EscalationChain escalationChain;
		TEscalationChain tEscalationChain;
		for (Iterator it=escalationChains.iterator();it.hasNext();) {
			escalationChain = (EscalationChain)it.next();
			tEscalationChain = transformEscalationChain(escalationChain, hasBusinessCalendar);
			tEscalationSettings.getEscalationChain().add(tEscalationChain);
		}
		return tEscalationSettings;
		
	}
	
	/**
	 * Transform the escalation chain in Human Task to TEL escalation chain
	 * @param escalationChain
	 * @return
	 */
	private TEscalationChain transformEscalationChain(EscalationChain escalationChain, boolean hasBusinessCalendar) {
		TEscalationChain tEscalationChain = TaskFactory.eINSTANCE.createTEscalationChain();
		// Set the Activation State of the escalation chain
		if (escalationChain.getActivationState()==WorkItemState.READY_LITERAL) {
			tEscalationChain.setActivationState(TActivationStates.READY_LITERAL);
		} else if (escalationChain.getActivationState()==WorkItemState.CLAIMED_LITERAL) {
			tEscalationChain.setActivationState(TActivationStates.CLAIMED_LITERAL);
		} else if (escalationChain.getActivationState()==WorkItemState.WAITING_FOR_SUB_TASK_LITERAL) {
			tEscalationChain.setActivationState(TActivationStates.WAITING_FOR_SUB_TASK_LITERAL);
		}
		Escalation escalation;
		TEscalation tEscalation;
		for (Iterator it=escalationChain.getEscalations().iterator();it.hasNext();) {
			escalation = (Escalation)it.next();
			tEscalation = transformEscalation(escalation, hasBusinessCalendar);
			tEscalationChain.getEscalation().add(tEscalation);
			escalationsForPostTransform.put(escalation,tEscalation);
		}
		return tEscalationChain;
	}
	
	/**
	 * Transforma BOM LiteralDuration to runtime duration string
	 * @param literalDuration
	 * @return
	 */
	private String transformDuration(LiteralDuration literalDuration) {
		BTDuration dataType = (BTDuration)BTDataTypeManager.instance.newInstance(IBTDataTypeRegistry.PREDEF_DURATION);
		try {
			dataType.setValue(literalDuration.getValue());	
			int day = dataType.getDay();
			int hour = dataType.getHour();
			int minute = dataType.getMinute();
			double second = dataType.getSecond();
			StringBuffer transformedDuration = new StringBuffer();
            
			//defect 162816
//			if(second < 0) second = 0.0;

			//defect 164950
			Double d = new Double(second);
			int secondInt = d.intValue();
			//transformedDuration.append("0");
			if (secondInt>0) {
				transformedDuration.append(secondInt);
				transformedDuration.append(HumanTaskUtil.DURATION_SECS);									
			}			
//			if (minute<0) minute=0;
			if (minute>0) {
				if (secondInt>0) {
					transformedDuration.append(" ");
				}
				transformedDuration.append(minute);
				transformedDuration.append(HumanTaskUtil.DURATION_MINS);									
			}
							
//			if (hour<0) hour=0;
			if (hour>0) {
				if (minute > 0 || second > 0) {
					transformedDuration.append(" ");										
				}
				transformedDuration.append(hour);
				transformedDuration.append(HumanTaskUtil.DURATION_HRS);									
			}
//			if (day<0) day=0;
			if (day>0) {
				if (hour>0 || minute > 0 || second > 0) {
					transformedDuration.append(" ");										
				}
				transformedDuration.append(day);
				transformedDuration.append(HumanTaskUtil.DURATION_DAYS);									
			}

//			if (day!=0) {
//				transformedDuration.append(day);
//				if (day==1) {
//					transformedDuration.append(HumanTaskUtil.DURATION_DAY);
//				} else {
//					transformedDuration.append(HumanTaskUtil.DURATION_DAYS);					
//				}
//			}
//			if (hour != 0) {
//				transformedDuration.append(hour);
//				if (hour==1) {
//					transformedDuration.append(HumanTaskUtil.DURATION_HR);
//				} else {
//					transformedDuration.append(HumanTaskUtil.DURATION_HRS);					
//				}				
//			}
//			if (minute != 0) {
//				transformedDuration.append(minute);
//				if (minute==1) {
//					transformedDuration.append(HumanTaskUtil.DURATION_MIN);
//				} else {
//					transformedDuration.append(HumanTaskUtil.DURATION_MINS);					
//				}				
//			}
			return transformedDuration.toString();
		} catch (BTDataTypeException dtException) {
			return null;
		}
	}
	
	/**
	 * Transform escalation details
	 * @param escalation
	 * @return
	 */
	private TEscalation transformEscalation (Escalation escalation, boolean hasBusinessCalendar) {
		TEscalation tEscalation = TaskFactory.eINSTANCE.createTEscalation();
		// Set name
		String escalationName = HumanTaskUtil.getTelEscalationName(getContext(), (HumanTask)getSource().get(0), escalation.getName());
		if (escalationName==null) {
			escalationName = HumanTaskUtil.getTelName(getContext(),escalation,tEscalation,AttributeNameConstants.TEL_ESCALATION_NAME);
			HumanTaskUtil.registerTelEscalationName(getContext(), (HumanTask)getSource().get(0), escalation.getUid(), escalationName);
		}
		//defect 170483
		if(escalation.getOwnedComment() != null && !escalation.getOwnedComment().isEmpty()){
			TDocumentation documentation = TaskFactory.eINSTANCE.createTDocumentation();
			documentation.setValue(((Comment)escalation.getOwnedComment().get(0)).getBody());
			documentation.setLocale(LocaleRegistry.instance().getStartupLocale().toString());
			tEscalation.getDocumentation().add(documentation);
		}
		tEscalation.setName(escalationName);
		// Set display name
		TDisplayName tDisplayName=TaskFactory.eINSTANCE.createTDisplayName();
		tDisplayName.setLocale(LocaleRegistry.instance().getStartupLocale().toString());
		tDisplayName.setValue(HumanTaskUtil.getTelName(getContext(),escalation,tEscalation,AttributeNameConstants.TEL_ESCALATION_DISPLAY_NAME));
		tEscalation.getDisplayName().add(tDisplayName);

		// Set the Expected State
		if (escalation.getDesiredState()==WorkItemState.CLAIMED_LITERAL) {
			tEscalation.setAtLeastExpectedState(TAtLeastExpectedStates.CLAIMED_LITERAL);
		} else if (escalation.getDesiredState()==WorkItemState.SUB_TASKS_COMPLETED_LITERAL) {
			tEscalation.setAtLeastExpectedState(TAtLeastExpectedStates.SUB_TASKS_COMPLETED_LITERAL);
		}  else if (escalation.getDesiredState()==WorkItemState.ENDED_LITERAL) {
			tEscalation.setAtLeastExpectedState(TAtLeastExpectedStates.ENDED_LITERAL);
		}
		// transform escalation period
		ValueSpecification escalationPeriod = escalation.getEscalationPeriod();
		if (escalationPeriod!=null) {
			if (hasBusinessCalendar) {
				tEscalation.setDurationUntilEscalation(((LiteralDuration)escalationPeriod).getValue());
			} else {
				String tranformedDuration = transformDuration((LiteralDuration)escalationPeriod);
				tEscalation.setDurationUntilEscalation(tranformedDuration);				
			}
		} else {
			tEscalation.setDurationUntilEscalation(HumanTaskUtil.DEFAULT_DURATION_UNTIL_ESCALATION);
		}
		// Transform escalation action
		EscalationAction escalationAction = escalation.getEscalationAction();
		// transform repetition period
		ValueSpecification repetitionPeriod = escalationAction.getRepetitionPeriod();
		if (repetitionPeriod!=null) {
			if (hasBusinessCalendar) {
				tEscalation.setAutoRepeatDuration(((LiteralDuration)repetitionPeriod).getValue());
			} else {
				String transformedDuration = transformDuration((LiteralDuration)repetitionPeriod);
				tEscalation.setAutoRepeatDuration(transformedDuration);
			}
		}		
		return tEscalation;
	}	

	/**
	 * The reason for doing a post-transformation of the escalation is to ensure the name of all escalations are transformed
	 * before the query is transformed because the query the uses context variable may require putting a reference name of the
	 * escalation.  Hence we have to ensure the transformed names are all available first
	 * returns true if email escalation is defined.  this is to allow to set the correct people directory 
	 * @param escalationsMap
	 */
	private boolean postTransformEscalations(HashMap escalationsMap) {
		boolean ret=false;
		Set escalations = escalationsMap.keySet();
		for (Iterator it = escalations.iterator();it.hasNext();) {
			Escalation escalation = (Escalation)it.next();
			TEscalation tEscalation = (TEscalation)escalationsMap.get(escalation);
			// transform escalation receiever
			EscalationAction escalationAction = escalation.getEscalationAction();
			ResourceRequirement receiver = escalationAction.getEscalationReceiver();
			TVerb tVerb = null;
			if (receiver!=null) {
				tVerb = transformResourceRequirement(receiver);
				// Temporary fix for 165011
				if (escalationAction instanceof EmailEscalationAction) {
					if (tVerb.getName().equals(StaffUtil.StaffConstants.SSS_USERS_BY_USERID )){
						tVerb.setName(StaffUtil.StaffConstants.SSS_USER_RECORDS_BY_USERID);
					}					
				}
				TEscalationReceiver tEscalationReceiver = TaskFactory.eINSTANCE.createTEscalationReceiver();
				tEscalationReceiver.setVerb(tVerb);
				tEscalation.setEscalationReceiver(tEscalationReceiver);			
			}
			if (escalationAction instanceof EmailEscalationAction) {
				ret=true;
				tEscalation.setEscalationAction(TEscalationActions.EMAIL_LITERAL);	
				TEmail tEmail = (TEmail)getEmailMessageMap().get(((EmailEscalationAction)escalationAction).getEmailMessage());
				if (tEmail!=null) {
					tEscalation.setEmail(tEmail.getName());
				}
				//defect 73906/73464. Do not generate EmailReceiver
				// set emailReceiver query to be the escalation receiver query
//				TEMailReceiver tEMailReceiver = TaskFactory.eINSTANCE.createTEMailReceiver();
//				tEMailReceiver.setVerb((TVerb)EcoreUtil.copy(tVerb));
//				tEscalation.setEMailReceiver(tEMailReceiver);				
			} else {
				// This is WorkItemEscalationAction
				tEscalation.setEscalationAction(TEscalationActions.WORK_ITEM_LITERAL);
			}
			
		}
		return ret;
	}
	/**
	 * Transform ResourceRequirement to Staff Verb
	 * @param resourceRequirement
	 * @return
	 */
	private TVerb transformResourceRequirement(ResourceRequirement resourceRequirement) {
		StaffActivityRule staffActivityRule = AbstractbpelFactory.eINSTANCE.createStaffActivityRule();
		staffActivityRule.getSource().add(resourceRequirement);
		getChildRules().add(staffActivityRule);
		staffActivityRule.transformSource2Target();
		return (TVerb)staffActivityRule.getTarget().get(0);

	}
	
	
	/**
	 * Transform BusinessCalendar for HumanTask
	 * @param ht HumanTask
	 * @param documentRoot DocumentRoot
	 * @return
	 */
	private void processBusinessCalendar(HumanTask ht, DocumentRoot documentRoot) {
		if (ht == null || documentRoot == null || ht.getRecurringTime() == null || documentRoot.getTask() == null) {
			return;
		}
		
		// GJD: process recurringTimeIntervals in the TimeIntervalsRule	
		TimeIntervals timeIntervals = ht.getRecurringTime();
		HTBusinessCalendarRule ti = AbstractbpelFactory.eINSTANCE.createHTBusinessCalendarRule();
		getChildRules().add(ti);
		ti.getSource().add(timeIntervals);
		ti.getSource().add(documentRoot);
		ti.transformSource2Target();
			
		ICalendar iCalendar = null;
		if (ti.getTarget() != null) {
			Iterator it = ti.getTarget().iterator();
			Object obj = null;
			while (it.hasNext()) {
				obj = it.next();
				if (obj instanceof ICalendar) {
					iCalendar = (ICalendar)obj;					
					getTarget().add(iCalendar);					
					
					if (iCalendar.getTargetNamespace() == null) { // Note: only the calendar created for this HumanTask does not have the namespace set
						String actionNamespace = createTargetNamespace(ht);
						iCalendar.setTargetNamespace(actionNamespace + TimeConstants.CALENDAR_SUFFIX);
							
						// set the iCalendar on HT:
						documentRoot.getTask().setCalendarJNDIName(TimeConstants.CALENDAR_JNDI_NAME);
						documentRoot.getTask().setCalendarName(actionNamespace + TimeConstants.CALENDAR_SUFFIX +
									 TimeConstants.COLON +
									 // GJD 20080415 - use iCalendar name
									 iCalendar.getName());
						
						ExportSession exportSession = BpelOptionsUtil.getExportSession();
						if(BpelOptions.isBestPracticePatternEnabed(exportSession)  ||
								BpelOptionsUtil.isLibraryOnly() ||
								BpelOptionsUtil.isLibraryEnabledWithModuleProject()){
							Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
							if(docRootRegistry instanceof Set){
								((Set)docRootRegistry).add(iCalendar);
							}			
						}				
					}
				}
			}
		}		
	}
} //HumanTaskRuleImpl
