/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * $Id: InputRuleImpl.java,v 1.10.2.1 2009/04/13 18:47:11 build Exp $
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Input;
import com.ibm.wbit.bpelpp.Parameter;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Input Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class InputRuleImpl extends AbstractProcDefRuleImpl implements InputRule {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	protected InputRuleImpl() {
		super();
	}
	/* attribute indicates whether it is first time execution of this rule */
	 protected boolean ivFirstTimeExec = true;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.INPUT_RULE;
	}

	/**
	 * source = BOM InputPinSet/OutputSet
	 * target = BPELPlus Input
	 * Iterates through each of the ObjectPins in the PinSet and creates for each of them a BPELPlus Parameter.
	 * Reuse the Variable when creating the Parameter.
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
			return true;				
		}
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no sources");
			return false;
		}
		PinSet pinSet = (PinSet)getSource().get(0);
		if(!ivFirstTimeExec && !isComplete()){
			 reExecute(pinSet); 
			 LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			 return true;
		}
		ivFirstTimeExec = false;
		Input input = BPELPlusFactory.eINSTANCE.createInput();
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		if(objectPins != null && !objectPins.isEmpty()){
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin objectPin = (ObjectPin)it.next();
				Parameter parameter = BPELPlusFactory.eINSTANCE.createParameter();
				BPELVariable variable = getVariableForObjectPin(objectPin);
				if(variable != null){
					// GJD 20060831
					if (variable.getType() != null) { // original code
						NameProvider nameProvider = NameProviderFactory.getNameProvider(parameter);		
						nameProvider.setContext(getContext());
						parameter.setName(nameProvider.getDefaultName(parameter, null, objectPin, NamingUtil.findRegistry(this), null));						
					} else {	// match the name of the parameter with the element declaration
						try{							
							parameter.setName(OriginalWsdlUtil.retrievePartNameFromWSDL(getContext(),objectPin));
						}catch(Exception e){
							String params[]={objectPin.getName()};
							LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME, params, e);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME));
							runtimeEx.setChainedException(e);			
							throw runtimeEx;
						}						
					}
					parameter.setVariable(variable); 
				}else{
					setComplete(false);
				}
				input.getParameter().add(parameter);
			}
		}
		
		getTarget().add(input);
		executeHandlers();
		propagateChildTargets();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		
	}
	
	private void reExecute(PinSet pinSet){
	    //	  always execute child rules
		List childRules = getChildRules();
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			((TransformationRule)childRulesIterator.next()).transformSource2Target();		
		}	
	    
		//get the target of this rule.. the Input. 
		//check the Parameter without type.. and get the Pin in the set for that Parameter... 
		//try to set the variable for it based on the incoming link
		//if one still cannot execute, set the rule to false.. 
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		Input input = (Input)getTarget().get(0);
		List parameters = input.getParameter();
		if(parameters == null || parameters.isEmpty()){
			setComplete(true);
			return;
		}else{
			BPELVariable variable = null;
			for(int i=0; i<parameters.size(); i++){
				Parameter parameter = (Parameter)parameters.get(i);
				if(parameter.getVariable()== null){
					ObjectPin objectPin =(ObjectPin) objectPins.get(i);
					//ObjectPin sourcePin = (ObjectPin)((InputObjectPin)objectPin).getIncoming().getSource();
					ArrayList sources = new ArrayList();
					sources.add(objectPin.getType());
					sources.add(objectPin);
					TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), VariableRule.class, sources, BPELVariable.class);
						//if a rule was already created for this Pin, use the variable and add the Pin as source of this rule 
					if(rule != null && !rule.getTarget().isEmpty()){
						variable = (BPELVariable)rule.getTarget().get(0);
						rule.getSource().add(objectPin);
						NameProvider nameProvider = NameProviderFactory.getNameProvider(parameter);
						parameter.setName(nameProvider.getDefaultName(parameter, null, objectPin, NamingUtil.findRegistry(this), null));
						parameter.setVariable(variable);
					}else{
						//the Pin was not yet processed
						setComplete(false);
					}
				}
			}
		}
	}
	
	private BPELVariable getVariableForObjectPin(ObjectPin objectPin){
		
		BPELVariable variable = createVariable(objectPin);
		return variable;

	}
	

} //InputRuleImpl
