/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.StaffActivityRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.PinSetRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.Input;
import com.ibm.wbit.bpelpp.Output;
import com.ibm.wbit.bpelpp.Staff;
import com.ibm.wbit.bpelpp.Task;
import com.ibm.wbit.tel.DocumentRoot;
import com.ibm.wbit.tel.TTask;

/**
 * <!-- begin-user-doc -->
 *  source: 0: InputPinSet of BOM Task, 
 * 		   1: the associated Action, 
 * 		   2: WSDL Operation,
 * 		   3: BPEL Parterner  
 * target: BPEL Invoke
 * 
 * Maps BOM invocation of Task or CallBehaviourAction of the InputPinSet to BPEL Invoke.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class InvokeRuleImpl extends ConnectableRuleImpl implements InvokeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/* attribute indicates whether it is first time execution of this rule */
	protected boolean ivFirstTimeExec = true;
	protected Input ivInput = null;
	protected Output ivOutput = null;
	protected List<Output> ivOutputList = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected InvokeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.INVOKE_RULE;
	}

	/**
	 * Maps BOM invocation of Task or CallBehaviourAction of the InputPinSet to BPEL Invoke.
	 * 
	 * SOURCE = [InputSet, Action, Operation, Partner, (Optional) Callback Operation] 
	 * TARGET = BPEL Invoke (+ objects propagated by children rules)
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

	/* 1. create Container Rules for the InputPinSet and correlated OutputPinSet
	 * 2. create BPEL Invoke using rule's sources - Operation and Parterner
	 * 3. for each pin in the Input Set, set Invoke as the target of the link that representing incoming edge of the pin.
	 * 4. if the Input Set is also a exclusive branch, create ExclusiveBranchRule and execute the rule.
	 * 	  if not, if the Output Set is concurrent branch, create ConcurrentBranchRule and execute the rule.
	 * 5. for each pin in the correlated OutputPinSet, set Invoke as the source of the link that representing outgoing edge of the pin.
	 * 6. propagate all targets produced by child rules.
	 * 7. if staff is required for the action in the input source will create and execute the StaffActivityRule. 
	 * Set the target of the StaffActivityRule to the invoke.  
	 * 	
	*/
	  //assertion
	  if (getSource() == null || getSource().isEmpty()) {
		LoggingUtil.traceExit(this, "transformSource2Target", "no source");
		return false;	  	
	  }
	  //first source of this rule is one input set
	  InputPinSet inputSet = (InputPinSet)getSource().get(0);
	  	
	  //check if this is first time execute this rule
	   if(!ivFirstTimeExec){
		  reExecute(inputSet); 
		  executeHandlers();
		  LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
		  return true;
	   }
	   
	   ivFirstTimeExec = false;
	   
	   createContainerRule();
	   

	   Action action = (Action)getSource().get(1);
	   Invoke invoke = createInvoke(inputSet);
	   getTarget().add(invoke); // Ray: should always set the invoke to target first because the assumption is the first target of this rule is always the invoke
	   	   
		// if "Staff" is assigned to the action, then create the StaffActivity
		// and set it as extensibleElement on the invoke.
	   if(!HumanTaskUtil.isUse61HumanTaskSupport() && StaffUtil.isStaffRequired(action)){
			boolean generate51CompatibleStaffActivity = BpelOptionsUtil.generateStaffActivity51Compliant();
			if(generate51CompatibleStaffActivity){
				StaffActivityRule staffRule = AbstractbpelFactory.eINSTANCE.createStaffActivityRule();
		   		getChildRules().add(staffRule);
		   		staffRule.getSource().addAll(action.getResourceRequirement());
				boolean done = staffRule.transformSource2Target();
		   		if(done){
		   			Staff staff = (Staff)staffRule.getTarget().get(0);
		   			invoke.getExtensibilityElements().add(staff);
		   		}
			}else{
				
				Operation operation = (Operation)getSource().get(2);
				PortType portType =(PortType)operation.eContainer();
				
				Collection htSources = new ArrayList();
				htSources.add(action);
				htSources.add(portType);

//				only generate an inline HumanTask if no component was already created at this point.
//				defect 113347
				ComponentRule rule = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class, htSources,com.ibm.wsspi.sca.scdl.DocumentRoot.class);
				if(rule != null && !rule.getTarget().isEmpty()){
					com.ibm.wsspi.sca.scdl.DocumentRoot docRoot = (com.ibm.wsspi.sca.scdl.DocumentRoot)rule.getTarget().get(0);
					if((docRoot.getComponent() == null || docRoot.getComponent().getImplementation() == null) && docRoot.getImport() == null){
						rule = null;
					}
				}
				if(rule == null){
					//check if there is already a humanTask created for these sources (this may be the case when of multiple InputSets
					HumanTaskRule humanTask = (HumanTaskRule)TransformationUtil.getRuleForSource(getRoot(), HumanTaskRule.class, htSources, DocumentRoot.class);
					if(humanTask == null){
						humanTask = AbstractbpelFactory.eINSTANCE.createHumanTaskRule();
						//attach the HumanTask to the ProcessDefinitionRule
						TransformationRule parentRule = ProcessUtil.getProcessDefinitionRule(getContext());
						parentRule.getChildRules().add(humanTask);
						humanTask.getSource().add(action);
						humanTask.getSource().add(portType);
						//humanTask.getSource().addAll(this.getSource());
			   			humanTask.transformSource2Target();
					}
		   			//AdminTask adminTask = BPELPlusFactory.eINSTANCE.createAdminTask();
					Task adminTask = BPELPlusFactory.eINSTANCE.createTask();
		   			TTask task = ((DocumentRoot)humanTask.getTarget().get(0)).getTask();
		   			//remove the task Display name. 
		   			if(task.getDisplayName() != null && !task.getDisplayName().isEmpty()){
		   				task.getDisplayName().remove(task.getDisplayName().get(0));
		   			}
		   			if (task.getDurationUntilDeleted() != null) {
		   				task.setDurationUntilDeleted(null);
		   			}
		   			adminTask.setName(task);
		   			invoke.getEExtensibilityElements().add(adminTask);
		   			ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(this.getContext());
		   			organizeImports((Process)currentProcessDefRule.getTarget().get(0), (DocumentRoot)humanTask.getTarget().get(0));
				}
				//else we may want to add a warning if the component created is not a HumanTask component.
			}
	   }
	   
	   doInputPinSetConnection(invoke);

	   createBranchRuleIfAny();
	   
	   doOutputPinSetConnection(invoke);

	   // Ray: added support for descriptions and documentations		
	   Description desc = AbstractbpelUtil.transformDescription(action);
	   if (desc!=null) {
		   invoke.addExtensibilityElement(desc);
	   }
	   Documentation doc = AbstractbpelUtil.transformDocumentation(action);
	   if (doc!=null) {
			invoke.addExtensibilityElement(doc);
	   }	   
	   //propagate all targets from children to parent
	   propagateChildTargets();
	   
	   //set complete to true
	   setComplete(true); 

	   executeHandlers();
	   
	   LoggingUtil.traceExit(this, "transformSource2Target");	   	   
		return true;
	}	
	
	/**
	 * Creates container rules for the input set and correlated output sets
	 *
	 */
	protected void createContainerRule(){
		  InputPinSet inputSet = (InputPinSet)getSource().get(0);
		  
		  InputRule cr = AbstractbpelFactory.eINSTANCE.createInputRule();
		  cr.getSource().add(inputSet);
		  getChildRules().add(cr);
		  cr.transformSource2Target();
		  
   		  ivInput = (Input)cr.getTarget().get(0);
   		  
		  // although we only support one-one correlation of input set and output set, 
		  // here implement correlation to multiple output set
		  List osList = inputSet.getOutputPinSet();
		  if(osList.size() > 1){
			  Iterator osIt = osList.iterator();
			  while(osIt.hasNext()){				 
					  OutputPinSet nextOS = (OutputPinSet)osIt.next();	
					if(CBPELUtil.isFaultSetNoConnections(nextOS)){
						 continue;
					  }
					  OutputRule or = AbstractbpelFactory.eINSTANCE.createOutputRule();
					  or.getSource().add(nextOS);
					  getChildRules().add(or);
					  or.transformSource2Target();
                      if(!or.getTarget().isEmpty() && or.getTarget().get(0) instanceof Output){
                    	  ivOutput = (Output)or.getTarget().get(0);
                    	  if(ivOutputList == null){
                    		  ivOutputList = new ArrayList<Output>();					  
                    	  }
                    	  ivOutputList.add(0,ivOutput);	
                      }
			  }
		  }else{		 
			  OutputPinSet nextOS = (OutputPinSet)osList.get(0);
			  OutputRule or = AbstractbpelFactory.eINSTANCE.createOutputRule();
			  or.getSource().add(nextOS);
			  getChildRules().add(or);
			  or.transformSource2Target();

			  ivOutput = (Output)or.getTarget().get(0);
			  if(ivOutputList == null){
				  ivOutputList = new ArrayList<Output>();				 
			  }
			  ivOutputList.add(0,ivOutput);
		  }
		  
	}
	
	/**
	 * Creates ExclusiveBranchRule or ConcurrentBranchRule. 
	 * The branch rule will map all elements in the branch region directly following the invoked Task or CallBehaviourAction.
	 * 
	 */
	protected void createBranchRuleIfAny(){
	    /* 
	     * First check if the InputPinSet is an exclusive branch, 
		 * if true, created ExclusiveBranchRule,
		 * if not, check if the correlated OutputPinSet is an concurrent branch, 
		 * 	if true, create a ConcurrentBranchRule.
		 */  
	
		TransformationRule rule;
	
		InputPinSet inputSet = ((InputPinSet)getSource().get(0));
		//check if input set is exclusive branch
		if(BomHelper.getInstance().isEBranch(inputSet)){
			rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
			rule.getSource().add(inputSet);
			getChildRules().add(rule);
			rule.transformSource2Target();
			
		} else { //check if output is Concurrent branch
			
			//is it safe to assume there is only one output set since it is not e-branch?
			//OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
			List opss = BomUtils.getCorrelatedPinSets(inputSet);
			if (opss.size()>0) {
				OutputPinSet outputSet=(OutputPinSet)opss.get(0);
				if(BomHelper.getInstance().isCBranch(outputSet)){
					rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
					rule.getSource().add(outputSet);
					getChildRules().add(rule);	
					rule.transformSource2Target();				 
				}else{
					//do nothing
				}
			}
		}		
	}
		
	/**
	 * For each pin in the input set, get the link created by parent rule, 
	 * and set <code>invoke</code> as the target of that link.
	 * Updates the join codition of the <code>invoke</code> to depend on all links coming into the input set. 
	 * @param invoke  BPEL Invoke
	 */	
	protected void doInputPinSetConnection(Invoke invoke){
	   
	   InputPinSet inputSet = (InputPinSet)getSource().get(0);

	   /* Connect the incoming and outgoing links for this action */
	   List isResultList = mapInputPinSet(inputSet);
	   Activity inputSetActivity = null;
	   
	   if(inputSet.getAction().getInStructuredNode() != null &&
				(BomUtils.isLocalProcess(inputSet.getAction().getInStructuredNode()) ||
				inputSet.getAction().getInStructuredNode() instanceof LoopNode) &&
				ProcessUtil.pinSetIsEntryToSubprocess(inputSet)) {
		   InputPinSet containerInputPinSet = ProcessUtil.findEntrySetToSubprocess(this.getParentRule());
		   if (containerInputPinSet != null && ProcessUtil.manyLinksRequired(containerInputPinSet)) {
				//retrieve activity from container
				Activity containerInputSetActivity = 
					PinSetRegistryUtil.getBpelActivity(getContext(), containerInputPinSet);
				BpelOptimizationUtil.registerNonOptimizableElement(getContext(), containerInputSetActivity);
		   }
		}

	   if (isResultList.isEmpty() || !(isResultList.get(0) instanceof Activity)){
		   inputSetActivity = invoke;
	   } else {
	   		// This is the case to optimize when there is only a single RetrieveArtifactPin, we just reuse the BPEL
	   		// Variable the represents the Local Repos as the InputVariable of the invoke activity.  This reduce the
	   		// additional assign/java snippet needed
	   		if (isResultList.size()==1 && isResultList.get(0) instanceof Variable) {
	   			inputSetActivity=invoke;
	   		} else {
	 		   inputSetActivity = (Activity)isResultList.get(0);
					
			   /* Link the new input set actibvity to the sequence */
			   Link link = linkActivities(inputSetActivity, 
										  invoke, 
										  BomUtils.getCanonicalName(inputSet) + "_to_" + inputSet.getAction().getName());
					
			   isResultList.add(link);
					
			   /* Propagate results from the input set mapping */
			   getTarget().addAll(isResultList);
	   		}
	   }

	   setTargetOfLinks(inputSet, inputSetActivity);	
	    
	}
	

	
	/**
	 * For each output pin in the Output Set correlated with the InputPinSet,
	 * finds the link created by the parent rule, sets the <code>invoke</code> as the source of the link.
	 * 
	 * @param invoke  BPEL invoke
	 */
	protected void doOutputPinSetConnection(Invoke invoke){
	
		List osList = ((InputPinSet)getSource().get(0)).getOutputPinSet();
		
		OutputPinSet nextOS;		
		Pin pin;
		//although we don't support multiple outputs correlated to one input, here implement for multiple outputs
		for(int i=0, count = osList.size(); i<count; i++){
			nextOS = (OutputPinSet) osList.get(i);				

			List osResultList = mapOutputPinSet(nextOS);
			Activity outputSetActivity = null;
			if (osResultList.isEmpty()){
				outputSetActivity = invoke;
			} else {
		   		// This is the case to optimize when there is only a single RetrieveArtifactPin, we just reuse the BPEL
		   		// Variable the represents the Local Repos as the InputVariable of the invoke activity.  This reduce the
		   		// additional assign/java snippet needed
		   		//if (osResultList.size()==1 && osResultList.get(0) instanceof Variable) {
				if (osResultList.get(0) instanceof Variable) {
		   			outputSetActivity=invoke;
		   		} else {//to handle multiple outputsets
					outputSetActivity = (Activity)osResultList.get(0);
					
					/* Link the new input set activity to the sequence */
					Link link = linkActivities(invoke, 
											   outputSetActivity, 
											   nextOS.getAction().getName() + "_to_" + BomUtils.getCanonicalName(nextOS));
					
					osResultList.add(link);
					
					/* Propagate the results from the input set mapping */
					getTarget().addAll(osResultList);
					
					//Register the link from the OPS to the generated fork activity
					if(nextOS.getIsException().booleanValue()){
						CBPELUtil.registerFaultSource(getContext(), this, nextOS.getName(), link);
						CBPELUtil.createGeneratedAttribute(link, CBPELUtil.GEN_BPC_FAULT_LINK);
					}
		   		}				
			}

			setSourceOfLinks(nextOS, outputSetActivity);
		
		}		
	}
	
	protected Invoke createInvoke(InputPinSet inputSet) {
		Operation operation;
		PartnerLink partner=null;
		operation = (Operation)getSource().get(2);
		if (getSource().size()>3) {
			partner = (PartnerLink)getSource().get(3);			
		}
		PortType portType = (PortType)operation.eContainer();
		//create BPEL invoke
		Invoke invoke = BPELFactory.eINSTANCE.createInvoke();
		
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
		valueProvider.setContext(this.getContext()); 
		String convertedName = (String)valueProvider.getAttributeValueForType(invoke, inputSet, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));
		invoke.setName(convertedName);		

		//AR parameter support
		invoke.getExtensibilityElements().add(ivInput);
		//invoke.getExtensibilityElements().add(ivOutput);
		if(ivOutputList != null && ivOutputList.size() > 0){//we need to control the sequence
			int size = ivOutputList.size();
			for(int i=0;i<size;i++){
				invoke.getExtensibilityElements().add(ivOutputList.get(i));	
			}			
			ivOutputList.clear();
		}
		
		invoke.setOperation(operation);
		invoke.setPartnerLink(partner);
		invoke.setPortType(portType);
	  
		createDisplayName(invoke, inputSet, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);		
		return invoke;				
	}
	
	protected void replaceVariable(Variable oldVar,Variable newVar) {
		List childRules = getChildRules();
		TransformationRule tr;
		for (Iterator it=childRules.iterator();it.hasNext();) {
			tr = (TransformationRule)it.next();
			if (tr instanceof ContainerRule) {
				if (tr.getTarget().contains(oldVar)) {
					int i = tr.getTarget().indexOf(oldVar);
					tr.getTarget().remove(oldVar);
					tr.getTarget().add(i,newVar);
					break;
				}
			}
		}
	}
	
	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, String targetNamespace){
		
		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && thisImport.getNamespace().equals(targetNamespace)){
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, Definition toBeImportedDef){
		
		String targetNamespace = toBeImportedDef.getTargetNamespace();
		String uri = OriginalWsdlUtil.getOriginalFileNameFromContext(getContext(),toBeImportedDef);
		String location = null;
		if(uri != null && uri.length() > 0){
			location = uri;
		}else{
			location  = toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT;
		}
		List imports = process.getImports();
		String thisImportLocation = null;
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if (thisImport.getLocation().indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
					thisImportLocation = thisImport.getLocation().substring(thisImport.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);					
				} else {
					thisImportLocation = thisImport.getLocation();
				}
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && 
						thisImport.getNamespace().equals(targetNamespace) &&
						thisImportLocation.equals(location)){
					return true;
				}
			}
		}
		return false;
	}
	
	
	/**
	 * Adds the import for the <code>portType</code> namespace, if not the same as the
	 * WSDL <code>definition</code> namespace.
	 * @param definition WSDL Definition
	 * @param portType WSDL PortType
	 */	
	private void organizeImports(Process processDef, Object obj){
		if(obj instanceof PortType){
			Definition toBeImportedDef = ((PortType)obj).getEnclosingDefinition();
			String namespace = toBeImportedDef.getTargetNamespace();
			
			//should add wsdl import.. leave it for the one that is saving the resource
			if(!containsImport(processDef, toBeImportedDef)){
				//we need to import the schema 
				com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
				if (toBeImportedDef.getLocation() != null && ExportOperationUtil.isExternalFile(toBeImportedDef.getLocation())) {
					wsdlImport.setLocation(ExportOperationUtil.replaceConvertedSpaces(toBeImportedDef.getLocation()));
				} else {
					wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
				}
				wsdlImport.setNamespace(namespace);
				wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
				if (!wsdlImport.getLocation().equals(ExportOperation.WSDL_FILE_EXT)) {
					processDef.getImports().add(wsdlImport);
				}
			}
		}else if(obj instanceof DocumentRoot){
			URI namespaceURI = ((DocumentRoot)obj).getTask().getTargetNamespace();
			String namespace = namespaceURI.toString();
			if(!namespace.equals(processDef.getTargetNamespace())){
				if(!containsImport(processDef,namespace)){
					//we need to import the tel file 
					com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
					wsdlImport.setLocation(((DocumentRoot)obj).getTask().getName() + ExportOperation.ITEL_FILE_EXT );
					wsdlImport.setNamespace(namespace);
					wsdlImport.setImportType(AbstractbpelConstants.XSD_IMPORT_TYPE);
					if (!wsdlImport.getLocation().equals(ExportOperation.ITEL_FILE_EXT)) {
						processDef.getImports().add(wsdlImport);
					}
				}
			}
			
		}
	}
} //InvokeRuleImpl
