/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeWithCallbackRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.PinSetRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Correlation;
import com.ibm.wbit.bpel.CorrelationPattern;
import com.ibm.wbit.bpel.CorrelationSet;
import com.ibm.wbit.bpel.Correlations;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Initiate;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.OnMessage;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.Input;
import com.ibm.wbit.bpelpp.Output;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Invoke With Callback Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class InvokeWithCallbackRuleImpl extends OneWayInvokeRuleImpl implements InvokeWithCallbackRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private List outputPinSets=null;
	private List outputVariables =new ArrayList();
	
	/* The callback activity (either a <receive> or <pick>), if one is created */
	private Activity fCallbackActivity = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected InvokeWithCallbackRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.INVOKE_WITH_CALLBACK_RULE;
	}

	/** 
	 * Returns the callback activity (either a <receive> or a <pick>), if one
	 * was created as part of the execution of this rule.
	 *
	 * @return The callback activity
	 */
	public Activity getCallbackActivity(){
	    return fCallbackActivity;
	}
	
	/**
	 * Generate callback activity based on the output pinSet
	 */	
	protected void doOutputPinSetConnection(Invoke invoke){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		List osList = getOutputPinSets(inputSet);
		OutputPinSet nextOS;		
		Pin pin;
		
		fCallbackActivity = null;
		
		if (isCallbackNeeded()) {
			CorrelationSet cSetForInvoke = createCorrelationSet(invoke,inputSet);
			int count = osList.size();
			if (count > 0) {
			    fCallbackActivity =PinSetRegistryUtil.getBpelActivity(getContext(),(OutputPinSet)osList.get(0));
				if (fCallbackActivity == null) {
					if (count > 1) {
					    fCallbackActivity = BPELFactory.eINSTANCE.createPick();
						Pick pick = (Pick)fCallbackActivity;
						pick.setCreateInstance(Boolean.FALSE);
						
						NameProvider nameProvider = NameProviderFactory.getNameProvider(pick);
						String convertedName = nameProvider.getName(pick, inputSet.getAction(), NamingUtil.findRegistry(this));

						pick.setName(convertedName);
						//String name = getDisplayName(inputSet.getAction());
						//name = name+" Callback";
						createDisplayName(pick, inputSet, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
					} else {
						//callbackActivity = createReceive(inputSet,(Operation)getSource().get(4),(Variable)outputVariables.get(0),cSetForInvoke);
					    fCallbackActivity = createReceive(inputSet,(Operation)getSource().get(4),(Output)outputVariables.get(0),cSetForInvoke);
					}
					PinSetRegistryUtil.registerPinSet(getContext(),(OutputPinSet)osList.get(0),fCallbackActivity);
					
					Activity callbackOSActivity;
					OnMessage onMsg=null;
					Output output; 
					Operation op;
					List previousVars=new ArrayList();
					boolean reuseVar;
					for(int i=0; i<count; i++){
						nextOS = (OutputPinSet) osList.get(i);				

						// Now create the callback receive/pick
						if (count == 1) {
							callbackOSActivity = fCallbackActivity;
						} else {
							Pick pick = (Pick)fCallbackActivity;
							//bpelVar=(BPELVariable)outputVariables.get(i);
							output = (Output)outputVariables.get(i);
							// Attempt to reuse the same variables on the pick
							reuseVar=false;
							op = (Operation)getSource().get(4+i);
							onMsg = createOnMessage(inputSet,op,output,cSetForInvoke);
							Empty empty = BPELFactory.eINSTANCE.createEmpty();
						
							NameProvider nameProvider = NameProviderFactory.getNameProvider(pick);
							String convertedName = nameProvider.getName(empty, nextOS, NamingUtil.findRegistry(this));
							empty.setName(convertedName);
							createDisplayName(empty,nextOS.getName());
							callbackOSActivity = empty;
							onMsg.setActivity(empty);
							pick.getMessages().add(onMsg);
							
						}
						List osResultList = mapOutputPinSet(nextOS);
						Activity outputSetActivity = null;
						if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity)){
							outputSetActivity = callbackOSActivity;
						} else {
							outputSetActivity = (Activity)osResultList.get(0);				
							Link link2 = linkActivities(callbackOSActivity, 
													   outputSetActivity, 
														callbackOSActivity.getName() + "_to_" + BomUtils.getCanonicalName(nextOS));
				
							if (fCallbackActivity instanceof Pick) {
								Flow flow;
								Links links;
								if (onMsg.getActivity() instanceof Empty) {
									flow = BPELFactory.eINSTANCE.createFlow();
									//defect 139867. generate name for the flow
									String convertedName = NamingUtil.getFlowName(flow, this);
									flow.setName(convertedName);
									createDisplayName(flow, convertedName);
									links = BPELFactory.eINSTANCE.createLinks();
									flow.setLinks(links);
									onMsg.setActivity(flow);
								} else {
									flow = (Flow)onMsg.getActivity();
									links = flow.getLinks();
								}
								flow.getActivities().add(callbackOSActivity);
								flow.getActivities().add(outputSetActivity);
								links.getChildren().add(link2);
							} else {
								osResultList.add(link2);													
								/* Propagate the results from the input set mapping */
								getTarget().addAll(osResultList);
							}
				
						}
						setSourceOfLinks(nextOS, outputSetActivity);
		
					} 
				} else {
					// this is the case when there are multiple inputSets correlates to the same outputSet. 
					// although in most cases things will be handled automatically (or magically).  There is the case
					// for implicit termination (where the outputSet has no outgoing pin or pin has no outgoing link) where
					// during the processing of the non-first inputSet of the Task, a link will be created and the link needs
					// to have the source connected to the outputSetActivity.  Without this code here to handle this case, the
					// source of the link will never be set and thus will have a dangling link created
					for(int i=0; i<count; i++){
						nextOS = (OutputPinSet) osList.get(i);				
						setSourceOfLinks(nextOS, fCallbackActivity);
					}
				}
				/* Link the new input set activity to the sequence */
				Link link = linkActivities(invoke, 
				        					fCallbackActivity, 
											inputSet.getAction().getName() + "_to_" + fCallbackActivity.getName());
				getTarget().add(link);
				getTarget().add(fCallbackActivity);
			}
		} else {	
			for(int i=0, count = osList.size(); i<count; i++){
				nextOS = (OutputPinSet) osList.get(i);				

				List osResultList = mapOutputPinSet(nextOS);
				Activity outputSetActivity = null;
				if (osResultList.isEmpty() || osResultList.get(0) instanceof Variable){
					outputSetActivity = invoke;
				} else {
					outputSetActivity = (Activity)osResultList.get(0);
				
					/* Link the new input set activity to the sequence */
					Link link = linkActivities(invoke, 
											   outputSetActivity, 
											   nextOS.getAction().getName() + "_to_" + BomUtils.getCanonicalName(nextOS));
				
					osResultList.add(link);
				
					/* Propagate the results from the input set mapping */
					getTarget().addAll(osResultList);
				}

				setSourceOfLinks(nextOS, outputSetActivity);
		
			}		

		}

	}
	
	/**
	 * Check if there is an correlationSet already created for other invokes representing other inputSet.  Reuse it if one is there.
	 * Create a new one otherwise
	 * @param invoke
	 * @param inputSet
	 * @return
	 */
	private CorrelationSet createCorrelationSet(Invoke invoke,InputPinSet inputSet) {

		CorrelationSet cSetForInvoke = findExistingCorrelationSet(inputSet);
		if (cSetForInvoke == null) {
			// If 1-way longrunning process is used, need to create empty correlation set
			cSetForInvoke = BPELFactory.eINSTANCE.createCorrelationSet();
			cSetForInvoke.setName(inputSet.getAction().getName()); // correlationSet name will be automatically converted later ... See ProcessDefinitionRuleImpl.processChildTarget() method
			getTarget().add(cSetForInvoke);			
		}
		Correlation cor = BPELFactory.eINSTANCE.createCorrelation();
		cor.setSet(cSetForInvoke);
//		cor.setInitiate(AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES);
		cor.setInitiate(Initiate.YES_LITERAL);
		Correlations cors = invoke.getCorrelations();
		if (cors==null) {
			cors=BPELFactory.eINSTANCE.createCorrelations();
			invoke.setCorrelations(cors);
		}
		
		cor.setPattern(CorrelationPattern.OUT_LITERAL);
		cors.getChildren().add(cor);
	   
		//Ray: added for displayname on BPELP
		//String name = getDisplayName(inputSet.getAction());
		String name = getBPELActivityDisplayName(invoke, inputSet);
		LoggingUtil.logWarning(MessageKeys.INCOMPLETE_CORRELATION_SET_WARNING, new String[]{name==null?inputSet.getAction().getName():name}, null);	
		return cSetForInvoke;
	}

	private CorrelationSet findExistingCorrelationSet(InputPinSet inputSet) {
		List ips = inputSet.getAction().getInputPinSet();
		if (ips.size()>1) {
			InputPinSet tmpPS;
			for (Iterator it=ips.iterator();it.hasNext();) {
				tmpPS = (InputPinSet)it.next();
				if (tmpPS != inputSet) {
					Activity bpelAct = PinSetRegistryUtil.getBpelActivity(getContext(),tmpPS);
					if (bpelAct instanceof Invoke) {
						Correlations cors = ((Invoke)bpelAct).getCorrelations();
						if (cors != null && !cors.getChildren().isEmpty()) {
							Correlation cor = (Correlation) cors.getChildren().get(0);
							return cor.getSet();
						}
					}
				}
			}
		}	
		return null;
	}
	protected Receive createReceive(InputPinSet inputSet, Operation op, Output var, CorrelationSet cSetForInvoke) {
		//Operation op = (Operation)getSource().get(4);
		PartnerLink partner = (PartnerLink)getSource().get(3);
		//switch with the callback partner
		TransformationRule partnerRule = TransformationUtil.getRuleForTarget(getRoot(), PartnerRule.class, partner);
		if(partnerRule != null && partnerRule.getTarget().size()>1){
			partner = (PartnerLink)partnerRule.getTarget().get(1);
		}
		Receive receive = BPELFactory.eINSTANCE.createReceive();
				
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
		valueProvider.setContext(this.getContext());
		//defect 115295
		//the name of the callback should be attached to the outputPinSet
		//String convertedName = (String)valueProvider.getAttributeValueForType(receive, inputSet, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));
		OutputPinSet outputSet = (OutputPinSet)outputPinSets.get(0);
		String convertedName = (String)valueProvider.getAttributeValueForType(receive, outputSet, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));

		receive.setName(convertedName);
		receive.setOperation(op);
		receive.setPortType((PortType) op.eContainer());
		receive.getExtensibilityElements().add(var);
		receive.setPartnerLink(partner);	
		receive.setCreateInstance(Boolean.FALSE);
		// Ray: added display name support
//		String name = getDisplayName(inputSet.getAction());
//		name = name+" Callback";
		//createDisplayName(receive, inputSet, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
		createDisplayName(receive, outputSet, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
		
		Correlation cor = BPELFactory.eINSTANCE.createCorrelation();
		cor.setSet(cSetForInvoke);
		Correlations cors = receive.getCorrelations();
		if (cors==null) {
			cors=BPELFactory.eINSTANCE.createCorrelations();
			receive.setCorrelations(cors);
		}
		cors.getChildren().add(cor);
		return receive;
	}

	protected OnMessage createOnMessage(InputPinSet inputSet,Operation op, Output var,CorrelationSet cSetForInvoke) {
		//Operation op = (Operation)getSource().get(4+ind);
		PartnerLink partner = (PartnerLink)getSource().get(3);
		//switch with the callback partner
		TransformationRule partnerRule = TransformationUtil.getRuleForTarget(getRoot(), PartnerRule.class, partner);
		if(partnerRule != null && partnerRule.getTarget().size()>1){
			partner = (PartnerLink)partnerRule.getTarget().get(1);
		}
		
		OnMessage receive = BPELFactory.eINSTANCE.createOnMessage();
		receive.setOperation(op);
		receive.setPortType((PortType) op.eContainer());
		receive.getExtensibilityElements().add(var);
		receive.setPartnerLink(partner);	
		Correlation cor = BPELFactory.eINSTANCE.createCorrelation();
		cor.setSet(cSetForInvoke);
		Correlations cors = receive.getCorrelations();
		if (cors==null) {
			cors=BPELFactory.eINSTANCE.createCorrelations();
			receive.setCorrelations(cors);
		}
		cors.getChildren().add(cor);
		return receive;
	}
	
	private List getOutputPinSets(InputPinSet inputSet) {
		if (outputPinSets==null) {
				outputPinSets = BomUtils.getCorrelatedPinSets(inputSet);
				if (inputSet.getAction() instanceof CallBehaviorAction) {
					// if it is a CBA, need to re-adjust the order of the outputPinSets and make sure they match the CBA's order
					// It is important because the operation (Source of this rule) is in order
					List filteredOrigOPS = new ArrayList();
					List origOPS = inputSet.getAction().getOutputPinSet();
					OutputPinSet tempOPS;
					for (Iterator it=origOPS.iterator();it.hasNext();) {
						tempOPS = (OutputPinSet)it.next();
						if (outputPinSets.contains(tempOPS)) {
							filteredOrigOPS.add(tempOPS);
						}
					}
					outputPinSets = filteredOrigOPS;
				}
			}
		return outputPinSets;		
	}
	
	/**
	 * Creates Input/Output rules for the input set and correlated output sets
	 *
	 */
	protected void createContainerRule(){
		  InputPinSet inputSet = (InputPinSet)getSource().get(0);
		 
		 //create the Input rule for input set
		  InputRule cr = AbstractbpelFactory.eINSTANCE.createInputRule();
		  cr.getSource().add(inputSet);
		  getChildRules().add(cr);
		  cr.transformSource2Target();
		  
   		  ivInput = (Input)cr.getTarget().get(0);
   		  
 		if (isCallbackNeeded()) {
			List osList = getOutputPinSets(inputSet);
			OutputPinSet nextOS;
			
			for (int i=0, cnt=osList.size();i<cnt;i++) {
				nextOS = (OutputPinSet)osList.get(i);
				OutputRule or = AbstractbpelFactory.eINSTANCE.createOutputRule();
				or.getSource().add(nextOS);
				getChildRules().add(or);
				or.transformSource2Target();
							
				Output output = (Output)or.getTarget().get(0);
				outputVariables.add(output);			
			}
 		}
		  
	}	
	
	// If the number of source elements of this rule is <= 4, then there is no callbacks
	protected boolean isCallbackNeeded() {
		return (getSource().size()>4);
	}
	

	
} //InvokeWithCallbackRuleImpl
