/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.ExtensibilityElement;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;
import org.eclipse.xsd.util.XSDUtil;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.model.BinaryMathematicalOperator;
import com.ibm.btools.expression.model.BinaryNumericExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LoopNodeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Loop Node Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class LoopNodeRuleImpl extends ProcessRuleImpl implements LoopNodeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* Keeps track of whether this is the first time this rule has been activated */
	private boolean fFirstTimeExec = true;
	
	/* The variable that corresponds to the internal output set variable */
	private BPELVariable fOsStorageVar;
	
	/* The variable that keeps track of the indices for each of the parts of the internalOSVar */
	private BPELVariable fOsIndexVar;
	
	/* The part that is used for the assignment of each item from the primary output set variable
	 * to osStorageVar */
	private Part fAssignIndexPart;
	
	/* The NCNameConverter used throughout this rule */
	private JavaNCNameConverter fConverter;
	
	private boolean isAllVariablesMapped = false;
	
	/* Keeps track of whether or not we need to use indices or not, based on the configuration
	 * of the output pin set */
	private boolean fIndicesNeeded;
	
	private ExtensibilityElement exprElement;
	
	/* This is the type the parts that are used to keep track of the current indices */
	private static XSDSimpleTypeDefinition intXSDType = XSDUtil.getSchemaForSchema(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001).resolveSimpleTypeDefinition("int");
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected LoopNodeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.LOOP_NODE_RULE;
	}

	/**
	 * source: BOM InputPinSet of a LoopNode (not a ForLoopNode)
	 * target: BPEL While and an activity to map the output set, temp variables
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
				
		/* The source of this rule is always a single input set */
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		/* There is only allowed to be one non-exception, non-streaming output set on a LoopNode */
		OutputPinSet outputPinSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
		

		if (!fFirstTimeExec) {
			//reExecute(inputSet);
			SANNestedProcessRule sanRule=null;
			sanRule=findSanNestedProcessRuleWithOriginalScope(getChildRules(),(While)getTarget().get(0));
//			for (int i=0; i<getChildRules().size();i++) {
//				if (getChildRules().get(i) instanceof SANNestedProcessRule) {
//					sanRule = (SANNestedProcessRule)getChildRules().get(i);
//					for (int j=0; j<sanRule.getTarget().size();j++) {
//						if (sanRule.getTarget().get(j) instanceof Scope) {
//							Scope scope = ((Scope)sanRule.getTarget().get(j));
//							scope.setActivity(((While)getTarget().get(0)).getActivity());
//							break;
//						}
//					}					
//					break;
//				}
//			}
			sanRule.transformSource2Target();
			processSANNestedProcessRuleTargets(sanRule);
			//re-execute the expression only if all variables are mapped . Defect 74612
			List variables = getVariablesForPinSet(inputSet);
			if(variables.size() == BomUtils.getObjectPinsForSet(inputSet).size()){
				isAllVariablesMapped = true;
				While bpelWhile;
				bpelWhile = (While)getTarget().get(0);
				if(exprElement != null)bpelWhile.getExtensibilityElements().remove(exprElement);
				exprElement = transformWhileCondition((LoopNode)inputSet.getAction());
				if (exprElement != null){
					bpelWhile.getExtensibilityElements().add(exprElement);
				}
			}
			
			/* We don't need to re-execute the handlers in this case because all this
			 * re-execution did was re-run the SANNestedProcessRule... */
			
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		}
		fFirstTimeExec = false;

		fConverter = new JavaNCNameConverter();

		// TODO: Ray testing
		boolean bDelegateToSANNestedProcessRule=true;
		if (bDelegateToSANNestedProcessRule) {
			LoopNodeRule rule = null;
			While bpelWhile;
			InputPinSet ips = (InputPinSet)getSource().get(0);
			List inputSets = ips.getAction().getInputPinSet();
			// Find the BPEL while activity that has been already created for this while loop node
		    for (Iterator it=inputSets.iterator();it.hasNext()&&rule==null;) {
				rule = (LoopNodeRule)TransformationUtil.getRuleForSource(
								ProcessUtil.getProcessDefinitionRule(getContext()),
								LoopNodeRule.class,
								(InputPinSet)it.next(),
								While.class);
		    }    	
		    SANNestedProcessRule sanRule=null;
			if (rule!=null) {
				bpelWhile = (While)rule.getTarget().get(0);
				getTarget().add(bpelWhile);
				findSanNestedProcessRuleWithOriginalScope(rule.getChildRules(),bpelWhile);
				// Now, create a new SANNestedProcessRule and let it execute with the inputPinSet (not the first in the list)
				sanRule = AbstractbpelFactory.eINSTANCE.createSANNestedProcessRule();
				sanRule.getSource().addAll(getSource());
				getChildRules().add(sanRule);
				sanRule.transformSource2Target();
				//only execute if all the variables are mapped. defect 74612
				//first remove the old one
				List variables = getVariablesForPinSet(inputSet);
				if(variables.size() == BomUtils.getObjectPinsForSet(inputSet).size()){
					isAllVariablesMapped = true;
					if(exprElement != null)bpelWhile.getExtensibilityElements().remove(exprElement);
					exprElement = transformWhileCondition((LoopNode)inputSet.getAction());
					if (exprElement != null){
						bpelWhile.getExtensibilityElements().add(exprElement);
					}
				}
			} else {
				sanRule = AbstractbpelFactory.eINSTANCE.createSANNestedProcessRule();
				sanRule.getSource().addAll(getSource());
				getChildRules().add(sanRule);
				sanRule.transformSource2Target();
				/* Create a BPEL <while>, which is the primary representation of the BOM LoopNode */ 
				bpelWhile = BPELFactory.eINSTANCE.createWhile();
				
				AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME_WHILE);
				valueProvider.setContext(getContext());
				String convertedName = (String)valueProvider.getAttributeValueForType(bpelWhile, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_NAME_WHILE, NamingUtil.findRegistry(this));
				String displayName = (String)valueProvider.getAttributeValueForType(bpelWhile, inputSet.getAction(), AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME_WHILE, null);
				createDisplayName(bpelWhile,displayName);
				
				/* Convert the BOM LoopCondition into a Expession-Model instance of a While Condition, 
				 * and add it as an ExtensibilityElement to the While activity */
				List variables = getVariablesForPinSet(inputSet);
				if(variables.size() == BomUtils.getObjectPinsForSet(inputSet).size()){
					isAllVariablesMapped = true;
					exprElement = transformWhileCondition((LoopNode)inputSet.getAction());
					if (exprElement != null){
						bpelWhile.getExtensibilityElements().add(exprElement);
					}
				}
//				exprElement = transformWhileCondition((LoopNode)inputSet.getAction());
//				if (exprElement != null){
//					bpelWhile.getExtensibilityElements().add(exprElement);
//				}
				getTarget().add(bpelWhile);
				bpelWhile.setName(convertedName);
			}
			processSANNestedProcessRuleTargets(sanRule);
//			for (int i=0; i<sanRule.getTarget().size();i++) {
//				if (sanRule.getTarget().get(i) instanceof Scope) {
//					Scope scope = ((Scope)sanRule.getTarget().get(i));
//					if (bpelWhile.getActivity()==null) {
//						bpelWhile.setActivity(scope.getActivity());
//						bpelWhile.getEExtensibilityElements().addAll(scope.getEExtensibilityElements());
//						////bpelWhile.setName(scope.getName());
//						//bpelWhile.setName(convertedName);												
//					}
//					
//					//migration 11.09
////					bpelWhile.getSources().addAll(scope.getSources());
////					bpelWhile.getTargets().addAll(scope.getTargets());
//					if(scope.getSources()!=null && scope.getSources().getChildren() != null && !scope.getSources().getChildren().isEmpty()){
//						if(bpelWhile.getSources() == null){
//							Sources sources  = BPELFactory.eINSTANCE.createSources();
//							bpelWhile.setSources(sources);
//						}
//						bpelWhile.getSources().getChildren().addAll(scope.getSources().getChildren());
//					}
//					if(scope.getTargets()!=null && scope.getTargets().getChildren() != null && !scope.getTargets().getChildren().isEmpty()){
//						if(bpelWhile.getTargets() == null){
//							Targets targets  = BPELFactory.eINSTANCE.createTargets();
//							bpelWhile.setTargets(targets);
//						}
//						//bpelWhile.setJoinCondition(scope.getJoinCondition());
//						if(scope.getTargets().getJoinCondition()!= null){
//							bpelWhile.getTargets().setJoinCondition(scope.getTargets().getJoinCondition());	
//						}
//						bpelWhile.getTargets().getChildren().addAll(scope.getTargets().getChildren());
//					}
//					
////					bpelWhile.getSources().getChildren().addAll(scope.getSources().getChildren());
////					bpelWhile.getTargets().getChildren().addAll(scope.getTargets().getChildren());
//
//				} else {
//					getTarget().add(sanRule.getTarget().get(i));
//				}
//			}
		} else {
			if (outputPinSet.getOutputObjectPin().isEmpty()){
				fIndicesNeeded = false;
			} else {
				fIndicesNeeded = true;
			}

			/* Create all the BPEL variables we need */
			transformBOMVariable();
			transformTerminationNode();
			if (fIndicesNeeded){
				createStorageVariables(outputPinSet);
				createBpelIndexVariable(outputPinSet);
			}		
			
			// create variables for InputPinSet and OutputPinSet used for entry/exit transitions for local container (loop/subprocess)			
			List variables = createEntryExitVariables(fConverter);
			getTarget().addAll(variables); //propagate all these types to parent
			
			/* Create the sequence that will hold the <while> and the final assignment from the
			 * temporary storage variable to the variable associated with the output pin set (if needed) */
			Sequence sequence = BPELFactory.eINSTANCE.createSequence();
			
			/* Create the representation of the input pin set (if required), and put it in sequence */
			doInputPinSetConnection(sequence);

			/* Create the initialization activity to initialize the index variable, if required based
			 * on the number of object pins in the output set*/
			if (fIndicesNeeded){
				sequence.getActivities().add(createInitActivity(outputPinSet)); 
			}

			/* Create a BPEL <while>, which is the primary representation of the BOM LoopNode */ 
			While bpelWhile = BPELFactory.eINSTANCE.createWhile();
			
			/* Create the Flow that will be the top-level activity in the <while> */
			Flow flow = BPELFactory.eINSTANCE.createFlow();
			
			//defect 139867. generate name for the flow
			String convertedName = NamingUtil.getFlowName(flow, this);
			flow.setName(convertedName);
			createDisplayName(flow, convertedName);
			
			bpelWhile.setActivity(flow);
			sequence.getActivities().add(bpelWhile);
			getTarget().add(sequence);

			/* Convert the BOM LoopCondition into a Expession-Model instance of a While Condition, 
			 * and add it as an ExtensibilityElement to the While activity */
			ExtensibilityElement exprElement = transformWhileCondition((LoopNode)inputSet.getAction());
			if (exprElement != null){
				bpelWhile.getExtensibilityElements().add(exprElement);
			}


			BomWalker bw = ProcessUtil.getBomWalker(getContext());
			List inputSetList = bw.getNext(inputSet, BomWalker.INPUT_SET, false);

			/* Add the activity to initialize the termination node and connect to the first 
			 * activities in the flow */
			Activity termInitActivity = getTerminationActivity(inputSet, true);
			if (termInitActivity != null){
				connectTerminationInitActivity(flow, termInitActivity, inputSetList);
			}
			

			/* Walk through each pin of the input set, creating and executing child rules */
			
			InputPinSet nextIS = null;
			List nextInputPinSets = null;
			for (int i = 0, count = inputSetList.size(); i<count; i++){
				nextIS = (InputPinSet)inputSetList.get(i);
				do {			
					nextIS = stepThroughBOM(nextIS);
					if (nextIS != null){ 	//means we just created a child rule
						//we must continue
						// It is safe just to use the first elment since we always skip 
						// the branch point, we should only have one InputPinSet back
						nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
						if (nextInputPinSets == null || nextInputPinSets.isEmpty()) {
							nextIS = null;
						} else {
							nextIS = (InputPinSet) nextInputPinSets.get(0);
						}					
					}
					//else nextIS == null, we want stop walk down that path
						 
				} while(nextIS!= null);				 
			}
			
			// check if LoopNode is branch point 
			TransformationRule rule = createBranchRuleIfAny();		
			
			// process target produced by child including propagate target to parent
			processChildTargets(flow, rule);
			
			// connect OutputPinSet with acvtivites inside and outside of this LoopNode
			doOutputPinSetConnection(outputPinSet, sequence, flow);	
			
		}

		executeHandlers();
		
		//d170286
		if (inputSet.eContainer() != null) {
			getSource().add(inputSet.eContainer());
		}
		
		setComplete(true);		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return true;
	}
	
	private SANNestedProcessRule findSanNestedProcessRuleWithOriginalScope(List childRules, While bpelWhile) {
		SANNestedProcessRule sanRule=null;
		// Need to find the first Scope created and change the owner of the Flow from the While activity 
		// back to the Scope.
		for (int i=0; i<childRules.size();i++) {
			if (childRules.get(i) instanceof SANNestedProcessRule) {
				sanRule = (SANNestedProcessRule)childRules.get(i);
				for (int j=0; j<sanRule.getTarget().size();j++) {
					if (sanRule.getTarget().get(j) instanceof Scope) {
						Scope scope = ((Scope)sanRule.getTarget().get(j));
						scope.setActivity(bpelWhile.getActivity());
						break;
					}
				}
				break;
			}
		}	
		return sanRule;
	}
	
	/**
	 * Set the Flow of the scope as the While's Flow and throw away the Scope
	 * @param sanRule
	 */
	protected void processSANNestedProcessRuleTargets(SANNestedProcessRule sanRule) {
		While bpelWhile = (While)getTarget().get(0);
		for (int i=0; i<sanRule.getTarget().size();i++) {
			if (sanRule.getTarget().get(i) instanceof Scope) {
				Scope scope = ((Scope)sanRule.getTarget().get(i));
				boolean skipScope = shouldSkipScope(sanRule);
				//Loop Node w/o fault links, prior to fault handling
				if(skipScope){
				if (bpelWhile.getActivity()==null) {
					bpelWhile.setActivity(scope.getActivity());
					bpelWhile.getEExtensibilityElements().addAll(scope.getEExtensibilityElements());
					////bpelWhile.setName(scope.getName());
					//bpelWhile.setName(convertedName);												
				}
				
				//migration 11.09
//				bpelWhile.getSources().addAll(scope.getSources());
//				bpelWhile.getTargets().addAll(scope.getTargets());
				if(scope.getSources()!=null && scope.getSources().getChildren() != null && !scope.getSources().getChildren().isEmpty()){
					if(bpelWhile.getSources() == null){
						Sources sources  = BPELFactory.eINSTANCE.createSources();
						bpelWhile.setSources(sources);
					}
					bpelWhile.getSources().getChildren().addAll(scope.getSources().getChildren());
				}
				if(scope.getTargets()!=null && scope.getTargets().getChildren() != null && !scope.getTargets().getChildren().isEmpty()){
					if(bpelWhile.getTargets() == null){
						Targets targets  = BPELFactory.eINSTANCE.createTargets();
						bpelWhile.setTargets(targets);
					}
					//bpelWhile.setJoinCondition(scope.getJoinCondition());
					if(scope.getTargets().getJoinCondition()!= null){
						bpelWhile.getTargets().setJoinCondition(scope.getTargets().getJoinCondition());	
					}
					bpelWhile.getTargets().getChildren().addAll(scope.getTargets().getChildren());
				}				
//				bpelWhile.getSources().getChildren().addAll(scope.getSources().getChildren());
//				bpelWhile.getTargets().getChildren().addAll(scope.getTargets().getChildren());
				}else{//Handle fault links for the scope
					if (bpelWhile.getActivity()==null) {
						bpelWhile.setActivity(scope.getActivity());										
					}
					scope.setActivity(bpelWhile);
					//Remove the bpel while activity from the target list
					getTarget().remove(bpelWhile);
					//propagate the scope to the parent
					getTarget().add(scope);
				}

			} else {
				getTarget().add(sanRule.getTarget().get(i));
			}
		}
	}
    /**
     * The nestedProcessRule's source is the inputpinset of the while loop, we walk through
     * its correlated output pinset and detect if the fault pins has outgoing link. If all output pinsets 
     * are regular ones or no outgoing links from the fault pins, we should skip the scope.
     * @param sanRule
     * @return boolean if a fault pin w/ outgoing link
     */
	private boolean shouldSkipScope(SANNestedProcessRule sanRule) {
		boolean skipScope = true;
		if(sanRule == null){
			return skipScope;
		}
		if(sanRule.getSource()!= null && !sanRule.getSource().isEmpty()){
			if(sanRule.getSource().get(0) instanceof InputPinSet){
				InputPinSet ips = (InputPinSet)sanRule.getSource().get(0);
				if(ips.getOutputPinSet()!= null){
					for(Object eachOps:ips.getOutputPinSet()){
						if(eachOps instanceof OutputPinSet){
							OutputPinSet ops = (OutputPinSet)eachOps;
							if(ops.getIsException() != null && ops.getIsException().booleanValue()){
								List pinList = BomUtils.getPinsInSet(ops);
								for(Object eachPin:pinList){
									if(eachPin instanceof Pin){
										Pin pin = (Pin)eachPin;
										if(pin.getOutgoing() instanceof ActivityEdge){
											//If we find one of the fault pins w/ outgoing link
											//we should not skip the scope and jump out the loop
											skipScope = false;
											break;
										}
									}
								}
							}
						}
						//Continue loop until we find one
						if(!skipScope){
							break;
						}
					}
				}
			}
		}
		return skipScope;
	}
	
	/**
	 * Connect InputPinSet of the LoopNode including:
	 * 1. create BPEL Activity (using methods in ConnectableRule) to represent the Inputset, 
	 *    and set it as the target of links coming into the InputPinSet.
	 * 	  Update joinCodition for the activity	
	 * 		
	 * Note that for a LoopNode it is not possible to have outgoing links from the input pins
	 * in the input pin sets.   	 
	 * @param sequence - BPEL sequence that is the container for Activity
	 */
	protected void doInputPinSetConnection(Sequence sequence){
	   
		InputPinSet inputSet = (InputPinSet)getSource().get(0);	
		
		/* Create a container for the input pin set, if one hasn't already been created */
		if (!inputSet.getInputObjectPin().isEmpty()) {
			ContainerRule cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
			cr.getSource().add(inputSet);
			getChildRules().add(cr);
			cr.transformSource2Target();
		}
		       
		List resultList = mapInputPinSet(inputSet);
		
		Activity inputSetActivity = null;
		
		if (!resultList.isEmpty() && resultList.get(0) instanceof Activity){
			inputSetActivity = (Activity)resultList.get(0);	
			sequence.getActivities().add(inputSetActivity);
			getTarget().addAll(resultList.subList(1, resultList.size()));
			
			inputSetActivity = sequence;
		} else {
			/* Since there is no activity required to represent the input set, just use
			 * the sequence as the target for the incoming links */
			inputSetActivity = sequence;
		}
		inputSetActivity.setName(
			fConverter.convertName(NamingUtil.findRegistry(this), 
								   inputSetActivity, 
								   BomUtils.getCanonicalName(inputSet.getAction())));
		
		NameProvider nameProvider = NameProviderFactory.getNameProvider(inputSetActivity);
		String convertedName = nameProvider.getName(inputSetActivity, inputSet.getAction(), NamingUtil.findRegistry(this));
		
		
		createDisplayName(inputSetActivity,AbstractbpelConstants.LOOP_NODE_SEQUENCE_DSP_NAME);
		setTargetOfLinks(inputSet, inputSetActivity);
		
		/* NOTE: We don't have to deal with connections that are outgoing from the input pins on a LoopNode
		 * because it is illegal for a LoopNode to have input pins that have outgoing edges... nothing
		 * can travel from the input set into the inside of the loop node */
	   
	}
		
	/**
	 * Process targets produced by child rules.
	 * Containers and Partners are propagated to parent rule, but Links are not since
	 * it is not possible to flatten the flow that is inside the While.  Therefore, for 
	 * compatibility with the runtime, we need to keep the link definitions on the Flow
	 * inside the While.
	 *  
	 * If the LoopNode is also a branch poin (e-branch or c-branch), 
	 * the flow created by the branch rule is propagated to parent rule.
	 * All other activities are contained by this <code>flow</code>. 
	 * 
	 * @param flow The BPEL <flow> to which certain child targets should be added
	 * @param branchRule The branchRule that was created to map the branch region that
	 * starts at the output pin set.  (If null, indicates that there was no branch point).
	 * <link> elements that are targets of this rule must not be absorbed into the while <flow>
	 */
	protected void processChildTargets(Flow flow, TransformationRule branchRule){
		List childRules = getChildRules();
		TransformationRule nextRule = null;
		List targetList= null;
		Object target;
		for(int i=0, count = childRules.size(); i<count; i++){
			nextRule = (TransformationRule)childRules.get(i);
			targetList = nextRule.getTarget();
			int targetCount = targetList.size();
			for (int j=0; j< targetCount; j++){
				target = targetList.get(j);
				
				if (target instanceof Link && !nextRule.equals(branchRule)){
					/* Although we normally propagate links up to the top-level flow, here we want
					 * to keep them contained in this flow. We can't flatten this flow out later
					 * as we normally can, because it is inside a <while> which will have to stay. */
					if (flow.getLinks()==null)
					{
						Links links = BPELFactory.eINSTANCE.createLinks();
						flow.setLinks(links);
					}
					flow.getLinks().getChildren().add(target);
					
				} else if (target instanceof BPELVariable 
							|| target instanceof PartnerLink 
							|| target instanceof Link){	// These links must have come from branch region
					getTarget().add(target); //propagate all these types to parent
						
				} else if (target instanceof Activity){

					if((target instanceof Flow) && (target == getBranchFlow())){
						// propagate branch rule's flow to parent
						getTarget().add(target);
					} else if ((target instanceof Flow) && (((Flow)target).getActivities().isEmpty())){
						//do nothing since we don't want empty flow 						
					} else {	
						//add all other activity to this flow
						flow.getActivities().add(target);
					}
				} else{
					// There are no other types that this could be, but just in case...
					getTarget().add(target); //propagate all other unknown type to parent
				}
			}
		}
	
	}	
	/**
	 * Converts a BOM LoopNode expression into a BPEL While Condition (as an Expression Model 
	 * instance, wrapped by an ExtensibilityExpression).  The Expression will be added as a E
	 * 
	 * @param loopNode The BOM LoopNode that has the LoopCondition
	 * @return ExtensibilityExpression - The extensibility element that wraps the Expression Model
	 */
	private ExtensibilityExpression transformWhileCondition(LoopNode loopNode){
		
		// Ray: try to handle null expression
		if (loopNode.getLoopCondition() instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)loopNode.getLoopCondition()).getExpression()!=null){
			ExpressionRule expRule = AbstractbpelFactory.eINSTANCE.createExpressionRule();
			expRule.getSource().add(((StructuredOpaqueExpression)loopNode.getLoopCondition()).getExpression());
			this.getChildRules().add(expRule);
			expRule.transformSource2Target();
			Expression expr =(Expression)expRule.getTarget().get(0);
			ExtensibilityExpression extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			extExpr.setExpression(expr);
			return 	extExpr;
		}
		// Log a warning
		String params[] = new String[1];
		params[0] = BomUtils.getCanonicalName(loopNode,false,true);
		LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);		 	

		return null;
		
	}
	
	/**
	 * Creates an alternative activity to initialize the variable part(s) that keeps track of
	 * the number of elements in the final output variable for each part.  This is only really 
	 * required for BPEL, since in BPEL+ (Java) there are no index variables needed.
	 *  
	 * @return Activity - The AlternativeActivity
	 */
	private Activity createInitActivity(OutputPinSet outputPinSet){
		
		AlternativeActivity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
// Ray May 05/04:  Since for BPELP, we will only create an empty which is not useful.  There we should skip it and not create it at all
//		Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		
		NameProvider nameProvider = NameProviderFactory.getNameProvider(altActivity);
		String convertedName = nameProvider.getName(altActivity, outputPinSet, NamingUtil.findRegistry(this));
		
//		String convertedName = fConverter.convertName(NamingUtil.findRegistry(this), altActivity, BomUtils.getCanonicalName(outputPinSet) + AbstractbpelConstants.LOOP_NODE_INDEX_INIT_ACTIVITY_NAME_POSFIX);
		altActivity.setName(convertedName);
		
		bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL);
//		Ray May 05/04:  Since for BPELP, we will only create an empty which is not useful.  There we should skip it and not create it at all
//		bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
//		bpelpAlternative.setActivity(BPELFactory.eINSTANCE.createEmpty());

		Assign initAssign = BPELFactory.eINSTANCE.createAssign();
		
		Part curOsPart = null;
		/* Initialize each of the index parts associated with the primary output variable.  */
		for (Iterator it = fOsIndexVar.getMessageType().getEParts().iterator(); it.hasNext(); ){
			curOsPart = (Part)it.next();
			
			if (curOsPart.equals(fAssignIndexPart)){
				/* No need to initialize this part now... it will be done later */
				continue;
			}
			
			Copy initCopy = BPELFactory.eINSTANCE.createCopy();
			From initFrom = BPELFactory.eINSTANCE.createFrom();
			To initTo = BPELFactory.eINSTANCE.createTo();
	
			ExtensibilityExpression extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			IntegerLiteralExpression intLitExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
			intLitExpr.setIntegerSymbol(new Integer(1));
			extExpr.setExpression(intLitExpr);
			initFrom.getExtensibilityElements().add(extExpr);
			
//			initFrom.setExpression("'1'");
			initTo.setVariable(fOsIndexVar);
			initTo.setPart(curOsPart);

			initAssign.getCopy().add(initCopy);
			initCopy.setFrom(initFrom);
			initCopy.setTo(initTo);
		
			
		}
		
		if (initAssign.getCopy().size() > 0){
			bpelAlternative.setActivity(initAssign);		
			altActivity.getAlternative().add(bpelAlternative);
//			Ray May 05/04:  Since for BPELP, we will only create an empty which is not useful.  There we should skip it and not create it at all
//			altActivity.getAlternative().add(bpelpAlternative);
		
			return altActivity;					
		} else {
			/* There are no indices needed since there are no parts in the output pin set */
			return null;
		}
	
	}
	
	/**
	 * Connect the LoopNode OutputPinSet with internal links, connect the internal representation
	 * of the OutputPinSet to the external representation, and connect the external represention
	 * to the external links
	 * @return
	 */
	private void doOutputPinSetConnection(OutputPinSet outputPinSet, Sequence parentSequence, Flow flow){

		Activity assignFromOSVarToTempVar = null;	// done each iteration (only exists if FIndicesNeeded)
		Activity terminationActivity = null;		// done each iteration (always exists)
		Activity assignFromTempVarToOSVar = null;	// done at the end of all iterations (only exists if fIndicesNeeded)
		Activity outputSetActivity = null;			// done at the end of all iterations (only exists if there are special pins in the output set)
				
		/* Create the actual output set representation.  We have to do this step before we set an activity as
		 * the target of the links or else that activity will cached as the target and will be returned by the 
		 * mapOutputPinSet(...) method */
		List resultList = mapOutputPinSet(outputPinSet);
		if (!resultList.isEmpty()&& resultList.get(0) instanceof Activity){
			outputSetActivity = (Activity)resultList.get(0);
			getTarget().addAll(resultList.subList(1, resultList.size())); 
			
			NameProvider nameProvider = NameProviderFactory.getNameProvider(outputSetActivity);
			String convertedName = nameProvider.getName(outputSetActivity, outputPinSet, NamingUtil.findRegistry(this));
			
			//String convertedName = fConverter.convertName(NamingUtil.findRegistry(this), outputSetActivity, BomUtils.getCanonicalName(outputPinSet));
			outputSetActivity.setName(convertedName);
		}

				
		/* We always need an activity to set the termination variable value to end each iteration */
		Activity termActivity = getTerminationActivity(outputPinSet, false);
		

		if (fIndicesNeeded){

			/* Create the alternative activity for the two ways in which we can update the storage
			 * variable with the values in the acutal output set variable */
			assignFromOSVarToTempVar = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//			TODO ??? change  AR to access NameProvider instead
			assignFromOSVarToTempVar
				.setName(fConverter.convertName(NamingUtil.findRegistry(this), 
											   assignFromOSVarToTempVar, 
											   BomUtils.getCanonicalName((NamedElement)outputPinSet.eContainer()) 
											   		+ AbstractbpelConstants.LOOP_NODE_OS_VAR_TO_TEMP_VAR_ACTIVITY_NAME_POSTFIX));
			
			
			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL);
			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
			bpelAlternative.setActivity(createBpelTempStorageActivity(outputPinSet));
			bpelpAlternative.setActivity(createJavaTempStorageActivity(outputPinSet));
			((AlternativeActivity)assignFromOSVarToTempVar).getAlternative().add(bpelAlternative);
			((AlternativeActivity)assignFromOSVarToTempVar).getAlternative().add(bpelpAlternative);

			
			/* Create the activity that replicates the data from the storage variable into the actual
			 * output set variable. This is where the activities that need the data will look for it.
			 * This must be done after all the iterations have finished and the final data is in the temp var */ 			
			assignFromTempVarToOSVar = createPrimaryStorageActivity(outputPinSet);
			assignFromTempVarToOSVar
				.setName(fConverter.convertName(NamingUtil.findRegistry(this), 
											   assignFromOSVarToTempVar, 
											   BomUtils.getCanonicalName((NamedElement)outputPinSet.eContainer()) 
													+ AbstractbpelConstants.LOOP_NODE_TEMP_VAR_TO_OS_VAR_ACTIVITY_NAME_POSTFIX));
		}
		
		
		/* Now, based on what activities are null and which ones are not, configure the 
		 * Flow and parent Sequence */
		 
		if (assignFromOSVarToTempVar != null){
			flow.getActivities().add(assignFromOSVarToTempVar);
			setTargetOfLinks(outputPinSet, assignFromOSVarToTempVar);	
			Link termLink = 
				linkActivities(assignFromOSVarToTempVar, 
							   termActivity, 
							   assignFromOSVarToTempVar.getName() 
							   	+ "_to_" 
							   	+ termActivity.getName());
			if (flow.getLinks()==null)
			{
				Links links = BPELFactory.eINSTANCE.createLinks();
				flow.setLinks(links);
			}							   	
			flow.getLinks().getChildren().add(termLink);
			
		} else {
			setTargetOfLinks(outputPinSet, termActivity);		
		}
		
		/* We will always have the termActivity, and it must always be last in the flow */
		flow.getActivities().add(termActivity);
		
		/* The existence and form of outputSetActivity determines where we put it and also where
		 * we put assignFromTempVarToOSVar, since we only need once <sequence>  */
		if (outputSetActivity == null){
			if (assignFromTempVarToOSVar != null){
				parentSequence.getActivities().add(assignFromTempVarToOSVar);
			}
			
		} else if (outputSetActivity instanceof Sequence){
			if (assignFromTempVarToOSVar != null){
				/* Put the assign at the front, since it will put the data into the variable in which
				 * any output set activities will look for it. */
				((Sequence)outputSetActivity).getActivities().add(0, assignFromTempVarToOSVar);
			}
			parentSequence.getActivities().add(outputSetActivity);
			
		} else if (outputSetActivity instanceof Activity){
			if (assignFromTempVarToOSVar != null){
				/* Put the assign at the front, since it will put the data into the variable in which
				 * any output set activities will look for it. */
				Sequence osSequence = BPELFactory.eINSTANCE.createSequence();
				osSequence.getActivities().add(assignFromTempVarToOSVar);
				osSequence.getActivities().add(outputSetActivity);
				parentSequence.getActivities().add(osSequence);
			} else {
				parentSequence.getActivities().add(outputSetActivity);
			}
		}
				
		/* We could have also set the outputSetSequence as the source of the links, but then 
		 * we would have a link that crosses a <sequence> boundary, which may not be allowed */
		setSourceOfLinks(outputPinSet, parentSequence);
		
	}


	/**
	 * Creates the primary variable to represent the output pin set, and the variable used as the 
	 * temporary storage place while the loop is iterating.  With each interation of the loop, 
	 * more information is added to the temporary storage variable until the loop is finished 
	 * and all the information is copied back to the primary variable associated with the output set.
	 * 
	 * The temporary storage variable created will be stored in the <code>osStorageVar</code> 
	 * instance variable so it can be accessed by other methods in this class.
	 * 
	 * @param outputPinSet The single regular OutputPinSet on the BOM LoopNode 
	 */
	private void createStorageVariables(OutputPinSet outputPinSet){
		
		/* Create the primary variable associated with the output pin set */
		if (!outputPinSet.getOutputObjectPin().isEmpty()) {
			ContainerRule cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
			cr.getSource().add(outputPinSet);
			getChildRules().add(cr);
			cr.transformSource2Target();
		}
		 
		/* Get the WSDL Definition into which the messages will be placed */
		TransformationContext context = getContext();
		String key = createNamespace(outputPinSet);
		TransformationRule piRule = ProcessUtil.getProcessInterfaceRule(context, outputPinSet);
		Definition wsdlDef = (Definition)piRule.getTarget().get(0);
		
		String osName = BomUtils.getCanonicalName(outputPinSet);
		
		/* Create the storage variable */
		fOsStorageVar = BPELFactory.eINSTANCE.createBPELVariable();
		fOsStorageVar.setName(fConverter.convertName(NamingUtil.findRegistry(this), 
													 fOsStorageVar, 
													 osName + "LoopStorageVar"));

		/* Create and set the Message that will be used for the temporary storage variable */
		MessageRule internalOSMessageRule = WsdlFactory.eINSTANCE.createMessageRule();
		internalOSMessageRule.getSource().add(outputPinSet);
		piRule.getChildRules().add(internalOSMessageRule);
		internalOSMessageRule.transformSource2Target();
		Message internalOSMessage = (Message)internalOSMessageRule.getTarget().get(0);
		fOsStorageVar.setMessageType(internalOSMessage);
		
		/* Add the variables created by hand to the targets of this rule so they will be propagated */
		getTarget().add(fOsStorageVar);
		
		
	}
	
	/**
	 * Creates the temporary variable that has parts used as the index counters while
	 * the loop is iterating.  
	 * 
	 * The variable created will be stored in the <code>osIndexVar</code> instance variable
	 * so that they can be accessed by other methods in this class.
	 * 
	 * @param outputPinSet The single regular OutputPinSet on the BOM LoopNode 
	 */
	private void createBpelIndexVariable(OutputPinSet outputPinSet){
		
		/* If there are no object pins in the outputPinSet, then we don't need any index vars */
		if (outputPinSet.getOutputObjectPin().isEmpty()){
			return;
		}
		
		/* Get the WSDL Definition into which the messages will be placed */
		TransformationContext context = getContext();
		String key = createNamespace(outputPinSet);
		TransformationRule piRule = ProcessUtil.getProcessInterfaceRule(context, outputPinSet);
		Definition wsdlDef = (Definition)piRule.getTarget().get(0);
		
		String osName = BomUtils.getCanonicalName(outputPinSet);
		
		/* We will create indices based on the parts that are in the temporary storage variable
		 * previously created. */
		Message internalOSMessage = (Message)fOsStorageVar.getMessageType();
		
		/* Create the variable that keeps track of the current index in each part of the storage variable */
		fOsIndexVar = BPELFactory.eINSTANCE.createBPELVariable();
		fOsIndexVar.setName(fConverter.convertName(NamingUtil.findRegistry(this), 
												   fOsIndexVar, 
												   osName + "IndexVar"));
		Message osIndexMessage = WSDLFactory.eINSTANCE.createMessage();
		osIndexMessage.setQName(new QName(wsdlDef.getTargetNamespace(), 
										  fConverter.convertName(NamingUtil.findRegistry(this), 
																 osIndexMessage,
																 osName + "IndexMessage")));
		fOsIndexVar.setMessageType(osIndexMessage);
		/* We need an index part for each data part in the output set variable */
		for (int i=0, count = internalOSMessage.getEParts().size(); i<count; i++){
			Part indexPart = WSDLFactory.eINSTANCE.createPart();
			indexPart.setName(fConverter.convertName(NamingUtil.findRegistry(this), 
												     indexPart,
												     (((Part)internalOSMessage.getEParts().get(i)).getName()+ "IndexPart")));
			indexPart.setTypeDefinition(intXSDType);
			osIndexMessage.addPart(indexPart);
		}

		/* We need one more part to keep track of the index in the variable that we
		 * are assigning *from*. We only need one (and not one for each part) because we can
		 * reuse it for each of the parts.  */
		fAssignIndexPart = WSDLFactory.eINSTANCE.createPart();
		fAssignIndexPart.setName(fConverter.convertName(NamingUtil.findRegistry(this),
														fAssignIndexPart,
														osName + "AssignmentIndexPart"));
		fAssignIndexPart.setTypeDefinition(intXSDType);
		osIndexMessage.addPart(fAssignIndexPart);

		/* Add the message to the WSDL definition so that it will show up in the WSDL */
		wsdlDef.addMessage(osIndexMessage);	
		
		/* Add the variable created by hand to the targets of this rule so it will be propagated */
		getTarget().add(fOsIndexVar);
		
		
	}
	
	/**
	 * This method creates the activity that assigns the values from the actual output set variables
	 * into the temporary storage variables using the indices calculated so far, 
	 * and then updates the indices to point to the next spot. 
	 * 
	 * @param loopNode The BOM LoopNode
	 * @return
	 */
	private Activity createBpelTempStorageActivity(OutputPinSet outputPinSet){
		
		/* If there are no output object pins in the output pin set, there is nothing to copy */
		if (outputPinSet.getOutputObjectPin().isEmpty()){
			return null;
		}
		
		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
		
		/* Get the variable associated with the output set. This is where we will copy from */
		BPELVariable outputSetVar = getVariable(outputPinSet);
				
		Part curOSPart = null;
		Part curStorageIndexPart = null;
		for (int i=0, count = outputSetVar.getMessageType().getEParts().size(); i< count; i++){

			/* Before we copy the data from the part associated with a particular input pin into the 
			 * corresponding part in the temp storage variable, we have to reset the reset the index that 
			 * says what element in the source part we are copying, since we always want to start at 1 */
			Assign initAssign = BPELFactory.eINSTANCE.createAssign();
			Copy initCopy = BPELFactory.eINSTANCE.createCopy();
			From initFrom = BPELFactory.eINSTANCE.createFrom();
			To initTo = BPELFactory.eINSTANCE.createTo();

			ExtensibilityExpression extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			IntegerLiteralExpression intLitExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
			intLitExpr.setIntegerSymbol(new Integer(1));
			extExpr.setExpression(intLitExpr);
			initFrom.getExtensibilityElements().add(extExpr);

//			initFrom.setExpression("'1'");
			initTo.setVariable(fOsIndexVar);
			initTo.setPart(fAssignIndexPart);
	
			initAssign.getCopy().add(initCopy);
			initCopy.setFrom(initFrom);
			initCopy.setTo(initTo);		
		
			sequence.getActivities().add(initAssign);


			curOSPart = (Part)outputSetVar.getMessageType().getEParts().get(i);
			curStorageIndexPart = (Part)fOsStorageVar.getMessageType().getEParts().get(i);
			
			/* Create the <while> that will copy each item in a partcular output set pin part into
			 * the corresponding part in the temp storage variable */
			While storageWhile = BPELFactory.eINSTANCE.createWhile();
			extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();

			FunctionExpression countExpr = ModelFactory.eINSTANCE.createFunctionExpression();
			FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
			def.setFunctionID(CoreFunctions.SIZE_FUNCTION);
			def.setFunctionName("count");
			countExpr.setDefinition(def);
			
			FunctionArgument countArg = ModelFactory.eINSTANCE.createFunctionArgument();
			countArg.setArgumentValue(createGetVariableDataExpr(outputSetVar, curOSPart));
			countExpr.getArguments().add(countArg);
			
			ComparisonExpression whileExpr = ModelFactory.eINSTANCE.createComparisonExpression();
			whileExpr.setFirstOperand(createGetVariableDataExpr(fOsIndexVar, fAssignIndexPart));
			whileExpr.setOperator(ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL);
			whileExpr.setSecondOperand(countExpr);
			extExpr.setExpression(whileExpr);

			storageWhile.getExtensibilityElements().add(extExpr);
			
//			BooleanExpression storageExpression = BPELFactory.eINSTANCE.createBooleanExpression();
//			storageExpression.setBody(
//						"bpws:getVariableData('" + fOsIndexVar.getName() + "', "
//										   + "'" + fAssignIndexPart.getName() + "') "
//					  + " <= count(bpws:getVariableData('" + outputSetVar.getName() + "', "
//													+ "'" + curOSPart.getName() + "'))");
//			storageWhile.setCondition(storageExpression);

			sequence.getActivities().add(storageWhile);


			Sequence storageSequence = BPELFactory.eINSTANCE.createSequence();
			storageWhile.setActivity(storageSequence);
		
			/* This assign replicates from the outputSetVar to the osStorageVar, from the position defined
			 * by assignIndexPart, to the position defined by the index corresponding to each part */ 
			Assign toStorageAssign = BPELFactory.eINSTANCE.createAssign();
			Copy toStorageCopy = BPELFactory.eINSTANCE.createCopy();
			From toStorageFrom = BPELFactory.eINSTANCE.createFrom();
			//ToExtension toStorageTo = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createToExtension();
			To toStorageTo = BPELFactory.eINSTANCE.createTo();
			toStorageAssign.getCopy().add(toStorageCopy);
			toStorageCopy.setFrom(toStorageFrom);
			toStorageCopy.setTo(toStorageTo);
			
			toStorageFrom.setVariable(outputSetVar);
			toStorageFrom.setPart(curOSPart);
			extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			extExpr.setExpression(createSelectFunctionByPosition(curOSPart, fOsIndexVar, fAssignIndexPart));
			toStorageFrom.getExtensibilityElements().add(extExpr);
			
//			toStorageFrom.setSelect("/" + curOSPart.getType().getName() + "[position() = bpws:getVariableData('" + fOsIndexVar.getName() + "', "
//																	+ "'" + fAssignIndexPart.getName() + "')]");

			toStorageTo.setVariable(fOsStorageVar);
			toStorageTo.setPart(curStorageIndexPart);
			extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			extExpr.setExpression(createSelectFunctionByPosition(curStorageIndexPart, fOsIndexVar, curStorageIndexPart));
			toStorageTo.getExtensibilityElements().add(extExpr);

//			toStorageTo.setSelect("/" + curStorageIndexPart.getType().getName() + "[position() = bpws:getVariableData('" + fOsIndexVar.getName() + "', "
//																	+ "'" + curStorageIndexPart.getName() + "')]");
																	
			storageSequence.getActivities().add(toStorageAssign);																	

			/* Now create the assign to increment both index variables */ 			
			Assign indexUpdateAssign = BPELFactory.eINSTANCE.createAssign();
			Copy indexUpdateCopy1 = BPELFactory.eINSTANCE.createCopy();
			From indexUpdateFrom1 = BPELFactory.eINSTANCE.createFrom();
			To indexUpdateTo1 = BPELFactory.eINSTANCE.createTo();
			
			indexUpdateAssign.getCopy().add(indexUpdateCopy1);
			indexUpdateCopy1.setFrom(indexUpdateFrom1);
			indexUpdateCopy1.setTo(indexUpdateTo1);

			extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			BinaryNumericExpression binNumExpr = ModelFactory.eINSTANCE.createBinaryNumericExpression();

			intLitExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
			intLitExpr.setIntegerSymbol(new Integer(1));

			binNumExpr.setFirstOperand(createGetVariableDataExpr(fOsIndexVar, fAssignIndexPart));
			binNumExpr.setOperator(BinaryMathematicalOperator.ADDITION_LITERAL);
			binNumExpr.setSecondOperand(intLitExpr);

			extExpr.setExpression(binNumExpr);
			indexUpdateFrom1.getExtensibilityElements().add(extExpr);
			
//			indexUpdateFrom1.setSelect("bpws:getVariableData('" + fOsIndexVar.getName() + "', "
//																+ "'" + fAssignIndexPart.getName() + "') + 1");
			indexUpdateTo1.setVariable(fOsIndexVar);
			indexUpdateTo1.setPart(fAssignIndexPart);

			Copy indexUpdateCopy2 = BPELFactory.eINSTANCE.createCopy();
			From indexUpdateFrom2 = BPELFactory.eINSTANCE.createFrom();
			To indexUpdateTo2 = BPELFactory.eINSTANCE.createTo();
			
			indexUpdateAssign.getCopy().add(indexUpdateCopy2);
			indexUpdateCopy2.setFrom(indexUpdateFrom2);
			indexUpdateCopy2.setTo(indexUpdateTo2);
			
			extExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			binNumExpr = ModelFactory.eINSTANCE.createBinaryNumericExpression();

			intLitExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
			intLitExpr.setIntegerSymbol(new Integer(1));

			binNumExpr.setFirstOperand(createGetVariableDataExpr(fOsIndexVar, curStorageIndexPart));
			binNumExpr.setOperator(BinaryMathematicalOperator.ADDITION_LITERAL);
			binNumExpr.setSecondOperand(intLitExpr);

			extExpr.setExpression(binNumExpr);
			indexUpdateFrom2.getExtensibilityElements().add(extExpr);

//			indexUpdateFrom2.setSelect("bpws:getVariableData('" + fOsIndexVar.getName() + "', "
//																+ "'" + curStorageIndexPart.getName() + "') + 1");
			indexUpdateTo2.setVariable(fOsIndexVar);
			indexUpdateTo2.setPart(curStorageIndexPart);

			storageSequence.getActivities().add(indexUpdateAssign);																	

		}
		
		return sequence;
	}
	
	private FunctionExpression createSelectFunctionByPosition(Part part, BPELVariable indexVar, Part indexPart){
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();					
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID(CoreFunctions.SELECT_FUNCTION);

		/* First Argument (elements) for the select Function Expression is a ModelPathExpression*/
		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		ModelPathExpression pathExpr = ModelFactory.eINSTANCE.createModelPathExpression();
		StaticStep step1 = ModelFactory.eINSTANCE.createStaticStep();
		step1.setStepName(part.getName());
		StaticStep step2 = ModelFactory.eINSTANCE.createStaticStep();
		step2.setStepName(part.getTypeName().getLocalPart());
		pathExpr.setIsAbsolute(Boolean.TRUE);
		pathExpr.getSteps().add(step1);
		pathExpr.getSteps().add(step2);					
		arg1.setArgumentValue(pathExpr);

		/* Second argument (selection criteria) for the select Function Expression is a ComparisonExpression */
		FunctionArgument arg2 = ModelFactory.eINSTANCE.createFunctionArgument();
		ComparisonExpression cmpExpr = ModelFactory.eINSTANCE.createComparisonExpression();	
					
		/* first operand of ComparisonExpression is position() Function Expresssion with no arguments */				
		FunctionExpression posFunctionExpr = ModelFactory.eINSTANCE.createFunctionExpression();					
		FunctionDefinition def2 = ModelFactory.eINSTANCE.createFunctionDefinition();
		def2.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.position");
		def2.setFunctionName("position");
		posFunctionExpr.setDefinition(def2);

		cmpExpr.setFirstOperand(posFunctionExpr);		

		cmpExpr.setOperator(ComparisonOperator.EQUAL_TO_LITERAL);

		/* Second operand is a function */
		cmpExpr.setSecondOperand(createGetVariableDataExpr(indexVar, indexPart)); 				

		arg2.setArgumentValue(cmpExpr);					
		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		functionExpr.getArguments().add(arg2);
		
		return functionExpr;
		
	}

	private FunctionExpression createGetVariableDataExpr(BPELVariable var, Part part){
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData");
		def.setFunctionName("bpws:getVariableData");

		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr1.setStringSymbol(var.getName());
		arg1.setArgumentValue(strExpr1);

		FunctionArgument arg2 = ModelFactory.eINSTANCE.createFunctionArgument();
		StringLiteralExpression strExpr2 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr2.setStringSymbol(part.getName());
		arg2.setArgumentValue(strExpr2);

		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		functionExpr.getArguments().add(arg2);	
		
		return functionExpr;
		
	}
	
	private Invoke createJavaTempStorageActivity(OutputPinSet outputPinSet){
		//Invoke javaInvoke = BPELFactory.eINSTANCE.createInvoke();
		//AR change with a java invoke
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();

		/* Get the variable associated with the output set. This is where we will copy from */
		BPELVariable outputSetVar = getVariable(outputPinSet);
				
		String javaString = "";
		Part curOSPart = null;
		Part curTempStoragePart = null;
		ObjectPin pin = null;
		int outListInd=0;
		boolean lastIndexInitialized=false;
		for (int i=0, count = outputSetVar.getMessageType().getEParts().size(); i< count; i++){

			curOSPart = (Part)outputSetVar.getMessageType().getEParts().get(i);
			curTempStoragePart = (Part)fOsStorageVar.getMessageType().getEParts().get(i);

			/* To determine whether we need to treat these parts as lists or single values, 
			 * we need to check the multiplicity of the pin for which the part was created. */
			PartRule partRule = (PartRule)TransformationUtil.getRuleForTarget(getRoot(), PartRule.class, curOSPart);

			boolean treatAsList = false;
			if (partRule != null){
				if (!partRule.getSource().isEmpty() && partRule.getSource().get(0) instanceof ObjectPin){
					pin = (ObjectPin)partRule.getSource().get(0);
					//Ray
					treatAsList = isListOfElement(pin,curOSPart);
//					String upperBound = 
//						AbstractbpelUtil.getLiteralValue((LiteralSpecification)pin.getUpperBound())
//										.toString();
//					
//					/* If the upper bound is neither 1 nor unspecified (in which case it defaults
//					 * to 1), it must be a list... */
//					if (!("".equals(upperBound) || "1".equals(upperBound))){
//						treatAsList = true;
//					}
				}
			}
			
			/* Because the temp storage part and the output set part were both created based 
			 * on the same pin, they are either both lists, or both not lists... */
			if (treatAsList){
//				javaString += AbstractbpelUtil.generateJavaGetterForVariable(fOsStorageVar.getName()) + "(true)"
//							  + ".get" + curTempStoragePart.getName() + "()"
//							  +	".addAll("+AbstractbpelUtil.generateJavaGetterForVariable(outputSetVar.getName()) + "()"
//										 + ".get" + curOSPart.getName() + "());\n";
				// Ray - handle WSADIE 5.1 code gen (uses array '[]' instead of list)
				StringBuffer sb = new StringBuffer();
				if (!lastIndexInitialized) {
				   sb.append("int lastIndex=");
				   sb.append(outListInd);
				   sb.append(";\n");
				   lastIndexInitialized=true;
				}
				String curOSPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(outputSetVar.getName())+"()"
					   + ".get" + curOSPart.getName() + "()" 
					   + ".get" + AbstractbpelUtil.capitalizeFirstChar(getListOfElementName((XSDComplexTypeDefinition)curOSPart.getTypeDefinition()));
				String curTempStoragePartSetterName = AbstractbpelUtil.generateJavaGetterForVariable(fOsStorageVar.getName()) + "(true)"
					   + ".get"	+ curTempStoragePart.getName() + "()" 
					   + ".set" + AbstractbpelUtil.capitalizeFirstChar(getListOfElementName((XSDComplexTypeDefinition)curTempStoragePart.getTypeDefinition()));
				sb.append("for (int i=0; i<");
				sb.append(curOSPartGetterName);
				sb.append("().length; i++) {\n\t");
				sb.append(curTempStoragePartSetterName);
				sb.append("(lastIndex++,");
				sb.append(curOSPartGetterName);
				sb.append("(i));\n}\n");
				javaString += sb.toString();
				
			} else {
				javaString += AbstractbpelUtil.generateJavaGetterForVariable(fOsStorageVar.getName()) + "(true)"
							  + ".set" + curTempStoragePart.getName() + "("
							  +	AbstractbpelUtil.generateJavaGetterForVariable(outputSetVar.getName()) + "()"
										 + ".get" + curOSPart.getName() + "());\n";
				
			}
			
		}
//		TODO AR migration ???
//		javaCode.setValue(javaString);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaString);
		javaInvoke.getExtensibilityElements().add(jsActivity);
		
		return javaInvoke;
	}

	//Ray
	private boolean isListOfElement(Pin pin,Part part) {
		boolean isList=false;
		if (pin instanceof InputObjectPin) {
			isList = BomUtils.isMultipled((InputObjectPin)pin);
		} else {
			isList = BomUtils.isMultipled((OutputObjectPin)pin);
		}
		if (isList) {
			XSDTypeDefinition partType = part.getTypeDefinition();
			if (partType != null) {
				if (partType instanceof XSDComplexTypeDefinition) {
					// it may not be the safest to just check the name ...
					if (!partType.getName().startsWith("ListOf")) {
						isList = false;
					}
				} else {
					isList = false;
				}
			}
		}	
		return isList;	
	}

	//Ray
	private String getListOfElementName (XSDComplexTypeDefinition type) {
		String elementName = null;
		elementName = ((XSDElementDeclaration)((XSDParticle)((XSDModelGroup)type.getComplexType().getContent()).getContents().get(0)).getContent()).getName();
		return elementName;
	}	
		
	/**
	 * Creates the BPEL activity that assigns the values from the temporary storage variable
	 * into which the loop results were accumulated into the primary variable associated
	 * with the output set.  This activity will be executed after the LoopNode iterations 
	 * have completed.
	 *  
	 * @return Assign - The BPEL Activity that performs the assignment.
	 */
	private Assign createPrimaryStorageActivity(OutputPinSet outputPinSet){
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		assign.getCopy().add(copy);
		copy.setFrom(from);
		copy.setTo(to);
		
		/* Get the variable associated with the output set. This is where we will copy to */
		BPELVariable outputSetVar = getVariable(outputPinSet);

		from.setVariable(fOsStorageVar);
		to.setVariable(outputSetVar);
			
		return assign;
		
	}
	
	
	/**
	 * This method should be able to be reused for all named elements
	 * @param namedElement - a named element
	 * @return
	 */
	public static String createNamespace(NamedElement namedElement) {
		
		EObject element = namedElement;
		while (!(element instanceof PackageableElement) && (element != null)){
			element = element.eContainer();
		}
		
		if (element == null){
			return null;
		}
		
		String ns = ((PackageableElement)element).getName();
		Package belongingPackage = ((PackageableElement)element).getOwningPackage();
		while (belongingPackage != null && !ProcessUtil.isRootPackage(belongingPackage)) {
			ns = belongingPackage.getName() + "/" + ns;
			belongingPackage = belongingPackage.getOwningPackage();
		}
		return ProcessUtil.DEFAULT_PREFIX+ns;
	}	

	private BPELVariable getVariable(EObject ruleSource){
		
		BPELVariable var = (BPELVariable)((ContainerRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), 
																	ContainerRule.class, 
																	ruleSource)).getTarget().get(0);
		
		return var;
		
	}
	/**
	*	Returns the list of variables created for the pinSet. Will not attempt to create other variables.
	*/
	private List getVariablesForPinSet(PinSet pinSet){
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		List variables = new LinkedList();
		if(!objectPins.isEmpty()) {
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin op = (ObjectPin)it.next();
			
				ArrayList thisSources = new ArrayList();
				thisSources.add(((TypedElement)op).getType());
				thisSources.add(op);
				ArrayList targets = new ArrayList();
		//		BPELVariable variable = createVariable(op);
				TransformationRule rule = TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), VariableRule.class, thisSources, targets);
				if(rule != null && !rule.getTarget().isEmpty()){
					variables.add ((BPELVariable)rule.getTarget().get(0));
				}
			}
		}
		return variables;
	}
	
	/**
	 * Places the activity that initializes the termination node variable into the flow, and
	 * connects it to the activities that are immediately inside the LoopNode.  This causes
	 * the termination initialization activity to be executed at the beginning of each
	 * loop iteration.
	 * 
	 * @param flow The flow to insert the termination initialization activity into
	 * @param termInitActivity The activity that initializes the termination node variable
	 * @param inputSetList The list of input sets that represent the actions immediately inside
	 * the loop node (i.e. the actions that should be executed after the termination node 
	 * is initialized) 
	 */
	private void connectTerminationInitActivity(Flow flow, Activity termInitActivity, List inputSetList){
		
		flow.getActivities().add(0, termInitActivity);
		
		InputPinSet inputSet = null;
		LinkRule newLinkRule = null;
		for (Iterator it = inputSetList.iterator(); it.hasNext(); ){
			inputSet = (InputPinSet)it.next();
			
			newLinkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
			newLinkRule.getSource().add(inputSet);

			/* Execute the new link rule */
			getChildRules().add(newLinkRule);
			newLinkRule.transformSource2Target();
			
			/* Set the termination initialization activity as the source of the link. 
			 * This won't be done automatically later as it normally is for links because we
			 * never call setSourceOfLinks(...) using the termInitActivity, since there is no
			 * corresponding BOM object for the activity  */
			Source source = BPELFactory.eINSTANCE.createSource();
			source.setActivity(termInitActivity);
			source.setLink((Link)newLinkRule.getTarget().get(0));
		
		}

	}
	
} //LoopNodeRuleImpl
