/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.MergeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CycleLinkUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.abstractbpel.JoinExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Query;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Merge Rule</b></em>'.
 * 
 * <p>This class is responsible for mapping a BOM merge node into the appropriate BPEL Activities</p> 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class MergeRuleImpl extends ConnectableRuleImpl implements MergeRule {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/* The activities that represents the input set, output set and merge itself */
	private Activity fIsActivity = null;
	private Activity fOsActivity = null;
	private Activity fAssign = null;
	private boolean createdEmpty = false;
	boolean bNeedAssign = false;
	boolean allVariablesMapped = false;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MergeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.MERGE_RULE;
	}

	/**
	 * source = input set on a BOM Merge node
	 * target = [<activity>, [ (<link>,<assign>,<link>) | <link>], <acitivity>] 
	 * 			 + non-activites from input and output set mapping
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource() == null || getSource().isEmpty() || !(getSource().get(0) instanceof InputPinSet)){
			/* We were not fed the correct source, so throw an exception */
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			throw new IllegalArgumentException("Must supply a source before the rule can be executed");
		}

		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
		
		if (getTarget() != null && !getTarget().isEmpty()){
			/* We must have previously executed this rule */
			reExecute(inputSet);
			/* Return true, event though there may always be more links to connect to
			 * the input / output pins */
			
			executeHandlers();
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return false;
		}
		/* An input set must have at least one correlated output set, so this shouldn't fail */
		List isVariables = null;
		List osVariables = null;
		List objectsToPropagate = new ArrayList();

		/* Since this is the first time we are executing this rule, create the input set container */
		if (!inputSet.getInputObjectPin().isEmpty()) {
			isVariables = getVariablesForPinSet(inputSet);
		}
		if (!outputSet.getOutputObjectPin().isEmpty()) {
			osVariables = getVariablesForPinSet(outputSet);
		}
//20090522		boolean bNeedAssign;
		int numUnconnectedPins = 0;
		List objectPinsInSet = BomUtils.getObjectPinsForSet(inputSet);
		if (objectPinsInSet != null && !objectPinsInSet.isEmpty()) {
			// count unconnected pins (no need to map Variables for empty pins)
			for(Object thisPin: BomUtils.getObjectPinsForSet(inputSet)){
				if(((Pin)thisPin).getIncoming() == null) numUnconnectedPins = numUnconnectedPins++;
			}
		
			// EXCLUDING empty pins - check if all variables are mapped
			if(isVariables.size() == (objectPinsInSet.size() - numUnconnectedPins)){
				allVariablesMapped = true;
			}
		}
		if(isVariables == null && osVariables == null){
			bNeedAssign = false;
		}else{
			bNeedAssign = !(isVariables != null && osVariables != null &&  isVariables.size() == osVariables.size() && isVariables.containsAll(osVariables));
		}
		/* Map the input set and deal with any complex pins, if necessary */
		List isResults = mapInputPinSet(inputSet);
		if (isResults.isEmpty() || !(isResults.get(0) instanceof Activity)){
			/* There were no activities created, so create a BPEL <empty> that will sync the flow */
			fIsActivity = BPELFactory.eINSTANCE.createEmpty();
			NameProvider nameProvider = NameProviderFactory.getNameProvider(fIsActivity);
			String convertedName = nameProvider.getName(fIsActivity, inputSet, NamingUtil.findRegistry(this));
			fIsActivity.setName(convertedName);
			createDisplayName(fIsActivity,inputSet.getAction().getName()+" "+inputSet.getName());			
		} else {
			/* An activity has been created for input set synchronization, so use it */
		    fIsActivity = (Activity)isResults.get(0);
			objectsToPropagate.addAll(isResults.subList(1, isResults.size()));
		}

// check if this was created by Decision's merge point and create a link from decision if this is the case
//Flow flowInExclusiveBranch = null; 		
//if (getParentRule() instanceof ExclusiveBranchRuleImpl) {
//	ExclusiveBranchRuleImpl exBranchRule = (ExclusiveBranchRuleImpl)getParentRule();
//	if ((exBranchRule.getMergePoint() == inputSet.getAction() || exBranchRule.getMergePoint() == outputSet) &&
//			exBranchRule.getTarget() != null && 
//			!exBranchRule.getTarget().isEmpty() && exBranchRule.getTarget().get(0) instanceof Flow) {
//		flowInExclusiveBranch = (Flow)exBranchRule.getTarget().get(0);
//		String linkName =  flowInExclusiveBranch.getName() + "_to_" + BomUtils.getCanonicalName(inputSet);
//		Link link = linkActivities(flowInExclusiveBranch, fIsActivity, linkName);
//		if (flowInExclusiveBranch.getLinks() == null) {
//			Links links = BPELFactory.eINSTANCE.createLinks();
//			flowInExclusiveBranch.setLinks(links);
//		}
//		flowInExclusiveBranch.getLinks().getChildren().add(link);
//	}
//}
		/* Add this <activity> to the list of targets right away, so that it will appear before
		 * any upcoming <assign>s if persisted in list order.  Note that this must be BEFORE the <activity>
		 * corresponding to the output set, as will be seen later. */		
		getTarget().add(fIsActivity);
		
		/* For each input pin in the source input set, set isActivity as the target and 
		 * update the join condition */	
		setTargetOfLinks(inputSet, fIsActivity);
		
		/* Figure out if a merge rule has already been created for another input set on the same
		 * Merge.  If it has, we won't need to create an output set <activity>... it will exist already */
		MergeRule mr = null;
		InputPinSet tempInputPinSet = null;
		for (Iterator it = inputSet.getAction().getInputPinSet().iterator(); it.hasNext() && mr == null; ){
			tempInputPinSet = (InputPinSet)it.next();
			if (tempInputPinSet != inputSet){
				mr = (MergeRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
															MergeRule.class,
															tempInputPinSet); 
			}
		}
		
		if (mr != null){
			/* If the MergeRule above exists, then the container rule must also exist */
			/* There should be two or three <activities> at the beginning of the 
			 * list of targets from mr.  If there are input objects pins in InputSet, it means that
			 * the third <activity> corresponds to the output set, otherwise, the second <activity> 
			 * will, since only a <link> was created between the two activities, and not an <assign> */
			List targets = mr.getTarget();
			Object target = null;
			int activityCount = 0;
			//int activityNum = (inputSet.getInputObjectPin().isEmpty() ? 2 : 3);
			List tempIsVariables = null;
			if (!tempInputPinSet.getInputObjectPin().isEmpty()){
				tempIsVariables = getVariablesForPinSet(tempInputPinSet);	
			}
			
			boolean bNeededAssign;
			if(tempIsVariables == null && osVariables == null){
				bNeededAssign = false;
			}else{
				bNeededAssign = !(tempIsVariables != null && osVariables != null &&  tempIsVariables.size() == osVariables.size() && tempIsVariables.containsAll(osVariables));
			}
			int activityNum = (!bNeededAssign ? 2 : 3);
				for (Iterator it = targets.iterator(); it.hasNext() && activityCount < activityNum; ){
				target = it.next();
				if (target instanceof Activity){
					if (++activityCount == activityNum){
						fOsActivity = (Activity)target;
					}
				}
			}		
				
		} else {
			/* No existing merge rule was found, so create the <activity> corresponding to the output set,
			 * and the container rule to create the associated container */
			List osResults = mapOutputPinSet(outputSet);
			if (osResults.isEmpty() || !(osResults.get(0) instanceof Activity)){
				/* No activities were created to handle the output set, so create an <empty> */
			    fOsActivity = BPELFactory.eINSTANCE.createEmpty();
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), fOsActivity, BomUtils.getCanonicalName(outputSet));
				fOsActivity.setName(convertedName);
			} else {
			    fOsActivity = (Activity)osResults.get(0);
				/* Keep track of the rest of the objects to propage, because the target order is important */
				objectsToPropagate.addAll(osResults.subList(1, osResults.size()));
			}
			createDisplayName(fOsActivity,inputSet.getAction().getName());
		}

		if (bNeedAssign && allVariablesMapped){
			
			createAssign(inputSet, outputSet);
			/* Create an <assign> to copy the incoming data to the container associated 
			 * with the output set */
			/**
			 * Assumption :: the BOM Type between the input and output pin of Merge must be the same
			 * Then there are 2 cases.
			 * 1.  They have same lower and upper bound.  In such case they will share the same WSDL message def and a simple assign will handle this
			 * 2.  They have different lower and upper bound.  In such case, for BPEL, we need assign with XPath.  For BPELP, we need java snippets
			 *  
			 */
//			fAssign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL);
//			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
//			//	bpelAlternative.setActivity(createMultiplicityCopyAssign(inputSet,inputSetContainer,outputSet,outputSetContainer));
//			bpelpAlternative.setActivity(createMultiplicityCopyJavaInvoke(inputSet,outputSet,true));				
//			((AlternativeActivity)fAssign).getAlternative().add(bpelAlternative);
//			((AlternativeActivity)fAssign).getAlternative().add(bpelpAlternative);
//	
//			
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), fAssign, BomUtils.getCanonicalName(inputSet.getAction()));
//			fAssign.setName(convertedName);
//			DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
//			dspName.setText(inputSet.getAction().getName()+" Assign");
//			fAssign.getEExtensibilityElements().add(dspName);
//			/* Add this <assign> to the list of targets of this rule */
//			getTarget().add(fAssign);
//			
//			/* Create the links that connect the input set <activity> to this <assign>, and this assign
//			 * to the output set <activity> */
//			Link link = BPELFactory.eINSTANCE.createLink();
//
//			Source isActivitySrc = BPELFactory.eINSTANCE.createSource();
//			isActivitySrc.setActivity(fIsActivity);
//			link.getSources().add(isActivitySrc);
//
//			Target assignTar = BPELFactory.eINSTANCE.createTarget();
//			assignTar.setActivity(fAssign);
//			link.getTargets().add(assignTar);
//
//				
//			convertedName = converter.convertName(NamingUtil.findRegistry(this), link, fIsActivity.getName() + "_to_" + inputSet.getName() + "Assign");
//			link.setName(convertedName);
//			/* Add this link to the list of targets of this rule */
//			getTarget().add(link);
//			
//			LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
//			this.getChildRules().add(lr);
//			lr.getSource().add(outputSet);
//			lr.transformSource2Target();
//			link = (Link)lr.getTarget().get(0);
//			
//			Source assignSrc = BPELFactory.eINSTANCE.createSource();
//			assignSrc.setActivity(fAssign);
//			link.getSources().add(assignSrc);
//
//			Target osActivityTar = BPELFactory.eINSTANCE.createTarget();
//			osActivityTar.setActivity(fOsActivity);
//			link.getTargets().add(osActivityTar);
//				
//			convertedName = converter.convertName(NamingUtil.findRegistry(this), link, inputSet.getName() + "Assign_to_" + fOsActivity.getName());
//			link.setName(convertedName);
//			/* Add this link to the list of targets of this rule */
//			getTarget().add(link);

		} else {
			/* There are no object pins, so we need to create a single link that will synchronize all
			 * the control pins, and link from the opening <empty> to the closing <empty> 			 */
			LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
			this.getChildRules().add(lr);
			lr.getSource().add(outputSet);
			lr.transformSource2Target();
			Link link = (Link)lr.getTarget().get(0);

			Source osActivitySrc = BPELFactory.eINSTANCE.createSource();
//if (flowInExclusiveBranch != null) {
//	osActivitySrc.setActivity(flowInExclusiveBranch);
//} else { // code prior to GFlow changes
			osActivitySrc.setActivity(fIsActivity);
//}
			link.getSources().add(osActivitySrc);
			
			//We need to capture the empty here to 
//20090522	if(CycleLinkUtil.keepEmptyAsStartActivty(inputSet,getContext())){
			// we need to ensure the non-cycle input pin set of the merge node	
			MergeRuleImpl mrCycle = null;
			InputPinSet tempInputPinSetCycle = null;
			boolean createdAlready = false;
			for (Iterator it = inputSet.getAction().getInputPinSet().iterator(); it.hasNext() && !createdAlready; ){
				tempInputPinSetCycle = (InputPinSet)it.next();
				if (tempInputPinSetCycle != inputSet){
					mrCycle = (MergeRuleImpl)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
																MergeRule.class,
																tempInputPinSet);
					if (mrCycle!= null && mrCycle.createdEmpty) {
						createdAlready = true;
					}
				}
			}
			//This should be one of the regular input pin set
			//20090522	if(mrCycle instanceof MergeRuleImpl){
			if(!createdAlready){
				//20090522		Activity retEmpty = ((MergeRuleImpl)mrCycle).getInputSetActivity();					
				//20090522	if(!CycleLinkUtil.hasStartElement(inputSet)&& retEmpty != null){
				if(CycleLinkUtil.isMergeInputSetFirstNodeConnectedInLocalContainer(inputSet)){
					//20090522	BpelOptimizationUtil.registerNonOptimizableElement(getContext(), retEmpty);	
					BpelOptimizationUtil.registerNonOptimizableElement(getContext(), getInputSetActivity());
					createdEmpty = true;
				}
			}
//20090522}
						
			Target osActivityTar = BPELFactory.eINSTANCE.createTarget();
			osActivityTar.setActivity(fOsActivity);
			link.getTargets().add(osActivityTar);

			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), link, fIsActivity.getName() + "_to_" + fOsActivity.getName());			
			link.setName(convertedName);
			/* Add this link to the list of targets of this rule */
			getTarget().add(link);		

		boolean faultLinkCandidate = false;
		OutputPinSet ops = null;int j=0;
		List pinList = BomUtils.getPinsInSet(inputSet);
		if(pinList != null && pinList.size() == 1){
			if 	(pinList.get(0) instanceof ObjectPin) {
				ObjectPin op = (ObjectPin)pinList.get(0);
				if(op.getIncoming().getSource()instanceof OutputObjectPin){
					OutputObjectPin oop = (OutputObjectPin)op.getIncoming().getSource();
					PinSet ps = BomUtils.findPinSetForPin(oop);
					if(ps instanceof OutputPinSet){
						ops = (OutputPinSet)ps;
						if(ops.getIsException().booleanValue()){
							faultLinkCandidate = true;
						}
					}
				}
			}else if(pinList.get(0) instanceof ControlPin){
				ControlPin op = (ControlPin)pinList.get(0);
				if(op.getIncoming().getSource()instanceof OutputControlPin){
					OutputControlPin ocp = (OutputControlPin)op.getIncoming().getSource();
					PinSet cps = BomUtils.findPinSetForPin(ocp);
					if(cps instanceof OutputPinSet){
						ops = (OutputPinSet)cps;
						if(ops.getIsException().booleanValue()){
							faultLinkCandidate = true;
						}
					}
				}
			}
		}
		if(faultLinkCandidate){
			//Find the InvokeRule that is the parent of the merge rule
			TransformationRule targetRule = this.getParentRule();
			while(!(targetRule instanceof InvokeRule) && targetRule != null){
				targetRule = targetRule.getParentRule();
			}
			if(targetRule instanceof InvokeRule){
				CBPELUtil.registerFaultSource(getContext(), targetRule, ops.getName(), link);
			}
		  }			
		}
		/* Update the join condition so closing <activity> reflects the links just created */
		updateMergeOutputPinSetJoinCondition(outputSet, fOsActivity);

		ConcurrentBranchRule cbr = null;
		if (BomHelper.getInstance().isCBranch(outputSet)){
			cbr = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
			getChildRules().add(cbr);
			cbr.getSource().add(outputSet);
			cbr.transformSource2Target();
		}

		/* Set the output set <activity> as the source of any links assigned to it by other rules */		
		setSourceOfLinks(outputSet, fOsActivity);
		/* The first <activity>, <link>s and <assign>s have already been added to the target list...
		 * Remember that the output set <activity> must be the second <activity> in the target list
		 * if there are no input object pins in inputSet, and third <activity> otherwise... */
		getTarget().add(fOsActivity);
		getTarget().addAll(objectsToPropagate);
		if (cbr != null){
			getTarget().addAll(cbr.getTarget());
		}
		mapDescriptionAndDocumentation(inputSet.getAction(),fOsActivity);

		if (BpelOptionsUtil.generateEmptyActivityForControlNode()) {
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(),fOsActivity);
		}	
		
		/* Consider this rule complete, even though there are other input sets that may still have 
		 * yet to be processed... they will be handled by different rules  */
		setComplete(true);

		executeHandlers();

		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();												
	}

	/**
	 * Returns the primary activity created to represent the input pin set 
	 * @return The input pin set activity
	 */
	public Activity getInputSetActivity(){
	    return fIsActivity;
	}
	
	/**
	 * Returns the primary activity created to represent the output pin set
	 * @return The output pin set activity
	 */
	public Activity getOutputSetActivity(){
	    return fOsActivity;
	}
	
	/**
	 * Returns the primary activity created to represent the Merge
	 * @return The primary MergeRule activity
	 */
	public Activity getMergeActivity(){
	    return fAssign;
	}
	

	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected Invoke createMultiplicityCopyJavaInvoke(InputPinSet is, OutputPinSet os, boolean alwaysCreateNewPart){
		Invoke joinActivity = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaAssign = "";
		javaAssign += createJavaInvokeForWSICompliantDiamond(is, os);
		jsActivity.setJavaCode(javaAssign);
		joinActivity.getEExtensibilityElements().add(jsActivity);
		
		return joinActivity;
	}
	
	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected String createJavaInvokeForWSICompliantDiamond(InputPinSet is, OutputPinSet os){

		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		//this list records all variables that are found for the os pins and are already initialized by code so that 
		//we do not initialize twice the same variable in the snippet
		List initializedVariable = new LinkedList();
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			//we should really look here for the next InputObjectPin linked to this Join OutputObjectPin 
			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
			//** outPin **
			ObjectPin pinLinkedToOutPin = null;
			if(outPin.getOutgoing() != null && outPin.getOutgoing().getTarget() instanceof ObjectPin){
				pinLinkedToOutPin = (ObjectPin)outPin.getOutgoing().getTarget();
			}else if(outPin instanceof StoreArtifactPin){
				pinLinkedToOutPin = outPin;	
			}else{
				continue;
			}
			//now get the Set, the Variable for this set and the index of the Pin in the set..
			PinSet inPinSet = BomUtils.findPinSetForPin(pinLinkedToOutPin);
			//get the variable for this Set... **osVariable**
			BPELVariable pinSetVariable = getVariableForPin(inPinSet, pinLinkedToOutPin, true);
			//check if we have already initialized this variable
			if(!initializedVariable.contains(pinSetVariable)){
				javaAssign += AbstractbpelUtil.initializeVariable(pinSetVariable);
				initializedVariable.add(pinSetVariable);
			}
			int outPinIndex;
				outPinIndex =-1;
			XSDTypeDefinition pinTypeDefinition = pinSetVariable.getType();	
			String pinElementName = pinSetVariable.getName();
			
			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
			boolean useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(pinLinkedToOutPin, pinSetVariable);
		
			int outListInd=0;
			boolean lastIndexInitialized = false;
			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				BPELVariable inSetVariable = null;
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
				//get the real pin to which this links 
				//we should really look here for the next InputObject pin linked to this OutputObjectPin 
				//** outPin **
				ObjectPin outLinkedToInPin = null;
				if(inPin.getIncoming()!= null){
					outLinkedToInPin = (ObjectPin)inPin.getIncoming().getSource();
				}else if(inPin instanceof RetrieveArtifactPin){
					outLinkedToInPin = inPin;
				}else{
					//check if there has been a variable created for this inPin. This is the case of 
					//a Pin that is not connected -> invalid model that we try to address... 
					inSetVariable = getVariableForPin(is, inPin, false);
					if(inSetVariable != null){
						outLinkedToInPin = inPin;
					}else{
						continue;	
					}
					
				}
				
				//get the variable for this Set... **osVariable**
				if(inSetVariable == null){
					//now get the Set, the Variable for this set and the index of the Pin in the set..
					PinSet outPinSet = BomUtils.findPinSetForPin(outLinkedToInPin);
					inSetVariable = getVariableForPin(outPinSet, outLinkedToInPin, false);
				}
					
				
				if (inSetVariable == null){
					return "";
				}
				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
				boolean useListForInput;
				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outLinkedToInPin, inSetVariable);
				//we should only create the assign if the variables are not identicals, else, there is not need for assign
				if(!inSetVariable.equals(pinSetVariable))
				javaAssign += AbstractbpelUtil.createJavaCopySnippetsDiamond(inSetVariable, useListForInput,pinSetVariable,useListForOutput,lastIndexInitialized,"lastIndex"+outPinIndex,Integer.toString(outListInd),false)
				+ ";\n";
				if (useListForOutput) {
					if (!useListForInput) {
						outListInd++;
					} else {
						lastIndexInitialized=true;
					}
				}
			}
		}
		return javaAssign;
	}
		
//	/**
//	 * This method creates a BPEL <assign> to copy the data from the given input set
//	 * variable to the given output set variable
//	 * 
//	 * @param cIS The input set variable
//	 * @param cOS The output set variable
//	 * @return Assign - The BPEL Assign activity that performs the copy operation
//	 */
//	private Assign createMergeAssign(InputPinSet is, Variable cIS, OutputPinSet os, Variable cOS){
//		/* Create the assign that will copy from the input set variable to the output set variable.  
//		 * 
//		 * Note: We can't just use an AssignRule because we don't have 
//		 * an ActivityEdge to set as the source of the rule.
//		 * 
//		 * Note: We don't have to specify parts because the structure of the output set must be
//		 * the same as the structure of the input set in a Merge
//		 */
//		Assign assign = BPELFactory.eINSTANCE.createAssign();
//		if(!BpelOptimizationUtil.bOptimizeMergeTransformation){
//			Copy copy = BPELFactory.eINSTANCE.createCopy();
//			From from = BPELFactory.eINSTANCE.createFrom();
//			To to = BPELFactory.eINSTANCE.createTo();
//	
//			from.setVariable(cIS);
//			to.setVariable(cOS);
//	
//			copy.setFrom(from);
//			copy.setTo(to);
//			assign.getCopy().add(copy);
//		// we need to create assigne from the isVariable to each pin of the Action the Merge connects to
//		}else{
//			//retrieve the OutputPinSet to which the Merge InputPinSet connects to... as this is the case of an 
//			//Assign, assume that is one OuptputPinSet...
//			List pins = is.getInputObjectPin();
//			if(pins!= null && !pins.isEmpty()){
//				InputObjectPin inPin = (InputObjectPin)pins.get(0);
//				OutputObjectPin outPin = (OutputObjectPin)inPin.getIncoming().getSource();
//				//retrieve the pinSet that links to the Merge inputSet
//				OutputPinSet pinSet = (OutputPinSet)BomUtils.findPinSetForPin(outPin);
//				//we must find that.. this pinSet was already processed
//				ContainerRule cr = (ContainerRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
//													ContainerRule.class,
//													pinSet); 
//				Variable fromVariable = (Variable)cr.getTarget().get(0);
//				
//				//all the pins are guaranteed to come from the same PinSet.... 
//				List mergeInputPins = is.getInputObjectPin();
//				List outputPins = pinSet.getOutputObjectPin();
//				
//				for (int i=0; i< mergeInputPins.size(); i++){
//					Copy copy = BPELFactory.eINSTANCE.createCopy();
//					From from = BPELFactory.eINSTANCE.createFrom();
//					copy.setFrom(from);
//					To to = BPELFactory.eINSTANCE.createTo();
//					copy.setTo(to);
//					
//					outPin = (OutputObjectPin)outputPins.get(i);
//					Part outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, pinSet)).getTarget().get(0); 
//					String queryFrom = createQueryString(pinSet, outPin, outPinPart);
//					from.setVariable(cIS);
//					from.setPart(outPinPart);
//
//					if(queryFrom != null && !queryFrom.equals("")){
//						Query bpelQueryFrom = BPELFactory.eINSTANCE.createQuery();
//						bpelQueryFrom.setQueryLanguage("");
//						bpelQueryFrom.setValue(queryFrom);
//						from.setQuery(bpelQueryFrom);
//					}
//					//we have to retrieve now the outputVariable linking to the corresponding outputSet Merge
//					OutputObjectPin outPinInMerge = (OutputObjectPin)os.getOutputObjectPin().get(i);
//					if(outPinInMerge.getOutgoing() != null){
//						//InputObjectPin pinLinkedFromMerge = (InputObjectPin)outPinInMerge.getOutgoing().getTarget();
//						Pin pinLinkedFromMerge = (Pin)outPinInMerge.getOutgoing().getTarget();
//						
//						PinSet pinSetLinkedFromMerge = BomUtils.findPinSetForPin(pinLinkedFromMerge);
//						Variable toVariable = getVariableForPinSet(pinSetLinkedFromMerge);
//						Part inPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, pinSetLinkedFromMerge)).getTarget().get(0); 
//						String queryTo = createQueryString(pinSetLinkedFromMerge, pinLinkedFromMerge, inPinPart);
//						to.setVariable(toVariable);
//						to.setPart(inPinPart);
//						if(queryTo != null && !queryTo.equals("")){
//							Query bpelQueryTo = BPELFactory.eINSTANCE.createQuery();
//							bpelQueryTo.setQueryLanguage("");
//							bpelQueryTo.setValue(queryFrom);
//							to.setQuery(bpelQueryTo);
//						}
//						assign.getCopy().add(copy);
//
//						
//					}
//				}
//			}
//		}
//		return assign;
//	}		
		
	/**
	 * Update the joinCondition for the merge output set.  This is a special case of a join condition
	 * where we have one link per input set, but that link doesn't point to a particular pin in the 
	 * output set... just to the output set as a whole.
	 * 
	 * @param set - Merge OutputPinSet
	 * @param target - BPEL activity that owns the join condition 
	 */
	protected void updateMergeOutputPinSetJoinCondition(OutputPinSet outputSet, Activity target){
			
		Collection linkRules = null;
		List pinSetLinkRules = new ArrayList();

		//get the list of link rules that are associated with this output set
		linkRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), outputSet, Link.class); 
		Link link = null;
		TransformationRule tRule = null;

		for (Iterator lrIterator = linkRules.iterator(); lrIterator.hasNext(); ){
			tRule = (TransformationRule)lrIterator.next();
			
			/* Only deal with link rules. */
			if (tRule instanceof LinkRule){
				link = (Link)tRule.getTarget().get(0);

				/* Because we asked for all the LinkRules with the output set as a source, we will get back those
				 * that are outgoing from the output set as well.  We only want to deal with those links that
				 * have the given empty as their *target*, not their *source*.			 */
				if (!link.getTargets().isEmpty() && ((Target)link.getTargets().get(0)).getActivity() == target){
					pinSetLinkRules.add(tRule);
				}
			}
		}

		// Ray: if the pinSetLinkRules.size()==1, that means it is a single branched merge.  So no need to
		// create a join condition
		if (pinSetLinkRules.size()==1) {
			return;
		}
		// Remove any existing join condition.
		for (int i=target.getExtensibilityElements().size()-1; i>=0; i--){
			if (target.getExtensibilityElements().get(i) instanceof JoinExtensibilityExpression){
				target.getExtensibilityElements().remove(i);
			}
		}

		/* Create Expression Model for the JoinCondition */
		JoinExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createJoinExtensibilityExpression(); 
		Expression joinExpr = createLogicalOr(pinSetLinkRules);
		bpelExpr.setExpression(joinExpr);			

		target.addExtensibilityElement(bpelExpr);		
		
		
	}

	/**
	 * Creates the BPEL Assign activity to copy each multiplicity item from the variable associated with
	 * the input pin set to the variable associated with the output pin set. (this is for Join and Merge)
	 * 
	 * @param is The InputPinSet that is providing the information to copy
	 * @param isVariable The BpEL variable that actually holds the source information
	 * @param os The OutputPinSet that the information is being copied to
	 * @param osVariable The BPEL variable that will hold the information once it is copied 
	 * @return Assign - The BPEL Assign activitiy with potentially many Copy elements
	 */
	protected Assign createMultiplicityCopyAssign(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable){
		
		Assign joinActivity = BPELFactory.eINSTANCE.createAssign();
		
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			Part outPinPart = (Part)((PartRule) 
				TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),
													PartRule.class,
													(Pin)os.getOutputObjectPin().get(i))).getTarget().get(0);

			int outPosition = 1;				

			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
				Part inPinPart = (Part)((PartRule) 
					TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),
														PartRule.class,
														inPin)).getTarget().get(0);

				/* NOTE: It is a restriction that we are requiring the upper bound = lower bound
				 * for the input object pin multiplicities. If there is no multiplicity defined, assume 1..1 */
//				if (inPin.getLowerBound() instanceof LiteralInteger && inPin.getUpperBound() instanceof LiteralUnlimitedNatural 
//						&& ((LiteralInteger)inPin.getLowerBound()).getValue().intValue() != Integer.parseInt(((LiteralUnlimitedNatural)inPin.getUpperBound()).getValue())){
//					
//					LoggingUtil.logWarning(MessageKeys.JOIN_MULTIPLICITY_NOT_SUPPORTED);		
//				}		

				 
				int count;
				if (inPin.getLowerBound() != null){
					/* Use the lower-bound to be on safe side. The LB and UB are supposed to be the same anyways. */
					count = ((LiteralInteger)inPin.getLowerBound()).getValue().intValue();
				} else {
					count = 1;
				}

				for (int k=0; k<count; k++){
					Copy copy = BPELFactory.eINSTANCE.createCopy();
					From from = BPELFactory.eINSTANCE.createFrom();
				
					from.setVariable(isVariable);
					from.setPart(inPinPart);
								   
					ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
					
					bpelExpr.setExpression(createCopyExpression(inPinPart, k+1));
					from.addExtensibilityElement(bpelExpr);
					
					/* Create an expression model instance to store the queries for From*/
					ExtensibilityExpression bpelExpr2 = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();					
					bpelExpr2.setExpression(createCopyExpression(outPinPart, outPosition));
					
					/* Create a ToExtension class */
					//ToExtension toExt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createToExtension();
					To toExt = BPELFactory.eINSTANCE.createTo();
					toExt.setVariable(osVariable);
					toExt.setPart(outPinPart);
					toExt.addExtensibilityElement(bpelExpr2); 	
								
					copy.setFrom(from);
					copy.setTo(toExt);
					((Assign)joinActivity).getCopy().add(copy);
					 
					outPosition++;
				}

			}						 
		}
		
		return joinActivity;
		
	}

	/** 
	 * Create the expression model representation of the BPEL assignment expression
	 * 
	 */	
	private FunctionExpression createCopyExpression(Part pinPart, int k)
	{
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();					
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID(CoreFunctions.SELECT_FUNCTION);

		/* First Argument (elements) for the select Function Expression is a ModelPathExpression*/
		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		ModelPathExpression pathExpr = ModelFactory.eINSTANCE.createModelPathExpression();
		StaticStep step1 = ModelFactory.eINSTANCE.createStaticStep();
		step1.setStepName(pinPart.getName());
		StaticStep step2 = ModelFactory.eINSTANCE.createStaticStep();
		step2.setStepName(pinPart.getTypeName().getLocalPart());
		pathExpr.setIsAbsolute(Boolean.TRUE);
		pathExpr.getSteps().add(step1);
		pathExpr.getSteps().add(step2);					
		arg1.setArgumentValue(pathExpr);

		/* Second argument (selection criteria) for the select Function Expression is a ComparisonExpression */
		FunctionArgument arg2 = ModelFactory.eINSTANCE.createFunctionArgument();
		ComparisonExpression cmpExpr = ModelFactory.eINSTANCE.createComparisonExpression();	
					
		/* first operand of ComparisonExpression is position() Function Expresssion with no arguments */				
		FunctionExpression posFunctionExpr = ModelFactory.eINSTANCE.createFunctionExpression();					
		FunctionDefinition def2 = ModelFactory.eINSTANCE.createFunctionDefinition();
		def2.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.position");
		def2.setFunctionName("position");
		posFunctionExpr.setDefinition(def2);

		cmpExpr.setFirstOperand(posFunctionExpr);		

		cmpExpr.setOperator(ComparisonOperator.EQUAL_TO_LITERAL);

		/* Second operand is an IntegerLiteralExpression */
		IntegerLiteralExpression intExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
		intExpr.setIntegerSymbol(new Integer(k));
		cmpExpr.setSecondOperand(intExpr); 				

		arg2.setArgumentValue(cmpExpr);					
		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		functionExpr.getArguments().add(arg2);
		
		return functionExpr;
	}
		
//	/**
//	 * Creates the Java version of the assignment to copy the input pin information to the 
//	 * corresponding output pin variable (this is for Join and Merge)
//	 */
//	protected Invoke createMultiplicityCopyJavaInvoke(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){
//		boolean bUseSinglePartWSDL = BpelOptionsUtil.isTargetWSICompliant(this.getContext());
//		Invoke joinActivity = AbstractbpelUtil.createJavaInvoke();
//		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
//		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
//		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
//		String javaAssign = "";
//		//Since we will only generate single element part wsdl for the Process, and implicitly for all the ControlActions, retrieve the Part 
//		//using as source the PinSet.
//		Part outPinPart = null;
//		Part inPinPart = null; 
//		if(bUseSinglePartWSDL){
//			outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, os)).getTarget().get(0);
//		}
//		
//		
//		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
//		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
//			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
//
//			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
//			boolean useListForOutput;
//			if(!bUseSinglePartWSDL){
//				outPinPart = (Part)((PartRule) 
//					TransformationUtil.getRuleForSource(getRoot(),
//														PartRule.class,
//														outPin)).getTarget().get(0);
//			}
//				
//			// Ray
//			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, i);
//			if (useListForOutput && alwaysCreateNewPart) {
//				// Ray - the output variable of Join hasn't been set prior to this and WSADIE code-gen will have a problem with it
//				// The solution is to create a new part object instance everytime and then set the contents.  This also will optimize the code on not
//				// needing to do 'get' on the part as this is costly
//				// 1. Get type of part
////					String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(outPinPart.getTypeName());
////				// Create partClass instance
////					javaAssign += fullPartTypeName + " " + outPinPart.getName() + " = new " + fullPartTypeName + "();\n";
//					if(!bUseSinglePartWSDL){
//						javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart,false);
//					}else{
//						//we need to pass in the index of the pin in order to retrieve the Pin information from the PinPart
//						javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart,false, i);	
//					}
//				
//			}
//			int outListInd=0;
//			boolean lastIndexInitialized=false;
//			String outVarName = AbstractbpelUtil.capitalizeFirstChar(osVariable.getName());
//			String outPartName = AbstractbpelUtil.capitalizeFirstChar(outPinPart.getName());
//			//Since we will only generate single element part wsdl for the Process, and implicitly for all the ControlActions, retrieve the Part 
//			//using as source the PinSet. 
//			if(bUseSinglePartWSDL){
//				inPinPart = (Part)((PartRule)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, is)).getTarget().get(0);
//			}
//			
//
//			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
//				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
//				//overwrite the behaviour for old multi part wsdl
//				if(!bUseSinglePartWSDL){
//					 inPinPart = (Part)((PartRule) 
//					TransformationUtil.getRuleForSource(getRoot(),
//														PartRule.class,
//														inPin)).getTarget().get(0);
//				}
//
//				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
//				boolean useListForInput;
//				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart, j);
//				if(!bUseSinglePartWSDL){
//					javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false);
//				}else{
//					javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i);
//				}
//				
//				if (useListForOutput) {
//					if (!useListForInput) {
//						outListInd++;
//					} else {
//						lastIndexInitialized=true;
//					}
//				}
//
//			}
//			if (useListForOutput && alwaysCreateNewPart) {
////				javaAssign +=
////				AbstractbpelUtil.generateJavaGetterForVariable(outVarName)+ 
////				"(true).set"+outPartName+"("+outPinPart.getName()+");\n";
//				//TODO AR changed
//				if(!bUseSinglePartWSDL){
//					javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart,outPinPart.getName());
//				}else{
//					String pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, i);
//					javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart, true);
//					javaAssign += outPinPart.getName() + ".set" + pinElementName +"(" + pinElementName +");\n";
//					javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart, outPinPart.getName());
////					String pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, i);
////					//javaAssign += AbstractbpelUtil.createJavaElementSetterCode(osVariable,outPinPart,outPinPart.getName(), i);
////					javaAssign += AbstractbpelUtil.createJavaElementSetterCode(osVariable,outPinPart,pinElementName, i);
//				}
//				
//			}
//		}
//		
//		javaCode.setValue(javaAssign);
//		jsActivity.setJavaCode(javaCode);
//		joinActivity.getEExtensibilityElements().add(jsActivity);
//		
//		return joinActivity;
//	}


	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected Invoke createMultiplicityCopyJavaInvoke(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){
		boolean bUseSinglePartWSDL = BpelOptionsUtil.isTargetWSICompliant();
		Invoke joinActivity = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		
		if(bUseSinglePartWSDL){
			if(!BpelOptionsUtil.isTargetDiamondCompatible()){
				javaAssign += createJavaInvokeForWSICompliant(is, isVariable, os, osVariable, alwaysCreateNewPart);
			}else{
				if(BpelOptimizationUtil.bOptimizeMergeTransformation){
					//javaAssign += createJavaInvokeForWSICompliantDiamond(is, isVariable, os, osVariable, alwaysCreateNewPart);
					javaAssign += createJavaInvokeForWSICompliantDiamond(is, os);
				}else{
					javaAssign += createJavaInvokeForWSICompliantDiamond(is, isVariable, os, osVariable, alwaysCreateNewPart);
				}
			}
		}else{
			javaAssign += createJavaInvokeForNonWSICompliant(is, isVariable, os, osVariable, alwaysCreateNewPart);
		}
		
//		javaCode.setValue(javaAssign);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaAssign);
		joinActivity.getEExtensibilityElements().add(jsActivity);
		
		return joinActivity;
	}

	




	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected String createJavaInvokeForNonWSICompliant(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){
		boolean bUseSinglePartWSDL = BpelOptionsUtil.isTargetWSICompliant();
		Invoke joinActivity = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		//Since we will only generate single element part wsdl for the Process, and implicitly for all the ControlActions, retrieve the Part 
		//using as source the PinSet.
		Part outPinPart = null;
		Part inPinPart = null; 
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);

			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
			boolean useListForOutput;
				outPinPart = (Part)((PartRule)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, outPin)).getTarget().get(0);
				
			// Ray
			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, i);
			if (useListForOutput && alwaysCreateNewPart) {
				// Ray - the output variable of Join hasn't been set prior to this and WSADIE code-gen will have a problem with it
				// The solution is to create a new part object instance everytime and then set the contents.  This also will optimize the code on not
				// needing to do 'get' on the part as this is costly
				// 1. Get type of part
//					String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(outPinPart.getTypeName());
//				// Create partClass instance
//					javaAssign += fullPartTypeName + " " + outPinPart.getName() + " = new " + fullPartTypeName + "();\n";
					javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart,false);
			}
			int outListInd=0;
			boolean lastIndexInitialized=false;
			String outVarName = AbstractbpelUtil.capitalizeFirstChar(osVariable.getName());
			String outPartName = AbstractbpelUtil.capitalizeFirstChar(outPinPart.getName());
			//Since we will only generate single element part wsdl for the Process, and implicitly for all the ControlActions, retrieve the Part 
			//using as source the PinSet. 

			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
					 inPinPart = (Part)((PartRule) 
					TransformationUtil.getRuleForSource(getRoot(),
														PartRule.class,
														inPin)).getTarget().get(0);

				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
				boolean useListForInput;
				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart, j);

					javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false);
				
				if (useListForOutput) {
					if (!useListForInput) {
						outListInd++;
					} else {
						lastIndexInitialized=true;
					}
				}

			}
			if (useListForOutput && alwaysCreateNewPart) {
//				javaAssign +=
//				AbstractbpelUtil.generateJavaGetterForVariable(outVarName)+ 
//				"(true).set"+outPartName+"("+outPinPart.getName()+");\n";
				//TODO AR changed

					javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart,outPinPart.getName());
				
			}
		}
		
		return javaAssign;
	}

	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected String createJavaInvokeForWSICompliant(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){
		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		Part outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, os)).getTarget().get(0);
		boolean partCreated = false;
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
			boolean useListForOutput;
			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, i);
			if (useListForOutput && alwaysCreateNewPart) {
				javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart,false, i);	
			}
			int outListInd=0;
			boolean lastIndexInitialized=false;
			String outVarName = AbstractbpelUtil.capitalizeFirstChar(osVariable.getName());
			String outPartName = AbstractbpelUtil.capitalizeFirstChar(outPinPart.getName());
			Part inPinPart = (Part)((PartRule)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, is)).getTarget().get(0);
			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
				boolean useListForInput;
				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart, j);
				//javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i);
				javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i, partCreated);
				partCreated = true;
				if (useListForOutput) {
					if (!useListForInput) {
						outListInd++;
					} else {
						lastIndexInitialized=true;
					}
				}
			}
			if (useListForOutput && alwaysCreateNewPart) {
				  String pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, i);
				  if(!partCreated){
					javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart, true);
					partCreated = true;
				  }
				  javaAssign += outPinPart.getName() + ".set" + pinElementName +"(" + pinElementName +");\n";
				//  javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart, outPinPart.getName());
			}
			
		}
		javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart, outPinPart.getName());
		return javaAssign;
	}
	
	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected String createJavaInvokeForWSICompliantDiamond(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){

		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		Part outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, os)).getTarget().get(0);
//		initialize the outputVariable if not already initialized
		javaAssign += AbstractbpelUtil.initializeVariable(osVariable, outPinPart, true);
		boolean partCreated = false;
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
			int outPinIndex;
			if(outPinCount == 1 && !BpelOptionsUtil.isAlwaysGenerateWraperComplexType()){
				outPinIndex =-1;
				partCreated = true;
			}else{
				outPinIndex = i;
			}
//			XSDTypeDefinition pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(outPinPart, i);
//			String pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, i);
			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
//			boolean useListForOutput;
//			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, i);
//			if (useListForOutput && alwaysCreateNewPart) {
//				//javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart, i);
//				javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart,false, i);
//				//javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(osVariable,outPinPart,false, i);	
//			}
			
			XSDTypeDefinition pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(outPinPart, outPinIndex);
			String pinElementName = null;
			if(outPinIndex != -1){
				pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, outPinIndex);	
			}else{
				pinElementName = osVariable.getName();
			}
			
			
			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
			boolean useListForOutput;
			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, outPinIndex);
			if (useListForOutput && alwaysCreateNewPart) {
				//javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart, i);
				if(outPinIndex != -1)
					javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(osVariable, outPinPart, false, outPinIndex);
				javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart,true, outPinIndex);
				partCreated = true;
				//javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(osVariable,outPinPart,false, i);	
			}
			int outListInd=0;
			boolean lastIndexInitialized=false;
			String outVarName = AbstractbpelUtil.capitalizeFirstChar(osVariable.getName());
			String outPartName = AbstractbpelUtil.capitalizeFirstChar(outPinPart.getName());
			Part inPinPart = (Part)((PartRule)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, is)).getTarget().get(0);
			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
				int inPinIndex;
				if(inPinCount == 1 && !BpelOptionsUtil.isAlwaysGenerateWraperComplexType()){
					inPinIndex =-1;
				}else{
					inPinIndex = j;
				}
				boolean useListForInput;
//				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart, j);
//				javaAssign += AbstractbpelUtil.createJavaCopySnippetsDiamond(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i, partCreated)
//				+ ";\n";

				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart, inPinIndex);
				javaAssign += AbstractbpelUtil.createJavaCopySnippetsDiamond(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+outPinIndex,Integer.toString(outListInd),false, inPinIndex, outPinIndex, partCreated)
				+ ";\n";
				//javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i);
				//javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i, partCreated);
				partCreated = true;
				if (useListForOutput) {
					if (!useListForInput) {
						outListInd++;
					} else {
						lastIndexInitialized=true;
					}
				}
			}
			if (useListForOutput && alwaysCreateNewPart) {
				 // String pinElementName = AbstractbpelUtil.getPinElementName(outPinPart, i);
				  if(!partCreated){
//				  	javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart, false, i)+ ";\n";
				  	javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(osVariable,outPinPart, false, outPinIndex)+ ";\n";
				  	
					//javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart, true);
				  	//javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(osVariable, outPinPart, true, i) + ";\n";
					partCreated = true;
				  }
				  //javaAssign += outPinPart.getName() + ".set" + pinElementName +"(" + pinElementName +");\n";
				//  javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart, outPinPart.getName());
			}
			String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)pinTypeDefinition);
			javaAssign += AbstractbpelUtil.generateJavaSetterForVariableDiamond(pinElementName, listElement, pinTypeDefinition, listElement) + ";\n";
			
		}
		//javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart, outPinPart.getName());
		return javaAssign;
	}
	

	/**
	 * Creates the Java version of the assignment to copy the input pin information to the 
	 * corresponding output pin variable (this is for Join and Merge)
	 */
	protected String createJavaInvokeForWSICompliantDiamond_bak(InputPinSet is, OutputPinSet os){

		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
		String javaAssign = "";
		boolean partCreated = true;
		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
		//this list records all variables that are found for the os pins and are already initialized by code so that 
		//we do not initialize twice the same variable in the snippet
		List initializedVariable = new LinkedList();
		
		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
			//we should really look here for the next InputObjectPin linked to this Join OutputObjectPin 
			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
			//if there is no outgoing link from this pin so break
			if(outPin.getOutgoing() == null)
				break;
			//InputObjectPin pinLinkedToOutPin = (InputObjectPin)outPin.getOutgoing().getTarget();
			ObjectPin pinLinkedToOutPin = (ObjectPin)outPin.getOutgoing().getTarget();
			//now get the Set, the Variable for this set and the index of the Pin in the set..
			PinSet inPinSet = BomUtils.findPinSetForPin(pinLinkedToOutPin);
			//get the variable for this Set... **osVariable**
			Variable pinSetVariable = getVariableForPinSet(inPinSet);

			Part inPinPart = (Part) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, inPinSet).getTarget().get(0);
			//check if we have already initialized this variable
			if(!initializedVariable.contains(pinSetVariable)){
				javaAssign += AbstractbpelUtil.initializeVariable(pinSetVariable, inPinPart, true);
				initializedVariable.add(pinSetVariable);
			}
			int outPinIndex;
			if(BomUtils.isSinglePinInSet(inPinSet)  && !BpelOptionsUtil.isAlwaysGenerateWraperComplexType()){
				outPinIndex =-1;
				partCreated = true;
			}else{
				//outPinIndex = i;
				//outPinIndex = BomUtils.getPinsInSet(inPinSet).indexOf(pinLinkedToOutPin);
				outPinIndex = BomUtils.getObjectPinsForSet(inPinSet).indexOf(pinLinkedToOutPin);
			}
			XSDTypeDefinition  pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(inPinPart, outPinIndex);
			String pinElementName = null;
			if(outPinIndex != -1){
				pinElementName = AbstractbpelUtil.getPinElementName(inPinPart, outPinIndex);	
			}else{
				pinElementName = pinSetVariable.getName();
			}
			
			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
			boolean useListForOutput;
			
			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(pinLinkedToOutPin,inPinPart, outPinIndex);
			if (useListForOutput) {
				if(outPinIndex != -1)
					javaAssign += AbstractbpelUtil.createJavaVariableForPartCodeDiamond(pinSetVariable, inPinPart, false, outPinIndex);
					javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(pinSetVariable,inPinPart,true, outPinIndex);
					partCreated = true;
			}
			
			int outListInd=0;
			boolean lastIndexInitialized = false;
			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
				
				//get the real pin to which this links 
				//we should really look here for the next InputObject pin linked to this OutputObjectPin 
				if(inPin.getIncoming() == null)
					break;
				OutputObjectPin outLinkedToInPin = (OutputObjectPin)inPin.getIncoming().getSource();
				//now get the Set, the Variable for this set and the index of the Pin in the set..
				PinSet outPinSet = BomUtils.findPinSetForPin(outLinkedToInPin);
				//get the variable for this Set... **osVariable**
				Variable inSetVariable = getVariableForPinSet(outPinSet);
				
				Part outPinPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, outPinSet).getTarget().get(0);
				
				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
				int inPinIndex;
				//if(BomUtils.getPinsInSet(outPinSet).size()==1  && !BpelOptionsUtil.isAlwaysGenerateWraperComplexType()){
				if(BomUtils.isSinglePinInSet(outPinSet)  && !BpelOptionsUtil.isAlwaysGenerateWraperComplexType()){
					inPinIndex =-1;
				}else{
					//inPinIndex1 = j;
					inPinIndex = BomUtils.getObjectPinsForSet(outPinSet).indexOf(outLinkedToInPin);
				}
				boolean useListForInput;
				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outLinkedToInPin,outPinPart, inPinIndex);
				javaAssign += AbstractbpelUtil.createJavaCopySnippetsDiamond(inSetVariable,outPinPart,useListForInput,pinSetVariable,inPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+outPinIndex,Integer.toString(outListInd),false, inPinIndex, outPinIndex, partCreated)
				+ ";\n";
				partCreated = true;
				if (useListForOutput) {
					if (!useListForInput) {
						outListInd++;
					} else {
						lastIndexInitialized=true;
					}
				}
			}
			if (useListForOutput) {
				  if(!partCreated){
				  	javaAssign += AbstractbpelUtil.initializeJavaListPropertyDiamond(pinSetVariable,inPinPart, false, outPinIndex)+ ";\n";
					partCreated = true;
				  }
				  String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)pinTypeDefinition);
				javaAssign += AbstractbpelUtil.generateJavaSetterForVariableDiamond(pinElementName, listElement, pinTypeDefinition, listElement) + ";\n";
			}
//			String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)pinTypeDefinition);
//			javaAssign += AbstractbpelUtil.generateJavaSetterForVariableDiamond(pinElementName, listElement, pinTypeDefinition, listElement) + ";\n";
		}
		return javaAssign;
	}
	
	/**
	 * Modify all the Assigns that copy into the MergeOutputSetVariable so that the Assign will copy To the Variable (Part) of 
	 * the Pin of the next task that is linked to the Merge.  
	 * @param inputSet the Merge inputSet
	 * @param outputSet the Merge outputSet
	 */
	private void transformMergeAssign(InputPinSet inputSet, OutputPinSet outputSet){
		
		Collection assignRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()), AssignRule.class, (EObject)inputSet, Assign.class);
		if(assignRules != null && !assignRules.isEmpty()){
			Iterator it = assignRules.iterator();
			while(it.hasNext()){
				AssignRule assignRule = (AssignRule)it.next();
				//check is this assignRule has as target the inputSet of the 
				if(assignRule.getSource().get(2) == inputSet){
					//this is the rule we are interested in...
					Assign thisAssign = getTargetAssign(assignRule.getTarget());
					if(thisAssign.getCopy()!= null && !thisAssign.getCopy().isEmpty()){
						Copy copy = (Copy)thisAssign.getCopy().get(0);
						//we need to change the To from the MergeOutputSet criteria to the corresponding
						//OutputSet on the next Merge Task
						ActivityEdge activityEdge = (ActivityEdge)assignRule.getSource().get(0);
						InputObjectPin inMergePin = (InputObjectPin)activityEdge.getTarget();
						//find the index to retrieve the OutputOjectPin on the Merge... 
						InputPinSet mergeSet = (InputPinSet)BomUtils.findPinSetForPin(inMergePin);
						//get the index of the Pin in the Merge InputPinSet
						int inputPinIndex = BomUtils.getObjectPinsForSet(mergeSet).indexOf(inMergePin);
						//retrieve the Pin in the Merge OutputPinSet that corresponds to the Pin in the InputPinSet 
						OutputObjectPin outMergePin = (OutputObjectPin)outputSet.getOutputObjectPin().get(inputPinIndex);
						//find the link to where it goes.. 
						if(outMergePin.getOutgoing() == null){
							//we need to register this assign so that it will be cleaned in the end
							BpelOptimizationUtil.registerOptimizableElement(getContext(), thisAssign);
							break;
						}
						//this is the Pin to which the Merge Pin links to
						ObjectPin mergedInputPin = (ObjectPin)outMergePin.getOutgoing().getTarget();
						//the PinSet to which it belongs
						PinSet linkedPinSet = BomUtils.findPinSetForPin(mergedInputPin);
						//retrive the Part for this OutputPin
						Part outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, linkedPinSet)).getTarget().get(0); 
						//get the query for assign
						String queryTo = createQueryString(linkedPinSet, mergedInputPin, outPinPart);
						//get the variable created for this PinSet
						Variable toVariable = getVariableForPinSet(linkedPinSet);
						//change the To of the Copy
						copy.getTo().setVariable(toVariable);
						copy.getTo().setPart(outPinPart);
						if(queryTo != null && !queryTo.equals("")){
							Query bpelQueryTo = BPELFactory.eINSTANCE.createQuery();
							bpelQueryTo.setQueryLanguage("");
							bpelQueryTo.setValue(queryTo);
							copy.getTo().setQuery(bpelQueryTo);
						}
					}
				}
				
			}
		}
	}
	

//	/**
//	 * Creates the Java version of the assignment to copy the input pin information to the 
//	 * corresponding output pin variable (this is for Join and Merge)
//	 */
//	protected Invoke createMultiplicityCopyJavaInvoke(InputPinSet is, Variable isVariable, OutputPinSet os, Variable osVariable, boolean alwaysCreateNewPart){
//		Invoke joinActivity = AbstractbpelUtil.createJavaInvoke();
//		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
//		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
//		boolean useSinglePartWSDL = !ProcessUtil.isMultiplePartWSDL(is.getAction());		
//		/* The output container is represented as a list, accessible via 'get[VariableName]()' */
//		String javaAssign = "";
//		//TODO AR the part is always created for the element part wsdl of the process
//		Part outPinPart = (Part)((PartRule) TransformationUtil.getRuleForSource(getRoot(), PartRule.class, os)).getTarget().get(0);
//		/* For each pin in the OutputSet, there is one or more corresponding pins in the input set */
//		for (int i=0, outPinCount = os.getOutputObjectPin().size(); i<outPinCount; i++){
//			OutputObjectPin outPin = (OutputObjectPin)os.getOutputObjectPin().get(i);
//			String outPinName = null;
//			if(useSinglePartWSDL){
//				outPinName = AbstractbpelUtil.getPinElementName(os, outPin, outPinPart);
//			}
//			
//
//			/* Get the multiplicity of outPin.  This will affect whether use Java Lists or not */
//			boolean useListForOutput;
////			Part outPinPart = (Part)((PartRule) 
////				TransformationUtil.getRuleForSource(getRoot(),
////													PartRule.class,
////													outPin)).getTarget().get(0);
//			
//			// Ray
//			useListForOutput = AbstractbpelUtil.isTransformedMultiplicityElementAList(outPin,outPinPart, i);
//			if (useListForOutput && alwaysCreateNewPart) {
//				// Ray - the output variable of Join hasn't been set prior to this and WSADIE code-gen will have a problem with it
//				// The solution is to create a new part object instance everytime and then set the contents.  This also will optimize the code on not
//				// needing to do 'get' on the part as this is costly
//				// 1. Get type of part
////				String fullPartTypeName = AbstractbpelUtil.getFullJavaClassName(outPinPart.getTypeName());
////				// Create partClass instance
////				javaAssign += fullPartTypeName + " " + outPinPart.getName() + " = new " + fullPartTypeName + "();\n";
//				javaAssign += AbstractbpelUtil.createJavaVariableForPartCode(osVariable,outPinPart,false, i);				
//			}
//			int outListInd=0;
//			boolean lastIndexInitialized=false;
//			String outVarName = AbstractbpelUtil.capitalizeFirstChar(osVariable.getName());
//			String outPartName = AbstractbpelUtil.capitalizeFirstChar(outPinPart.getName());
//			//TODO AR the Part is created always for the element part wsdl created for this process..
//			Part inPinPart = (Part)((PartRule)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, is)).getTarget().get(0);
//
//			for (int j=i, inPinCount = is.getInputObjectPin().size(); j<inPinCount; j+=outPinCount){
//				InputObjectPin inPin = (InputObjectPin)is.getInputObjectPin().get(j);
//
////				Part inPinPart = (Part)((PartRule) 
////					TransformationUtil.getRuleForSource(getRoot(),
////														PartRule.class,
////														inPin)).getTarget().get(0);
//
//				/* Get the multiplicity of inPin.  This will affect whether use Java Lists or not */
//				boolean useListForInput;
//				useListForInput = AbstractbpelUtil.isTransformedMultiplicityElementAList(inPin,inPinPart);
//
//				javaAssign += AbstractbpelUtil.createJavaCopySnippets(isVariable,inPinPart,useListForInput,osVariable,outPinPart,useListForOutput,lastIndexInitialized,"lastIndex"+i,Integer.toString(outListInd),false, j, i);
//				if (useListForOutput) {
//					if (!useListForInput) {
//						outListInd++;
//					} else {
//						lastIndexInitialized=true;
//					}
//				}
//				/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */
//				/*
//				String inVarName = AbstractbpelUtil.capitalizeFirstChar(isVariable.getName());
//				String inPartName = AbstractbpelUtil.capitalizeFirstChar(inPinPart.getName());
//				if (useListForOutput){
//					// Ray
//					if (!useListForInput) {
//						javaAssign += outPinPart.getName()
//								   + ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)outPinPart.getType()))+"("+(outListInd++)+","
//								   +  AbstractbpelUtil.generateJavaGetterForVariable(inVarName) + "()"
//								   + ".get" + inPartName + "()"
//								   + ");\n";
//					} else {
//						 StringBuffer sb = new StringBuffer();
//						 if (!lastIndexInitialized) {
//							sb.append("int lastIndex"+i+"=");
//							sb.append(outListInd);
//							sb.append(";\n");
//							lastIndexInitialized=true;
//						 }
//						 String inListPartGetterName = AbstractbpelUtil.generateJavaGetterForVariable(inVarName)+"()"
//								+ ".get" + inPartName + "()" 
//								+ ".get" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)inPinPart.getType()));
//						 String outListPartSetterName = outPinPart.getName()
//							   + ".set" + AbstractbpelUtil.capitalizeFirstChar(getNameOfListElement((XSDComplexTypeDefinition)outPinPart.getType()));
//						 sb.append("for (int i=0; i<");
//						 sb.append(inListPartGetterName);
//						 sb.append("().length; i++) {\n\t");
//						 sb.append(outListPartSetterName);
//						 sb.append("(lastIndex"+i+"++,");
//						 sb.append(inListPartGetterName);
//						 sb.append("(i));\n}\n");
//						 javaAssign += sb.toString();
//					}
//				} else if (!useListForOutput){
//					javaAssign += AbstractbpelUtil.generateJavaGetterForVariable(outVarName) + "(true)"
//							   + ".set"	+ outPartName + "("
//										   +  AbstractbpelUtil.generateJavaGetterForVariable(inVarName) + "()"
//										   + ".get" + inPartName + "()"
//							   + ");\n";					
//				}
//				*/
//			}
//			if (useListForOutput && alwaysCreateNewPart) {
////				javaAssign +=
////				AbstractbpelUtil.generateJavaGetterForVariable(outVarName)+ 
////				"(true).set"+outPartName+"("+outPinPart.getName()+");\n";
////TODO AR changed
//				//javaAssign += AbstractbpelUtil.createJavaPartSetterCode(osVariable,outPinPart,outPinPart.getName());
//				javaAssign += AbstractbpelUtil.createJavaElementSetterCode(osVariable,outPinPart,outPinPart.getName(), i);
//			}
//		}
//		
//		javaCode.setValue(javaAssign);
//		jsActivity.setJavaCode(javaCode);
//		joinActivity.getEExtensibilityElements().add(jsActivity);
//		
//		return joinActivity;
//	}

	private String createQueryString( NamedElement set, Pin pin, Part part){
		if(part.getTypeDefinition() != null){
			return "";
		}else if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && BomUtils.isSinglePinInSet((PinSet)set)){
			return "";
		}
		String pinElementDeclaration = AbstractbpelUtil.getPinElementName(set, pin, part);
		return  "/" + pinElementDeclaration;

	}	
	
	private Variable getVariableForPinSet(PinSet pinSet){
		ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
		crIS.getSource().add(pinSet);
		getChildRules().add(crIS);
		crIS.transformSource2Target();
		getTarget().addAll(crIS.getTarget());
		Variable variable = (Variable)crIS.getTarget().get(0);
		return variable;
	}
	
	
	private Assign getTargetAssign(Collection assignTargets){
		if(assignTargets != null && !assignTargets.isEmpty()){
			Iterator it = assignTargets.iterator();
			while(it.hasNext()){
				Object target = it.next();
				if(target instanceof Assign){
					return (Assign)target;
				}
			}
		}
		
		return null;
	}
	
	private List getVariablesForPinSet(PinSet pinSet){
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		List variables = new LinkedList();
		if(!objectPins.isEmpty()) {
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin op = (ObjectPin)it.next();
				BPELVariable variable = createVariable(op);
				if(variable != null){
					getTarget().add(variable);
					variables.add(variable);
				}
			}
		}
		return variables;
	}
	
//	private BPELVariable getVariableForPin(PinSet pinSet, ObjectPin objectPin ){
//		BPELVariable variable = createVariable(objectPin);
//		getTarget().add(variable);
//		return variable;
//	}
	
	/**
	 * Returns the BPELVariable created for this objectPin. Creates the variable if the flag indicates so, else, 
	 * if the BPELVariable is not set, return null.
	 * @param pinSet
	 * @param objectPin
	 * @param createNew
	 * @return
	 */
	private BPELVariable getVariableForPin(PinSet pinSet, ObjectPin objectPin , boolean createNew){
		if(!createNew){
			ArrayList thisSources = new ArrayList();
			thisSources.add(((TypedElement)objectPin).getType());
			thisSources.add(objectPin);
			//check if there if this srcObj was already processed for Variable.
			TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), VariableRule.class, thisSources, BPELVariable.class );
			if(rule != null && !rule.getTarget().isEmpty()){
				return (BPELVariable)rule.getTarget().get(0);
			}else{
				return null;
			}
		}
		BPELVariable variable = createVariable(objectPin);
		getTarget().add(variable);
		return variable;

	}
	
	/**
	 * Re-executes this rule. After reExecuting the super, will attempt to rebuild the JavaSnippet that might have been created for
	 * this rule. If the input branches in this Join belong to parallel branches that were not processed at the time of processing 
	 * the flow (ex. parallel flow starting from the Process starts and/or pinSet), the Snippet created for this Join will be left
	 * as empty "". It will be created on a subsequent path (reExecute) when all the variables are known. 
	 * This is currently a workaround.. it should be handled by a ConcurrentBranchRule for the Process. 
	 */
	protected void reExecute(InputPinSet inputSet){
		super.reExecute(inputSet);
		if(bNeedAssign && allVariablesMapped && fAssign == null){
			createAssign(inputSet, (OutputPinSet)inputSet.getOutputPinSet().get(0));
		}
		reExecuteBPELPAlternativeActivity();
	}
	
	/**
	 * Attempts to rebuild the JavaSnippet that might have been created for this Join, if left unfinished. 
	 *
	 */
	private void reExecuteBPELPAlternativeActivity(){
		Activity bpelpActivity = getBPELPAlternativeActivity(getTarget());
		
		if(bpelpActivity != null){
			if(bpelpActivity instanceof Invoke){
				List extensibilityElements = bpelpActivity.getExtensibilityElements();
				JavaScriptActivity javaScript = (JavaScriptActivity)extensibilityElements.get(0);
				if(javaScript.getJavaCode().equals("")){
					InputPinSet is = (InputPinSet)getSource().get(0);
					Variable isVariable=null;
					Variable osVariable = null;
					/* For a join, the input set must be correlated to a single output set */
					OutputPinSet os = (OutputPinSet)is.getOutputPinSet().get(0);
					List isVariables = getVariablesForPinSet(is);
					List osVariables = getVariablesForPinSet(os);
					String assign = createJavaInvokeForWSICompliantDiamond(is, os);
					javaScript.setJavaCode(assign);
				}
			}
		}
	}
	
	/** 
	 * Returns the BPELP Alternative Activity that might have been created for this Join.
	 * @param targets
	 * @return
	 */
	private Activity getBPELPAlternativeActivity(List targets){
		if(targets == null || targets.isEmpty()){
			return null;
		}
		Activity bpelpActivity = null;
		Iterator it = targets.iterator();
		while(it.hasNext()){
			Object targetActivity = it.next();
			if(targetActivity instanceof AlternativeActivity){
				AlternativeActivity activity = (AlternativeActivity)targetActivity;
				bpelpActivity = ((Alternative)activity.getAlternative().get(1)).getActivity();
			}
		}
		return bpelpActivity;
	}
	
	private void createAssign(InputPinSet inputSet, OutputPinSet outputSet){

		/* Create an <assign> to copy the incoming data to the container associated 
		 * with the output set */
		/**
		 * Assumption :: the BOM Type between the input and output pin of Merge must be the same
		 * Then there are 2 cases.
		 * 1.  They have same lower and upper bound.  In such case they will share the same WSDL message def and a simple assign will handle this
		 * 2.  They have different lower and upper bound.  In such case, for BPEL, we need assign with XPath.  For BPELP, we need java snippets
		 *  
		 */
		fAssign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
		Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
		bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL);
		bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
		//	bpelAlternative.setActivity(createMultiplicityCopyAssign(inputSet,inputSetContainer,outputSet,outputSetContainer));
		bpelpAlternative.setActivity(createMultiplicityCopyJavaInvoke(inputSet,outputSet,true));				
		((AlternativeActivity)fAssign).getAlternative().add(bpelAlternative);
		((AlternativeActivity)fAssign).getAlternative().add(bpelpAlternative);

		
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), fAssign, BomUtils.getCanonicalName(inputSet.getAction()));
		fAssign.setName(convertedName);
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		dspName.setText(inputSet.getAction().getName()+" Assign");
		fAssign.getEExtensibilityElements().add(dspName);
		/* Add this <assign> to the list of targets of this rule */
		getTarget().add(fAssign);
		
		/* Create the links that connect the input set <activity> to this <assign>, and this assign
		 * to the output set <activity> */
		Link link = BPELFactory.eINSTANCE.createLink();

		Source isActivitySrc = BPELFactory.eINSTANCE.createSource();
		isActivitySrc.setActivity(fIsActivity);
		link.getSources().add(isActivitySrc);

		Target assignTar = BPELFactory.eINSTANCE.createTarget();
		assignTar.setActivity(fAssign);
		link.getTargets().add(assignTar);

			
		convertedName = converter.convertName(NamingUtil.findRegistry(this), link, fIsActivity.getName() + "_to_" + inputSet.getName() + "Assign");
		link.setName(convertedName);
		/* Add this link to the list of targets of this rule */
		getTarget().add(link);
		
		LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
		this.getChildRules().add(lr);
		lr.getSource().add(outputSet);
		lr.transformSource2Target();
		link = (Link)lr.getTarget().get(0);
		
		Source assignSrc = BPELFactory.eINSTANCE.createSource();
		assignSrc.setActivity(fAssign);
		link.getSources().add(assignSrc);

		Target osActivityTar = BPELFactory.eINSTANCE.createTarget();
		osActivityTar.setActivity(fOsActivity);
		link.getTargets().add(osActivityTar);
			
		convertedName = converter.convertName(NamingUtil.findRegistry(this), link, inputSet.getName() + "Assign_to_" + fOsActivity.getName());
		link.setName(convertedName);
		/* Add this link to the list of targets of this rule */
		getTarget().add(link);

	
	}

} //MergeRuleImpl
