/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * $Id: OutputRuleImpl.java,v 1.11.2.1 2009/04/13 18:47:11 build Exp $
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Output;
import com.ibm.wbit.bpelpp.Parameter;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Output Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class OutputRuleImpl extends AbstractProcDefRuleImpl implements OutputRule {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	protected OutputRuleImpl() {
		super();
	}
	private List currentPathCompatibleVariablesRules = new LinkedList();
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.OUTPUT_RULE;
	}

	/**
	 * source = BOM InputPinSet/OutputSet
	 * target = BPELPlus Input
	 * Iterates through each of the ObjectPins in the PinSet and creates for each of them a BPELPlus Parameter.
	 * Reuse the Variable when creating the Parameter.
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
			return true;				
		}
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no sources");
			return false;
		}
		
		
		PinSet pinSet = (PinSet)getSource().get(0);
		Output output = BPELPlusFactory.eINSTANCE.createOutput();
		
		TransformationRule containerActivityRule = CBPELUtil.findContainerBPELActivityRule(this);
		
		List objectPins = BomUtils.getObjectPinsForSet(pinSet);
		if(objectPins != null && !objectPins.isEmpty()){
			Iterator it = objectPins.iterator();
			while(it.hasNext()){
				ObjectPin objectPin = (ObjectPin)it.next();
				Parameter parameter = BPELPlusFactory.eINSTANCE.createParameter();
				BPELVariable variable = getVariableForObjectPin(objectPin);
				Message targetMsg = null;
				if(CBPELUtil.isFaultLinkVariable(pinSet)){
					CBPELUtil.registerFaultLinkVariable(getContext(),containerActivityRule,pinSet.getName(),variable);
				}				
				parameter.setVariable(variable);
				// GJD 20060831
				if (variable.getType() != null) { // original code
					NameProvider nameProvider = NameProviderFactory.getNameProvider(parameter);
					nameProvider.setContext(getContext());
					parameter.setName(nameProvider.getDefaultName(parameter, null, objectPin, NamingUtil.findRegistry(this), null));					
				} else {	// match the name of the parameter with the element declaration				   
					try{							
						parameter.setName(OriginalWsdlUtil.retrievePartNameFromWSDL(getContext(),objectPin));
					}catch(Exception e){
						String params[]={objectPin.getName()};
						LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME, params, e);
						BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_PARAMETER_NAME));
						runtimeEx.setChainedException(e);			
						throw runtimeEx;
					}			
				}				
				output.getParameter().add(parameter);
			}
		}
		//We skip add the fault output, BPEL activity supports only one input and one output
		//Fault output is supported by fault link semantics
		if(pinSet instanceof OutputPinSet){
			OutputPinSet ops = (OutputPinSet)pinSet;
			if(ops.getIsException().booleanValue()){
				if(CBPELUtil.shouldCreateWrapperContainer(getContext(), ops)){
					//Create wrapper variable reference to the wrapper element of the anonymous complex type
					//the original message will be retrieved and the wrapper element will be referenced
					Message targetMsg = CBPELUtil.retrieveMessageForPinSet(getContext(), ops);
					BPELVariable wrapperFaultVariable = createContainerForAnonymousType(targetMsg);
					CBPELUtil.registerFaultLinkVariable(getContext(),containerActivityRule,pinSet.getName(),wrapperFaultVariable);
					//add the mapping between the outputPinSet and wrapper variable
					getTarget().add(wrapperFaultVariable);
					executeHandlers();
				}				
			}else{
				getTarget().add(output);			
				executeHandlers();
			}
		}else if(pinSet instanceof InputPinSet){
			getTarget().add(output);			
			executeHandlers();
		}			
		propagateChildTargets();
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		
	}
	
	private BPELVariable createContainerForAnonymousType(Message targetMsg) {
		BPELVariable container = null;
		if(targetMsg != null){
			if(targetMsg.getEParts()!= null && targetMsg.getEParts().size() > 0){
				Part part = (Part)targetMsg.getEParts().get(0);
				if(part.getElementDeclaration() instanceof XSDElementDeclaration){
					container = BPELFactory.eINSTANCE.createBPELVariable();
					// for anonymous types set the element
					container.setXSDElement(part.getElementDeclaration());
					//pass the name through the registry... 
					String convertedName = null;							
					JavaNCNameConverter converter = new JavaNCNameConverter();		
					String name = part.getElementDeclaration().getName();									 
					convertedName = converter.convertName(NamingUtil.findRegistry(this), container, name + "Variable");
					container.setName(convertedName);	
					container.setMessageType(targetMsg);
					//organizeImports(container);
					getTarget().add(container);
				}
			}
		}
		return container;
	}
	
	private void organizeImports(BPELVariable variable){
		ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(this.getContext());
		Process process = (Process)currentProcessDefRule.getTarget().get(0);
		XSDSchema xsdSchema = null;
		if (variable.getType() != null) {
		    xsdSchema = variable.getType().getSchema();
		} else if (variable.getXSDElement() != null) {
		    xsdSchema = variable.getXSDElement().getSchema();
		}
		String xsdNamespace = "";
		if (xsdSchema != null) {
		    xsdNamespace = xsdSchema.getTargetNamespace();
		}
		if(xsdNamespace != null && !xsdNamespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) && xsdSchema != null && 
				!containsImport(process, xsdNamespace, xsdSchema.getSchemaLocation())){
			//we need to import the schema 
			com.ibm.wbit.bpel.Import xsdImport = BPELFactory.eINSTANCE.createImport();
			if (this.getSource().get(0) instanceof Class &&
			        ((Class)this.getSource().get(0)).getOwningPackage() instanceof ExternalSchema) {
			    xsdImport.setLocation(xsdSchema.getSchemaLocation());
			} else {	// original code
			    xsdImport.setLocation(ProcessUtil.createFileName(xsdNamespace)+ExportOperation.XSD_FILE_EXT);
			}	
			
			xsdImport.setNamespace(xsdNamespace);
			
			if (xsdImport.getLocation() != null && xsdImport.getLocation().endsWith(ExportOperation.WSDL_FILE_EXT)) {
			    xsdImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
			} else {
			    xsdImport.setImportType(AbstractbpelConstants.XSD_IMPORT_TYPE);
			}
			if (xsdImport.getLocation() != null && !xsdImport.getLocation().equals(ExportOperation.XSD_FILE_EXT)) {
				process.getImports().add(xsdImport);
			}
		}		
	}
	
	/**
	 * Checks is the Process already contains a wsdl import for the <code>targetNamespace</code> 
	 * @param process
 	 * @param targetNamespace
 	 * @param location 
	 * @return
	 */
	private boolean containsImport(Process process, String targetNamespace, String location){
		if (location == null) { //there is no schema location - internally created
			List imports = process.getImports();
			if(imports != null && !imports.isEmpty()){
				Iterator it = imports.iterator();
				while(it.hasNext()){
					com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
					if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.XSD_IMPORT_TYPE) && 
							thisImport.getNamespace().equals(targetNamespace)){
						return true;
					}
				}
			}
		} else {
			String fileLocation = null;
			if (location.indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
				fileLocation = location;//.substring(location.lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);
			} else {
				fileLocation = location;
			}
			List imports = process.getImports();
			String thisImportLocation = null;
			if(imports != null && !imports.isEmpty()){
				Iterator it = imports.iterator();
				while(it.hasNext()){
					com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
					if (thisImport.getLocation().indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
						thisImportLocation = thisImport.getLocation();//.substring(thisImport.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);					
					} else {
						thisImportLocation = thisImport.getLocation();
					}
					if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.XSD_IMPORT_TYPE) && 
							thisImport.getNamespace().equals(targetNamespace) && 
							thisImportLocation.equals(fileLocation)){
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Returns the variable generated for this <code>objectPin</code>
	 * @param objectPin a BOM ObjectPin
	 * @return BPELVariable
	 */
	private BPELVariable getVariableForObjectPin(ObjectPin objectPin){
		BPELVariable variable = createVariable(objectPin);
		return variable;
		
		
	}
	

} //OutputRuleImpl
