/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELPackage;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;
import com.ibm.wbit.bpel.services.partnerlinktype.Role;
import com.ibm.wbit.bpel.services.partnerlinktype.RolePortType;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.ProcessResolver;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Reference;
import com.ibm.wsspi.sca.scdl.mfc.MediationFlowImplementation;

/**
 * Checks if a BOM Acitivity is representing a resuable Process or Task.
 * Based on that creates the appropriate child rules.
 * Source: BOM Activity
 * Target:
 * Target Propagated: BPEL constructs - Variables, Partners, Sequence, Recieve, Reply 
 */
public class PartnerRuleImpl extends AbstractProcDefRuleImpl implements PartnerRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private static String PARTNER_SUFFIX="Partner";
	private Reference singleReference = null;
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PartnerRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.PARTNER_RULE;
	}

	/*
	 * Source: BOM Action OR BOM Datastore + (Optional) ProcessInterfaceRule (used for searching the PartnerLinkType)
	 * Target: BPEL Partner
	 * 
	 * Note: This rule requires that the associated WSDL PartnerLinkType rule has 
	 * previously been executed.
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		NamedElement srcElement = (NamedElement)getSource().get(0);
		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
			if(this.getParentRule() instanceof CallBehaviorActionRule){
				PartnerLink transformedPartner = (PartnerLink)this.getTarget().get(0);
				createSCDLReference(transformedPartner,srcElement,transformedPartner.getName());
			}
			return true;
		}
		
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}

		// Initialization
//		NamedElement srcElement = (NamedElement)getSource().get(0);
		TransformationContext context = getContext();
		
		// Obtain the PartnerLinkType
		ProcessInterfaceRule currentPIRule= (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context);
		ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(context);;
		ProcessInterfaceRule partnerLinkTypePIRule;
		// Ray: If there is a ProcessInterfaceRule passed in as the source of this rule,
		// we'll use it for searching the PartnerLinkType.  This is the case that we are
		// calling another Process and the PartnerLinkType is defined for the other Process' WSDL
		if (getSource().size()==2 && getSource().get(1) instanceof ProcessInterfaceRule) { // this check could be pretty fragile!!
			partnerLinkTypePIRule = (ProcessInterfaceRule)getSource().get(1);
		} else {
			partnerLinkTypePIRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context);
		}
		organizeImports((Process)currentProcessDefRule.getTarget().get(0),(Definition)partnerLinkTypePIRule.getTarget().get(0));
		PartnerLinkType partnerLinkType;
		TransformationRule rule = null;
		//Applicable to CBA rule created partner rule
		if(getSource().size() > 3 && getSource().get(getSource().size()-2) instanceof PartnerLinkRule){
			rule = (PartnerLinkRule)getSource().get(getSource().size()-2);
		}else{
			rule = TransformationUtil.getRuleForSource(
					//ProcessUtil.getProcessInterfaceRule(getContext(),action),
					partnerLinkTypePIRule, 
					getSource(),
					PartnerLinkType.class);
		}
		partnerLinkType = (PartnerLinkType) rule.getTarget().get(0);
		List partnerLinkTypeList = rule.getTarget();
		for(int i=0; i<partnerLinkTypeList.size(); i++){
			partnerLinkType = (PartnerLinkType)partnerLinkTypeList.get(i);
			// Create partner	
			PartnerLink partner = BPELFactory.eINSTANCE.createPartnerLink();
				//defect 75458
			String partnerName = null;
			if(srcElement instanceof Action || srcElement instanceof Datastore){
					boolean isProcess = false;
					if (srcElement instanceof Action && BomUtils.isTopLevelProcess((Action)srcElement)){
						partnerName = ((Activity)srcElement.eContainer()).getName();
						isProcess = true;
					}else { 
						partnerName = BomUtils.getCanonicalName(srcElement);
					}
					//if this is a service operation, use the PortType name for the partner
					if (srcElement instanceof Action && BomUtils.isServiceOperation((Action)srcElement)){
						if(getSource().size() >= 3 && getSource().get(2) instanceof PortType){
							partnerName = ((PortType)getSource().get(2)).getQName().getLocalPart();
							isProcess = true;
						}
					}
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), partner, partnerName + PARTNER_SUFFIX);
				partner.setName(convertedName);
				partner.setPartnerLinkType(partnerLinkType);
				// There are 2 cases that the action is a top level process
				// 1. It is the process itself
				// 2. It is the remote process (the process that is being invoked by this process)
				boolean isRemoteProcess=false;
				if (isProcess) {
					// Need an additional check for the case if a process is calling itself
					if (!isRemoteProcess) {
						if (this.eContainer instanceof CallBehaviorActionRule) {
							isRemoteProcess = true;
						}
					}
				}
				if (isProcess && !isRemoteProcess) {
					for (Iterator it = partnerLinkType.getRole().iterator(); it.hasNext(); ) {
						Role role = (Role)it.next();
						//if (partner.eIsSet(BPELPackage.eINSTANCE.getPartnerLink_MyRole()) && i==1) {
						if (i==1) {
							partner.setPartnerRole(role);
							// This is for callback
							createSCDLReference(partner,srcElement,partner.getName());
							// Need to update the callback reference to wire to this process
							// Since we never know the order of the processes we are transforming,
							// we either update the reference here or when we transform the CBA
							// that calls this process
							updateCallbackReference((PortType)role.getPortType().getName());
							organizeImports(role.getPortType().getName());
						} else {
							partner.setMyRole(role);
							organizeImports(role.getPortType().getName());
						}
					}
				} else {
					Role role = (Role)partnerLinkType.getRole().get(0);
					if (i==1) {
						partner.setMyRole(role);
						organizeImports(role.getPortType().getName());
					} else {
						partner.setPartnerRole(role);
						//we should not generate the PartnerLink Template for the case where the code generates 
						//as part of the Test in the d2d case. In this case, having both the Import and the PartnerLink value will cause problems
						//for the Test in isolation scenario where things should be emulated.
						boolean isCallingProcess = isProcess && isRemoteProcess && "PROCESS".equals(((Action)srcElement).getAspect());
						if(!BpelOptionsUtil.isGenerateForTestDeploy() && isCallingProcess){
							//enable the TemplateName in the partnerLink for lateBinding
							ProcessResolver processResolver = BPELPlusFactory.eINSTANCE.createProcessResolver();
							List<NamedElement> sources = new ArrayList<NamedElement>();
							sources.add(srcElement);
							ComponentRule compRule = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,sources,DocumentRoot.class);
							if(compRule != null && !compRule.getTarget().isEmpty()){
								DocumentRoot dr = (DocumentRoot)compRule.getTarget().get(0);
								String name = null;
								if( dr.getComponent() != null){
									name = dr.getComponent().getName();
								}else if(dr.getImport() != null){
									name = dr.getImport().getName();
								}
								if(name != null && !name.equals("") ){
									String templateName = name.substring(name.lastIndexOf("/") +1);
									if(templateName != null && !templateName.equals("")){
										javax.xml.namespace.QName qname = new javax.xml.namespace.QName(null,templateName);
										processResolver.setProcessTemplateName(qname);
										partner.getExtensibilityElements().add(processResolver);
									}
								}
							}
						}
						organizeImports(role.getPortType().getName());
						createSCDLReference(partner,srcElement,partner.getName());
					}
				}
			}
		getTarget().add(partner);
		}

        //AZ WMT 20070327 add map test
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/*
	 * Source: BOM Action OR BOM Datastore + (Optional) ProcessInterfaceRule (used for searching the PartnerLinkType)
	 * Target: BPEL Partner
	 * 
	 * Note: This rule requires that the associated WSDL PartnerLinkType rule has 
	 * previously been executed.
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target_bak()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "already completed");
			return true;
		}
		
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}

		// Initialization
		NamedElement srcElement = (NamedElement)getSource().get(0);
		TransformationContext context = getContext();
		
		// Obtain the PartnerLinkType
		ProcessInterfaceRule currentPIRule= (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context);
		ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(context);;
		ProcessInterfaceRule partnerLinkTypePIRule;
		// Ray: If there is a ProcessInterfaceRule passed in as the source of this rule,
		// we'll use it for searching the PartnerLinkType.  This is the case that we are
		// calling another Process and the PartnerLinkType is defined for the other Process' WSDL
		if (getSource().size()==2 && getSource().get(1) instanceof ProcessInterfaceRule) { // this check could be pretty fragile!!
			partnerLinkTypePIRule = (ProcessInterfaceRule)getSource().get(1);
			//AR WSDL import support.. add WSDL import for the PartnerLinkType to the Process definition file.
			//organizeImports((Process)currentProcessDefRule.getTarget().get(0),(Definition)partnerLinkTypePIRule.getTarget().get(0));
			//organizeImports((Definition)currentPIRule.getTarget().get(0),(Definition)partnerLinkTypePIRule.getTarget().get(0));
		} else {
			partnerLinkTypePIRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context);
		}
		organizeImports((Process)currentProcessDefRule.getTarget().get(0),(Definition)partnerLinkTypePIRule.getTarget().get(0));
		PartnerLinkType partnerLinkType;
		TransformationRule rule = 
			TransformationUtil.getRuleForSource(
				//ProcessUtil.getProcessInterfaceRule(getContext(),action),
				partnerLinkTypePIRule, 
				srcElement,
				PartnerLinkType.class);
		partnerLinkType = (PartnerLinkType) rule.getTarget().get(0);

		// Create partner	
		PartnerLink partner = BPELFactory.eINSTANCE.createPartnerLink();
		String partnerName;

		if (srcElement instanceof Action || srcElement instanceof Datastore){
			boolean isProcess = false;
			if (srcElement instanceof Action && BomUtils.isTopLevelProcess((Action)srcElement)){
				partnerName = ((Activity)srcElement.eContainer()).getName();
				isProcess = true;
			} else { 
				partnerName = BomUtils.getCanonicalName(srcElement);
			}
			
				
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), partner, partnerName + PARTNER_SUFFIX);
			partner.setName(convertedName);
			//partner.setName(partnerName + PARTNER_SUFFIX);
			partner.setPartnerLinkType(partnerLinkType);
			// There are 2 cases that the action is a top level process
			// 1. It is the process itself
			// 2. It is the remote process (the process that is being invoked by this process)
			boolean isRemoteProcess=false;
			if (isProcess) {
//				Definition thisProcess = (Definition)currentPIRule.getTarget().get(0);
//				Definition actionProcess = (Definition)ProcessUtil.getProcessInterface(context,srcElement);
//				// this test will not work if we ever (for whatever reason) want to allow
//				// sharing of WSDLs between processes
//				isRemoteProcess = (thisProcess!=actionProcess);
				isRemoteProcess = isRemoteProcess((Action)srcElement);
				// Need an additional check for the case if a process is calling itself
				if (!isRemoteProcess) {
					if (this.eContainer instanceof CallBehaviorActionRule) {
						isRemoteProcess = true;
					}
				}
			}
	
			if (isProcess && !isRemoteProcess) {
				for (Iterator it = partnerLinkType.getRole().iterator(); it.hasNext(); ) {
					Role role = (Role)it.next();
					if (partner.eIsSet(BPELPackage.eINSTANCE.getPartnerLink_MyRole())) {
						partner.setPartnerRole(role);
						// This is for callback
						createSCDLReference(partner,srcElement,partner.getName());
						// Need to update the callback reference to wire to this process
						// Since we never know the order of the processes we are transforming,
						// we either update the reference here or when we transform the CBA
						// that calls this process
						updateCallbackReference((PortType)role.getPortType().getName());
						organizeImports(role.getPortType().getName());
					} else {
						partner.setMyRole(role);
						organizeImports(role.getPortType().getName());
					}
				}
			} else {
				for (Iterator it = partnerLinkType.getRole().iterator(); it.hasNext(); ) {
					Role role = (Role)it.next();
					if (partner.eIsSet(BPELPackage.eINSTANCE.getPartnerLink_PartnerRole())) {
						partner.setMyRole(role);
						organizeImports(role.getPortType().getName());
					} else {
						partner.setPartnerRole(role);
						organizeImports(role.getPortType().getName());
						createSCDLReference(partner,srcElement,partner.getName());
					}
				}
			}
			
			
		
		} else if(srcElement instanceof com.ibm.btools.bom.model.services.Operation){
			partnerName = BomUtils.getCanonicalName((com.ibm.btools.bom.model.services.Operation)getSource().get(0));
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), partner, partnerName + PARTNER_SUFFIX);
			partner.setName(convertedName);			
			//partner.setName(partnerName + PARTNER_SUFFIX);
			partner.setPartnerLinkType(partnerLinkType);
				
			for (Iterator it = partnerLinkType.getRole().iterator(); it.hasNext(); ) {
				Role role = (Role)it.next();
				if (partner.eIsSet(BPELPackage.eINSTANCE.getPartnerLink_PartnerRole())) {
					partner.setMyRole(role);
				} else {
					partner.setPartnerRole(role);
				}
			}
		}


		getTarget().add(partner);
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	
	/**
	 * returns true if the process is a remote process (accessed through CBA)
	 * @param process
	 * @return
	 */
	private boolean isRemoteProcess(Action process) {
		Definition thisProcess = (Definition)((ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext())).getTarget().get(0);
		Definition actionProcess = (Definition)ProcessUtil.getProcessInterface(getContext(),process);
		// this test will not work if we ever (for whatever reason) want to allow
		// sharing of WSDLs between processes
		return (thisProcess!=actionProcess);		
	}
	

	/**
	 * Adds the import for the <code>portType</code> namespace, if not the same as the
	 * WSDL <code>definition</code> namespace.
	 * @param definition WSDL Definition
	 * @param portType WSDL PortType
	 */	
	private void organizeImports(Process processDef, Definition toBeImportedDef){
		String namespace = toBeImportedDef.getTargetNamespace();
		if(!containsImport(processDef,toBeImportedDef)){		
			//we neet to import the schema 
			com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
			String uri = OriginalWsdlUtil.getOriginalFileNameFromContext(getContext(),toBeImportedDef);
			if (toBeImportedDef.getLocation() != null && ExportOperationUtil.isExternalFile(toBeImportedDef.getLocation())) {
				wsdlImport.setLocation(ExportOperationUtil.replaceConvertedSpaces(toBeImportedDef.getLocation()));
			} else if(uri != null && uri.length() > 0){
				wsdlImport.setLocation(uri);
			}else{
				wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
			}		
			
			wsdlImport.setNamespace(namespace);
			wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
			if (!wsdlImport.getLocation().equals(ExportOperation.WSDL_FILE_EXT)) {
				processDef.getImports().add(wsdlImport);
			}
		}
	}
	
	/**
	 * Checks is the Process already contains a wsdl import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, String targetNamespace){
		
		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && thisImport.getNamespace().equals(targetNamespace)){
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Create SCDL reference
	 * @param partner
	 * @param srcElement
	 * @param refName
	 * @param isMultiple
	 */
	private void createSCDLReference(PartnerLink partner,EObject srcElement,String refName) {
		// don't create a reference if the action is an inline task
		if (srcElement instanceof Action) {
			Action action = (Action)srcElement;
			if(StaffUtil.isInlineTask(action)){
				return;
			}	
		}
		// get the Partner's PortType
		Role role = partner.getPartnerRole();
		RolePortType rpt = role.getPortType();
		PortType refPt = (PortType)rpt.getName();

		// Create reference for Process Component
		Component procCmp = ProcessUtil.getProcessComponent(getContext());
		DocumentRoot source = (DocumentRoot)procCmp.eContainer();

		ArrayList sources = new ArrayList();

		//d144756
//		if (srcElement != null && srcElement instanceof StructuredActivityNode &&
//				((StructuredActivityNode)srcElement).getAspect() != null &&
//				!((StructuredActivityNode)srcElement).getAspect().equals(BOMUtil.ASPECT_SERVICE_OPERATION)) {
			sources.add(srcElement);
//		}			
		sources.add(refPt);
		ComponentRule cr = null;
//		if(srcElement instanceof Action && FabricIntegrationUtil.getComponentRuleForServiceOperationIfAny(getContext(), (Action)srcElement) != null){
//			cr = FabricIntegrationUtil.getComponentRuleForServiceOperationIfAny(getContext(), (Action)srcElement);
//		}else{
		if(getSource().size() >= 1 && getSource().get(getSource().size()-1) instanceof DocumentRoot){
			DocumentRoot docRoot = (DocumentRoot)getSource().get(getSource().size()-1);
			//find the rule for the target, reuse the component for the non-overriden CBAs, which should have been created already 
			if(getSource().get(0) instanceof Action && getSource().get(1) instanceof CallBehaviorAction && !FabricIntegrationUtil.isOverriden((CallBehaviorAction)getSource().get(1)) && FabricIntegrationUtil.getComponentRuleForCbaAction(getContext(), (Action)getSource().get(0)) != null){
//				cr = (ComponentRule)TransformationUtil.getRuleForTarget(getRoot(), docRoot);
				cr = FabricIntegrationUtil.getComponentRuleForCbaAction(getContext(), (Action)getSource().get(0));
			}else{
				//we need to use the action and cba to do look up
				List lookupCriterion = new ArrayList();
				lookupCriterion.add(getSource().get(0));
				lookupCriterion.add(getSource().get(1));
//				cr = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,sources,DocumentRoot.class);
				cr = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,lookupCriterion,DocumentRoot.class);
			}	
		}else{
			cr = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,sources,DocumentRoot.class);
		}
//		}
		DocumentRoot importTarget=null;
		DocumentRoot mediationTarget=null;
		DocumentRoot target = null;
		if (cr != null) {
			//GJD find import target
			Iterator it = cr.getTarget().iterator();
			Object obj = null;
			while (it.hasNext()) {
				obj = it.next();
				if (obj instanceof DocumentRoot && ((DocumentRoot)obj).getImport() != null){					
					if(ScdlUtil.isCallbackImport(obj)){
						continue;
					}
					importTarget = (DocumentRoot)obj;
				}else if (obj instanceof DocumentRoot && ((DocumentRoot)obj).getComponent() != null && ((DocumentRoot)obj).getComponent().getImplementation() != null 
						&& ((DocumentRoot)obj).getComponent().getImplementation() instanceof MediationFlowImplementation) {
					mediationTarget = (DocumentRoot)obj;
				}
			}
			if(mediationTarget != null){ target = mediationTarget;}
			else if (importTarget != null){ target = importTarget;}
			if (target == null){
				target =(DocumentRoot)cr.getTarget().get(0);
			}
		}
		if (source!=null) {			
			this.singleReference = ScdlUtil.createSCDLComponentReference(source,target,refPt,refName,false);
		}
	}
	
//	private String getComponentPath(EObject targetElement) {
//		if (targetElement.eContainer() instanceof Activity) {
//			if (targetElement instanceof Action && isRemoteProcess((Action)targetElement)) {
//				return SAUtil.createPackage(targetElement);				
//			}
//		}
//		return null;
//	}
	
	/**
	 * Create SCDL Reference
	 * @param partner
	 * @param srcElement
	 * @param refName
	 * @param isMultiple - create 0..n multiplicity for the Reference is this param is true
	 */
//	private void createSCDLReference(PartnerLink partner,EObject srcElement,String refName,boolean isMultiple) {
//		// get the Partner's PortType
//		Role role = partner.getPartnerRole();
//		RolePortType rpt = role.getPortType();
//		PortType pt = (PortType)rpt.getName();
//		
//		// Create reference for Process Component
//		Component procCmp = ProcessUtil.getProcessComponent(getContext());
//		if (procCmp != null) {
//			Reference reference = SCDLFactory.eINSTANCE.createReference();
//			reference.setName(refName);
//			WSDLPortType wsdlPT = com.ibm.wsspi.sca.scdl.wsdl.WSDLFactory.eINSTANCE.createWSDLPortType();
//			Object qname = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),pt.getQName().getLocalPart(),null);
//			wsdlPT.setPortType(qname);			
//			reference.getInterfaces().add(wsdlPT);
//			if (isMultiple) {
//				reference.setMultiplicity(Reference.MULTIPLICITY_0_N);
//			}
//			// Update namespace prefix
//			DocumentRoot procCmpRoot = (DocumentRoot)procCmp.eContainer();
//			if (!procCmpRoot.getXMLNSPrefixMap().containsValue(((QName)qname).getNamespaceURI())) {
//				String prefix = createNewSCDLNSPrefix(procCmpRoot.getXMLNSPrefixMap());
//				procCmpRoot.getXMLNSPrefixMap().put(prefix,((QName)qname).getNamespaceURI());
//			}				
//			if (procCmp.getReferenceSet()==null) {
//				procCmp.setReferenceSet(SCDLFactory.eINSTANCE.createReferenceSet());
//			}
//			procCmp.getReferenceSet().getReferences().add(reference);
//
//			// Create Wire
//			ArrayList sources = new ArrayList();
//			sources.add(srcElement);
//			sources.add(pt);
//			ComponentRule cr = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,sources,DocumentRoot.class);
//			if (cr != null) {
//				DocumentRoot root =(DocumentRoot)cr.getTarget().get(0);
//				Part cmp=null;
//				if (root.getComponent()!=null) {
//					cmp = root.getComponent();
//				} else {
//					cmp = root.getImport();
//				}
//				Wire wire = SCDLFactory.eINSTANCE.createWire();
//				wire.setTargetName(cmp.getName());
//				reference.getWires().add(wire);
//			}
//		}		
//	}
//	
//	public String createNewSCDLNSPrefix(Map nsMap) {
//		int i = 1;
//		
//		String prefixName = ComponentRuleImpl.SCDL_NS_PREFIX + i++;
//		while (nsMap.containsKey(prefixName))
//			prefixName = ComponentRuleImpl.SCDL_NS_PREFIX + i++;
//		
//		return prefixName;
//	}		
	
//	private void updateCallbackReference(PortType callbackPT) {	
//		ComponentRule rule = (ComponentRule)TransformationUtil.getRuleForSource(getRoot(),ComponentRule.class,callbackPT,DocumentRoot.class);
//		if (rule!=null) {
//			DocumentRoot dr = (DocumentRoot)rule.getTarget().get(0);
//			if (dr.getComponent()!=null) {
//				Component tarComp = dr.getComponent();
//				Component srcComp = ProcessUtil.getProcessComponent(getContext());
//				ScdlUtil.updateSCDLComponentReference(srcComp,tarComp,callbackPT);
//				
//			}
//		}
//	}
	private void updateCallbackReference(PortType callbackPT) {	
		Collection rules = TransformationUtil.getRulesForSource(getRoot(),ComponentRule.class,callbackPT,DocumentRoot.class);
		if (!rules.isEmpty()) {
			for (Iterator it=rules.iterator();it.hasNext();) {
				TransformationRule rule = (TransformationRule)it.next();
				DocumentRoot dr = (DocumentRoot)rule.getTarget().get(0);
				if (dr.getComponent()!=null) {
					Component tarComp = dr.getComponent();
					Component srcComp = ProcessUtil.getProcessComponent(getContext()); 
					String targetProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(tarComp.eContainer());
					String sourceProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(srcComp.eContainer());
					if(targetProjectName != null && !targetProjectName.equals(sourceProjectName)){
						ScdlUtil.updateSCDLComponentReference(srcComp,tarComp,callbackPT,true);		
					}else{
						ScdlUtil.updateSCDLComponentReference(srcComp,tarComp,callbackPT);
					}
				}
			}
		}
	}
	
	/**
	 * Adds the import for the <code>portType</code> namespace, if not the same as the
	 * WSDL <code>definition</code> namespace.
	 * @param definition WSDL Definition
	 * @param portType WSDL PortType
	 */	
	private void organizeImports(Object obj){
		
		ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(this.getContext());
		Process processDef = (Process)currentProcessDefRule.getTarget().get(0);
		if(obj instanceof PortType){
			Definition toBeImportedDef = ((PortType)obj).getEnclosingDefinition();
			String namespace = toBeImportedDef.getTargetNamespace();
			if(!namespace.equals(processDef.getTargetNamespace())){
				//should add wsdl import.. leave it for the one that is saving the resource
				if(!containsImport(processDef, toBeImportedDef)){
					//we need to import the schema 
					com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
					String uri = OriginalWsdlUtil.getOriginalFileNameFromContext(getContext(),toBeImportedDef);
					if (toBeImportedDef.getLocation() != null && ExportOperationUtil.isExternalFile(toBeImportedDef.getLocation())) {
						wsdlImport.setLocation(ExportOperationUtil.replaceConvertedSpaces(toBeImportedDef.getLocation()));
					} else if(uri != null && uri.length() > 0){		// original code
						wsdlImport.setLocation(uri);
					}else{
						wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
					}					
					
					wsdlImport.setNamespace(namespace);
					wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
					if (!wsdlImport.getLocation().equals(ExportOperation.WSDL_FILE_EXT)) {
						processDef.getImports().add(wsdlImport);
					}
				}	
			}
		}
	}
	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, Definition toBeImportedDef){
		
		String targetNamespace = toBeImportedDef.getTargetNamespace();
		String uri = OriginalWsdlUtil.getOriginalFileNameFromContext(getContext(),toBeImportedDef);
		String location = null;
		if(uri != null && uri.length() > 0){
			location = uri;
		}else{
			location  = toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT;
		}
		List imports = process.getImports();
		String thisImportLocation = null;
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if (thisImport.getLocation().indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
					thisImportLocation = thisImport.getLocation().substring(thisImport.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);					
				} else {
					thisImportLocation = thisImport.getLocation();
				}
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && 
						thisImport.getNamespace().equals(targetNamespace) &&
						thisImportLocation.equals(location)){
					return true;
				}
			}
		}
		return false;
	}
    //AZ Return the reference for the rule handler to retrieve
	public Reference getSingleReference() {
		return singleReference;
	}
} //PartnerRuleImpl
