/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.impl.ArtifactsFactoryImpl;
import com.ibm.btools.bom.model.models.InformationModel;
import com.ibm.btools.bom.model.models.impl.ModelsFactoryImpl;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InitialNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.processes.activities.impl.ActivitiesFactoryImpl;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.LinkRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.PinSetRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CycleLinkUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.ElementDeclarationRule;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Throw;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Catch;
import com.ibm.wbit.bpelpp.FaultSource;
import com.ibm.wbit.bpelpp.FaultSources;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;
//import com.ibm.xtq.xslt.drivers.OptimizationRegimen;


/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Process Rule</b></em>'.
 * 
 * <p> This method is responsible for 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ProcessRuleImpl extends ProcessWalkingRuleImpl implements ProcessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* attribute indicate whether it is first time execution of this rule, default to true */
	 private boolean ivFirstTimeExec = true;
	 
	/* Holds the mapping from pin set to the activity created to represent it */
	private HashMap fPinSetMap = new HashMap(); 
	
	/* attribute to keep the flow created by branch rule*/
	private Flow ivBranchFlow = null; 
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.PROCESS_RULE;
	}

	/* *
	 * source: BOM InputPinSet of StructureActivityNode (non-task) and Flow
	 * target: BPEL Flow
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/*
		 * Map all elements in the StructuredActivityNode reachable from the InputPinSet and include them in the flow.
		 * This include action connected from InitialNode. 
		 * 
		 * 1. create flow
		 * 2. create BPEL variables for the variables associated with this StructuredActivityNode.
		 * 2.5 create BPEL variable corresponding to the termination node(s) of the SAN
		 * 3. do input set links.
		 * 4. for each input set reachable from the source input set, 
		 *    step through it, create child rules and execute the rule, 
		 *    until reached end of StructuredActivityNode.
		 * 	4.1 If there is more then one input set that is reachable from the source input set, this qualifies for parallel flow so we should 
		 * record the variables that were assigned on each branch and update the registry of "forbiden" variable - variables
		 * 	assigned on one branch that cannot be assigned on the next branch. 2 Registries are kept : one for the VariablesOnParallelFlow - variables 
		 * assigned on a previous parallel branch that cannot be used for another branch, VariableOnCurrentPath- variable created on a Parallel branch that should
		 * be assigned with priority.
		 * 5. do output set links.
		 * 6. consume and propagate targets produced by child rules.
		 *   
		 */
		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
		
		//source of this rule is only one input set
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		setEntryInputPinSet(inputSet);
		
//		/* Check the attributes that are not supported by the TE */
//		if (inputSet.getAction().getInputPinSet().size() > 1){
//			BTRuntimeException e = new BTRuntimeException(MessageKeys.SAN_MULTIPLE_INPUT_SETS_NOT_SUPPORTED);
//			LoggingUtil.logError(MessageKeys.SAN_MULTIPLE_INPUT_SETS_NOT_SUPPORTED);
//			throw e;		
//		}
				
		//check if this is not first time execute this rule
		if (!ivFirstTimeExec) {
			reExecute(inputSet);
			
			executeHandlers();
			LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			return true;
		}
		
		//else it is first time execution of the rule
		ivFirstTimeExec = false;
		//Flow flow = BPELFactory.eINSTANCE.createFlow();
		Flow flow = (Flow)getSource().get(1);
		//set flow as the first target of this rule
		getTarget().add(flow);
		
		initializePathsVariablesForBranchRegion();
		
		//For each BOM variable, v, that is contained in the structured activity node that owns the source input set, Create a ContainerRule, cr
		transformBOMVariable();
		
		// Create a BOM Variable to keep track of whether this SAN has yet been terminated.
		transformTerminationNode();
		
		//create variables for InputPinSet and OutputPinSet used for entry/exit transitions for local container (loop/subprocess)
		JavaNCNameConverter converter = new JavaNCNameConverter();
		List variables = createEntryExitVariables(converter);
		if (variables != null) {
			getTarget().addAll(variables); //propagate all these types to parent
		}
					
		doInputPinSetConnection(flow);
		
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		
		List inputSetList=bw.getNext(inputSet, BomWalker.INPUT_SET, false);
		//walkThruInputPinSets(bw,inputSetList,currentPathVariables,currentPathVariablesSnapshot,forbiddenVariables);
		walkThruInputPinSets(bw,inputSetList);
		
		//!!main logic of this method is here
		//walk through each of the input set, create and execute child rules
//		InputPinSet nextIS = null;
//		List nextInputPinSets = null;
//		for (int i = 0, count = inputSetList.size(); i<count; i++){
//			nextIS = (InputPinSet)inputSetList.get(i);
//			do{			
//				nextIS = stepThroughBOM(nextIS);
//				if (nextIS != null){ 	//means we just created a child rule
//					//we must continue
//					// It is safe just to use the first elment since we always skip 
//					// the branch point, we should only have one InputPinSet back
//					nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
//					if (nextInputPinSets == null || nextInputPinSets.isEmpty()) {
//						nextIS = null;
//						//doOutputPinSetConnection(flow, true);
//					} else {
//						nextIS = (InputPinSet) nextInputPinSets.get(0);
//					}					
//				}
//				//else nextIS == null, we want stop walk down that path
//					 
//			} while(nextIS!= null);	
//			//start recording the "forbiden" variables
//			currentPathVariables.addAll(ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext()));
//			List addedVariables = getPathAddedVariables(currentPathVariablesSnapshot, currentPathVariables);
//			ProcessUtil.registerVariableForParallelFlow(this.getContext(), addedVariables, false);
//			//reset the current list of variables on the current path
//			ProcessUtil.registerVariablesForCurrentPath(this.getContext(), currentPathVariablesSnapshot, true);
//		}
//		ProcessUtil.registerVariableForParallelFlow(this.getContext(), forbiddenVariables, true);
	
		//check if SAN is branch point 
		createBranchRuleIfAny();		
		
		//process target produced by child including propagate target to parent
		processChildTargets(flow);
		
		//connect OutputPinSet with acvtivites inside and outside of this SAN
		doOutputPinSetConnection(flow);	
	
		setComplete(true);	

		executeHandlers();

		// now handles all non-connected nodes.  we do it here at the end to minimize any
		// side-effects of the regular transformation
		StructuredActivityNode san = (StructuredActivityNode)inputSet.getAction();
		if (BpelOptionsUtil.isTransformNonConnectedNode() && san.getInputPinSet().indexOf(inputSet)==san.getInputPinSet().size()-1) { // ensure we do it once and do it at the end
			List nonConnectedInputPinSet = getNonConnectedInputPinSet(san,inputSet);
			if (!nonConnectedInputPinSet.isEmpty()) {
				// We don't care about reusing variables or not (in fact we shouldn't register them to affect
				// regular transformation). So set those lists to null
				walkThruInputPinSets(bw,nonConnectedInputPinSet);
			}
		}
		
		//reinitialize the Variable Registry at the exit from this rule..
		ProcessUtil.registerVariablesForCurrentPath(getContext(), currentPathVariables, false);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return true;
	}
	
	public Activity getPinSetActivity(PinSet ps){
	    return (Activity)fPinSetMap.get(ps);
	}
	
	private void walkThruInputPinSets(BomWalker bw, List inputSetList) {
		InputPinSet nextIS = null;
		List nextInputPinSets = null;
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		for (int i = 0, count = inputSetList.size(); i<count; i++){
			nextIS = (InputPinSet)inputSetList.get(i);
			InputPinSet mergeSet = nextIS;
			List nonCbVariables = getVariablesForCBPoint(inputSet, mergeSet, false);;
			
		    //for the CB the BCPoint Variables are already added to the variablesOnCurrentPath. We should 
		    //remove the CB variable created for the CB set that is not in focus - not linked to this inputPinSet
		    //and add them to the list of forbiddenVariables. 
			if(count >1){
				updatePathRegistryAtBranchPoint(inputSet, mergeSet);
			}

			do{	
				nextIS = stepThroughBOM(nextIS);
				if (nextIS != null){ 	//means we just created a child rule
					
					//we must continue
					// It is safe just to use the first elment since we always skip 
					// the branch point, we should only have one InputPinSet back
					nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
					if (nextInputPinSets == null || nextInputPinSets.isEmpty()) {
						nextIS = null;
						//doOutputPinSetConnection(flow, true);
					} else {
						nextIS = (InputPinSet) nextInputPinSets.get(0);
					}					
				}
				//else nextIS == null, we want stop walk down that path
					 
			} while(nextIS!= null);	
			if (currentPathVariables!=null && currentPathVariablesSnapshot!=null && forbiddenVariables!=null) {
				resetPathRegistryInBranchPoint(inputSet, mergeSet);
			}
		}
		
		if (currentPathVariables!=null && currentPathVariablesSnapshot!=null && forbiddenVariables!=null) {
			ProcessUtil.registerVariableForParallelFlow(this.getContext(), forbiddenVariables, true);
		}
	}
	
	/**
	 * Process targets produced by child rules.
	 * Links, Containers and Partners are propagated to parent rule. 
	 * If the StructuredActivityNode is also a branch poin (e-branch or c-branch), 
	 * the flow created by the branch rule is propagated to parent rule.
	 * All other activities are contained by this <code>flow</code>. 
	 * @param flow The BPEL <flow> to which certain child targets should be added
	 */
	private void processChildTargets(Flow flow){
		List childRules = getChildRules();
		TransformationRule nextRule = null;
		List targetList= null;
		Object target;
		int count= childRules.size();
		int targetCount;
		for(int i=0; i<count; i++){
			nextRule = (TransformationRule)childRules.get(i);
			targetList = nextRule.getTarget();
			targetCount = targetList.size();
			for (int j=0; j< targetCount; j++){
				target = targetList.get(j);
				if(target instanceof Link || target instanceof BPELVariable || target instanceof PartnerLink){
//				if (target instanceof Link) {
//					Links links = flow.getLinks(); 
//					if (links==null) {
//						links = BPELFactory.eINSTANCE.createLinks();
//						flow.setLinks(links);
//					}
//					links.getChildren().add(target);
//				}
//				else if(target instanceof BPELVariable || target instanceof PartnerLink){
					getTarget().add(target); //propagate all these types to parent	
				} else if (target instanceof Activity){

					if((target instanceof Flow) && (target == getBranchFlow())){
						// propagate branch rule's flow to parent
						getTarget().add(target);
					} else if ((target instanceof Flow) && (((Flow)target).getActivities().isEmpty())){
						//do nothing since we don't want empty flow 						
					} else {	
						//add all other activity to this flow
						flow.getActivities().add(target);
					}
				} else{
//					TODO: what other type could be?
					getTarget().add(target); //propagate all other unknown type to parent
				}
			}
		}
	
	}
	
	/**
	 * Check if the Input Set associtated StructuredActivityNode is also an exclusive branch point.
	 * if yes, Create exclusive branch rule 
	 * if not, create Concurrent branch rule if the OutputPinSet correlated with the InputPinSet is a concurrent branch rule. 
	 * Save the branch flow created so it can be propagated to parent later on. 
	 */
	protected TransformationRule createBranchRuleIfAny(){		
		
		TransformationRule rule = null;		
		InputPinSet inputSet = ((InputPinSet)getSource().get(0));
		
		//if this processRule is create by top level StructuredActivityNode, then don't need to do this.
		if(BomUtils.isTopLevelProcess(inputSet.getAction())){
		//if(BomUtils.isProcess(inputSet.getAction())){ // TODO: RAY:tesing for subprocess
			return null;
		}
		
		//check if input set is exclusive branch
		if(BomHelper.getInstance().isEBranch(inputSet)){
			rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
			rule.getSource().add(inputSet);
			getChildRules().add(rule);
			rule.transformSource2Target();
			
			//save the flow
			setBranchFlow((Flow)rule.getTarget().get(0));
		} else { //check if output is Concurrent branch
			// it is safe to assume there is only one output set since it is not e-branch.
			List opss = BomUtils.getCorrelatedPinSets(inputSet);
			if (opss.size()>0) {
				OutputPinSet outputSet=(OutputPinSet)opss.get(0);
//				if (inputSet.getOutputPinSet().size()!=0) {
//					outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
//				} else {
//					outputSet = (OutputPinSet)inputSet.getAction().getOutputPinSet().get(0);
//				}
				if (BomHelper.getInstance().isCBranch(outputSet)){
					rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
					rule.getSource().add(outputSet);
					getChildRules().add(rule);
					rule.transformSource2Target();
				
					//save the flow
					setBranchFlow((Flow)rule.getTarget().get(0));	 
				} else {
					//do nothing
				}
			}
		}
		
		return rule;
	
	}

	
	/**
	 * Connect InputPinSet of the SAN including:
	 * 1. create BPEL Activity (using methods in ConnectableRule) to represent the Inputset, 
	 *    and set it as the target of links coming into the InputPinSet.
	 * 	  Update joinCodition for the activity	
	 * 		
	 * 2. Create AssignRule or LinkRule for each pin in the InputPinSet to 
	 *    represent data flows into the internal activities of the SAN.
	 * 	  Set the Activity as source of the links that are just created by AssignRule or LinkRule.
	 *  	 
	 * @param flow - BPEL flow that is the container for Activity
	 */
	private void doInputPinSetConnection(Flow flow){
	   
		InputPinSet inputSet = (InputPinSet)getSource().get(0);	
       
		List resultList = mapInputPinSet(inputSet);
       
		Activity inputSetActivity = null;
		//for local Process and LoopNode with multiple InputPinSets, create Assign activity		
		//Activity terminationInitActivity = getTerminationActivity(inputSet, true);
		Activity terminationInitActivity = null;
		if ((inputSet.getAction() instanceof LoopNode || BomUtils.isLocalProcess(inputSet.getAction())) &&
				inputSet.getAction().getInputPinSet().size() > 1) {
			terminationInitActivity = getTerminationActivityForLocalMultiSet(inputSet, true);
		} else {	// code prior to changes
			terminationInitActivity = getTerminationActivity(inputSet, true);		
		}

		if (resultList.isEmpty()|| !(resultList.get(0) instanceof Activity)){
			inputSetActivity = terminationInitActivity;
			
		} else if (resultList.get(0) instanceof Sequence){
			inputSetActivity = (Sequence)resultList.get(0);
//			((Sequence)inputSetActivity).getActivities().add(terminationInitActivity);
			/* We need to over-write the registry entry which says the terminateSetActivity is the 
			 * representative activity for the input set, and instead use the <sequence> */
			PinSetRegistryUtil.registerPinSet(getContext(),inputSet,inputSetActivity);
			
		} else if (resultList.get(0) instanceof Activity){
			inputSetActivity = BPELFactory.eINSTANCE.createSequence();
			((Sequence)inputSetActivity).getActivities().add(resultList.get(0));
//			((Sequence)inputSetActivity).getActivities().add(terminationInitActivity);
			/* We need to over-write the registry entry which says the terminateSetActivity is the 
			 * representative activity for the input set, and instead use the <sequence> */
			PinSetRegistryUtil.registerPinSet(getContext(),inputSet,inputSetActivity);
		}
		
		fPinSetMap.put(inputSet, inputSetActivity);
						
		NameProvider nameProvider = NameProviderFactory.getNameProvider(inputSetActivity);
		String convertedName = nameProvider.getName(inputSetActivity, inputSet, NamingUtil.findRegistry(this));
		inputSetActivity.setName(convertedName);
		
		flow.getActivities().add(inputSetActivity);
		if (resultList.size() > 1){
			getTarget().addAll(resultList.subList(1, resultList.size()));
		}
		
		setTargetOfLinks(inputSet, inputSetActivity);
	   
		/* Create and execute one link rule for every input set in this process node
		* that is fulfilled by only a StartNode.   */
		createInitialNodeLinks(inputSet);
	   
		//pin in the input set is connected to an input pin on the inside of the SAN, 
		//it is necessary to create a BPEL representaiton of this connection.
		//check each pin in the input set	  
		Pin pin;
//		boolean doAssign = (BomHelper.getInstance().isNextSetCompatible(inputSet, true) == null
//							? true
//							: false);
		
		boolean doAssign = false;
		List pinList = BomUtils.getPinsInSet(inputSet);
		for (int i = 0, count = pinList.size(); i < count; i++) {
			pin = (Pin) pinList.get(i);
			
			/* Ignore pins that have no outgoing connection */
			if (pin.getOutgoing() == null){
			   continue;
			}

			if (pin.getOutgoing().getTarget() instanceof TerminationNode) {
				//LinkRule linkRule = createTerminationLinkRule(pin);
				LinkRule linkRule = createTerminationLinkRule(pin,inputSet);
				getChildRules().add(linkRule);
				linkRule.transformSource2Target();

			} else if (pin.getOutgoing().getTarget() instanceof Pin) {

				if ((pin instanceof ObjectPin) && doAssign) {
	
					//create assign rule for only ObjectPin
					AssignRule ar = AbstractbpelFactory.eINSTANCE.createAssignRule();
					ar.getSource().add(pin.getOutgoing()); //edge
					ar.getSource().add(inputSet); //input set
					ar.getSource().add(BomUtils.findTargetSet(pin.getOutgoing()));
	
					//input set on the target side of the edge
					//add Assign rule as child rule
					getChildRules().add(ar);
					ar.transformSource2Target();
	
				} else { //create link rule for control pin and object pin in not compatible set
					LinkRule lr = AbstractbpelFactory.eINSTANCE.createLinkRule();
					lr.getSource().add(pin.getOutgoing());
	
					//set the outgoing edge as the source
					lr.getSource().add(pin); //set the input pin as the source
					lr.getSource().add(pin.getOutgoing().getTarget());
	
					//input pin on the target side of the edge
					lr.getSource().add(inputSet); //set on source side of the edge
					lr.getSource().add(BomUtils.findTargetSet(pin.getOutgoing()));
	
					//input set on target side of the edge
					//add link rule as child rule
					getChildRules().add(lr);
					lr.transformSource2Target();
				}
			}	
		}
		setSourceOfLinks(inputSet, inputSetActivity);			
	}
	
	/**
	 * Connect the SAN OutputPinSet empty with internal links and external links including:
	 * 1. create Empty for each outputSet if doesn't exist.
	 * 2. set the Empty as the target of the links that are connected to the OutputPinSet 
	 *    from internal activities of the SAN.
	 * 3. if this SAN is not a top level SAN, set Empty as the source of the links 
	 *    that are connected from the OutputPinSet to external activities of the SAN 
	 * @return
	 */
	private void doOutputPinSetConnection(Flow flow){

		// Handle the outputPinSets that are correlated to an inputPinSet
		InputPinSet inputSet = (InputPinSet)getSource().get(0); 
		List osList = inputSet.getOutputPinSet();
		//List osList = BomUtils.getCorrelatedPinSets(inputSet);
		
		OutputPinSet os;
	
		for(int i=0, count = osList.size(); i<count; i++){
			os = (OutputPinSet) osList.get(i);
			doOutputPinSetConnection(flow,inputSet,os);
		}		
		
		// Handle those non-correlated OutputPinSets
		List nonCorrelatedOutputPinSets = getNonCorrelatedOutputPinSets(inputSet);
		for(int i=0, count = nonCorrelatedOutputPinSets.size(); i<count; i++){
			os = (OutputPinSet) nonCorrelatedOutputPinSets.get(i);
			doOutputPinSetConnection(flow,inputSet,os);
		}
	}

	private void doOutputPinSetConnection(Flow flow, InputPinSet is, OutputPinSet os) {	
		List resultList = null;
		Activity outputSetActivity = null;
		Activity terminationSetActivity = null;
		
		resultList = mapOutputPinSet(os);
			
		terminationSetActivity = getTerminationActivity(os, false);
		
		//Retain the end activity for GFlow where applicable
		if(CycleLinkUtil.keepTerminationSetActivity(terminationSetActivity, os, getContext())){
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(), terminationSetActivity);
		}
		
		if (resultList.isEmpty()){
			JavaNCNameConverter converter = new JavaNCNameConverter();

			//set the lastActivity for OutputPinSet
			if (os.getAction() instanceof LoopNode || (BomUtils.isLocalProcess(os.getAction()) && 
														os.getAction().getOutputPinSet().size() > 1)) {			//is.getOutputPinSet().size() > 1)) {
				NamingRegistry registry = NamingUtil.findRegistry(this);
				outputSetActivity = ScopeRegistryUtil.getBpelActivity(getContext(), os);
				
				if (outputSetActivity == null) {
					if (os.getIsException() != null && os.getIsException().booleanValue()) {
						Throw throwAct = BPELFactory.eINSTANCE.createThrow();
						String activityName = BomUtils.getCanonicalName((NamedElement)os.eContainer());
						String convertedName = converter.convertName(registry, throwAct, activityName);
						throwAct.setName(convertedName);
						createDisplayName(throwAct, os.getName());
						outputSetActivity = throwAct;
					
						//set the fault Name and Variable
						BPELVariable bpelVariable = ScopeRegistryUtil.getBpelVariable(getContext(), os);
						String pinSetName = BomUtils.getCanonicalName(os);
						String osConvertedName = converter.convertName(registry, throwAct, pinSetName);
						String encodedNamespace = null;
						if (bpelVariable != null) {
							throwAct.setFaultVariable(bpelVariable);
							
							if (bpelVariable.getXSDElement() != null) {
								encodedNamespace = bpelVariable.getXSDElement().getTargetNamespace();
							} else if (bpelVariable.getType() != null) {
								encodedNamespace = bpelVariable.getType().getTargetNamespace();
							}

							QName qname = new QName(encodedNamespace, osConvertedName + ExportOperationConstants.FAULT_SUFFIX);
							throwAct.setFaultName(qname);
						} else { // no variable
							ProcessInterfaceRule interfaceRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext(), os.getAction());
							if (interfaceRule != null && !interfaceRule.getTarget().isEmpty() && interfaceRule.getTarget().get(0) instanceof Definition) {
								Definition def = (Definition)interfaceRule.getTarget().get(0);
								encodedNamespace = def.getTargetNamespace();
							} else {
								AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_TARGET_NAMESPACE);
								valueProvider.setContext(this.getContext());
								encodedNamespace = (String)valueProvider.getAttributeValueForType(throwAct, os.getAction(), AttributeNameConstants.WSDL_TARGET_NAMESPACE, NamingUtil.findRegistry(this));
							}
							QName qname = new QName(encodedNamespace, osConvertedName + ExportOperationConstants.FAULT_SUFFIX);
							throwAct.setFaultName(qname);							
						}
					} else if (BomUtils.hasMoreThanOneRegularOutputSet(os.getAction())){
						Assign assignAct = BPELFactory.eINSTANCE.createAssign();
						String activityName = BomUtils.getCanonicalName((NamedElement)os.eContainer());
						String convertedName = converter.convertName(registry, assignAct, activityName);
						assignAct.setName(convertedName);
						createDisplayName(assignAct, os.getName());
						ScopeRegistryUtil.createCopyForPinSetsINContainer(assignAct, os, getContext());				
						outputSetActivity = assignAct;
					} else {
						outputSetActivity = terminationSetActivity;
					}
					//register for future use
					ScopeRegistryUtil.registerNamedElement(getContext(), os, outputSetActivity);
				}
			}
			else{
				outputSetActivity = terminationSetActivity;			
			}			
		} else if (resultList.get(0) instanceof Sequence){
			outputSetActivity = (Activity)resultList.get(0);
			((Sequence)outputSetActivity).getActivities().add(terminationSetActivity);			
			
			/* We need to over-write the registry entry which says the terminateSetActivity is the 
			 * representative activity for the output set, and instead use the <sequence> */
			PinSetRegistryUtil.registerPinSet(getContext(),os,outputSetActivity,((InputPinSet)getSource().get(0)));

		} else if (resultList.get(0) instanceof Activity){
			outputSetActivity = BPELFactory.eINSTANCE.createSequence();
			((Sequence)outputSetActivity).getActivities().add(resultList.get(0));
			((Sequence)outputSetActivity).getActivities().add(terminationSetActivity);			

			/* We need to over-write the registry entry which says the terminateSetActivity is the 
			 * representative activity for the output set, and instead use the <sequence> */
			PinSetRegistryUtil.registerPinSet(getContext(),os,outputSetActivity,((InputPinSet)getSource().get(0)));
			
		} else {
			outputSetActivity = terminationSetActivity; // defect 128664
		}
		
		fPinSetMap.put(os, outputSetActivity);
		if (!flow.getActivities().contains(outputSetActivity)){
			flow.getActivities().add(outputSetActivity);
		}				

		NameProvider nameProvider = NameProviderFactory.getNameProvider(outputSetActivity);
		String convertedName = nameProvider.getName(outputSetActivity, os, NamingUtil.findRegistry(this));

//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String convertedName = converter.convertName(NamingUtil.findRegistry(this), outputSetActivity, BomUtils.getCanonicalName(os));
	
		outputSetActivity.setName(convertedName);

		setTargetOfLinks(os, outputSetActivity);
			
		//connect external link except top level process
		if(!BomUtils.isTopLevelProcess(is.getAction())) {
			//set source for links from the scope 
			if ((BomUtils.isLocalProcess(os.getAction()) || os.getAction() instanceof LoopNode) && 
//					is.getOutputPinSet().size() > 1 &&
					os.getAction().getOutputPinSet().size() > 1 &&
					getParentRule() instanceof SANNestedProcessRule &&
					getParentRule().getTarget().size() > 0 &&
					getParentRule().getTarget().get(0) instanceof Scope) {
				setSourceOfLinks(os, (Scope)getParentRule().getTarget().get(0));	
				//inject logic to add the fault sources for the scope
				optimizeFaultLinkForScope(os, (Scope)getParentRule().getTarget().get(0));
			} else { // code used prior to cross links clean-up
				setSourceOfLinks(os, outputSetActivity);
			}
		}

	}
	
	/**
	 * This method is responsible for optimizing the fault links associated with the scope which represent the sub-process
	 * or while loop
	 * If the fault output set includes multiple pins, we will create an assign activity to assign the wrapper variable parts
	 * to individual variables leading to subsequent activities
	 * @param os The exceptional output pinset of the sub-process or while loop
	 * @param scope The scope activity generated to represent the sub-process or while loop
	 */
	@SuppressWarnings("unchecked")
	private void optimizeFaultLinkForScope(OutputPinSet os, Scope scope) {	
		if(CycleLinkUtil.existFaultSource(getContext(), os,scope)){
			return;
		}
		if(os == null || scope == null){
			return;
		}
		QName faultName = null;
		Variable variable = null;
		Link faultLink = null;
		Activity throwActivity = ScopeRegistryUtil.getBpelActivity(getContext(), os);	
		if(throwActivity instanceof Throw){
			Throw theThrow = (Throw)throwActivity;
			faultName = theThrow.getFaultName();
			variable = theThrow.getFaultVariable();
			
			if(variable instanceof BPELVariable){
				BPELVariable bpelVar = (BPELVariable)variable;
				if(bpelVar.getXSDElement() != null && CBPELUtil.shouldGenerateForkActivity(os)){							
					Assign genAssign = CBPELUtil.createAssign(getContext(), os);
					//create a name for the assign
					JavaNCNameConverter converter = new JavaNCNameConverter();
					String convertedName = converter.convertName(NamingUtil.findRegistry(this), genAssign, BomUtils.getCanonicalName(os)+"GeneratedAssign");
					genAssign.setName(convertedName);
					
					//Create the fault link
					faultLink = linkActivities(scope, 
								genAssign, 
							   os.getAction().getName() + "-to-" + BomUtils.getCanonicalName(os));	
					
					//propagate to the parent
					getTarget().add(faultLink);
					
					//Find out the OS associated linkRules and their targets and reset the source of the links
					setSourceOfLinks(os,genAssign);
					
					//Retain the generated assign not to be optimized in the optimization rule
					BpelOptimizationUtil.registerNonOptimizableElement(getContext(), genAssign);
					
					//Propagate to the parent
					getTarget().add(genAssign);
				}else{			
					//retrieve fault link for the registry
					if(!CBPELUtil.handleParallelLinks(os)){
						faultLink = CBPELUtil.getFautlLinkForScope(getContext(),os);
					}else{
						//defect 91127
						faultLink = injectEmptyActivity(os, scope);
					}
				}
			}
		}
		if(faultName != null && faultLink != null){
			//create FaultSources on the scope			
			FaultSource faultSource = BPELPlusFactory.eINSTANCE.createFaultSource();	
			Catch eachCatch = BPELPlusFactory.eINSTANCE.createCatch();
			eachCatch.setFaultName(faultName);
			if(variable != null){
//				eachCatch.setFaultVariable(variable.getName());
				eachCatch.setFaultVariable(variable);
			}
			faultSource.setLink(faultLink);
			faultSource.setCatch(eachCatch);	
			
			Sources sources = CBPELUtil.getSources(scope);
//			if(faultSources  != null){				
//				faultSources.getFaultSource().add(faultSource);			
//			}
			if(sources != null){
				sources.getChildren().add(faultSource);
			}
			
			//Remove the fault link from the regular sources
//			if(scope.getSources() != null&& !scope.getSources().getChildren().isEmpty()){
//				for(Object eachSource:scope.getSources().getChildren()){
//					if(eachSource instanceof Source){
//						Source  theSource = (Source)eachSource;
//						if(faultLink == theSource.getLink()){
//							scope.getSources().getChildren().remove(theSource);
//							break;
//						}
//					}
//				}
//			}
		}		
	}
/**
 * Empty activity injection logic for the case that a subprocess with exceptional pin set and there is 
 * a parallel link between a regular and a fault link on the generated scope which represents the subprocess
 * @param os
 * @param scope
 * @return
 */
	private Link injectEmptyActivity(OutputPinSet os, Scope scope) {
		Link faultLink = null;
		Empty genEmpty = BPELFactory.eINSTANCE.createEmpty();
		//create a name for the empty activity
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), genEmpty, BomUtils.getCanonicalName(os)+"GeneratedEmpty");
		genEmpty.setName(convertedName);

		//Create the fault link
		faultLink = linkActivities(scope, 
				genEmpty, 
				os.getAction().getName() + "-to-" + BomUtils.getCanonicalName(os));	

		//propagate to the parent
		getTarget().add(faultLink);

		//Find out the OS associated linkRules and their targets and reset the source of the links
		setSourceOfLinks(os,genEmpty);

		//Retain the generated assign not to be optimized in the optimization rule
		BpelOptimizationUtil.registerNonOptimizableElement(getContext(), genEmpty);

		//Propagate to the parent
		getTarget().add(genEmpty);			

		//We need to wire the empty to the process output
		List pinList = BomUtils.getPinsInSet(os);
		if(pinList != null && !pinList.isEmpty()){
			Pin pin = (Pin)pinList.get(0);
			
			if(pin.getOutgoing().getTarget()instanceof TerminationNode){
				
				TerminationNode tn = (TerminationNode)pin.getOutgoing().getTarget();
				
				if(tn.getOutcome() instanceof OutputPinSet){
					
					//create a link between the injected empty and the process output pinset termination activity
					OutputPinSet ops = (OutputPinSet)tn.getOutcome();
					
					//Ensure it's top level process
					if(BomUtils.isProcess(ops.getAction())&& !BomUtils.isLocalProcess(ops.getAction())){
						Link tempLink = linkActivities(genEmpty, 
								//This is the mapped empty activity for the top level process' output pin set
								getTerminationActivity(ops, false),
								genEmpty.getName() + "-to-" + BomUtils.getCanonicalName(ops));
						
						//Propagate this target
						getTarget().add(tempLink);
					}
				}
			}
		}
		return faultLink;
	}

	private List getNonCorrelatedOutputPinSets(InputPinSet ips) {
		Action action = ips.getAction();
		List outpinsets = new ArrayList();
		OutputPinSet ops;
		for (Iterator it=action.getOutputPinSet().iterator(); it.hasNext();) {
			ops = (OutputPinSet)it.next();
			if (ops.getInputPinSet().size()==0) {
				outpinsets.add(ops);
			}
		}
		return outpinsets;
	}
	
	/**
	 * Create and execute container rules for the variables associcated with the StructuredActivityNode.
	 *
	 */
	protected void transformBOMVariable(){
		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);	
		List vList = ((StructuredActivityNode)inputSet.getAction()).getVariable();		
		com.ibm.btools.bom.model.processes.activities.Variable v;
		//ContainerRule cr;
		VariableRule cr;
		for(int i= 0, count=vList.size(); i<count; i++){
			v = (com.ibm.btools.bom.model.processes.activities.Variable)vList.get(i);
			BPELVariable variable = createVariable(v);
		}
	}
	
	/**
	 * Creates a variable that will be used to record whether or not this SAN has been
	 * terminated (i.e. whether a TerminationNode in the SAN has been reached).
	 *
	 */
	protected void transformTerminationNode(){
		
		if (BpelOptimizationUtil.bDoTerminationLogic) {
			InputPinSet inputSet = (InputPinSet)getSource().get(0);

			/* Only create the termination node variable if it hasn't already been done */
			ContainerRule cr = (ContainerRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), ContainerRule.class, inputSet.getAction());

			if (cr != null){
				return;
			}
		
			/* The easist way to do this is to create a BOM Variable with the proper name
			 * and assign it the Boolean predefined type.  Then the variable will be created
			 * properly, even though the BOM Variable never actually existed in the model.
			 */
			com.ibm.btools.bom.model.processes.activities.Variable v;
			// Ray: should create PrimitiveType instance instead
//			com.ibm.btools.bom.model.artifacts.Class booleanTypeClass;
			PrimitiveType booleanTypeClass;
		
			/* TODO: Use the BTDataTypeManager to create a new instance of BTBoolean as the type, when it is ready */
			InformationModel primitiveTypesModel = ModelsFactoryImpl.eINSTANCE.createInformationModel();
			//primitiveTypesModel.setName("PrimitiveTypesModel");
			primitiveTypesModel.setName(PredefTypesConst.ROOT_INFO_MODEL_NAME);
			//booleanTypeClass = ArtifactsFactoryImpl.eINSTANCE.createClass();
			// Ray: should create PrimitiveType instance instead
			booleanTypeClass = ArtifactsFactoryImpl.eINSTANCE.createPrimitiveType();
			booleanTypeClass.setName("Boolean");
			primitiveTypesModel.getOwnedMember().add(booleanTypeClass);
		
			v = ActivitiesFactoryImpl.eINSTANCE.createVariable();
			v.setType(booleanTypeClass);
		
				
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), v, inputSet.getAction().getName() + "Termination");
			v.setName(convertedName);
			//v.setName(inputSet.getAction().getName() + "Termination");
//			v.setScope((StructuredActivityNode)inputSet.getAction());
		
			LiteralInteger lowerBound = ArtifactsFactoryImpl.eINSTANCE.createLiteralInteger();
			LiteralUnlimitedNatural upperBound = ArtifactsFactoryImpl.eINSTANCE.createLiteralUnlimitedNatural();
		
			lowerBound.setValue(new Integer(1));
			upperBound.setValue("1");
		
			v.setLowerBound(lowerBound);
			v.setUpperBound(upperBound);

			/* Create and execute the container rule.  If we're here, it hasn't already been created */
			cr = AbstractbpelFactory.eINSTANCE.createContainerRule();
			cr.getSource().add(v);
			/* Also add the action as a source, so we can later find the termination variable by 
			 * looking for a ContainerRule that has the given action as a source*/
			cr.getSource().add(inputSet.getAction());

			getChildRules().add(cr);
			cr.transformSource2Target();
		
				
			convertedName = converter.convertName(NamingUtil.findRegistry(this), ((BPELVariable)cr.getTarget().get(0)), BomUtils.getCanonicalName(inputSet.getAction(), false, true) + "Termination");
			((BPELVariable)cr.getTarget().get(0)).setName(convertedName);
			//((Variable)cr.getTarget().get(0)).setName(BomUtils.getCanonicalName(inputSet.getAction(), false, true) + "Termination");
		}

	}
	
	
	/**
	 * Creates the links from the input set of a SAN to the input sets that are connected to by
	 * only an InitialNode.
	 *  
	 * @param inputSet The input set on the SAN to link to the Initial Node-initialized tasks 
	 */
	protected void createInitialNodeLinks(InputPinSet inputSet){
		List result = new ArrayList();
		
		/* Find the SAN that contains inputSet... 
		 * We will use that SAN to find the associated Start Nodes */
		StructuredActivityNode san = (StructuredActivityNode)inputSet.getAction();
		
		/* Find all start nodes associated with this input set, or with no input set at all */
		List initialNodeList = BomUtils.getInitialNodesForSet(inputSet);

		for (int i = initialNodeList.size()-1; i>=0; i--){
			if (!isOnlyInitiator( (InitialNode)initialNodeList.get(i) )){
				initialNodeList.remove(initialNodeList.get(i));
			 }			
		}
		
		/* For each start node, create and execute a LinkRule and collect the Link objects */
		InitialNode iNode = null;
		LinkRule newLinkRule = null;
		Pin pin = null;
		for (Iterator it = initialNodeList.iterator(); it.hasNext(); ){
			iNode = (InitialNode)it.next();
			
			pin = (Pin)iNode.getOutgoing().getTarget();
			
			newLinkRule = AbstractbpelFactory.eINSTANCE.createLinkRule();
			newLinkRule.getSource().add(inputSet);
			newLinkRule.getSource().add(iNode);
			newLinkRule.getSource().add(iNode.getOutgoing());
			newLinkRule.getSource().add(pin);

			/* The pin could be an output pin if the start node is connected to the output set */
			if (pin instanceof InputControlPin){
				newLinkRule.getSource().addAll(BomUtils.getInputPinSets(pin));	
			} else if (pin instanceof OutputControlPin){
				newLinkRule.getSource().addAll(BomUtils.getOutputPinSets(pin));
			}
			
			/* Execute the new link rule */
			getChildRules().add(newLinkRule);
			newLinkRule.transformSource2Target();
		
		}
		
	}
			
	/**
	 * Returns true if the given InitialNode is connected to an input set that consists
	 * of control pins  (all of which are connected to by InitialNodes), or object pins
	 * that don't have an incoming connection (such as RetrieveArtifactPins)
	 *  
	 * @param initialNode The initial node that is used to initiate an action 
	 * @return boolean - True if only initial nodes are initiators, false otherwise
	 */			 
	public boolean isOnlyInitiator(InitialNode initialNode){
		
		ActivityEdge edge = initialNode.getOutgoing();
		if (edge == null){
			return false;
		}
		
		PinSet pinSet = (PinSet)BomUtils.findTargetSet(edge);
		if (pinSet == null){
			return false;
		}
		
//		if (pinSet instanceof InputPinSet){
//			
//			InputObjectPin oPin = null;
//			for (Iterator it=((InputPinSet)pinSet).getInputObjectPin().iterator(); it.hasNext(); ){
//				oPin = (InputObjectPin)it.next();
////				if (!(oPin instanceof RetrieveArtifactPin) || oPin.getIncoming() != null){
//				//Ray testing
//				if (!(oPin instanceof InputValuePin) && !(oPin instanceof RetrieveArtifactPin) && oPin.getIncoming() != null){
//					/* There is at least one object pin that is has an incoming connection */
//					return false;		
//				}
//			}
//	
//			InputControlPin cPin = null;
//			for (Iterator it=((InputPinSet)pinSet).getInputControlPin().iterator(); it.hasNext(); ){
//				cPin = (InputControlPin)it.next();
//				
//				//if (cPin.getIncoming() == null || !(cPin.getIncoming().getSource() instanceof InitialNode)){
//				if (cPin.getIncoming() != null && !(cPin.getIncoming().getSource() instanceof InitialNode)){ // Ray
//					/* There is at least one control pin that is not connected to an initial node */
//					return false;		
//				}
//			}
//		}  else if (pinSet instanceof OutputPinSet){
//			if (((OutputPinSet)pinSet).getOutputObjectPin().size() != 0){
//				/* There are more than just control pins in this set */
//				return false;
//			}
//	
//			OutputControlPin cPin = null;
//			for (Iterator it=((OutputPinSet)pinSet).getOutputControlPin().iterator(); it.hasNext(); ){
//				cPin = (OutputControlPin)it.next();
//				
//				if (cPin.getIncoming() == null || !(cPin.getIncoming().getSource() instanceof InitialNode)){
//					/* There is at least one control pin that is not connected to an initial node */
//					return false;		
//				}
//			}
//		} 	
		/* If we've made it this far, the intialNode is either connected to the only 
		 * pin in the pin set, or all other pins are also connected to by initial nodes. */
		return true;
			
	}

	/**
	 * Creates the Activity required to initialize / set the termination node variable for
	 * the SAN that contains the given pin set.  Both BPEL and BPEL+ versions are created, using
	 * the AlternativeActivity
	 * @param pinSet The pin set on the SAN for which to initialize / set the termination onde
	 * @return Assign The BPEL Assign activity that initializes / sets the termination node variable
	 */
	protected Activity getTerminationActivity(PinSet pinSet, boolean forInit){
		
		Activity termAct;
		termAct=PinSetRegistryUtil.getBpelActivity(getContext(),pinSet);
		if (termAct != null) {
			return termAct;
		}
		if (BpelOptimizationUtil.bDoTerminationLogic) {
			/* Find the SAN that contains pinSet... 
			 * We will use that SAN to find the associated BPEL Variable */
			StructuredActivityNode san = (StructuredActivityNode)pinSet.eContainer();
		
			ContainerRule termVarRule = (ContainerRule)		
				TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
													ContainerRule.class,
													san);

			com.ibm.wbit.bpel.BPELVariable termVar = (com.ibm.wbit.bpel.BPELVariable)termVarRule.getTarget().get(0);

			/* A termination variable always has only one part */
			Part termVarPart = (Part)termVar.getMessageType().getEParts().get(0);

			/* We need to initalize / set the variable with the constant expression 'FALSE' / 'TRUE'.  
			 * But because BPEL+ doesn't support XPath, we need to create an AlternativeActivity so that 
			 * we have both a BPEL and a BPEL+ version of the <assign> */
			AlternativeActivity altActivity = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlt = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			altActivity.getAlternative().add(bpelAlt);
			altActivity.getAlternative().add(bpelpAlt);
		
			bpelAlt.setType(ProcessFlowType.BPEL_LITERAL);
			bpelpAlt.setType(ProcessFlowType.BPELPP_LITERAL);

			String activityName = null;
			if (forInit){
				bpelAlt.setActivity(createTerminationInitAssign(termVar, termVarPart));
				bpelpAlt.setActivity(createTerminationInitJava(termVar, termVarPart));	
				activityName = BomUtils.getCanonicalName((NamedElement)pinSet.eContainer()) 
								+ "TerminationInitialization";
			} else {
				bpelAlt.setActivity(createTerminationSetAssign(termVar, termVarPart));
				bpelpAlt.setActivity(createTerminationSetJava(termVar, termVarPart));				
				activityName = BomUtils.getCanonicalName((NamedElement)pinSet.eContainer()) 
								+ "Termination";
			}

			// Check to see if it's a valid name by calling NameConverter
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), altActivity, activityName);
			altActivity.setName(convertedName);
			termAct = altActivity;
			//return altActivity;			
		} else {
			// If we don't need to implement the terminationNode logic, just create an empty activity
			Activity emptyAct = BPELFactory.eINSTANCE.createEmpty();
			String activityName = BomUtils.getCanonicalName((NamedElement)pinSet.eContainer());
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), emptyAct, activityName);
			emptyAct.setName(convertedName);
			termAct = emptyAct;			
		}
		if (pinSet instanceof InputPinSet) {
			PinSetRegistryUtil.registerPinSet(getContext(),pinSet,termAct);
		} else {
			PinSetRegistryUtil.registerPinSet(getContext(),pinSet,termAct,((InputPinSet)getSource().get(0)));
		}
		createDisplayName(termAct,pinSet.getName());
		return termAct;
	}
			
	/**
	 * Creates the Activity required to initialize the input pin set variables for the local SAN (process/loop).
	 * @param inputPinSet The pin set on the SAN for which to initialize the variable
	 * @return Assign The BPEL Assign activity
	 */
	protected Activity getTerminationActivityForLocalMultiSet(InputPinSet pinSet, boolean forInit){
		Activity termAct = PinSetRegistryUtil.getBpelActivity(getContext(),pinSet);
		if (termAct != null) {
			return termAct;
		}
		Assign assignAct = BPELFactory.eINSTANCE.createAssign();
		String activityName = BomUtils.getCanonicalName((NamedElement)pinSet.eContainer());
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = converter.convertName(NamingUtil.findRegistry(this), assignAct, activityName);
		assignAct.setName(convertedName);
		termAct = assignAct;
		PinSetRegistryUtil.registerPinSet(getContext(), pinSet, termAct);
		createDisplayName(termAct, pinSet.getName());
		// find and set Variable created for this pin set
		ScopeRegistryUtil.createCopyForPinSetsINContainer(assignAct, pinSet, getContext());

		return termAct;
	}

	
	/**
	 * Create the <assign> version of the termination variable initialization assignment.
	 * @param termVar The BPEL Variable that represents the termination node to initialize
	 * @param termVarPart The BPEL Variable part the contains the termination value
	 * @return Assign - The BPEL Assign that initialize the termination variable
	 */
	private Assign createTerminationInitAssign(com.ibm.wbit.bpel.BPELVariable termVar, Part termVarPart){
		/* Create the BPEL Assign that we will use to set the value of the termination variable */
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		/* Create an Expression Model to initialize the termination variable with 'FALSE' */
		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
		BooleanLiteralExpression strExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
		strExpr.setBooleanSymbol(Boolean.FALSE);
		bpelExpr.setExpression(strExpr);
		from.addExtensibilityElement(bpelExpr);
		
		/* Do we need to use a ToExtension object? No, because the expression is on the <from>, which
		 * is already capable of havine extensibility elements built in */
		to.setVariable(termVar);
		to.setPart(termVarPart);
		
		copy.setFrom(from);
		copy.setTo(to);
		assign.getCopy().add(copy);
		
		return assign;
		
	}
	
	/**
	 * Create the JavaScriptActivity version of the termination variable initialization assignment.
	 * @param termVar The BPEL Variable that represents the termination node to initialize
	 * @param termVarPart The BPEL Variable part the contains the termination value
	 * @return Invoke - The BPEL Invoke that houses the java code to initialize the variable
	 */
	private Invoke createTerminationInitJava(com.ibm.wbit.bpel.BPELVariable termVar, Part termVarPart){
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity javaScriptActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();

		/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */
		String varName = AbstractbpelUtil.capitalizeFirstChar(termVar.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(termVarPart.getName());			
		String javaString = AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true).set" + partName + "(false);"; 
//		javaCode.setValue(javaString);
//		javaScriptActivity.setJavaCode(javaCode);
		javaScriptActivity.setJavaCode(javaString);
		javaInvoke.getEExtensibilityElements().add(javaScriptActivity);
		
		return javaInvoke;
	}


	/**
	 * Creates the BPEL version  of the assign to set the value of the termination node
	 * variable to true
	 * 
	 * @param termVar The termination node variable to set the value of 
	 * @param termVarPart The part of the termination node variable to set the value of
	 * @return Assign - The BPEL Assign that sets the termination value
	 */
	private Assign createTerminationSetAssign(BPELVariable termVar, Part termVarPart){
		/* Create the BPEL Assign that we will use to set the value of the termination variable */
		Assign assign = BPELFactory.eINSTANCE.createAssign();
		Copy copy = BPELFactory.eINSTANCE.createCopy();
		From from = BPELFactory.eINSTANCE.createFrom();
		To to = BPELFactory.eINSTANCE.createTo();
		
		/* Create an expression model to assign the value TRUE to the terminationNode */
		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
		BooleanLiteralExpression strExpr = ModelFactory.eINSTANCE.createBooleanLiteralExpression();
		strExpr.setBooleanSymbol(Boolean.TRUE);
		bpelExpr.setExpression(strExpr);
		from.addExtensibilityElement(bpelExpr);
		
		to.setVariable(termVar);
		to.setPart(termVarPart);
		
		copy.setFrom(from);
		copy.setTo(to);
		assign.getCopy().add(copy);
		
		return assign;
		
	}
	
	/**
	 * Creates the BPEL Invoke that contains a JavaScriptActivity that updates the value of the 
	 * termination node variable to TRUE.
	 * 
	 * @param termVar The termination node variable to update
	 * @param termVarPart The part of the termination node variable to update.
	 * @return Invoke - the BPEL Invoke the contains a JavaScriptActivity as an extensible element, 
	 * which in turn contains a JavaCode element that contains the actual java code!
	 */
	private Invoke createTerminationSetJava(BPELVariable termVar, Part termVarPart){
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaString = new String();
		
		/* TODO: Do we have to use eGet(...) and eSet(...) calls to be generic and avoid casting? */
		/* TODO: NOTE: Assuming that we have to captilize the first character of the part and var name */
		String varName = AbstractbpelUtil.capitalizeFirstChar(termVar.getName());
		String partName = AbstractbpelUtil.capitalizeFirstChar(termVarPart.getName());	
		javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true).set" + partName + "(true);";
		
//		javaCode.setValue(javaString);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaString);
		javaInvoke.getExtensibilityElements().add(jsActivity);
		
		return javaInvoke;
	}
			
	/**
	 * Returns the branch BPEL flow
	 * @return Flow - the branch flow
	 */
	protected Flow getBranchFlow() {
		return ivBranchFlow;
	}

	/**
	 * Sets the branch BPEL flow
	 * @param flow The branch flow
	 */
	private void setBranchFlow(Flow flow) {
		ivBranchFlow = flow;
	}
	
	private List getPathAddedVariables(List currentPathVariablesSnapshot, List variablesOnCurrentPath){
		//the actual variables on CurrentPath
	//	List variablesOnCurrentPath = ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext());
		
//		int initialSize = currentPathVariablesSnapshot.size();
//		int finalSize = variablesOnCurrentPath.size();
		List addedVariables = new LinkedList();
//		if(finalSize > initialSize){
//			for(int j= initialSize; j< finalSize; j++){
//				addedVariables.add(variablesOnCurrentPath.get(j));
//			}
//		}
		if(variablesOnCurrentPath != null){
			Iterator it = variablesOnCurrentPath.iterator();
			while(it.hasNext()){
				Object var = it.next();
				if(!currentPathVariablesSnapshot.contains(var) && !addedVariables.contains(var)){
					addedVariables.add(var);
				}
			}
		}

		return addedVariables;
		
	}

	/**
	 * Go through all nodes in the san and get a list of InputPinSet that doesn't 
	 * have any connected pins (pins that has an incoming edge)
	 * @param san
	 * @param ips
	 * @return
	 */
	private List getNonConnectedInputPinSet(StructuredActivityNode san, InputPinSet ips) {
		List nonConnectedInputPinSet = new ArrayList();
		for (Iterator it=san.getNodeContents().iterator();it.hasNext();) {
			Element cnode = (Element) it.next();
			if (cnode instanceof Action) {
				Action action = (Action)cnode;
				List ipss = action.getInputPinSet();
				for (Iterator it2=ipss.iterator();it2.hasNext();) {
					InputPinSet aips = (InputPinSet)it2.next();
					boolean bNotConnected=true;
					List ops = aips.getInputObjectPin();
					for (Iterator it3=ops.iterator();it3.hasNext() && bNotConnected;) {
						InputObjectPin iop = (InputObjectPin)it3.next();
						if (iop.getIncoming()!=null) {
							bNotConnected=false;
						}
					}
					if (bNotConnected) {
						List cps = aips.getInputControlPin();
						for (Iterator it3=cps.iterator();it3.hasNext() && bNotConnected;) {
							InputControlPin icp = (InputControlPin)it3.next();
							if (icp.getIncoming()!=null) {
								bNotConnected=false;
							}
						}							
					}
					if (bNotConnected) {
						nonConnectedInputPinSet.add(aips);
						String params[]={aips.getName(),aips.getAction().getName()};
						//LoggingUtil.logError(MessageKeys.NON_CONNECTED_INPUTPINSET,params,null);						
					}
				}
			}
		}
		return nonConnectedInputPinSet;
	}
	/**
	 * Returns he list of variables created for the concurrent branch point, starting at <code>cbSet</code>
	 * that merges at <code>mergeSet</code>
	 * @param cbSet concurrent branch point, a PinSet
	 * @param mergeSet concurrent merge Point, a PinSet
	 * @return List of BPELVariables.
	 */
	private List getVariablesForCBPoint(PinSet cbSet, PinSet mergeSet){
		List var = new LinkedList();
		//List outObjectPins = os.getOutputObjectPin();
		List outObjectPins = BomUtils.getObjectPinsForSet(cbSet);
		List inputObjectPins = BomUtils.getObjectPinsForSet(mergeSet);
		//List inputObjectPins = is.getInputObjectPin();
		Iterator it = outObjectPins.iterator();
		while(it.hasNext()){
			ObjectPin oop = (ObjectPin)it.next();
			if(oop instanceof OutputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}
			}else if(oop instanceof InputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}
			}
		}
		return var;
	}
	
	/**
	 * Returns the list of variables created for the concurrent branch point, starting at <code>cbSet</code>
	 * that merges at <code>mergeSet</code>
	 * @param cbSet concurrent branch point, a PinSet
	 * @param mergeSet concurrent merge Point, a PinSet
	 * @return List of BPELVariables.
	 */
	private List getVariablesForCBPoint(PinSet cbSet, PinSet mergeSet, boolean included){
		List var = new LinkedList();
		List outVar = new LinkedList();
		//List outObjectPins = os.getOutputObjectPin();
		List outObjectPins = BomUtils.getObjectPinsForSet(cbSet);
		List inputObjectPins = BomUtils.getObjectPinsForSet(mergeSet);
		//List inputObjectPins = is.getInputObjectPin();
		Iterator it = outObjectPins.iterator();
		while(it.hasNext()){
			ObjectPin oop = (ObjectPin)it.next();
			if(oop instanceof OutputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}else{
					outVar.add(createVariable(oop));
				}
			}else if(oop instanceof InputObjectPin && oop.getOutgoing() != null){
				if(inputObjectPins.contains(oop.getOutgoing().getTarget())){
					var.add(createVariable(oop));
				}else{
					outVar.add(createVariable(oop));
				}
			}
		}
		if(included){
			return var;
		}else{
			return outVar;
		}
	}
	
	/**
	 * Creates variables that will be used for entry/exit set for local process and loop
	 *
	 */
	protected List createEntryExitVariables(JavaNCNameConverter converter){
		InputPinSet inputSet = (InputPinSet)getSource().get(0);

		// check whether variables were created for this container
		if (ScopeRegistryUtil.hasVariable(getContext(), inputSet)) {
			return null;
		}
		
		Action action = inputSet.getAction();
		ArrayList variables = null;
		if (action instanceof LoopNode || BomUtils.isLocalProcess(action)) {
			NamingRegistry nameRegistry = NamingUtil.findRegistry(this);
			variables = new ArrayList();
			if (action.getInputPinSet().size() > 1) {
				Iterator <InputPinSet> it = action.getInputPinSet().iterator();
				InputPinSet iSet = null;
				while (it.hasNext()) {
					iSet = it.next();
					variables.add(ScopeRegistryUtil.createBooleanVariable(iSet, getContext(), nameRegistry, converter));
				}
			}
			boolean hasMultipleRegularOS = BomUtils.hasMoreThanOneRegularOutputSet(action);
			Iterator <OutputPinSet> it = action.getOutputPinSet().iterator();
			OutputPinSet oSet = null;
			while (it.hasNext()) {
				oSet = it.next();
				if ((oSet.getIsException() != null && oSet.getIsException().booleanValue())) {
					if (oSet.getOutputObjectPin().isEmpty()) {
						continue;
					} else if (oSet.getOutputObjectPin().size() == 1) {
						OutputObjectPin ooPin = (OutputObjectPin) oSet.getOutputObjectPin().get(0);
						BPELVariable bpelVariable = ScopeRegistryUtil.getBpelVariable(getContext(), oSet);
						if (bpelVariable == null) {
							TransformationRule defRule = ProcessUtil.getProcessDefinitionRule(getContext());
							ArrayList sources = new ArrayList();
							sources.add(ooPin.getType());
							sources.add(ooPin);
							TransformationRule rule = TransformationUtil.getRuleForSource(defRule, VariableRule.class, sources, BPELVariable.class);
							//if a rule was already created for this Pin, use the variable and add the Pin as source of this rule 
							if(rule != null && !rule.getTarget().isEmpty()){
								bpelVariable = (BPELVariable)rule.getTarget().get(0);
							}
						}
						
						if (bpelVariable == null) {
							if (ooPin.getIncoming() == null && ooPin.getOutgoing() == null) {
								bpelVariable = createVariable(ooPin);
								if (bpelVariable != null) {
									variables.add(bpelVariable);
								}
							} else if (ooPin.getIncoming() != null) {	// source pin inside the container
								ObjectPin sourcePin = (ObjectPin)(ooPin.getIncoming().getSource());
								bpelVariable = createVariable(sourcePin);
							} else {	// target pin inside the container
								ObjectPin targetPin = (ObjectPin)(ooPin.getOutgoing().getTarget());
								bpelVariable = createVariable(targetPin);
							}
						}
						if (bpelVariable != null) {
							ScopeRegistryUtil.registerNamedElement(getContext(), oSet, bpelVariable);
						}
					} else { // for exception pin set with multiple pins create wrapper variable
						BPELVariable bpelVariable = ScopeRegistryUtil.getBpelVariable(getContext(), oSet);
						if (bpelVariable == null) {
							List sources = new ArrayList();
							sources.add(oSet);
							TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), ElementDeclarationRule.class, sources, XSDElementDeclaration.class);
							if (rule != null && rule instanceof ElementDeclarationRule &&
									!rule.getTarget().isEmpty() && rule.getTarget().get(0) instanceof XSDElementDeclaration) {
								XSDElementDeclaration xsdElement = (XSDElementDeclaration)rule.getTarget().get(0);
								bpelVariable = ScopeRegistryUtil.createVariableForExceptionPinSet(oSet, getContext(), nameRegistry, xsdElement, converter);
								if (bpelVariable != null) {
									variables.add(bpelVariable);
									organizeImports(bpelVariable);
									ScopeRegistryUtil.registerNamedElement(getContext(), oSet, bpelVariable);
								}
							}
						}
					}
				} else if (oSet.getIsStream() != null && oSet.getIsStream().booleanValue()) {
					continue;					
				} else if (hasMultipleRegularOS) {			
					variables.add(ScopeRegistryUtil.createBooleanVariable(oSet, getContext(), NamingUtil.findRegistry(this), converter));
				}
			}	
		}
		return variables;
	}
		
	// organize imports for BPEL
	private void organizeImports(BPELVariable variable){
		ProcessDefinitionRule currentProcessDefRule= (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(this.getContext());
		Process process = (Process)currentProcessDefRule.getTarget().get(0);
		XSDSchema xsdSchema = null;
		if (variable.getType() != null) {
		    xsdSchema = variable.getType().getSchema();
		} else if (variable.getXSDElement() != null) {
		    xsdSchema = variable.getXSDElement().getSchema();
		}

		if (xsdSchema == null || xsdSchema.getTargetNamespace() == null) {
			return;
		}
		String xsdNamespace = xsdSchema.getTargetNamespace();
		if (xsdSchema.eContainer() instanceof XSDSchemaExtensibilityElement) {
			XSDSchemaExtensibilityElement xsdExtElement = (XSDSchemaExtensibilityElement)xsdSchema.eContainer();
			if (xsdExtElement != null && xsdExtElement.eContainer() instanceof Types) {
				Definition toBeImportedDef = ((Types)xsdExtElement.eContainer()).getEnclosingDefinition();
				if(toBeImportedDef != null && !xsdNamespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) && 
						!containsImport(process, xsdNamespace)){
					com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
					wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
					wsdlImport.setNamespace(xsdNamespace);
					wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
					process.getImports().add(wsdlImport);
				}
			}
		}
	}

	/**
	 * Checks is the Process already contains a wsdl import for the <code>targetNamespace</code> 
	 * @param process
 	 * @param targetNamespace
	 * @return boolean
	 */
	private boolean containsImport(Process process, String targetNamespace){
		if (process == null || targetNamespace == null) {
			return false;
		}
		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && 
						thisImport.getNamespace().equals(targetNamespace)){
					return true;
				}
			}
		}
		return false;
	}

} //ProcessRuleImpl
