/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.TransformationSession;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.OutputRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANReusableProcessRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.StaffActivityRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.PinSetRegistryUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil.StaffConstants;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Correlation;
import com.ibm.wbit.bpel.CorrelationPattern;
import com.ibm.wbit.bpel.CorrelationSet;
import com.ibm.wbit.bpel.Correlations;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Initiate;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.OnMessage;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Receive;
import com.ibm.wbit.bpel.Reply;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Input;
import com.ibm.wbit.bpelpp.Output;
import com.ibm.wbit.bpelpp.Parameter;
import com.ibm.wbit.bpelpp.Parameters;
import com.ibm.wbit.bpelpp.Staff;
import com.ibm.wbit.bpelpp.Task;
import com.ibm.wbit.tel.TPotentialInstanceCreator;
import com.ibm.wbit.tel.TPotentialStarter;
import com.ibm.wbit.tel.TStaffSettings;
import com.ibm.wbit.tel.TTask;
import com.ibm.wbit.tel.TUISettings;
import com.ibm.wbit.tel.TVerb;
import com.ibm.wbit.tel.TaskFactory;
import com.ibm.wbit.tel.util.CustomTaskFactory;
import com.ibm.wbit.tel.util.TaskResourceImpl;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * 
 * Creates a BPEL Sequence as the top-level BPEL structured activity
 * that represents the top-level BOM StructuredActivityNode.  It creates
 * BPEL recieve or pick (in the case of multiple inputsets) and reply
 * for the Sequence.  Rules for Partner, PartnerLink, and PortType representing
 * the top-level SAN will be created and invoked.
 * 
 * Source: BOM StructuredActivityNode
 * Target: BPEL Sequence
 * Target Propagated: BPEL Variables, Partners
 */
public class SANReusableProcessRuleImpl	extends ActionRuleImpl implements SANReusableProcessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private BomUtils.PinSetAnalyzer ipsa=null;
	private static final String WSDL_NAMESPACE_PREFIX = "wsdl"; 

	// capture a map of input pin set --> startable BPEL activity.  
	// All non-startable activity must be preceed by a startable activity
	private Map fStartableActivities= new HashMap(); 
	private Map fNonStartableActivities = new HashMap();
	private Map fLinks = new HashMap();

	/* The map of output pin set --> activity created to handle the response back to the caller */
	private Map fResponseActivities = new HashMap();
	
	private com.ibm.btools.bom.model.processes.activities.Activity interfaceActivity = null;



	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANReusableProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.SAN_REUSABLE_PROCESS_RULE;
	}

	private Flow flow; // bpel process body
	private StructuredActivityNode san; // bom process body
	private ProcessInterfaceRule processIntRule;
	private ProcessDefinitionRule processDefRule;
	private CorrelationSet processCorrelationSet=null; // correlation set for the process

	/**
	 * source: BOM StructuredActivityNode
	 * target: BPEL flow
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		// this activity rule is for activity with SAN
		boolean ret=false;
		san = (StructuredActivityNode)getSource().get(0);	
		//Cache the interface activity of the process if there is any
		interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny(san);
			
		// Create Flow
		flow=BPELFactory.eINSTANCE.createFlow();

		//generate name for flow
		String convertedName = NamingUtil.getFlowName(flow, this);
		flow.setName(convertedName);
		createDisplayName(flow, convertedName);

		getTarget().add(flow);

		// Create Process Interface and Definition  
		if(interfaceActivity != null){
			processIntRule = createProcessInterfaceRule(interfaceActivity.getImplementation());
		}else{
			processIntRule = createProcessInterfaceRule(san);
		}		
		processDefRule = (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext());

		// Handle different cases for InputPinSets of a SAN:
		ipsa = new BomUtils.PinSetAnalyzer(san);
		// Case 1: A list of InputPinSets that has no correlation predicate defined
		List ipsNoCP = ipsa.getInputPinSetsWithNoCorrelationPredicate();
		// Case 2: A list of InputPinSets that has correlation predicated defined and noCorrelationMatches=createNewInstance
		List ipsCPCreateNewInstance = ipsa.getInputPinSetsWithCorrelationPredicateCreateNewInstance();
		// If there exists an inputPinSet with noCorrelationMatches=createNewInstance, then need to set BPEL initiate flag of the correlationSet to 'rendezvous'
		//		String initiateCorrelation;
		//		if (ipsCPCreateNewInstance.size()==0) {
		//			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES;
		//		} else {
		//			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_RENDEZVOUS;						
		//		}
		String initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES;
		// Case 3: A list of inputPinSets that has correlation predicated defined and noCorrelationMatches!=createNewInstance
		//This will be handled by validation against WPS export, only case 2 is the right input delivery option based on the current runtime support
		//defect 89371 89373
		List ipsCP = ipsa.getInputPinSetsWithCorrelationPredicate();

		// Handle case 1
		// If there is only one InputPinSet that has no correlation predicate defined, then create a single recieve.  Otherwise, a pick will be created
		if (ipsNoCP.size()==1) {
			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES;
			fStartableActivities.put(ipsNoCP.get(0), createSingleReceive((InputPinSet)ipsNoCP.get(0),true,initiateCorrelation));
		} else if (ipsNoCP.size()>0) {
			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES;
			fStartableActivities.put(ipsNoCP, createPick(ipsNoCP,true,initiateCorrelation));
		}

		// Handle case 2
		if (ipsCPCreateNewInstance.size()>0) {
			InputPinSet inputPinSet;
			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_RENDEZVOUS;
			for (Iterator it=ipsCPCreateNewInstance.iterator();it.hasNext();) {
				inputPinSet=(InputPinSet)it.next();
				fStartableActivities.put(inputPinSet, createSingleReceive(inputPinSet,true,initiateCorrelation));
			}
		}

		// Handle case 3
		if (ipsCP.size()>0) {
			InputPinSet inputPinSet;
//			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_NO;	
			initiateCorrelation=AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES;
			for (Iterator it=ipsCP.iterator();it.hasNext();) {
				inputPinSet=(InputPinSet)it.next();
//				fNonStartableActivities.put(inputPinSet, createSingleReceive(inputPinSet,false,AbstractbpelUtil.BPEL_CORRELATION_INITIATE_NO));
				fStartableActivities.put(inputPinSet, createSingleReceive(inputPinSet,true,initiateCorrelation));
			}
		}

		// Make sure the non-startable activities are preced by startable activities
		Activity nonStartableActivity,startableActivity;
		for (Iterator nsa=fNonStartableActivities.values().iterator();nsa.hasNext();) {
			nonStartableActivity = (Activity)nsa.next();
			for (Iterator sa=fStartableActivities.values().iterator();sa.hasNext();) {
				startableActivity = (Activity)sa.next();
				addLinkToFlow(flow,connectActivities(startableActivity,nonStartableActivity));
			}
		}

		setComplete(true);
		processChildTargets();

		executeHandlers();

		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();	    
	}

	/**
	 * Returns the map of pinset(s) of activities that when activated may result in a new instance of the
	 * process being created, assuming they do not correlate to an existing instance.  
	 * Each key is either a pin set or a list of pin sets.  Each value is either 
	 * a <receive> or <pick> activity.
	 * 
	 * @return The map of pin sets to activities that can start the process
	 */
	public Map getStartableActivities(){
		return fStartableActivities;
	}

	/**
	 * Returns the map of pinset(s) of activities that when activated may result in a new instance of the
	 * process being created, assuming they do not correlate to an existing instance.  
	 * Each key is either a pin set or a list of pin sets.  Each value is either 
	 * a <receive> or <pick> activity.
	 * 
	 * @return The map of pin sets to activities that can start the process
	 */
	public Map getLinks(){
		return fLinks;
	}

	/**
	 * Returns the map of pin sets of entry-point activities that when activated will not result in a new instance of the
	 * process being created.  Each key is a pin set.  Each value is a <receive> activity.
	 * 
	 * @return The map of pin sets to entry-point activities that can not start the process
	 */
	public Map getNonStartableActivities(){
		return fNonStartableActivities;
	}

	/**
	 * Returns map list of pin sets to activities that represent the possible responses from this
	 * process back to the caller.  Keys of the map consist of output pin sets, and 
	 * values consist of <reply> or <invoke> activities.
	 * 
	 * @return The map of pin sets to response activities created for this process
	 */
	public Map getResponseActivities(){
		return fResponseActivities;
	}

	private CorrelationSet getProcessCorrelationSet() {
		if (processCorrelationSet == null) {
			processCorrelationSet = BPELFactory.eINSTANCE.createCorrelationSet();
			com.ibm.wbit.bpel.Process procDef = (com.ibm.wbit.bpel.Process)ProcessUtil.getProcessDefinition(getContext());
			if (procDef != null) {
				processCorrelationSet.setName(procDef.getName());
			} else {
				processCorrelationSet.setName("processCorrelationSet");
			}
		}
		return processCorrelationSet;
	}


	private void transformInputPinSet(InputPinSet inputSet, ExtensibleElement inputSetActivity, Parameters inputContainer, String initateCorrelation) {
		// Create PortType for the process
		TransformationRule portTypeRule = createPortTypeRule(processIntRule,san);	
		//Record the process ips
		InputPinSet processIPS = inputSet;
		// Create SCDL Component
		PortType processPT = (PortType)portTypeRule.getTarget().get(0);
		DocumentRoot componentRoot =  createSCDLComponent(san,processPT);
		ProcessUtil.putProcessComponent(getContext(),componentRoot.getComponent());
		// Create SCDL Export
		DocumentRoot exportRoot = createSCDLExport(componentRoot,processPT);
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(BpelOptions.isBestPracticePatternEnabed(exportSession) && BomUtils.isTopLevelProcess(san)){
			Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
			if(docRootRegistry instanceof Set){
				((Set)docRootRegistry).add(componentRoot);
				((Set)docRootRegistry).add(exportRoot);
			}			
		}

		// Walk thru and transform the process flow starting from the InputPinSet
		ProcessRule pr = AbstractbpelFactory.eINSTANCE.createProcessRule();
		pr.getSource().add(inputSet);
		pr.getSource().add(flow);
		getChildRules().add(pr);
		pr.transformSource2Target();
        
		//Remove callback interfaces from the calling process export since we split the exports 
		ScdlUtil.consolidateTargetExportInterfaces(componentRoot.getComponent(), exportRoot.getExport());

		// Create PartnerLinkType and PartnerLink 
		OperationAnalyzer oa = new OperationAnalyzer(ipsa);
		int operationType = oa.getOperationType(inputSet);
		PartnerLinkRule partnerLinkRule;
		//AR new 29.04.. the currentProcessIntRule is the Process Interface Rule that will contain collect all PartnerLinks
		//in the end, on save, a separate WSDL the "Artifacts" WSDL will be created for the PartnerLinks.
		ProcessInterfaceRule currentProcessIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext());;

		//if(operationType==oa.ONE_WAY_OPERATION_WITH_CALLBACK){
		if (portTypeRule.getTarget().size()>1) {
			partnerLinkRule = createPartnerLinkRule(currentProcessIntRule,san,(PortType)portTypeRule.getTarget().get(0),(PortType)portTypeRule.getTarget().get(1));
		} else {
			partnerLinkRule = createPartnerLinkRule(currentProcessIntRule,san,(PortType)portTypeRule.getTarget().get(0), this);
		}
		PartnerLink partner = ProcessUtil.getProcessPartner(getContext(),san);
		if (partner == null) {
			PartnerRule partnerRule = AbstractbpelFactory.eINSTANCE.createPartnerRule();
			partnerRule.getSource().add(san);
			getChildRules().add(partnerRule);
			partnerRule.transformSource2Target();
			partner = (PartnerLink)partnerRule.getTarget().get(0);
			ProcessUtil.putProcessPartner(getContext(),san,partner);
			getTarget().addAll(partnerRule.getTarget());
		}

		// create receive and reply
		// find operation based on inputset and related outputset (if exists)
		Vector sources = new Vector();
		com.ibm.btools.bom.model.processes.activities.Activity interfaceActivity = null;
		if(FabricIntegrationUtil.getInterfaceActivityIfAny(san) != null){
			interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny(san);
		}
		if(interfaceActivity != null){
			inputSet = (InputPinSet)interfaceActivity.getImplementation().getInputPinSet().get(0);
			sources.add(inputSet);		
		}else{
			if(operationType != oa.TWO_WAY_OPERATION){
				sources.add(inputSet);
			} else {
				sources.add(inputSet);
				sources.add(inputSet.getOutputPinSet().get(0));
			}	
		}
		//Make the return type as generic rule since this operation rule may be an external service operation
		TransformationRule opRule =
					TransformationUtil.getRuleForSource(
					ProcessUtil.getProcessInterfaceRule(getContext()),
					sources,
					Operation.class);
		Operation op = (Operation) opRule.getTarget().get(0);

		DisplayName recName = BPELPlusFactory.eINSTANCE.createDisplayName();			
		if (inputSetActivity instanceof Receive) {		
			Receive receive = (Receive)inputSetActivity;

			AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
			valueProvider.setContext(this.getContext());
//			String convertedName = (String)valueProvider.getAttributeValueForType(receive, inputSet, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));
			//defect 85258 The receive should take the process activity name as the receive name
			String convertedName = (String)valueProvider.getAttributeValueForType(receive, processIPS, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));

			receive.setName(convertedName);
			receive.setOperation(op);
			receive.setPortType((PortType) op.eContainer());
			receive.getExtensibilityElements().add(inputContainer);
			receive.setPartnerLink(partner);				
//			createDisplayName(receive, inputSet, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
			//defect 85258 The receive should take the process activity name as the receive name
			createDisplayName(receive, processIPS, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
		} else {
			OnMessage receive = (OnMessage)inputSetActivity;
			AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
			receive.setOperation(op);
			receive.setPortType((PortType) op.eContainer());
			receive.getExtensibilityElements().add(inputContainer);
			receive.setPartnerLink(partner);	
			recName.setText(inputSet.getAction().getName()+" "+inputSet.getName());
		}

		CorrelationSet cSetForReceive = null;		
		if (operationType==oa.ONE_WAY_OPERATION_WITH_CALLBACK || 
				(operationType==oa.TWO_WAY_OPERATION && (ipsa.getInputPinSetsWithCorrelationPredicate().size()>0 || ipsa.getInputPinSetsWithCorrelationPredicateCreateNewInstance().size()>0)) ||
				ipsa.getInputPinSetsWithCorrelationPredicate().contains(inputSet) ||
				ipsa.getInputPinSetsWithCorrelationPredicateCreateNewInstance().contains(inputSet) ||
				(operationType==oa.ONE_WAY_OPERATION && (ipsa.getInputPinSetsWithCorrelationPredicate().size()>0 || ipsa.getInputPinSetsWithCorrelationPredicateCreateNewInstance().size()>0))
		){
			//if (operationType!=oa.TWO_WAY_OPERATION || ipsa.getInputPinSetsWithCorrelationPredicate().size()>0 || ipsa.getInputPinSetsWithCorrelationPredicateCreateNewInstance().size()>0) {
			// If 1-way longrunning process is used, need to create empty correlation set
			cSetForReceive = getProcessCorrelationSet();
			getTarget().add(cSetForReceive);
			Correlation cor = BPELFactory.eINSTANCE.createCorrelation();
			cor.setSet(cSetForReceive);
			//migration 11.09
			//cor.setInitiate(initateCorrelation?Boolean.TRUE:Boolean.FALSE);
			//cor.setInitiate(initateCorrelation);
			//defect89373 add support for "join"
			if(AbstractbpelUtil.BPEL_CORRELATION_INITIATE_RENDEZVOUS.equals(initateCorrelation)){
				cor.setInitiate(Initiate.JOIN_LITERAL);
			}else{
				cor.setInitiate(AbstractbpelUtil.BPEL_CORRELATION_INITIATE_YES.equals(initateCorrelation)?Initiate.YES_LITERAL:Initiate.NO_LITERAL);
			}
			Correlations cors;
			if (inputSetActivity instanceof Receive) {		
				Receive receive = (Receive)inputSetActivity;
				cors = receive.getCorrelations();
				if (cors==null) {
					cors=BPELFactory.eINSTANCE.createCorrelations();
					receive.setCorrelations(cors);
				}
			} else {
				OnMessage receive = (OnMessage)inputSetActivity;
				cors = receive.getCorrelations();
				if (cors==null) {
					cors=BPELFactory.eINSTANCE.createCorrelations();
					receive.setCorrelations(cors);
				}				
			}
			cors.getChildren().add(cor);
			LoggingUtil.logWarning(MessageKeys.INCOMPLETE_CORRELATION_SET_WARNING, new String[]{recName.getText()==null?inputSet.getAction().getName():recName.getText()}, null);
		}
		if(!HumanTaskUtil.isUse61HumanTaskSupport() && StaffUtil.isStaffRequired(san)){
			StaffActivityRule staffRule = AbstractbpelFactory.eINSTANCE.createStaffActivityRule();
			staffRule.getSource().addAll(san.getResourceRequirement());
			getChildRules().add(staffRule);
			staffRule.transformSource2Target();
			if(staffRule.getTarget()!= null && !staffRule.getTarget().isEmpty()){
				Staff staff = (Staff)staffRule.getTarget().get(0);
				inputSetActivity.getExtensibilityElements().add(staff);
			}
		}
		propagateChildTargetsOfProcessRule(pr,flow);	
	}	
	private void transformOutputPinSet(InputPinSet inputSet, ExtensibleElement inputSetActivity) {
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName;
		Operation op;
		PartnerLink partner;
		if (inputSetActivity instanceof Receive) {
			partner = ((Receive)inputSetActivity).getPartnerLink();
		} else {
			partner = ((OnMessage)inputSetActivity).getPartnerLink();
		}
		//Variable outputContainer;
		OutputPinSet os;
		Activity outputSetAct;
		OperationAnalyzer oa = new OperationAnalyzer(ipsa);
		int operationType = oa.getOperationType(inputSet);
		if (operationType==oa.TWO_WAY_OPERATION) {
			os=(OutputPinSet)inputSet.getOutputPinSet().get(0); 
			Input output = createInput(os);
			// Create reply
			if (inputSetActivity instanceof Receive) {
				op = ((Receive)inputSetActivity).getOperation();
			} else {
				op = ((OnMessage)inputSetActivity).getOperation();
			}
			Reply reply = BPELFactory.eINSTANCE.createReply();
			fResponseActivities.put(os, reply);		

			AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
			valueProvider.setContext(this.getContext());
			convertedName = (String)valueProvider.getAttributeValueForType(reply, os, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));


			reply.setName(convertedName);
			reply.setOperation(op);
			reply.setPortType((PortType) op.eContainer());
			reply.getExtensibilityElements().add(output);

			//reply.setVariable(outputContainer);
			reply.setPartnerLink(partner);
			createDisplayName(reply, os, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
			flow.getActivities().add(reply);
			// Connect the outputPinSet activity with the reply
			outputSetAct = PinSetRegistryUtil.getBpelActivity(getContext(),os);
			addLinkToFlow(flow,connectActivities(outputSetAct,reply));	
			//Handle fault reply creation
			createFaultReply(inputSet, op, partner);
		} else if (operationType!=oa.ONE_WAY_OPERATION) {

			TransformationRule partnerRule = TransformationUtil.getRuleForTarget(getRoot(), PartnerRule.class, partner);
			if(partnerRule != null && partnerRule.getTarget().size()>1){
				partner = (PartnerLink)partnerRule.getTarget().get(1);
			}
			// Create Callback invoke
			List outputPinSets = PinSetRegistryUtil.getPinSetsForRelatedPinSet(getContext(),inputSet);			
			for (Iterator it=outputPinSets.iterator();it.hasNext();) {
				os = (OutputPinSet)it.next();
				Input input = createInput(os);
				Vector sources = new Vector();
				sources.add(os);
				OperationRule opRule =
					(OperationRule) TransformationUtil.getRuleForSource(
							ProcessUtil.getProcessInterfaceRule(getContext()),
							sources,
							Operation.class);
				if (opRule!=null && !opRule.getTarget().isEmpty()) {
					op = (Operation) opRule.getTarget().get(0);

					Invoke callbackInvoke = BPELFactory.eINSTANCE.createInvoke();
					fResponseActivities.put(os, callbackInvoke);

					AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_ACTIVITY_NAME);
					convertedName = (String)valueProvider.getAttributeValueForType(callbackInvoke, os, AttributeNameConstants.BPEL_ACTIVITY_NAME, NamingUtil.findRegistry(this));
					callbackInvoke.setName(convertedName);
					callbackInvoke.setOperation(op);
					callbackInvoke.setPortType((PortType) op.eContainer());
					callbackInvoke.getEExtensibilityElements().add(input);
					callbackInvoke.setPartnerLink(partner);	
					createDisplayName(callbackInvoke, os, AttributeNameConstants.BPEL_ACTIVITY_DISPLAY_NAME);
					flow.getActivities().add(callbackInvoke);

					// Create empty correlation
					Correlation cor = BPELFactory.eINSTANCE.createCorrelation();
					CorrelationSet cSetForReceive;
					if (inputSetActivity instanceof Receive) {
						cSetForReceive = ((Correlation)((Receive)inputSetActivity).getCorrelations().getChildren().get(0)).getSet();
					} else {
						cSetForReceive = ((Correlation)((OnMessage)inputSetActivity).getCorrelations().getChildren().get(0)).getSet();
					}	
					cor.setSet(cSetForReceive);
					cor.setPattern(CorrelationPattern.OUT_LITERAL);
					Correlations cors = callbackInvoke.getCorrelations();
					if (cors==null) {
						cors=BPELFactory.eINSTANCE.createCorrelations();
						callbackInvoke.setCorrelations(cors);
					}
					cors.getChildren().add(cor);
					// Connect the outputPinSet activity with the callback invoke
					outputSetAct = PinSetRegistryUtil.getBpelActivity(getContext(),os);
					addLinkToFlow(flow,connectActivities(outputSetAct,callbackInvoke));								
				}
			}	
		}		
	}
    
	/**
	 * This function is to handle the creation of fault reply for the global process in case of
	 * exceptional output pin set exists
	 * @param inputSet
	 * @param op
	 * @param partner
	 */
	@SuppressWarnings("unchecked")
	private void createFaultReply(InputPinSet inputSet, Operation op,PartnerLink partner) {
		if(inputSet == null || op == null || partner == null){
			return;
		}
		if(inputSet.getOutputPinSet().size() > 1){
			for(Object eachOS:inputSet.getOutputPinSet()){
				OutputPinSet ops = (OutputPinSet)eachOS;
				if(ops.getIsException() != null && ops.getIsException().booleanValue()){
					if(BomUtils.isTopLevelProcess(ops.getAction())){
						//Handle top level process fault output pin set														
						Reply replyAct = BPELFactory.eINSTANCE.createReply();
						//d89909 - Incorrect mapping of exception OutputPinSet
						fResponseActivities.put(ops, replyAct);
						createDisplayName(replyAct, ops.getName());		
						
						QName qname = null;	
						MessageRule msgRule = null;
						OutputPinSet interfaceFaultOPS = null;
						
						if(interfaceActivity != null){
							//we need to use the interface output pin set to look up the <code>MessageRule</code>							
							InputPinSet interfaceIPS = (InputPinSet)interfaceActivity.getImplementation().getInputPinSet().get(0);
							if(interfaceIPS.getOutputPinSet().size() > 1){
								for(Object eachInterfaceOPS:interfaceIPS.getOutputPinSet()){
									OutputPinSet eachIntOPS = (OutputPinSet)eachInterfaceOPS;
								if(eachIntOPS.getName().equals(ops.getName())&& eachIntOPS.getIsException() != null && eachIntOPS.getIsException().booleanValue()){
										interfaceFaultOPS = eachIntOPS;
										break;
									}
								}
							}
					}
						if(interfaceFaultOPS != null){
							msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext(),interfaceFaultOPS.getAction()), MessageRule.class, interfaceFaultOPS);
						}else{
							msgRule = (MessageRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext(),ops.getAction()), MessageRule.class, ops);
						}						
						if(msgRule != null && msgRule.getTarget() != null && !msgRule.getTarget().isEmpty()){
							Message msg = (Message)msgRule.getTarget().get(0);
							qname = msg.getQName();
						}
				        
						//Look-up the fault on the operation
						Fault targetFault = null;
						if(op.getEFaults() != null && !op.getEFaults().isEmpty()){
							for(Object eachFault:op.getEFaults()){
								if(eachFault instanceof Fault){
									Fault fault = (Fault)eachFault;
									if(fault.getMessage() != null && fault.getMessage().getQName() != null){
										QName instQName = fault.getMessage().getQName();
										if(qname != null && instQName != null){
											if(qname.getLocalPart().equals(instQName.getLocalPart()) && qname.getNamespaceURI().equals(instQName.getNamespaceURI())){
												targetFault = fault;
												break;
											}
										}
									}
								}
							}
						}
						
						//Assemble QName for the operation's fault, where enclosing portType namespace uri is utilized
						QName faultQName = null;
						if(targetFault != null){
							String literalFaultName = targetFault.getName();
							if(op.eContainer() instanceof PortType){
								PortType pt = (PortType) op.eContainer();
								faultQName = new QName(pt.getQName().getNamespaceURI(),literalFaultName);
							}
						}
						
						replyAct.setOperation(op);
						replyAct.setPortType((PortType) op.eContainer());						
						replyAct.setPartnerLink(partner);	
						if(faultQName != null){
							replyAct.setFaultName(faultQName);	
						}

						NameProvider nameProvider = NameProviderFactory.getNameProvider(replyAct);
						String convertedNameReply = nameProvider.getName(replyAct, ops, NamingUtil.findRegistry(this));
						replyAct.setName(convertedNameReply);	
						
						Input faultInput = createInput(ops);
						replyAct.getExtensibilityElements().add(faultInput);
						
						if(!flow.getActivities().contains(replyAct)){
							flow.getActivities().add(replyAct);
						}
						//connect the fault output set activity to the fault reply
						Activity faultoutputSetAct = PinSetRegistryUtil.getBpelActivity(getContext(),ops);
						addLinkToFlow(flow,connectActivities(faultoutputSetAct,replyAct));	
					}		
				}
			}		
		}
	}

	private Pick createPick(List inputPinSets, boolean createInstance, String initiateCorrelation) {

		// Create Pick
		Pick pick = BPELFactory.eINSTANCE.createPick();
		pick.setCreateInstance(createInstance?Boolean.TRUE:Boolean.FALSE);
		NameProvider nameProvider = NameProviderFactory.getNameProvider(pick);
		String convertedName = nameProvider.getName(Activity.class, san, NamingUtil.findRegistry(this));
		pick.setName(convertedName);
		// Add display name support
		createDisplayName(pick,san.getName());
		flow.getActivities().add(pick);

		InputPinSet inputSet;
		BPELVariable var,tmpVar;
		OnMessage onMsg;
		int operationType;
		List currentPathVariables = ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(getContext());
		List previousVars=new ArrayList();
		previousVars.addAll(currentPathVariables);
		boolean reuseVar;
		OutputRule cr;
		for (Iterator it=inputPinSets.iterator();it.hasNext();){	
			inputSet = (InputPinSet)it.next();

			cr = AbstractbpelFactory.eINSTANCE.createOutputRule();
			getChildRules().add(cr);
			cr.getSource().add(inputSet);
			cr.transformSource2Target();
			Output input = (Output)cr.getTarget().get(0);

			List pinSetVariables = getVariablesForPinSet(input);
			List assignedPerSet = new LinkedList();
			List tempVars = new LinkedList();
			// Attempt to reuse the same variables on the pick
			reuseVar=false;

			for(int i=0; i<input.getParameter().size(); i++){
				Parameter thisParameter = (Parameter)input.getParameter().get(i);
				var = (BPELVariable)thisParameter.getVariable();

				for (Iterator it2=previousVars.iterator();it2.hasNext();) {
					tmpVar = (BPELVariable)it2.next();
					if (tmpVar!=var && tmpVar.getType() == var.getType() && !assignedPerSet.contains(tmpVar)) {
						thisParameter.setVariable(tmpVar);
						assignedPerSet.add(tmpVar);
						//change the variable rule target
						VariableRule rule = (VariableRule)TransformationUtil.getRuleForTarget(getRoot(), VariableRule.class, var);
						rule.getTarget().add(0,tmpVar);
						rule.getTarget().remove(var);
						cr.getTarget().remove(var);
						cr.getTarget().add(tmpVar);
						ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(getContext()).remove(var);
						//						PMR54206,999,616
						var = tmpVar;
						break;
					}else{
						//try to find a variable in the current path.. 
						//get the Variable Rule that created this variable
						List fobiddenVariables =  ProcessUtil.getParallelFlowsRegisteredVariables(getContext());
						for(int j=0; j<currentPathVariables.size(); j++){
							BPELVariable thisVariable = (BPELVariable)currentPathVariables.get(j);
							if(thisVariable != var && thisVariable.getType() == var.getType() && !fobiddenVariables.contains(thisVariable)&& !assignedPerSet.contains(thisVariable)){
								thisParameter.setVariable(thisVariable);
								assignedPerSet.add(thisVariable);
								//change the variable rule target
								VariableRule rule = (VariableRule)TransformationUtil.getRuleForTarget(getRoot(), VariableRule.class, var);
								rule.getTarget().add(0,thisVariable);
								rule.getTarget().remove(var);
								cr.getTarget().remove(var);
								cr.getTarget().add(thisVariable);
								ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(getContext()).remove(var);
								//								PMR54206,999,616
								var = tmpVar;
								break;
							}
						}
						tempVars.add(var);
					}

				}
				if(previousVars.size() == 0){
					previousVars.add(var);
					//assignedPerSet.add(var);
				}
				//defect 148755 , populate assignedPerSet list
				//defect 163860 regressional
				if(!assignedPerSet.contains(var)){
					assignedPerSet.add(var);
				}
				previousVars.addAll(tempVars);
			}

			onMsg = BPELFactory.eINSTANCE.createOnMessage();
			createInitiatingHumanTask(inputSet, onMsg);
			transformInputPinSet(inputSet, onMsg, input, initiateCorrelation);
			Activity inputSetAct = PinSetRegistryUtil.getBpelActivity(getContext(),inputSet);
			if (inputSetAct != null) {
				onMsg.setActivity(inputSetAct);
				if (ProcessUtil.manyLinksRequired(inputSet)) {
					//retrieve activity from container
					BpelOptimizationUtil.registerNonOptimizableElement(getContext(), inputSetAct);
				}
			}
			pick.getMessages().add(onMsg);
			//add for ruleHandler mapping
			fStartableActivities.put(inputSet, onMsg);
			transformOutputPinSet(inputSet,onMsg);
		}

		return pick;
	}


	private List getVariablesForPinSet(Output output){
		List vars = new LinkedList();
		if(output.getParameter() != null && !output.getParameter().isEmpty()){
			for (Iterator it2=output.getParameter().iterator();it2.hasNext();) {
				vars.add(((Parameter)it2.next()).getVariable());
			}
		}
		return vars;
	}

	private Receive createSingleReceive(InputPinSet inputSet, boolean createInstance, String initiateCorrelation) {

		Receive receive = BPELFactory.eINSTANCE.createReceive();
		receive.setCreateInstance(createInstance?Boolean.TRUE:Boolean.FALSE);	
		flow.getActivities().add(receive);
		//we need to register the variables for the parallel flows here. In case there is only one receive created, this will not do anything. else. 
		//this will register all the variables created in by the ProcessRule of the previous Receive in the registry as "forbidden" variables
		List variables = getVariablesForSet(inputSet);
		List forbiddenVariables = new LinkedList();
		forbiddenVariables.addAll(ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext()));
		forbiddenVariables.removeAll(variables);
		ProcessUtil.registerVariableForParallelFlow(this.getContext(), forbiddenVariables, false);
		//ProcessUtil.registerVariableForParallelFlow(this.getContext(), ProcessUtil.getCurrentPathParallelFlowsRegisteredVariables(this.getContext()), false);

		// Create Input Variable
		Output inputContainer = createOutput(inputSet);

		transformInputPinSet(inputSet,receive,inputContainer,initiateCorrelation);

		// Connect the receive with the inputPinSet activity
		Activity inputSetAct = PinSetRegistryUtil.getBpelActivity(getContext(),inputSet);
		addLinkToFlow(flow,connectActivities(receive,inputSetAct));			

		// Create the reply or callbacks
		transformOutputPinSet(inputSet,receive);	

		createInitiatingHumanTask(inputSet, receive);

		return receive;		
	}


	private void addLinkToFlow(Flow flow, Link link) {	
		if (flow.getLinks()==null)
		{
			Links links = BPELFactory.eINSTANCE.createLinks();
			flow.setLinks(links);
		}
		flow.getLinks().getChildren().add(link);
	}

	private void propagateChildTargetsOfProcessRule(ProcessRule pr, Flow flow) {
		//propagate child target of pr, except the flow and link, add all links to flow
		List prTargets = pr.getTarget();
		for(int i= 1, count= prTargets.size(); i< count; i++){
			Object target = prTargets.get(i);
			if(target instanceof Link)
			{
				addLinkToFlow(flow,(Link)target);
			}
			else	
				getTarget().add(target);
		}			
	}

	private Link connectActivities(Activity srcAct, Activity tarAct) {
		Link link = BPELFactory.eINSTANCE.createLink();
		Source src = BPELFactory.eINSTANCE.createSource();
		Target tar = BPELFactory.eINSTANCE.createTarget();
		link.getSources().add(src);
		link.getTargets().add(tar);
		link.setName(srcAct.getName()+"To"+tarAct.getName());
		Sources sources = srcAct.getSources();
		if(sources == null){
			sources = BPELFactory.eINSTANCE.createSources();
			srcAct.setSources(sources);
		}
		Targets targets = tarAct.getTargets();
		if(targets == null){
			targets = BPELFactory.eINSTANCE.createTargets();
			tarAct.setTargets(targets);
		}

		sources.getChildren().add(src);
		targets.getChildren().add(tar);
		fLinks.put(link, this);
		return link;

	}

	private void processChildTargets() {
		EList childRules = getChildRules();
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()) {
			TransformationRule childRule =	(TransformationRule) childRulesIterator.next();
			EList childTargets = childRule.getTarget();
			for (int i = 0; i < childTargets.size(); i++) {
				EObject childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof Variable || childTarget instanceof DocumentRoot) {
					getTarget().add(childTarget);
				}
			}
		}
	}

	/**
	 * A simple util class to obtain the type of the operation based on inputPinSet 
	 */
	public class OperationAnalyzer {
		public int TWO_WAY_OPERATION=1;
		public int ONE_WAY_OPERATION=2;
		public int ONE_WAY_OPERATION_WITH_CALLBACK=3;
		private BomUtils.PinSetAnalyzer ipsa=null;

		public OperationAnalyzer(BomUtils.PinSetAnalyzer ipsa) {
			this.ipsa = ipsa;
		}
		public OperationAnalyzer(InputPinSet pinSet) {
			ipsa = new BomUtils.PinSetAnalyzer(pinSet.getAction());
		}
		public int getOperationType(InputPinSet pinSet) {
			// Need to find out if a callback is needed or not.  There are the following cases:
			// Case 1: It is a two-way operation.  Thus no callback
			// Case 2: It is a one-way operation that requires callback
			// Case 3: It is a one-way operation that doesn't require callback
			int caseNo=TWO_WAY_OPERATION;
			if (ipsa.getInputPinSetsAsTwoWayOperation().contains(pinSet)) {
				caseNo=TWO_WAY_OPERATION;
			} else {
				if (ipsa.getCallBackOutputPinSets(pinSet).size()>0) {
					caseNo=ONE_WAY_OPERATION_WITH_CALLBACK;
				} else {
					caseNo=ONE_WAY_OPERATION;					
				}
			}			
			return caseNo;			
		}

	}

	private void createInitiatingHumanTask(InputPinSet inputSet, ExtensibleElement bpelActivity){

		if(BomUtils.isTopLevelProcess(inputSet.getAction())){
			//create the initiating HumanTask
			// Create TEL
			TransformationRule portTypeRule = createPortTypeRule(processIntRule,san);	


			PortType portType = (PortType)portTypeRule.getTarget().get(0);
			//Operation operation = (Operation)portType.getOperations().get(0);
			com.ibm.btools.bom.model.processes.activities.Activity interfaceActivity = null;
			InputPinSet interfaceInputPinSet = null;
			if(FabricIntegrationUtil.getInterfaceActivityIfAny(san) != null){
				interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny((StructuredActivityNode)inputSet.getAction());
			}
			if(interfaceActivity != null){
				interfaceInputPinSet = (InputPinSet)interfaceActivity.getImplementation().getInputPinSet().get(0);
			}
			Operation operation = null;
			if(interfaceActivity != null && BomUtils.isServiceOperation(interfaceActivity.getImplementation())){
				if(interfaceInputPinSet != null){
					operation =(Operation)TransformationUtil.getRuleForSource(processIntRule, com.ibm.btools.te.ilm.heuristics.extservice.OperationRule.class, interfaceInputPinSet).getTarget().get(0);
				}else{
					operation =(Operation)TransformationUtil.getRuleForSource(processIntRule, com.ibm.btools.te.ilm.heuristics.extservice.OperationRule.class, inputSet).getTarget().get(0);
				}
			}else{
				if(interfaceInputPinSet != null){
					operation =(Operation)TransformationUtil.getRuleForSource(processIntRule, OperationRule.class, interfaceInputPinSet).getTarget().get(0);
				}else{
					operation =(Operation)TransformationUtil.getRuleForSource(processIntRule, OperationRule.class, inputSet).getTarget().get(0);
				}
			}
			com.ibm.wbit.tel.DocumentRoot humanTaskRoot = createTELFromProcess(inputSet, portType, operation);
			TTask task = humanTaskRoot.getTask();
			TUISettings uiSettings =task.getUiSettings();
			getTarget().add(humanTaskRoot);
			HumanTaskUtil.registerInlineHumanTask(getContext(),humanTaskRoot);

			TransformationSession session = TransformationEngine.getTransformationSession();
			TransformationSessionUtil.setHasHumanTask(session);

			ExportSession exportSession = BpelOptionsUtil.getExportSession();		
			if(BpelOptions.isBestPracticePatternEnabed(exportSession)){
				Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TOP_LEVEL_DOCUMENT_ROOT);
				if(docRootRegistry instanceof Set){
					((Set)docRootRegistry).add(humanTaskRoot);
				}			
			}

			Task adminTask = BPELPlusFactory.eINSTANCE.createTask();
			
//			if(san.getActivity() != null && san.getActivity().getOwnedComment() != null && !san.getActivity().getOwnedComment().isEmpty()){
//				Comment comment = (Comment)san.getActivity().getOwnedComment().get(0);
//				TDescription documentation = TaskFactory.eINSTANCE.createTDescription();
//				documentation.setLocale(task.getDefaultLocale());
//				documentation.setValue(comment.getBody());
//				task.getDescription().add(documentation);
//				
//			}
			
			
			//remove the task Display name. 
			if(task.getDisplayName() != null && !task.getDisplayName().isEmpty()){
				task.getDisplayName().remove(task.getDisplayName().get(0));
			}
			if (task.getDurationUntilDeleted() != null) {
				task.setDurationUntilDeleted(null);
			}

			adminTask.setName(task);
			bpelActivity.getEExtensibilityElements().add(adminTask);
			Process process = (Process)processDefRule.getTarget().get(0);

			// All itel needs to have a name prepended by the process name
			String newTaskName = task.getName();
			
			newTaskName = process.getName()+"_"+newTaskName;
			task.setName(newTaskName);

			organizeImports(process, portType);
			CrossProjectReferenceUtil.registerTranformTargetWithProjectName(inputSet, humanTaskRoot);
		}
	}

	// d159506
	private void organizeImports(Process processDef,Object obj) {
		if(obj instanceof PortType){
			Definition toBeImportedDef = ((PortType)obj).getEnclosingDefinition();
			String namespace = toBeImportedDef.getTargetNamespace();

			//should add wsdl import.. leave it for the one that is saving the resource
			if(!containsImport(processDef, namespace)){
				//we need to import the schema 
				com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
				if (toBeImportedDef.getLocation() != null && ExportOperationUtil.isExternalFile(toBeImportedDef.getLocation())) {
					wsdlImport.setLocation(ExportOperationUtil.replaceConvertedSpaces(toBeImportedDef.getLocation()));
				} else {
					wsdlImport.setLocation(toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT);
				}
				wsdlImport.setNamespace(namespace);
				wsdlImport.setImportType(AbstractbpelConstants.WSDL_IMPORT_TYPE);
				if (!wsdlImport.getLocation().equals(ExportOperation.WSDL_FILE_EXT)) {
					processDef.getImports().add(wsdlImport);
				}
			}
		} else {
			com.ibm.wbit.tel.DocumentRoot docRoot = (com.ibm.wbit.tel.DocumentRoot)obj;
			URI namespaceURI = docRoot.getTask().getTargetNamespace();
			String namespace = namespaceURI.toString();
			if(!namespace.equals(processDef.getTargetNamespace())){
				if(!containsImport(processDef,namespace)){
					//we need to import the tel file 
					com.ibm.wbit.bpel.Import wsdlImport = BPELFactory.eINSTANCE.createImport();
					wsdlImport.setLocation(docRoot.getTask().getName() + ExportOperation.ITEL_FILE_EXT );
					wsdlImport.setNamespace(namespace);
					wsdlImport.setImportType(AbstractbpelConstants.XSD_IMPORT_TYPE);
					if (!wsdlImport.getLocation().equals(ExportOperation.ITEL_FILE_EXT)) {
						processDef.getImports().add(wsdlImport);
					}
				}
			}

		}
	}

	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Process process, String targetNamespace){

		List imports = process.getImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				com.ibm.wbit.bpel.Import thisImport = (com.ibm.wbit.bpel.Import)it.next();
				if(thisImport.getImportType() != null && thisImport.getImportType().equals(AbstractbpelConstants.WSDL_IMPORT_TYPE) && thisImport.getNamespace().equals(targetNamespace)){
					return true;
				}
			}
		}
		return false;
	}

	private com.ibm.wbit.tel.DocumentRoot  createTELFromProcess(InputPinSet inputSet, PortType portType, Operation operation){
		Action action = inputSet.getAction();
		String name = action.getName();
		String namespace =  ProcessUtil.createNamespace(action,true);
		URI uri = URI.createURI(namespace);
		URI relativeWSDLLocation = URI.createURI(portType.getEnclosingDefinition().getQName().getLocalPart() + ExportOperation.WSDL_FILE_EXT);
		//set the TaskResource flag to true. Even if a little early as we do not have a Resource until now, 
		//we have to set the TaskResource flag to true in order to avoid attempting to use the ArtifactLoader.
		TaskResourceImpl.setUseImports(true);
		TTask task = CustomTaskFactory.getInstance().createDefaultOTask(name,uri, portType, operation, relativeWSDLLocation, portType.getQName().getNamespaceURI(), AbstractbpelConstants.WSDL_IMPORT_TYPE, false);
		task.setJndiNameStaffPluginProvider(HumanTaskUtil.JNDI_NAME_STAFF_PLUGIN_PROVIDER_VMM);

		com.ibm.wbit.tel.DocumentRoot documentRoot = TaskFactory.eINSTANCE.createDocumentRoot();
		documentRoot.getXMLNSPrefixMap().put(WSDL_NAMESPACE_PREFIX, portType.getQName().getNamespaceURI());
		documentRoot.setTask(task);
		// get unique name
		name = HumanTaskUtil.getTelName(getContext(),action,task,AttributeNameConstants.TEL_TASK_NAME);
		task.setName(name);
		//consistently set the duration until deleted to null.
		if (task.getDurationUntilDeleted() != null) {
			task.setDurationUntilDeleted(null);
		}
		TVerb verb =TaskFactory.eINSTANCE.createTVerb(); 
		verb.setName(StaffConstants.EVERYBODY);
		TStaffSettings staffSettings = TaskFactory.eINSTANCE.createTStaffSettings();
		TPotentialInstanceCreator potOwner = TaskFactory.eINSTANCE.createTPotentialInstanceCreator();
		potOwner.setVerb(verb);
		staffSettings.setPotentialInstanceCreator(potOwner);

		TVerb verbStarter =TaskFactory.eINSTANCE.createTVerb(); 
		verbStarter.setName(StaffConstants.EVERYBODY);

		TPotentialStarter potStarter = TaskFactory.eINSTANCE.createTPotentialStarter();
		potStarter.setVerb(verbStarter);
		staffSettings.setPotentialStarter(potStarter);

		task.setStaffSettings(staffSettings);

		// Transform UI Setting
		TUISettings uiSettings = HumanTaskUtil.createUISetting(inputSet, documentRoot, this);
		
		task.setUiSettings(uiSettings);
		return documentRoot;

	}
	


} //SANReusableProcessRuleImpl
