package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.SCA;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANTaskRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * <!-- begin-user-doc -->
 * Source: InputPinSet of StructureActivityNode (task)
 * Target: none	
 * Creates and executes other rules to map BOM Task to BPEL constructs.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANTaskRuleImpl extends ActionRuleImpl implements SANTaskRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* attribute indicate whether it is first time execution of this rule */
	  private boolean firstTimeExec = true;	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANTaskRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.SAN_TASK_RULE;
	}

	/**
	 * Creates and executes other rules to map BOM Task to BPEL construct.
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		//assertion
		if (getSource() == null || getSource().isEmpty()) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
			 
		//get source of this rule- input set of the task		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		Action action = inputSet.getAction();
		
  		//check if this is first time execute this rule
		if(!firstTimeExec){
			 InvokeRule invokeRule = (InvokeRule)getChildRules().get(0); 
			 invokeRule.transformSource2Target();
			 LoggingUtil.traceExit(this, "transformSource2Target", "re-execute");
			 return true;		  
		}
   
		firstTimeExec = false;
		BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(action); 
		List osList = psa.getCallBackOutputPinSets(inputSet);	

		InvokeRule ir;
		boolean isOneWayOperation = psa.getInputPinSetsAsOneWayOperation().contains(inputSet);
		if(isOneWayOperation){
			ir = AbstractbpelFactory.eINSTANCE.createInvokeWithCallbackRule();			
		} else {
			ir = AbstractbpelFactory.eINSTANCE.createInvokeRule();
		}
		
		// create child rule - invoke rule
		// add ir as child rule (and make sure we add it as the first child rule)
		getChildRules().add(ir);	
		
		// Create PortType, ServiceLink, and Partner  
		ProcessInterfaceRule processIntRule = createProcessInterfaceRule(action);
		//AR new 29.04.. the currentProcessIntRule is the Process Interface Rule that contains will contain collect all PartnerLinks
		//in the end, on save, a sepparate WSDL the "Artifacts" WSDL will be created for the PartnerLinks.
		ProcessInterfaceRule currentProcessIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(getContext());;
		ProcessDefinitionRule processDefRule = (ProcessDefinitionRule)ProcessUtil.getProcessDefinitionRule(getContext());
		TransformationRule portTypeRule = createPortTypeRule(processIntRule,action);

		Operation op=null;					
		if (isOneWayOperation) {
			op = findOneWayOperation(processIntRule,inputSet);
		} else {
			op = findOperation(processIntRule,inputSet);
		}
		PortType pt = (PortType)portTypeRule.getTarget().get(0);
		
		// Create SCDL Component
		DocumentRoot docRoot = createSCDLComponent(action,pt);
		
		//AZ 20070322 support pattern creation
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(BpelOptions.isBestPracticePatternEnabed(exportSession)&& docRoot != null){
			AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
			Object implementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
			boolean skipModuleExport=false;
			if(implementation instanceof SCA){
				skipModuleExport = true;
			}
			if(!skipModuleExport){
				createSCDLExport(docRoot,pt);
			}
		}
		
		PartnerLinkRule partnerLinkRule;
		if (osList.size()>0) {
			partnerLinkRule = createPartnerLinkRule(currentProcessIntRule,action,pt,(PortType)portTypeRule.getTarget().get(1));
		} else {
			partnerLinkRule = createPartnerLinkRule(currentProcessIntRule,action,pt, this);
		}
		PartnerRule partnerRule = null;
		// Partner Rule will create component reference
		partnerRule = createPartnerRule(processDefRule,action);
		
		// Add the operation to the Component Reference
		ScdlUtil.updateSCDLComponentReference(ProcessUtil.getProcessComponent(getContext()),pt,op);
		
		// Now that we have the partner and operation, set them as
		// sources of the InvokeRule
		ir.getSource().add(inputSet);
		ir.getSource().add(action);
		ir.getSource().add(op);
		ir.getSource().add(partnerRule.getTarget().get(0));
		
		if (osList.size()>0) {
			OutputPinSet ops;
			for (Iterator it=osList.iterator();it.hasNext();) {
				ops = (OutputPinSet)it.next();
				op = findOneWayOperation(processIntRule,ops);
				ir.getSource().add(op);
			}
		}		
		// NOTE: Mapping and connecting the input sets / output sets is done by the InvokeRule
		
		// Execute the invoke rule
		ir.transformSource2Target();			
		
		//propagate all child target
		propagateChildTargets();
		
		if(CBPELUtil.shouldHanleFaultLink(inputSet)){
			CBPELUtil.processFaultSources(getContext(),ir,portTypeRule);
		}
		
		executeHandlers();
		
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}	
	
	private boolean isOneWayOperation(List osList, InputPinSet inputSet){
		boolean isOneWayOperation = false;
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			isOneWayOperation = (osList.size()>0 || inputSet.getOutputPinSet().size()==0);
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			Boolean isOneWayOperationObj  = (Boolean)TechnicalAttributesHelper.getAttributeValue(inputSet, AttributeNameConstants.IS_ONE_WAY_OPERATION);
			if(isOneWayOperationObj == null){
				isOneWayOperation = (osList.size()>0 || inputSet.getOutputPinSet().size()==0);
			}else{
				isOneWayOperation = isOneWayOperationObj.booleanValue();
			}
		}else if(BpelOptionsUtil.runTransformation() || BpelOptionsUtil.runValidation()){
			Boolean isOneWayOperationObj  = (Boolean)TechnicalAttributesHelper.getAttributeValue(inputSet, AttributeNameConstants.IS_ONE_WAY_OPERATION);
			if(isOneWayOperationObj == null){
				isOneWayOperation = (osList.size()>0 || inputSet.getOutputPinSet().size()==0);
			}else{
				isOneWayOperation = isOneWayOperationObj.booleanValue();
			}
		}
		return isOneWayOperation;
	}
	
} //SANTaskRuleImpl
