/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.abstractbpel.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.processes.actions.RepositoryAction;
import com.ibm.btools.bom.model.processes.actions.RetrieveArtifactAction;
import com.ibm.btools.bom.model.processes.actions.StoreArtifactAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelPackage;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ExclusiveBranchRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableAccessRule;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Expression;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpel.Wait;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Variable Access Rule</b></em>'.
 * 
 * <p>This rule is responsible for mapping a BOM Store/RetrieveArtifactPin and 
 * Store/RetrieveArtifactAction, in the case that they reference a variable and not
 * a datstore. </p>
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class VariableAccessRuleImpl extends ConnectableRuleImpl implements VariableAccessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private static final boolean optimizeVariableAccess=true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected VariableAccessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return AbstractbpelPackage.Literals.VARIABLE_ACCESS_RULE;
	}

	/**
	 * source = {StoreArtifactPin || RetrieveArtifactPin || Input set of a Store/RetrieveArtifactAction}
	 * target = { [<sequence>], <sequence>, [<sequence>] }
	 * 
	 * RESTRICTION: Variable access is done at the part level, which means that the entire
	 * part is either read from or written to.  It is not possible to <i>insert</i> a value
	 * into part.
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;
		}
		
		List results = new ArrayList();
		
		com.ibm.btools.bom.model.processes.activities.Variable bomVariable = null;
		
		
		if (getSource().get(0) instanceof StoreArtifactPin){
			Activity target = null;
			StoreArtifactPin pin = (StoreArtifactPin)getSource().get(0);

			/* Check the attributes that are not supported by the TE */
			if (pin.getAtBeginning() != null){
				String params[]={pin.getName(),pin.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
			}
			if (pin.getIsInsert() != null){
				String params[]={pin.getName(),pin.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.VAR_IS_INSERT_NOT_SUPPORTED,params,null);		
			}

			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)pin.getRepository();
			BPELVariable pinVariable  = getBPELVariable(pin);
			BPELVariable repoVariable = getBPELVariable(bomVariable);
			
			/* It is a restriction that pin be in only one set */
			OutputPinSet pinSet = (OutputPinSet)BomUtils.getOutputPinSets(pin).get(0);
		//	if (useVariableForActivity(pinSet,pin,bomVariable)) {
			if (useVariableForActivity(pinVariable, repoVariable)) {
				results.add(getBPELVariable(bomVariable));
			} else {
				String upperBound = 
					AbstractbpelUtil.getLiteralValue((LiteralSpecification)pin.getUpperBound())
									.toString();
				if ("".equals(upperBound)){
					upperBound = "*";
				}
				target = 
					transformVariableStorage(getBPELVariable(pin),
											 pin,
											 false, 	// pin.getAtBeginning().booleanValue(), // not supported
											 false, 	// pin.getIsInsert().booleanValue(), // not supported
											 false,			// There is no overflow value for a pin
											 getBPELVariable(bomVariable),
											 bomVariable);
				JavaNCNameConverter converter = new JavaNCNameConverter();
				String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
				target.setName(convertedName);
				//target.setName(BomUtils.getCanonicalName(pin));	
				addDisplayName(pin,bomVariable,target);	 
				results.add(target);
			}
			
		} else if (getSource().get(0) instanceof RetrieveArtifactPin){
			Activity target = null;
			RetrieveArtifactPin pin = (RetrieveArtifactPin)getSource().get(0);

			/* Check the attributes that are not supported by the TE */
			if (pin.getAtBeginning() != null){
				String params[]={pin.getName(),pin.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
			}
			if (pin.getIsRemove() != null){
				String params[]={pin.getName(),pin.getAction().getName()};
				LoggingUtil.logWarning(MessageKeys.VAR_IS_REMOVE_NOT_SUPPORTED,params,null);		
			}

			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)pin.getRepository();
			/* It is a restriction that pin be in only one set */
			InputPinSet pinSet = (InputPinSet)BomUtils.getInputPinSets(pin).get(0);
			BPELVariable pinVariable  = getBPELVariable(pin);
			BPELVariable repoVariable = getBPELVariable(bomVariable);
//			if (useVariableForActivity(pinSet,pin,bomVariable)) {
			if (useVariableForActivity(pinVariable, repoVariable)) {
				results.add(getBPELVariable(bomVariable));
			} else {
			//int pinIndex = pinSet.getInputObjectPin().indexOf(pin);
			String upperBound = 
				AbstractbpelUtil.getLiteralValue((LiteralSpecification)pin.getUpperBound())
								.toString();
			if ("".equals(upperBound)){
				upperBound = "*";
			}
			target = 
				transformVariableRetrieval(	getBPELVariable(bomVariable),
											bomVariable,
											false, 	// pin.getAtBeginning().booleanValue(), // not supported
											false, // pin.getIsRemove().booleanValue(), // not supported
											getBPELVariable(pin),
											pin);
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
			target.setName(convertedName);
			addDisplayName(pin,bomVariable,target);	 
			results.add(target);
			}
			
		} else if (getSource().get(0) instanceof InputPinSet){
			//TODO check pinIndex for transformVariable API
			InputPinSet pinSet = (InputPinSet)getSource().get(0);
			RepositoryAction action = (RepositoryAction)pinSet.getAction();
			
			/* Create and execute the container rule that corresponds to the input pin set */
			ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
			crIS.getSource().add(pinSet);
			this.getChildRules().add(crIS);
			crIS.transformSource2Target();
			results.addAll(crIS.getTarget());
			
			/* Create and execute the container rule that corresponds to the output pin set */
			// TODO: handle the case where there is more than one output set (possibly exception output sets - must be done in all rules)
			OutputPinSet resultSet = (OutputPinSet)pinSet.getOutputPinSet().get(0);
			ContainerRule crOS = AbstractbpelFactory.eINSTANCE.createContainerRule();
			crOS.getSource().add(resultSet);
			this.getChildRules().add(crOS);
			crOS.transformSource2Target();
			results.addAll(crOS.getTarget());

			/* Get the BOM Variable around which this action is based */
			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)action.getRepository();

			Activity target = null;
			
			if (action instanceof StoreArtifactAction){
				StoreArtifactAction saAction = (StoreArtifactAction)action;

				/* Check the attributes that are not supported by the TE */
				if (saAction.getAtBeginning() != null){
					String params[]={saAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
				}
				if (saAction.getIsInsert() != null){
					String params[]={saAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_IS_INSERT_NOT_SUPPORTED,params,null);		
				}
				if (saAction.getOverflow() != null){
					String params[]={saAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_OVERFLOW_NOT_SUPPORTED,params,null);		
				}

				String upperBound = 
					AbstractbpelUtil.getLiteralValue((LiteralSpecification)saAction.getValue().getUpperBound())
									.toString();

				target = 
//					transformVariableStorage(getBPELVariable(pinSet),
//											 getBPELVariablePart(saAction.getValue()),
//											 upperBound,
//											 false, // saAction.getAtBeginning().booleanValue(),	// atBeginning not supported
//											 false, // saAction.getIsInsert().booleanValue(),		// isInsert not supported
//											 false, // saAction.getOverflow().booleanValue(),		// overflow not supported
//											 getBPELVariable(bomVariable),
//											 getBPELVariablePart(bomVariable));
					transformVariableStorage(getBPELVariable(pinSet),
										 saAction.getValue(),
										 false, // saAction.getAtBeginning().booleanValue(),	// atBeginning not supported
										 false, // saAction.getIsInsert().booleanValue(),		// isInsert not supported
										 false, // saAction.getOverflow().booleanValue(),		// overflow not supported
										 getBPELVariable(bomVariable),
										 bomVariable);				
			} else if (action instanceof RetrieveArtifactAction){
				RetrieveArtifactAction raAction = (RetrieveArtifactAction)action;

				/* When (if) the 'length' attribute is supported, we must add this... 
				String length ;
				if ((raAction.getLength() instanceof InputValuePin) && !((InputValuePin)raAction.getLength()).getValue().isEmpty()){
					length = ((LiteralUnlimitedNatural)((InputValuePin)raAction.getLength()).getValue().get(0)).getValue();	
				} else {
					length = "*";
				}
				*/
				
				/* Check the attributes that are not supported by the TE */
				if (raAction.getLength() != null){
					String params[]={raAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_LENGTH_NOT_SUPPORTED,params,null);		
				}
				if (raAction.getAtBeginning() != null){
					String params[]={raAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
				}
				if (raAction.getIsRemove() != null){
					String params[]={raAction.getName()};
					LoggingUtil.logWarning(MessageKeys.VAR_IS_REMOVE_NOT_SUPPORTED,params,null);		
				}

				target = 
//					transformVariableRetrieval(	getBPELVariable(bomVariable),
//												getBPELVariablePart(bomVariable),
//												"*", 	// length not supported
//												raAction.getAtBeginning().booleanValue(),
//												raAction.getIsRemove().booleanValue(),
//												getBPELVariable(resultSet),
//												getBPELVariablePart((Pin)resultSet.getOutputObjectPin().get(0)));
//TODO
				transformVariableRetrieval(	getBPELVariable(bomVariable),
											bomVariable,
											raAction.getAtBeginning().booleanValue(),
											raAction.getIsRemove().booleanValue(),
											getBPELVariable(resultSet),
											
											(OutputObjectPin)resultSet.getOutputObjectPin().get(0)
											);

			}
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(action));
			target.setName(convertedName);	
			//target.setName(BomUtils.getCanonicalName(action));

			/* Map the input sets and connect the incoming and links for this action */
			connectInputSet(pinSet, target);

			addDisplayName(action,bomVariable,target);	 
			
			/* Add the target AFTER we have mapped and added the input set */
			results.add(target);
			
			/* Check if this action is an Exclusive Branch Point */
			if (BomHelper.getInstance().isEBranch(pinSet)){
				ExclusiveBranchRule rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
				rule.getSource().add(pinSet);
				getChildRules().add(rule);
				rule.transformSource2Target();
				results.addAll(rule.getTarget());
				
			} else if (BomHelper.getInstance().isCBranch(resultSet)){
				ConcurrentBranchRule rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
				rule.getSource().add(resultSet);
				getChildRules().add(rule);
				rule.transformSource2Target();
				results.addAll(rule.getTarget());
			}

			/* Map the output sets and connect the outgoing links for this action */
			connectOutputSet(resultSet, target);
			
		} 
		
		this.getTarget().addAll(results);
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;
	}

//	/**
//	 * source = {StoreArtifactPin || RetrieveArtifactPin || Input set of a Store/RetrieveArtifactAction}
//	 * target = { [<sequence>], <sequence>, [<sequence>] }
//	 * 
//	 * RESTRICTION: Variable access is done at the part level, which means that the entire
//	 * part is either read from or written to.  It is not possible to <i>insert</i> a value
//	 * into part.
//	 * 
//	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
//	 */
//	public boolean transformSource2Target_bak() {
//		LoggingUtil.traceEntry(this, "transformSource2Target");
//		
//		if (getSource().isEmpty()){
//			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
//			return false;
//		}
//		
//		List results = new ArrayList();
//		
//		com.ibm.btools.bom.model.processes.activities.Variable bomVariable = null;
//		
//		
//		if (getSource().get(0) instanceof StoreArtifactPin){
//			Activity target = null;
//			StoreArtifactPin pin = (StoreArtifactPin)getSource().get(0);
//
//			/* Check the attributes that are not supported by the TE */
//			if (pin.getAtBeginning() != null){
//				String params[]={pin.getName(),pin.getAction().getName()};
//				LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
//			}
//			if (pin.getIsInsert() != null){
//				String params[]={pin.getName(),pin.getAction().getName()};
//				LoggingUtil.logWarning(MessageKeys.VAR_IS_INSERT_NOT_SUPPORTED,params,null);		
//			}
//
//			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)pin.getRepository();
//			/* It is a restriction that pin be in only one set */
//			OutputPinSet pinSet = (OutputPinSet)BomUtils.getOutputPinSets(pin).get(0);
//			if (useVariableForActivity(pinSet,pin,bomVariable)) {
//				results.add(getBPELVariable(bomVariable));
//			} else {
//				int pinIndex;
//				if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && pinSet.getOutputObjectPin().size() == 1){
//					//this is the case where we do not have a wrapper complex type for the PinSet... only one Pin in the Set
//					pinIndex = -1;
//				}else{
//					pinIndex = pinSet.getOutputObjectPin().indexOf(pin);	
//					
//				}
//				
//
//				String upperBound = 
//					AbstractbpelUtil.getLiteralValue((LiteralSpecification)pin.getUpperBound())
//									.toString();
//				if ("".equals(upperBound)){
//					upperBound = "*";
//				}
////				String upperBound = pin.getUpperBound() == null 
////									? "*" 
////									: ((LiteralUnlimitedNatural)pin.getUpperBound()).getValue();
//				/*
//				target = 
//					transformVariableStorage(getBPELVariable(pinSet),
//											 getBPELVariablePart(pin),
//											 upperBound,
//											 false, 	// pin.getAtBeginning().booleanValue(), // not supported
//											 false, 	// pin.getIsInsert().booleanValue(), // not supported
//											 false,			// There is no overflow value for a pin
//											 getBPELVariable(bomVariable),
//											 getBPELVariablePart(bomVariable));
//				*/	
//				target = 
//					transformVariableStorage(getBPELVariable(pinSet),
//											 getBPELVariablePart(pin),
//											 pin,
//											 false, 	// pin.getAtBeginning().booleanValue(), // not supported
//											 false, 	// pin.getIsInsert().booleanValue(), // not supported
//											 false,			// There is no overflow value for a pin
//											 getBPELVariable(bomVariable),
//											 getBPELVariablePart(bomVariable),
//											 bomVariable, pinIndex, 0
//											 );
//				JavaNCNameConverter converter = new JavaNCNameConverter();
//				String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
//				target.setName(convertedName);
//				//target.setName(BomUtils.getCanonicalName(pin));	
//				addDisplayName(pin,bomVariable,target);	 
//				results.add(target);
//			}
//			
//		} else if (getSource().get(0) instanceof RetrieveArtifactPin){
//			Activity target = null;
//			RetrieveArtifactPin pin = (RetrieveArtifactPin)getSource().get(0);
//
//			/* Check the attributes that are not supported by the TE */
//			if (pin.getAtBeginning() != null){
//				String params[]={pin.getName(),pin.getAction().getName()};
//				LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
//			}
//			if (pin.getIsRemove() != null){
//				String params[]={pin.getName(),pin.getAction().getName()};
//				LoggingUtil.logWarning(MessageKeys.VAR_IS_REMOVE_NOT_SUPPORTED,params,null);		
//			}
//
//			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)pin.getRepository();
//			/* It is a restriction that pin be in only one set */
//			InputPinSet pinSet = (InputPinSet)BomUtils.getInputPinSets(pin).get(0);
//			if (useVariableForActivity(pinSet,pin,bomVariable)) {
//				results.add(getBPELVariable(bomVariable));
//			} else {
//				int pinIndex;
//				//support the case where the destination is a PinSet with a single Pin.. the pinIndex = -1
//				if(!BpelOptionsUtil.isAlwaysGenerateWraperComplexType() && pinSet.getInputObjectPin().size() == 1){
//					//this is the case where we do not have a wrapper complex type for the PinSet... only one Pin in the Set
//					pinIndex = -1;
//				}else{
//					pinIndex = pinSet.getInputObjectPin().indexOf(pin);	
//				
//				}
//			//int pinIndex = pinSet.getInputObjectPin().indexOf(pin);
//			String upperBound = 
//				AbstractbpelUtil.getLiteralValue((LiteralSpecification)pin.getUpperBound())
//								.toString();
//			if ("".equals(upperBound)){
//				upperBound = "*";
//			}
////			String upperBound = pin.getUpperBound() == null 
////								? "*" 
////								: ((LiteralUnlimitedNatural)pin.getUpperBound()).getValue();
//
//			target = 
////				transformVariableRetrieval(	getBPELVariable(bomVariable),
////											getBPELVariablePart(bomVariable),
////											upperBound,
////											false, 	// pin.getAtBeginning().booleanValue(), // not supported
////											false, // pin.getIsRemove().booleanValue(), // not supported
////											getBPELVariable(pinSet),
////											getBPELVariablePart(pin));
//				transformVariableRetrieval(	getBPELVariable(bomVariable),
//											getBPELVariablePart(bomVariable),
//											bomVariable,
//											false, 	// pin.getAtBeginning().booleanValue(), // not supported
//											false, // pin.getIsRemove().booleanValue(), // not supported
//											getBPELVariable(pinSet),
//											getBPELVariablePart(pin),
//											pin, 0, pinIndex
//											);
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(pin));
//			target.setName(convertedName);
//			addDisplayName(pin,bomVariable,target);	 
//			//target.setName(BomUtils.getCanonicalName(pin));		 
//			results.add(target);
//			}
//			
//		} else if (getSource().get(0) instanceof InputPinSet){
////TODO check pinIndex for transformVariable API
//			InputPinSet pinSet = (InputPinSet)getSource().get(0);
//			RepositoryAction action = (RepositoryAction)pinSet.getAction();
//			
//			/* Create and execute the container rule that corresponds to the input pin set */
//			ContainerRule crIS = AbstractbpelFactory.eINSTANCE.createContainerRule();
//			crIS.getSource().add(pinSet);
//			this.getChildRules().add(crIS);
//			crIS.transformSource2Target();
//			results.addAll(crIS.getTarget());
//			
//			/* Create and execute the container rule that corresponds to the output pin set */
//			// TODO: handle the case where there is more than one output set (possibly exception output sets - must be done in all rules)
//			OutputPinSet resultSet = (OutputPinSet)pinSet.getOutputPinSet().get(0);
//			ContainerRule crOS = AbstractbpelFactory.eINSTANCE.createContainerRule();
//			crOS.getSource().add(resultSet);
//			this.getChildRules().add(crOS);
//			crOS.transformSource2Target();
//			results.addAll(crOS.getTarget());
//
//			/* Get the BOM Variable around which this action is based */
//			bomVariable = (com.ibm.btools.bom.model.processes.activities.Variable)action.getRepository();
//
//			Activity target = null;
//			
//			if (action instanceof StoreArtifactAction){
//				StoreArtifactAction saAction = (StoreArtifactAction)action;
//
//				/* Check the attributes that are not supported by the TE */
//				if (saAction.getAtBeginning() != null){
//					String params[]={saAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
//				}
//				if (saAction.getIsInsert() != null){
//					String params[]={saAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_IS_INSERT_NOT_SUPPORTED,params,null);		
//				}
//				if (saAction.getOverflow() != null){
//					String params[]={saAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_OVERFLOW_NOT_SUPPORTED,params,null);		
//				}
//
//				String upperBound = 
//					AbstractbpelUtil.getLiteralValue((LiteralSpecification)saAction.getValue().getUpperBound())
//									.toString();
//
//				target = 
////					transformVariableStorage(getBPELVariable(pinSet),
////											 getBPELVariablePart(saAction.getValue()),
////											 upperBound,
////											 false, // saAction.getAtBeginning().booleanValue(),	// atBeginning not supported
////											 false, // saAction.getIsInsert().booleanValue(),		// isInsert not supported
////											 false, // saAction.getOverflow().booleanValue(),		// overflow not supported
////											 getBPELVariable(bomVariable),
////											 getBPELVariablePart(bomVariable));
//					transformVariableStorage(getBPELVariable(pinSet),
//										 getBPELVariablePart(saAction.getValue()),
//						 				 saAction.getValue(),
//										 false, // saAction.getAtBeginning().booleanValue(),	// atBeginning not supported
//										 false, // saAction.getIsInsert().booleanValue(),		// isInsert not supported
//										 false, // saAction.getOverflow().booleanValue(),		// overflow not supported
//										 getBPELVariable(bomVariable),
//										 getBPELVariablePart(bomVariable),
//										 bomVariable, 0, 0
//										 );				
//			} else if (action instanceof RetrieveArtifactAction){
//				RetrieveArtifactAction raAction = (RetrieveArtifactAction)action;
//
//				/* When (if) the 'length' attribute is supported, we must add this... 
//				String length ;
//				if ((raAction.getLength() instanceof InputValuePin) && !((InputValuePin)raAction.getLength()).getValue().isEmpty()){
//					length = ((LiteralUnlimitedNatural)((InputValuePin)raAction.getLength()).getValue().get(0)).getValue();	
//				} else {
//					length = "*";
//				}
//				*/
//				
//				/* Check the attributes that are not supported by the TE */
//				if (raAction.getLength() != null){
//					String params[]={raAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_LENGTH_NOT_SUPPORTED,params,null);		
//				}
//				if (raAction.getAtBeginning() != null){
//					String params[]={raAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_AT_BEGINNING_NOT_SUPPORTED,params,null);		
//				}
//				if (raAction.getIsRemove() != null){
//					String params[]={raAction.getName()};
//					LoggingUtil.logWarning(MessageKeys.VAR_IS_REMOVE_NOT_SUPPORTED,params,null);		
//				}
//
//				target = 
////					transformVariableRetrieval(	getBPELVariable(bomVariable),
////												getBPELVariablePart(bomVariable),
////												"*", 	// length not supported
////												raAction.getAtBeginning().booleanValue(),
////												raAction.getIsRemove().booleanValue(),
////												getBPELVariable(resultSet),
////												getBPELVariablePart((Pin)resultSet.getOutputObjectPin().get(0)));
////TODO
//				transformVariableRetrieval(	getBPELVariable(bomVariable),
//											getBPELVariablePart(bomVariable),
//											bomVariable,
//											raAction.getAtBeginning().booleanValue(),
//											raAction.getIsRemove().booleanValue(),
//											getBPELVariable(resultSet),
//											getBPELVariablePart((Pin)resultSet.getOutputObjectPin().get(0)),
//											(OutputObjectPin)resultSet.getOutputObjectPin().get(0), 0, 0
//											);
//
//			}
//			JavaNCNameConverter converter = new JavaNCNameConverter();
//			String convertedName = converter.convertName(NamingUtil.findRegistry(this), target, BomUtils.getCanonicalName(action));
//			target.setName(convertedName);	
//			//target.setName(BomUtils.getCanonicalName(action));
//
//			/* Map the input sets and connect the incoming and links for this action */
//			connectInputSet(pinSet, target);
//
//			addDisplayName(action,bomVariable,target);	 
//			
//			/* Add the target AFTER we have mapped and added the input set */
//			results.add(target);
//			
//			/* Check if this action is an Exclusive Branch Point */
//			if (BomHelper.getInstance().isEBranch(pinSet)){
//				ExclusiveBranchRule rule = AbstractbpelFactory.eINSTANCE.createExclusiveBranchRule();
//				rule.getSource().add(pinSet);
//				getChildRules().add(rule);
//				rule.transformSource2Target();
//				results.addAll(rule.getTarget());
//				
//			} else if (BomHelper.getInstance().isCBranch(resultSet)){
//				ConcurrentBranchRule rule = AbstractbpelFactory.eINSTANCE.createConcurrentBranchRule();
//				rule.getSource().add(resultSet);
//				getChildRules().add(rule);
//				rule.transformSource2Target();
//				results.addAll(rule.getTarget());
//			}
//
//			/* Map the output sets and connect the outgoing links for this action */
//			connectOutputSet(resultSet, target);
//			
//		} 
//		
//		this.getTarget().addAll(results);
//		LoggingUtil.traceExit(this, "transformSource2Target");		
//		return true;
//	}
	
	
//	protected Activity transformVariableRetrieval_bak(BPELVariable srcVariable,
//												Part srcVariablePart,
//												MultiplicityElement srcBomElement,
//												boolean atBeginning,
//												boolean isRemove,
//												BPELVariable destVariable,
//												Part destVariablePart,
//												MultiplicityElement destBomElement, int srcIndex, int destIndex
//												){
//
//		Activity assign = null;
//
//		if (srcVariable.getMessageType() == destVariable.getMessageType() ) {
//			assign = AbstractbpelUtil.createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
//		} else { 
//			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
//			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
//			// TODO: for BPEL should handle creation of multiple Assign
//			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart);
//			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariablePart, destIndex);
//			if(destBomElement instanceof RetrieveArtifactPin){
//				if(!isWSICompliant(((RetrieveArtifactPin)destBomElement).getAction())){
//					destIndex = -1;
//					//bpelpAlternative.setActivity(createComplexAssignJavaForNonWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//				}else{
//					if(!BpelOptionsUtil.isTargetDiamondCompatible()){
//						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					}else{
//						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliantDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					}
//				}
//				
//			}
//			//bpelAlternative.setActivity(AbstractbpelUtil.createComplexAssign(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//			//bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));				
//			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
//			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
//		}		
//
//		return assign;
//	
//												  	
//	}
	
	protected Activity transformVariableRetrieval(BPELVariable srcVariable,MultiplicityElement srcBomElement,boolean atBeginning, boolean isRemove,
			BPELVariable destVariable, MultiplicityElement destBomElement){
		
		  
		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
		Activity assign=null;
		if (srcVariable.equals(destVariable)) {
			assign = BPELFactory.eINSTANCE.createEmpty();
		} else { 
			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
			// TODO: for BPEL should handle creation of multiple Assign
			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariable);
			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariable);
			if(destBomElement instanceof RetrieveArtifactPin){
				Invoke rapInvoke = createComplexAssignJavaForWSICompliantDiamond(srcVariable,isSrcAList,destVariable, isDestAList);
				bpelpAlternative.setActivity(rapInvoke);
				//defect 155921 do not map the Snippet
				//revert 1611243
				MappingManager mapMgr = MappingManager.getInstance(this.getRoot().getContext());
				mapMgr.addMapDefinition(destBomElement, rapInvoke);
			}
			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
		}		
		return assign;
	
		
//		Activity assign = null;
		
//		if (srcVariable.getMessageType() == destVariable.getMessageType() ) {
//			assign = AbstractbpelUtil.createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
//		} else { 
//			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
//			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
//			// TODO: for BPEL should handle creation of multiple Assign
//			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart);
//			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariablePart, destIndex);
//			if(destBomElement instanceof RetrieveArtifactPin){
//				if(!isWSICompliant(((RetrieveArtifactPin)destBomElement).getAction())){
//					destIndex = -1;
//					//bpelpAlternative.setActivity(createComplexAssignJavaForNonWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//				}else{
//					if(!BpelOptionsUtil.isTargetDiamondCompatible()){
//						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliant(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					}else{
//						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliantDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					}
//				}
//				
//			}
//			//bpelAlternative.setActivity(AbstractbpelUtil.createComplexAssign(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//			//bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));				
//			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
//			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
//		}		
		
//		return assign;
		
		
	}
	
	/**
	 * This method determines whether the variable that represents the Local Repos is quailifed to be the input/output variable
	 * of the invoke Activity without the need of generating an assign/java snippet activity
	 * @param pinSet
	 * @param pin
	 * @param var
	 * @return
	 */
	private boolean useVariableForActivity(PinSet pinSet, ObjectPin pin, com.ibm.btools.bom.model.processes.activities.Variable var) {
		if (!optimizeVariableAccess) {
			return false;
		}
		if (!(AbstractbpelUtil.getMultiplicity(pin.getUpperBound()).equals(AbstractbpelUtil.getMultiplicity(var.getUpperBound()))) || !AbstractbpelUtil.isSameType((TypedElement)pin,(TypedElement)var)) {
			return false;
		}
		if (pinSet instanceof InputPinSet) {
			if (!BomUtils.isTask(((InputPinSet)pinSet).getAction()) || ((InputPinSet)pinSet).getInputObjectPin().size()>1) {
				return false;
			}
		} else {
			if (!BomUtils.isTask(((OutputPinSet)pinSet).getAction()) || ((OutputPinSet)pinSet).getOutputObjectPin().size()>1) {
				return false;
			}			
		}
		
		return true;
	}
	
	/**
	 * This method determines whether the variable that represents the Local Repos is quailifed to be the input/output variable
	 * of the invoke Activity without the need of generating an assign/java snippet activity
	 * @param pinSet
	 * @param pin
	 * @param var
	 * @return
	 */
	private boolean useVariableForActivity(BPELVariable pinVariable, BPELVariable repoVariable) {
		if(pinVariable != repoVariable){
			return false;
		}else{
			return true;
		}
//		if(pinVariable.getType() == repoVariable.getType()){
//			return true;
//		}else{
//			return false;
//		}
	}
	/**
	 * Generates the BPEL Activites required to retrieve information from the given BOM
	 * variable.
	 * @param srcBOMVariable The BOM Variable to retrieve the information from.
	 * @param length Not currently supported
	 * @param atBeginning Not currently supported
	 * @param isRemove Not currently suppored
	 * @param destVariable The BPEL Variable into which the information will be placed
	 * @param destVariablePart The part of the BPEL variable into which the info will be placed
	 * @return Activity - The BPEL Activity required to retrieve the information
	 */
//	protected Activity transformVariableRetrieval(BPELVariable srcVariable,
//												Part srcVariablePart,
//												String length,
//												boolean atBeginning,
//												boolean isRemove,
//												BPELVariable destVariable,
//												Part destVariablePart){
//
//		Activity activity = null;
//
//		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
//		Assign assign = BPELFactory.eINSTANCE.createAssign();
//		
//		if ("0".equals(length) || "*".equals(length)){
//			/* There is no polling in this case.  Values are withdrawn from the variable immediately */
//			activity = assign;			
//		} else {
///* Use this code if we want to poll variables until they meet the minimum multiplicity requirements */
////			/* Create a BPEL Sequence to contain the tasks required to get the value from the variable */ 
////			activity = BPELFactory.eINSTANCE.createSequence();
////			// Should we wait for 'length' values, or the lower-bound of the pin?
////			/* Poll the variable until it has enough values to satisfy length */
////			((Sequence)activity).getActivities().add(createPollingWhile(srcVariable, length));		
////
////			((Sequence)activity).getActivities().add(assign);
//
//			activity = assign;
//		}
//											  	
//		Copy copy = BPELFactory.eINSTANCE.createCopy();
//		From from = BPELFactory.eINSTANCE.createFrom();
//		To to = BPELFactory.eINSTANCE.createTo();
//		 												
//		from.setVariable(srcVariable);	
//		from.setPart(srcVariablePart);
//			
//		to.setVariable(destVariable);
//		to.setPart(destVariablePart);
//		
//		copy.setFrom(from);
//		copy.setTo(to);
//		assign.getCopy().add(copy);
//
//		return activity;
//	
//												  	
//	}


	/*
	 * Returns an Assign activity that updates the destVariable with the information provided
	 * in the srcVariable
	 */
	protected Activity transformVariableStorage(BPELVariable srcVariable,
											    MultiplicityElement srcBomElement,
											  boolean atBeginning,
											  boolean isInsert,
											  boolean overflow,
											  BPELVariable destVariable,
											  MultiplicityElement destBomElement ){
											  	
		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
		Activity assign=null;
		//if (srcVariable.getType() == destVariable.getType() ) {
		if(srcVariable == destVariable){
		
		//	assign = AbstractbpelUtil.createSimpleAssign(srcVariable,destVariable);
			assign = BPELFactory.eINSTANCE.createEmpty();
		} else { 
			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
			// TODO: for BPEL should handle creation of multiple Assign
			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariable);
			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariable);
			if(srcBomElement instanceof StoreArtifactPin){
				Invoke sapInvoke = createComplexAssignJavaForWSICompliantDiamond(srcVariable,isSrcAList,destVariable, isDestAList);
				bpelpAlternative.setActivity(sapInvoke);
				//defect 155921 do not map the Snippet   
				//revert defect 161243
				MappingManager mapMgr = MappingManager.getInstance(this.getRoot().getContext());
				mapMgr.addMapDefinition(destBomElement, sapInvoke);
			}
			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
		}		
		return assign;
	
												  	
	}
	
//	/*
//	 * Returns an Assign activity that updates the destVariable with the information provided
//	 * in the srcVariable
//	 */
//	protected Activity transformVariableStorage_bak(BPELVariable srcVariable,
//											  Part srcVariablePart,
//											  MultiplicityElement srcBomElement,
//											  boolean atBeginning,
//											  boolean isInsert,
//											  boolean overflow,
//											  BPELVariable destVariable,
//											  Part destVariablePart,
//											  MultiplicityElement destBomElement, int srcIndex, int destIndex
//											  ){
//											  	
//		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
//		Activity assign=null;
//		if (srcVariable.getMessageType() == destVariable.getMessageType() ) {
//			assign = AbstractbpelUtil.createSimpleAssign(srcVariable,srcVariablePart,destVariable,destVariablePart);
//		} else { 
//			assign = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternativeActivity();
//			Alternative bpelAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			Alternative bpelpAlternative = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createAlternative();
//			bpelAlternative.setType(ProcessFlowType.BPEL_LITERAL); 
//			bpelpAlternative.setType(ProcessFlowType.BPELPP_LITERAL);
//			// TODO: for BPEL should handle creation of multiple Assign
//			boolean isSrcAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(srcBomElement,srcVariablePart, srcIndex);
//			boolean isDestAList = AbstractbpelUtil.isTransformedMultiplicityElementAList(destBomElement,destVariablePart);
//			bpelAlternative.setActivity(AbstractbpelUtil.createComplexAssign(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//			destIndex = -1;
//			if(srcBomElement instanceof StoreArtifactPin){
//				if(!isWSICompliant(((StoreArtifactPin)srcBomElement).getAction())){
//					srcIndex = -1;
//				}
//			}
//			if(srcBomElement instanceof StoreArtifactPin){
//				if(!isWSICompliant(((StoreArtifactPin)srcBomElement).getAction())){
//					bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList));
//				}else{
//					if(!BpelOptionsUtil.isTargetDiamondCompatible()){
//						bpelpAlternative.setActivity(createComplexAssignJava(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, -1, destIndex));
//					}else{
//						bpelpAlternative.setActivity(createComplexAssignJavaForWSICompliantDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList, srcIndex, destIndex));
//					}
//				}
//			}
//			((AlternativeActivity)assign).getAlternative().add(bpelAlternative);
//			((AlternativeActivity)assign).getAlternative().add(bpelpAlternative);
//		}		
//		return assign;
//	
//												  	
//	}

	/*
	 * Returns an Assign activity that updates the destVariable with the information provided
	 * in the srcVariable
	 */
//	protected Activity transformVariableStorage(BPELVariable srcVariable,
//											  Part srcVariablePart,
//											  String length,
//											  boolean atBeginning,
//											  boolean isInsert,
//											  boolean overflow,
//											  BPELVariable destVariable,
//											  Part destVariablePart){
//											  	
//		/* Create a BPEL <assign> to copy the contents of srcVariable to destVariable */
//		Assign assign = BPELFactory.eINSTANCE.createAssign();
//		Copy copy = BPELFactory.eINSTANCE.createCopy();
//		From from = BPELFactory.eINSTANCE.createFrom();
//		To to = BPELFactory.eINSTANCE.createTo();
//		 												
//		from.setVariable(srcVariable);
//		from.setPart(srcVariablePart);
//			
//		to.setVariable(destVariable);
//		to.setPart(destVariablePart);
//			
//		copy.setFrom(from);
//		copy.setTo(to);
//		assign.getCopy().add(copy);
//		return assign;							  	
//	}
	
	/**
	 * Returns the BPEL Variable that is the target of a rule that has the BOM object as its source
	 * @param bomObject The BOM object that is the source of a rule with a Variable target
	 * @return Variable - The Variable target.
	 */
	private BPELVariable getBPELVariable(EObject bomObject){

//		BPELVariable destVariable = (BPELVariable)
//			TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
//												ContainerRule.class,
//												bomObject)
//							  .getTarget()
//							  .get(0);
		BPELVariable destVariable = createVariable((NamedElement) bomObject);
			
		return destVariable;
	}

	/**
	 * Returns the WSDL Part associated with the given bomObject;
	 *  
	 * @param bomObject The BOM Object that is the source of a rule with a WSDL Part as the target
	 * @return Part - The WSDL Part that is associated to the bomObject via a TransformationRule
	 */
	private Part getBPELVariablePart(EObject bomObject){

		Part destPart = null;
		// Ray
		/* Start searching from the root, because the bomObject could be a pin on a CallBehaviourAction
		 * etc., that is within a WSDL other than the one that contains this Datastore */		
//			TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule(getContext()),
		
		if(bomObject instanceof Pin){
			Action action = null;
			PinSet pinSet = null;
			if(bomObject instanceof InputObjectPin){
				action = ((InputObjectPin)bomObject).getAction();
				pinSet = (InputPinSet)BomUtils.getInputPinSets((Pin)bomObject).get(0);
			}else if(bomObject instanceof OutputObjectPin){
				action = ((OutputObjectPin)bomObject).getAction();
				pinSet = (OutputPinSet)BomUtils.getOutputPinSets((Pin)bomObject).get(0);
			}
			//if(ProcessUtil.isMultiplePartWSDL(action)){
			if(!isWSICompliant(action)){
				destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, bomObject).getTarget().get(0);
			}else{
				
				destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, pinSet).getTarget().get(0);
			}
		}else{
			destPart = (Part)TransformationUtil.getRuleForSource(getRoot(), PartRule.class, bomObject).getTarget().get(0);
		}
			
		  
			
		return destPart;
	}


	/**
	 * This method creates the BPEL While that will poll the given BPEL variable at a regular
	 * interval to determine if the number of elements in the variable are sufficiently 
	 * numerous to satisfy the given minimum.
	 * 
	 * @param varToPoll The BPEL Variable to poll for elements.
	 * @param minCount The number of items that must be in the variable for the polling to
	 * be satisfied.
	 * @return While - The resultant While activity that polls the given variable
	 */
	private While createPollingWhile(BPELVariable varToPoll, String minCount){
		While pollWhile = BPELFactory.eINSTANCE.createWhile();
		
		/* A BPEL variable is represented by a message with a single part */ 
		Part part = (Part) varToPoll.getMessageType().getEParts().get(0);
		//Part part = getBPELVariablePart(varToPoll);

		// Do we really want to have the concept of polling variables in BPEL??
		// No... it was taken out of the spec by August 23, 2003

//		BooleanExpression boolExp = BPELFactory.eINSTANCE.createBooleanExpression();
//		boolExp.setBody("bpws:getVariableData('"
//						+ varToPoll.getName()
//						+ "', '" 
//						+ part.getName()
//						+ "', '"
//						+ "count(/" 
//						+ part.getName() 
//						+ "/"
//						+ part.getTypeName().getLocalPart()
//						+ ") < "
//						+ minCount 
//						+ "'"); 
//		pollWhile.setCondition(boolExp);
		
		/* Create an expression model for the while boolean Expression */
		ExtensibilityExpression bpelExpr = com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();		
		bpelExpr.setExpression(createExpressionModel(part, minCount, varToPoll));
		pollWhile.addExtensibilityElement(bpelExpr);		
		
		/* Put the delay inside the body of the while loop */
		Wait wait = BPELFactory.eINSTANCE.createWait();
		Expression exp = BPELFactory.eINSTANCE.createExpression();
		exp.setBody("PT0.5S");
		wait.setUntil(exp);
		pollWhile.setActivity(wait);
		
		return pollWhile;
		
	}
		
	
	/**
	 * Calls the method to generate the mapped representation of the input set, and connects
	 * the result via a BPEL link to the target activity, if required
	 * @param pinSet The input set to map
	 * @param target The target activity to connect to the input set representation
	 */
	private void connectInputSet(InputPinSet pinSet, Activity target){
		/* Connect the incoming and outgoing links for this action */
		List isResultList = mapInputPinSet(pinSet);
		Activity inputSetActivity = null;
		if (isResultList.isEmpty()|| !(isResultList.get(0) instanceof Activity)){
			inputSetActivity = target;
		} else {
			inputSetActivity = (Activity)isResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(inputSetActivity, 
									   target, 
									   BomUtils.getCanonicalName(pinSet) + "_to_" + pinSet.getAction().getName());
				
			isResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(isResultList);
		}

		setTargetOfLinks(pinSet, inputSetActivity);	
	}

	/**
	 * Calls the method to generate the mapped representation of the output set, and connects
	 * the source to the result via a BPEL link, if required.
	 * 
	 * @param pinSet The output pin set to map
	 * @param source The BPEL activity to connect the output set reprsentation to
	 */	
	private void connectOutputSet(OutputPinSet pinSet, Activity source){
		List osResultList = mapOutputPinSet(pinSet);
		Activity outputSetActivity = null;
		if (osResultList.isEmpty() || !(osResultList.get(0) instanceof Activity)){
			outputSetActivity = source;
		} else {
			outputSetActivity = (Activity)osResultList.get(0);
				
			/* Link the new input set activity to the sequence */
			Link link = linkActivities(source, 
									   outputSetActivity, 
									   pinSet.getAction().getName() + "_to_" + BomUtils.getCanonicalName(pinSet));
				
			osResultList.add(link);
				
			/* Propagate the results from the input set mapping */
			getTarget().addAll(osResultList);
		}

		setSourceOfLinks(pinSet, outputSetActivity);	
	}

	/**
	 * Feturns a ComparisonExpression
	 * @param part - The part of a BPEL variable
	 * @param minCount - A String that represents the number of elements in the variable
	 * @return ComparisonExpression - A ComparisonExpression creation
	 */
	private FunctionExpression createExpressionModel(Part part, String minCount, BPELVariable varToPoll)
	{		
		ComparisonExpression cmpExpr = ModelFactory.eINSTANCE.createComparisonExpression();

		/* First operand of the ComparisonExpression */
		/*Adding the predefined count FunctionExpression */
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID(CoreFunctions.SIZE_FUNCTION);

		/* First argument for count Function Expression */
		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		ModelPathExpression mpeExpr = ModelFactory.eINSTANCE.createModelPathExpression();
		StaticStep step1 = ModelFactory.eINSTANCE.createStaticStep();
		step1.setStepName(part.getName());
		StaticStep step2 = ModelFactory.eINSTANCE.createStaticStep();
		step2.setStepName(part.getTypeName().getLocalPart());
		mpeExpr.setIsAbsolute(Boolean.TRUE);
		mpeExpr.getSteps().add(step1);
		mpeExpr.getSteps().add(step2);
		arg1.setArgumentValue(mpeExpr);

		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		cmpExpr.setFirstOperand(functionExpr);

		cmpExpr.setOperator(ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL);

		/* Second operand of ComparisonExpression */
		IntegerLiteralExpression intExpr = ModelFactory.eINSTANCE.createIntegerLiteralExpression();
		intExpr.setIntegerSymbol(new Integer(Integer.parseInt(minCount)));
		cmpExpr.setSecondOperand(intExpr);
		
		/* String Literal Expressions */
		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr1.setStringSymbol(varToPoll.getName());	
		StringLiteralExpression strExpr2 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr2.setStringSymbol(part.getName());
		
		/* Function Expression - GetVariableData */
		FunctionExpression getVarExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition getVarDefinition = ModelFactory.eINSTANCE.createFunctionDefinition();
		FunctionArgument getVarArg1 = ModelFactory.eINSTANCE.createFunctionArgument();
		getVarArg1.setArgumentValue(strExpr1);
		FunctionArgument getVarArg2 = ModelFactory.eINSTANCE.createFunctionArgument();
		getVarArg2.setArgumentValue(strExpr2);		
		getVarDefinition.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData");
		getVarDefinition.setFunctionName("bpws:getVariableData");
		getVarExpr.setDefinition(getVarDefinition);
		getVarExpr.getArguments().add(getVarArg1);
		getVarExpr.getArguments().add(getVarArg2);

		return getVarExpr;		
	}

	protected Invoke createComplexAssignJava(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();

		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable,destVariablePart,true, destIndex);
		}
		//javaStr += AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex );
		javaStr += AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex, true );
		if (isDestAList) {
			String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
			javaStr += AbstractbpelUtil.createJavaElementSetterCode(destVariable,destVariablePart,pinElementName, destIndex);

		}
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}
	
	
	protected Invoke createComplexAssignJava(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();

		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable,destVariablePart,true);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaPartSetterCode(destVariable,destVariablePart,destVariablePart.getName());
		}
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);

		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}


	protected Invoke createComplexAssignJavaForNonWSICompliant(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();

		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable,destVariablePart,true, destIndex);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex );
		if (isDestAList) {
			String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
			javaStr += AbstractbpelUtil.createJavaElementSetterCode(destVariable,destVariablePart,pinElementName, destIndex);

		}
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}	
	
	protected Invoke createComplexAssignJavaForWSICompliant(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();
		
		javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable, destVariablePart, true);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.createJavaVariableForPartCode(destVariable,destVariablePart,true, destIndex);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippets(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex, true );
//		if (isDestAList) {
//			String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
//			javaStr += AbstractbpelUtil.createJavaElementSetterCode(destVariable,destVariablePart,pinElementName, destIndex);	
			if(isDestAList){
				String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
				javaStr += destVariablePart.getName() + "." +
									AbstractbpelUtil.generateJavaSetterForVariable(pinElementName) +"(" + pinElementName +");\n";
			}
			javaStr += AbstractbpelUtil.createJavaPartSetterCode(destVariable, destVariablePart, destVariablePart.getName());
//		}

		
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}	

	protected Invoke createComplexAssignJavaForWSICompliantDiamond(BPELVariable srcVariable, boolean isSrcAList, BPELVariable destVariable, boolean isDestAList) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();
//		initialize the outputVariable if not already initialized
		javaStr += AbstractbpelUtil.initializeVariable(destVariable);
		javaStr += AbstractbpelUtil.createJavaCopySnippetsDiamond(srcVariable,isSrcAList,destVariable,isDestAList,false,"ind","0", false) + ";\n";
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}	
	
	protected Invoke createComplexAssignJavaForWSICompliantDiamond_bak(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();
//		initialize the outputVariable if not already initialized
		javaStr += AbstractbpelUtil.initializeVariable(destVariable, destVariablePart, true);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.initializeJavaListPropertyDiamond(destVariable, destVariablePart, true, destIndex);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippetsDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex, true )
		+ ";\n";
			if(isDestAList){
				String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
				String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex));
				XSDTypeDefinition pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex);
				javaStr += AbstractbpelUtil.generateJavaSetterForVariableDiamond(destVariable.getName(), pinElementName, pinTypeDefinition, listElement) + ";\n";
			}
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}	
	
	
	
	protected Invoke createComplexAssignJavaForWSICompliantDiamond_bak1(Variable srcVariable,Part srcVariablePart,boolean isSrcAList, Variable destVariable,Part destVariablePart, boolean isDestAList, int srcIndex, int destIndex) {
		Invoke invoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaStr = new String();
//		initialize the outputVariable if not already initialized
		javaStr += AbstractbpelUtil.initializeVariable(destVariable, destVariablePart, true);
		if (isDestAList) {
			javaStr += AbstractbpelUtil.initializeJavaListPropertyDiamond(destVariable, destVariablePart, true, destIndex);
		}
		javaStr += AbstractbpelUtil.createJavaCopySnippetsDiamond(srcVariable,srcVariablePart,isSrcAList,destVariable,destVariablePart,isDestAList,false,"ind","0",false, srcIndex, destIndex, true )
		+ ";\n";
			if(isDestAList){
				String pinElementName = AbstractbpelUtil.getPinElementName(destVariablePart, destIndex);
				String listElement = AbstractbpelUtil.getNameOfListElement((XSDComplexTypeDefinition)AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex));
				XSDTypeDefinition pinTypeDefinition = AbstractbpelUtil.getPinTypDefinition(destVariablePart, destIndex);
				javaStr += AbstractbpelUtil.generateJavaSetterForVariableDiamond(destVariable.getName(), pinElementName, pinTypeDefinition, listElement) + ";\n";
			}
//		javaCode.setValue(javaStr);
//		jsActivity.setJavaCode(javaCode);
		jsActivity.setJavaCode(javaStr);
		invoke.getEExtensibilityElements().add(jsActivity);
		
		return invoke;

	}	


	private void addDisplayName(Object action,Repository ds,Activity bpelAct) {
		DisplayName dspName = BPELPlusFactory.eINSTANCE.createDisplayName();
		bpelAct.getEExtensibilityElements().add(dspName);
		String actionName;
		if (action instanceof RetrieveArtifactPin || action instanceof RetrieveArtifactAction) {
			actionName="Read from";
		} else {
			actionName="Write to";
		}
		dspName.setText(actionName+" "+(ds==null?"":ds.getName()));		
	}

} //VariableAccessRuleImpl
