package com.ibm.btools.te.ilm.heuristics.abstractbpel.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.wbit.bpel.Activity;


/**
 * This util class maintains a registry of the mapping of input and outputPinSet of a process (global/local) to BPEL activity
 */
public class PinSetRegistryUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	public static String REGISTRY_KEY="ProcessPinSetRegistry";
	
	public static class RelatedPinSetsEntry {
		public List relatedPinSets=new ArrayList();;
		public Activity activity;
		public RelatedPinSetsEntry() {
			super();
		}
		public void setActivity(Activity activity) {
			this.activity = activity;
		}
		public Activity getActivity() {
			return activity;
		}
		public void setRelatedPinSet(PinSet pinSet) {
			relatedPinSets.add(pinSet);
		}
		public List getRelatedPinSets() {
			return relatedPinSets;
		}
		public boolean isPinSetRelated (PinSet pinSet) {
			return getRelatedPinSets().contains(pinSet);
		}
	}
	public static Map getRegistry(TransformationContext context) {
		Map registry = (Map)context.get(REGISTRY_KEY);
		if (registry==null) {
			registry = new HashMap();
			context.put(REGISTRY_KEY,registry);
		}
		return registry;
	}
	
	/**
	 * register a pinSet with the bpel activity
	 * @param context
	 * @param pinSet
	 * @param bpelAct
	 */
	public static void registerPinSet(TransformationContext context, PinSet pinSet, Activity bpelAct) {
		getRegistry(context).put(pinSet,bpelAct);
	}

	/**
	 * register a pinSet that has a related pinSet (InputPinSet correlated to an OutputPinSet) with the bpel activity 
	 * @param context
	 * @param pinSet
	 * @param bpelAct
	 * @param relatedPinSet
	 */
	public static void registerPinSet(TransformationContext context, PinSet pinSet, Activity bpelAct, PinSet relatedPinSet) {
		RelatedPinSetsEntry entry = (RelatedPinSetsEntry)getRegistry(context).get(pinSet);
		if (entry == null) {
			entry = new RelatedPinSetsEntry();
			getRegistry(context).put(pinSet,entry);
		}
		entry.setActivity(bpelAct);
		entry.setRelatedPinSet(relatedPinSet);
	}

	public static Activity getBpelActivity(TransformationContext context, PinSet pinSet) {
		if (getRegistry(context).get(pinSet) instanceof Activity) {
			return (Activity)getRegistry(context).get(pinSet);
		} else if (getRegistry(context).get(pinSet) instanceof RelatedPinSetsEntry) {
			return ((RelatedPinSetsEntry)getRegistry(context).get(pinSet)).getActivity();
		}
		return null;
	}

	public static List getBpelActivitiesForRelatedPinSet(TransformationContext context, PinSet relatedPinSet) {
		Set keys = getRegistry(context).keySet();
		PinSet ps;
		List ret=new ArrayList();
		for (Iterator it=keys.iterator();it.hasNext();) {
			ps = (PinSet)it.next();
			Object entry = getRegistry(context).get(ps);
			if (entry instanceof RelatedPinSetsEntry) {
				if (((RelatedPinSetsEntry)entry).isPinSetRelated(relatedPinSet)) {
					ret.add(((RelatedPinSetsEntry)entry).getActivity());
				}
			}
		}
		return ret;
	}

	public static List getPinSetsForRelatedPinSet(TransformationContext context, PinSet relatedPinSet) {
		Set keys = getRegistry(context).keySet();
		PinSet ps;
		List ret=new ArrayList();
		for (Iterator it=keys.iterator();it.hasNext();) {
			ps = (PinSet)it.next();
			Object entry = getRegistry(context).get(ps);
			if (entry instanceof RelatedPinSetsEntry) {
				if (((RelatedPinSetsEntry)entry).isPinSetRelated(relatedPinSet)) {
					ret.add(ps);
				}
			}
		}
		return ret;
	}

}
