/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.bpelp.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Part;

import com.ibm.btools.blm.ie.exprt.ExportOptions;
import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpFactory;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpPackage;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.JavaExprSerializerRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.util.BpelpFlags;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.model.abstractbpel.Alternative;
import com.ibm.btools.te.ilm.model.abstractbpel.AlternativeActivity;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.btools.te.ilm.model.abstractbpel.JoinExtensibilityExpression;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.From;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Sequence;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Targets;
import com.ibm.wbit.bpel.To;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpel.impl.BPELFactoryImpl;
import com.ibm.wbit.bpelpp.Autonomy;
import com.ibm.wbit.bpelpp.BPELPlusFactory;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.DisplayName;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.ExecutionMode;
import com.ibm.wbit.bpelpp.JavaCode;
import com.ibm.wbit.bpelpp.JavaScriptActivity;
import com.ibm.wbit.bpelpp.Task;
import com.ibm.wbit.wpc.AutonomyEnum;
import com.ibm.wbit.wpc.ExecutionModeEnum;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BpelpRuleImpl extends TransformationRuleImpl implements BpelpRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	// This is used for consolidation of the links that have the same source and target 
	private Map consolidateLinksMap=new HashMap();
	// This is used to capture the scopes in the process to allow fixing crossing links in
	// the scope after the process has been flatten out (can't fix it during flattening as
	// it won't be accurate)
//	private List scopes = new ArrayList();
//	private List whiles = new ArrayList();
	private List crossLinkAct = new ArrayList();
	// This is used for removing any PartnerLink that are not referenced by any <invoke> activity
	// A PartnerLink is created but not referenced is caused by the fact that if the <invoke> is
	// in fact a StaffActivity, it will not need to reference any PartnerLink.  See fixStaffActivity method
	private Map partnerLinksMap=new HashMap();
	
	private HashMap fSwitchToEmptyMap = new HashMap();
	
	private List whileWithConditionList = new ArrayList();
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BpelpRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return BpelpPackage.Literals.BPELP_RULE;
	}

	/* 
	 * Transforms an Abstract BPEL process flow into a BPEL+ process flow
	 * SOURCE: AbstractBPEL <process> activity
	 * TARGET: BPEL+ <process> activity with all children conforming to the BPEL+ Specification,
	 * 		   also satisfying the runtime limitations in effect
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (getSource().isEmpty() || !(getSource().get(0) instanceof Process)){
			LoggingUtil.traceExit(this, "transformSource2Target","no source");
			return false;
		}
		
		/* Iterate through the containership hierarchy and delegate the transformation 
		 * of a particular item to the appropriate method */
		Process process = (Process)getSource().get(0);
		
		/*
		 * Ray: Add Bpelp specific header
		 */	

		TransformationRule definitionRule = ProcessUtil.getProcessDefinitionRule(getContext());
		if(definitionRule != null){
			com.ibm.btools.bom.model.processes.activities.Activity activity = (com.ibm.btools.bom.model.processes.activities.Activity)definitionRule.getSource().get(0);
			
			AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
			valueProvider.setContext(this.getContext());
			Boolean longRunningProcess = (Boolean)valueProvider.getAttributeValueForType(activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_IS_LONG_RUNNING);
			Boolean runAsChild = (Boolean)valueProvider.getAttributeValueForType(activity.getImplementation(), AttributeNameConstants.BPEL_PROCESS_RUN_AS_CHILD);
			
			ExecutionMode execMode = BPELPlusFactory.eINSTANCE.createExecutionMode();
			if(longRunningProcess.booleanValue()){
				execMode.setExecutionMode(ExecutionModeEnum.LONG_RUNNING_LITERAL);	
			}else{
				execMode.setExecutionMode(ExecutionModeEnum.MICROFLOW_LITERAL);
			}
			process.addExtensibilityElement(execMode);
			Autonomy autonomy = BPELPlusFactory.eINSTANCE.createAutonomy();
			if(runAsChild.booleanValue()){
				autonomy.setAutonomy(AutonomyEnum.CHILD_LITERAL);
			}
			process.addExtensibilityElement(autonomy);
		}
		
/*
 ExecutionMode should be set based on the TechnicalAttributes
		ExecutionMode execMode = BPELPlusFactory.eINSTANCE.createExecutionMode();
	//	if (ExportOperation.bSupportMicroFlow) {
		if(BpelOptionsUtil.isTargetMicroflow()){
			execMode.setExecutionMode("microflow");
		} else {
			execMode.setExecutionMode("longRunning");
		}
		
	
		process.addExtensibilityElement(execMode);
*/	
		
//		AutoDelete autoDelete = BPELPlusFactory.eINSTANCE.createAutoDelete();
//		autoDelete.setAutoDelete(Boolean.TRUE);
//		process.addExtensibilityElement(execMode);
		 
		/* Transform the process recursively, starting from root activity */ 	
		transformActivity(process.getActivity());
		
		// In the case where the top-level process container is a flow (instead of a sequence), need to attempt to fix the links
		if (process.getActivity() instanceof Flow) {
			recordLinksForConsolidation(((Flow)process.getActivity()).getLinks());			
		}
		
		/* Ray: Need to consolidate links that have same source and target because
		 * WSADIE 5.1 and runtime won't support it
		 */
		consolidateLinks(consolidateLinksMap);

// RK : JR24578 - don't need to fix any crossing links here (which doesn't properly handle while loop in getContainingStructure method). Optimization will handle them later.		
//		int nCLAct = crossLinkAct.size();
//		// Should go through the scope list from end to beginning because the list
//		// is created from top-down.  Therefore we should fix the inner scopes first
//		// and work it to the top.
//		EObject obj;
//		for (int i=nCLAct-1; i>=0; i--) {
//			obj = (EObject)crossLinkAct.get(i);
//			if (obj instanceof Scope) {
//				Scope scope = (Scope)obj;
//				//fixCrossingLinksInScope(scope);
//				//Ray: Now that we support multiple inputsets on a subprocess and diamond supports crossing link, don't need to fix it anymore
//				//fixCrossingLinksInStructuredActivity(scope,scope.getActivity());
//			} else if (obj instanceof While) {
//				While _while = (While)obj;
//				fixCrossingLinksInStructuredActivity(_while,_while.getActivity());
//			}
//		}

/*
		int nScopes = scopes.size();
		// Should go through the scope list from end to beginning because the list
		// is created from top-down.  Therefore we should fix the inner scopes first
		// and work it to the top.
		for (int i=nScopes-1; i>=0; i--) {
			Scope scope = (Scope)scopes.get(i);
			//fixCrossingLinksInScope(scope);
			fixCrossingLinksInStructuredActivity(scope,scope.getActivity());
		}

		int nWhiles = whiles.size();
		// Should go through the scope list from end to beginning because the list
		// is created from top-down.  Therefore we should fix the inner scopes first
		// and work it to the top.
		for (int i=nWhiles-1; i>=0; i--) {
			While _while = (While)whiles.get(i);
			//fixCrossingLinksInScope(scope);
			fixCrossingLinksInStructuredActivity(_while,_while.getActivity());
		}
*/
/*		
		for (Iterator it=scopes.iterator();it.hasNext();) {
			Scope scope = (Scope)it.next();
			fixCrossingLinksInScope(scope);
		}
*/		

		fixPartnerLinks(partnerLinksMap,process);
		
		getTarget().add(process);
		
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceEntry(this, "transformSource2Target");
		return isComplete();
	}
	
	
	/**
	 * Recursively walks through each object in the abstract bpel process to execute any 
	 * child rules required to map to the desired ILM target. 
	 * 
	 * @param bpelObject The abstract bpel model object to apply transformation rules to
	 */
	private void transformActivity(EObject bpelObject){
		
		/* Recursive base case */
		if (bpelObject == null){
			return;
		}
	
		ArrayList objsToBeRemoved = new ArrayList();
		EObject content;
		for (int i=0; i<bpelObject.eContents().size(); i++){
			content = (EObject)bpelObject.eContents().get(i);

			/* Perform the transformations that require object substitutions *before* we
			 * recurse thorough the child elements */
			content = performSubstitutionRules(content);
			// Ray May 05,04: Normally the content afte performSubstitutionRules should not be
			// null.  This should only be the case where an Alt Activity is presented but there
			// is no BPELP activity available.  That means for BPELP, we don't need this activity.
			// Therefore we should remove this from its container
			if (content == null) {
				objsToBeRemoved.add(bpelObject.eContents().get(i));
			} else {
				// if the content is in fact a StaffActivity, need to perform some additional updates
				fixStaffActivity(content);
				
				/* Recursively transform this object's children */
				transformActivity(content);
			
				/* Actually transform the object to the pure BPEL version */
				performChildRules(content);

				if (content instanceof Flow){
					/* Ray: Need to consolidate links that have the same source and targets because
					 * WSADIE 5.1 and runtime won't support it
					 */		
					 recordLinksForConsolidation(((Flow)content).getLinks());
				}
			}
		}
		// Ray May 05,04: In some cases the altActivity for BPELP could be null.  In this case
		// we need to remove the alternativeActivity from its container.  Note that this can only
		// happen if the container is a <sequence>.  If the container is a flow then links have already
		// been created that links to this altActivity.  In this case the altActivity for BPELP
		// should never be null in the first place.
		for (int i=0; i < objsToBeRemoved.size(); i++) {
			EObject obj = (EObject)objsToBeRemoved.get(i);
			if (bpelObject instanceof Sequence) {
				((Sequence)bpelObject).getActivities().remove(obj);
			}
		}
		
	}
	
	private void fixPartnerLinks(Map map,Process proc) {
		Set keys = map.keySet();
		PartnerLink key;
		List vals;
		for (Iterator it=keys.iterator();it.hasNext();) {
			key = (PartnerLink)it.next();
			vals = (List)map.get(key);
			if (vals.isEmpty()) {
				proc.getPartnerLinks().getChildren().remove(key);
			}
		}
	}
	
	private void fixStaffActivity(EObject object) {
		if (object instanceof Invoke) {
			Invoke invoke = (Invoke)object;
			List extElements = invoke.getExtensibilityElements();
			boolean bIsStaff=false;
			for (Iterator it=extElements.iterator();it.hasNext()&&!bIsStaff;) {
				if (it.next() instanceof Task) {
					bIsStaff=true;
				}
			}
			PartnerLink plink = invoke.getPartnerLink();
			List refList = (List)partnerLinksMap.get(plink);
			if (refList == null) {
				refList = new ArrayList();
				partnerLinksMap.put(plink,refList);
			}
			if (bIsStaff) {
				PartnerLink dummy = BPELFactoryImpl.eINSTANCE.createPartnerLink();
				dummy.setName("null");
				invoke.setPartnerLink(dummy);				
			} else {
				// register a PartnerLink reference
				refList.add(invoke);
			}
		}
	}
	
	public void cleanupExtensibleElement(EObject object) {
		if (isTEExtensibleElement(object)) {
			object.eContainer().eContents().remove(object);
		}
	}
	
	public boolean isTEExtensibleElement(EObject object) {
		if (object instanceof AlternativeActivity) {
			return true;
		} else if (object instanceof ExtensibilityExpression) {
			return true;
		}
		return false;
	}
	
	/**
	 * This method determines if any substitution rules are necessary for the given object,
	 * and creates and executes them if they are.
	 * 
	 * @param absBpelObject The object to check for the need for substitution
	 * @return EObject The new object that was substituted for the parameter.
	 */
	private EObject performSubstitutionRules(EObject absBpelObject){
		EObject subObject = absBpelObject;
				
		if (absBpelObject instanceof Flow){
			/* We don't need to return anything if we flatten the process because the objects
			 * being replaced are the *children* of the Flow, not the flow itself */
			flattenProcess((Flow)absBpelObject);
			
			/* Ray: Need to consolidate links that have the same source and targets because
			 * WSADIE 5.1 and runtime won't support it
			 */		
//			 recordLinksForConsolidation(((Flow)absBpelObject).getLinks());
		} else if (absBpelObject instanceof AlternativeActivity){
			subObject = transformAlternativeActivity((AlternativeActivity)absBpelObject);

		} else if (absBpelObject instanceof Switch){
			subObject = transformSwitch((Switch)absBpelObject);
			//TODO AR also change the Assign.. we need to change the Assign that use path queries into Java Invokes.
		} else if (absBpelObject instanceof Assign){
			subObject = transformAssign((Assign)absBpelObject);
		}
		
		return subObject;
	}
	
	
	/**
	 * Performs the transformation of an Assign to a Java code invoke. It the Assign is a part to part assign, 
	 * remains unchanged, else, if it is a part to element assign, -> using a query assignment, a Java code will
	 * be generated as per runtime limitation....
	 * @param bpelAssign
	 * @return
	 */
	private EObject transformAssign(Assign bpelAssign){
		//we know that we only create a copy with one from and one to per Assign
		//TODO AR if the target is Diamond, as XPATH queries are supported for the FROM and TO we can 
		//return
		if(BpelOptionsUtil.isTargetDiamondCompatible()){
			return bpelAssign;
		}
		if(bpelAssign.getCopy().size()==1){
			Copy sourceCopy = (Copy)bpelAssign.getCopy().get(0);
			From from = sourceCopy.getFrom();
			To to = sourceCopy.getTo();

			if((from.getQuery() == null || from.getQuery().equals(""))){
				if((to.getQuery() == null || to.getQuery().equals(""))){
					return bpelAssign;
				}
			}
		}
			
		Invoke javaInvoke = AbstractbpelUtil.createJavaInvoke();
		JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
		JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
		String javaString = new String();
			for(int i=0; i<bpelAssign.getCopy().size(); i++){
				Copy sourceCopy = (Copy)bpelAssign.getCopy().get(i);
				From from = sourceCopy.getFrom();
				To to = sourceCopy.getTo();
				
//				JavaScriptActivity jsActivity = BPELPlusFactory.eINSTANCE.createJavaScriptActivity();
//				JavaCode javaCode = BPELPlusFactory.eINSTANCE.createJavaCode();
				
				BPELVariable srcCon = (BPELVariable)from.getVariable();
				BPELVariable tarCon = (BPELVariable)to.getVariable();
			
				String tarVarName = AbstractbpelUtil.capitalizeFirstChar(tarCon.getName());
				String tarPartName = null;
				if(to.getPart() != null){
					 tarPartName = to.getPart().getName();
				}else{
					tarPartName = AbstractbpelUtil.capitalizeFirstChar(((Part)tarCon.getMessageType().getEParts().get(0)).getName());
				}
			
				String srcVarName = AbstractbpelUtil.capitalizeFirstChar(srcCon.getName());
				String srcPartName = null; 
				if(from.getPart() != null){
					srcPartName = from.getPart().getName();
				}else{
					srcPartName = AbstractbpelUtil.capitalizeFirstChar(((Part)srcCon.getMessageType().getEParts().get(0)).getName());
				}
			 
			
//						if(!ExportOperation.bInterfaceWISCompliant){
//							//Ray Apr 13,04
//							if(obj==null || obj instanceof LiteralNull || !(obj instanceof LiteralSpecification)){
//								javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true)"
//														+ ".set" + partName + "(null);\n";
//							}else{
//								javaString += AbstractbpelUtil.generateJavaGetterForVariable(varName) + "(true)"
//											+ ".set" + partName + "(" + AbstractbpelUtil.getLiteralAssignmentString(value, destPart) + ");\n";
//							}
//						}else{
							//TODO AR 26.07
						//	InputPinSet is = (InputPinSet)BomUtils.getInputPinSets(sourcePin).get(0);
//				String tarPinElementName = AbstractbpelUtil.getPinElementName(tarSet, tarPin, findPart(tarSet, tarPin, tarCon));		
//				String srcPinElementName = AbstractbpelUtil.getPinElementName(srcSet, srcPin, findPart(srcSet, srcPin, srcCon));
				String srcPinElementName = null;
				String tarPinElementName = null;
				if(from.getQuery() != null && !from.getQuery().equals("")){
					//migration
					srcPinElementName = from.getQuery().getValue().substring(1);
				}else{
					srcPinElementName = "";
				}
				if(to.getQuery() != null && !to.getQuery().equals("")){
					//migration
					tarPinElementName = to.getQuery().getValue().substring(1);
					//tarPinElementName = to.getQuery().substring(1);
				}else{
					tarPinElementName = "";
				}
			
//				javaString += AbstractbpelUtil.generateJavaGetterForVariable(tarVarName) + "(true)" 
//												+ "." + AbstractbpelUtil.generateJavaGetterForVariable(tarPartName) +"()" 
//														+ ".set" + tarPinElementName + "(" 
//																+ AbstractbpelUtil.generateJavaGetterForVariable(srcVarName) +"()"
//																	+ "." + AbstractbpelUtil.generateJavaGetterForVariable(srcPartName) +"()" +
//																			"." + AbstractbpelUtil.generateJavaGetterForVariable(srcPinElementName) + "()"
//																 
//														 + ");\n";
//
//



				javaString += AbstractbpelUtil.generateJavaGetterForVariable(tarVarName) + "(true)" 
												+ "."  +
												((tarPinElementName != null && !tarPinElementName.equals(""))?
													 AbstractbpelUtil.generateJavaGetterForVariable(tarPartName) +"()" + "." + AbstractbpelUtil.generateJavaSetterForVariable(tarPinElementName) 
													 :AbstractbpelUtil.generateJavaSetterForVariable(tarPartName)) 
													 + "("
														+ AbstractbpelUtil.generateJavaGetterForVariable(srcVarName) +"()"
																	+ "." + AbstractbpelUtil.generateJavaGetterForVariable(srcPartName) +"()" +
																		((srcPinElementName != null && !srcPinElementName.equals(""))?
																			"." + AbstractbpelUtil.generateJavaGetterForVariable(srcPinElementName) + "()"
																			:"")
																 
														 + ");\n";







				
				
			//now we should change the links to point to the new activity
			//createDisplayName(javaInvoke,"Set value for " + srcPin.getName());
			//TODO 
			//createDisplayName(javaInvoke,"Set value for " + srcPinElementName);		
			//	createDisplayName(javaInvoke,"Set Constant Value");
			}
			//TODO migration .. revisit ??
		//	javaCode.getValue(javaString);
		//	jsActivity.setJavaCode(javaCode);
			jsActivity.setJavaCode(javaString);
			javaInvoke.getExtensibilityElements().add(jsActivity);
			/* Replace the <assign> with the java Invoke in the the parent container */
			if (bpelAssign.eContainer().eGet(bpelAssign.eContainmentFeature()) instanceof List){
				List activityList = (List)bpelAssign.eContainer().eGet(bpelAssign.eContainmentFeature());
				int index = activityList.indexOf(bpelAssign);
				activityList.add(index, javaInvoke);
				/* The bpelSwitch has shifted up one in the list -- remove it from its new position */
				activityList.remove(index + 1);
			} else {
				/* If the container isn't a list, just set the new containment value directly */
				bpelAssign.eContainer().eSet(bpelAssign.eContainmentFeature(), javaInvoke);
			}
			//migration AR 11.09
//			javaInvoke.getTargets().addAll(bpelAssign.getTargets());
//			javaInvoke.getSources().addAll(bpelAssign.getSources());
			Sources javaInvokeSources = javaInvoke.getSources();
			if(javaInvokeSources == null){
				javaInvokeSources = BPELFactory.eINSTANCE.createSources();
				javaInvoke.setSources(javaInvokeSources);
				
			}
			Targets javaInvokeTargets = javaInvoke.getTargets();
			if(javaInvokeTargets == null){
				javaInvokeTargets = BPELFactory.eINSTANCE.createTargets();
				javaInvoke.setTargets(javaInvokeTargets);
				
			}
			javaInvokeTargets.getChildren().addAll(bpelAssign.getTargets().getChildren());
			javaInvokeSources.getChildren().addAll(bpelAssign.getSources().getChildren());
		

		return javaInvoke;

	}
	
	
	/**
	 * Consolidate the links that have connects the same source and target
	 * @param recordedLinksMap
	 */
	private void consolidateLinks(Map recordedLinksMap) {
		Collection values = recordedLinksMap.values();
		Set keySet = recordedLinksMap.keySet();
		Set valSet = recordedLinksMap.entrySet();
		Iterator it = values.iterator();
		Map srcMap;
		List linksList;
		while (it.hasNext()) {
			srcMap = (Map)it.next();
			Collection tarVals = srcMap.values();
			Iterator tarIt = tarVals.iterator();
			while (tarIt.hasNext()) {
				linksList = (List)tarIt.next();
				// If the list entries > 1, that means there is more than one link connects the same source and target
				if (linksList.size() > 1) {
//					for (int i=0; i<linksList.size(); i++) {
//						System.out.println(((Link)linksList.get(i)).getName());
//					}
					//BpelOptimizationUtil.consolidateLinks(linksList);
					//TODO AR Dec 20
					BpelOptimizationUtil.consolidateLinks(linksList);
				}
			}
		}
	}
	
	/**
	 * Returns the map of <src activity> --> <target activity>* --> <link>*
	 * (where * means 0 or more) for all the links in the process.
	 * 
	 * @return The Map of links
	 */
	public Map getConsolidatedLinksMap(){
	    return consolidateLinksMap;
	}
	
//
//	/**
//		 * Obtain the transition condition from the source of the link
//		 * @param link
//		 * @return
//		 */	
//		private JoinCondition getJoinCondition(Link link) {
//			
//			Activity targetActivity = ((Target)link.getTargets().get(0)).getActivity();
//			if (targetActivity != null) {
//				List exElements = targetActivity.getExtensibilityElements();
//				// find the JoinCondition extensibility element.  There should only be one 
//				for (int i=0; i<exElements.size(); i++) {
//					if (exElements.get(i) instanceof JoinCondition) {
//						return (JoinCondition)exElements.get(i);
//					}
//				}
//			}
//			
////			List extensibilityElements = ((Target)link.getTargets().get(0)).getExtensibilityElements();
////			Iterator it = extensibilityElements.iterator();
////			Object extElement;
////			while (it.hasNext()) {
////				extElement = it.next();
////				if (extElement instanceof JoinCondition) {
////					return (JoinCondition)extElement;
////				}
////			}
//			return null;
//		}
//	/**
//	 * Obtain the transition condition from the source of the link
//	 * @param link
//	 * @return
//	 */	
//	private TransitionCondition getTransitionCondition(Link link) {
//		List extensibilityElements = ((Source)link.getSources().get(0)).getExtensibilityElements();
//		Iterator it = extensibilityElements.iterator();
//		Object extElement;
//		while (it.hasNext()) {
//			extElement = it.next();
//			if (extElement instanceof TransitionCondition) {
//				return (TransitionCondition)extElement;
//			}
//		}
//		return null;
//	}
//	
//	/**
//	 * Remove the link from the flow and disconnect the links from the source and target
//	 * @param link
//	 */
//	private void removeLink (Link link) {
//		// disconnect the link from the source and target
//		((Source)link.getSources().get(0)).setActivity(null);
//		((Target)link.getTargets().get(0)).setActivity(null);
//		
//		EObject links= link.eContainer();
//		while ((links!=null) && !(links instanceof Links)) {
//			links = link.eContainer();
//		}
//		((Links)links).getChildren().remove(link);
//	}
//	
//	/**
//	 * Consolidate two transition conditions
//	 * @param masterTC
//	 * @param secondaryTC
//	 */
//	private void consolidateTransitionCondition(TransitionCondition masterTC, TransitionCondition secondaryTC) {
//		if (masterTC != null && secondaryTC != null) {
//			if (!masterTC.getJavaCode().getValue().equals(secondaryTC.getJavaCode().getValue())) {
//				String masterJavaCode = masterTC.getJavaCode().getValue();
//				String secondPartJavaCode = secondaryTC.getJavaCode().getValue().substring(secondaryTC.getJavaCode().getValue().indexOf("return")+6);
//				String firstPartJavaCode = masterJavaCode.substring(0,masterJavaCode.indexOf(";"));
//				masterTC.getJavaCode().setValue(firstPartJavaCode+ " || " + secondPartJavaCode);
//			}
//		}
//	}
//	
//	private Link getMasterLink(List duplicatedLinksList){
//		int size = duplicatedLinksList.size();
//		Link masterLink = null;
//		if(size > 1){
//			
//			for(int i=0; i<size; i++){
//				masterLink = (Link)duplicatedLinksList.get(i);
//				JoinCondition joinCondition = getJoinCondition(masterLink);
//				if(joinCondition != null){
//					String body = joinCondition.getJavaCode().getValue();
//					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(masterLink);
//					if(body.indexOf("|| (" + getLinkStatusSnippet) != -1 || body.indexOf(getLinkStatusSnippet + ") ||") != -1){
//							return masterLink;
//					}
//				}
//			}
//		}
//		if(masterLink == null){
//			masterLink = (Link)duplicatedLinksList.get(0);
//		}
//	return masterLink;	
//	}
//	
//	/**
//	 * consolidate a list of links that connects the same source and targets, 
//	 * @param duplicatedLinksList
//	 */
//	private void consolidateLinks(List duplicatedLinksList) {
//		int size = duplicatedLinksList.size();
//		Link masterLink, secondaryLink;
//		// pick the first one as the master link
//		masterLink = getMasterLink(duplicatedLinksList);
//		//masterLink = (Link)duplicatedLinksList.get(0);
//		TransitionCondition masterTC = getTransitionCondition(masterLink);
//		TransitionCondition secondaryTC;
////		for (int i=1; i<size; i++) {
////			secondaryLink = (Link)duplicatedLinksList.get(i);
////			secondaryTC = getTransitionCondition(secondaryLink);
////			if (masterTC == null && secondaryTC != null) {
////				masterLink.addExtensibilityElement(secondaryTC);
////			} else if (secondaryTC != null && masterTC != null) {
////				consolidateTransitionCondition(masterTC,secondaryTC);
////			} 
////			updateJoinCondition(secondaryLink);
////			removeLink(secondaryLink);
////		}
//		for (int i=0; i<size; i++) {
//			if(!(duplicatedLinksList.get(i).equals(masterLink))){
//				secondaryLink = (Link)duplicatedLinksList.get(i);
//				secondaryTC = getTransitionCondition(secondaryLink);
//				if (masterTC == null && secondaryTC != null) {
//					masterLink.addExtensibilityElement(secondaryTC);
//				} else if (secondaryTC != null && masterTC != null) {
//					consolidateTransitionCondition(masterTC,secondaryTC);
//				} 
//				updateJoinCondition(secondaryLink);
//				removeLink(secondaryLink);
//			}
//			
//		}
//	}
//	
//	private String buildGetLinkStatusJavaSnippet(Link link) {
//		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
//		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
//		def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getLinkStatus");
//		def.setFunctionName("bpws:getLinkStatus");
//
//		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
//		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
//		strExpr1.setStringSymbol(link.getName());
//		arg1.setArgumentValue(strExpr1);
//
//		functionExpr.setDefinition(def);
//		functionExpr.getArguments().add(arg1);
//		JavaExpressionSerializer javaExprSerializer = new JavaExpressionSerializer();
//		return javaExprSerializer.serialize(functionExpr);
//	}
//	
//	/**
//	 * Upon removal of the link, we need to remove the getLinkStatus for the link in the JoinCondition
//	 * @param linkToBeRemoved
//	 */
//	private void updateJoinCondition(Link linkToBeRemoved) {
//		Activity targetActivity = ((Target)linkToBeRemoved.getTargets().get(0)).getActivity();
//		if (targetActivity != null) {
//			List exElements = targetActivity.getExtensibilityElements();
//			// find the JoinCondition extensibility element.  There should only be one 
//			JoinCondition joinCondition=null;
//			for (int i=0; i<exElements.size() && joinCondition==null; i++) {
//				if (exElements.get(i) instanceof JoinCondition) {
//					joinCondition = (JoinCondition)exElements.get(i);
//				}
//			}
//			if (joinCondition != null) {
//				List targets = targetActivity.getTargets();
//				if (targets.size()==2) {
//					// if there are 2 targets, then there will only be one left when the link
//					// is removed.  That means we don't need any JoinCondition (i.e. only 1 link goes
//					// into this activity)
//					exElements.remove(joinCondition);
//				} else {
//					// Clean up the getLinkStatus snippet of the link from the JoinCondition
//					String body = joinCondition.getJavaCode().getValue();
//					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(linkToBeRemoved);
//					body = StringUtil.replaceAll(body,"&& ("+getLinkStatusSnippet+")","");
//					body = StringUtil.replaceAll(body,"|| ("+getLinkStatusSnippet+")","");
//					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") &&","");
//					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") ||","");
//					joinCondition.getJavaCode().setValue(body);
//				}
//			}
//		}
//	}
	
	/**
	 * Create the Map that will be used for links consolidation.  Map key is the source.  Value is
	 * another Map that has key as target, and value is a list of links.  If there are more than one
	 * links in the list, that means there is more than one link that connects the same source and target
	 * @param links
	 */
	private void recordLinksForConsolidation(Links links) {
		if (links == null) {
			return;
		}
		List allLinks = links.getChildren();
		Iterator it = allLinks.iterator();
		Link link;
		Map targetsToLinkMap;
		List recordLinksList;
		Activity srcAct,tarAct;
		while (it.hasNext()) {
			link = (Link)it.next();
			// TODO: Ray Testing
			if (link.getSources().size()==0 || link.getTargets().size()==0) {
				continue;
			}	
			
			srcAct=((Source)link.getSources().get(0)).getActivity();
			tarAct=((Target)link.getTargets().get(0)).getActivity();
			
			if(CBPELUtil.isFaultLink(srcAct, link)){
				continue;
			}
			
			targetsToLinkMap = (Map)consolidateLinksMap.get(srcAct);
			if (targetsToLinkMap == null) {
				targetsToLinkMap = new HashMap();
				recordLinksList = new ArrayList();
				recordLinksList.add(link);
				targetsToLinkMap.put(tarAct,recordLinksList);
				consolidateLinksMap.put(srcAct,targetsToLinkMap);
			} else {
				recordLinksList = (List)targetsToLinkMap.get(tarAct);
				if (recordLinksList == null) {
					recordLinksList = new ArrayList();
					recordLinksList.add(link);
					targetsToLinkMap.put(tarAct,recordLinksList);					
				} else {
					recordLinksList.add(link);
				}
			}
		}
	}
	
	
	/**
	 * This method determines if a particular child rule is required to map the given
	 * object, creates the rule if it is necessary and executes it.
	 * 
	 * @param absBpelObject The abstract bpel object against which to apply the child rule
	 */
	private void performChildRules(EObject absBpelObject){
		List childRules = new ArrayList();
				
		if (containsExpression(absBpelObject)){
			transformExpression((ExtensibleElement)absBpelObject);
		}

	}


	/**
	 * Determines if the given abstract bpel object contains an expression or not.  This is
	 * determined by looking for an ExtensibleExpression in the list of Extensibility Elements
	 * 
	 * @param absBpelObject The abstract bpel object to check for a condition.
	 * 
	 * @return True if the object contains an extensibility element, false otherwise.
	 */	
	private boolean containsExpression(EObject absBpelObject){
		/* Only ExtensibleElements can contain expressions */
		if (absBpelObject instanceof ExtensibleElement){
			for (Iterator it = ((ExtensibleElement)absBpelObject).getEExtensibilityElements().iterator(); it.hasNext(); ){
				if (it.next() instanceof ExtensibilityExpression){
					return true;
				}
			}
		}
				
		return false;
	}
	
	/**
	 * Creates a JavaExprSerializerRule to transform the expression model instances associated
	 * with extElement into a java string.  The java string is then assigned to the element as
	 * per the BPEL specification.
	 *  
	 * @param extElement The ExtensibleElement that contains an ExtensibleExpression.
	 */
	private void transformExpression(ExtensibleElement extElement){
		Object element = null;
		List elementsToAdd = new ArrayList();
		List elementsToRemove = new ArrayList();
		
		for (Iterator it = extElement.getEExtensibilityElements().iterator(); it.hasNext(); ){
			element = it.next();
			if (element instanceof ExtensibilityExpression){
				
				/* Mark the expression that we are about to process as 'to be removed'.  We can't
				 * remove it right away or we will get a ConcurrentModificationException */
				elementsToRemove.add(element);
				
				JavaExprSerializerRule javaSerializerRule = BpelpFactory.eINSTANCE.createJavaExprSerializerRule();
				javaSerializerRule.getSource().add(((ExtensibilityExpression)element).getExpression());
				getChildRules().add(javaSerializerRule);
				javaSerializerRule.transformSource2Target();
				//TODO migration ???? 
				//	JavaCode javaCode = (JavaCode)javaSerializerRule.getTarget().get(0);
				String javaCode = (String)((com.ibm.wbit.bpel.Expression)javaSerializerRule.getTarget().get(0)).getBody();
				
				/* Of the following cases, only one can be true at a time */
				//AR !! the transformExpression is called after the substitutionRules are done, so do we have Case anymore? 
				//this case is not migrated ...
				if (extElement instanceof Case){
//					Condition condition = BPELPlusFactory.eINSTANCE.createCondition();
//					condition.setJavaCode(javaCode);
//					elementsToAdd.add(condition);
					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(javaCode);
					((Case)extElement).setCondition(condition);	
				} else if (extElement instanceof While && !(element instanceof JoinExtensibilityExpression)){
					/**
					 * Ray Mar02/04 - WSADIE BPEL editor supports 2 types of While condition: 'Expression' vs 'Visual Expression'.
					 * To annotate the condition as an 'Expression' type condition, BPEL editor requires the bpel 
					 * condition="DefinedByJavaCode".  For example:
					 *             <while condition="DefinedByJavaCode" name="While" wpc:displayName="While" wpc:id="7">
					 */
//					BooleanExpression boolExpr = BPELFactory.eINSTANCE.createBooleanExpression();
//					boolExpr.setBody("DefinedByJavaCode");
					//((While)extElement).setCondition(boolExpr);
//					Condition condition = BPELPlusFactory.eINSTANCE.createCondition();
//					condition.setJavaCode(javaCode);
								
//					TODO migration ??? verify this and update with java namespace .?.. this most probably will not work as in 5.1
					//we can set the condition in the bpel.Condition do not set extensibilityElements... 
					//check with ui when ready what is expected... !!!!
					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(javaCode);
					While theWhile = (While)extElement;
					theWhile.setCondition(condition);
					if(this.whileWithConditionList != null){
						this.whileWithConditionList.add(theWhile);
					}
					//elementsToAdd.add(condition);

				} else if (extElement instanceof Source){
//					TransitionCondition transCond = BPELPlusFactory.eINSTANCE.createTransitionCondition();
//					transCond.setJavaCode(javaCode);
//					elementsToAdd.add(transCond);

					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(javaCode);
					((Source)extElement).setTransitionCondition(condition);	

				} else if (extElement instanceof Activity && element instanceof JoinExtensibilityExpression){
//					JoinCondition joinCond = BPELPlusFactory.eINSTANCE.createJoinCondition();
//					joinCond.setJavaCode(javaCode);
//					elementsToAdd.add(joinCond);

					com.ibm.wbit.bpel.Condition condition = BPELFactory.eINSTANCE.createCondition();
					condition.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
					condition.setBody(javaCode);
					//((Activity)extElement).setJoinCondition(condition);
					((Activity)extElement).getTargets().setJoinCondition(condition);

				}
			

			}
		}
		
		extElement.getExtensibilityElements().removeAll(elementsToRemove);
		extElement.getExtensibilityElements().addAll(elementsToAdd);
		
	}
		
	
	/**
	 * Replaces the given alternative activity with the BPEL PLUS alternative.
	 * 
	 * @param altActivity The alternative activity to replace.
	 * @param Activity The activity that replaced AlternativeActivity
	 */	
	private Activity transformAlternativeActivity(AlternativeActivity altActivity){
		Alternative alt = null;
		Activity result = null;

		for (int i=0; i<altActivity.getAlternative().size(); i++){
			alt = (Alternative)altActivity.getAlternative().get(i);

			if (alt.getType().equals(ProcessFlowType.BPELPP_LITERAL)){

				/* Find the position of altActivity in its container, remove it, and put
				 * the BPELP alternative in the same spot */
				Activity bpelpActivity = alt.getActivity();

				if (altActivity.eContainer().eGet(altActivity.eContainmentFeature()) instanceof List){
					List activityList = (List)altActivity.eContainer().eGet(altActivity.eContainmentFeature());
					int index = activityList.indexOf(altActivity);
					activityList.add(index, bpelpActivity);
					/* The altActivity has shifted up one in the list -- remove it from its new position */
					activityList.remove(index + 1);
				} else {
					/* If the container isn't a list, just set the new containment value directly */
					altActivity.eContainer().eSet(altActivity.eContainmentFeature(), bpelpActivity);
				}
					
				/* Copy the generic activity attributes to the actual activity */
				bpelpActivity.setName(altActivity.getName());
				
				
				//migration AR 11.09
//				bpelpActivity.setJoinCondition(altActivity.getJoinCondition());
//				bpelpActivity.getTargets().addAll(altActivity.getTargets());
//				bpelpActivity.getSources().addAll(altActivity.getSources());
				
				
				if(altActivity.getTargets() != null && altActivity.getTargets().getChildren() != null){
					Targets bpelpTargets = bpelpActivity.getTargets();
					if( bpelpTargets == null){
						bpelpTargets = BPELFactory.eINSTANCE.createTargets();
						bpelpActivity.setTargets(bpelpTargets);
						
					}
					bpelpTargets.getChildren().addAll(altActivity.getTargets().getChildren());
					bpelpActivity.getTargets().setJoinCondition(altActivity.getTargets().getJoinCondition());
				}
			
				
				if(altActivity.getSources() != null && altActivity.getSources().getChildren() != null){
					Sources bpelpSources = bpelpActivity.getSources();
					if( bpelpSources == null){
						bpelpSources = BPELFactory.eINSTANCE.createSources();
						bpelpActivity.setSources(bpelpSources);
					}
					bpelpSources.getChildren().addAll(altActivity.getSources().getChildren());
				}
	
				bpelpActivity.getEExtensibilityElements().addAll(altActivity.getEExtensibilityElements());
				
				result = bpelpActivity;
			}
		}
		return result;
	}

	private Activity transformSwitch(Switch bpelSwitch){
		boolean isSwitchInclusive = false;
		Decision decision = null;
		TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),DecisionRule.class,bpelSwitch);
		if (tr != null) {
			if (tr.getSource().get(0) instanceof InputPinSet) {
				Action action = ((InputPinSet)tr.getSource().get(0)).getAction();
				if (action instanceof Decision) {
					isSwitchInclusive = ((Decision)action).getMultiplePaths().booleanValue();
					decision = (Decision)action;
				}
			}
		}
		//if (BpelOptionsUtil.generateSwitchForExclusiveDecision() && !isSwitchInclusive) {
		if (!isSwitchInclusive && generateSwitchForExclusiveDecision(decision)) {
			return transformSwitchToBpelpSwitch(bpelSwitch);			
		} else {
			return transformSwitchToEmpty(bpelSwitch);
		}
	}	
	
	private boolean generateSwitchForExclusiveDecision(NamedElement origSrcObj){
		/* We never want to generate a switch when we're invoked from the BME Export */
		//defect 115460
		ExportSession exportSession = TransformationSessionUtil.getExportSession(TransformationEngine.getTransformationSession());
		if(exportSession != null){
			ExportOptions options = exportSession.getExportOptions();
			if(options != null){
				if(options.getAdditionalOption("CREATE_OM_HASHMAP") != null){
					return false;
				}
			}
		}
//		if (TransformationSessionUtil.getExportSession(TransformationEngine.getTransformationSession()).getExportOptions().getAdditionalOption("CREATE_OM_HASHMAP") != null){
//			return false;
//		}
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.GENERATE_SWITCH);
		boolean generateSwitch = ((Boolean)valueProvider.getAttributeValueForType(origSrcObj,AttributeNameConstants.GENERATE_SWITCH)).booleanValue();
		return generateSwitch;
		
	}


	/**
	 * Transforms the given BPEL Switch activity, which isn't supported in BPEL+, into a BPEL 
	 * Sequence that contains the child activities of the Cases of the original Switch, augmented
	 * with transition conditions that reflect the decision conditions.
	 * 
	 * @param bpelSwitch The BPEL Switch activity to replace with a Sequence
	 * @return Activity - The BPEL Activity that replaced the Switch 
	 */
	private Activity transformSwitchToEmpty(Switch bpelSwitch){
		Empty empty = BPELFactory.eINSTANCE.createEmpty();
		empty.setName(bpelSwitch.getName());
		//migration AR 11.09
//		empty.getTargets().addAll(bpelSwitch.getTargets());
//		empty.getSources().addAll(bpelSwitch.getSources());



		if(bpelSwitch.getTargets() != null){
			Targets targets = empty.getTargets();
			if(targets == null){
				targets = BPELFactory.eINSTANCE.createTargets();
				empty.setTargets(targets);
			}
			empty.getTargets().setJoinCondition(bpelSwitch.getTargets().getJoinCondition());
			targets.getChildren().addAll(bpelSwitch.getTargets().getChildren());	
		}
		if(bpelSwitch.getSources() != null && !bpelSwitch.getSources().getChildren().isEmpty()){
			Sources sources = empty.getSources();
			if(sources == null){
				sources = BPELFactory.eINSTANCE.createSources();
				empty.setSources(sources);
			}
			sources.getChildren().addAll(bpelSwitch.getSources().getChildren());	
		}
	
		empty.getEExtensibilityElements().addAll(bpelSwitch.getEExtensibilityElements());
		
		Case switchCase = null;
		Activity caseActivity = null;
		for (Iterator it = bpelSwitch.getCases().iterator(); it.hasNext(); ){
			switchCase = (Case)it.next();
			caseActivity = switchCase.getActivity();
			/* Set the decision condition as the join condition on the activity, for completeness... */
			//TODO migration ???? 
//			caseActivity.setJoinCondition(switchCase.getCondition());

			if(switchCase.getCondition() != null){
				com.ibm.wbit.bpel.Condition booleanExpression = BPELFactory.eINSTANCE.createCondition();
				booleanExpression.setBody(switchCase.getCondition().getBody());	
				booleanExpression.setExpressionLanguage(AbstractbpelConstants.EXPRESSION_JAVA_NAMESPACE);
				//caseActivity.setJoinCondition(booleanExpression);
				caseActivity.getTargets().setJoinCondition(booleanExpression);
			}
			
			ExtensibilityExpression caseCondition = null;
			Documentation doc = null;
			Description desc = null;			
			for (int i=0; i<switchCase.getExtensibilityElements().size(); i++){
				if (switchCase.getExtensibilityElements().get(i) instanceof ExtensibilityExpression){
					/* This must be the switch case condition, which we will translate into a 
					 * transition condition on the case's contained activity */
					caseCondition = (ExtensibilityExpression)switchCase.getExtensibilityElements().get(i);
					switchCase.getExtensibilityElements().remove(caseCondition);
					i--;
				} else if (switchCase.getExtensibilityElements().get(i) instanceof Documentation) {
					doc = (Documentation) switchCase.getExtensibilityElements().get(i);
					switchCase.getExtensibilityElements().remove(doc);
					i--;
				} else if (switchCase.getExtensibilityElements().get(i) instanceof Description) {
					desc = (Description) switchCase.getExtensibilityElements().get(i);
					switchCase.getExtensibilityElements().remove(desc);
					i--;					
				} else {
					/* Propagate anything that's not the switch case condition to the dest activity */
					switchCase.getActivity().getExtensibilityElements().add(switchCase.getExtensibilityElements().get(i));
					i--;
				}
			}

			if (caseActivity instanceof Empty) { 
				/* Now find the existing transition condition for each source on the activity, if there is one.	 */
				Source source = null;
				ExtensibilityExpression transCondition = null;
				Object element = null;
				List sourcesToMove = new ArrayList();
				//migration AR 11.09
			//	for (Iterator srcIt = caseActivity.getSources().iterator(); srcIt.hasNext(); ){
				if(caseActivity.getSources() != null && caseActivity.getSources().getChildren() != null){
					for (Iterator srcIt = caseActivity.getSources().getChildren().iterator(); srcIt.hasNext(); ){
						source = (Source)srcIt.next();
						for (Iterator it2 = source.getExtensibilityElements().iterator(); it2.hasNext(); ){
							element = it2.next();
							if (element instanceof ExtensibilityExpression 
								&& !(element instanceof JoinExtensibilityExpression)){
		
								transCondition = (ExtensibilityExpression)element;
							}
						}
	
						// Ray - Try to handle the case where opaqueExpression is not set
						if (caseCondition != null) {	
							/* Remove the old transition condition and update it with the new one */
							source.getExtensibilityElements().remove(transCondition);
							/* Create a copy of the case condition, so we can use it in multiple places */
							source.getExtensibilityElements().add(updateTransitionCondition(transCondition, 
																 (ExtensibilityExpression)cloneExpression(caseCondition)));
						}
						Link link = source.getLink();
						if (link != null) {
							if (doc != null) {
								// need to make a copy of the doc because there could be multiple links going out from the same branch
								Documentation docCopy = BPELPlusFactory.eINSTANCE.createDocumentation();
//GJD JR24896 - limit documentation to 4096 chars and description to 254 chars
								String documentation = doc.getValue();
								if (documentation.length() > 4096) {
									doc.setValue(documentation.substring(0, 4096));
									String name = caseActivity.getName();
									String[] params = new String[1];
									params[0] = name;
									LoggingUtil.logWarning(MessageKeys.TRUNCATED_ANNOTATION_WARNING, params, null);	
								} else {
									docCopy.setValue(documentation);
								}
								link.getExtensibilityElements().add(docCopy);
							}
							if (desc != null) {
								// need to make a copy of the doc because there could be multiple links going out from the same branch
								Description descCopy = BPELPlusFactory.eINSTANCE.createDescription();
								String descrip = desc.getValue();
								if (descrip.length() > 254) {
									descCopy.setValue(descrip.substring(0, 254));
//d160626									String name = caseActivity.getName();
//									String[] params = new String[1];
//									if (name == null) {
//										params[0] = caseActivity.getElement().getLocalName(); 
//									} else {
//										params[0] = name;
//									}
//									LoggingUtil.logWarning(MessageKeys.TRUNCATED_DESCRIPTION_WARNING, params, null);	
								} else {
									descCopy.setValue(descrip);
								}
								link.getExtensibilityElements().add(descCopy);
							}
						}
						
						sourcesToMove.add(source);
					}
				}

				for (Iterator moveIt = sourcesToMove.iterator(); moveIt.hasNext(); ){
					((Source)moveIt.next()).setActivity(empty);
				}
			} else {
				if (caseActivity instanceof AlternativeActivity) {
					caseActivity = transformAlternativeActivity((AlternativeActivity)caseActivity);
				}
				// Create a link that connects the Empty (representing the switch) to the caseActivity
				Link link = BPELFactory.eINSTANCE.createLink();
				Source src = BPELFactory.eINSTANCE.createSource();
				Target tar = BPELFactory.eINSTANCE.createTarget();
				src.setActivity(empty);
				tar.setActivity(caseActivity);
				link.getSources().add(src);
				link.getTargets().add(tar);
				link.setName(empty.getName()+"To"+caseActivity.getName());
				if (caseCondition != null) {
					src.getExtensibilityElements().add(caseCondition);
				}
				if (doc != null) {
					link.getExtensibilityElements().add(doc);
				}
				if (desc != null) {
					link.getExtensibilityElements().add(desc);
				}				
				// add the activity to the nearest Flow
				EObject container=bpelSwitch.eContainer();
				if (container instanceof Flow) {
					((Flow)container).getActivities().add(caseActivity);
				} else if (container instanceof Sequence) {
					((Sequence)container).getActivities().add(caseActivity);
				}
				
				Flow flow=null;
				while (container!=null) {
					if (container instanceof Flow) {
						flow = (Flow)container;
						break;
					}
					container = container.eContainer();
				}
				if (flow != null) {
					Links links = flow.getLinks();
					if (links == null) {
						links = BPELFactory.eINSTANCE.createLinks();
						flow.setLinks(links);
					}
					links.getChildren().add(link);
				}
			}

		}		

		/* Replace the <switch> with the <empty> in the the parent container */
		if (bpelSwitch.eContainer().eGet(bpelSwitch.eContainmentFeature()) instanceof List){
			List activityList = (List)bpelSwitch.eContainer().eGet(bpelSwitch.eContainmentFeature());
			int index = activityList.indexOf(bpelSwitch);
			activityList.add(index, empty);
			/* The bpelSwitch has shifted up one in the list -- remove it from its new position */
			activityList.remove(index + 1);
		} else {
			/* If the container isn't a list, just set the new containment value directly */
			bpelSwitch.eContainer().eSet(bpelSwitch.eContainmentFeature(), empty);
		}
		if (BpelOptionsUtil.generateEmptyActivityForControlNode()) {
			BpelOptimizationUtil.registerNonOptimizableElement(getContext(),empty);
		}
		/* Put the empty in the map regardless of whether it will be optimized out later */
		fSwitchToEmptyMap.put(bpelSwitch, empty);			
		return empty;
	}

	/**
	 * Returns the BPEL Empty activity that was created to represent the
	 * BPEL Switch activity in the case where Switch is not desired.
	 * 
	 * @param originalSwitch The original Switch activity
	 * @return The Empty created for the switch, or null if no Empty was created
	 */
	public Empty getDecisionActivity(Switch originalSwitch){
	    return (Empty)fSwitchToEmptyMap.get(originalSwitch);
	}

	private Activity transformSwitchToBpelpSwitch(Switch bpelSwitch){
	
		Case switchCase = null;
		for (Iterator it = bpelSwitch.getCases().iterator(); it.hasNext(); ){
			switchCase = (Case)it.next();
			transformExpression(switchCase);
			// remove unsupported extensibility elements
			for (int i=switchCase.getExtensibilityElements().size()-1; i>=0; i--){
				Object exElement = switchCase.getExtensibilityElements().get(i);
				if (exElement instanceof Documentation ||
						exElement instanceof Description ||
						exElement instanceof DisplayName	
					) {
					switchCase.getExtensibilityElements().remove(i);
				}
			}
			
		}		
		return bpelSwitch;
	}

//	private Activity transformSwitch(Switch bpelSwitch){
//		Sequence sequence = BPELFactory.eINSTANCE.createSequence();
//		sequence.setName(bpelSwitch.getName());
//		sequence.getTargets().addAll(bpelSwitch.getTargets());
//		sequence.getSources().addAll(bpelSwitch.getSources());
//		sequence.setJoinCondition(bpelSwitch.getJoinCondition());
//		sequence.getEExtensibilityElements().addAll(bpelSwitch.getExtensibilityElements());
//		
//		Case switchCase = null;
//		Activity caseActivity = null;
//		for (Iterator it = bpelSwitch.getCases().iterator(); it.hasNext(); ){
//			switchCase = (Case)it.next();
//			caseActivity = switchCase.getActivity();
//			/* Set the decision condition as the join condition on the activity, for completeness... */
//			caseActivity.setJoinCondition(switchCase.getCondition());
//			
//			ExtensibilityExpression caseCondition = null;
//			for (int i=0; i<switchCase.getExtensibilityElements().size(); i++){
//				if (switchCase.getExtensibilityElements().get(i) instanceof ExtensibilityExpression){
//					/* This must be the switch case condition, which we will translate into a 
//					 * transition condition on the case's contained activity */
//					caseCondition = (ExtensibilityExpression)switchCase.getExtensibilityElements().get(i);
//					switchCase.getExtensibilityElements().remove(caseCondition);
//					i--;
//				} else {
//					/* Propagate anything that's not the switch case condition to the dest activity */
//					switchCase.getActivity().getExtensibilityElements().add(switchCase.getExtensibilityElements().get(i));
//					i--;
//				}
//			}
//
//			/* Now find the existing transition condition for each source on the activity, if there is one.	 */
//			Source source = null;
//			ExtensibilityExpression transCondition = null;
//			Object element = null;
//			for (Iterator srcIt = caseActivity.getSources().iterator(); srcIt.hasNext(); ){
//				source = (Source)srcIt.next();
//				for (Iterator it2 = source.getExtensibilityElements().iterator(); it2.hasNext(); ){
//					element = it2.next();
//					if (element instanceof ExtensibilityExpression 
//						&& !(element instanceof JoinExtensibilityExpression)){
//	
//						transCondition = (ExtensibilityExpression)element;
//					}
//				}
//	
//				/* Remove the old transition condition and update it with the new one */
//				source.getExtensibilityElements().remove(transCondition);
//				/* Create a copy of the case condition, so we can use it in multiple places */
//				source.getExtensibilityElements().add(updateTransitionCondition(transCondition, 
//													 (ExtensibilityExpression)cloneExpression(caseCondition)));
//			}
//
//			/* Move the case activity from the case statement and into the newly created sequence */			
//			sequence.getActivities().add(caseActivity);
//
//		}		
//
//		/* Replace the <switch> with the <sequence> in the the parent container */
//		if (bpelSwitch.eContainer().eGet(bpelSwitch.eContainmentFeature()) instanceof List){
//			List activityList = (List)bpelSwitch.eContainer().eGet(bpelSwitch.eContainmentFeature());
//			int index = activityList.indexOf(bpelSwitch);
//			activityList.add(index, sequence);
//			/* The bpelSwitch has shifted up one in the list -- remove it from its new position */
//			activityList.remove(index + 1);
//		} else {
//			/* If the container isn't a list, just set the new containment value directly */
//			bpelSwitch.eContainer().eSet(bpelSwitch.eContainmentFeature(), sequence);
//		}
//		
//		return sequence;
//	}

	/**
	 * Performs a cloning of an expression (recursively)
	 * 
	 * @param oldExp The old expression object (or part of an expression object)
	 * @return EObject The cloned version of the oldExp object given as a parameter
	 */
	private EObject cloneExpression(EObject oldExp){
		
		/* Create a copy of the same type of object */
		EObject newExp = ((EObject)oldExp).eClass().getEPackage().getEFactoryInstance().create(oldExp.eClass());
			
		/* For the given object, go through all the structural features it has, and duplicate them,
		 * as long as the structural feature is not a containment feature, since they will be done recursively */
		for (int i=0; i<oldExp.eClass().getEAllAttributes().size(); i++){
			newExp.eSet( (EAttribute)newExp.eClass().getEAllAttributes().get(i), 
						 oldExp.eGet( (EAttribute)oldExp.eClass().getEAllAttributes().get(i)) );
		}

		EReference eRef = null;
		EReference eNewRef = null;
		for (int i=0; i<oldExp.eClass().getEAllReferences().size(); i++){
			eRef = (EReference)oldExp.eClass().getEAllReferences().get(i);
			eNewRef = (EReference)newExp.eClass().getEAllReferences().get(i);
			if (oldExp.eGet(eRef) instanceof EObject && oldExp.equals(((EObject)oldExp.eGet(eRef)).eContainer())){
				newExp.eSet(eNewRef, cloneExpression((EObject)oldExp.eGet(eRef)));
			} else if (oldExp.eGet(eRef) instanceof EList){
				for (int j=0; j<((EList)oldExp.eGet(eRef)).size(); j++){
					((EList)newExp.eGet(eNewRef)).add(j, cloneExpression((EObject)((EList)oldExp.eGet(eRef)).get(j)));
				}
				
			} else {
				newExp.eSet(eNewRef, oldExp.eGet(eRef));
			}
		}
		
		return newExp;
		
	}

	/**
	 * Creates a new transition condition, built by logically ANDing the two paramter conditions
	 * together.  If one of the parameters is null, the other becomes the complete transition 
	 * condition. 
	 *  
	 * @param transCondition - The original transition condition
	 * @param caseCondition - The part of the transition condition to AND to the end
	 * @return ExtensibilityExpression - The object containing the new transition condition
	 */
	private ExtensibilityExpression updateTransitionCondition(ExtensibilityExpression transCondition, ExtensibilityExpression caseCondition){
		
		if (transCondition == null){
			return caseCondition;
		} else if (caseCondition == null){
			return transCondition;
		} else {
			/* Neither is null, so return the logical AND of the two conditions */
			
			BinaryLogicalBooleanExpression boolExpr = ModelFactory.eINSTANCE.createBinaryLogicalBooleanExpression();						
			boolExpr.setOperator(LogicalBooleanOperator.AND_LITERAL);
			boolExpr.setFirstOperand((Expression)transCondition.getExpression());
			boolExpr.setSecondOperand((Expression)caseCondition.getExpression());
			
			ExtensibilityExpression newTransCondition = AbstractbpelFactory.eINSTANCE.createExtensibilityExpression();
			newTransCondition.setExpression(boolExpr);
			return newTransCondition;
		}		
	}

	/**
	 * Applies the flattening algorithm to the given abstract bpel object, removing children
	 * <scpoe> and <flow> objects from parent <flow>s to leave a single top-level <flow>
	 * in the process.
	 * 
	 * <p>All child <flow> and <scope>s are removed from the parent <flow> given as a parameter</p>
	 * 
	 * @param absBpelObject The abstract bpel object previously identified as requiring flattening.
	 */
	private void flattenProcess(Flow parentFlow){
		Activity childActivity = null;

		/* Working from the front of the list means that we will replace all nested flows in one shot, since the 
		 * activities that replace a child flow will then be searched for flows themselves 	 */
		for (int i=0; i<parentFlow.getActivities().size(); i++){

			childActivity = (Activity)parentFlow.getActivities().get(i);
			if (childActivity instanceof Flow){
				if (BpelpFlags.needToFlattenFlow) // Ray 04 Feb 04: We may want to flatten flows ...
				{
					parentFlow.getActivities().remove(childActivity);
					parentFlow.getActivities().addAll(i, ((Flow)childActivity).getActivities());
					if (parentFlow.getLinks() != null && ((Flow)childActivity).getLinks() != null) {				
						parentFlow.getLinks().getChildren().addAll(((Flow)childActivity).getLinks().getChildren());
					}
					/* Re-check the current position in the list, since we just inserted a new element */
					i--;
				}
			} else if (childActivity instanceof Scope){
				if (BpelpFlags.needToFlattenScope) // Ray 04 Feb 04: WPC team said Scope will be supported if there is no crossing links.  So we may not want to flatten scope (and the flow of the scope)
				{
					parentFlow.getActivities().remove(childActivity);
					/* Find the first non-<scope> child to replace the first <scope> with */
					do {
						childActivity = ((Scope)childActivity).getActivity();
					} while (childActivity instanceof Scope);
				
					if (childActivity instanceof Flow){
						parentFlow.getActivities().addAll(i, ((Flow)childActivity).getActivities());
						if (parentFlow.getLinks() != null && ((Flow)childActivity).getLinks() != null) {				
							parentFlow.getLinks().getChildren().addAll(((Flow)childActivity).getLinks().getChildren());
						}
					} else {
						parentFlow.getActivities().add(i, childActivity);
					}

					/* Re-check the current position in the list, since we just inserted a new element */
					i--;
				} else {
					// Ray: add the scope in the list to fix crossing link later
					//scopes.add(childActivity);	
					crossLinkAct.add(childActivity);				
				}
			} else if (childActivity instanceof While) {
				//whiles.add(childActivity);
				crossLinkAct.add(childActivity);				
			}
		}			

	}		

	/**
	 * Fix the crossing links: 
	 * 1) link coming from an activity outside of the scope to an activity inside the Flow of the scope
	 * 2) link coming from an activity inside of the flow of the scope to an activity outside of the scope
	 * @param scope
	 */
	private void fixCrossingLinksInStructuredActivity (Activity parent, Activity structuredActivity) {
		if (structuredActivity instanceof Flow) {
			List activities = ((Flow)structuredActivity).getActivities();
			Activity activity=null;
			ArrayList srcToBeFixed = new ArrayList();
			ArrayList tarToBeFixed = new ArrayList();
			for (Iterator it=activities.iterator();it.hasNext();) {
				activity = (Activity)it.next();
				//migration 11.09
			//	for (Iterator srcIt = activity.getSources().iterator(); srcIt.hasNext();) {
				if(activity.getSources() != null){
					for (Iterator srcIt = activity.getSources().getChildren().iterator(); srcIt.hasNext();) {
						Source src = (Source)srcIt.next();
						Link link = src.getLink();
						if (link.getTargets().isEmpty()) {
							continue; // This case should be an error
						}
						Target target = (Target)link.getTargets().get(0); // assume a link has only one target
						Activity act = target.getActivity();
						if (!activities.contains(act)) {
							EObject structure = getContainingStructure(act);
							// Make sure the links is not going inside the scope nested within a scope
							if (structure!=null && !activities.contains(structure)) {
								srcToBeFixed.add(src);
							} else {
								// Set the link back to the flow in the scope
								setLinkToFlow((Flow)structuredActivity,link);
							}
						} else {
							// Set the link back to the flow in the scope
							setLinkToFlow((Flow)structuredActivity,link);
						}
					}
				}
				//migration AR 11.09
				//for (Iterator tarIt = activity.getTargets().iterator(); tarIt.hasNext();) {
				if(activity.getTargets() != null){
					for (Iterator tarIt = activity.getTargets().getChildren().iterator(); tarIt.hasNext();) {
						Target tar = (Target)tarIt.next();
						Link link = tar.getLink();
						Source src = (Source)link.getSources().get(0); // assume a link has only one source
						Activity act = src.getActivity();
						if (!activities.contains(act)) {
							EObject structure = getContainingStructure(act);
							// Make sure the links is not going inside the scope nested within a scope
							if (structure!=null && !activities.contains(structure)) {
								tarToBeFixed.add(tar);
							}  else {
								// Set the link back to the flow in the scope
								setLinkToFlow((Flow)structuredActivity,link);
							}
						}  else {
							// Set the link back to the flow in the scope
							setLinkToFlow((Flow)structuredActivity,link);
						}
					}
				}
			}
			Source nextSrc;
			Link duplicatedLink;
			for (Iterator it = srcToBeFixed.iterator(); it.hasNext();) {
				nextSrc=(Source)it.next();
				duplicatedLink = findDuplicatedLink(nextSrc,parent);
				nextSrc.setActivity(parent);
				if (duplicatedLink!=null) {
					ArrayList linksNeedConsolidation = new ArrayList();
					linksNeedConsolidation.add(duplicatedLink);
					linksNeedConsolidation.add(nextSrc.getLink());
					//TODO AR DEC 20
					//BpelOptimizationUtil.consolidateLinks(linksNeedConsolidation);
					BpelOptimizationUtil.consolidateLinks(linksNeedConsolidation);
				}
			}
			Target nextTar;
			for (Iterator it = tarToBeFixed.iterator(); it.hasNext();) {
				nextTar=(Target)it.next();
				duplicatedLink = findDuplicatedLink(nextTar,parent);				
				nextTar.setActivity(parent);
				if (duplicatedLink!=null) {
					ArrayList linksNeedConsolidation = new ArrayList();
					linksNeedConsolidation.add(duplicatedLink);
					linksNeedConsolidation.add(nextTar.getLink());
					//TODO AR 
					//BpelOptimizationUtil.consolidateLinks(linksNeedConsolidation);
					BpelOptimizationUtil.consolidateLinks(linksNeedConsolidation);
				}
			}			
		}
	}

	
	/**
	 * Fix the crossing links: 
	 * 1) link coming from an activity outside of the scope to an activity inside the Flow of the scope
	 * 2) link coming from an activity inside of the flow of the scope to an activity outside of the scope
	 * @param scope
	 */
	/*
	private void fixCrossingLinksInScope (Scope scope) {
		Activity structuredActivity = scope.getActivity();
		if (structuredActivity instanceof Flow) {
			List activities = ((Flow)structuredActivity).getActivities();
			Activity activity=null;
			ArrayList srcToBeFixed = new ArrayList();
			ArrayList tarToBeFixed = new ArrayList();
			for (Iterator it=activities.iterator();it.hasNext();) {
				activity = (Activity)it.next();
				for (Iterator srcIt = activity.getSources().iterator(); srcIt.hasNext();) {
					Source src = (Source)srcIt.next();
					Link link = src.getLink();
					if (link.getTargets().isEmpty()) {
						continue; // This case should be an error
					}
					Target target = (Target)link.getTargets().get(0); // assume a link has only one target
					Activity act = target.getActivity();
					if (!activities.contains(act)) {
						EObject structure = getContainingStructure(act);
						// Make sure the links is not going inside the scope nested within a scope
						if (structure!=null && !activities.contains(structure)) {
							srcToBeFixed.add(src);
						} else {
							// Set the link back to the flow in the scope
							setLinkToFlow((Flow)structuredActivity,link);
						}
					} else {
						// Set the link back to the flow in the scope
						setLinkToFlow((Flow)structuredActivity,link);
					}
				}
				for (Iterator tarIt = activity.getTargets().iterator(); tarIt.hasNext();) {
					Target tar = (Target)tarIt.next();
					Link link = tar.getLink();
					Source src = (Source)link.getSources().get(0); // assume a link has only one source
					Activity act = src.getActivity();
					if (!activities.contains(act)) {
						EObject structure = getContainingStructure(act);
						// Make sure the links is not going inside the scope nested within a scope
						if (structure!=null && !activities.contains(structure)) {
							tarToBeFixed.add(tar);
						}  else {
							// Set the link back to the flow in the scope
							setLinkToFlow((Flow)structuredActivity,link);
						}
					}  else {
						// Set the link back to the flow in the scope
						setLinkToFlow((Flow)structuredActivity,link);
					}
				}
			}
			Source nextSrc;
			Link duplicatedLink;
			for (Iterator it = srcToBeFixed.iterator(); it.hasNext();) {
				nextSrc=(Source)it.next();
				duplicatedLink = findDuplicatedLink(nextSrc,scope);
				nextSrc.setActivity(scope);
				if (duplicatedLink!=null) {
					ArrayList linksNeedConsolidation = new ArrayList();
					linksNeedConsolidation.add(duplicatedLink);
					linksNeedConsolidation.add(nextSrc.getLink());
					consolidateLinks(linksNeedConsolidation);
				}
			}
			Target nextTar;
			for (Iterator it = tarToBeFixed.iterator(); it.hasNext();) {
				nextTar=(Target)it.next();
				duplicatedLink = findDuplicatedLink(nextTar,scope);				
				nextTar.setActivity(scope);
				if (duplicatedLink!=null) {
					ArrayList linksNeedConsolidation = new ArrayList();
					linksNeedConsolidation.add(duplicatedLink);
					linksNeedConsolidation.add(nextTar.getLink());
					consolidateLinks(linksNeedConsolidation);
				}
			}			
		}
	}
*/
	private Link findDuplicatedLink(Target tar, Activity newTarAct) {
		Activity srcAct=null;
		if (!tar.getLink().getSources().isEmpty()) {
			srcAct = ((Source)tar.getLink().getSources().get(0)).getActivity(); // assume there is always one target per link
		}
		if (srcAct != null) {
			//migration AR 11.09
		//	List tars = newTarAct.getTargets();
			List tars = newTarAct.getTargets().getChildren();
			Target nextTar;
			for (Iterator it=tars.iterator();it.hasNext();) {
				nextTar = (Target)it.next();
				if (!nextTar.getLink().getSources().isEmpty()) {
					if (((Source)nextTar.getLink().getSources().get(0)).getActivity()==srcAct) {
						return nextTar.getLink();
					}
				}
			}
		}
		return null;
	}
	
	private Link findDuplicatedLink(Source src, Activity newSrcAct) {
		Activity tarAct=null;
		if (!src.getLink().getTargets().isEmpty()) {
			tarAct = ((Target)src.getLink().getTargets().get(0)).getActivity(); // assume there is always one target per link
		}
		if (tarAct != null) {
			//migration AR 11.09
			//List srcs = newSrcAct.getSources();
			if (newSrcAct.getSources()!=null) {
				List srcs = newSrcAct.getSources().getChildren();
				Source nextSrc;
				for (Iterator it=srcs.iterator();it.hasNext();) {
					nextSrc = (Source)it.next();
					if (!nextSrc.getLink().getTargets().isEmpty()) {
						if (((Target)nextSrc.getLink().getTargets().get(0)).getActivity()==tarAct) {
							return nextSrc.getLink();
						}
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * Set the link to the flow
	 * @param flow
	 * @param link
	 */
	private void setLinkToFlow (Flow flow, Link link) {
		if (flow.getLinks()==null)
		{
			Links links = BPELFactory.eINSTANCE.createLinks();
			flow.setLinks(links);
		}
		flow.getLinks().getChildren().add(link);		
	}
	
	/**
	 * Find the structure container (Scope or Process) of the activity
	 * @param act
	 * @return
	 */
	private EObject getContainingStructure(Activity act) {
		EObject container = act.eContainer();
		while (container!=null && (!(container instanceof Scope) && !(container instanceof Process))) {
			container = container.eContainer();
		}
		return container;
	}
    
//	AZ WMT 20070321 migrate to 3.2 platform added for rule handler
	public List getTransformableProcessSwitch() {
		List switches = new LinkedList();
		Set s2eMap = fSwitchToEmptyMap.keySet();
		if(s2eMap != null && !s2eMap.isEmpty()){
			Iterator seIt = s2eMap.iterator();
			while(seIt.hasNext()){
				Object key = seIt.next();
				if(key instanceof Switch){
					switches.add(key);
				}
			}
		}
		return switches;
	}

	public List getWhileWithConditionList() {
		return whileWithConditionList;
	}
} //BpelpRuleImpl
