package com.ibm.btools.te.ilm.heuristics.bpelp.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDParticle;

import com.ibm.btools.expression.function.CoreFunctions;
import com.ibm.btools.expression.language.serialization.ExpressionSerializer;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BinaryMathematicalOperator;
import com.ibm.btools.expression.model.BinaryNumericExpression;
import com.ibm.btools.expression.model.BinaryOperatorExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.DateLiteralExpression;
import com.ibm.btools.expression.model.DateTimeLiteralExpression;
import com.ibm.btools.expression.model.DurationLiteralExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.LiteralExpression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.NegationExpression;
import com.ibm.btools.expression.model.NotExpression;
import com.ibm.btools.expression.model.NumericLiteralExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.Step;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.expression.model.TimeLiteralExpression;
import com.ibm.btools.expression.model.UnaryOperatorExpression;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.model.abstractbpel.ExtensibilityExpression;
import com.ibm.wbit.bpelpp.BPELPVariable;


public class JavaExpressionSerializer extends ExpressionSerializer implements JavaConstants{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private static boolean isMultiple = false;
	private static boolean isStandAloneSelect = false;
	private static String prefix = "";
	private static String containerPrefix = "";
	private static String variableName = "";
	private static int numCountFunctions = 0;
	private static int numSumFunctions = 0;
	private static int numExistsFunctions = 0;
	private static int numEveryFunctions = 0;
	private static int numBooleanExpr = 0;
	private static int numTimeExpr = 0;	
	private static boolean isWithinFunction = false;	
	private static boolean hasPreviousPath = false;
	private static boolean isGetAllExpr = false;
	private static String forLoopVariable = null;
	private static ReferenceStep lastRefStep = null;

	private Expression countConditionExpression=null;
	private String serializedCountConditionExpression="";
	/**
	 * Serializes an expression into an equivalent Java String
	 * @param expression the expression
	 * @return String - the Java String
	 */
	public String serialize(Expression expression) 
	{		
		LoggingUtil.traceEntry(this, "serialize");
		String javaStr = null;
		if (expression instanceof LiteralExpression) {		
			javaStr =  serialize((LiteralExpression)expression);		
		} else if (expression instanceof BinaryOperatorExpression) {
			javaStr =  serialize((BinaryOperatorExpression)expression);
		} else if (expression instanceof UnaryOperatorExpression) {
			javaStr =  serialize((UnaryOperatorExpression)expression);
		} else if (expression instanceof FunctionExpression) {			
			javaStr =  serialize((FunctionExpression)expression);
		} else if (expression instanceof ModelPathExpression) {
			javaStr =  serialize((ModelPathExpression)expression);
		} 
		if (isTopLevelExpression(expression))
			resetAttributes();
		LoggingUtil.traceExit(this, "serialize");							
		return javaStr;
	}
	
	
	/**
	 * Serializes a LiteralExpression into an equivalent Java String
	 * @param litExpression the LiteralExpression
	 * @return String - the Java String
	 */	
	private String serialize(LiteralExpression litExpression) 
	{	
		StringBuffer sb = new StringBuffer();		
		if (litExpression instanceof StringLiteralExpression)
		{			
			sb.append(STRING_DECOR);
			sb.append(((StringLiteralExpression)litExpression).getStringSymbol());
			sb.append(STRING_DECOR);
		}else if (litExpression instanceof BooleanLiteralExpression) {	
			if (isTopLevelExpression(litExpression))		
				sb.append(RETURN);
			if (((BooleanLiteralExpression)litExpression).getBooleanSymbol().booleanValue()) 
				sb.append(TRUE_VALUE);
			else 
				sb.append(FALSE_VALUE);	
			if (isTopLevelExpression(litExpression))			
				sb.append(SEMICOLON);			
		}else if (litExpression instanceof NumericLiteralExpression){
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_START);			
			if (((NumericLiteralExpression)litExpression).getNumericSymbol().indexOf(".")==-1
					&& Long.valueOf(((NumericLiteralExpression)litExpression).getNumericSymbol()).longValue()>Integer.MAX_VALUE){
				sb.append("new Long(\"");
				sb.append(((NumericLiteralExpression)litExpression).getNumericSymbol());
				sb.append("\").longValue()");				
			}else if (((NumericLiteralExpression)litExpression).getNumericSymbol().indexOf(".")!=-1
			&& Double.valueOf(((NumericLiteralExpression)litExpression).getNumericSymbol()).doubleValue()>Float.MAX_VALUE){
				sb.append("new Double(\"");
				sb.append(((NumericLiteralExpression)litExpression).getNumericSymbol());
				sb.append("\").doubleValue()");
			}else{
				sb.append(((NumericLiteralExpression)litExpression).getNumericSymbol());	
			}
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_END);		
		/*			
		}else if (litExpression instanceof TimeLiteralExpression){
			sb.append("new GregorianCalendar(");
			sb.append("new GregorianCalendar().get(Calendar.YEAR)");			
			sb.append(COMMA);
			sb.append("new GregorianCalendar().get(Calendar.MONTH)");
			sb.append(COMMA);
			sb.append("new GregorianCalendar().get(Calendar.DATE)");	
			sb.append(COMMA);
			sb.append(((TimeLiteralExpression)litExpression).getHour());
			sb.append(COMMA);
			sb.append(((TimeLiteralExpression)litExpression).getMinute());
			sb.append(COMMA);
			sb.append(((TimeLiteralExpression)litExpression).getSecond());	
			sb.append(")");
		*/			
		}else if (litExpression instanceof DateLiteralExpression){
			sb.append("new java.util.GregorianCalendar(");
			sb.append(((DateLiteralExpression)litExpression).getYear());
			sb.append(COMMA);
			sb.append(((DateLiteralExpression)litExpression).getMonth()-1);
			sb.append(COMMA);
			sb.append(((DateLiteralExpression)litExpression).getDay());			
			sb.append(")");	
		}else if (litExpression instanceof DateTimeLiteralExpression){
			sb.append("new java.util.GregorianCalendar(");
			sb.append(((DateTimeLiteralExpression)litExpression).getYear());
			sb.append(COMMA);
			sb.append(((DateTimeLiteralExpression)litExpression).getMonth()-1);
			sb.append(COMMA);
			sb.append(((DateTimeLiteralExpression)litExpression).getDay());
			sb.append(COMMA);
			sb.append(((DateTimeLiteralExpression)litExpression).getHour());
			sb.append(COMMA);
			sb.append(((DateTimeLiteralExpression)litExpression).getMinute());
			sb.append(COMMA);
			sb.append(((DateTimeLiteralExpression)litExpression).getSecond());						
			sb.append(").getTime()");
		}else if (litExpression instanceof DurationLiteralExpression){
			//Logging - we do not support duration expression
		}
		/*
		else if (litExpression instanceof IntegerLiteralExpression) 
		{
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_START);
			sb.append(((IntegerLiteralExpression)litExpression).getIntegerSymbol().intValue());
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_END);
		} 
		else if (litExpression instanceof RealLiteralExpression) 
		{
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_START);
			sb.append(((RealLiteralExpression)litExpression).getRealSymbol().floatValue());
			if (areParenthesesRequired(litExpression))
				sb.append(PARENTHESIS_END);
		} 
		*/		
		else 
		{
		
		}
		return sb.toString();
	}

	/**
	 * Check to see if we need parentheses for the NegationExpression
	 * @param litExpression the expression
	 * @return boolean - true or false whether we need parentheses
	 */	
	private boolean areParenthesesRequired(LiteralExpression litExpression) 
	{		
		EObject container = litExpression.eContainer();	
		if (container instanceof NegationExpression) {
			/*
			if (litExpression instanceof IntegerLiteralExpression) {
				if (((IntegerLiteralExpression)litExpression).getIntegerSymbol().intValue() < 0) {
					return true;
				}
			}
			if (litExpression instanceof RealLiteralExpression) {
				if (((RealLiteralExpression)litExpression).getRealSymbol().floatValue() < 0) {
					return true;
				}
			}
			*/
			return true;
		}
		return false;
	}	

	/**
	 * Serializes a UnaryOperatorExpression into an equivalent Java String
	 * @param unaryExpression the UnaryOperatorExpression
	 * @return String - the Java String
	 */	
	private String serialize(UnaryOperatorExpression unaryExpression) 
	{	
		if (unaryExpression instanceof NotExpression) {
			return serialize((NotExpression)unaryExpression);
		} else if (unaryExpression instanceof NegationExpression) {
			return serialize((NegationExpression)unaryExpression);
		} else {
			return null;
		}
	}

	/**
	 * Serializes a NotExpression into an equivalent Java String
	 * @param notExpression the NotExpression
	 * @return String - the Java String
	 */
	private String serialize(NotExpression notExpression)  
	{	
		StringBuffer sb = new StringBuffer();			
		String tempString;		
		/* recursively serialize argument expression */
		tempString = serialize(notExpression.getExpression());
		String varName = getVariableName();
		/*
		if ((!isTopLevelExpression(notExpression)) && isMultiple())
		{
			// Creating a java variable
			setVariableName("tempBool" + String.valueOf(numBooleanExpr++));
			if (notExpression.getEvaluatesToType().equalsIgnoreCase("Integer"))				
				sb.append("int ");
			else if (notExpression.getEvaluatesToType().equalsIgnoreCase("Float"))
				sb.append("float ");			
			sb.append(getVariableName());
			sb.append(" = 0;");
			sb.append(NEWLINE);
		}
		*/
		if (isMultiple())
			/* Put the multiple Java Statements first */
			sb.append(tempString);		
		if (isTopLevelExpression(notExpression))
			sb.append(RETURN);			
			
		StringBuffer tempBuffer = new StringBuffer();		
		tempBuffer.append(NOT_FUNC);
		tempBuffer.append(PARENTHESIS_START);
		if (isStandAloneSelect()){
			tempBuffer.append(getPrefix());
			setStandAloneSelect(false);
		}			
		else if (isMultiple())			
			tempBuffer.append(varName);				
		else
			tempBuffer.append(tempString);
		tempBuffer.append(PARENTHESIS_END);		
		if (isTopLevelExpression(notExpression)){
			tempBuffer.append(SEMICOLON);						
		}
		if (isMultiple())
			setVariableName(tempBuffer.toString());
		else
			sb.append(tempBuffer.toString());
		if (isTopLevelExpression(notExpression)){
			setMultiple(false);							
		}
				
		
		return sb.toString();
	}

	/**
	 * Serializes a NegationExpression into an equivalent Java String
	 * @param negExpression the NegationExpression
	 * @return String - the Java String
	 */
	private String serialize(NegationExpression negExpression) 
	{
		StringBuffer sb = new StringBuffer();			
		String tempString;
		/* recursively serialize argument expression */
		tempString = serialize(negExpression.getExpression());
		String varName = getVariableName();
		/*
		if (isMultiple())
		{
			// Creating a java variable
			setVariableName("tempBool" + String.valueOf(numBooleanExpr++));
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(" = false;");
			sb.append(NEWLINE);		
			// Put the multiple Java Statements first
			sb.append(tempString);
			sb.append(getVariableName());
			sb.append(EQUALS);
		}	
		*/
		StringBuffer tempBuffer = new StringBuffer();
		if (isMultiple())
			sb.append(tempString);					
		tempBuffer.append(NEGATION_OP);		
		tempBuffer.append(PARENTHESIS_START);		
		if (isStandAloneSelect()){
			tempBuffer.append(getPrefix());			
			setStandAloneSelect(false);
		}
		else if (isMultiple())					
			tempBuffer.append(varName);
		else
			tempBuffer.append(tempString);
		tempBuffer.append(PARENTHESIS_END);
		
		if (isMultiple())
			setVariableName(tempBuffer.toString());
		else
			sb.append(tempBuffer.toString());		
		return sb.toString();
	}

	/**
	 * Serializes a BinaryOperatorExpression into an equivalent Java String
	 * @param binExpression the BinaryOperatorExpression
	 * @return String - the Java String
	 */	
	private String serialize(BinaryOperatorExpression binExpression) 
	{	
		if (binExpression instanceof BinaryLogicalBooleanExpression) {
			return serialize((BinaryLogicalBooleanExpression)binExpression);
		} else if (binExpression instanceof BinaryNumericExpression) {
			return serialize((BinaryNumericExpression)binExpression);
		} else if (binExpression instanceof ComparisonExpression) {
			return serialize((ComparisonExpression)binExpression);
		} else {
			return null;
		}
	}

	/**
	 * Serializes a Binary expression into an equivalent Java String
	 * @param expr - the BinaryOperatorExpression to be serialized
	 * @param operator - the String operator for the BinaryOperatorExpression
	 * @return String - the Java String
	 */	
	private String serializeBinary(BinaryOperatorExpression expr, String operator)
	{
		StringBuffer sb = new StringBuffer();
		String tempString;
		StringBuffer tempBuffer = new StringBuffer();					
		tempString = serialize(expr.getFirstOperand());
		String varName = getVariableName();		
		boolean combinedMultiple = false;		
		/*
		//(!(isTopLevelExpression(logicExpression))) && (!isWithinFunction) &&
		if (  isMultiple()){
			setVariableName("tempBool" + String.valueOf(numBooleanExpr++));
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(" = false;");
			sb.append(NEWLINE);							
		}
		*/
		combinedMultiple = isMultiple();
		tempBuffer.append(PARENTHESIS_START);	
		if (isMultiple()){		
			/* Put the multiple Java Statements first */
			sb.append(tempString);	
			//sb.append(getVariableName());
			//sb.append(EQUALS);			
			if (isStandAloneSelect()){
				tempBuffer.append(getPrefix());
				tempBuffer.append(SEMICOLON);
				tempBuffer.append(NEWLINE);	
			}
			else{
				tempBuffer.append(varName);
			//	sb.append(SEMICOLON);
			//	sb.append(NEWLINE);						
			}
			//tempBuffer.append(getVariableName());
		}		
		else{
			tempBuffer.append(tempString);
		}	
		tempBuffer.append(PARENTHESIS_END);
		tempBuffer.append(operator);
		
		setMultiple(false);				
		
		tempString = serialize(expr.getSecondOperand());			
		
		varName = getVariableName();
		StringBuffer tempBuffer2 = new StringBuffer();
		tempBuffer2.append(PARENTHESIS_START);
		// (!(isTopLevelExpression(logicExpression))) && (!isWithinFunction) &&*
		/*
		if (isMultiple()){
			setVariableName("tempBool" + String.valueOf(numBooleanExpr++));
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(" = false;");
			sb.append(NEWLINE);						
			sb.append(tempString);	
			sb.append(getVariableName());
			sb.append(EQUALS);
		}
		*/
		//else if (isMultiple()){ 
		//	tempBuffer2.append(tempString);		
		//}		
		if (isMultiple())
			sb.append(tempString);
		else if (isStandAloneSelect()){
			tempBuffer2.append(getPrefix());
			//tempBuffer2.append(getVariableName());
			//sb.append(SEMICOLON);
			//sb.append(NEWLINE);	
		}			
		if (isMultiple()){	
			tempBuffer2.append(varName);	
			//tempBuffer2.append(getVariableName());
			//sb.append(SEMICOLON);
			//sb.append(NEWLINE);			
		}							
		else
			tempBuffer2.append(tempString);
		tempBuffer2.append(PARENTHESIS_END);
	
		StringBuffer finalBuffer = new StringBuffer();			
		finalBuffer.append(PARENTHESIS_START);
		finalBuffer.append(tempBuffer.toString());
		//finalBuffer.append(PARENTHESIS_START);	
		
		finalBuffer.append(tempBuffer2.toString());
		//finalBuffer.append(PARENTHESIS_END);
		finalBuffer.append(PARENTHESIS_END);		
		
		setMultiple(isMultiple() || combinedMultiple);
		if (isTopLevelExpression(expr)){
			sb.append(RETURN);	
			sb.append(finalBuffer.toString());	
			sb.append(SEMICOLON);
			sb.append(NEWLINE);	
		}		
		else if ( isMultiple() ){
			setVariableName("tempVar" + String.valueOf(numBooleanExpr++));
			// TODO check evaluates to type
			if (expr instanceof BinaryNumericExpression)
				sb.append("int ");
			else
				sb.append("boolean ");
			sb.append(getVariableName());
			if (expr instanceof BinaryNumericExpression)
				sb.append(" = 0;");
			else
				sb.append(" = false;");
			sb.append(NEWLINE);
			sb.append(getVariableName());
			sb.append(EQUALS);
			sb.append(finalBuffer.toString());
			sb.append(SEMICOLON);
			sb.append(NEWLINE);	
		}
		else{
			sb.append(finalBuffer.toString());
		}
		
			
		if (isTopLevelExpression(expr) || isWithinFunction)
			setMultiple(false);			
		
		return sb.toString();

	}
	
	/**
	 * Serializes a BinaryLogicalBooleanExpression into an equivalent Java String
	 * @param logicExpression the BinaryLogicalBooleanExpression
	 * @return String - the Java String
	 */	
	private String serialize(BinaryLogicalBooleanExpression logicExpression)
	{
		String operator = "";
		LogicalBooleanOperator op = logicExpression.getOperator();
		if (LogicalBooleanOperator.AND_LITERAL.equals(op)) {
			operator=AND_OP;
		} else if (LogicalBooleanOperator.OR_LITERAL.equals(op)) {
			operator = OR_OP;
		} else {
			return null;
		}
		return serializeBinary(logicExpression, operator);
	}
	
	/**
	 * Serializes a BinaryNumericExpression into an equivalent Java String
	 * @param numExpression the BinaryNumericExpression
	 * @return String - the Java String
	 */
	private String serialize(BinaryNumericExpression numExpression)  
	{	
		String operator = "";		
		BinaryMathematicalOperator op = numExpression.getOperator();
		if (BinaryMathematicalOperator.ADDITION_LITERAL.equals(op)) {
			operator = ADDITION_OP;
		} else if (BinaryMathematicalOperator.SUBTRACTION_LITERAL.equals(op)) {
			operator = SUBTRACTION_OP;
		} else if (BinaryMathematicalOperator.MULTIPLICATION_LITERAL.equals(op)) {
			operator = MULTIPLICATION_OP;
		} else if (BinaryMathematicalOperator.DIVISION_LITERAL.equals(op)) {
			operator = DIVISION_OP;
		} else if (BinaryMathematicalOperator.MODULUS_LITERAL.equals(op)) {
			operator = MODULUS_OP;
		} else {
			return null;
		}	
		return serializeBinary(numExpression, operator);
	}

	private boolean isStringExpression (Expression expr){
		if (expr instanceof StringLiteralExpression){
			return true;
		}else if (expr instanceof FunctionExpression
					&& "com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData".equals(((FunctionExpression)expr).getDefinition().getFunctionID())){
			List arguments = ((FunctionExpression)expr).getArguments();			
			if (arguments.size()>2)			
			{
				FunctionArgument argument = (FunctionArgument)arguments.get(2);
				Expression argValue = argument.getArgumentValue();
				if (argValue instanceof ModelPathExpression)
				{	
					Object lastStep = ((ModelPathExpression)argValue).getSteps().get(((ModelPathExpression)argValue).getSteps().size()-1);
					if (lastStep instanceof ReferenceStep && ((ReferenceStep)lastStep).getReferencedObject() instanceof XSDParticle){ 
						if (((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent() instanceof XSDElementDeclaration)
							if ( ((XSDElementDeclaration)((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent()).getTypeDefinition().getName().equals("string") ){
								return true;
							}
					}	
				}
			}	
		}	
		return false;
		
	}

	private boolean isDateTimeExpression (Expression expr){
		if (expr instanceof DateLiteralExpression || expr instanceof DateTimeLiteralExpression || expr instanceof TimeLiteralExpression){
			return true;
		}else if (expr instanceof FunctionExpression
					&& "com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData".equals(((FunctionExpression)expr).getDefinition().getFunctionID())){
			List arguments = ((FunctionExpression)expr).getArguments();			
			if (arguments.size()>2)			
			{
				FunctionArgument argument = (FunctionArgument)arguments.get(2);
				Expression argValue = argument.getArgumentValue();
				if (argValue instanceof ModelPathExpression)
				{	
					Object lastStep = ((ModelPathExpression)argValue).getSteps().get(((ModelPathExpression)argValue).getSteps().size()-1);
					if (lastStep instanceof ReferenceStep && ((ReferenceStep)lastStep).getReferencedObject() instanceof XSDParticle){ 
						if (((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent() instanceof XSDElementDeclaration)
							if ( ((XSDElementDeclaration)((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent()).getTypeDefinition().getName().equals("date") 
									|| ((XSDElementDeclaration)((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent()).getTypeDefinition().getName().equals("time")
									|| ((XSDElementDeclaration)((XSDParticle)((ReferenceStep)lastStep).getReferencedObject()).getContent()).getTypeDefinition().getName().equals("dateTime")){
								return true;
							}
					}	
				}
			}	
		}	
		return false;
		
	}
	/**
	 * Serializes a ComparisonExpression into an equivalent Java String
	 * @param compExpression the ComparisonExpression
	 * @return String - the Java String
	 */	
	private String serialize(ComparisonExpression compExpression) 
	{
		//If the first and second Operands are StringLiteralExpression 
		if ( (isStringExpression(compExpression.getFirstOperand() )) 
				|| (isStringExpression(compExpression.getSecondOperand())) )
		 {
			StringBuffer sb = new StringBuffer();
			 // If the operands are of type String, we will never have the multiple fragments 
			 sb.append(stringCompareSerialize(compExpression));
			 return sb.toString();
		 }
		 else if (isDateTimeExpression(compExpression.getFirstOperand()) 
					|| isDateTimeExpression(compExpression.getSecondOperand()) )
		 {
			StringBuffer sb = new StringBuffer();			 
			sb.append(dateTimeCompareSerialize(compExpression));
			return sb.toString();	 	
		 }
		 
		 else
		 {
		 	String operator = "";
			ComparisonOperator op = compExpression.getOperator();
			if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
				operator = EQUAL_TO_OP;
			} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
				operator = NOT_EQUAL_TO_OP;
			} else if (ComparisonOperator.LESS_THAN_LITERAL.equals(op)) {
				operator = LESS_THAN_OP;
			} else if (ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
				operator = LESS_THAN_OR_EQUAL_TO_OP; 
			} else if (ComparisonOperator.GREATER_THAN_LITERAL.equals(op)) {
				operator = GREATER_THAN_OP;
			} else if (ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
				operator = GREATER_THAN_OR_EQUAL_TO_OP;
			} else {
				return null;
			}	
			return serializeBinary(compExpression, operator);	 	
		 }		
	}	
	
	/**
	 * Serializes a String Comparison expression into an equivalent Java String
	 * @param compExpression the expression
	 * @return String - the Java String
	 */
	private String stringCompareSerialize(ComparisonExpression compExpression)
	{
		StringBuffer sb = new StringBuffer();
		if (isTopLevelExpression(compExpression))
			sb.append(RETURN);	
		sb.append(PARENTHESIS_START);	
		/* Java String comparison using s1.compareTo(s2) */
		sb.append(serialize(compExpression.getFirstOperand()));
		sb.append(".compareTo");
		sb.append(PARENTHESIS_START);
		sb.append(serialize(compExpression.getSecondOperand()));
		sb.append(PARENTHESIS_END);
		ComparisonOperator op = compExpression.getOperator();
		if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
			sb.append(STRING_EQUAL_TO_OP);
		} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
			sb.append(STRING_NOT_EQUAL_TO_OP);
		} else if (ComparisonOperator.LESS_THAN_LITERAL.equals(op)) {
			sb.append(STRING_LESS_THAN_OP);
		} else if (ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			sb.append(STRING_LESS_THAN_OR_EQUAL_TO_OP); 
		} else if (ComparisonOperator.GREATER_THAN_LITERAL.equals(op)) {
			sb.append(STRING_GREATER_THAN_OP);
		} else if (ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL.equals(op)) {
			sb.append(STRING_GREATER_THAN_OR_EQUAL_TO_OP);
		} else {
			return null;
		}
		sb.append(PARENTHESIS_END);
		if (isTopLevelExpression(compExpression))
			sb.append(SEMICOLON);		
		return sb.toString();
	}

	private String printTimeCurrentDate(TimeLiteralExpression litExpression){
		StringBuffer sb = new StringBuffer();
		sb.append("new java.util.GregorianCalendar(");
		sb.append("new java.util.GregorianCalendar().get(java.util.Calendar.YEAR)");			
		sb.append(COMMA);
		sb.append("new java.util.GregorianCalendar().get(java.util.Calendar.MONTH)");
		sb.append(COMMA);
		sb.append("new java.util.GregorianCalendar().get(java.util.Calendar.DATE)");	
		sb.append(COMMA);
		sb.append(litExpression.getHour());
		sb.append(COMMA);
		sb.append(litExpression.getMinute());
		sb.append(COMMA);
		sb.append(litExpression.getSecond());	
		sb.append(")");
		return sb.toString();
	}
	
	private String printTimeGivenDate(TimeLiteralExpression litExpression, String date){
		StringBuffer sb = new StringBuffer();
		sb.append("java.util.Calendar c = java.util.Calendar.getInstance();");
		sb.append(NEWLINE);
		sb.append("c.setTime(");
		sb.append(date);
		sb.append(".getTime());");
		sb.append(NEWLINE);
		sb.append("c.set(java.util.Calendar.HOUR, ");
		sb.append(litExpression.getHour());
		sb.append(");");
		sb.append(NEWLINE);
		sb.append("c.set(java.util.Calendar.MINUTE, ");
		sb.append(litExpression.getMinute());
		sb.append(");");
		sb.append(NEWLINE);
		sb.append("c.set(java.util.Calendar.SECOND, ");
		sb.append(litExpression.getSecond());
		sb.append(");");
		sb.append(NEWLINE);	
		return sb.toString();
	}

	private String timeLiteralSerialize(ComparisonExpression cmpExpr){
		StringBuffer sb = new StringBuffer();
		if (cmpExpr.getFirstOperand() instanceof TimeLiteralExpression &&
			cmpExpr.getSecondOperand() instanceof TimeLiteralExpression){
			if (isTopLevelExpression(cmpExpr))
				sb.append(RETURN);	
			sb.append(PARENTHESIS_START);				
			sb.append(printTimeCurrentDate((TimeLiteralExpression)cmpExpr.getFirstOperand()));
			sb.append(printDateTimeOperator(cmpExpr.getOperator()));
			sb.append(PARENTHESIS_START);			
			sb.append(printTimeCurrentDate((TimeLiteralExpression)cmpExpr.getSecondOperand()));
			sb.append(PARENTHESIS_END);
			sb.append(PARENTHESIS_END);
			if (isTopLevelExpression(cmpExpr))
				sb.append(SEMICOLON);	
						
		}else{
			boolean isFirstOperandMPE = false;
			String date = null;
			if (cmpExpr.getSecondOperand() instanceof TimeLiteralExpression){
				isFirstOperandMPE = true;
			}
			setMultiple(true);
			setVariableName("calendar" + String.valueOf(numTimeExpr++));
			if (isFirstOperandMPE){
				date=serialize(cmpExpr.getFirstOperand());			
				sb.append(printTimeGivenDate((TimeLiteralExpression)cmpExpr.getSecondOperand(), date));
			}else{
				date=serialize(cmpExpr.getSecondOperand());
				sb.append(printTimeGivenDate((TimeLiteralExpression)cmpExpr.getFirstOperand(), date));
			}
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(EQUALS);
			if (isFirstOperandMPE){
				sb.append(date);
				sb.append(printDateTimeOperator(cmpExpr.getOperator()));
				sb.append(PARENTHESIS_START);
				sb.append("c.getTime()");
				sb.append(PARENTHESIS_END);
			}else{
				sb.append("c.getTime()");
				sb.append(printDateTimeOperator(cmpExpr.getOperator()));
				sb.append(PARENTHESIS_START);
				sb.append(date);
				sb.append(PARENTHESIS_END);				
			}
			sb.append(SEMICOLON);
			sb.append(NEWLINE);
			if (isTopLevelExpression(cmpExpr)){
				sb.append(RETURN);
				sb.append(getVariableName());
				sb.append(SEMICOLON);		
			}			
		}
		return sb.toString();
	}

	private String printDateTimeOperator(ComparisonOperator op){
		StringBuffer sb = new StringBuffer();
		if (ComparisonOperator.EQUAL_TO_LITERAL.equals(op)) {
			sb.append(DATE_EQUAL);
		} else if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
			sb.append(DATE_EQUAL);
		} else if (ComparisonOperator.IS_AFTER_LITERAL.equals(op)) {
			sb.append(DATE_AFTER);
		} else if (ComparisonOperator.IS_BEFORE_LITERAL.equals(op)) {
			sb.append(DATE_BEFORE);
		} else {
			//LoggingUtil.logWarning(MessageKeys.DATE_TIME_OPERATOR_LITERAL_NOT_SUPPORTED);
			return null;
		}	
		return sb.toString();	
	}

	/**
	 * Serializes a Date, Time and DateTime Comparison expression into an equivalent Java code
	 * @param compExpression the expression
	 * @return String - the Java String
	 */
	private String dateTimeCompareSerialize(ComparisonExpression compExpression)
	{
		if (compExpression.getFirstOperand() instanceof TimeLiteralExpression || compExpression.getSecondOperand() instanceof TimeLiteralExpression){
			return timeLiteralSerialize(compExpression);		
		}else{
			StringBuffer sb = new StringBuffer();
			if (isTopLevelExpression(compExpression))
				sb.append(RETURN);	
			sb.append(PARENTHESIS_START);			
			ComparisonOperator op = compExpression.getOperator();
			if (ComparisonOperator.NOT_EQUAL_TO_LITERAL.equals(op)) {
				sb.append(NOT_FUNC);
			}			
			sb.append(serialize(compExpression.getFirstOperand()));	
			sb.append(printDateTimeOperator(op));

			sb.append(PARENTHESIS_START);
			sb.append(serialize(compExpression.getSecondOperand()));
			sb.append(PARENTHESIS_END);		
			sb.append(PARENTHESIS_END);
			if (isTopLevelExpression(compExpression))
				sb.append(SEMICOLON);		
			return sb.toString();
		}

	}


	/**
	 * Serializes a ModelPathExpression into an equivalent Java String
	 * @param pathExpression the ModelPathExpression
	 * @return String - the Java String
	 */	
	private String serialize(ModelPathExpression pathExpression) 
	{	
		StringBuffer sb = new StringBuffer();	
		List steps = pathExpression.getSteps();		
		StaticStep staticStep;
		ReferenceStep referenceStep;
		String stepName;
		
		for (int i=0; i<steps.size(); i++) 
		{
			if (steps.get(i) instanceof StaticStep) 
			{			
				staticStep = (StaticStep)steps.get(i);			
				stepName = staticStep.getName();
				if (stepName == null) {
					return null;
				}	
				if (i==0)
					sb.append(getPrefix());	
				else	
				sb.append(composeJavaMethodName(stepName));
				if (isHasPreviousPath()){
					setContainerPrefix(sb.toString());
					setHasPreviousPath(false);
				}					
				setPrefix(sb.toString());
			} 
			else if (steps.get(i) instanceof ReferenceStep) 
			{	
				
				if ( !(isGetAllExpr && i==steps.size()-1) ){
					referenceStep = (ReferenceStep)steps.get(i);			
					stepName = referenceStep.getReferenceName();
					//stepName="Variable";
					if (stepName == null) {
						return null;
					}		
				
					if (referenceStep.getReferencedObject() instanceof BPELPVariable) {
						if (i==(steps.size()-1)){						
							sb.append(composeJavaMethodVariableName(stepName, referenceStep, true));
							if (forLoopVariable!=null && lastRefStep==null){
								lastRefStep = referenceStep;
							}
						}
						else{						
							sb.append(composeJavaMethodVariableName(stepName, referenceStep, false));
						}
							
					} else {
						if (i==(steps.size()-1)){						
							sb.append(composeJavaMethodName(referenceStep, true));
							if (forLoopVariable!=null && lastRefStep==null){
								lastRefStep = referenceStep;
							}
						}
						else{						
							sb.append(composeJavaMethodName(referenceStep, false));
						}
					}
					if (isHasPreviousPath()){
						setContainerPrefix(sb.toString());
						setHasPreviousPath(false);
					}							
					setPrefix(sb.toString());			
				}

			} 
			else 			
				return null;
		}		
		return sb.toString();
	}
	
	private String composeJavaLoopCondition(Expression expr) {
		return composeJavaLoopCondition(expr,"i");
	}
	
	private String composeJavaLoopCondition(Expression expr, String iteratorVar) {
		StringBuffer tempBuffer = new StringBuffer();
		// the first argument of select will always be ModelPathExpression unless
		// the expression framework was updated to support Select
		if ((expr instanceof ModelPathExpression) && (!isArray((ModelPathExpression)expr)))
		{
			if (!isJavaPrimitiveType((ModelPathExpression)expr)) {
				tempBuffer.append("if (");
				
				tempBuffer.append(serialize(expr));	
				tempBuffer.append("!=null)");
			}
		} else {
			tempBuffer.append("for (int ");
			tempBuffer.append(iteratorVar);
			tempBuffer.append("=0;");
			tempBuffer.append(iteratorVar);
			tempBuffer.append("<");
			//tempBuffer.append("for (int i=0; i<");

			tempBuffer.append(serialize(expr));	
			tempBuffer.append(getMultiplicityMethod());
			tempBuffer.append(" ;");			
			tempBuffer.append(iteratorVar);
			tempBuffer.append("++)");
		}		
		return tempBuffer.toString();
	}
	
	private String composeJavaInnerLoopCondition(Expression expr, String iteratorVar1, String iteratorVar2) {
		StringBuffer tempBuffer = new StringBuffer();
		// the first argument of select will always be ModelPathExpression unless
		// the expression framework was updated to support Select
		if ((expr instanceof ModelPathExpression) && (!isArray((ModelPathExpression)expr)))
		{
			if (!isJavaPrimitiveType((ModelPathExpression)expr)) {
				tempBuffer.append("if (");
				
				tempBuffer.append(serialize(expr));	
				tempBuffer.append("!=null)");
			}
		} else {
			tempBuffer.append("for (int ");
			tempBuffer.append(iteratorVar2);
			tempBuffer.append("=0;");
			tempBuffer.append(iteratorVar2);
			tempBuffer.append("<");
			
			tempBuffer.append(serialize(expr));		
			tempBuffer.append(getArrayPositionMethod(iteratorVar1));
			
			if (expr instanceof ModelPathExpression){
				List steps = ((ModelPathExpression)expr).getSteps();
				String name = ((Step)steps.get(steps.size()-1)).getName();
				tempBuffer.append(composeJavaMethodName(name));
			}else{
				return null;
			}				
			tempBuffer.append(getMultiplicityMethod());
			tempBuffer.append(" ;");			
			tempBuffer.append(iteratorVar2);
			tempBuffer.append("++)");			
			
		}		
		return tempBuffer.toString();
	}	
	/**
	 * Creates a Java Method call name from a String variable for a BPEL Part or Business Item
	 * @param name the string variable
	 * @return String - the Java Method Call Name
	 */	
	private String composeJavaMethodName(ReferenceStep step, boolean isLastStep)
	{
		String name =step.getName();
		if (lastRefStep!=null && step!=null && lastRefStep.getReferencedObject().equals(step.getReferencedObject())){
			return ".get" + Character.toUpperCase(name.charAt(0)) + name.substring(1) + "(" + forLoopVariable + ")";
		}else if (isLastStep || !isArray((ReferenceStep)step) || isGetAllExpr){
			return ".get" + Character.toUpperCase(name.charAt(0)) + name.substring(1) + "()";
		}else{
			return ".get" + Character.toUpperCase(name.charAt(0)) + name.substring(1) + "(0)";
		}
	}
	
	/**
	 * Creates a Java Method call name from a String variable for a BPEL Part or Business Item
	 * @param name the string variable
	 * @return String - the Java Method Call Name
	 */	
	private String composeJavaMethodName(String name)
	{
		/* Need to capitalize the first character of name */
		if (Character.isUpperCase(name.charAt(0)))
			return ".get" + name + "()";
		else
			return ".get" + Character.toUpperCase(name.charAt(0)) + name.substring(1) + "()";
	}
	
	
	
	/**
	 * Creates a Java Method call name from a String variable for a BPEL Variable
	 * @param name the string variable
	 * @return String - the Java Method Call Name
	 */
	private String composeJavaMethodVariableName(String name, ReferenceStep step, boolean isLastStep)
	{
		/* Need to capitalize the first character of name */
		if (lastRefStep!=null && step!=null && lastRefStep.getReferencedObject().equals(step.getReferencedObject())){
			return AbstractbpelUtil.generateJavaGetterForVariable(name)+ "(" + forLoopVariable + ")";
		}else if (step!=null && isArray((ReferenceStep)step) && !isLastStep){
			return AbstractbpelUtil.generateJavaGetterForVariable(name)+"(0)";
		}else
			return AbstractbpelUtil.generateJavaGetterForVariable(name)+"()";
	}
	
	/**
	 * Serializes a FunctionExpression into an equivalent Java String
	 * @param funcExpression the FunctionExpression
	 * @return String - the Java String
	 */
	private String serialize(FunctionExpression funcExpression)  {
	
		String funcId = funcExpression.getDefinition().getFunctionID();
		
		if ((CoreFunctions.CONTAINS_FUNCTION).equals(funcId)) 			
			return serializeContains(funcExpression);		
		else if ((CoreFunctions.STARTS_WITH_FUNCTION).equals(funcId))		
			return serializeStartsWith(funcExpression);				
		else if ("com.ibm.btools.te.ilm.heuristics.abstractbpel.getVariableData".equals(funcId))		
			return serializeGetVariableData(funcExpression);		
		else if ("com.ibm.btools.te.ilm.heuristics.abstractbpel.getLinkStatus".equals(funcId))		
			return serializeGetLinkStatus(funcExpression);
		else if ((CoreFunctions.SIZE_FUNCTION).equals(funcId))
			return serializeCount(funcExpression);
		else if ((CoreFunctions.SUM_FUNCTION).equals(funcId))
			return serializeSum(funcExpression);
		else if ((CoreFunctions.EXISTS_FUNCTION).equals(funcId))
			return serializeExists(funcExpression);
		else if ((CoreFunctions.FOR_ALL_FUNCTION).equals(funcId))
			return serializeEvery(funcExpression);
		else if ((CoreFunctions.SELECT_FUNCTION).equals(funcId))
			return serializeSelect(funcExpression);
		
		return null;
	}

	/**
	 * Serializes the Function Expression bpws:getLinkStatus into an equivalent Java String
	 * @param FunctionExpression the FunctionExpression
	 * @return String - the Java String
	 */	
	private String serializeGetLinkStatus(FunctionExpression funcExpr)
	{
		/* Always an argument to a Comparison Expression */
		String funcId = funcExpr.getDefinition().getFunctionID();
		StringBuffer sb = new StringBuffer();
		sb.append("getLinkStatus");
		sb.append(PARENTHESIS_START);
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		/* argValue should always be a StringLiteralExpression */		
		sb.append(serialize(argValue));		
		sb.append(PARENTHESIS_END);
		return sb.toString();
	}
	
	/**
	 * Serializes the starts with function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */	
	private String serializeStartsWith(FunctionExpression funcExpr)
	{		
		String funcId = funcExpr.getDefinition().getFunctionID();
		StringBuffer sb = new StringBuffer();
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		if (isTopLevelExpression(funcExpr))
			sb.append(RETURN);	
		/* argValue should be of type StringLiteralExpression or getVariableData FunctionExpr */
		if (argValue instanceof ModelPathExpression)
			setHasPreviousPath(true);
		sb.append(serialize(argValue));	
		sb.append(".startsWith(");
		argument = (FunctionArgument)arguments.get(1);
		argValue = argument.getArgumentValue();
		if (argValue instanceof ModelPathExpression)
			setPrefix(getContainerPrefix());
		sb.append(serialize(argValue));
		sb.append(")");		
		if (isTopLevelExpression(funcExpr))
			sb.append(SEMICOLON);	
		setHasPreviousPath(false);
		return sb.toString();	
	}

	/**
	 * Serializes the contains function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpressionFunctionExpression
	 * @return String - the Java String
	 */
	private String serializeContains(FunctionExpression funcExpr)
	{
		String funcId = funcExpr.getDefinition().getFunctionID();
		StringBuffer sb = new StringBuffer();
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		if (isTopLevelExpression(funcExpr))
			sb.append(RETURN);	
		sb.append(PARENTHESIS_START);
		/* argValue should be of type StringLiteralExpression or getVariableData FunctionExpr */
		if (argValue instanceof ModelPathExpression)
			setHasPreviousPath(true);
		sb.append(serialize(argValue));	
		sb.append(".indexOf(");
		argument = (FunctionArgument)arguments.get(1);
		argValue = argument.getArgumentValue();
		if (argValue instanceof ModelPathExpression)
			setPrefix(getContainerPrefix());
		sb.append(serialize(argValue));
		sb.append(PARENTHESIS_END);
		sb.append(")!=-1");
		if (isTopLevelExpression(funcExpr))
			sb.append(SEMICOLON);	
		setHasPreviousPath(false);
		return sb.toString();
	}

	/**
	 * Serializes the bpws:getVariableData funciton expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */	
	private String serializeGetVariableData(FunctionExpression funcExpr)
	{				
		StringBuffer sb = new StringBuffer();
		StringBuffer prefixBuffer = new StringBuffer();
		/* set a local static variable prefix to be the BPEL part of a variable */		
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);		
		prefixBuffer.append(composeJavaMethodVariableName(((StringLiteralExpression)argument.getArgumentValue()).getStringSymbol(), null, false));
		if (arguments.size()>1)
		{
			argument = (FunctionArgument)arguments.get(1);	
			prefixBuffer.append(composeJavaMethodName(((StringLiteralExpression)argument.getArgumentValue()).getStringSymbol()));			
		}
		setPrefix(prefixBuffer.toString());
		
		if (arguments.size()>2)
		{
			argument = (FunctionArgument)arguments.get(2);
			Expression argValue = argument.getArgumentValue();
			if (argValue instanceof ModelPathExpression)
			{
				if (isTopLevelExpression(funcExpr))
					sb.append(RETURN);	
				sb.append(serialize((ModelPathExpression)argValue));
				if (isTopLevelExpression(funcExpr))
					sb.append(SEMICOLON);
				return sb.toString();
			}				
			else if (argValue instanceof FunctionExpression)
			{
				String funcId = ((FunctionExpression)argValue).getDefinition().getFunctionID();	
				if ((CoreFunctions.SELECT_FUNCTION).equals(funcId)) 
				{
					return serializeSelect((FunctionExpression)argValue);						
				} 
				else if ((CoreFunctions.SIZE_FUNCTION).equals(funcId))
				{
					return serializeCount((FunctionExpression)argValue);
				}
				else if ((CoreFunctions.SUM_FUNCTION).equals(funcId))
				{
					return serializeSum((FunctionExpression)argValue);
				}
				else if ((CoreFunctions.EXISTS_FUNCTION).equals(funcId))
				{
					return serializeExists((FunctionExpression)argValue);
				}
				else if ((CoreFunctions.FOR_ALL_FUNCTION).equals(funcId))
				{
					return serializeEvery((FunctionExpression)argValue);
				}
				else if ((CoreFunctions.CONTAINS_FUNCTION).equals(funcId))
				{
					return serializeContains((FunctionExpression)argValue);
				}	
				else if ((CoreFunctions.STARTS_WITH_FUNCTION).equals(funcId))
				{
					return serializeStartsWith((FunctionExpression)argValue);
				}		
				//Note: The third argument of getVariableData should never be a
				//getAll function.
			}
		}
		else
		{	
			if (isTopLevelExpression(funcExpr))
				sb.append(RETURN);	
			sb.append(prefixBuffer.toString());
			if (isTopLevelExpression(funcExpr))
				sb.append(SEMICOLON);
			return sb.toString();
		}
		return null;
	}

	/**
	 * Serializes the select function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */	
	private String serializeSelect(FunctionExpression funcExpr)
	{		
		StringBuffer tempBuffer = new StringBuffer();
		List selectArguments = (funcExpr).getArguments();
		FunctionArgument selectArgument = (FunctionArgument)selectArguments.get(0);
		Expression selectArgValue = selectArgument.getArgumentValue();
		String thisPrefix = null;
		String loopIterator = ((numCountFunctions>0)?String.valueOf((char)('i'+numCountFunctions)):"i");
		numCountFunctions++;
		String loopIterator2 = ((numCountFunctions>0)?String.valueOf((char)('i'+numCountFunctions)):"i");
		ModelPathExpression mpe = ModelFactory.eINSTANCE.createModelPathExpression();
		if (selectArgValue instanceof FunctionExpression && ((FunctionExpression)selectArgValue).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)){
			isGetAllExpr=true;
			mpe=(ModelPathExpression)((FunctionArgument)((FunctionExpression)selectArgValue).getArguments().get(0)).getArgumentValue();
		}else{
			mpe=(ModelPathExpression)selectArgValue;
		}
			
//		tempBuffer.append("for (int i=0; i<");
//		tempBuffer.append(serialize(selectArgValue));	
//		tempBuffer.append(".size(); i++)");
		thisPrefix = getPrefix();
		tempBuffer.append(composeJavaLoopCondition(mpe, loopIterator));
		tempBuffer.append(NEWLINE);		
		if (isGetAllExpr){
			setPrefix(thisPrefix);
			tempBuffer.append(composeJavaInnerLoopCondition(mpe, loopIterator, loopIterator2));			
			tempBuffer.append(NEWLINE);
		}
			
		selectArgument = (FunctionArgument)selectArguments.get(1);
		tempBuffer.append("if ");
		tempBuffer.append(PARENTHESIS_START);
		/* Should be a Boolean Expression */
		isWithinFunction = true;
		tempBuffer.append(serialize(selectArgument.getArgumentValue()));
		isWithinFunction = false;
		tempBuffer.append(PARENTHESIS_END);			
		tempBuffer.append(NEWLINE);		

		
		if ((selectArgValue instanceof ModelPathExpression) && (!isArray((ModelPathExpression)selectArgValue))) {
			setPrefix(getPrefix());
		} else if (selectArgValue instanceof FunctionExpression && ((FunctionExpression)selectArgValue).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)){
			List steps = null;	
			if (((FunctionArgument)((FunctionExpression)selectArgValue).getArguments().get(0)).getArgumentValue() instanceof ModelPathExpression){
				steps = ((ModelPathExpression)((FunctionArgument)((FunctionExpression)selectArgValue).getArguments().get(0)).getArgumentValue()).getSteps();		
			}
			else{
					return null;
			}
			//append ".getName()			
			setPrefix(getPrefix()+getArrayPositionMethod(loopIterator)+ 
						composeJavaMethodName( ((Step)steps.get(steps.size()-1)).getName())+getArrayPositionMethod(loopIterator2));
		} else{
			setPrefix(getPrefix()+getArrayPositionMethod(loopIterator));
		}
		
//		setPrefix(getPrefix() + ".get(i)");
		/*
		if (isTopLevelExpression(funcExpr))
		{
			tempBuffer.append(RETURN);
			tempBuffer.append(getPrefix());
			tempBuffer.append(SEMICOLON);
			setStandAloneSelect(false);
			setMultiple(false);
		}	
		*/			
		setVariableName(getPrefix());
		setStandAloneSelect(true);
		setMultiple(true);
		isGetAllExpr=false;
		return tempBuffer.toString();
	}

	/**
	 * Serializes the count function expression into an equivalent Java String
	 * @param funcExpr the funcExpr
	 * @return String - the Java String
	 */	
	private String serializeCount(FunctionExpression funcExpr)
	{			
		StringBuffer sb = new StringBuffer();
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		
		if ( (argValue instanceof ModelPathExpression) ){ 
			serializedCountConditionExpression = serialize(argValue);
//			if (serializedCountConditionExpression.length() == 0) {
//				serializedCountConditionExpression=getPrefix();
//			}

			if (argValue instanceof ModelPathExpression && !isArray((ModelPathExpression)argValue)) {
				if (!isJavaPrimitiveType((ModelPathExpression)argValue)) {
					sb.append("(");
					sb.append(serializedCountConditionExpression);
					sb.append("!=null)?1:0");
				} else {
					sb.append("1");
				}
			} else if (argValue instanceof ModelPathExpression){
				sb.append(serializedCountConditionExpression);
				sb.append(getMultiplicityMethod());
			}
			// This CountExpression is needed by the serializeEvery method to correctly
			// create the 'count' comparison string - see serializeEvery method
			setCountConditionExpression(argValue);
		
		}else if (argValue instanceof FunctionExpression && ((FunctionExpression)argValue).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)){
			isGetAllExpr=true;				
			setVariableName("count" + String.valueOf(numCountFunctions++));
			sb.append("int ");
			sb.append(" ");
			sb.append(getVariableName());
			sb.append(" = 0;");
			sb.append(NEWLINE);
			
			String loopIterator = ((numCountFunctions>0)?String.valueOf((char)('i'+numCountFunctions)):"i");
			ModelPathExpression mpe = (ModelPathExpression)((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue();
			sb.append(composeJavaLoopCondition(mpe,loopIterator));
			sb.append(NEWLINE);
			sb.append(getVariableName());
			sb.append("+=");			
			sb.append(getPrefix() + getArrayPositionMethod(loopIterator));
			List steps = null;	
			if (((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue() instanceof ModelPathExpression){
				steps = ((ModelPathExpression)((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue()).getSteps();		
			}else
				return null;			
			sb.append(composeJavaMethodName( ((Step)steps.get(steps.size()-1)).getName()));
			sb.append(getMultiplicityMethod());	
			sb.append(";")	;	
			sb.append(NEWLINE);
			setMultiple(true);
			isGetAllExpr = false;
			
		}else if (argValue instanceof FunctionExpression && ((FunctionExpression)argValue).getDefinition().getFunctionID().equals(CoreFunctions.SELECT_FUNCTION))	{		
			String loopIterator = ((numCountFunctions>0)?String.valueOf((char)('i'+numCountFunctions)):"i");
			numCountFunctions++;			
			String loopIterator2 = ((numCountFunctions>0)?String.valueOf((char)('i'+numCountFunctions)):"i");
			/* FunctionExpression is a select */
			StringBuffer ifBuffer = new StringBuffer();
			StringBuffer criteriaBuffer = new StringBuffer();
			StringBuffer tempBuffer = new StringBuffer();
			String varName = "";
			setVariableName("count" + String.valueOf(numCountFunctions++));
			tempBuffer.append("int ");
			tempBuffer.append(getVariableName());
			tempBuffer.append(" = 0;");	
			tempBuffer.append(NEWLINE);	
			
			/* Get the arguments from FunctionExpression */
			List selectArguments = ((FunctionExpression)argValue).getArguments();
			FunctionArgument selectArgument = (FunctionArgument)selectArguments.get(0);
			Expression selectArgValue = selectArgument.getArgumentValue();
			ModelPathExpression argumentMPE = ModelFactory.eINSTANCE.createModelPathExpression();
			
			if (selectArgValue instanceof FunctionExpression){
				//it's a getall function
				isGetAllExpr = true;
				argumentMPE=(ModelPathExpression)((FunctionArgument)((FunctionExpression)selectArgValue).getArguments().get(0)).getArgumentValue();				
			}else{			
				argumentMPE = (ModelPathExpression)selectArgValue;
			}
			
			// This CountExpression is needed by the serializeEvery method to correctly
			// create the 'count' comparison string - see serializeEvery method
			setCountConditionExpression(selectArgValue);
			
			// the first argument of select will always be ModelPathExpression unless
			// the expression framework was updated to support Select within a Select function
			String tempPrefix = getPrefix();
			tempBuffer.append(composeJavaLoopCondition(argumentMPE,loopIterator));
			serializedCountConditionExpression=getPrefix();
			if (lastRefStep!=null){
				forLoopVariable = loopIterator;
			}
			
			if (isGetAllExpr){	
				tempBuffer.append(NEWLINE);	
				setPrefix(tempPrefix);
				tempBuffer.append(composeJavaInnerLoopCondition(argumentMPE,loopIterator, loopIterator2));									
			}
			
			tempBuffer.append(CURLYBRACKET_START);
			tempBuffer.append(NEWLINE);			
			selectArgument = (FunctionArgument)selectArguments.get(1);
			/* Print if First if there are not multiple statements within the for loop */
			ifBuffer.append("if ");
			ifBuffer.append(PARENTHESIS_START);
			isWithinFunction = true;
			varName = getVariableName();
			
			/* Serialize the criteria argument */
			criteriaBuffer.append(serialize(selectArgument.getArgumentValue()));			
			if (!isMultiple()){
			  tempBuffer.append(ifBuffer.toString());
			  tempBuffer.append(criteriaBuffer.toString());
			}				
			else {
				tempBuffer.append(criteriaBuffer.toString());
				tempBuffer.append(ifBuffer.toString());		
				tempBuffer.append(getVariableName());		
			}
		
			setVariableName(varName);
			tempBuffer.append(PARENTHESIS_END);
			isWithinFunction = false;			
			tempBuffer.append(NEWLINE);	
						
			tempBuffer.append(varName);
			tempBuffer.append("++;");			
			tempBuffer.append(NEWLINE);		
			tempBuffer.append(CURLYBRACKET_END);
			tempBuffer.append(NEWLINE);
			sb.append(tempBuffer.toString());						
			setMultiple(true);
			isGetAllExpr=false;
		}
		
		return sb.toString();
		
	}

	/**
	 * Serializes the sum function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */
	private String serializeSum(FunctionExpression funcExpr)
	{
		String funcId = funcExpr.getDefinition().getFunctionID();
		StringBuffer sb = new StringBuffer();
		StringBuffer tempBuffer = new StringBuffer();
		List arguments = funcExpr.getArguments();
		FunctionArgument argument = (FunctionArgument)arguments.get(0);
		Expression argValue = argument.getArgumentValue();
		String loopIterator = ((numSumFunctions>0)?String.valueOf((char)('i'+numSumFunctions)):"i");
		numSumFunctions++;			
		String loopIterator2 = ((numSumFunctions>0)?String.valueOf((char)('i'+numSumFunctions)):"i");
		// Assumption: sum applies only to 'numbers'
		setVariableName("sum" + String.valueOf(numSumFunctions++));
		tempBuffer.append(getJavaType(argValue));
		tempBuffer.append(" ");
		tempBuffer.append(getVariableName());
		tempBuffer.append(" = 0;");
		tempBuffer.append(NEWLINE);
		String thisPrefix = null;
		if ( (argValue instanceof ModelPathExpression) ||
			(argValue instanceof FunctionExpression && ((FunctionExpression)argValue).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)) ){			
//			tempBuffer.append("for (int i=0; i<");
//			tempBuffer.append(serialize(argValue));	
//			tempBuffer.append(".size(); i++)");

			ModelPathExpression mpe = ModelFactory.eINSTANCE.createModelPathExpression();
			if (argValue instanceof FunctionExpression){
				isGetAllExpr=true;
				mpe = (ModelPathExpression)((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue();
			}else{
				mpe = (ModelPathExpression)argValue;
			}
				

			thisPrefix=getPrefix();
			tempBuffer.append(composeJavaLoopCondition(mpe, loopIterator));
			tempBuffer.append(NEWLINE);
				
			if (isGetAllExpr){
				setPrefix(thisPrefix);
				tempBuffer.append(composeJavaInnerLoopCondition(mpe, loopIterator, loopIterator2));	
				tempBuffer.append(NEWLINE);			
			}
			
			tempBuffer.append(getVariableName());
			tempBuffer.append("+=");

				
			if ((argValue instanceof ModelPathExpression) && (!isArray((ModelPathExpression)argValue))) {
				tempBuffer.append(getPrefix());
			} else if (argValue instanceof ModelPathExpression){
				tempBuffer.append(getPrefix());
				tempBuffer.append(getArrayPositionMethod(loopIterator));
			}else if (isGetAllExpr){
				List steps = null;	
				if (((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue() instanceof ModelPathExpression){
					steps = ((ModelPathExpression)((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue()).getSteps();		
				}
				else{
						return null;
				}
				tempBuffer.append(getPrefix());
				tempBuffer.append(getArrayPositionMethod(loopIterator));
				//tempBuffer.append(".");
				tempBuffer.append(composeJavaMethodName( ((Step)steps.get(steps.size()-1)).getName()));
				tempBuffer.append(getArrayPositionMethod(loopIterator2));
			}
			tempBuffer.append(";");
			tempBuffer.append(NEWLINE);		
			sb.append(tempBuffer.toString());
			isGetAllExpr=false;	
		}
		else if (argValue instanceof FunctionExpression)
		{
			/* FunctionExpression must be a select */
			/* Get the arguments from FunctionExpression */
			String varName = "";
			StringBuffer ifBuffer = new StringBuffer();
			StringBuffer criteriaBuffer = new StringBuffer();
			List selectArguments = ((FunctionExpression)argValue).getArguments();
			FunctionArgument selectArgument = (FunctionArgument)selectArguments.get(0);
			Expression selectArgValue = selectArgument.getArgumentValue();
			ModelPathExpression mpe = ModelFactory.eINSTANCE.createModelPathExpression();
			if (selectArgValue instanceof FunctionExpression && ((FunctionExpression)selectArgValue).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)){
				isGetAllExpr=true;
				mpe = (ModelPathExpression)((FunctionArgument)((FunctionExpression)selectArgValue).getArguments().get(0)).getArgumentValue();
			}else{
				mpe = (ModelPathExpression)selectArgValue;
			}
			

//			tempBuffer.append("for (int i=0; i<");
//			tempBuffer.append(serialize(selectArgValue));	
//			tempBuffer.append(".size(); i++)");
			
			thisPrefix=getPrefix();
			tempBuffer.append(composeJavaLoopCondition(mpe, loopIterator));			
			tempBuffer.append(CURLYBRACKET_START);
			tempBuffer.append(NEWLINE);
			
			if (isGetAllExpr){	
				setPrefix(thisPrefix);
				tempBuffer.append(composeJavaInnerLoopCondition(mpe, loopIterator, loopIterator2));
				tempBuffer.append(NEWLINE);
			}
			
			selectArgument = (FunctionArgument)selectArguments.get(1);
			ifBuffer.append("if ");
			ifBuffer.append(PARENTHESIS_START);
			isWithinFunction = true;
			String prefixString=getPrefix();
			varName = getVariableName();
			/* Serialize the criteria argument */
			criteriaBuffer.append(serialize(selectArgument.getArgumentValue()));			
			if (!isMultiple()){
			  tempBuffer.append(ifBuffer.toString());
			  tempBuffer.append(criteriaBuffer.toString());
		    }				
		    else {
		    	tempBuffer.append(criteriaBuffer.toString());
		    	tempBuffer.append(ifBuffer.toString());		
		    	tempBuffer.append(getVariableName());		
		    }
			isWithinFunction = false;
			tempBuffer.append(PARENTHESIS_END);									
			tempBuffer.append(NEWLINE);	
			setVariableName(varName);
			tempBuffer.append(varName);
			tempBuffer.append("+=");
			
			if ((argValue instanceof ModelPathExpression) && (!isArray((ModelPathExpression)argValue))) {
				tempBuffer.append(prefixString);
			} else if (isGetAllExpr){
				 tempBuffer.append(prefixString);
				 tempBuffer.append(getArrayPositionMethod(loopIterator));
				List steps = null;	
				FunctionExpression getAllexpr = (FunctionExpression)((FunctionArgument)((FunctionExpression)argValue).getArguments().get(0)).getArgumentValue();
				if ( ((FunctionArgument)getAllexpr.getArguments().get(0)).getArgumentValue() instanceof ModelPathExpression){
					steps = ((ModelPathExpression)((FunctionArgument)getAllexpr.getArguments().get(0)).getArgumentValue()).getSteps();		
				}
				else{
						return null;
				}				
				//tempBuffer.append(".");
				tempBuffer.append(composeJavaMethodName( ((Step)steps.get(steps.size()-1)).getName()));
				tempBuffer.append(getArrayPositionMethod(loopIterator2));
				 
			}else {
				tempBuffer.append(prefixString);
				tempBuffer.append(getArrayPositionMethod(loopIterator));
			}	
			tempBuffer.append(";");
			tempBuffer.append(NEWLINE);	
			tempBuffer.append(CURLYBRACKET_END);
			tempBuffer.append(NEWLINE);	
			sb.append(tempBuffer.toString());
			isGetAllExpr=false;
		}	
		setMultiple(true);	
		return sb.toString();
		
	}
	
	/**
	 * Serializes the exists function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */	
	private String serializeExists(FunctionExpression funcExpr)
	{
		StringBuffer sb = new StringBuffer();
		forLoopVariable= "";
		sb.append(serialize(createCountExpression(funcExpr)));
		String varName = getVariableName();	
		if (!isTopLevelExprforExistsEvery(funcExpr))		
		{
			setVariableName("exists" + String.valueOf(numExistsFunctions++));
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(" = ");			
		}
		else
			sb.append(RETURN);
		
		sb.append(PARENTHESIS_START);	
		sb.append(varName);
		sb.append(">0");	
		sb.append(PARENTHESIS_END);		
		sb.append(SEMICOLON);
		sb.append(NEWLINE);
		if (isTopLevelExprforExistsEvery(funcExpr))
			setMultiple(false);
		forLoopVariable= null;
		lastRefStep=null;
		return sb.toString();
	}
	
	
	/**
	 * Serializes the every function expression into an equivalent Java String
	 * @param funcExpr the FunctionExpression
	 * @return String - the Java String
	 */		
	private String serializeEvery(FunctionExpression funcExpr)
	{
		StringBuffer sb = new StringBuffer();	
		forLoopVariable= "";			
		sb.append(serialize(createCountExpression(funcExpr)));
		String varName = getVariableName();	
		if (isTopLevelExprforExistsEvery(funcExpr))
			sb.append(RETURN);
		else {
			setVariableName("every" + String.valueOf(numEveryFunctions++));
			sb.append("boolean ");
			sb.append(getVariableName());
			sb.append(" = ");			
		}	
		sb.append(PARENTHESIS_START);
		sb.append(varName);
		sb.append(EQUAL_TO_OP);
		// Handle the case where the Count expression was created on a non-array type
		// element.  Consider this is a rare case and perhaps should treat it as an
		// invalid expression in the first place
		if (getCountConditionExpression() instanceof ModelPathExpression) {

			if (isArray((ModelPathExpression)getCountConditionExpression())) 
			{
				//sb.append(getPrefix());
				sb.append(serializedCountConditionExpression);
				sb.append(getMultiplicityMethod());
			} else if (!isJavaPrimitiveType((ModelPathExpression)getCountConditionExpression())){
				sb.append("(");
				//sb.append(getPrefix());
				sb.append(serializedCountConditionExpression);
				sb.append("!=null)?1:0");
			} else {
				sb.append("1");
			}
		} else {
			//sb.append(getPrefix());
			sb.append(serializedCountConditionExpression);
			sb.append(getMultiplicityMethod());
		}
		sb.append(PARENTHESIS_END);		
		sb.append(SEMICOLON);
		sb.append(NEWLINE);
		if (isTopLevelExprforExistsEvery(funcExpr))
			setMultiple(false);
		forLoopVariable= null;
		lastRefStep=null;
		return sb.toString();
	}

	/**
	 * Creates a count function expression 
	 * @param funcExpr the FunctionExpression
	 * @return FunctionExpression - the count function expresssion
	 */		
	private FunctionExpression createCountExpression (FunctionExpression funcExpr)
	{
		/* Get the arguments from the parameter funcExpr */		
		List selectArguments = funcExpr.getArguments();
		FunctionArgument selectArgument = (FunctionArgument)selectArguments.get(0);

		/* Create count function Expression */
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID(CoreFunctions.SIZE_FUNCTION);
		def.setFunctionName("count");

		/* First argument for count Function Expression */
		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();
		
		/* Create select Function Expression */ 	
		FunctionExpression selectExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition selectDef = ModelFactory.eINSTANCE.createFunctionDefinition();
		selectDef.setFunctionID(CoreFunctions.SELECT_FUNCTION);	
		selectExpr.setDefinition(selectDef);
		FunctionArgument selectArg1 = ModelFactory.eINSTANCE.createFunctionArgument();	
		
//		if it's a mpe	
	  if (selectArgument.getArgumentValue() instanceof ModelPathExpression){
		  ModelPathExpression mpeExpr = ModelFactory.eINSTANCE.createModelPathExpression();		
		  mpeExpr = (ModelPathExpression)selectArgument.getArgumentValue();
		  selectArg1.setArgumentValue(mpeExpr);		
	  }else if (selectArgument.getArgumentValue() instanceof FunctionExpression
				  && (((FunctionExpression)selectArgument.getArgumentValue()).getDefinition().getFunctionID().equals(CoreFunctions.GET_ALL_FUNCTION)) ){
		  /* Create select Function Expression */ 	
		  FunctionExpression getallExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		  FunctionDefinition getallDef = ModelFactory.eINSTANCE.createFunctionDefinition();
		  getallDef.setFunctionID(CoreFunctions.GET_ALL_FUNCTION);	
		  getallExpr.setDefinition(getallDef);
		  FunctionArgument getallArg1 = ModelFactory.eINSTANCE.createFunctionArgument();
		  ModelPathExpression mpeExpr = ModelFactory.eINSTANCE.createModelPathExpression();
		  //getting the mpe from the first argument to getall function
		  mpeExpr = (ModelPathExpression)((FunctionArgument)((FunctionExpression)selectArgument.getArgumentValue()).getArguments().get(0)).getArgumentValue();
		  getallArg1.setArgumentValue(mpeExpr);
		  getallExpr.getArguments().add(getallArg1);						
		  selectArg1.setArgumentValue(getallExpr);					
	  }
			
		FunctionArgument selectArgument1 = ModelFactory.eINSTANCE.createFunctionArgument();
		selectArgument1 = (FunctionArgument)selectArguments.get(1);
		FunctionArgument selectArg2 = ModelFactory.eINSTANCE.createFunctionArgument();	
		
		Expression boolExpr = selectArgument1.getArgumentValue();
		selectArg2.setArgumentValue(boolExpr);	
		selectExpr.getArguments().add(selectArg1);
		selectExpr.getArguments().add(selectArg2);

		arg1.setArgumentValue(selectExpr);
		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);		
		return functionExpr;
	}

	/**
	 * resets all the global attributes	 
	 */	
	private void resetAttributes ()	
	{ 
		isMultiple = false;
		isStandAloneSelect = false;
		prefix = "";
		containerPrefix = "";
		variableName = "";
		numCountFunctions = 0;
		numSumFunctions = 0;
		numExistsFunctions = 0;
		numEveryFunctions = 0;
		numBooleanExpr = 0;	
		numTimeExpr=0;
		isWithinFunction = false;	
		hasPreviousPath = false;
		countConditionExpression = null;
		serializedCountConditionExpression = null;
		forLoopVariable=null;
		lastRefStep=null;
	}
	
	/**
	 * Checks to see if the expression is a top level expression
	 * @param expr the expression
	 * @return boolean 
	 */	
	private boolean isTopLevelExpression(Expression expr)
	{
		if (expr != null && expr.eContainer() instanceof ExtensibilityExpression )
			return true;
		else
			return false;		
	}
	
	/**
	 * Checks to see if the exist or every expression is a top level expression
	 * @param expr the expression
	 * @return boolean 
	 */	
	private boolean isTopLevelExprforExistsEvery(Expression expr)
	{		
		if (expr.eContainer().eContainer().eContainer() instanceof ExtensibilityExpression )
			return true;
		else
			return false;
	}

	private String getJavaType(Expression expr) {
		if (expr instanceof ModelPathExpression){
			List steps = ((ModelPathExpression)expr).getSteps();
			// Get the last step to find out if it is a ReferenceStep where the ReferencedObject
			// is an element with upperbound > 1
			if (steps.size() > 0) {
				Step step = (Step)steps.get(steps.size()-1);
				if (step instanceof ReferenceStep) {
					Object refObj = ((ReferenceStep)step).getReferencedObject();
					if (refObj instanceof XSDParticle) {
						String typeName = ((XSDElementDeclaration)((XSDParticle)refObj).getContent()).getTypeDefinition().getName();
						if (typeName.equals("int")) {
							return "int";
						} else if (typeName.equals("long")) {
							return "long";
						} else if (typeName.equals("short")) {
							return "short";
						} else if (typeName.equals("double")) {
							return "double";
						} else if (typeName.equals("float")) {
							return "float";
						}
					}
				}
			}
		}
		return "int"; 
	}
	
	private boolean isJavaPrimitiveType(ModelPathExpression mpe) {
		List steps = mpe.getSteps();
		// Get the last step to find out if it is a ReferenceStep where the ReferencedObject
		// is an element with upperbound > 1
		if (steps.size() > 0) {
			Step step = (Step)steps.get(steps.size()-1);
			if (step instanceof ReferenceStep) {
				Object refObj = ((ReferenceStep)step).getReferencedObject();
				if (refObj instanceof XSDParticle) {
					String typeName = ((XSDElementDeclaration)((XSDParticle)refObj).getContent()).getTypeDefinition().getName();
					if (typeName.equals("int") || typeName.equals("long") || typeName.equals("short")
						|| typeName.equals("double") || typeName.equals("float")) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
	/**
	 * Check if the ModelPathExpression is referencing an object that is an Array
	 * @param mpe
	 * @return
	 */
	private boolean isArray(ModelPathExpression mpe) {
		List steps = mpe.getSteps();
		// Get the last step to find out if it is a ReferenceStep where the ReferencedObject
		// is an element with upperbound > 1
		if (steps.size() > 0) {
			Step step = (Step)steps.get(steps.size()-1);
			if (step instanceof ReferenceStep) {
				Object refObj = ((ReferenceStep)step).getReferencedObject();
				if (refObj instanceof XSDParticle) {
					int maxOccur = ((XSDParticle)refObj).getMaxOccurs();
					// maxOccur = -1 means it is unbounded
					if ((maxOccur <= 1) && (maxOccur > 0))
						return false;
				}
			}
		}	
		// Return true by default.  Assume the expression makes sense	
		return true;
	}
	
	private boolean isArray(ReferenceStep step){
		Object refObj = step.getReferencedObject();
		if (refObj instanceof XSDParticle) {
			int maxOccur = ((XSDParticle)refObj).getMaxOccurs();
			// maxOccur = -1 means it is unbounded
			if ((maxOccur <= 1) && (maxOccur > 0))
				return false;
		}
		return true;
	}
	
	/**
	 * @return
	 */
	private static boolean isMultiple() {
		return isMultiple;
	}

	/**
	 * @return
	 */
	private static boolean isStandAloneSelect() {
		return isStandAloneSelect;
	}

	/**
	 * @return
	 */
	private static String getPrefix() {
		return prefix;
	}

	/**
	 * @param b
	 */
	private static void setMultiple(boolean b) {
		isMultiple = b;
	}

	/**
	 * @param b
	 */
	private static void setStandAloneSelect(boolean b) {
		isStandAloneSelect = b;
	}

	/**
	 * @param string
	 */
	private static void setPrefix(String string) {
		prefix = string;
	}

	/**
	 * @return
	 */
	private static String getVariableName() {
		return variableName;
	}

	/**
	 * @param string
	 */
	private static void setVariableName(String string) {
		variableName = string;
	}

	/**
	 * @return
	 */
	private static boolean isHasPreviousPath() {
		return hasPreviousPath;
	}

	/**
	 * @param b
	 */
	private static void setHasPreviousPath(boolean b) {
		hasPreviousPath = b;
	}

	/**
	 * @return
	 */
	private static String getContainerPrefix() {
		return containerPrefix;
	}

	/**
	 * @param string
	 */
	private static void setContainerPrefix(String string) {
		containerPrefix = string;
	}

	/**
	 * @return
	 */
	private Expression getCountConditionExpression() {
		return countConditionExpression;
	}

	/**
	 * @param expression
	 */
	private void setCountConditionExpression(Expression expression) {
		countConditionExpression = expression;
	}

	/**
	 * This method returns the Java code that returns the length of an Array or List, depends
	 * on what WSADIE generates.  As of 5.1, an xsd element with maxOccurs > 1 with have an array (i.e. [])
	 * being generated
	 * @return
	 */
	private String getMultiplicityMethod() {
		return ".length";
	}
	
	/**
	 * Same idea as getMultiplicityMethod ... see its comment
	 * @param iteratorVar
	 * @return
	 */
	private String getArrayPositionMethod(String iteratorVar) {
		return "["+iteratorVar+"]";
	}	
}
