package com.ibm.btools.te.ilm.heuristics.br.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.IfThenRule;
import com.ibm.btools.bom.model.processes.businessrules.TemplateParameter;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.br.BRSetRule;
import com.ibm.btools.te.ilm.heuristics.br.BRTemplateRule;
import com.ibm.btools.te.ilm.heuristics.br.BrPackage;
import com.ibm.btools.te.ilm.heuristics.br.util.BRActionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BRConditionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BRExpressionSerializer;
import com.ibm.btools.te.ilm.heuristics.br.util.BrUtil;
import com.ibm.wbit.br.core.model.ActionBlock;
import com.ibm.wbit.br.core.model.Condition;
import com.ibm.wbit.br.core.model.ConditionExpression;
import com.ibm.wbit.br.core.model.Expression;
import com.ibm.wbit.br.core.model.ModelFactory;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>BR Template Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class BRTemplateRuleImpl extends TransformationRuleImpl implements BRTemplateRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/* attribute indicate whether it is first time execution of this rule */
	private boolean firstTimeExec = true;	

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BRTemplateRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return BrPackage.Literals.BR_TEMPLATE_RULE;
	}

	/**
	 * Map BOM BusinessRuleTemplate to Business Rule Template constructs.
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		//assertion
		if (getSource() == null || getSource().size() != 1) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
			 
		//get source of this rule - BusinessRuleTemplate	
		BusinessRuleTemplate businessRuleTemplate = (BusinessRuleTemplate)getSource().get(0);
		
  		//check if this is first time execute this rule
		if(!firstTimeExec){
			 return true;		  
		} else {
			firstTimeExec = false;
		}

		// create Business Rule Template(businessRuleTemplate);
		// create all parameters
		// create a Rule that a template is based on
		// 		- create and populate condition
		// 		- create and populate action
		// populate presentation string

		// create Business Rule Template(businessRuleTemplate);
		RuleTemplate ruleTemplate = ModelFactory.eINSTANCE.createRuleTemplate();
		
		// set attributes
		ruleTemplate.setId(businessRuleTemplate.getUid());
		// set attributes
		AttributeValueProvider avp = null;
		BRConditionSerializer brConditionSerializer = BRConditionSerializer.getInstance();		
		BRActionSerializer brActionSerializer = BRActionSerializer.getInstance();
		BRExpressionSerializer brExpressionSerializer = BRExpressionSerializer.getInstance();
		if (brExpressionSerializer.getRoot() == null) {
			brExpressionSerializer.setRoot(getRoot());
		}
		if (getParentRule() instanceof BRSetRule) {
			BRSetRuleImpl brSetRule = (BRSetRuleImpl)getParentRule();
			avp = brSetRule.getAvp();
		}
		
		String rtName = BrUtil.createRuleTemplateName(businessRuleTemplate, ruleTemplate, avp, getContext());
		ruleTemplate.setName(rtName);
		// no display name, only description for template
		if (businessRuleTemplate.getOwnedComment() != null && !businessRuleTemplate.getOwnedComment().isEmpty()) {
			ruleTemplate.setDescription(((Comment)businessRuleTemplate.getOwnedComment().get(0)).getBody());
		}
		getTarget().add(ruleTemplate);

		// create all parameters
		createParameters(businessRuleTemplate, ruleTemplate, avp);
		
		// create rule
		createRule(businessRuleTemplate, ruleTemplate, brConditionSerializer, brActionSerializer);
		
		// populate presentation string
		ruleTemplate.setWebPresentation(BrUtil.formatPresentation(businessRuleTemplate));	

		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}	
	

	// create all parameters
	@SuppressWarnings("unchecked")
	private void createParameters(BusinessRuleTemplate businessRuleTemplate, RuleTemplate ruleTemplate, AttributeValueProvider avp) {
		if (businessRuleTemplate.getParameters() == null || businessRuleTemplate.getParameters().isEmpty() ||
				ruleTemplate == null) {
			return;
		}
		
		Iterator it = businessRuleTemplate.getParameters().iterator();
		TemplateParameter templateParameter = null;
		Variable parameterVariable = null; 
		String parName = null;
			
		while (it.hasNext()) {
			templateParameter = (TemplateParameter) it.next();
			
			parameterVariable = ModelFactory.eINSTANCE.createVariable();
			parName = BrUtil.createParameterName(templateParameter, parameterVariable, avp, getContext());
			parameterVariable.setVarName(parName);
			parameterVariable.setDisplayName(templateParameter.getName());
			if (templateParameter.getOwnedComment() != null && !templateParameter.getOwnedComment().isEmpty()) {
				parameterVariable.setComment(((Comment)templateParameter.getOwnedComment().get(0)).getBody());
			}
			parameterVariable.setDataTypeQName(BrUtil.findQNameForType(templateParameter.getType()));
			ruleTemplate.getParameter().add(parameterVariable);
		}
	}
	
	
	// create and set in rule template a Rule
	// 		- create and populate condition
	// 		- create and populate action
	@SuppressWarnings("unchecked")
	private void createRule(BusinessRuleTemplate businessRuleTemplate, RuleTemplate ruleTemplate, BRConditionSerializer brConditionSerializer, BRActionSerializer brActionSerializer) {
		if (businessRuleTemplate == null || ruleTemplate == null) {
			return;
		}
		
		com.ibm.wbit.br.core.model.IfThenRule rule = ModelFactory.eINSTANCE.createIfThenRule();
		
		Condition condition = ModelFactory.eINSTANCE.createCondition();
		ConditionExpression conditionExpression = null;
		Expression actionExpression = null;
		
		// get, convert and set the expression for condition
		if (businessRuleTemplate.getRule() != null && businessRuleTemplate.getRule() instanceof IfThenRule) {
			IfThenRule ifThenRule = (IfThenRule)businessRuleTemplate.getRule();
			
			if (ifThenRule.getRuleCondition() != null) {
				if (ifThenRule.getRuleCondition() != null && ifThenRule.getRuleCondition() instanceof StructuredOpaqueExpression &&
						((StructuredOpaqueExpression)ifThenRule.getRuleCondition()).getExpression() != null &&
						brConditionSerializer != null) {
					conditionExpression = brConditionSerializer.createConditionExpression(((StructuredOpaqueExpression)ifThenRule.getRuleCondition()).getExpression(), null);
					condition.setConditionExpression(conditionExpression);
				}
			}

			if (condition.getConditionExpression() == null) {
				conditionExpression = ModelFactory.eINSTANCE.createBooleanExpression();
			}

			condition.setConditionExpression(conditionExpression);
			rule.setIf(condition);

			rule.setThen(brActionSerializer.createActionBlock(ifThenRule.getRuleConsequence()));
		} else {
			conditionExpression = ModelFactory.eINSTANCE.createBooleanExpression();
			condition.setConditionExpression(conditionExpression);
			rule.setIf(condition);

			ActionBlock actionBlock = ModelFactory.eINSTANCE.createActionBlock();
			actionExpression = ModelFactory.eINSTANCE.createExpression();
			actionBlock.getAction().add(actionExpression);				
			rule.setThen(actionBlock);
		}
		
		ruleTemplate.setRule(rule);
	}

} //BRTemplateRuleImpl