package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;

/**
 * Naming registry for SCDL component artifacts to ensure the unqiuenes of their name
 */
public class BRNamingRegistry implements NamingRegistry {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private  HashMap registry = new HashMap();
	private  HashMap brGroupNameRegistry = new HashMap();
	private  HashMap brSetNameRegistry = new HashMap();
	private  HashMap brTemplateNameRegistry = new HashMap();
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#isUniqueName(java.lang.Object, java.lang.String)
	 */
	public boolean isUniqueName(Object obj, String name) {
		LoggingUtil.traceEntry(this, "isUniqueName");
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}
		else if (map.values().contains(name)){
			return false;
		}
		else {
			//Store the name in the Registry
			map.put(obj, name);
			return true;
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#isUniqueName(java.lang.Object, java.lang.Object, java.lang.String)
	 */
	public boolean isUniqueName(Object obj, Object parentObject, String name)
	{
		HashMap map = getRegistry(obj);
		List entries = null;
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}else if(map.get(parentObject) != null){
			entries = (List)map.get(parentObject);
			if(entries.contains(name)){
				return false;
			}else{
				entries.add(name);
				return true;
			}
		}else {
			entries = new LinkedList();
			entries.add(name);
			//Store the name in the Registry
			map.put(parentObject, entries);
			return true;
		}
	}	

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#deregister(java.lang.Object, java.lang.String)
	 */
	public void deregister(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return;		 		
		}
		else if (map.values().contains(name)){
			map.remove(obj);
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#retrieveFromRegistry(java.lang.Class, java.lang.String)
	 */
	public Object retrieveFromRegistry(Class obj, String registryEntry) {	
		HashMap registry = getRegistry(obj);
		
		if(registry.containsValue(registryEntry)){
			//get the object container that was registered for that entry.. 
			Iterator it = registry.keySet().iterator();
			while(it.hasNext()){
				Object variable = (EObject)it.next();
				if(registry.get(variable).equals(registryEntry)){
					return variable;
				}
			}
			
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#register(java.lang.Object, java.lang.String)
	 */
	public void register(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if(map == null){
			return;
		}
		map.put(obj, name);
	}
	
	
	public void register(Object obj, Object parentObject, String name) {
		HashMap map = getRegistry(obj);
		List entries = null;
		if (map.get(parentObject) != null){
			entries = (List)map.get(parentObject);
			if(entries.contains(name)){
				return;
			}else{
				entries.add(name);
				return;
			}
		}else {
			entries = new LinkedList();
			entries.add(name);
			//Store the name in the Registry
			map.put(parentObject, entries);
			return;
		}
	}

	
	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */		
	public HashMap getRegistry(Object obj)
	{
		if (obj instanceof BusinessRuleGroup || obj instanceof RuleSet) {
			return brGroupNameRegistry;
		} else if (obj instanceof RuleTemplate || 
				obj instanceof Rule) {
			return brSetNameRegistry;
		} else if (obj instanceof Variable) {
			return brTemplateNameRegistry;
		} else {
			return registry;
		}
	}

	public HashMap getBrGroupNameRegistry() {
		return brGroupNameRegistry;
	}

	public HashMap getBrSetNameRegistry() {
		return brSetNameRegistry;
	}

	public HashMap getBrTemplateNameRegistry() {
		return brTemplateNameRegistry;
	}

	public HashMap getRegistry() {
		return registry;
	}

}
