package com.ibm.btools.te.ilm.heuristics.br.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.xml.type.util.XMLTypeUtil;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.blm.ie.BusinessRuleTaskPresentationUtil;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralSpecification;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRule;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSet;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.TemplateParameter;
import com.ibm.btools.expression.ExpressionPlugin;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionArgumentDefinition;
import com.ibm.btools.expression.resource.MessageKeys;
import com.ibm.btools.expression.util.LogUtil;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.model.specification.processmodel.wps.TaskAttributes;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;
import com.ibm.btools.util.logging.LogHelper;
import com.ibm.wbiservers.businessrule.model.brg.BrgFactory;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brg.ReferenceGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BrgtFactory;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbiservers.common.componentdef.CodeParameterDef;
import com.ibm.wbiservers.common.componentdef.ComponentDef;
import com.ibm.wbiservers.common.componentdef.ComponentdefFactory;
import com.ibm.wbiservers.common.componentdef.OperationDef;
import com.ibm.wbiservers.common.componentdef.WSDL;
import com.ibm.wbiservers.common.selectiontables.OperationSelectionTable;
import com.ibm.wbiservers.common.selectiontables.SelectionTables;
import com.ibm.wbiservers.common.selectiontables.SelectiontablesFactory;
import com.ibm.wbit.br.core.model.DocumentRoot;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;
import com.ibm.wbit.br.core.util.RuleConstants;

public class BrUtil implements BRExpressionConstants {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Create component name
	 * @param action Action 
	 * @param component Component
	 * @param avp AttributeValueProvider
	 * @param tContext TransformationContext 
	 * @return componentName
	 */
	public static String createRuleGroupName(Action action, BusinessRuleGroup businessRuleGroup, AttributeValueProvider avp, TransformationContext tContext) {
		if (action == null || avp == null || businessRuleGroup == null) {
			return null;
		}
		
		String componentName = null;
		if (action.getOwnedDescriptor() != null && !action.getOwnedDescriptor().isEmpty() &&
				((BusinessRuleAction)action).getOwnedDescriptor().get(0) instanceof TaskAttributes) {
			TaskAttributes ta = (TaskAttributes)((BusinessRuleAction)action).getOwnedDescriptor().get(0);
			if (ta.getServiceComponent() != null) {
				componentName = ta.getServiceComponent().getName();
			}
			if (componentName != null) {
				BRNamingRegistry brNamingRegistry = (BRNamingRegistry)NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY);
				Object contextContainer = NamingUtil.getContextContainer(action); 
				
				String pathName = NamingUtil.convertNameForAction(action, tContext, componentName);
				if (contextContainer instanceof StructuredActivityNode) {
					brNamingRegistry.register(businessRuleGroup, contextContainer, pathName);
				} else {
					brNamingRegistry.register(businessRuleGroup, pathName);
				}
				return componentName;
			}
		}
		
		componentName = (String)avp.getAttributeValueForType(businessRuleGroup, 
																action, 
																AttributeNameConstants.BUSINESS_RULE_NAME, 
																NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY));
		
		return componentName;
	}

	/**
	 * Create rule set name
	 * @param brSet BusinessRuleSet
	 * @param rSet RuleSet
	 * @param avp AttributeValueProvider
	 * @param tContext TransformationContext 
	 * @return componentName
	 */
	public static String createRuleSetName(BusinessRuleSet brSet, RuleSet rSet, AttributeValueProvider avp, TransformationContext tContext) {
		if (brSet == null || avp == null || rSet == null) {
			return null;
		}
		
		String rsName = (String)avp.getAttributeValueForType(rSet, 
																	brSet, 
																	AttributeNameConstants.BUSINESS_RULE_NAME, 
																	NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY));
		
		return rsName;
	}
	
	/**
	 * Create template name
	 * @param brtemplate BusinessRuleTemplate
	 * @param rTemplate RuleTemplate
	 * @param avp AttributeValueProvider
	 * @param tContext TransformationContext 
	 * @return componentName
	 */
	public static String createRuleTemplateName(BusinessRuleTemplate brtemplate, RuleTemplate rTemplate, AttributeValueProvider avp, TransformationContext tContext) {
		if (brtemplate == null || avp == null || rTemplate == null) {
			return null;
		}
		
		String rtName = (String)avp.getAttributeValueForType(rTemplate, 
																	brtemplate, 
																	AttributeNameConstants.BUSINESS_RULE_NAME, 
																	NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY));
		
		return rtName;
	}
	
	/**
	 * Create rule name
	 * @param br BusinessRule
	 * @param rule Rule
	 * @param avp AttributeValueProvider
	 * @param tContext TransformationContext 
	 * @return componentName
	 */
	public static String createRuleName(BusinessRule br, Rule rule, AttributeValueProvider avp, TransformationContext tContext) {
		if (br == null || avp == null || rule == null) {
			return null;
		}
		
		String rName = (String)avp.getAttributeValueForType(rule, 
																	br, 
																	AttributeNameConstants.BUSINESS_RULE_NAME, 
																	NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY));
		
		return rName;
	}
	
	/**
	 * Create parameter variable name
	 * @param templateParameter TemplateParameter
	 * @param var Variable
	 * @param avp AttributeValueProvider
	 * @param tContext TransformationContext 
	 * @return parameter name
	 */
	public static String createParameterName(TemplateParameter templateParameter, Variable var, AttributeValueProvider avp, TransformationContext tContext) {
		if (templateParameter == null || avp == null || var == null) {
			return null;
		}
		
		String parName = (String)avp.getAttributeValueForType(var, 
																	templateParameter, 
																	AttributeNameConstants.BUSINESS_RULE_NAME, 
																	NamingUtil.getBRNamingRegistry(tContext,NamingUtil.BR_NAMING_REGISTRY));
		
		return parName;
	}

	@SuppressWarnings("unchecked")
	/**
	 * Add port type to component
	 * @param def ComponentDef 
	 * @param table SelectionTables
	 * @param port PortType
	 * @return
	 */
	public static void addPortTypeToComponent(ComponentDef def, SelectionTables table, PortType port) {
		// Create the interface.
		com.ibm.wbiservers.common.componentdef.WSDLPortType pt = ComponentdefFactory.eINSTANCE.createWSDLPortType();
		javax.xml.namespace.QName ptQName = port.getQName();
		pt.setName(XMLTypeUtil.createQName(ptQName.getNamespaceURI(),ptQName.getLocalPart(),null));
		Iterator operations = port.getOperations().iterator();
		// Create the opdefs.
		while (operations.hasNext()) {
			Operation operation = (Operation) operations.next();
			//addOperationToComponent
			OperationDef operationDef = ComponentdefFactory.eINSTANCE.createOperationDef();
			operationDef.setOperationName(operation.getName());
			operationDef.setSelector(RuleConstants.GENERIC_SELECTOR_ID);
			CodeParameterDef javaParam = ComponentdefFactory.eINSTANCE.createCodeParameterDef();
			javaParam.setCode(CURRENT_TIME_JAVACODE);
			operationDef.getParameterDefs().add(javaParam);
			def.getOperationDefs().add(operationDef);	
		}
		def.getWSDL().getPortTypes().add(pt);
	}
	
	/**
	 * Update the rule group for rule sets
	 * @param targets List 
	 * @param brGroup BusinessRuleGroup
	 * @return
	 */
	public static void updateBusinessRuleGroupForRuleSet(List targets, BusinessRuleGroup brGroup) {
		if (targets == null || targets.isEmpty() || brGroup == null) {
			return;
		}
		
		Iterator it = targets.iterator();
		Object obj = null;
		RuleSet ruleSet = null;
		
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof RuleSet) {
				ruleSet = (RuleSet) obj;
				
				Object qName = XMLTypeUtil.createQName(brGroup.getTargetNameSpace(), brGroup.getName(), "brgroup"); //$NON-NLS-1$
				ruleSet.setRulegroup(qName);
			} else if (obj instanceof DocumentRoot && ((DocumentRoot)obj).getRuleSet() != null) {
				ruleSet = ((DocumentRoot)obj).getRuleSet();
				
				Object qName = XMLTypeUtil.createQName(brGroup.getTargetNameSpace(), brGroup.getName(), "brgroup"); //$NON-NLS-1$
				ruleSet.setRulegroup(qName);
			}		
		}
	}
	
	/**
	 * Generate QName for a type
	 * @param type Type 
	 * @return QName
	 */
	public static org.eclipse.emf.ecore.xml.type.internal.QName findQNameForType (Type type) {
		if (type == null || !DataDefinitionUtil.isPrimitiveType(type)) {
			return null;
		}
		
		XSDTypeDefinition xsdType = DataDefinitionUtil.getSimpleXSDType(type);
		
		org.eclipse.emf.ecore.xml.type.internal.QName qNameGen = 
			new org.eclipse.emf.ecore.xml.type.internal.QName(xsdType.getTargetNamespace(), xsdType.getName(), "xsd");

		return qNameGen;
	}

	/**
	 * Get string value from ValueSpecification
	 * @param valueSpecification ValueSpecification 
	 * @return name
	 */
	public static String getStringValueFromValueSpecification(ValueSpecification valueSpecification) {
		if (valueSpecification == null) {
			return null;
		}
		
		if (valueSpecification instanceof LiteralSpecification) {
			LiteralSpecification literalSpecification = (LiteralSpecification) valueSpecification;
			if (literalSpecification instanceof LiteralString) {
				LiteralString literalString = (LiteralString)literalSpecification;
				return literalString.getValue();
			} else if (literalSpecification instanceof LiteralReal) {
				LiteralReal literalReal = (LiteralReal) literalSpecification;
				return literalReal.getValue().toString();
			} else if (literalSpecification instanceof LiteralInteger) {
				LiteralInteger literalInteger = (LiteralInteger) literalSpecification;
				return literalInteger.getValue().toString();
			} else if (literalSpecification instanceof LiteralUnlimitedNatural) {
				LiteralUnlimitedNatural literalUnlimitedNatural = (LiteralUnlimitedNatural) literalSpecification;
				return literalUnlimitedNatural.getValue();
			} else if (literalSpecification instanceof LiteralString) {
				LiteralString literalString = (LiteralString)literalSpecification;
				return literalString.getValue();
			} else if (literalSpecification instanceof LiteralBoolean) {
				LiteralBoolean literalBoolean = (LiteralBoolean)literalSpecification;
				if (literalBoolean.getValue()) {
					return BRExpressionConstants.BOOLEAN_TRUE;
				} else {
					return BRExpressionConstants.BOOLEAN_FALSE;
				}
			}
			
		} 
		return null;
	}
	

	@SuppressWarnings("unchecked")
	/**
	 * Update the Business Rule document (.brg)
	 * @param businessRuleGroup BusinessRuleGroup
	 * @param brAction BusinessRuleAction
	 * @param pt PortType
	 * @param baseName String 
	 * @param brGroupTable BusinessRuleGroupTable 
	 * @param targets List
	 * @return componentName
	 */	
	public static void updateBusinessRuleDocument(BusinessRuleGroup businessRuleGroup, 
							BusinessRuleAction brAction, 
							PortType pt, String baseName, 
							BusinessRuleGroupTable brGroupTable,
							List targets) {
		if (businessRuleGroup == null || brAction == null || pt == null || baseName == null) {
			return;
		}
				
		//populate attributes for BusinessRuleGroup  and set it as target
		businessRuleGroup.setTargetNameSpace(ProcessUtil.createNamespace(brAction, true));
		businessRuleGroup.setPresentationTimezone(LOCAL_TIME);
		//display name and description
		if (brAction.getActivity() != null) {	//reusable br task - comment attached to activity
			Activity activity = brAction.getActivity();
			if (activity.getOwnedComment() != null && !activity.getOwnedComment().isEmpty()) {
				businessRuleGroup.setDescription(((Comment)activity.getOwnedComment().get(0)).getBody());
			}			
		} else{
			if (brAction.getOwnedComment() != null && !brAction.getOwnedComment().isEmpty()) {
				businessRuleGroup.setDescription(((Comment)brAction.getOwnedComment().get(0)).getBody());
			}
		}
		if (brAction.getName() != null) {
			businessRuleGroup.setDisplayName(brAction.getName());
		}
		
		WSDL wsdl = ComponentdefFactory.eINSTANCE.createWSDL();
		businessRuleGroup.setWSDL(wsdl);
		
		// Create the ref group (needed by WID editors)
		ReferenceGroup refGroup = BrgFactory.eINSTANCE.createReferenceGroup();
		businessRuleGroup.setReferenceGroup(refGroup);
		
		Object qName = XMLTypeUtil.createQName(brGroupTable.getTargetNameSpace(), brGroupTable.getName(), "selt"); //$NON-NLS-1$
		businessRuleGroup.setBusinessRuleGroupTableName(qName);
		businessRuleGroup.setBusinessRuleGroupTable(brGroupTable);		

		BrUtil.addPortTypeToComponent(businessRuleGroup, brGroupTable, pt);
		
		// update the rule group in rule sets
		updateBusinessRuleGroupForRuleSet(targets, businessRuleGroup);
		return;
	}
	
	
	/**
	 * Create empty business rule group table
	 * @param businessRuleAction BusinessRuleAction
	 * @param componentName String 
	 * @param pt PortType
	 * @return businessRuleGroupTable
	 */	
	@SuppressWarnings("unchecked")
	public static BusinessRuleGroupTable createEmptyBRGroupTable(BusinessRuleAction businessRuleAction, String componentName, PortType portType) {
		if (businessRuleAction == null || componentName == null || portType == null) {
			return null;
		}
		
		// create and populate BusinessRuleTable
		BusinessRuleGroupTable brgTable = BrgtFactory.eINSTANCE.createBusinessRuleGroupTable();
		
		brgTable.setTargetNameSpace(ProcessUtil.createNamespace(businessRuleAction, true));
		String baseName = componentName.substring(componentName.lastIndexOf(FILE_SEP) + 1);
		brgTable.setName(baseName);	//normalize names
		
		OperationSelectionTable opSelectionTable = SelectiontablesFactory.eINSTANCE.createOperationSelectionTable();
		
		// set the operation name, only 1 operation allowed
		opSelectionTable.setOperationName(((Operation)portType.getOperations().get(0)).getName());	
		//there is only one operation and one operation table
		brgTable.getOperationSelectionTables().add(opSelectionTable);

		return brgTable;
	}
	
	
	/**
	 * Format presentation string using template parameters.
	 * @param businessRuleTemplate BusinessRuleTemplate
	 * @return String
	 */	
	public static String formatPresentation(BusinessRuleTemplate businessRuleTemplate) {
		if (businessRuleTemplate == null) {
			return null;
		}
		
		String presentationString = null;
		if (businessRuleTemplate.getPresentation() == null || businessRuleTemplate.getPresentation().length() == 0) {
			presentationString = BusinessRuleTaskPresentationUtil.getInstance().getDisplayablePresentation(businessRuleTemplate);
		} else {
			presentationString = businessRuleTemplate.getPresentation();
		}
		
		// if there are no parameters - return modeler presentation
		if (businessRuleTemplate.getParameters() == null || businessRuleTemplate.getParameters().isEmpty()) {
			return presentationString;
		}
		
		// create list of parameters embedded in tokens
		TemplateParameter templateParameter = null;
		String [] parameterTokens = new String [businessRuleTemplate.getParameters().size()];		
		for (int i = 0; i< businessRuleTemplate.getParameters().size(); i++) {
			templateParameter = (TemplateParameter)businessRuleTemplate.getParameters().get(i);
			parameterTokens[i] = PARAMETER_START + templateParameter.getName() + PARAMETER_END;
		}

		// replace the parameters in tokens (PARAMETER_START+templateName+PARAMETER_END) with
		//	(PRESENTATION_PARAMETER_START+templateName+PRESENTATION_PARAMETER_END)
		// eg. replace "The discount is {discountRate}" with "The discount is {0}"
		StringBuffer sb = new StringBuffer(presentationString);

		for (int i = 0; i < parameterTokens.length; i++) {
			while (sb.toString().indexOf(parameterTokens[i]) != -1) {
				sb.replace(sb.toString().indexOf(parameterTokens[i]),
						sb.toString().indexOf(parameterTokens[i]) + parameterTokens[i].length(), 
						(PRESENTATION_PARAMETER_START + i + PRESENTATION_PARAMETER_END));
			}
			// Note: error to use parameter more that once in presentation string - WID code: CWLBT7063
		}
		
		//d78299 - escape apostrophy with double apostrophy character
		if (sb.toString().indexOf(PARAMETER_START) != -1) {
			String replaceApostrophy = sb.toString().replaceAll(PARAMETER_START, "''");
			return replaceApostrophy;
		}
		
		return sb.toString();
	}

	
	/**
	 * Create and return an BRSerializationException.
	 */
	public static BRSerializationException createException(final String messageCode, final String[] params, final String methodName) {
		
		BRSerializationException brException = new BRSerializationException(null, 
																					ExpressionPlugin.getPlugin().getString(messageCode), 
																					 messageCode, 
																					 params, 
																					 BRSerializationException.ERROR, 
																					 MessageKeys.MESSAGE_RESOURCE_BUNDLE_NAME, 
																					 BRExpressionSerializer.class.getName(), 
																					 methodName);
		LogUtil.logException(messageCode, params, brException);
		
		return brException;
	}

	/**
	 * Find and return the FunctionArgument with the given ID, or null if one doesn't exist.
	 * @param argName
	 * @param arguments
	 * @return
	 */
	public static FunctionArgument findArgument(final String argID, final List arguments) {
						
		LogUtil.traceEntry(BrUtil.class, "findArgument(final String argID, final List arguments)", new String[] {"argName", "arguments"}, new Object[] {argID, arguments});
				
		FunctionArgument argument = null;
		if (argID != null && arguments != null) {
			final Iterator iterator = arguments.iterator();
			FunctionArgument thisArg = null;
			FunctionArgumentDefinition thisArgDefn = null;
			boolean found = false;
			while (iterator.hasNext() && !found) {
				thisArg = (FunctionArgument)iterator.next();
				if (thisArg != null) {
					thisArgDefn = thisArg.getDefinition();
					if (thisArgDefn != null) {
						if (argID.equals(thisArgDefn.getArgumentID())) {
							found = true;
							argument = thisArg;
						}
					}
				}
			}
		}
		
		LogUtil.traceExit(BrUtil.class, "findArgument(final String argID, final List arguments)", argument);

		return argument;				
	}
	
	/**
	 * Find and return index of this Pin in Pin Set
	 * @param pin InputObjectPin
	 * @param iPinSet InputPinSet
	 * @return index
	 */ 
	public static int findInputPinIndex(InputObjectPin pin, InputPinSet iPinSet) {
		if (pin == null || iPinSet == null) {
			return -1;
		}

		int i;
		List pins = iPinSet.getInputObjectPin();
		for (i = 0; i < pins.size(); i++) {
			if (((InputObjectPin)pins.get(i)).getUid().equals(pin.getUid())) {
					break;
			}
		}
		if (i == pins.size()) {
			return -1;	//exception?
		}
		return i;
	}

	/**
	 * Find and return index of this Pin in Pin Set
	 * @param pin OutputObjectPin
	 * @param oPinSet OutputPinSet
	 * @return index
	 */ 
	public static int findOutputPinIndex(OutputObjectPin pin, OutputPinSet oPinSet) {
		if (pin == null || oPinSet == null) {
			return -1;
		}

		int i;
		List pins = oPinSet.getOutputObjectPin();
		for (i = 0; i < pins.size(); i++) {
			if (((OutputObjectPin)pins.get(i)).getUid().equals(pin.getUid())) {
					break;
			}
		}
		if (i == pins.size()) {
			return -1;
		}
		return i;
	}

	/**
	 * Find and return index of this template in parameters' list
	 * @param templateParameter TemplateParameter
	 * @param businessRuleTemplate BusinessRuleTemplate
	 * @return index
	 */ 
	public static int findIndexOfTemplateParameter(TemplateParameter templateParameter, BusinessRuleTemplate businessRuleTemplate) {
		if (templateParameter == null || businessRuleTemplate == null || businessRuleTemplate.getParameters() == null ||
				businessRuleTemplate.getParameters().isEmpty()) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.INVALID_OPERAND, null, "findIndexOfTemplateParamter");
		}
		
		int index = 0;
		
		for (; index < businessRuleTemplate.getParameters().size(); index++) {
			if (businessRuleTemplate.getParameters().get(index).equals(templateParameter)) {
				break;
			}
		}
		if (index == businessRuleTemplate.getParameters().size()) {
			throw BrUtil.createException(com.ibm.btools.te.ilm.resource.MessageKeys.INVALID_OPERAND, null, "findIndexOfTemplateParamter");
		}
		
		return index;
	}
	
	/**
	 * Get name from PinSetRule
	 * @param pinSetRule TransformationRule
	 * @param pinIndex int
	 * @return String
	 */ 
	public static String getNameFromPinSetRule(TransformationRule pinSetRule, int pinIndex) {
		if (pinSetRule == null || pinIndex == -1) {
			return null;
		}
		
		String serializedName = null;
		if (pinSetRule.getTarget().get(0) instanceof XSDTypeDefinition) {
			XSDTypeDefinition xsdType = (XSDTypeDefinition)pinSetRule.getTarget().get(0);
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
				if (xsdGroup.getContents().size() > pinIndex) {
					if (xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
						if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
							serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
						} else {
							serializedName = xsdElement.getName();
						}
					} else if (xsdGroup.getContents().get(pinIndex) instanceof XSDParticle &&
							((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent() instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent();
						if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
							serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
						} else {
							serializedName = xsdElement.getName();
						}
					}											
				}
			}
		} else if (pinSetRule.getTarget().get(0) instanceof XSDElementDeclaration) {
			XSDElementDeclaration xsdElementWrapper = (XSDElementDeclaration)pinSetRule.getTarget().get(0);
			XSDTypeDefinition xsdType = xsdElementWrapper.getType();
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
				if (xsdGroup.getContents().size() > pinIndex && xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
					XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
					if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
							!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
						serializedName = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
					} else {
						serializedName = xsdElement.getName();
					}
				}
			}								
		}

		return serializedName;
	}
	
	/**
	 * Get name for type declaration for the collection pin (max > 1) from PinSetRule
	 * @param pinSetRule TransformationRule
	 * @param pinIndex int
	 * @return String
	 */ 
	public static String getTypeForCollectionPinFromPinSetRule(TransformationRule pinSetRule, int pinIndex) {
		if (pinSetRule == null || pinIndex == -1) {
			return null;
		}
		String wrappedType = null;
		XSDTypeDefinition cType = null;
		if (pinSetRule.getTarget().get(0) instanceof XSDTypeDefinition) {
			XSDTypeDefinition xsdType = (XSDTypeDefinition)pinSetRule.getTarget().get(0);
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
				if (xsdGroup.getContents().size() > pinIndex) {
					if (xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
						cType = xsdElement.getTypeDefinition();
					} else if (xsdGroup.getContents().get(pinIndex) instanceof XSDParticle &&
							((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent() instanceof XSDElementDeclaration) {
						XSDElementDeclaration xsdElement = (XSDElementDeclaration)((XSDParticle)xsdGroup.getContents().get(pinIndex)).getContent();
						cType = xsdElement.getTypeDefinition();
					}											
				}
			}
		} else if (pinSetRule.getTarget().get(0) instanceof XSDElementDeclaration) {
			XSDElementDeclaration xsdElementWrapper = (XSDElementDeclaration)pinSetRule.getTarget().get(0);
			XSDTypeDefinition xsdType = xsdElementWrapper.getType();
			if (xsdType instanceof XSDComplexTypeDefinition && ((XSDComplexTypeDefinition)xsdType).getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)((XSDComplexTypeDefinition)xsdType).getContent()).getContent();
				if (xsdGroup.getContents().size() > pinIndex && xsdGroup.getContents().get(pinIndex) instanceof XSDElementDeclaration) {
					XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(pinIndex);
					cType = xsdElement.getTypeDefinition();
				}
			}								
		}

		if (cType != null && cType instanceof XSDComplexTypeDefinition) {
			XSDComplexTypeDefinition xsdComplexType = (XSDComplexTypeDefinition)cType;		// wrapper type for the list is always a complex type, has only 1 element
			if (xsdComplexType.getContent() instanceof XSDParticle) {
				XSDModelGroup xsdGroup = (XSDModelGroup)((XSDParticle)xsdComplexType.getContent()).getContent();
				if (xsdGroup.getContents().get(0) instanceof XSDElementDeclaration) {
					XSDElementDeclaration xsdElement = (XSDElementDeclaration)xsdGroup.getContents().get(0);
					if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
							!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
						wrappedType = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
					} else {
						wrappedType = xsdElement.getName();
					}
				} else if (xsdGroup.getContents().get(0) instanceof XSDParticle &&
							((XSDParticle)xsdGroup.getContents().get(0)).getContent() instanceof XSDElementDeclaration) {
					XSDElementDeclaration xsdElement = (XSDElementDeclaration)((XSDParticle)xsdGroup.getContents().get(0)).getContent();
					if (xsdElement.getResolvedElementDeclaration() != null && xsdElement.getResolvedElementDeclaration().getName() != null &&
								!(xsdElement.getResolvedElementDeclaration().getName().equals(xsdElement.getName()))) {
						wrappedType = xsdElement.getResolvedElementDeclaration().getName();	// anonymous type - reference to xsdelement
					} else {
						wrappedType = xsdElement.getName();
					}
				}											
			}						
		}
		return wrappedType;
	}

	
	/**
	 * Convert LiteralTime to Calendar (based on com.ibm.btools.blm.ui.content.businessruletask.TimeStringConverterWithoutTimezone)
	 * @param timeString String
	 * @return Calendar
	 */ 
	public static Calendar timeStringToDateTime(String timeString) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(null, null, "timeStringToTime", 
					"timeString -->, " + timeString, "com.ibm.btools.blm.ui");
		}
		
		Calendar calendar = Calendar.getInstance();
		
		String dateTimeString = "";
		if (timeString.lastIndexOf("-") > 20) {
			dateTimeString = timeString.substring(0, timeString.lastIndexOf("-"));
		} else if (timeString.indexOf("+") != -1) {
			dateTimeString = timeString.substring(0, timeString.indexOf("+"));
		} else {
			dateTimeString = timeString.substring(0, timeString.length());
		}
		
		String yearString = dateTimeString.substring(0, dateTimeString.indexOf(DATE_SEPARATOR));
		calendar.set(Calendar.YEAR, new Integer(yearString).intValue());
		
		String monthString = dateTimeString.substring(dateTimeString.indexOf(DATE_SEPARATOR) + 1, dateTimeString.lastIndexOf(DATE_SEPARATOR));
		calendar.set(Calendar.MONTH, new Integer(monthString).intValue() - 1);
		
		if (dateTimeString.indexOf(TIME_KEY) != -1) {
			String dateString = dateTimeString.substring(dateTimeString.lastIndexOf(DATE_SEPARATOR) + 1, dateTimeString.indexOf(TIME_KEY));
			calendar.set(Calendar.DATE, new Integer(dateString).intValue());			
		} else {
			String dateString = dateTimeString.substring(dateTimeString.lastIndexOf(DATE_SEPARATOR) + 1);
			calendar.set(Calendar.DATE, new Integer(dateString).intValue());						
		}
		
		// for date time, set the time attributes
		if (timeString.length() > 10) {
			String hourString = dateTimeString.substring(dateTimeString.indexOf(TIME_KEY) + 1, dateTimeString.indexOf(TIME_SEPARATOR));
			calendar.set(Calendar.HOUR_OF_DAY, new Integer(hourString).intValue());
			
			String minuteString = dateTimeString.substring(dateTimeString.indexOf(TIME_SEPARATOR) + 1, dateTimeString.lastIndexOf(TIME_SEPARATOR));
			calendar.set(Calendar.MINUTE, new Integer(minuteString).intValue());
			
			String secondString = dateTimeString.substring(dateTimeString.lastIndexOf(TIME_SEPARATOR) + 1);
			int decimalIndex = secondString.indexOf('.'); 
			if (decimalIndex == -1) {
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, 0);
			} else {
				String milliSecondString = secondString.substring(decimalIndex + 1);
				secondString = secondString.substring(0, decimalIndex);
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, new Integer(milliSecondString).intValue());
			} 	
		}
		return calendar;
	}

	/**
	 * Convert LiteralTime - time value to Calendar (based on com.ibm.btools.blm.ui.content.businessruletask.TimeStringConverterWithoutTimezone)
	 * @param timeString String
	 * @return Calendar
	 */ 
	public static Calendar timeStringToTime(String timeString) {
		if (LogHelper.isTraceEnabled()) {
			LogHelper.traceEntry(null, null, "timeStringToTime", 
					"timeString -->, " + timeString, "com.ibm.btools.blm.ui");
		}
		
		Calendar calendar = Calendar.getInstance();
		
		// for date time, set the time attributes
		String hourString = timeString.substring(0, timeString.indexOf(TIME_SEPARATOR));
		calendar.set(Calendar.HOUR_OF_DAY, new Integer(hourString).intValue());
		
		String minuteString = timeString.substring(timeString.indexOf(TIME_SEPARATOR) + 1, timeString.lastIndexOf(TIME_SEPARATOR));
		calendar.set(Calendar.MINUTE, new Integer(minuteString).intValue());
		
		String secondString = null; 
			
		if (timeString.indexOf(UTC_TIME_ZONE_INDICATOR) != -1) {	// if string ends with "Z"
			secondString = timeString.substring(timeString.lastIndexOf(TIME_SEPARATOR) + 1, timeString.length() - 1);
			int decimalIndex = secondString.indexOf('.'); 
			if (decimalIndex == -1) {
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, 0);
			} else {
				String milliSecondString = secondString.substring(decimalIndex + 1);
				secondString = secondString.substring(0, decimalIndex);
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, new Integer(milliSecondString).intValue());
			}
		} else {
			secondString = timeString.substring(timeString.lastIndexOf(TIME_SEPARATOR) + 1);
			int decimalIndex = secondString.indexOf('.'); 
			if (decimalIndex == -1) {
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, 0);
			} else {
				String milliSecondString = secondString.substring(decimalIndex + 1);
				secondString = secondString.substring(0, decimalIndex);
				calendar.set(Calendar.SECOND, new Integer(secondString).intValue());
				calendar.set(Calendar.MILLISECOND, new Integer(milliSecondString).intValue());
			}			
		}
		return calendar;
	}

	/**
	 * Check if this type was created for simple types.
	 * @param type Type
	 * @return boolean
	 */ 
	public static boolean isSimpleType(Type type) {
		if (type == null || type.getOwnedComment().isEmpty()) {
			return false;
		}
		
		Iterator commentsIt = type.getOwnedComment().iterator();
		Object comment = null;
		while (commentsIt.hasNext()) {
			comment = commentsIt.next();
			if (comment != null && ((Comment)comment).getBody() != null) {
				if (((Comment)comment).getBody().equalsIgnoreCase(WBM_IMPORT_TYPE_DERIVED_SIMPLE_TYPE_COMMENT)) {
					return true;
				}
			}
		}
		
		return false;	
	}
}
