package com.ibm.btools.te.ilm.heuristics.extservice.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtserviceFactory;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtservicePackage;
import com.ibm.btools.te.ilm.heuristics.extservice.OperationRule;
import com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.util.OriginalWsdlUtil;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Port Type Rule</b></em>'.
 * Source: ServiceInterface
 * Target: WSDL PortType
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class PortTypeRuleImpl extends TransformationRuleImpl implements PortTypeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/**
	 * The WSDL port type
	 */
	private PortType portType; 	
	
	/**
	 * Boolean indicated if the class was executed more than once. Child rules should
	 * not be created in this case.
	 */
	private boolean executedOnce;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PortTypeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return ExtservicePackage.Literals.PORT_TYPE_RULE;
	}

	/**
	 * Source: BOM StructuredActivityNode with aspect="ServiceOperation"
	 * Target: WSDL PortType
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target(){
		/*
		 * The source of this rule is a StructuredActivityNode with aspect="ServiceOperation". 
		 * The target of the rule is a WSDL PortType
		 * ASSUMPTION: There is only one PortType per Action. 
		 * The method performs as follows:
		 * If a target already exist for this rule (the rule already executed 
		 * once, a PortType is not created and the method will only execute the 
		 * possible unfinished (complete = false) rules.	
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if (isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}	
		NamedElement ne = (NamedElement)getSource().get(0);
		
		//look up the original WSDL Definition 
		Definition origDef = OriginalWsdlUtil.getOriginalWsdl(getContext(),BomUtils.resolveFileAttachment((StructuredActivityNode)ne));
		PortType origPT=null;
		if (origDef != null) {
			// Lookup the Original PortType from the original WSDL Definition if available
			origPT = OriginalWsdlUtil.getPortType(origDef,origDef.getTargetNamespace(),BomUtils.resolveOwningInterface((StructuredActivityNode)ne).getName());
			if (origPT != null) {
				OriginalWsdlUtil.registerWsdlElement(getContext(),BomUtils.resolveOwningInterface((StructuredActivityNode)ne),origPT);				
			}
		} 
		if (origPT!=null) {
			portType=origPT;
		}
		// Create the OperationRule (childRules)
		if (!executedOnce ){
			if (getSource().get(0) instanceof Action) {
				OperationRule or;
				InputPinSet ips;
				OutputPinSet ops;
				Action action = (Action)getSource().get(0);
				BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(action, getContext());
				
				// Handle two way operations
				List twoWayOps = psa.getInputPinSetsAsTwoWayOperation();
				for (int i=0; i< twoWayOps.size(); i++){
					ips = (InputPinSet)twoWayOps.get(i);
					or = ExtserviceFactory.eINSTANCE.createOperationRule();
					or.getSource().add(ips);
					or.getSource().add(ips.getOutputPinSet().get(0));
					getChildRules().add(or);
				}
				boolean ret = executeChildRules();
				
//				// Handle one way operations
//				List oneWayOps = psa.getInputPinSetsAsOneWayOperation();
//				for (int i=0; i< oneWayOps.size(); i++){
//					ips = (InputPinSet)oneWayOps.get(i);
//					or = ExtserviceFactory.eINSTANCE.createOperationRule();
//					or.getSource().add(ips);
//					getChildRules().add(or);
//				}
//				ret = ret & executeChildRules();	
//				
//				//Handle call back operations
//				List callbackOps = psa.getOutputPinSetsAsCallbackOperation();
//				for (int i=0; i< callbackOps.size(); i++){
//					ops = (OutputPinSet)callbackOps.get(i);
//					or = ExtserviceFactory.eINSTANCE.createOperationRule();
//					or.getSource().add(ops);
//					getChildRules().add(or);
//				}
//				ret = ret & executeChildRules();
				
				setComplete(ret);
			}
		}
		executedOnce = true;
		ProcessUtil.putCurrentPortType(getContext(), portType);
		getTarget().add(portType);
		//AZ 20070504 handle external port type
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}	
} //PortTypeRuleImpl
