/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.AbstractFdlDataStructRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;


import com.ibm.btools.te.ilm.heuristics.fdl.MemberDeclRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlNameRegistry;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.MemberDeclaration;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Abstract Fdl Data Struct Rule</b></em>'.
 * Superclass of all the DataStructure related rules that provide some common methods to
 * facilitate data structures transformation 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class AbstractFdlDataStructRuleImpl extends TransformationRuleImpl implements AbstractFdlDataStructRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected AbstractFdlDataStructRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.ABSTRACT_FDL_DATA_STRUCT_RULE;
	}

	/**
	 * This method checks the FDL multiplicity restriction of a BOM multiplicity element
	 * and log warning message if it violates the restriction
	 * @param me BOM MultiplicityElement
	 * @return boolean
	 */
	protected boolean validateMultiplicity (MultiplicityElement me)
	{
		// Initialize logging params
		String elementName="";
		if (me instanceof NamedElement) {
			elementName = BomUtils.getCanonicalName((NamedElement)me); 
		}
		String params[] = new String[1];
		params[0] = elementName;

		// ignore this test if 
		if (me.getLowerBound()==null || me.getUpperBound()==null)
		{
			// assuming a valid BOM, this shouldn't happen
			return false;
		}
		
		String lowerBound = getMultiplicityString(me.getLowerBound());
		String upperBound = getMultiplicityString(me.getUpperBound());
		boolean bValid=true;
		if (!lowerBound.equals(upperBound))
		{
			LoggingUtil.logWarning(MessageKeys.LOWER_UPPER_BOUND_NOT_MATCH,params,null);			
			bValid=false;
		} else {		
			if (upperBound.toLowerCase().equals("n") || upperBound.toLowerCase().equals("*"))
			{
				LoggingUtil.logWarning(MessageKeys.UNBOUNDED_MULTIPLCITY_NOT_SUPPORTED,params,null);			
			} else if (upperBound.equals("0")){
				LoggingUtil.logWarning(MessageKeys.ZERO_MULTIPLCITY_NOT_SUPPORTED,params,null);			
			}
		}
		return bValid;
	}

	/**
	 * Obtains the multiplicity in a String format 
	 * @param mul
	 * @return multiplicity
	 */
	private String getMultiplicityString(ValueSpecification mul)
	{
		if (mul instanceof LiteralUnlimitedNatural)
		{
			return ((LiteralUnlimitedNatural)mul).getValue();
		} else {
			return ((LiteralInteger)mul).getValue().toString();
		}
	}
	
	/**
	 * Obtains the multiplicity in an int format
	 * @param lun
	 * @return multiplicity
	 */
	protected int getMultiplicity(ValueSpecification lun)
	{
		// if no multiplicity specify, default to 1
		if (lun == null)
			return 1;
		String val = getMultiplicityString(lun);
		// if unbound or zero multiplicity is found, default to 1
		if (val.toLowerCase().equals("n") || val.equals("0") || val.toLowerCase().equals("*") )
		{
			val="1";
		}
		return Integer.parseInt(val);
	}

	/**
	 * Creates a FDL MemberDeclaration based on BOM TypedElement
	 * @param element
	 * @return
	 */	
	protected MemberDeclaration createMemberDecl(TypedElement element)
	{
		MemberDeclRule mdRule;
		mdRule = FdlFactory.eINSTANCE.createMemberDeclRule();
		mdRule.getSource().add(element);
		getChildRules().add(mdRule);
		mdRule.transformSource2Target();
		// Ray: try to handle null type
//		if (mdRule.getTarget().isEmpty()) {
//			return null;
//		}
		return (MemberDeclaration)mdRule.getTarget().get(0);
	}
	
	/**
	 * Obtains the NameRegistry in the current context
	 * @return FdlNameRegistry
	 */
	protected FdlNameRegistry getNameRegistry(){
			return FdlUtil.getFdlNameRegistry(getContext());
		}
	
} //AbstractFdlDataStructRuleImpl
