/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;


import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Concurrent Branch Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ConcurrentBranchRuleImpl extends ProcessWalkingRuleImpl implements ConcurrentBranchRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ConcurrentBranchRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONCURRENT_BRANCH_RULE;
	}

	/**
	 * Source: BOM OuputPinSet
	 * Target: None
	 * 
	 * Maps all elements in the concurrent branch region marked by the Input Pin Set and concurrent merge point(join piont).
	 * The node of concurrent merge point is NOT mapped by this rule.
	 * This rule creates child rules to map the concurrent branch region, hence it doesn't have any rule target itself. 
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/* 
		 * tranformation includes following steps:
		 * 1. create connectors for the concurrent branch point output set.
		 * 2. for each input set reachable from the branch point output set, 
		 *    step through it, create child rules and execute the rule, 
		 *    until reached the merge point or reached end of SAN.
		 * 3. propagate all targets produced by child rules.  
		 */			 
	 		
		 if (getSource() == null || getSource().isEmpty())
			return false;
			
		//check if this is first time execute this rule
		if(!isFirstTimeExec()){
			//there is nothing to re-execute
			// reExecute(null); 
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
		  return true;
		}		
  
		setFirstTimeExec(false);	
		 //first source of this rule is one output set
		OutputPinSet outputSet = (OutputPinSet)getSource().get(0);	
		

		createConnectorRulesForOutputPinSet(outputSet);
			
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		//find the concurrent merge point
		NamedElement mergePoint = BomHelper.getInstance().findConcurrentMergePoint(outputSet);
		Action mergePointAction = null;
		if(mergePoint instanceof InputPinSet)
			mergePointAction = ((InputPinSet)mergePoint).getAction();
		else if(mergePoint instanceof StructuredActivityNode)
			mergePointAction = (Action)mergePoint;
				
		//!!main logic of this method is here
		  InputPinSet nextIS;
		  //get a list of input set reachable from the output set. since output set is concurrent branch point, this should never step out of the branch region.
		  List inputSetList = bw.getNext(outputSet, BomWalker.INPUT_SET, false);			  
		  if(inputSetList == null || inputSetList.isEmpty()){
			//we have reached end of SAN ??? what to do??
			//need do the connection for the output set of SAN to internal activities of SAN? 
			//No, because the process rule will take care of doing output set internal and external connection
		  }else{
			int count = inputSetList.size();
			for (Iterator it=inputSetList.iterator(); it.hasNext(); ){ 
				nextIS = (InputPinSet)it.next();
				if(nextIS.getAction() != mergePointAction){				
					do{									 
						nextIS = stepThroughBOM(nextIS);
						if(nextIS != null){ //means we just created a child rule
						  //we must continue
						  //bw.getNext(...) return list fo inputSet. Is it safe just to use the first elment?
						  //The answer is: since we always skip branch point, we should only have one InputPinSet back
						  List nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
						  if(nextInputPinSets == null || nextInputPinSets.isEmpty() ){						  
							  nextIS = null; //reached end of SAN
							  //Is it needed to do the connection for the output set of SAN to internal activities of SAN?
							  //The answer is no. Process rule take care of doing it. 
							  //doOutputPinSetConnection(flow, true);
						  }else{						  
							  nextIS= (InputPinSet)nextInputPinSets.get(0);	
							  if(nextIS.getAction() == mergePointAction)
								nextIS = null; //we are at the coucurrent merge point, so exclude the concurrent merge point
						  }
					  }else{ // stepTrhoughBOM didn't create child rule, so we should stop tracing this path
						//do nothing
					  }
		 
					 }while(nextIS!= null);	
				}
		  } 			 
	  }	    
  
	propagateChildTargets();	

	setComplete(true);
	LoggingUtil.traceExit(this, "transformSource2Target");
	return isComplete();
	}

} //ConcurrentBranchRuleImpl
