/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.CallOperationAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.actions.RetrieveArtifactAction;
import com.ibm.btools.bom.model.processes.actions.StoreArtifactAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.InputValuePin;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StoreArtifactPin;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;


import com.ibm.btools.te.ilm.heuristics.fdl.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.CallOperationActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ConnectableRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ConnectorRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataFlowRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataMappingRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ForkRule;
import com.ibm.btools.te.ilm.heuristics.fdl.JoinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.LoopNodeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.MapRule;
import com.ibm.btools.te.ilm.heuristics.fdl.MergeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RetrieveDatastoreArtifactActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RetrieveDatastoreArtifactPinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANNestedProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANTaskRule;
import com.ibm.btools.te.ilm.heuristics.fdl.StoreDatastoreArtifactActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.StoreDatastoreArtifactPinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;

import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ActivityExtensionSetting;
import com.ibm.btools.fdl.model.BasicActivity;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.ContainerInitial;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.StartConditionEnum;
import com.ibm.btools.fdl.model.SynchronizationType;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

/**
 * @author jiehu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Connectable Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ConnectableRuleImpl extends AbstractFdlProcDefRuleImpl implements ConnectableRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/* cache the targetActivity for re-execute */
	protected Activity targetActivity = null;
	/* keep a map of pin and the inserted program activity has been created for its ORed connectors */
	protected Map processedPins = new HashMap();
	/* keep a list of RetrieveArtifactPin mapped activity */
	protected List artifactPinActivityList = null; 
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ConnectableRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONNECTABLE_RULE;
	}

/**
 * Re-executes this rule. The re-execute including following actions:
 * If there is a target that this rule has cached, set target of incoming connectors to the cached target.
 * If there is any cached RetrieveArtifactPin mapped activity, create control connector to 
 * link them with the activity before the cached activity.
 * 
 * @param inputPinSet the InputPinSet the rule is created from.
 */
	protected void reExecute(InputPinSet inputPinSet){
	
		if(inputPinSet != null){		
			//If targetActivity is not null, re-execute the links connect to and from this activity the inputSet represent		
			if(getTargetActivity() != null){				
				setTargetOfConnectors(inputPinSet, getTargetActivity());
				
				if(artifactPinActivityList != null && !artifactPinActivityList.isEmpty()){
						//create control links to link first head of newly created activity with the task before targetActivity
						linkWithSourceActivity(inputPinSet, (Activity)artifactPinActivityList.get(0));			
				}	
					
			} //end of if is not null
					
		}
		
		//this is very bad performance. move the execute child rule responsibility to subclass itself when it has that need.
		//always execute child rules
		/*	List childRules = getChildRules();
			Iterator childRulesIterator = childRules.iterator();
			while (childRulesIterator.hasNext()){
				((TransformationRule)childRulesIterator.next()).transformSource2Target();		
			}	*/
	}

	
	/**
	 * Sets incoming connectors' target to the <code>targetActivity</code>
	 * This method also updates the join condition for the <code>targetActivity </code>.
	 * @param pinSet 		  the PinSet that its incoming connectors' target is going to be set.
	 * @param targetActivity  the activity that is going to be set as the target of incoming connectors. 
	 */
	protected void setTargetOfConnectors(PinSet pinSet, Activity targetActivity ){
		
		//set target for all incoming connectors to this activity
		
		ActivityEdge edge;
		Collection sources = new ArrayList();
		Collection connectorRules;
		ArrayList pins = new ArrayList();
		if(pinSet instanceof OutputPinSet){
		  pins.addAll(((OutputPinSet)pinSet).getOutputObjectPin());
		  pins.addAll(((OutputPinSet)pinSet).getOutputControlPin());
		}else if(pinSet instanceof InputPinSet){
		  pins.addAll(((InputPinSet)pinSet).getInputObjectPin());
		  pins.addAll(((InputPinSet)pinSet).getInputControlPin());
		}

		ControlFlow cf;
		DataFlow df;
		ConnectorRule cr;
		Collection targetTypes = new ArrayList();
		HashMap allConnectorRulesMap = new HashMap();
		for(Iterator it = pins.iterator(); it.hasNext(); ){
			Pin pin = (Pin)it.next();
			edge = pin.getIncoming();
			sources.clear();
			sources.add(edge);
			sources.add(pinSet);	
			//get FlowRules for each edge.	
			connectorRules = TransformationUtil.getRulesForSource(
			  ProcessUtil.getProcessDefinitionRule(getContext()), sources, ControlFlow.class);
			if(!connectorRules.isEmpty()){   
				allConnectorRulesMap.put(pin, new ArrayList(connectorRules));
			}   
			for(Iterator it2 = connectorRules.iterator(); it2.hasNext(); ){
				cr = (ConnectorRule) it2.next();
				//assume control flow is the first target of connector rule
				cf = (ControlFlow)cr.getTarget().get(0);
				if(cf.getToActivity() == null){	
					cf.setToActivity(targetActivity);
				}
				if(cr.getTarget().size() == 2){				
					df = (DataFlow) cr.getTarget().get(1);
					if(df.getToActivity() == null){					
						df.setToActivity(targetActivity);
						//re-execute the connector rule to cause it doing a data mapping
						cr.transformSource2Target();
					}		
				}		
					
			}
			//get DataFlowRule, there should be only one for each edge.
			targetTypes.clear();
			targetTypes.add(DataFlow.class);
			DataFlowRule dfr = (DataFlowRule)TransformationUtil.getRuleForSource(
								  ProcessUtil.getProcessDefinitionRule(getContext()), DataFlowRule.class, sources, targetTypes); 
			if(dfr != null){					
				df = (DataFlow) dfr.getTarget().get(0);
				if(df.getToActivity() == null){	
					df.setToActivity(targetActivity);
					//re-execute the connector rule to cause it doing a data mapping
					dfr.transformSource2Target();
				}						
			}
		}
		
		doUpdateJoinCondition(allConnectorRulesMap, pinSet, targetActivity);

	}
	
	/**
	 * Updates the join condition for the <code>targetActivity</code>
	 * If ANDed and ORed join condition is detected, then a NOOP activity will be inserted before the <code>targetActivity<code>.
	 * @param allConnectorRulesMap a map contains the pin as the key, list of incoming ConnectorRules for that pin as the value.
	 */
	private void doUpdateJoinCondition(Map allConnectorRulesMap, PinSet pinSet, Activity targetActivity){
		/*
		 * Join condition is determined by looking at the pinSet's incoming connectors.
		 * Incoming connectors for different pins are ANDed.
		 * Incoming connectors for same pins are ORed.
		 * If ANDed and ORed join condition is detected, a NOOP Activity will be inserted.   
		 */
		
		  //assume we only can deal with either All connectors true or one connector true as restricted by FDL

		  boolean and = false;
		  boolean or = false;
		  ControlFlow cf;
		  ConnectorRule cr;
		  
		  Map normConnectorRulesMap = consolidateConnectorRules(allConnectorRulesMap); 
		  int crSize = normConnectorRulesMap.size();	
		  if( crSize > 1){
			  and = true;
		  }else if(pinSet instanceof InputPinSet){
		  	int n = getNumberOfRetrieveArtifactPins((InputPinSet)pinSet);
		  	if( n> 1){
		  		and = true;
		  	} else if(n == 1 && crSize == 1 ){
		  		and = true;
		  	}
		  }
		  
		 List pinConnectorRules;
		  for(Iterator it= normConnectorRulesMap.keySet().iterator(); it.hasNext(); ){
		  	  Pin pin = (Pin)it.next();
			  pinConnectorRules = (List)normConnectorRulesMap.get(pin);				
			  if(pinConnectorRules.size() > 1){
				  or = true;
				  if(and){	
				  		//we need to add NOOP program to fix the ORed and ANDed combination problem
				  		insertActivity(pinConnectorRules, pin, pinSet, targetActivity);
				  		or = false;					  		
				  					  	
				  }				 
			  }
		  }		
		  
	      StartConditionEnum joinCondition =  StartConditionEnum.ALL_CONNECTORS_LITERAL;
		  if(and && or){
			  // we should never get here.
		  }else if(!and && !or){
			  // this is the case there is only one control flow
			joinCondition = StartConditionEnum.AT_LEAST_ONE_CONNECTOR_LITERAL;
		  }else if(and && !or){
			  // this is the case there are multiple control flows no ORed
			joinCondition= StartConditionEnum.ALL_CONNECTORS_LITERAL;
		  }else if(!and && or){
			  // this is the case there are multiple distinct control flows for an edge
			joinCondition = StartConditionEnum.AT_LEAST_ONE_CONNECTOR_LITERAL; 
		  }
							
		  if(targetActivity instanceof BasicActivity){	
			ActivityExtensionSetting paSetting = ((BasicActivity)targetActivity).getActivityExtensionSetting();
			if(paSetting == null){
			  paSetting = getFdlModelFactoryInstance().createActivityExtensionSetting();				  		
			}					 
			 paSetting.setStartWhen(joinCondition);
			((BasicActivity)targetActivity).setActivityExtensionSetting(paSetting);
		  } else if(targetActivity instanceof Block){
		  		((Block)targetActivity).setStartWhen(joinCondition);
		  }else{
		  	//we don't have other type		
		  }		  
			  
	}
	
	/**
	 * Returns the number of RetrieveArtifactPins in the <code>inputPinSet</code>
	 * 
	 * @param inputPinSet
	 * @return int - number of retriveArtifactPins in the <code>inputPinSet</code>
	 */
	private int getNumberOfRetrieveArtifactPins(InputPinSet inputPinSet){
		
		List objPins = inputPinSet.getInputObjectPin();
		
		int count = 0;
		for(Iterator it = objPins.iterator(); it.hasNext(); ){
			Object pin = it.next();
			if(pin instanceof RetrieveArtifactPin){
				count++;
			}
		}
		
		return count;
	}
	
	
	/**
	 * Inserts an empty activity before the <code>targetActivity</code>
	 * This is to handle the case that when there are start condition for the <code>targetActivity</code> 
	 * has ANDed and ORed combination which cannot be expressed in FDL. An empty activity is inserted to 
	 * intercept the ORed connectors and then connect with the <code>targetActivity</code>
	 * 
	 * @param pinConnectorRules	all the incoming connector rules for the pinSet  
	 * @param pin				the pin that has ORed join condition. 
	 * @param pinSet			the PinSet that the <code>pin</pin> is in.			
	 * @param targetActivity	the target activity the inserted NOOP Activity is going to link with.
	 */
	private void insertActivity(List pinConnectorRules, Pin pin, PinSet pinSet, Activity targetActivity ){
		
		if(processedPins.keySet().contains(pin)){
			//we don't need to create ProgramActivity again, simply get the one from the map
			ProgramActivity cachedActivity = (ProgramActivity)processedPins.get(pin);		
			setTargetForConnectorRules(pinConnectorRules, cachedActivity);
			return;		
		}
		
		//else, create everything
		
		DataStructure ds = null;
		  if(pin instanceof ObjectPin){
		  	ds = createDataStructure((ObjectPin)pin);
		  	
		  }else{//control pin don't need data mapping 
		  	//empty datastructure					 
			ds = createEmptyContainer();
								
		  } 
		  
		ProgramActivity insertActivity = createInsertedProgramActivity(pin, pinSet, ds);
		processedPins.put(pin, insertActivity);
		
		setTargetForConnectorRules(pinConnectorRules, insertActivity);
		
		//create connectors to connect insertActivity to targetActivity
		ControlFlow cf = getFdlModelFactoryInstance().createControlFlow();
		cf.setFromActivity(insertActivity);
		cf.setToActivity(targetActivity);
		//must add to containing parent's target, because of the timing, this rule's target already propagated.
		getContainingParentRule().getTarget().add(cf);
		if(pin instanceof ObjectPin){
			DataFlow df = getFdlModelFactoryInstance().createDataFlow();
			df.setFromActivity(insertActivity);
			df.setToActivity(targetActivity);
			//create data mapping
			List dataMappings = df.getDataMappings();
			DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
			if (FdlUtil.bOptimizeSingleObjPinContainer) {
				dm.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
				if (FdlUtil.isPinSetDataStructureOptimizable(pinSet)) {
					dm.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);			
				} else {
					int indexOfPin = (pinSet instanceof InputPinSet)? 
						(((InputPinSet)pinSet).getInputObjectPin().indexOf(pin)) :
						(((OutputPinSet)pinSet).getOutputObjectPin().indexOf(pin));
					String targetMemberName = 
						((MemberDeclaration)targetActivity.getInputDataStructure().getMemberDeclarations().get(indexOfPin)).getMemberName();
					dm.setToDataStructureMemberName(targetMemberName);
				}
			} else {
				int indexOfPin = (pinSet instanceof InputPinSet)? 
					(((InputPinSet)pinSet).getInputObjectPin().indexOf(pin)) :
					(((OutputPinSet)pinSet).getOutputObjectPin().indexOf(pin));
				String srcMemberName = 
					((MemberDeclaration)ds.getMemberDeclarations().get(0)).getMemberName();
				dm.setFromDataStructureMemberName(srcMemberName);
				String targetMemberName = 
					((MemberDeclaration)targetActivity.getInputDataStructure().getMemberDeclarations().get(indexOfPin)).getMemberName();
				dm.setToDataStructureMemberName(targetMemberName);
			}
			dataMappings.add(dm);
			//*must* add to containing parent's target, because of the timing, this rule's target already propagated.
			getContainingParentRule().getTarget().add(df);			
		}		

		
	}

/**
 * Creates an empty container.
 * @return DataStructure - the empty data structure.
 */	
	private DataStructure createEmptyContainer(){	
		DataStructure emptyContainer = null;
		//fake a outputPinSet
		  //create a fake inputPinSet so that we can use the container rule to create datastructure
		OutputPinSet fakePinSet = ActivitiesFactory.eINSTANCE.createOutputPinSet();
		fakePinSet.setName("empty");		
		//use containerRule create the empty dataStructure 
		ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
		cr.getSource().add(fakePinSet);
		getRoot().getChildRules().add(cr);
		cr.transformSource2Target();
		emptyContainer = (DataStructure)cr.getTarget().get(0);
		return emptyContainer;	
	}

	
	/**
	 * Sets target for control flow and dataflow created by connector rules.
	 * 
	 * @param pinConnectorRules connector rules for a pin.
	 * @param targetActivity 	the target activity.
	 */
	private void setTargetForConnectorRules(List pinConnectorRules, Activity targetActivity){
		//set the all DataFlow and ControlFlow' target to targetActivity
		  for(Iterator it = pinConnectorRules.iterator(); it.hasNext(); ){
			  ConnectorRule rule = (ConnectorRule)it.next();
			  ControlFlow cf = (ControlFlow)rule.getTarget().get(0);
			  cf.setToActivity(targetActivity);
			  if(rule.getTarget().size() > 1){
				  DataFlow df =(DataFlow) rule.getTarget().get(1);
				  df.setToActivity(targetActivity);
				  //force it re-do mapping
				  rule.transformSource2Target();
			  }
		}
	}	
	
	/**
	 * Creates NOOP Program Activity for the pin.
	 * 
	 * @param pin
	 * @param ds
	 * @return ProgramActivity
	 */
	private ProgramActivity createInsertedProgramActivity(Pin pin, PinSet pinSet, DataStructure ds){
		
		//create noop program actvity
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();
		//must add to containing parent's target, because of the timing, this rule's target already propagated.
		getContainingParentRule().getTarget().add(pa);
		Action action = null;
		if(pinSet instanceof InputPinSet){
			action = ((InputPinSet)pinSet).getAction();
		}else{
			action = ((OutputPinSet)pinSet).getAction();
		}
		String pinName = getNameRegistry().getActivityName(action) +"_" + pin.getName();	
		pa.setName(getNameRegistry().getActivityName(pinName, "_NOOP"));
		//pa.setName(getNameRegistry().getActivityName(FdlUtil.getCanonicalName(pin), "_NOOP"));
		//set program to the MQ predefined NOOP program
		pa.setDescription("TE inserted NOOP Activity for ORed connectors of pin: " + FdlUtil.getCanonicalName(pin));
		pa.setProgramName(FdlConstants.NOOP_PROGRAM_NAME);
		pa.setSynchronization(SynchronizationType.CHAINED_LITERAL);
		pa.setInputDataStructure(ds);
		pa.setOutputDataStructure(ds);
		//set join condition of new activity to OR
		ActivityExtensionSetting paSetting = getFdlModelFactoryInstance().createActivityExtensionSetting();
		paSetting.setStartWhen(StartConditionEnum.AT_LEAST_ONE_CONNECTOR_LITERAL);	
		pa.setActivityExtensionSetting(paSetting);
		 
		//create noop program		
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		
		//Set Default DataFlow for the empty program activity
		DataFlow df = getFdlModelFactoryInstance().createDataFlow();
		DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
		dm.setFromDataStructureMemberName("_STRUCT");
		dm.setToDataStructureMemberName("_STRUCT");
		df.getDataMappings().add(dm);
		df.setDefaultActivity(pa);
		//*must* add to containing parent's target, because of the timing, this rule's target already propagated.
		getContainingParentRule().getTarget().add(df);
		
		return pa;
		
	}
	
	/**
	 * Creates data structure for the pin.
	 * The pin will be the only field in the datastructure.
	 * 
	 * @param pin
	 * @return DataStructure
	 */
	private DataStructure createDataStructure(ObjectPin pin){
		
		DataStructure ds = null;
	
		Type type = ((ObjectPin)pin).getType();
		ValueSpecification lowerBound = null, upperBound = null;
		if(((ObjectPin)pin).getLowerBound() != null){
			lowerBound = (ValueSpecification)EcoreUtil.copy(((ObjectPin)pin).getLowerBound());
		}
		if(((ObjectPin)pin).getUpperBound() != null){
			upperBound = (ValueSpecification)EcoreUtil.copy(((ObjectPin)pin).getUpperBound());
		}			
		
		//create a fake inputPinSet so that we can use the container rule to create datastructure
		InputPinSet fakePinSet = ActivitiesFactory.eINSTANCE.createInputPinSet();
		//add object pin to it
		InputObjectPin fakePin = ActivitiesFactory.eINSTANCE.createInputObjectPin();
		fakePin.setType(pin.getType());
		fakePin.setLowerBound(lowerBound);
		fakePin.setUpperBound(upperBound);
		fakePinSet.getInputObjectPin().add(fakePin);
		
		ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
		cr.getSource().add(fakePinSet);
		getRoot().getChildRules().add(cr);
		cr.transformSource2Target();
		ds = (DataStructure)cr.getTarget().get(0);			
	
		return ds;  
	}
	
	
	/**
	 * Sets the source of outgoing connectors from the <code>pinSet</code> to <code>sourceActivity</code>.
	 * 
	 * @param pinSet 			the PinSet that its outgoing connectors' source is going to be set.
	 * @param sourceActivity 	the activity that is going to be set as the source of outgoing connectors.
	 */
	protected void setSourceOfConnectors(PinSet pinSet, Activity sourceActivity){
		
		//	set source for all outging connectors to this activity			
		  ActivityEdge edge;
		  Collection sources = new ArrayList();
		  Collection connectorRules;
		  ArrayList pins = new ArrayList();
		  if(pinSet instanceof OutputPinSet){
		  	pins.addAll(((OutputPinSet)pinSet).getOutputObjectPin());
		  	pins.addAll(((OutputPinSet)pinSet).getOutputControlPin());
		  }else if(pinSet instanceof InputPinSet){
			pins.addAll(((InputPinSet)pinSet).getInputObjectPin());
			pins.addAll(((InputPinSet)pinSet).getInputControlPin());
		  }
		 
		  ControlFlow cf;
		  DataFlow df;
		  ConnectorRule cr;
		  Collection targetTypes = new ArrayList();
	
		  for(Iterator it = pins.iterator(); it.hasNext(); ){
			  edge =((Pin)it.next()).getOutgoing();
			  sources.clear();
			  sources.add(edge);
			  sources.add(pinSet);			
			  connectorRules = TransformationUtil.getRulesForSource(
					  ProcessUtil.getProcessDefinitionRule(getContext()), sources, ControlFlow.class ); 
			  for(Iterator it2 = connectorRules.iterator(); it2.hasNext(); ){
				  cr = (ConnectorRule) it2.next();
				  //assume control flow is the first target of connector rule
				  cf = (ControlFlow)cr.getTarget().get(0);
				  if(cf.getFromActivity() == null){
				  	cf.setFromActivity(sourceActivity);
				  }
				  if(cr.getTarget().size() == 2){				
					  df = (DataFlow) cr.getTarget().get(1);
					  if(df.getFromActivity() == null){	
					  	df.setFromActivity(sourceActivity);	
					 	//re-execute the connector rule to cause it doing a data mapping
					 	cr.transformSource2Target();
					  }	
				  }						
			  }
			  
		
		
				targetTypes.clear();
				targetTypes.add(DataFlow.class);
				DataFlowRule dfr = (DataFlowRule)TransformationUtil.getRuleForSource(
									  ProcessUtil.getProcessDefinitionRule(getContext()), DataFlowRule.class, sources, targetTypes);
				if(dfr != null){				
					df = (DataFlow) dfr.getTarget().get(0);
					if(df.getFromActivity() == null){	
						df.setFromActivity(sourceActivity);
						//re-execute the connector rule to cause it doing a data mapping
						dfr.transformSource2Target();
					}
				}
		  }

	//update join condition for all the activity that the edge is leading to
	doUpdateTargetJoinCondition(pins); 
	}
	
	
	/**
	 * Updates start condition of the activity that the pin's outgoing edge leads to.
	 * @param pins
	 */
	private void doUpdateTargetJoinCondition(List pins){
	
		Collection targetSets= new HashSet();
		PinSet targetSet;	
		ActivityEdge edge;
		Collection sources = new ArrayList();
	    Collection connectorRules;
	    ConnectorRule cr;
	    ControlFlow cf;
		
		
		for(Iterator it=pins.iterator(); it.hasNext(); ){
			edge =((Pin)it.next()).getOutgoing();
			if((edge != null) && ((targetSet = (PinSet)BomUtils.findTargetSet(edge)) != null) ){
				if(!targetSets.contains(targetSet)){
					targetSets.add(targetSet);
					sources.clear();
					sources.add(targetSet);
					sources.add(edge);
					cr =(ConnectorRule)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule(getContext()), 
						sources, 
						ControlFlow.class);
					if(cr != null){
						if(cr.getTarget()!= null){
							cf = (ControlFlow)cr.getTarget().get(0);
							if(cf.getToActivity() != null){
								TransformationRule rule = getRuleForPinSet(targetSet);
								rule.transformSource2Target();					
				
							}
						}
					}
				} 	
			}
		}	
	}
	
	/**
	 * Returns a map contains all connector rules for each pin in the pinSet.
	 * The map's key is the pin, value is the list of connector rules for that pin.
	 * @param pinSet
	 * @return Map
	 */
	private Map getAllConnectorsMapForSet(PinSet pinSet){		
		
		ArrayList pins = new ArrayList();
		if(pinSet instanceof OutputPinSet){
		  pins.addAll(((OutputPinSet)pinSet).getOutputObjectPin());
		  pins.addAll(((OutputPinSet)pinSet).getOutputControlPin());
		}else if(pinSet instanceof InputPinSet){
		  pins.addAll(((InputPinSet)pinSet).getInputObjectPin());
		  pins.addAll(((InputPinSet)pinSet).getInputControlPin());
		}
	

		ActivityEdge edge;
		Collection sources = new ArrayList();
		Collection connectorRules;
		ControlFlow cf;
		DataFlow df;
		HashMap allConnectorsMap = new HashMap();
		for(Iterator it = pins.iterator(); it.hasNext(); ){
			Pin pin = (Pin)it.next();
			edge =pin.getIncoming();
			sources.clear();
			sources.add(edge);
			sources.add(pinSet);			
			connectorRules = TransformationUtil.getRulesForSource(
			  ProcessUtil.getProcessDefinitionRule(getContext()), sources, ControlFlow.class );
			if(!connectorRules.isEmpty()){			
				allConnectorsMap.put(pin, new ArrayList(connectorRules));
			} 
		}
		
		return allConnectorsMap;
		
	}
	
	/**
	 * Removes any duplicated connector rules from the connectorRulesMap and return a new map.
	 * 
	 * @param connectorRulesMap
	 * @return Map
	 */
	private Map consolidateConnectorRules(Map connectorRulesMap){
		//remove duplicate connector rules
		//duplicate rules are created because there are multiple edges going from the same souce pinSet to target pinSet 
		
		Map normMap = new HashMap();
		Map tempMap = new HashMap();
		PinSet ts, ss;
		HashSet set;
		for(Iterator it = connectorRulesMap.keySet().iterator(); it.hasNext(); ){
			Pin pin = (Pin)it.next();
			ActivityEdge edge = pin.getIncoming();
			ss =(PinSet) BomUtils.findSourceSet(edge);
			ts = (PinSet)BomUtils.findTargetSet(edge);		
			set = new HashSet();
			set.add(ss);
			set.add(ts);
			boolean found = false;
			Pin keyPin = null; 
			for(Iterator it2 = tempMap.keySet().iterator(); it2.hasNext(); ){
				keyPin = (Pin)it2.next();
				Set nextSet = (Set)tempMap.get(keyPin);
				if(nextSet.equals(set)){
					found = true;
					break;
				}
			}
			if(found){
				//see if we need replace it with objectPin's connector rule. 
				//this is needed because we will need the dataflow in the rule in creating inserted activity
				if(pin instanceof ObjectPin){
					if(!(keyPin instanceof ObjectPin)){
						tempMap.remove(keyPin);
						tempMap.put(pin, set);
						normMap.remove(keyPin);
						normMap.put(pin, connectorRulesMap.get(pin)); 		
					}
				}
				
			}else{				
				tempMap.put(pin, set);
				normMap.put(pin, connectorRulesMap.get(pin));
			}
		}
		
		return normMap;		
	}

	/**
	 * Returns the cached target activity
	 * @return Activity
	 */
	protected Activity getTargetActivity() {
		return targetActivity;
	}

	/**
	 * Caches the target activity for re-execution
	 * @param activity 
	 */
	protected void setTargetActivity(Activity activity) {
		targetActivity = activity;
	}
	
	
	/**
	 * Maps any StoreArtifactPin that reference datastore in the outputPinSet to store datastore program activity and link it with the srcActivity.
	 * 
	 * @param outputPinSet	the outputPinSet that might contain StoreArtifactPin.
	 * @param srcActivity	the activity that mapped store datastore program activity will be connected with.
	 */
	protected void mapOutputPinSet(OutputPinSet outputPinSet, Activity srcActivity){
		
		//fist map any storeArtifact pin
		Map targetMap = mapArtifactOutputPins(outputPinSet);
		if(targetMap != null){
			//for each target in the map, create connector to link it with pa
			for(Iterator it = targetMap.keySet().iterator(); it.hasNext(); ){
				OutputObjectPin pin = (OutputObjectPin)it.next();			
				Activity targetActivity = (Activity)targetMap.get(pin);
				//directly create control link
				ControlFlow ctrlFlow = getFdlModelFactoryInstance().createControlFlow();
				//ctrlFlow.setName(pin.getName());
				ctrlFlow.setFromActivity(srcActivity);
				ctrlFlow.setToActivity(targetActivity);
				getTarget().add(ctrlFlow);	
				DataFlow dataFlow = getFdlModelFactoryInstance().createDataFlow();
				//dataFlow.setName(pin.getName());
				dataFlow.setFromActivity(srcActivity);
				dataFlow.setToActivity(targetActivity);
				//create datamapping
				List dataMappings = dataFlow.getDataMappings();				
				if (FdlUtil.bOptimizeSingleObjPinContainer) {
					DataMappingRule dmr = FdlFactory.eINSTANCE.createDataMappingRule();
					if (outputPinSet.getOutputObjectPin().size()==1) {
						dmr.getSource().add(srcActivity.getOutputDataStructure());
					} else {
						int indexOfPin = outputPinSet.getOutputObjectPin().indexOf(pin);
						dmr.getSource().add(srcActivity.getOutputDataStructure().getMemberDeclarations().get(indexOfPin));
					}
//					if (pin instanceof StoreArtifactPin) {
//						Repository repos = ((StoreArtifactPin)pin).getRepository();
//						if (getMultiplicityInInt(repos.getUpperBound())==1 &&
//							repos.getType()!=null && 
//							!FdlUtil.isConvertibleToBasicDataType(repos.getType().getName())	
//						) {			
//							dmr.getSource().add(targetActivity.getInputDataStructure());
//						} else {
//							dmr.getSource().add(targetActivity.getInputDataStructure().getMemberDeclarations().get(0));				
//						}
//					} else {
//						dmr.getSource().add(targetActivity.getInputDataStructure().getMemberDeclarations().get(0));
//					}
					dmr.getSource().add(targetActivity.getInputDataStructure().getMemberDeclarations().get(0));
					getChildRules().add(dmr);
					dmr.transformSource2Target();
					dataMappings.addAll(dmr.getTarget());
				} else {
					DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
					int indexOfPin = outputPinSet.getOutputObjectPin().indexOf(pin);
					String srcMemberName = 
						((MemberDeclaration)srcActivity.getOutputDataStructure().getMemberDeclarations().get(indexOfPin)).getMemberName();
					dm.setFromDataStructureMemberName(srcMemberName);
					String targetMemberName = 
						((MemberDeclaration)targetActivity.getInputDataStructure().getMemberDeclarations().get(0)).getMemberName();
					dm.setToDataStructureMemberName(targetMemberName);
					dataMappings.add(dm);
				}							
				getTarget().add(dataFlow);					
			}

	}
		
}		

	
/**
 * Maps any StoreArtifactPin in the <code>outputPinSet</code>.
 * It uses StoreDatastoreArtifactPinRule to create store to datastore ProgramActivity.
 * @param outputPinSet
 * @return Map - a map contains the StoreArtifactPin as the key, the mapped program activity as the value. 
 */	
private Map mapArtifactOutputPins(OutputPinSet outputPinSet){
	
	Map targetMap = new HashMap();
	//create child rule for any StoreArtifactPin
	List pinList = outputPinSet.getOutputObjectPin();
	for(Iterator it = pinList.iterator(); it.hasNext(); ){
		OutputObjectPin pin = (OutputObjectPin)it.next();
		if(pin instanceof StoreArtifactPin){
			//create rule to map it
			StoreDatastoreArtifactPinRule rule = FdlFactory.eINSTANCE.createStoreDatastoreArtifactPinRule();
			rule.getSource().add(pin);
			getChildRules().add(rule);
			rule.transformSource2Target();
			targetMap.put(pin, rule.getTarget().get(0));
		}
	
	}
	return targetMap;	
}
	
	
/**
 * Maps any RetrieveArtifactPin that reference datastore in the <code>inputPinSet</code>.
 * RetrieveArtifactActionPin is mapped to a retrieve from datastore ProgramActivity.
 * If there are multiple RetrieveArtifactPin in the <code>inputPinSet<code>,
 * then their mapped activities will be connected in sequence by control connectors.
 * Each mapped activity is connected with <code>targetActivity<code> by control connector and data connector.
 *     
 * @param inputPinSet		The input pin set that may contain RetrieveArtifactPin
 * @param targetActivity	The activity that mapped activities will be connected to
 */
protected void mapInputPinSet(InputPinSet inputPinSet, Activity targetActivity){
	
	//fist map any RetrieveArtifact pin
	Map srcActivityMap = mapArtifactInputPins(inputPinSet);
	if(srcActivityMap != null && !srcActivityMap.isEmpty() ){
		artifactPinActivityList = new ArrayList();
		//for all target in the map, create control connector to link them in sequence
		for(Iterator it = srcActivityMap.keySet().iterator(); it.hasNext(); ){
			InputObjectPin pin1 = (InputObjectPin)it.next();
			Activity srcAct1 = (Activity)srcActivityMap.get(pin1);
			artifactPinActivityList.add(srcAct1);
			
			linkWithTargetActivity(pin1, inputPinSet, srcAct1, targetActivity );
			
			if(it.hasNext()){ //we link pin1's activiy with pin2's activiy				
				InputObjectPin pin2 =(InputObjectPin)it.next();					
				Activity srcAct2 = (Activity)srcActivityMap.get(pin2);
				artifactPinActivityList.add(srcAct2);
				//directly create control link
				ControlFlow ctrlFlow = getFdlModelFactoryInstance().createControlFlow();
				//ctrlFlow.setName(pin1.getName() + "-" + pin2.getName());
				ctrlFlow.setFromActivity(srcAct1);
				ctrlFlow.setToActivity(srcAct2);
				getTarget().add(ctrlFlow);
				
				linkWithTargetActivity(pin2, inputPinSet, srcAct2, targetActivity );					
		  }else{
		  	//do nothing
			}
					
		}

	}
	
	if(artifactPinActivityList != null && !artifactPinActivityList.isEmpty()){
		//create control links to link head of the newly created activity with the task that precedes targetActivity
		linkWithSourceActivity(inputPinSet, (Activity)artifactPinActivityList.get(0));			
	}

}


/**
 * Creates control connector to connect the targetActivity with 
 * the pinSet's all the incoming connectors' source activity.
 * @param pinSet
 * @param targetActivity
 */
private void linkWithSourceActivity(PinSet pinSet, Activity targetActivity){
	//only need to link using ControlFlow
	List pins = new ArrayList();
	if(pinSet instanceof InputPinSet){
		pins.addAll(((InputPinSet)pinSet).getInputControlPin());
		pins.addAll(((InputPinSet)pinSet).getInputObjectPin());
	}else{
		pins.addAll(((OutputPinSet)pinSet).getOutputControlPin());
		pins.addAll(((OutputPinSet)pinSet).getOutputObjectPin());	
	}
	
	List sources = new ArrayList();
	for(Iterator it = pins.iterator(); it.hasNext(); ){
		Pin pin = (Pin)it.next();
		ActivityEdge edge = pin.getIncoming();
		sources.clear();
		sources.add(edge);
		sources.add(pinSet);	
		//get FlowRules for each edge.	
		Collection connectorRules = TransformationUtil.getRulesForSource(
		  ProcessUtil.getProcessDefinitionRule(getContext()), sources, ControlFlow.class);
		if(!connectorRules.isEmpty()){		
			ConnectorRule cr = (ConnectorRule) connectorRules.iterator().next();
			//assume control flow is the first target of connector rule
			ControlFlow cf = (ControlFlow)cr.getTarget().get(0);
			Activity sourceActivity = cf.getFromActivity();
			if(sourceActivity != null){	
				ControlFlow ctrlFlow = getFdlModelFactoryInstance().createControlFlow();
				ctrlFlow.setFromActivity(sourceActivity);
				ctrlFlow.setToActivity(targetActivity);	
				//ctrlFlow.setName(sourceActivity.getName() + "-to-" + targetActivity.getName());
				getContainingParentRule().getTarget().add(ctrlFlow);				
			}		
		}		
	}
	
	//set targetActivity join condition to ORed
	ActivityExtensionSetting paSetting = 
		paSetting = getFdlModelFactoryInstance().createActivityExtensionSetting();				  							 
	paSetting.setStartWhen(StartConditionEnum.AT_LEAST_ONE_CONNECTOR_LITERAL);
	((BasicActivity)targetActivity).setActivityExtensionSetting(paSetting);
	
}

/**
 * Creates control and data connectors to link srcPin's activity srcAct with target activity tarAct.
 * Creates data mapping to map from the srcAct's output container to tarAct's input container.  
 * @param srcPin		Input object pin that has been mapped to an activity.
 * @param inputPinSet	The input pin set that contains the srcPin
 * @param srcAct		The activity that the srcPin is mapped to. 
 * 						The output container of srcAct has the srcPin as the first field.
 * @param tarAct		The target activity that srcAct will be connected to.
 * 						The inputPinSet is its input container. 
 */
private void linkWithTargetActivity(InputObjectPin srcPin, InputPinSet inputPinSet, Activity srcAct, Activity tarAct){
	//we link srcPin's activity with targetActivity					
	  ControlFlow ctrlFlow = getFdlModelFactoryInstance().createControlFlow();
	  //ctrlFlow.setName(srcPin.getName());
	  ctrlFlow.setFromActivity(srcAct);
	  ctrlFlow.setToActivity(targetActivity);
	  getTarget().add(ctrlFlow);
	  	
	  DataFlow dataFlow = getFdlModelFactoryInstance().createDataFlow();
	  //dataFlow.setName(srcPin.getName());
	  dataFlow.setFromActivity(srcAct);
	  dataFlow.setToActivity(targetActivity);
	  
	  //create datamapping
	  List dataMappings = dataFlow.getDataMappings();
	  if (FdlUtil.bOptimizeSingleObjPinContainer) {
		DataMappingRule dmr = FdlFactory.eINSTANCE.createDataMappingRule();
		if (srcPin instanceof RetrieveArtifactPin) {
			Repository repos = ((RetrieveArtifactPin)srcPin).getRepository();
			if (FdlUtil.getMultiplicity(repos.getUpperBound())==1 &&
				repos.getType()!=null && 
				!FdlUtil.isConvertibleToBasicDataType(repos.getType().getName())	
			) {			
				dmr.getSource().add(srcAct.getOutputDataStructure());
			} else {
				dmr.getSource().add(srcAct.getOutputDataStructure().getMemberDeclarations().get(0));				
			}
		} else {
			dmr.getSource().add(srcAct.getInputDataStructure().getMemberDeclarations().get(0));
		}
		if (inputPinSet.getInputObjectPin().size()==1) {
			dmr.getSource().add(tarAct.getInputDataStructure());
		} else {
			int indexOfPin = inputPinSet.getInputObjectPin().indexOf(srcPin);
			dmr.getSource().add(tarAct.getInputDataStructure().getMemberDeclarations().get(indexOfPin));
		}
		getChildRules().add(dmr);
		dmr.transformSource2Target();
		dataMappings.addAll(dmr.getTarget());
	} else {
		DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
	  
		int indexOfPin = inputPinSet.getInputObjectPin().indexOf(srcPin);
		//pin is the first field in srcAct output datastructure, index = 0
		String srcMemberName = 
			((MemberDeclaration)srcAct.getOutputDataStructure().getMemberDeclarations().get(0)).getMemberName();
		dm.setFromDataStructureMemberName(srcMemberName);
		String targetMemberName = 
			((MemberDeclaration)targetActivity.getInputDataStructure().getMemberDeclarations().get(indexOfPin)).getMemberName();
		dm.setToDataStructureMemberName(targetMemberName);
		dataMappings.add(dm);
	}	  
	  getTarget().add(dataFlow);
}				  		

/**
 * Maps any RetrieveArtifactPin in the inputPinSet.
 * It uses RetrieveDatastoreArtifactPinRule to create retrieve from datastore ProgramActivity.
 * @param inputPinSet
 * @return Map
 */	
private Map mapArtifactInputPins(InputPinSet inputPinSet){

	Map targetMap = new HashMap();
	//create child rule for any RetrieveArtifactPin
	List pinList = inputPinSet.getInputObjectPin();
	for(Iterator it = pinList.iterator(); it.hasNext(); ){
		InputObjectPin pin = (InputObjectPin)it.next();
		if(pin instanceof RetrieveArtifactPin){
			//create rule to map it
			RetrieveDatastoreArtifactPinRule rule = 
				FdlFactory.eINSTANCE.createRetrieveDatastoreArtifactPinRule();
			rule.getSource().add(pin);
			getChildRules().add(rule);
			rule.transformSource2Target();
			targetMap.put(pin, rule.getTarget().get(0));
		}

	}
	return targetMap;	
}

/**
 * Returns the existing rule for the pinSet. 
 * Searching start from Process Definition Rule (SANReusableProcessRule)
 * Returns null if no rule found.
 * @param pinSet the source of the rule to be searched
 * @return TransformationRule - the rule found
 */		
protected TransformationRule getRuleForPinSet(PinSet pinSet){
		//see if a rule already exists for this input set.
		Class ruleType = null;
		Action sourceAction = null;
		if(pinSet instanceof InputPinSet){		
			sourceAction = ((InputPinSet)pinSet).getAction();
		}else{
			sourceAction = ((OutputPinSet)pinSet).getAction();
		}

		if(BomUtils.isTask(sourceAction)){ 
			ruleType = SANTaskRule.class;
		}else if(BomUtils.isReusableTask(sourceAction) || BomUtils.isReusableService(sourceAction)){
			ruleType = SANReusableTaskRule.class;
		}else if (sourceAction instanceof CallBehaviorAction){ //must check this after check isReusableTask
			ruleType = CallBehaviorActionRule.class;
		}else if(BomUtils.isProcess(sourceAction)){
			ruleType = SANNestedProcessRule.class;
		}else if(sourceAction instanceof Decision){		    
			ruleType = DecisionRule.class;
		}else if(sourceAction instanceof Merge){		
			ruleType = MergeRule.class;
		}else if(sourceAction instanceof Join){
			ruleType = JoinRule.class;
		}else if(sourceAction instanceof Fork){
			ruleType = ForkRule.class;
		}else if(sourceAction instanceof RetrieveArtifactAction){
			ruleType = RetrieveDatastoreArtifactActionRule.class;
		}else if(sourceAction instanceof StoreArtifactAction){
			ruleType = StoreDatastoreArtifactActionRule.class;
		}else if(sourceAction instanceof com.ibm.btools.bom.model.processes.actions.Map){
			ruleType = MapRule.class;
		}else if(sourceAction instanceof CallOperationAction){
			ruleType = CallOperationActionRule.class;
		}else if(sourceAction instanceof LoopNode){
			ruleType = LoopNodeRule.class;
		}else{		
			//TODO: more types here
			// ruleType = null;
			// Ray 04/27/04: Try to handle non-supported Action, treat it like a task by default:
			ruleType = SANTaskRule.class;			
		}	
		TransformationRule tr = TransformationUtil.getRuleForSource(
									ProcessUtil.getProcessDefinitionRule(getContext()), 
									ruleType, 
									pinSet);
		return tr;
	 }	
	 
/**
 * Gets the rule that can contain the targets in the parent tree.
 * The containing parent rule is the one that the rule's targets will not be further propagated. 
 * @return TransformationRule
 */	 	
private TransformationRule getContainingParentRule(){
	TransformationRule parentRule;
	parentRule = getParentRule();
	while(parentRule != null){
		if(parentRule instanceof ProcessRule){
			break;
		}else{
			parentRule = parentRule.getParentRule();
		}
	}
	return parentRule;
}


/**
 * Maps any InputValuePin in the <code>inputPinSet</code> to input container initialization.
 * Returns a list of the container initialization.
 *   
 * @param inputPinSet The input pin set that may contain input value pin
 * @param inputDs The input datastructure the inputPinSet has mapped to
 * @return List - a list of ContainerInitilization objects or empty list
 */
protected List mapInputValuePin(InputPinSet inputPinSet, DataStructure inputDs){
	
	//transform valuePin to initialization of dataStructure
	List pins = inputPinSet.getInputObjectPin();
	List ciList = new ArrayList();
	for(Iterator it = pins.iterator(); it.hasNext(); ){
		InputObjectPin pin = (InputObjectPin)it.next();
		if(pin instanceof InputValuePin){
			int index = pins.indexOf(pin);
			List valSpecList = ((InputValuePin)pin).getValue();
			int size = valSpecList.size();										
			for(int i = 0; i < size; i++){
				String initValue = FdlUtil.getValueSpecificationString((ValueSpecification)valSpecList.get(i));
				MemberDeclaration md = (MemberDeclaration)inputDs.getMemberDeclarations().get(index);
				Integer arraySize = md.getArraySize();
				if(initValue != null){
					ContainerInitial ci = null;
					String memberName = md.getMemberName();
					if(size == 1 || ((i == 0) && (arraySize == null)) ){
						//arraySize == null or ==1 means there is 1 multiplicity of the pin
						ci = getFdlModelFactoryInstance().createContainerInitial();
						ci.setMemberName(memberName);						
					}else if( (arraySize != null) && (i < arraySize.intValue()) ){
						//must guard here so we throw away any extra initilization
						ci = getFdlModelFactoryInstance().createContainerInitial();
						ci.setMemberName(memberName + "(" + i + ")");
					}else{
						//do nothing, we don't want to create container initialization for value specifications that are not in the multiplicity range
					}
					if(ci != null){					
						ci.setInitialValue(initValue);
						//activity.getInputContainerInitials().add(ci);
						ciList.add(ci);
					}
				}
			}
				
			
		}
	}
	
	return ciList;
	
}


/**
 * Creates and executes branch rule (exclusive or concurrent) 
 * if the source of this rule is a branch point.
 * 
 * @param inputPinSet	The input pin set that the rule is traversing on.
 * @return TransformationRule - The branch rule created or null if no branch rule created 
 */
protected TransformationRule createBranchRuleIfAny(InputPinSet inputPinSet){		
		
		TransformationRule rule = null;		
	
		//check if input set is exclusive branch
		if(BomHelper.getInstance().isEBranch(inputPinSet)){
			rule = FdlFactory.eINSTANCE.createExclusiveBranchRule();
			rule.getSource().add(inputPinSet);
			getChildRules().add(rule);
			rule.transformSource2Target();
		
		} else { //check if output is Concurrent branch
			// it is safe to assume there is only one output set since it is not e-branch.
			OutputPinSet outputSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
			if (BomHelper.getInstance().isCBranch(outputSet)){
				rule = FdlFactory.eINSTANCE.createConcurrentBranchRule();
				rule.getSource().add(outputSet);
				getChildRules().add(rule);
				rule.transformSource2Target();	 
			} else {
				//do nothing
			}
		}
	return rule;
	}
	
	/**
	 * Creates and executes container rule for the pin set.
	 * Returns the target of the container rule - data structure.
	 * 
	 * @param pinSet The pin set the container rule is to be created
	 * @return DataStructure - The data structure representing for the pinSet
	 */
	protected  DataStructure createDataStructure(PinSet pinSet){
		
		//Ray May 07,04 - handle inputPinSet with no correlated outputPinSet
		if (pinSet == null) {
			return null; 
		}
		//get source of this rule 		
		ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
		getRoot().getChildRules().add(cr);
		cr.getSource().add(pinSet);
		cr.transformSource2Target();
		return (DataStructure)cr.getTarget().get(0);
	}	


} //ConnectableRuleImpl
