/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataTypeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Container Rule</b></em>'.
 * Source: BOM PinSet
 * Target: FDL DataStructure
 * Transform a BOM Pin Set to an FDL DataStructure.  Each pin will be transformed to a field
 * in the FDL DataStructure
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ContainerRuleImpl extends AbstractFdlDataStructRuleImpl implements ContainerRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static DataStructure emptyContainer=null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ContainerRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONTAINER_RULE;
	}

	/**
	 * Source: BOM PinSet
	 * Target: FDL DataStructure
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		PinSet ps = (PinSet)getSource().get(0);
		List objPins;
		DataStructure ds=null;
		if (ps instanceof InputPinSet)
		{
			objPins = ((InputPinSet)ps).getInputObjectPin();		
		} else {
			objPins = ((OutputPinSet)ps).getOutputObjectPin();					
		}
		// if there is at least one object pin, we will proceed and create the data structure
		// otherwise, we will create a Default Empty Container data structure
		if (objPins.size()>0)
		{	
			if (FdlUtil.isPinSetDataStructureOptimizable(ps)) {
				ObjectPin objPin = (ObjectPin)objPins.get(0);
				Type type = objPin.getType();
				DataTypeRule dtr = FdlFactoryImpl.eINSTANCE.createDataTypeRule();
				dtr.getSource().add(type);
				getChildRules().add(dtr);
				dtr.transformSource2Target();
				ds = (DataStructure)dtr.getTarget().get(0);
				} else {
					String dsNameKey = getNameRegistry().computeDataStructureKey(ps); 
					 // First check if the same data structure has already been created
					 ds = (DataStructure)getContext().get(FdlConstants.FDL_CONTAINER_CONTEXT_KEY_PREFIX+dsNameKey);
					 // If no data structure is found in the context, creates a new one
					 // Ray- May 19,04 - Fix the name collision problem when processes are exported in different session
					 // String dsName = getNameRegistry().getDataStructureName(ps);
					 String dsName = getNameRegistry().getUniqueDataStructureName(dsNameKey);
					 if (ds == null)
					 {
						 ds = com.ibm.btools.fdl.model.impl.ModelFactoryImpl.eINSTANCE.createDataStructure();
						 ds.setName(dsName);
						 //Ray May 19,04 
						 //ds.setDescription(getNameRegistry().getDescription(FdlUtil.getFullName(ps)));
						 // store this in the context for later retrieval
						 getContext().put(FdlConstants.FDL_CONTAINER_CONTEXT_KEY_PREFIX+dsNameKey,ds);
						 Iterator it = objPins.iterator();
						 ObjectPin objPin;
						 MemberDeclaration md;
						 int arraySize;
						 while (it.hasNext())
						 {
							 objPin = (ObjectPin)it.next();
							 md = createMemberDecl(objPin);
							 // Ray: try to handle null type
//							 if (md != null) {
								 md.setMemberName(createMemberName(ds,objPin));
								 ds.getMemberDeclarations().add(md);						
//							 }
						 }
					 }								
				}
		} else {
			ds = createEmptyContainer();
		}
		getTarget().add(ds);
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();		
	}

	/**
	 * Creates an empty data structure message
	 * @return
	 */
	private DataStructure createEmptyContainer() {
		if (emptyContainer == null) {
			emptyContainer = com.ibm.btools.fdl.model.impl.ModelFactoryImpl.eINSTANCE.createDataStructure();
			emptyContainer.setName(FdlConstants.EMPTY_CONTAINER_NAME);
		}
		return emptyContainer;
	}
	
	/**
	 * Creates a valid name for the FDL member declaration
	 * @param ds
	 * @param objPin
	 * @return
	 */
	private String createMemberName(DataStructure ds, ObjectPin objPin)
	{
		String name = getNameRegistry().getDataStructureMemberName(ds, objPin);
		return name;
	}
	
} //ContainerRuleImpl
