/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.heuristics.fdl.ConnectorRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainmentRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataFlowRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FlowRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.BooleanExpression;
import com.ibm.btools.fdl.model.ConnectorExpression;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.Expression;
import com.ibm.btools.fdl.model.FDLOperator;
import com.ibm.btools.fdl.model.NumericExpression;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.fdl.model.Source;
import com.ibm.btools.fdl.model.Process;
import com.ibm.btools.fdl.model.util.FDLExpressionSerializer;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Containment Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ContainmentRuleImpl extends ConnectableRuleImpl implements ContainmentRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private HashMap cfMap = new HashMap();
	private HashMap dfMap = new HashMap();
	private HashMap ccfMap = new HashMap();
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ContainmentRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.CONTAINMENT_RULE;
	}

/**
 * Processes targets propagated by child rules and set it to Process or Block according to their type.
 * 
 * @param obj - FDL Process or Block
 */
	protected void processChildTargets(Object obj){
		List constructs = null;
		boolean isProcess = true;
		if(obj instanceof Process){
			constructs = ((Process)obj).getConstructs();
			
		}else if(obj instanceof Block){
			constructs = ((Block)obj).getConstructs();
			isProcess = false;
		}
			
		Object target;
		DataFlow df;
		ControlFlow cf;
		TransformationRule childRule;
	
		for(Iterator it = getChildRules().iterator(); it.hasNext(); ){
			childRule = (TransformationRule)it.next();
			for(Iterator it2 = childRule.getTarget().iterator(); it2.hasNext(); ){
				target = it2.next();
				if(target instanceof Activity){					
					constructs.add(target);				
				}else if(target instanceof ControlFlow){
						if((cf = validateControlFlow((ControlFlow)target)) != null){
							constructs.add(cf);
						}
				}else if(target instanceof DataFlow){
						if((df = consolidateDataFlow((DataFlow)target)) != null){
							constructs.add(df);				
						}
				}else if(target instanceof Source){
					if(isProcess){
//						((Process)obj).setSource((Source)target);
						((Process)obj).getSource().add((Source)target);
					}
					else{
						((Block)obj).getSource().add((Source)target);
//						((Block)obj).setSource((Source)target);						
					}
							
				}else if(target instanceof Sink){
					if(isProcess){
//						((Process)obj).setSink((Sink)target);
						((Process)obj).getSink().add((Sink)target);
					}
					else{
						((Block)obj).getSink().add((Sink)target);
//						((Block)obj).setSink((Sink)target);
					}
				}else{
					//propogate all other target (DataStructure, Program etc)
					getTarget().add(target); 
				}
			}
		}
	
		//add conditioned control flow collected in ccfMap
		for(Iterator it=ccfMap.values().iterator(); it.hasNext(); ){
			constructs.add(it.next());
		}
	}
/**
 * Validates the control flow passed in. 
 * Control flow is not valid if it doesn't have source or target, 
 * or a control flow from the same source and target already exist. 
 * @param cf ControlFlow that is to be validated 
 * @return ControlFlow or null if invalid or already exist.
 */
	
	private ControlFlow validateControlFlow(ControlFlow cf){
		//here eliminate any invalid control flows.
		if(cf.getFromActivity() == null || cf.getToActivity() == null){
			//TODO: Ray March 18 ... now that we add optimization at the end to remove noops, the analysis
			// requires to analyze the number of incoming and outgoing control flows so we should fix it here as well
			cf.setFromActivity(null);
			cf.setToActivity(null);			
			return null;
		}
	
		if(cf.getWhen()!= null){
			consolidateConditionedControlFlow(cf);
			return null;
		}
		//check if the control flow already exist
		ConnectorKey key = new ConnectorKey();
		key.setSource(cf.getFromActivity());
		key.setTarget(cf.getToActivity()); 
		if(cfMap.containsKey(key)){
			//TODO: Ray March 18 ... now that we add optimization at the end to remove noops, the analysis
			// requires to analyze the number of incoming and outgoing control flows so we should fix it here as well
			cf.setFromActivity(null);
			cf.setToActivity(null);
			//TODO: Ray March 18
			 return null;
		}else {			
			cfMap.put(key, cf);
			return cf;			
		}
	}
	
	/**
	 * Merges condtions of control flows have same source and target into one condition.  
	 * @param cf ControlFlow that is to be merged.
	 */	
	protected void consolidateConditionedControlFlow(ControlFlow cf){
		/* This is to handle cases that when multiple control flow from decision goes to the same target acticity.
			we need to consolidate the flow condition into one since we can only have one control flow to the target.
			we ORed transition conditions of all control flows
			//e.g. testCase#42 with 3 decisions
		*/
	
		//check if we already have this control flow
		ConnectorKey key = new ConnectorKey();
		key.setSource(cf.getFromActivity());
		key.setTarget(cf.getToActivity());
	
		if(!ccfMap.containsKey(key)){
			ccfMap.put(key, cf);
			return;	
		}
	
		ControlFlow oldCf = (ControlFlow)ccfMap.get(key);
		//String oldEx = oldCf.getWhen().getBody();		
		//String ex = cf.getWhen().getBody();	
		FDLExpressionSerializer ser = new FDLExpressionSerializer();
		String oldEx = ser.serialize(oldCf.getWhen());
		String ex = ser.serialize(cf.getWhen());	
		if(oldEx.equals(ex)){
			//this could be multiple pins in the same Decision outputPinSet goes to same target
			//don't need to update anything
			return; 
		}		
	
		//oldEx = oldEx.replaceAll("\\) AND NOT \\(", "\\) OR \\(");
		//ex = ex.replaceAll("\\) AND NOT \\(", "\\) OR \\(");
		Expression oldExpression = null;
		Expression newExpression = null;
		oldExpression = replaceConnectorExpr(oldCf.getWhen());
		newExpression = replaceConnectorExpr(cf.getWhen());
		oldEx = ser.serialize(oldExpression);
		ex = ser.serialize(newExpression);
		if(ex.indexOf(oldEx) != -1){
			//use the ex as the expression
			//oldCf.getWhen().setBody(ex);
			oldCf.setWhen(newExpression);
		} else if(oldEx.indexOf(ex) != -1){
			//do nothing
		} else{
			//log debug info
			//System.out.println("condition not correct in one of the control flow");
		}

	}
	
	/**
	 * Validates data flow.
	 * Flows are not valid if it doesn't have source or target. 
	 * If the data flow is valid, then consolidates data flows from same source and targets 
	 * including merging the data mappings. 
	 * @param df
	 * @return DataFlow or null if invalid or alreay exist.
	 */	
	protected DataFlow consolidateDataFlow(DataFlow df){
		if(df.getDefaultActivity() != null){
			return df;
		}

		if((df.getFromActivity() == null && df.getSource()== null) || 
			(df.getToActivity()== null && df.getSink() == null)){
				//TODO: Ray March 18 ... now that we add optimization at the end to remove noops, the analysis
				// requires to analyze the number of incoming and outgoing control flows so we should fix it here as well
				df.setFromActivity(null);
				df.setToActivity(null);
				//TODO: Ray March 18				
			return null; 
		}
	
		Object src = df.getFromActivity();
		if (src == null){
			src = df.getSource();
		}
		Object target = df.getToActivity();
		if(target == null){ 
			target = df.getSink();
		}
		ConnectorKey key = new ConnectorKey();
		key.setSource(src);
		key.setTarget(target);
 
		if(dfMap.containsKey(key)){
			DataFlow oldDf = (DataFlow)dfMap.get(key);
			DataMapping oldDm, newDm = null;
			boolean hasDm;
			List mappings = df.getDataMappings();
			for(int i= 0, size= mappings.size(); i<size; i++){					
				newDm = (DataMapping) mappings.get(i);
				hasDm = false;
				List oldMappings = oldDf.getDataMappings();
				for(int j= 0, size2= oldMappings.size(); j<size2; j++){
					oldDm = (DataMapping)oldMappings.get(j);
					if((oldDm.getFromDataStructureMemberName().equals(newDm.getFromDataStructureMemberName())) &&
						(oldDm.getToDataStructureMemberName().equals(newDm.getToDataStructureMemberName()))){
						hasDm = true;										
						break;
						}
				}
				if(!hasDm){								
					oldDf.getDataMappings().add(newDm);
				}
			}
			//TODO: Ray March 18 ... now that we add optimization at the end to remove noops, the analysis
			// requires to analyze the number of incoming and outgoing control flows so we should fix it here as well
			df.setFromActivity(null);
			df.setToActivity(null);
			//TODO: Ray March 18
			return null;
		}else{ //save new dataflow to map
			dfMap.put(key, df);
			return df;
		}

	}	
			
			
/**
 * Creates FDL Source to represent the BOM InputPinSet.
 * If <code>inputPinSet</code> contains only control pin or is empty,
 * no source is created. 
 * @param inputPinSet
 * @return Source - FDL Source object or null
 */
protected Source createSource(InputPinSet inputPinSet){
	
	//if inputPinSet doesn't contain any object pin or it is empty, no need to create source
	List inObjPinSetList = inputPinSet.getInputObjectPin();
	if(inObjPinSetList == null || inObjPinSetList.isEmpty() ){
		return null;
	}
		
	//create Source 
	 Source src = getFdlModelFactoryInstance().createSource();
	 src.setId(new Integer(1));		 
	 return src;	 	
		 	
}	


/**
 * Creates FDL Sink to represent the BOM OutputPinSet.
 * If <code>outputPinSet</code> contains only control pin or is empty,
 * no source is created. 
 * @param outputPinSet
 * @return Sink - FDL Sink object or null
 */
protected Sink createSink(OutputPinSet outputPinSet){
	
	//if outputPinSet doesn't contain any object pin or it is empty, no need to create Sink
	  List outObjPinSetList = outputPinSet.getOutputObjectPin();
	  if(outObjPinSetList == null || outObjPinSetList.isEmpty() ){
		  return null;
	  }	
	//create Sink
	 Sink sink = getFdlModelFactoryInstance().createSink();
	 sink.setId(new Integer(1));		 
	 return sink;	 	
		 	
}		



/**
 *	Connects OutputPinSet of the SAN including:  
 *   1. Gets all ConnectorRules that represent the incoming edges of the OutputPinSet, 
 *   2. Set FDL Sink as the sink of the data flows.
 *
 *  @param outputSet
 *  @param fdlSink  	    	 
 */
			
	protected void connectOutputPinSet(OutputPinSet outputSet, Sink fdlSink){				

	
		List pinList = new ArrayList();
	  
		pinList.addAll(outputSet.getOutputObjectPin());  
		Pin pin;
		ActivityEdge edge;
		Collection sources = new ArrayList();
		Collection connectorRules;
		ConnectorRule cr;
		DataFlow df;
			
		for(Iterator it = pinList.iterator(); it.hasNext(); ){
			pin = (Pin)it.next();			
			edge = pin.getIncoming();
			sources.clear();
			sources.add(edge);
			sources.add(outputSet);			
			connectorRules = TransformationUtil.getRulesForSource(
			  ProcessUtil.getProcessDefinitionRule(getContext()), sources, DataFlow.class );  
		  
			for(Iterator it2 = connectorRules.iterator(); it2.hasNext(); ){
				cr = (ConnectorRule) it2.next();
				if(cr instanceof DataFlowRule){					
					df = (DataFlow)cr.getTarget().get(0);
				} else if(cr instanceof FlowRule){
					//assume data flow is the second target of connector rule
					df = (DataFlow)cr.getTarget().get(1);
				}else{
					return;
				}
				df.setSink(fdlSink);
				//re-execute the rule to cause the cr to do the mapping at this time.
				cr.transformSource2Target();			
			
			}		
			
		}		
	}		


	/**
	 * Connects InputPinSet of the SAN including:
	 *  
	 * 1. Creates ConnectorRules representing the InputPinSet outgoing edges.	  
	 * 2. Sets FDL Source as the source of DataFlows.	
	 */
	protected void connectInputPinSet(InputPinSet inputSet, Source fdlSource){
		
 
	   List pinList = new ArrayList();
	 	
	   //get object pin from the input Set
	   pinList.addAll(inputSet.getInputObjectPin());  
   
		//pin in the input set is connected to an input pin on the inside of the SAN, 
		//create a ConnectorRule representaiton of this connection.
		//check each pin in the input set	  
		Pin pin;	
		ConnectorRule cr;	
		for (Iterator it= pinList.iterator(); it.hasNext(); ) {
			pin = (Pin) it.next();
			PinSet targetPinSet = (PinSet)BomUtils.findTargetSet(pin.getOutgoing());
			if(targetPinSet != null){		
				cr = FdlFactory.eINSTANCE.createDataFlowRule();
				cr.getSource().add(pin.getOutgoing()); //edge
				cr.getSource().add(inputSet); //input set
				cr.getSource().add(targetPinSet);
				
				getChildRules().add(cr);
				cr.transformSource2Target();
				//set source for the DataFlow created by the cr.
				((DataFlow)cr.getTarget().get(0)).setSource(fdlSource);
			}
		} 
	}	
		/**
	 * Replaces the operator in any ConnectorExpressions from 
	 * operator = AND NOT to operator=OR
	 * @param expr - Expression
	 * @return Expresion - the accumulated expression
	 */
	private Expression replaceConnectorExpr(Expression expr)
	{	
		if (expr instanceof BooleanExpression){
			if (((BooleanExpression)expr).getFirstOperand()!=null)
				((BooleanExpression)expr).setFirstOperand(replaceConnectorExpr(((BooleanExpression)expr).getFirstOperand()));
			((BooleanExpression)expr).setSecondOperand(replaceConnectorExpr(((BooleanExpression)expr).getSecondOperand()));
		}
		/*
		else if (expr instanceof IntegerExpression){
			if (((IntegerExpression)expr).getFirstOperand()!=null)
				((IntegerExpression)expr).setFirstOperand(replaceConnectorExpr(((IntegerExpression)expr).getFirstOperand()));
			((IntegerExpression)expr).setSecondOperand(replaceConnectorExpr(((IntegerExpression)expr).getSecondOperand()));
		}
		*/
		else if (expr instanceof NumericExpression){
			if (((NumericExpression)expr).getFirstOperand()!=null)
				((NumericExpression)expr).setFirstOperand(replaceConnectorExpr(((NumericExpression)expr).getFirstOperand()));
			((NumericExpression)expr).setSecondOperand(replaceConnectorExpr(((NumericExpression)expr).getSecondOperand()));
		}
		else if (expr instanceof ConnectorExpression){
			((ConnectorExpression)expr).setOperator(FDLOperator.OR_LITERAL);
			((ConnectorExpression)expr).setFirstOperand(replaceConnectorExpr(((ConnectorExpression)expr).getFirstOperand()));
			((ConnectorExpression)expr).setSecondOperand(replaceConnectorExpr(((ConnectorExpression)expr).getSecondOperand()));
		}		
		return expr;
	}
			
		
} //ContainmentRuleImpl
