/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DataMappingRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataStructureRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.MemberDeclRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DataType;
import com.ibm.btools.fdl.model.MemberDeclaration;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Data Mapping Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DataMappingRuleImpl extends AbstractFdlDataStructRuleImpl implements DataMappingRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DataMappingRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.DATA_MAPPING_RULE;
	}

	/**
	 * Source: Source and Target FDL Member Declaration
	 * Target: One or more FDL DataMapping
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		Collection mappings=null;
		
		if (getSource().get(0) instanceof MemberDeclaration) {
			MemberDeclaration srcMd = (MemberDeclaration)getSource().get(0);
			if(getSource().size()>1){ 
				if (getSource().get(1) instanceof MemberDeclaration) {
					mappings = createDataMappings(srcMd,(MemberDeclaration)getSource().get(1));
				} else if (getSource().get(1) instanceof DataStructure) {
					mappings = createDataMappings(srcMd,(DataStructure)getSource().get(1));
				}
			}else{ 
				mappings = createDataMappings(srcMd,srcMd);
			}	
		} else if (getSource().get(0) instanceof DataStructure) {
			DataStructure srcType = (DataStructure)getSource().get(0);
			if(getSource().size()>1){ 
				if (getSource().get(1) instanceof MemberDeclaration) {
					mappings = createDataMappings(srcType,(MemberDeclaration)getSource().get(1));
				} else if (getSource().get(1) instanceof DataStructure) {
					DataStructure tarType = (DataStructure)getSource().get(1);
					if (srcType == tarType) { // will this case ever happen?  if srcType=tarType then getSource().size() will be 1
						mappings = new ArrayList();
						mappings.add(createDataMapping(FdlConstants.TOP_STRUCTURE_NAME,FdlConstants.TOP_STRUCTURE_NAME));
					}
				}
			}else{ 
				mappings = new ArrayList();
				mappings.add(createDataMapping(FdlConstants.TOP_STRUCTURE_NAME,FdlConstants.TOP_STRUCTURE_NAME));
			}	
		}
		
		if (mappings != null) {
			getTarget().addAll(mappings);
		}
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();				
		
	}

	private Collection createDataMappings(MemberDeclaration srcMd,DataStructure tarType)
	{

		ArrayList mappings = new ArrayList();
		DataType srcMdType = srcMd.getType();
		// Ray: try to handle null type
		if (srcMdType == null || tarType == null) {
			return mappings;
		}
		if (srcMdType == tarType)
		{
			mappings.add(createDataMapping(srcMd.getMemberName(),FdlConstants.TOP_STRUCTURE_NAME));
		} else {
			// this is the case where the source Pin type is a subClass of the target Pin type
			//mappings.addAll(createSuperClassDataMapping(srcType,tarMd));			
		}
		return mappings;
	}

	private Collection createDataMappings(DataStructure srcType, MemberDeclaration tarMd)
	{

		ArrayList mappings = new ArrayList();
		DataType tarMdType = tarMd.getType();
		// Ray: try to handle null type
		if (srcType == null || tarMdType == null) {
			return mappings;
		}
		if (srcType == tarMdType)
		{
			mappings.add(createDataMapping(FdlConstants.TOP_STRUCTURE_NAME,tarMd.getMemberName()));
		} else {
			// this is the case where the source Pin type is a subClass of the target Pin type
			//mappings.addAll(createSuperClassDataMapping(srcType,tarMd));			
		}
		return mappings;
	}
	
	private Collection createDataMappings(MemberDeclaration srcMd, MemberDeclaration tarMd)
	{

		ArrayList mappings = new ArrayList();
		DataType srcMdType = srcMd.getType();
		DataType tarMdType = tarMd.getType();
		// Ray: try to handle null type
		if (srcMdType == null || tarMdType == null) {
			return mappings;
		}
		if (srcMdType == tarMdType)
		{
			mappings.addAll(createDataMapping(srcMd,tarMd));
		} else {
			// this is the case where the source Pin type is a subClass of the target Pin type
			mappings.addAll(createSuperClassDataMapping(srcMd,tarMd));			
		}
		return mappings;
	}
	
	private Collection createSuperClassDataMapping (MemberDeclaration srcMd, MemberDeclaration tarMd)
	{
		DataType srcMdType = srcMd.getType();
		DataType tarMdType = tarMd.getType();

		// Need to get the DataStructureRule that transformed the type of the source Pin
		// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
		// the MemberDeclaration for the flatten super class
		DataStructureRule srcPinDsr = (DataStructureRule)
			TransformationUtil.getRuleForTarget(getRoot(),DataStructureRule.class,srcMdType);
				
		// Need to get the DataStructureRule that transformed the type of the target Pin
		// to FDL DataStructure so that we can later on use getRuleForSource to retrieve 
		// the MemberDeclaration for the super class		
		DataStructureRule tarPinDsr = (DataStructureRule)
			TransformationUtil.getRuleForTarget(getRoot(),DataStructureRule.class,tarMdType);

		ArrayList mappings = new ArrayList();
		int srcMdNum = getArraySize(srcMd);
		int tarMdNum = getArraySize(tarMd);;		
		String srcMdName=srcMd.getMemberName();
		String tarMdName=tarMd.getMemberName();
		String ind="";		

		Collection superClassMappings;
		com.ibm.btools.bom.model.artifacts.Classifier superClass =
			(com.ibm.btools.bom.model.artifacts.Classifier)tarPinDsr.getSource().get(0);
		if (srcMdNum==0 || tarMdNum==0) {
			superClassMappings = 
				createSuperClassMemberMappings(superClass,srcPinDsr,tarPinDsr);
			if (tarMdNum==srcMdNum) {
				updateSuperClassMappings(superClassMappings,srcMdName,tarMdName,ind,ind);
			} else if (srcMdNum==0){
				updateSuperClassMappings(superClassMappings,srcMdName,tarMdName,ind,"(0)");
			} else {
				updateSuperClassMappings(superClassMappings,srcMdName,tarMdName,"(0)",ind);
			}
			mappings.addAll(superClassMappings);	
		} else {		 
			String indexStr;
			// If the array sizes between the source and target pins
			// are different, we will pick the smaller size for mapping
			int arrayInd=(tarMdNum < srcMdNum)?tarMdNum:srcMdNum;
			for (int i=0; i<arrayInd;i++)
			{
				indexStr="("+i+")";
				superClassMappings = 
					createSuperClassMemberMappings(superClass,srcPinDsr,tarPinDsr);
				updateSuperClassMappings(superClassMappings,srcMdName,tarMdName,indexStr,indexStr);
				mappings.addAll(superClassMappings);
			}
		}	
		return mappings;		
//
//			
//		// Create superclass mappings	
//		Collection superClassMappings = createSuperClassMemberMappings(
//		(com.ibm.btools.bom.model.artifacts.Class)tarPinDsr.getSource().get(0),srcPinDsr,tarPinDsr);
//			
//		// Need to append the original attribute name to the mappings
//		int numMappings = superClassMappings.size();
//		String srcMN= srcMd.getMemberName();
//		String tarMN= tarMd.getMemberName();
//		String ind="";
//		// Handle array
//		int srcMdNum = getArraySize(srcMd);
//		int tarMdNum = getArraySize(tarMd);
//		if ((srcMdNum==0) && (tarMdNum==0)) {
//			updateSuperClassMappings(superClassMappings,srcMN,tarMN,ind,ind);
//		} else if (srcMdNum==0){
//			updateSuperClassMappings(superClassMappings,srcMN,tarMN,ind,"(1)");
//		} else {
//			Collection mappingsCopy=cloneMappings(superClassMappings);
//			for (int i=0; i<srcMdNum; i++)
//			{
//				updateSuperClassMappings(mappingsCopy,srcMN,tarMN,ind,"(1)");
//				mappings.addAll(mappingsCopy);
//				mappingsCopy=cloneMappings(superClassMappings);
//			}
//		}
//		mappings.addAll(superClassMappings);
	}
	
	private void updateSuperClassMappings(Collection superClassMappings,String srcMN,String tarMN,String srcInd,String tarInd)
	{
		Iterator it = superClassMappings.iterator();
		DataMapping superClassMapping;
		while (it.hasNext())
		{
			superClassMapping = (DataMapping)it.next();
			superClassMapping.setFromDataStructureMemberName(
				srcMN+srcInd+"."+superClassMapping.getFromDataStructureMemberName());
			superClassMapping.setToDataStructureMemberName(
				tarMN+tarInd+"."+superClassMapping.getToDataStructureMemberName());
		}
			
	}
	private Collection createSuperClassMemberMappings(
		com.ibm.btools.bom.model.artifacts.Classifier superClass,
		DataStructureRule srcPinDsr,
		DataStructureRule tarPinDsr
		)
	{
		ArrayList mappings = new ArrayList();
		List attributes = FdlUtil.getOwnedAttribute(superClass);
		Iterator it = attributes.iterator();
		Property prop;
		MemberDeclaration srcMd, tarMd;
		while (it.hasNext())
		{
			prop = (Property)it.next();
			// get the source member declaration that represents the flatten super class property/attribute
			srcMd = (MemberDeclaration)
					((MemberDeclRule)TransformationUtil.getRuleForSource(srcPinDsr,MemberDeclRule.class,prop)).getTarget().get(0);
			// get the target member declaration that represents the super class property/attribute
			tarMd =	(MemberDeclaration)
					((MemberDeclRule)TransformationUtil.getRuleForSource(tarPinDsr,MemberDeclRule.class,prop)).getTarget().get(0);
			mappings.addAll(createDataMapping(srcMd,tarMd));
		}
		// Recursively create mappings for superclasses
		List superClasses = superClass.getSuperClassifier();
		it = superClasses.iterator();
		while (it.hasNext())
		{
			mappings.addAll(
				createSuperClassMemberMappings(
					(com.ibm.btools.bom.model.artifacts.Classifier)it.next(),
					srcPinDsr,
					tarPinDsr
					)
				);
		}
		return mappings;
	}
	
	private Collection createDataMapping (MemberDeclaration srcMd, MemberDeclaration tarMd)
	{
		ArrayList mappings = new ArrayList();
		int srcMdNum = getArraySize(srcMd);
		int tarMdNum = getArraySize(tarMd);;		
		String srcMdName=srcMd.getMemberName();
		String tarMdName=tarMd.getMemberName();
		// if both Source and Target arraysizes is null (that means they
		// are not arrays) or if the arraysizes are the same, then
		// we only need to create a single data mapping.  Otherwise we
		// have to create multiple mappings for the array
		if (srcMdNum==tarMdNum)
		{
			mappings.add(createDataMapping(srcMdName,tarMdName));				
		} else {
//			if (tarMdNum < srcMdNum)
//			{
//				// TODO: Loggin Error
//				System.out.println("ERROR: target array index smaller than source array index");
//				throw new BTRunTimeException("target array index smaller than source array index");
//			}
			
			if (srcMdNum==0) {
				mappings.add(createDataMapping(srcMdName,tarMdName+"(0)"));
			} else if (tarMdNum==0) {	
				mappings.add(createDataMapping(srcMdName+"(0)",tarMdName));
			} else {
				String indexStr;
				// If the array sizes between the source and target pins
				// are different, we will pick the smaller size for mapping
				int arrayInd=(tarMdNum < srcMdNum)?tarMdNum:srcMdNum;
				for (int i=0; i<arrayInd;i++)
				{
					indexStr="("+i+")";
					mappings.add(createDataMapping(srcMdName+indexStr,tarMdName+indexStr));									
				}
			}
		}	
		return mappings;		
	}
	
	private DataMapping createDataMapping(String fromName, String toName)
	{
		DataMapping dMap = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataMapping();
		dMap.setFromDataStructureMemberName(fromName);
		dMap.setToDataStructureMemberName(toName);
		return dMap;		
		
	}

	private int getArraySize (MemberDeclaration md)
	{
		Integer mdInt = md.getArraySize();
		if (mdInt == null) {
			return 0;
		} else {
			return mdInt.intValue();
		}
	}
} //DataMappingRuleImpl
