/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.DataStructureRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataTypeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DataType;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Data Structure Rule</b></em>'.
 * Source: BOM Class
 * Target: FDL DataStructure
 * Tranforms from a BOM Class element to an FDL DataStructure
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DataStructureRuleImpl extends AbstractFdlDataStructRuleImpl implements DataStructureRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DataStructureRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.DATA_STRUCTURE_RULE;
	}

	/**
	 * Source: BOM Class
	 * Target: FDL DataStructure
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		com.ibm.btools.bom.model.artifacts.Classifier type = (com.ibm.btools.bom.model.artifacts.Classifier)getSource().get(0);	
		getTarget().add(createDataStructure(type));
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();				
	}

	/**
	 * Creates a FDL DataStructure from a BOM Class.  Since FDL DataStructure doesn't support
	 * Class inheritency, then this method will flatten the super classes 
	 * @param type
	 * @return
	 */	
	private DataStructure createDataStructure(com.ibm.btools.bom.model.artifacts.Classifier type)
	{
		DataStructure ds = com.ibm.btools.fdl.model.impl.ModelFactoryImpl.eINSTANCE.createDataStructure();
	
		String nameKey = FdlUtil.getFullName(type);
		String name = getNameRegistry().getDataStructureName(type);
		ds.setName(name);
		ds.setDescription(getNameRegistry().getDescription(FdlUtil.getFullName(type)));
		getContext().put(nameKey,ds);		
		// Flatten all super class' attributes
		flattenSuperClass(type,ds, type);
		// Transform owned attributes
		List attributes = FdlUtil.getOwnedAttribute(type);
		Iterator iterator = attributes.iterator();
		Property attribute;
		while (iterator.hasNext())
		{
			attribute = (Property)iterator.next();
		
			MemberDeclaration md = createMemberDecl(attribute);
			md.setMemberName(getNameRegistry().getDataStructureMemberName(attribute));
			ds.getMemberDeclarations().add(md);
		}
		// Detect if there is cyclic references of DataStructure
		ArrayList al = new ArrayList();
		al.add(ds);
		al=detectCycle(ds,al);
		if (al!=null) {
			String params[]=new String[2];
			DataTypeRule r1 = (DataTypeRule)TransformationUtil.getRuleForTarget(getRoot(),DataTypeRule.class,(EObject)al.get(0));
			DataTypeRule r2 = (DataTypeRule)TransformationUtil.getRuleForTarget(getRoot(),DataTypeRule.class,(EObject)al.get(1));
			if (r1 != null) {
				params[0] = ((NamedElement)r1.getSource().get(0)).getName();
			} else {
				params[0] = "";
			}
			if (r2 != null) {
				params[1] = ((NamedElement)r2.getSource().get(0)).getName();
			} else {
				params[1] = "";
			}
			LoggingUtil.logError(MessageKeys.CYCLIC_REFERENCE_IN_DATA_STRUCTURE,params,null);
//			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.CYCLIC_REFERENCE_IN_DATA_STRUCTURE));
//			throw runtimeEx;
		}
		// Ray: added support for descriptions and documentations		
		String desc = FdlUtil.transformDescription(type);
		if (desc!=null) {
			ds.setDescription(getNameRegistry().getDescription(desc));
		}
		return ds;
	}
	
	/**
	 * Detect if there is cyclic references in the DataStructure
	 * a-> c,b
	 * b-> e,c,d
	 * c-> e
	 * d-> a
	 * e->
	 * @param ds
	 * @return
	 */
	private ArrayList detectCycle(DataStructure ds, ArrayList al)
	{ 
		ArrayList cyclicNodes=null;
		List mds = ds.getMemberDeclarations();
		Iterator it = mds.iterator();
		MemberDeclaration md;
		DataType dt;
		while (it.hasNext())
		{
			md=(MemberDeclaration)it.next();
			dt=md.getType();
			if (dt instanceof DataStructure) {
				if (al.contains((DataStructure)dt)) {
					cyclicNodes = new ArrayList(2);
					cyclicNodes.add(ds);
					cyclicNodes.add(dt);
					return cyclicNodes;
				} else {
					ArrayList nextAl = new ArrayList(); 
					nextAl.addAll(al);
					nextAl.add((DataStructure)dt);
					cyclicNodes = detectCycle((DataStructure)dt,nextAl);
					if (cyclicNodes!=null)
						return cyclicNodes;
				}
			}
		}
		return cyclicNodes;
	}
	
	/**
	 * Flatten the superclasses to the subclass.  I.e. move all the attributes from the
	 * superclass hierarchy to the subclass
	 * @param type
	 * @param ds
	 * @param flattenToType
	 */
	private void flattenSuperClass (com.ibm.btools.bom.model.artifacts.Classifier type, DataStructure ds, com.ibm.btools.bom.model.artifacts.Classifier flattenToType)
	{
		// Need to flatten superclass because FDL doesn't support type inheritancy
		if (type.getSuperClassifier() != null && !type.getSuperClassifier().isEmpty()){
			List superClassList = type.getSuperClassifier();
			Iterator it = superClassList.iterator();
			com.ibm.btools.bom.model.artifacts.Classifier superClass;
			MemberDeclaration md;
			List attributes;
			while (it.hasNext()){
				superClass = (com.ibm.btools.bom.model.artifacts.Classifier)it.next();
				// TODO: Ray testing - try to avoid exporting predefined Process Data superclass
//				if (FdlUtil.isPredefinedProcessDataStructure(superClass)) {
//					continue;
//				}
				attributes = FdlUtil.getOwnedAttribute(superClass);
				if (attributes != null && ! attributes.isEmpty()){
					Iterator iterator = attributes.iterator();
					Property attribute;
					while (iterator.hasNext()) {
						attribute = (Property)iterator.next();
						md = createMemberDecl(attribute);
						String memberName = getNameRegistry().getDataStructureMemberNameForType(attribute, flattenToType);
						md.setMemberName(memberName); 
						//md.setMemberName(fixSuperClassMemberName(superClass,md.getMemberName()));
						ds.getMemberDeclarations().add(md);
					}
				}			
				// recursively flatten the superClass' superclass	
				flattenSuperClass(superClass,ds, flattenToType);			
			}
		}

	}

} //DataStructureRuleImpl
