/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.ArtifactsFactory;
import com.ibm.btools.bom.model.artifacts.DataType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DataTypeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DatastoreRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Datastore Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DatastoreRuleImpl extends AbstractFdlProcDefRuleImpl implements DatastoreRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DatastoreRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.DATASTORE_RULE;
	}

	/**
	 * Source: Repository
	 * Target: FDL Programs
	 * There will be one retrieve and one store Program 
	 * for datastores that belong to the same namespace (BOM package).
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (getSource() == null || getSource().isEmpty())
			 return false;
			 
		Repository datastore = (Repository)getSource().get(0);
		
		if(datastore instanceof Variable){
			//log ERROR
			LoggingUtil.logWarning(MessageKeys.VARIABLE_REPOSITORY_NOT_SUPPORTED);
		}
		
		//first test if there is already a rule for this datastore created.
		TransformationRule existingRule =  TransformationUtil.getRuleForSource(getRoot(), datastore);
		if(existingRule != null && existingRule != this){
			getTarget().addAll(existingRule.getTarget());					
			setComplete(true);
			LoggingUtil.traceExit(this, "transformSource2Target","rule exist");
			return isComplete();
		}		
	
		//create predefined ProgramActivity for retrieve datastore program
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();

		pa.setName(getNameRegistry().getActivityName(datastore.getName(), FdlConstants.DATASTORE_RETRIEVE_PROGRAM_SUFFIX)); //"_R"
		pa.setProgramName(getNameRegistry().getProgramName(datastore.getName(), FdlConstants.DATASTORE_RETRIEVE_PROGRAM_SUFFIX)); //"_R"		

		DataStructure inDs = createInputDataStructureForRetrieve(datastore);		
		pa.setInputDataStructure(inDs);	
			
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		Program prm = (Program)pr.getTarget().get(0);
		prm.setStructuresFromActivity(new Boolean(true));
		getTarget().add(prm);	
		
			
	//	create fake ProgramActivity for store datastore program
		pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(getNameRegistry().getActivityName(datastore.getName(), FdlConstants.DATASTORE_STORE_PROGRAM_SUFFIX)); //"_W"
		pa.setProgramName(getNameRegistry().getProgramName(datastore.getName(), FdlConstants.DATASTORE_STORE_PROGRAM_SUFFIX)); //"_W"
	
	   //create Empty datastructure for store output  
	   
	    DataStructure outDs = createEmptyContainer();  
		
	    pa.setOutputDataStructure(outDs);
			
	    pr = FdlFactory.eINSTANCE.createProgramRule();
	    pr.getSource().add(pa);
	    getRoot().getChildRules().add(pr);
	  
	    pr.transformSource2Target();
	    prm = (Program)pr.getTarget().get(0);
	    prm.setStructuresFromActivity(new Boolean(true));
	    getTarget().add(prm);
		
	    setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");	    
	    return isComplete();
	}
	
	
	
private DataStructure createInputDataStructureForRetrieve(Repository datastore){
	
		/* Ray: why do we need them?
		ValueSpecification lowerBound = (ValueSpecification)EcoreUtil.copy((EObject)(datastore.getLowerBound()));
		ValueSpecification upperBound = (ValueSpecification)EcoreUtil.copy((EObject)(datastore.getUpperBound()));
		*/
		//TODO: re-evaluate how to create a primitive datatype
		// Ray: should create PrimitiveType instance instead
		//DataType booleanType = ArtifactsFactory.eINSTANCE.createDataType();
		DataType booleanType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
		booleanType.setName("Boolean");
		//DataType intType = ArtifactsFactory.eINSTANCE.createDataType();
		DataType intType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
		//intType.setName("Integer");
		intType.setName("String"); // length is a string to handle the 'n' length case
		//	create input datastructure for retrieve program		
		//create a new BOM datatype to represent the input		
		 
		  Type type = datastore.getType();
		  com.ibm.btools.bom.model.artifacts.Class newType = 
			  ArtifactsFactory.eINSTANCE.createClass();
		  newType.setName(datastore.getName() + FdlConstants.DATASTORE_RETRIEVE_IN_SUFFIX);
		
		//create the datastore type attribute with multiplicity value
		  com.ibm.btools.bom.model.artifacts.Property prop =  
		  	ArtifactsFactory.eINSTANCE.createProperty();
		  prop.setName(FdlConstants.DATASTORE_IS_REMOVE_NAME);		
		  prop.setType(booleanType);
		  newType.getOwnedAttribute().add(prop);
		
		  prop =  ArtifactsFactory.eINSTANCE.createProperty();
		  prop.setName(FdlConstants.DATASTORE_AT_BEGINNING_NAME);
		  prop.setType(booleanType);		
		  newType.getOwnedAttribute().add(prop);
		
		  prop =  ArtifactsFactory.eINSTANCE.createProperty();
		  prop.setName(FdlConstants.DATASTORE_LENGTH_NAME);
		  prop.setType(intType);		
		  newType.getOwnedAttribute().add(prop);
		  
			//	input datastructure
		  DataTypeRule dsr = FdlFactory.eINSTANCE.createDataTypeRule();
		  dsr.getSource().add(newType);
		  getRoot().getChildRules().add(dsr);
		  dsr.transformSource2Target();
		  DataStructure inDs = (DataStructure)dsr.getTarget().get(0);
		  
		  return inDs;
}	


private DataStructure createEmptyContainer(){
	
	//fake a outputPinSet
	  //create a fake inputPinSet so that we can use the container rule to create datastructure
	OutputPinSet fakePinSet = ActivitiesFactory.eINSTANCE.createOutputPinSet();
	fakePinSet.setName("empty");		
	//use containerRule create the datastructure
  
	ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
	cr.getSource().add(fakePinSet);
	getRoot().getChildRules().add(cr);
	cr.transformSource2Target();
	DataStructure emptyContainer = (DataStructure)cr.getTarget().get(0);
	return emptyContainer;	
}

} //DatastoreRuleImpl
