/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.DecisionOutputSet;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ExclusiveBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FlowRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.BooleanExpression;
import com.ibm.btools.fdl.model.ConnectorExpression;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Expression;
import com.ibm.btools.fdl.model.FDLOperator;
import com.ibm.btools.fdl.model.Member;
import com.ibm.btools.fdl.model.MemberNameExpression;
import com.ibm.btools.fdl.model.NumericExpression;
import com.ibm.btools.fdl.model.NumericLiteralExpression;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.StringExpression;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Decision Rule</b></em>'.
 * Source: BOM InputPinSet
 * Target: FDL noop Program Activity representing the Decision Node
 * Transform a BOM Decision to one or more FDL noop Program Activity
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DecisionRuleImpl extends ControlActionRuleImpl implements DecisionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private ProgramActivity nopa;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DecisionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.DECISION_RULE;
	}

	/**
	 * Source: BOM InputPinSet
	 * Target: FDL noop Program Activity representing the Decision Node
	 */
	public boolean transformSource2Target()
	{
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		Decision decision = (Decision)inputPinSet.getAction();
		if (isFirstTimeExec()) {
			setFirstTimeExec(false);
			
			nopa = createNoopProgramActivity();
			nopa.setName(getNameRegistry().getActivityName(decision));
			getTarget().add(nopa);
			
			createExclusiveBranchRule(inputPinSet);
			
			DataStructure inDataStruct=null;
			ContainerRule isCR = createContainerRule(inputPinSet);
			List targets = isCR.getTarget();
			if (targets.size() > 0) {
				inDataStruct = (DataStructure)(targets.get(0));
			}
			nopa.setInputDataStructure(inDataStruct);
			nopa.setOutputDataStructure(inDataStruct);
			//map input value pin if any to nopa input container initialization
			List initialList = mapInputValuePin(inputPinSet, inDataStruct);
			if((initialList != null) && (!initialList.isEmpty())){
				nopa.getInputContainerInitials().addAll(initialList);
			}	
			
			HashMap tcMap = handleOutputSets(nopa, inputPinSet, isCR);
			updateTransitionCondition(inputPinSet.getAction().getOutputPinSet(),tcMap,decision.getMultiplePaths().booleanValue());
			DataFlow df = createDataDefaultConnector(nopa);
			getTarget().add(df);

			// Ray: added support for descriptions and documentations		
			Action action = inputPinSet.getAction();
			String desc = FdlUtil.transformDescription(action);
			if (desc!=null) {
				nopa.setDescription(getNameRegistry().getDescription(desc));
			}
			String doc = FdlUtil.transformDocumentation(action);
			if (doc!=null) {
				nopa.setDocumentation(getNameRegistry().getDocumentation(doc));
			}			
			//propagate all targets from children to parent
			propagateChildTargets();			
		}
		setTargetOfConnectors(inputPinSet, nopa);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;		
	}
	
	/**
	 * Creates a HashMap to collect the pair of OutputSet 
	 * (as the key) and the FDL control flows (as the value) 
	 * corresponds to the edges coming out from outputset in order to 
	 * create transitionCondition later on
	 * @param nopa
	 * @param is
	 * @param isCR
	 * @return
	 */
	private HashMap handleOutputSets(ProgramActivity nopa,InputPinSet is,ContainerRule isCR)
	{
		// Creates a HashMap tcMap to collect the pair of OutputSet 
		// os (as the key) and the FDL control flows (as the value) 
		// corresponds to the edges coming out from os in order to 
		// create transitionCondition later on
		HashMap tcMap = new HashMap();
		List outputPinSets = is.getAction().getOutputPinSet();
		OutputPinSet os;
		Iterator it = outputPinSets.iterator();
		// ArrayList ctrlFlows collects the FDL control flows coming 
		// out from nopa corresponds to the edges coming out from os.  
		// This is needed for setting transitionCondition later on.		
		ArrayList ctrlFlows;
		Activity compatContainerAct;
		ContainerRule osCR;
		while (it.hasNext()) {
			os = (OutputPinSet)it.next();
			osCR = createContainerRule(os);
			ctrlFlows = new ArrayList();
			if (!isPinSetIdentical(is,os)) {
				compatContainerAct = createCompatibleContainerActivity(nopa,null,isCR,osCR,false);
				getTarget().add(compatContainerAct);
				ctrlFlows.addAll(compatContainerAct.getIncomingControlFlows());
			} else {
				setSourceOfConnectors(os,nopa);
				Collection ctrlFlowsCollected = collectControlFlowsForOutputset(os);
				if (ctrlFlowsCollected!=null) {
					ctrlFlows.addAll(ctrlFlowsCollected);
				}
				//ctrlFlows.addAll(nopa.getOutgoingControlFlows());
			}
			tcMap.put(os,ctrlFlows);
		}
		return tcMap;
	}
	
	/**
	 * Collect all the control flows that is coming out from the same output pin set
	 * @param os
	 * @return
	 */
	private Collection collectControlFlowsForOutputset(OutputPinSet os)
	{
		ArrayList ctrlFlows=null;
		Collection rules =
			TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(getContext()),
								FlowRule.class,os,ControlFlow.class);
		if (rules != null && !rules.isEmpty())
		{
			ctrlFlows = new ArrayList();
			Iterator it = rules.iterator();
			FlowRule tr;
			while (it.hasNext())
			{
				tr = (FlowRule)it.next();
				ctrlFlows.add(tr.getTarget().get(0));
			}
		}
		return ctrlFlows;
	}
	
	/**
	 * Creates a transitionCondition expression for the control flow
	 * @param outputPinSets
	 * @param tcMap
	 * @param multiplePath
	 */
	private void updateTransitionCondition (List outputPinSets,HashMap tcMap,boolean multiplePath)
	{
		Iterator it = outputPinSets.iterator();		
		ArrayList ctrlFlows;
		Iterator cfit;
		ControlFlow cf;
		Expression condition;
		DecisionOutputSet os;
		//String transformedCondition=null,bodyCondition=null,accumulatedCondition=null;
		Expression transformedExpr, bodyExpr, accumulatedExpr;
		bodyExpr = null;
		accumulatedExpr = null;
		transformedExpr = null;
		
		while (it.hasNext())
		{		
			os = (DecisionOutputSet)it.next();
			/* Call transformBOMExpression */
			transformedExpr = transformBOMExpression(os.getCondition());	
			if (transformedExpr != null)
			{			
				if (multiplePath) {
					bodyExpr = transformedExpr;
				}
				else {
					if (accumulatedExpr != null){
						bodyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createConnectorExpression();
						((ConnectorExpression)bodyExpr).setOperator(FDLOperator.AND_NOT_LITERAL);
						((ConnectorExpression)bodyExpr).setFirstOperand(transformedExpr);
						Expression cloneExpr = cloneExpression(accumulatedExpr);					
						((ConnectorExpression)bodyExpr).setSecondOperand(cloneExpr);
						accumulatedExpr = bodyExpr;					
					}
					else {
						bodyExpr = transformedExpr;
						accumulatedExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createExpression();
						accumulatedExpr=bodyExpr;
					}
				}
				
				// Create transition condition
				/*
				transformedCondition = transformBOMExpression(os.getCondition());
				if (multiplePath) {
					bodyCondition = transformedCondition;
				} else {
					if (accumulatedCondition != null) {
						bodyCondition = "("+transformedCondition+")"+accumulatedCondition;
						accumulatedCondition = " AND NOT ("+transformedCondition+")"+accumulatedCondition;
					} else {
						bodyCondition = transformedCondition;
						accumulatedCondition = " AND NOT ("+transformedCondition+")";
					}
				}
				*/			
				ctrlFlows = (ArrayList)tcMap.get(os);
				cfit = ctrlFlows.iterator();
				
				String desc=os.getCondition().getDescription(), name=os.getCondition().getName();
				while (cfit.hasNext()) {
					//condition=com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createExpression();
					//condition.setBody(bodyCondition);	
					condition = (Expression) EcoreUtil.copy(bodyExpr);								
					cf = (ControlFlow)cfit.next();
					cf.setWhen(condition);
					cf.setName(getNameRegistry().getActivityName(name,""));
					cf.setDescription(getNameRegistry().getDescription(desc));
				}
			} else {
				// Ray: try to handle null expression
				// Log a warning
				String params[] = new String[1];
				params[0] = BomUtils.getCanonicalName(os,false,true);
				LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);		 	
				if (os.getCondition()!=null) {
					ctrlFlows = (ArrayList)tcMap.get(os);
					cfit = ctrlFlows.iterator();
					
					String desc=os.getCondition().getDescription(), name=os.getCondition().getName();
					while (cfit.hasNext()) {
						cf = (ControlFlow)cfit.next();
						if (desc != null) {
							cf.setDescription(getNameRegistry().getDescription(desc));							
						}
						if (name != null) {
							cf.setName(getNameRegistry().getActivityName(name,""));
						}
					}					
				}
			}
		}
	}
	
	/**
	 * Converts BOM expression to BPEL expression for the case condition 
	 * @param expr - OpaqueExpression
	 * @return Expression - the expression suitable for BPEL logic condition
	 */
	private Expression transformBOMExpression(OpaqueExpression expr){
		 //create an ExpressionRule rule to transform the expression in a bom context
		 // an expression in a BPEL context		
		 // Ray: try to handle expression = null case
		 if (expr instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)expr).getExpression()!=null){
			 StructuredOpaqueExpression opaqueExpression = (StructuredOpaqueExpression)expr;
			 ExpressionRule exprRule = FdlFactory.eINSTANCE.createExpressionRule();
			 exprRule.getSource().add(opaqueExpression.getExpression());
			 getChildRules().add(exprRule);
			 exprRule.transformSource2Target(); 
			 if (!exprRule.getTarget().isEmpty()){
			 	Expression targetExpr = (Expression)exprRule.getTarget().get(0);
			 	targetExpr.setName(expr.getName());
			 	targetExpr.setDescription(getNameRegistry().getDescription(expr.getDescription()));
				return targetExpr;	
			 }		 
		 } 
		 return null;
	}
	
	/**
	 * Clone an FDL expression
	 * @param expr
	 * @return
	 */
	private Expression cloneExpression(Expression expr)
	{
		if (expr instanceof NumericLiteralExpression){
			NumericLiteralExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createNumericLiteralExpression();
			copyExpr.setNumericSymbol(((NumericLiteralExpression)expr).getNumericSymbol());
			return copyExpr;			
		}
		/*
		else if (expr instanceof FloatingPointLiteralExpression){
			FloatingPointLiteralExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createFloatingPointLiteralExpression();
			copyExpr.setRealSymbol(((FloatingPointLiteralExpression)expr).getRealSymbol());
			return copyExpr;	
		}
		*/
		else if (expr instanceof StringExpression){
			StringExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createStringExpression();
			copyExpr.setStringSymbol(((StringExpression)expr).getStringSymbol());
			return copyExpr;
		}
		/*
		else if (expr instanceof IntegerExpression){
			IntegerExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createIntegerExpression();
			copyExpr.setFirstOperand(cloneExpression(((IntegerExpression)expr).getFirstOperand()));
			copyExpr.setOperator(((IntegerExpression)expr).getOperator());
			copyExpr.setSecondOperand(cloneExpression(((IntegerExpression)expr).getSecondOperand()));
			return copyExpr;
		}
		*/
		else if (expr instanceof NumericExpression){
			NumericExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createNumericExpression();
			copyExpr.setFirstOperand(cloneExpression(((NumericExpression)expr).getFirstOperand()));
			copyExpr.setOperator(((NumericExpression)expr).getOperator());
			copyExpr.setSecondOperand(cloneExpression(((NumericExpression)expr).getSecondOperand()));
			return copyExpr;
		}
		else if (expr instanceof BooleanExpression){
			BooleanExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createBooleanExpression();
			copyExpr.setFirstOperand(cloneExpression(((BooleanExpression)expr).getFirstOperand()));
			copyExpr.setOperator(((BooleanExpression)expr).getOperator());
			copyExpr.setSecondOperand(cloneExpression(((BooleanExpression)expr).getSecondOperand()));
			return copyExpr;
		}
		else if (expr instanceof ConnectorExpression){
			ConnectorExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createConnectorExpression();
			copyExpr.setFirstOperand(cloneExpression(((ConnectorExpression)expr).getFirstOperand()));
			copyExpr.setOperator(((ConnectorExpression)expr).getOperator());
			copyExpr.setSecondOperand(cloneExpression(((ConnectorExpression)expr).getSecondOperand()));
			return copyExpr;
		}
		else if (expr instanceof MemberNameExpression){
			MemberNameExpression copyExpr = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createMemberNameExpression();
			List members = new LinkedList();
			members = ((MemberNameExpression)expr).getMember();
			for (int i=0; i<members.size(); i++){
				Member member = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createMember();
				member.setName(((Member)members.get(i)).getName());
				copyExpr.getMember().add(member);
			}
				
			return copyExpr;
		}
		else
			return null;
		
	}
	
	/**
	 * Creates an ExclusiveBranchRule for the DecisionNode
	 * @param inputPinSet
	 * @return
	 */
	private ExclusiveBranchRule createExclusiveBranchRule(InputPinSet inputPinSet)
	{
		ExclusiveBranchRule rule = null;
		if (BomHelper.getInstance().isEBranch(inputPinSet)) {
			rule = FdlFactory.eINSTANCE.createExclusiveBranchRule();
			rule.getSource().add(inputPinSet);
			getChildRules().add(rule);
			rule.transformSource2Target();
		}
		return rule;
	}
	
	/**
	 * Get the noop activity representing the decision node
	 * @return
	 */
	public ProgramActivity getNoopActivity() {
		return nopa;
	}
} //DecisionRuleImpl
