/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;

import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ExclusiveBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;


import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Exclusive Branch Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ExclusiveBranchRuleImpl extends ProcessWalkingRuleImpl implements ExclusiveBranchRule {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExclusiveBranchRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.EXCLUSIVE_BRANCH_RULE;
	}

	/**
	 * Source: BOM InputPinSet
	 * Target: None 
	 * 
	 * Maps all elements in the exclusive branch region marked by the Input Pin Set and exclusive merge point.
	 * The action of exclusive merge point is also mapped by this rule.
	 * This rule creates child rules to map the exclusive branch region, hence it doesn't have any rule target itself. 
	 *	  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

	 /*
	 * 1. for each output set correlated with the input set, check if it is concurrent branch
	 * 		if it is conrrent branch, create ConcurrentBranchRule and map it.
	 * 		if not, create connectors representing the outgoing edge from the pins in the output set.
	 * 2. for each input set reachable by each output set, 
	 *    step through it, create child rule for each element encountered and execute child rules,
	 *    until reached the merge point including the merge point as well or util reached the end of Structured Activity Node.
	 * 3. propagate child rules' targets.
	 */
	
	
	   //assertion
	  if (getSource() == null || getSource().isEmpty())
			return false;
  
	  //source of this rule is one input set
	  InputPinSet inputSet = (InputPinSet)getSource().get(0);
	
	  //check if this is first time execute this rule
	   if(!isFirstTimeExec()){
	   	  //there is nothing to re-execute
		  //reExecute(null); 
		LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
		  return true;
	   }
	
		setFirstTimeExec(false);	
	
		//for each ouput set, check if need to create Concurrent branch rule
		List osList = inputSet.getOutputPinSet();	
		OutputPinSet nextOS;
		boolean isCBranch = false;				
		for(Iterator it = osList.iterator(); it.hasNext();){
			nextOS = (OutputPinSet)it.next();
			//create concurrent branch point for nextOS if any
			isCBranch = createCBranchRuleIfAny(nextOS);
			if(!isCBranch){
				//create links for each pins in the output set
				createConnectorRulesForOutputPinSet(nextOS);
			}		
				
		}
			
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		NamedElement mergePoint = BomHelper.getInstance().findExclusiveMergePoint(inputSet);
		Action mergePointAction = null;
		if(mergePoint instanceof OutputPinSet)
			mergePointAction = ((OutputPinSet)mergePoint).getAction();
		else if(mergePoint instanceof StructuredActivityNode)
			mergePointAction = (Action)mergePoint;
			
		List nextISList;
		InputPinSet nextIS;
		//!!main logic of this method is here
		  //walk through each of the output set, create and execute child rules
		for (Iterator it=osList.iterator(); it.hasNext(); ){
			  nextOS = (OutputPinSet)it.next();
			  nextISList = bw.getNext(nextOS, BomWalker.INPUT_SET, true);
			  if(nextISList == null || nextISList.isEmpty()){
				//we have reached end of SAN ??? what to do??
				//need do the connection for the output set of SAN to internal activities of SAN?
				// The answer is no. ProcessRule takes care of doing this. 
				//doOutputPinSetConnection(flow, true);				
			  }
			  else{
						  
				for (Iterator it2 = nextISList.iterator(); it2.hasNext(); ){ //TODO: watch for if this loop is needed. It is believed there will be only one inputSet returned 
					nextIS = (InputPinSet)it2.next();
				  do{
					  if(nextIS.getAction() == mergePointAction){
						//we get to merge pint, but we need to include merge point inside the exclusive branch flow, so execute the do..while loop one more time				  	  	
						stepThroughBOM(nextIS); 				
						nextIS = null;
					  }
					  else{ //nextIS not on merge point				  	  
						nextIS = stepThroughBOM(nextIS);
						if(nextIS != null){ //means we just created a child rule
						  //we must continue
						  //bw.getNext(...) return list fo inputSet. Is it safe just to use the first element?
						  //the answer is yes, since we always skip branch point, we should only have one InputPinSet back
						  List nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
						  if(nextInputPinSets == null || nextInputPinSets.isEmpty() ){						  
							  nextIS = null; //reached end of SAN
							  //need do the connection for the output set of SAN to internal activities of SAN?
							  //the answer is no. 
							  //doOutputPinSetConnection(flow, true);
						  }else{						  
							  nextIS= (InputPinSet)nextInputPinSets.get(0);	
							  //must pass the output set to BomHelper otherwise it doesn't work
							  if(!BomHelper.getInstance().isInBranchRegion(inputSet, (OutputPinSet)nextIS.getOutputPinSet().get(0))) {
								// Ray: Additional checking - if nextIS is an EBP but is contained within the current EB region, we
								// shouldn't skip it
								if (BomHelper.getInstance().isEBranch(nextIS)) {
									NamedElement emp = BomHelper.getInstance().findExclusiveMergePoint(nextIS);
									if (!BomHelper.getInstance().isInBranchRegion(inputSet,emp)) {
										nextIS=null;
									}
								} else {
									// we stepped over merge point, so should stop it here
									nextIS = null;
								}
							  }
						  }
					  }else{ // stepTrhoughBOM didn't create child rule, so we should stop tracing this path
						//do nothing
					  }
				 
					 }
				  }while(nextIS!= null);	
			  } 	
			}		 
		  }
	
		propagateChildTargets();
	
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	}
	
	
/**
 * Creates and executes the ConcurrentBranchRule if the outputSet is a concurrent branch.
 * @param outputSet  the output set of the exclusive branch point.
 * @return boolean - true if created concurrentBranchRule
 */
private boolean createCBranchRuleIfAny(OutputPinSet outputSet){		

	TransformationRule rule;	
	 //check if output is Concurrent branch
	if(BomHelper.getInstance().isCBranch(outputSet)){
		rule = FdlFactory.eINSTANCE.createConcurrentBranchRule();
		rule.getSource().add(outputSet);
		getChildRules().add(rule);
		rule.transformSource2Target();			
		return true;	 
	}else{
		//do nothing
		return false;
	}
}	

} //ExclusiveBranchRuleImpl
