/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.expression.model.BinaryLogicalBooleanExpression;
import com.ibm.btools.expression.model.BinaryMathematicalOperator;
import com.ibm.btools.expression.model.BinaryNumericExpression;
import com.ibm.btools.expression.model.BinaryOperatorExpression;
import com.ibm.btools.expression.model.BooleanLiteralExpression;
import com.ibm.btools.expression.model.ComparisonExpression;
import com.ibm.btools.expression.model.ComparisonOperator;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.IntegerLiteralExpression;
import com.ibm.btools.expression.model.LiteralExpression;
import com.ibm.btools.expression.model.LogicalBooleanOperator;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.NegationExpression;
import com.ibm.btools.expression.model.NotExpression;
import com.ibm.btools.expression.model.NumericLiteralExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StaticStep;
import com.ibm.btools.expression.model.Step;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.expression.model.UnaryOperatorExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.MemberDeclRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.BooleanExpression;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.DataStructureMember;
import com.ibm.btools.fdl.model.FDLOperator;
import com.ibm.btools.fdl.model.ModelFactory;
import com.ibm.btools.fdl.model.MemberDeclaration;
import com.ibm.btools.fdl.model.MemberItem;
import com.ibm.btools.fdl.model.MemberNameExpression;
import com.ibm.btools.fdl.model.NumericExpression;
import com.ibm.btools.fdl.model.StringExpression;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Expression Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ExpressionRuleImpl extends AbstractFdlProcDefRuleImpl implements ExpressionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExpressionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.EXPRESSION_RULE;
	}

	/**
	 * source = An Expression Model in the BOM context
	 * target = A FDL Expression Model in the FDL context
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */	
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		com.ibm.btools.fdl.model.Expression exprModelTarget = null;
		//assertion
		if (getSource() == null || getSource().isEmpty())
			return false;
			
		Expression exprSource = (Expression)EcoreUtil.copy((Expression)getSource().get(0));
		exprModelTarget = mapToFDLExpressionModel(exprSource);	
	
		//save target
		if (exprModelTarget!=null)
			getTarget().add(exprModelTarget);	
		else
			
			LoggingUtil.logWarning(MessageKeys.INVALID_FDL_EXPRESSION);								
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Maps from a BOM Expression to a FDL Expression.	 
	 * @param exprModelSrce - a Expression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */	
	private com.ibm.btools.fdl.model.Expression mapToFDLExpressionModel(Expression exprModelSrce)
	{
		//top level expressions for DecisionoutputConditions
		if (exprModelSrce instanceof UnaryOperatorExpression)
			return mapUnaryOperatorExpression((UnaryOperatorExpression)exprModelSrce);		
		else if (exprModelSrce instanceof BinaryOperatorExpression)
			return mapBinaryOperatorExpression((BinaryOperatorExpression)exprModelSrce);
		else if (exprModelSrce instanceof LiteralExpression)
			return mapLiteralExpression((LiteralExpression)exprModelSrce);
		else if (exprModelSrce instanceof ModelPathExpression)			
			return mapModelPathExpression((ModelPathExpression)exprModelSrce);
		else if (exprModelSrce instanceof FunctionExpression)
			//throw new BTRuntimeException("Function Expressions are not supported in FDL Mapping");
			return null;			
		
		return null;
	}

	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a LiteralExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */		
	private com.ibm.btools.fdl.model.Expression mapLiteralExpression(LiteralExpression exprSource)
	{
		if (exprSource instanceof NumericLiteralExpression){
			com.ibm.btools.fdl.model.NumericLiteralExpression intLitExpr = ModelFactory.eINSTANCE.createNumericLiteralExpression();
			intLitExpr.setNumericSymbol( ((NumericLiteralExpression)exprSource).getNumericSymbol());
			return intLitExpr;
		}
		/*
		else if (exprSource instanceof RealLiteralExpression){
			FloatingPointLiteralExpression floatExpr = ModelFactory.eINSTANCE.createFloatingPointLiteralExpression();
			floatExpr.setRealSymbol( ((RealLiteralExpression)exprSource).getRealSymbol());
			return floatExpr;
		}
		*/
		else if(exprSource instanceof StringLiteralExpression){
			StringExpression strExpr = ModelFactory.eINSTANCE.createStringExpression();
			strExpr.setStringSymbol(((StringLiteralExpression)exprSource).getStringSymbol());
			return strExpr;
		}
		else if (exprSource instanceof BooleanLiteralExpression){
			com.ibm.btools.fdl.model.NumericLiteralExpression intLitExpr = ModelFactory.eINSTANCE.createNumericLiteralExpression();
			if ( ((BooleanLiteralExpression)exprSource).getBooleanSymbol().booleanValue() )
				intLitExpr.setNumericSymbol("1");
			else //false
				intLitExpr.setNumericSymbol("0");
			return intLitExpr;		
		}else{
			//TODO  lOGGING - do not support time/date
			return null;
		}
	
	}

	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a BinaryOperatorExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */			
	private com.ibm.btools.fdl.model.Expression mapBinaryOperatorExpression(BinaryOperatorExpression exprSource)
	{
		if (exprSource instanceof BinaryLogicalBooleanExpression)
			return mapBinaryLogicalBooleanExpression((BinaryLogicalBooleanExpression)exprSource);		
		else if (exprSource instanceof BinaryNumericExpression)
			return mapBinaryNumericExpression((BinaryNumericExpression)exprSource);		
		else  //comparison Expression			
			return mapComparisonExpression((ComparisonExpression)exprSource);	
	}

	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a BinaryNumericExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */		
	private com.ibm.btools.fdl.model.Expression mapBinaryNumericExpression(BinaryNumericExpression exprSource)
	{
		//if ( (exprSource.getFirstOperand().getEvaluatesToType().equalsIgnoreCase("Integer")) && (exprSource.getSecondOperand().getEvaluatesToType().equalsIgnoreCase("Integer")) ){
		if ( (exprSource.getFirstOperand() instanceof IntegerLiteralExpression) && (exprSource.getSecondOperand() instanceof IntegerLiteralExpression) ){
			// Create an integer expression
			NumericExpression intExpr = ModelFactory.eINSTANCE.createNumericExpression();
			com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getFirstOperand());
			if (tempExpr!=null)
				intExpr.setFirstOperand(tempExpr);
			else
				return null;
			tempExpr = mapToFDLExpressionModel(exprSource.getSecondOperand());
			if (tempExpr!=null)
				intExpr.setSecondOperand(tempExpr);
			else
				return null;
			
			// Setting the operators
			if (exprSource.getOperator().equals(BinaryMathematicalOperator.ADDITION_LITERAL))
				intExpr.setOperator(FDLOperator.ADDITION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.SUBTRACTION_LITERAL))
				intExpr.setOperator(FDLOperator.SUBTRACTION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.MULTIPLICATION_LITERAL))
				intExpr.setOperator(FDLOperator.MULTIPLICATION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.DIVISION_LITERAL))
				intExpr.setOperator(FDLOperator.DIVISION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.MODULUS_LITERAL))
				intExpr.setOperator(FDLOperator.MODULUS_LITERAL);				
			return intExpr;
		}
		else{
			// Create a Numeric expression
			NumericExpression numericExpr = ModelFactory.eINSTANCE.createNumericExpression();
			com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getFirstOperand());
			if (tempExpr!=null)
				numericExpr.setFirstOperand(tempExpr);
			else
				return null;
			tempExpr = mapToFDLExpressionModel(exprSource.getSecondOperand());
			if (tempExpr!=null)
				numericExpr.setSecondOperand(tempExpr);
			else
				return null;
			//numericExpr.setFirstOperand(mapToFDLExpressionModel(exprSource.getFirstOperand()));
			//numericExpr.setSecondOperand(mapToFDLExpressionModel(exprSource.getSecondOperand()));			
			// Setting the operators
			if (exprSource.getOperator().equals(BinaryMathematicalOperator.ADDITION_LITERAL))
				numericExpr.setOperator(FDLOperator.ADDITION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.SUBTRACTION_LITERAL))
				numericExpr.setOperator(FDLOperator.SUBTRACTION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.MULTIPLICATION_LITERAL))
				numericExpr.setOperator(FDLOperator.MULTIPLICATION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.DIVISION_LITERAL))
				numericExpr.setOperator(FDLOperator.DIVISION_LITERAL);
			else if (exprSource.getOperator().equals(BinaryMathematicalOperator.MODULUS_LITERAL))
				//throw new BTRuntimeException("Modulus operation is not supported for Real Expressions");
				return null;
			return numericExpr;
		}		
	}
	
	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a BinaryLogicalBooleanExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */		
	private com.ibm.btools.fdl.model.Expression mapBinaryLogicalBooleanExpression(BinaryLogicalBooleanExpression exprSource)
	{
		//checks to see if one of the operands is a BooleanLiteralExpression.
		//maps to a Boolean expression
		BooleanExpression boolExpr = ModelFactory.eINSTANCE.createBooleanExpression();
		//BooleanExpression tempBoolExpr = ModelFactory.eINSTANCE.createBooleanExpression();
		
		//Check operator - throw an exception for operators that are not supported by FDL
		// Set Operators
		if (((BinaryLogicalBooleanExpression)exprSource).getOperator().equals(LogicalBooleanOperator.AND_LITERAL)) 
			boolExpr.setOperator(FDLOperator.AND_LITERAL);
		else // OR_LITERAL
			boolExpr.setOperator(FDLOperator.OR_LITERAL); 
		
		// First Operand is a BooleanLiteral or a MPE that evaluate to a boolean, need to convert to a BooleanExpr
		if ( (exprSource.getFirstOperand() instanceof BooleanLiteralExpression) ||  
				((exprSource.getFirstOperand() instanceof ModelPathExpression) &&
				(exprSource.getFirstOperand().getEvaluatesToType().equalsIgnoreCase("Boolean"))) ){					
					boolExpr.setFirstOperand(createBooleanExpression(exprSource.getFirstOperand()));
		}
		else{
			com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getFirstOperand());
			if (tempExpr!=null)
				boolExpr.setFirstOperand(tempExpr);
			else
				return null;
			//boolExpr.setFirstOperand( mapToFDLExpressionModel(exprSource.getFirstOperand()) );
		}

		// Second Operand is a BooleanLiteral or a MPE that evaluate to a boolean, need to convert to a BooleanExpr
		if ( (exprSource.getSecondOperand() instanceof BooleanLiteralExpression) ||  
				((exprSource.getSecondOperand() instanceof ModelPathExpression) &&
				(exprSource.getSecondOperand().getEvaluatesToType().equalsIgnoreCase("Boolean"))) ){
					boolExpr.setSecondOperand(createBooleanExpression(exprSource.getSecondOperand()));
		}
		else {
			com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getSecondOperand());
			if (tempExpr!=null)
				boolExpr.setSecondOperand(tempExpr);
			else
				return null;
			//boolExpr.setSecondOperand( mapToFDLExpressionModel(exprSource.getSecondOperand()) );
		}
		return boolExpr;		
	}
	
	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a ComparisonExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */	
	private com.ibm.btools.fdl.model.Expression mapComparisonExpression(ComparisonExpression exprSource)
	{
		BooleanExpression boolExpr = ModelFactory.eINSTANCE.createBooleanExpression();
		//boolExpr.setFirstOperand(mapToFDLExpressionModel(exprSource.getFirstOperand()));
		//boolExpr.setSecondOperand(mapToFDLExpressionModel(exprSource.getSecondOperand()));
		com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getFirstOperand());
		if (tempExpr!=null)
			boolExpr.setFirstOperand(tempExpr);
		else
			return null;
		tempExpr = mapToFDLExpressionModel(exprSource.getSecondOperand());
		if (tempExpr!=null)
			boolExpr.setSecondOperand(tempExpr);
		else
			return null;
		// Setting the operators
		if (exprSource.getOperator().equals(ComparisonOperator.EQUAL_TO_LITERAL))
			boolExpr.setOperator(FDLOperator.EQUAL_TO_LITERAL);
		else if (exprSource.getOperator().equals(ComparisonOperator.NOT_EQUAL_TO_LITERAL))
			boolExpr.setOperator(FDLOperator.NOT_EQUAL_TO_LITERAL);
		else if (exprSource.getOperator().equals(ComparisonOperator.LESS_THAN_LITERAL))
			boolExpr.setOperator(FDLOperator.LESS_THAN_LITERAL);
		else if (exprSource.getOperator().equals(ComparisonOperator.LESS_THAN_OR_EQUAL_TO_LITERAL))
			boolExpr.setOperator(FDLOperator.LESS_THAN_OR_EQUAL_TO_LITERAL);
		else if (exprSource.getOperator().equals(ComparisonOperator.GREATER_THAN_LITERAL))
			boolExpr.setOperator(FDLOperator.GREATER_THAN_LITERAL);
		else if (exprSource.getOperator().equals(ComparisonOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL))
			boolExpr.setOperator(FDLOperator.GREATER_THAN_OR_EQUAL_TO_LITERAL);
		return boolExpr;
	}

	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a UnaryOperatorExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */	
	private com.ibm.btools.fdl.model.Expression mapUnaryOperatorExpression(UnaryOperatorExpression exprSource)
	{
		if (exprSource instanceof NotExpression)
		{
			BooleanExpression boolExpr = ModelFactory.eINSTANCE.createBooleanExpression();
			boolExpr.setOperator(FDLOperator.NOT_LITERAL);
			// Maps to a FDL Boolean Expression with only 1 operand
			if ( (exprSource.getExpression() instanceof BooleanLiteralExpression) ||  
					 ((exprSource.getExpression() instanceof ModelPathExpression) &&
					 (exprSource.getExpression().getEvaluatesToType().equalsIgnoreCase("Boolean"))) ){					
						 boolExpr.setSecondOperand(createBooleanExpression(exprSource.getExpression()));
			}
			else {
				com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getExpression());
				if (tempExpr!=null)
					boolExpr.setSecondOperand(tempExpr);
				else
					return null;						
				//boolExpr.setSecondOperand( mapToFDLExpressionModel(exprSource.getExpression()) );
			}
			return boolExpr;
		}
		else // Negation Expression
		{
			// Maps to Integer Expression if evaluatestoType is an integer
			if (exprSource.getEvaluatesToType().equalsIgnoreCase("Integer")){
				NumericExpression intExpr = ModelFactory.eINSTANCE.createNumericExpression();
				com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(exprSource.getExpression());
				if (tempExpr!=null)
					intExpr.setSecondOperand(tempExpr);
				else
					return null;
				//intExpr.setSecondOperand(mapToFDLExpressionModel(exprSource.getExpression()));
				intExpr.setOperator(FDLOperator.NEGATION_LITERAL);
				return intExpr;
			}			
			// Maps to Numeric Expression if evaluatesToType is a real
			else {
				NumericExpression numericExpr = ModelFactory.eINSTANCE.createNumericExpression();
				com.ibm.btools.fdl.model.Expression tempExpr = mapToFDLExpressionModel(((NegationExpression)exprSource).getExpression());
				if (tempExpr!=null)
					numericExpr.setSecondOperand(tempExpr);
				else
					return null;
				//numericExpr.setSecondOperand(mapToFDLExpressionModel( ((NegationExpression)exprSource).getExpression() ));
				numericExpr.setOperator(FDLOperator.NEGATION_LITERAL);
				return numericExpr;
			}
			
		}
	}

	/**
	 * Maps to a Expression in the FDL context.	 
	 * @param exprSource - a ModelPathExpression in the BOM context
	 * @return Expression - A Expression in the FDL context
	 */		
	private com.ibm.btools.fdl.model.Expression mapModelPathExpression(ModelPathExpression exprSource)
	{
		MemberNameExpression memberExpr = ModelFactory.eINSTANCE.createMemberNameExpression();
		List steps = new LinkedList();
		steps = exprSource.getSteps();
		Step tempStep;
		// Check the type of exprSource
		
		for (int i=0; i< steps.size(); i++)
		{
			tempStep = (Step)steps.get(i);
			if (tempStep instanceof ReferenceStep){				
				if ( (((ReferenceStep)tempStep).getReferencedObject() instanceof InputPinSet) || 
					(((ReferenceStep)tempStep).getReferencedObject() instanceof OutputPinSet) ){
						// Must call the rule to get the names of the DataStructures				
						//DataStructure dataStruct = (DataStructure)TransformationUtil.getRuleForSource(ProcessUtil.getProcessDefinitionRule
						//							(getContext()),ContainerRule.class,((ReferenceStep)tempStep).getReferencedObject()).getTarget().get(0);
						//structMember.setName(dataStruct.getName());
						//DataStructureMember structMember = ModelFactory.eINSTANCE.createDataStructureMember();
						//structMember.setName(FdlUtil.getDataStructureName( (PinSet)((ReferenceStep)tempStep).getReferencedObject() ));					
						//memberExpr.getMember().add(structMember);	
				}
				else if ( ((ReferenceStep)tempStep).getReferencedObject() instanceof ObjectPin)
				{
					//MemberDeclaration item = (MemberDeclaration)TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule
					//							(getContext()),MemberDeclRule.class,((ReferenceStep)tempStep).getReferencedObject()).getTarget().get(0);
					//structMember.setName(item.getMemberName());
					DataStructureMember structMember = ModelFactory.eINSTANCE.createDataStructureMember();
					//String dsNameKey = FdlUtil.computeDataStructureKey((PinSet)((ReferenceStep)steps.get(i-1)).getReferencedObject());
					//ds = (DataStructure)getContext().get("FDLContainer"+dsNameKey);
					//structMember.setName(FdlUtil.getDataStructureMemberName(ds, (ObjectPin)((ReferenceStep)tempStep).getReferencedObject()));					
					DataStructure ds=null;
					int index=-1;
					//PinSet pinSet = (PinSet)((ReferenceStep)steps.get(i-1)).getReferencedObject();
					// Ray: Need to handle the case where the MPE contains no pinset
					PinSet pinSet = null;
					if (i>0 && ((ReferenceStep)steps.get(i-1)).getReferencedObject() instanceof PinSet) { 
						pinSet = (PinSet)((ReferenceStep)steps.get(i-1)).getReferencedObject();
					} else {
						pinSet = findPinSetForPin((Pin)((ReferenceStep)tempStep).getReferencedObject());
					}
					if (pinSet instanceof InputPinSet)
						index = (((InputPinSet)pinSet).getInputObjectPin()).indexOf(((ReferenceStep)tempStep).getReferencedObject());
					else if (pinSet instanceof OutputPinSet)
						index = (((OutputPinSet)pinSet).getOutputObjectPin()).indexOf(((ReferenceStep)tempStep).getReferencedObject());
					
					ds = (DataStructure)TransformationUtil.getRuleForSource(getRoot(),ContainerRule.class, pinSet).getTarget().get(0);
					
					if (!FdlUtil.isPinSetDataStructureOptimizable(pinSet)) {
						String name = ((MemberDeclaration)ds.getMemberDeclarations().get(index)).getMemberName();
						structMember.setName(name);
						memberExpr.getMember().add(structMember);					
					}
				}	
				else if (((ReferenceStep)tempStep).getReferencedObject() instanceof Property){
					
					//Ray
					MemberDeclaration md = (MemberDeclaration)((MemberDeclRule)TransformationUtil.getRuleForSource(getRoot(),MemberDeclRule.class,((ReferenceStep)tempStep).getReferencedObject())).getTarget().get(0);
					MemberItem item = ModelFactory.eINSTANCE.createMemberItem();
					//item.setName(((ReferenceStep)steps.get(i)).getReferenceName());
					item.setName(md.getMemberName());
					memberExpr.getMember().add(item);
				} else {
					if (((ReferenceStep)tempStep).getReferencedObject() instanceof Variable) {
						LoggingUtil.logWarning(MessageKeys.VARIABLE_REF_IN_EXPRESSION_NOT_SUPPORTED,null,null);
						break;
					}
				}
			}
			else { //Static Step
				MemberItem item = ModelFactory.eINSTANCE.createMemberItem();
				item.setName(((StaticStep)steps.get(i)).getName());
				memberExpr.getMember().add(item);
			}
		}
		return memberExpr;
	}

	/**
	 * Maps to a BooleanExpression in the FDL context.	 
	 * @param expr - a Expression in the BOM context
	 * @return BooleanExpression - A BooleanExpression in the FDL context
	 */		
	private BooleanExpression createBooleanExpression(Expression expr)
	{
		BooleanExpression boolExpr = ModelFactory.eINSTANCE.createBooleanExpression();
		boolExpr.setOperator(FDLOperator.EQUAL_TO_LITERAL);
		boolExpr.setFirstOperand(mapToFDLExpressionModel(expr));
		com.ibm.btools.fdl.model.NumericLiteralExpression intExpr = ModelFactory.eINSTANCE.createNumericLiteralExpression();
		intExpr.setNumericSymbol("1");
		boolExpr.setSecondOperand(intExpr);
		return boolExpr;
	}

	/**
	 * This method attempts to find the first pinSet (input or output) that contains a Pin.  Notice a pin can never appear
	 * in both input and output pinset.
	 * @param pin
	 * @return
	 */
	private PinSet findPinSetForPin(Pin pin) {
		Action action = (Action)pin.eContainer();
		Iterator it = action.getInputPinSet().iterator();
		InputPinSet inPinSet;
		while (it.hasNext()) {
			inPinSet = (InputPinSet)it.next();
			// it can never be control pin
			if (inPinSet.getInputObjectPin().contains(pin)) {
				return inPinSet;
			}
			
		}
		it = action.getOutputPinSet().iterator();
		OutputPinSet outPinSet;
		while (it.hasNext()) {
			outPinSet = (OutputPinSet)it.next();
			// it can never be control pin
			if (outPinSet.getOutputObjectPin().contains(pin)) {
				return outPinSet;
			}
		}
		return null;  // This should never happened!!
	}	
} //ExpressionRuleImpl
