/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ConcurrentBranchRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.JoinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Join Rule</b></em>'.
 * Source: BOM InputPinSet of the Join Action
 * Target: FDL Noop Program Activity representing the Join
 * Transform a Join action to a FDL Noop Program Activity representing the Join
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class JoinRuleImpl extends ControlActionRuleImpl implements JoinRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private ProgramActivity nopa;
	private ProgramActivity isNopa;	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected JoinRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.JOIN_RULE;
	}

	/**
	 * Source: BOM InputPinSet of the Join Action
	 * Target: FDL Noop Program Activity representing the Join
	 */
	public boolean transformSource2Target() {		
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty())
			return false;
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		if (isFirstTimeExec()) {
			setFirstTimeExec(false);
			
			Join join = (Join)inputPinSet.getAction();
			
			nopa = createNoopProgramActivity();
			nopa.setName(getNameRegistry().getActivityName(join));
			getTarget().add(nopa);
				
			OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getAction().getOutputPinSet().get(0);
			createConcurrentBranchRule(outputPinSet);
			setSourceOfConnectors(outputPinSet,nopa);

			DataStructure outDataStruct=null;
			ContainerRule osCR = createContainerRule(outputPinSet);
			List targets = osCR.getTarget();
			if (targets.size() > 0) {
				outDataStruct = (DataStructure)(targets.get(0));
			}
			nopa.setInputDataStructure(outDataStruct);
			nopa.setOutputDataStructure(outDataStruct);
			isNopa = handleInputSets(nopa,inputPinSet,outputPinSet,osCR);
			DataFlow df = createDataDefaultConnector(nopa);
			getTarget().add(df);

			// Ray: added support for descriptions and documentations		
			Action action = inputPinSet.getAction();
			String desc = FdlUtil.transformDescription(action);
			if (desc!=null) {
				nopa.setDescription(getNameRegistry().getDescription(desc));
			}
			String doc = FdlUtil.transformDocumentation(action);
			if (doc!=null) {
				nopa.setDocumentation(getNameRegistry().getDocumentation(doc));
			}						
			//propagate all targets from children to parent
			propagateChildTargets();					
		}
		setTargetOfConnectors(inputPinSet, isNopa);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return true;		
	}
	
	/**
	 * Check if the InputPinSet and the corresponding OutputPinSet (see createCorrespondingInputPinSets method)
	 * are identical and creates a FDL no-op activity if they are not.
	 * @param nopa
	 * @param is
	 * @param os
	 * @param osCR
	 * @return
	 */
	private ProgramActivity handleInputSets(ProgramActivity nopa, InputPinSet is, OutputPinSet os, ContainerRule osCR) {
		ArrayList corIPSets = createCorrespondingInputPinSets(is, os);
		Iterator it=corIPSets.iterator();
		InputPinSet corIs;
		ContainerRule isCR;
		ProgramActivity compatContainerAct = nopa;
		boolean allOutputPinSetIdentical=true;
		while (it.hasNext() && allOutputPinSetIdentical) {
			corIs = (InputPinSet)it.next();
			if (!isPinSetIdentical(corIs,os)) {
				allOutputPinSetIdentical=false;
				isCR = createContainerRule(is);
				compatContainerAct = (ProgramActivity)createCompatibleContainerActivity(null,nopa,osCR,isCR,true);
				getTarget().add(compatContainerAct);
			}
		}
		return compatContainerAct;
	}
	
	/**
	 * Try to find the matching pins in the inputpinset that matches to the pins 
	 * in an outputpinset of a Join
	 * @param is
	 * @param os
	 * @return
	 */
	private ArrayList createCorrespondingInputPinSets(InputPinSet is, OutputPinSet os) {
		ArrayList corIPSets = new ArrayList();
		int numObjPins = os.getOutputObjectPin().size();
		if (numObjPins > 0) {
			List isObjPins = is.getInputObjectPin();
			int totalIteration = isObjPins.size()/numObjPins;
			InputPinSet corIPSet;
			InputObjectPin isObjPin,corIsObjPin;
			for (int i=0; i < totalIteration; i++) {
				corIPSet = ActivitiesFactory.eINSTANCE.createInputPinSet();
				for (int j=0; j<numObjPins; j++) {
					isObjPin = (InputObjectPin)isObjPins.get(j+(i*numObjPins));
					//corOsObjPin = (OutputObjectPin)copyObjectPin(osObjPin);
					corIPSet.getInputObjectPin().add(isObjPin);
				}
				corIPSets.add(corIPSet);
			}
		}
		return corIPSets;
	}
	
	/**
	 * Creates a ConcurrentBranchRule is a concurrent branch exists
	 * @param outputPinSet
	 * @return
	 */
	private ConcurrentBranchRule createConcurrentBranchRule(OutputPinSet outputPinSet) {
		ConcurrentBranchRule rule=null;
		if (BomHelper.getInstance().isCBranch(outputPinSet)){
			rule = FdlFactory.eINSTANCE.createConcurrentBranchRule();
			rule.getSource().add(outputPinSet);
			getChildRules().add(rule);
			rule.transformSource2Target();	 
		}
		return rule;
	}
	
	public ProgramActivity getIsNoopActivity() {
		return isNopa;
	}

	public ProgramActivity getNoopActivity() {
		return nopa;
	}

} //JoinRuleImpl
