/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.fdl.model.Block;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Expression;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.Sink;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ExpressionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.LoopNodeRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.resource.MessageKeys;

/**
 * <!-- begin-user-doc -->
 * Source: BOM InputPinSet (on a LoopNode)
 * Target: FDL Block
 * 
 * Transforms Bom LoopNode to FDL Block with exist condition (while condition)
 * Note: LoopNode may have multiple input pin sets, if so, there will be multiple block created.
 * 		 this rule only can handle when LoopNode's attribute isTestedFirst = false, which is a do-while loop.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class LoopNodeRuleImpl extends BlockRuleImpl implements LoopNodeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected LoopNodeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.LOOP_NODE_RULE;
	}

	/**
	 * Source: BOM InputPinSet of LoopNode
	 * Target: FDL Block
	 * Transform Bom LoopNode to FDL Block with exist condition (while condition)
	 * Note: LoopNode may have multiple input pin sets, if so, there will be multiple block created.
	 * 		 this rule only can handle when LoopNode attribute isTestedFirst = false, which is a do-while loop.
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
		
		if(!isFirstTimeExec()){
			//will connect additional connectors with Block 
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target");
			return isComplete();
		}

		setFirstTimeExec(false);
		
		LoopNode loopNode = (LoopNode)inputPinSet.getAction();		

		// log error for loopNode isTestedFirst = true
	/*	if(loopNode.getIsTestedFirst().booleanValue()){
			LoggingUtil.logError(MessageKeys.LOOPNODE_IS_TESTED_FIRST_NOT_SUPPORTED);
			BTRuntimeException e = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.LOOPNODE_IS_TESTED_FIRST_NOT_SUPPORTED));							
			throw e;
		} */
		//create Block
		Block block = getFdlModelFactoryInstance().createBlock();
		block.setName(getNameRegistry().getActivityName(inputPinSet));
		getTarget().add(block);	
				
		//cache the block for re-execution of this rule
		setTargetActivity(block);	
		
		//create input and output containers for block
		OutputPinSet outputPinSet = (OutputPinSet)inputPinSet.getOutputPinSet().get(0);
		DataStructure inputDs = createDataStructure(inputPinSet);
		DataStructure outputDs = createDataStructure(outputPinSet);
		block.setInputDataStructure(inputDs);
		block.setOutputDataStructure(outputDs);	
		
		//map input value pin if any to block input container initialization
		List initialList = mapInputValuePin(inputPinSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			block.getInputContainerInitials().addAll(initialList);
		}
		
		//connect incoming connectors of the InputPinSet to the Block 
		setTargetOfConnectors(inputPinSet, block);	
		
		//create sink only since there will be no real inputOjbectPin (except inputValuePin)in input set for loopNode
		Sink sink = createSink(outputPinSet);
		if(sink != null){
			block.getSink().add(sink);
		}
		
		//set block's exit condition
		Expression exp =  transformBOMExpression(loopNode.getLoopCondition());
		if (exp != null) {
			block.setExitWhen(exp);
		} else {
			// Ray: try to handle null expression
			// Log a warning
			String params[] = new String[1];
			params[0] = BomUtils.getCanonicalName(loopNode,false,true);
			LoggingUtil.logWarning(MessageKeys.EXPRESSION_NULL_OR_UNKNOWN_WARNING,params,null);		 	
			
		}
		
		//create connectors for outgoing edges of the InputPinSet
		//** for LoopNode, there is no object flow from inputPinset to internal
		//connectInputPinSet(inputPinSet, src);		
		
		//create process rule to map internal of the block
	
		ProcessRule pr = FdlFactory.eINSTANCE.createProcessRule();
		getChildRules().add(pr);		
		pr.getSource().add(inputPinSet);
		pr.transformSource2Target();	
		
		//if there are object pin in the output pin set
		if(sink != null){
			//add additional program activity to accumulate the output object pin
			insertActivityForAccumulation(outputPinSet, sink);
		}
		
		//process all child rules targets so far, so we exclude any rules may be created later
		processChildTargets(block);
		
		TransformationRule branchRule =  createBranchRuleIfAny(inputPinSet);		
		if(branchRule != null){
			getTarget().addAll(branchRule.getTarget());
		}
				
		//connect outgoing connectors of the OutputPinSet to the Block
		setSourceOfConnectors(outputPinSet, block);		

		// Ray: added support for descriptions and documentations		
		Action action = inputPinSet.getAction();
		String desc = FdlUtil.transformDescription(action);
		if (desc!=null) {
			block.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			block.setDocumentation(getNameRegistry().getDocumentation(doc));
		}		
		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");

		return isComplete();
		
	}
	
	/**
	 * Converts BOM expression to FDL expression for the case condition 
	 * @param expr - OpaqueExpression
	 * @return Expression - the expression suitable for FDL logic condition
	 */
	private Expression transformBOMExpression(OpaqueExpression expr){
		 //create an ExpressionRule rule to transform the expression in a bom context
		 // an expression in a BPEL context		
		 if (expr instanceof StructuredOpaqueExpression && ((StructuredOpaqueExpression)expr).getExpression()!=null){
			 StructuredOpaqueExpression opaqueExpression = (StructuredOpaqueExpression)expr;
			 ExpressionRule exprRule = FdlFactory.eINSTANCE.createExpressionRule();
			 exprRule.getSource().add(opaqueExpression.getExpression());
			 getChildRules().add(exprRule);
			 exprRule.transformSource2Target(); 
			if (!exprRule.getTarget().isEmpty()){
			   Expression targetExpr = (Expression)exprRule.getTarget().get(0);
			   targetExpr.setName(expr.getName());
			   targetExpr.setDescription(getNameRegistry().getDescription(expr.getDescription()));
			   return targetExpr;	
			}		 
		 }
		 return null;
	}
	
	/**
	 * Inserts an FDL Program Activity that handles accumulation of object pins for the outputPinSet
	 *	 
	 * Note: the Program implementation must accumulate the data of the input container for each call during the loop.
	 *    
	 * @param outputPinSet
	 * @param sink
	 */
	private void insertActivityForAccumulation(OutputPinSet outputPinSet, Sink sink){
		
		//create Program activity for accumulation of the object pins in the ouput pin set
		String actName = getNameRegistry().getActivityName(FdlUtil.getCanonicalName(outputPinSet), "");
		String prmName = getNameRegistry().getProgramName(actName, "");
		//assume input datastructure is the same as output datastructure since the number of object is accumulated at run time.
		DataStructure inputDs = createDataStructure(outputPinSet);
			
		ProgramActivity pa = getFdlModelFactoryInstance().createProgramActivity();
			pa.setName(actName);		
			pa.setProgramName(prmName);		
			pa.setInputDataStructure(inputDs);
			pa.setOutputDataStructure(inputDs);
		//add pa to block
		Block block = (Block)getTarget().get(0);
		block.getConstructs().add(pa);	
		
		//create program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();	
		
		Program p = (Program)pr.getTarget().get(0);
		//override description
		p.setDescription(FdlConstants.LOOP_ACCUMULATION_PROGRAM_DESCRIPTION);
		
		//connect all incoming connectors of outputPinset to pa
		setTargetOfConnectors(outputPinSet, pa);	
		
		//create data connector to link the pa with sink
		//directly create data flow
		// the data mapping from pa to sink only can be assumed default
		DataFlow df = getFdlModelFactoryInstance().createDataFlow();
		DataMapping dm = getFdlModelFactoryInstance().createDataMapping();
		dm.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dm.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		df.getDataMappings().add(dm);
		df.setFromActivity(pa);
		df.setSink(sink);
		//add df to block
		block.getConstructs().add(df);
	}

} //LoopNodeRuleImpl
