/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.processes.actions.Map;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.fdl.model.Activity;
import com.ibm.btools.fdl.model.ControlFlow;
import com.ibm.btools.fdl.model.DataFlow;
import com.ibm.btools.fdl.model.DataMapping;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.Program;
import com.ibm.btools.fdl.model.ProgramActivity;
import com.ibm.btools.fdl.model.SynchronizationType;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.MapRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProgramRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;

/**
 * <!-- begin-user-doc -->
 * 	 
 * Source: BOM InputPinSet (on a Map)
 * Target: FDL ProgramActivity
 * 
 * Maps a BOM Map to an FDL Program Activity.
 * Connects the mapped activity with the incoming and outgoing connectors.
 * Note: we assume the FDL program will be the implementation of the map function 
 * 		 to convert the input to output data structure. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class MapRuleImpl extends ConnectableRuleImpl implements MapRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private boolean createNoOp=true;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MapRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.MAP_RULE;
	}

	/**
	 * Source: BOM InputPinSet (on a Map)
	 * Target: FDL ProgramActivity
	 * 
	 * Maps a BOM Map to an FDL Program Activity.
	 * Connects the mapped activity with the incoming and outgoing connectors.
	 * Note: we assume the FDL program will be the implementation of the map function 
	 * 		 to convert the input to output data structure. 
	 * 
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);		
		
		if(!(isFirstTimeExec())){
			reExecute(inputPinSet);
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
		}

		//first time execute this rule
		setFirstTimeExec(false);
		//map's input set and output set may not correlated. There should be only one input set and only output set.
		OutputPinSet outputPinSet = (OutputPinSet)(((Map)inputPinSet.getAction()).getOutputPinSet().get(0));
		DataStructure inputDs = createDataStructure(inputPinSet);
		DataStructure outputDs = createDataStructure(outputPinSet);

		ProgramActivity pa;
		if (createNoOp) {
			pa = createNoopProgramActivity(inputDs);
		} else {
			pa = createProgramActivity(inputDs, outputDs);
		}
		//map InputPinSet to see if there is any activity created
		mapInputPinSet(inputPinSet, pa); 
	
		//do InputPinSet connection		
		setTargetOfConnectors(inputPinSet, pa);
		
		createBranchRuleIfAny(inputPinSet);
		
		if (createNoOp && inputDs != outputDs) {
			ProgramActivity compatContainerAct = createCompatibleContainerActivity(pa,null,inputDs,outputDs,outputPinSet);
			getTarget().add(compatContainerAct);			
			mapOutputPinSet(outputPinSet, compatContainerAct);
		} else {
			mapOutputPinSet(outputPinSet, pa);
			//do OutputPinSet connection
			setSourceOfConnectors(outputPinSet, pa);
			
		}

		// Ray: added support for descriptions and documentations		
		Action action = inputPinSet.getAction();
		String desc = FdlUtil.transformDescription(action);
		if (desc!=null) {
			pa.setDescription(getNameRegistry().getDescription(desc));
		}
		String doc = FdlUtil.transformDocumentation(action);
		if (doc!=null) {
			pa.setDocumentation(getNameRegistry().getDocumentation(doc));
		}		
		//all targets propagate to parent
		propagateChildTargets();

		setComplete(true);
		
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	}
		
	/**
	 * Creates FDL Program Activity and Program to represent the Map.
	 * Note: each function has only one Program implementing it. 
	 * 		 The program uses the input and output data structure from the activity.
	 * 
	 * @param inputDs Input data structure for the activity
	 * @param outputDs Output data structure for the activity
	 * @return ProgramActivity - The program activity created for the map action 
	 */
	private ProgramActivity createProgramActivity(DataStructure inputDs, DataStructure outputDs){	

		ProgramActivity pa;
	
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		Map mapAction = (Map)inputPinSet.getAction(); 
		String paName = getNameRegistry().getActivityName(inputPinSet);
		//for each function, we will create one program, that program can handle any input and output
		String funcUid = mapAction.getFunction().getUid();
		 
		String prmName = getNameRegistry().getProgramName(funcUid, FdlConstants.FUNCTION_PROGRAM_NAME_SUFFIX);

		pa = getFdlModelFactoryInstance().createProgramActivity();
		pa.setName(paName);
		pa.setProgramName(prmName);
	
		pa.setInputDataStructure(inputDs);
		pa.setOutputDataStructure(outputDs);
	
		String dsc = getNameRegistry().getDescription(FdlUtil.getFullName(mapAction));
		pa.setDescription(dsc);
	
		//map input value pin if any to activity input container initialization
		List initialList = mapInputValuePin(inputPinSet, inputDs);
		if((initialList != null) && (!initialList.isEmpty())){
			pa.getInputContainerInitials().addAll(initialList);
		}

		//set pa as target of this rule
		getTarget().add(pa);
		//** cache pa for reExecute
		setTargetActivity(pa);

		//create program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(pa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		Program p = (Program)pr.getTarget().get(0);
		//override description	
		p.setDescription(FdlConstants.FUNCTION_PROGRAM_DESCRIPTION); //"map function program"
		String doc = getNameRegistry().getDocumentation(
			"<" + mapAction.getFunction().getLanguage() + "> " + 
			mapAction.getFunction().getBody());
		p.setDocumentation(getNameRegistry().getDocumentation(doc));
		p.setStructuresFromActivity(Boolean.TRUE);
		
		return pa;
	}

	/**
	 * Creates am FDL noop program activity
	 * @return
	 */
	private ProgramActivity createNoopProgramActivity(DataStructure ds)
	{
		ProgramActivity nopa = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createProgramActivity();
		InputPinSet inputPinSet = (InputPinSet)getSource().get(0);
		String paName = getNameRegistry().getActivityName(inputPinSet);
		nopa.setName(paName);
		nopa.setProgramName(FdlConstants.NOOP_PROGRAM_NAME);
		nopa.setSynchronization(SynchronizationType.CHAINED_LITERAL);
		//create noop program
		ProgramRule pr = FdlFactory.eINSTANCE.createProgramRule();
		pr.getSource().add(nopa);
		getRoot().getChildRules().add(pr);
		pr.transformSource2Target();
		nopa.setInputDataStructure(ds);
		nopa.setOutputDataStructure(ds);
		getTarget().add(nopa);
		getTarget().add(createDataDefaultConnector(nopa));
		return nopa;
	}

	/**
	 * Creates an 'dummy' noop activity for the case when the inputpinset and outputpinset of
	 * a control action are not identical and therefore not be able to share the same input/ouput
	 * DataStructure.  If a noop activity is creates, re-wiring the flow connector will be handled
	 * as well
	 * @param srcAct
	 * @param tarAct
	 * @param srcConRule
	 * @param tarConRule
	 * @param accumulateCopy
	 * @return
	 */
	protected ProgramActivity createCompatibleContainerActivity(
		Activity srcAct, Activity tarAct, DataStructure srcCon, DataStructure tarCon, PinSet pinSet)
	{
		ProgramActivity noopAct = createNoopProgramActivity(tarCon);
		ControlFlow ctrlFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createControlFlow();
		DataFlow dataFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataFlow();
		getTarget().add(ctrlFlow);
		getTarget().add(dataFlow);

		noopAct.setName(createNoopActivityName(srcAct,pinSet));

		// link between the srcAct and noopAct
		ctrlFlow.setFromActivity(srcAct);
		ctrlFlow.setToActivity(noopAct);
		dataFlow.setFromActivity(srcAct);
		dataFlow.setToActivity(noopAct);

		setSourceOfConnectors(pinSet,noopAct);
		
		return noopAct;
		
	}

	/**
	 * Creates an FDL DataDefault connector
	 * @param act
	 * @return
	 */
	protected DataFlow createDataDefaultConnector(Activity act)
	{
		DataFlow dataFlow = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataFlow();
		dataFlow.setDefaultActivity(act);
		DataMapping dataMapping = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDataMapping();
		dataMapping.setFromDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dataMapping.setToDataStructureMemberName(FdlConstants.TOP_STRUCTURE_NAME);
		dataFlow.getDataMappings().add(dataMapping);
		return dataFlow;
	}

	/**
	 * Creates a name for the noop activity based on the pinset
	 * @param act
	 * @param pinset
	 * @return
	 */
	private String createNoopActivityName(Activity act,PinSet pinSet)
	{
		//return act.getName()+"_"+pinset.getName();
	
		//return getNameRegistry().getActivityName(pinSet);
		String noopName = getNameRegistry().getActivityName(pinSet);
		// The returned noopName will be the same as the original control action name in the case
		// of Join (how about fork?).  Therefore need to call the registry and recreate a unique name again
		// using a different method
		return getNameRegistry().getActivityName(noopName,"");
	}	
} //MapRuleImpl
