/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.OrganizationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.fdl.model.ModelFactory;
import com.ibm.btools.fdl.model.Organization;
import com.ibm.btools.fdl.model.Person;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Organization Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class OrganizationRuleImpl extends AbstractFdlProcDefRuleImpl implements OrganizationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected OrganizationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.ORGANIZATION_RULE;
	}

	/**
	 * SOURCE: BOM OrganizationUnit
	 * TARGET: FDL Organization
	 * Maps ResponsibleOrganization to FDL Organization
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;
		if (getSource() == null || getSource().isEmpty()){
			return false;
		}		
		OrganizationUnit orgUnit = (OrganizationUnit)getSource().get(0);
		Organization org = buildOrganization(orgUnit);		
		getTarget().add(org);  
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");

		return isComplete();
	}

	/**
	 * Creates a FDL Organization to represent the a department or a company of a Staff
	 * @param orgUnit - a BOM action	 
	 * @return Organization - a resulting FDL Organization
	 */	
	private Organization buildOrganization(OrganizationUnit orgUnit)
	{		
		Organization org = ModelFactory.eINSTANCE.createOrganization();		
		List slots = orgUnit.getSlot();		
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();	
				if (thisSlot.getDefiningFeature().getName()!=null){					
					if(thisSlot.getDefiningFeature().getName().equals("organizationID")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){	
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							if (value!=null)
								value=FdlUtil.getResourceOrganizationName(value);
							org.setName(value);
							getTarget().add(org); 
						}
					}
				}			
			}
		}
		//second pass to map references slots
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();			
				if(thisSlot.getDefiningFeature().getName().equals("manager")){	
					if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){	
						IndividualResource resource = (IndividualResource)((InstanceValue)thisSlot.getValue().get(0)).getInstance();														
						org.setManagerID(assignManager(resource));
					}
				}
				else if(thisSlot.getDefiningFeature().getName().equals("parentOrganization")){															
					if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
						OrganizationUnit parentOrg = (OrganizationUnit)((InstanceValue)thisSlot.getValue().get(0)).getInstance();
						Organization tempParent = assignParentOrg(parentOrg);
						if (tempParent==null){
							//parent Organization not created yet			;
							org.setParentOrganization(createOrganization(parentOrg));
						}else {
							org.setParentOrganization(tempParent);
						}
					}
						
				}
				else if(thisSlot.getDefiningFeature().getName().equals("members")){
					List values = thisSlot.getValue();
					if(values != null && !values.isEmpty()){
						Iterator valueIt = values.iterator();
						while (valueIt.hasNext()){
							IndividualResource resource = (IndividualResource)((InstanceValue)valueIt.next()).getInstance();
							Person tempPerson = assignMembers(resource);
							if (tempPerson==null)
								org.getGroupMembers().add(createPerson(resource));
							else
								org.getGroupMembers().add(tempPerson);
								
						}
					}	
				}	
			}
		}		
		
		
		
		return org;
	}

	/**
	 * Creates a Parent Organization if not already declared 
	 * @param orgUnit - a BOM action	 
	 * @return Organization - a resulting parent FDL Organization
	 */	
	private Organization createOrganization(OrganizationUnit orgUnit){
			OrganizationRule orgRule = com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory.eINSTANCE.createOrganizationRule();			
			orgRule.getSource().add(orgUnit);
			getRoot().getChildRules().add(orgRule);
			orgRule.transformSource2Target();	
			return (Organization)orgRule.getTarget().get(0);		
	}	

	/**
	 * Creates a FDL Person
	 * @param resource - an Individual Resource 
	 * @return Person - a resulting  FDL Person
	 */		
	private Person createPerson(IndividualResource resource){
		PersonRule personRule = com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory.eINSTANCE.createPersonRule();
		personRule.getSource().add(resource);
		getRoot().getChildRules().add(personRule);
		personRule.transformSource2Target();
		return (Person)personRule.getTarget().get(0);
	}

	/**
	 * Creates a FDL Person
	 * @param resource - an Individual Resource 
	 * @return String - a string manager ID
	 */		
	private String assignManager(IndividualResource resource){
		String value = null;				
		List resourceSlots = resource.getSlot();		
		if(resourceSlots != null && !resourceSlots.isEmpty()){
			Iterator resourceIt = resourceSlots.iterator();
			while (resourceIt.hasNext()){		
				Slot thisResourceSlot = (Slot)resourceIt.next();					
				if (thisResourceSlot.getDefiningFeature().getName()!=null){					
					if(thisResourceSlot.getDefiningFeature().getName().equals("personId")){
						if (thisResourceSlot.getValue()!=null && !thisResourceSlot.getValue().isEmpty()){
							String personId = ((LiteralString)thisResourceSlot.getValue().get(0)).getValue();
							if (personId!=null){
								personId=FdlUtil.getResourceOrganizationName(personId);
								return personId.toUpperCase();	
							}
						}													
					}
				}
			}
		}
		return null;		
	}

	/**
	 * Finds an already exisitng Organization
	 * @param parentOrg - an OrganizationUnit
	 * @return Organization - FDL Organization
	 */		
	private Organization assignParentOrg (OrganizationUnit parentOrg){			
		List orgSlots = parentOrg.getSlot();
		String orgID = null;
		if(orgSlots != null && !orgSlots.isEmpty()){
			Iterator orgIt = orgSlots.iterator();
			while (orgIt.hasNext()){		
				Slot thisOrgSlot = (Slot)orgIt.next();					
				if (thisOrgSlot.getDefiningFeature().getName()!=null){					
					if(thisOrgSlot.getDefiningFeature().getName().equals("organizationID")){
						if (thisOrgSlot.getValue()!=null && !thisOrgSlot.getValue().isEmpty()){
							orgID =((LiteralString)thisOrgSlot.getValue().get(0)).getValue();
						}					
						break;								
					}
				}
			}
		}
		Iterator ruleIt = getRoot().getChildRules().iterator();
		while (ruleIt.hasNext() && orgID!=null){
			TransformationRule rule = (TransformationRule)ruleIt.next();
			if (rule instanceof OrganizationRule && rule.getTarget()!=null && !rule.getTarget().isEmpty()){						
				Organization existOrg = (Organization)((OrganizationRule)rule).getTarget().get(0);
				if ( existOrg.getName().equals(FdlUtil.getResourceOrganizationName(orgID)) ){
					return existOrg;					
				}
			}
		}
		return null;
		
	}

	/**
	 * Finds already existing Perons as Members of organization
	 * @param resource - an IndividualResource
	 * @return Person - FDL Person
	 */			
	private Person assignMembers (IndividualResource resource){		
		String personID = null;			
		List resourceSlots = resource.getSlot();		
		if(resourceSlots != null && !resourceSlots.isEmpty()){
			Iterator resourceIt = resourceSlots.iterator();
			while (resourceIt.hasNext()){		
				Slot thisResourceSlot = (Slot)resourceIt.next();					
				if (thisResourceSlot.getDefiningFeature().getName()!=null){					
					if(thisResourceSlot.getDefiningFeature().getName().equals("personId")){
						if (thisResourceSlot.getValue()!=null && !thisResourceSlot.getValue().isEmpty()){
							personID = ((LiteralString)thisResourceSlot.getValue().get(0)).getValue();	
							if (personID!=null)
								personID=FdlUtil.getResourceOrganizationName(personID).toUpperCase();
						}						
						break;								
					}
				}
			}
		}
		Iterator ruleIt = getRoot().getChildRules().iterator();
		//boolean isExistsPerson = false;
		while (ruleIt.hasNext() && personID!=null){
			TransformationRule rule = (TransformationRule)ruleIt.next();
			if (rule instanceof PersonRule && rule.getTarget()!=null && !rule.getTarget().isEmpty()){						
				Person person = (Person)((PersonRule)rule).getTarget().get(0);
				if ( person.getUserID().equals(personID) ){
					return person;
				}
			}
		}
		return null;
	}									
			

} //OrganizationRuleImpl
