/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.Qualification;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.OrganizationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.PersonRule;
import com.ibm.btools.te.ilm.heuristics.fdl.RoleRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.fdl.model.ModelFactory;
import com.ibm.btools.fdl.model.Organization;
import com.ibm.btools.fdl.model.Person;
import com.ibm.btools.fdl.model.Role;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Person Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class PersonRuleImpl extends AbstractFdlProcDefRuleImpl implements PersonRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PersonRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.PERSON_RULE;
	}

	/**
	 * SOURCE: BOM IndividualResource
	 * TARGET: FDL Person
	 * Maps ResourceRequirement to FDL Person
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {	
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete())
			return true;			
		if (getSource() == null || getSource().isEmpty()){
			return false;
		}		
		IndividualResource resource = (IndividualResource)getSource().get(0);
		Person person = buildPerson(resource);	
		if (person!=null)	
			getTarget().add(person);  
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Creates a Person with all the corresponding attributes
	 * @param resource - an IndividualResource
	 * @return Person - FDL Person
	 */		
	private Person buildPerson(IndividualResource resource)
	{
		Person person = ModelFactory.eINSTANCE.createPerson();
		List slots = resource.getSlot();		
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();	
				if (thisSlot.getDefiningFeature().getName()!=null){					
					if(thisSlot.getDefiningFeature().getName().equals("personId")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							if (value!=null){
								value = FdlUtil.getResourceOrganizationName(value).toUpperCase();
							}						
							person.setPersonID(value);
							person.setUserID(value);
							getTarget().add(person);  
						}
					}
					else if(thisSlot.getDefiningFeature().getName().equals("lastName")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							person.setLastName(value);
						}
						
					}
					else if(thisSlot.getDefiningFeature().getName().equals("firstName")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							person.setFirstName(value);		
						}
					}
					else if(thisSlot.getDefiningFeature().getName().equals("middleName")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							person.setMiddleName(value);
						}
					}	
					else if(thisSlot.getDefiningFeature().getName().equals("phone")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							String value =((LiteralString)thisSlot.getValue().get(0)).getValue();
							person.setPhone(value);
						}
					}				
				}			
			}
		}
		
		
		//Second pass - to map department that has references		
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();		
				if( (thisSlot.getDefiningFeature().getName().equals("department")) || (thisSlot.getDefiningFeature().getName().equals("company")) ){
					if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
						OrganizationUnit orgUnit = (OrganizationUnit)((InstanceValue)thisSlot.getValue().get(0)).getInstance();
						Organization tempOrg = assignOrganization(orgUnit);						
						if (tempOrg==null){
							Organization org = createOrganization(orgUnit);
							org.getGroupMembers().add(person);							
						}else {
							if (!tempOrg.getGroupMembers().contains(person))
								tempOrg.getGroupMembers().add(person);
						}		
					}	
				}	
			}
		}
		//build role members
		buildRoleMembers(resource, person);
		
		
		return person;
	}

	/**
	 * Creates a FDL Organization if this organization has not been declared previously
	 * @param orgUnit - an OrganizationUnit
	 * @return Organization - FDL Organization
	 */			
	private Organization createOrganization(OrganizationUnit orgUnit){
			OrganizationRule orgRule = com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory.eINSTANCE.createOrganizationRule();			
			orgRule.getSource().add(orgUnit);
			getRoot().getChildRules().add(orgRule);
			orgRule.transformSource2Target();
			return (Organization)orgRule.getTarget().get(0);
	}
	
	/**
	 * Creates a FDL Role if this role has not been declared previously
	 * @param role - a role
	 * @return bom Role - FDL Role
	 */			
	private Role createRole(com.ibm.btools.bom.model.resources.Role  role){
			RoleRule roleRule = com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory.eINSTANCE.createRoleRule();			
			roleRule.getSource().add(role);
			getRoot().getChildRules().add(roleRule);
			roleRule.transformSource2Target();
			return (Role)roleRule.getTarget().get(0);
	}	

	/**
	 * Put the Person IDs into the corresponding Role Member list under FDL Role
	 * @param resource - an IndividualResource
	 * @param person - an FDL Person
	 * @return
	 */		
	private void buildRoleMembers(IndividualResource resource, Person person){
		
		List qualifications = resource.getQualification();
		if(qualifications != null && !qualifications.isEmpty()){
			Iterator it = qualifications.iterator();
			while(it.hasNext()){
				boolean roleFound = false;
				Qualification thisQualification = (Qualification)it.next();
				if (thisQualification.getRole()!=null){
					Iterator ruleIt = getRoot().getChildRules().iterator();
					while (ruleIt.hasNext()){
						TransformationRule rule = (TransformationRule)ruleIt.next();
						if (rule instanceof RoleRule){						
							Role role = (Role)((RoleRule)rule).getTarget().get(0);
							String qualificationName = thisQualification.getRole().getName();
							if (qualificationName!=null){
								if ( role.getName().equals(FdlUtil.getResourceOrganizationName(qualificationName)) ){
									//add person to FDL Role
									role.getRoleMembers().add(person);
									roleFound=true;
									break;
								}
							}
						}
					}
				}
				if (!roleFound){
					//role has not been created yet, create role
					Role role = createRole(thisQualification.getRole());
					role.getRoleMembers().add(person);

				}				
			}
			
			
		}
	}	

	/**
	 * Finds an exisit Organization
	 * @param orgUnit - an OrganizationUnit
	 * @return Organization - FDL Organization
	 */		
	private Organization assignOrganization(OrganizationUnit orgUnit){
		List orgSlots = orgUnit.getSlot();
		String orgID = null;
		if(orgSlots != null && !orgSlots.isEmpty()){
			Iterator orgIt = orgSlots.iterator();
			while (orgIt.hasNext()){		
				Slot thisOrgSlot = (Slot)orgIt.next();					
				if (thisOrgSlot.getDefiningFeature().getName()!=null){					
					if(thisOrgSlot.getDefiningFeature().getName().equals("organizationID")){
						if (thisOrgSlot.getValue()!=null && !thisOrgSlot.getValue().isEmpty()){
							orgID =((LiteralString)thisOrgSlot.getValue().get(0)).getValue();	
							if (orgID!=null)
								orgID=FdlUtil.getResourceOrganizationName(orgID);
						}
						break;								
					}
				}
			}
		}
		if (orgUnit!=null && orgID!=null){
			//Check to see if organization already exists
			Iterator ruleIt = getRoot().getChildRules().iterator();
			boolean isExistsOrganization=false;			
			while (ruleIt.hasNext()){
				TransformationRule rule = (TransformationRule)ruleIt.next();
				if (rule instanceof OrganizationRule && rule.getTarget()!=null && !rule.getTarget().isEmpty()){						
					Organization existOrg = (Organization)((OrganizationRule)rule).getTarget().get(0);
					if ( existOrg.getName().equals(orgID) ){
						//isExistsOrganization=true;
						 return existOrg;
						//break;
					}
				}
			}			
		}
		return null;
	}

} //PersonRuleImpl
