/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessRule;


import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;


/**
 * <!-- begin-user-doc -->
 *  source: BOM InputPinSet of StructuredActivityNode (non-task)
 * target: none 
 * Walks through BOM process graph from the source InputPinSet, creates child rule to 
 * maps all elements in the StrucuredActivityNode reachable from the InputPinset 
 * excluding the InputPinset and its outgoing edges.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ProcessRuleImpl extends ProcessWalkingRuleImpl implements ProcessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
		
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.PROCESS_RULE;
	}

/**
 * source: BOM InputPinSet of StructuredActivityNode (non-task)
 * target: none 
 * Walks through BOM process graph from the source InputPinSet, creates child rule to 
 * maps all elements in the StrucuredActivityNode reachable from the InputPinset 
 * excluding the InputPinset and its outgoing edges.
 *  
 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		/*
		 * Map all elements in the StructuredActivityNode reachable from the InputPinSet.
		 * This include action connected from InitialNode. 
		 * 
		 * 1. for each input set reachable from the source input set, 
		 *    step through it, create child rules and execute the rule, 
		 *    until reached end of StructuredActivityNode.
		 * 2. propagate targets produced by child rules.
		 *   
		 */
		//assertion
		if (getSource() == null || getSource().isEmpty())
			return false;
		
		//source of this rule is only one input set
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		
		//check if this is not first time execute this rule
		 if(!isFirstTimeExec()){
			reExecute(inputSet); 
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;
		 }
		 
		//else it is first time execution of the rule
		setFirstTimeExec(false);
		 		
		BomWalker bw = ProcessUtil.getBomWalker(getContext());
		
		List inputSetList=bw.getNext(inputSet, BomWalker.INPUT_SET, false);
		
		//!!main logic of this method is here
		//walk through each of the input set, create and execute child rules
		InputPinSet nextIS = null;
		List nextInputPinSets = null;
		for (int i = 0, count = inputSetList.size(); i<count; i++){
			nextIS = (InputPinSet)inputSetList.get(i);
			do{			
				nextIS = stepThroughBOM(nextIS);
				if (nextIS != null){ 	//means we just created a child rule
					//we must continue
					// It is safe just to use the first elment since we always skip 
					// the branch point, we should only have one InputPinSet back
					nextInputPinSets = bw.getNext(nextIS, BomWalker.INPUT_SET, true);
					if (nextInputPinSets == null || nextInputPinSets.isEmpty()) {
						nextIS = null;
					} else { 
						nextIS = (InputPinSet) nextInputPinSets.get(0);
					}					
				}
				//else nextIS == null, we want stop walk down that path
					 
			} while(nextIS!= null);				 
		}
			
		//propagate all child targets to parent
		propagateChildTargets();
		
		setComplete(true);		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return true;
	}
	
	
} //ProcessRuleImpl
