/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.CallOperationAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Map;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.actions.RetrieveArtifactAction;
import com.ibm.btools.bom.model.processes.actions.StoreArtifactAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;

import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.heuristics.fdl.ConnectorRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FlowRule;
import com.ibm.btools.te.ilm.heuristics.fdl.ProcessWalkingRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BomWalker;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.fdl.model.ControlFlow;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * The abstract class provides common functions for stepping through BOM process graph rules.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public abstract class ProcessWalkingRuleImpl extends ConnectableRuleImpl implements ProcessWalkingRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessWalkingRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.PROCESS_WALKING_RULE;
	}

/**
 * If there is a rule already exist for the inputSet as the source, re-execute that rule.
 * Otherwise, do the mapping for the inputSet. 
 * Mapping includes following: 
 * 1. If the inputSet is not a branch point, creates connectors for the OutputPinSet that the inputSet can jump to. 
 * 2. Creates child rule to map the node that the inputSet is on.
 *
 * @param inputSet	the InputPinSet that just stepping on
 * @return InputPinSet - the inputSet passed in if new rule is created for the inputSet 
 * 						or null if no new rule created
 */
	protected  InputPinSet stepThroughBOM(InputPinSet inputSet){
			TransformationRule tr = getRuleForPinSet(inputSet);
			if(tr != null){
				/* if one does exist, this input set has already been mapped, but we must execute it again 
				 so that the links that have been created that point to the input set can have their target set.
				 */
				tr.transformSource2Target(); 
				//Return null to indicate that we did not create a new child rule.
				return null;
			}
		
			/* Otherwise, if a rule for this input set does not yet exist,
			 * Based on the type of action associated with inputSet, a transformation rule must be created.  
			 * But first, we must determine whether the action inputSet either an exclusive branch point or a concurrent branch point.  
			 * This is important because if it is a branch point, we do not create the links immediately following the action, 
			 * but instead, leave the child rule to create those links itself. 
			 */
			//Check whether the Action associated with inputSet is a branch point 	
			boolean isEBranch = false;
			boolean isCBranch = false;
			OutputPinSet outputSet;
		
			isEBranch = BomHelper.getInstance().isEBranch(inputSet);
			if(!isEBranch){
				BomWalker bw = ProcessUtil.getBomWalker(getContext());
				//if inputSet is not an exclusive branch, then the there should never be multiple output set returned from following method

				// Ray May 07,04: Try to handle inputset with no correlation to any outputset
				List outputSets = bw.getNext(inputSet, BomWalker.OUTPUT_SET, true);
				if (outputSets.size()>0) {
					outputSet = (OutputPinSet)bw.getNext(inputSet, BomWalker.OUTPUT_SET, true).get(0);
					isCBranch = BomHelper.getInstance().isCBranch(outputSet);					
				} else {
					outputSet=null;
					isCBranch = false;
				}
//			 	outputSet = (OutputPinSet)bw.getNext(inputSet, BomWalker.OUTPUT_SET, true).get(0);
//				isCBranch = BomHelper.getInstance().isCBranch(outputSet);
//				if(!isCBranch){
				if (!isCBranch && outputSet!=null) {	// Ray
				/*If it is neither of the two types of branch, then this Rule is 
				//responsible for creating the connectors that extend from the action that contains inputSet.  
				//This will involve creating a ConnectorRule for 
				//each pin in the output set that is correlated to is.  
				//(We know that there is only one output set correlated to is, 
				//and also that the output set only has one pin, 
				//because it is neither an exclusive nor concurrent branch point). 
				*/
			
				//Create ConnectorRule for the only pin in output set
				createConnectorRulesForOutputPinSet(outputSet);
				}
			 	
			} //end if !branch	 
		
			//	Next, create the child rule depending on the type of action.  The source of the child rule will be inputSet.
			//	Call the following method to create the child rule based on the type of action the contains isNext
			TransformationRule ctr = createChildRule(inputSet);

			if (ctr!= null){		
				//Add the child rule to this rule
				getChildRules().add(ctr);
				ctr.transformSource2Target();
			}
		    
			return inputSet;
		
		}
	/**
	 * Creates child rule according to the type of the inputSet.
	 * @param inputSet The input set that a child rule is to be created for
	 * @return TransformationRule - The child rule that represents <code>inputSet</code>
	 */	
	private TransformationRule createChildRule(InputPinSet inputSet){
		Action sourceAction = inputSet.getAction();
		TransformationRule rule;	
		if(BomUtils.isTask(sourceAction)){ 
			rule = FdlFactory.eINSTANCE.createSANTaskRule();
		}else if(BomUtils.isReusableTask(sourceAction) || BomUtils.isReusableService(sourceAction)){ 
			rule = FdlFactory.eINSTANCE.createSANReusableTaskRule();
		}else if (sourceAction instanceof CallBehaviorAction){ //must do this after check on isReusable task since reusable task is also a CBA
			rule = FdlFactory.eINSTANCE.createCallBehaviorActionRule();
		}else if(sourceAction instanceof LoopNode){
			rule = FdlFactory.eINSTANCE.createLoopNodeRule();
		}else if(BomUtils.isProcess(sourceAction)){
			rule = FdlFactory.eINSTANCE.createSANNestedProcessRule();
		}else if(sourceAction instanceof Decision){		    
			rule = FdlFactory.eINSTANCE.createDecisionRule();
		}else if(sourceAction instanceof Merge){		
			rule = FdlFactory.eINSTANCE.createMergeRule();
		}else if(sourceAction instanceof Join){
			rule = FdlFactory.eINSTANCE.createJoinRule();
		}else if(sourceAction instanceof Fork){
			rule = FdlFactory.eINSTANCE.createForkRule();
		}else if(sourceAction instanceof RetrieveArtifactAction) {
			rule = FdlFactory.eINSTANCE.createRetrieveDatastoreArtifactActionRule();
		}else if(sourceAction instanceof StoreArtifactAction){
			rule = FdlFactory.eINSTANCE.createStoreDatastoreArtifactActionRule();
		}else if(sourceAction instanceof CallOperationAction){
			rule = FdlFactory.eINSTANCE.createCallOperationActionRule();		
		}else if(sourceAction instanceof Map){
			rule = FdlFactory.eINSTANCE.createMapRule();
		}else{
//			rule = null;
			// Ray 04/27/04: Try to handle non-supported Action, treat it like a task by default:
			rule = FdlFactory.eINSTANCE.createSANTaskRule();
		}
		
		if(rule != null){		
			rule.getSource().add(inputSet);
		}
		return rule;
	
	}
	
	/**
	 * Creates connector rules to transform edges outgoing from the <code>outputPinSet</code>.
	 * @param outputPinSet
	 */
	protected void createConnectorRulesForOutputPinSet(OutputPinSet outputPinSet){
		Pin pin;
		List pinList = new ArrayList();
		ConnectorRule cr;		
		pinList.addAll(((OutputPinSet)outputPinSet).getOutputControlPin());
		pinList.addAll(((OutputPinSet)outputPinSet).getOutputObjectPin());
		for(Iterator it = pinList.iterator(); it.hasNext(); ){			
			pin = (Pin)it.next();
			ActivityEdge edge = pin.getOutgoing();
			PinSet targetPinSet = null;
			if((edge != null) && ((targetPinSet = (PinSet)BomUtils.findTargetSet(edge)) != null)){			
			 // pin's target is not any ControlNode			
			 	if(needConnectorRule(pin, outputPinSet, targetPinSet)){	
					cr = FdlFactory.eINSTANCE.createFlowRule();	
					getChildRules().add(cr);	
					cr.getSource().add(pin.getOutgoing()); //edge			
					cr.getSource().add(outputPinSet); //outputSet
					cr.getSource().add(targetPinSet);						
					cr.transformSource2Target();
			 	}
			}
		}			 
	}
	
	/**
	 * Handles special case that outgoing connector from a Merge Action should only created once.
	 * This method checkes existence of connector rule for the pin, 
	 * if found one, returns false, otherwise return true.
	 *  
	 * @param pin 
	 * @param sourceSet
	 * @param targetSet
	 * @return boolean true means outgoing connector should be created for the pin 
	 */
	private boolean needConnectorRule(Pin pin, OutputPinSet sourceSet, PinSet targetSet ){ 
		if(sourceSet.getAction() instanceof Merge){
			//check if ConnectorRule exist
			Collection sources = new ArrayList();
			sources.add(pin.getOutgoing());
			sources.add(sourceSet);
			sources.add(targetSet);
			ConnectorRule rule = (ConnectorRule)TransformationUtil.getRuleForSource(
							  ProcessUtil.getProcessDefinitionRule(getContext()),
							  FlowRule.class,
							  sources,
							  ControlFlow.class);
			if(rule != null)
				return false;							   		
		}	
 	
		return true;			
	}		

} //ProcessWalkingRuleImpl
