/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.ActivitiesFactory;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.RetrieveDatastoreArtifactPinRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlConstants;
import com.ibm.btools.fdl.model.DataStructure;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * Source: RetrieveArtifactPin (referencing Datastore only)
 * Target: ProgramActivity (referencing retrieve from Datastore Program)
 * 
 * Maps a RetrieveArtifactPin to a FDL ProgramActivity.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class RetrieveDatastoreArtifactPinRuleImpl extends RetrieveDatastoreArtifactRuleImpl implements RetrieveDatastoreArtifactPinRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RetrieveDatastoreArtifactPinRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.RETRIEVE_DATASTORE_ARTIFACT_PIN_RULE;
	}

	/**
	 * Source: RetrieveArtifactPin (referencing Datastore only)
	 * Target: ProgramActivity (referencing retrieve from Datastore Program)
	 * 
	 * Maps a BOM RetrieveArtifactPin to a FDL ProgramActivity.
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		if (getSource() == null || getSource().isEmpty())
			return false;		
		
		RetrieveArtifactPin pin = (RetrieveArtifactPin)getSource().get(0);			
		if(!(isFirstTimeExec())){
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
		}

		//first time execute this rule
		setFirstTimeExec(false);
		
		createProgramActivity(pin);

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
}
		

	/**
	 * Creates a FDL retrieve datastore ProgramActivity for the <code>pin</code>.
	 * 	
	 * @param pin The retrieve artifact pin
	 * @return ProgramActivity - The retrieve datastore program activity created
	 */
	private ProgramActivity createProgramActivity(RetrieveArtifactPin pin){
			
		Repository datastore = pin.getRepository();

		DataStructure outputDs = createOutputDataStructure(pin);
		
		String pinName = getNameRegistry().getActivityName(pin.getAction()) + "_" + pin.getName();
	//	String paName = getNameRegistry().getActivityName(FdlUtil.getCanonicalName(pin), "");	
		String paName = getNameRegistry().getActivityName(pinName, "");
			
		//need to extract the multiplicity of the pin and set as initial value to program input datastructure
	    ValueSpecification valueSpec = (ValueSpecification)pin.getUpperBound();
	    String lengthValueString = FdlUtil.getValueSpecificationString(valueSpec);
	    if(lengthValueString == null){
			lengthValueString = "1";
	    }

		ProgramActivity pa = createRetrieveDatastoreProgramActivity(
			datastore,
			paName,
			outputDs,
			pin.getIsRemove(),
			pin.getAtBeginning(),
			lengthValueString);	
	
		String dsc = FdlUtil.getFullName(pin);
		pa.setDescription(getNameRegistry().getDescription(dsc));
	  	
		getTarget().add(pa);
		return pa;
}	
		
		
/**
 * Creates the output datastructure from the pin.
 * @param pin
 * @return
 */
private DataStructure createOutputDataStructure(RetrieveArtifactPin pin){
	
	//	create output datastructure according to the pin multiplicity, assuming datstore program can handle any multiplicity
	  
	ValueSpecification lowerBound = pin.getLowerBound();
	if(lowerBound != null){
		lowerBound = (ValueSpecification)EcoreUtil.copy((EObject)(lowerBound));
	}
	ValueSpecification upperBound = pin.getUpperBound();
		if(upperBound != null){
			upperBound = (ValueSpecification)EcoreUtil.copy((EObject)(upperBound));
	}
	
	  //create output datastructure for retrieve program		
	  Type type = pin.getType();
	  //create a fake OutputPinSet so that we can use the container rule to create datastructure
	  OutputPinSet fakePinSet = ActivitiesFactory.eINSTANCE.createOutputPinSet();
	  String size = FdlUtil.getValueSpecificationString(upperBound);
	  if(size == null){
	  	size = "1";
	  }
	  fakePinSet.setName(pin.getRepository().getName() + "-" + size + FdlConstants.DATASTORE_RETRIEVE_OUT_SUFFIX); //"_R_OUT"
	  //add object pin to it
	  OutputObjectPin objPin = ActivitiesFactory.eINSTANCE.createOutputObjectPin();
	  objPin.setType(type);
	  objPin.setName(pin.getName());
	  objPin.setLowerBound(lowerBound);
	  objPin.setUpperBound(upperBound);	
	  fakePinSet.getOutputObjectPin().add(objPin);		
	  //use containerRule create the datastructure
  
	  ContainerRule cr = FdlFactory.eINSTANCE.createContainerRule();
	  cr.getSource().add(fakePinSet);
	  getRoot().getChildRules().add(cr);
	  cr.transformSource2Target();
	  DataStructure outDs = (DataStructure)cr.getTarget().get(0);	  
	 return outDs;
	}
} //RetrieveDatastoreArtifactPinRuleImpl
