/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.BlockRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FlattenRule;
import com.ibm.btools.te.ilm.heuristics.fdl.SANNestedProcessRule;


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>SAN Nested Process Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class SANNestedProcessRuleImpl extends AbstractFdlProcDefRuleImpl implements SANNestedProcessRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SANNestedProcessRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.SAN_NESTED_PROCESS_RULE;
	}

	/**
	 * Source: inputPinSet
	 * target: none
	 * Creates BlockRule or FaltternRule to map the nested StructuredActivityNode.
	 * Propagates all child targets.	  
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");

		/*
		 * Decide whether can map to FDL Block or need to flatten the SAN. 
		 * It creates either BlockRule or FlattenRule to map the InputPinSet of the SAN.
		 * 1. If there are multiple InputPinSets, must flatten.
		 * 2. If there is one InputPinSet but correlated to multiple OutputPinSets, must flatten.
		 */
	   if(!isFirstTimeExec()){
	   		//need to re-execute child rule
			((TransformationRule)getChildRules().get(0)).transformSource2Target();
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
	   }

		//first time execute this rule
	   setFirstTimeExec(false); 
	   InputPinSet inputPinSet = (InputPinSet)(getSource().get(0));
	   if(inputPinSet.getAction().getInputPinSet().size()>1){
	   		//there are multiple InputPinSet for the NestedProcess, must flatten it.
	   		createFlattenRule(inputPinSet); 
	   }else if (inputPinSet.getOutputPinSet().size()>1){
			createFlattenRule(inputPinSet);	   		
	   }else{
			createBlockRule(inputPinSet);
	   }
	   
		//propogate all child rules targets
		propagateChildTargets();
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	private void createFlattenRule(InputPinSet inputPinSet){
		
		FlattenRule fr = FdlFactory.eINSTANCE.createFlattenRule();
		getChildRules().add(fr);
		
		fr.getSource().add(inputPinSet);
		fr.transformSource2Target();		
	}
	
	private void createBlockRule(InputPinSet inputPinSet){
		BlockRule br = FdlFactory.eINSTANCE.createBlockRule();
		getChildRules().add(br);

		br.getSource().add(inputPinSet);
		br.transformSource2Target();	
		
	}

} //SANNestedProcessRuleImpl
