/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.Constraint;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.RequiredRole;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.expression.model.Expression;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.ModelPathExpression;
import com.ibm.btools.expression.model.ReferenceStep;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.OrganizationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.StaffAssignmentRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.DoneBy;
import com.ibm.btools.fdl.model.DoneByEnum;
import com.ibm.btools.fdl.model.OrganizationEnum;
import com.ibm.btools.fdl.model.Person;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Staff Assignment Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class StaffAssignmentRuleImpl extends AbstractFdlProcDefRuleImpl implements StaffAssignmentRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StaffAssignmentRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.STAFF_ASSIGNMENT_RULE;
	}

	/**
	 * SOURCE: BOM ResourceRequirement
	 * TARGET: FLD DoneBy Object
	 * Maps ResourceRequirement constraint queries to a string to be put inside
	 * the ActivityExtensionSetting attribute doneBy
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {	
		LoggingUtil.traceEntry(this, "transformSource2Target");
			
		if (isComplete())
			return true;		
		if (getSource() == null || getSource().isEmpty()){
			return false;
		}		
		ResourceRequirement thisRequirement = (ResourceRequirement)getSource().get(0);
		DoneBy doneBy = null;
		
		doneBy = createQuery(thisRequirement);
		if (doneBy!=null)
			getTarget().add(doneBy);  
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	}

	/**
	 * Creates a Staff Query
	 * @param thisRequirement - a ResourceRequirement
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */		
	private DoneBy createQuery(ResourceRequirement thisRequirement)
	{
		DoneBy doneBy = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDoneBy();
		ResourceRequirement candidateRequirement = null;	
		if(thisRequirement instanceof IndividualResourceRequirement){
			IndividualResourceRequirement individual = (IndividualResourceRequirement)thisRequirement;
			//IndividualResourcesRequirement with assigned IndividualResources of predefinedType si map to the Users by userID  
			doneBy.setPrefix(DoneByEnum.PERSON_LITERAL);	
			if (individual.getIndividualResource()!= null && StaffUtil.isPredefinedStaff(individual.getIndividualResource().getClassifier())){				
				//UserByUserID Query	
				String personUserID = retrieveUserId(individual.getIndividualResource());
				if (personUserID!=null)
					doneBy.getValue().add("'" + personUserID + "'");	
			}else if (individual.getResourceType()!= null && StaffUtil.isPredefinedStaff(individual.getResourceType())){
				if(thisRequirement.getOwnedConstraint()!= null && !thisRequirement.getOwnedConstraint().isEmpty()){
					//only known constraints are mapped to predefined queries.
					if(isKnownConstraint((Constraint)thisRequirement.getOwnedConstraint().get(0))){
						//Query suport for GroupMembers and Manager of Organization
						doneBy = createQueryFromConstraint((Constraint)thisRequirement.getOwnedConstraint().get(0));
					}else{ 
						candidateRequirement = thisRequirement;
					}
				}else{					
					//Everybody Query
					//doneBy.setPrefix(DoneByEnum.ALL_LITERAL);			
				}
			}
		//Only RequiredRoles with predefined specified ResourceType are mapped to Staff Queries		
		}else if(thisRequirement instanceof RequiredRole && thisRequirement.getResourceType() != null && StaffUtil.isPredefinedStaff(thisRequirement.getResourceType())){
			//Role Members Query			
			doneBy.setPrefix(DoneByEnum.MEMBEROFROLE_LITERAL);	
			if (thisRequirement.getResourceType() != null && StaffUtil.isPredefinedStaff(thisRequirement.getResourceType()) 
				&& ((RequiredRole)thisRequirement).getRole()!=null){
				String roleName = ((RequiredRole)thisRequirement).getRole().getName();
				if (roleName!=null)
					doneBy.getValue().add("'" + FdlUtil.getResourceOrganizationName(roleName) + "'");	
			}
		}
		if(candidateRequirement != null){			
			LoggingUtil.logWarning(MessageKeys.UNKNOWN_FDL_STAFF_ALLOCATION_CONSTRAINT);
			return null; // Ray May 07,04 - if it is an unknown staff query, return null such that no done_by will be created
		}	
		if (doneBy.getValue().isEmpty())
			return null;
		else
			return doneBy;
	}
	/**
	 * Returns true if the input Constraint is a known predefined SSS query constraint
	 * @param constraint BOM Constraint
	 * @return boolean
	 */
	private boolean isKnownConstraint(Constraint constraint){
		if(constraint.getSpecification() instanceof StructuredOpaqueExpression){
			StructuredOpaqueExpression bomExpression = (StructuredOpaqueExpression)constraint.getSpecification();
			Expression expression = bomExpression.getExpression();
			if(expression instanceof FunctionExpression){
				FunctionExpression function = (FunctionExpression)expression;
				String functionID = function.getDefinition().getFunctionID();
				if(StaffUtil.getPredefinedFDLResourceRequirementConstraints().contains(functionID)){
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Retrieves the Person ID from the resource
	 * @param resource - a IndividualResource
	 * @return String - string user ID
	 */			
	private String retrieveUserId(IndividualResource resource)
	{
		String userId = null;
		Person person = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createPerson();
		List slots = resource.getSlot();		
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();	
				if (thisSlot.getDefiningFeature().getName()!=null){					
					if(thisSlot.getDefiningFeature().getName().equals("personId")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							userId =((LiteralString)thisSlot.getValue().get(0)).getValue();
						}
						break;
					}
				}
			}
		}
		if (userId!=null)
			return FdlUtil.getResourceOrganizationName(userId).toUpperCase();
		else
			return null;
	}

	/**
	 * Calls the corresponding createQuery method from the constraint
	 * @param constraint - a BOM Constraint
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */		
	private DoneBy createQueryFromConstraint(Constraint constraint){
		DoneBy doneBy = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDoneBy();
		StructuredOpaqueExpression resReqConstraint = (StructuredOpaqueExpression)constraint.getSpecification();
		FunctionExpression function = (FunctionExpression)resReqConstraint.getExpression();
		String functionID = function.getDefinition().getFunctionID();				
		if(functionID.equals(StaffUtil.GROUP_MEMBERS_FUNCTION)){
			doneBy = createGroupMembersQuery(function);
		}else if(functionID.equals(StaffUtil.GROUP_SEARCH_FUNCTION)){
			doneBy = createGroupSearchQuery(function);
		}else if (functionID.equals(StaffUtil.MGR_OF_ORG_FUNCTION)){
			doneBy = createMgrOfOrgQuery(function);
		}
		return doneBy;
	}

	/**
	 * Creates a Group Members Query
	 * @param function - a  BOM FunctionExpression
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */		
	private DoneBy createGroupMembersQuery(FunctionExpression function)
	{
		DoneBy doneBy = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDoneBy();		
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();				
				String argumentID = functionArgument.getArgumentID();		
				if(argumentID.equals(StaffUtil.GROUP_MEMBERS_NAME_ARG)){
					doneBy.setPrefix(DoneByEnum.ORGANIZATION_LITERAL);					
				}
				if(doneBy.getPrefix()!= null){
					Expression valueParameterExpression = functionArgument.getArgumentValue();
					if(valueParameterExpression instanceof StringLiteralExpression){
						doneBy.getValue().add("'" + ((StringLiteralExpression)valueParameterExpression).getStringSymbol() + "'");
						doneBy.setOption(OrganizationEnum.MEMBERS_ONLY_LITERAL);
					}else if (valueParameterExpression instanceof ModelPathExpression){
						ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
						setOrgDoneBy(pathExpression, doneBy, true);
						/*
						//get the last step
						ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
						EObject eObject = lastStep.getReferencedObject();
						if(eObject instanceof OrganizationUnit){
							//Should be an OrganizationUnit
							List orgSlots = ((OrganizationUnit)eObject).getSlot();							
							if(orgSlots != null && !orgSlots.isEmpty()){
								Iterator orgIt = orgSlots.iterator();
								while (orgIt.hasNext()){		
									Slot thisOrgSlot = (Slot)orgIt.next();					
									if (thisOrgSlot.getDefiningFeature().getName()!=null){					
										if(thisOrgSlot.getDefiningFeature().getName().equals("organizationId")){
											doneBy.getValue().add("'" + ((LiteralString)thisOrgSlot.getValue().get(0)).getValue() + "' ");
											doneBy.setOption(OrganizationEnum.MEMBERS_ONLY_LITERAL);	
											break;								
										}
									}
								}
							}
						}*/
					}					
				}
			}
		}
		return doneBy;
	}

	/**
	 * Creates a Group Search Query
	 * @param function - a  BOM FunctionExpression
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */		
	private DoneBy createGroupSearchQuery(FunctionExpression function)
	{
		DoneBy doneBy = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDoneBy();
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();				
				String argumentID = functionArgument.getArgumentID();		
				if(argumentID.equals(StaffUtil.GROUP_SEARCH_GROUPID_ARG)){
					doneBy.setPrefix(DoneByEnum.ORGANIZATION_LITERAL);					
				}
				if(doneBy.getPrefix()!= null){
					Expression valueParameterExpression = functionArgument.getArgumentValue();
					if(valueParameterExpression instanceof StringLiteralExpression){
						doneBy.getValue().add("'" + ((StringLiteralExpression)valueParameterExpression).getStringSymbol() + "' ");
						doneBy.setOption(OrganizationEnum.MEMBERS_ONLY_LITERAL);		
					}							
				}
			}
		}
		return doneBy;
	}

	/**
	 * Creates a Manager of Organization Query
	 * @param function - a  BOM FunctionExpression
	 * @return DoneBy - FDL Done By, this is a query specified in Activity Extension Setting
	 */	
	private DoneBy createMgrOfOrgQuery(FunctionExpression function)
	{
		DoneBy doneBy = com.ibm.btools.fdl.model.ModelFactory.eINSTANCE.createDoneBy();
		List arguments = function.getArguments();
		if(arguments != null && !arguments.isEmpty()){
			Iterator it = arguments.iterator();
			while(it.hasNext()){
				FunctionArgument functionArgument = (FunctionArgument)it.next();				
				String argumentID = functionArgument.getArgumentID();		
				if(argumentID.equals(StaffUtil.MGR_OF_ORG_ORGANIZATION_ARG)){
					doneBy.setPrefix(DoneByEnum.MANAGEROFORGANIZATION_LITERAL);					
				}
				if(doneBy.getPrefix()!= null){
					Expression valueParameterExpression = functionArgument.getArgumentValue();
					if(valueParameterExpression instanceof StringLiteralExpression){
						doneBy.getValue().add("'" + ((StringLiteralExpression)valueParameterExpression).getStringSymbol() + "'");
					}else if (valueParameterExpression instanceof ModelPathExpression){
						
						ModelPathExpression pathExpression = (ModelPathExpression)valueParameterExpression;
						setOrgDoneBy(pathExpression, doneBy, false);
						/*
						if(eObject instanceof OrganizationUnit){
							//Should be an OrganizationUnit							
							List resourceSlots = ((OrganizationUnit)eObject).getSlot();		
							if(resourceSlots != null && !resourceSlots.isEmpty()){
								Iterator resourceIt = resourceSlots.iterator();
								while (resourceIt.hasNext()){		
									Slot thisResourceSlot = (Slot)resourceIt.next();					
									if (thisResourceSlot.getDefiningFeature().getName()!=null){					
										if(thisResourceSlot.getDefiningFeature().getName().equals("manager")){
											String value = null;								
											IndividualResource resource = (IndividualResource)((InstanceValue)thisResourceSlot.getValue().get(0)).getInstance();
											List managerSlots = resource.getSlot();		
											if(managerSlots != null && !managerSlots.isEmpty()){
												Iterator managerIt = managerSlots.iterator();
												while (managerIt.hasNext()){		
													Slot thisManagerSlot = (Slot)managerIt.next();					
													if (thisManagerSlot.getDefiningFeature().getName()!=null){					
														if(thisManagerSlot.getDefiningFeature().getName().equals("personId")){
															value =((LiteralString)thisManagerSlot.getValue().get(0)).getValue();									
														}
													}
												}
											}
											doneBy.getValue().add("'" + value + "'");									
										}
									}
								}
							}		
						}*/
					}					
				}
			}
		}		
		return doneBy;
	}

	private void setOrgDoneBy(ModelPathExpression pathExpression, DoneBy doneBy, boolean isOrgQuery){
		
		//get the last step
		if (pathExpression.getSteps()!=null && !(pathExpression.getSteps().isEmpty()) ){
			ReferenceStep lastStep = (ReferenceStep)pathExpression.getSteps().get(pathExpression.getSteps().size()-1);
			EObject eObject = lastStep.getReferencedObject();
			if(eObject instanceof OrganizationUnit){
				//Should be an OrganizationUnit
				List orgSlots = ((OrganizationUnit)eObject).getSlot();							
				if(orgSlots != null && !orgSlots.isEmpty()){
					Iterator orgIt = orgSlots.iterator();
					while (orgIt.hasNext()){		
						Slot thisOrgSlot = (Slot)orgIt.next();					
						if (thisOrgSlot.getDefiningFeature().getName()!=null){					
							if(thisOrgSlot.getDefiningFeature().getName().equals("organizationID")){
								if (thisOrgSlot.getValue()!=null && !thisOrgSlot.getValue().isEmpty()){
									String orgID = ((LiteralString)thisOrgSlot.getValue().get(0)).getValue();
									if (orgID==null)
										return;
									doneBy.getValue().add("'" + FdlUtil.getResourceOrganizationName(orgID) + "' ");
									if (isOrgQuery)
										doneBy.setOption(OrganizationEnum.MEMBERS_ONLY_LITERAL);	
									//Check to see if Organization is created
									findOrganization((OrganizationUnit)eObject, thisOrgSlot.getDefiningFeature().getName());										
								}	
								return;											
							}
						}
					}
				}
			}
		}
	}
	
	private void findOrganization(OrganizationUnit orgUnit, String orgID){
		
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), OrganizationRule.class, orgUnit); 
				
		if (rule==null){
			//check if organization is of predefined type
			if (orgUnit!=null && StaffUtil.isPredefinedOrganization(orgUnit.getClassifier())){
				//create Organization			
				OrganizationRule orgRule = FdlFactory.eINSTANCE.createOrganizationRule();
				orgRule.getSource().add(orgUnit);
				getRoot().getChildRules().add(orgRule);							
				orgRule.transformSource2Target();					
			}

		}
	}



} //StaffAssignmentRuleImpl
