/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.fdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.InternalEList;

import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.StoreArtifactAction;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Repository;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.StoreDatastoreArtifactActionRule;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.fdl.model.ProgramActivity;

/**
 * <!-- begin-user-doc -->
 * Source: BOM StoreArtifactAction (referencing BOM Datastore only)
 * Target: BOM ProgramActivity (referencing store to datastore program)
 * 
 * Maps a StoreArtifactAction to a store to datastore program activity.
 * Connects the mapped activity with incoming and outgoing connectors.
 * 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class StoreDatastoreArtifactActionRuleImpl extends StoreDatastoreArtifactRuleImpl implements StoreDatastoreArtifactActionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StoreDatastoreArtifactActionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return FdlPackage.Literals.STORE_DATASTORE_ARTIFACT_ACTION_RULE;
	}

	/**
	 * Source: BOM StoreArtifactAction (referencing BOM Datastore only)
	 * Target: BOM ProgramActivity (referencing store to datastore program)
	 * 
	 * Maps a StoreArtifactAction to a FDL ProgramActivity for storing to the datastore.
	 * Connects the activity with incoming and outgoing connectors.
	 *  
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target() {
		LoggingUtil.traceEntry(this, "transformSource2Target");
		
		InputPinSet inputSet = (InputPinSet)getSource().get(0);
		OutputPinSet outputSet = (OutputPinSet)inputSet.getOutputPinSet().get(0);
		StoreArtifactAction action = (StoreArtifactAction)inputSet.getAction();
	
		if(!(isFirstTimeExec())){
			LoggingUtil.traceExit(this, "transformSource2Target","re-execute");
			return true;	
		}

		//first time execute this rule
		setFirstTimeExec(false);
		
		ProgramActivity pa = createProgramActivity(action);
		//do InputPinSet connection
		setTargetOfConnectors(inputSet, pa);
			
		createBranchRuleIfAny(inputSet);
		//do OutputPinSet connection
		setSourceOfConnectors(outputSet, pa);
		//all targets propagate to parent
		propagateChildTargets();

		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
		}
		
	/**
	 * Creates the ProgramActivity to represent the action.
	 * Initializes input container by retrieving values from the action's attributes.
	 * 
	 * @param action The store artifact action
	 * @return ProgramActivity - The program activity created
	 */
	private ProgramActivity createProgramActivity(StoreArtifactAction action){
		
		Repository datastore = action.getRepository();
		
		//	get inputObjectPin, assume that is the one and only one reference the datastore
		//InputObjectPin objPin = (InputObjectPin)action.getInputObjectPin().get(0);
		InputObjectPin objPin = (InputObjectPin)action.getValue();
		ValueSpecification lowerBound = objPin.getLowerBound();
		if(lowerBound != null){
			lowerBound = (ValueSpecification)EcoreUtil.copy((EObject)(lowerBound));
		}
		ValueSpecification upperBound = objPin.getUpperBound();
		if(upperBound != null){
			upperBound = (ValueSpecification)EcoreUtil.copy((EObject)(upperBound));
		}
		String paName = getNameRegistry().getActivityName(action);	
		  
		ProgramActivity pa = createStoreDatastoreProgramActivity(
			datastore, 
			paName,
			lowerBound,
			upperBound,
			action.getIsInsert(),
			action.getAtBeginning(),
			action.getOverflow()
			 );	    	
		
		String dsc = FdlUtil.getFullName(action);
		pa.setDescription(getNameRegistry().getDescription(dsc));	
		
		getTarget().add(pa);
		return pa;
	
	
	}

} //StoreDatastoreArtifactActionRuleImpl
