package com.ibm.btools.te.ilm.heuristics.fdl.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.Element;
import com.ibm.btools.bom.model.artifacts.LiteralBoolean;
import com.ibm.btools.bom.model.artifacts.LiteralDuration;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralNull;
import com.ibm.btools.bom.model.artifacts.LiteralReal;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.LiteralTime;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.fdl.model.BasicType;
import com.ibm.btools.fdl.model.BasicTypeEnum;
import com.ibm.btools.fdl.model.DataType;
import com.ibm.btools.fdl.model.ModelFactory;

/**
 * This utility class provides utility functions related to FDL transformation.
 */
public class FdlUtil {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	public static boolean bOptimizeSingleObjPinContainer=true;
	public static boolean bApplyOptimization=true;
		
	static BasicType floatType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType doubleType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType stringType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType integerType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType longType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType shortType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType booleanType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType byteType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType dateType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType timeType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType dateTimeType = ModelFactory.eINSTANCE.createBasicType();
	static BasicType durationType = ModelFactory.eINSTANCE.createBasicType();	
	
	static String supportedPredefinedDataTypes[]= {
		"Boolean",
		"String",
		"Long",
		"Integer",
		"Short",
		"Byte",
		"Double",
		"Float",		
		"Date",		
		"Time",		
		"DateTime",		
		"Duration"		
	};

	static final char[] RESOURCE_ORG_NAME_CHAR_NOT = 
					{'*', '?', '"', ';', ':', '.'};
	static final int MAX_RESOURCE_ORG_NAME_LENGTH = 32;

	static HashMap predefinedDataTypesRegistry=new HashMap();
	
	static {
		stringType.setType(BasicTypeEnum.STRING_LITERAL);
		floatType.setType(BasicTypeEnum.FLOAT_LITERAL);
		doubleType.setType(BasicTypeEnum.FLOAT_LITERAL);
		integerType.setType(BasicTypeEnum.LONG_LITERAL);			
		longType.setType(BasicTypeEnum.LONG_LITERAL);			
		shortType.setType(BasicTypeEnum.LONG_LITERAL);			
		booleanType.setType(BasicTypeEnum.LONG_LITERAL);					
		byteType.setType(BasicTypeEnum.LONG_LITERAL);					
		dateType.setType(BasicTypeEnum.STRING_LITERAL);
		timeType.setType(BasicTypeEnum.STRING_LITERAL);
		dateTimeType.setType(BasicTypeEnum.STRING_LITERAL);
		durationType.setType(BasicTypeEnum.STRING_LITERAL);
	
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[0],booleanType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[1],stringType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[2],longType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[3],integerType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[4],shortType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[5],byteType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[6],doubleType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[7],floatType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[8],dateType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[9],timeType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[10],dateTimeType);
		predefinedDataTypesRegistry.put(supportedPredefinedDataTypes[11],durationType);
	}
	
	
/**
 * Returns the canonical name for the given NamedElement by prepending the name of
 * the owner object up through the ownership hierarchy. Names are separated by '_'. 
 * 
 * @param element The <code>NamedElement</code> to generate the name for
 * @return String - The canonical name of <code>element</code>
 * @see com.ibm.btools.te.ilm.heuristics.helper.BomUtils#getCanonicalName(NamedElement, String)
 */
public static String getCanonicalName(NamedElement element){
	return BomUtils.getCanonicalName(element, "_");
}
	

/**
 * Returns true if the <code>typeName</code> can be converted to basic data type.
 * 
 * @param typeName
 * @return
 */
public static boolean isConvertibleToBasicDataType (String typeName) {
	return ((getPredefinedDataType(typeName)!=null)&&(getPredefinedDataType(typeName)instanceof BasicType))?true:false;
}

/**
 * Returns true if the <code>typeName</code> is supported predefined data type.
 * @param typeName
 * @return
 */
public static boolean isSupportedPredefinedDataType (String typeName) {
	return (getPredefinedDataType(typeName)!=null)?true:false;
}

/**
 * Returns the predefined data type instance for the type name.
 * @param typeName
 * @return
 */
public static DataType getPredefinedDataType(String typeName) {
	return (DataType)predefinedDataTypesRegistry.get(typeName);
}
	
/**
 * Gets full name for the <code>pkgElement</code>.
 * @param pkgElement
 * @return
 */
public static String getFullName(PackageableElement pkgElement)
{
	String name = null;
	if (pkgElement instanceof Type) {
		Type type = (Type) pkgElement;
		name = getFullPackageName(type);
	} else {
		EObject container = pkgElement.eContainer();
		Activity belongingActivity = null;
		while (!(container instanceof Activity)) {
			container = container.eContainer();
		}
		name = pkgElement.getName()+"_"+ getFullPackageName((Activity)container);
	}
	return name;
}
/**
 * Gets the full name of the <code>namedElement</code>.
 * The full name including its full package name.
 * @param namedElement
 * @return
 */	
	public static String getFullName(NamedElement namedElement)
		{
			String name = null;
			name = namedElement.getName();
			NamedElement container = (NamedElement)namedElement.eContainer();
			if(container == null){
				return name;
			}
			
			while (!(container instanceof Activity)) {
				name = container.getName() + "_" + name;
				container = (NamedElement)container.eContainer();
			}
			name = getFullPackageName((Activity)container) + "_" + name;
			
			return name;
		}


	/**
	 * Returns the full package name for the PackageableElement by prepending the owning package name 
	 * up through the ownership hierarchy. Names are seperated by "_".
	 * @param pkgElement
	 * @return
	 */
	private static String getFullPackageName(PackageableElement pkgElement)
	{
		String name=pkgElement.getName();
		Package belongingPackage = pkgElement.getOwningPackage();
		while (belongingPackage != null && !ProcessUtil.isRootPackage(belongingPackage)) {
			name = belongingPackage.getName() + "_" + name;
			belongingPackage = belongingPackage.getOwningPackage();
		}
		return name;
	}
	

	/**
	 * Replaces the given String with its first letter lower case 
	 */
	public static String getLowerCase(String s){
			if(Character.isUpperCase(s.charAt(0))){		
				char c = Character.toLowerCase(s.charAt(0));
				s = String.valueOf(c) + s.substring(1,s.length());
			}
			return s;
	    
	}	
	
	
	/**
	 * Gets the value from <code>valueSpec</code>.
	 * If the <code>valueSpec</code> is not a LiteralValueSpecification, log warning and return null.
	 * @param valueSpec
	 * @return
	 */
	public static String getValueSpecificationString(ValueSpecification valueSpec){
		if(valueSpec == null){
			return null;
		}
		
		String valueString = null;
		//we only support LiteralValueSpecification
		if(valueSpec instanceof LiteralString){
			 valueString = ((LiteralString)valueSpec).getValue();
		}else if(valueSpec instanceof LiteralUnlimitedNatural){
			//shoud just return whatever it is specified by user
			valueString = ((LiteralUnlimitedNatural)valueSpec).getValue();
		
		}else if(valueSpec instanceof LiteralInteger){	
				valueString = ((LiteralInteger)valueSpec).getValue().toString();		
		}else if(valueSpec instanceof LiteralReal){
				valueString = ((LiteralReal)valueSpec).getValue().toString();
		}else if(valueSpec instanceof LiteralBoolean){
				Boolean b = ((LiteralBoolean)valueSpec).getValue();
				valueString = b.booleanValue()? "1" : "0";
		}else if(valueSpec instanceof LiteralNull){
			//return null to make the same effect as the structure member doesn't have any value
				valueString = null;	
		}else if(valueSpec instanceof LiteralTime){
				valueString = ((LiteralTime)valueSpec).getValue().toString();
		}else if(valueSpec instanceof LiteralDuration){
				valueString = ((LiteralDuration)valueSpec).getValue();		
		}else{
			String params[]={valueSpec.getType().getName()};
			LoggingUtil.logWarning(MessageKeys.CONSTANT_NOT_SUPPORTED,params,null);		
		} 
		
		return valueString;
}

/**
 * Puts the Fdl name registry in context
 * @param context		the transformation context
 * @param nameRegistry	the Fdl name registry 
 */
public static void putFdlNameRegistry(TransformationContext context, FdlNameRegistryImpl nameRegistry){
	
	context.put(FdlConstants.FDL_NAME_REGISTRY_KEY, nameRegistry);
	
}

/**
 * Gets the Fdl name registry in context.
 * Creates one and put on in context is not found
 * @param context	the transformation context
 * @return FdlNameRegistry - the name registry in context
 */
public static FdlNameRegistry getFdlNameRegistry(TransformationContext context){
	FdlNameRegistryImpl registry = (FdlNameRegistryImpl)context.get(FdlConstants.FDL_NAME_REGISTRY_KEY);
	if (registry == null) {
		registry = new FdlNameRegistryImpl();
		putFdlNameRegistry(context,registry);
	}
	return registry;
}

/**
 * Retrieves the ownedAttribute for different types of 'Classifier'
 * @param classifier
 * @return list of owned attributes
 */
public static List getOwnedAttribute(com.ibm.btools.bom.model.artifacts.Classifier classifier) {
	if (classifier instanceof com.ibm.btools.bom.model.artifacts.Class) {
		return ((com.ibm.btools.bom.model.artifacts.Class)classifier).getOwnedAttribute();
	} else if (classifier instanceof BulkResourceType) {
		return ((BulkResourceType)classifier).getOwnedAttribute();
	} else if (classifier instanceof IndividualResourceType) {
		return ((IndividualResourceType)classifier).getOwnedAttribute();
	} else if (classifier instanceof OrganizationUnitType) {
		return ((OrganizationUnitType)classifier).getOwnedAttribute();
	} else if (classifier instanceof LocationType) {
		return ((LocationType)classifier).getOwnedAttribute();
	} else if (classifier instanceof Signal) {
		return ((Signal)classifier).getOwnedAttribute();
	}
	return null;
}

public static String transformDescription(Element element) {
	Comment desc = BomUtils.getDescription(element);
	if (desc!=null) {
		return desc.getBody();
	}
	return null;
}
	
public static String transformDocumentation(Element element) {
	List comments = BomUtils.getAnnotatedComments(element);
	if (comments.size() > 0) {
		StringBuffer sb = new StringBuffer();
		for (int i=0; i<comments.size(); i++) {
			Comment comment = (Comment)comments.get(i);
			sb.append(comment.getBody());
			if (i!=(comments.size()-1)) {
				sb.append("\r\n");
			}
		}
		return sb.toString();
	}		
	return null;
}

/**
 *  Returns the resource/organization memeber name for the string name.
 *  strip reserved characters and make sure it's within the maximum length
 * 
 * @param oldName	name of resource/organization	
 * @return String
 */		
public static String getResourceOrganizationName(String oldName){
	String name = oldName;
	name = name.trim();
	if (name!=null){
		name = stripChars(name, RESOURCE_ORG_NAME_CHAR_NOT);
		if(name.length()> MAX_RESOURCE_ORG_NAME_LENGTH){
			//trucate to make it in the range
			name = name.substring(0,  MAX_RESOURCE_ORG_NAME_LENGTH);					
		}		
		return name;	
	}
	else{
		return null;
	}
	
}

private static String stripChars(String name, char[] strippedChars){
		StringTokenizer st = new StringTokenizer(name, new String(strippedChars));
		StringBuffer sb = new StringBuffer();
		while(st.hasMoreTokens()){
			sb.append(st.nextToken());
		}
		return sb.toString();		
	}

public static boolean isPredefinedProcessDataStructure(Classifier classifier) {
	String className = classifier.getName();
	if (className.equals("FDL Predefined Data Structure") ||
		className.equals("_PROCESS_INFO") ||
		className.equals("_ACTIVITY_INFO")
	) {
		Package pkg = classifier.getOwningPackage();
		if (pkg != null) {
			if (pkg.getName().equals("Business items")) {
				Package rootPkg = pkg.getOwningPackage();
				if (ProcessUtil.isRootPackage(rootPkg)) {
					return true;
				}
			}
		}
	}
	return false;
}

public static boolean isPinSetDataStructureOptimizable(PinSet ps) {
	List objPins;
	if (ps instanceof InputPinSet)
	{
		objPins = ((InputPinSet)ps).getInputObjectPin();		
	} else {
		objPins = ((OutputPinSet)ps).getOutputObjectPin();					
	}
	if (FdlUtil.bOptimizeSingleObjPinContainer && 
					objPins.size()==1 && 
					((ObjectPin)objPins.get(0)).getType()!=null && 
					!FdlUtil.isConvertibleToBasicDataType(((ObjectPin)objPins.get(0)).getType().getName()) &&
					getMultiplicity(((ObjectPin)objPins.get(0)).getUpperBound()) <=1
					) {
						return true;
					}
	return false;
}

/**
 * Obtains the multiplicity in a String format 
 * @param mul
 * @return multiplicity
 */
private static String getMultiplicityString(ValueSpecification mul)
{
	if (mul instanceof LiteralUnlimitedNatural)
	{
		return ((LiteralUnlimitedNatural)mul).getValue();
	} else {
		return ((LiteralInteger)mul).getValue().toString();
	}
}
	
/**
 * Obtains the multiplicity in an int format
 * @param lun
 * @return multiplicity
 */
public static int getMultiplicity(ValueSpecification lun)
{
	// if no multiplicity specify, default to 1
	if (lun == null)
		return 1;
	String val = getMultiplicityString(lun);
	// if unbound or zero multiplicity is found, default to 1
	if (val.toLowerCase().equals("n") || val.equals("0") || val.toLowerCase().equals("*") )
	{
		val="1";
	}
	return Integer.parseInt(val);
}
}