package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Message;

import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.actions.CallOperationAction;
import com.ibm.btools.bom.model.processes.actions.ControlAction;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.actions.Fork;
import com.ibm.btools.bom.model.processes.actions.Join;
import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.actions.RepositoryAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ConnectableNode;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.FlowFinalNode;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.TerminationNode;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.bom.model.services.Activity;
import com.ibm.btools.bom.model.services.InputParameterSet;
import com.ibm.btools.bom.model.services.OutputParameterSet;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.VariableRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.heuristics.xsd.ElementDeclarationRule;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.wbit.bpel.BPELVariable;


/**
 * This class provides methods that aid the mapping of BPEL to BOM.
 */
public class BomHelper {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	/* The singleton instance of this class */
	private static BomHelper singleton = null;

	/* The instance of the merge point finder that is used */
	private MergePointFinder fMPFinder = null;

	/* The cache that maps branch points to merge points  */
	private Map fBranchMergeCache;

	/* The cache that maps branch points to merge regions */
	private Map fBranchRegionCache;

	/* Ray: Need to maintain a list of visited branch point to avoid infinite loop in the case of cycle */
	private List visitedEBranchPoint;
	private List visitedCBranchPoint;
	 
	/**
	 * Private constructor for singleton pattern 
	 */
	private BomHelper(){
		this.fBranchMergeCache = new HashMap();
		this.fBranchRegionCache = new HashMap();
		this.fMPFinder = new MergePointFinder();
		this.visitedEBranchPoint = new ArrayList(); // Ray
		this.visitedCBranchPoint = new ArrayList(); // Ray
	}
	
	// Ray: For each transformation session should start with new caches 
	public void reset() {
		fBranchMergeCache = new HashMap();
		fBranchRegionCache = new HashMap();
		fMPFinder = new MergePointFinder();		
		visitedEBranchPoint = new ArrayList(); // Ray
		visitedCBranchPoint = new ArrayList(); // Ray
	}
	
	/**
	 * The singleton accessor method
	 * @return BomHelper - The singleton instance of this class
	 */
	public static BomHelper getInstance(){
		if (singleton == null){
			singleton = new BomHelper();
		}
		return singleton;
	}
	
	
	/**
	 * Returns whether the given output set constitutes a concurrent branch point.
	 * @param branchPoint - The output set that may or may not be a branch point
	 * @return boolean - True if the output set is a branch point, false otherwise
	 */
	public boolean isCBranch(OutputPinSet branchPoint){
		if (branchPoint.getOutputControlPin().size() + branchPoint.getOutputObjectPin().size() > 1){
			// Ray: Additional check-if more than one pin had outgoing edges and that they don't connect to a flow final or termination node, then it is a CBranch
//			Iterator it = branchPoint.getOutputControlPin().iterator();
//			OutputControlPin outCtrlPin;
//			int validCBranchPinCount=0;
//			while (it.hasNext() && validCBranchPinCount<2) {
//				outCtrlPin = (OutputControlPin)it.next();
//				if (outCtrlPin.getOutgoing()!=null) {
//					if (outCtrlPin.getOutgoing().getTarget()!=null && 
//						!((outCtrlPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
//						 (outCtrlPin.getOutgoing().getTarget() instanceof TerminationNode))
//						) {
//							validCBranchPinCount++;
//						}
//				}
//			}
//			it = branchPoint.getOutputObjectPin().iterator();
//			OutputObjectPin outObjPin;
//			while (it.hasNext() && validCBranchPinCount<2) {
//				outObjPin = (OutputObjectPin)it.next();
//				if (outObjPin.getOutgoing()!=null) {
//					if (outObjPin.getOutgoing().getTarget()!=null && 
//						!((outObjPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
//						 (outObjPin.getOutgoing().getTarget() instanceof TerminationNode))
//						) {
//							validCBranchPinCount++;
//						}
//				}
//			}		
//			if (validCBranchPinCount < 2) {
//				return false;	
//			}
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Returns whether the given named element qualifies as a possible concurrent merge point.
	 * Note that if this method returns true, it does not indicate that the named element is a merge
	 * point for any particular branch point.  It only indicates that the named element *could* be
	 * a merge point.
	 * 
	 * @param possibleMergePoint - The NamedElement that may or may not be a merge point.  This object
	 * must be an instance of InputPinSet, or an OutputPinSet owned by a non-task StructuredActivityNode. 
	 * @return boolean - True if the NamedElement is a potential merge point, false otherwise
	 */
	public boolean isCMerge(NamedElement possibleMergePoint){
		if (possibleMergePoint instanceof Action 
				&& (BomUtils.isProcess((Action)possibleMergePoint) 
					|| possibleMergePoint instanceof LoopNode)){
		    
		    /* An output set on a non-task SAN is always a possible merge point */
		    return true;

		} else if (possibleMergePoint instanceof InputPinSet 
				   && (((InputPinSet)possibleMergePoint).getInputControlPin().size() 
					  + ((InputPinSet)possibleMergePoint).getInputObjectPin().size()) > 1){
			
			/* An input set with more than one pin is always a potential merge point */
			return true;

		} else {
			return false;
		}

	}

	/**
	 * Returns whether the given named element is the full concurrent merge point for the given branch point.
	 * If branchPoint is not actually a branch point, false is returned.
	 * 
	 * @param branchPoint - The branch point in question 
	 * @param possibleMergePoint - The NamedElement that may or may not be the full merge point for the
	 * given branch point.  This object must be an instance of InputPinSet, or an OutputPinSet owned 
	 * by a non-task StructuredActivityNode.
	 * @return boolean - True if the NamedElement is the full merge point, false otherwise
	 */
	public boolean isCMerge(OutputPinSet branchPoint, NamedElement possibleMergePoint){
		if (!isCBranch(branchPoint)){
			return false;
		}
		
		if (!isCMerge(possibleMergePoint)){
			return false;
		}
		
		NamedElement realMergePoint = findConcurrentMergePoint(branchPoint);
		
		return (realMergePoint == possibleMergePoint);

	}	
	
	
	/**
	 * Returns whether the given input set constitutes an exclusive branch point.
	 * @param branchPoint - The input set that may or may not be an exclusive branch point
	 * @return boolean - True if the input set is an exclusive branch point, false otherwise
	 */
	public boolean isEBranch(InputPinSet branchPoint){
		if (branchPoint.getOutputPinSet().size()==0) {
			// Ray: if the branch leads to no OutputPinSet (no association to any OutputPinSet)
			// then assume the branch points lead to all OutputPinSet of the containing action
			List ops = BomUtils.getCorrelatedPinSets(branchPoint);
			if (ops.size()>1) {
				return true;
			} else {
				return false;
			}
		} else
		if (branchPoint.getOutputPinSet().size() > 1){
			// Ray: Unfortunately we cannot do the optimization like the commented code below for
			// EBP because we have to make sure each EBP (outputPinSet) will be examined even if there is only one
			// branch has out going edges to non termination/flow final nodes
			
//			// Ray: Additional check - if at least one pin in the outputPinSet has outgoing edge and that edge is not connected
//			// to a flow final or termination node, that means it is qualified as a branching outputPinSet
			//
			////AR added back to support the case where we have a BS with multiple PinSets with no outgoing links. This is 
			//not really a branch Point
			int numOfQualifiedBranchingPinSet=0;
			Iterator itPinSet = branchPoint.getOutputPinSet().iterator();
			OutputPinSet outPinSet;
			while (itPinSet.hasNext() && numOfQualifiedBranchingPinSet < 2) {
				outPinSet = (OutputPinSet)itPinSet.next();
				Iterator it = outPinSet.getOutputControlPin().iterator();
				OutputControlPin outCtrlPin;
				int validBranchPinCount=0;
				while (it.hasNext() && validBranchPinCount<1) {
					outCtrlPin = (OutputControlPin)it.next();
					if (outCtrlPin.getOutgoing()!=null) {
						if (outCtrlPin.getOutgoing().getTarget()!=null && 
							!((outCtrlPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
							 (outCtrlPin.getOutgoing().getTarget() instanceof TerminationNode))
							) {
								validBranchPinCount++;
							}
					}
				}
				it = outPinSet.getOutputObjectPin().iterator();
				OutputObjectPin outObjPin;
				while (it.hasNext() && validBranchPinCount<1) {
					outObjPin = (OutputObjectPin)it.next();
					if (outObjPin.getOutgoing()!=null) {
						if (outObjPin.getOutgoing().getTarget()!=null && 
							!((outObjPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
							 (outObjPin.getOutgoing().getTarget() instanceof TerminationNode))
							) {
								validBranchPinCount++;
							}
					}
				}								
				if (validBranchPinCount > 0) {
					numOfQualifiedBranchingPinSet++;	
				}
			}
			if (numOfQualifiedBranchingPinSet == 0) {
				return false;			
			}
			return true;
		} else {
			// Ray: The Decision transformation heavily relies on the fact that we always need to treat
			// it as an Exclusive branch.  In the case that a Decision has only 1 branch (i.e. 1 outputPinSet)
			// we still want to return true such that the Decision transformation will be handled as an
			// Exclusive branch. Otherwise, the decision transformation will fall aparts ...
			// Until we find a better place to handle Decision with single branch, we have to put the fix in
			// here to treat as a special case for Exclusive Branch detection.
			if (branchPoint.getAction() instanceof Decision) {
				return true;
			}
			return false;
		}
	}

//	private int countValidBranchPin (OutputPinSet outPinSet) {
//		Iterator it = outPinSet.getOutputControlPin().iterator();
//		OutputControlPin outCtrlPin;
//		int validBranchPinCount=0;
//		while (it.hasNext() && validBranchPinCount<2) {
//			outCtrlPin = (OutputControlPin)it.next();
//			if (outCtrlPin.getOutgoing()!=null) {
//				if (outCtrlPin.getOutgoing().getTarget()!=null && 
//					!((outCtrlPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
//					 (outCtrlPin.getOutgoing().getTarget() instanceof TerminationNode))
//					) {
//						validBranchPinCount++;
//					}
//			}
//		}
//		it = outPinSet.getOutputObjectPin().iterator();
//		OutputObjectPin outObjPin;
//		while (it.hasNext() && validBranchPinCount<2) {
//			outObjPin = (OutputObjectPin)it.next();
//			if (outObjPin.getOutgoing()!=null) {
//				if (outObjPin.getOutgoing().getTarget()!=null && 
//					!((outObjPin.getOutgoing().getTarget() instanceof FlowFinalNode) ||
//					 (outObjPin.getOutgoing().getTarget() instanceof TerminationNode))
//					) {
//						validBranchPinCount++;
//					}
//			}
//		}			
//	}
	
	/**
	 * Returns whether the given NamedElement qualifies as a possible exclusive merge point.
	 * Note that if this method returns true, it does not indicate that the named element is a merge
	 * point for any particular branch point.  It only indicates that the named element *could* be
	 * a merge point.
	 * 
	 * @param possibleMergePoint - The NamedElement that may or may not be a merge point.  Must be an
	 * instance of either OutputPinSet or StructuredActivityNode that is a nested process
	 * @return boolean - True if the Action is a potential merge point, false otherwise
	 */
	public boolean isEMerge(NamedElement possibleMergePoint){
		List inputSets = null;
		
		if (possibleMergePoint instanceof Action 
				&& (BomUtils.isProcess((Action)possibleMergePoint)
					|| possibleMergePoint instanceof LoopNode)){
			/* All nested subprocesses are potential merge points for any branches in their contents */
			return true;
		
		} else if (!(possibleMergePoint instanceof OutputPinSet)){
			return false; 
		}
		
		inputSets = ((OutputPinSet)possibleMergePoint).getInputPinSet();
		
		List outputSets = new ArrayList();
		
		/* An action can be an exclusive merge point if there is more than one input set 
		 * correlated to a single output set. */
		if (inputSets.size() < 2){
			return false;
		}
		 
		InputPinSet inputSet = null;
		OutputPinSet candidate = null;
		for (int i=0, len = inputSets.size(); i<len; i++ ){
			inputSet = (InputPinSet)inputSets.get(i);
			if ( inputSet.getOutputPinSet().size() == 1){
				candidate = (OutputPinSet)inputSet.getOutputPinSet().get(0);
				if (outputSets.contains(candidate)){
					/* Another input set also correlated to (only) this output set, so it's a possible merge */
					return true;
				} 
				outputSets.add(candidate);
			}
		}
		    
		/* We must not have detected more than one input set that correlated to the same (single) output set */
		return false;

	}

	/**
	 * Returns whether the given Action is the full exclusive merge point for the given branch point.
	 * If branchPoint is not actually an exclusive branch point, false is returned.
	 * 
	 * @param branchPoint - The exclusive branch point in question 
	 * @param possibleMergePoint - The NamedElement that may or may not be the full merge point for the
	 * given exclusive branch point.  Must be an instance of either OutputPinSet or StructuredActivityNode
	 * that is a nested subprocess
	 * @return boolean - True if the Action is the full merge point, false otherwise
	 */
	public boolean isEMerge(InputPinSet branchPoint, NamedElement possibleMergePoint){
		if (!isEBranch(branchPoint)){
			return false;
		}
		
		if (!isEMerge(possibleMergePoint)){
			return false;
		}
		
		NamedElement realMergePoint = findExclusiveMergePoint(branchPoint);

		return (realMergePoint == possibleMergePoint);
		
	}	

	/**
	 * Given a particular branch point, finds the corresponding full concurrent merge point 
	 * 
	 * @param branchPoint The OutputPinSet that is a concurrent branch point
	 * @return NamedElement - The merge point which will be either an InputPinSet or a 
	 * StructuredActivityNode that is not a task
	 */
	public NamedElement findConcurrentMergePoint(OutputPinSet branchPoint){
		NamedElement result = null;
		
		/* If we have previously found the merge point for this branch point, get
		 * it from the cache to save the lookup. */
		if (fBranchMergeCache.containsKey(branchPoint)){
			result = (NamedElement)fBranchMergeCache.get(branchPoint);
		} else {
			/* Must use a new instance of MergePointFinder, because this could be called (recursively) by MergePointFinder
			 * itself, and we don't want to overwrite the values the current instance is in the process of figuring out */
//			result = fMPFinder.findConcurrentMergePoint(branchPoint);
			if (!visitedCBranchPoint.contains(branchPoint)) { // Ray
				visitedCBranchPoint.add(branchPoint); // Ray
				MergePointFinder mpf = new MergePointFinder();
				result = mpf.findConcurrentMergePoint(branchPoint);

				fBranchMergeCache.put(branchPoint, result);
//				fBranchRegionCache.put(branchPoint, fMPFinder.getNodePathContext().getBranchRegion(branchPoint));
				fBranchRegionCache.put(branchPoint, mpf.getNodePathContext().getBranchRegion(branchPoint));
			} // Ray
			else {
//				System.out.println("Export:Cycle detected in CBranch");
				// Cycle is detected in this case.  Return the containing SAN
				Action action = branchPoint.getAction();
				action = action.getInStructuredNode();
				result = action;
			}
			visitedCBranchPoint.remove(branchPoint); // Ray

		}

		return result;
	}


	/**
	 * Returns the full exclusive merge point for the branch point given as an argument. 
	 * 
	 * A full exclusive merge point is defined as the first output set that is reached by 
	 * all of the original branch paths.
	 * 
	 * @param branchPoint - An InputPinSet that is a branch point
	 * @return NamedElement - The merge point that corresponds to the branch point.  Must be either an
	 * OutputPinSet or a StructuredActivityNode  
	 */
	public NamedElement findExclusiveMergePoint(InputPinSet branchPoint){
		NamedElement result = null;
		if (fBranchMergeCache.containsKey(branchPoint)){
			result = (NamedElement)fBranchMergeCache.get(branchPoint);
		} else {
			if (!visitedEBranchPoint.contains(branchPoint)) { // Ray
				visitedEBranchPoint.add(branchPoint); // Ray
				/* Must use a new instance of MergePointFinder, because this could be called (recursively) by MergePointFinder
				 * itself, and we don't want to overwrite the values the current instance is in the process of figuring out */
				MergePointFinder mpf = new MergePointFinder();
//				result = fMPFinder.findExclusiveMergePoint(branchPoint);
				result = mpf.findExclusiveMergePoint(branchPoint);
				fBranchMergeCache.put(branchPoint, result);
				if (result != null){
//					fBranchRegionCache.put(branchPoint, fMPFinder.getNodePathContext().getBranchRegion(branchPoint));
					fBranchRegionCache.put(branchPoint, mpf.getNodePathContext().getBranchRegion(branchPoint));
				}
				//defect 85566
				visitedEBranchPoint.remove(branchPoint); // Ray
			} // Ray
			else {
//				System.out.println("Export:Cycle detected");
				// Cycle is detected in this case.  Return the containing SAN
				Action action = branchPoint.getAction();
				action = action.getInStructuredNode();
				result = action;
			}
		//defect 85566 moved up above, else we will fail into infinite loop in case of loopback
		//	visitedEBranchPoint.remove(branchPoint); // Ray
		}
		return result;
	}
	
	/**
	 * Determines whether the set adjacent to the starting point is compatible with the starting point.
	 * If start is an InputPinSet, then compatibility is tested with the previous output set (or possibly
	 * input set if the previous set was the input set of a non-task SAN).
	 * If start is an OutputPinSet, then compatibility is tested with the next input set.  See 
	 * <code>isNextInputPinSetCompatible(OutputPinSet)</code> for more information on this case
	 * 
	 * @param start - The InputPinSet/OutputPinSet from which to begin the compatibility tests.  Note that
	 * this must be either an instance of InputPinSet or OutputPinSet.
	 * @return NamedElement - In the case that the sets are compatible, returns the set against which 
	 * the start parameter is tested.  Otherwise, returns null. 
	 */
	public NamedElement isNextSetCompatible(NamedElement start, TransformationContext context)
	{
		//TODO AR 14.14
		NamedElement owner = (NamedElement)start.eContainer();
		//Action owner = (Action)start.eContainer();
		// In the case of a CallBehviorAction, which is (in BPEL perspective) invoking
		// another process operation.  The message definition of the process operation
		// being invoked belongs to that process (the one that is being invoked). 
		// Therefore should always treat the next set as incompatible.
		// In the case that the process being invoked is not a first class artifact (FCA),
		// i.e. not a participant in the solution, we will treat a CBA like a normal
		// task.  However, it is not possible in here to check whether a process that the CBA
		// references to is a FCA or not unless we can get a hold of
		// the Heuristics transformation root which is the only one that has the
		// knowledge on determining whether a process is a FCA. To simplify matters, the
		// assumption is to always treat input/outputsets of a CBA not compatible
		
//		if (owner instanceof CallBehaviorAction)
//			return null;
//		if(owner instanceof Operation)
//			return null;
		//if the owner is an Activity, this can only happen if calling through a CallOperationAction, 
		//and in this case the nextSet will always be incompatible.
		if(owner instanceof Activity)
			return null;	
		if ((start instanceof OutputPinSet && !BomUtils.isTopLevelProcess((Action)owner)) ||
			(start instanceof InputPinSet && BomUtils.isTopLevelProcess((Action)owner)))
			return getForwardCompatibleSet(start,context);
		else 
			return getBackwardCompatibleSet(start, context);
	}


	/*
	 * The algorithm is as follow:
	 * 1. For each pin in the set (input or outputset), traverses the outgoing edge
	 * 2. Collect the pin on the other end of the edge (store in 'tarPins').
	 * 3. As soon as a pin in the set is found connecting to a different target node, return null
	 * 4. Otherwise, we know that all pins in the set is connecting to the same target node
	 * 5. Next thing is to find out which (if any) set (input or outputset depend if the target
	 * node is a SAN that contains the source node) of the target node contains all object pins that are collected 
	 * in 'tarPins' (see step 2)
	 * 5.1.AR. Technical Attributes enhancememts. Additionally checks if the 2 sets are compatibles. The 2 sets are compatibles if the 
	 * Technical Attributes (if any) are compatibles .. meaning.. same variable name, same Message/Part and XSDElementDeclaration names. 
	 * 6. Return the set if found, otherwise return null
	 */
	private NamedElement getForwardCompatibleSet(NamedElement origSet, TransformationContext context)
	{	
		List srcPins;
		boolean controlPinsOnly=false;
		if (origSet instanceof OutputPinSet)
			srcPins = ((OutputPinSet)origSet).getOutputObjectPin();
		else
			srcPins = ((InputPinSet)origSet).getInputObjectPin();
		if (srcPins != null && srcPins.isEmpty())
		{
			controlPinsOnly=true;
			if (origSet instanceof OutputPinSet)
				srcPins = ((OutputPinSet)origSet).getOutputControlPin();
			else
				srcPins = ((InputPinSet)origSet).getInputControlPin();
		}
		Vector tarPins = new Vector();
		//check if all the pins are connected to the same source for all pins 
		if (srcPins != null && !srcPins.isEmpty()){
			Action tarNode = null;
			Action tempTarNode = null;
			//OutputObjectPin iop;
			Pin tmpPin;
			ActivityEdge edge;
			Pin tarPin;
			for (int i=0; i<srcPins.size(); i++)
			{
				tmpPin = (Pin)srcPins.get(i);
				edge = tmpPin.getOutgoing();
				if (edge != null)
				{
					ConnectableNode cn = edge.getTarget();
					// Test if a pin is connect to a control node
					if (!(cn instanceof Pin))
						return null;					
					tarPin = (Pin)cn;	
					tarPins.add(tarPin);
					tempTarNode = (Action)tarPin.eContainer();
				} else {
					return null;
				}
				if (tarNode == null)
				{
					tarNode = tempTarNode;
				} else if (tempTarNode != tarNode)
				{
					// if a pin is found connecting to a different node
					// then the set must be incompatible
					return null;
				}
			}
			// if we are here, all the links from the inputPins of the inputSet take us to the same Action

			// If the node found is a CBA, then always assume it is not compatible
			// TODO: this may need to be optimized in the case where the two connected
			// nodes are CBA that reference to the same Process
			Action owner = (Action)origSet.eContainer();
			//if (!checkCBACaseForCompatibleSet(tarNode,owner) || !checkCOACaseForCompatibleSet(tarNode,owner)) {
			//TODO AR 14.10
		//	if (!checkCBACaseForCompatibleSet(tarNode,owner) || !checkRuntimeImplementationForCompatibleSet(tarNode, owner)) {
			if (!checkCBACaseForCompatibleSet(tarNode,owner)) {
				return null;
			}
//			boolean tarNodeIsCBA=tarNode instanceof CallBehaviorAction;
//			boolean origSetOwnerIsCBA = owner instanceof CallBehaviorAction;				
//			if (tarNodeIsCBA)
//			{
//				if (origSetOwnerIsCBA)
//				{
//					// TODO: This may fail if the referenced Process is not available.
//					// We may want to catch the exception and ignore it 
//					if (((CallBehaviorAction)owner).getBehavior() != ((CallBehaviorAction)tarNode).getBehavior())
//						return null; 
//				} else 
//					return null;
//			} else if (origSetOwnerIsCBA)
//				return null;

			// Now find the action that has the input/outputset contains exactly the same object pins
			List sets;
			if (tarNode==origSet.eContainer().eContainer())
			{
				// If the target node is a nested SAN, then we should always
				// treat the set as not compatible because we always want a
				// container to be created for the outputset of a nested SAN
				// and that we want to have an assign to be created to map data
				// to the outputset container
				
				// Ray: but why??
				//if (BomUtils.isTopLevelProcess(tarNode))
				if (BomUtils.isProcess(tarNode) || tarNode instanceof LoopNode)
					sets = tarNode.getOutputPinSet();
				else
					return null;
			} else {
				sets = tarNode.getInputPinSet();
			}
			Iterator it = sets.iterator();
			List pinsInSet;
			NamedElement is;
			while (it.hasNext())
			{
				is = (NamedElement)it.next();
				if (is instanceof InputPinSet)
					pinsInSet = ((InputPinSet)is).getInputObjectPin();				
				else
					pinsInSet = ((OutputPinSet)is).getOutputObjectPin();				
				/* Note: It isn't sufficient to check that all the target pins are contained in
				 * the given input set... All the pins must also be in the same order.  */			
//				if (pinsInSet.size() == tarPins.size() && pinsInSet.containsAll(tarPins))
				if (pinsInSet.size() == tarPins.size() && pinsInSet.equals(tarPins)){
					//additional check fro PinSet compatibility
//					if(TechnologyProfileHelper.compatibleSets((PinSet)origSet, (PinSet)is, context)){
//						return is;	
//					}
				}
					
			}
		}
		return null;
	}
	

	private boolean checkCBACaseForCompatibleSet(Action tarNode, Action owner) {	
		boolean tarNodeIsCBA=tarNode instanceof CallBehaviorAction;
		boolean origSetOwnerIsCBA = owner instanceof CallBehaviorAction;				
		if (tarNodeIsCBA)
		{
			if (origSetOwnerIsCBA)
			{
				// TODO: This may fail if the referenced Process is not available.
				// We may want to catch the exception and ignore it 
				if (((CallBehaviorAction)owner).getBehavior() != ((CallBehaviorAction)tarNode).getBehavior())
					return false; 
			} else 
				return false;
		} else if (origSetOwnerIsCBA)
			return false;
		return true;
	}

	private boolean checkCOACaseForCompatibleSet(Action tarNode, Action owner) {	
		boolean tarNodeIsCOA=tarNode instanceof CallOperationAction;
		boolean origSetOwnerIsCOA = owner instanceof CallOperationAction;				
		if (tarNodeIsCOA)
		{
			if (origSetOwnerIsCOA)
			{
				// TODO: This may fail if the referenced Process is not available.
				// We may want to catch the exception and ignore it 
				if (((CallOperationAction)owner).getOperation().eContainer() != ((CallOperationAction)tarNode).getOperation().eContainer())
					return false; 
			} else 
				return false;
		} else if (origSetOwnerIsCOA)
			return false;
		return true;
	}
	

	
	
	/*
	 * The algorithm is as follow:
	 * 1. For each pin in the set (input or outputset), traverses the incoming edge
	 * 2. Collect the pin on the other end of the edge (store in 'tarPins').
	 * 3. As soon as a pin in the set is found connecting to a different source node, return null
	 * 4. Otherwise, we know that all pins in the set is connecting to the same source node
	 * 5. Next thing is to find out which (if any) set (output or inputset depend if the source
	 * node is a SAN that contains the target node) of the source node contains all object pins that are collected 
	 * in 'tarPins' (see step 2)
	 * 5.1.AR. Technical Attributes enhancememts. Additionally checks if the 2 sets are compatibles. The 2 sets are compatibles if the 
	 * Technical Attributes (if any) are compatibles .. meaning.. same variable name, same Message/Part and XSDElementDeclaration names. 
	 * 6. Return the set if found, otherwise return null
	 */
	private NamedElement getBackwardCompatibleSet(NamedElement origSet, TransformationContext context){	
		boolean controlPinsOnly=false;
		List srcPins;
		if (origSet instanceof OutputPinSet) 
			srcPins = ((OutputPinSet)origSet).getOutputObjectPin();
		else
			srcPins = ((InputPinSet)origSet).getInputObjectPin();
		if (srcPins != null && srcPins.isEmpty())
		{
			controlPinsOnly=true;
			if (origSet instanceof OutputPinSet)
				srcPins = ((OutputPinSet)origSet).getOutputControlPin();
			else
				srcPins = ((InputPinSet)origSet).getInputControlPin();
		}
		Vector tarPins = new Vector();
		//check if all the pins are connected to the same source for all pins 
		if (srcPins != null && !srcPins.isEmpty()){
			Action tarNode = null;
			Action tempTarNode = null;
			Pin tmpPin;
			ActivityEdge edge;
			Pin tarPin;
			for (int i=0; i<srcPins.size(); i++)
			{
				tmpPin = (Pin)srcPins.get(i);
				edge = tmpPin.getIncoming();
				if (edge != null)
				{
					ConnectableNode cn = edge.getSource();
					// Test if a pin is connect to a control node
					if (!(cn instanceof Pin))
						return null;
					tarPin = (Pin)cn;	
					tarPins.add(tarPin);
					tempTarNode = (Action)tarPin.eContainer();
				} else {
					return null;
				}
				if (tarNode == null)
				{
					tarNode = tempTarNode;
				} else if (tempTarNode != tarNode)
				{
					// if a pin is found connecting to a different node
					// then the set must be incompatible
					return null;
				}
			}
			// if we are here, all the links from the inputPins of the inputSet take us to the same Action
			// If the node found is a CBA, then always assume it is not compatible
			// TODO: this may need to be optimized in the case where the two connected
			// nodes are CBA that reference to the same Process
			// TODO: this may need to be optimized in the case where the two connected
			// nodes are CBA that reference to the same Process
			Action owner = (Action)origSet.eContainer();
			//TODO AR WS_I the sets are not compatibles if they belong to 2 activities that have implementation specified... 
		//	if (!checkCBACaseForCompatibleSet(tarNode,owner) || !checkCOACaseForCompatibleSet(tarNode,owner) || !checkRuntimeImplementationForCompatibleSet(tarNode, owner)) {
		if (!checkCBACaseForCompatibleSet(tarNode,owner) || !checkCOACaseForCompatibleSet(tarNode,owner)) {
				return null;
			}
//			boolean tarNodeIsCBA=tarNode instanceof CallBehaviorAction;
//			boolean origSetOwnerIsCBA = owner instanceof CallBehaviorAction;				
//			if (tarNodeIsCBA)
//			{
//				if (origSetOwnerIsCBA)
//				{
//					// TODO: This may fail if the referenced Process is not available.
//					// We may want to catch the exception and ignore it 
//					if (((CallBehaviorAction)owner).getBehavior() != ((CallBehaviorAction)tarNode).getBehavior())
//						return null; 
//				} else 
//					return null;
//			} else if (origSetOwnerIsCBA)
//				return null;
			// Now find the action that has the input/outputset contains exactly the same object pins
			List sets;
			if (tarNode==origSet.eContainer().eContainer())
				sets = tarNode.getInputPinSet();
			else
				sets = tarNode.getOutputPinSet();
			Iterator it = sets.iterator();
			List pinsInSet,tarObjPins=null;
			NamedElement is;
			while (it.hasNext())
			{
				is = (NamedElement)it.next();
				if (!controlPinsOnly)
				{
					if (is instanceof InputPinSet) 
						pinsInSet = ((InputPinSet)is).getInputObjectPin();				
					else
						pinsInSet = ((OutputPinSet)is).getOutputObjectPin();				
				} else {
					if (is instanceof InputPinSet) {
						pinsInSet = ((InputPinSet)is).getInputControlPin();	
						tarObjPins = ((InputPinSet)is).getInputObjectPin();		
					} else {
						pinsInSet = ((OutputPinSet)is).getOutputControlPin();
						tarObjPins = ((OutputPinSet)is).getOutputObjectPin();
					}
				}
				/* Note: It isn't sufficient to check that all the target pins are contained in
				 * the given input set... All the pins must also be in the same order.  */			
//				if (pinsInSet.size() == tarPins.size() && pinsInSet.containsAll(tarPins))
				if (pinsInSet.size() == tarPins.size() && pinsInSet.equals(tarPins)) {
					if (controlPinsOnly && (tarObjPins!=null && tarObjPins.size()>0)) {
						return null;
					}
					//additional check for PinSet compatibility
//					if(TechnologyProfileHelper.compatibleSets((PinSet)is, (PinSet)origSet, context)){
//						return is;
//					}
				}
			}
		}
		return null;
	}


	
	/**
	 * Creates the specific subclass of TransitionRule that applies to the action that owns
	 * the specified source.
	 * @param ruleSource - The object that will initially be set as the source of the rule to be created.
	 * Must be an instance of either InputPinSet, OutputPinSet, or Action.
	 * @return TransformationRule - The rule applicable for the given source.
	 */
	public TransformationRule createChildRule(NamedElement ruleSource){
		TransformationRule tr = null;
		Action ruleSourceAction = null; 
		
		/* Get the action that contains this input set to help in deciding which rule to create */
		if (ruleSource instanceof InputPinSet){
			ruleSourceAction = ((InputPinSet)ruleSource).getAction();

		} else if (ruleSource instanceof OutputPinSet){
			/* The only rule that has an output set as the source is the ExclusiveBranchRule, and
			 * those are created manually */

		} else if (ruleSource instanceof Action){
			ruleSourceAction = (Action)ruleSource;						

		} 
			
		if (BomUtils.isTask(ruleSourceAction)){
			tr = AbstractbpelFactory.eINSTANCE.createSANTaskRule();

		} else if (BomUtils.isProcess(ruleSourceAction)){
			tr = AbstractbpelFactory.eINSTANCE.createSANNestedProcessRule();

		} else if (ruleSourceAction instanceof Decision){
			tr = AbstractbpelFactory.eINSTANCE.createDecisionRule();

		} else if (ruleSourceAction instanceof Merge){
			tr = AbstractbpelFactory.eINSTANCE.createMergeRule();

		} else if (ruleSourceAction instanceof CallBehaviorAction){
			tr = AbstractbpelFactory.eINSTANCE.createCallBehaviorActionRule();

		} else if (ruleSourceAction instanceof CallOperationAction){
			tr = AbstractbpelFactory.eINSTANCE.createCallOperationActionRule();
		
		} else if (ruleSourceAction instanceof Fork){
			tr = AbstractbpelFactory.eINSTANCE.createForkRule();

		} else if (ruleSourceAction instanceof Join){
			tr = AbstractbpelFactory.eINSTANCE.createJoinRule();
		} else if (ruleSourceAction instanceof com.ibm.btools.bom.model.processes.actions.Map){
			tr = AbstractbpelFactory.eINSTANCE.createMapRule();
		} else if (ruleSourceAction instanceof LoopNode){
			tr = AbstractbpelFactory.eINSTANCE.createLoopNodeRule();

		} else if (ruleSourceAction instanceof RepositoryAction){
			if (((RepositoryAction)ruleSourceAction).getRepository() instanceof Variable){
				tr = AbstractbpelFactory.eINSTANCE.createVariableAccessRule();

			} else if (((RepositoryAction)ruleSourceAction).getRepository() instanceof Datastore){
				tr = AbstractbpelFactory.eINSTANCE.createDatastoreAccessRule();
			}
		} else if (BomUtils.isHumanTask(ruleSourceAction)) {
				tr = AbstractbpelFactory.eINSTANCE.createHTTaskRule();
		} else if (BomUtils.isBusinessRuleTask(ruleSourceAction)){
			tr = AbstractbpelFactory.eINSTANCE.createBRTaskRule();				
		} else {
			/* We cannot create a transformation rule based on this type of action */
			//BTRuntimeException e = new BTRuntimeException(null, TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.BOM_OBJECT_NOT_SUPPORTED), MessageKeys.BOM_OBJECT_NOT_SUPPORTED, new String[]{ruleSourceAction.getClass().toString()}, BTGeneralException.ERROR, MessageKeys.RESOURCE_PROPERTY_FILE, this.getClass().toString(), "createChildRule");
			String params[]={ruleSourceAction.getName()};
			LoggingUtil.logError(MessageKeys.BOM_OBJECT_NOT_SUPPORTED,params,null);
			//throw e;
			// Ray: try to handle non-supported action
			tr = AbstractbpelFactory.eINSTANCE.createGenericActionRule(); 
		}
		
		if (ruleSourceAction.getLocalPostcondition().size() > 0 || ruleSourceAction.getLocalPrecondition().size() > 0){
			String params[]={ruleSourceAction.getName()};
			LoggingUtil.logError(MessageKeys.PRE_POST_CONDITIONS_NOT_SUPPORTED,params,null);
		}
		
		/* Add the input parameter to the source of the newly created rule */
		if (tr != null){
			tr.getSource().add(ruleSource);
		}
				
		return tr;	
	}


	/**
	 * Returns whether or not the given element is in the branch region starting at the 
	 * given branch point.  Note that the input set [output set] of the merge point are 
	 * considered to be inside the concurrent [exclusive] branch region. 
	 * 
	 * @param branchPoint The branch point that starts the branch region in question. This must
	 * be an instance of either InputPinSet or OutputPinSet, and can be either an exclusive or concurrent
	 * branch point.
	 * @param element The element to test for inclusion in the branch region.
	 * @return boolean - True if <code>element</code> is in the branch region, false otherwise.
	 */
	public boolean isInBranchRegion(NamedElement branchPoint, NamedElement element){
		List branchRegion = getBranchRegion(branchPoint);
				
		return branchRegion.contains(element);		

	}

	/**
	 * Gets the branch region for the given branch point.  This causes the branch region
	 * to be cached.
	 * 
	 * @param branchPoint The branch point for which to get the branch region
	 * @return List - The contents of the branch region
	 */	
	public List getBranchRegion(NamedElement branchPoint){
		/* Make sure we've previously found the merge point for the given branch point.  In the case
		 * where we have, this will simply return the cached value, so it's not terribly inefficient.
		 * Finding the merge point causes the branch region to be cached.  */
		if (branchPoint instanceof OutputPinSet && isCBranch((OutputPinSet)branchPoint)){
			findConcurrentMergePoint((OutputPinSet)branchPoint);
		} else if (branchPoint instanceof InputPinSet && isEBranch((InputPinSet)branchPoint)){
			findExclusiveMergePoint((InputPinSet)branchPoint);
		}
		
		List branchRegion = (List)fBranchRegionCache.get(branchPoint);
		if (branchRegion == null){
			branchRegion = new ArrayList();
		} 
		return branchRegion;
		
	}
	/**
	 * Reuse the <code>containerRule</code> for the <code>targetPinSet</code>. 
	 * by copying the <code>containerRule</code> target ans registering the Parts with
	 * the targetPinSet pins.
	 * @param containerRule
	 * @param targetPinSet
	 */
	public void reuseContainer(ContainerRule containerRule, PinSet targetPinSet){
		
		ContainerRule newContainerRule = AbstractbpelFactory.eINSTANCE.createContainerRule();
		
		newContainerRule.getTarget().addAll(containerRule.getTarget());
		
		newContainerRule.getSource().add(targetPinSet);
		
		
		containerRule.getChildRules().add(newContainerRule);
		
		containerRule.setComplete(true);
		// if the 2 sets are compatibles we need to register the Variable with the set
		registerContainerWithSet((BPELVariable)containerRule.getTarget().get(0), targetPinSet);
		
		registerPartsWithPins(newContainerRule, (NamedElement)containerRule.getSource().get(0), targetPinSet);
		
	}
	
	/**
	 * Reuse the <code>containerRule</code> for the <code>targetPinSet</code>. 
	 * by copying the <code>containerRule</code> target ans registering the Parts with
	 * the targetPinSet pins.
	 * @param containerRule
	 * @param targetPinSet
	 */
	public void reuseContainer(TransformationRule rule, PinSet targetPinSet, PinSet sourcePinSet){
		
		List targetObjectPins = BomUtils.getObjectPinsForSet(targetPinSet);
		List sourceObjectPins = BomUtils.getObjectPinsForSet(sourcePinSet);
		if(sourceObjectPins != null && !sourceObjectPins.isEmpty()){
			for(int i=0; i<sourceObjectPins.size(); i++){
				ObjectPin op = (ObjectPin)sourceObjectPins.get(i);
				ArrayList sources = new ArrayList();
				sources.add(op.getType());
				sources.add(op);
				VariableRule variableRule = (VariableRule)TransformationUtil.getRuleForSource(rule, VariableRule.class, sources, BPELVariable.class);
				if(variableRule != null){
					if(!variableRule.getSource().contains(targetObjectPins.get(i)))
						variableRule.getSource().add(targetObjectPins.get(i));
				}
			}
		}
	}
	
	/**
	 * Finds the PartRule associated with each pin/parameter in the input for which a container 
	 * already exists, and adds the corresponding pin/parameter in the unregistered set to the list of the
	 * sources associated with the part.
	 * 
	 * NOTE: The two sets must previously have been found to be compatible, and the 
	 * set that is the first argument must have an associated container with its pins/parameters registered
	 * for the parts of the message that type the container.
	 *  
	 * @param setWithContainer The [Pin/Parameter]Set whose pins/parameters have already been associated with parts
	 * @param setToRegister The [Pin/Parameter]Set whose pins/parameters are to be associated with the parts
	 */
	public void registerPartsWithPins(TransformationRule rule, NamedElement setWithContainer, NamedElement setToRegister){
		
		List registeredPins = new ArrayList();
		List unregisteredPins = new ArrayList();
		
		if (setWithContainer instanceof InputPinSet){
			registeredPins.addAll( ((InputPinSet)setWithContainer).getInputObjectPin());	
		} else if (setWithContainer instanceof OutputPinSet){
			registeredPins.addAll( ((OutputPinSet)setWithContainer).getOutputObjectPin());
		} else if (setWithContainer instanceof InputParameterSet){
			registeredPins.addAll( ((InputParameterSet)setWithContainer).getParameter());
		} else if (setWithContainer instanceof OutputParameterSet){
			registeredPins.addAll( ((OutputParameterSet)setWithContainer).getParameter());
		}
		
		if (setToRegister instanceof InputPinSet){
			unregisteredPins.addAll( ((InputPinSet)setToRegister).getInputObjectPin());
		} else if (setToRegister instanceof OutputPinSet){
			unregisteredPins.addAll( ((OutputPinSet)setToRegister).getOutputObjectPin());
		} else if (setToRegister instanceof InputParameterSet){
			unregisteredPins.addAll( ((InputParameterSet)setToRegister).getParameter());
		} else if (setToRegister instanceof OutputParameterSet){
			unregisteredPins.addAll( ((OutputParameterSet)setToRegister).getParameter());
		}
		
		/* If there are no unregistered object pins, return */
		if (unregisteredPins.isEmpty()){
			return;
		}
		
		/* Because the two sets are matching, they must have the same number of input pins, 
		 * and the object pins must be in the same order, so match them up by index	 */
		
		ObjectPin registeredPin = null;
		PartRule partRule = null;
		ElementDeclarationRule elementDeclaration = null;
		//		TODO CHECK !!!! 		
		if(BpelOptionsUtil.isTargetWSICompliant()){
			partRule = (PartRule)TransformationUtil.getRuleForSource(rule.getRoot(), PartRule.class,	setWithContainer);
			if(partRule != null){
				partRule.getSource().add(setToRegister);
			}
			elementDeclaration = (ElementDeclarationRule)TransformationUtil.getRuleForSource(rule.getRoot(),ElementDeclarationRule.class, setWithContainer);
			if(elementDeclaration != null){
				elementDeclaration.getSource().add(setToRegister);
			}
			//additional registers the message with the set
			MessageRule messageRule = (MessageRule)TransformationUtil.getRuleForSource(rule.getRoot(),	MessageRule.class, setWithContainer);
			if(messageRule != null){
				messageRule.getSource().add(setToRegister);
			}
		}else{
			for (int i=0, count = registeredPins.size(); i<count; i++){
				/* We have to start our search from the root (as opposed to the ProcessInterfaceRule
				 * because we may be looking for a part that is in another WSDL, if we are dealing 
				 * with a CallBehaviourAction or CallOperationAction. */
				partRule = (PartRule)TransformationUtil.getRuleForSource(rule.getRoot(),	PartRule.class,	(EObject)registeredPins.get(i));
				/* Add the new corresponding pin to the PartRule so that we can find this PartRule
				 * later based on this pin */
				partRule.getSource().add(unregisteredPins.get(i));
			
			}
			
		}
	}
	
	/**
	 * Determines whether the set adjacent to the starting point in a given direction is compatible
	 * or not.
	 * 
	 * @param start The PinSet to compare with another for compatibility
	 * @param lookForward True if the set should be compared the following set, False if it should be compared
	 * with the previous set.
	 * @return NamedElement - The PinSet that is compatible.  If no compatible PinSet is found, null is returned.
	 */
	public NamedElement isNextSetCompatible(NamedElement start, boolean lookForward){
	
		if (lookForward){
			return getForwardCompatibleSet(start, null);
		} else {
			return getBackwardCompatibleSet(start, null);
		}	
	}
	
	
	/**
	 * Return true if the <code>inputPinSet</code> is compatible with the <code>outputPinSet</code> 
	 * The two sets are considered compatible if they compatible ObjectPins (same type and multiplicity)
	 * @param inputPinSet
	 * @param outputPinSet
	 * @return
	 */
	public boolean compatibleSets(PinSet startPinSet, PinSet endPinSet,  TransformationContext context){
		//the 2 sets are not compatible if the Technical Attributes associated with them (Variable, Message, etc are not compatibles... )
//		if(!TechnologyProfileHelper.compatibleSets(startPinSet, endPinSet, context)){
//			return false;
//		}
		TransformationRule startPinSetMessageRule = TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule((context)), MessageRule.class, startPinSet, Message.class);
		TransformationRule endPinSetMessageRule = TransformationUtil.getRuleForSource(ProcessUtil.getProcessInterfaceRule((context)), MessageRule.class, endPinSet, Message.class);
		if(startPinSetMessageRule != null && startPinSetMessageRule.getTarget().get(0) != null && endPinSetMessageRule != null && endPinSetMessageRule.getTarget().get(0) != null){
			if(startPinSetMessageRule.getTarget().get(0).equals(endPinSetMessageRule.getTarget().get(0))){
				return true;
			}else{
				return false;
			}
		}else{
			List startPins = BomUtils.getObjectPinsForSet(startPinSet);
			List endPins = BomUtils.getObjectPinsForSet(endPinSet);
			if(startPins.size() != endPins.size()){
				return false;
			}
			for(int i=0; i< startPins.size(); i++){
				ObjectPin inputPin = (ObjectPin)startPins.get(i);
				ObjectPin outputPin = (ObjectPin)endPins.get(i);
				if(!inputPin.getType().equals(outputPin.getType())){
					return false;
				}else{
					if(!sameMultiplicity(inputPin, outputPin)){
						return false;
					}
				}
			}
		}
		return true;
	}
	/**
	 * Return true if the <code> inputPin</code> has same multiplicity as the <code>outputPin</code>
	 * @param inputPin
	 * @param outputPin
	 * @return boolean
	 */
	public boolean sameMultiplicity(MultiplicityElement inputPin, MultiplicityElement outputPin){
		
		ValueSpecification inLowerBound = inputPin.getLowerBound();
		ValueSpecification outLowerBound = outputPin.getLowerBound();
		
		ValueSpecification inUpperBound = inputPin.getUpperBound();
		ValueSpecification outUpperBound = outputPin.getUpperBound();
		
//		if(inputPin instanceof RetrieveArtifactPin){
//			inLowerBound = ((RetrieveArtifactPin)inputPin).getRepository().getLowerBound();
//			inUpperBound = ((RetrieveArtifactPin)inputPin).getRepository().getUpperBound();
//		}
//		if(inputPin instanceof StoreArtifactPin){
//			inLowerBound = ((StoreArtifactPin)inputPin).getRepository().getLowerBound();
//			inUpperBound = ((StoreArtifactPin)inputPin).getRepository().getUpperBound();
//		}
//		
//		if(outputPin instanceof RetrieveArtifactPin){
//			outLowerBound = ((RetrieveArtifactPin)outputPin).getRepository().getLowerBound();
//			outUpperBound = ((RetrieveArtifactPin)outputPin).getRepository().getUpperBound();
//		}
//		if(outputPin instanceof StoreArtifactPin){
//			outLowerBound = ((StoreArtifactPin)outputPin).getRepository().getLowerBound();
//			outUpperBound = ((StoreArtifactPin)outputPin).getRepository().getUpperBound();
//		}
		
		
		if(inUpperBound instanceof LiteralUnlimitedNatural){
			if(outUpperBound instanceof LiteralUnlimitedNatural){
				if(!(((LiteralUnlimitedNatural)inUpperBound).getValue().equals(((LiteralUnlimitedNatural)inUpperBound).getValue()))){
					return false;
				}
			}else{
				return false;
			}
		}else{
			if(outUpperBound instanceof LiteralUnlimitedNatural){
				return false;
			}else{
				if(inUpperBound instanceof LiteralInteger && outUpperBound instanceof LiteralInteger){
					if(!(((LiteralInteger)inUpperBound).getValue().intValue() == ((LiteralInteger)outUpperBound).getValue().intValue())){
						return false;
					}
				}
			}
		}
		if(inLowerBound instanceof LiteralInteger && outLowerBound instanceof LiteralInteger){
			if(!(((LiteralInteger)inLowerBound).getValue().intValue() == ((LiteralInteger)outLowerBound).getValue().intValue())){
				//take care of the case where lowerBound is 0.. this should be similar
				if((((LiteralInteger)inLowerBound).getValue().intValue() ==1 || ((LiteralInteger)inLowerBound).getValue().intValue() ==0) &&
				((LiteralInteger)outLowerBound).getValue().intValue() ==1 || ((LiteralInteger)outLowerBound).getValue().intValue() ==0){
					return true;
				}
				return false;
			}
		}
		return true;	
	}
	
	/** 
	 * We are now registering this container with the scrObj for technical attributes....
	 * @param container
	 * @param srcObj
	 */
	public void registerContainerWithSet(BPELVariable container, PinSet srcObj){
		if(srcObj.eContainer() instanceof ControlAction || srcObj.eContainer() instanceof com.ibm.btools.bom.model.processes.actions.Map){
			return;
		}
	//	TechnologyProfileHelper.registerContainerWithSet(container, srcObj);
	}
	
	/** 
	 * Returns the BPELVariable that may have been created for PinSet that correlated with the <code>start</code> OutputPinSet.
	 * (Looks for a backward compatible PinSet on the OutputSet).  
	 * @param start
	 * @return
	 */
	public BPELVariable getContainerForCompatibleCorrelatedSet(TransformationRule rule, NamedElement start, TransformationContext context){
		BPELVariable container = null;
		NamedElement owner = (NamedElement)start.eContainer();
		Map nonReusableVaribaleContext = (Map)context.get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY);
		if(owner instanceof Activity){
			return null;	
		}
		//if(start instanceof OutputPinSet && !BomUtils.isTopLevelProcess((Action)owner)){
		if(start instanceof OutputPinSet){
			List inputPinSets = ((OutputPinSet)start).getInputPinSet();
			if(!inputPinSets.isEmpty()){
				Iterator it = inputPinSets.iterator();
				while(it.hasNext()){
					InputPinSet inputPinSet = (InputPinSet)it.next();
					if(nonReusableVaribaleContext != null && nonReusableVaribaleContext.containsKey(owner) && (((List)nonReusableVaribaleContext.get(owner)).contains(inputPinSet))){
					break;
				}
					if(BomHelper.getInstance().compatibleSets((OutputPinSet)start, inputPinSet, context)){
						ContainerRule containerRule = (ContainerRule)TransformationUtil.getRuleForSource(
									ProcessUtil.getProcessDefinitionRule(context),
									ContainerRule.class,
									inputPinSet,
									BPELVariable.class);
						if (containerRule != null ) {
							container = (BPELVariable)containerRule.getTarget().get(0);
							//the container was found compatible so we need to reuse the container.. meaning, register it with the set
							registerContainerWithSet(container, (PinSet)start);
							registerPartsWithPins(rule, (PinSet)inputPinSet, (PinSet)start);
							break;
							
						}
					}
				}
			}
		//}else if(start instanceof InputPinSet && !BomUtils.isTopLevelProcess((Action)owner)){
		}else if(start instanceof InputPinSet){
			List outputPinSets = ((InputPinSet)start).getOutputPinSet();
			if(!outputPinSets.isEmpty()){
				Iterator it = outputPinSets.iterator();
				while(it.hasNext()){
					OutputPinSet outputPinSet = (OutputPinSet)it.next();
					
					if(nonReusableVaribaleContext != null && nonReusableVaribaleContext.containsKey(owner) && (((List)nonReusableVaribaleContext.get(owner)).contains(outputPinSet))){
					break;
				}
					if(BomHelper.getInstance().compatibleSets((InputPinSet)start, outputPinSet,context)){
						ContainerRule containerRule = (ContainerRule)TransformationUtil.getRuleForSource(
									ProcessUtil.getProcessDefinitionRule(context),
									ContainerRule.class,
									outputPinSet,
									BPELVariable.class);
						if (containerRule != null) {
							container = (BPELVariable)containerRule.getTarget().get(0);
							//the container was found compatible so we need to reuse the container.. meaning, register it with the set
							BomHelper.getInstance().registerContainerWithSet(container, (PinSet)start);
							BomHelper.getInstance().registerPartsWithPins(rule,(PinSet)outputPinSet, (PinSet)start);
							break;
							
						}
					}
				}
			}
		}
		return container;
	}

	
	/** 
	 * Returns the BPELVariable that may have been created for PinSet that correlated with the <code>start</code> OutputPinSet.
	 * (Looks for a backward compatible PinSet on the OutputSet).  
	 * In the case the testing is done on a CBA OutputPinSet, the CBA OutputPinSet that will be used for creating the Container is used for testing. 
	 * @param start
	 * @return
	 */
	public BPELVariable getContainerForCompatibleCorrelatedSet(TransformationRule rule, NamedElement start, TransformationContext context, NamedElement origSrc){
		BPELVariable container = null;
		NamedElement owner = (NamedElement)start.eContainer();
		Map nonReusableVaribaleContext = (Map)context.get(AbstractbpelConstants.NOT_QUALIFY_FOR_VARIABLE_REUSE_CONTEXT_KEY);
		if(owner instanceof Activity){
			return null;	
		}
		if(start instanceof OutputPinSet){
			List inputPinSets = ((OutputPinSet)start).getInputPinSet();
			if(!inputPinSets.isEmpty()){
				for(int i=0; i<inputPinSets.size(); i++){
					InputPinSet inputPinSet = (InputPinSet)inputPinSets.get(i);
					if(nonReusableVaribaleContext != null && nonReusableVaribaleContext.containsKey(owner) && (((List)nonReusableVaribaleContext.get(owner)).contains(inputPinSet))){
					break;
				}
					if(BomHelper.getInstance().compatibleSets((OutputPinSet)start, inputPinSet, context)){
						boolean compatibleVariables = true;
						//we are handling the CBA case
						if(origSrc != start){
							inputPinSet = (InputPinSet)((OutputPinSet)origSrc).getInputPinSet().get(i);
	//						compatibleVariables = TechnologyProfileHelper.compatibleVariable(inputPinSet, (PinSet)origSrc);
						}
						ContainerRule containerRule = (ContainerRule)TransformationUtil.getRuleForSource(
									ProcessUtil.getProcessDefinitionRule(context),
									ContainerRule.class,
									inputPinSet,
									BPELVariable.class);
						if (containerRule != null && compatibleVariables) {
							container = (BPELVariable)containerRule.getTarget().get(0);
							//the container was found compatible so we need to reuse the container.. meaning, register it with the set
							registerContainerWithSet(container, (PinSet)origSrc);
							registerPartsWithPins(rule, (PinSet)inputPinSet, (PinSet)start);
							break;
							
						}
					}
				}
			}
		//}else if(start instanceof InputPinSet && !BomUtils.isTopLevelProcess((Action)owner)){
		}else if(start instanceof InputPinSet){
			List outputPinSets = ((InputPinSet)start).getOutputPinSet();
			if(!outputPinSets.isEmpty()){
				for(int i=0; i<outputPinSets.size(); i++){
					OutputPinSet outputPinSet = (OutputPinSet)outputPinSets.get(i);
					if(nonReusableVaribaleContext != null && nonReusableVaribaleContext.containsKey(owner) && (((List)nonReusableVaribaleContext.get(owner)).contains(outputPinSet))){
					break;
				}
					if(BomHelper.getInstance().compatibleSets((InputPinSet)start, outputPinSet,context)){
						boolean compatibleVariables = true;
						//we are handling the CBA case
						if(origSrc != start){
							outputPinSet = (OutputPinSet)((InputPinSet)origSrc).getOutputPinSet().get(i);
					//		compatibleVariables = TechnologyProfileHelper.compatibleVariable(outputPinSet, (PinSet)origSrc);

						}
						ContainerRule containerRule = (ContainerRule)TransformationUtil.getRuleForSource(
									ProcessUtil.getProcessDefinitionRule(context),
									ContainerRule.class,
									outputPinSet,
									BPELVariable.class);
						if (containerRule != null) {
							container = (BPELVariable)containerRule.getTarget().get(0);
							//the container was found compatible so we need to reuse the container.. meaning, register it with the set
							BomHelper.getInstance().registerContainerWithSet(container, (PinSet)start);
							BomHelper.getInstance().registerPartsWithPins(rule,(PinSet)outputPinSet, (PinSet)start);
							break;
							
						}
					}
				}
			}
		}
		return container;
	}

	
}
