package com.ibm.btools.te.ilm.heuristics.helper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.expression.model.FunctionArgument;
import com.ibm.btools.expression.model.FunctionDefinition;
import com.ibm.btools.expression.model.FunctionExpression;
import com.ibm.btools.expression.model.ModelFactory;
import com.ibm.btools.expression.model.StringLiteralExpression;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AssignRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ContainerRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.util.JavaExpressionSerializer;
import com.ibm.btools.util.StringHelper;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Condition;
import com.ibm.wbit.bpel.Copy;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.OnMessage;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.JoinCondition;
import com.ibm.wbit.bpelpp.TransitionCondition;


/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


/**
 * Util class to maintain optimization info
 */
public class BpelOptimizationUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$


	public static final String OPTIMIZATION_REGISTRY="OPTIMIZATION_REGISTRY";
	public static final String NON_OPTIMIZABLE_ELEMENTS="NON_OPTIMIZABLE_ELEMENTS";
	public static final String OPTIMIZABLE_ELEMENTS="OPTIMIZABLE_ELEMENTS";

	public static final boolean bApplyOptimization=true;
	public static final boolean bDoTerminationLogic=false;
	public static final boolean bUseSameInOutVar=true;
	public static final boolean bRebuildExclusiveStructure=true;
	public static final boolean bOptimizeForkTransformation = true;
	public static final boolean bOptimizeJoinTransformation = true;
	public static final boolean bOptimizeMergeTransformation = true;
	
	//public static final boolean bUseSameInOutVar=false;
	
	public static HashMap getOptimizationRegistry(TransformationContext context) {
		HashMap registry = (HashMap)context.get(OPTIMIZATION_REGISTRY);
		if (registry == null) {
			registry = new HashMap();
			context.put(OPTIMIZATION_REGISTRY,registry);
		}
		return registry;
	}
	
	public static List getNonOptimizableElements(TransformationContext context) {
		HashMap registry = getOptimizationRegistry(context);
		List elements = (List)registry.get(NON_OPTIMIZABLE_ELEMENTS);
		if (elements == null) {
			elements = new ArrayList();
			registry.put(NON_OPTIMIZABLE_ELEMENTS,elements);
		}
		return elements;
	}
	
	public static List getOptimizableElements(TransformationContext context) {
		HashMap registry = getOptimizationRegistry(context);
		List elements = (List)registry.get(OPTIMIZABLE_ELEMENTS);
		if (elements == null) {
			elements = new ArrayList();
			registry.put(OPTIMIZABLE_ELEMENTS,elements);
		}
		return elements;
	}
	
	public static void registerNonOptimizableElement(TransformationContext context, Object element) {
		getNonOptimizableElements(context).add(element);
	}
	
	public static void registerOptimizableElement(TransformationContext context, Object element) {
		getOptimizableElements(context).add(element);
	}
	
	public static Variable replaceVariable_bak (TransformationContext context, Variable origVar, PinSet origPinSet, Variable newVar) {
		if(origVar == newVar)
		return origVar;
		if (((BPELVariable)newVar).getMessageType() == ((BPELVariable)origVar).getMessageType()) {
			Collection cRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(context),ContainerRule.class,origPinSet);
			ContainerRule cr;
			List crTargets;
			int ind;
			for (Iterator it=cRules.iterator();it.hasNext();) {
				cr = (ContainerRule)it.next();
				crTargets = cr.getTarget();
				ind = crTargets.indexOf(origVar);
				if (ind != -1) {
					crTargets.add(ind,newVar);
					crTargets.remove(ind+1);
				}
			}
			// Update all the assign previously created
			Collection assignRules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(context),AssignRule.class,origPinSet);
			AssignRule assignRule;
			List ruleTargets;
			Assign assign;
			List copies;
			Copy copy;
			for (Iterator it = assignRules.iterator();it.hasNext();) {
				assignRule = (AssignRule)it.next();
				ruleTargets = assignRule.getTarget();
				assign = (Assign)ruleTargets.get(ruleTargets.size()-1); // last target is assign
				copies = assign.getCopy();
				for (Iterator it2=copies.iterator(); it2.hasNext();) {
					copy = (Copy)it2.next();
					if (copy.getFrom().getVariable()==origVar) {
						copy.getFrom().setVariable(newVar);
					}
					if (copy.getTo().getVariable()==origVar) {
						copy.getTo().setVariable(newVar);
					}
				}
			}
			return newVar;				
		}
		return origVar;
	}

	/**
	 * Obtain the transition condition from the source of the link
	 * @param link
	 * @return
	 */	
	public static TransitionCondition getTransitionCondition_bak(Link link) {
		if (link.getSources().size()>0) {
			List extensibilityElements = ((Source)link.getSources().get(0)).getExtensibilityElements();
			Iterator it = extensibilityElements.iterator();
			Object extElement;
			while (it.hasNext()) {
				extElement = it.next();
				if (extElement instanceof TransitionCondition) {
					return (TransitionCondition)extElement;
				}
			}
		}
		return null;
	}
	
	/**
	 * Obtain the transition condition from the source of the link
	 * @param link
	 * @return
	 */	
	public static Condition getTransitionCondition(Link link) {
		if (link.getSources().size()>0) {
			return ((Source)link.getSources().get(0)).getTransitionCondition();
		}
		return null;
	}

	private static Link getMasterLink_bak(List duplicatedLinksList){
		int size = duplicatedLinksList.size();
		Link masterLink = null;
		if(size > 1){
			
			for(int i=0; i<size; i++){
				masterLink = (Link)duplicatedLinksList.get(i);
				JoinCondition joinCondition = getJoinCondition_bak(masterLink);
				if(joinCondition != null){
					//TODO migration ?? revisit???
					//String body = joinCondition.getJavaCode().getValue();
					String body = joinCondition.getJavaCode();
					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(masterLink);
					if(body.indexOf("|| (" + getLinkStatusSnippet) != -1 || body.indexOf(getLinkStatusSnippet + ") ||") != -1){
							return masterLink;
					}
				}
			}
		}
		if(masterLink == null){
			masterLink = (Link)duplicatedLinksList.get(0);
		}
	return masterLink;	
	}
	
	private static Link getMasterLink(List duplicatedLinksList){
		int size = duplicatedLinksList.size();
		Link masterLink = null;
		if(size > 1){
			
			for(int i=0; i<size; i++){
				masterLink = (Link)duplicatedLinksList.get(i);
				Condition joinCondition = getJoinCondition(masterLink);
				if(joinCondition != null){
					//TODO migration ?? revisit???
					//String body = joinCondition.getJavaCode().getValue();
					String body = (String)joinCondition.getBody();
					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(masterLink);
					if(body.indexOf("|| (" + getLinkStatusSnippet) != -1 || body.indexOf(getLinkStatusSnippet + ") ||") != -1){
							return masterLink;
					}
				}
			}
		}
		if(masterLink == null){
			masterLink = (Link)duplicatedLinksList.get(0);
		}
	return masterLink;	
	}

	public static JoinCondition getJoinCondition_bak(Activity targetActivity) {
			
		if (targetActivity != null) {
			List exElements = targetActivity.getExtensibilityElements();
			// find the JoinCondition extensibility element.  There should only be one 
			for (int i=0; i<exElements.size(); i++) {
				if (exElements.get(i) instanceof JoinCondition) {
					return (JoinCondition)exElements.get(i);
				}
			}
		}
		return null;
	}
	
	public static Condition getJoinCondition(Activity targetActivity) {
		
	if (targetActivity != null && targetActivity.getTargets() != null) {
		//return targetActivity.getJoinCondition();
		return targetActivity.getTargets().getJoinCondition();
		
	}
	return null;
}

	/**
	 * Obtain the transition condition from the source of the link
	 * @param link
	 * @return
	 */	
	public static JoinCondition getJoinCondition_bak(Link link) {
			
			Activity targetActivity = ((Target)link.getTargets().get(0)).getActivity();
			return getJoinCondition_bak(targetActivity);
			/*
			if (targetActivity != null) {
				List exElements = targetActivity.getExtensibilityElements();
				// find the JoinCondition extensibility element.  There should only be one 
				for (int i=0; i<exElements.size(); i++) {
					if (exElements.get(i) instanceof JoinCondition) {
						return (JoinCondition)exElements.get(i);
					}
				}
			}
			*/
//			List extensibilityElements = ((Target)link.getTargets().get(0)).getExtensibilityElements();
//			Iterator it = extensibilityElements.iterator();
//			Object extElement;
//			while (it.hasNext()) {
//				extElement = it.next();
//				if (extElement instanceof JoinCondition) {
//					return (JoinCondition)extElement;
//				}
//			}
			//return null;
		}
	
	/**
	 * Obtain the transition condition from the source of the link
	 * @param link
	 * @return
	 */	
	public static Condition getJoinCondition(Link link) {
			
			Activity targetActivity = ((Target)link.getTargets().get(0)).getActivity();
			return getJoinCondition(targetActivity);
			/*
			if (targetActivity != null) {
				List exElements = targetActivity.getExtensibilityElements();
				// find the JoinCondition extensibility element.  There should only be one 
				for (int i=0; i<exElements.size(); i++) {
					if (exElements.get(i) instanceof JoinCondition) {
						return (JoinCondition)exElements.get(i);
					}
				}
			}
			*/
//			List extensibilityElements = ((Target)link.getTargets().get(0)).getExtensibilityElements();
//			Iterator it = extensibilityElements.iterator();
//			Object extElement;
//			while (it.hasNext()) {
//				extElement = it.next();
//				if (extElement instanceof JoinCondition) {
//					return (JoinCondition)extElement;
//				}
//			}
			//return null;
		}

	public static void fixParallelLinksForSourceActivity(Activity act) {
		//List sources = act.getSources();
		//migration Ar 11.09
		List sources = act.getSources().getChildren();
		Source src;
		Target tar;
		Activity tarAct;
		Link link;
		HashMap tarLinksRegistry=new HashMap();
		List linksList;
		for (Iterator it=sources.iterator();it.hasNext();) {
			src = (Source)it.next();
			tar = null;
			link = src.getLink();
			if (!link.getTargets().isEmpty()) {
				tar = (Target)src.getLink().getTargets().get(0);
			}
			if (tar != null) {
				tarAct = tar.getActivity();
				linksList = (List)tarLinksRegistry.get(tarAct);
				if (linksList == null) {
					linksList = new ArrayList();
					tarLinksRegistry.put(tarAct,linksList);
				}
				linksList.add(link);
			}
		}
		Collection linksListsToBeConsolidated = tarLinksRegistry.values();
		for (Iterator it=linksListsToBeConsolidated.iterator();it.hasNext();) {
			linksList = (List)it.next();
			if (linksList.size()>1) {
				//TODO AR Dec 20
				//consolidateLinks(linksList);
				consolidateLinks(linksList);
			}
		}
	}

	public static void fixParallelLinksForTargetActivity(Activity act) {
		//List targets = act.getTargets();
		//migration Ar 11.09
		List targets = act.getTargets().getChildren();
		Source src;
		Target tar;
		Activity srcAct;
		Link link;
		HashMap srcLinksRegistry=new HashMap();
		List linksList;
		for (Iterator it=targets.iterator();it.hasNext();) {
			tar = (Target)it.next();
			src = null;
			link = tar.getLink();
			if (!link.getSources().isEmpty()) {
				src = (Source)tar.getLink().getSources().get(0);
			}
			if (src != null) {
				srcAct = src.getActivity();
				linksList = (List)srcLinksRegistry.get(srcAct);
				if (linksList == null) {
					linksList = new ArrayList();
					srcLinksRegistry.put(srcAct,linksList);
				}
				if(!CBPELUtil.isFaultLink(srcAct,link)){
					linksList.add(link);
				}
			}
		}
		Collection linksListsToBeConsolidated = srcLinksRegistry.values();
		for (Iterator it=linksListsToBeConsolidated.iterator();it.hasNext();) {
			linksList = (List)it.next();
			if (linksList.size()>1) {
				//TODO AR DEC 20
				//consolidateLinks(linksList);
				consolidateLinks(linksList);
			}
		}
	}	
	public static void consolidateLinks_bak(List duplicatedLinksList) {
		int size = duplicatedLinksList.size();
		Link masterLink, secondaryLink;
		// pick the first one as the master link
		masterLink = getMasterLink(duplicatedLinksList);
		//masterLink = (Link)duplicatedLinksList.get(0);
		TransitionCondition masterTC = getTransitionCondition_bak(masterLink);
		TransitionCondition secondaryTC;
	
		for (int i=0; i<size; i++) {
			if(!(duplicatedLinksList.get(i).equals(masterLink))){
				secondaryLink = (Link)duplicatedLinksList.get(i);
				secondaryTC = getTransitionCondition_bak(secondaryLink);
				if (masterTC == null && secondaryTC != null) {
					masterLink.addExtensibilityElement(secondaryTC);
				} else if (secondaryTC != null && masterTC != null) {
					consolidateTransitionCondition_bak(masterTC,secondaryTC);
				} 
				updateJoinCondition_bak(secondaryLink);
				removeLink(secondaryLink);
			}
			
		}
	}
	
	public static void consolidateLinks(List duplicatedLinksList) {
		int size = duplicatedLinksList.size();
		Link masterLink, secondaryLink;
		// pick the first one as the master link
		masterLink = getMasterLink(duplicatedLinksList);
		//masterLink = (Link)duplicatedLinksList.get(0);
		Condition masterTC = getTransitionCondition(masterLink);
		Condition secondaryTC;
	
		for (int i=0; i<size; i++) {
			if(!(duplicatedLinksList.get(i).equals(masterLink))){
				secondaryLink = (Link)duplicatedLinksList.get(i);
				secondaryTC = getTransitionCondition(secondaryLink);
				if (masterTC == null && secondaryTC != null) {
					//masterLink.addExtensibilityElement(secondaryTC);
					((Source)masterLink.getSources().get(0)).setTransitionCondition(secondaryTC);
				} else if (secondaryTC != null && masterTC != null) {
					consolidateTransitionCondition(masterTC,secondaryTC);
				} 
				updateJoinCondition(secondaryLink);
				removeLink(secondaryLink);
			}
			
		}
	}	

	/**
	 * Remove the link from the flow and disconnect the links from the source and target
	 * @param link
	 */
	public static void removeLink (Link link) {
		// disconnect the link from the source and target
		//TODO AR migration ??? 
		
		if(((Source)link.getSources().get(0)).getActivity() != null){
			//Source source = (Source)link.getSources().get(0);
			Activity act = ((Source)link.getSources().get(0)).getActivity();
			act.getSources().getChildren().remove(link.getSources().get(0));
			if(act.getSources().getChildren().isEmpty()){
				act.setSources(null);
			}
		//	((Source)link.getSources().get(0)).setActivity(null);
		}
		if(((Target)link.getTargets().get(0)).getActivity() != null){
			Activity act = ((Target)link.getTargets().get(0)).getActivity();
			act.getTargets().getChildren().remove(link.getTargets().get(0));
			if(act.getTargets().getChildren().isEmpty()){
				act.setTargets(null);
			}
	//		((Target)link.getTargets().get(0)).setActivity(null);
//			Target target = (Target)link.getTargets().get(0);
//			((Target)link.getTargets().get(0)).eUnset(target.eClass().getEStructuralFeature("activity"));
		}
		
		
		EObject links= link.eContainer();
		while ((links!=null) && !(links instanceof Links)) {
			links = link.eContainer();
		}
		((Links)links).getChildren().remove(link);
	}
	
	/**
	 * Consolidate two transition conditions
	 * @param masterTC
	 * @param secondaryTC
	 */
	public static void consolidateTransitionCondition_bak(TransitionCondition masterTC, TransitionCondition secondaryTC) {
		if (masterTC != null && secondaryTC != null) {
//			TODO migration ?? revisit???
			//if (!masterTC.getJavaCode().getValue().equals(secondaryTC.getJavaCode().getValue())) {
//				String masterJavaCode = masterTC.getJavaCode().getValue();
//				String secondPartJavaCode = secondaryTC.getJavaCode().getValue().substring(secondaryTC.getJavaCode().getValue().indexOf("return")+6);
//				String firstPartJavaCode = masterJavaCode.substring(0,masterJavaCode.indexOf(";"));
//				masterTC.getJavaCode().setValue(firstPartJavaCode+ " || " + secondPartJavaCode);

			if (!masterTC.getJavaCode().equals(secondaryTC.getJavaCode())) {
				String masterJavaCode = masterTC.getJavaCode();
				String secondPartJavaCode = secondaryTC.getJavaCode().substring(secondaryTC.getJavaCode().indexOf("return")+6);
				String firstPartJavaCode = masterJavaCode.substring(0,masterJavaCode.indexOf(";"));
				masterTC.setJavaCode(firstPartJavaCode+ " || " + secondPartJavaCode);
			}
		}
	}
	
	/**
	 * Consolidate two transition conditions
	 * @param masterTC
	 * @param secondaryTC
	 */
	public static void consolidateTransitionCondition(Condition masterTC, Condition secondaryTC) {
		if (masterTC != null && secondaryTC != null) {
//			TODO migration ?? revisit???
			//if (!masterTC.getJavaCode().getValue().equals(secondaryTC.getJavaCode().getValue())) {
//				String masterJavaCode = masterTC.getJavaCode().getValue();
//				String secondPartJavaCode = secondaryTC.getJavaCode().getValue().substring(secondaryTC.getJavaCode().getValue().indexOf("return")+6);
//				String firstPartJavaCode = masterJavaCode.substring(0,masterJavaCode.indexOf(";"));
//				masterTC.getJavaCode().setValue(firstPartJavaCode+ " || " + secondPartJavaCode);

			if (!masterTC.getBody().equals(secondaryTC.getBody())) {
				String masterJavaCode =(String) masterTC.getBody();
				String secondPartJavaCode = ((String)secondaryTC.getBody()).substring(((String)secondaryTC.getBody()).indexOf("return")+6);
				String firstPartJavaCode = masterJavaCode.substring(0,masterJavaCode.indexOf(";"));
				masterTC.setBody(firstPartJavaCode+ " || " + secondPartJavaCode);
			}
		}
	}

	public static String buildGetLinkStatusJavaSnippet(Link link) {
		FunctionExpression functionExpr = ModelFactory.eINSTANCE.createFunctionExpression();
		FunctionDefinition def = ModelFactory.eINSTANCE.createFunctionDefinition();
		def.setFunctionID("com.ibm.btools.te.ilm.heuristics.abstractbpel.getLinkStatus");
		def.setFunctionName("bpws:getLinkStatus");

		FunctionArgument arg1 = ModelFactory.eINSTANCE.createFunctionArgument();			
		StringLiteralExpression strExpr1 = ModelFactory.eINSTANCE.createStringLiteralExpression();
		strExpr1.setStringSymbol(link.getName());
		arg1.setArgumentValue(strExpr1);

		functionExpr.setDefinition(def);
		functionExpr.getArguments().add(arg1);
		JavaExpressionSerializer javaExprSerializer = new JavaExpressionSerializer();
		return javaExprSerializer.serialize(functionExpr);
	}

	/**
	 * Upon removal of the link, we need to remove the getLinkStatus for the link in the JoinCondition
	 * @param linkToBeRemoved
	 */
	public static void updateJoinCondition_bak(Link linkToBeRemoved) {
		Activity targetActivity = ((Target)linkToBeRemoved.getTargets().get(0)).getActivity();
		if (targetActivity != null) {
			List exElements = targetActivity.getExtensibilityElements();
			// find the JoinCondition extensibility element.  There should only be one 
			JoinCondition joinCondition=null;
			for (int i=0; i<exElements.size() && joinCondition==null; i++) {
				if (exElements.get(i) instanceof JoinCondition) {
					joinCondition = (JoinCondition)exElements.get(i);
				}
			}
			if (joinCondition != null) {
				//migration AR 11.09
				//List targets = targetActivity.getTargets();
				List targets = targetActivity.getTargets().getChildren();
				if (targets.size()==2) {
					// if there are 2 targets, then there will only be one left when the link
					// is removed.  That means we don't need any JoinCondition (i.e. only 1 link goes
					// into this activity)
					exElements.remove(joinCondition);
				} else {
					// Clean up the getLinkStatus snippet of the link from the JoinCondition
					//TODO migration ?? revisit
					//	String body = joinCondition.getJavaCode().getValue();
					String body = joinCondition.getJavaCode();
					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(linkToBeRemoved);
//					body = StringUtil.replaceAll(body,"&& ("+getLinkStatusSnippet+")","");
//					body = StringUtil.replaceAll(body,"|| ("+getLinkStatusSnippet+")","");
//					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") &&","");
//					body = StringUtil.replaceAll(body,"("+getLinkStatusSnippet+") ||","");
//					body = StringUtil.replaceAll(body,"&& ("+getLinkStatusSnippet+")","");
					body = body.replaceAll("|| ("+getLinkStatusSnippet+")","");
					body = body.replaceAll("("+getLinkStatusSnippet+") &&","");
					body = body.replaceAll("("+getLinkStatusSnippet+") ||","");
				//	joinCondition.getJavaCode().setValue(body);
					joinCondition.setJavaCode(body);
				}
			}
		}
	}
	
	/**
	 * Upon removal of the link, we need to remove the getLinkStatus for the link in the JoinCondition
	 * @param linkToBeRemoved
	 */
	public static void updateJoinCondition(Link linkToBeRemoved) {
		Activity targetActivity = ((Target)linkToBeRemoved.getTargets().get(0)).getActivity();
		if (targetActivity != null) {
//			List exElements = targetActivity.getExtensibilityElements();
//			// find the JoinCondition extensibility element.  There should only be one 
//			JoinCondition joinCondition=null;
//			for (int i=0; i<exElements.size() && joinCondition==null; i++) {
//				if (exElements.get(i) instanceof JoinCondition) {
//					joinCondition = (JoinCondition)exElements.get(i);
//				}
//			}
			Condition joinCondition = getJoinCondition(linkToBeRemoved);
			if (joinCondition != null) {
				//migration AR 11.09
				//List targets = targetActivity.getTargets();
				List targets = targetActivity.getTargets().getChildren();
				if (targets.size()<=2) {
					// if there are 2 or less targets, then there will only be one left when the link
					// is removed.  That means we don't need any JoinCondition (i.e. only 1 link goes
					// into this activity)
					//targetActivity.setJoinCondition(null);
					targetActivity.getTargets().setJoinCondition(null);
					//exElements.remove(joinCondition);
				} else {
					// Clean up the getLinkStatus snippet of the link from the JoinCondition
					//	String body = joinCondition.getJavaCode().getValue();
					//String body = joinCondition.getJavaCode();
					String body = (String)joinCondition.getBody();
					//Feature 109954
					String getLinkStatusSnippet = buildGetLinkStatusJavaSnippet(linkToBeRemoved);
//					body = StringHelper.replaceAll(body,"&& ("+getLinkStatusSnippet+")","");
//					body = StringHelper.replaceAll(body,"|| ("+getLinkStatusSnippet+")","");
//					body = StringHelper.replaceAll(body,"("+getLinkStatusSnippet+") &&","");
//					body = StringHelper.replaceAll(body,"("+getLinkStatusSnippet+") ||","");						
					body = removeGetLinkStatusSnippet(body,getLinkStatusSnippet);
				//	joinCondition.getJavaCode().setValue(body);
					joinCondition.setBody(body);
				}
			}
		}
	}	
	
	private static String removeGetLinkStatusSnippet(String body, String getLinkStatusSnippet) {
		String copyOfBody = new String(body);
		copyOfBody=StringHelper.replaceAll(copyOfBody," ","");
		String copyOfSnippet = new String(getLinkStatusSnippet);
		int snippetLen = copyOfSnippet.length();
		int ind = copyOfBody.indexOf(copyOfSnippet);
		while (ind != -1) {
			int indLeft=ind-1, indRight=ind+snippetLen;
			if (indLeft>=0 && indRight<copyOfBody.length()) {
				while (copyOfBody.charAt(indLeft)=='(' && copyOfBody.charAt(indRight)==')') {
					copyOfSnippet="("+copyOfSnippet+")";
					indLeft--;
					indRight++;
					// Ray - PMR 12183,100,838
					while(copyOfBody.charAt(indLeft)=='\r') {
						copyOfSnippet="\r"+copyOfSnippet;
						indLeft--;
					}
					while(copyOfBody.charAt(indRight)=='\r') {
						copyOfSnippet=copyOfSnippet+"\r";
						indRight++;
					}
				}				
			}
			if (indLeft>0) {
				if (copyOfBody.charAt(indLeft)=='&') {
					copyOfSnippet="&&"+copyOfSnippet;
				} else if (copyOfBody.charAt(indLeft-1)=='|') {
					copyOfSnippet="||"+copyOfSnippet;					
				}
			}
			if (indRight<copyOfBody.length()) {
				if (copyOfBody.charAt(indRight)=='&') {
					copyOfSnippet=copyOfSnippet+"&&";
				} else if (copyOfBody.charAt(indRight)=='|') {
					copyOfSnippet=copyOfSnippet+"||";					
				}
			}
			copyOfBody=StringHelper.replaceFirst(copyOfBody,copyOfSnippet,"");
			copyOfSnippet = getLinkStatusSnippet;
			ind = copyOfBody.indexOf(copyOfSnippet);
		}
		return copyOfBody;
	}
	/**
	 * Populates the 'allActivities' list argument with all the activities
	 * that are contained within the given activity.
	 * 
	 * @param activity The activities whose contents to search (recursively) for activities
	 * @param allActivities The list to fill with the ativities found within 'activity'
	 */
	public static void getAllActivities(EObject activity, List allActivities){
		for (int i=0; i<activity.eContents().size(); i++){
			if (activity.eContents().get(i) instanceof Activity){
				allActivities.add(activity.eContents().get(i));
			}
			getAllActivities((EObject)activity.eContents().get(i), allActivities);
		}
	}
	
	/**
	 * Populates the 'allVariables' list argument with all the BPEL variables
	 * that are contained within the given container.
	 * 
	 * @param container The object whose contents to search (recursively) for variables
	 * @param allVariables The list to fill with the variables found within 'container'
	 */
	public static void getAllVariables(EObject container, List allVariables){
		for (int i=0; i<container.eContents().size(); i++){
			if (container.eContents().get(i) instanceof BPELVariable){
				allVariables.add(container.eContents().get(i));
			}
			getAllVariables((EObject)container.eContents().get(i), allVariables);
		}
	}
	/**
	 * Find the parent of the activity (act) that has to be either Pick or Switch
	 */
	public static EObject findPickOrSwitchParent(EObject act) {
		if (act == null) {
			return null;
		}
		
		while (act.eContainer()!=null && !(act.eContainer() instanceof Pick || act.eContainer() instanceof Switch)) {
			act = (EObject)act.eContainer();
		}
		
		return (EObject)act.eContainer();
	}


	/**
	 * Find the parent of the activity (act) that has to be either Case or OnMessage
	 */
	public static EObject findCaseOrOnMessageParent(EObject act) {
		if (act == null || act.eContainer() == null || act.eContainer().eContainer() == null) {
			return null;
		}
		act = act.eContainer().eContainer();
		
		while (act != null && act.eContainer() != null 
				&& !(act instanceof Case || act instanceof OnMessage)) {
			act = act.eContainer().eContainer();
		}
		
		return act;
	}

	//check if this activity does not have links to any other activities in its branch (Case for Switch or OnMessage for Pick)	
	public static boolean isNoSourceForLinksWithinBranchAct(Activity act) {
		if (act == null || act.getSources() == null || act.getSources().getChildren() == null || act.getSources().getChildren().isEmpty()) {
			return true;
		}
		EObject containerBranch = findCaseOrOnMessageParent(act);
		if (containerBranch == null) {
			return true;
		}
		
		List<Source> sources = act.getSources().getChildren();
		Target target = null;
		Link link = null;
		Activity targetActivity = null;
		EObject targetContainerBranch = null;
		for (Source source: sources) {
			if (source == null || source.getLink() == null || source.getLink().getTargets() == null || source.getLink().getTargets().isEmpty()) {
				continue;
			}
			link = source.getLink();
			target = (Target)link.getTargets().get(0);
			targetActivity = target.getActivity();
			if (targetActivity == null) {
				continue;	//problem with links?
			}
			targetContainerBranch = findCaseOrOnMessageParent(targetActivity);
			if (targetContainerBranch == containerBranch) {
				return false;
			}
		}

		return true;
	}	
}
