package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.InvokeRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.SANTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.ConnectableRuleImpl;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.InvokeRuleImpl;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Assign;
import com.ibm.wbit.bpel.Invoke;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpelpp.FaultSource;
import com.ibm.wbit.bpelpp.FaultSources;

public class CBPELMappingUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	public static final String FLOW_MAPPING = "FLOW_MAPPING";
	
	public static Map<EObject, EObject> getFlowMapping(TransformationContext context) {
		Map<EObject, EObject> flowMapping = (Map<EObject, EObject>) context.get(FLOW_MAPPING);
		if (flowMapping == null) {
			flowMapping = new HashMap<EObject, EObject>();
			context.put(FLOW_MAPPING, flowMapping);
		}
		
		return flowMapping;
	}
	
	public static void registerFlowMapping(EObject activity, EObject flow, TransformationContext context) {
		Map <EObject, EObject> flowMapping = getFlowMapping(context);
		
		flowMapping.put(activity, flow);
	}
	
	/**
	 * Creates the following mapping for a task:
	 * IPS --> FaultSources
	 * For all faultSource inside the FaultSources :
	 * 		EPS --> FaultSource
	 * If an assign is generated :
	 * For all generated assigns :
	 * 		IPS --> Assign Sources
	 * 		EPS --> Assign Source
	 * 		all connections from the corresponding EPS --> generated link from invoke to assign
	 * 
	 * @param rule
	 * @param mapMgr
	 */
	public static void createFaultMappingsForTask(TransformationRule rule, MappingManager mapMgr) {
		//d81057
		//Note : Source elements don't need to be mapped now that feedback doesn't matter
		//That's because bpel matcher has a customized id generation for source elements
		if (!(rule instanceof SANTaskRule) &&
				!(rule instanceof CallBehaviorActionRule)) {
			return;
		}
		
		List<OutputPinSet> opsList = CBPELMappingUtil.getExceptionPS(rule);
		if (opsList.size() == 0) {
			return;
		}
		
		//if (true) return;
		
		//SAN IPS --> FaultSources
		Map<TransformationRule,Map<String, List<Object>>> faultSourcesMap = CBPELUtil.getFaultSourcesMap(rule.getRoot().getContext());
		Map<String, List<Object>> epsMap = faultSourcesMap.get(rule.getChildRules().get(0));
		
		Invoke invoke = (Invoke) rule.getTarget().get(0);
		List<FaultSource> faultSources = getFaultSources(invoke);
		if(faultSources == null){
			return;
		}
		//List<FaultSource> faultSourceList = faultSources.getFaultSource();
		//mapMgr.addMapDefinition(opsList.get(0).getAction(), faultSources);
		//IPS --> FaultSources
//		mapMgr.addMapDefinition((InputPinSet)rule.getSource().get(0), faultSources);
		
		List<OutputPinSet> exceptionOPSList = getExceptionPS(rule);
		for (OutputPinSet exceptionOPS : exceptionOPSList) {
			List<FaultSource> faultSourceForThisOPS = getFaultSourceForEPS(faultSources, exceptionOPS, rule);
//			for (FaultSource faultSource : faultSourceForThisOPS) {
//					//EPS --> FaultSource
//					mapMgr.addMapDefinition(exceptionOPS, faultSource);
//			}
			
//			List<EObject> targetObjects = epsMap.get(exceptionOPS.getName());
//			
//			//Map this EPS to all FaultSource
//			for (FaultSource faultSource : faultSourceList) {
//				if (targetObjects.contains(faultSource.getLink())) {
//					//EPS --> FaultSource
//					mapMgr.addMapDefinition(exceptionOPS, faultSource);
//				}
//			}
			if(CBPELUtil.shouldGenerateForkActivity(exceptionOPS)) {
				if (!(rule.getChildRules().get(0) instanceof InvokeRule)) {
					return;
				}
				Activity generatedActivity = ((InvokeRuleImpl)rule.getChildRules().get(0)).getMappedActivity(exceptionOPS);
				//((ConnectableRuleImpl)rule).getMappedActivity(exceptionOPS);
				for (FaultSource faultSource : faultSourceForThisOPS) {
					//Figure out which faultSource this assign pertains to.
					if (((Target)(faultSource.getLink().getTargets().get(0))).eContainer().eContainer() == generatedActivity) {
						//IPS --> Assign Sources
						mapMgr.addMapDefinition((InputPinSet)rule.getSource().get(0), generatedActivity.getSources());
						//EPS --> Assign Source
//						for (Source src : (List<Source>)generatedActivity.getSources().getChildren()) {
//							mapMgr.addMapDefinition(exceptionOPS, src);
//						}
						//All connections from this EPS --> link from invoke to assign
						for (OutputObjectPin op : (List<OutputObjectPin>)exceptionOPS.getOutputObjectPin()) {
							mapMgr.addMapDefinition(op.getOutgoing(), faultSource.getLink());
						}
				}
			}
		}
		
//		//Map assigns
//		for (Object targetObj : rule.getTarget()) {
//			if (targetObj instanceof Assign) {
//				//Assign assign = (Assign) rule.getTarget().get(1);
//				Assign assign = (Assign) targetObj;
//				OutputPinSet exceptionOPS = getEPSForAssign(assign, faultSources, exceptionOPSList, rule);
//				
//				for (FaultSource faultSource : getFaultSourceForEPS(faultSources, exceptionOPS, rule)) {
//					//Figure out which faultSource this assign pertains to.
//					if (((Target)(faultSource.getLink().getTargets().get(0))).eContainer().eContainer() == assign) {
//						//EPS --> Assign
//						//mapMgr.addMapDefinition(exceptionOPS, assign);
//						//IPS --> Assign Sources
//						mapMgr.addMapDefinition((InputPinSet)rule.getSource().get(0), assign.getSources());
//						//EPS --> Assign Source
//						for (Source src : (List<Source>)assign.getSources().getChildren()) {
//							mapMgr.addMapDefinition(exceptionOPS, src);
//						}
//						//All connections from this EPS --> link from invoke to assign
//						for (OutputObjectPin op : (List<OutputObjectPin>)exceptionOPS.getOutputObjectPin()) {
//							mapMgr.addMapDefinition(op.getOutgoing(), faultSource.getLink());
//						}
//					}
//				}//Each assign should only be connected to one fault source
//			}//for each assign
		}
		
	}

	private static List<OutputPinSet> getExceptionPS(TransformationRule rule)
	{
		List<OutputPinSet> result = new LinkedList<OutputPinSet>();
		List<OutputPinSet> opsList = (((InputPinSet)rule.getSource().get(0)).getAction()).getOutputPinSet();
		for (OutputPinSet ops : opsList) {
			if (ops.getIsException()) {
				result.add(ops);
			}
		}
		
		return result;
	}
	
	private static List<FaultSource> getFaultSources(Invoke invoke) {
		List<FaultSource> result = new LinkedList<FaultSource>();
		for (Source source : (List<Source>)invoke.getSources().getChildren()) {
			if (source instanceof FaultSource) {
				result.add((FaultSource) source);
			}
		}
		
		return result;
//		List extensibilityElements = invoke.getExtensibilityElements();
//		for (Iterator it = extensibilityElements.iterator(); it.hasNext();) {
//			Object extElement = it.next();
//			if (extElement instanceof FaultSources) {
//				return (FaultSources) extElement;
//			}
//		}
		
//		return null;
	}
	
	//Returns a list of FaultSource that was generated from a certain exception output pinset
	private static List<FaultSource> getFaultSourceForEPS(List<FaultSource> faultSources, OutputPinSet exceptionOPS, TransformationRule rule) {
		if(faultSources == null){
			return null;
		}
		List<FaultSource> result = new LinkedList<FaultSource>();
		
		Map<TransformationRule,Map<String, List<Object>>> faultSourcesMap = CBPELUtil.getFaultSourcesMap(rule.getRoot().getContext());
		Map<String, List<Object>> epsMap = faultSourcesMap.get(rule.getChildRules().get(0));
		//A list of objects containing artifacts (links, bpel variables etc) generated for this exception output pinset
		if(epsMap == null){
			return null;
		}
		List<Object> targetObjects = epsMap.get(exceptionOPS.getName());
		
		for (FaultSource faultSource : faultSources) {
			//If this FaultSource corresponds to a link in the targetObject list, then it is generated from this exception output pinset
			if (targetObjects.contains(faultSource.getLink())) {
				result.add(faultSource);
			}
		}
		
		return result;
	}
	
	//Returns the EPS that corresponds to a certain assign
//	private static OutputPinSet getEPSForAssign(Assign assign, FaultSources faultSources, List<OutputPinSet> exceptionOPSList, 
//			TransformationRule rule) {
//		for (OutputPinSet exceptionOPS : exceptionOPSList) {
//			List<FaultSource> faultSourceForThisEPS = getFaultSourceForEPS(faultSources, exceptionOPS, rule);
//			
//			for (FaultSource faultSource : faultSourceForThisEPS) {
//				if (((Target)(faultSource.getLink().getTargets().get(0))).eContainer().eContainer() == assign) {
//					return exceptionOPS;
//				}
//			}
//		}
//		
//		return null;
//	}
	
}
