package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.eclipse.emf.common.util.EList;

import com.ibm.btools.bom.model.processes.actions.Merge;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.ActivityEdge;
import com.ibm.btools.bom.model.processes.activities.ConnectableNode;
import com.ibm.btools.bom.model.processes.activities.FinalNode;
import com.ibm.btools.bom.model.processes.activities.InitialNode;
import com.ibm.btools.bom.model.processes.activities.InputControlPin;
import com.ibm.btools.bom.model.processes.activities.InputObjectPin;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputControlPin;
import com.ibm.btools.bom.model.processes.activities.OutputObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.Pin;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.RetrieveArtifactPin;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.util.ScopeRegistryUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELFactory;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Links;
import com.ibm.wbit.bpel.Scope;
import com.ibm.wbit.bpel.Source;
import com.ibm.wbit.bpel.Sources;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.Throw;
import com.ibm.wbit.bpel.Variable;
import com.ibm.wbit.bpelpp.FaultSource;

/**
 * This class provides utilities for cBPEL cycle link support to ensure at least one start activity and one end activity
 * in a generalized flow activity. We need to support the following scenarios:
 * 1. Cycle link between a merge and decision (split) node both in a top level process and subprocess/while loop
 * 2. Cycle link between a merge and a fault pin of a task/service both in a top level process and subprocess/while loop
 * 3. Cycle link between input pin set and output pin set of a subprocess with multiple input pin sets and multiple output pin sets,
 * add validation guard for top level process used as a CBA and cycle link between  between input pin set and output pin set of the
 * process CBA
 * 4. Support control flow fault link in both cycle and non-cycle cases
 */

public class CycleLinkUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$


	/**
	 * This method is to assert the case that the merge activity is the very beginning construct in a 
	 * nested SAN with cycle link
	 * @param ips
	 * @param context TODO
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean keepEmptyAsStartActivty(InputPinSet ips, TransformationContext context){
		boolean isLinkFromDownstreamElement = false;
		Action containerSAN = null;
		if(ips != null && ips.getAction() instanceof Merge){			
			List pinList = BomUtils.getPinsInSet(ips);
			if(pinList != null && pinList.size() > 0){
				//We assume the cycle has only single pin in its source and target pinsets
				Pin iop = (Pin)pinList.get(0);
				//defect 85719
				if(iop.getIncoming() != null && (iop.getIncoming().getSource() instanceof InputObjectPin || iop.getIncoming().getSource() instanceof InputControlPin) ){
					//Check if its nested SAN input pin set
					List ipss = BomUtils.getInputPinSets((InputObjectPin)(iop.getIncoming().getSource()));
					if(ipss != null && !ipss.isEmpty()){
						InputPinSet contanerSANIps = (InputPinSet)ipss.get(0);
						containerSAN = contanerSANIps.getAction();
						if(BomUtils.isLocalProcess(contanerSANIps.getAction())||BomUtils.isTopLevelProcess(contanerSANIps.getAction())){
							if(containerSAN == ips.getAction().getInStructuredNode()){
								isLinkFromDownstreamElement = false;
							}
						}
					}
				}else if(iop.getIncoming() != null && iop.getIncoming().getSource() instanceof InitialNode){
					//This should cover the while loop case since while loop doesn't allow the outside
					//data coming into the nested SAN
					isLinkFromDownstreamElement = false;
				}else if(iop.getIncoming() != null && (iop.getIncoming().getSource() instanceof OutputObjectPin || iop.getIncoming().getSource() instanceof OutputControlPin)){
					//The link is from an upstream activity. how to detect this case???
					List opss = BomUtils.getOutputPinSets((Pin)(iop.getIncoming().getSource()));
					if(opss != null && !opss.isEmpty()){
						OutputPinSet sourceOPS = (OutputPinSet)opss.get(0);						
						isLinkFromDownstreamElement = isCycleLink(sourceOPS, ips);
						//isLinkFromDownstreamElement = cycleDetected(sourceOPS.getAction(), ips, context);
					}

				}else{
					isLinkFromDownstreamElement = true;
				}
			}
		}
		return isLinkFromDownstreamElement;
	}

	/**
	 * This method traverse back the connections and check if the target action in the list of all the upstream elements,
	 * if no, the source is in front the target, otherwise, it's a cycle link	
	 * @param sourceOPS
	 * @param targetIPS
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean isCycleLink(OutputPinSet sourceOPS, InputPinSet targetIPS){	
		List upstreatElements = new ArrayList();	
		Action targetAction = targetIPS.getAction();
		if(sourceOPS != null && targetIPS != null){		
			if(sourceOPS.getInputPinSet()!= null && !sourceOPS.getInputPinSet().isEmpty()){				
				InputPinSet sourceIPS = (InputPinSet)sourceOPS.getInputPinSet().get(0);				
				traceUpstreamElements(targetAction,sourceIPS,upstreatElements);						
			}
		}
		return upstreatElements.contains(targetAction);
	}

	/**
	 * Walk backwards recursively and put all the connected upstream nodes into a collection. When we find the target action in the
	 * upstream element list, we terminate the look-up. This would be more efficient
	 * @param targetAction
	 * @param sourceIPS
	 * @param upstreatElements
	 */
	@SuppressWarnings("unchecked")
	private static void traceUpstreamElements(Action targetAction,
			InputPinSet sourceIPS,
			List upstreatElements) {
		List pinList = BomUtils.getPinsInSet(sourceIPS);
		if(pinList != null && !pinList.isEmpty()){
			for(Object eachPin:pinList){
				if(eachPin instanceof Pin){
					Pin linkedPin = (Pin)eachPin;
					if(linkedPin.getIncoming() instanceof ActivityEdge){
						ActivityEdge activityEdge = linkedPin.getIncoming();
						if(activityEdge.getSource()instanceof Pin){
							Pin srcPin = (Pin)activityEdge.getSource();
							if(srcPin.eContainer() instanceof Action){
								Action upstremAction = (Action)srcPin.eContainer();
								
								//defend the case that there is a cycle in the merge nodes' upstream node 
								//to avoid infinite loop. defect 89955
								if(upstremAction != null && upstreatElements.contains(upstremAction)){
									return;
								}
								
								if(BomUtils.isLocalProcess(upstremAction)||BomUtils.isTopLevelProcess(upstremAction)){									
									continue;
								}else{
									upstreatElements.add(upstremAction);
									//we terminate the look-up once we find the target action for performance consideration
									if(upstremAction == targetAction ){
										return;
									}
									InputPinSet childIPS = (InputPinSet)upstremAction.getInputPinSet().get(0);																		
									traceUpstreamElements(targetAction,childIPS,upstreatElements);
								}
							}
						}else if(activityEdge.getSource()instanceof InitialNode ||activityEdge.getSource() instanceof RetrieveArtifactPin){
							continue;
						}
					}
				}
			}
		}		
	}

	/**
	 * This method is used to detect whether the merge incoming links are from the input pin sets or an initial node
	 * The in structured node can be either  a top level process or a sub-process or while loop. We will ignore top
	 * level process, since it already has start activity <Recieve> and end Activity <Relply>
	 * @param inputSet
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean hasStartElement(InputPinSet inputSet){
		boolean fromAstartElement = false;
		if(inputSet != null && inputSet.getAction() instanceof Merge){
			if(BomUtils.isTopLevelProcess( inputSet.getAction().getInStructuredNode())){
				fromAstartElement = true;
				return fromAstartElement;
			}
			for (Iterator it = inputSet.getAction().getInputPinSet().iterator(); it.hasNext();){
				InputPinSet tempInputPinSetCycle = (InputPinSet)it.next();
				if (tempInputPinSetCycle != inputSet){
					List pinList = BomUtils.getPinsInSet(tempInputPinSetCycle);
					if(pinList != null){
						for(Object eachPin:pinList){
							if(eachPin instanceof Pin){
								Pin thePin = (Pin)eachPin;
								//20090515
								if (thePin.getIncoming() == null) {
									continue;
								}
								
								if(thePin.getIncoming()!= null && thePin.getIncoming().getSource() instanceof Pin){
									if(thePin.getIncoming().getSource().eContainer() == inputSet.getAction().getInStructuredNode()){										
										continue;
									}else{
										fromAstartElement = true;
										break;
									}
								}else if(thePin.getIncoming().getSource() instanceof InitialNode || thePin.getIncoming().getSource() instanceof RetrieveArtifactPin){
									continue;
								}
							}
						}
					}
				}
			}
		}
		return fromAstartElement;
	}

	/**
	 * This method is used to detect whether the merge incoming links are from the input pin sets or an initial node in
	 * 	local structured node (sub-process or while loop).
	 * @param inputSet
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean isMergeInputSetFirstNodeConnectedInLocalContainer(InputPinSet inputSet){
		if (inputSet == null || !(inputSet.getAction() instanceof Merge) ||
				BomUtils.isTopLevelProcess(inputSet.getAction().getInStructuredNode())) {
			return false;
		}
		if (allPinsConnectedOnlyFromInitialOrProcessBoundary(inputSet.getAction())) {
			return false;		// return false when all pins are coming from initial nodes or local SAN boundary (no need to keep Empty)
		}
		
		boolean connectedFromContainerEdgeOrInitialNode = false;
		
		Pin thePin = null;
		List pinList = BomUtils.getPinsInSet(inputSet);
		if(pinList != null){
			for(Object eachPin: pinList){
				thePin = (Pin)eachPin;
				if (thePin.getIncoming() == null) {
					continue;
				}
				
				if (thePin.getIncoming().getSource() instanceof Pin){
					if(thePin.getIncoming().getSource().eContainer() == inputSet.getAction().getInStructuredNode()){										
						connectedFromContainerEdgeOrInitialNode = true;
						break;
					}
				} else if(thePin.getIncoming().getSource() instanceof InitialNode){
					connectedFromContainerEdgeOrInitialNode = true;
					break;
				}
			}
		}
		return connectedFromContainerEdgeOrInitialNode;
	}

	/**
	 * This method is used to detect merge is connected only to InitialNodes or Process boundary
	 * @param inputSet
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean allPinsConnectedOnlyFromInitialOrProcessBoundary(Action merge){
		if (merge == null) {
			return false;
		}
		List pinList = new ArrayList();	
		for (Object pinSet: merge.getInputPinSet()) {	// traverse all pinsets of Merge
			pinList.addAll(BomUtils.getPinsInSet((PinSet)pinSet));
		}	
		
		if (pinList.isEmpty()) {
			return false;
		}
		
		boolean connectedFromContainerEdgeOrInitialNode = true;
		
		Pin thePin = null;
		for(Object eachPin: pinList){
			thePin = (Pin)eachPin;
			if (thePin.getIncoming() == null) {
				continue;
			}
			
			if (thePin.getIncoming().getSource() instanceof Pin){
				if(thePin.getIncoming().getSource().eContainer() != merge.getInStructuredNode()){										
					connectedFromContainerEdgeOrInitialNode = false;
					break;
				}
			}
		}
		return connectedFromContainerEdgeOrInitialNode;
	}
	
	/**
	 * This method analyzes the output pin sets of a sub-process or while loop to determine whether we need to retain the termination set activity
	 * as end activity of a generalized flow, which is created during connecting the output pin sets, the criterion should be:
	 * 1. There is only ONE output pin set, since we already handle splitting the cross boundary link by creating either <Assign> or <Throw> activities
	 * for regular and exceptional output pin sets respectively for multiple output pin sets case
	 * 2. The immediate upstream element contains cycle link
	 * This search algorithm may be expensive, we may have better way to determine the case---TODO
	 * @param os
	 * @param context TODO
	 * @param is
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean keepTerminationSetActivity(Activity terminationSetActivity, OutputPinSet os, TransformationContext context){
		if(os != null && os.getAction() != null && terminationSetActivity instanceof Empty){
			if(BomUtils.isLocalProcess(os.getAction()) || os.getAction() instanceof LoopNode){
				if(os.getAction().getOutputPinSet()!= null && os.getAction().getOutputPinSet().size() == 1){
					List pinList = BomUtils.getPinsInSet(os);
					int noIncomingLinkCounter = 0;
					for(Object eachPin:pinList){
						if(eachPin instanceof Pin){
							Pin pinObj = (Pin)eachPin;
							ActivityEdge ae = pinObj.getIncoming();							
							if(ae != null && ae.getSource() instanceof Pin){
								Pin upstreamPin = (Pin)ae.getSource();
								if(upstreamPin.eContainer() instanceof Action){
									Action upstreamAction = (Action)upstreamPin.eContainer();
									EList upstreamOpsList = upstreamAction.getOutputPinSet();
									boolean foundCycleLink = false;
									for(Object eachUpOps:upstreamOpsList){
										if(eachUpOps instanceof OutputPinSet){
											OutputPinSet upOPS = (OutputPinSet)eachUpOps;
											List upPinList = BomUtils.getPinsInSet(upOPS);
											for(Object eachUpStPin:upPinList){
												if(eachUpStPin instanceof Pin){
													Pin upStPinObj = (Pin)eachUpStPin;
													if(upStPinObj.getOutgoing()== null){
														continue;
													}else if(upStPinObj.getOutgoing().getTarget()!= null){
														ConnectableNode  targetConnectableNode = upStPinObj.getOutgoing().getTarget();
														if(targetConnectableNode instanceof FinalNode){
															continue;
														}else if(targetConnectableNode instanceof Pin){
															if(targetConnectableNode.eContainer() == os.getAction()){
																continue;
															}else{
																if(BomUtils.findPinSetForPin((Pin)targetConnectableNode) instanceof InputPinSet){
																	InputPinSet targetIps = (InputPinSet)BomUtils.findPinSetForPin((Pin)targetConnectableNode);
																	//																	foundCycleLink = isCycleLink(upOPS, targetIps);
																	//																	if(foundCycleLink){
																	//																		break;
																	//																	}else{
																	//we need to traverse the node until reach a termination node or hit its container
																	foundCycleLink = cycleDetected(upstreamAction, targetIps, context);
																	//																	}
																}																
															}
														}
													}
												}
											}
										}
									}
									return foundCycleLink;
								}								
							}else{
								noIncomingLinkCounter++;
							}
						}
					}
					/**
					 * In the case that there is no outgoing links connected to the output pin sets of the sub-process or while loop,
					 * we need to detect if there is an action with cycle link DIRECTLY connected to a termination node. If yes, we
					 * need to have the empty activity as an end activity for the GFlow. Otherwise, we skip the empty activity
					 */
					if(noIncomingLinkCounter == pinList.size()){						
						//we need to find out if the cycle link in front of a element which is connected to a termination node
						StructuredActivityNode san = (StructuredActivityNode)os.getAction();							
						Object content;
						for (Iterator it = san.getNodeContents().iterator(); it.hasNext(); ){
							content = it.next();
							if (content instanceof FinalNode){
								FinalNode fn = (FinalNode)content;
								if(fn.getIncoming() != null && fn.getIncoming().getSource()instanceof Pin){
									Pin fnSourcePin = (Pin)fn.getIncoming().getSource();
									if(fnSourcePin.eContainer() instanceof Action){
										PinSet upStreamPS = BomUtils.findPinSetForPin(fnSourcePin);
										if(upStreamPS instanceof OutputPinSet){
											OutputPinSet upstreamOPS = (OutputPinSet)upStreamPS;
											Action upstreamAction = upstreamOPS.getAction();
											EList opsList = upstreamAction.getOutputPinSet();
											boolean includeCycleLink = false;
											for(Object eachOPS:opsList){
												if(eachOPS instanceof OutputPinSet){
													OutputPinSet theOPS = (OutputPinSet)eachOPS;
													List eachPinList = BomUtils.getPinsInSet(theOPS);
													for(Object eachpin:eachPinList){
														if(eachpin instanceof Pin){
															Pin eachPinObj = (Pin)eachpin;
															if(eachPinObj.getOutgoing() != null && eachPinObj.getOutgoing().getTarget() instanceof Pin){
																Pin targetPin = (Pin)eachPinObj.getOutgoing().getTarget();
																PinSet targetPinSet = BomUtils.findPinSetForPin(targetPin);
																//if(targetPinSet instanceof InputPinSet && isCycleLink(theOPS, (InputPinSet)targetPinSet)){
																if(targetPinSet instanceof InputPinSet && cycleDetected(theOPS.getAction(), (InputPinSet)targetPinSet, context)){
																	includeCycleLink = true;
																	break;
																}
																continue;
															}
														}
													}
													if(includeCycleLink){
														return true;
													}
												}
											}
										}
									}
								}
							}
						}		
					}
				}
			}
		}
		return false;
	}

	public static boolean existFaultSource(TransformationContext context,
			OutputPinSet os, Scope scope) {
		boolean matchFound = false;
		Activity throwActivity = ScopeRegistryUtil.getBpelActivity(context, os);
		if(throwActivity instanceof Throw){
			Throw theThrow = (Throw)throwActivity;
			QName faultName = theThrow.getFaultName();
			Variable variable = theThrow.getFaultVariable();

			Sources sources = CBPELUtil.getSources(scope);

			if(sources != null && !sources.getChildren().isEmpty()){
				for(Object eachSource:sources.getChildren()){
					if(eachSource instanceof FaultSource){
						FaultSource faultSource = (FaultSource)eachSource;
						if(faultSource.getCatch() != null){
							//TODO should we compare the link??
							if(faultSource.getCatch().getFaultName() == faultName && faultSource.getCatch().getFaultVariable() == variable){
								matchFound = true;
								break;
							}
						}
					}
				}
			}
		}
		return matchFound;
	}

	/**
	 * This method is to remove the source which has duplicate fault source under the scope activity. This
	 * utility should be invoked from the BPEL optimization rule. It's not supposed to be called from the ProcessRule
	 * or SANNestedProcessRule, where the link will be screwed up in the case of cycle link on the subprocess
	 * @param scope
	 */
	public static void removeDuplicatedSourceForScope(Scope scope){
		if(scope != null){			
			List<Source> regulareLinkList = new ArrayList<Source>();
			List<FaultSource> faultLinkList = new ArrayList<FaultSource>();
			List<Source> removableLinkList = new ArrayList<Source>();

			Sources sources = CBPELUtil.getSources(scope);
			if(sources != null&& !sources.getChildren().isEmpty()){
				for(Object eachSource:sources.getChildren()){
					if(eachSource instanceof Source){
						if(eachSource instanceof FaultSource){
							FaultSource  theFaultSource = (FaultSource)eachSource;
							faultLinkList.add(theFaultSource);
						}else{
							Source  theSource = (Source)eachSource;
							regulareLinkList.add(theSource);
						}						
					}				
				}
			}
			if(!faultLinkList.isEmpty()&& !regulareLinkList.isEmpty()){
				for(FaultSource eachFaultSource:faultLinkList){
					for(Source eachSource:regulareLinkList){
						//find the duplicates and record it
						if(eachFaultSource.getLink() == eachSource.getLink()){							
							removableLinkList.add(eachSource);
							break;
						}
					}
				}
			}
			//Remove the fault link from the regular sources
			if(!removableLinkList.isEmpty()){
				for(Source removableSource:removableLinkList){
					sources.getChildren().remove(removableSource);
				}
			}
		}
		return;
	}

	/**
	 * This method makes use of the BOM walker to detect the existence of a cycle link. This is 
	 * the core algorithm for cycle link support. The BOM walker will track through all the
	 * source action's down stream elements until it finds source action itself, which means 
	 * there exists a cycle link. However, this method cannot be used for detecting whether we need 
	 * start empty activity, which leads to stackoverflow exception in some cases
	 * @param sourceAction
	 * @param targetIPS
	 * @param context
	 * @return
	 */
	private static boolean cycleDetected(Action sourceAction, InputPinSet targetIPS, TransformationContext context ){		
		boolean foundCycleLink = false;
		BomWalker bw = ProcessUtil.getBomWalker(context);			
		ArrayList<InputPinSet> visitedISets = new ArrayList<InputPinSet>();
		if(targetIPS != null && context != null){
			List<InputPinSet> nextPinSetList = bw.getNext(targetIPS, BomWalker.INPUT_SET, false);
			if(nextPinSetList != null && nextPinSetList.size()>0){
				for(InputPinSet eachIPS:nextPinSetList){
					visitedISets.add(eachIPS);
					if(eachIPS.getAction() == sourceAction){
						foundCycleLink = true;
						break;
					}else{
						// 20090515: use the list of visited InputPinSets to prevent infinite loop (for cases of a loop back WITHIN the loop back)
						//	foundCycleLink = cycleDetected(sourceAction,eachIPS,context);
						foundCycleLink = cycleDetected(sourceAction,eachIPS,context, visitedISets);
						if(foundCycleLink){
							break;
						}
					}
				}					
			}				
		}		
		return foundCycleLink;
	}

	/**
	 * Based on boolean cycleDetected(Action sourceAction, InputPinSet targetIPS, TransformationContext context ) - method above
	 * Check for the cycle, record visited InputPinSets
	 * @param sourceAction
	 * @param targetIPS
	 * @param context
	 * @param context
	 * @return
	 */
	private static boolean cycleDetected(Action sourceAction, InputPinSet targetIPS, TransformationContext context, List <InputPinSet> visitedInputPinSets){		
		boolean foundCycleLink = false;
		BomWalker bw = ProcessUtil.getBomWalker(context);			
		if(targetIPS != null && context != null){
			List<InputPinSet> nextPinSetList = bw.getNext(targetIPS, BomWalker.INPUT_SET, false);
			if(nextPinSetList != null && nextPinSetList.size()>0){
				for(InputPinSet eachIPS:nextPinSetList){
					if(eachIPS.getAction() == sourceAction){
						foundCycleLink = true;
						break;
					} else if(visitedInputPinSets.contains(eachIPS)){	// if the InputPinSet was visited - break the cycle - return from recursive call
						break;
					}else{
						visitedInputPinSets.add(eachIPS);
						foundCycleLink = cycleDetected(sourceAction,eachIPS,context, visitedInputPinSets);
						if(foundCycleLink){
							break;
						}
					}
				}					
			}				
		}		
		return foundCycleLink;
	}

	/**
	 * This method is to determine the whether the link is a fault link within the current context
	 * This utility will be called during Empty activity optimization to consolidate the link in parallel 
	 * with the fault link.
	 * @param context
	 * @param link
	 * @return
	 */
	public static boolean isFaultSourceLink(TransformationContext context,Link link){
		boolean isFaultSource = false;
		// assume single source and single target of a link
		Source src = (Source)link.getSources().get(0);		
		Activity srcAct = src.getActivity();
		if(srcAct instanceof Scope){
			Map<OutputPinSet,Link> scopeFaultLinkRegistry = CBPELUtil.getScopeFaultLinkRegistry(context);
			if(scopeFaultLinkRegistry != null){
				Collection<Link> valueCollection = scopeFaultLinkRegistry.values();
				for(Link eachLink:valueCollection){
					if(link == eachLink){
						isFaultSource = true;
						break;
					}
				}
			}
		}else{
			Map<Activity,List<Source>> linkRegistry = CBPELUtil.getFaultLinkRegistry(context);
			if(linkRegistry != null && linkRegistry.containsKey(srcAct)){
				List<Source> srcList = linkRegistry.get(srcAct);
				if(srcList != null && srcList.contains(src)){
					isFaultSource = true;
				}
			}
		}
		//double check the case where a target empty has two sources with the same link (fault link) and the source of link are from 
		//different sources, one case would be an invoke (fault link) and empty for the invoke activity's pinset
		if(!isFaultSource){
			if(link.getSources().size() > 1){
				for(Object eachSrc:link.getSources()){
					if(eachSrc instanceof FaultSource){
						isFaultSource = true;
					}
				}
			}
		}
		return isFaultSource;
	}

	/**
	 * if there exists a fault link, we consolidate:
	 * 1. Remove duplicated source
	 * 2. Remove duplicated target
	 * 3. Remove duplicated link
	 * 4. Re-target the fault link
	 * This method is for reducing empty activity during optimization
	 * @param empty
	 * @param link
	 * @param srcAct
	 * @param isFaultSource
	 */
	@SuppressWarnings("unchecked")
	public static void consolidateLinkInParallelWithFaultLink(Activity empty,
			Link link, 
			Activity srcAct, 
			boolean isFaultSource) {
		if(isFaultSource){							
			if(empty.getSources()!= null && !empty.getSources().getChildren().isEmpty()){
				Source emptySrc = (Source)empty.getSources().getChildren().get(0);
				Target tar2 = (Target)emptySrc.getLink().getTargets().get(0);
				if(tar2.eContainer()!= null && tar2.eContainer().eContainer() instanceof Activity){
					Activity newTarget = (Activity)tar2.eContainer().eContainer();
					//we need to remove the parallel links from the newTarget first
					if(newTarget.getTargets()!= null && !newTarget.getTargets().getChildren().isEmpty()){
						Target targetToReduce = (Target)newTarget.getTargets().getChildren().get(0);
						Link dupLink = targetToReduce.getLink();

						//Remove duplicated source
						Source sourceToReduce = (Source)dupLink.getSources().get(0);
						if(srcAct.getSources()!= null && !srcAct.getSources().getChildren().isEmpty()){
							srcAct.getSources().getChildren().remove(sourceToReduce);
						}

						//Remove duplicated target
						if(newTarget.getTargets()!= null && !newTarget.getTargets().getChildren().isEmpty()){
							newTarget.getTargets().getChildren().remove(targetToReduce);
						}

						//Remove duplicated link
						if(dupLink.eContainer() instanceof Links){
							Links linkContainer = (Links)dupLink.eContainer();
							linkContainer.getChildren().remove(dupLink);
						}							
					}
					//Re-target the fault link
					Target tag = BPELFactory.eINSTANCE.createTarget();
					tag.setActivity(newTarget);
					link.getTargets().add(tag);
				}
			}
		}
	}
}
