package com.ibm.btools.te.ilm.heuristics.helper;
/*
 * Licensed Materials - Property of IBM
 * 5724-I74, 5724-I75
 *  Copyright IBM Corporation 2008, 2009. All rights reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */


import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.wst.wsdl.Definition;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.AdaptiveEntity;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.BusinessRule;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.DynamicAssembler;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.HumanTask;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.Java;
import com.ibm.btools.te.attributes.model.definition.implementation.wps.impl.WebServicesImpl;
import com.ibm.btools.te.attributes.model.specification.processmodel.wps.ServiceAttributes;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.ProcessFlowRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractProcDefRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.PartnerRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.model.sa.ProcessFlow;
import com.ibm.wbiservers.brules.scdl.brgimpl.BusinessRuleGroupImplementation;
import com.ibm.wbit.bpel.PartnerLink;
import com.ibm.wbit.bpel.PartnerLinks;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerlinktypeFactory;
import com.ibm.wbit.bpel.services.partnerlinktype.Role;
import com.ibm.wbit.bpel.services.partnerlinktype.RolePortType;
import com.ibm.wbit.sca.scdl.ae.AdaptiveEntityImplementation;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Implementation;
import com.ibm.wsspi.sca.scdl.SCAImportBinding;
import com.ibm.wsspi.sca.scdl.java.JavaImplementation;
import com.ibm.wsspi.sca.scdl.mfc.MediationFlowImplementation;
import com.ibm.wsspi.sca.scdl.process.ProcessImplementation;
import com.ibm.wsspi.sca.scdl.task.TaskImplementation;

/**
 * This utility class includes methods to handle process implements service/business service and
 * CBA level implementation variations for modeler global elements
 */
public class FabricIntegrationUtil {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$


	//key for the process flow register map, which is keyed by the process flow's source BOM activity
	private static final String PROCESS_FLOW_RULE_REGISTRY = "_PROCESS_FLOW_RULE_REGISTRY";

	private static final String INTERFACE_ACTIVITY_REGISTRY = "_INTERFACE_ACTIVITY_REGISTRY";

	private static final String COMPONENT_RULE_FOR_INTERFACE_PORTTYPE = "_COMPONENT_RULE_FOR_INTERFACE_PORTTYPE";

	private static final String PARTNER_LINK_RULE_FOR_ACTION = "PARTNER_LINK_RULE_FOR_ACTION";

	private static final String PARTNER_RULE_FOR_ACTION = "_PARTNER_RULE_FOR_ACTION";

	private static final String PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY = "_PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY";

	/**
	 * Register the ProcessFlowRule into the context as a flag to check whether we need to skip the ProcessFlow SA process when 
	 * persisting the targets
	 * @param context
	 * @param processSAN
	 * @param interfaceSAN
	 */
	@SuppressWarnings("unchecked")
	public static void registerProcessFlowForActivity(Activity sourceActivity, ProcessFlow processFlow){
		ExportSession session = BpelOptionsUtil.getExportSession();
		if(session != null){
			Map sessionContext = session.getContext();
			Map<Activity,ProcessFlow> processFlowRuleRegistry = null;
			if(sessionContext != null){
				if(sessionContext.get(PROCESS_FLOW_RULE_REGISTRY) instanceof Map){
					processFlowRuleRegistry = (Map<Activity,ProcessFlow>)sessionContext.get(PROCESS_FLOW_RULE_REGISTRY);
				}				
				if(processFlowRuleRegistry == null){
					processFlowRuleRegistry = new HashMap<Activity,ProcessFlow>();
					processFlowRuleRegistry.put(sourceActivity,processFlow);
					sessionContext.put(PROCESS_FLOW_RULE_REGISTRY, processFlowRuleRegistry);
				}else{
					processFlowRuleRegistry.put(sourceActivity,processFlow);
				}
			}
		}		
	}	
	/**
	 * Find the interface for this process SAN if there is any (process ---> (0...1) interface)
	 * @param processSAN
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Activity getInterfaceActivityIfAny(StructuredActivityNode processSAN){
		if(processSAN == null){
			return null;
		}

		Activity intfaceActivity = null;	
		if(!BomUtils.isProcess(processSAN)){
			return intfaceActivity;
		}	
		//This might be the sub-process
		if(processSAN.getActivity() == null){
			return intfaceActivity;
		}
		EList interfaceList = processSAN.getActivity().getInterfaces();
		if(interfaceList != null && interfaceList.size() == 1){
			Behavior behavior = (Behavior)interfaceList.get(0);
			if(behavior instanceof com.ibm.btools.bom.model.processes.activities.Activity){
				intfaceActivity = (com.ibm.btools.bom.model.processes.activities.Activity)behavior;
			}
		}
		//Register the interface activity for future look-up
		ExportSession session = BpelOptionsUtil.getExportSession();
		if(session != null && intfaceActivity != null){
			if(session != null){
				Map sessionContext = session.getContext();
				List<Activity> interfaceActivityRegistry = null;
				if(sessionContext != null){
					if(sessionContext.get(INTERFACE_ACTIVITY_REGISTRY) instanceof List){
						interfaceActivityRegistry = (List<Activity>)sessionContext.get(INTERFACE_ACTIVITY_REGISTRY);
						if(!interfaceActivityRegistry.contains(intfaceActivity)){
							interfaceActivityRegistry.add(intfaceActivity);
						}
					}else{
						interfaceActivityRegistry = new ArrayList<Activity>();
						interfaceActivityRegistry.add(intfaceActivity);
						sessionContext.put(INTERFACE_ACTIVITY_REGISTRY, interfaceActivityRegistry);						
					}
				}			
			}		
		}		
		return intfaceActivity;
	}    
	/**
	 * Sort out the process flow which source activity is an interface activity of another process in the transformation session
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<ProcessFlow> sortInterfaceActivityProcessFlow() {
		List<ProcessFlow> pfList =  new ArrayList<ProcessFlow>();
		ExportSession session = BpelOptionsUtil.getExportSession();
		if(session != null){
			Map sessionContext = session.getContext();
			Map<Activity,ProcessFlow> processFlowRuleRegistry = null;
			if(sessionContext != null){
				if(sessionContext.get(PROCESS_FLOW_RULE_REGISTRY) instanceof Map){
					processFlowRuleRegistry = (Map<Activity,ProcessFlow>)sessionContext.get(PROCESS_FLOW_RULE_REGISTRY);
					Set activitySet = processFlowRuleRegistry.keySet();
					List tempList = new ArrayList();
					tempList.addAll(activitySet);

					if(activitySet!= null && !activitySet.isEmpty()){
						for(Object eachElement:activitySet){
							if(eachElement instanceof Activity){
								Activity act = (Activity)eachElement;
								Activity interfaceActivity = getInterfaceActivityIfAny(act.getImplementation());
								if(interfaceActivity != null && tempList.contains(interfaceActivity)){
									ProcessFlow candidate = processFlowRuleRegistry.get(interfaceActivity);
									pfList.add(candidate);
								}
							}
						}
					}
				}					
			}			
		}		
		return pfList;
	}
	/**
	 * If the source activity of the process flow is an interface activity, we skip the process flow SA processing
	 * @param pf
	 * @return
	 */
	public static boolean shouldSkipProcessFlow(ProcessFlow pf){
		List<ProcessFlow> skipList = sortInterfaceActivityProcessFlow();
		boolean isBusinessService = false;
		EList definitions = pf.getProcessInterface();
		if(!definitions.isEmpty() && definitions.size() == 1){
			Definition interfaceDef = (Definition)definitions.get(0);
			isBusinessService = isBusinessServiceInterface(interfaceDef);
		}
		if(skipList != null){
			return (skipList.contains(pf) && isBusinessService);
		}
		return false;

	}

	/**
	 * Resolve process interface activity's input pin set, since we use the interface WSDL to represent the process
	 * @param processInputPinSet
	 * @return
	 */
	public static InputPinSet getInterfaceActivityInputPinSetForProcessIPS(InputPinSet processInputPinSet){
		InputPinSet interfaceActivityIPS = null;
		//For compatibility
		if(processInputPinSet != null && !BomUtils.isProcess(processInputPinSet.getAction())){
			interfaceActivityIPS = processInputPinSet;
		}
		if(processInputPinSet.eContainer() instanceof StructuredActivityNode){
			StructuredActivityNode processSAN = (StructuredActivityNode)processInputPinSet.eContainer();
			Activity interfaceActivity  = getInterfaceActivityIfAny(processSAN);
			if(interfaceActivity != null){
				//The interface activity can only have one inputpinset
				interfaceActivityIPS =  (InputPinSet)interfaceActivity.getImplementation().getInputPinSet().get(0);
			}
		}
		return interfaceActivityIPS;
	}

	/**
	 * Resolve process interface activity's output pin set, since we use the interface WSDL to represent the process
	 * @param processInputPinSet
	 * @return
	 */
	public static OutputPinSet getInterfaceActivityOutputPinSetForProcessOPS(OutputPinSet processOutputPinSet){
		OutputPinSet interfaceActivityOPS = null;
		//For compatibility
		if(processOutputPinSet != null && !BomUtils.isProcess(processOutputPinSet.getAction())){
			interfaceActivityOPS = processOutputPinSet;
		}
		if(processOutputPinSet.eContainer() instanceof StructuredActivityNode){
			StructuredActivityNode processSAN = (StructuredActivityNode)processOutputPinSet.eContainer();
			Activity interfaceActivity  = getInterfaceActivityIfAny(processSAN);
			if(interfaceActivity != null){
				//The interface activity can only have one inputpinset
				EList opsList = interfaceActivity.getImplementation().getOutputPinSet();
				for(Object eachOPS:opsList){
					if(eachOPS instanceof OutputPinSet){
						OutputPinSet ops = (OutputPinSet)eachOPS;
						if(ops.getName().equalsIgnoreCase(processOutputPinSet.getName())){
							interfaceActivityOPS = ops;
							break;
						}
					}
				}
			}
		}
		return interfaceActivityOPS;
	}

	/**
	 * check if the source of the wsdl definition is a process interface definition.
	 * @param def
	 * @return
	 */
	public static boolean isProcessInterfaceDefinition(Definition def){
		if(def == null){
			return false;
		}
		NamedElement bomSrc = CrossProjectReferenceUtil.getBOMSOurceObjectFor(def);
		ExportSession session = BpelOptionsUtil.getExportSession();
		if(session != null){
			Map sessionContext = session.getContext();
			List<Activity> interfaceActivityRegistry = null;
			if(sessionContext.get(INTERFACE_ACTIVITY_REGISTRY) instanceof List){
				interfaceActivityRegistry = (List)sessionContext.get(INTERFACE_ACTIVITY_REGISTRY);
				if(interfaceActivityRegistry != null && !interfaceActivityRegistry.isEmpty()){
					for(Activity eachAct:interfaceActivityRegistry){
						if(eachAct != null && bomSrc instanceof ExternalService){
							Package pkg  = eachAct.getOwningPackage();
							while(!(pkg instanceof ExternalService) && pkg != null){
								pkg = pkg.getOwningPackage();
							}
							if(pkg == bomSrc){
								return true;
							}
						}else if(bomSrc instanceof Activity){
							return interfaceActivityRegistry.contains(bomSrc);
						}
					}
				}
			}
		}
		return false;
	}

	public static boolean isBusinessServiceInterface(Definition def){
		if(def != null){
			NamedElement bomSrc = CrossProjectReferenceUtil.getBOMSOurceObjectFor(def);
			if(bomSrc instanceof Activity ){
				Action action = ((Activity)bomSrc).getImplementation();
				return 	BomUtils.isServiceOperation(action);
			}else if(bomSrc instanceof ExternalService){
				return true;
			}
		}
		return false;
	}

	/**
	 * Create a copy of the existing partner links and update the partner link in the process. The reason to do in this way is
	 * due to the fact that the setEnclosingDefinition() method will lead to duplicated
	 * Meanwhile, update the partner link type in process
	 * Role created for portType in the process interface shallow interface *Artifacts.wsdl
	 * @param partnerLinks
	 * @return
	 */
	public static List<PartnerLinkType> clonePartnerLink(List<Object> partnerLinks,Definition artifactsDef,com.ibm.wbit.bpel.Process process){
		List<PartnerLinkType> clonePartnerLinkTypes = new ArrayList<PartnerLinkType>();			
		for(Object eachplt:partnerLinks){
			PartnerLinkType plt = (PartnerLinkType)eachplt;

			PartnerLinkType serviceLink = PartnerlinktypeFactory.eINSTANCE.createPartnerLinkType();	
			serviceLink.setName(plt.getName());	

			Role role = PartnerlinktypeFactory.eINSTANCE.createRole();			
			Role originalRole = (Role)plt.getRole().get(0);
			role.setName(originalRole.getName());

			RolePortType rpt = PartnerlinktypeFactory.eINSTANCE.createRolePortType();
			rpt.setName(originalRole.getPortType().getName());
			role.setPortType(rpt);
			serviceLink.getRole().add(role);

			serviceLink.setEnclosingDefinition(artifactsDef);
			role.setEnclosingDefinition(artifactsDef);
			rpt.setEnclosingDefinition(artifactsDef);

			PartnerLinks processPartnerLinks = process.getPartnerLinks();
			EList linkList = processPartnerLinks.getChildren();
			if(linkList != null && !linkList.isEmpty()){
				for(Object eachLink:linkList){
					if(eachLink instanceof PartnerLink){
						PartnerLink pl = (PartnerLink)eachLink;
						PartnerLinkType processPLT = pl.getPartnerLinkType();
						if(processPLT.getName() != null && processPLT.getName().equals(serviceLink.getName())){
							pl.setPartnerLinkType(serviceLink);
							break;
						}
					}
				}
			}			
			clonePartnerLinkTypes.add(serviceLink);
		}		
		return clonePartnerLinkTypes;
	}

	public static ComponentRule getComponentRuleForCbaAction(TransformationContext context, Action action) {
		if(context.get(COMPONENT_RULE_FOR_INTERFACE_PORTTYPE) instanceof Map){
			Map<NamedElement,ComponentRule> compRuleRegister = (Map<NamedElement,ComponentRule>)context.get(COMPONENT_RULE_FOR_INTERFACE_PORTTYPE);
			if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
				Activity activity = (Activity)action.eContainer();
				Package serviceInterface = activity.getOwningPackage();
				return compRuleRegister.get(serviceInterface);
			}else{
				return compRuleRegister.get(action);
			}
		}
		return null;
	}

	public static void registerComponentRuleForCbaAction(TransformationContext context, ComponentRule rule, Action action) {
		if(context == null || rule == null || action == null){
			return;
		}	
		Map<NamedElement,ComponentRule> compRuleRegister = null;
		if(context.get(COMPONENT_RULE_FOR_INTERFACE_PORTTYPE) instanceof Map){
			compRuleRegister = (Map<NamedElement,ComponentRule>)context.get(COMPONENT_RULE_FOR_INTERFACE_PORTTYPE);
			if(!compRuleRegister.containsKey(action)){
				if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
					Activity activity = (Activity)action.eContainer();
					Package serviceInterface = activity.getOwningPackage();
					compRuleRegister.put(serviceInterface, rule);
				}else{
					compRuleRegister.put(action,rule);
				}
			}
		}else{
			compRuleRegister = new HashMap<NamedElement,ComponentRule>(); 
			if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
				Activity activity = (Activity)action.eContainer();
				Package serviceInterface = activity.getOwningPackage();
				compRuleRegister.put(serviceInterface, rule);
			}else{
				compRuleRegister.put(action,rule);
			}
			context.put(COMPONENT_RULE_FOR_INTERFACE_PORTTYPE, compRuleRegister);
		}
	}

	/**
	 * Compare the cba's implementation type with the global element's implementation
	 * @param abstractProcDefRule
	 * @param action
	 * @param rule
	 * @return
	 */
	public static boolean isCBAImplementationChanged(AbstractProcDefRule abstractProcDefRule, Action action,ComponentRule rule) {	
		boolean implementationChanged = false;
		if(abstractProcDefRule instanceof CallBehaviorActionRule){			
			if(rule != null && !rule.getTarget().isEmpty()){
				for(Object eachTarget:rule.getTarget()){
					if(eachTarget instanceof DocumentRoot && ((DocumentRoot)eachTarget).getComponent() != null){
						Component component = ((DocumentRoot)eachTarget).getComponent();
						Implementation baseImple = component.getImplementation();					
						InputPinSet inputSet = (InputPinSet)abstractProcDefRule.getSource().get(0);
						if(inputSet.getAction() instanceof CallBehaviorAction){
							CallBehaviorAction cba = (CallBehaviorAction)inputSet.getAction();
							AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
							//Find out the CBA's technical attribute implementation type						
							Object implementation = avp.getAttributeValueForType(cba, AttributeNameConstants.IMPLEMENTATION);
							if (implementation != null && !implementation.equals(AttributeNameConstants.IMPLEMENTATION)) {							
								if(implementation instanceof HumanTask){									
									if(!(baseImple instanceof TaskImplementation)){
										implementationChanged = true;
									}
								} else if (implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.Process) {
									if(!(baseImple instanceof ProcessImplementation)){
										implementationChanged = true;
									}
								} else if (implementation instanceof Java) {
									if(!(baseImple instanceof JavaImplementation)){
										implementationChanged = true;
									}
								} else if (implementation instanceof BusinessRule) {
									if(!(baseImple instanceof BusinessRuleGroupImplementation)){
										implementationChanged = true;
									}

								} else if (implementation instanceof AdaptiveEntity) {
									if(!(baseImple instanceof AdaptiveEntityImplementation)){
										implementationChanged = true;
									}

								} else if (implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.SCA) {
									if(!(baseImple instanceof SCAImportBinding)){
										implementationChanged = true;
									}

								}
								else if(implementation instanceof DynamicAssembler){	
									if(!(baseImple instanceof com.ibm.ws.fabric.da.scdl.DynamicAssembler)){
										implementationChanged = true;
									}																		
								}
								else if(implementation instanceof WebServicesImpl){
									if(!(baseImple instanceof MediationFlowImplementation)){
										implementationChanged = true;
									}			
								}								
							}
						}
					}
				}			
			}			
		}
		return implementationChanged;
	}
	/**
	 * Check whether the CBA's implementation type has been changed as different from their global element's
	 * implementation type
	 * @param action
	 * @param cba
	 * @return
	 */
	public static boolean isCBAImplementationChanged(Action action, CallBehaviorAction cba){
		if(action != null && cba != null){
			//The component attribute value provider
			AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
			Object globalImplementation = avp.getAttributeValueForType(action, AttributeNameConstants.IMPLEMENTATION);
			Object cbaImplementation = avp.getAttributeValueForType(cba, AttributeNameConstants.IMPLEMENTATION);
			if(globalImplementation.getClass()== cbaImplementation.getClass()){
				return false;
			}else{
				return true;
			}
		}
		return false;
	}

	/**
	 * Retrieve the source CBA from the rule
	 * @param rule
	 * @return
	 */
	public static CallBehaviorAction getCBAForRule(TransformationRule rule){
		if(rule instanceof CallBehaviorActionRule){
			CallBehaviorActionRule cbaRule = (CallBehaviorActionRule)rule;
			if(cbaRule.getSource().get(0) instanceof InputPinSet){
				InputPinSet ips = (InputPinSet)cbaRule.getSource().get(0);
				if(ips.getAction() instanceof CallBehaviorAction){
					return (CallBehaviorAction)ips.getAction();
				}
			}
		}
		return null;
	}

	/**
	 * Determine whether we need to use the CBA's implementation
	 * @param compRule
	 * @param implementationChanged
	 * @return
	 */
	public static boolean shouldUseCBAImplementation(ComponentRule compRule,boolean implementationChanged){
		if(compRule != null){
			int numberOfSrc = compRule.getSource().size();		
			return numberOfSrc >= 3 && ((compRule.getSource().get(numberOfSrc-1) instanceof CallBehaviorAction) && implementationChanged);
		}
		return false;		
	}

	/**
	 * We support the CBA implementation type overwrite for global task/service, and business service
	 * @param action
	 * @return
	 */
	public static boolean isSupportedCBA(Action action){
		if(action != null){
			if(BomUtils.isReusableTask(action)||BomUtils.isReusableService(action)||BomUtils.isServiceOperation(action)){
				return true;
			}
		}
		return false;
	}

	/**
	 * If the returned value is true, we need to use the implementation on the CBA. Otherwise, we should query the global element's
	 * implementation
	 * @param cba The callbehavioraction of the global element
	 * @return
	 */
	public static boolean isOverriden(CallBehaviorAction cba){		
		if(cba != null){
			EList descriptors = cba.getOwnedDescriptor();
			EList classifications = cba.getResponsibleOrganization();
			if(descriptors != null && !descriptors.isEmpty()){
				for(Object eachDescriptor:descriptors){
					if(eachDescriptor instanceof ServiceAttributes){
						ServiceAttributes serviceAttributes = (ServiceAttributes)eachDescriptor;
						if(serviceAttributes.getServiceComponent() != null){
							//If the implementation is null, which means the user sets the implementation as "NONE"
							//Otherwise, the user needs to override the implementation as different kind of type
							return true;
						}
					}
				}
				//defect 84336
				//we did not choose to override but the actual implementation on the CBA is NONE or Webservice and the list of
				//classifications on the Action to be called are different then the list on the CBA, in this case override.
			}else if(classifications != null && !classifications.isEmpty()){
				Action toBeProcessedAction = null;
				Behavior behavior = ((CallBehaviorAction)cba).getBehavior();
				if (behavior instanceof Activity){
					toBeProcessedAction = ((Activity)behavior).getImplementation();
					//defect 89077 exempt the check for global process, global human task, and global business rule task
					if(toBeProcessedAction != null){
						if(BomUtils.isReusableHumanTask(toBeProcessedAction) || BomUtils.isReusableBusinessRuleTask(toBeProcessedAction) || BomUtils.isTopLevelProcess(toBeProcessedAction)){
							return false;
						}
						AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
						Object implementation = avp.getAttributeValueForType(toBeProcessedAction, AttributeNameConstants.IMPLEMENTATION);
						//if (implementation != null && !implementation.equals(AttributeNameConstants.IMPLEMENTATION)) {
						if(implementation == null || implementation instanceof WebServicesImpl){
							EList actionClassifications = toBeProcessedAction.getResponsibleOrganization();
							//if(classifications.size()== actionClassifications.size()){
							List<Object> diff = new LinkedList<Object>();
							diff.addAll(classifications);
							diff.removeAll(actionClassifications);
							if(diff.size() >0){
								for(Object classifier: diff){
									if(classifier instanceof OrganizationUnit){
										List<Comment> comments= ((OrganizationUnit)classifier).getOwnedComment();
										for(Comment comment: comments){
											if(comment.getBody().startsWith(PredefTypesConst.CATEGORY_CLASSIFICATION_COMMENT)){
												return true;
											}
										}
									}
								}										
								return false;
							}						
						}
					}
				}
			}
		}
		return false;
	}

	/**
	 * Determine whether we need to use the portType to search partner, this applies to global HMT/BR CBA, possibly process CBA
	 * @param action
	 * @return
	 */
	public static boolean searchPartnerByPortType(Action action){
		if(action != null){
			return BomUtils.isReusableBusinessRuleTask(action)||BomUtils.isReusableHumanTask(action);
		}
		return false;
	}

	/**
	 * Partner Link Rule has to be per process flow rule, i.e. per process
	 * @param context
	 * @param processFlowRule
	 * @param action
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static PartnerLinkRule getPartnerLinkRuleForAction(TransformationContext context, TransformationRule processFlowRule, Action action) {
		if(context.get(PARTNER_LINK_RULE_FOR_ACTION) instanceof Map){
			Map<ProcessFlowRule, Map<NamedElement,PartnerLinkRule>> prtnerLinkRuleRegister = (Map<ProcessFlowRule, Map<NamedElement,PartnerLinkRule>>)context.get(PARTNER_LINK_RULE_FOR_ACTION);
			if(prtnerLinkRuleRegister != null && prtnerLinkRuleRegister.get(processFlowRule) instanceof Map){
				Map<NamedElement,PartnerLinkRule> registryMap = (Map<NamedElement,PartnerLinkRule>) prtnerLinkRuleRegister.get(processFlowRule);
				if(BomUtils.isServiceOperation(action)&& action.eContainer() instanceof Activity){				
					Activity activity = (Activity)action.eContainer();
					Package serviceInterface = activity.getOwningPackage();						
					return registryMap.get(serviceInterface);
				}else{
					return registryMap.get(action);
				}
			}			
		}
		return null;
	}

	/**
	 * Register the partner link rule with the process flow rule, which represents the process. This is to support the CBA implementation override.
	 * @param context
	 * @param processFlowRule
	 * @param partnerLinkRule
	 * @param action
	 */
	@SuppressWarnings("unchecked")
	public static void registerPartnerLinkRuleForAction(TransformationContext context,  TransformationRule processFlowRule,PartnerLinkRule partnerLinkRule,	Action action) {
		if(context == null || partnerLinkRule == null || action == null || !(processFlowRule instanceof ProcessFlowRule)){
			return;
		}	
		//we should base off the serviceinterface to handle multiple operations are reused 
		Map<TransformationRule, Map<NamedElement,PartnerLinkRule>> prtnerLinkRuleRegister = null;
		if(context.get(PARTNER_LINK_RULE_FOR_ACTION) instanceof Map){
			prtnerLinkRuleRegister = (Map<TransformationRule, Map<NamedElement,PartnerLinkRule>>)context.get(PARTNER_LINK_RULE_FOR_ACTION);
			if(!prtnerLinkRuleRegister.containsKey(processFlowRule)){
				Map<NamedElement,PartnerLinkRule> registryMap = new HashMap<NamedElement,PartnerLinkRule>();
				if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
					Activity activity = (Activity)action.eContainer();
					Package serviceInterface = activity.getOwningPackage();
					registryMap.put(serviceInterface, partnerLinkRule);
				}else{
					registryMap.put(action, partnerLinkRule);
				}
				prtnerLinkRuleRegister.put(processFlowRule,registryMap);
			}
		}else{
			prtnerLinkRuleRegister = new HashMap<TransformationRule, Map<NamedElement,PartnerLinkRule>>(); 
			Map<NamedElement,PartnerLinkRule> registryMap = new HashMap<NamedElement,PartnerLinkRule>();
			if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
				Activity activity = (Activity)action.eContainer();
				Package serviceInterface = activity.getOwningPackage();
				registryMap.put(serviceInterface, partnerLinkRule);
			}else{
				registryMap.put(action, partnerLinkRule);
			}
			prtnerLinkRuleRegister.put(processFlowRule,registryMap);
			context.put(PARTNER_LINK_RULE_FOR_ACTION, prtnerLinkRuleRegister);
		}

	}

	/**
	 * Partner Rule has to be per process flow rule, i.e. per process
	 * @param context
	 * @param processFlowRule
	 * @param action
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static PartnerRule getPartnerRuleForAction(TransformationContext context, TransformationRule processFlowRule,Action action) {
		if(context.get(PARTNER_RULE_FOR_ACTION) instanceof Map){
			Map<ProcessFlowRule, Map<NamedElement,PartnerRule>> prtnerLinkRuleRegister = (Map<ProcessFlowRule, Map<NamedElement,PartnerRule>>)context.get(PARTNER_RULE_FOR_ACTION);
			if(prtnerLinkRuleRegister != null && prtnerLinkRuleRegister.get(processFlowRule) instanceof Map){
				Map<NamedElement,PartnerRule> registryMap = (Map<NamedElement,PartnerRule>) prtnerLinkRuleRegister.get(processFlowRule);
				if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
					Activity activity = (Activity)action.eContainer();
					Package serviceInterface = activity.getOwningPackage();
					return registryMap.get(serviceInterface);
				}else{
					return registryMap.get(action);
				}
			}			
		}
		return null;
	}
	/**
	 * Register the partner rule with the process flow rule, which represents the process. This is to support the CBA implementation override.
	 * @param context
	 * @param processFlowRule
	 * @param partnerRule
	 * @param action
	 */
	@SuppressWarnings("unchecked")
	public static void registerPartnerRuleForAction(TransformationContext context, TransformationRule processFlowRule,PartnerRule partnerRule,Action action) {
		if(context == null || partnerRule == null || action == null || !(processFlowRule instanceof ProcessFlowRule)){
			return;
		}	
		Map<TransformationRule, Map<NamedElement,PartnerRule>> prtnerLinkRuleRegister = null;
		if(context.get(PARTNER_RULE_FOR_ACTION) instanceof Map){
			prtnerLinkRuleRegister = (Map<TransformationRule, Map<NamedElement,PartnerRule>>)context.get(PARTNER_RULE_FOR_ACTION);
			if(!prtnerLinkRuleRegister.containsKey(processFlowRule)){
				Map<NamedElement,PartnerRule> registryMap = new HashMap<NamedElement,PartnerRule>();
				if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
					Activity activity = (Activity)action.eContainer();
					Package serviceInterface = activity.getOwningPackage();
					registryMap.put(serviceInterface, partnerRule);
				}else{
					registryMap.put(action, partnerRule);
				}
				prtnerLinkRuleRegister.put(processFlowRule,registryMap);
			}
		}else{
			prtnerLinkRuleRegister = new HashMap<TransformationRule, Map<NamedElement,PartnerRule>>(); 
			Map<NamedElement,PartnerRule> registryMap = new HashMap<NamedElement,PartnerRule>();
			if(BomUtils.isServiceOperation(action) && action.eContainer() instanceof Activity){
				Activity activity = (Activity)action.eContainer();
				Package serviceInterface = activity.getOwningPackage();
				registryMap.put(serviceInterface, partnerRule);
			}else{
				registryMap.put(action, partnerRule);
			}
			prtnerLinkRuleRegister.put(processFlowRule,registryMap);
			context.put(PARTNER_RULE_FOR_ACTION, prtnerLinkRuleRegister);
		}

	}

	public static List<PartnerLinkType> getPLTFrProcessFlow(TransformationContext context,ProcessFlowRule processFlowRule){
		List<PartnerLinkType> pltList = null;
		if(context.get(PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY) instanceof Map){
			Map<ProcessFlowRule,List<PartnerLinkType>> register = (Map<ProcessFlowRule,List<PartnerLinkType>>)context.get(PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY);			
			if(processFlowRule != null && register.get(processFlowRule) instanceof List){
				pltList = register.get(processFlowRule);
			}
		}
		return pltList;
	}

	/**
	 * Register the process the partner link types on the process interface to support process variation
	 * @param context
	 * @param processInterfaceRule
	 * @param plt
	 */
	public static void registerProcessFlowForPLT(TransformationContext context,ProcessInterfaceRule processInterfaceRule,	PartnerLinkType plt) {
		if(context == null || processInterfaceRule == null || plt == null ){
			return;
		}
		//Find out the interface rule's parent rule - process flow rule
		ProcessFlowRule pfr = null;
		TransformationRule parent = processInterfaceRule.getParentRule();
		while(!(parent instanceof ProcessFlowRule) && parent != null){
			parent = parent.getParentRule();
		}
		if(parent instanceof ProcessFlowRule){
			pfr = (ProcessFlowRule)parent;
			if(!pfr.getSource().isEmpty()){
				Activity activity = (Activity)pfr.getSource().get(0);	
				Activity interfaceActivity = getInterfaceActivityIfAny(activity.getImplementation());
				if(interfaceActivity == null){
					return;
				}
			}

		}
		Map<ProcessFlowRule,List<PartnerLinkType>> register = null;
		List<PartnerLinkType> pltList = null;
		if(context.get(PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY) instanceof Map){
			register = (Map<ProcessFlowRule,List<PartnerLinkType>>)context.get(PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY);			
			if(pfr != null && register.get(pfr) instanceof List){
				pltList = (List<PartnerLinkType>)register.get(pfr);
				if(!pltList.contains(plt)){
					pltList.add(plt);
				}
			}else{
				pltList = new ArrayList<PartnerLinkType>();
				pltList.add(plt);
				register.put(pfr, pltList);
			}
		}else{
			register = new HashMap<ProcessFlowRule, List<PartnerLinkType>>();
			pltList = new ArrayList<PartnerLinkType>();
			pltList.add(plt);
			register.put(pfr, pltList);
			context.put(PROCESS_FLOW_TO_PARTNER_LINK_TYPE_REGISTRY, register);
		}		
	}
}
