package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.List;

import com.ibm.btools.bom.model.artifacts.NamedElement;

/**
 * This class is used to keep track of a path of nodes that are encountered while searching
 * for an exclusive or concurrent merge point. 
 */
public class NodePath {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private List fVisited;
	private int fLastVisitedCount;
	
	/* This NamedElement is the BOM object from which the last step was taken, or next step 
	 * will be taken.  */
	private NamedElement fElement;
	
	/**
	 * Default constructor.
	 */
	public NodePath(){
		fVisited = new ArrayList();
		fLastVisitedCount = 0;
		fElement = null;
	}
	
	public void setElement(NamedElement element){
		fElement = element;
	}
	
	/**
	 * Returns the NamedElement stored by this NodePath
	 * 
	 * @return NamedElement
	 */
	public NamedElement getElement(){
		return fElement;
	}
	
	/**
	 * When used for concurrent merge point identification, this method records the input sets 
	 * that were visited along the way, or the containing SAN if no merge
	 * point could be found earlier.
	 * 
	 * @param visitedNodes - The input sets and/or the containing SAN visited
	 */
	public void recordVisited(List visitedNodes){
		/**
		 *  TODO: Change this to record just the UUID in the real version, for performance (?)
		 */ 
		fVisited.addAll(visitedNodes);
		fLastVisitedCount = visitedNodes.size();
	}
	
	/**
	 * @see recordVisited(List)
	 * 
	 * @param visitedNode - The input set or output set visited
	 */
	public void recordVisited(NamedElement visitedNode){
		/**
		 *  TODO: Change this to record just the UUID in the real version, for performance (?)
		 */ 
		fVisited.add(visitedNode);
		fLastVisitedCount = 1;
	}	

	/**
	 * Makes a new copy of the nodePath
	 * 
	 * @return Object The new NodePath object 
	 */
	public Object clone(){
		NodePath newNodePath = new NodePath();
		newNodePath.recordVisited(fVisited);
		return newNodePath;
	}

	/**
	 * Returns the list of NamedElements that were last recorded as visited.
	 * 
	 * @return List The list of last-visited NamedElements 
	 */
	public List getLastVisited(){
		return fVisited.subList(fVisited.size()-fLastVisitedCount, fVisited.size());	
	}
	
	/**
	 * Returns the complete list of NamedElements visited by this NodePath
	 * 
	 * @return List - The list of NamedElements
	 */
	public List getNodeHistory(){
		return fVisited;
	}
}
