package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.beans.Encoder;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.resources.BulkResourceRequirement;
import com.ibm.btools.bom.model.resources.IndividualResourceRequirement;
import com.ibm.btools.bom.model.resources.ResourceRequirement;
import com.ibm.btools.expression.bom.function.ResourceRequirementCriteriaFunctions;
import com.ibm.btools.ie.ui.ilm.model.ConvertToTESettings;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.TransformationSession;

/**
 * This class contains methods that are used in staff assignment.
 */
public class StaffUtil implements ResourceRequirementCriteriaFunctions{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * Class that keeps all constants related to predefined Resource types ans SSS Verbs.
	 */
	public static class StaffConstants{	
		//Predefined resource type names
//		public static final String STAFF_CATEGORY 							= "Staff_Category";
//		public static final String PERSON_CATEGORY 							= "Person_Category";
//		public static final String PERSON 									= "Person";
//		public static final String STAFF 									= "Staff";
		//public static final String DEFAULT_PACKAGE 							= "Predefined Resources";
//		public static final String DEFAULT_PACKAGE 							= "RootResourceModel";
		public static final String STAFF_IDENTIFIER							= "personId";
		public static final String GROUP_IDENTIFIER							= "organizationID";
		public static final String ORGANIZATION_NAME						= "organization_name";
		//Everybody criteria
		public final static String EVERYBODY 								= "Everybody";
		
		//Members of Role criteria
		public final static String SSS_MEMBERS_OF_ROLE 						= "Role Members";
		public final static String SSS_ROLE_NAME 							= "RoleName";
		public final static String SSS_INCLUDE_NESTED_ROLES 				= "IncludeNestedRoles";
		public final static String SSS_DOMAIN 								= "Domain";
		public final static String SSS_ALTERNATIVE_ROLE_NAME1 				= "AlternativeRoleName1";
		public final static String SSS_ALTERNATIVE_ROLE_NAME2 				= "AlternativeRoleName2";
		
		//Users by UserID criteria	
		public final static String SSS_USERS_BY_USERID						= "Users by user ID";
		public final static String SSS_USER_ID 								= "UserID";
		public final static String SSS_ALTERNATIVE_ID1 						= "AlternativeID1";
		public final static String SSS_ALTERNATIVE_ID2 						= "AlternativeID2";
		// d165011
		public final static String SSS_USER_RECORDS_BY_USERID				= "User Records by user ID";

		 // Manager of employee criteria
		public final static String SSS_MGR_OF_EMPL_FUNCTION					= "Manager of Employee";
		public final static String SSS_MGR_OF_EMPL_EMPLNAME_ARG				= "EmployeeName";
	 	
		 // Manager of employee by user id criteria
		public final static String SSS_MGR_OF_EMPL_BY_USERID_FUNCTION		= "Manager of Employee by user ID";
		public final static String SSS_MGR_OF_EMPL_BY_USERID_USERID_ARG		= "EmployeeUserID";

		// Person search criteria
		public final static String SSS_PERSON_SEARCH_FUNCTION				= "Person Search";
		public final static String SSS_PERSON_SEARCH_USERID_ARG				= "UserID";
		public final static String SSS_PERSON_SEARCH_PROFILE_ARG			= "Profile";
		public final static String SSS_PERSON_SEARCH_LASTNAME_ARG			= "LastName";
		public final static String SSS_PERSON_SEARCH_FIRSTNAME_ARG			= "FirstName";
		public final static String SSS_PERSON_SEARCH_MIDDLENAME_ARG			= "MiddleName";
		public final static String SSS_PERSON_SEARCH_EMAIL_ARG				= "Email";
		public final static String SSS_PERSON_SEARCH_COMPANY_ARG			= "Company";
		public final static String SSS_PERSON_SEARCH_DISPLAYNAME_ARG		= "DisplayName";
		public final static String SSS_PERSON_SEARCH_SECRETARY_ARG			= "Secretary";
		public final static String SSS_PERSON_SEARCH_ASSISTANT_ARG			= "Assistant";
		public final static String SSS_PERSON_SEARCH_MANAGER_ARG			= "Manager";
		public final static String SSS_PERSON_SEARCH_DEPARTMENT_ARG			= "Department";
		public final static String SSS_PERSON_SEARCH_EMPLOYEEID_ARG			= "EmployeeID";
		public final static String SSS_PERSON_SEARCH_TAXPAYERID_ARG			= "TaxPayerID";
		public final static String SSS_PERSON_SEARCH_PHONE_ARG				= "Phone";
		public final static String SSS_PERSON_SEARCH_FAX_ARG				= "Fax";
		public final static String SSS_PERSON_SEARCH_GENDER_ARG				= "Gender";
		public final static String SSS_PERSON_SEARCH_TIMEZONE_ARG			= "Timezone";
		public final static String SSS_PERSON_SEARCH_PREFLANGUAGE_ARG		= "PreferredLanguage";
		
		// Person by name criteria
		public final static String SSS_PERSON_BY_NAME_FUNCTION				= "Users";
		public final static String SSS_PERSON_BY_NAME_NAME_ARG				= "Name";
		public final static String SSS_PERSON_BY_NAME_ALTNAME1_ARG			= "AlternativeName1";
		public final static String SSS_PERSON_BY_NAME_ALTNAME2_ARG	  		= "AlternativeName2";
	  	
		//Group members criteria
		public final static String SSS_GROUP_MEMBERS_FUNCTION				= "Group Members";
		public final static String SSS_GROUP_MEMBERS_NAME_ARG 				= "GroupName";
		public final static String SSS_GROUP_MEMBERS_INCLUDE_SUBGROUPS_ARG 	= "IncludeSubgroups";	
		public final static String SSS_GROUP_MEMBERS_ALTNAME1_ARG			= "AlternativeGroupName1";
		public final static String SSS_GROUP_MEMBERS_ALTNAME2_ARG	  		= "AlternativeGroupName2";
		
		
		//Group search criteria
		 public final static String SSS_GROUP_SEARCH_FUNCTION				= "Group Search";
		 public final static String SSS_GROUP_SEARCH_GROUPID_ARG			= "GroupID";
		 public final static String SSS_GROUP_SEARCH_TYPE_ARG			    = "Type";
		 public final static String SSS_GROUP_SEARCH_INDUSTRY_TYPE_ARG		= "IndustryType";
		 public final static String SSS_GROUP_SEARCH_BUSINESS_TYPE_ARG		= "BusinessType";
		 public final static String SSS_GROUP_SEARCH_GEOGRAPHIC_LOCATION_ARG= "GeographicLocation";
		 public final static String SSS_GROUP_SEARCH_AFFILIATES_ARG			= "Affiliates";
		 public final static String SSS_GROUP_SEARCH_DISPLAYNAME_ARG		= "DisplayName";
		 public final static String SSS_GROUP_SEARCH_SECRETARY_ARG			= "Secretary";
		 public final static String SSS_GROUP_SEARCH_ASSISTANT_ARG			= "Assistant";
		 public final static String SSS_GROUP_SEARCH_MANAGER_ARG			= "Manager";
		 public final static String SSS_GROUP_SEARCH_DEPARTMENT_ARG			= "BusinessCategory";
		 public final static String SSS_GROUP_SEARCH_PARENT_COMPANY_ARG		= "ParentCompany";
		 
		 public final static String SSS_DEPARTMENT_MEMBERS					= "Department Members";
		 public final static String SSS_DEPARTMENT_MEMBERS_INCLUDENESTEDEPARTMENTS_ARG="IncludeNestedDepartments";
		 public final static String SSS_DEPARTMENT_MEMBERS_PARAM_DEPARTMENT_NAME = "DepartmentName";
		 public final static String SSS_DEPARTMENT_MEMBERS_PARAM_ALTERNATIVE_DEPARTMENT_NAME1 = "AlternativeDepartmentName1";
		 public final static String SSS_DEPARTMENT_MEMBERS_PARAM_ALTERNATIVE_DEPARTMENT_NAME2 = "AlternativeDepartmentName2";
		 
		// Constants for additional query params
		public static final String QUERY_PARAM_VALUE_FALSE = "false"; //$NON-NLS-1$
			  	
	}
	
	public static class VMMConstants{	
		
		public static final String VMM_DEFAULT_SUFFIX = ",o=defaultWIMFileBasedRealm";
		public static final String VMM_PERSON = "VMM_PersonAccount";
		public static final String VMM_GROUP_ROLE = "VMM_Group_Role";
		public static final String VMM_GROUP_ORG = "VMM_Group_Org";
		public static final String VMM_SCRIPT_FILENAME = "WBM_ResourcesVMMCreateScript_";
		public static final String VMM_SCRIPT_SESSION_CLEANUP_FILENAME = "WBM_ResourcesVMMCleanupSessionScript_";
		public static final String VMM_SCRIPT_CLEANUP_FILENAME = "WBM_ResourcesVMMCleanupAllScript";
		public static final String VMM_SCRIPT_EXTENSION = ".py";
			  	
	}
	//a list fully qualified names of resource types supported by TE. 
	//TODO change when ResourceManager is ready	
	static List predefinedListTypes = new LinkedList();
	
	static {
		predefinedListTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.STAFF_CATEGORY_TYPE_NAME );
		predefinedListTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.STAFF_TYPE_NAME);
		predefinedListTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.PERSON_TYPE_NAME );
		predefinedListTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.PERSON_CATEGORY_TYPE_NAME);
		
//		predefinedListTypes.add(StaffConstants.DEFAULT_PACKAGE + "/" + StaffConstants.STAFF);
//		predefinedListTypes.add(StaffConstants.DEFAULT_PACKAGE + "/" + StaffConstants.STAFF_CATEGORY);
//		predefinedListTypes.add(StaffConstants.DEFAULT_PACKAGE + "/" + StaffConstants.PERSON);
//		predefinedListTypes.add(StaffConstants.DEFAULT_PACKAGE + "/" + StaffConstants.PERSON_CATEGORY);

	}
	static List predefinedOrgTypes = new LinkedList();
	static {
		predefinedOrgTypes.add(PredefTypesConst.ROOT_ORG_MODEL_NAME +"/" + PredefTypesConst.ORGANIZATION_CATEGORY_TYPE_NAME );
		predefinedOrgTypes.add(PredefTypesConst.ROOT_ORG_MODEL_NAME +"/" + PredefTypesConst.ORGANIZATION_TYPE_NAME );
	}

	static List predefinedBulkTypes = new LinkedList();
	static {
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.COMMUNICATION_SERVICE_TYPE_NAME);
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.EQUIPMENT_TYPE_NAME );
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.FACILITY_TYPE_NAME);
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.GENERAL_SERVICE_TYPE_NAME);
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.MACHINE_TYPE_NAME);
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.TOOL_TYPE_NAME);
		predefinedBulkTypes.add(PredefTypesConst.ROOT_RES_MODEL_NAME +"/" + PredefTypesConst.RESOURCE_CATEGORY_TYPE_NAME);		
	}
	
	//list of supported predefined functions 
	static List predefinedResourceRequirement = new LinkedList();
	
	static{
		predefinedResourceRequirement.add(MGR_OF_EMPL_FUNCTION);
		predefinedResourceRequirement.add(MGR_OF_EMPL_BY_USERID_FUNCTION);
		predefinedResourceRequirement.add(PERSON_SEARCH_FUNCTION);
		predefinedResourceRequirement.add(PERSON_BY_NAME_FUNCTION);
		//TODO to add support to ORG
		predefinedResourceRequirement.add(GROUP_MEMBERS_FUNCTION);
		predefinedResourceRequirement.add(GROUP_SEARCH_FUNCTION);
		predefinedResourceRequirement.add(MGR_OF_ORG_FUNCTION);
	}
	
	//list of supported predefined FDL functions
	static List predefinedFDLResourceRequirement = new LinkedList();
	static{		
		predefinedFDLResourceRequirement.add(GROUP_MEMBERS_FUNCTION);
		predefinedFDLResourceRequirement.add(GROUP_SEARCH_FUNCTION);
		predefinedFDLResourceRequirement.add(MGR_OF_ORG_FUNCTION);
	}
	
	/**
	 * Returns a list of predefined identifiers for the predefined resource requirements contraints.
	 * @return list
	 */
	public static List getPredefinedResourceRequirementConstraints(){
		return predefinedResourceRequirement;
	}
	
	/**
	 * Returns a list of predefined identifiers for the predefined FDL resource requirements contraints.
	 * @return list
	 */
	public static List getPredefinedFDLResourceRequirementConstraints(){
		return predefinedFDLResourceRequirement;
	}	
	
	/**
	 * Returns the canonical name of this type. 
	 * @param type BOM Type (normally a resource type)
	 * @return String canonical name
	 */
	public static String getCanonicalName(PackageableElement type){
		String canonicalName = null;
			if(type.getOwningPackage()!= null){
				Package model = type.getOwningPackage();
				canonicalName = model.getName() + "/" + type.getName();
				Package belongingModel = model.getOwningPackage();
				while (belongingModel != null && !ProcessUtil.isRootPackage(belongingModel)) {
					canonicalName = belongingModel.getName() + "/" + canonicalName;
					belongingModel = belongingModel.getOwningPackage();
			}
		}
		return canonicalName;
	}
	
	/**
	 * Returns true if the action has assigned at least one resource requirement
	 * with predefined resource types (or subclasses of those).
	 * @param action a BOM action. 
	 * @return boolean
	 */
	public static boolean isStaffRequired(Action action){
		if(action.getResourceRequirement()== null || action.getResourceRequirement().isEmpty()){
			return false;	
		}
		List resourceRequirements = action.getResourceRequirement();
		Iterator it = resourceRequirements.iterator();
		while (it.hasNext()){
			ResourceRequirement thisRequirement = (ResourceRequirement)it.next();
			//we do not handle BulkResourceRequirement
			if(!(thisRequirement instanceof BulkResourceRequirement)){
				if(thisRequirement.getResourceType() != null && isPredefinedStaff(thisRequirement.getResourceType())){
					return true;
				}else if(thisRequirement instanceof IndividualResourceRequirement){
					IndividualResourceRequirement individualResource = (IndividualResourceRequirement)thisRequirement;
					if (individualResource.getIndividualResource()!= null && isPredefinedStaff(individualResource.getIndividualResource().getClassifier())){
						return true;
					}
				}
			}
		}
		return false;
	}
	
	public static boolean isPredefinedType(Type type) {
		String canonicalName = StaffUtil.getCanonicalName(type);
		if(predefinedListTypes.contains(canonicalName) || predefinedOrgTypes.contains(canonicalName) || predefinedBulkTypes.contains(canonicalName)){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if the type is one of the predefined Staff Resource or a subclass of those.
	 * @param type BOM Type
	 * @return boolean
	 */	
	public static boolean isPredefinedStaff(Type type){

		String canonicalName = StaffUtil.getCanonicalName(type);
		if(predefinedListTypes.contains(canonicalName) || isKindOfStaff(type)){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns true if the type is one of the predefined Organization or a subclass of those.
	 * @param type BOM Type
	 * @return boolean
	 */	
	public static boolean isPredefinedOrganization(Type type){

		String canonicalName = StaffUtil.getCanonicalName(type);
		if(predefinedOrgTypes.contains(canonicalName) || isKindOfOrg(type)){
			return true;
		}
		return false;
	}	

	/**
	 * Returns true if the type is one of the predefined Organization or a subclass of those.
	 * @param type BOM Type
	 * @return boolean
	 */	
	public static boolean isPredefinedBulkResource(Type type){

		String canonicalName = StaffUtil.getCanonicalName(type);
		if(predefinedBulkTypes.contains(canonicalName) || isKindOfBulkResCat(type)){
			return true;
		}
		return false;
	}	

	/**
	 * Returns true if at least one Type in the input list is a predefined Organization or a subclass of those.
	 * @param types a list of BOM Types
	 * @return boolean
	 */
	 public static boolean isPredefinedOrganization(List types){
	  if (types != null && !types.isEmpty()){
		  Iterator it = types.iterator();
		  while(it.hasNext()){
			  if(isPredefinedOrganization((Type)it.next())){
				  return true;
			  }
		  }
	  }
	  return false;
	 }
		
	/**
	 * Returns true if at least one Type in the input list is a predefined Staff Resource or a subclass of those.
	 * @param types a list of BOM Types
	 * @return boolean
	 */
	 public static boolean isPredefinedStaff(List types){
	  if (types != null && !types.isEmpty()){
		  Iterator it = types.iterator();
		  while(it.hasNext()){
			  if(isPredefinedStaff((Type)it.next())){
				  return true;
			  }
		  }
	  }
	  return false;
	 }
	/**
	 * Returns true if the input type is a subclass of the predefined Resource types. 
	 * @param type BOM Type
	 * @return boolean
	 */ 
	public static boolean isKindOfStaff(Type type){
		List superClassifiers = ((Classifier)type).getSuperClassifier();
		if(superClassifiers != null && !superClassifiers.isEmpty()){
			Iterator it = superClassifiers.iterator();
			while(it.hasNext()){
				Classifier classifier = (Classifier)it.next();
				if(isPredefinedStaff(classifier)){
					return true;
				}
			}
		}
		return false;
	}	
	
	/**
	 * Returns true if the input type is a subclass of the predefined Organization types. 
	 * @param type BOM Type
	 * @return boolean
	 */ 
	public static boolean isKindOfOrg(Type type){
		List superClassifiers = ((Classifier)type).getSuperClassifier();
		if(superClassifiers != null && !superClassifiers.isEmpty()){
			Iterator it = superClassifiers.iterator();
			while(it.hasNext()){
				Classifier classifier = (Classifier)it.next();
				if(isPredefinedOrganization(classifier)){
					return true;
				}
			}
		}
		return false;
	}	

	/**
	 * Returns true if the input type is a subclass of the predefined Organization types. 
	 * @param type BOM Type
	 * @return boolean
	 */ 
	public static boolean isKindOfBulkResCat(Type type){
		List superClassifiers = ((Classifier)type).getSuperClassifier();
		if(superClassifiers != null && !superClassifiers.isEmpty()){
			Iterator it = superClassifiers.iterator();
			while(it.hasNext()){
				Classifier classifier = (Classifier)it.next();
				if(isPredefinedBulkResource(classifier)){
					return true;
				}
			}
		}
		return false;
	}	

	/**
	 * Check if the action is to be generated as inlineTask
	 * @param action
	 * @return
	 */
	public static boolean isInlineTask(Action action) {
		// For 6.1 HumanTask support, we can check the action directly to determine whether we need inline HT or not 
		if (HumanTaskUtil.isUse61HumanTaskSupport()) {
			return HumanTaskUtil.isLocalHumanTask(action);
		}
		if(StaffUtil.isStaffRequired(action) && !(BomUtils.isReusableService(action) || BomUtils.isReusableTask(action))){
			String componentName = (String)TechnicalAttributesHelper.getAttributeValue(action,AttributeNameConstants.COMPONENT_NAME);
			Object implementation = TechnicalAttributesHelper.getAttributeValue(action,AttributeNameConstants.IMPLEMENTATION);
			//is it enough only to check for componentName and implementation? what about component DisplayName?
			if(implementation == null){
				return true;
			}
		}		
		return false;
	}


	/**
	 * Determines if parameter is one requiring unique name format
	 * groupName, roleName, AlternativeRoleName1, AlternativeRoleName2
	 */
	public static boolean isNameParameter(String funcParameterID) {
		return (funcParameterID.equals(StaffConstants.SSS_GROUP_MEMBERS_NAME_ARG) || funcParameterID.equals(StaffConstants.SSS_ROLE_NAME)
				|| funcParameterID.equals(StaffConstants.SSS_ALTERNATIVE_ROLE_NAME1) || funcParameterID.equals(StaffConstants.SSS_ALTERNATIVE_ROLE_NAME2)
				|| funcParameterID.equals(StaffConstants.SSS_GROUP_MEMBERS_ALTNAME1_ARG) || funcParameterID.equals(StaffConstants.SSS_GROUP_MEMBERS_ALTNAME2_ARG)
				|| funcParameterID.equals(StaffConstants.SSS_DEPARTMENT_MEMBERS_PARAM_ALTERNATIVE_DEPARTMENT_NAME1) || funcParameterID.equals(StaffConstants.SSS_DEPARTMENT_MEMBERS_PARAM_ALTERNATIVE_DEPARTMENT_NAME2)
				|| funcParameterID.equals(StaffConstants.SSS_DEPARTMENT_MEMBERS_PARAM_DEPARTMENT_NAME));
	}
	
	/**
	 * Pass person to store into the current context
	 */	
	public static void registerVMMPersonEntity(TransformationContext context, String type, VMMPersonEntry VMMPerson){
		
		TransformationSession session = TransformationEngine.getTransformationSession();
		TransformationSessionUtil.registerVMMPerson(session, VMMPerson);
	}
	
	/**
	 * Pass group from role to store into the current context
	 */
	public static void registerVMMGroupEntityForRole(TransformationContext context, VMMGroupEntry VMMGroup){
		TransformationSession session = TransformationEngine.getTransformationSession();
		TransformationSessionUtil.registerVMMGroupsFromRole(session, VMMGroup);
	}
	
	/**
	 * Pass group from organization to store into the current context
	 */
	public static void registerVMMGroupEntityForOrg(TransformationContext context, VMMGroupEntry VMMGroup){
		TransformationSession session = TransformationEngine.getTransformationSession();
		TransformationSessionUtil.registerVMMGroupsFromOrg(session, VMMGroup); 
	}

	/**
	 * Generate Jython script to add VMM entries
	 * @param timestamp 
	 */
	public static StringBuffer generateVMMJythonScript(List groupEntries, List personEntries, List groupEntriesOrg, String timestamp){
		StringBuffer sb = new StringBuffer();
		
		String WBM_all_groupName = "WBM_generated_all";
		String WBM_all_groupIdentifier = "WBM_generated_all";
		sb.append(WBM_all_groupIdentifier + " = AdminTask.searchGroups('[-cn " + WBM_all_groupName + "]')\n");
		sb.append ("if not " + WBM_all_groupIdentifier + ":\n");
		sb.append("\t" + WBM_all_groupIdentifier + " = AdminTask.createGroup('[-cn " + WBM_all_groupName + "]')\n");
		
		//Create WBM_generated group
		String WBM_groupName = "WBM_generated_session_" + timestamp;
		String WBM_groupIdentifier = "WBM_generated";
		sb.append(WBM_groupIdentifier + " = AdminTask.searchGroups('[-cn " + WBM_groupName + "]')\n");
		sb.append ("if not " + WBM_groupIdentifier + ":\n");
		sb.append("\t" + WBM_groupIdentifier + " = AdminTask.createGroup('[-cn " + WBM_groupName + "]')\n");
		
		sb.append("memberOfGroupResult = AdminTask.getMembersOfGroup('[-uniqueName cn=" + WBM_all_groupName +",o=defaultWIMFileBasedRealm]')\n");
		sb.append("from java.lang import String\n");
		sb.append("result = String(memberOfGroupResult)\n");
		sb.append("if not (result.contains(String(\"" + WBM_groupName +"\"))):\n");
		sb.append("\tAdminTask.addMemberToGroup('[-memberUniqueName cn=" + WBM_groupName +",o=defaultWIMFileBasedRealm -groupUniqueName cn=" + WBM_all_groupName + ",o=defaultWIMFileBasedRealm]')\n");
		
		
		//Generate script to check if VMM groups from roles exist already
		//Add if not found 
		if(groupEntries != null && !groupEntries.isEmpty()){
			for(int i=0; i<groupEntries.size(); i++){
				
				VMMGroupEntry groupEntry = (VMMGroupEntry)groupEntries.get(i);
				if (!groupEntry.isVMMMissingMandatoryAttribute()) {
					String groupIdentifier = "group" + i;
//					String groupName = groupEntry.getVMMGroupName();
//					String description = groupEntry.getVMMDescription();
					String groupName = convertToUTF8(groupEntry.getVMMGroupName());
					String description = convertToUTF8(groupEntry.getVMMDescription());
					sb.append(groupIdentifier + " = AdminTask.searchGroups(['-cn', u'" + groupName + "'])\n");
					sb.append ("if " + groupIdentifier + ":\n");
					sb.append("\tprint u'Group " + groupName + " exists.  " + groupName + " will not be created'\n");
					sb.append("else:\n");
					//defect 73912
					sb.append("\t" + groupIdentifier + " = AdminTask.createGroup(['-cn', u'" + groupName);
					if(description != null && description.trim() != ""){
						sb.append("', '-description', u'" + description + "'])\n");
					} else {
						sb.append("'])\n");
					}
	//				Add to WBM_generated group
					sb.append("\tAdminTask.addMemberToGroup(['-memberUniqueName',  u'cn=" + groupName +",o=defaultWIMFileBasedRealm', '-groupUniqueName', u'cn=" + WBM_groupName + ",o=defaultWIMFileBasedRealm'])\n");
					sb.append("\tprint u'Created group " + groupName + "'\n");
				} else {
					// String resourceName = groupEntry.getVMMResourceName();
					String resourceName = convertToUTF8(groupEntry.getVMMResourceName());
					sb.append("print u'Entity " + resourceName + " is missing mandatory attribute: Role name.  No group created for role " + resourceName +"'\n");
				}
					
			}
		}
		//Generate script to check if VMM persons exist already
		//Add if not found 
		if(personEntries != null && !personEntries.isEmpty()){
			for(int i=0; i<personEntries.size(); i++){				
				VMMPersonEntry personEntry = (VMMPersonEntry)personEntries.get(i);
				if (!personEntry.isVMMMissingMandatoryAttribute()) {
//					String personID = personEntry.getVMMPersonID();
//					String personIdentifier = "user" + i;
//					String email = personEntry.getVMMemail();
//					String resourceName = personEntry.getVMMResourceName();
					String personID = convertToUTF8(personEntry.getVMMPersonID());
					String personIdentifier = "user" + i;
					String email = convertToUTF8(personEntry.getVMMemail());
					String resourceName = convertToUTF8(personEntry.getVMMResourceName());
					String firstName = convertToUTF8(personEntry.getVMMfirstName());
					String lastName = convertToUTF8(personEntry.getVMMlastName());
					if (personEntry.isVMMfirstNameDefault()) {
						sb.append("print u'Person " + resourceName + " is missing mandatory attribute: firstName.  firstName set to " + personID +"'\n");
					}
					if (personEntry.isVMMLastNameDefault()) {
						sb.append("print u'Person " + resourceName + " is missing mandatory attribute: lastName.  lastName set to " + personID +"'\n");
					}
					sb.append(personIdentifier + " = AdminTask.searchUsers(['-uid', u'" + personID + "'])\n");
					sb.append ("if " + personIdentifier + ":\n");
					sb.append("\tprint u'User " + personID + " exists.  " + personID + " will not be created'\n");
					sb.append("else:\n");
					if (email==null || email.equals("")){
						sb.append("\t" + personIdentifier + " = AdminTask.createUser(['-uid', u'" + personID + "', '-password', 'password', '-sn', u'" + lastName + "', '-cn', u'" + firstName + "'])\n");
					} else {
						sb.append("\t" + personIdentifier + " = AdminTask.createUser(['-uid', u'" + personID + "', '-password', 'password', '-sn', u'" + lastName + "', '-cn', u'" + firstName + "', '-mail', u'" + email + "'])\n");
					}
					sb.append("\tprint u'Created user " + personID + "'\n");
	//				Add to WBM_generated group
					sb.append("\tAdminTask.addMemberToGroup(['-memberUniqueName', u'uid=" + personID +",o=defaultWIMFileBasedRealm', '-groupUniqueName', u'cn=" + WBM_groupName + ",o=defaultWIMFileBasedRealm'])\n");
						
					List roles = personEntry.getVMMroles();
					if(roles != null && !roles.isEmpty()){
						Iterator it = roles.iterator();
						while(it.hasNext()){
							//String roleName = (String)it.next();
							String roleName = convertToUTF8((String)it.next());
							sb.append("memberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', u'cn=" + roleName +",o=defaultWIMFileBasedRealm'])\n");
							sb.append("from java.lang import String\n");
							sb.append("result = String(memberOfGroupResult)\n");
							sb.append("if(result.contains(String(\"" + personID +"\"))):\n");
							sb.append("\tprint u'User " + personID + " is already in the group " + roleName + "'\n");
							sb.append("else:\n");
							sb.append("\tAdminTask.addMemberToGroup(['-memberUniqueName', u'uid=" + personID +",o=defaultWIMFileBasedRealm', '-groupUniqueName', u'cn=" + roleName + ",o=defaultWIMFileBasedRealm'])\n");	
						}
					}
				} else {
//					String resourceName = personEntry.getVMMResourceName();
					String resourceName = convertToUTF8(personEntry.getVMMResourceName());
					sb.append("print u'Person " + resourceName + " is missing mandatory attribute: personId.  No user created for person "+ resourceName +"'\n");
				}
			}
		}
		//Generate script to check if VMM groups from organizations exist already
		//Add if not found 
		if(groupEntriesOrg != null && !groupEntriesOrg.isEmpty()){
			for(int i=0; i<groupEntriesOrg.size(); i++){
				VMMGroupEntry groupEntry = (VMMGroupEntry)groupEntriesOrg.get(i);
				if (!groupEntry.isVMMMissingMandatoryAttribute()) {
					String groupIdentifier = "org" + i;
//					String groupName = groupEntry.getVMMGroupName();
//					String description = groupEntry.getVMMDescription();
					String groupName = convertToUTF8(groupEntry.getVMMGroupName());
					String description = convertToUTF8(groupEntry.getVMMDescription());
					sb.append(groupIdentifier + " = AdminTask.searchGroups(['-cn', u'" + groupName + "'])\n");
					sb.append ("if " + groupIdentifier + ":\n");
					sb.append("\tprint u'Group " + groupName + " exists.  " + groupName + " will not be created'\n");
					sb.append("else:\n");
					//sb.append("\t" + groupIdentifier + " = AdminTask.createGroup(['-cn', '" + groupName + "'])\n");
					sb.append("\t" + groupIdentifier + " = AdminTask.createGroup(['-cn', u'" + groupName);
					if(description != null && description.trim() != ""){
						sb.append("', '-description', u'" + description + "'])\n");
					} else {
						sb.append("'])\n");
					}
					sb.append("\tprint u'Created group " + groupName + "'\n");
	//				Add to WBM_generated group
					sb.append("\tAdminTask.addMemberToGroup(['-memberUniqueName', u'cn=" + groupName +",o=defaultWIMFileBasedRealm', '-groupUniqueName', u'cn=" + WBM_groupName + ",o=defaultWIMFileBasedRealm'])\n");
	
					List members = groupEntry.getVMMMembers();
					if(members != null && !members.isEmpty()){
						Iterator it = members.iterator();
						while(it.hasNext()){
							//String memberID = (String)it.next();
							String memberID = convertToUTF8((String)it.next());
							sb.append("memberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', u'cn=" + groupName +",o=defaultWIMFileBasedRealm'])\n");
							sb.append("from java.lang import String\n");
							sb.append("r=String(memberOfGroupResult)\n");
							sb.append("if(r.contains(String(\"" + memberID +"\"))):\n");
							sb.append("\tprint u'User " + memberID + " is already in the group " + groupName + "'\n");
							sb.append("else:\n");
							sb.append("\tAdminTask.addMemberToGroup(['-memberUniqueName', u'uid=" + memberID +",o=defaultWIMFileBasedRealm',  '-groupUniqueName', u'cn=" + groupName + ",o=defaultWIMFileBasedRealm'])\n");
							
						}
					}
				} else {
//					String resourceName = groupEntry.getVMMResourceName();
					String resourceName = convertToUTF8(groupEntry.getVMMResourceName());
					sb.append("print u'Entity " + resourceName + " is missing mandatory attribute: Organization name.  No group created for organization " + resourceName + "'\n");
				}
			}
		}
		if(!sb.toString().equals("")){
			sb.append("print 'Finished executing VMM script'\n");
			return sb;
		}
		return null;
	}

	/**
	 * Generate Jython script to remove VMM entries added by VMM entry creation script
	 * @param timestamp 
	 */
	public static StringBuffer generateSessionCleanupVMMJythonScript(List groupEntries, List personEntries, List groupEntriesOrg, String timestamp) {
		StringBuffer sb = new StringBuffer();
		String WBM_groupName = "WBM_generated_session_" + timestamp;
		String WBM_all_groupName = "WBM_generated_all";
		sb.append("group_WBM = AdminTask.searchGroups(['-cn', '" + WBM_groupName +"'])\n");
		sb.append("if group_WBM:\n");
		sb.append("\tmemberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', 'cn="+ WBM_groupName + ",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\tif memberOfGroupResult:\n");
		sb.append("\t\tGroupList = memberOfGroupResult.split(java.lang.System.getProperty(\"line.separator\"))\n");
		sb.append("\t\tfor i in GroupList:\n");
		sb.append("\t\t\tfrom java.lang import String\n");
		sb.append("\t\t\tWBMEntry=String(i)\n");
		sb.append("\t\t\tuniqueName = String(WBMEntry.substring(WBMEntry.indexOf(',')+2, WBMEntry.length()))\n");
		sb.append("\t\t\tentryID = String(uniqueName.substring(uniqueName.indexOf('=')+1, uniqueName.indexOf(',')))\n");
		sb.append("\t\t\tentryType = String(WBMEntry.substring(0, WBMEntry.indexOf(',')))\n");
		sb.append("\t\t\tif (entryType.contains(String(\"PersonAccount\"))):\n");
		sb.append("\t\t\t\tperson = AdminTask.searchUsers(['-uid', entryID])\n");
		sb.append("\t\t\t\tif uniqueName:\n");
		sb.append("\t\t\t\t\tperson = AdminTask.deleteUser(['-uniqueName', uniqueName])\n");
		sb.append("\t\t\t\t\tprint 'Deleted person', entryID\n");
		sb.append("\t\t\telse:\n");
		sb.append("\t\t\t\tgroup = AdminTask.searchUsers(['-cn', entryID])\n");
		sb.append("\t\t\t\tif uniqueName:\n");
		sb.append("\t\t\t\t\tgroup = AdminTask.deleteGroup(['-uniqueName', uniqueName])\n");
		sb.append("\t\t\t\t\tprint 'Deleted group', entryID\n");
		sb.append("\t\tmemberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', 'cn="+ WBM_groupName +",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\t\tif not memberOfGroupResult:\n");
		sb.append("\t\t\tgroup = AdminTask.deleteGroup(['-uniqueName', 'cn="+ WBM_groupName +",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\t\tmemberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', 'cn="+ WBM_all_groupName +",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\t\tif not memberOfGroupResult:\n");
		sb.append("\t\t\tgroup = AdminTask.deleteGroup(['-uniqueName', 'cn="+ WBM_all_groupName +",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\t\tprint 'Finished executing VMM cleanup script'\n");
		sb.append("\telse:\n");
		sb.append("\t\tprint 'No entries to clean up.  Finished executing VMM cleanup script'\n");
		sb.append("else:\n");
		sb.append("\tprint 'No entries to clean up.  Finished executing VMM cleanup script'\n");
		return sb;
		
		//Generate script to check if VMM persons exist
		//Remove if found 
//		if(personEntries != null && !personEntries.isEmpty()){
//			for(int i=0; i<personEntries.size(); i++){
//				String personIdentifier = "user" + i;
//				VMMPersonEntry personEntry = (VMMPersonEntry)personEntries.get(i);
//				String personID = personEntry.getVMMPersonID();
//				sb.append(personIdentifier + " = AdminTask.searchUsers(['-uid', u'" + personID + "'.encode('utf-8')])\n");
//				sb.append ("if " + personIdentifier + ":\n");
//				sb.append("\t" + personIdentifier + " = AdminTask.deleteUser(['-uniqueName', u'uid=" + personID + ",o=defaultWIMFileBasedRealm'.encode('utf-8')])\n");
//				sb.append("\tprint u'Deleted user " + personID + "'.encode('utf-8')\n");
//			}
//		}
//		//Generate script to check if VMM groups from roles exist
//		//Remove if found 
//		if(groupEntries != null && !groupEntries.isEmpty()){
//			for(int i=0; i<groupEntries.size(); i++){
//				String groupIdentifier = "group" + i;
//				VMMGroupEntry groupEntry = (VMMGroupEntry)groupEntries.get(i);
//				String groupName = groupEntry.getVMMGroupName();
//				sb.append(groupIdentifier + " = AdminTask.searchGroups(['-cn', u'" + groupName + "'.encode('utf-8')])\n");
//				sb.append ("if " + groupIdentifier + ":\n");
//				sb.append("\t" + groupIdentifier + " = AdminTask.deleteGroup(['-uniqueName', u'cn=" + groupName + ",o=defaultWIMFileBasedRealm'.encode('utf-8')])\n");
//				sb.append("\tprint u'Deleted group " + groupName + "'.encode('utf-8')\n");
//			}
//		}
//		//Generate script to check if VMM groups from organizations exist
//		//Remove if found 
//		if(groupEntriesOrg != null && !groupEntriesOrg.isEmpty()){
//			for(int i=0; i<groupEntriesOrg.size(); i++){
//				String groupIdentifier = "org" + i;
//				
//				VMMGroupEntry groupEntry = (VMMGroupEntry)groupEntriesOrg.get(i);
//				String groupName = groupEntry.getVMMGroupName();
//				sb.append(groupIdentifier + " = AdminTask.searchGroups(['-cn', u'" + groupName + "'.encode('utf-8')])\n");
//				sb.append ("if " + groupIdentifier + ":\n");
//				sb.append("\t" + groupIdentifier + " = AdminTask.deleteGroup(['-uniqueName', u'cn=" + groupName + ",o=defaultWIMFileBasedRealm'.encode('utf-8')])\n");
//				sb.append("\tprint u'Deleted group " + groupName + "'.encode('utf-8')\n");
//			}
//		}
//		if(!sb.toString().equals("")){
//			sb.append("print 'Finished executing VMM cleanup script'\n");
//			return sb;
//		}
		// return null;
	}

	public static StringBuffer generateCleanupVMMJythonScript(List groupEntries, List personEntries, List groupEntriesOrg, String timestamp) {
		StringBuffer sb = new StringBuffer();
		String WBM_groupName = "WBM_generated_all";
		sb.append("group_WBM = AdminTask.searchGroups(['-cn', '" + WBM_groupName +"'])\n");
		sb.append("if group_WBM:\n");
		sb.append("\tmemberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', 'cn="+ WBM_groupName + ",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\tif memberOfGroupResult:\n");
		sb.append("\t\tGroupList = memberOfGroupResult.split(java.lang.System.getProperty(\"line.separator\"))\n");
		sb.append("\t\tfor i in GroupList:\n");
		sb.append("\t\t\tfrom java.lang import String\n");
		sb.append("\t\t\tWBMEntry=String(i)\n");
		sb.append("\t\t\tuniqueName_session = String(WBMEntry.substring(WBMEntry.indexOf(',')+2, WBMEntry.length()))\n");
		sb.append("\t\t\tentryID = String(uniqueName_session.substring(uniqueName_session.indexOf('=')+1, uniqueName_session.indexOf(',')))\n");
		sb.append("\t\t\tentryType = String(WBMEntry.substring(0, WBMEntry.indexOf(',')))\n");
		sb.append("\t\t\tif (entryType.contains(String(\"Group\"))):\n");
		sb.append("\t\t\t\tmemberOfGroupResult = AdminTask.getMembersOfGroup(['-uniqueName', uniqueName_session])\n");
		

		sb.append("\t\t\t\tif memberOfGroupResult:\n");
		sb.append("\t\t\t\t\tGroupList = memberOfGroupResult.split(java.lang.System.getProperty(\"line.separator\"))\n");
		sb.append("\t\t\t\t\tfor j in GroupList:\n");
		sb.append("\t\t\t\t\t\tfrom java.lang import String\n");
		sb.append("\t\t\t\t\t\tWBMEntry=String(j)\n");
		sb.append("\t\t\t\t\t\tuniqueName = String(WBMEntry.substring(WBMEntry.indexOf(',')+2, WBMEntry.length()))\n");
		sb.append("\t\t\t\t\t\tentryID = String(uniqueName.substring(uniqueName.indexOf('=')+1, uniqueName.indexOf(',')))\n");
		sb.append("\t\t\t\t\t\tentryType = String(WBMEntry.substring(0, WBMEntry.indexOf(',')))\n");
		sb.append("\t\t\t\t\t\tif (entryType.contains(String(\"PersonAccount\"))):\n");
		sb.append("\t\t\t\t\t\t\tperson = AdminTask.searchUsers(['-uid', entryID])\n");
		sb.append("\t\t\t\t\t\t\tif uniqueName:\n");
		sb.append("\t\t\t\t\t\t\t\tperson = AdminTask.deleteUser(['-uniqueName', uniqueName])\n");
		sb.append("\t\t\t\t\t\t\t\tprint 'Deleted person', entryID\n");
		sb.append("\t\t\t\t\t\telse:\n");
		sb.append("\t\t\t\t\t\t\tgroup = AdminTask.searchUsers(['-cn', entryID])\n");
		sb.append("\t\t\t\t\t\t\tif uniqueName:\n");
		sb.append("\t\t\t\t\t\t\t\tgroup = AdminTask.deleteGroup(['-uniqueName', uniqueName])\n");
		sb.append("\t\t\t\t\t\t\t\tprint 'Deleted group', entryID\n");
		
		sb.append("\t\t\tgroup = AdminTask.deleteGroup(['-uniqueName', uniqueName_session])\n");
		
		sb.append("\tgroup = AdminTask.deleteGroup(['-uniqueName', 'cn="+ WBM_groupName +",o=defaultWIMFileBasedRealm'])\n");
		sb.append("\tprint 'Finished executing VMM cleanup script'\n");
		sb.append("else:\n");
		sb.append("\tprint 'No entries to clean up.  Finished executing VMM cleanup script'\n");
		return sb;
		
	}
	
	private static String convertToUTF8(String unicodeString) {
		if (unicodeString == null)
			return null;
		
		char[] wordChars = new char[unicodeString.length()];
		unicodeString.getChars(0, unicodeString.length(), wordChars, 0);
		StringBuffer jsEncoded = new StringBuffer();
		
		for (int j = 0; j < wordChars.length; j++) {
			int unicode = wordChars[j];
			// to enhance readability, do not encode A-Z,a-z
			if (('A' <= unicode && unicode <= 'Z')
					|| ('a' <= unicode && unicode <= 'z')) {
				jsEncoded.append(wordChars[j]);
				continue;
			}
			// encode the character
			String charInHex = Integer.toString(unicode, 16).toUpperCase();
			switch (charInHex.length()) {
				case 1 :
					jsEncoded.append("\\u000").append(charInHex); //$NON-NLS-1$
					break;
				case 2 :
					jsEncoded.append("\\u00").append(charInHex); //$NON-NLS-1$
					break;
				case 3 :
					jsEncoded.append("\\u0").append(charInHex); //$NON-NLS-1$
					break;
				default :
					jsEncoded.append("\\u").append(charInHex); //$NON-NLS-1$
					break;
			}
		}
		return jsEncoded.toString();
	}

}
