package com.ibm.btools.te.ilm.heuristics.helper;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.ie.ui.ilm.config.TESettingHelper;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTESetting;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTargetType;
import com.ibm.btools.ie.ui.ilm.config.model.BOM2ILMTransformationType;
import com.ibm.btools.ie.ui.ilm.config.model.TargetConfig;
import com.ibm.btools.ie.ui.ilm.config.model.TargetType;
import com.ibm.btools.ie.ui.ilm.config.model.TransformationSetting;
import com.ibm.btools.te.framework.TransformationContext;

/**
 * Provides a set of helper methods to perform TESetting related operations
 */
public class TESettingUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	public static final String TE_SETTING_KEY = "TE_SETTING";
	
	/**
	 * Register the TESetting object in the context
	 * @param context
	 * @param teSetting
	 */
	public static void putTESetting (TransformationContext context, BOM2ILMTESetting teSetting) {
		context.put(TE_SETTING_KEY,teSetting);
	}
	
	/**
	 * Obtain the TESetting from the context
	 * @param context
	 * @return
	 */
	public static BOM2ILMTESetting getTESetting(TransformationContext context) {
		return (BOM2ILMTESetting)context.get(TE_SETTING_KEY);
	}
	
	/**
	 * Obtain a list of transformation target types for a specific modelObject with a specific type of transformation
	 * @param context
	 * @param modelObject
	 * @param transformationType
	 * @return
	 */
	public static Collection getTargetTypes(TransformationContext context,EObject modelObject, BOM2ILMTransformationType transformationType) {
		BOM2ILMTESetting teSetting = getTESetting(context);
		if (teSetting != null) { 
			TransformationSetting transformationSetting = TESettingHelper.getTransformationSetting(teSetting,transformationType);
			TargetConfig targetConfig = TESettingHelper.getTargetConfig(transformationSetting,modelObject);
			if (targetConfig != null) {
				return targetConfig.getTargetType();
			}
			return transformationSetting.getDefaultTargetType();
		}
		return null;
	}
	
	/**
	 * Get a list of Target transformation types for  a specific modelObject
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static Collection getTargetTypes(TransformationContext context,EObject modelObject) {
		BOM2ILMTESetting teSetting = getTESetting(context);
		if (teSetting != null) {
			TransformationSetting transformationSetting;
			EList transformationSettings = teSetting.getTransformationSetting();
			Iterator it = transformationSettings.iterator();
			while (it.hasNext()) {
				transformationSetting=(TransformationSetting)it.next();
				TargetConfig targetConfig = TESettingHelper.getTargetConfig(transformationSetting,modelObject);
				if (targetConfig != null) {
					return targetConfig.getTargetType();
				}				
			}
		}
		return null;
	}

	/**
	 * Check to see if a modelObject is configured to be transformed to a 'targetType'
	 * @param context
	 * @param modelObject
	 * @param transformationType
	 * @param targetType
	 * @return
	 */
	private static boolean isTarget(TransformationContext context,EObject modelObject,BOM2ILMTransformationType transformationType, BOM2ILMTargetType targetType) {
		Collection targetTypes=getTargetTypes(context,modelObject,transformationType);
		if (targetTypes != null) {
			Iterator it=targetTypes.iterator();
			TargetType itTargetType;
			while (it.hasNext()) {
				itTargetType=(TargetType)it.next();
				if (itTargetType.getType() == targetType) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Check if the BOM Process is configured to transform to BPEL
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isProcessTargetBPEL(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.PROCESS_LITERAL,BOM2ILMTargetType.BPEL_LITERAL);
	}

	/**
	 * Check if the BOM Process is configured to transform to BPEL_Plus
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isProcessTargetBPEL_PLUS(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.PROCESS_LITERAL,BOM2ILMTargetType.BPELP_LITERAL);
	}

	/**
	 * Check if the BOM Process is configured to transform to FDL
	 * @param context
	 * @param modelObject
	 * @return
	 */	
	public static boolean isProcessTargetFDL(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.PROCESS_LITERAL,BOM2ILMTargetType.FDL_LITERAL);

	}

	/**
	 * Check if the BOM Business Item is configured to transform to XSD
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isBusinessItemTargetXSD(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.INFORMATION_LITERAL,BOM2ILMTargetType.XSD_LITERAL);

	}
	
	/**
	 * Check if the BOM Business Item is configured to transform to FDL
	 * @param context
	 * @param modelObject
	 * @return
	 */	
	public static boolean isBusinessItemTargetFDL(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.INFORMATION_LITERAL,BOM2ILMTargetType.FDL_LITERAL);
	}

	/**
	 * Check if the BOM Business Item is configured to transform to WDO
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isBusinessItemTargetWDO(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.INFORMATION_LITERAL,BOM2ILMTargetType.WDO_LITERAL);
	}

	/**
	 * Check if the BOM Resource is configured to transform to FDL
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isResourceTargetFDL(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.RESOURCE_LITERAL,BOM2ILMTargetType.FDL_LITERAL);
	}

	/**
	 * Check if the BOM Organization is configured to transform to FDL
	 * @param context
	 * @param modelObject
	 * @return
	 */
	public static boolean isOrganizationTargetFDL(TransformationContext context,EObject modelObject) {
		return isTarget(context,modelObject,BOM2ILMTransformationType.ORGANIZATION_LITERAL,BOM2ILMTargetType.FDL_LITERAL);
	}
}
