/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.bom.model.artifacts.DescriptorType;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.observation.EventType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.SAUtil;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TESettingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Data Definition Rule</b></em>'.
 * Source = BOM Type
 * Target = SA contains IT representation of the BOM Type
 * Takes care of BOM Business Item transformation.   
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DataDefinitionRuleImpl extends TransformationRuleImpl implements DataDefinitionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static final String DATA_DEFINITION_SA="DATA_DEFINITION_SA";
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DataDefinitionRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.DATA_DEFINITION_RULE;
	}

	/**
	 * Source = BOM Type
	 * Target = SA contains IT representation of the BOM Type
	 */
	public boolean transformSource2Target()
	{	
		LoggingUtil.traceEntry(this, "transformSource2Target");
		TransformationContext context = getContext();
		Type type = (Type)getSource().get(0);
		if (!(type instanceof PrimitiveType) && !BomUtils.isClassifier(type)) {
			if (TESettingUtil.isBusinessItemTargetXSD(context,type)){
				handleXSDTransformation(type);
			} else if (TESettingUtil.isBusinessItemTargetFDL(context,type)) {
				handleFDLTransformation(type);
			}
		}
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	
	/**
	 * Handle the case for XSD Transformation
	 * @param type
	 */
	private void handleXSDTransformation(Type type) {
		//feature 110941
		if (!BomUtils.isPredefinedType(type) && !(type instanceof EventType || type instanceof DescriptorType)) { 
			handleDataDefinitionTransformation(XsdFactory.eINSTANCE.createXSDSchemaRule(),type);
		}
	}
	
	/**
	 * Handle the case for FDL Transformation
	 * @param type
	 */
	private void handleFDLTransformation(Type type) {
//		feature 110941
		//if (!StaffUtil.isPredefinedType(type)) { 
		if (!BomUtils.isPredefinedType(type) && !(type instanceof EventType || type instanceof DescriptorType)) {
			handleDataDefinitionTransformation(FdlFactory.eINSTANCE.createDataTypeRule(),type);
		}		
	}
	
	/**
	 * Create SA for the type
	 * @param rule
	 * @param type
	 */
	private void handleDataDefinitionTransformation (TransformationRule rule, Type type) {
//		if (!(StaffUtil.isPredefinedStaff(type) || StaffUtil.isPredefinedOrganization(type))) {
			getChildRules().add(rule);
			rule.getSource().add(type);
			rule.transformSource2Target();
			if (!rule.getTarget().isEmpty()) {
				//SolutionArtifact unclassifiedSA = getSolutionArtifact();
				//getTarget().add(unclassifiedSA);
				//updateUnclassifiedSA(unclassifiedSA,rule.getTarget());
				getTarget().addAll(createUnclassifiedSAs(rule.getTarget()));
			}
//		}		
	}
	
	/**
	 * update the unclassified SA to contains the list of targets
	 * @param unclassifiedSA
	 * @param targets
	 */
//	private void updateUnclassifiedSA (SolutionArtifact unclassifiedSA, Collection targets)
//	{
//		Iterator it = targets.iterator();
//		EObject target;
//		UnclassifiedArtifact artifact=null;
//		while (it.hasNext()) {
//			target = (EObject)it.next();
//			SAUtil.updateUnclassifiedSA(unclassifiedSA,target);
//		}
//	}
//
//	private SolutionArtifact getSolutionArtifact() {
//		SolutionArtifact sa = SAUtil.getSolutionArtifact(getContext(),DATA_DEFINITION_SA);
//		if (sa==null) {
//			sa = SAUtil.createUnclassifiedSolutionArtifact();
//			SAUtil.registerSolutionArtifact(getContext(),DATA_DEFINITION_SA,sa);
//		}
//		return sa;
//	}
	
	private SolutionArtifact getSolutionArtifact(EObject key) {
		SolutionArtifact sa = SAUtil.getSolutionArtifact(getContext(),key);
		if (sa==null) {
			sa = SAUtil.createUnclassifiedSolutionArtifact();
			SAUtil.registerSolutionArtifact(getContext(),key,sa);
		}
		return sa;
	}	
	private List createUnclassifiedSAs (Collection targets)
	{
		Iterator it = targets.iterator();
		EObject target;
		UnclassifiedArtifact artifact=null;
		SolutionArtifact unclassifiedSA;
		List unclassifiedSAs = new ArrayList();
		while (it.hasNext()) {
			target = (EObject)it.next();
			//unclassifiedSA=SAUtil.createUnclassifiedSolutionArtifact();
			unclassifiedSA=getSolutionArtifact(target);
			unclassifiedSAs.add(unclassifiedSA);
			SAUtil.updateUnclassifiedSA(unclassifiedSA,target);
			if (target instanceof XSDSchema) {
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule.class,target);
				if (tr!=null) {
					unclassifiedSA.setPackage(SAUtil.createPackage((EObject)tr.getSource().get(0),false));
				}				
			}
		}
		return unclassifiedSAs;
	}
	
} //DataDefinitionRuleImpl
