/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.InstanceValue;
import com.ibm.btools.bom.model.artifacts.LiteralString;
import com.ibm.btools.bom.model.artifacts.Slot;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.resources.IndividualResource;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.bom.model.resources.Qualification;
import com.ibm.btools.bom.model.resources.Role;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;

import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;

import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.OrganizationRule;
import com.ibm.btools.te.ilm.heuristics.ResourceRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.helper.StaffUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TESettingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.VMMGroupEntry;
import com.ibm.btools.fdl.model.Organization;
import com.ibm.btools.te.ilm.model.sa.SAType;
import com.ibm.btools.te.ilm.model.sa.SaFactory;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifactType;
import com.ibm.btools.te.ilm.resource.MessageKeys;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.util.EList;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;

import org.eclipse.emf.ecore.util.InternalEList;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Organization Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class OrganizationRuleImpl extends TransformationRuleImpl implements OrganizationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected OrganizationRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.ORGANIZATION_RULE;
	}

	/**
	 * SOURCE: BOM OrganizationUnit
	 * TARGET: FDL Organization
	 * Maps BOM OrganizationUnit to a FDL Organization
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */	
	public boolean transformSource2Target()
	{	
		LoggingUtil.traceEntry(this, "transformSource2Target");
		TransformationContext context = getContext();
		OrganizationUnit orgUnit = (OrganizationUnit)getSource().get(0);
		if (TESettingUtil.isOrganizationTargetFDL(context,orgUnit)){
			handleFDLTransformation(orgUnit);	
		}else{
			handleBPELOrganizationTransformation(orgUnit);
		}
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Verify that name of organization is provided, collect uids of members that are also being exported,
	 * Make call to store VMM group into context
	 */	
	private void handleBPELOrganizationTransformation(OrganizationUnit orgUnit) {	
		//defect 75299
		if(StaffUtil.isKindOfOrg((Type)(orgUnit).getClassifier().get(0))){ 
			String groupName = null;
			VMMGroupEntry VMMGroup = new VMMGroupEntry();
			List slots = orgUnit.getSlot();		
			if(slots != null && !slots.isEmpty()){
				Iterator it = slots.iterator();
				while(it.hasNext()){
					Slot thisSlot = (Slot)it.next();	
					if (thisSlot.getDefiningFeature().getName()!=null){		
						if(thisSlot.getDefiningFeature().getName().equals("name")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								groupName =((LiteralString)thisSlot.getValue().get(0)).getValue();
							}
						}
						if(thisSlot.getDefiningFeature().getName().equals("members")){
							if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
								for (int i=0; i<thisSlot.getValue().size(); i++) {	
									IndividualResource instValue = (IndividualResource)((InstanceValue)thisSlot.getValue().get(i)).getInstance();
									if(findResourcePerson(instValue)){
										String memberID = getPersonIDValueFromInstValue(instValue);
										List<String> memberIDs = new ArrayList();
										if (VMMGroup.getVMMMembers()!=null) {
											memberIDs = VMMGroup.getVMMMembers();
										}
										memberIDs.add(memberID);
										VMMGroup.setVMMMembers(memberIDs);
									}
								}
								
							}
						}
						
						
					}			
				}
			}
			if(groupName == null){
				String orgName = orgUnit.getName();	
				VMMGroup.setVMMMissingMandatoryAttribute(orgName);
				StaffUtil.registerVMMGroupEntityForOrg(getContext(), VMMGroup);
				//LoggingUtil.logWarning(MessageKeys.VMM_GROUP_MISSING_MANDATORY_ATTRIBUTES, param, null);
				return;
			}
			//defect 73912
			if(orgUnit.getOwnedComment() != null && !orgUnit.getOwnedComment().isEmpty()){
				Comment description = (Comment)orgUnit.getOwnedComment().get(0);
				String orgDescription = description.getBody();
				VMMGroup.setVMMDescription(orgDescription);
			}
			VMMGroup.setVMMGroupName(groupName);
			StaffUtil.registerVMMGroupEntityForOrg(getContext(), VMMGroup);
		}
	}
	
	private String getPersonIDValueFromInstValue(IndividualResource resource) {
		String personID = null;
		if(resource == null)
		{
			return null;
		}
		List slots = resource.getSlot();		
		if(slots != null && !slots.isEmpty()){
			Iterator it = slots.iterator();
			while(it.hasNext()){
				Slot thisSlot = (Slot)it.next();	
				if (thisSlot.getDefiningFeature().getName()!=null){					
					if(thisSlot.getDefiningFeature().getName().equals("personId")){
						if(thisSlot.getValue() != null && !thisSlot.getValue().isEmpty()){
							personID =((LiteralString)thisSlot.getValue().get(0)).getValue();
						}
					}
					
				}			
			}
		}
		return personID;
	}

	private boolean findResourcePerson (IndividualResource resource){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), ResourceRule.class, resource);
		if (rule==null)
			return false;
		else
			return true;
	}
	
	/**
	 * Create SA for a OrganizationUnit
	 * @param orgUnit - OrganizationUnit
	 * @return
	 */		
	private void handleFDLTransformation(OrganizationUnit orgUnit) {
		if (orgUnit!=null && !findOrganization(orgUnit) && StaffUtil.isPredefinedOrganization(orgUnit.getClassifier())){
			TransformationContext context = getContext();
			com.ibm.btools.te.ilm.heuristics.fdl.OrganizationRule orgRule = FdlFactory.eINSTANCE.createOrganizationRule();
			getChildRules().add(orgRule);
			orgRule.getSource().add(orgUnit);
			setComplete(orgRule.transformSource2Target());
			if (isComplete()) {
				EList targets = orgRule.getTarget();
				Iterator it = targets.iterator();
				EObject organization;
				SolutionArtifact sa;
				UnclassifiedArtifact orgArtifact;
				while (it.hasNext()) {
					organization = (EObject)it.next();
					if (organization instanceof Organization) {
						sa = SaFactory.eINSTANCE.createSolutionArtifact();
						sa.setType(SAType.UNCLASSIFIED_LITERAL);
						sa.setName(orgUnit.getName());
						orgArtifact = SaFactory.eINSTANCE.createUnclassifiedArtifact();
						orgArtifact.setType(UnclassifiedArtifactType.WORKFLOW_DEFINITION_LITERAL);
						orgArtifact.getArtifacts().add(organization);
						sa.getComposedArtifacts().add(orgArtifact);
						getTarget().add(sa);		
					}
				}
			}
		}
	}
	
	private boolean findOrganization (OrganizationUnit orgUnit){
		TransformationRule rule = TransformationUtil.getRuleForSource(getRoot(), com.ibm.btools.te.ilm.heuristics.fdl.OrganizationRule.class, orgUnit);
		if (rule==null)
			return false;
		else
			return true;
	}

} //OrganizationRuleImpl
