/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.internal.impl.DefinitionImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.ProcessFlowRule;
import com.ibm.btools.te.ilm.heuristics.TimeIntervalsRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.AbstractbpelFactory;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRReusableTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BRTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.BpelOptimizationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.HumanTaskRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.bpel.BpelFactory;
import com.ibm.btools.te.ilm.heuristics.bpel.BpelRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpFactory;
import com.ibm.btools.te.ilm.heuristics.bpelp.BpelpRule;
import com.ibm.btools.te.ilm.heuristics.extservice.ExtserviceFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.ActivityRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FDLOptimizationRule;
import com.ibm.btools.te.ilm.heuristics.fdl.FdlFactory;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlProcessGraphLayoutManager;
import com.ibm.btools.te.ilm.heuristics.fdl.util.FdlUtil;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelConstants;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptimizationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.SAUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TESettingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NoHashNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.naming.ProjectNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ExportRule;
import com.ibm.btools.te.ilm.heuristics.scdl.util.JavaImplementation;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.model.sa.HumanTask;
import com.ibm.btools.te.ilm.model.sa.ProcessFlow;
import com.ibm.btools.te.ilm.model.sa.ProcessFlowType;
import com.ibm.btools.te.ilm.model.sa.SAType;
import com.ibm.btools.te.ilm.model.sa.SaFactory;
import com.ibm.btools.te.ilm.model.sa.ScdlArtifact;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;
import com.ibm.btools.te.ilm.model.sa.UnclassifiedArtifact;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.br.core.model.DocumentRoot;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.businesscalendar.ICalendar;

/**
 * Transform a BOM Activity to a ProcessDefinition (BPEL Process) and a ProcessInterface (WSDL).
 * It creates the ProcessDefinitonRule.  It also creates the ProcessInterfaceRule
 * if it has not been created already.
 * Source: Activity
 * Target: ProcessFlowGraph
 */
public class ProcessFlowRuleImpl extends TransformationRuleImpl implements ProcessFlowRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	//Indicate whether the interface activity has been transformed to handle process variation
	private boolean isInterfaceActionTransformed = false;
	
	public boolean isInterfaceActionTransformed() {
		return isInterfaceActionTransformed;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessFlowRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.PROCESS_FLOW_RULE;
	}

	/**
	 * Transform a BOM Activity to a ProcessDefinition (BPEL Process) and a ProcessInterface (WSDL).
	 * It creates the ProcessDefinitonRule.  It also creates the ProcessInterfaceRule
	 * if it has not been created already.
	 * Source: Activity
	 * Target: ProcessFlowGraph
	 */
	public boolean transformSource2Target()
	{				
		LoggingUtil.traceEntry(this, "transformSource2Target");
		TransformationContext context = getContext();
		Activity activity = (Activity)getSource().get(0);			
		boolean isProcessTargetBPEL = TESettingUtil.isProcessTargetBPEL(context,activity);
		boolean isProcessTargetBPEL_PLUS = TESettingUtil.isProcessTargetBPEL_PLUS(context,activity);		
		boolean isProcessTargetFDL = TESettingUtil.isProcessTargetFDL(context,activity);
		if (isProcessTargetBPEL || isProcessTargetBPEL_PLUS){
			handleBPELTransformation(activity);
		} else if (isProcessTargetFDL) {
			handleFDLTransformation(activity);
		}
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	private void handleFDLTransformation(Activity activity) {
		TransformationContext context = getContext();
		ActivityRule activityRule = FdlFactory.eINSTANCE.createActivityRule();
		getChildRules().add(activityRule);
		activityRule.getSource().add(activity);
		setComplete(activityRule.transformSource2Target());
		if (isComplete()) {
			EList targets = activityRule.getTarget();
			Iterator it = targets.iterator();
			EObject process;
			SolutionArtifact sa;
			ProcessFlow processFlow;
			while (it.hasNext()) {
				process = (EObject)it.next();
				if (process instanceof com.ibm.btools.fdl.model.Process) {
					if (FdlUtil.bApplyOptimization) {
						FDLOptimizationRule foRule = FdlFactory.eINSTANCE.createFDLOptimizationRule();
						foRule.getSource().add(process);
						getChildRules().add(foRule);
						foRule.transformSource2Target();
						process = (EObject)foRule.getTarget().get(0);
					}					
					//add auto layout
					new FdlProcessGraphLayoutManager((com.ibm.btools.fdl.model.Process)process).doLayout();
					sa = SaFactory.eINSTANCE.createSolutionArtifact();
					sa.setType(SAType.PROCESS_FLOW_LITERAL);
					sa.setName(activity.getName());
					processFlow = SaFactory.eINSTANCE.createProcessFlow();
					processFlow.setType(ProcessFlowType.FDL_LITERAL);
					processFlow.setProcessDefinition(process);	
					sa.getComposedArtifacts().add(processFlow);
					getTarget().add(sa);		
				}
			}
		}
	}
	
	private void handleBPELTransformation(Activity activity) {
		TransformationContext context = getContext();
		ProcessUtil.registerVariableForParallelFlow(context, new LinkedList(), true);
		ProcessUtil.registerVariablesForCurrentPath(context, new LinkedList(), true);
		
		// The processFlowRule creates the ProcessFlowDefiniton will check if there is a ProcessInterfaceRule in the context.
		// If yes, it will add it as child. If no, it will create it and add the ProcessInterfaceRule to the context
		// of the rules key=namespace of the process, value ProcessInterfaceRule
		Activity interfaceActivity = null;		
		if(FabricIntegrationUtil.getInterfaceActivityIfAny(activity.getImplementation()) != null){
			interfaceActivity = FabricIntegrationUtil.getInterfaceActivityIfAny(activity.getImplementation());
		}
		ProcessInterfaceRule interfaceRule = null;//(ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context,activity);
		if(interfaceActivity != null){
			interfaceRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context,interfaceActivity);
		}else{
			interfaceRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context,activity);
		}
	//	ProcessInterfaceRule interfaceRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context,activity);
		//Create new interface rule for each process
			if (interfaceRule == null) {
				interfaceRule = AbstractbpelFactory.eINSTANCE.createProcessInterfaceRule();
				// Put the InterfaceRule in context using the activity namespace as the key
			//	Activity interfaceActivity = ProcessUtil.getInterfaceActivityIfAny(activity.getImplementation());
				if(interfaceActivity != null){
					ProcessUtil.putProcessInterfaceRule(context,interfaceActivity,interfaceRule);
				}else{
					ProcessUtil.putProcessInterfaceRule(context,activity,interfaceRule);
				}
				interfaceRule.setRoot(getRoot());
			//For process variation, if the interface action has been transformed, we empty the target
			//and let the rule create a new instance of WSDL definition to carry on the partner link
			//types for process
			}else if(interfaceRule != null && interfaceActivity != null){
				isInterfaceActionTransformed = true;
				interfaceRule.getTarget().clear();
				if(interfaceActivity != null){
					ProcessUtil.putProcessInterfaceRule(context,interfaceActivity,interfaceRule);
				}else{
					ProcessUtil.putProcessInterfaceRule(context,activity,interfaceRule);
				}
			}
	//	}
		getChildRules().add(interfaceRule);
		//Ray May 12,05 InterfaceRule's source is supposed to be a Process SAN.  if we put activity, somewhere down
		// the road things break
		//interfaceRule.getSource().addAll(source);
		if(interfaceActivity == null){
			interfaceRule.getSource().add(activity.getImplementation());
		}else{
			interfaceRule.getSource().add(interfaceActivity.getImplementation());
		}

		// Put InterfaceRule in the context with preset key for easy access
		// during the traversal of this process
		ProcessUtil.putProcessInterfaceRule(context,interfaceRule);
		// Execute the rule
		interfaceRule.transformSource2Target();
		ProcessDefinitionRule definitionRule = null;
		if(activity.getImplementation() != null){
			Action san = (Action)activity.getImplementation();
			if(BomUtils.isTask((Action)san) || BomUtils.isService((Action)san)||BomUtils.isServiceOperation(san)||
					BomUtils.isHumanTask(san) || BomUtils.isBusinessRuleTask(san)){
				//fill in the WSDL Definition with port, message, operation informations
				processInterfaceDefinition(interfaceRule, san);
				List commands = ProcessUtil.getUpdateCommandsFromContext(getContext());
				if(commands != null && !commands.isEmpty()){
					Iterator it = commands.iterator();
					while(it.hasNext()){
						TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), (Command)it.next());	
					}
					
				}
				// Let the Activity rule to handle any other runtime artifacts transformation
				com.ibm.btools.te.ilm.heuristics.abstractbpel.ActivityRule activityRule = AbstractbpelFactory.eINSTANCE.createActivityRule();
				getChildRules().add(activityRule);
				activityRule.getSource().addAll(getSource());
				activityRule.transformSource2Target();
				getTarget().addAll(processScdlArtifacts(activityRule.getTarget()));
				//feature 117494
				//updateDefinitionDocumentation((Definition)interfaceRule.getTarget().get(0), activity);
				setComplete(true);
			}else{
				definitionRule = AbstractbpelFactory.eINSTANCE.createProcessDefinitionRule();
				definitionRule.getSource().addAll(getSource());
				getChildRules().add(definitionRule);
				// Put ProcessDefinitionRule in the context with preset key for easy access
				// during the traversal of this process
				ProcessUtil.putProcessDefinitionRule(context,definitionRule);
				// Execute the rule
				setComplete(definitionRule.transformSource2Target());
			}
		}
			
		if (isComplete())
		{
			SolutionArtifact sa = SaFactory.eINSTANCE.createSolutionArtifact();
			sa.setType(SAType.PROCESS_FLOW_LITERAL);
			sa.setName(activity.getName());
			ProcessFlow processFlow = SaFactory.eINSTANCE.createProcessFlow();
			for(int i=0; i<interfaceRule.getTarget().size(); i++){
				processFlow.getProcessInterface().add((EObject)interfaceRule.getTarget().get(i));
			}
				
			/*  
			 * 1. Read the ILMTransformationConfig that contains the information specified by 
			 *    the user as to what ILM ComposedArtifact(s) to create.
			 * 2. Create the appropriate Transformation rule to transform the Abstract BPEL
			 *    into either BPEL or BPEL+, depending on the user settings.
			 * 3. Set the AbstractBpel representation as the source of the rule
			 * 4. Execute the transformation rule
			 * 5. Get the target, and set it in the ProcessFlow ComposedArtifact. 
			 */

			if (TESettingUtil.isProcessTargetBPEL(context,activity)){
				BpelRule bpelRule = BpelFactory.eINSTANCE.createBpelRule();
				if(definitionRule != null){
					bpelRule.getSource().add(definitionRule.getTarget().get(0));
					getChildRules().add(bpelRule);
					bpelRule.transformSource2Target();
					processFlow.setType(ProcessFlowType.BPEL_LITERAL);
					EObject process = (EObject)bpelRule.getTarget().get(0);
					if (BpelOptimizationUtil.bApplyOptimization) {
						BpelOptimizationRule boRule = AbstractbpelFactory.eINSTANCE.createBpelOptimizationRule();
						boRule.getSource().add(process);
						boRule.getSource().add(activity);
						getChildRules().add(boRule);
						boRule.transformSource2Target();
						process = (EObject)boRule.getTarget().get(0);
					}
					processFlow.setProcessDefinition(process);
				}
					
				
					
			} else if (TESettingUtil.isProcessTargetBPEL_PLUS(context,activity)){
				BpelpRule bpelpRule = BpelpFactory.eINSTANCE.createBpelpRule();
				if(definitionRule != null){
					bpelpRule.getSource().add(definitionRule.getTarget().get(0));
					getChildRules().add(bpelpRule);
					bpelpRule.transformSource2Target();
					processFlow.setType(ProcessFlowType.BPELPP_LITERAL);
					EObject process = (EObject)bpelpRule.getTarget().get(0);
					if (BpelOptimizationUtil.bApplyOptimization) {
						BpelOptimizationRule boRule = AbstractbpelFactory.eINSTANCE.createBpelOptimizationRule();
						boRule.getSource().add(process);
						boRule.getSource().add(activity);
						getChildRules().add(boRule);
						boRule.transformSource2Target();
						process = (EObject)boRule.getTarget().get(0);
					}
					processFlow.setProcessDefinition(process);

				}
			}			
			String registeredName = null;
			if (activity.getImplementation() != null) {
				StructuredActivityNode action = activity.getImplementation();
			
				NoHashNamingRegistry noHashReg = null;
				if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
				} else {
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
				}
				Object contextContainer = NamingUtil.getContextContainer(action);
			
				if(noHashReg.actionExistInGlobalRegistry(action)) {
					registeredName = noHashReg.retrieveNameFromGlobalRegistry(action);
					if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						registeredName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						registeredName = registeredName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}
				} else if (noHashReg.actionExistInLocalRegistry(action, contextContainer)) {
					registeredName = noHashReg.retrieveNameFromLocalRegistry(action, contextContainer);
					if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						registeredName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						registeredName = registeredName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}		
				}
			}
			if (registeredName != null) {
				sa.setPackage(registeredName);	
			} else {	//GJD 20070914 current default behaviour
				sa.setPackage(SAUtil.createPackage(activity));
			}
			sa.getComposedArtifacts().add(processFlow);
			getTarget().add(sa);
			
			if (definitionRule != null) {
				getTarget().addAll(processScdlArtifacts(definitionRule.getTarget()));
			}
			//Register the process flow keyed by its BOM source activity for target persistence consolidation
			FabricIntegrationUtil.registerProcessFlowForActivity(activity, processFlow);
		}
	}

	/**
	 * This method insures the WSDL Interface is completed with port, messages, operation information.
	 * 
	 */
	private void processInterfaceDefinition(ProcessInterfaceRule processIntRule, Action action){
		TransformationRule portTypeRule = null;			
		if(BomUtils.isServiceOperation(action)){
			portTypeRule = TransformationUtil.getRuleForSource(processIntRule,
					com.ibm.btools.te.ilm.heuristics.extservice.PortTypeRule.class, 
					action, 
					PortType.class);
			if(portTypeRule == null){
				portTypeRule = ExtserviceFactory.eINSTANCE.createPortTypeRule();
			}
		}else{
			portTypeRule = TransformationUtil.getRuleForSource(processIntRule,PortTypeRule.class, action, PortType.class);
			if(portTypeRule == null){
				portTypeRule = WsdlFactory.eINSTANCE.createPortTypeRule();
			}
		}
		portTypeRule.getSource().add(action);
		processIntRule.getChildRules().add(portTypeRule);
		processIntRule.transformSource2Target();	
	if(!portTypeRule.isComplete()){
		portTypeRule.transformSource2Target();
	}
}
	
	/**
	 * Handle all SCA components created by putting them in the SA
	 * @param targets
	 * @return
	 */
	private List processScdlArtifacts(List targets) {
		SolutionArtifact sa=null;
		ArrayList sas = new ArrayList();
		EObject obj;

		for (Iterator it=targets.iterator(); it.hasNext();) {
			obj = (EObject)it.next();			
			if (obj instanceof com.ibm.wsspi.sca.scdl.DocumentRoot) {
				Class classForSearch;
				//feature 118131
				if (((com.ibm.wsspi.sca.scdl.DocumentRoot)obj).getComponent()!=null ||((com.ibm.wsspi.sca.scdl.DocumentRoot)obj).getImport()!=null) {
					classForSearch=ComponentRule.class;
				} else {
					classForSearch=ExportRule.class;
				}
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(),classForSearch,obj);
				if (tr!=null) {
					ScdlArtifact scdlArtifact = SaFactory.eINSTANCE.createScdlArtifact();
					scdlArtifact.setArtifact(obj);
					sa = SaFactory.eINSTANCE.createSolutionArtifact();
					sa.setType(SAType.SCDL_LITERAL);
					sa.getComposedArtifacts().add(scdlArtifact);
					Action action=null;
					if (classForSearch==ComponentRule.class) {
						action = (Action)tr.getSource().get(0);
					} else {
						TransformationRule tr2 = TransformationUtil.getRuleForTarget(getRoot(),ComponentRule.class,(com.ibm.wsspi.sca.scdl.DocumentRoot)tr.getSource().get(0));;
						if (tr2!=null) {
							action = (Action)tr2.getSource().get(0);							
						}
					}
					if (action != null) {
						//GJD 20070907
						String path = null;
						com.ibm.wsspi.sca.scdl.DocumentRoot docRoot = (com.ibm.wsspi.sca.scdl.DocumentRoot)obj;
						if (docRoot.getComponent() != null && docRoot.getComponent().getName() != null) {
							path = docRoot.getComponent().getName();
							if (path.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
								path = path.substring(0, path.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
								path = path.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
							}
						} else if (docRoot.getImport() != null && docRoot.getImport().getName() != null) {
							path = docRoot.getImport().getName();
							if (path.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
								path = path.substring(0, path.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
								path = path.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
							}
						} else if (docRoot.getExport() != null && docRoot.getExport().getName() != null) {
							path = docRoot.getExport().getName();
							if (path.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
								path = path.substring(0, path.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
								path = path.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
							}
						} 
						
						if (path != null) {
							boolean isOverride = false;
							if(tr instanceof ComponentRule && tr.getSource().size() >= 3 && (tr.getSource().get(tr.getSource().size()-1) instanceof CallBehaviorAction)){					
								isOverride = true;
							}else if(tr instanceof ExportRule && tr.getParentRule() instanceof CallBehaviorActionRule){
								CallBehaviorActionRule cabaRule = (CallBehaviorActionRule)tr.getParentRule();
								if(!cabaRule.getSource().isEmpty() && cabaRule.getSource().get(0)instanceof InputPinSet){
									InputPinSet ips = (InputPinSet)cabaRule.getSource().get(0);
									if(ips.eContainer() instanceof CallBehaviorAction){
										if(BomUtils.isServiceOperation((CallBehaviorAction)ips.eContainer())||BomUtils.isService((CallBehaviorAction)ips.eContainer())){
											if(FabricIntegrationUtil.isOverriden((CallBehaviorAction)ips.eContainer())){
												isOverride = true;
											}
										}
									}
								}
							}
							if(ExportOperationUtil.skipDefaultCatalog() && !isOverride){
								sa.setPackage(SAUtil.createPackage(action));
							}else{
								sa.setPackage(path);
							}
						} else {
							sa.setPackage(SAUtil.createPackage(action));
						}
						sas.add(sa);						
					}
				}
			}else if(obj instanceof com.ibm.wbit.tel.DocumentRoot){
				HumanTask humanTaskArtifact = SaFactory.eINSTANCE.createHumanTask();
				humanTaskArtifact.setInline(HumanTaskUtil.isInlineHumanTask(getContext(),(com.ibm.wbit.tel.DocumentRoot)obj));
				humanTaskArtifact.setHumanTask(obj);
				sa = SaFactory.eINSTANCE.createSolutionArtifact();
				sa.setType(SAType.HUMAN_TASK_LITERAL);
				sa.getComposedArtifacts().add(humanTaskArtifact);
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(), HumanTaskRule.class,obj);
				Action action = null;
				if(tr != null){
					action = (Action)tr.getSource().get(0);
				}else{
					//L017093 handle the case of the inline tel generated by the from the Receive or the OnMessage, this is done in the SanReusableProcessRule
					//lookup the rule based on the target only
					tr = TransformationUtil.getRuleForTarget(getRoot(), obj);
					if(tr != null){
						//source is the Activity
						if(tr.getSource().get(0) instanceof Activity){
							Activity processActivity = (Activity)tr.getSource().get(0);
							action = processActivity.getImplementation();
						}
					}
				}

				NoHashNamingRegistry noHashReg = null;
				if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
				} else {
					noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
				}
				String registeredName = null;	
				
				if(noHashReg.actionExistInGlobalRegistry(action)) {
					registeredName = noHashReg.retrieveNameFromGlobalRegistry(action);
					if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						registeredName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						registeredName = registeredName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}
				}
			
				if (registeredName != null) {
					sa.setPackage(registeredName);
				} else {	
					sa.setPackage(SAUtil.createPackage(action));
				}
				sas.add(sa);
				
			
				//GJD put business calendar in the same location as human task that owns it				
				if (tr != null && tr.getTarget() != null) {
					Iterator targetIt = tr.getTarget().iterator();
					Object htObj = null;
					String packageForHT = null;
					String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
					while (targetIt.hasNext()) {
						htObj = targetIt.next();
						if (htObj instanceof ICalendar) {
							com.ibm.wbit.tel.DocumentRoot htDoc = (com.ibm.wbit.tel.DocumentRoot)obj;
							ICalendar cal = (ICalendar)htObj;
							if (cal.getTargetNamespace() != null &&
									htDoc.getTask() != null &&
									htDoc.getTask().getTargetNamespace() != null &&
									cal.getTargetNamespace().equals(htDoc.getTask().getTargetNamespace().toString() + TimeConstants.CALENDAR_SUFFIX)) {
								packageForHT = sa.getPackage();
								sa = SAUtil.createUnclassifiedSolutionArtifact();
								UnclassifiedArtifact ua = SaFactory.eINSTANCE.createUnclassifiedArtifact();
								ua.getArtifacts().add(htObj);
								sa.getComposedArtifacts().add(ua);		
								sa.setPackage(packageForHT);
								sas.add(sa);
								break;
							}
						}
					}
				}			
			} else if(obj instanceof ICalendar){
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(), TimeIntervalsRule.class, obj);
				if (tr != null) {	// only for top level timetables (not associated with HT)
					RecurringTimeIntervals rti = (RecurringTimeIntervals)tr.getSource().get(0);

					sa = SAUtil.createUnclassifiedSolutionArtifact();
					UnclassifiedArtifact ua = SaFactory.eINSTANCE.createUnclassifiedArtifact();
					ua.getArtifacts().add(obj);
					sa.getComposedArtifacts().add(ua);	
					
					String packageName = NamingUtil.registerNameForTimetable(rti, getContext());					
					if (packageName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
						packageName = packageName.substring(0, packageName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
						packageName = packageName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
					}
					sa.setPackage(packageName);
					sas.add(sa);
				}
				//AR 1609 add the MediationFlow and mdf on the list
			}else if(obj instanceof SACLRoot || obj instanceof JavaImplementation || obj instanceof BusinessRuleGroup || obj instanceof RuleSet || obj instanceof DocumentRoot 
					|| obj instanceof com.ibm.wbit.sib.mediation.operation.model.DocumentRoot || obj instanceof EPackage){
				ScdlArtifact scdlArtifact = SaFactory.eINSTANCE.createScdlArtifact();
				scdlArtifact.setArtifact(obj);
				sa = SaFactory.eINSTANCE.createSolutionArtifact();
				sa.setType(SAType.SCDL_LITERAL);
				sa.getComposedArtifacts().add(scdlArtifact);
				
				Action action = null;
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(), ComponentRule.class, obj);
				// GJD look for rules in BR Rules
				if (tr == null) {
					tr = TransformationUtil.getRuleForTarget(getRoot(), BRTaskRule.class, obj);
					if (tr == null) {
						tr = TransformationUtil.getRuleForTarget(getRoot(), BRReusableTaskRule.class, obj);
						if (tr != null && !tr.getSource().isEmpty() && tr.getSource().get(0) instanceof BusinessRuleAction) {
							action = (Action)tr.getSource().get(0);
						}
					} else {					
						action = ((InputPinSet)tr.getSource().get(0)).getAction();
					}
				}
				
				String registeredName = null;							
				if (action == null) {	// anything other than BRTaskRule
					action = (Action)tr.getSource().get(0);					
					//defect 84336
					if(obj instanceof SACLRoot || obj instanceof JavaImplementation || obj instanceof BusinessRuleGroup || obj instanceof RuleSet || obj instanceof com.ibm.wbit.sib.mediation.operation.model.DocumentRoot || obj instanceof EPackage){
						if(tr instanceof ComponentRule && tr.getSource().size() >= 3 && (tr.getSource().get(tr.getSource().size()-1) instanceof CallBehaviorAction)){					
							CallBehaviorAction cba = (CallBehaviorAction)(tr.getSource().get(tr.getSource().size()-1));
							sa.setPackage(SAUtil.createPackage(cba));
						}else{
							sa.setPackage(SAUtil.createPackage(action));
						}
					}else{
						sa.setPackage(SAUtil.createPackage(action));
					}

				} else {
					NoHashNamingRegistry noHashReg = null;
					if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
						String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(action);
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
					} else {
						noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(getContext(), NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
					}
					Object contextContainer = NamingUtil.getContextContainer(action);
				
					if (contextContainer instanceof StructuredActivityNode && noHashReg.actionExistInLocalRegistry(action, contextContainer)) {
						registeredName = noHashReg.retrieveNameFromLocalRegistry(action, contextContainer);
						if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
							registeredName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
							registeredName = registeredName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
						}
					} else if(noHashReg.actionExistInGlobalRegistry(action)) {
						registeredName = noHashReg.retrieveNameFromGlobalRegistry(action);
						if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
							registeredName = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR) + 1);
							registeredName = registeredName.replace(ExportOperation.FILE_SEPARATOR_CHAR, File.separatorChar);
						}
					}
					if (registeredName != null) {
						sa.setPackage(registeredName);
					} else {	
						sa.setPackage(SAUtil.createPackage(action));
					}
				}
 
//				sa.setPackage(SAUtil.createPackage(action));
				sas.add(sa);
				
				if(obj instanceof BusinessRuleGroup){
					BusinessRuleGroup brg = (BusinessRuleGroup)obj;
					if (brg.getBusinessRuleGroupTable() != null) {
						BusinessRuleGroupTable brgt = ((BusinessRuleGroup)obj).getBusinessRuleGroupTable();
						scdlArtifact = SaFactory.eINSTANCE.createScdlArtifact();
						scdlArtifact.setArtifact(brgt);
						sa = SaFactory.eINSTANCE.createSolutionArtifact();
						sa.setType(SAType.SCDL_LITERAL);
						sa.getComposedArtifacts().add(scdlArtifact);
						if (registeredName != null) {
							sa.setPackage(registeredName);
						} else {	
							if(tr instanceof ComponentRule && tr.getSource().size() >= 3 && (tr.getSource().get(tr.getSource().size()-1) instanceof CallBehaviorAction)){					
								CallBehaviorAction cba = (CallBehaviorAction)(tr.getSource().get(tr.getSource().size()-1));
								sa.setPackage(SAUtil.createPackage(cba));
							}else{
								sa.setPackage(SAUtil.createPackage(action));
							}
//							sa.setPackage(SAUtil.createPackage(action));
						}
						sas.add(sa);
					}
				}
			}else if (obj instanceof Process || obj instanceof Definition){
				TransformationRule tr = TransformationUtil.getRuleForTarget(getRoot(), ComponentRule.class, obj);
				if(tr != null){
					ScdlArtifact scdlArtifact = SaFactory.eINSTANCE.createScdlArtifact();
					scdlArtifact.setArtifact(obj);
					sa = SaFactory.eINSTANCE.createSolutionArtifact();
					sa.setType(SAType.SCDL_LITERAL);
					sa.getComposedArtifacts().add(scdlArtifact);
		
					Action action = (Action)tr.getSource().get(0);
					
					if(tr instanceof ComponentRule && tr.getSource().size() >= 3 && (tr.getSource().get(tr.getSource().size()-1) instanceof CallBehaviorAction)){					
						CallBehaviorAction cba = (CallBehaviorAction)(tr.getSource().get(tr.getSource().size()-1));
						sa.setPackage(SAUtil.createPackage(cba));
					}else{
						sa.setPackage(SAUtil.createPackage(action));
					}
//					sa.setPackage(SAUtil.createPackage(action));
					sas.add(sa);
				}
			}
		}
		return sas;
	}

	
	/**
	 * Creates a custom documentation for the WSDL Interface that will allow us to determine this interface was
	 * generated with Modeler. /Note: As WID does not support documentation at the WSDL Interface level, we will 
	 * attach the documentation at the PortType level/
	 * @param wsdlDefinition
	 * @param activity
	 */
	private void updateDefinitionDocumentation(Definition wsdlDefinition){
		//feature 117494
			((DefinitionImpl)wsdlDefinition).updateElement();
			List portTypes = wsdlDefinition.getEPortTypes();
			if(portTypes != null && !portTypes.isEmpty()){
				Iterator it = portTypes.iterator();
				while(it.hasNext()){
					PortType portType = (PortType)it.next();
					Element element = portType.getElement();
					Node firstChild = element.getFirstChild();
					
					Document portTypeDoc = portType.getElement().getOwnerDocument();
					//Element doc = wsdlDefinition.getDocument().createElementNS(AttributeNameConstants.WSDL_TARGET_NAMESPACE, "documentation");
					Element modelerDocumentation = portTypeDoc.createElementNS("http://schemas.xmlsoap.org/wsdl/" , "wsdl:documentation");
					StringBuffer serviceComment = new StringBuffer();
					serviceComment.append(AbstractbpelConstants.MODELER_VERSION_TAG);
					Node modelerText = portTypeDoc.createTextNode(serviceComment.toString());
					modelerDocumentation.appendChild(modelerText);
					if(firstChild != null){
						element.insertBefore(modelerDocumentation, firstChild);
					}else{
						element.appendChild(modelerDocumentation);
					}
					
				}
			}

			
		}

} //ProcessFlowRuleImpl
