/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.ecore.EClass;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.bom.model.time.TimeInterval;
import com.ibm.btools.bom.model.time.TimeIntervals;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.ExportOperationConstants;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.HeuristicsFactory;
import com.ibm.btools.te.ilm.heuristics.HeuristicsPackage;
import com.ibm.btools.te.ilm.heuristics.HeuristicsTransformation;
import com.ibm.btools.te.ilm.heuristics.TimeIntervalsRule;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.util.TimeConstants;
import com.ibm.btools.te.ilm.heuristics.util.TimeIntervalsUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.businesscalendar.CalFactory;
import com.ibm.wbit.businesscalendar.ICalendar;
import com.ibm.wbit.businesscalendar.Vcalendar;
import com.ibm.wbit.businesscalendar.Vevent;
import com.ibm.wbit.businesscalendar.util.CalendarUtil;

/**
 * Transform a TimeIntervals to a Business Calendar (ICalendar).
 * Sources: RecurringTimeIntervals
 * Target: ICalendar
 */
public class TimeIntervalsRuleImpl extends TransformationRuleImpl implements TimeIntervalsRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected TimeIntervalsRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return HeuristicsPackage.Literals.TIME_INTERVALS_RULE;
	}

	
	/**
	 * SOURCE: BOM TimeIntervals
	 * TARGET: Business Calendar
	 * Maps BOM TimeIntervals to BusinessCalendar as defined in schema.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target()	{	
		LoggingUtil.traceEntry(this, "transformSource2Target");

		if (isComplete()) {
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		
		//assertion
		if (getSource() == null || getSource().size() != 1) {
			LoggingUtil.traceExit(this, "transformSource2Target", "no source");
			return false;			
		}
			 
		// RecurringTimeIntervals from the Resource Model
		RecurringTimeIntervals rTimeIntervals = (RecurringTimeIntervals)getSource().get(0);

		if (rTimeIntervals == null) { 
			LoggingUtil.traceExit(this, "transformSource2Target", "no content for calendars");
			return false;			
		}
		
		ICalendar businessCalendar = CalFactory.eINSTANCE.createICalendar();
		
		//set iCal name for resource to recurring time name
		String calendarPath = NamingUtil.registerNameForTimetable(rTimeIntervals, getContext());
		String calendarName = null;
		if (calendarPath != null) {
			if (calendarPath.indexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) > -1) {
				calendarName = calendarPath.substring(calendarPath.lastIndexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) + 1);	
			} else {
				calendarName = calendarPath;
			}
		} else {
			calendarName = rTimeIntervals.getName();
		}
		businessCalendar.setName(calendarName);
		
		//create namespace
		businessCalendar.setTargetNamespace(createTargetNamespace(rTimeIntervals, calendarPath));

		Vcalendar busCalendar = createVCalendar(rTimeIntervals, calendarName);
		businessCalendar.setVcalendar(busCalendar);
		
		//set table as a target
		getTarget().add(businessCalendar);
	    	
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(BpelOptions.isBestPracticePatternEnabed(exportSession) ||
				BpelOptionsUtil.isLibraryOnly() ||
				BpelOptionsUtil.isLibraryEnabledWithModuleProject()){
			Object docRootRegistry = exportSession.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.TIMETABLE_NAME_REGISTRY);
			if(docRootRegistry instanceof Set){
				((Set)docRootRegistry).add(businessCalendar);
			}			
		}
		
		executeHandlers();
		setComplete(true);
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	
	// create calendar for RecurringTimeIntervals
	private Vcalendar createVCalendar (RecurringTimeIntervals rti, String calendarName) {
		Vcalendar busCalendar = CalFactory.eINSTANCE.createVcalendar();
		busCalendar.setProdid(TimeConstants.PRODUCT_ID + TimeConstants.INTERNAL_ID + rti.getUid() + TimeConstants.CLOSING_BRACKET);
		// set the name of the resource calendar to the rti's name
		busCalendar.setXCalname(calendarName);
		busCalendar.setVersion(CalendarUtil.VERSION_1000);
		populateFromTopCalendar(rti, busCalendar);
		
		return busCalendar;
	}		
			
	// populate from top level calendar
	private Vcalendar populateFromTopCalendar(RecurringTimeIntervals rTimeIntervals, Vcalendar busCalendar) { 
		if (rTimeIntervals == null) {
			return busCalendar;
		}
						
		if (rTimeIntervals.getInterval() != null && !rTimeIntervals.getInterval().isEmpty()) {
			Iterator tiIterator = rTimeIntervals.getInterval().iterator();
			TimeInterval ti = null;
			Vevent vEvent = null;
			
			while (tiIterator.hasNext()) {
				ti = (TimeInterval) tiIterator.next();
				vEvent = TimeIntervalsUtil.createVevent(ti, rTimeIntervals);	//this is work period 
				if (vEvent != null) {
					busCalendar.getVevent().add(vEvent);
				}
			}
			
			// process exemptions
			if (rTimeIntervals.getExemptPeriod() != null && !rTimeIntervals.getExemptPeriod().isEmpty()) {
				Iterator exemptionIterator = rTimeIntervals.getExemptPeriod().iterator();
				
				TimeIntervals exemptionTimeIntervals = null;
				while (exemptionIterator.hasNext()) {
					exemptionTimeIntervals = (TimeIntervals)exemptionIterator.next();
					populateCalendar(exemptionTimeIntervals, busCalendar);
				}
			}
		}
		
		return busCalendar;
	}
	
	// populate based on the exemption timetables included in the timetable
	private Vcalendar populateCalendar(TimeIntervals timeIntervals, Vcalendar busCalendar) { 
		if (timeIntervals == null || timeIntervals.getRecurringTimeIntervals() == null ||
				timeIntervals.getRecurringTimeIntervals().isEmpty()) {
			return null;
		}
				
		// populate recurrance and create vevent for every TimeInterval
		RecurringTimeIntervals rTI = null;
		Iterator rtiIterator = timeIntervals.getRecurringTimeIntervals().iterator();
		TimeIntervalsRule tRule = null;
		
		while (rtiIterator.hasNext()) {
			rTI = (RecurringTimeIntervals) rtiIterator.next();
			
			tRule = TimeIntervalsUtil.findTimeIntervalsRule(rTI, getRoot());
			if (tRule == null) {
				tRule = HeuristicsFactory.eINSTANCE.createTimeIntervalsRule();
				tRule.getSource().add(rTI);
				getParentRule().getChildRules().add(tRule);		// if no parent, add to the parent rule
				tRule.transformSource2Target();
				
				if (!(tRule.getParentRule() instanceof HeuristicsTransformation) && tRule.getTarget() != null && !tRule.getTarget().isEmpty()) {
					tRule.getParentRule().getTarget().addAll(tRule.getTarget());
				}
			} else if (tRule.getTarget() == null || tRule.getTarget().isEmpty()) {
				tRule.transformSource2Target();
			}
			
			// retrieve from target
			if (tRule.getTarget() != null && !tRule.getTarget().isEmpty() && 
					tRule.getTarget().get(0) instanceof ICalendar) {
				ICalendar iCal = (ICalendar)tRule.getTarget().get(0);
				String tns = iCal.getTargetNamespace();
				String name = iCal.getName();
				busCalendar.getVexclude().add(tns + TimeConstants.COLON + name);
			} else { 	// no target
				String params[] = {rTI.getName()};
				LoggingUtil.logError(MessageKeys.TE_FAILED_TO_CREATE_TIMETABLE, params, null);
				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_FAILED_TO_CREATE_TIMETABLE));
				throw runtimeEx;
			}
		}
		
		return busCalendar;
	}
	
	/**
	 * Builds the TargetNamespace for the RecurringTimeIntervals.
	 * @param rti RecurringTimeIntervals
	 * @return String targetNamespace
	 */
	private String createTargetNamespace(RecurringTimeIntervals rti, String calendarPath){
		String uniqueName = null;
		// get the name made unique for the last folder of the Calendar (i.e. MondayToFri2 for resources/mondaytofri2/MondayToFri)
		if (calendarPath != null) {
			if (calendarPath.indexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) > -1) {
				String lastSegment = calendarPath.substring(calendarPath.lastIndexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) + 1);
				String pathNoLastSegment = calendarPath.substring(0, calendarPath.lastIndexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR));
				if (pathNoLastSegment.indexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) > -1) {
					uniqueName = pathNoLastSegment.substring(pathNoLastSegment.lastIndexOf(ExportOperationConstants.FILE_SEPARATOR_CHAR) + 1);
				} else {
					uniqueName = pathNoLastSegment;
				}
				if (uniqueName.length() > lastSegment.length()) {
					uniqueName = lastSegment + uniqueName.substring(lastSegment.length());
				} else {
					uniqueName = lastSegment;
				}
			} else {
				uniqueName = calendarPath;
			}
		} else {
			uniqueName = rti.getName();
		}
		String encodedNamespace = ProcessUtil.createNamespace(rti, true);
		JavaNCNameConverter nc = new JavaNCNameConverter();
		String convertedRTI = nc.convertName(uniqueName);
		return encodedNamespace + "/" +convertedRTI ;
	}

} //TimeIntervalsRuleImpl