package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRule;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleBlock;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSet;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.bom.model.processes.businessrules.TemplateParameter;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.heuristics.br.util.BRNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.SAUtil;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.Variable;

public class BusinessRuleNameProvider extends NameProvider {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	public static final String BUSINESSRULE_NAME_PREFIX="businessRule";	
	public static final String NAME_SEPARATOR="_";

	/**
	 * Constructor for the class that instantiates the name validator.
	 *
	 */
	public BusinessRuleNameProvider() {
		nameValidator = new NameValidator();
	}
	
	/**
	 * Returns the default name of a Component that is unique within the given registry and in the context of the parent object.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		if (fromSource instanceof BusinessRuleAction || forType instanceof RuleSet || forType instanceof RuleTemplate || 
				forType instanceof Rule || forType instanceof Variable) {
			return getUniqueNameForBR(forType, fromSource, registry);
		}
		
		return getUniqueName(forType,fromSource);		
	}

	protected String getUniqueName(Object forType,NamedElement fromSource) {
		String fullName = BomUtils.getFullName(fromSource);
		JavaNCNameConverter converter = new JavaNCNameConverter();
		return converter.convertName(composeName(fromSource.getName(),NamingUtil.getKeyFromString(fullName)));
	}

	private String getUniqueNameForBR(Object forType, NamedElement fromSource, NamingRegistry registry){
		Object container = null;
		String registeredName = null;
		String convertedName = null;
		NoHashNamingRegistry noHashReg = null;

		if (fromSource instanceof BusinessRuleAction) {
			BusinessRuleAction brAction = (BusinessRuleAction)fromSource;
						
			if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
				String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(brAction);
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
			} else {
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
			}
			
			container = NamingUtil.getContextContainer(brAction);
			if(noHashReg.actionExistInGlobalRegistry(brAction)) {
				registeredName = noHashReg.retrieveNameFromGlobalRegistry(brAction);
			} else if (noHashReg.actionExistInLocalRegistry(brAction, container)) {
				registeredName = noHashReg.retrieveNameFromLocalRegistry(brAction, container);
			}		
			if (registeredName != null) {
				String fileName = NamingUtil.getNameForArtifact(registeredName);	
				if (fileName != null) {
					convertedName = fileName;
					((BRNamingRegistry)registry).register(forType, container, registeredName);
				}
			} else {
				registeredName = NamingUtil.convertNameForAction((Action)fromSource, getContext());
				String fileName = NamingUtil.getNameForArtifact(registeredName);	
				((BRNamingRegistry)registry).register(forType, container, registeredName);
				convertedName = fileName;
			}
		} else if (fromSource instanceof BusinessRuleSet) {
			BusinessRuleAction brAction = ((BusinessRuleSet)fromSource).getOwningAction();;
//			if (brAction.getInStructuredNode() != null) {	//local
//				container = brAction.getInStructuredNode();
//			} else { //reusable
//				container = brAction.getActivity();
//			}
			if (NamingUtil.USE_PER_PROJECT_UNIQUENESS) {
				String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(brAction);
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(projectName);
			} else {
				noHashReg = ((ProjectNamingRegistry)NamingUtil.getProjectNamingRegistry(context, NamingUtil.PROJECT_NAMING_REGISTRY)).getProjectRegistry(NamingUtil.GLOBAL_XPROJECT_UNIQUENESS);
			}
			container = NamingUtil.getContextContainer(brAction);
			if(noHashReg.actionExistInGlobalRegistry(brAction)) {
				registeredName = noHashReg.retrieveNameFromGlobalRegistry(brAction);
			} else if (noHashReg.actionExistInLocalRegistry(brAction, container)) {
				registeredName = noHashReg.retrieveNameFromLocalRegistry(brAction, container);
			}		
			if (registeredName != null) {
				if (registeredName.indexOf(ExportOperation.FILE_SEPARATOR) != -1) {
					container = registeredName.substring(0, registeredName.lastIndexOf(ExportOperation.FILE_SEPARATOR));
				} else {
					container = registeredName;
				}
			} else {
				container = SAUtil.createPackage(brAction, ExportOperation.FILE_SEPARATOR);
			}
		} else if (fromSource instanceof BusinessRule) {
			BusinessRuleBlock brBlock = ((BusinessRule)fromSource).getRuleBlock();
			container = brBlock.getRuleSet();
		} else if (fromSource instanceof BusinessRuleTemplate) {
			container = ((BusinessRuleTemplate)fromSource).getRuleSet();
		} else if (fromSource instanceof TemplateParameter) {
			container = (BusinessRuleTemplate)fromSource.eContainer();
		}
		
		if (container == null) {
			if (registeredName != null  && registry != null) {
				String fileName = NamingUtil.getNameForArtifact(registeredName);
			
				if (fileName != null) {
					convertedName = fileName;
					registry.register(forType, registeredName);
				} else {
					String sourceName = fromSource.getName();
					JavaNCNameConverter converter = new JavaNCNameConverter();
					convertedName = converter.convertName(registry, forType,  sourceName, container);
				}
			} else {
				String sourceName = fromSource.getName();
				JavaNCNameConverter converter = new JavaNCNameConverter();
						convertedName = converter.convertName(registry, forType,  sourceName);
			}
		} else if (convertedName == null) {
			String sourceName = fromSource.getName();
			JavaNCNameConverter converter = new JavaNCNameConverter();
			convertedName = converter.convertName(registry, forType,  sourceName, container);
		}
		return convertedName;
		
	}
	
	public static String composeName(String name,String key) {
		return name+NAME_SEPARATOR+key;
	}
	
}
