package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.StringTokenizer;

import org.apache.xerces.util.XMLChar;

import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleSet;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbit.bpelpp.Parameter;
import com.ibm.wbit.br.core.model.RuleSet;

public class JavaNCNameConverter implements NameConverter {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	// TODO: what should this max length be set to
	static final int MAX_NAME_LENGTH = 100;
	static final char[] NC_NAME_INVALID_CHAR =
						{':', ';', '<', '>', '?', '/', '\\', '{', '}', 
						'[', ']', '~', '`', '!', '@', '#', '$', '%', 
						'^', '&', '(', ')', '=', '+', '|',' '};
						
	static final String[] JAVA_KEY_WORD =
				{"abstract", "default", "if", "private", "this", "boolean", 
				"do", "implements", "protected", "throw", "break", "double", 
				"import", "public", "throws", "byte", "else", "instanceof", 
				"return", "transient", "case", "extends", "int", "short", 
				"try", "catch", "final", "interface", "static", "void", 
				"char", "finally", "long", "strictfp", "volatile", "class", 
				"float", "native", "super", "while", "const", "for", "new", 
				"switch", "continue", "goto", "package", "synchronized", 
				"true", "false", "null"};				

	/**
	 * Returns a String that is a valid name
	 * It converts the name into a valid name that is also unique in the registry	 
	 * @param registry  the registry to store the source name
	 * @param obj  the object of the construct
	 * @param sourceName  the String name to be converted to a valid name	 
	 * @return String the converted String name
	 */				
	public String convertName(NamingRegistry registry, Object obj, String sourceName)
	{		
		if (sourceName == null) {
			return null;
		}
		
		// 1. Replace any invalid characters in the sourceName 
		String newName = replaceInvalidName(sourceName);
		
		//EObject containerObj = findContainerObject(obj, rule);
		//NamingRegistry registry = NamingUtil.getRegistry(context, containerObj);
		
		// 2. check to see if the name is unique, generate one if it's not unique
		if (registry != null && !registry.isUniqueName(obj, newName)){
			return generateNewName(newName, obj, registry);
		}
		else{
			return newName;
		}
		
	}
	
	
	/**
	 * Returns a String that is a valid name
	 * It converts the name into a valid name that is also unique in the registry	 
	 * @param registry  the registry to store the source name
	 * @param obj  the object of the construct
	 * @param sourceName  the String name to be converted to a valid name	 
	 * @return String the converted String name
	 */				
	public String convertName(NamingRegistry registry, Object obj, String sourceName, Object inParentContext)
	{	
		if(inParentContext == null){
			return convertName(registry, obj, sourceName);
		}
		// 1. Replace any invalid characters in the sourceName
		String newName = replaceInvalidName(sourceName);		
		
		//EObject containerObj = findContainerObject(obj, rule);
		//NamingRegistry registry = NamingUtil.getRegistry(context, containerObj);
		
		// 2. check to see if the name is unique, generate one if it's not unique
		if (obj instanceof BusinessRuleGroup || obj instanceof RuleSet ||
				inParentContext instanceof BusinessRuleSet || inParentContext instanceof BusinessRuleTemplate) {
			if (registry != null && (!registry.isUniqueName(obj, inParentContext, newName))){	// checks for existance, and registers if not found
				return generateNewName(newName, obj, inParentContext, registry);
			} else {
				return newName;
			}
		} else if (registry != null && !registry.isUniqueName(obj, inParentContext, newName)){
			return generateNewName(newName, obj, registry);
		} else {
			return newName;
		}
		
	}
	
	
	public String convertName(String sourceName){
		return replaceInvalidName(sourceName);
	}

	
	/**
	 * Returns a String with the specified characters removed	 
	 * @param name  the source string name
	 * @param strippedChars  an array of characters that need to be stripped	
	 * @return String the String name with the specified characters removed
	 */	
	private static String stripCharacters(String name, char[] strippedChars)
	{
		StringTokenizer st = new StringTokenizer(name, new String(strippedChars));
		StringBuffer sb = new StringBuffer();
		while(st.hasMoreTokens()){
			sb.append(st.nextToken());
		}
		return sb.toString();		
	}
		
	/**
	 * Returns a new generated name that is unique in the registry 
	 * @param duplicatedName  the duplicated name that needs to be replaced
	 * @param registry  the namingRegistry
	 * @param obj the EObject construct
	 * @return String the newly generated string name
	 */	
	private String generateNewName(String duplicatedName, Object obj, NamingRegistry registry)
	{
		int count = 2;
		String newName = duplicatedName  + "_" + count++;
		// Keep trying to generate new name until name is unique
		while (!registry.isUniqueName(obj, newName))
		{
			newName = duplicatedName  + "_" + count++;
		}
		return newName;
	}
	
	/**
	 * Returns a new generated name that is unique in the registry 
	 * @param duplicatedName  the duplicated name that needs to be replaced
	 * @param registry  the namingRegistry
	 * @param obj the EObject construct
	 * @return String the newly generated string name
	 */	
	private String generateNewName(String duplicatedName, Object obj, Object parentContext, NamingRegistry registry)
	{
		int count = 2;
		String newName = duplicatedName  + "_" + count++;
		// Keep trying to generate new name until name is unique
		while (!registry.isUniqueName(obj, parentContext, newName))
		{
			newName = duplicatedName  + "_" + count++;
		}
		return newName;
	}


	/**
	 * Returns a valid String name
	 * @param name  the source string name
	 * @return String the valid string name
	 */	
	public static String replaceInvalidName(String name)
	{		
		if (name == null) {
			return null;
		}

		//0. check to see if it the first character of the name is valid
		//name = replaceInvalidFirstCharacter(name);
		
		//1. check to see if it contains any invalid NCName character
		//name = stripCharacters(name, NC_NAME_INVALID_CHAR);
			
		char[] invalidNCNameList= new char[MAX_NAME_LENGTH];
		int count=0;
		
		while(!XMLChar.isValidNCName(name)){
			for(int i=0; i<name.length(); i++){
				if (!XMLChar.isNCName(name.charAt(i)) &&  name.indexOf(name.charAt(i))==i){					
					invalidNCNameList[count++]=name.charAt(i);
				}
			}
			name = stripCharacters(name, invalidNCNameList);
			if (name.length()==0){
				name="defaultName";			
			}
			if (!XMLChar.isNCNameStart(name.charAt(0))){
				if (name.charAt(0) == '.'){
					name = stripCharacters(name, new char[]{'.'});
				}else{
					name = 'a'+name;
				}
				
			}
		}
		
		//2. check to see if it contains any invalid Java character or if name is a java keyword
		name = replaceInvalidJavaIdentifier(name);
		
		return name;		
	}


	/**
	 * Returns a valid String name
	 * Checks to see if the first character of the name is valid
	 * @param name the source string name
	 * @return String the valid string name
	 */		
	private static String replaceInvalidFirstCharacter(String name)
	{		
		// invalid start if first character is not a java letter
		if (!Character.isJavaIdentifierStart(name.charAt(0))){
			//Add an 'a'
			StringBuffer sb = new StringBuffer();
			sb.append('a');
			sb.append(name);
			return sb.toString();			
		}
		else if (name.charAt(0) == '$'){
			//'$' is a valid identifier start in java, but invalid for NCName, 
			//therefore, replace '$' with 'a'
			StringBuffer sb = new StringBuffer();
			sb.append('a');
			sb.append(name.substring(1));
			return sb.toString();			
		}		
		else{
			return name;
		}
	}

	/**
	 * Returns a valid String name
	 * Replaces any invalid Java characters in the name and checks to make sure it's not a 
	 * predefined java identifier
	 * @param name the source string name
	 * @return String the valid string name
	 */		
	private static String replaceInvalidJavaIdentifier(String name)
	{
		String newName = name;
		char[] invalidChars = new char[MAX_NAME_LENGTH];
		int count=0;
		//replace any invalid Java characters
		for (int i=0; i<name.length(); i++)
		{
			if (!Character.isJavaIdentifierPart(name.charAt(i))){
				//strip the invalid characters
				invalidChars[count++]=name.charAt(i);
			}				
		}
		newName = stripCharacters(name, invalidChars);
		
		//check to see if the name is a java keyword, boolean literal or null literal
		if (isJavaKeyword(newName)){
			//add an 'a' in the front of the name 
			StringBuffer sb = new StringBuffer();
			sb.append('a');
			sb.append(newName);
			newName = sb.toString();
		}
		
		//check to see if it the first character of the name is valid
		newName = replaceInvalidFirstCharacter(newName);
		
		return newName;
	}

	/**
	 * Returns a valid String name
	 * Checks to make sure the name is not a Java keyword, boolean literal or null literal
	 * @param name the source string name
	 * @return String the valid string name
	 */			
	private static boolean isJavaKeyword(String name)
	{
		for (int i=0; i<JAVA_KEY_WORD.length; i++)
		{
			if (0==name.compareTo(JAVA_KEY_WORD[i]))
				return true;
		}
		return false;
	}

}
