package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.HashMap;
import java.util.Iterator;

import com.ibm.btools.bom.model.externalmodels.ServiceInterface;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;

/**
 * Naming registry for artifacts to ensure the unqiuenes of their name
 */
public class NoHashNamingRegistry implements NamingRegistry {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private  HashMap<Object, HashMap<Action, String>> localRegistry = new HashMap<Object, HashMap<Action, String>>();
	private  HashMap<Action, String> globalRegistry = new HashMap<Action, String>();
	private  HashMap<Action, String> wsdlRegistry = new HashMap<Action, String>();
	private  HashMap<RecurringTimeIntervals, String> timetableRegistry = new HashMap<RecurringTimeIntervals, String>();

	
	/**
	 * Check if this name is unique in global name registry	 
	 * @param obj Object	
	 * @param name String 
	 * @return boolean
	 */		
	public boolean isUniqueName(Object obj, String name) {
		LoggingUtil.traceEntry(this, "isUniqueName");
		if (name == null) {
			return true;	//could be set to false - but then infinite loop in name generation
		}
		if (obj instanceof Action) {
			if (getGlobalContextRegistry().values() == null) {
				return true;
			}
//TODO GJD 20080527		Iterator<String> it = getGlobalContextRegistry().values().iterator();
			Action thisAction = (Action)obj;
			Iterator<Action> it = getGlobalContextRegistry().keySet().iterator();
			Action action = null;
			while (it.hasNext()) {
				action = (Action)it.next();
				if (name.equals(getGlobalContextRegistry().get(action))) {
					if (BomUtils.isServiceOperation(action)) {
						if (action.eContainer() instanceof Activity && 
								((Activity)action.eContainer()).getOwningPackage() instanceof ServiceInterface &&
								thisAction.eContainer() instanceof Activity && 
								((Activity)thisAction.eContainer()).getOwningPackage() instanceof ServiceInterface &&
								((Activity)action.eContainer()).getOwningPackage() == ((Activity)thisAction.eContainer()).getOwningPackage()) {
							continue;
						} else {					
							return false;
						}
					} else {
						return false;
					}
				}
			}
			return true;
		}
		LoggingUtil.traceExit(this, "isUniqueName");
		return false;
	}

	/**
	 * Check if this name is unique in local name registry based on its container 
	 * @param obj Object	
	 * @param parentObject Object	
	 * @param name String 
	 * @return boolean
	 */		
	public boolean isUniqueName(Object obj, Object parentObject, String name){
		LoggingUtil.traceEntry(this, "isUniqueName");
		if (name == null) {
			return true;	//could be set to false - but then infinite loop in name generation
		}
		HashMap<Action, String> map = getLocalContextRegistry(parentObject);
		if (map == null){
			return true;		 		
		} else {
//			if (map.containsValue(name)) {		
			if (map.values() == null) {
				return true;
			}
			Iterator<String> it = map.values().iterator();
			while (it.hasNext()) {
				if (name.equalsIgnoreCase((String)it.next())) {
					return false;
				}
			}
			LoggingUtil.traceExit(this, "isUniqueName");
			return true;
		}
	}	

	/**
	 * Check if this name is unique in wsdl name registry 
	 * @param name String 
	 * @return boolean
	 */		
	public boolean isUniqueNameInWSDLRegistry(String name){
		LoggingUtil.traceEntry(this, "isUniqueNameInWSDLRegistry");
//		if (wsdlRegistry.containsValue(name)){
		if (name == null) {
			return true;	//could be set to false - but then infinite loop in name generation
		}
		if (wsdlRegistry.values() == null) {
			return true;
		}
		Iterator<String> it = wsdlRegistry.values().iterator();
		while (it.hasNext()) {
			if (name.equalsIgnoreCase((String)it.next())) {
				return false;
			}
		}
		LoggingUtil.traceExit(this, "isUniqueNameInWSDLRegistry");
		return true;
	}	
	
	/**
	 * Check if this name is unique in timetable name registry 
	 * @param name String 
	 * @return boolean
	 */		
	public boolean isUniqueNameInTimetableRegistry(String name){
		LoggingUtil.traceEntry(this, "isUniqueNameInTimetableRegistry");
		if (name == null) {
			return true;	//could be set to false - but then infinite loop in name generation
		}
		if (timetableRegistry.values() == null) {
			return true;
		}
		Iterator<String> it = timetableRegistry.values().iterator();
		while (it.hasNext()) {
			if (name.equalsIgnoreCase((String)it.next())) {
				return false;
			}
		}
		LoggingUtil.traceExit(this, "isUniqueNameInTimetableRegistry");
		return true;
	}	


	/**
	 * Deregister this object from global registry
	 * @param obj Object
	 * @param name String 
	 * @return 
	 */		
	public void deregister(Object obj, String name) {
		if (obj instanceof Action) {
			if (getGlobalContextRegistry().containsKey(obj)){
				getGlobalContextRegistry().remove(obj);
			}
		}
		return;
	}

	/**
	 * Deregister this object from local registry
	 * @param obj Object
	 * @param container Object
	 * @param name String 
	 * @return 
	 */		
	public void deregisterLocal(Object obj, Object container, String name) {
		if (obj instanceof Action) {
			HashMap<Action, String> map = getLocalContextRegistry(container);
			if (map == null){
				return;		 		
			} else if (map.containsKey(obj)){
				map.remove(obj);
			}
		}
		return;
	}

	/**
	 * Class based retrieval not implemented for this registry
	 * @param obj Class
	 * @param name String 
	 * @return Object
	 */		
	public Object retrieveFromRegistry(Class obj, String registryEntry) {	
		return null;
	}

	/**
	 * Register this object in global registry
	 * @param obj Object
	 * @param name String 
	 * @return
	 */		
	public void register(Object obj, String name) {
		if (obj instanceof Action) {
			Action action = (Action)obj;
			HashMap<Action, String> map = getGlobalContextRegistry();			
			map.put(action, name);
		}
		return;
	}
	
	/**
	 * Register this object in local registry
	 * @param obj Object
	 * @param parentObject Object
	 * @param name String 
	 * @return
	 */		
	public void register(Action action, Object parentObject, String name) {
		HashMap<Action, String> map = getLocalContextRegistry(parentObject);			
		if (map == null){
			HashMap<Action, String> entries = new HashMap<Action, String>();
			entries.put(action, name);
			localRegistry.put(parentObject, entries);
			return;		 		
		} else {
			map.put(action, name);
		}
		return;
	}
	
	/**
	 * Register this wsdl in wsdl registry
	 * @param action Action
	 * @param name String 
	 * @return
	 */		
	public void registerWSDL(Action action, String name) {
		if (!wsdlRegistry.containsKey(action)){
			wsdlRegistry.put(action, name);
			return;		 		
		}
		return;
	}
	
	/**
	 * Register this timetable in timetable registry
	 * @param rti RecurringTimeIntervals
	 * @param name String 
	 * @return
	 */		
	public void registerTimetable(RecurringTimeIntervals rti, String name) {
		if (!timetableRegistry.containsKey(rti)){
			timetableRegistry.put(rti, name);
			return;		 		
		}
		return;
	}
	
	/**
	 * Returns a local registry for the specific obj	 
	 * @param obj Object	
	 * @return HashMap
	 */		
	public HashMap<Action, String> getLocalContextRegistry(Object ctx) {
		HashMap<Action, String> hashMap = (HashMap<Action,String>)localRegistry.get(ctx);
		return hashMap;
	}

	/**
	 * Returns a local registry	 
	 * @param obj Object	
	 * @return HashMap
	 */		
	public HashMap<Object, HashMap<Action, String>> getLocalContextRegistries() {
		return localRegistry;
	}

	/**
	 * Returns a global registry 	
	 * @return HashMap
	 */		
	public HashMap<Action, String> getGlobalContextRegistry() {
		return globalRegistry;
	}

	/**
	 * Returns an WSDL registry 
	 * @return HashMap
	 */		
	public HashMap<Action, String> getWSDLRegistry() {
		return wsdlRegistry;
	}

	/**
	 * Returns an Timetable registry 
	 * @return HashMap
	 */		
	public HashMap<RecurringTimeIntervals, String> getTimetableRegistry() {
		return timetableRegistry;
	}

	/**
	 * Returns a name for the specific Action from global registry	 
	 * @param obj  the Action	
	 * @return String the corresponding name for obj
	 */		
	public String retrieveNameFromGlobalRegistry(Action action) {	
		if (getGlobalContextRegistry().containsKey(action)) {
			return (String)getGlobalContextRegistry().get(action);
		}
		return null;
	}
	
	/**
	 * Returns a name from local registry for the specific Action	 
	 * @param action Action	
	 * @param container Object	
	 * @return String the corresponding name for obj
	 */		
	public String retrieveNameFromLocalRegistry(Action action, Object container) {	
		HashMap<Action, String> map = getLocalContextRegistry(container);			
		if (map != null){
			return (String)map.get(action);		 		
		}
		return null;
	}
	
	/**
	 * Returns a wsdl name for the specific Action	 
	 * @param action Action	
	 * @return String the corresponding name for obj
	 */		
	public String retrieveNameFromWSDLRegistry(Action action) {	
		if (getWSDLRegistry().containsKey(action)) {
			return (String)getWSDLRegistry().get(action);
		}
		return null;
	}
	
	/**
	 * Returns a timetable name for the specific timetable	 
	 * @param rti RecurringTimeIntervals	
	 * @return String the corresponding name for obj
	 */		
	public String retrieveNameFromTimetableRegistry(RecurringTimeIntervals rti) {	
		if (getTimetableRegistry().containsKey(rti)) {
			return (String)getTimetableRegistry().get(rti);
		}
		return null;
	}
	
	/**
	 * Check if this action exists in local name registry based on its container 
	 * @param obj Object	
	 * @param parentObject Object	
	 * @return boolean
	 */		
	public boolean actionExistInLocalRegistry(Object obj, Object parentObject){
		LoggingUtil.traceEntry(this, "actionExistInRegistry");
		HashMap<Action, String> map = getLocalContextRegistry(parentObject);
		if (map == null){
			return false;		 		
		} else {
			if (map.containsKey(obj)){
				return true;
			}else {
				return false;
			}
		}
	}	

	/**
	 * Check if this action exists in global name registry 
	 * @param obj Object	
	 * @return boolean
	 */		
	public boolean actionExistInGlobalRegistry(Object obj){
		LoggingUtil.traceEntry(this, "actionExistInGlobalRegistry");
		if (getGlobalContextRegistry().containsKey(obj)){
			return true;
		}else {
			return false;
		}
	}	

	/**
	 * Check if this action exists in wsdl name registry 
	 * @param obj Object	
	 * @return boolean
	 */		
	public boolean actionExistInWSDLRegistry(Object obj){
		LoggingUtil.traceEntry(this, "actionExistInWSDLRegistry");
		if (getWSDLRegistry().containsKey(obj)){
			return true;
		}else {
			return false;
		}
	}	

	/**
	 * Check if this timetable exists in timetable name registry 
	 * @param obj Object	
	 * @return boolean
	 */		
	public boolean timetableExistInTimetableRegistry(Object obj){
		LoggingUtil.traceEntry(this, "timetableExistInTimetableRegistry");
		if (getTimetableRegistry().containsKey(obj)){
			return true;
		}else {
			return false;
		}
	}	

	/**
	 * Check if this name exists in local name registry based on its container 
	 * @param parentObject Object	
	 * @param fullName String	
	 * @return boolean
	 */		
	public boolean nameExistInLocalRegistry(Object parentObject, String fullName){
		LoggingUtil.traceEntry(this, "nameExistInLocalRegistry");
		if (fullName == null) {
			return false;	
		}
		HashMap<Action, String> map = getLocalContextRegistry(parentObject);
		if (map == null){
			return false;		 		
		} else {
//			if (map.containsValue(fullName)){
			if (map.values() == null) {
				return false;
			}
			Iterator<String> it = map.values().iterator();
			while (it.hasNext()) {
				if (fullName.equalsIgnoreCase((String)it.next())) {
					return true;
				}
			}
			LoggingUtil.traceExit(this, "nameExistInLocalRegistry");
			return false;
		}
	}	

	/**
	 * Check if this name exists in global name registry 
	 * @param fullName String	
	 * @return boolean
	 */		
	public boolean nameExistInGlobalRegistry(String fullName){
		LoggingUtil.traceEntry(this, "nameExistInGlobalRegistry");
		if (fullName == null) {
			return false;
		}
//		if (getGlobalContextRegistry().containsValue(fullName)){
		if (getGlobalContextRegistry().values() == null) {
			return false;
		}
		Iterator<String> it = getGlobalContextRegistry().values().iterator();
		while (it.hasNext()) {
			if (fullName.equalsIgnoreCase((String)it.next())) {
				return true;
			}
		}
		LoggingUtil.traceExit(this, "nameExistInGlobalRegistry");

		return false;
	}
	
}
