package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.humantasks.Escalation;
import com.ibm.btools.bom.model.processes.humantasks.HumanTask;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.HumanTaskUtil;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.BPELVariable;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.tel.TEmail;
import com.ibm.wbit.tel.TEscalation;
import com.ibm.wbit.tel.TTask;

/**
* Provider of BPEL Elements names.
*/

public class TELNameProvider extends NameProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static String VARIABLE_NAME_SUFFIX = "Variable";
	public TELNameProvider(){
		nameValidator = new NameValidator();
	}
	/**
	 * Generates an UNIQUE, VALID value for the <code>propertyAttribute</code> of the <code>type</code>
	 * @param forType  a BPEL or WSDL Object
	 * @param propertyName name of the attribute property
	 * @param fromSource  BOM source object
	 * @param registry a NamingRegistry  
	 * @return String default name.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		if(forType instanceof TTask){
			return getDefaultTELElementName(forType, fromSource, registry,getParentContext(fromSource));
		}else if(forType instanceof TEscalation){
			return getDefaultTELElementName(forType, fromSource, registry,getParentContext(fromSource));
		}else if(forType instanceof TEmail){
			return getDefaultTELElementName(forType, fromSource, registry,getParentContext(fromSource));
		}
		else{
			return getDefaultTELElementName(forType, fromSource, registry,getParentContext(fromSource));
		}
	}

	private Object getParentContext(NamedElement fromSource) {
		if (parentContext==null) {
			if (fromSource instanceof HumanTask) {
				HumanTask humanTask = (HumanTask)fromSource;
				if (HumanTaskUtil.isLocalHumanTask(humanTask)) {
					// iTel all goes under the process directory
					parentContext = BomUtils.getContainingProcess(humanTask);
				} else {
					parentContext = humanTask.getActivity().getOwningPackage();
				}		
			}else if(fromSource instanceof Action && BomUtils.isTopLevelProcess((Action)fromSource)){ 
				parentContext = fromSource;
			}else {
				parentContext = fromSource.eContainer();
			}
		}
		return parentContext;
	}
	
	/**
	 * Returns a BPEL ElementName (unique in the registry) based on the <code>fromSource</code> name. If the registry is not set
	 * will create the name based on the source Name, not checking for uniqueness, else, the generated name will be 
	 * UNIQUE withing the given registry.
	 * @param forType a WSDL Element Type
	 * @param fromSource a source BOM NamedElement, source of the Name
	 * @param registry a Registry for identical type name that insures that the name is unique. 
	 * @return String the default BPEL Element name
	 */
	private String getDefaultTELElementName(Object forType, NamedElement fromSource, NamingRegistry registry, Object parentContext){
		String convertedName = null;
		if(registry == null){
			convertedName = getName(fromSource);
		}else{
			String sourceName = fromSource.getName();
			JavaNCNameConverter converter = new JavaNCNameConverter();
			convertedName = converter.convertName(registry, forType,  sourceName, parentContext);
		}
		return convertedName;
	}
}
