package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.apache.xerces.util.URI;
import org.apache.xerces.util.URI.MalformedURIException;
import org.eclipse.emf.common.command.Command;
import org.eclipse.wst.common.uriresolver.internal.util.URIEncoder;
import org.eclipse.wst.wsdl.Definition;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PackageableElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.time.RecurringTimeIntervals;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.attributes.helper.TechnicalAttributesHelper;
import com.ibm.btools.te.attributes.helper.UpdateCommandHelper;
import com.ibm.btools.te.attributes.helper.ValidationRecordHelper;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.TransformationEngine;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.NamespaceNamingRegistry;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;
import com.ibm.btools.te.validation.model.TeValidationRecord;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.model.utils.NamespaceUtils;

/**
* Generic provider for the targetNamespace.
*/
public class TargetNamespaceProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	//validator for this provider
	TargetNamespaceValidator validator; 
	TransformationContext context;
	public static final String DEFAULT_PREFIX = "http://";
	public static final String CALLBACK_SUFFIX = "Callback";
	/**
	 * TargetNamespce validator.
	 */
	public class TargetNamespaceValidator{
		private  String validName;
		private String errorCode;

		public String getErrorCode(){
			return errorCode;
		}
		/** 
		 * Validates the <code>targetNamespace</code> value.
		 * @param targetNamespace
		 * @return boolean true if the value is correct.
		 */
		public boolean validateNamespace(String targetNamespace){
			try {
				//verifies the targetNamespace by trying to create an URI. If the
				//value is not correct or the value is not a generic URI, returns false.
				URI uri = new URI(targetNamespace);
				boolean isGenericURI = uri.isGenericURI();
				if(isGenericURI){
					return true;
				}else{
					errorCode = "invalid Namespace";
					//errorCode = MessageKeys.TE_ATTRIBUTES_INVALID_TARGET_NAMESPACE;
				    return false;
				}
			} catch (MalformedURIException e) {
				errorCode = "invalid Namespace";
				//errorCode = MessageKeys.TE_ATTRIBUTES_INVALID_TARGET_NAMESPACE;
			    return false;
			}
		}
		public boolean validateEncodedNamespace(String targetNamespace){
			try {
				//verifies the targetNamespace by trying to create an URI. If the
				//value is not correct or the value is not a generic URI, returns false.
				URI uri = new URI(targetNamespace);
			} catch (MalformedURIException e) {
				errorCode = "invalid Namespace";
			    return false;
			}
			return true;
		}

		/**
		 * Returns the correct value of the targetNamespace
		 * @param namedElement
		 * @return
		 */
		public String getValidTargetNamespace(NamedElement namedElement){
			return getDefaultTargetNamespace(namedElement, false);
		}
	}

	/**
	 * Creates a TargetNamespaceProvider. Initializes the validator for this TargetNamespaceProvider.
	 */
	public TargetNamespaceProvider() {
		validator = new TargetNamespaceValidator();
		
	}
	/**
	 * Returns the value of the targetNamespace.
	 * @param forType a BPEL or WSDL Object
	 * @param fromSource BOM source object
	 * @param propertyName the name of the technical attribute 
	 * @param context a TransformationContext
	 * @param useEncoded a flag indicating if the namespace should be encoded.
	 * @return
	 */
	public String getTargetNamespace(Object forType, NamedElement fromSource, String propertyName, TransformationContext context, boolean useEncoded){
		String targetNamespace = null;
		boolean isCBA = isFromCBA(fromSource);
		//In case the option is generateDefaultValuesWithOverwrite should do:
		//1. generate the default Value
		//2. Generate the command to update the Model with the new value
		//3. Register the command.
		if(BpelOptionsUtil.generateDefaultValuesWithOverwrite()){
			//1.the default value is false
			targetNamespace = getDefaultTargetNamespace(fromSource, true);
			if(forType instanceof Definition){
				if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_TARGET_NAMESPACE)){
					targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;
				}else if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE)){
					targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + CALLBACK_SUFFIX + ExportOperation.WSDL_SUFFIX;
				}else{
					//d164778 add the last segment for local elements (when process and local task have the same names)
					//targetNamespace = targetNamespace + ExportOperation.WSDL_SUFFIX;
					targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;						
				}
			}
			//2c. call the commandUpdateHelper to update the BOM Model TA with new value
			Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, propertyName, targetNamespace);
			if(isCBA){
				ProcessUtil.putUpdateCommandsInContext(context, command);
			}else{
				TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);	
			}
			
		}else if(BpelOptionsUtil.generateDefaultValuesNoOverwrite()){
			//1. retrieve the technical attribute for this type... 
			String optionObject = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, propertyName);
			if(optionObject == null || optionObject.trim().equals("")){
				targetNamespace = getDefaultTargetNamespace(fromSource, true);
				if(forType instanceof Definition){
					if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_TARGET_NAMESPACE)){
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;
					}else if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE)){
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + CALLBACK_SUFFIX + ExportOperation.WSDL_SUFFIX;
					}else{
						//d164778 add the last segment for local elements (when process and local task have the same names)					
						//targetNamespace = targetNamespace + ExportOperation.WSDL_SUFFIX;
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;						
					}
				}
				Command command = UpdateCommandHelper.createUpdateCommand((NamedElement)fromSource, propertyName, targetNamespace);
				//2d. call the Registry Helper to add the command to the list of Commands
				if(isCBA){
					ProcessUtil.putUpdateCommandsInContext(context, command);
				}else{
					TransformationSessionUtil.appendAndExecuteUpdateCommand(TransformationEngine.getTransformationSession(), command);	
				}
				
			}else{
				targetNamespace = (String)optionObject;
			}
		}else if(BpelOptionsUtil.runTransformation()){
			//this means we are running the transformation with user setting without validating.. 
			//1. retrieve the technical attribute for this type... 
			String optionObject = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, propertyName);
			if(optionObject != null && !optionObject.trim().equals("")){
				targetNamespace = optionObject;
				//defect 167595 We need to encode the target namespace in case that
				//the user modifies the namespace using DBCS characters
				String encodedNamespace = null;				
				try{
                   //We need to decode the uri to namespace first, then encode it defect 167821
				   //This is to defend the user modify the target namespace using DBCS character
					targetNamespace = NamespaceUtils.convertUriToNamespace(targetNamespace);
					encodedNamespace = URIEncoder.encode(targetNamespace, "UTF8");
					if(encodedNamespace.getBytes().length > 220){
						String elementName = fromSource.getName();
				   		String params[]={elementName, targetNamespace};
						if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
							LoggingUtil.logError(MessageKeys.NAMESPACE_SIZE_UNSUPPORTED, params, null);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NAMESPACE_SIZE_UNSUPPORTED));
							throw runtimeEx;					
						}
					}
					StringBuffer sb = new StringBuffer();										
					//now check if the namespace is also java compliant.. 
					if (!Character.isJavaIdentifierStart(encodedNamespace.charAt(0))){
						//Add an 'a'
						sb = new StringBuffer();
						sb.append('a');
						sb.append(encodedNamespace);
						encodedNamespace = sb.toString();							
					}
					targetNamespace = encodedNamespace;
				}catch(Exception ex){
					String elementName = null;
					if (fromSource instanceof Action) {
						Action action = (Action)fromSource;
						elementName = NamingUtil.convertNameForAction(action, context);
						if (action.getInStructuredNode() != null && elementName.indexOf("/") != -1 && elementName.length() > 1) {
							String localName = elementName.substring(elementName.lastIndexOf("/") + 1);
							String containerPath = elementName.substring(0, elementName.lastIndexOf("/"));
					   		String params[] = {localName, containerPath, targetNamespace};
							LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_LOCAL_NAMESPACE, params, null);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_LOCAL_NAMESPACE));
							throw runtimeEx;						
						}
					} else {
						elementName = fromSource.getName();
					}
			   		String params[]={elementName, targetNamespace};
					LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_NAMESPACE, params, null);
					BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_NAMESPACE));
					throw runtimeEx;					
				}
				
			}else{
				targetNamespace = getDefaultTargetNamespace(fromSource, true);
				if(forType instanceof Definition){
					if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_TARGET_NAMESPACE)){
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;
					}else if(fromSource instanceof Action && BomUtils.isProcess((Action)fromSource) && propertyName.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_TARGET_NAMESPACE)){
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + CALLBACK_SUFFIX + ExportOperation.WSDL_SUFFIX;
					}else{
						//d161682 add the last segment for local elements (when process and local task have the same names)					
						//targetNamespace = targetNamespace + ExportOperation.WSDL_SUFFIX;
						targetNamespace = targetNamespace + "/" + targetNamespace.substring(targetNamespace.lastIndexOf("/")+1) + ExportOperation.WSDL_SUFFIX;						
					}
					if(targetNamespace.getBytes().length > 220){
						String elementName = fromSource.getName();
				   		String params[]={elementName};
						if(BpelOptionsUtil.isGenerateForDeployPatternEnabled()){
							LoggingUtil.logError(MessageKeys.NAMESPACE_SIZE_UNSUPPORTED, params, null);
							BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NAMESPACE_SIZE_UNSUPPORTED));
							throw runtimeEx;					
						}
					}
				}
	
			}
			//d159393
			if (!validator.validateEncodedNamespace(targetNamespace)) {
				String elementName = null;
				if (fromSource instanceof Action) {
					Action action = (Action)fromSource;
					elementName = NamingUtil.convertNameForAction(action, context);
					if (action.getInStructuredNode() != null && elementName.indexOf("/") != -1 && elementName.length() > 1) {
						String localName = elementName.substring(elementName.lastIndexOf("/") + 1);
						String containerPath = elementName.substring(0, elementName.lastIndexOf("/"));
				   		String params[] = {localName, containerPath, targetNamespace};
						LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_LOCAL_NAMESPACE, params, null);
						BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_LOCAL_NAMESPACE));
						throw runtimeEx;						
					}
				} else {
					elementName = fromSource.getName();
				}
		   		String params[]={elementName, targetNamespace};
				LoggingUtil.logError(MessageKeys.TE_ATTRIBUTES_INVALID_NAMESPACE, params, null);
				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.TE_ATTRIBUTES_INVALID_NAMESPACE));
				throw runtimeEx;
			}
		}else if(BpelOptionsUtil.runValidation()){
			//1. retrieve the technical attribute for this type... 
			String optionObject = (String)TechnicalAttributesHelper.getAttributeValue((NamedElement)fromSource, propertyName);
			if(optionObject == null || optionObject.trim().equals("")){
				targetNamespace = getDefaultTargetNamespace(fromSource, true);
				//2d. call the Registry Helper to add the command to the list of Commands
			}else{
				boolean isValid = validator.validateNamespace((String)optionObject);
				if(!isValid){
					String convertedNamespace = validator.getValidTargetNamespace((NamedElement)fromSource);
					TeValidationRecord validationRecord = ValidationRecordHelper.createRecord(fromSource, convertedNamespace, null, propertyName, validator.getErrorCode() );
					TransformationSessionUtil.addValidationRecord(TransformationEngine.getTransformationSession(), validationRecord);
					targetNamespace = convertedNamespace;
				}else{
					targetNamespace = (String)optionObject;
				}
			}
		}
		
		return targetNamespace;
	}
	
	/**
	 * Returns a default targetNamespace created based on the canonical name of the NamedElement.
	 * @param namedElement
	 * @param encoded
	 * @return
	 */
	public String getDefaultTargetNamespace (NamedElement namedElement, boolean encoded){
		return createNamespace (namedElement, DEFAULT_PREFIX, encoded);
	}
	/**
	 * Returns a default targetNamespace created based on the canonical nameof the NamedElement prefixed by the <code>namespacePrefix</code>.
	 * @param namedElement
	 * @param encoded
	 * @return
	 */
	private String createNamespace (NamedElement namedElement, String namespacePrefix, boolean encoded){
		if(encoded){
			String encodedNamespace = null;
			String targetNamespace = createNamespace(namedElement, namespacePrefix);
			try{
				encodedNamespace = URIEncoder.encode(targetNamespace, "UTF8");
				//now replace strip the % out
//				StringTokenizer st = new StringTokenizer(encodedNamespace, new String("%"));
				StringBuffer sb = new StringBuffer();
//				while(st.hasMoreTokens()){
//					sb.append(st.nextToken());
//				}
//				encodedNamespace = sb.toString();		
				//now check if the namespace is also java compliant.. 
				if (!Character.isJavaIdentifierStart(encodedNamespace.charAt(0))){
					//Add an 'a'
					sb = new StringBuffer();
					sb.append('a');
					sb.append(encodedNamespace);
					encodedNamespace = sb.toString();			
				}
				
			}catch(Exception ex){
				ex.printStackTrace();
				return targetNamespace;
			}
			return encodedNamespace;
		}else{
			return createNamespace (namedElement, DEFAULT_PREFIX);
		}
	}
	/**
	 * Createh the namespace String for the <code>namedElement</code> prefixed by the <code>namespacePrefix</code>
	 * @param namedElement a BOM NamedElement
	 * @param namespacePrefix prefix to be used for the targetNamespace
	 * @return string representing the targetNamespace 
	 */
	private String createNamespace(NamedElement namedElement, String namespacePrefix) {
		//AZ prepend the project name in the default target namespace URI
		String projectName = null;
		projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(namedElement);
		//Handle generate default TE action 
		if(projectName == null){
			NamedElement rootContainer = namedElement;
			if(!(namedElement instanceof Activity)){
				rootContainer = (NamedElement)namedElement.eContainer();
				if(namedElement instanceof StructuredActivityNode && namedElement.eContainer() instanceof NamedElement ||namedElement instanceof PinSet){
					while(rootContainer != null && !(rootContainer instanceof Activity )){
						rootContainer = (NamedElement)rootContainer.eContainer();
					}
				}
			}
			projectName = MappingUtil.getProjectNameForBOM(rootContainer);
		}
		URINameConverter uriConverter = new URINameConverter();		
		String ns = null;
		if(namedElement instanceof Activity){
			ns = getFullNamespace(namedElement);
			ns = projectName + "/" + ns;
		}else if (namedElement instanceof Type) {
			Type type = (Type) namedElement;
			ns = buildOwningPackageStructure(type,projectName);
//			Package belongingModel = type.getOwningPackage();
//			if(belongingModel == null || isHostNamePackage(belongingModel) || ProcessUtil.isRootPackage(belongingModel)){
//				ns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, projectName+"/"+belongingModel.getName(), false);
//			}else{
//				ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName());
//				belongingModel =belongingModel.getOwningPackage();
//				while (belongingModel != null && !isHostNamePackage(belongingModel) && !ProcessUtil.isRootPackage(belongingModel)) {
//					//ns = belongingModel.getName() + "/" + ns;
//					ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName() , false) + "/" + ns;
//					belongingModel = belongingModel.getOwningPackage();
//				}
//				if(belongingModel != null){
//					if(ns != null){
//						ns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName(), false) + "/" + ns;
//					}
//				}
//			}
		} else if (namedElement instanceof RecurringTimeIntervals) {
			ns = buildOwningPackageStructureForTimetable((RecurringTimeIntervals)namedElement, projectName);
		} else if (namedElement instanceof PackageableElement) {
			ns = getFullNamespace(namedElement);
			ns = projectName + "/" + ns;
		} else {
			ns = getFullNamespace(namedElement);
			ns = projectName + "/" + ns;
		}
		return namespacePrefix + ns;
	}

	/**
	 * Returns the BOM Namespace of the <code>namedElement</code>
	 * @param namedElement
	 * @return
	 */
	private String getFullNamespace(NamedElement namedElement){
		String topLevelActivityNamespace = null;
		String ns = "";
		if(namedElement instanceof Activity){
			ns = getTopLevelActivityNamespace((PackageableElement)namedElement);
		}else if(namedElement instanceof StructuredActivityNode && namedElement.eContainer() instanceof Activity){
			ns = getTopLevelActivityNamespace((PackageableElement)namedElement.eContainer());
		}else if(namedElement instanceof StructuredActivityNode){
			URINameConverter uriConverter = new URINameConverter();
			ns = namedElement.getName();
			ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, ns);
			// Ray : Form generation support
			if (namedElement.eContainer()==null) {
				return ns;
			}
			ns = getFullNamespace((NamedElement)namedElement.eContainer()) + "/" + ns;
		}else{
			URINameConverter uriConverter = new URINameConverter();
			NamedElement belongingElement = (NamedElement)namedElement.eContainer();
			if (belongingElement != null) {
				while(!(belongingElement instanceof StructuredActivityNode)){
						belongingElement = (NamedElement)belongingElement.eContainer();
				}
				ns = getFullNamespace((StructuredActivityNode)belongingElement);
			} else if (namedElement instanceof ExternalSchema) {
				ns = ((ExternalSchema)namedElement).getTargetNamespace();
			}
		}
		return ns;	
	}
	
	/**
	 * Returns the namespace for the top level BOM Activity.
	 * @param topLevelElement
	 * @return
	 */
	private String getTopLevelActivityNamespace(PackageableElement topLevelElement) {
		URINameConverter uriConverter = new URINameConverter();
		String ns = topLevelElement.getName();
		ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, ns);
		Action action = ((Activity)topLevelElement).getImplementation();
		//try to replace the Process/Task or Service which can be a working copy with the Master copy that is saved in the context.. 
		if(topLevelElement instanceof Activity && (BomUtils.isProcess(action) || BomUtils.isTask(action) || BomUtils.isService(action) || BomUtils.isBusinessRuleTask(action) || BomUtils.isHumanTask(action))){
			//PackageableElement registeredProcess = (PackageableElement)TransformationSessionUtil.getMasterCopy(TransformationEngine.getTransformationSession());
	//defect 139703, registrer the MasterCopy with the working copy
			PackageableElement registeredProcess = (PackageableElement)TransformationSessionUtil.getMasterCopy(TransformationEngine.getTransformationSession(), (Activity)topLevelElement);
			
			if(registeredProcess != null){
				topLevelElement = registeredProcess;
			}
		}
		Package belongingPackage = topLevelElement.getOwningPackage();
		if(belongingPackage == null || isHostNamePackage(belongingPackage) || ProcessUtil.isRootPackage(belongingPackage)){
			ns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, belongingPackage.getName(), false) + "/" + ns;
		}else{
			ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingPackage.getName())+  "/" + ns;
			belongingPackage =belongingPackage.getOwningPackage();
			while (belongingPackage != null && !isHostNamePackage(belongingPackage) && !ProcessUtil.isRootPackage(belongingPackage)) {
				ns = uriConverter.convertName(NamespaceNamingRegistry.getRegistry(), null, belongingPackage.getName() , false) + "/" + ns;
				belongingPackage = belongingPackage.getOwningPackage();
			}
			ns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, belongingPackage.getName(), false) + "/" + ns;

		}
		return ns;
	}
	/**
	 * Returns true if the namedElement, an Input/OutputPinSet belong to an Action which is called
	 * from the main Process through a CBA
	 * @param namedElement
	 * @return
	 */
	private boolean isFromCBA(NamedElement namedElement){
		Action action = null;
		if(namedElement instanceof InputPinSet){
			action = ((InputPinSet)namedElement).getAction();
		}else if (namedElement instanceof OutputPinSet){
			action = ((OutputPinSet)namedElement).getAction();
		}else if(namedElement instanceof StructuredActivityNode){
			action = (StructuredActivityNode)namedElement;
		}
		Action thisProcess = null;
		if(getContext() == null){
			return false;
		}
		TransformationRule procDefRule = ProcessUtil.getProcessDefinitionRule(getContext());
		if(procDefRule != null){
			thisProcess = ((com.ibm.btools.bom.model.processes.activities.Activity)ProcessUtil.getProcessDefinitionRule(getContext()).getSource().get(0)).getImplementation();
		}
		if(thisProcess == null){
			return false;
		}
		if((action instanceof StructuredActivityNode) && ((StructuredActivityNode)action).getInStructuredNode() == null && 
				("SERVICE".equals(((StructuredActivityNode)action).getAspect() )|| 
						"TASK".equals(((StructuredActivityNode)action).getAspect()))){
			return true;
		}else if(BomUtils.isTopLevelProcess(action) && thisProcess != action ){
			return true;
		}else{
			return false;
		}
		
	}
	
	/**
	 * @return Returns the context.
	 */
	public TransformationContext getContext() {
		return context;
	}
	/**
	 * @param context The context to set.
	 */
	public void setContext(TransformationContext context) {
		this.context = context;
	}
	/**
	 * Returns true if this is the first Package under the RootPackage. This package name will generate the hostPort  
	 * @param packageElement
	 * @return
	 */
	private boolean isHostNamePackage(Package packageElement){
		if(packageElement.getOwningPackage() != null && ProcessUtil.isRootPackage(packageElement.getOwningPackage())){
			return true;
		}
		return false;
	}
	
	/**
	 * This method is to build the data catalog structure based on the WBM workspace
	 * structure, prepend project name before the data catalog hierarchy
	 * @param type
	 * @param projectName
	 * @return
	 */
	private String buildOwningPackageStructure(Type type,String projectName){
		String tns = "";
		if(type == null ||projectName == null){
			return tns;
		}
		StringBuffer nsBuffer = new StringBuffer();		
		URINameConverter uriConverter = new URINameConverter();	
		Package belongingModel = type.getOwningPackage();		
		while(belongingModel != null && !ProcessUtil.isRootPackage(belongingModel)){
			tns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName(), false);
			nsBuffer.insert(0, "/" + tns);
			belongingModel = belongingModel.getOwningPackage();			
		}
		nsBuffer.insert(0, projectName);		
		return nsBuffer.toString();
	}
	
	
	/**
	 * This method is to build the data catalog structure based on the WBM workspace
	 * structure, prepend project name before the data catalog hierarchy
	 * @param type
	 * @param projectName
	 * @return
	 */
	private String buildOwningPackageStructureForTimetable(RecurringTimeIntervals rti, String projectName){
		String tns = "";
		if(rti == null || projectName == null){
			return tns;
		}
		StringBuffer nsBuffer = new StringBuffer();		
		URINameConverter uriConverter = new URINameConverter();	
		Package belongingModel = rti.getOwningPackage();		
		while(belongingModel != null && !ProcessUtil.isRootPackage(belongingModel)){
			tns = uriConverter.convertHostName(NamespaceNamingRegistry.getRegistry(), null, belongingModel.getName(), false);
			nsBuffer.insert(0, "/" + tns);
			belongingModel = belongingModel.getOwningPackage();			
		}
		nsBuffer.insert(0, projectName);		
		return nsBuffer.toString();
	}	

}
