package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.StringTokenizer;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.te.ilm.heuristics.helper.NamespaceNamingRegistry;


public class URINameConverter {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private int[] arrayURI = { 0, -902, -1, 2147483647 };
	//	TODO: what should this max length be set to
	 static final int MAX_NAME_LENGTH = 100;

	static final char[] URI_INVALID_CHAR =
							{';', '/', '?', ':', '@', '&', '=', '+', 
							'$', ',', '-', '_', '!', '~', 
							'*', '\'', '(', ')', '.'};
	static final char[] URI_INVALID_HOSTPORT_CHAR = {';', '/', '?', '=', '+',  '~', '\'', '(', ')', '.'};

	/**
	 * Returns a String with the specified characters removed	 
	 * @param name  the source string name
	 * @param strippedChars  an array of characters that need to be stripped	
	 * @return String the String name with the specified characters removed
	 */	
	private String stripCharacters(String name, char[] strippedChars)
	{
		StringTokenizer st = new StringTokenizer(name, new String(strippedChars));
		StringBuffer sb = new StringBuffer();
		while(st.hasMoreTokens()){
			sb.append(st.nextToken());
		}
		return sb.toString();		
	}


	/**
	 * Returns a String that is a valid name
	 * It converts the name into a valid name that is also unique in the registry	 
	 * @param registry  the registry to store the source name
	 * @param obj  the object of the construct
	 * @param sourceName  the String name to be converted to a valid name	 
	 * @return String the converted String name
	 */				
	public String convertName(NamingRegistry registry, EObject obj, String sourceName)
	{	
			
		if (!((NamespaceNamingRegistry)registry).isNameSpaceExists(sourceName))
		{		
			// 1. Replace any invalid characters in the sourceName		
			String newName = replaceInvalidName(sourceName);
				
			// 2. check to see if the name is unique, generate one if it's not unique		
			if (!registry.isUniqueName(sourceName, newName)){
				return generateNewName(newName, sourceName, registry);
			}
			else{
				return newName;
			}	
		}
		else {
			return ((NamespaceNamingRegistry)registry).getNamespaceName(sourceName);
		}		
	}

	public String convertName(NamingRegistry registry, EObject obj, String sourceName, boolean isCreateUnique)
	{		
		if (!((NamespaceNamingRegistry)registry).isNameSpaceExists(sourceName))
		{		
			// 1. Replace any invalid characters in the sourceName		
			String newName = replaceInvalidName(sourceName);
				
			// 2. check to see if the name is unique, generate one if it's not unique		
			if (isCreateUnique && !registry.isUniqueName(sourceName, newName)){
				return generateNewName(newName, sourceName, registry);
			}
			else{
				return newName;
			}	
		}
		else {
			return ((NamespaceNamingRegistry)registry).getNamespaceName(sourceName);
		}		
	}
	/**
	 * Returns a String that is a valid hostPort name.
	 * @param registry  the registry to store the source name
	 * @param obj  the object of the construct
	 * @param sourceName  the String name to be converted to a valid name	 
	 * @return String the converted String name
	 */		
	public String convertHostName(NamingRegistry registry, EObject obj, String sourceName, boolean isCreateUnique)
	{		
		if (!((NamespaceNamingRegistry)registry).isNameSpaceExists(sourceName))
		{		
			// 1. Replace any invalid characters in the sourceName		
			String newName = replaceInvalidHostName(sourceName);
				
			// 2. check to see if the name is unique, generate one if it's not unique		
			if (isCreateUnique && !registry.isUniqueName(sourceName, newName)){
				return generateNewName(newName, sourceName, registry);
			}
			else{
				return newName;
			}	
		}
		else {
			return ((NamespaceNamingRegistry)registry).getNamespaceName(sourceName);
		}		
	}


	/**
	 * Returns a new generated name that is unique in the registry 
	 * @param duplicatedName  the duplicated name that needs to be replaced
	 * @param registry  the namingRegistry
	 * @param obj the EObject construct
	 * @return String the newly generated string name
	 */	
	private String generateNewName(String duplicatedName, String sourceName, NamingRegistry registry)
	{
		int count = 2;
		String newName = duplicatedName + count++;
		// Keep trying to generate new name until name is unique
		while (!registry.isUniqueName(sourceName, newName))
		{
			newName = duplicatedName + count++;
		}
		return newName;
	}	
	
	private boolean isValidCharacter(char c) {
		return c >= 128 || ((arrayURI[c / 32] & (1 << (c % 32))) != 0);			
	}
	
	/**
	 * Returns true is the character is a valid character for a host/port character.
	 * @param c
	 * @return
	 */
	private boolean isValidHostChar(char c){
		//alpha
		 if (isValidCharacter(c) || (c > 'A' && c < 'Z') || (c > 'a' && c < 'z') ||
		 		//digit
                (c > '0' && c < '9') 
				//safe
				|| ( c == '$' || c=='-' || c=='_' || c=='@' || c=='.' || c=='&' || c=='+' )
				//extras
				|| ( c=='!' || c=='*' || c=='(' || c==')' || c=='\'' || c==',')){
		 	return true;
		 }
		 return false;
	}
	
	/**
	 * Returns true if the character is a valid character for a URI Port.
	 * @param c
	 * @return
	 */
	private boolean isValidPortChar(char c){
		//digit
		if ((c > '0' || c < '9')){
		 	return true;
		 }
		 return false;
	}
	
	private String replaceInvalidName(String sourceName)
	{		
		char[] invalidChars = new char[MAX_NAME_LENGTH];
		int count=0;
		//char[] invalidNCNameList= new char[MAX_NAME_LENGTH];
		
		for (int i=0; i<sourceName.length(); i++)
		{
			if (!isValidCharacter(sourceName.charAt(i))){
				//strip the invalid characters
				invalidChars[count++]=sourceName.charAt(i);
			}				
		}		
		sourceName = stripCharacters(sourceName, invalidChars);		
		sourceName = stripCharacters(sourceName, URI_INVALID_CHAR);
		sourceName = checkMultipleDots(sourceName); 
		
		//check NCName
		/*
		while(!XMLChar.isValidNCName(sourceName)){
			for(int i=0; i<sourceName.length(); i++){
				if (!XMLChar.isNCName(sourceName.charAt(i)) &&  sourceName.indexOf(sourceName.charAt(i))==i){					
					invalidNCNameList[count++]=sourceName.charAt(i);
				}
			}
			sourceName = stripCharacters(sourceName, invalidNCNameList);
			if (sourceName.length()==0){
				sourceName="defaultName";			
			}
			if (!XMLChar.isNCNameStart(sourceName.charAt(0))){
				sourceName = 'a'+sourceName;
			}
		}
		*/
		JavaNCNameConverter javaConverter = new JavaNCNameConverter();
		sourceName = javaConverter.convertName(sourceName);
		
		return sourceName; 
	}
	
	/**
	 * Strips invalid host/port characters.
	 * @param sourceName
	 * @return
	 */
	private String replaceInvalidHostName(String sourceName)
	{		
		char[] invalidHostChars = new char[MAX_NAME_LENGTH];
		char[] invalidPortChars = new char[MAX_NAME_LENGTH];
		int count=0;
		//eliminate whitespaces
		int portIndex = sourceName.indexOf(":");
		if (portIndex != -1){ 
			for (int i=0; i<portIndex; i++){
				if (!isValidHostChar(sourceName.charAt(i))){
					//strip the invalid characters
					invalidHostChars[count++]=sourceName.charAt(i);
				}				
			}		
			sourceName = stripCharacters(sourceName.substring(0, portIndex), invalidHostChars) + sourceName.substring(portIndex+1);	
			//get again the portIndex
			portIndex = sourceName.indexOf(":");
			for (int i=portIndex+1; i<sourceName.length(); i++)
			{
				if (!isValidPortChar(sourceName.charAt(i))){
					//strip the invalid characters
					invalidPortChars[count++]=sourceName.charAt(i);
				}				
			}		
			sourceName = sourceName.substring(0, portIndex+1) + stripCharacters(sourceName.substring(portIndex+1), invalidPortChars);
			//there was no digit after the ":" and we stripped all characters.. so remove the ":"
			if(sourceName.length() == portIndex +1){
				sourceName = sourceName.substring(0, portIndex);
			}
		}else{
			for (int i=0; i<sourceName.length(); i++){
				if (!isValidHostChar(sourceName.charAt(i))){
					//strip the invalid characters
					invalidHostChars[count++]=sourceName.charAt(i);
				}				
			}		
			sourceName = stripCharacters(sourceName, invalidHostChars);	
			
		}

		if(sourceName.length() == 0){
			sourceName = "defaultName";
		}
		return sourceName; 
	}
	
	/**
	 * Strips the multiple dots from the <code>sourceName</code>,
	 * @param sourceName
	 * @return string fixed
	 */
	private String checkMultipleDots(String sourceName){
		if (sourceName.indexOf("..")==-1){
			return sourceName;
		}else {			
			while (sourceName.indexOf("..")!=-1){
				String tempName = null;
				//find all dots and set tempName to the new name
				int startIndex = sourceName.indexOf("..");
				tempName = sourceName.substring(0, startIndex);				
				while(sourceName.charAt(startIndex++)=='.');
				tempName = tempName + sourceName.substring(startIndex-1);
				sourceName=tempName;
			}
			return sourceName;
		}
	}
	
}
