package com.ibm.btools.te.ilm.heuristics.naming;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;

/**
* Provider of WSDL Elements names.
*/
public class WSDLNameProvider extends NameProvider {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static String PORT_TYPE_CALLBACK_SUFFIX = "Callback";
	
	/**
	 * Constructor for the class that instantiates the name validator.
	 *
	 */
	public WSDLNameProvider(){
		nameValidator = new NameValidator();
	}
	
	/**
	 * Returns the name for the WSDL Element.
	 */
	public String getName(Object forType, NamedElement fromSource, NamingRegistry registry, String propertyName){
		if(fromSource instanceof Variable){
			return getDefaultWSDLElementName(forType, fromSource, registry, parentContext);
		}
		return super.getName(forType, fromSource, registry, propertyName);
	}
	
	
	/**
	 * Generates an UNIQUE, VALID value for the <code>propertyAttribute</code> of the <code>type</code>
	 * @param forType  a BPEL or WSDL Object
	 * @param propertyName name of the attribute property
	 * @param fromSource  BOM source object
	 * @param registry a NamingRegistry  
	 * @return String default name.
	 */
	public String getDefaultName(Object forType, String propertyName, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		if(propertyName == null){
			return getDefaultWSDLElementName(forType, fromSource, registry, parentContext);
		}else if(forType instanceof PortType && propertyName.equals(AttributeNameConstants.WSDL_PORT_TYPE_NAME)){
			return getDefaultWSDLElementName(forType, fromSource, registry, parentContext);
		}else if(forType instanceof PortType && propertyName.equals(AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_NAME)){
			return getDefaultCallbackPortTypeName(forType, fromSource, registry, parentContext);
		}else if(forType instanceof Message || forType instanceof Part || forType instanceof Operation){
			return getDefaultWSDLElementName(forType, fromSource, registry, parentContext);
		}else{
			return getDefaultWSDLElementName(forType, fromSource, registry, parentContext);
		}
	}	
	
	/**
	 * Returns an UNIQUE WSDL PortType callback Name (in the registry) based on the <code>fromSource</code> name.
	 * @param fromSource
	 * @param context
	 * @return String the default name of a callback PortType
	 */
	private String getDefaultCallbackPortTypeName(Object forType, NamedElement fromSource, NamingRegistry registry, Object parentContext) {
		JavaNCNameConverter converter = new JavaNCNameConverter();
		String convertedName = null;
		convertedName = converter.convertName(registry, forType,  fromSource.getName() + PORT_TYPE_CALLBACK_SUFFIX, parentContext);
	   	return convertedName;
	}

	/**
	 * Returns an UNIQUE WSDL ElementName (in the registry) based on the <code>fromSource</code> name.
	 * @param forType a WSDL Element Type
	 * @param fromSource a source BOM NamedElement, source of the Name
	 * @param registry a Registry for identical type name that insures that the name is unique. 
	 * @return String a defaultl WSDL element name
	 */
	private String getDefaultWSDLElementName(Object forType, NamedElement fromSource, NamingRegistry registry, Object parentContext){
		String convertedName = null;
		if(registry == null){
			convertedName = getName(fromSource);
		}else{
		    JavaNCNameConverter converter = new JavaNCNameConverter();
	    	convertedName = converter.convertName(registry, forType,  fromSource.getName(), parentContext);	
		}
		return convertedName;
	}


}
