/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 */
package com.ibm.btools.te.ilm.heuristics.scdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.xml.type.internal.QName;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.naming.ComponentNameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ExportRule;
import com.ibm.btools.te.ilm.heuristics.scdl.ScdlPackage;
import com.ibm.btools.te.ilm.heuristics.scdl.util.ScdlUtil;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Export;
import com.ibm.wsspi.sca.scdl.Import;
import com.ibm.wsspi.sca.scdl.InterfaceSet;
import com.ibm.wsspi.sca.scdl.SCAImportBinding;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType;

/**
 * <!-- begin-user-doc -->
 * An implementation of the model object '<em><b>Export Rule</b></em>'.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ExportRuleImpl extends TransformationRuleImpl implements ExportRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	
	private boolean executedOnce=false;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExportRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return ScdlPackage.Literals.EXPORT_RULE;
	}

	/**
	 * Source= DocumentRoot that contains the Component (target of the Export) and one or more wInterface (PortType)
	 * Target= SCDL Export
	 */
	@SuppressWarnings("unchecked")
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if (executedOnce) {
			updateInterfaceSet();
			return isComplete();
		}
		// Get the input params
        //AZ WBT 20070322 
		DocumentRoot docRoot = (DocumentRoot)getSource().get(0);
		
		PortType pt = (PortType)getSource().get(1);
		Component target = (Component)((DocumentRoot)getSource().get(0)).getComponent();
		
		//AZ WBT 20070322 -start
		String targetName = null;
		String targetDisplayName = null;
		if(target == null){
			Import targetImprt = (Import)docRoot.getImport();
			//Don't create export component for the sca import binding impelementation
			if(targetImprt.getBinding() instanceof SCAImportBinding){
				SCAImportBinding scaImportBinding = (SCAImportBinding)targetImprt.getBinding();
				if(scaImportBinding.getModuleName() == null && scaImportBinding.getExportName()== null){
					return isComplete();
				}
			}
			targetName = targetImprt.getName();
			targetDisplayName = targetImprt.getDisplayName();
		}else{
			targetName = target.getName();
			targetDisplayName = target.getDisplayName();
		}		
        //AZ WBT 20070322 -end
		
		// Create SCDL Export
		Export export = SCDLFactory.eINSTANCE.createExport();
		//export.setName(getUniqueName(target,pt));
		//export.setName(ComponentNameProvider.updateName(target.getName(),NamingUtil.getKeyFromString(target.getName()+"Export"),false));
		String convertedName = null; 
		List<Object> parentSources = getParentRule().getSource();	
		if (parentSources != null) {		
			Iterator<Object> it = parentSources.iterator();
			Object obj = null;
			Action fromSource = null;
			while (it.hasNext()) {
				obj = it.next();
				if (obj instanceof Action) {
					fromSource = (Action)obj;
					break;
				} else if (obj instanceof PinSet) {
					fromSource = (Action)((PinSet)obj).eContainer();
					break;
				}
			}
	
			if (fromSource != null) {	
				if(fromSource instanceof CallBehaviorAction){
					CallBehaviorAction cba = (CallBehaviorAction)fromSource;
					boolean isOverriden = FabricIntegrationUtil.isOverriden(cba);
					if(isOverriden){
						convertedName = NamingUtil.convertNameForAction((Action)fromSource, getContext());
					}else{
						convertedName = NamingUtil.convertNameForAction(((Activity)cba.getBehavior()).getImplementation(), getContext());
					}
				}else{
					convertedName = NamingUtil.convertNameForAction((Action)fromSource, getContext());
				}
			}		
		}
		if (convertedName != null) {
			export.setName(convertedName);
		} else {
			export.setName(ComponentNameProvider.updateName(targetName,NamingUtil.getKeyFromString(targetName +"Export"),false));
		}
		//export.setDisplayName(target.getDisplayName()+" Export");
		export.setDisplayName(targetDisplayName + " Export");
		// Set target Component
		//export.setTargetName(target.getName());
		//export.setTargetName(targetName);
		export.setTargetName(convertedName);
		// Set Interface
		// Ray - d155869
		export.setInterfaceSet(ScdlUtil.createInterfaceSet(pt,false));	
		// Create document root and update the namespace prefixes
		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setExport(export);
		//ScdlUtil.createNSPrefixForWSDLPortType(root);
		getTarget().add(root);
		executedOnce=true;
		setComplete(true);
		
		//AZ WMT 20070322
		executeHandlers();
		
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}

	/**
	 * Get a unique name for the Export
	 * @param fromSource
	 * @return
	 */
	private String getUniqueName(Component target,PortType portType) {
		String fullName = target.getName();
		return fullName+"_"+
			NamingUtil.getKeyFromString(fullName+portType.getQName().getNamespaceURI()+portType.getQName().getLocalPart());
	}	
	
	/**
	 * Add additional interfaces to the interfaceSet of the Component
	 *
	 */
	private void updateInterfaceSet() {
		Export export = ((DocumentRoot)getTarget().get(0)).getExport();
		if (export != null) {
			InterfaceSet is = export.getInterfaceSet();
			if (is != null) {
				for (int i=0; i<getSource().size();i++) {
					if (getSource().get(i) instanceof PortType) {
						PortType pt = (PortType)getSource().get(i);
						for (int j=0; j<is.getInterfaces().size();j++) {
							if (is.getInterfaces().get(j) instanceof WSDLPortType) {
								WSDLPortType wpt = (WSDLPortType)is.getInterfaces().get(j);
								QName qname = (QName)wpt.getPortType();
								if (qname.getNamespaceURI()!=pt.getQName().getNamespaceURI() && qname.getLocalPart()!=pt.getQName().getLocalPart()) {
										wpt = ScdlUtil.createwInterface(pt);
										if(!exist(is, pt)){
											is.getInterfaces().add(wpt);	
										}
								}
							}
						}
					}
				}				
			}	
			//ScdlUtil.createNSPrefixForWSDLPortType((DocumentRoot)getTarget().get(0));
		}			
	}
	/**
	 * We need to check if the interface has already been added to avoid duplication in the case of 
	 * multiple ( size > 1) call back CBA's in the process PMR 66655 004 000
	 * @param is 
	 * @param portType
	 * @return
	 */
	private boolean exist(InterfaceSet is, PortType portType){
		if(is != null && portType != null){
			List<WSDLPortType> interfaces = is.getInterfaces();
			for(WSDLPortType eachInterface:interfaces){
				QName qname = (QName)eachInterface.getPortType();
				if (qname.getNamespaceURI()== portType.getQName().getNamespaceURI() && qname.getLocalPart()== portType.getQName().getLocalPart()) {
					return true;
				}
			}
		}
		return false;
	}
} //ExportRuleImpl
