package com.ibm.btools.te.ilm.heuristics.scdl.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xml.type.util.XMLTypeUtil;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.bom.command.compound.PredefTypesConst;
import com.ibm.btools.bom.model.artifacts.Comment;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnit;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.services.Behavior;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.etools.eflow2.model.eflow.Composition;
import com.ibm.etools.eflow2.model.eflow.FCMBlock;
import com.ibm.etools.eflow2.model.eflow.FCMComposite;
import com.ibm.etools.eflow2.model.eflow.FCMConnection;
import com.ibm.etools.eflow2.model.eflow.Terminal;
import com.ibm.etools.eflow2.model.eflow.ViewPoint;
import com.ibm.etools.eflow2.model.eflow.emf.MOF;
import com.ibm.etools.eflow2.utils.model.type.CompositeType;
import com.ibm.etools.eflow2.utils.model.type.TypeElement;
import com.ibm.etools.eflow2.utils.model.type.TypeFactory;
import com.ibm.etools.eflow2.utils.model.type.WSDLMessageType;
import com.ibm.etools.eflow2.utils.model.utility.ConstantString;
import com.ibm.etools.eflow2.utils.model.utility.UtilityFactory;
import com.ibm.wbit.sib.eflow.EFlowConstants;
import com.ibm.wbit.sib.eflow.EFlowURIConverterImpl;
import com.ibm.wbit.sib.eflow.MedFlowResourceFactoryImpl;
import com.ibm.wbit.sib.mediation.operation.model.DocumentRoot;
import com.ibm.wbit.sib.mediation.operation.model.ExtendedOperationBinding;
import com.ibm.wbit.sib.mediation.operation.model.FlowBinding;
import com.ibm.wbit.sib.mediation.operation.model.InterfaceMediationFlow;
import com.ibm.wbit.sib.mediation.operation.model.MediationFlowControlFactory;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.InterfaceSet;
import com.ibm.wsspi.sca.scdl.Reference;
import com.ibm.wsspi.sca.scdl.ReferenceSet;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType;


public class MediationFlowGenerator {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2008, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	
	private ResourceSet resourceSet = null;
	private Resource flowResource = null;
	private EPackage flowNodePackage = null;
	Component component = null;
	private String medFlowName= null;
	private String operationName = null;
	Map<Fault, CompositeType> compositeTypesFaultsMap = new HashMap<Fault, CompositeType>();
	Map<Fault, String> inputFaultToTerminalIDMap = new HashMap<Fault, String>();
	Map<Fault, String> calloutFaultToTerminalIDMap = new HashMap<Fault, String>();
	
	
	private final String MEDIATION_FLOW_FILE_EXTENSION = ".medflow";
	private final String MEDIATION_FLOW_MESSAGE = "message";
	private final String MEDIATION_OUT_TERMINAL = "OutTerminal.out";
	private final String MEDIATION_IN_TERMINAL = "InTerminal.in";
	private final String MEDIATION_OUT_TERMINAL_NO_MATCH = "OutTerminal.noMatch";
	private final String MEDIATION_OUT_TERMINAL_FAILURE = "OutTerminal.Failure";
	private final String MEDIATION_IN_PREFIX_DINAMIC_FAULT="InTerminal.dynamic:in:";
	private final String MEDIATION_OUT_PREFIX_DINAMIC_FAULT="OutTerminal.dynamic:out:";
	private final String  MAIN_FLOW_NAME = "root";
	//as per observed delimter
	private final String MEDIATION_FLOW_DELIMITER = "^";

	Action action = null;
	Action callBehaviorAction = null;
	PortType portType = null;
	InterfaceMediationFlow interfaceMediationFlow = null;
	private int currentRootID = 1;
	
	
	public MediationFlowGenerator(Component component, Action action, PortType portType){
		this.component = component;
		if(action instanceof CallBehaviorAction){
			this.callBehaviorAction = action;
			this.action = findToBeProcessedAction(action);
		}else{
			this.action = action;
		}
		this.portType = portType;
	}
	

	/**
	 * Creates the ResponseFlow composite
	 * @param wsdlMessageInput
	 * @param wsdlMessageOutput
	 * @param faults
	 * @param operationName
	 * @param referenceName
	 * @return
	 */
	private FCMComposite createResponseFlow(Message wsdlMessageInput, Message wsdlMessageOutput, List<Fault> faults, String operationName, String referenceName) {
		
		// Create the flow type using dynamic MOF and add to the package
		FCMComposite flowNode = MOF.eflowFactory.createFCMComposite();
		flowNode.setName(getFlowNodeName());
		flowNode.getESuperTypes().add(MOF.eflowPackage.getFCMBlock());

		flowNodePackage.getEClassifiers().add(flowNode);

		Composition topLevelComposition = MOF.eflowFactory.createComposition();
		flowNode.setComposition(topLevelComposition);


		CompositeType compositeTypeInputWSDL = TypeFactory.eINSTANCE.createCompositeType();

		TypeElement typeElementRequest = TypeFactory.eINSTANCE.createTypeElement();
		typeElementRequest.setId(MEDIATION_FLOW_MESSAGE);
		WSDLMessageType wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
		wsdlType.setWsdlMessage(wsdlMessageInput.getQName().toString());
		typeElementRequest.setType(wsdlType);
		compositeTypeInputWSDL.getElements().add(typeElementRequest);

		CompositeType compositeTypeOutputWSDL = TypeFactory.eINSTANCE.createCompositeType();
		//defect 84991
		if(wsdlMessageOutput != null){
			TypeElement typeElementResponse = TypeFactory.eINSTANCE.createTypeElement();
			//typeElementResponse.setId( TerminalType.MESSAGE_TYPE );
			typeElementResponse.setId(MEDIATION_FLOW_MESSAGE);
			wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
			wsdlType.setWsdlMessage(wsdlMessageOutput.getQName().toString());
			typeElementResponse.setType(wsdlType);
			compositeTypeOutputWSDL.getElements().add(typeElementResponse); 
			topLevelComposition.getType().add(compositeTypeOutputWSDL);
		}
		topLevelComposition.getType().add(compositeTypeInputWSDL);
		//topLevelComposition.getType().add(compositeTypeOutputWSDL);

		List<CompositeType> compositeTypesFaults = new ArrayList<CompositeType>();
		if(faults != null && !faults.isEmpty()){
			for(Fault fault: faults){
				CompositeType compositeTypeFaultWSDL = TypeFactory.eINSTANCE.createCompositeType();
				TypeElement typeElementFault = TypeFactory.eINSTANCE.createTypeElement();
				//typeElementResponse.setId( TerminalType.MESSAGE_TYPE );
				typeElementFault.setId(MEDIATION_FLOW_MESSAGE);
				wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
				if(fault.getMessage() != null){
					wsdlType.setWsdlMessage(fault.getMessage().getQName().toString());
					typeElementFault.setType(wsdlType);
					compositeTypeFaultWSDL.getElements().add(typeElementFault);
					topLevelComposition.getType().add(compositeTypeFaultWSDL);
					compositeTypesFaults.add(compositeTypeFaultWSDL);
					compositeTypesFaultsMap.put(fault, compositeTypeFaultWSDL);
				}
			}
		}
		
		
		EPackage ePackageRequest = MOF.getEPackage(flowResource);
		
		FCMBlock inputResponseNode = generateInputResponse(compositeTypeOutputWSDL, wsdlMessageOutput);
				//defect 84991
		if(inputResponseNode != null)
		topLevelComposition.getNodes().add(inputResponseNode);
	
		FCMBlock calloutNodeResponse = generateCalloutResponse(compositeTypeOutputWSDL, compositeTypeInputWSDL, operationName, referenceName);
		topLevelComposition.getNodes().add(calloutNodeResponse);
		
		FCMConnection calloutResponseToInputResponse = MOF.eflowFactory.createFCMConnection();
		calloutResponseToInputResponse.setSourceNode(calloutNodeResponse);
		calloutResponseToInputResponse.setTargetNode(inputResponseNode);
		calloutResponseToInputResponse.setSourceTerminalName(MEDIATION_OUT_TERMINAL);
		calloutResponseToInputResponse.setTargetTerminalName(MEDIATION_IN_TERMINAL);

		topLevelComposition.getConnections().add(calloutResponseToInputResponse);

		if(faults != null && !faults.isEmpty()){
				FCMBlock inputFaultNode = generateInputFault(compositeTypesFaults, faults, operationName);
				topLevelComposition.getNodes().add(inputFaultNode);
			
				FCMBlock calloutFaultResponse = generateCalloutResponseFault(compositeTypesFaults, faults, operationName, referenceName);
				topLevelComposition.getNodes().add(calloutFaultResponse);
				
				for(Fault fault: faults){
					FCMConnection calloutFaultToInputFault = MOF.eflowFactory.createFCMConnection();
					calloutFaultToInputFault.setSourceNode(calloutFaultResponse);
					calloutFaultToInputFault.setTargetNode(inputFaultNode);
					calloutFaultToInputFault.setSourceTerminalName(calloutFaultToTerminalIDMap.get(fault));
					calloutFaultToInputFault.setTargetTerminalName(inputFaultToTerminalIDMap.get(fault));
					topLevelComposition.getConnections().add(calloutFaultToInputFault);
			}
		}

		
		return flowNode;
	}


	/**
	 * Creates the Callout for the FaultResponse
	 * @param compositeTypesFaults
	 * @param faults
	 * @param operationName
	 * @param referenceName
	 * @return
	 */
	private FCMBlock generateCalloutResponseFault(List<CompositeType> compositeTypesFaults, List<Fault> faults,	String operationName, String referenceName) {
		
		String nameSpaceURI = "mednode://mednodes/CalloutFault.mednode/CalloutFault";
		String nodeName = "CalloutFault";
		org.eclipse.emf.ecore.resource.Resource newResource = resourceSet.getResource(URI.createURI(nameSpaceURI), true);
		EPackage ePackageInput = MOF.getEPackage(newResource);
		FCMComposite nestedComposite = MOF.getFCMComposite(ePackageInput);

		Composition nestedComposition = MOF.eflowFactory.createComposition();
		nestedComposite.setComposition(nestedComposition);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock calloutNode = (FCMBlock) nestedFlowFactory.create(nestedComposite);
		addLocation(calloutNode);
	   	for(Fault fault: faults){
	   		if(fault.getMessage() != null){
				String label = fault.getMessage().getQName().getLocalPart();
				Terminal terminal = calloutNode.getOutTerminal(MEDIATION_OUT_TERMINAL);
				if(terminal == null){
					terminal = MOF.eflowFactory.createOutTerminal();
					calloutNode.getOutTerminals().add(terminal);
					terminal.setTerminalNodeID(MEDIATION_OUT_PREFIX_DINAMIC_FAULT +label);	
				}
				terminal.setType(compositeTypesFaultsMap.get(fault));
				terminal.setExplicitType(true);	
				terminal.setDynamic(true);
				terminal.setLabel(label);
				calloutFaultToTerminalIDMap.put(fault,terminal.getTerminalNodeID());
	   		}
	   	}

		String shortDescription = referenceName;
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		calloutNode.setTranslation(value);	
		
	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
    	String flowName = referenceName + "_" + nodeName;
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.REFERENCE_NAME_QUALIFIER, null, true);
    	}
    	
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER), referenceName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature("operationName") == null) {
    		addStringAttribute(calloutNode.eClass(), "operationName", null, true);
    	}
    	
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature("operationName"), operationName);

		return calloutNode;
	}


	/**
	 * Creates the InputFault, with one input terminal for each Fault.
	 * @param compositeTypesFaults
	 * @param faults
	 * @param operationName
	 * @return
	 */
	private FCMBlock generateInputFault(List<CompositeType> compositeTypesFaults, List<Fault> faults, String operationName) {
		
		String nameSpaceURI = "mednode://mednodes/InputFault.mednode/InputFault";
		String nodeType = "InputFault";
		org.eclipse.emf.ecore.resource.Resource newResource = resourceSet.getResource(URI.createURI(nameSpaceURI), true);
		EPackage ePackageInput = MOF.getEPackage(newResource);
		FCMComposite nestedComposite = MOF.getFCMComposite(ePackageInput);

		Composition nestedComposition = MOF.eflowFactory.createComposition();
		nestedComposite.setComposition(nestedComposition);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock nestedFlow = (FCMBlock) nestedFlowFactory.create(nestedComposite);
		addLocation(nestedFlow);
	   	if (nestedFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(nestedFlow.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
	   	String flowName = portType.getQName().getLocalPart() + "_" + nodeType;
 	   	nestedFlow.eSet(nestedFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);

	   	for(Fault fault: faults){
	   		if(fault.getMessage() != null){
		   		String label = fault.getMessage().getQName().getLocalPart();
				Terminal terminal = nestedFlow.getInTerminal(MEDIATION_IN_TERMINAL);
				if(terminal == null){
					terminal = MOF.eflowFactory.createInTerminal();
					nestedFlow.getInTerminals().add(terminal);
					terminal.setTerminalNodeID(MEDIATION_IN_PREFIX_DINAMIC_FAULT +label);	
				}
				terminal.setType(compositeTypesFaultsMap.get(fault));
				terminal.setExplicitType(true);	
				terminal.setDynamic(true);
				terminal.setLabel(label);
				inputFaultToTerminalIDMap.put(fault,terminal.getTerminalNodeID());
	   		}
	   	}

		String shortDescription = portType.getQName().getLocalPart();
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		nestedFlow.setTranslation(value);	

		return nestedFlow;
	
	}


	/**
	 * Generates the calloutResponse.
	 * @param compositeTypeOutputWSDL
	 * @param compositeTypeInputWSDL
	 * @param operationname
	 * @param referenceName
	 * @return
	 */
	private FCMBlock generateCalloutResponse(CompositeType compositeTypeOutputWSDL, CompositeType compositeTypeInputWSDL, String operationname, String referenceName) {
		String nameSpaceURI = "mednode://mednodes/CalloutResponse.mednode/CalloutResponse";
		String nodeName = "CalloutResponse";
		org.eclipse.emf.ecore.resource.Resource newResource = resourceSet.getResource(URI.createURI(nameSpaceURI), true);
		EPackage ePackageInput = MOF.getEPackage(newResource);
		FCMComposite nestedComposite = MOF.getFCMComposite(ePackageInput);

		Composition nestedComposition = MOF.eflowFactory.createComposition();
		nestedComposite.setComposition(nestedComposition);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock calloutNode = (FCMBlock) nestedFlowFactory.create(nestedComposite);
		addLocation(calloutNode);

		Terminal terminal = calloutNode.getOutTerminal(MEDIATION_OUT_TERMINAL);
		if(terminal == null){
			terminal = MOF.eflowFactory.createOutTerminal();
			calloutNode.getOutTerminals().add(terminal);
			terminal.setTerminalNodeID(MEDIATION_OUT_TERMINAL);	
		}

		terminal.setType(compositeTypeOutputWSDL);
		terminal.setExplicitType(true);	
		
		Terminal terminalFailure = calloutNode.getOutTerminal(MEDIATION_OUT_TERMINAL_FAILURE);
		if(terminalFailure == null){
			terminalFailure = MOF.eflowFactory.createOutTerminal();
			calloutNode.getOutTerminals().add(terminalFailure);
			terminalFailure.setTerminalNodeID(MEDIATION_OUT_TERMINAL_FAILURE);	
		}

		terminalFailure.setType(compositeTypeInputWSDL);
		terminalFailure.setExplicitType(true);	
		
		String shortDescription = operationname + ":" + referenceName;
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		calloutNode.setTranslation(value);	

	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
    	String flowName = referenceName + "_" + operationName +"_" + nodeName;
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.REFERENCE_NAME_QUALIFIER, null, true);
    	}
    	
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER), referenceName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature("operationName") == null) {
    		addStringAttribute(calloutNode.eClass(), "operationName", null, true);
    	}
    	
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature("operationName"), operationname);
	   	


		return calloutNode;
	}


	/**
	 * Generates the RequestFlow. Will contain the CompositeTypes for each WSDLMessage, (input/output and each Fault).
	 * Also, an inputNode, the EndpointLookupNode, responseNode, calloutNode and the inputFaultNode.
	 * @param wsdlMessageInput
	 * @param wsdlMessageOutput
	 * @param faults
	 * @param operationName
	 * @param referenceName
	 * @return
	 */
	private FCMComposite createRequestFlow(Message wsdlMessageInput, Message wsdlMessageOutput, List<Fault> faults, String operationName, String referenceName) {
		// Create the flow type using dynamic MOF and add to the package
		FCMComposite flowNode = MOF.eflowFactory.createFCMComposite();
		
		flowNode.setName(getFlowNodeName());
		flowNode.getESuperTypes().add(MOF.eflowPackage.getFCMBlock());
		
		flowNodePackage.getEClassifiers().add(flowNode);

		Composition topLevelComposition = MOF.eflowFactory.createComposition();
		flowNode.setComposition(topLevelComposition);


		CompositeType compositeTypeInputWSDL = TypeFactory.eINSTANCE.createCompositeType();
		
		TypeElement typeElementRequest = TypeFactory.eINSTANCE.createTypeElement();
		typeElementRequest.setId(MEDIATION_FLOW_MESSAGE);
		WSDLMessageType wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
		wsdlType.setWsdlMessage(wsdlMessageInput.getQName().toString());
		typeElementRequest.setType(wsdlType);
		compositeTypeInputWSDL.getElements().add(typeElementRequest);

		CompositeType compositeTypeOutputWSDL = TypeFactory.eINSTANCE.createCompositeType();
		//defect 84991
		if(wsdlMessageOutput != null){
			TypeElement typeElementResponse = TypeFactory.eINSTANCE.createTypeElement();
			typeElementResponse.setId(MEDIATION_FLOW_MESSAGE);
			wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
			wsdlType.setWsdlMessage(wsdlMessageOutput.getQName().toString());
			typeElementResponse.setType(wsdlType);
			compositeTypeOutputWSDL.getElements().add(typeElementResponse); 
			topLevelComposition.getType().add(compositeTypeOutputWSDL);
		}
		topLevelComposition.getType().add(compositeTypeInputWSDL);

		//for each of the Faults also create the CompositeType
		List<CompositeType> compositeTypesFaults = new ArrayList<CompositeType>();
		if(faults != null && !faults.isEmpty()){
			for(Fault fault: faults){
				if(fault.getMessage() != null){
					CompositeType compositeTypeFaultWSDL = TypeFactory.eINSTANCE.createCompositeType();
					TypeElement typeElementFault = TypeFactory.eINSTANCE.createTypeElement();
					typeElementFault.setId(MEDIATION_FLOW_MESSAGE);
					wsdlType = TypeFactory.eINSTANCE.createWSDLMessageType();
					wsdlType.setWsdlMessage(fault.getMessage().getQName().toString());
					typeElementFault.setType(wsdlType);
					compositeTypeFaultWSDL.getElements().add(typeElementFault);
					topLevelComposition.getType().add(compositeTypeFaultWSDL);
					compositeTypesFaults.add(compositeTypeFaultWSDL);
				}
			}
		}


		FCMBlock inputNode = generateInputNode(compositeTypeInputWSDL, wsdlMessageInput);
		topLevelComposition.getNodes().add(inputNode);

		FCMBlock inputResponseNode = generateInputResponse(compositeTypeOutputWSDL, wsdlMessageOutput);
		//defect 84991
		if(inputResponseNode != null)
		topLevelComposition.getNodes().add(inputResponseNode);


		FCMBlock endPointLookupNode = generateEndPointLookup(compositeTypeInputWSDL);
		topLevelComposition.getNodes().add(endPointLookupNode);


		FCMBlock calloutNode = generateCallout(compositeTypeInputWSDL, operationName, referenceName);
		topLevelComposition.getNodes().add(calloutNode);
		
		if(faults != null && !faults.isEmpty()){
			FCMBlock inputFaultNode = generateInputFault(compositeTypesFaults, faults, operationName);
			topLevelComposition.getNodes().add(inputFaultNode);
		}

		FCMConnection inputToEndpointLookup = MOF.eflowFactory.createFCMConnection();
		inputToEndpointLookup.setSourceNode(inputNode);
		inputToEndpointLookup.setTargetNode(endPointLookupNode);
		inputToEndpointLookup.setSourceTerminalName(MEDIATION_OUT_TERMINAL);
		inputToEndpointLookup.setTargetTerminalName(MEDIATION_IN_TERMINAL);


		FCMConnection endpointLookupToCallout = MOF.eflowFactory.createFCMConnection();
		endpointLookupToCallout.setSourceNode(endPointLookupNode);
		endpointLookupToCallout.setTargetNode(calloutNode);
		endpointLookupToCallout.setSourceTerminalName(MEDIATION_OUT_TERMINAL);
		endpointLookupToCallout.setTargetTerminalName(MEDIATION_IN_TERMINAL);

		FCMConnection endpointLookupToCallout2 = MOF.eflowFactory.createFCMConnection();
		endpointLookupToCallout2.setSourceNode(endPointLookupNode);
		endpointLookupToCallout2.setTargetNode(calloutNode);
		endpointLookupToCallout2.setSourceTerminalName(MEDIATION_OUT_TERMINAL_NO_MATCH);
		endpointLookupToCallout2.setTargetTerminalName(MEDIATION_IN_TERMINAL);

		FCMConnection endpointLookupToCallout3 = MOF.eflowFactory.createFCMConnection();
		endpointLookupToCallout3.setSourceNode(endPointLookupNode);
		endpointLookupToCallout3.setTargetNode(calloutNode);
		endpointLookupToCallout3.setSourceTerminalName(MEDIATION_OUT_TERMINAL_FAILURE);
		endpointLookupToCallout3.setTargetTerminalName(MEDIATION_IN_TERMINAL);

		
		topLevelComposition.getConnections().add(inputToEndpointLookup);
		topLevelComposition.getConnections().add(endpointLookupToCallout);
		topLevelComposition.getConnections().add(endpointLookupToCallout2);
		topLevelComposition.getConnections().add(endpointLookupToCallout3);
		
		return flowNode;
	}


	/**
	 * Generated the CalloutNode
	 * @param compositeTypeInputWSDL
	 * @param operationname
	 * @param referenceName
	 * @return
	 */
	private FCMBlock generateCallout(CompositeType compositeTypeInputWSDL, String operationname, String referenceName) {

		String nameSpaceURI = "mednode://mednodes/Callout.mednode/Callout";
		String nodeName = "Callout";
		org.eclipse.emf.ecore.resource.Resource newResource = resourceSet.getResource(URI.createURI(nameSpaceURI), true);
		EPackage ePackageInput = MOF.getEPackage(newResource);
		FCMComposite nestedComposite = MOF.getFCMComposite(ePackageInput);

		Composition nestedComposition = MOF.eflowFactory.createComposition();
		nestedComposite.setComposition(nestedComposition);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock calloutNode = (FCMBlock) nestedFlowFactory.create(nestedComposite);
		addLocation(calloutNode);

		Terminal terminal = calloutNode.getInTerminal(MEDIATION_IN_TERMINAL);
		if(terminal == null){
			terminal = MOF.eflowFactory.createInTerminal();
			calloutNode.getInTerminals().add(terminal);
			terminal.setTerminalNodeID(MEDIATION_IN_TERMINAL);	
		}

		terminal.setType(compositeTypeInputWSDL);
		terminal.setExplicitType(true);	
		String shortDescription = operationname + ":" + referenceName;
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		calloutNode.setTranslation(value);	

		String flowName = referenceName + "_" + operationName +"_" + nodeName;
	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER) == null) {
    		addStringAttribute(calloutNode.eClass(), EFlowConstants.REFERENCE_NAME_QUALIFIER, null, true);
    	}
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature(EFlowConstants.REFERENCE_NAME_QUALIFIER), referenceName);
	   	
	   	if (calloutNode.eClass().getEStructuralFeature("operationName") == null) {
    		addStringAttribute(calloutNode.eClass(), "operationName", null, true);
    	}
	   	calloutNode.eSet(calloutNode.eClass().getEStructuralFeature("operationName"), operationname);
		return calloutNode;
	}


	/**
	 * Generated the InputNode.
	 * @param compositeTypeInputWSDL
	 * @param wsdlMessageInput
	 * @return
	 */
	private FCMBlock generateInputNode( CompositeType compositeTypeInputWSDL, Message wsdlMessageInput){

		String nameSpaceURI = "mednode://mednodes/Input.mednode/Input";
		String nodeType = "Input";
		org.eclipse.emf.ecore.resource.Resource newResource = resourceSet.getResource(URI.createURI(nameSpaceURI), true);
		EPackage ePackageInput = MOF.getEPackage(newResource);
		FCMComposite nestedComposite = MOF.getFCMComposite(ePackageInput);

		Composition nestedComposition = MOF.eflowFactory.createComposition();
		nestedComposite.setComposition(nestedComposition);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock nestedFlow = (FCMBlock) nestedFlowFactory.create(nestedComposite);
		addLocation(nestedFlow);

		String flowName = getFlowName(nodeType);
		if (nestedFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(nestedFlow.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
	   	nestedFlow.eSet(nestedFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);

		Terminal terminal = nestedFlow.getOutTerminal(MEDIATION_OUT_TERMINAL);
		if(terminal == null){
			terminal = MOF.eflowFactory.createOutTerminal();
			nestedFlow.getOutTerminals().add(terminal);
			terminal.setTerminalNodeID(MEDIATION_OUT_TERMINAL);	
		}
		terminal.setType(compositeTypeInputWSDL);
		terminal.setExplicitType(true);	
		String shortDescription = wsdlMessageInput.getQName().getLocalPart();

		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		nestedFlow.setTranslation(value);	

		return nestedFlow;
	}

	/**
	 * Returns the flow name for the given nodeType
	 * @param nodeType
	 * @return
	 */
	private String getFlowName(String nodeType) {
		//return medFlowName+ "_" + operationName +"_" + nodeType;
		return portType.getQName().getLocalPart() + "_" + operationName +"_" + nodeType;
		
	}

	
	/**
	 * Add an empty location to the nestedFlow.
	 * @param nestedFlow
	 */
	private void addLocation(FCMBlock nestedFlow) {
		ViewPoint location = nestedFlow.getLocation();
		if(location == null){
			location = MOF.eflowFactory.createViewPoint();
			nestedFlow.setLocation(location);
		}
	}



	private FCMBlock generateInputResponse( CompositeType compositeTypeOutputWSDL,Message wsdlMessageOutput){
				//defect 84991
		if(wsdlMessageOutput == null) return null;

		String nameSpaceURIResponse = "mednode://mednodes/InputResponse.mednode/InputResponse";
		String nodeType = "InputResponse";
		org.eclipse.emf.ecore.resource.Resource newResourceResponse = resourceSet.getResource(URI.createURI(nameSpaceURIResponse), true);

		EPackage ePackageResponse = MOF.getEPackage(newResourceResponse);
		FCMComposite nestedCompositeResponse = MOF.getFCMComposite(ePackageResponse);

		Composition nestedCompositionResponse = MOF.eflowFactory.createComposition();
		nestedCompositeResponse.setComposition(nestedCompositionResponse);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactoryResponse = ePackageResponse.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock blockResponse = (FCMBlock) nestedFlowFactoryResponse.create(nestedCompositeResponse);
		addLocation(blockResponse);
	   	if (blockResponse.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(blockResponse.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
    	String flowName = getFlowName(nodeType);
	   	blockResponse.eSet(blockResponse.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), flowName);
		Terminal terminalResponse = blockResponse.getInTerminal(MEDIATION_IN_TERMINAL);
		if(terminalResponse == null){
			terminalResponse = MOF.eflowFactory.createInTerminal();
			blockResponse.getInTerminals().add(terminalResponse);
			terminalResponse.setTerminalNodeID(MEDIATION_IN_TERMINAL);
		}
		//terminal.setTerminalNodeID(MOF.getID(terminal.getTerminalNode()));
		terminalResponse.setType(compositeTypeOutputWSDL);
		terminalResponse.setExplicitType(true);	
		String shortDescription = wsdlMessageOutput.getQName().getLocalPart();
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		blockResponse.setTranslation(value);	

		return blockResponse;

	}

	/**
	 * Creates the EndPointLookup node.
	 * @param compositeTypeInputWSDL
	 * @return
	 */
	private FCMBlock generateEndPointLookup(CompositeType compositeTypeInputWSDL){
		
		String classifier = getClassifier();

		String nameSpaceURIEndpoint = "mednode://mednodes/EndpointLookup.mednode/EndpointLookup";

		org.eclipse.emf.ecore.resource.Resource resourceLookup = resourceSet.getResource(URI.createURI(nameSpaceURIEndpoint), true);
		EPackage ePackageLookup = MOF.getEPackage(resourceLookup);
		FCMComposite nestedCompositeLookup = MOF.getFCMComposite(ePackageLookup);

		Composition nestedCompositionLookup = MOF.eflowFactory.createComposition();
		nestedCompositeLookup.setComposition(nestedCompositionLookup);

		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactoryLookup = ePackageLookup.getEFactoryInstance();

		// Use the nested flow factory to create an instance of FCMBlock
		// The instance side is empty, the nested flow is a type!
		FCMBlock blockLookup = (FCMBlock) nestedFlowFactoryLookup.create(nestedCompositeLookup);
		addLocation(blockLookup);
	   	if (blockLookup.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
    		addStringAttribute(blockLookup.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
    	}
  
	   	blockLookup.eSet(blockLookup.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), "EndpointLookup");

	   	
	   	if (blockLookup.eClass().getEStructuralFeature("classifications.classification") == null) {
    		addStringAttribute(blockLookup.eClass(), "classifications.classification", null, true);
    	}
    	
	   	blockLookup.eSet(blockLookup.eClass().getEStructuralFeature("classifications.classification"), classifier);
	   	//defect 83261
	   	if (blockLookup.eClass().getEStructuralFeature("portTypeName") == null) {
    		addStringAttribute(blockLookup.eClass(), "portTypeName", null, true);
    	}
    	
	   	blockLookup.eSet(blockLookup.eClass().getEStructuralFeature("portTypeName"), portType.getQName().getLocalPart());
	   	
	   	if (blockLookup.eClass().getEStructuralFeature("portTypeNamespace") == null) {
    		addStringAttribute(blockLookup.eClass(), "portTypeNamespace", null, true);
    	}
    	
	   	blockLookup.eSet(blockLookup.eClass().getEStructuralFeature("portTypeNamespace"), portType.getQName().getNamespaceURI());

	
		Terminal terminalLookupIn = blockLookup.getInTerminal(MEDIATION_IN_TERMINAL);
		if(terminalLookupIn == null){
			terminalLookupIn = MOF.eflowFactory.createInTerminal();
			blockLookup.getInTerminals().add(terminalLookupIn);
			terminalLookupIn.setTerminalNodeID(MEDIATION_IN_TERMINAL);

		}
		terminalLookupIn.setType(compositeTypeInputWSDL);

		Terminal terminalLookupOut = blockLookup.getOutTerminal(MEDIATION_OUT_TERMINAL);
		if(terminalLookupOut == null){
			terminalLookupOut = MOF.eflowFactory.createOutTerminal();
			blockLookup.getOutTerminals().add(terminalLookupOut);
			terminalLookupOut.setTerminalNodeID(MEDIATION_OUT_TERMINAL);

		}
		terminalLookupOut.setType(compositeTypeInputWSDL);

		Terminal terminalLookupNoMatch = blockLookup.getOutTerminal(MEDIATION_OUT_TERMINAL_NO_MATCH);
		if(terminalLookupNoMatch == null){
			terminalLookupNoMatch = MOF.eflowFactory.createOutTerminal();
			blockLookup.getOutTerminals().add(terminalLookupNoMatch);
			terminalLookupNoMatch.setTerminalNodeID(MEDIATION_OUT_TERMINAL_NO_MATCH);

		}
		terminalLookupNoMatch.setType(compositeTypeInputWSDL);

		Terminal terminalLookupFailure = blockLookup.getOutTerminal(MEDIATION_OUT_TERMINAL_FAILURE);
		if(terminalLookupFailure == null){
			terminalLookupFailure = MOF.eflowFactory.createOutTerminal();
			blockLookup.getOutTerminals().add(terminalLookupFailure);
			terminalLookupFailure.setTerminalNodeID(MEDIATION_OUT_TERMINAL_FAILURE);

		}
		terminalLookupFailure.setType(compositeTypeInputWSDL);

		String shortDescription = "EndpointLookup";
		ConstantString value = UtilityFactory.eINSTANCE.createConstantString();
		value.setString(shortDescription);
		blockLookup.setTranslation(value);	


		return blockLookup;

	}
	/**
	 * Returns the classification string. Will contain classification values
	 * @return
	 */
	private String getClassifier() {
		StringBuffer classifierValue = new StringBuffer();
		String classifierString = null;
		List<OrganizationUnit> classifiers = null;
		//defect 84336
		//use the CBA classifiers if exist, else use the action classifier.
		if(callBehaviorAction != null){
			//defect 86136, check if this is not an override, we should get it from the action.
			boolean isOverriden = FabricIntegrationUtil.isOverriden((CallBehaviorAction)callBehaviorAction);
			if(isOverriden){
				classifiers = callBehaviorAction.getResponsibleOrganization();
			}else{
				classifiers = action.getResponsibleOrganization();
			}
		}else{
			classifiers = action.getResponsibleOrganization();
		}
		for(OrganizationUnit classifier: classifiers){
			List<Comment> comments= classifier.getOwnedComment();
			for(Comment comment: comments){
			//defect 90598
				if(comment.getBody() != null && comment.getBody().startsWith(PredefTypesConst.CATEGORY_CLASSIFICATION_COMMENT)){
					String thisClassification = comment.getBody().substring(PredefTypesConst.CATEGORY_CLASSIFICATION_COMMENT.length()); 
					classifierValue.append(thisClassification);
					classifierValue.append(MEDIATION_FLOW_DELIMITER);
				}
			}
		}
		if(classifierValue != null && classifierValue.length() > 0){
			classifierString = classifierValue.toString();
			if(classifierString.lastIndexOf(MEDIATION_FLOW_DELIMITER) != -1){
				classifierString = classifierString.substring(0, classifierString.lastIndexOf(MEDIATION_FLOW_DELIMITER));
			}
		}
		return classifierString;
	}


	/**
	 * Generates the content of the Mediation flow.
	 * @return
	 */
	public EObject generateMediationFlow(){
		
		if(interfaceMediationFlow == null){
			DocumentRoot root = createInterfaceMediationFlow();
			interfaceMediationFlow = root.getInterfaceMediationFlow();
		}

		medFlowName = ScdlUtil.getMediationFlowName(component);
		String fileName = medFlowName + MEDIATION_FLOW_FILE_EXTENSION;
		
			
		String projectName = ResourceMGR.getResourceManger().getProjectName(action);
		//defect 83998, if the Project name cannot be found we are probably generating for default attributes
		//so we can use any constant. The resourceSet created is temporary
		if(projectName == null){
			projectName = "Test";
		}
		resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(EFlowConstants.EFLOW_EXTENSION, new MedFlowResourceFactoryImpl());
		resourceSet.setURIConverter(new EFlowURIConverterImpl());

		flowResource = resourceSet.createResource(URI.createPlatformResourceURI(projectName).appendSegment(fileName));
		flowNodePackage = MOF.ecoreFactory.createEPackage();
		flowNodePackage.setNsPrefix(fileName);
		flowNodePackage.setNsURI(fileName);
		flowNodePackage.setName(MAIN_FLOW_NAME );
		flowResource.getContents().add(flowNodePackage);
		
		
		FCMComposite composite = MOF.eflowFactory.createFCMComposite();
		composite.setName(MAIN_FLOW_NAME);
		composite.getESuperTypes().add(MOF.eflowPackage.getFCMBlock());

		Composition topLevelComposition = MOF.eflowFactory.createComposition();
		composite.setComposition(topLevelComposition);
		

		EPackage ePackageInput = MOF.getEPackage(flowResource);
		// Get the factory associated with the nested flow package	
		EFactory nestedFlowFactory = ePackageInput.getEFactoryInstance();
		
		//for each operation on the Component interface create the MediationFlowRequest/Response. 
		//if the Operation is one way, should only create the MediationFlowRequest... 
		List<Operation> operations = portType.getOperations();
		for(int i=0; i<operations.size(); i++){
			Operation operation = (Operation)operations.get(i);
			ExtendedOperationBinding extendedOperationBinding = (ExtendedOperationBinding)interfaceMediationFlow.getExtendedOperationBinding().get(i);
			operationName = extendedOperationBinding.getSource();
			String referenceName = ((Reference)((ReferenceSet)interfaceMediationFlow.getReferences()).getReferences().get(0)).getName();
			Message messageInput = (Message)operation.getInput().getMessage();
			Message messageOutput = null;
			if(operation.getOutput() != null){
				messageOutput = (Message)operation.getOutput().getMessage();
			}
			//Message messageOutput = (Message)operation.getOutput().getMessage();
			List<Fault> faults = operation.getEFaults();
			

			FCMComposite flowNodeRequest = createRequestFlow(messageInput, messageOutput, faults, operationName, referenceName); 
			flowNodePackage.getEClassifiers().add(flowNodeRequest);
			//defect 84991
			FCMComposite flowNodeResponse = null;
			if(messageOutput != null){
				flowNodeResponse = createResponseFlow(messageInput, messageOutput, faults, operationName, referenceName);
				flowNodePackage.getEClassifiers().add(flowNodeResponse);
			}
			
			FCMBlock firstFlow = (FCMBlock) nestedFlowFactory.create(flowNodeRequest);
		   	if (firstFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
	    		addStringAttribute(firstFlow.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
	    	}
	    	String firstFlowName = portType.getQName().toString() + ":" + operation.getName() +"::" + "request";
		   	firstFlow.eSet(firstFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), firstFlowName);

			topLevelComposition.getNodes().add(firstFlow);
			//defect 84991
			if(flowNodeResponse != null){
				FCMBlock secondFlow = (FCMBlock) nestedFlowFactory.create(flowNodeResponse);
				
			   	if (secondFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME) == null) {
		    		addStringAttribute(secondFlow.eClass(), EFlowConstants.PROPERTY_NAME, null, true);
		    	}
			   	String secondFlowName = portType.getQName().toString() + ":" + operation.getName() +"::" +"response";	    	
			   	secondFlow.eSet(secondFlow.eClass().getEStructuralFeature(EFlowConstants.PROPERTY_NAME), secondFlowName);
	
				topLevelComposition.getNodes().add(secondFlow);
			}
		}
		flowNodePackage.getEClassifiers().add(composite);

		return flowNodePackage;
	}
	
	/**
	 * Return the InterfactMediationFlow.
	 * @return
	 */
	public DocumentRoot createInterfaceMediationFlow(){
		String name = component.getName();
		DocumentRoot documentRoot = MediationFlowControlFactory.eINSTANCE.createDocumentRoot();
		
		interfaceMediationFlow = MediationFlowControlFactory.eINSTANCE.createInterfaceMediationFlow();
		InterfaceSet is = component.getInterfaceSet();
		InterfaceSet newIS = ScdlUtil.copyInterfaceSet(is);
		
		interfaceMediationFlow.setInterfaces(newIS);
		interfaceMediationFlow.setName(name);
		TargetNamespaceProvider prov = new TargetNamespaceProvider();
		String targetNamespace = prov.getDefaultTargetNamespace(action, true);
		interfaceMediationFlow.setTargetNamespace(targetNamespace);
		
		ReferenceSet rs = SCDLFactory.eINSTANCE.createReferenceSet();
		 
		Reference reference = SCDLFactory.eINSTANCE.createReference();
		reference.setName(ScdlUtil.MEDIATION_FLOW_REFERENCE_NAME);
		WSDLPortType wsdlPT = com.ibm.wsspi.sca.scdl.wsdl.WSDLFactory.eINSTANCE.createWSDLPortType();
		Object qname = XMLTypeUtil.createQName(portType.getQName().getNamespaceURI(),portType.getQName().getLocalPart(),null);
		wsdlPT.setPortType(qname);			
		reference.getInterfaces().add(wsdlPT);

		rs.getReferences().add(reference);
		
		interfaceMediationFlow.setReferences(rs);

		List<Operation> operations = portType.getOperations();
		for(Operation op:operations){
			
			ExtendedOperationBinding extOp = MediationFlowControlFactory.eINSTANCE.createExtendedOperationBinding();
			extOp.setSource(op.getName());
			extOp.setTarget(op.getName());
			extOp.setSourcePortType(portType.getQName());
			extOp.setTargetReference(reference.getName());
			interfaceMediationFlow.getExtendedOperationBinding().add(extOp);
		}
			
		FlowBinding flowBinding = MediationFlowControlFactory.eINSTANCE.createFlowBinding();
		String medFlowName = ScdlUtil.getMediationFlowName(component);
		flowBinding.setMedflow(medFlowName + ExportOperation.MEDFLOW_FILE_EXT);
		interfaceMediationFlow.getOperationFlow().add(flowBinding);
		
		documentRoot.setInterfaceMediationFlow(interfaceMediationFlow);
		return documentRoot;
	}
	
	/**
	 * Creates and adds a String EAttribute to the given EClass.
	 *
	 * @return The created attribute
	 * @param eClass 
	 * @param name The name of the attribute, e.g. "dataSource"
	 * @param value The attribute default value (optional)
	 * @param required True if the attribute is required
	 */
	private EAttribute addStringAttribute(EClass eClass, String name, String value, boolean required) {
		
		EAttribute field = MOF.ecoreFactory.createEAttribute();

		field.setName(name);
		MOF.setID(flowResource, field, MOF.makeEAttributeID(name));
		
		
		field.setEType(MOF.ecorePackage.getEString());
		if (value != null)
		{
			field.setDefaultValue(value);
		}
		// Add attribute to flowNode
		eClass.getEStructuralFeatures().add(field);

		return field;
	}
	
	private String getFlowNodeName(){
		String flowNodeNmae =  MAIN_FLOW_NAME + "_I_" + currentRootID;
		currentRootID= currentRootID + 1;
		return flowNodeNmae;
	}
	
	/**
	 * Returns the Activity referecned by the CallBehaviorAction, or itself, if <code>action</code>
	 * is not a CallBehaviorAction.
	 * @param action - a BOM Action if Activity
	 * @return - namedElement
	 */
	private Action findToBeProcessedAction(Action action){
		Action san= null;
		if (action instanceof CallBehaviorAction){
			Behavior behavior = ((CallBehaviorAction)action).getBehavior();
		
			if (behavior instanceof Activity){
					san = ((Activity)behavior).getImplementation();
			}
			return san;
		}else{
			return action;
		}
	}	


	

}
