package com.ibm.btools.te.ilm.heuristics.scdl.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xml.type.internal.QName;
import org.eclipse.emf.ecore.xml.type.util.XMLTypeUtil;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDCompositor;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DeliverAsyncAt;
import com.ibm.wsspi.sca.scdl.DeliverAsyncAtAttribute;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Export;
import com.ibm.wsspi.sca.scdl.Implementation;
import com.ibm.wsspi.sca.scdl.InteractionStyle;
import com.ibm.wsspi.sca.scdl.Interface;
import com.ibm.wsspi.sca.scdl.InterfaceSet;
import com.ibm.wsspi.sca.scdl.JoinTransaction;
import com.ibm.wsspi.sca.scdl.Module;
import com.ibm.wsspi.sca.scdl.Part;
import com.ibm.wsspi.sca.scdl.Reference;
import com.ibm.wsspi.sca.scdl.SCDLFactory;
import com.ibm.wsspi.sca.scdl.Wire;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLFactory;
import com.ibm.wsspi.sca.scdl.wsdl.WSDLPortType;


/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 * Provides a set of helper methods for creating and updating SCDL artifacts
 */
public class ScdlUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Namespace prefix
	 */
	static public final String SCDL_NS_PREFIX="ns";
	
	public static final int INPUT_VARIABLE_TYPE = 1;
	public static final int OUTPUT_VARIABLE_TYPE = 2;
	public static final String MEDIATION_FLOW_REFERENCE_NAME= "ReferencePartner";
		
	/**
	 * Update the Source Component by adding a new wire to the Target Component
	 * @param srcComp - Source Component
	 * @param tarComp - Target Component
	 * @param pt - The Referenced PortType
	 */
	public static void updateSCDLComponentReference(Component srcComp, Component tarComp, PortType pt) {
		// Update the reference
		List refs = srcComp.getReferences();
		for (Iterator it=refs.iterator();it.hasNext();) {
			Reference ref = (Reference)it.next();
			for (int j=0; j<ref.getInterfaces().size();j++) {
				if (ref.getInterfaces().get(j) instanceof WSDLPortType) {
					WSDLPortType wpt = (WSDLPortType)ref.getInterfaces().get(j);
					QName qname = (QName)wpt.getPortType();
					// if a matching reference is found, add a wire to it
					if (qname.getNamespaceURI()==pt.getQName().getNamespaceURI() && qname.getLocalPart()==pt.getQName().getLocalPart()) {
						List wires = ref.getWires();
						boolean isWireExist=false;
						Wire wire;
						for (int i=0; i<wires.size();i++) {
							wire = (Wire)wires.get(i);
							if (wire.getTargetName().equals(tarComp.getName())) {
								isWireExist = true;
								break; 
							}								
						}
						if (!isWireExist) {
							wire = SCDLFactory.eINSTANCE.createWire();
							wire.setTargetName(tarComp.getName());
							ref.getWires().add(wire);								
						}
					}
				}
			}
			
		}
	}

	/**
	 * Get the Component Reference of the same PortType.  Assuming there is only one Reference with the same
	 * PortType can exist in the Component  
	 * @param srcComp
	 * @param pt
	 * @return
	 */
	public static Reference getReference(Component srcComp, PortType pt) {
		List refs = srcComp.getReferences();
		for (Iterator it=refs.iterator();it.hasNext();) {
			Reference ref = (Reference)it.next();
			for (int j=0; j<ref.getInterfaces().size();j++) {
				if (ref.getInterfaces().get(j) instanceof WSDLPortType) {
					WSDLPortType wpt = (WSDLPortType)ref.getInterfaces().get(j);
					QName qname = (QName)wpt.getPortType();
					// if a matching reference is found, add a wire to it
					if (qname.getNamespaceURI()==pt.getQName().getNamespaceURI() && qname.getLocalPart()==pt.getQName().getLocalPart()) {
						return ref;
					}
				}
			}
		}
		return null;
	}
	
//	public static void updateSCDLComponentReference(Component srcComp, Component tarComp, PortType pt) {
//		// Update the reference
//		List refs = srcComp.getReferences();
//		for (Iterator it=refs.iterator();it.hasNext();) {
//			Reference ref = (Reference)it.next();
//			for (int j=0; j<ref.getInterfaces().size();j++) {
//				if (ref.getInterfaces().get(j) instanceof WSDLPortType) {
//					WSDLPortType wpt = (WSDLPortType)ref.getInterfaces().get(j);
//					QName qname = (QName)wpt.getPortType();
//					// if a matching reference is found, add a wire to it
//					if (qname.getNamespaceURI()==pt.getQName().getNamespaceURI() && qname.getLocalPart()==pt.getQName().getLocalPart()) {
//						List wires = ref.getWires();
//						boolean isWireExist=false;
//						Wire wire;
//						for (int i=0; i<wires.size();i++) {
//							wire = (Wire)wires.get(i);
//							if (wire.getTargetName().equals(tarComp.getName())) {
//								isWireExist = true;
//								break; 
//							}								
//						}
//						if (!isWireExist) {
//							wire = SCDLFactory.eINSTANCE.createWire();
//							wire.setTargetName(tarComp.getName());
//							ref.getWires().add(wire);								
//						}
//					}
//				}
//			}
//			
//		}
//	}
	
	/**
	 * Add the operation to the Component Wire
	 * @param srcComp
	 * @param pt
	 * @param op
	 */
	public static void updateSCDLComponentReference(Component srcComp, PortType pt, Operation op) {
		// Update the reference
		List refs = srcComp.getReferences();
		for (Iterator it=refs.iterator();it.hasNext();) {
			Reference ref = (Reference)it.next();
			for (int j=0; j<ref.getInterfaces().size();j++) {
				if (ref.getInterfaces().get(j) instanceof WSDLPortType) {
					WSDLPortType wpt = (WSDLPortType)ref.getInterfaces().get(j);
					QName qname = (QName)wpt.getPortType();
					// if a matching reference is found, set the operaton
					if (qname.getNamespaceURI()==pt.getQName().getNamespaceURI() && qname.getLocalPart()==pt.getQName().getLocalPart()) {
						com.ibm.wsspi.sca.scdl.Operation scdlOp = SCDLFactory.eINSTANCE.createOperation();
						scdlOp.setName(op.getName());
						wpt.getOperations().add(scdlOp);
					}
				}
			}
			
		}
	}	

	/**
	 * Create a component reference
	 * @param source
	 * @param target
	 * @param refPT
	 * @param refName
	 * @param isMultiple
	 */
	public static Reference createSCDLComponentReference(DocumentRoot source, DocumentRoot target, PortType refPT, String refName, boolean isMultiple) {

		if(source.getComponent() != null && source.getComponent().getReferenceSet() != null){ 
			Reference ref = source.getComponent().getReferenceSet().getReference(refName);
			if(ref != null){
				return ref;
			}
		}
		Component srcComponent = source.getComponent();
		//AZ WMT 20070322 return reference to be captured in the partnerruleimpl
		Reference reference = null;
		if (srcComponent != null) {
			reference = SCDLFactory.eINSTANCE.createReference();
			reference.setName(refName);
			WSDLPortType wsdlPT = com.ibm.wsspi.sca.scdl.wsdl.WSDLFactory.eINSTANCE.createWSDLPortType();
			Object qname = XMLTypeUtil.createQName(refPT.getQName().getNamespaceURI(),refPT.getQName().getLocalPart(),null);
			wsdlPT.setPortType(qname);			
			reference.getInterfaces().add(wsdlPT);
			if (isMultiple) {
				reference.setMultiplicity(Reference.MULTIPLICITY_0_N);
			}
			// Update namespace prefix
//			if (!source.getXMLNSPrefixMap().containsValue(((QName)qname).getNamespaceURI())) {
//				String prefix = createNewSCDLNSPrefix(source.getXMLNSPrefixMap());
//				source.getXMLNSPrefixMap().put(prefix,((QName)qname).getNamespaceURI());
//			}		
			// Add the default reference qualifier
			setDefaultReferenceSetting(reference);
			
			if (srcComponent.getReferenceSet()==null) {
				srcComponent.setReferenceSet(SCDLFactory.eINSTANCE.createReferenceSet());
			}
			srcComponent.getReferenceSet().getReferences().add(reference);

			// Create Wire
			if (target != null) {
				Part cmp=null;
				if (target.getComponent()!=null) {
					cmp = target.getComponent();
				} else {
					cmp = target.getImport();
				}
				Wire wire = SCDLFactory.eINSTANCE.createWire();
//				if (refPath!=null) {
//					wire.setTargetName(refPath+cmp.getName());
//				} else {
					wire.setTargetName(cmp.getName());				
//				}
				reference.getWires().add(wire);				
			}
		}
		return reference;
	}

//	public static void createNSPrefixForWSDLPortType(DocumentRoot root) {
//		List interfaces=null;
//		if (root.getComponent()!=null) {
//			Component component = root.getComponent();
//			interfaces = component.getInterfaceSet().getInterfaces();
//		} else if (root.getImport()!=null) {
//			Import extImport = root.getImport();
//			interfaces = extImport.getInterfaceSet().getInterfaces();
//		} else if (root.getExport()!=null) {
//			Export extExport = root.getExport();
//			interfaces = extExport.getInterfaceSet().getInterfaces();			
//		}
//		if (interfaces != null) {
//			for (Iterator iterator = interfaces.iterator(); iterator.hasNext();) {
//				Object obj = iterator.next();
//				if (obj instanceof WSDLPortType) {
//					QName qname = (QName) ((WSDLPortType) obj).getPortType();
//					if (!root.getXMLNSPrefixMap().containsValue(qname.getNamespaceURI())) {
//						String prefix = ScdlUtil.createNewSCDLNSPrefix(root.getXMLNSPrefixMap());
//						root.getXMLNSPrefixMap().put(prefix,qname.getNamespaceURI());
//					}
//					
//				}
//			}			
//		}
//	}	
//	/**
//	 * Create a new SCDL NamespacePrefix
//	 * @param nsMap - namespace registry
//	 * @return
//	 */
//	public static String createNewSCDLNSPrefix(Map nsMap) {
//		int i = 1;
//		
//		String prefixName = SCDL_NS_PREFIX + i++;
//		while (nsMap.containsKey(prefixName))
//			prefixName = SCDL_NS_PREFIX + i++;
//		
//		return prefixName;
//	}		

	
	/**
	 * Create WSDL InterfaceSet for SCDL Component/Import
	 * @param pt
	 * @return
	 */
	public static InterfaceSet createInterfaceSet(PortType pt) {
		// Ray - d155869
//		WSDLPortType wsdlPT = createwInterface(pt);
//		InterfaceSet interfaceSet = SCDLFactory.eINSTANCE.createInterfaceSet();
//		interfaceSet.getInterfaces().add(wsdlPT);
//		return interfaceSet;
		return createInterfaceSet(pt,true);
	}

	/**
	 * Create WSDL InterfaceSet for SCDL Component/Import
	 * @param pt
	 * @param bDefaultSetting - whether to create default setting or not
	 * @return
	 */
	public static InterfaceSet createInterfaceSet(PortType pt, boolean bDefaultSetting) {
		//	 Ray - d155869
		WSDLPortType wsdlPT = createwInterface(pt,bDefaultSetting);
		InterfaceSet interfaceSet = SCDLFactory.eINSTANCE.createInterfaceSet();
		interfaceSet.getInterfaces().add(wsdlPT);
		return interfaceSet;
	}
	
	/**
	 * Update SCDL Interface with Operations from WSDL PortType for SCDL Component/Import
	 * @param interfaceSCA
	 * @param pt
	 * @return interface
	 */
	public static Interface updateInterfaceWithOperations(Interface interfaceSCA, PortType pt) {
	    if (pt.getOperations() == null) {
	        return interfaceSCA;
	    }
	    Iterator it = pt.getOperations().iterator();
	    while (it.hasNext()) {
	        com.ibm.wsspi.sca.scdl.Operation operation = SCDLFactory.eINSTANCE.createOperation();
	        operation.setName(((Operation)it.next()).getName());
	        interfaceSCA.getOperations().add(operation);
	    }
	    
	    return interfaceSCA;
	}
	

	
	/**
	 * Create wInterface
	 * @param pt
	 * @return
	 */
	public static WSDLPortType createwInterface(PortType pt) {
//		 Ray - d155869
//		WSDLPortType wsdlPT = WSDLFactory.eINSTANCE.createWSDLPortType();
//		Object qname = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),pt.getQName().getLocalPart(), null);
//		wsdlPT.setPortType(qname);
//
//		setDefaultInterfaceSetting(wsdlPT);		
//				
//		return wsdlPT;
		return createwInterface(pt,true);
	}	

	/**
	 * Create wInterface
	 * @param pt
	 * @return
	 */
	public static WSDLPortType createwInterface(PortType pt, boolean bDefaultSetting) {
		//	Ray - d155869
		WSDLPortType wsdlPT = WSDLFactory.eINSTANCE.createWSDLPortType();
		//we need to find out the prefix for the original WSDL
		String prefix = null;
		if(pt.getEnclosingDefinition() != null && pt.getEnclosingDefinition().getLocation() != null && ExportOperationUtil.isExternalFile(pt.getEnclosingDefinition().getLocation())){
			prefix =  pt.getEnclosingDefinition().getPrefix(pt.getQName().getNamespaceURI());
		}
//		Object qname = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),pt.getQName().getLocalPart(), null);
		Object qname = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),pt.getQName().getLocalPart(), prefix);
		wsdlPT.setPortType(qname);

		if (bDefaultSetting) {
			setDefaultInterfaceSetting(wsdlPT);					
		}
				
		return wsdlPT;
	}	
	
	/**
	 * Set default Interface setting
	 * @param wsdlPT
	 */
	public static void setDefaultInterfaceSetting(WSDLPortType wsdlPT) {
		// create default interactionStyle
		wsdlPT.setPreferredInteractionStyle(InteractionStyle.ASYNC_LITERAL);
		// create default txn qualifier
		JoinTransaction jt = SCDLFactory.eINSTANCE.createJoinTransaction();
		jt.setValue(Boolean.FALSE);
		wsdlPT.getInterfaceQualifiers().add(jt);		
	}
	

	/**
	 * Set default Reference setting
	 * @param ref
	 */
	public static void setDefaultReferenceSetting(Reference ref) {
		DeliverAsyncAt daa = SCDLFactory.eINSTANCE.createDeliverAsyncAt();
		daa.setValue(DeliverAsyncAtAttribute.COMMIT_LITERAL);
		ref.getReferenceQualifiers().add(daa);
	}
	
	/**
	 * Check if the DocumentRoot is a component
	 * @param dr
	 * @return
	 */
	public static boolean isComponent (DocumentRoot dr) {
		return (dr != null && dr.getComponent()!=null);
	}
	
	/**
	 * Check if the DocumentRoot is a component
	 * @param dr
	 * @return
	 */
	public static boolean isImport (DocumentRoot dr) {
		return (dr != null && dr.getImport()!=null);
	}

	/**
	 * Check if the DocumentRoot is an Export
	 * @param dr
	 * @return
	 */
	public static boolean isExport (DocumentRoot dr) {
		return (dr != null && dr.getExport()!=null);
	}
	
	/**
	 * Clone the component (EcoreUtil.copy does not work.  it throws a NullPointerException for some reason
	 * @param ocmp
	 * @return
	 */
	public static Component copyComponent(Component ocmp) {
		Component ncmp = SCDLFactory.eINSTANCE.createComponent();
		ncmp.setName(ocmp.getName());
		ncmp.setDisplayName(ocmp.getDisplayName());
		ncmp.setDescription(ocmp.getDescription());
		if (ocmp.getImplementation()!=null) {
			ncmp.setImplementation((Implementation)EcoreUtil.copy(ocmp.getImplementation()));				
		}
		List orefs  = ocmp.getReferences();
		for (Iterator it=orefs.iterator();it.hasNext();) {
			Reference oref = (Reference)it.next();
			Reference nref = SCDLFactory.eINSTANCE.createReference();
			nref.setName(oref.getName());
			nref.setDescription(oref.getDescription());
			nref.setMultiplicity(oref.getMultiplicity());
			setDefaultReferenceSetting(nref);
			for (Iterator it2=oref.getInterfaces().iterator();it2.hasNext();) {
				WSDLPortType opt = (WSDLPortType)it2.next();
				nref.getInterfaces().add(copyPortType(opt));
			}
			List owires = oref.getWires();
			for (Iterator it3=owires.iterator();it3.hasNext();) {
				Wire owire = (Wire)it3.next();
				Wire nwire = SCDLFactory.eINSTANCE.createWire();
				nwire.setTargetName(owire.getTargetName());
				nref.getWires().add(nwire);
			}
			if (ncmp.getReferenceSet()==null) {
				ncmp.setReferenceSet(SCDLFactory.eINSTANCE.createReferenceSet());
			}
			ncmp.getReferenceSet().getReferences().add(nref);
		}
		InterfaceSet ois = ocmp.getInterfaceSet();
		if (ois!=null) {
			ncmp.setInterfaceSet(copyInterfaceSet(ois));			
		}
		return ncmp;
	}
	
	private static WSDLPortType copyPortType(WSDLPortType opt) {
		WSDLPortType npt = com.ibm.wsspi.sca.scdl.wsdl.WSDLFactory.eINSTANCE.createWSDLPortType();
		QName qname = (QName)opt.getPortType();
		npt.setPortType(XMLTypeUtil.createQName(qname.getNamespaceURI(),qname.getLocalPart(),null));
		List oops = npt.getOperations();
		for (Iterator it=oops.iterator();it.hasNext();) {
			com.ibm.wsspi.sca.scdl.Operation oop = (com.ibm.wsspi.sca.scdl.Operation)it.next();
			com.ibm.wsspi.sca.scdl.Operation nop = SCDLFactory.eINSTANCE.createOperation();
			nop.setName(oop.getName());
			npt.getOperations().add(nop);
		}
		setDefaultInterfaceSetting(npt);
		return npt;
	}	
	
	public static InterfaceSet copyInterfaceSet(InterfaceSet ois) {
		InterfaceSet nis = SCDLFactory.eINSTANCE.createInterfaceSet();
		for (Iterator it2=ois.getInterfaces().iterator();it2.hasNext();) {
			WSDLPortType opt = (WSDLPortType)it2.next();
			nis.getInterfaces().add(copyPortType(opt));
		}
		return nis;
	}
	
	public static Export copyExport(Export oexp) {
		Export nexp = SCDLFactory.eINSTANCE.createExport();
		nexp.setName(oexp.getName());
		nexp.setDisplayName(oexp.getDisplayName());
		InterfaceSet ois = oexp.getInterfaceSet();
		if (ois!=null) {
			nexp.setInterfaceSet(copyInterfaceSet(ois));			
		}
		return nexp;
	}
	
	   public static List getTypeDeclarationElements(Operation op, int  type) {
		org.eclipse.wst.wsdl.Definition def=  op.getEnclosingDefinition();
		org.eclipse.wst.wsdl.Message message = getMessage(op, type);
    	if(message !=null)
    	{
   			List partsList = message.getEParts();
   			if(message.getEParts().size() > 0){
   				org.eclipse.wst.wsdl.Part part =  (org.eclipse.wst.wsdl.Part) message.getEParts().get(0); // there is only one part in a doc/lit wrapped scenario
   				return getUnwrappedElementDeclarations(part);
   			}	
    	}
    	return Collections.EMPTY_LIST;
    }

    /**
     * @param part to unwrap
     * @param context TransformationContext
     * @return List of elements that make up the part
     */
    public static List getUnwrappedElementDeclarations(org.eclipse.wst.wsdl.Part part)
    {
        List unwrappedElementsList = new ArrayList();
        XSDElementDeclaration wrapperElement = part.getElementDeclaration();
        if (wrapperElement == null)
            return Collections.EMPTY_LIST;
        XSDModelGroup modelGroup = getUnwrappedElementModelGroup(part);
        if (modelGroup != null)
        {
            XSDCompositor xsdCompositor = modelGroup.getCompositor();
            EList particleList = modelGroup.getContents();
            for (Iterator iterator = particleList.iterator(); iterator.hasNext();)
            {
                Object particleObj = iterator.next();
                if(particleObj instanceof XSDParticle)
                {
                    Object unwrappedElementDeclaration = ((XSDParticle)particleObj).getContent();
                    if(unwrappedElementDeclaration instanceof XSDElementDeclaration) {
                        unwrappedElementsList.add(unwrappedElementDeclaration);
                    }
                }
            }
        }
        return unwrappedElementsList;
    }
    
    /**
     * handy method to unwrap elements in DocLitWrap wsdl. Returns null if it is not a wrapper element.
     * @param part
     * @return xsdmodelgroup of complex element that part refers to
     */
    public static XSDModelGroup getUnwrappedElementModelGroup(org.eclipse.wst.wsdl.Part part)
    {
        XSDElementDeclaration wrapperElement = part.getElementDeclaration();
        XSDModelGroup modelGroup = null;
        if (wrapperElement != null)
        {
            // Start unwrapping the wrapper elements
            XSDTypeDefinition wrapperElementTypeDefinition = wrapperElement.getTypeDefinition();
            if(wrapperElementTypeDefinition == null || !(wrapperElementTypeDefinition instanceof XSDComplexTypeDefinition))  // i.e. the element was not probably resolved or  it did not conform to the format we were expecdting
            	return null;
            XSDParticle xsdParticle = (XSDParticle)((XSDComplexTypeDefinition)wrapperElementTypeDefinition).getContent();
            if(xsdParticle == null)
            	return null;
            XSDParticleContent xsdParticleContent = xsdParticle.getContent();
            if(xsdParticle == null)
            	return null;
            if (xsdParticleContent instanceof XSDModelGroup)
            {
                modelGroup = (XSDModelGroup) xsdParticleContent;
            }
        }
        return modelGroup;
    }
    
    /** Common method to get Input/Output/Fault messages for an operation
     * @param op Operation 
     * @param type can be WSDLUtils.INPUT_VARIABLE_TYPE / OUTPUT_VARIABLE_TYPE / FAULT_VARIABLE_TYPE
     * @return message for that type.  null if nothing exists
     */
    public static org.eclipse.wst.wsdl.Message getMessage(Operation op, int type) {
    	org.eclipse.wst.wsdl.Message message  = null;
    	if(type == INPUT_VARIABLE_TYPE)
    	{
    		org.eclipse.wst.wsdl.Input input = op.getEInput();
    		if(input != null)
    			 message=  input.getEMessage();
    	}
    	else if(type == OUTPUT_VARIABLE_TYPE)
    	{
    		org.eclipse.wst.wsdl.Output output = op.getEOutput();
    		if(output != null)
    			 message=  output.getEMessage();
    	}

    	return message;
    }

    
    public static String createJavaClassName(Component component){
    	
    	return getJavaPackageName(component)+  "." + getTargetSimpleTypeName(component);
    	
    }
    
	/**
	 * @return the package name for the {@link JavaImplementation}
	 * from the {@link Component} that was passed in the constructor.
	 */
	public static String getJavaPackageName(Component component) {
		
		String target = component.getName().substring(0, component.getName().lastIndexOf(ExportOperation.FILE_SEPARATOR));
		String namespace = target.replaceAll(ExportOperation.FILE_SEPARATOR, ".");
		
		return namespace;
	}	
	
	public static String getTargetSimpleTypeName(Component component) {
		String className = component.getName().substring(component.getName().lastIndexOf(ExportOperation.FILE_SEPARATOR)+1) + "Impl";
		return className;		
	}
	
	public static String getMediationFlowName(Component component) {
		String className = component.getName().substring(component.getName().lastIndexOf(ExportOperation.FILE_SEPARATOR)+1);
		return className;		
	}
	

	
	public static DocumentRoot createScdlModule(String name) {
		Module module = SCDLFactory.eINSTANCE.createModule();
		module.setName(name);
		DocumentRoot root = SCDLFactory.eINSTANCE.createDocumentRoot();
		root.setModule(module);		
		return root;
	}
    /**
     * Register the target component with a list of source component where the source component
     * has callback partner which is the calling process component, we need to create import 
     * component for the calling process export component
     * @param tarComp
     * @param srcComp
     */
	public static void registerCallbackComponent(DocumentRoot tarComp,DocumentRoot srcComp) {
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || tarComp == null || srcComp == null){
			return;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_REGISTRY_KEY);
			if(!(registry instanceof Map)){
				registry = new HashMap<DocumentRoot,List<DocumentRoot>>();
				additionalOptions.put(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_REGISTRY_KEY, registry);
			}
			List<DocumentRoot> srcComponentList = null;
			if(!((Map<DocumentRoot,List<DocumentRoot>>)registry).containsKey(tarComp)){
				srcComponentList = new ArrayList<DocumentRoot>();
				srcComponentList.add(srcComp);
				((Map<DocumentRoot,List<DocumentRoot>>)registry).put(tarComp, srcComponentList);
			}else{
				srcComponentList = ((Map<DocumentRoot,List<DocumentRoot>>)registry).get(tarComp);
				srcComponentList.add(srcComp);
			}		
		}
		Map<DocumentRoot,DocumentRoot> srcRegistry = (Map<DocumentRoot,DocumentRoot>)additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_SOURCE_REGISTRY_KEY);
		if(!(srcRegistry instanceof Map)){
			srcRegistry = new HashMap<DocumentRoot,DocumentRoot>();
			additionalOptions.put(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_SOURCE_REGISTRY_KEY, srcRegistry);			
		}
		srcRegistry.put(srcComp, tarComp);
	}
	
	/**
	 * If the callbak import is created, we skip 
	 * @param srcComponent
	 * @return
	 */
	public static boolean isCallbackImportCreated(Component srcComponent){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return false;
		}
		if(srcComponent != null && srcComponent.eContainer() instanceof DocumentRoot){
			DocumentRoot srcRoot = (DocumentRoot)srcComponent.eContainer();			
			Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
			if(additionalOptions != null){
				Map<DocumentRoot,DocumentRoot> srcRegistry = (Map<DocumentRoot,DocumentRoot>)additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_SOURCE_REGISTRY_KEY);
				if(srcRegistry instanceof Map){
					return srcRegistry.containsKey(srcRoot);
				}				
			}
			
		}
		return false;
	}
	/**
	 * Avoid create duplicate import and export component for the same call back interface
	 * @param tarComp
	 * @param srcComp
	 * @return
	 */
	public static boolean isCallbackHandled(Component tarComp,Component srcComp){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || tarComp == null || tarComp.eContainer() == null || srcComp == null || srcComp.eContainer() == null){
			return false;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_REGISTRY_KEY);
			if(registry instanceof Map){
				Map<DocumentRoot,List<DocumentRoot>> record = (Map<DocumentRoot,List<DocumentRoot>>)registry;
				if(record.containsKey(tarComp.eContainer())){
					List<DocumentRoot> srcList = record.get(tarComp.eContainer());
					if(srcList.contains(srcComp.eContainer())){
						return true;
					}
				}
			}
		}
		return false;
	}
/**
 * Find the call back dependency project to resolve the reference
 * @param projectName
 * @return
 */
	public static List<String> findCallbackDependency(Object component){
		String projectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(component);
		List<String> dependentProjects = new ArrayList<String>();
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || projectName == null){
			return dependentProjects;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_REGISTRY_KEY);
			DocumentRoot targetComponent = null;
			if(registry instanceof Map){
				Map<DocumentRoot,List<DocumentRoot>> register = (Map<DocumentRoot,List<DocumentRoot>>)registry;
				if(!register.containsKey(component)){
					return dependentProjects;
				}
				if(register.keySet()!= null && !register.keySet().isEmpty()){
					for(DocumentRoot eachKey:register.keySet()){
						String eachProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(eachKey);
						if(eachProjectName != null && eachProjectName.equals(projectName) && component == eachKey){
							targetComponent = eachKey;
							break;
						}
					}
				}
				if(targetComponent != null){
					List<DocumentRoot> compList = register.get(targetComponent);
					if(compList != null && !compList.isEmpty()){
						for(DocumentRoot eachComp:compList){
							String eachSrcProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(eachComp);
							if(eachSrcProjectName != null){
								if(!dependentProjects.contains(eachSrcProjectName)&& !(projectName.equals(eachSrcProjectName))){
									dependentProjects.add(eachSrcProjectName);	
								}
							}
						}
					}
				}				
			}			
		}
		return dependentProjects;
	}
	
	/**
	 * This method is called to resolve the calling process residing project library to add in the callee's module 
	 * project dependency
	 * @param srcProjectName
	 * @return
	 */
	public static List<String> resolveAdditionalModuleProjectDependency(String srcProjectName){
		List<String>   dependants = new ArrayList<String>();
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || srcProjectName == null){
			return dependants;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
		if(additionalOptions != null){
			Map<DocumentRoot,DocumentRoot> srcRegistry = (Map<DocumentRoot,DocumentRoot>)additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_REFERRENCE_SOURCE_REGISTRY_KEY);
			if(srcRegistry != null && srcRegistry.keySet() != null && srcRegistry.keySet().size() > 0){
				Set<DocumentRoot> keys = srcRegistry.keySet();
				for(DocumentRoot eachSrc:keys){
					String eachSrcProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(eachSrc);
					if(eachSrcProjectName != null && eachSrcProjectName.equals(srcProjectName)){
						DocumentRoot targetComponent  = srcRegistry.get(eachSrc);
						String eachTargetProjectName = (String)CrossProjectReferenceUtil.getProjectNameForTargetObject(targetComponent);						
						if(eachTargetProjectName != null && !eachTargetProjectName.equals(srcProjectName)){							
							String targetLib = BpelOptionsUtil.getWPSLibraryName(eachTargetProjectName);							
							if(!dependants.contains(targetLib)){
								dependants.add(targetLib);
							}							
						}
					}
				}
			}
		}
		return dependants;
	}
    /**
     * This registry is for removing the callback interfaces on the export component
     * @param tarComp
     * @param callbackPT
     */
	public static void regsiterImportName2QName(Component tarComp,PortType callbackPT) {
		if(tarComp != null && callbackPT != null){
			javax.xml.namespace.QName portTypeQName = callbackPT.getQName();
			ExportSession exportSession = BpelOptionsUtil.getExportSession();
			if(exportSession == null){
				return;
			}
			Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
			if(additionalOptions != null){
				Object registry = additionalOptions.get(TransformationSessionKeyConstants.QNAME_TO_IMPORT_REGISTRY_KEY);
				Map<Component,List<javax.xml.namespace.QName>> register = null;
				if(!(registry instanceof Map)){
					register = new HashMap<Component,List<javax.xml.namespace.QName>>();		
					additionalOptions.put(TransformationSessionKeyConstants.QNAME_TO_IMPORT_REGISTRY_KEY, register);
				}else{
					register = (Map<Component,List<javax.xml.namespace.QName>>)registry;
				}
				
				if(register != null){
					List<javax.xml.namespace.QName> qNames = register.get(tarComp);
					if(qNames == null){
						qNames = new ArrayList<javax.xml.namespace.QName>();
						register.put(tarComp, qNames);
					}
					qNames.add(portTypeQName);					
				}			
			}
		}		
	}
	
	public static  boolean consolidateTargetExportInterfaces(Component tarComp,Export exportComp){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return false;
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.QNAME_TO_IMPORT_REGISTRY_KEY);
			Map<Component,List<javax.xml.namespace.QName>> register = null;
			if(registry instanceof Map){
				register = (Map<Component,List<javax.xml.namespace.QName>>)registry;		
				List<javax.xml.namespace.QName> qNames = register.get(tarComp);
				List<WSDLPortType> removableInterfaces = new ArrayList<WSDLPortType>();
				if(qNames==null){
					return false;
				}
				for(javax.xml.namespace.QName eachQName:qNames){
					List<WSDLPortType> interfaces = exportComp.getInterfaceSet().getInterfaces();
					for(WSDLPortType eachPT:interfaces){
						javax.xml.namespace.QName qName = (javax.xml.namespace.QName)eachPT.getPortType();
						if(qName.getLocalPart()!= null && qName.getLocalPart().equals(eachQName.getLocalPart())&& 
								qName.getNamespaceURI()!= null && qName.getNamespaceURI().equals(eachQName.getNamespaceURI())){
							removableInterfaces.add(eachPT);
							break;
						}
					}
				}
				if(!removableInterfaces.isEmpty()){
					exportComp.getInterfaceSet().getInterfaces().removeAll(removableInterfaces);				
				}
			}
		}
		return false;
	}

	public static void updateSCDLComponentReference(Component srcComp,Component tarComp, PortType callbackPT, boolean isCallback) {
		if(isCallback){
			List refs = srcComp.getReferences();
			for (Iterator it=refs.iterator();it.hasNext();) {
				Reference ref = (Reference)it.next();
				for (int j=0; j<ref.getInterfaces().size();j++) {
					if (ref.getInterfaces().get(j) instanceof WSDLPortType) {
						WSDLPortType wpt = (WSDLPortType)ref.getInterfaces().get(j);
						QName qname = (QName)wpt.getPortType();
						// if a matching reference is found, add a wire to it
						if (qname.getNamespaceURI()==callbackPT.getQName().getNamespaceURI() && qname.getLocalPart()==callbackPT.getQName().getLocalPart()) {
							List wires = ref.getWires();
							boolean isWireExist=false;
							Wire wire;
							for (int i=0; i<wires.size();i++) {
								wire = (Wire)wires.get(i);
								if (wire.getTargetName().equals(tarComp.getName())) {
									isWireExist = true;
									break; 
								}								
							}
							if (!isWireExist) {
								wire = SCDLFactory.eINSTANCE.createWire();
								wire.setTargetName(tarComp.getName()+callbackPT.getQName().getLocalPart());
								ref.getWires().add(wire);								
							}
						}
					}
				}
			}
		}
	
		
	}
   /**
    * Record for import persistence purpose
    * @param importRoot
    */
	public static void registerCallbackImport(DocumentRoot importRoot) {
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null){
			return ;			
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_IMPORT_REGISTRY_KEY);
			List<DocumentRoot> importRootList = null;
			if(!(registry instanceof List)){
				importRootList = new ArrayList<DocumentRoot>();
				additionalOptions.put(TransformationSessionKeyConstants.CALLBACK_IMPORT_REGISTRY_KEY,importRootList);
			}else{
				importRootList = (List<DocumentRoot>)registry;
			}
			if(!importRootList.contains(importRoot)){
				importRootList.add(importRoot);
			}
		}		
	}	
	/**
	 * Check if the document root is callback import
	 * @param importRoot
	 * @return
	 */
	public static boolean isCallbackImport(Object importRoot){
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		if(exportSession == null || !(importRoot instanceof DocumentRoot)){
			return false;			
		}
		Map additionalOptions = exportSession.getExportOptions().getAdditionalOptions();		
		if(additionalOptions != null){
			Object registry = additionalOptions.get(TransformationSessionKeyConstants.CALLBACK_IMPORT_REGISTRY_KEY);
			List<DocumentRoot> importRootList = null;
			if(registry instanceof List){
				importRootList = (List<DocumentRoot>)registry;
				return importRootList.contains(importRoot);
			}			
		}		
		return false;
	}
	
	public static boolean isActivityInExportSession(Activity action){
		if(action != null){
			ExportSession exportSession = BpelOptionsUtil.getExportSession();
			if(exportSession == null || exportSession.getExportOptions() == null){
				return false;			
			}
			Map options = exportSession.getExportOptions().getAdditionalOptions();
			if(options != null){
				Object registry = options.get(TransformationSessionKeyConstants.MODEL_ELEMENTS_PER_SESSION);
				if(registry instanceof List){
					List<EObject> modelElementList = (List)registry;
					return modelElementList.contains(action);
				}
			}			
		}
		return false;
	}
}
