package com.ibm.btools.te.ilm.heuristics.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import com.ibm.btools.util.datatype.BTDataTypeException;
import com.ibm.btools.util.datatype.BTDuration;

/**
 * Duration class contains the information to represent a duration.
 */
public class Duration {	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private int years = 0;
	private int months = 0;
	private int weeks = 0;
	private int days = 0;
	private int hours = 0;
	private int minutes = 0;
	private int seconds = 0;
	private boolean isNegative = false;
	
	
	public Duration() {
	}
	
	public Duration(String duration) {
		
		if (duration == null) {
			return;
		}
		
		// "T" is not mandatory in duration strings
		String tString = null;
		if (duration.indexOf(TimeStringConverter.TIME_KEY) != -1) {
			tString = duration.substring(duration.indexOf(TimeStringConverter.TIME_KEY));
		}		
		String pString = null;
		if (tString == null) {
			pString = duration;
		} else {
			pString = duration.substring(0, duration.indexOf(TimeStringConverter.TIME_KEY));
		}
		if (pString.indexOf(TimeStringConverter.YEAR_KEY) != -1) {
			String yearString = pString.substring(1, pString.indexOf(TimeStringConverter.YEAR_KEY));
			int years = new Integer(yearString).intValue();
			setYears(years);
		}
		
		if (pString.indexOf(TimeStringConverter.MONTH_KEY) != -1) {
			String monthString = pString.substring(indexOfFirstNonNumericStringBefore(pString, pString.indexOf(TimeStringConverter.MONTH_KEY)) + 1, 
													pString.indexOf(TimeStringConverter.MONTH_KEY));
			int months = new Integer(monthString).intValue();
			setMonths(months);
		}
		
		if (pString.indexOf(TimeStringConverter.WEEK_KEY) != -1) {
			String weekString = pString.substring(indexOfFirstNonNumericStringBefore(pString, pString.indexOf(TimeStringConverter.WEEK_KEY)) + 1, 
												 pString.indexOf(TimeStringConverter.WEEK_KEY));
			int weeks = new Integer(weekString).intValue();
			setWeeks(weeks);
		}
		
		if (pString.indexOf(TimeStringConverter.DAY_KEY) != -1) {
			String dayString = pString.substring(indexOfFirstNonNumericStringBefore(pString, pString.indexOf(TimeStringConverter.DAY_KEY)) + 1, 
												 pString.indexOf(TimeStringConverter.DAY_KEY));
			int days = new Integer(dayString).intValue();
			setDays(days);
		}
		if (tString != null) {
			if (tString.indexOf(TimeStringConverter.HOUR_KEY) != -1) {
				String hourString = tString.substring(1, tString.indexOf(TimeStringConverter.HOUR_KEY));
				int hours = new Integer(hourString).intValue();
				setHours(hours);
			}

			if (tString.indexOf(TimeStringConverter.MINUTE_KEY) != -1) {
				String minuteString = tString.substring(indexOfFirstNonNumericStringBefore(tString, tString.indexOf(TimeStringConverter.MINUTE_KEY)) + 1, 
														tString.indexOf(TimeStringConverter.MINUTE_KEY));
				int minutes = new Integer(minuteString).intValue();
				setMinutes(minutes);
			}
		
			// Ray - d160822
			// Need to convert old duration format like P0Y0M0DT0H0M9.99999999E8S to be P0Y0M0DT0H0M999999999S
			// Otherwise import will die because the EMF support of xsd:duration cannot handle 9.99999999E8S
//			if (tString.indexOf(TimeStringConverter.SECOND_KEY) != -1) {
//				String secondString = tString.substring(indexOfFirstNonNumericStringBefore(tString, tString.indexOf(TimeStringConverter.SECOND_KEY)) + 1, 
//														tString.indexOf(TimeStringConverter.SECOND_KEY));
//				int seconds = new Double(secondString).intValue();
//				setSeconds(seconds);
//			}
			if (tString.indexOf(TimeStringConverter.SECOND_KEY) != -1) {
				String secondString = null;
				if (tString.indexOf("E") > -1) {
					if (tString.indexOf(TimeStringConverter.MINUTE_KEY) > -1) {
						secondString = tString.substring(tString.indexOf(TimeStringConverter.MINUTE_KEY) + 1, 
								tString.indexOf(TimeStringConverter.SECOND_KEY));
					}
					else {
						secondString = tString.substring(1, tString.indexOf(TimeStringConverter.SECOND_KEY));
					}
				}
				else {
					secondString = tString.substring(indexOfFirstNonNumericStringBefore(tString, tString.indexOf(TimeStringConverter.SECOND_KEY)) + 1, 
							tString.indexOf(TimeStringConverter.SECOND_KEY));
				}
				Double lvTotalSeconds = new Double(secondString); 
				int seconds = lvTotalSeconds.intValue();
				setSeconds(seconds);
//				int milliseconds = Math.round((float)((lvTotalSeconds.doubleValue() - seconds) *1000));
//				setMilliseconds(milliseconds);
			}					
		}
	}
	
	public Duration(int quantity, int timeUnit) {
		
		if (timeUnit == TimeStringConverter.YEAR_INDEX) {
			setYears(quantity);
		} else if (timeUnit == TimeStringConverter.MONTH_INDEX) {
			setMonths(quantity);
		} else if (timeUnit == TimeStringConverter.WEEK_INDEX) {
			setWeeks(quantity);
		} else if (timeUnit == TimeStringConverter.DAY_INDEX) {
			setDays(quantity);
		} else if (timeUnit == TimeStringConverter.HOUR_INDEX) {
			setHours(quantity);
		} else if (timeUnit == TimeStringConverter.MINUTE_INDEX) {
			setMinutes(quantity);
		} else if (timeUnit == TimeStringConverter.SECOND_INDEX) {
			setSeconds(quantity);
		}
	}
	
	
	/**
	 * See http://www.w3.org/TR/xmlschema-2/ for more details
	 * For example, to indicate a duration of 1 year, 2 months, 3 days, 10 hours, and 30 minutes, one would write: P1Y2M3DT10H30M. One could also indicate a duration of minus 120 days as: -P120D. 
     * Reduced precision and truncated representations of this format are allowed provided they conform to the following: 
     * If the number of years, months, days, hours, minutes, or seconds in any expression equals zero, the number and its corresponding designator may be omitted. However, at least one number and its designator must be present. 
	 * The seconds part may have a decimal fraction. 
	 * The designator 'T' shall be absent if all of the time items are absent. The designator 'P' must always be present. 
	 * For example, P1347Y, P1347M and P1Y2MT2H are all allowed; P0Y1347M and P0Y1347M0D are allowed. P-1347M is not allowed although -P1347M is allowed. P1Y2MT is not allowed. 
	 */
	public String toString() {		
		BTDuration btDuration = new BTDuration();
		try {
			btDuration.setYear(getYears());
			btDuration.setMonth(getMonths());
			btDuration.setDay(getDays());
			btDuration.setHour(getHours());
			btDuration.setMinute(getMinutes());
			btDuration.setSecond(getSeconds());
		} catch (BTDataTypeException e) {
			//ignore - should never happen
		}
		return btDuration.getValue().toString();		
	}

	private int indexOfFirstNonNumericStringBefore(String str, int index) {
		
		for (int i = index - 1; i >= 0; i--) {
			if (str.charAt(i) != '0' && str.charAt(i) != '1' && str.charAt(i) != '2' &&
				str.charAt(i) != '3' && str.charAt(i) != '4' && str.charAt(i) != '5' &&
				str.charAt(i) != '6' && str.charAt(i) != '7' && str.charAt(i) != '8' &&
				str.charAt(i) != '9' && str.charAt(i) != '.') {
				return i;
			}
		}
		return -1;
	}	
	
	/**
	 * @return
	 */
	public int getDays() {
		return days;
	}

	/**
	 * @return
	 */
	public int getHours() {
		return hours;
	}

	/**
	 * @return
	 */
	public int getMinutes() {
		return minutes;
	}

	/**
	 * @return
	 */
	public int getMonths() {
		return months;
	}

	/**
	 * @return
	 */
	public int getSeconds() {
		return seconds;
	}

	/**
	 * @return
	 */
	public int getYears() {
		return years;
	}

	/**
	 * @param i
	 */
	public void setDays(int i) {
		days = i;
	}

	/**
	 * @param i
	 */
	public void setHours(int i) {
		hours = i;
	}

	/**
	 * @param i
	 */
	public void setMinutes(int i) {
		minutes = i;
	}

	/**
	 * @param i
	 */
	public void setMonths(int i) {
		months = i;
	}

	/**
	 * @param i
	 */
	public void setSeconds(int i) {
		seconds = i;
	}

	/**
	 * @param i
	 */
	public void setYears(int i) {
		years = i;
	}

	/**
	 * @return
	 */
	public int getWeeks() {
		return weeks;
	}

	/**
	 * @param i
	 */
	public void setWeeks(int i) {
		weeks = i;
	}
	
	/**
	 * @return
	 */
	public boolean isNegative() {
		return isNegative;
	}

	/**
	 * @param b
	 */
	public void setNegative(boolean b) {
		isNegative = b;
	}
	
	/**
	 * Retrieves the zero duration	
	 * @return String
	 */
	public static String getZeroDuration() {
		BTDuration btDuration = new BTDuration();
		try {
			btDuration.setYear(0);
			btDuration.setMonth(0);
			btDuration.setDay(0);
			btDuration.setHour(0);
			btDuration.setMinute(0);
			btDuration.setSecond(0);
		} catch (BTDataTypeException e) {
			//ignore - should never happen
		}
		return btDuration.getValue().toString();	
	}
}