/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.WSDLFactory;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.ParameterRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM InputPinSet and/or OutputPinSet
 * TARGET: WSDL Operation
 * The Rule is repsonsible for transforming a BOM InputPinSet, OutputPinSet or pair Input/OutputPinSet to a WSDL Operation. 
 *
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 */
public class OperationRuleImpl extends AbstractProcIntRuleImpl implements OperationRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	private static String OPERATION_REQUEST_SUFFIX = "Request";
	private static String OPERATION_RESPONSE_SUFFIX = "Response";
	private static String OPERATION_FAULT_SUFFIX = "Fault";
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected OperationRuleImpl() {
		super();
	}

	/**
	* A WSDL operation, target of this rule. 
	*/
	Operation operation = null;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.OPERATION_RULE;
	}

	/**
	 * SOURCE: Pairs of BOM Input/OutputPinSet or Input/OutputParameterSet
	 * TARGET: WSDL Operation 
	 * Transforms the source a BOM Input and/or Output set a WSDL operation. It is responsible for 
	 * creating child Message and ParameterRules.  
	 */
	public boolean transformSource2Target(){
		/* The source of an OperationRule is a InputPinSet, an OutputPinSet or a pair Input/OutputPinSet
		 * Creates a  WSDL Operation and sets its name as follows.
		 * 	-  for only one OutputPinSet in the source, this is a callback operation, 
		 *  -  more than one Set as source, set the operation name prefixing with Send 
		 * Create childRules as follows:
		 * - For each inputSet and/or output set create a parameter (inputSet -> inputParameter
		 * outputSet -> outputParameter)
		 * The Rule will create as childs ParameterRules and MessageRules.
		 * Execute the child rules and attach the input/output to the operation. 
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		if(getSource().get(0) instanceof Datastore){
			//we need to create 2 operations, one for set Variable and the other one for getVariable
			Operation setOperation = WSDLFactory.eINSTANCE.createOperation();
			setOperation.setName("setVariable");
			Operation getOperation = WSDLFactory.eINSTANCE.createOperation();
			getOperation.setName("getVariable");
			
			MessageRule messageRule = (MessageRule)TransformationUtil.getRuleForSource(getParentRule(),(EObject)getSource().get(0), Message.class);
				// could not find the message, create a message rule for the input/outputSet source
				if(messageRule == null){
					messageRule = WsdlFactory.eINSTANCE.createMessageRule();
					messageRule.getSource().add(getSource().get(0));
					getChildRules().add(messageRule);
			}		
			messageRule.transformSource2Target();
				
			ParameterRule parameter = parameter = WsdlFactory.eINSTANCE.createParameterRule();
			parameter.getSource().add(getSource().get(0));
			getChildRules().add(parameter);
			//execute twice so we get both the input and the Output
			parameter.transformSource2Target();
			
			setOperation.setInput((Input)parameter.getTarget().get(0));
			getOperation.setInput((Input)parameter.getTarget().get(1));
			getOperation.setOutput((Output)parameter.getTarget().get(2));
			getTarget().add(setOperation);
			getTarget().add(getOperation);
			executeChildRules();
			setComplete(true);
			return isComplete();
			
		}
		
		
		String operationName = null;
		operation = WSDLFactory.eINSTANCE.createOperation();
		getTarget().add(operation);

		Object parentContext = getParentRule().getTarget().get(0);
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_OPERATION_NAME);
		valueProvider.setContext(this.getContext());
		valueProvider.setInParentContext(parentContext);
		operationName = (String)valueProvider.getAttributeValueForType(operation, (NamedElement)getSource().get(0), AttributeNameConstants.WSDL_OPERATION_NAME, NamingUtil.findRegistry(this));
		
		operation.setName(operationName);
		// setup child parameter rules. For each input/output set in the source, a ParameterRule and a MessageRule will be created
		setupChildRules();
		boolean ret = executeChildRules();
		processChildTargets();
		setComplete(ret);
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	/**
	 * Create child ParameterRules for each source element (BOM Input and/or OutputPinSet)
	 */
	private void createParameterRules(){
		// create a parameter for the each pin in the input/output Set
		ParameterRule parameter; 
		for (int i=0; i< getSource().size(); i++){
			parameter = WsdlFactory.eINSTANCE.createParameterRule();
			parameter.getSource().add(getSource().get(i));
			// Add another source to indicate this is for callback such that the parameter rule will create an input for an OutputPinSet
			// May need to think of a better way ....
			if ((getSource().get(0) instanceof OutputPinSet) && (getSource().size() == 1)) {
				parameter.getSource().add(((OutputPinSet)(getSource().get(0))).getAction());
			}
			getChildRules().add(parameter);
		}
	}
	/**
	 * Creates a MessageRule for each Input/OutputSet (Input/OutputParameterSet). If a WSDL Message was already 
	 * created this source, the message should be reused.
	 */	
	private void createMessageRules(){
		MessageRule rule = null;
		for (int i=0; i< getSource().size(); i++){
			//check first if the message was already created.
			rule = (MessageRule)TransformationUtil.getRuleForSource(getRoot(),(EObject)getSource().get(i), Message.class);
			// could not find the message, create a message rule for the input/outputSet source
			if(rule == null){
				rule = WsdlFactory.eINSTANCE.createMessageRule();
				rule.getSource().add(getSource().get(i));
				getChildRules().add(rule);
			}
		}
	}
	/**
	 * Creates child rules for that will create WSDL messages and parameters.
	 */
	private void setupChildRules(){
		// NOTE: to be called in this order, as a WSDL Parameter will reference a Message.
		createMessageRules();
		createParameterRules();	
	} 
	/**
	 * Process child targets. This rule will only process WSDL Input and/or Output. 
	 */
	protected void processChildTargets(){
		List childRules = getChildRules();
		EObject childTarget;
		List childTargets;
		Iterator childRulesIterator = childRules.iterator();
		while (childRulesIterator.hasNext()){
			TransformationRule childRule = (TransformationRule)childRulesIterator.next();
			childTargets = childRule.getTarget();
			for (int i=0; i<childTargets.size(); i++){
				childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof Output){
					operation.setOutput((Output)childTarget);
					((Output)childTarget).setName(operation.getName() + OPERATION_RESPONSE_SUFFIX);
				}else if (childTarget instanceof Input){
					operation.setInput((Input)childTarget);
					((Input)childTarget).setName(operation.getName() + OPERATION_REQUEST_SUFFIX);
					//TODO iteration 4 add faults
				} else if (childTarget instanceof Fault){
					operation.getEFaults().add(childTarget);
				}	
			}
		}
	}

} //OperationRuleImpl
