/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Import;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.WSDLFactory;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartnerLinkRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerLinkType;
import com.ibm.wbit.bpel.services.partnerlinktype.PartnerlinktypeFactory;
import com.ibm.wbit.bpel.services.partnerlinktype.Role;
import com.ibm.wbit.bpel.services.partnerlinktype.RolePortType;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM Action
 * TARGET: WSDL PartnerLink
 * The Rule is responsible for transforming a BOM Action to a WSDL PartnerLink.. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 */
public class PartnerLinkRuleImpl extends AbstractProcIntRuleImpl implements PartnerLinkRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	public static final String PARTNER_LINK_SUFFIX="PartnerLink";
	public static final String PARTNER_LINK_CALLBACK_SUFFIX = "CallbackPartnerLink";
	public static final String PARTNER_ROLE_SUFFIX="Service";
	public static final String PARTNER_ROLE_CALLBACK_SUFFIX="ServiceCallback";
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PartnerLinkRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.PARTNER_LINK_RULE;
	}

	/**
	 * SOURCE = [ Action], PortType 
	 * TARGET = PartnerLinkType
	 * Transforms a BOM Action to a WSDL PartnerLink.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if( isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if( getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		String serviceName;		
		NamedElement srcElement = (NamedElement)getSource().get(0);
	
		if (srcElement instanceof Action && BomUtils.isTopLevelProcess((Action)srcElement)){
				serviceName = ((Activity)srcElement.eContainer()).getName();
		} else {
			serviceName = BomUtils.getCanonicalName(srcElement);
		}
		//if this is a ServiceOperation, use the PortTypeName for the partnerLink.
			//defect 75458
		if (srcElement instanceof Action 
			&& (BomUtils.isServiceOperation((Action)srcElement))){
			if(getSource().size() == 3 && getSource().get(2) instanceof CallBehaviorAction){
//			serviceName = ((PortType)getSource().get(1)).getQName().getLocalPart();
			//We need to use the CBA's name 
			serviceName = BomUtils.getCanonicalName((CallBehaviorAction)getSource().get(2));
			getSource().remove(2);
			}
		} 
		String[] partnerLinkSuffix = new String[]{PARTNER_LINK_SUFFIX, PARTNER_LINK_CALLBACK_SUFFIX};
		String [] partnerRoleSuffix = new String[]{PARTNER_ROLE_SUFFIX, PARTNER_ROLE_CALLBACK_SUFFIX};
		
		for(int i=1; i<getSource().size(); i++){
			PartnerLinkType serviceLink = PartnerlinktypeFactory.eINSTANCE.createPartnerLinkType();
			// Check to see if it's a valid name by calling NameConverter
			JavaNCNameConverter converter = new JavaNCNameConverter();
			String convertedName = converter.convertName(NamingUtil.findRegistry(this), serviceLink, serviceName + partnerLinkSuffix[i-1]);
			serviceLink.setName(convertedName);
			PortType portType = (PortType)getSource().get(i);
			organizeImports(getWSDLDefinition(), portType);
			// Create Role - Assume single role and no callback role for now
			Role role = PartnerlinktypeFactory.eINSTANCE.createRole();		
			String convertedRoleName = converter.convertName(NamingUtil.findRegistry(this), role, serviceName + partnerRoleSuffix[i-1]);
			role.setName(convertedRoleName);
			RolePortType rpt = PartnerlinktypeFactory.eINSTANCE.createRolePortType();
			rpt.setName(portType);
			role.setPortType(rpt);
			serviceLink.getRole().add(role);
			getTarget().add(serviceLink);
		}
		setComplete(true);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();		
	}
	/**
	 * Adds the import for the <code>portType</code> namespace, if not the same as the
	 * WSDL <code>definition</code> namespace.
	 * @param definition WSDL Definition
	 * @param portType WSDL PortType
	 */	
	private void organizeImports(Definition definition, PortType  portType){
		Definition portTypeDefinition = portType.getEnclosingDefinition();
		String namespace = portType.getQName().getNamespaceURI();
		if(!namespace.equals(definition.getTargetNamespace())){
			if(!containsImport(definition, portTypeDefinition)){
				//we neet to import the schema 
				Import wsdlImport = WSDLFactory.eINSTANCE.createImport();
				//wsdlImport.setLocationURI(portType.getEnclosingDefinition().getTargetNamespace());
				if (ExportOperation.bUseWSDLSuffix) {
					String relativeFilePath = ProcessUtil.createFilePathFromNamespace(namespace);
					int i=relativeFilePath.lastIndexOf(ExportOperation.WSDL_SUFFIX);
					if (i!=-1) {
						relativeFilePath = relativeFilePath.substring(0,i-1);
					}
					String suffix = ExportOperation.WSDL_SUFFIX.substring(0, 1).toUpperCase() + ExportOperation.WSDL_SUFFIX.substring(1);
					// GJD 20060830 preserve the file name for original wsdl
					String fileName = null;
					if (portTypeDefinition.getLocation() != null && portTypeDefinition.getLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
					    fileName = portTypeDefinition.getLocation().substring(portTypeDefinition.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR.charAt(0)) + 1);
					} else { // code prior to wsdl original changes
					    fileName = portTypeDefinition.getQName().getLocalPart() + ExportOperation.WSDL_FILE_EXT;
					}
					wsdlImport.setLocationURI(fileName);
				} else {
					String fileName = null;
					if (portTypeDefinition.getLocation() != null && portTypeDefinition.getLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
					    //fileName = portTypeDefinition.getLocation().substring(portTypeDefinition.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR.charAt(0)) + 1);
						//AZ use the original definition location as the import location
						fileName = ExportOperationUtil.replaceConvertedSpaces(portTypeDefinition.getLocation());
					} else { // code prior to wsdl original changes
					    fileName = portTypeDefinition.getQName().getLocalPart() + ExportOperation.WSDL_FILE_EXT;
					}
					wsdlImport.setLocationURI(fileName);
				}
				wsdlImport.setNamespaceURI(namespace);
				definition.addImport(wsdlImport);
				String prefix = DataDefinitionUtil.getPrefixFromNamespaceURI(namespace);
				definition.addNamespace( prefix, namespace);
			}	
		}
	}
	
	/**
	 * Checks is the Process already contains a itel import for the <code>targetNamespace</code> 
	 * @param targetNamespace 
	 * @return
	 */
	private boolean containsImport(Definition definition, Definition toBeImportedDef){
		
		String targetNamespace = toBeImportedDef.getTargetNamespace();
		//GJD20060830 get the proper location for external definition
		String location = null;
		if (toBeImportedDef.getLocation() != null && toBeImportedDef.getLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
		    location = toBeImportedDef.getLocation().substring(toBeImportedDef.getLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR.charAt(0)) + 1);
		} else { // code prior to wsdl original changes
		    location = toBeImportedDef.getQName().getLocalPart()+ExportOperation.WSDL_FILE_EXT;
		}
		List imports = definition.getEImports();
		if(imports != null && !imports.isEmpty()){
			Iterator it = imports.iterator();
			while(it.hasNext()){
				Import thisImport = (Import)it.next();
				if(thisImport.getNamespaceURI().equals(targetNamespace) && thisImport.getLocationURI().equals(location)){
					return true;
				}
			}
		}
		return false;
	}
//	/**
//	 * SOURCE = [ Action OR Datastore ], PortType 
//	 * TARGET = PartnerLinkType
//	 * Transforms a BOM Action to a WSDL PartnerLink.
//	 * @return boolean
//	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
//	 */
//	public boolean transformSource2Target_bak(){
//		LoggingUtil.traceEntry(this, "transformSource2Target");
//		if( isComplete()){
//			LoggingUtil.traceExit(this, "transformSource2Target");
//			return true;
//		}
//		if( getSource() == null || getSource().isEmpty()){
//			LoggingUtil.traceExit(this, "transformSource2Target");
//			return false;
//		}
//		String serviceName;		
//		NamedElement srcElement = (NamedElement)getSource().get(0);
//	
//		if (srcElement instanceof Action 
//			&& (BomUtils.isTopLevelProcess((Action)srcElement))){
//				serviceName = ((Activity)srcElement.eContainer()).getName();
//		} else {
//			serviceName = BomUtils.getCanonicalName(srcElement);
//		}
//		PartnerLinkType serviceLink = PartnerlinktypeFactory.eINSTANCE.createPartnerLinkType();
//		// Check to see if it's a valid name by calling NameConverter
//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String convertedName = converter.convertName(NamingUtil.findRegistry(this), serviceLink, serviceName + PARTNER_LINK_SUFFIX);
//		serviceLink.setName(convertedName);
//		
//		//serviceLink.setName(serviceName + PARTNER_LINK_SUFFIX);
//		//Definition wsdlInterface = (Definition)ProcessUtil.getProcessInterface(getContext());
//		/*
//		PortTypeRule interfaceRule = (PortTypeRule) 
//			TransformationUtil.getRuleForSource(pocIntRule,getSource(),PortType.class);
//		// Assuming PortType is the first target of the PortType rule
//		PortType portType = (PortType)interfaceRule.getTarget().get(0);
//		*/
//		PortType portType = (PortType)getSource().get(1);
//		
//		organizeImports(getWSDLDefinition(), portType);
//		// Create Role - Assume single role and no callback role for now
//		Role role = PartnerlinktypeFactory.eINSTANCE.createRole();		
//		String convertedRoleName = converter.convertName(NamingUtil.findRegistry(this), role, serviceName + PARTNER_ROLE_SUFFIX);
//		role.setName(convertedRoleName);
//		RolePortType rpt = PartnerlinktypeFactory.eINSTANCE.createRolePortType();
//		rpt.setName(portType);
//		role.setPortType(rpt);
//		serviceLink.getRole().add(role);
//		
//		//if the PartnerLinkRule has 2 PortTypes as source, it is assumed it was called 
//		//from a SANReusableProcessRule and CallBehaviorActionRule and should also create
//		//a role for callback
//		if(getSource().size() >2 && getSource().get(2) instanceof PortType){
//			portType = (PortType)getSource().get(2);
//			organizeImports(getWSDLDefinition(), portType);
//			role = PartnerlinktypeFactory.eINSTANCE.createRole();		
//			convertedRoleName = converter.convertName(NamingUtil.findRegistry(this), role, serviceName + PARTNER_ROLE_CALLBACK_SUFFIX);
//			role.setName(convertedRoleName);
//			rpt = PartnerlinktypeFactory.eINSTANCE.createRolePortType();
//			rpt.setName(portType);
//			role.setPortType(rpt);
//			serviceLink.getRole().add(role);
//		}
//		//serviceLink.setEnclosingDefinition(wsdlInterface);
//		getTarget().add(serviceLink);
//		setComplete(true);
//		LoggingUtil.traceExit(this, "transformSource2Target");
//		return isComplete();		
//	}
	
	
} //PartnerLinkRuleImpl
