/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.wsdl.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.WSDLFactory;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Datastore;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.wsdl.OperationRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PortTypeRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlFactory;
import com.ibm.btools.te.ilm.heuristics.wsdl.WsdlPackage;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM Action or a BOM Activity
 * TARGET: WSDL PortType
 * 
 * The rule is responsible for transforming a BOM Action to a WSDL PortType. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 */
public class PortTypeRuleImpl extends AbstractProcIntRuleImpl implements PortTypeRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * The WSDL port type
	 */
	private PortType portType; 
	
	/**
	 * Boolean indicated if the class was executed more than once. Child rules should
	 * not be created in this case.
	 */
	private boolean executedOnce;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PortTypeRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return WsdlPackage.Literals.PORT_TYPE_RULE;
	}

	/**
	 * SOURCE: BOM Action or BOM Activity. In case of a BOM Activity, it should be an Activity without implementation.
	 * TARGET: WSDL PortType
	 * Transforms a BOM Action or a BOM Interface to a WSDL PortType. Will perform a simple algorithm to create 
	 * the child OperationRules that will lead to the creation of WSDL Operation. Adds the target of its child rules
	 * - the WSDL Operations to the WSDL PortType.
	 * @return boolean
	 */
	public boolean transformSource2Target(){
		/*
	 	* The source of this rule is a BOM Action or a BOM Activity without implementation. The target of the rule id a WSDL PortType
	 	* ASSUMPTION: There is only one PortType per Action. 
		* The method performs as follows:
		* If a target already exist for this rule (the rule already executed 
		* once, a PortType is not created and the method will only execute the 
		* possible unfinished (complete = false) rules.
		* Else it proceed as follows. 
		* Creates a PortType and set it as target of the rule.
		* - Set the name of the PortType 
		* Creates and execute the child rules:
		* - Creates Operation rules as per the algorithm ..  
		*/
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if (isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		//if the target is not found for this the rule, create a PortType, 
		// set the PortType name, QName and create the OperationRule (childRules)
		if ((getTarget() == null || getTarget().isEmpty()) && ! executedOnce ){
			if (getSource().get(0) instanceof Action) {
				Action action = (Action)getSource().get(0);
				OperationRule or;
				InputPinSet ips;
				OutputPinSet ops;
				// GJD 200807 - create schema with wrapper types for Local container with multiple pins in exception OutputPinSet
				if (action instanceof LoopNode || BomUtils.isLocalProcess(action)) {
					portType = createPortType(action,"");
					Iterator <OutputPinSet> it = action.getOutputPinSet().iterator();
					while (it.hasNext()) {
						ops = it.next();
						if (ops.getIsException() != null && ops.getIsException().booleanValue() &&
								ops.getOutputObjectPin().size() > 1) {
							//for pin set with multiple pins create wrapper variable
							or = WsdlFactory.eINSTANCE.createOperationRule();
							or.getSource().add(ops);
							getChildRules().add(or);
						}
					}
					boolean ret = executeChildRules();
					setComplete(ret);
				} else { // code prior to fault wrapper generation for local container
					BomUtils.PinSetAnalyzer psa = new BomUtils.PinSetAnalyzer(action, getContext());
				
					// Handle two way operations
					List twoWayOps = psa.getInputPinSetsAsTwoWayOperation();
					if (twoWayOps.size()>0) {
						portType = createPortType(action,"");
					}
					for (int i=0; i< twoWayOps.size(); i++){
						ips = (InputPinSet)twoWayOps.get(i);
						or = WsdlFactory.eINSTANCE.createOperationRule();
						or.getSource().add(ips);
						// TODO add only regular OS as a source?
						or.getSource().addAll(ips.getOutputPinSet());
						//or.getSource().add(ips.getOutputPinSet().get(0));
						getChildRules().add(or);
					}
					boolean ret = executeChildRules();
				
					// Handle one way operations
					List oneWayOps = psa.getInputPinSetsAsOneWayOperation();
					if (oneWayOps.size()>0 && portType==null) {
						portType = createPortType(action,"");
					}
					for (int i=0; i< oneWayOps.size(); i++){
						ips = (InputPinSet)oneWayOps.get(i);
						or = WsdlFactory.eINSTANCE.createOperationRule();
						or.getSource().add(ips);
						getChildRules().add(or);
					}
					ret = ret & executeChildRules();
				
					// Handle call back operations
					List callbackOps = psa.getOutputPinSetsAsCallbackOperation();
					if (callbackOps.size()>0) {
						portType = createPortType(action,"Callback");
						MappingUtil.registerCallBackArtifacts(getContext(),portType);
					}
					for (int i=0; i< callbackOps.size(); i++){
						ops = (OutputPinSet)callbackOps.get(i);
						or = WsdlFactory.eINSTANCE.createOperationRule();
						or.getSource().add(ops);
						getChildRules().add(or);
					}
					ret = ret & executeChildRules();
					setComplete(ret);
					//	AR added support for Datastore
				}					
			} else if(getSource().get(0) instanceof Datastore){
				Datastore repository = (Datastore)getSource().get(0);
				createPortType(repository, "");
				OperationRule or  = WsdlFactory.eINSTANCE.createOperationRule();
				or.getSource().add(repository);
				getChildRules().add(or);
				boolean ret = executeChildRules();
				setComplete(ret);
			}else{
	//					 the source is an activity ... This is for service model which is not supported.  So ignore this case for now
			}
		}
		executedOnce = true;
		//AZ WMT 20070322
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}

	/**
	 * Creates a WSDL PortType from the <code>ne</code> with the <code>suffix</code> A "Callback" suffix is an indication that a callback PortType is to be 
	 * created. 
	 * @param ne BOM NamedElement
	 * @param suffix a suffix indicating if this a a "callback" portType.
	 * @return a WSDL PortType
	 */
	private PortType createPortType(NamedElement ne, String suffix) {
		portType = WSDLFactory.eINSTANCE.createPortType();
		getTarget().add(portType);
		AttributeValueProvider valueProvider = AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.WSDL_PORT_TYPE_NAME);
		valueProvider.setContext(this.getContext());
		String portTypeName = null;
		if(suffix.equals("Callback")){
			portTypeName = (String)valueProvider.getAttributeValueForType(portType, (NamedElement)getSource().get(0), AttributeNameConstants.WSDL_CALLBACK_PORT_TYPE_NAME, NamingUtil.findRegistry(this));
		}else{
			portTypeName = (String)valueProvider.getAttributeValueForType(portType, (NamedElement)getSource().get(0), AttributeNameConstants.WSDL_PORT_TYPE_NAME, NamingUtil.findRegistry(this));	
		}
//		JavaNCNameConverter converter = new JavaNCNameConverter();
//		String portTypeName = converter.convertName(NamingUtil.findRegistry(this), portType, BomUtils.getCanonicalName((NamedElement)getSource().get(0)) + suffix);
		portType.setQName(new QName(getWSDLDefinition().getTargetNamespace(), portTypeName));
		//once a PortType is created, register it as the current PortType in the context
		ProcessUtil.putCurrentPortType(getContext(), portType);
		return portType;			
	}	
	

	/**
	 * The rule only process BPEL Operations from child's rules target.
	 * @param - rule the child transformation rule to process
	 */
	protected void processChildTargets(TransformationRule rule){
		EObject childTarget = null;
		List childTargets = rule.getTarget();
			for (int i=0; i<childTargets.size(); i++){
				childTarget = (EObject) childTargets.get(i);
				if (childTarget instanceof Operation){
					portType.getEOperations().add(childTarget);
				}
			}	
	}
	/**
	 * Executes each child rule and immediately process the child's rule targets.
	 * @return boolean
	 */
	protected boolean executeChildRules(){
		boolean allComplete = true;
		TransformationRule r;
		if (childRules != null) {
			for (int i = 0; i < childRules.size(); i++) {
				r = (TransformationRule) childRules.get(i);
				if (!r.isComplete() && !r.isFailed()) {
					boolean  ret = r.transformSource2Target();
					processChildTargets(r);
					if(!ret){
						allComplete = false;
					}
				}
			}
		}
		return allComplete;		
	}
	
} //PortTypeRuleImpl
