package com.ibm.btools.te.ilm.heuristics.wsdl.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xml.type.util.XMLTypeUtil;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Fault;
import org.eclipse.wst.wsdl.Import;
import org.eclipse.wst.wsdl.Input;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Operation;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.Part;
import org.eclipse.wst.wsdl.Port;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.wst.wsdl.Service;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.wst.wsdl.binding.soap.SOAPAddress;
import org.eclipse.wst.wsdl.internal.impl.DefinitionImpl;
import org.eclipse.wst.wsdl.internal.impl.ImportImpl;
import org.eclipse.wst.wsdl.internal.impl.MessageImpl;
import org.eclipse.wst.wsdl.internal.impl.OperationImpl;
import org.eclipse.wst.wsdl.internal.impl.PortTypeImpl;
import org.eclipse.wst.wsdl.util.WSDLResourceImpl;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDInclude;
import org.eclipse.xsd.XSDRedefine;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.impl.XSDImportImpl;
import org.eclipse.xsd.impl.XSDSchemaImpl;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Package;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.ObjectPin;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.ProcessInterfaceRule;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.wsdl.MessageRule;
import com.ibm.btools.te.ilm.heuristics.wsdl.PartRule;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;
import com.ibm.wbit.model.utils.wsdl.WSDLUtils;


/**
 * This utility class is responsuble for external service related wps export, where the original 
 * WSDL document will be used for look up the artificats instaead of transformation. However, the 
 * existing trnasformation scheme will still be simulated
 * 
 * ============= Original Wsdl registry should have the following mapping ==================
 * 
 * BOM:				WSDL:
 * ExernalService   Definition
 * ServiceInterface PortType
 * Activity         Operation 
 * InputPinSet		InputParam/Message
 * Input			Part
 * OutputPinSet		OutputParam/Message
 * Output			Part
 * 
 */
public class OriginalWsdlUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	public static String ORIG_WSDLS_REGISTRY = "ORIG_WSDLS_REGISTRY";
	public static String ORIG__PATH_WSDL_REGISTRY = "ORIG__PATH_WSDL_REGISTRY";
	public static final String SERVICE_OPERATION_ASPECT = "ServiceOperation";

	private static final String PROCESS_ASPECT = "PROCESS";
	//Key for original WSDL cache registry
	private static final String ORIGINAL_WSDL_SESSION_REGISTRY = "_ORIGINAL_WSDL_SESSION_REGISTRY";
	
	public static Definition getNamedWSDL(TransformationContext context,NamedElement ne) {
		if(ne == null || ne.getName()== null ||ne.getName().length()== 0 ){
			return null;
		}
		Definition originalDef = getOriginalWSDLDefinition(context,ne);
		if (originalDef != null) {
			OriginalWsdlUtil.registerWsdlElement(context,ne,originalDef);
		}
		return originalDef;
	}

	public static Map getWsdlElementRegistry (TransformationContext context) {
		Map registry = (Map) context.get(ORIG_WSDLS_REGISTRY);
		if (registry==null) {
			registry = new HashMap();		
			context.put(ORIG_WSDLS_REGISTRY,registry);
		}
		return registry;
	}	
	public static void cleanWSDLRegistry(TransformationContext context){
		Map registry = (Map) context.get(ORIG_WSDLS_REGISTRY);
		if(registry != null){
			registry.clear();
			context.remove(ORIG_WSDLS_REGISTRY);
		}
	}

	/**
	 * Register the BOM element and WSDL element mapping to the registry so that we can retrieve the WSDL element
	 * based on the BOM element later. 
	 * @param context
	 * @param ne
	 * @param wsdlElement
	 */
	public static void registerWsdlElement(TransformationContext context,NamedElement ne, Object wsdlElement) {
		Map registry = getWsdlElementRegistry(context);
		// In the case of Action, InputPinSet, and OutputPinSet, it can be 1 to n mapping.  Hence register
		// the bom element with a hashmap of WSDL elements.  The hashmap key will be the WSDL Element type (class)
		// and the value will be the wsdl element
		if (ne instanceof Class ||ne instanceof Action || ne instanceof InputPinSet || ne instanceof OutputPinSet) {
			Map reg = (Map)registry.get(ne);
			if (reg == null) {
				reg = new HashMap();
				registry.put(ne,reg);
			}
			reg.put(wsdlElement.getClass(),wsdlElement);
		} else {
			registry.put(ne,wsdlElement);			
		}
	}

	public static Definition getOriginalWsdl(TransformationContext context,NamedElement ne) {			
		Map registry = getWsdlElementRegistry(context);
		//lazy loading
		if(!registry.containsKey(ne)){
			getNamedWSDL(context,ne);
		}
		Map reg = (Map)registry.get(ne);
		if (reg == null) {
			return null;
		}
		return (Definition)reg.get(DefinitionImpl.class);
	}

	public static PortType getPortType(TransformationContext context, NamedElement ne) {
		Map registry = getWsdlElementRegistry(context);
		Object portType = registry.get(ne);
		if(portType instanceof PortType){
			return (PortType)portType;
		}
		Map reg = (Map)portType;
		if (reg == null) {
			return null;
		}
		return (PortType)reg.get(PortTypeImpl.class);
	}

	public static Message getMessage(TransformationContext context, NamedElement ne) {
		Map registry = getWsdlElementRegistry(context);
		Map reg = (Map)registry.get(ne);
		if (reg == null) {
			return null;
		}
		return (Message)reg.get(MessageImpl.class);
	}

	public static Operation getOperation(TransformationContext context, NamedElement ne) {
		Map registry = getWsdlElementRegistry(context);
		Map reg = (Map)registry.get(ne);
		if (reg == null) {
			return null;
		}
		return (Operation)reg.get(OperationImpl.class);
	}

	public static PortType getPortType(Definition wsdlDef, String tns, String localName) {
		PortType pt=null;
		List pts = wsdlDef.getEPortTypes();
		for (Iterator it=pts.iterator();it.hasNext();) {
			pt = (PortType)it.next();
			if (pt.getQName().getLocalPart().equals(localName) && pt.getQName().getNamespaceURI().equals(tns)) {
				return pt;
			}
		}
		return null;
	}

	public static Message getMessage(Definition wsdlDef, String tns, String localName, TransformationContext context) {		
		Message message=null;		
		if(wsdlDef == null || tns == null || localName == null){
			return message;
		}		
		List elements = wsdlDef.getEMessages();
		for (Iterator it=elements.iterator();it.hasNext();) {
			Message eachMessage = (Message)it.next();			
			if (localName.equals(eachMessage.getQName().getLocalPart()) && 					
					tns.equals(eachMessage.getQName().getNamespaceURI())) {
				message = eachMessage;
				break;
			}
		}	
		if(message == null){
			//look up import definition
			EList imprts = wsdlDef.getEImports();
			if(imprts != null && imprts.size() > 0){
				Iterator imprtIt = imprts.iterator();
				while(imprtIt.hasNext()){
					Import imprtDef = (Import)imprtIt.next();	
					if(imprtDef.getESchema() != null && imprtDef.getEDefinition() == null ){
						//we just need to deal with WSDL import, schema import will be skipped
						continue;
					}
					Definition resolvedDefinition = imprtDef.getEDefinition();
					if(resolvedDefinition == null){
						resolvedDefinition = loadOriginalWSDLResource(ExportOperationUtil.resolveAbsolutePathForImport(imprtDef), context);
						if(resolvedDefinition == null){
							return message;
						}else{
							updateObjectToFilePathRegistry(context, resolvedDefinition, ExportOperationUtil.resolveAbsolutePathForImport(imprtDef));
						}
					}
					EList imprtMsgs = resolvedDefinition.getEMessages();	
					//The original WSDL namespace cannot be used for comparison in this case
					//The original message within the operation has to be looked up for comparison
					for(Iterator it=imprtMsgs.iterator();it.hasNext();){
						Message imprtMessage = (Message)it.next();
						//					EList portTypes = wsdlDef.getEPortTypes();
						//					if(portTypes.size()>0){
						//						for(int i = 0; i < portTypes.size();i++){
						//							PortType eachPT = (PortType)portTypes.get(i);
						//							EList operations = eachPT.getEOperations();
						//							if(operations.size() > 0){
						//								for(int j = 0; j < operations.size();j++){
						//									Operation eachOP = (Operation)operations.get(j);
						//									Input input = eachOP.getEInput();
						//									Message inputMsg = input.getEMessage();
						//									if(localName.equals(inputMsg.getQName().getLocalPart()) && 											
						//											message.getQName().getNamespaceURI().equals(inputMsg.getQName().getNamespaceURI())){
						//										return message;
						//									}else{
						//										Output output = eachOP.getEOutput();
						//										Message outputMsg = output.getEMessage();
						//										if(localName.equals(outputMsg.getQName().getLocalPart())&&
						//												outputMsg.getQName().getNamespaceURI()!= null &&
						//												message.getQName().getNamespaceURI().equals(outputMsg.getQName().getNamespaceURI())){
						//											return message;
						//										}
						//									}
						//								}
						//								
						//							}
						//						}
						//					}
						if(localName.equals(imprtMessage.getQName().getLocalPart())&&
								tns.equals(imprtMessage.getQName().getNamespaceURI())){
							message = imprtMessage;
							break;
						}

					}
					if(message != null){
						break;
					}
				}
			}
		}
		return message;		
	}	

	public static Operation getOperation(PortType pt, String opName) {
		Operation op=null;
		List ops = pt.getEOperations();
		for (Iterator it=ops.iterator();it.hasNext();) {
			op = (Operation)it.next();
			if (op.getName().equals(opName)) {
				return op;
			}
		}
		return null;
	}

	public static Input getInput (Operation op) {
		if (op.getEInput()!=null) {
			return op.getEInput();
		}
		return null;
	}

	public static Output getOutput (Operation op) {
		if (op.getEOutput()!=null) {
			return op.getEOutput();
		}
		return null;
	}	

	//TODO find correct fault
	public static Fault getFault (Operation op) {
		if (op.getEFaults()!=null) {
			//TODO temp - return the first in the list			
			return (Fault)op.getEFaults().get(0);
		}
		return null;
	}	


	public static String getTargetFileName(TransformationContext context,NamedElement bomElement){
		if(context == null || bomElement == null){
			return null;
		}
        // Ray 06/06/07 fix for Share Across Project
        //String projectName = (String)context.get(TransformationSessionKeyConstants.PROJECT_NAME);		
        String projectName = ResourceMGR.getResourceManger().getProjectName(bomElement);		
				
		String projectPath=FileMGR.getProjectPath(projectName);	
		ResourceMGR resMgr = ResourceMGR.getResourceManger();

		String bomRelURI = resMgr.getRelativeURI(bomElement);    		
		String bomActivityAbsolutePath = null;		

		bomActivityAbsolutePath = projectPath + 
		File.separator + 
		bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator)); 
		//String wsdlFile = null;
		File targetDirectory = new File(bomActivityAbsolutePath);
		if(targetDirectory.isDirectory()){
			//return bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator))+
			return bomActivityAbsolutePath +
			File.separator +
			getWSDLFile(targetDirectory);
		}	
		return null;
	}

	private static Definition getOriginalWSDLDefinition(TransformationContext context,NamedElement bomElement){		
		Definition originalWSDL = null;	
		if(!(bomElement instanceof Class)){
			return originalWSDL;
		}	
        // Ray 06/06/07 fix for Share Across Project
        //String projectName = (String)context.get(TransformationSessionKeyConstants.PROJECT_NAME);		
        String projectName = ResourceMGR.getResourceManger().getProjectName(bomElement);		
		String projectPath=FileMGR.getProjectPath(projectName);	
		ResourceMGR resMgr = ResourceMGR.getResourceManger();

		String bomRelURI = resMgr.getRelativeURI(bomElement);    		
		String bomActivityAbsolutePath = null;		

		bomActivityAbsolutePath = projectPath + 
		File.separator + 
		bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator)); 
		String wsdlFile = null;
		File targetDirectory = new File(bomActivityAbsolutePath);
		if(targetDirectory.isDirectory()){
			wsdlFile = getWSDLFile(targetDirectory);
		}	
		if(wsdlFile == null){
			return originalWSDL;
		}
		String targetWSDLFile = bomActivityAbsolutePath + File.separator + wsdlFile;
		originalWSDL = loadOriginalWSDLResource(targetWSDLFile, context);	
		
		//Register the original source in the export session, so we can load the intact
		//WSDL for persistence
		ExportSession session = BpelOptionsUtil.getExportSession();		
		if(session != null && originalWSDL != null){
			final ResourceSet resourceSet = new ResourceSetImpl();
			Resource resource = null;	
			Definition sessionDef = null;
			File inputFile = new File(targetWSDLFile);		
			try {
				URI fileURI = URI.createFileURI(inputFile.getAbsolutePath());				
				resource = resourceSet.getResource(fileURI, true);				
			} catch (Throwable throwable) {
				String params[]={targetWSDLFile};
				LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_WSDL_DEFINITION, params, throwable);
				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_WSDL_DEFINITION));
				runtimeEx.setChainedException(throwable);			
				throw runtimeEx;				
			}
			if (!resourceSet.getResources().isEmpty()) {			
				for (Iterator it=resourceSet.getResources().iterator();it.hasNext();) {
					resource = (Resource) it.next();
					if(resource instanceof WSDLResourceImpl){								
						if (!resource.getContents().isEmpty()) {
							Object resObject = resource.getContents().get(0);
							sessionDef = (Definition)resObject;
						}
					}
				}
			}
			
			Map sessionContext = session.getContext();
			Map<NamedElement, Definition> interfaceDefCacheRegistry = null;
			if(sessionContext.get(ORIGINAL_WSDL_SESSION_REGISTRY) instanceof Map){
				interfaceDefCacheRegistry = (Map<NamedElement, Definition>)sessionContext.get(ORIGINAL_WSDL_SESSION_REGISTRY);
				interfaceDefCacheRegistry.put(bomElement, sessionDef);
			}else{
				interfaceDefCacheRegistry = new HashMap<NamedElement, Definition>();
				interfaceDefCacheRegistry.put(bomElement, sessionDef);
				 sessionContext.put(ORIGINAL_WSDL_SESSION_REGISTRY, interfaceDefCacheRegistry);
			}
		}		
		return 	originalWSDL;
	}
	
	/**
	 * This method is used to load the original WSDL cache during the business service transformation, which is not
	 * modified and implemented by a top level process as interface
	 * @param ne
	 * @return
	 */
	public static Definition getCachedOriginalWSDL(NamedElement ne){
		ExportSession session = BpelOptionsUtil.getExportSession();		
		if(session != null){
			Map sessionContext = session.getContext();
			if(sessionContext.get(ORIGINAL_WSDL_SESSION_REGISTRY) instanceof Map){
				Map<NamedElement, Definition>  interfaceDefCacheRegistry = (Map<NamedElement, Definition> )sessionContext.get(ORIGINAL_WSDL_SESSION_REGISTRY);
				if(ne instanceof ExternalService){
					ne = resolveFileAttachmentForExternalService((ExternalService)ne);
				}
				return interfaceDefCacheRegistry.get(ne);
			}
		}
		return null;
	}
	
	/**
	 * Resolve the file attachment class for the external service, since we register the external service as the business service's
	 * BOM source object instead of the service operation activity
	 * @param extService
	 * @return
	 */
	private static NamedElement resolveFileAttachmentForExternalService(ExternalService extService){
		EList  members = extService.getOwnedMember();
		if(members != null && members.size() != 0){
			Iterator mbrIterator = members.iterator();
			while(mbrIterator.hasNext()){
				NamedElement mbr = (NamedElement)mbrIterator.next();
				if("__FILE__ATTACHMENT".equals(mbr.getName()) && 
						"fileAttachment".equals(mbr.getAspect()) &&
						mbr instanceof ExternalSchema){
					EList fileAttachments = ((ExternalSchema)mbr).getOwnedMember();
					if(!fileAttachments.isEmpty()){
						return (NamedElement)fileAttachments.get(0);
					}
				}				
			}
		} 
		return null;
	}

	private static String getWSDLFile(File dir) {
		String[] children = dir.list();
		if (children == null) {
			// Either dir does not exist or is not a directory
		} else {
			for (int i=0; i<children.length; i++) {
				// Get filename of file or directory
				String filename = children[i];
				if(filename != null && filename.endsWith(".wsdl")){
					return filename;
				}
			}
		}
		return null;
	}

	/**
	 * This method is invoked outside of the transformation context
	 * @param targetFileName
	 * @return
	 */
	public static Definition loadOriginalWSDLResource(String  targetFileName){
		return loadOriginalWSDLResource(targetFileName,null);
	}

	/**
	 * This method is invoked inside the transformation context
	 * @param targetFileName
	 * @param context
	 * @return
	 */
	public static Definition loadOriginalWSDLResource(String targetFileName, TransformationContext context){
		if(targetFileName == null){
			return null;
		}
		if(context != null){
			Object cachedWSDL = getFilePathToWSDLRegistry(context).get(targetFileName);
			if(cachedWSDL instanceof Definition ){
				return (Definition)cachedWSDL;
			}
		}
		final ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource=null;		
		File inputFile = new File(targetFileName);		
		try {
			URI fileURI = URI.createFileURI(inputFile.getAbsolutePath());				
			resource = resourceSet.getResource(fileURI, true);				
		} catch (Throwable throwable) {
			String params[]={targetFileName};
			LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_WSDL_DEFINITION, params, throwable);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_WSDL_DEFINITION));
			runtimeEx.setChainedException(throwable);			
			throw runtimeEx;				
		}

		if (!resourceSet.getResources().isEmpty()) {			
			for (Iterator it=resourceSet.getResources().iterator();it.hasNext();) {
				resource = (Resource) it.next();
				if(resource instanceof WSDLResourceImpl){								
					if (!resource.getContents().isEmpty()) {
						Object resObject = resource.getContents().get(0);
						if(resObject instanceof Definition){
							if(context != null){
								getFilePathToWSDLRegistry(context).put(targetFileName, resObject);
							}
							return (Definition)resObject;
						}

					}
				}
			}
		}
		return null;
	}

	/**
	 * Get original WSDL Definition file name from the transformation context
	 * @param context
	 * @param originalDefintion
	 * @return original file name
	 */
	public static String getOriginalFileNameFromContext(TransformationContext context,Definition originalDefintion){
		String fileName = "";
		if(context == null || originalDefintion == null){
			return fileName;
		}
		ExportSession session = (ExportSession)context.get(TransformationSessionKeyConstants.EXPORT_SESSION);
		if(session != null){
			Object option = session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY);
			if( option instanceof HashMap){
				fileName = (String)((HashMap)option).get(originalDefintion);
				if(fileName != null){
					fileName = fileName.substring(fileName.lastIndexOf(File.separator)+1);
				}
				//AZ added to handle the file name look up for the original definition during default value generation
			}else{
				String defLocation = originalDefintion.getLocation();				
				if(defLocation != null && defLocation.indexOf("/") != -1){
					fileName = defLocation.substring(defLocation.lastIndexOf("/")+1);
					fileName = ExportOperationUtil.replaceConvertedSpaces(fileName);
				}else if(defLocation != null && defLocation.indexOf(File.separator)!= -1){
					fileName = defLocation.substring(defLocation.lastIndexOf(File.separator)+1);
				}
			}
		}
		return fileName;
	}	

	public static void checkUnresolvedImports(ResourceSet resourceSet) {
		//Iterate over resource set		
		//need to create list for iterating since a forced import will cause a 
		//concurrent modification error on the resource
		List contentObjects = new ArrayList();
		Iterator iter = resourceSet.getResources().iterator();
		while(iter.hasNext()) {
			Resource resource = (Resource) iter.next();	
			contentObjects.add(resource.getContents().get(0));
		}
		iter = contentObjects.iterator();
		while(iter.hasNext()) {
			Object ob = iter.next();
			if(ob instanceof XSDSchemaImpl){
				checkUnresolvedSchemaImports((XSDSchema) ob);
			}
			else if(ob instanceof Definition){
				checkUnresolvedWSDLImports((Definition) ob);			
			}
		}
	}

	/**
	 * Recursively checks for missing imports, includes and redefines.  They are
	 * considered missing if they are not resolved.
	 * @param schema The XSD shema model.
	 */
	private static void checkUnresolvedSchemaImports(XSDSchema schema) {
		List imports = new ArrayList();		
		if(schema != null) {			
			List contents = schema.getContents();
			XSDSchema resolvedSchema = null;
			for(int i = 0; i < contents.size(); i++) {
				Object element = contents.get(i);
				//support <import
				if(element instanceof XSDImport) {
					resolvedSchema = (XSDSchema) ((XSDImport) element).getResolvedSchema();
					if(resolvedSchema != null) {
						checkUnresolvedSchemaImports(resolvedSchema);				
					}
					else {
						//check for existing file that was not imported because nothing was referenced in it
						//in this case the file does exist and should be imported
						XSDSchema importedSchema = ((XSDImportImpl) element).importSchema();
						if(importedSchema != null) {
							imports.add(importedSchema);
							checkUnresolvedSchemaImports(importedSchema);
						}						
					}
				}
				//				support <include
				else if(element instanceof XSDInclude) {
					resolvedSchema = (XSDSchema) ((XSDInclude) element).getResolvedSchema();
					if(resolvedSchema != null) {
						checkUnresolvedSchemaImports(resolvedSchema);				
					}					
				}
				//support <redefine  note: needs more testing for redefine
				else if(element instanceof XSDRedefine) {
					resolvedSchema = (XSDSchema) ((XSDRedefine) element).getResolvedSchema();
					if(resolvedSchema != null) {
						checkUnresolvedSchemaImports(resolvedSchema);				
					}			
				}				
			}
		}
	}

	/**
	 * Recursively checks for missing imports.  They are considered missing if they are not
	 * resolved 
	 * @param definition The WSDL definition model.
	 */
	private static void checkUnresolvedWSDLImports(Definition definition) {
		List imports = new ArrayList();
		//HashMap unresolvedObjects = new HashMap();
		if(definition != null) {			
			List wsdlImports = definition.getEImports();
			Types types = definition.getETypes();
			//check for unresolved WSDL references
			if(wsdlImports != null && !wsdlImports.isEmpty()) {
				for(int i = 0; i < wsdlImports.size(); i++) {
					Object element = wsdlImports.get(i);
					if(element instanceof Import) {
						Definition resolvedWSDLDefn = (Definition) ((Import) element).getDefinition();
						XSDSchema resolvedSchema = (XSDSchema) ((Import) element).getSchema();
						if(resolvedWSDLDefn != null) {
							checkUnresolvedWSDLImports(resolvedWSDLDefn);
						}
						else if(resolvedSchema != null) {
							checkUnresolvedSchemaImports(resolvedSchema);
						}
						else {
							//force the import
							((ImportImpl) element).importDefinitionOrSchema();
							resolvedWSDLDefn = (Definition) ((Import) element).getDefinition();
							resolvedSchema = (XSDSchema) ((Import) element).getSchema();
							if(resolvedWSDLDefn != null) {
								imports.add(resolvedWSDLDefn);
								checkUnresolvedWSDLImports(resolvedWSDLDefn);
							}
							else if(resolvedSchema != null) {
								imports.add(resolvedSchema);
								checkUnresolvedSchemaImports(resolvedSchema);
							}							
						}
					}
				}
			}
			//check for unresolved schema references
			if(types != null) {
				XSDSchemaExtensibilityElement extensibilityElement = (XSDSchemaExtensibilityElement) types.getExtensibilityElements().get(0);
				if(extensibilityElement != null) {
					XSDSchema schema = (XSDSchema) extensibilityElement.getSchema();
					checkUnresolvedSchemaImports(schema);
				}
			}
		}
	}

	/**
	 * This method applies to the resolution of the parameter name for the BPEL process
	 * For a variable that references to an element other than a complex type, the variable 
	 * name of a CBA invoke activity should be resolved to the part name instead of the element name
	 * For a reusable process or task, if the object pin references to a element, the variable name 
	 * should be resolved to the type name instead of the pin name
	 * @param context
	 * @param pin
	 * @return
	 */
	public static String retrievePartNameFromWSDL(TransformationContext context,ObjectPin pin){
		String targetPartName = null;
		if(context == null || pin == null){
			throw new RuntimeException("context or object pin cannot be null.");
		}
		if(!(pin.eContainer() instanceof CallBehaviorAction)){
			//WID 602 likes the name of the type instead of the pin's name
			PinSet ps = BomUtils.findPinSetForPin(pin);
			if(ps instanceof OutputPinSet && BomUtils.isProcess(((OutputPinSet)ps).getAction())&& ((OutputPinSet)ps).getIsException().booleanValue()){				
				ProcessInterfaceRule  processIntRule = (ProcessInterfaceRule)ProcessUtil.getProcessInterfaceRule(context, ps);			
				TransformationRule partRule = TransformationUtil.getRuleForSource(processIntRule, PartRule.class, ps );
				if(partRule != null){
					if(partRule.getTarget() != null && !partRule.getTarget().isEmpty()){
						Part wsdlPart = (Part)partRule.getTarget().get(0);	
						if(wsdlPart.getElementDeclaration() != null && wsdlPart.getElementDeclaration().getTypeDefinition() != null && wsdlPart.getElementDeclaration().getTypeDefinition().getName() == null)
						return wsdlPart.getName();						
					}
				}
	    	}
			return pin.getType().getName();			
		}
		if(pin.eContainer() instanceof CallBehaviorAction){
			if(BomUtils.isReusableService((Action)pin.eContainer()) || 
					BomUtils.isReusableTask((Action)pin.eContainer())||
					BomUtils.isReusableBusinessRuleTask((Action)pin.eContainer())||
					BomUtils.isReusableHumanTask((Action)pin.eContainer())||
					isProcessCBA((Action)pin.eContainer())){
				return pin.getType().getName();	
			}		
			Definition originalWSDL = null;		
			Activity behaviour  = null;
			CallBehaviorAction cba = (CallBehaviorAction)pin.eContainer();
			if(cba.getBehavior()instanceof Activity){
				behaviour = (Activity)cba.getBehavior();				
				if(behaviour != null){
					originalWSDL = OriginalWsdlUtil.getOriginalWsdl(context,BomUtils.resolveFileAttachment(behaviour.getImplementation()));
				}
			}		
			if(originalWSDL != null){				
				String qualifiedPartName = pin.getName();
				PinSet pinSet = BomUtils.findPinSetForPin(pin);
				String messagename = pinSet.getName();
				String partname = qualifiedPartName;				
				Message targetMsg = OriginalWsdlUtil.getMessage(originalWSDL,OriginalWsdlUtil.getTargetNamespaceForContainingService(originalWSDL,pin),messagename, context);
				if(targetMsg != null){
					if(targetMsg.getPart(partname) != null){				
						targetPartName = targetMsg.getPart(partname).getName();								
					}else{
						//The message name or part name modified, we need to take away the message name and try
						if(partname.startsWith(messagename)){
							String realPartName = partname.substring(messagename.length()+1);
							if(targetMsg.getPart(realPartName) != null){
								targetPartName =  targetMsg.getPart(realPartName).getName();
							}
						}						
					}
					//Go through the part, in case it refers to a elementdeclaration doc-lit wrapped style
					if(targetPartName == null){
						boolean isDocLitWrappedStype = false;
						PortType portType = OriginalWsdlUtil.getPortType(context,BomUtils.resolveOwningInterface(behaviour.getImplementation()));						
						if(portType != null){
							Operation operation = OriginalWsdlUtil.getOperation(portType,behaviour.getImplementation().getName());
							if(operation != null){
								WSDLUtils.Compliance isDocLitWrappedCompliance = WSDLUtils.isDocLitWrapped(operation);
								if(isDocLitWrappedCompliance == WSDLUtils.YES){
									isDocLitWrappedStype = true;								
								}	
							}
						}	
						if(isDocLitWrappedStype){
							Part singlePart = (Part)targetMsg.getEParts().get(0);
							List wrappedDeclarations = WSDLUtils.getUnwrappedElementDeclarations(singlePart);
							if(wrappedDeclarations != null && !wrappedDeclarations.isEmpty()){
								//for each declaration we should create a Pin set the name as per the Element name
								for(int i=0; i<wrappedDeclarations.size(); i++){
									XSDElementDeclaration declaration = (XSDElementDeclaration)wrappedDeclarations.get(i);
									if(declaration.isElementDeclarationReference()){
										declaration = declaration.getResolvedElementDeclaration();
									}
									if(declaration.getName() != null && declaration.getName().equals(partname)){
										targetPartName = partname;
										break;
									}
								}
							}else{
								//This is a fault message
								XSDElementDeclaration declaration = singlePart.getElementDeclaration();
								if(declaration.isElementDeclarationReference()){
									declaration = declaration.getResolvedElementDeclaration();
								}
								if(declaration.getName() != null && declaration.getName().equals(partname)){
									targetPartName = partname;											
								}else{
									//handle name appended with "_n", This is a lenient check, last resort
									if(partname.startsWith(declaration.getName())){
										targetPartName = declaration.getName();
									}
								}
							}			
									
						}else{
							//non-doc lit wrapper, handle name appended with "_n". This is a lenient check, last resort
							if(partname.startsWith(messagename)){											
								String realPartName = partname.substring(messagename.length()+1);
								EList eParts = targetMsg.getEParts();
								if(eParts != null && !eParts.isEmpty()){
									for(int k=0;k<eParts.size();k++){
										Part eachPart = (Part)eParts.get(k);
										String eachPartName = eachPart.getName();
										if(eachPartName != null && realPartName.startsWith(eachPartName)){
											targetPartName = eachPartName;
											break;
										}
									}
								}
							}

						}					
					}					
				}				
			}	
		}
		if(targetPartName == null){
			throw new RuntimeException("Cannot find appropriate name for the pin");
		}
		return targetPartName;
	}
	
	private static boolean isProcessCBA(Action action){
		if(action instanceof CallBehaviorAction){
			if(PROCESS_ASPECT.equals(action.getAspect())){
				return true;
			}
		}		
		return false;
	}

	/**
	 * if the pin refers to an inline schema type, the targetnamespace 
	 * should be it's owning external service's target namespace
	 * if it refers to a regular external schema type, then we use the original
	 * WSDL targetnamespace, which means the message is defined within the original
	 * WSDL file, not in the imported WSDL
	 * @param pin
	 * @return
	 */
	public static String getTargetNamespaceForContainingService(Definition originalWSDL,ObjectPin pin){
		if(originalWSDL == null && pin == null ||pin.getType() == null){
			return null;
		}
		if(pin.getType().getOwningPackage() instanceof ExternalSchema){
			ExternalSchema schema = (ExternalSchema)pin.getType().getOwningPackage();		
			if(schema.getOwningPackage() instanceof ExternalService){
				ExternalService extService = (ExternalService)schema.getOwningPackage();
				return extService.getTargetNamespace();
			}else{
				//This is not inline schema
				return originalWSDL.getTargetNamespace();
			}
		}else if(pin.getType() instanceof PrimitiveType){
			return originalWSDL.getTargetNamespace();
		}
		return null;
	}

	private static Map getFilePathToWSDLRegistry(TransformationContext context){
		Map registry = (Map) context.get(ORIG__PATH_WSDL_REGISTRY);
		if (registry==null) {
			registry = new HashMap();		
			context.put(ORIG__PATH_WSDL_REGISTRY,registry);
		}
		return registry;
	}

	/**
	 * The source file path should be unique in the WBM workspace, we only add the entry when the WSDL is first loaded
	 * to ensure we have one only instance across export session
	 * @param context
	 * @param originalWSDL
	 * @param sourceWSDLFilePath
	 */
	public static void updateObjectToFilePathRegistry(TransformationContext context,Definition originalWSDL,String sourceWSDLFilePath){
		ExportSession session = (ExportSession)context.get(TransformationSessionKeyConstants.EXPORT_SESSION);
		Object object2FileReg = session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY);
		if(object2FileReg instanceof HashMap){
			if(!((HashMap)object2FileReg).values().contains(sourceWSDLFilePath)){
				((HashMap)object2FileReg).put(originalWSDL,sourceWSDLFilePath);
			}
		}
	}
	/**
	 * This function is to test if the original wsdl includes binding information
	 * if there is binding information, so we update the import with binding information
	 * without creating component, otherwise we create component for the external service	 
	 * @param action
	 * @param pt
	 * @return boolean true if there is a binding information inside
	 */
	
	public static boolean isBindingInfoAvailable(Action action, PortType pt,TransformationContext context){
		if (action != null && action.getAspect() != null && action.getAspect().equals(SERVICE_OPERATION_ASPECT)) {
			Definition orgiginalDef = OriginalWsdlUtil.getOriginalWsdl(context,BomUtils.resolveFileAttachment((StructuredActivityNode)action));
			if(orgiginalDef != null){
				//AZ 2007 Assume the first service
				if(orgiginalDef.getEServices() == null || orgiginalDef.getEServices().isEmpty()){
					return false;
				}					
				Service svc = (Service)orgiginalDef.getEServices().get(0);
				Object svcQName=null;
				Object portQName=null;
				String endpoint=null;
				if(svc.getQName() != null){
					svcQName = XMLTypeUtil.createQName(svc.getQName().getNamespaceURI(),svc.getQName().getLocalPart(), null);
				}
				Port targetPort = null;
				if(svc != null && svc.getEPorts()!= null && !svc.getEPorts().isEmpty()){
					Iterator prtIt = svc.getEPorts().iterator();
					while(prtIt.hasNext()){
						Port each = (Port)prtIt.next();
						if(each.getEBinding()!= null && each.getEBinding().getPortType()!= null){
							if( each.getEBinding().getPortType().getQName() != null && pt.getQName() != null){
								javax.xml.namespace.QName eachQName = each.getEBinding().getPortType().getQName();
								if(eachQName.getLocalPart().equals(pt.getQName().getLocalPart())&&
										eachQName.getNamespaceURI().equals(pt.getQName().getNamespaceURI())){
									targetPort = each;
									break;
								}
							}
						}
					}
				}
				if(targetPort != null){
					if(pt.getQName() != null){
						portQName = XMLTypeUtil.createQName(pt.getQName().getNamespaceURI(),targetPort.getName(), null);
					}							
					//AZ 2007 According to the specification: 
					// 1) Binding extensibility elements (1) are used to specify the address information for the port
					// 2) A port MUST NOT specify more than one address.
					// 3) A port MUST NOT specify any binding information other than address information
					if(targetPort.getEExtensibilityElements() != null && !targetPort.getEExtensibilityElements().isEmpty()){
						Object each = targetPort.getEExtensibilityElements().get(0);
						if(each instanceof SOAPAddress){
							SOAPAddress address = (SOAPAddress)each;
							endpoint = address.getLocationURI();											
						}	
					}
				}
				if(svcQName != null && portQName!= null && endpoint!= null){
					return true;
				}
			}
		}
		return false;
	}
	
}

