/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.AbstractDataDefRule;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;

/**
 * This class contains methods that are generic and useful for all xsd transformation related rules. 
 */
public abstract class AbstractDataDefRuleImpl extends TransformationRuleImpl implements AbstractDataDefRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected AbstractDataDefRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.ABSTRACT_DATA_DEF_RULE;
	}

	/**
	 * Retrieves the XSD Schema from the Transformation context based on the <code>dataType</code> namespace
	 * @param - dataType BOM Type
	 * @return xsdSchema - XSDSchema or null if none is found
	*/
	public XSDSchema getXSDSchema(Type dataType) {
		
		//DataDefinitionRule rule = (DataDefinitionRule)getRoot().getContext().get(DataDefinitionUtil.createNamespace((NamedElement)dataType));
//		AZ roll back because of introduction of project name into the target namespace to make it unique
		DataDefinitionRule rule = (DataDefinitionRule)getRoot().getContext().get(ProcessUtil.createNamespace((NamedElement)dataType)+ DataDefinitionUtil.XSD_DATA_KEY);
		// Ray Fix for Share Across Project - we need to register the DataDefinitionRule in the context using a special namespace that includes the uid of the package 
		// representing the XSDSchema to avoid conflict because the namespace generated by names of the package structure no longer garanteed to be unqiue across projects
		//DataDefinitionRule rule = (DataDefinitionRule)getRoot().getContext().get(DataDefinitionUtil.getDataDefinitionNamespace((NamedElement)dataType)+ DataDefinitionUtil.XSD_DATA_KEY);
		if(rule != null){
			return (XSDSchema)rule.getTarget().get(0);
		}
		return null;
	}
	
	/**
	 * Returns true if this is a primitive type
	 * @param - dataType a BOM Type
	 * @return boolean - true if the <code>dataType</code> is a predefined simple xsd type.
	 */	
	public boolean isPrimitiveType(Type dataType){
		return DataDefinitionUtil.isPrimitiveType(dataType);
	}
	
	/**
	 * Retrieves the DataDefinitionRule that is the parent rule for all the
	 * XSD predefined types (string, int, boolean.. etc). This Rule is not 
	 * attached to the TransformationRoot, as it is not intended to be persisted.
	 * The DataDefinitionRule for the XSD predefined types is saved in the context 
	 * only with a key. TransformationUtil.XSD_Context_KEY.
	 * Creates the rule if it does not already exist.
	 * @return DataDefinitionRule 
	 */
	public DataDefinitionRule getXSDContext(){
		
		DataDefinitionRule xsdContext = DataDefinitionUtil.getXSDRulesContext(this);
			if(xsdContext == null){
				xsdContext = XsdFactory.eINSTANCE.createDataDefinitionRule();
				//add now the context to the 
				getRoot().getContext().put(DataDefinitionUtil.XSD_CONTEXT_KEY, xsdContext);
			}
			return xsdContext;
	}


} //AbstractDataDefRuleImpl
