/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.xsd.XSDAnnotation;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDComponent;
import org.eclipse.xsd.XSDCompositor;
import org.eclipse.xsd.XSDDerivationMethod;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDInclude;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDNamedComponent;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;
import org.w3c.dom.Element;

import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.DescriptorType;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Property;
import com.ibm.btools.bom.model.artifacts.Signal;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.observation.EventType;
import com.ibm.btools.bom.model.organizationstructures.LocationType;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NameProvider;
import com.ibm.btools.te.ilm.heuristics.naming.NameProviderFactory;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.AbstractDataDefRule;
import com.ibm.btools.te.ilm.heuristics.xsd.ClassRule;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.PropertyRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM DataType
 * TARGET: XSDTypeDefinition
 * Creates the XSDTypeDefinition for the source DataType. 
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ClassRuleImpl extends AbstractDataDefRuleImpl implements ClassRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ClassRuleImpl() {
		super();
	}

	/**
	* The BOM Type
	*/
	private Type dataType = null;
	
	/**
	 * The XSDComplexTypeDefinition for the BOM Type
	 */
	private XSDTypeDefinition xsdTypeDefinition = null;
	
	/**
	 * True if the Rule was already executed.
	 */
	boolean runOnce = false;

	// xsd import enhancement
	private boolean useOriginalXsd = false;
	
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.CLASS_RULE;
	}

	/**
	 * Maps a BOM Type tipically a BOM Class to a XSDTypeDefinition.
	 * SOURCE: BOM Type. 
	 * Target: XSDTypeDefinition.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target(){
		/*
		 * This rule will only execute once. 
		 * If the source dataType is a primitive Type, a PrimitiveDataType rule is created, 
		 * attached to the DataDefinitionRule for the XSD predefined types and executed. 
		 * The target of the PrimitiveDataTypeRule is passed to this rule.
		 * If the source is a complex type, it will create the XSDComplexType  definition.
		 * Will first check for the "super classifier" on the source data type, if exists, a ClassRule is created 
		 * for the "super classifier", attached to the corresponding data type DataDefinitionRule and executed.
		 * For all the complex type's attributes, based on the attribute multiplicity will either create a
		 * ClassRule or a ListRule. 
		 * The target of each rule is then attached to the complex type definition. 
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if(isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if (getSource() == null || getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		dataType = (Type)getSource().get(0);
		if (dataType == null){
			setComplete(true);
			return isComplete();
		}
		// this rule will only be executed once
		if (! runOnce){
			runOnce = true;
			DataDefinitionRule xsdContext = getXSDContext();
			//if this is a predefined type handle it differently
			if(isPrimitiveType(dataType) ){
				//check if there isn't already a rule for this type in the context
				AbstractDataDefRule dataTypeRule = (AbstractDataDefRule)TransformationUtil.getRuleForSource(xsdContext, dataType, XSDSimpleTypeDefinition.class);
				if(dataTypeRule != null){
					getTarget().add(dataTypeRule.getTarget().get(0));
					setComplete(true);
				}else{
					dataTypeRule = XsdFactory.eINSTANCE.createPrimitiveDataTypeRule();
					dataTypeRule.getSource().add(dataType);
					xsdContext.getChildRules().add(dataTypeRule);
					boolean ret = dataTypeRule.transformSource2Target();
					if(ret){
						getTarget().add(dataTypeRule.getTarget().get(0));
					}
					setComplete(true);
				}
			} else {
			    XSDElementDeclaration elementDeclaration = null;
			    XSDAttributeDeclaration attributeDeclaration = null;
				XSDComplexTypeDefinition complexTypeDefinition = XSDFactory.eINSTANCE.createXSDComplexTypeDefinition();
				NameProvider nameProvider = NameProviderFactory.getNameProvider(complexTypeDefinition);
				String convertedName = nameProvider.getName(complexTypeDefinition, ((NamedElement)getSource().get(0)), NamingUtil.findRegistry(this));
				complexTypeDefinition.setName(convertedName);
				complexTypeDefinition.setTargetNamespace(getXSDSchema(dataType).getTargetNamespace());
		
				if(((com.ibm.btools.bom.model.artifacts.Classifier)dataType).getIsAbstract().booleanValue()){
				    complexTypeDefinition.setAbstract(true);
				}

				NamedElement sourceElement = (NamedElement)getSource().get(0);
				
				if (OriginalXsdUtil.isExternalType(sourceElement)) {
					useOriginalXsd = true;
				    // Get from the registry if already used and stored.
				    XSDTypeDefinition typeDef = (XSDTypeDefinition)OriginalXsdUtil.getXSDTypeFromRegistry(getContext(), sourceElement);
				    if (typeDef == null) {
				        elementDeclaration = OriginalXsdUtil.getElementDeclarationFromRegistry(getContext(), sourceElement);
				        if (elementDeclaration != null && elementDeclaration.getType() != null) {
				            xsdTypeDefinition = elementDeclaration.getType();
				        } else {
				            attributeDeclaration =  OriginalXsdUtil.getAttributeDeclarationFromRegistry(getContext(), sourceElement);
				            if (attributeDeclaration != null && attributeDeclaration.getType() != null) {
				                xsdTypeDefinition = attributeDeclaration.getType();
				            }
				        }
				    } else {
						xsdTypeDefinition = typeDef;
				    }
				    
				    if (typeDef == null && elementDeclaration == null  && attributeDeclaration == null) { // get it from the schema
			            ExternalSchema externalSchema = (ExternalSchema)((Class)sourceElement).getOwningPackage();
			            XSDSchema origSchema = (XSDSchema)OriginalXsdUtil.getSchema(getContext(), externalSchema);
						if (origSchema == null && sourceElement instanceof Class) {	// if not found, load from URL, and store
							origSchema = OriginalXsdUtil.loadOriginalSchema(getContext(), externalSchema, sourceElement);
						}
						if (origSchema != null) {						    
						    // check if this type has been defined in the external elements type registry
							typeDef = (XSDTypeDefinition)OriginalXsdUtil.getXSDTypeFromSchema(origSchema, sourceElement.getName());
							if (typeDef != null) {
							    xsdTypeDefinition = typeDef;
								OriginalXsdUtil.registerXsdElement(getContext(), sourceElement, xsdTypeDefinition);
							} else if (typeDef == null) {
							    // get element: reuse if exists, otherwise register					    
							    elementDeclaration = OriginalXsdUtil.getElementDeclarationFromSchema(origSchema, sourceElement.getName());
							    if (elementDeclaration != null) {
							        OriginalXsdUtil.registerXsdElement(getContext(), sourceElement, elementDeclaration);
							        xsdTypeDefinition = elementDeclaration.getType();
							    } else {
							        attributeDeclaration = OriginalXsdUtil.getAttributeDeclarationFromSchema(origSchema, sourceElement.getName());
								    if (attributeDeclaration != null) {
								        OriginalXsdUtil.registerXsdElement(getContext(), sourceElement, attributeDeclaration);
								        xsdTypeDefinition = attributeDeclaration.getType();
								    }
							    }
							}
							if (xsdTypeDefinition == null) {
								String parentName = null;
								String childName = null;
								if (sourceElement.getName().indexOf(".") != -1) {
									childName = sourceElement.getName().substring(sourceElement.getName().lastIndexOf(".") + 1);	
									parentName = sourceElement.getName().substring(0, sourceElement.getName().indexOf("."));
								}

								if (getParentRule().getChildRules().get(0) instanceof ClassRule &&
										((ClassRule)getParentRule().getChildRules().get(0)).getTarget() != null &&
										!((ClassRule)getParentRule().getChildRules().get(0)).getTarget().isEmpty() &&
										((ClassRule)getParentRule().getChildRules().get(0)).getTarget() != this) {
									ClassRule parentRule = findParentRule(getParentRule().getChildRules(), parentName);
									if (parentRule != null) {
										XSDComponent xsdParent = (XSDComponent)parentRule.getTarget().get(0);	// this is the top level container
										xsdTypeDefinition = OriginalXsdUtil.getTypeFromParent(xsdParent, childName);
										if (xsdTypeDefinition != null) {
											OriginalXsdUtil.registerXsdElement(getContext(), sourceElement, xsdTypeDefinition);
										}
									}
								}
							}
						}
				    }
				}else{
					//this is not an external type so we need to register the typeDefinition with the displayName: 
//					XSDSchema thisSchema = getXSDSchema(dataType);
//					complexTypeDefinition.setTargetNamespace(getXSDSchema(dataType).getTargetNamespace());
//					thisSchema.updateElement();
//					XSDAnnotation annotation = XSDFactory.eINSTANCE.createXSDAnnotation();
//					thisSchema.getContents().add(annotation);
//					
//					Element appInfo = annotation.createApplicationInformation("http://someModelerNamespace");
//					
//					Element modelerInfo = thisSchema.getDocument().createElementNS("http://someModelerNamespace", "modeler:displayName");
//					
//					modelerInfo.setTextContent("some DisplayName");
//					
//					Element element = annotation.getElement();
//					if (element != null){
//						element.appendChild(appInfo);
//					}
//					appInfo.appendChild(modelerInfo);
//					complexTypeDefinition.setAnnotation(annotation);

					ProcessUtil.registerDisplayNameForXSDElement(this.getContext(), complexTypeDefinition, sourceElement.getName());	
				}
				
				// if not external type, set complexType attribute to newly created type
				if (xsdTypeDefinition == null) {
				    xsdTypeDefinition = complexTypeDefinition;
				}
				
				if (elementDeclaration != null && xsdTypeDefinition.getName() == null) {
				    getTarget().add(elementDeclaration);
				} else if (attributeDeclaration != null && xsdTypeDefinition.getName() == null) {
				    getTarget().add(attributeDeclaration);
				} else {
				    getTarget().add(xsdTypeDefinition);
				}
				
				if(xsdContext.getTarget() != null && !xsdContext.getTarget().isEmpty()){
					XSDSchema xsdSchema= (XSDSchema)xsdContext.getTarget().get(0);
					if(!useOriginalXsd && notInSchema(xsdSchema, xsdTypeDefinition) && !OriginalXsdUtil.isFromMetaSchema(xsdTypeDefinition)){
						xsdSchema.getContents().add(xsdSchema.getContents().size(), xsdTypeDefinition);
					}
				}
				setupChildRules();
			}
		}
		executeHandlers();
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
		
	}
	
	/**
	 * Returns true if the XSDNamedComponent is not already added to the schema.
	 * An additional check is made based on the type definition names. 
	 * @param xsdSchema the xsdSchema target of this Rule
	 * @param xsdDefinition a XSDTypeDefinition
	 * @return true - if the xsdDefinition is not already in schema. 
	 */
	private boolean notInSchema(XSDSchema xsdSchema, XSDNamedComponent xsdDefinition){
		if(!xsdSchema.getContents().contains(xsdDefinition)){
			if(xsdSchema.getTypeDefinitions()!= null){
				Iterator it = xsdSchema.getTypeDefinitions().iterator();
				XSDTypeDefinition thisDefinition = null;
				while(it.hasNext()){
					thisDefinition = (XSDTypeDefinition)it.next();
					if(thisDefinition.getName() != null && thisDefinition.getName().equals(xsdDefinition.getName())){	// GJD 20060820: should we check the elements for the anonymous type?
						xsdDefinition.setTargetNamespace(thisDefinition.getTargetNamespace());
						return false;
					}
				}
			}
			return true;
				
		}
		return false;
	
	}
	
	
	/**
	 * Creates and executes the child rules. Will create ClassRule for the superClassfier of the 
	 * <code>dataType</code> (if exists) and Class or ListRules for each of the <code>dataType</code> attributes. 
	 */
	private void setupChildRules(){
		// check first for superClass
		boolean finished = createSuperClassRule();
		//process class attributes
		boolean executed = createAttributesRules();
		
		if (finished && executed){
			setComplete(true);
		}
	}
	/**
	 * If the <code>dataType</code> has a superClassifier, creates and executes a ClassRule that has
	 * as source the superClassifier (type).
	 * The target of the ClassRule is the XSDTypeDefinition that will serve as baseTypeDefinition
	 * for the <code>dataType</code> XSDComplexType.
	 * Reuse the ClassRule if it already exists in the context.	
	 * @return boolean - true if the rule was executed successfully
	 */
	private boolean createSuperClassRule(){
		//NOTE: Currently only one superType is allowed for a BOM Class
		boolean ret = true;
		com.ibm.btools.bom.model.artifacts.Classifier type = (com.ibm.btools.bom.model.artifacts.Classifier)dataType;
		if(type.getSuperClassifier() != null && !type.getSuperClassifier().isEmpty()){
			List superClassList = type.getSuperClassifier();
			Iterator it = superClassList.iterator();
			com.ibm.btools.bom.model.artifacts.Classifier superClass;
			DataDefinitionRule parentRule;
			ClassRule propertyRule;
			XSDTypeDefinition superType;
				while(it.hasNext()){
					superClass = (com.ibm.btools.bom.model.artifacts.Classifier)it.next();
					propertyRule = (ClassRule)TransformationUtil.getRuleForSource(getRoot(), superClass, XSDTypeDefinition.class);
					if(propertyRule == null){
						parentRule = DataDefinitionUtil.getDataDefinitionRule(this, superClass);
						propertyRule = XsdFactory.eINSTANCE.createClassRule();
						propertyRule.getSource().add(superClass);
						parentRule.getChildRules().add(propertyRule);
						parentRule.transformSource2Target();
					}
					// xsd import enhancement
					if (useOriginalXsd) {
						continue;
					}
					superType = (XSDTypeDefinition)propertyRule.getTarget().get(0);
					organizeImports(getXSDSchema(dataType), superType, superClass);
					if (xsdTypeDefinition instanceof XSDComplexTypeDefinition) {
					    ((XSDComplexTypeDefinition)xsdTypeDefinition).setBaseTypeDefinition(superType);
					    ((XSDComplexTypeDefinition)xsdTypeDefinition).setDerivationMethod(XSDDerivationMethod.EXTENSION_LITERAL);
					}
					// 169413 KK  (WID)Xsd can not be resolved
					//xsdTypeDefinition.setTargetNamespace(getXSDSchema(superClass).getTargetNamespace());
			  }
		 }
		 return ret;
	}
	
	/**
	 * Retrieves the ownedAttribute for different types of 'Classifier'
	 * @param classifier
	 * @return list of owned attributes
	 */
	private List getOwnedAttribute(com.ibm.btools.bom.model.artifacts.Classifier classifier) {
		if (classifier instanceof com.ibm.btools.bom.model.artifacts.Class) {
			return ((com.ibm.btools.bom.model.artifacts.Class)classifier).getOwnedAttribute();
		} else if (classifier instanceof BulkResourceType) {
			return ((BulkResourceType)classifier).getOwnedAttribute();
		} else if (classifier instanceof IndividualResourceType) {
			return ((IndividualResourceType)classifier).getOwnedAttribute();
		} else if (classifier instanceof OrganizationUnitType) {
			return ((OrganizationUnitType)classifier).getOwnedAttribute();
		} else if (classifier instanceof LocationType) {
			return ((LocationType)classifier).getOwnedAttribute();
		}else if(classifier instanceof Signal){
			return ((Signal)classifier).getOwnedAttribute();
		}else if(classifier instanceof EventType){
			return ((EventType)classifier).getOwnedAttribute();
		}else if(classifier instanceof DescriptorType){
				return ((EventType)classifier).getOwnedAttribute();
		}else{
			return null;	
		}
		
	}
	
	/**
	 * Creates the <code>complexType</code> data structure. 
	 * For each of the <code>datatype</code> attributes will create and execute the ClassRule or ListRule 
	 * depending of the multiplicity of the attribute. Each of the rules created for the attributes will be 
	 * attached to the corresponding DataDefinitionRule. 
	 * @return boolean
	 */	
	private boolean createAttributesRules(){

		List attributes = getOwnedAttribute((com.ibm.btools.bom.model.artifacts.Classifier)dataType);
		if(attributes != null && ! attributes.isEmpty()){
			XSDModelGroup modelGroup = XSDFactory.eINSTANCE.createXSDModelGroup();
			modelGroup.setCompositor(XSDCompositor.SEQUENCE_LITERAL);
			XSDParticle particle = XSDFactory.eINSTANCE.createXSDParticle();
			particle.setContent(modelGroup);
			// xsd import enhancement
			if (!useOriginalXsd && xsdTypeDefinition instanceof XSDComplexTypeDefinition) {
			    ((XSDComplexTypeDefinition)xsdTypeDefinition).setContent(particle);
			}
			Iterator iterator = attributes.iterator();
			//for each of the attributes create a ClassRule or a ListRule.
			DataDefinitionRule parentRule;
			TransformationRule propertyRuleAttribute = null;
			while(iterator.hasNext()){	
				Property attribute = (Property)iterator.next();
				propertyRuleAttribute = (TransformationRule)TransformationUtil.getRuleForSource(this.getRoot(), attribute.getType(), XSDTypeDefinition.class);
				if(propertyRuleAttribute == null){
					propertyRuleAttribute = XsdFactory.eINSTANCE.createClassRule();
					if (useOriginalXsd) {
						((ClassRuleImpl)propertyRuleAttribute).setUseOriginalXsd(true);
					}
					if(attribute.getType() == null){
						PrimitiveType predefinedAnyType = DataDefinitionUtil.getAnyType();
						propertyRuleAttribute.getSource().add(predefinedAnyType);
						getChildRules().add(propertyRuleAttribute);
						propertyRuleAttribute.transformSource2Target();
					}else if (DataDefinitionUtil.isPrimitiveType(attribute.getType())){
						propertyRuleAttribute.getSource().add(attribute.getType());
						getChildRules().add(propertyRuleAttribute);
						propertyRuleAttribute.transformSource2Target();
					} else {
						propertyRuleAttribute.getSource().add(attribute.getType());
						parentRule = DataDefinitionUtil.getDataDefinitionRule(this, attribute);					
						parentRule.getChildRules().add(propertyRuleAttribute);
						parentRule.transformSource2Target();
					}			
				}
				if(propertyRuleAttribute.getTarget()== null || propertyRuleAttribute.getTarget().isEmpty()){
					// Log a warning
					if (attribute.getType() == null) {
						String params[] = new String[1];
						params[0] = BomUtils.getCanonicalName(attribute,false,true);
						LoggingUtil.logWarning(MessageKeys.NO_TYPE_SPECIFIED,params,null);					
					}
					break;
				}
				//add each target to the complex type definition
				XSDTypeDefinition property = null;
				if (propertyRuleAttribute.getTarget().get(0) instanceof XSDAttributeDeclaration) {
				    property = ((XSDAttributeDeclaration)propertyRuleAttribute.getTarget().get(0)).getType();
				} else if (propertyRuleAttribute.getTarget().get(0) instanceof XSDElementDeclaration) {
					property = ((XSDElementDeclaration)propertyRuleAttribute.getTarget().get(0)).getType();
				} else {
				    property = (XSDTypeDefinition)propertyRuleAttribute.getTarget().get(0);
				}
				organizeImports(getXSDSchema(dataType), property, attribute);
				
				PropertyRule propertyRule = XsdFactory.eINSTANCE.createPropertyRule();
				propertyRule.getSource().add(attribute);
				propertyRule.getSource().add(property);		
				getChildRules().add(propertyRule);
				propertyRule.transformSource2Target();
				// xsd import enhancement
				if (useOriginalXsd) {
					continue;
				}
				XSDParticle xsdParticle = (XSDParticle)propertyRule.getTarget().get(0);						
				modelGroup.getContents().add(xsdParticle);

				
				// Convert the returned XSD element name at this point because only at this point
				// the element is scoped to the complex type.  This is important for the NCName
				// conversion as it can perform properly for checking uniqueness of the name
				// within the scope of the complex type
				XSDElementDeclaration decl = (XSDElementDeclaration)xsdParticle.getContent();
				if (decl.getResolvedElementDeclaration() == decl) {
					JavaNCNameConverter converter = new JavaNCNameConverter();
					String convertedName =
						converter.convertName(
							NamingUtil.findRegistry(this),
							decl,
							decl.getName());
					decl.setName(convertedName);
					//feature 190156, form enhancement
					ProcessUtil.registerDisplayNameForXSDElement(this.getContext(), decl, attribute.getName());
				}
			}
		}
		return true;
	}

	/**
	 * If the <code>typeDefinition</code> is defined in another XSDSchema, add the import for that schema
	 * (if not already added) to this <code>schema</code>.
	 * @param schema - XSDSchema 
	 * @param typeDefinition - XSDTypeDefiniton 
	 * @param source TODO
	 */
	private void organizeImports(XSDSchema schema, XSDTypeDefinition typeDefinition, NamedElement source){
		// do not change for original XSD
		if (useOriginalXsd) {
			return;
		}
		if(typeDefinition != null && schema != null){
			String namespace = typeDefinition.getTargetNamespace();
			if(namespace != null && (namespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) ||!namespace.equals(schema.getTargetNamespace()) ||
							(namespace.equals(schema.getTargetNamespace()) &&
									typeDefinition.getSchema() != null &&
									typeDefinition.getSchema().getSchemaLocation() != null &&
									typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1))) {
				XSDTypeDefinition resolved = schema.resolveTypeDefinition(namespace, typeDefinition.getName());
				if(resolved != typeDefinition) {
					if (namespace.equals(schema.getTargetNamespace())) {
						if (!OriginalXsdUtil.hasIncludeForFile(schema, typeDefinition.getSchema().getSchemaLocation())) { 
							XSDInclude inc = XSDFactory.eINSTANCE.createXSDInclude();
							inc.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
							schema.getContents().add(0, inc);
						}
					// need to add namespaces etc
					//check if there already is an import for this namespace
					} else if(!OriginalXsdUtil.hasImportForNamespace(schema, namespace) ||
//d161889
//d161889 schema.getQNamePrefixToNamespaceMap().get(DataDefinitionUtil.getNamespacePrefix(namespace)) == null ||
							(typeDefinition.getSchema().getSchemaLocation() != null && !OriginalXsdUtil.hasImportForFile(schema, typeDefinition.getSchema().getSchemaLocation()))){
						XSDImport imp = XSDFactory.eINSTANCE.createXSDImport();
						imp.setNamespace(namespace);
						if (typeDefinition.getSchema().getSchemaLocation() != null && 
								typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
							imp.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
						} else {
							imp.setSchemaLocation(ProcessUtil.createFileName(namespace)+".xsd");
						}
						schema.getContents().add(0,imp);
						schema.getQNamePrefixToNamespaceMap().put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);
					}
				}
					//defect 88901. failed to handle the nonamespace type definition
			}else if(namespace == null){
				if((typeDefinition.getSchema().getSchemaLocation() != null && !OriginalXsdUtil.hasImportForFile(schema, typeDefinition.getSchema().getSchemaLocation()))){
					XSDImport imp = XSDFactory.eINSTANCE.createXSDImport();
					//imp.setNamespace(namespace);
					if (typeDefinition.getSchema().getSchemaLocation() != null && 
							typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
						imp.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
					} else {
						imp.setSchemaLocation(ProcessUtil.createFileName(namespace)+".xsd");
					}
					schema.getContents().add(0,imp);
//					schema.getQNamePrefixToNamespaceMap().put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);
				}				
			}
			
			//else {
				// Ray Fix for Share Across project - for persisting mutliple XSDSchemas with the same namespace
				// In the case that business item referencing other business item in the same namespace, we need to
				// create Include insteadof Import
//				if (schema != typeDefinition.getSchema() && namespace.equals(schema.getTargetNamespace())) {
//					XSDInclude incl = XSDFactory.eINSTANCE.createXSDInclude();
//					if (typeDefinition.getSchema().getSchemaLocation() != null && 
//							typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
//						incl.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
//					} else {
//						// Ensure the schemalocation matches the key used for registering the schema info in the SourcePathToTargetPathAbsoluteRegistry
//						// Here we include the schema hashcode to distinguish between different schemas with the same namespace
//						// See ExportOperation.java createResource method
//						//incl.setSchemaLocation(typeDefinition.getSchema().hashCode()+namespace+ProcessUtil.createFileName(namespace)+".xsd");
//						String includedTypeProjectName = null;
//						if(source instanceof Property){
//							Property property = (Property)source;
//							if(property.getType() instanceof Class){
//								includedTypeProjectName = CrossProjectReferenceUtil.getProjectNameForSourceObject(property.getType());
//							}
//						}else{
//							includedTypeProjectName = CrossProjectReferenceUtil.getProjectNameForSourceObject(source);
//						}						
//						incl.setSchemaLocation(namespace+ProcessUtil.createFileName(namespace)+".xsd"+"#"+includedTypeProjectName);					
//					}
//					schema.getContents().add(0,incl);
//				}
//			}

		}
	}

	/**
	 * @param useOriginalXsd The useOriginalXsd to set.
	 */
	public void setUseOriginalXsd(boolean useOriginalXsd) {
		this.useOriginalXsd = useOriginalXsd;
	}
	
	private ClassRule findParentRule (List childRules, String name) {
		if (childRules == null || childRules.isEmpty() || name == null) {
			return null;
		}
		
		Iterator it = childRules.iterator();
		ClassRule classRule = null;
		Object obj = null;
		ClassRule aClassRule = null;
		while (it.hasNext() && classRule == null) {
			obj = it.next();
			
			if (obj instanceof ClassRule) {
				aClassRule = (ClassRule) obj;
				if (aClassRule.getSource() != null && !aClassRule.getSource().isEmpty() &&
						aClassRule.getSource().get(0) instanceof Class &&
						((Class)aClassRule.getSource().get(0)).getName().equals(name)) {
					classRule = aClassRule;
				}
			}
		}
		
		return classRule;
	}
} //ClassRuleImpl
