/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDNamedComponent;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;

import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.impl.TransformationRuleImpl;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.XsdNamingRegistry;

/**
 * <!-- begin-user-doc -->
 * Source: BOM DataType
 * Target: XSD Schema
 * This Rule is the root rule of the transformation that is responsible for the creation of the XSDSchema.
 * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class DataDefinitionRuleImpl extends TransformationRuleImpl implements DataDefinitionRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected DataDefinitionRuleImpl() {
		super();
	}

	private XSDSchema xsdSchema;

	// GJD 20060720 added Xsd import enhancement
	private boolean useOriginalXsd = false;

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.DATA_DEFINITION_RULE;
	}

	/**
	 * Creates an XSDSchema for the namespace associated with the source.
	 */
	public XSDSchema createXSDSchema() {
		if(getSource() == null || getSource().isEmpty()){
			return null;
		}

		NamedElement sourceElement = (NamedElement)getSource().get(0);
		XSDSchema origSchema = null;
		  if (sourceElement instanceof Class && ((Class)sourceElement).getOwningPackage() instanceof ExternalSchema) {
		  	ExternalSchema externalSchema = (ExternalSchema)((Class)sourceElement).getOwningPackage();
            origSchema = (XSDSchema)OriginalXsdUtil.getSchema(getContext(), externalSchema);
        
            if (origSchema == null && sourceElement instanceof Class) {	// if not found, try to load from URL, and store
            	origSchema = OriginalXsdUtil.loadOriginalSchema(getContext(), externalSchema, sourceElement);
	        }
		}

	    if (origSchema != null) {
			xsdSchema = origSchema;
			useOriginalXsd = true;    
	    } else {
			xsdSchema = XSDFactory.eINSTANCE.createXSDSchema();
			TargetNamespaceProvider targetProvider = new TargetNamespaceProvider();
			String targetNamespace  = targetProvider.getDefaultTargetNamespace((NamedElement)getSource().get(0), true );
			xsdSchema.setTargetNamespace(targetNamespace);
			xsdSchema.setSchemaForSchemaQNamePrefix("xsd");
			Map namespaces = xsdSchema.getQNamePrefixToNamespaceMap();
			namespaces.put("tns", xsdSchema.getTargetNamespace());
			namespaces.put(xsdSchema.getSchemaForSchemaQNamePrefix(), XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
	    }
	    
		return xsdSchema;
	}
	
	/**
	 * SOURCE: BOM dataTpe
	 * TARGET: XSDSchema
	 * This rule is responsible for creating the XSD Schema.
	 * When first called, this method will create the XSDSchema for the BOM Type namespace that it has as source, 
	 * and set it as its target. All XSD Rules that share the same namespace are childs of this rule.
	 * As this rule is shared, it will only be set to complete at the end of the transformation cycle.
	 * @return boolean
	 * @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 **/
	public boolean transformSource2Target(){
		LoggingUtil.traceEntry(this, "transformSource2Target");
		if(isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		else{
			if(target == null || target.isEmpty()){
				xsdSchema = createXSDSchema();
				if(xsdSchema != null){
					getTarget().add(xsdSchema);
				}
				//Register the XsdNamingRegistry
				//this.eContainer = null if it's a simple type
				//if it's a simple type, it is always a valid name
				if (this.eContainer!=null){						
				 	TransformationContext context = getContext();
				 	if (NamingUtil.getRegistry(context, xsdSchema)==null)				
				  		NamingUtil.putRegistry(context, xsdSchema, new XsdNamingRegistry());
				}
			}
			executeChildRules();
			// this rule will be set to complete only at the of transformation
			setComplete(false);
		}
//		AZ WMT 20070322
		executeHandlers();
		setComplete(false);
		LoggingUtil.traceExit(this, "transformSource2Target");
		return isComplete();
	}
	
	/**
	 * Executes each child rule and immediately process the child rule targets.
	 * This is done in order to avoid processing the target of a rule if the rule is
	 * executed several times.
	 * @return boolean 
	 */
	protected boolean executeChildRules(){
		boolean allComplete = true;
		if (childRules != null) {
			for (int i = 0; i < childRules.size(); i++) {
				TransformationRule r = (TransformationRule) childRules.get(i);
				if (!r.isComplete() && !r.isFailed()) {
					boolean  ret = r.transformSource2Target();
					processChildTargets(r);
					if(!ret)
						allComplete = false;
				}
			}
		}
		return allComplete;		
	}	
		
	/**
	* Process the target of the rule. A DataDefinitionRule will process XSDTypes and XSDElementDeclarations from its target.
	* Adds the XSDTypeDefinitions and XSDElementDeclarations to the XSDSchema. Previously check
	* if same XSD elements were not already added. 
	* @param rule a child rule to be processed.  
	*/
	private void processChildTargets(TransformationRule rule){
		EObject childTarget = null;
		List childTargets = rule.getTarget();
		for (int i=0; i<childTargets.size(); i++){
			childTarget = (EObject) childTargets.get(i);
			if (childTarget instanceof XSDTypeDefinition && xsdSchema != null){
				XSDTypeDefinition typeDefinition = (XSDTypeDefinition)childTarget;
				if(!useOriginalXsd && notInSchema(typeDefinition) && !OriginalXsdUtil.isFromMetaSchema(typeDefinition)){
					xsdSchema.getContents().add(xsdSchema.getContents().size(), typeDefinition);
				}
			}
		}	
	}
	/**
	 * Returns true if the XSDNamedComponent is not already added to the schema.
	 * An additional check is made based on the type definition names. 
	 * @param xsdSchema the xsdSchema target of this Rule
	 * @param xsdDefinition a XSDTypeDefinition
	 * @return true - if the xsdDefinition is not already in schema. 
	 */
	private boolean notInSchema(XSDNamedComponent xsdDefinition){
		if(!xsdSchema.getContents().contains(xsdDefinition)){
			if(xsdSchema.getTypeDefinitions()!= null){
				Iterator it = xsdSchema.getTypeDefinitions().iterator();
				XSDTypeDefinition thisDefinition = null;
				while(it.hasNext()){
					thisDefinition = (XSDTypeDefinition)it.next();
					if(thisDefinition.getName() != null && thisDefinition.getName().equals(xsdDefinition.getName())){	// GJD 20060820: should we check the elements for the anonymous type?
						xsdDefinition.setTargetNamespace(thisDefinition.getTargetNamespace());
						return false;
					}
				}
			}
			return true;
				
		}
		return false;
	
	}
	

    /**
     * @return Returns the useOriginalXsd.
     */
    public boolean isUseOriginalXsd() {
        return useOriginalXsd;
    }
	/**
	 * @param useOriginalXsd The useOriginalXsd to set.
	 */
	public void setUseOriginalXsd(boolean useOriginalXsd) {
		this.useOriginalXsd = useOriginalXsd;
	}
} //DataDefinitionRuleImpl
