/**
 * <copyright>
 *
 *    Copyright IBM Corporation 2003, 2009. All rights reserved.
 *
 * </copyright>
 *
 * %W%
 * @version %I% %H%
 */
package com.ibm.btools.te.ilm.heuristics.xsd.impl;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2008, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDCompositor;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDTypeDefinition;

import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.Variable;
import com.ibm.btools.te.framework.FrameworkPackage;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.JavaNCNameConverter;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.heuristics.naming.TargetNamespaceProvider;
import com.ibm.btools.te.ilm.heuristics.xsd.ClassRule;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.ListRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdPackage;
import com.ibm.btools.te.ilm.heuristics.xsd.util.DataDefinitionUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.te.ilm.model.sa.SolutionArtifact;

/**
 * <!-- begin-user-doc -->
 * SOURCE: BOM TypedElement also a MultiplicityElement.
 * TARGET: XSDTypeDefinition 
 * The rule is responsible for transforming a BOM TypedElement which is also a 
 * MultiplicityElement to a new XSDTypeDefinition setting appropriate values for the minimum and maximum occurence .
  * <!-- end-user-doc -->
 * <p>
 * </p>
 *
 * @generated
 */
public class ListRuleImpl extends AbstractDataDefRuleImpl implements ListRule {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ListRuleImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EClass eStaticClass() {
		return XsdPackage.Literals.LIST_RULE;
	}

	/**
		 * True if the Rule was already executed.
		 */
		boolean runOnce = false;
	/**
	* SOURCE: BOM TypedElement (MultiplicityElement)
 	* TARGET: XSDTypeDefinition 
	* The rule is responsible for transforming a BOM TypedElement which is also a 
 	* MultiplicityElement to a new XSDTypeDefinition whith the corresponding values for 
 	* min and maxOccurs.
 	* @return boolean
 	* @see com.ibm.btools.te.framework.TransformationRule#transformSource2Target()
	 */
	public boolean transformSource2Target(){
		/*
		 * Reuse the XSDDefinition for the base type or creates a new ClassRule, if the base type 
		 * the type of the input source was not yet processed.  
		 * Creates the XSDComplexTypeDefinition using the base type. Min and maxOccurs are set based on the 
		 * type lower and upper bound. Check the schema if there was already a similar XSDTypeCreated (with same 
		 * lower and upper bound = the check will be done based on the type name). If yes, substitute the old type definition
		 * with the new one. 
		 * 
		 */
		LoggingUtil.traceEntry(this, "transformSource2Target"); 
		if(isComplete()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return true;
		}
		if(getSource() == null && getSource().isEmpty()){
			LoggingUtil.traceExit(this, "transformSource2Target");
			return false;
		}
		//only execute this rule once. 
		if (! runOnce){
			runOnce = true;
			TypedElement typedElement = (TypedElement)getSource().get(0);
			Type dataType = typedElement.getType();
			XSDTypeDefinition baseType = null;
			TransformationRule xsdRule = TransformationUtil.getRuleForSource(getRoot(), dataType, XSDTypeDefinition.class);
				if(xsdRule != null){
					baseType =  (XSDTypeDefinition)xsdRule.getTarget().get(0);
			}else{
				ClassRule classRule = XsdFactory.eINSTANCE.createClassRule();
				classRule.getSource().add(dataType);
				getParentRule().getChildRules().add(classRule);
				getParentRule().transformSource2Target();
				// GJD 20060821
				if (classRule.getTarget().get(0) instanceof XSDElementDeclaration) {
				    baseType = ((XSDElementDeclaration)classRule.getTarget().get(0)).getType();
				} else {
				    baseType = (XSDTypeDefinition)classRule.getTarget().get(0);
				}
			}
			MultiplicityElement element  = (MultiplicityElement)typedElement;
			XSDComplexTypeDefinition complexType = XSDFactory.eINSTANCE.createXSDComplexTypeDefinition();	// GJD20060821 This creates the complex type for the list 
			XSDModelGroup modelGroup = XSDFactory.eINSTANCE.createXSDModelGroup();
			modelGroup.setCompositor(XSDCompositor.SEQUENCE_LITERAL);
			XSDParticle particle = XSDFactory.eINSTANCE.createXSDParticle();
			particle.setContent(modelGroup);
			complexType.setContent(particle);
			XSDElementDeclaration decl = XSDFactory.eINSTANCE.createXSDElementDeclaration();
			// Check to see if it's a valid name by calling NameConverter
			JavaNCNameConverter converter = new JavaNCNameConverter();
			//GJD 20060821
			String convertedName = null;
			if (baseType.getName() != null) {
			    convertedName = converter.convertName(NamingUtil.findRegistry(this), decl, baseType.getName().toLowerCase()+ "Item");
				decl.setName(convertedName);
//			} else {
//			    convertedName = converter.convertName(NamingUtil.findRegistry(this), decl, ((XSDElementDeclaration) baseType.eContainer()).getName().toLowerCase()+ "Item");
			}
			//decl.setName(baseType.getName().toLowerCase()+ "Item");
			decl.setTypeDefinition(baseType);
			XSDParticle attributeParticle = XSDFactory.eINSTANCE.createXSDParticle();
			attributeParticle.setContent(decl);
			//TODO = check for Expressions....	
			ValueSpecification minOccursValue = element.getLowerBound();
			
			int minOccurs = 0;
			int maxOccurs = 0;
			if(minOccursValue != null){
				minOccurs = DataDefinitionUtil.convertToXSDMultiplicity(minOccursValue);
				//the tool default value for the min is 0. We should set it to 1 in order to cover the most common scenarios.
				//Defect 115293
				if(element instanceof Variable){
					if(DataDefinitionUtil.convertToXSDMultiplicity(minOccursValue) == 0){
						minOccurs = 1;
					}
				}
				attributeParticle.setMinOccurs(minOccurs);
			}
			ValueSpecification maxOccursValue = element.getUpperBound();
			if(maxOccursValue != null){
				maxOccurs = DataDefinitionUtil.convertToXSDMultiplicity(maxOccursValue);
				attributeParticle.setMaxOccurs(maxOccurs);
			}
		
			modelGroup.getContents().add(attributeParticle);
			//GJD 20060821
			String baseTypeName = null;
			//d140981 distinguish myInteger vs MyInteger
			if (baseType.getName() != null) {
//			    baseTypeName = DataDefinitionUtil.capitalizeFirstChar(baseType.getName());
			    baseTypeName = baseType.getName();
			} else {
//			    baseTypeName = DataDefinitionUtil.capitalizeFirstChar(((XSDElementDeclaration) baseType.eContainer()).getName());
			    baseTypeName = ((XSDElementDeclaration) baseType.eContainer()).getName();
			}
			//if unbounded, generate ListTypes
			if(attributeParticle.getMaxOccurs() == -1){
				if(attributeParticle.getMinOccurs() == 0){
					// Check to see if it's a valid name by calling NameConverter					
					convertedName = converter.convertName(NamingUtil.findRegistry(this), complexType, "ListOf" + baseTypeName +  "s");
					complexType.setName(convertedName);
					//complexType.setName("ListOf" + baseTypeName +  "s");
				}else{
					// Check to see if it's a valid name by calling NameConverter					
					convertedName = converter.convertName(NamingUtil.findRegistry(this), complexType, "ListOf" + attributeParticle.getMinOccurs() + "ToUnbounded" + baseTypeName + "s");
					complexType.setName(convertedName);
					//complexType.setName("ListOf" + attributeParticle.getMinOccurs() + "ToUnbounded" + baseTypeName + "s");
				}
			}else{
				//generate ListOfNToMTypes
				// Check to see if it's a valid name by calling NameConverter					
				convertedName = converter.convertName(NamingUtil.findRegistry(this), complexType, "ListOf" +attributeParticle.getMinOccurs() + "To" + attributeParticle.getMaxOccurs() + baseTypeName + "s");
				complexType.setName(convertedName);
				//complexType.setName("ListOf" +attributeParticle.getMinOccurs() + "To" + attributeParticle.getMaxOccurs() + baseTypeName + "s");
			}
			DataDefinitionRule parentRule = (DataDefinitionRule)getParentRule();
			
			if (parentRule instanceof DataDefinitionRuleImpl &&
			        !((DataDefinitionRuleImpl)parentRule).isUseOriginalXsd()) {	//GJD20060821: original code
			    XSDSchema schema = (XSDSchema)parentRule.getTarget().get(0);
			    complexType.setTargetNamespace(schema.getTargetNamespace());
			    //AR 2003.03.03
			    //	XSDComplexTypeDefinition sameType = (XSDComplexTypeDefinition)sameInSchema(schema, complexType);
			    XSDComplexTypeDefinition sameType = (XSDComplexTypeDefinition)sameInSchema(schema, baseType, minOccurs, maxOccurs);
			    if(sameType != null){
			        complexType = sameType;
			    }
			    getTarget().add(complexType);	
			    executeHandlers();
			    setComplete(true);
			} else {		// create/retrieve the schema XSD with the list type, referencing the original element
			    XSDSchema xsd = null;
				TargetNamespaceProvider targetProvider = new TargetNamespaceProvider();
				String targetNamespace = targetProvider.getDefaultTargetNamespace(typedElement, true);
				String originalSchemaNamespace = baseType.getTargetNamespace();
				String listSchemaLocation = null;
				if (baseType != null &&
						baseType.getSchema() != null &&
						baseType.getSchema().getSchemaLocation() != null &&
						baseType.getSchema().getSchemaLocation().length() > 0 &&
						baseType.getSchema().getSchemaLocation().lastIndexOf(".") != -1) {
					listSchemaLocation = baseType.getSchema().getSchemaLocation().substring(0, baseType.getSchema().getSchemaLocation().lastIndexOf(".")) +
											ExportOperation.COLLECTION_SUFFIX +
											ExportOperation.XSD_FILE_EXT;
				}
				
			    if (originalSchemaNamespace != null && OriginalXsdUtil.getSchema(getContext(), originalSchemaNamespace) != null) {	// this is NOT the schema for the typedElement, rather the child object - newly created schema for list types
			        xsd = (XSDSchema)OriginalXsdUtil.getSchema(getContext(), originalSchemaNamespace);
			    } else if (baseType.getSchema() != null && OriginalXsdUtil.getSchema(getContext(), listSchemaLocation) != null) {	// this is NOT the schema for the typedElement, rather the child object - newly created schema for list types
				    xsd = (XSDSchema)OriginalXsdUtil.getSchema(getContext(), listSchemaLocation);
			    } else {
			        xsd = OriginalXsdUtil.createSchemaForList(getContext(), originalSchemaNamespace, baseType);
			    }
			    complexType.setTargetNamespace(xsd.getTargetNamespace());
			    DataDefinitionUtil.organizeImports(xsd, baseType);

			    XSDComplexTypeDefinition sameType = (XSDComplexTypeDefinition)sameInSchema(xsd, baseType, minOccurs, maxOccurs);
			    if(sameType != null){
			        complexType = sameType;
			    }
			    
			    if (!xsd.getTypeDefinitions().contains(complexType)) {
					if (baseType.getName() == null && baseType.getContainer() instanceof XSDElementDeclaration) {
					    // anonymous type
						decl.setResolvedElementDeclaration((XSDElementDeclaration)baseType.getContainer());
					}
				    xsd.getContents().add(complexType);
			    }
			    
			    getTarget().add(complexType);	
			    parentRule.getTarget().add(xsd);	//add xsd to the parent rule (DataDefinitionRuleImpl)
			    TransformationRule dRule = TransformationUtil.getRuleForSource(getRoot(), dataType, SolutionArtifact.class);	// add xsd to DataDefinitionRule so SolutionArtifact can be created
			    if (dRule != null && dRule instanceof com.ibm.btools.te.ilm.heuristics.DataDefinitionRule && !dRule.getTarget().contains(xsd)) {
			    	dRule.getTarget().add(xsd);
			    }
			    executeHandlers();
			    setComplete(true);
			}
		}		
		LoggingUtil.traceExit(this, "transformSource2Target");		
		return isComplete();
	
	}
//		/**
//		 * If a XSDTypeDefinition similar to the <code>xsdDefinition</code> is found in the <code>xsdSchema</code>,
//		 * returns the typeDefinition in the schema.
//		 * @param xsdSchema the XSDSchema this type should be attached to
//		 * @param xsdDefinition the xsdDefinition to check
//		 * @return the similar typeDefinition in the schema or the input one, if none found in the xsdSchema.
//		 */
//		private XSDTypeDefinition sameInSchema(XSDSchema xsdSchema, XSDTypeDefinition xsdDefinition){
//			if(xsdSchema.getTypeDefinitions()!= null){
//				Iterator it = xsdSchema.getTypeDefinitions().iterator();
//				XSDTypeDefinition thisDefinition;
//				while(it.hasNext()){
//					thisDefinition = (XSDTypeDefinition)it.next();
//					if(thisDefinition.getName().equals(xsdDefinition.getName())){
//						return thisDefinition;
//					}
//				}
//			}
//			return null;	
//		}
	/**
	 * If a XSDTypeDefinition similar to the <code>xsdDefinition</code> is found in the <code>xsdSchema</code>,
	 * returns the typeDefinition in the schema.
	 * @param xsdSchema the XSDSchema this type should be attached to
	 * @param xsdDefinition the xsdDefinition to check
	 * @return the similar typeDefinition in the schema or the input one, if none found in the xsdSchema.
	 */	
	private XSDTypeDefinition sameInSchema(XSDSchema xsdSchema, XSDTypeDefinition baseType, int minOccurs, int maxOccurs){
			if(xsdSchema.getTypeDefinitions()!= null){
				Iterator it = xsdSchema.getTypeDefinitions().iterator();
				XSDTypeDefinition thisDefinition;
				while(it.hasNext()){
					thisDefinition = (XSDTypeDefinition)it.next();
					//this can be the definition of a list of "anyType" in which case the complexType is null
					if(thisDefinition instanceof XSDComplexTypeDefinition &&  thisDefinition.getComplexType()!= null && thisDefinition.getComplexType().getContent() != null){
						XSDParticleContent particle = thisDefinition.getComplexType().getContent();
						if(particle  instanceof XSDModelGroup){
							XSDModelGroup modelGroup = (XSDModelGroup)particle;
							if(modelGroup.getContents() != null && !modelGroup.getContents().isEmpty()){
								XSDParticle innerParticle = (XSDParticle)modelGroup.getContents().get(0);
								//there should only be one content declaration of the simple or context type in the list
								if (innerParticle.getContent() instanceof XSDElementDeclaration){
									XSDElementDeclaration elementDeclaration = (XSDElementDeclaration)innerParticle.getContent();
									if(elementDeclaration.getTypeDefinition()== baseType ||
											(elementDeclaration.getTypeDefinition().getName() != null &&
													baseType.getName() != null &&
													elementDeclaration.getTypeDefinition().getName().equals(baseType.getName()) &&
													elementDeclaration.getTypeDefinition().getTargetNamespace() != null &&
													elementDeclaration.getTypeDefinition().getTargetNamespace().equals(baseType.getTargetNamespace()))){
										if(innerParticle.getMaxOccurs() == maxOccurs && innerParticle.getMinOccurs() == minOccurs){
											return thisDefinition;
										}
									}
								}
							}
						}
					} else if(thisDefinition instanceof XSDComplexTypeDefinition && thisDefinition.getBaseType() != null &&
							thisDefinition.getBaseType().getName() != null && thisDefinition.getBaseType().getName().equals("anyType")) {
						
						XSDComplexTypeDefinition xsdCType = (XSDComplexTypeDefinition)thisDefinition;
					
						if (xsdCType.getContent() instanceof XSDParticle && ((XSDParticle)xsdCType.getContent()).getTerm() instanceof XSDModelGroup) {
							XSDModelGroup modelGroup = (XSDModelGroup)((XSDParticle)xsdCType.getContent()).getTerm();
							if(modelGroup.getContents() != null && !modelGroup.getContents().isEmpty()){
								XSDParticle innerParticle = (XSDParticle)modelGroup.getContents().get(0);
								//there should only be one content declaration of the simple or context type in the list
								if (innerParticle.getContent() instanceof XSDElementDeclaration){
									XSDElementDeclaration elementDeclaration = (XSDElementDeclaration)innerParticle.getContent();
									if(elementDeclaration.getTypeDefinition()== baseType ||
											(elementDeclaration.getTypeDefinition().getName() != null &&
													baseType.getName() != null &&
													elementDeclaration.getTypeDefinition().getName().equals(baseType.getName()) &&
													elementDeclaration.getTypeDefinition().getTargetNamespace() != null &&
													elementDeclaration.getTypeDefinition().getTargetNamespace().equals(baseType.getTargetNamespace()))){
										if(innerParticle.getMaxOccurs() == maxOccurs && innerParticle.getMinOccurs() == minOccurs){
											return thisDefinition;
										}
									}
								}
							}
						}
					}
				}
			}
			return null;	
		}

} //ListRuleImpl
