package com.ibm.btools.te.ilm.heuristics.xsd.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.emf.common.util.URI;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.Types;
import org.eclipse.wst.wsdl.WSDLFactory;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDInclude;
import org.eclipse.xsd.XSDNamedComponent;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.util.XSDConstants;
import org.eclipse.xsd.util.XSDUtil;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.blm.ie.exprt.option.WpsExportOptionsBean;
import com.ibm.btools.bom.model.artifacts.ArtifactsFactory;
import com.ibm.btools.bom.model.artifacts.LiteralInteger;
import com.ibm.btools.bom.model.artifacts.LiteralUnlimitedNatural;
import com.ibm.btools.bom.model.artifacts.MultiplicityElement;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.framework.TransformationRoot;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.heuristics.xsd.DataDefinitionRule;
import com.ibm.btools.te.ilm.heuristics.xsd.XsdFactory;
import com.ibm.btools.util.datatype.BTDataTypeManager;

/**
* This class contains static methods that are used in XSD transformation.
*/
public class DataDefinitionUtil {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	public static final String DEFAULT_PREFIX = "http://";
	public static final String XSD_CONTEXT_KEY = "xsdContext";
	public static final String XSD_ANY_TYPE="anyType";
	private static final String DEFAULT_INFO_ROOT_PACKAGE = "RootInformationModel";
	public static final String XSD_DATA_KEY = "DataDefInContext";
	
	private static PrimitiveType primitiveAnyType;

	/**
	 * Returns the namespace prefix for the input namespace. 
	 * @param namespace string 
	 * @return namespace prefix
	 */
	public static String getNamespacePrefix(String namespace){
		if(namespace != null || namespace.length() == 0){
		    String ns = null;
		    namespace = cleanEncodePercentSign(namespace);
		    if(namespace.lastIndexOf("/") > 0){
		        ns = namespace.substring(namespace.lastIndexOf("/")+1);
			}else{
				ns = namespace.substring(namespace.lastIndexOf(":")+1);
			}
			if (ns.endsWith(".xsd")) {
				ns = ns.substring(0, ns.length() - 4);
			} else if (ns.endsWith(".wsdl")) {
				ns = ns.substring(0, ns.length() - 5);
			}
		    ns = ns.replace('$', '_');	//replace all "$" with "_"
		    ns = ns.replace(':', '_');	//replace all "$" with "_"
		    if (ns != null && ns.length() > 0 && Character.isDigit(ns.charAt(0))) {
	    		return "n_" + ns;
		    } else if(ns != null && ns.length() == 0){
		    	return "n_" + String.valueOf(namespace.hashCode());
		    } else if (ns != null && ns.equalsIgnoreCase("xsd") && namespace != null && !namespace.equalsIgnoreCase(ExportOperation.PROCESS_SCHEMA_IMPORT)) {	// d139905, eg. ns = "xsd"
		    	return "n_" + String.valueOf(namespace.hashCode());
		    } else if (ns != null && ns.lastIndexOf(".") != -1) {	// d139905 e.g. ns = "www.ibm.com"
		    	return "n_" + String.valueOf(namespace.hashCode());
		    }
		    //defect 86400. It was observed that the ns will be the same if the last segment collide, hence, add
		    //the hash to make it unique
		    int hash = namespace.hashCode();
		    return ns+hash;
		}
		return null;	
		
	}

	/**
	 * Returns the DataDefinition rule which is the context of the xsd simple type definition or null if none
	 * was yet created
	 * @param rule a Transformation rule 
	 * @return DataDefinitionRule - the root rule responsible for holding the xsd simple type definition. 
	 */
		public static DataDefinitionRule getXSDRulesContext(TransformationRule rule){
		
		TransformationRule parentRule = null;
		rule = rule.getParentRule();
		while(rule != null && !(rule instanceof DataDefinitionRule)){
				rule = rule.getParentRule();
		}
		//if the top level rule is a DataDefinitionRule, this is the rule we are looking for
		if(rule instanceof DataDefinitionRule){
			return (DataDefinitionRule)rule;
		//if the rule is a TransformationRoot, look for the DataDefinitionRule in the context	
		}else if(rule instanceof TransformationRoot){
			return (DataDefinitionRule)((TransformationRoot)rule).getContext().get(XSD_CONTEXT_KEY);
		}else{
			return null;
		}
	}
	/**
	 * Returns the DataDefinitionRule responsible for holding the xsd simple type definition. 
	 * Lookup the rule in the TransformationRoot context, create a new one, if none find and save in 
	 * the transformation context. 
	 * @param rule TransformationRule
	 * @return DataDefinitionRule - the root rule responsible for holding the xsd simple type definition.
	 */
	private static DataDefinitionRule getXSDContext(TransformationRule rule){
		
		DataDefinitionRule xsdContext = DataDefinitionUtil.getXSDRulesContext(rule);
			if(xsdContext == null){
				xsdContext = XsdFactory.eINSTANCE.createDataDefinitionRule();
				//add now the context to the transformation context
				rule.getRoot().getContext().put(DataDefinitionUtil.XSD_CONTEXT_KEY, xsdContext);
			
			}
			return xsdContext;
	}
		
		
	/**
	 * Returns true if the <code>type</code> is one of the predefined types that are supplied by the tool.
	 * @param type BOM Type
	 * @return boolean - true if the <code>type</code> is a predefined simple type
	 */	
	public static boolean isPrimitiveType(Type type){
		if(type == null){
			return true;
		}

		if(type instanceof PrimitiveType && primitiveType2XsdSimpleTypeMap.keySet().contains(type.getName())){
			return true;
		}
		return false;
	}
	
	
	/**
	* Returns true if the <code>type</code> has specified a multiplicity different than 1. 
	* @param type a BOM MultiplicityType
	* @return boolean if the type has multiplicity specified.
	*/		
	public static boolean isMultipled(MultiplicityElement type){
		//TODO check LiteralUnlimitedNatural -> change to BTUnlimitedNatural
		ValueSpecification value = type.getUpperBound();
			if(value instanceof LiteralUnlimitedNatural ){
				if(value == null || ((LiteralUnlimitedNatural)value).getValue()!= null && ((LiteralUnlimitedNatural)value).getValue().equals("1")){
					return false;	
				}else{
					return true;
				}
				
			}else if (value instanceof LiteralInteger){
					if(value == null ||((LiteralInteger)value).getValue()!= null && ((LiteralInteger)value).getValue().intValue() == 1){
						return false;		
					}else{
						return true;
					}
				
			}
		ValueSpecification lowerBound = type.getLowerBound();
			if(lowerBound instanceof LiteralInteger){
				if(lowerBound == null || ((LiteralInteger)lowerBound).getValue()!= null && (((LiteralInteger)lowerBound).getValue().intValue() == 1 || ((LiteralInteger)lowerBound).getValue().intValue() == 0)){
					return false;
				}else{
					return true;
				}
			}
			return false;
	}
	
	
	/**
	 * Predefined map that maps the name of the primitiveTypes (as defined by the tool) to the xsd type
	 */
	private static Map primitiveType2XsdSimpleTypeMap = new HashMap();

	static {
		String [] registeredDataTypes = BTDataTypeManager.instance.getRegisteredDataTypes();
		for(int i=0; i< registeredDataTypes.length; i++){
			String typeName = registeredDataTypes[i];
			if(BTDataTypeManager.instance.getXSDType(typeName) != null)
			//	primitiveType2XsdSimpleTypeMap.put(DEFAULT_INFO_ROOT_PACKAGE + "/" + typeName, BTDataTypeManager.instance.getXSDType(typeName));
			primitiveType2XsdSimpleTypeMap.put(typeName, BTDataTypeManager.instance.getXSDType(typeName));
		}
		
		primitiveType2XsdSimpleTypeMap.put(XSD_ANY_TYPE, XSD_ANY_TYPE);
		
	}
	/**
	 * Returns the xsd type name for the <code>primitiveTypeName</code>.  
	 * @param primitiveTypeName the name of the primitive 
	 * @return xsdTypeName - xsd type name as contained in the <code>primitiveType2XSDSimpleTypeMap</code>. 
	 */
	public static String primitiveType2xsdSimpleType(String primitiveTypeName) {
		
		String xsdTypeName = (String) primitiveType2XsdSimpleTypeMap.get(primitiveTypeName);
		
		if (xsdTypeName == null){
			return primitiveTypeName;
		}
		return xsdTypeName;
	}
	/**
	 * Returns the <code>value</code> starting with capital letter.
	 * @param value
	 * @return - string
	 */
	public static String capitalizeFirstChar(String value){
		return value.substring(0, 1).toUpperCase() + value.substring(1); 
	}


	public static int convertToXSDMultiplicity(ValueSpecification valSpec) {
		if(valSpec != null){
			if(valSpec instanceof LiteralUnlimitedNatural){
				String value = ((LiteralUnlimitedNatural)valSpec).getValue();
				if(value.equals("n") || value.equals("N") || value.equals("*")){
					return -1;
				}else{
					return Integer.parseInt(value);
				}
			}else if (valSpec instanceof LiteralInteger){
				Integer value = ((LiteralInteger)valSpec).getValue();
				return value.intValue();
			}else{
				//else this is probably an opaque expression we can not handle, default it to unlimited
				return -1;
			}
		}
		return 1;
	}
	
	public static PrimitiveType getAnyType(){
		if(primitiveAnyType == null){
			primitiveAnyType = ArtifactsFactory.eINSTANCE.createPrimitiveType();
			primitiveAnyType.setName(XSD_ANY_TYPE);
		}
		return primitiveAnyType;
	}
	/**
		 * Returns the DataDefinitionRule, root of the XSDTransformation for the namespace of the <code>namedElement</code>
		 * under the <code>rule</code> root.
		 * If the type of the <code>typedElement</code> is a primitiveType and it does not have a multiplicity specified,
		 * the DataDefinitionRule is not attached to the <code>rule</code> root , rather it is saved in the transformation context. 
		 * @param rule 
		 * @param typedElement a BOM TypedElement, also a BOM MultiplicityElement
		 * @return DataDefinitionRule - DataDefinitionRule, root of the XSD trasnsformation for the <code>typedElement</code> namespace
		 */
	public static DataDefinitionRule getDataDefinitionRule(TransformationRule rule, NamedElement namedElement){
		
		TransformationContext context = null;
		if(rule instanceof TransformationRoot){
			context = ((TransformationRoot)rule).getContext();
		}else{
			context = rule.getRoot().getContext();
		}
		String namespace = null;
		DataDefinitionRule definitionRule = null;
		
		String namespaceSuffix = null;
			
		if(namedElement instanceof TypedElement){
				
			Type dataType = ((TypedElement)namedElement).getType();
			if(dataType == null || (isPrimitiveType(dataType)&& (namedElement instanceof MultiplicityElement) && !isMultipled((MultiplicityElement)namedElement))){
				definitionRule = DataDefinitionUtil.getXSDContext(rule); 

			}else{
				namespaceSuffix = XSD_DATA_KEY;
				// Ray Fix for Share Across Project - we need to register the DataDefinitionRule in the context using a special namespace that includes the uid of the package 
				// representing the XSDSchema to avoid conflict because the namespace generated by names of the package structure no longer garanteed to be unqiue across projects
				//namespace = getDataDefinitionNamespace((NamedElement)dataType);
				//AZ roll back because of introduction of project name into the target namespace to make it unique
				namespace = ProcessUtil.createNamespace((NamedElement)dataType);
				definitionRule = (DataDefinitionRule)rule.getRoot().getContext().get(namespace + namespaceSuffix);
			}
		//this is the case when the namedElement is an Action (Task, ReusableTask). We will create a dataDefinitionRule and save it under the 
		//a different key than the namespace to avoid possible collision 	
		}else{
//			AZ roll back because of introduction of project name into the target namespace to make it unique
			namespace = ProcessUtil.createNamespace(namedElement);
			namespaceSuffix = XSD_DATA_KEY; 
			// Ray Fix for Share Across Project - we need to register the DataDefinitionRule in the context using a special namespace that includes the uid of the package 
			// representing the XSDSchema to avoid conflict because the namespace generated by names of the package structure no longer garanteed to be unqiue across projects
			//namespace = getDataDefinitionNamespace(namedElement);
			//is there is already a DataDefinitionRule defined for this namespace?
			definitionRule =(DataDefinitionRule) context.get(namespace + namespaceSuffix);
//			if(rule instanceof TransformationRoot){
//				definitionRule = (DataDefinitionRule)((TransformationRoot)rule).getContext().get(namespace + namespaceSuffix);
//			}else{
//				definitionRule = (DataDefinitionRule)rule.getRoot().getContext().get(namespace + namespaceSuffix);
//			}
		}
		if(definitionRule == null){
			definitionRule = XsdFactory.eINSTANCE.createDataDefinitionRule();
			//save it in the context
			if(namespaceSuffix != null){
				//rule.getRoot().getContext().put(namespace + namespaceSuffix, definitionRule);
				context.put(namespace + namespaceSuffix, definitionRule);
			}else{
				context.put(namespace, definitionRule);
			//	rule.getRoot().getContext().put(namespace, definitionRule);
			}
			if(rule instanceof TransformationRoot){
				rule.getChildRules().add(definitionRule);
			}else{
				rule.getRoot().getChildRules().add(definitionRule);
			}
			// the source of this rule is the first dataType that will create it.
			if (namedElement instanceof TypedElement) {
				TypedElement typedElement = (TypedElement)namedElement;
				definitionRule.getSource().add(typedElement.getType());				
			} else {
				definitionRule.getSource().add(namedElement);
			}
			definitionRule.transformSource2Target();
			
		}
			return definitionRule;
		}

	/**
	 * Ray Fix for Share Across Project - we need to register the DataDefinitionRule in the context using a special namespace that includes the uid of the package 
	 * representing the XSDSchema to avoid conflict because the namespace generated by names of the package structure no longer garanteed to be unqiue across projects
	 * @param namedElement
	 * @return
	 */
	public static String getDataDefinitionNamespace(NamedElement namedElement) {
		String key=ProcessUtil.createNamespace(namedElement);
		ExportSession exportSession = BpelOptionsUtil.getExportSession();
		WpsExportOptionsBean wpsOptions = BpelOptions.getWpsOptions(exportSession);
		// In the case when we export to pure file system (i.e. no WID project will be created), then we don't
		// need to create the special key.  That means for export to file system, there will be a single xsd created per namespace 
//		if (wpsOptions.getExportOption()!=WpsExportOptionsBean.NONE) {
			if (namedElement instanceof Type) {
				key = key + ((Type)namedElement).getOwningPackage().getUid();
			} else if (namedElement instanceof Package) {
				key = key + namedElement.getUid();
			}			
//		}
		return key;
	}
	
	/**
	 * Checks if an XSDImport for the namespace of the <code>typeDefinition</code> already exists in 
	 * the <code>definition</code>. If not, it will add it. Registers the prefix with the namespace
	 * in the <code>definition</code>.
	 * @param definition the WSDL Definition 
	 * @param typeDefinition an XSDTypeDefinition to check in the WSDL Definition namespace.
	 */

	public static void organizeTypesImports(Definition definition, XSDNamedComponent typeDefinition){
		
		Types types = definition.getETypes();
		if(types == null){
			types = WSDLFactory.eINSTANCE.createTypes();
			definition.setTypes(types);
		}
		String namespace = typeDefinition.getTargetNamespace();
		List extElements = types.getSchemas();
		if(extElements.isEmpty()){	
			XSDImport xsdImport = XSDFactory.eINSTANCE.createXSDImport();
			if (typeDefinition.getSchema().getSchemaLocation() != null && 
					typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
				xsdImport.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
			} else {
				xsdImport.setSchemaLocation(ProcessUtil.createFileName(namespace)+ExportOperation.XSD_FILE_EXT);
			}
			xsdImport.setNamespace(namespace);

			XSDSchema xsdSchema = XSDFactory.eINSTANCE.createXSDSchema();
			xsdSchema.setSchemaForSchemaQNamePrefix("xsd");
			xsdSchema.getContents().add(0, xsdImport);
		
			Map namespaces = xsdSchema.getQNamePrefixToNamespaceMap();
			namespaces.put(xsdSchema.getSchemaForSchemaQNamePrefix(), XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
			namespaces.put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);
			types.getSchemas().add(xsdSchema);
		}else{
			Iterator it = extElements.iterator();
			boolean imported = false;
			while(it.hasNext()){
				Object thisElement = it.next();
				if(thisElement instanceof XSDSchema){
					if(((XSDSchema)thisElement).getQNamePrefixToNamespaceMap().get(DataDefinitionUtil.getNamespacePrefix(namespace)) != null){
						imported = true;
						break;
					}
				}
			}
			if(!imported){
				XSDImport xsdImport = XSDFactory.eINSTANCE.createXSDImport();
				if (typeDefinition.getSchema().getSchemaLocation() != null && 
						typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
					xsdImport.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
				} else {
					xsdImport.setSchemaLocation(ProcessUtil.createFileName(namespace)+ExportOperation.XSD_FILE_EXT);
				}
				xsdImport.setNamespace(namespace);
				

				XSDSchema xsdSchema = XSDFactory.eINSTANCE.createXSDSchema();
				xsdSchema.setSchemaForSchemaQNamePrefix("xsd");
				xsdSchema.getContents().add(0, xsdImport);

				Map namespaces = xsdSchema.getQNamePrefixToNamespaceMap();
				namespaces.put(xsdSchema.getSchemaForSchemaQNamePrefix(), XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
				namespaces.put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);

				types.getSchemas().add(xsdSchema);
			}
		}
	}
	
	/**
	 * If the <code>typeDefinition</code> is defined in another XSDSchema, add the import for that schema
	 * (if not already added) to this <code>schema</code>.
	 * @param schema - XSDSchema 
	 * @param typeDefinition - XSDTypeDefiniton 
	 */
	public static  void organizeImports(XSDSchema schema, XSDNamedComponent typeDefinition){
		if(typeDefinition != null && schema != null){
		    XSDSchema importedSchema = typeDefinition.getSchema();
		    if (importedSchema.getSchemaLocation() != null && importedSchema.getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) > -1) {
				String namespace = typeDefinition.getTargetNamespace();
				if (namespace == null && schema.getTargetNamespace() == null) {
					//check if there already is an include for this namespace
					if(!OriginalXsdUtil.hasIncludeForFile(schema, importedSchema.getSchemaLocation())) {
						XSDInclude include = XSDFactory.eINSTANCE.createXSDInclude();
						include.setSchemaLocation(importedSchema.getSchemaLocation());
						schema.getContents().add(0,include);
					}
				} else if (namespace == null) {
					//check if there already is an import for this namespace
					if(!OriginalXsdUtil.hasImportForFile(schema, importedSchema.getSchemaLocation())) {
						XSDImport imp = XSDFactory.eINSTANCE.createXSDImport();
						imp.setSchemaLocation(importedSchema.getSchemaLocation());
						schema.getContents().add(0,imp);
					}
				} else if(!namespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) && !namespace.equals(schema.getTargetNamespace())){
					XSDTypeDefinition resolved = schema.resolveTypeDefinition(namespace, typeDefinition.getName());
					if(resolved != typeDefinition) {
						// need to add namespaces etc
						//check if there already is an import for this namespace
						if(schema.getQNamePrefixToNamespaceMap().get(DataDefinitionUtil.getNamespacePrefix(namespace)) == null ||
								!OriginalXsdUtil.hasImportForFileName(schema, importedSchema.getSchemaLocation())){
							XSDImport imp = XSDFactory.eINSTANCE.createXSDImport();
							imp.setNamespace(namespace);
							imp.setSchemaLocation(importedSchema.getSchemaLocation());
							schema.getContents().add(0,imp);
							schema.getQNamePrefixToNamespaceMap().put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);
						}
					}
				}		        
		    } else { 	// original code
				String namespace = typeDefinition.getTargetNamespace();
				if(!namespace.equals(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001) && !namespace.equals(schema.getTargetNamespace())){
					XSDTypeDefinition resolved = schema.resolveTypeDefinition(namespace, typeDefinition.getName());
					if(resolved != typeDefinition) {
						// need to add namespaces etc
						//check if there already is an import for this namespace
						if(schema.getQNamePrefixToNamespaceMap().get(DataDefinitionUtil.getNamespacePrefix(namespace)) == null){
							XSDImport imp = XSDFactory.eINSTANCE.createXSDImport();
							imp.setNamespace(namespace);
							if (typeDefinition.getSchema().getSchemaLocation() != null && 
									typeDefinition.getSchema().getSchemaLocation().indexOf(ExportOperation.ROOT_EXTERNAL_MODEL) != -1) {
								imp.setSchemaLocation(typeDefinition.getSchema().getSchemaLocation());							
							} else {
								imp.setSchemaLocation(ProcessUtil.createFileName(namespace)+".xsd");
							}
							schema.getContents().add(0,imp);
							schema.getQNamePrefixToNamespaceMap().put(DataDefinitionUtil.getNamespacePrefix(namespace), namespace);
						}
					}
				}
			}
		}
	}
	
	
	public static String getPrefixFromNamespaceURI(String namespace){
		
		ArrayList prefixes = new ArrayList();
		for(StringTokenizer stringTokenizer = new StringTokenizer(namespace, ":/"); stringTokenizer.hasMoreElements(); prefixes.add(stringTokenizer.nextToken()));
		if(!prefixes.isEmpty()){
			URI uri = URI.createURI(namespace);
			if(prefixes.get(0).equals(uri.scheme())){
				prefixes.remove(0);
			}
		}
		if(prefixes.isEmpty()){
			return "ns";
		}else{
			String prefix = (String)prefixes.get(prefixes.size()-1);
			prefix = cleanEncodePercentSign(prefix);	
			return prefix;
		}
	}
	
	/**
	 * Returns true is the given simpleType is a XSD Buil-in Type.
	 * @param simpleType a XSDSimpleTypeDefinition
	 * @return
	 */	
	public static boolean isBuildInPrimitiveType(XSDTypeDefinition simpleType){
		if(simpleType instanceof XSDSimpleTypeDefinition){
			if(xsdSimpleType2primitiveTypeMap.containsKey(simpleType.getName()) || xsdSimpleTypeRestricted2primitiveTypeMap.containsKey(simpleType.getName())){
					return true;
			}	
		}
		
		return false;
	}
	
	
	private static Map xsdSimpleType2primitiveTypeMap = new HashMap();

	static {
		xsdSimpleType2primitiveTypeMap.put("string", "String");
		xsdSimpleType2primitiveTypeMap.put("byte", "Byte");
		xsdSimpleType2primitiveTypeMap.put("integer", "Long");
		xsdSimpleType2primitiveTypeMap.put("int", "Integer");
		xsdSimpleType2primitiveTypeMap.put("long", "Long");
		xsdSimpleType2primitiveTypeMap.put("short", "Short");
		xsdSimpleType2primitiveTypeMap.put("decimal", "Float");
		xsdSimpleType2primitiveTypeMap.put("float", "Float");
		xsdSimpleType2primitiveTypeMap.put("double", "Double");
		xsdSimpleType2primitiveTypeMap.put("boolean", "Boolean");
		xsdSimpleType2primitiveTypeMap.put("time", "Time");
		xsdSimpleType2primitiveTypeMap.put("dateTime", "DateTime");
		xsdSimpleType2primitiveTypeMap.put("duration", "Duration");
		xsdSimpleType2primitiveTypeMap.put("date", "Date");
		
	}

	private static Map xsdSimpleTypeRestricted2primitiveTypeMap = new HashMap();

	static {
		xsdSimpleTypeRestricted2primitiveTypeMap.put("normalizedString", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("token", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("unsignedByte", "Byte");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("base64Binary", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("hexBinary", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("positiveInteger", "Long");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("nonPositiveInteger", "Long");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("negativeInteger", "Long");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("nonNegativeInteger", "Long");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("unsignedInt", "Integer");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("unsignedLong", "Long");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("unsignedShort", "Short");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("gMonth", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("gYear", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("gYearMonth", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("gDay", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("gMonthDay", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("Name", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("QName", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("NCName", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("anyURI", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("language", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("ID", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("IDREF", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("IDREFS", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("ENTITY", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("ENTITIES", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("NOTATION", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("NMTOKEN", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("NMTOKENS", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("anyType", "String");
		xsdSimpleTypeRestricted2primitiveTypeMap.put("anySimpleType", "String");
			
	}
	
	/**
	 * @param definition
	 * @return
	 */
	public static XSDTypeDefinition getBaseTypeDefinition(XSDSimpleTypeDefinition definition) {
		XSDSimpleTypeDefinition baseDefinition = definition.getBaseTypeDefinition();
		while(!isBuildInPrimitiveType(baseDefinition)){
			baseDefinition = baseDefinition.getBaseTypeDefinition();
			
		}
		return baseDefinition;
	}


	/**
	 * Find the XSD type for a built in BOM Type. 
	 * @param dataType Type
	 * @return xsd simple type definition
	 */
	public static XSDTypeDefinition getSimpleXSDType(Type dataType) {
		if (dataType == null || !isPrimitiveType(dataType)) {
			return null;
		}
		
		String dataTypeName = dataType.getName();
		
		// look for a pre-defined, atomic type first (they're not in packages or schemas)
		String xsdTypeName = DataDefinitionUtil.primitiveType2xsdSimpleType(dataTypeName);
		if (xsdTypeName != null) {
			if(xsdTypeName.equals(DataDefinitionUtil.XSD_ANY_TYPE)){
				return XSDUtil.getSchemaForSchema(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001).resolveTypeDefinition(DataDefinitionUtil.XSD_ANY_TYPE);
			}else{
				return XSDUtil.getSchemaForSchema(XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001).resolveSimpleTypeDefinition(xsdTypeName);
			}
			
			
		}
		return null;
	}
	
	/**
	 * This utility applies to the method when creating prefix from namespace
	 * @param namespace
	 * @return
	 */
	private static String cleanEncodePercentSign(String namespace){		
		 if(namespace != null && namespace.indexOf("%") != -1){
		    	StringTokenizer st = new StringTokenizer(namespace, new String("%"));
		    	StringBuffer sb = new StringBuffer();
		    	while(st.hasMoreTokens()){
		    		sb.append(st.nextToken());
		    	}
		    	return sb.toString();	
		    }
		return namespace;
	}
}
