package com.ibm.btools.te.ilm.heuristics.xsd.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.XSDSchemaExtensibilityElement;
import org.eclipse.wst.wsdl.internal.impl.DefinitionImpl;
import org.eclipse.wst.wsdl.util.WSDLResourceImpl;
import org.eclipse.xsd.XSDAttributeDeclaration;
import org.eclipse.xsd.XSDAttributeGroupDefinition;
import org.eclipse.xsd.XSDAttributeUse;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDComponent;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDFactory;
import org.eclipse.xsd.XSDImport;
import org.eclipse.xsd.XSDInclude;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDModelGroupDefinition;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;
import org.eclipse.xsd.XSDRedefine;
import org.eclipse.xsd.XSDSchema;
import org.eclipse.xsd.XSDSimpleTypeDefinition;
import org.eclipse.xsd.XSDTypeDefinition;
import org.eclipse.xsd.impl.XSDAttributeDeclarationImpl;
import org.eclipse.xsd.impl.XSDComplexTypeDefinitionImpl;
import org.eclipse.xsd.impl.XSDElementDeclarationImpl;
import org.eclipse.xsd.impl.XSDSchemaImpl;
import org.eclipse.xsd.impl.XSDSimpleTypeDefinitionImpl;
import org.eclipse.xsd.util.XSDConstants;
import org.eclipse.xsd.util.XSDResourceImpl;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;
import com.ibm.btools.bom.model.externalmodels.ExternalService;
import com.ibm.btools.model.filemanager.FileMGR;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.framework.TransformationContext;
import com.ibm.btools.te.ilm.ExportOperation;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.LoggingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.TransformationSessionKeyConstants;
import com.ibm.btools.te.ilm.heuristics.naming.NamingUtil;
import com.ibm.btools.te.ilm.resource.MessageKeys;
import com.ibm.btools.te.ilm.resource.TEResourceBundleSingleton;
import com.ibm.btools.util.exception.BTRuntimeException;

/**
* This class contains static methods for reusing the original XSD definitions in transformation.
*/
public class OriginalXsdUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	/**
	 * Original XSD registry should have the following mapping
	 * 
	 * BOM:				XSD:
	 * PinSet			Wrapper ElementDeclaration
	 * Type				Schema/Type (Simple, Complex or Anonymous)
	 * 
	 */
	
	public static String ORIG_XSDS_REGISTRY = "ORIG_XSDS_REGISTRY";
	public static final String SCHEMA_TYPE_ASPECT = "SchemaType";
	public static final String SCHEMA_TYPE_INLINE_ASPECT = "SchemaType_Inline";
	public static final String INLINE_SCHEMA_ASPECT = "inlineData";
	public static final String EXTERNAL_SCHEMA_ASPECT = "data";

	// Note: this method should retrieve the XSD schema that contains the XSD Type represented by the ne.
	// The XSD schema can be in the original XSD file attached or as in-line schema in the original WSDL 
	// file attached.
	/**
	 * Returns the schema from the file name. 
	 * @param context TransformationContext 
	 * @param fileName String 
	 * @param ne NamedElement 
	 * @return schema XSDSchema
	 */
	public static XSDSchema getXSD(String fileName, NamedElement ne) {
	    if (fileName == null || ne == null) {
	        return null;
	    }
	    
		try {
			// Note: file can be XSD or WSDL (in-line types)
			File inputFile = new File(fileName);
			URI fileURI = URI.createFileURI(inputFile.getAbsolutePath());
			final ResourceSet resourceSet = new ResourceSetImpl();
			Resource resource = resourceSet.getResource(fileURI, true);
			if (!resourceSet.getResources().isEmpty()) {
				for (Iterator it = resourceSet.getResources().iterator(); it.hasNext(); ) {
					resource = (Resource)it.next();
					if(resource instanceof WSDLResourceImpl){
					    // Note: there can be a number of schemas, find the correct one
					    WSDLResourceImpl wsdlResource = (WSDLResourceImpl) resource;
					    DefinitionImpl definition = (DefinitionImpl)wsdlResource.getContents().get(0);
					    javax.wsdl.Types types = definition.getTypes();
					    List extensibilityElements = types.getExtensibilityElements();
					    if (extensibilityElements != null) {
					        Iterator iterator = extensibilityElements.iterator();
					        Object obj = null;
					        XSDSchema xsd = null;
					        while (iterator.hasNext()) {
					            obj = iterator.next();
					            if (obj instanceof XSDSchemaExtensibilityElement) {
					                xsd = ((XSDSchemaExtensibilityElement)obj).getSchema();
//					                if (getXSDTypeFromSchema(xsd, ne.getName()) != null || 
//					                        getElementDeclarationFromSchema(xsd, ne.getName()) != null ||
//					                        getAttributeDeclarationFromSchema(xsd, ne.getName()) != null) {
					                if (getXSDComponentFromSchema(xsd, ne.getName()) != null) {
//					            		if (xsd.getTargetNamespace() == null) {
//					            			String params[] = {xsd.getSchemaLocation()};
//					            			LoggingUtil.logError(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION, params, null);
//					            			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION));
//					            			throw runtimeEx;
//					            		}

					                	return xsd;
					                }
					            }
					        }					            
					    }
					} else if (resource instanceof XSDResourceImpl){
						XSDResourceImpl xsdResource = (XSDResourceImpl)resource;
						if (!xsdResource.getContents().isEmpty()) {
							XSDSchema xsdSchema = (XSDSchema)xsdResource.getContents().get(0);
//		            		if (xsdSchema.getTargetNamespace() == null) {
//		            			String params[] = {xsdSchema.getSchemaLocation()};
//		            			LoggingUtil.logError(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION, params, null);
//		            			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION));
//		            			throw runtimeEx;
//		            		}

		                	return xsdSchema;
						}
					}
				}
			}
		} catch (BTRuntimeException btException) {
			throw btException;	// rethrow
		}
		catch (Throwable throwable) {
			//defect 158164 abort export
			// Log error on failure to read file
			LoggingUtil.logError(MessageKeys.UNABLE_READ_FROM_FILE, new String[]{fileName}, throwable);
			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.UNABLE_READ_FROM_FILE));
			throw runtimeEx;
		}
		
		return null;
	}	

	
	/**
	 * Returns the XSD element registry from the context. 
	 * @param context TransformationContext 
	 * @return registry Map
	 */
	public static Map getXsdElementRegistry (TransformationContext context) {
		Map registry = (Map) context.get(ORIG_XSDS_REGISTRY);
		if (registry == null) {
			registry = new HashMap();
			context.put(ORIG_XSDS_REGISTRY,registry);
		}
		
		return registry;
	}
	
	
	/**
	 * Registers the XSD element in the registry. 
	 * @param context TransformationContext 
	 * @param ne NamedElement
	 * @param xsdElement Object
	 * @return void
	 */
	public static void registerXsdElement(TransformationContext context, NamedElement ne, Object xsdElement) {
		Map registry = getXsdElementRegistry(context);
		if (ne instanceof Type) {
			Map reg = (Map)registry.get(ne);
			if (reg == null) {
				reg = new HashMap();
				registry.put(ne, reg);
			}
			reg.put(xsdElement.getClass(), xsdElement);
		} else {
			registry.put(ne,xsdElement);
		}
	}

	
	/**
	 * Returns the schema for the named element. 
	 * @param context TransformationContext 
	 * @param ne NamedElement
	 * @return schema XSDSchema
	 */
	public static XSDSchema getSchema (TransformationContext context, NamedElement ne) {
		Map registry = getXsdElementRegistry(context);
		if (ne instanceof ExternalSchema) {
		    return (XSDSchema)registry.get(ne);
		} 
		Map reg = (Map)registry.get(ne);
		if (reg == null) {
			return null;
		}

		return (XSDSchema)reg.get(XSDSchemaImpl.class);		
	}

	
	/**
	 * Returns the schema registered with the namespace. 
	 * @param context TransformationContext 
	 * @param namespace String
	 * @return schema XSDSchema
	 */
	public static XSDSchema getSchema (TransformationContext context, String namespace) {
		if (namespace == null) {
			return null;
		}
		Map registry = getXsdElementRegistry(context);
		return (XSDSchema)registry.get(namespace);
	}


	/**
	 * Registers the XSD element in the registry. 
	 * @param context TransformationContext 
	 * @param namespace String
	 * @param xsdElement Object
	 * @return void
	 */
	public static void registerXsdElement(TransformationContext context, String namespace, Object xsdElement) {
		if (namespace == null) {
			return;
		}
		Map registry = getXsdElementRegistry(context);
		if (xsdElement instanceof XSDSchema) {
		    registry.put(namespace, xsdElement);
		}
	}

	/**
	 * Registers the XSD element in the registry. 
	 * @param context TransformationContext 
	 * @param namespace String
	 * @param xsdElement Object
	 * @return void
	 */
	public static void registerXsdElementBasedOnLocation(TransformationContext context, String fileLocation, Object xsdElement) {
		if (fileLocation == null) {
			return;
		}
		Map registry = getXsdElementRegistry(context);
		if (xsdElement instanceof XSDSchema) {
		    registry.put(fileLocation, xsdElement);
		}
	}

	/**
	 * Returns the type definition for the named element if defined in XSD registry. 
	 * @param context TransformationContext 
	 * @param ne NamedElement
	 * @return typeDefinition XSDTypeDefinition
	 */
	public static XSDTypeDefinition getXSDTypeFromRegistry (TransformationContext context, NamedElement ne) {
		Map registry = getXsdElementRegistry(context);
		Map reg = (Map)registry.get(ne);
		if (reg == null) {
			return null;
		}
		
		XSDTypeDefinition type = (XSDTypeDefinition)reg.get(XSDComplexTypeDefinitionImpl.class);
		if (type != null) {
		    return type;
		}
		
		return (XSDTypeDefinition)reg.get(XSDSimpleTypeDefinitionImpl.class);
	}


	/**
	 * Returns the element declaration for the named element if defined in XSD registry. 
	 * @param context TransformationContext 
	 * @param ne NamedElement
	 * @return elementDeclaration XSDElementDeclaration
	 */
	public static XSDElementDeclaration getElementDeclarationFromRegistry (TransformationContext context, NamedElement ne) {
		Map registry = getXsdElementRegistry(context);
		Object obj = registry.get(ne);
		
		if (obj instanceof XSDElementDeclaration) {
			return (XSDElementDeclaration)obj;
		} else if (obj instanceof Map) {
		    Map map = (Map) obj;
		    return (XSDElementDeclaration) map.get(XSDElementDeclarationImpl.class);
		}
		
		return null;
	}
	
	
	/**
	 * Returns the attribute declaration for the named element if defined in XSD registry. 
	 * @param context TransformationContext 
	 * @param ne NamedElement
	 * @return attributeDeclaration XSDAttributeDeclaration
	 */
	public static XSDAttributeDeclaration getAttributeDeclarationFromRegistry (TransformationContext context, NamedElement ne) {
		Map registry = getXsdElementRegistry(context);
		Object obj = registry.get(ne);
		
		if (obj instanceof XSDAttributeDeclaration) {
			return (XSDAttributeDeclaration)obj;
		} else if (obj instanceof Map) {
		    Map map = (Map) obj;
		    return (XSDAttributeDeclaration) map.get(XSDAttributeDeclarationImpl.class);
		}
		
		return null;
	}
	
	
	/**
	 * Finds if the component is defined in this schema. 
	 * @param schema XSDSchema 
	 * @param elementName String
	 * @return content Object
	 */	
	public static Object getXSDComponentFromSchema (XSDSchema schema, String elementName) {	    
	    if (schema == null || elementName == null || schema.getContents() == null) {
	        return null;
	    }

	    List contents = schema.getContents();
	    
	    Object component = null;
		for (Iterator it = contents.iterator(); it.hasNext(); ) {
			component = it.next();
			if (component instanceof XSDTypeDefinition  &&
					((XSDTypeDefinition)component).getName().equals(elementName)) {
				return component;
			} else if (component instanceof XSDElementDeclaration  &&
					((XSDElementDeclaration)component).getName().equals(elementName)) {
				return component;
			} else if (component instanceof XSDAttributeDeclaration  &&
					((XSDAttributeDeclaration)component).getName().equals(elementName)) {
				return component;
			} else if (component instanceof XSDRedefine) {
				XSDRedefine xsdRedefine = (XSDRedefine) component;
				if (xsdRedefine.getContents() == null || xsdRedefine.getContents().isEmpty()) {
					continue;
				}
				Iterator redefineContentsIt = xsdRedefine.getContents().iterator();
				Object redefinedContent = null;
				while (redefineContentsIt.hasNext()) {
					redefinedContent = redefineContentsIt.next();
					if (redefinedContent instanceof XSDTypeDefinition  &&
							((XSDTypeDefinition)redefinedContent).getName().equals(elementName)) {
						return redefinedContent;
					}
				}
			} 
		}
		
		return null;
	}
	
	
	/**
	 * Returns the type definition for the element name if defined in XSD schema. 
	 * @param schema XSDSchema 
	 * @param elementName String
	 * @return typeDefinition XSDTypeDefinition
	 */	
	public static XSDTypeDefinition getXSDTypeFromSchema (XSDSchema schema, String elementName) {	    
	    if (schema == null || elementName == null || schema.getContents() == null) {
	        return null;
	    }

	    List contents = schema.getContents();
	    
		Object element = null;
		for (Iterator it = contents.iterator(); it.hasNext(); ) {
			element = it.next();
			if (element instanceof XSDTypeDefinition &&
					((XSDTypeDefinition)element).getName().equals(elementName)) {
				return (XSDTypeDefinition)element;
			} else if (element instanceof XSDRedefine) {
				XSDRedefine xsdRedefine = (XSDRedefine) element;
				if (xsdRedefine.getContents() == null || xsdRedefine.getContents().isEmpty()) {
					continue;
				}
				Iterator redefineContentsIt = xsdRedefine.getContents().iterator();
				Object redefinedContent = null;
				while (redefineContentsIt.hasNext()) {
					redefinedContent = redefineContentsIt.next();
					if (redefinedContent instanceof XSDTypeDefinition  &&
							((XSDTypeDefinition)redefinedContent).getName().equals(elementName)) {
						return (XSDTypeDefinition)redefinedContent;
					}
				}
			}
		}
		
		return null;
	}
	
	
	// Note: this method will retrieve top level element declarations only and elements declared in groups
	/**
	 * Returns the element declaration for the element name if defined in XSD schema. 
	 * @param schema XSDSchema 
	 * @param elementName String
	 * @return elementDeclaration XSDElementDeclaration
	 */	
	public static XSDElementDeclaration getElementDeclarationFromSchema (XSDSchema schema, String elementName) {
	    if (schema == null || elementName == null || schema.getContents() == null) {
	        return null;
	    }
	    
		XSDElementDeclaration xsdElement = null;
	    
		List contents = schema.getElementDeclarations();
		XSDElementDeclaration xsdElementDeclaration = null;
		
		for (Iterator it = contents.iterator(); it.hasNext() && xsdElement == null; ) {
		    xsdElementDeclaration = (XSDElementDeclaration) it.next();
			if (xsdElementDeclaration.getName() != null &&
			        xsdElementDeclaration.getName().equals(elementName)) {
			    xsdElement = xsdElementDeclaration;
			}
		}
		
		// check the groups next
		if (xsdElement == null) {
			contents = schema.getModelGroupDefinitions();
			XSDModelGroupDefinition xsdModelGroupDefinition = null;
			
			for (Iterator it = contents.iterator(); it.hasNext() && xsdElement == null; ) {
			    xsdModelGroupDefinition = (XSDModelGroupDefinition) it.next();
			    xsdElement = getElementDeclarationFromModelGroup (xsdModelGroupDefinition.getModelGroup(), elementName);
			}
		}
		
		return xsdElement;
	}
	

	/**
	 * Returns the element declaration if defined in the group name in XSD schema. 
	 * @param xsdModelGroup XSDModelGroup 
	 * @param elementName String
	 * @return elementDeclaration XSDElementDeclaration
	 */	
	public static XSDElementDeclaration getElementDeclarationFromModelGroup (XSDModelGroup xsdModelGroup, String elementName) {
	    if (xsdModelGroup == null || elementName == null || xsdModelGroup.getParticles() == null) {
	        return null;
	    }

	    XSDElementDeclaration xsdElement = null;
	    
		List contents = xsdModelGroup.getContents();
		XSDParticle xsdParticle = null;
		XSDElementDeclaration xsdElementDeclaration = null;
		
		for (Iterator it = contents.iterator(); it.hasNext() && xsdElement == null; ) {
		    xsdParticle = (XSDParticle) it.next();
			if (xsdParticle.getContent() instanceof XSDElementDeclaration) {
			    xsdElementDeclaration = (XSDElementDeclaration) xsdParticle.getTerm();
				if (xsdElementDeclaration.getName() != null &&
				        xsdElementDeclaration.getName().equals(elementName)) {
				    xsdElement = xsdElementDeclaration;
				}			    
			} else if (xsdParticle.getContent() instanceof XSDModelGroup) {
			    xsdElement = getElementDeclarationFromModelGroup ((XSDModelGroup) xsdParticle.getContent(), elementName);
			} else if (xsdParticle.getContent() instanceof XSDModelGroupDefinition) {
			    XSDModelGroupDefinition xsdModelGroupDef = (XSDModelGroupDefinition) xsdParticle.getContent();
			    xsdModelGroupDef = xsdModelGroupDef.getResolvedModelGroupDefinition();
			    if (xsdModelGroupDef.getModelGroup() != null) {
			        xsdElement = getElementDeclarationFromModelGroup (xsdModelGroupDef.getModelGroup(), elementName);
			    }
			}		    
		}
		
		return xsdElement;
	}
	

	/**
	 * Returns the attribute declaration for the attribute name if defined in XSD schema. 
	 * @param schema XSDSchema 
	 * @param attributeName String
	 * @return attributeDeclaration XSDAttributeDeclaration
	 */	
	public static XSDAttributeDeclaration getAttributeDeclarationFromSchema (XSDSchema schema, String attributeName) {
	    if (schema == null || attributeName == null || schema.getContents() == null) {
	        return null;
	    }
	    
	    XSDAttributeDeclaration xsdAttribute = null;
	    
		List contents = schema.getAttributeDeclarations();
		XSDAttributeDeclaration xsdAttributeDeclaration = null;
		
		for (Iterator it = contents.iterator(); it.hasNext() && xsdAttribute == null; ) {
		    xsdAttributeDeclaration = (XSDAttributeDeclaration) it.next();
			if (xsdAttributeDeclaration.getName() != null &&
			        xsdAttributeDeclaration.getName().equals(attributeName)) {
				xsdAttribute = xsdAttributeDeclaration;
			}
		}

		if (xsdAttribute == null) {
			// check the Attribute groups next
			contents = schema.getAttributeGroupDefinitions();
			xsdAttributeDeclaration = null;
			XSDAttributeGroupDefinition xsdAttributeGroupDefinition = null;
			Object object = null;
			
			for (Iterator it = contents.iterator(); it.hasNext() && xsdAttribute == null; ) {
			    xsdAttributeGroupDefinition = (XSDAttributeGroupDefinition) it.next();
			    if (xsdAttributeGroupDefinition.getContents() != null) { 
					for (Iterator groupIt = xsdAttributeGroupDefinition.getContents().iterator(); groupIt.hasNext() && xsdAttribute == null; ) {
					    object = groupIt.next();
						if (object instanceof XSDAttributeUse) {  
							xsdAttributeDeclaration = ((XSDAttributeUse) object).getAttributeDeclaration();
							if (xsdAttributeDeclaration.getName() != null &&
							        xsdAttributeDeclaration.getName().equals(attributeName)) {
								xsdAttribute = xsdAttributeDeclaration;
							}
						} else if (object instanceof XSDAttributeGroupDefinition) {
						    return getAttributeDeclarationFromGroupDefinition((XSDAttributeGroupDefinition)object, attributeName);
						}
				    }
				}
			}
		}
		
		return xsdAttribute;
	}
	
	
	/**
	 * Returns the attribute declaration if defined in the attribute group name in XSD schema. 
	 * @param schema XSDSchema 
	 * @param attributeName String
	 * @return attributeDeclaration XSDAttributeDeclaration
	 */	
	public static XSDAttributeDeclaration getAttributeDeclarationFromGroupDefinition (XSDAttributeGroupDefinition xsdAttributeGroupDefinition, String attributeName) {
	    if (xsdAttributeGroupDefinition == null || attributeName == null || xsdAttributeGroupDefinition.getContents() == null) {
	        return null;
	    }
	    
		List contents = xsdAttributeGroupDefinition.getContents();
		Object object = null;
		XSDAttributeDeclaration xsdAttributeDeclaration = null;
		
		for (Iterator it = contents.iterator(); it.hasNext(); ) {
		    object = it.next();
			if (object instanceof XSDAttributeDeclaration) {
			    xsdAttributeDeclaration = (XSDAttributeDeclaration) object;
				if (xsdAttributeDeclaration.getName() != null &&
				        xsdAttributeDeclaration.getName().equals(attributeName)) {
					return xsdAttributeDeclaration;
				}
			} else if (object instanceof XSDAttributeGroupDefinition){
			    return getAttributeDeclarationFromGroupDefinition ((XSDAttributeGroupDefinition) object, attributeName);
			}
		}
		
		return null;
	}
	
	
	/**
	 * Check if the named element is defined in the external model. 
	 * @param ne NamedElement 
	 * @return isExternalType boolean
	 */		
	public static boolean isExternalType(NamedElement ne) {
	    if (ne == null || ne.getAspect() == null) {
	        return false;
	    }
	    
	    if (ne.getAspect().equals(SCHEMA_TYPE_ASPECT) || ne.getAspect().equals(SCHEMA_TYPE_INLINE_ASPECT))
	        return true;
	    
	    return false;
	}

	
	/**
	 * Check if the named element is defined in the registry. 
	 * @param ne NamedElement
	 * @param context TransformationContext 
	 * @return isExternalType boolean
	 */		
	public static boolean isRegisteredAsExternalElement(TransformationContext context, NamedElement ne) {
	    if (ne == null) {
	        return false;
	    }
	    
		Map registry = getXsdElementRegistry(context);
		Object obj = registry.get(ne);
		
		if (obj instanceof XSDElementDeclaration) {
			return true;
		} else if (obj instanceof XSDAttributeDeclaration) {
			return true;
		} else if (obj instanceof Map) {
		    Map reg = (Map) obj;
			if (reg == null) {
				return false;
			}
			
			if (reg.get(XSDComplexTypeDefinitionImpl.class) != null || 
			        reg.get(XSDSimpleTypeDefinitionImpl.class) != null ||
			        reg.get(XSDElementDeclarationImpl.class) != null ||
			        reg.get(XSDAttributeDeclarationImpl.class) != null) {
			    return true;
			}
		}

	    return false;
	}

	
	/**
	 * Get the URI Location for the ExternalDocument. 
	 * @param projectName String
	 * @param element ExternalDocument
	 * @return location String
	 */		
	public static String getLocationForElement(String projectName, NamedElement element) {
	    if (projectName == null || element == null) {
	        return null;
	    }
	    
		String projectPath = FileMGR.getProjectPath(projectName);	
		ResourceMGR resMgr = ResourceMGR.getResourceManger();
		
		String bomRelURI = resMgr.getRelativeURI(element);    
		
		String location = projectPath + 
					File.separator + 
					bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator)); 
		
		return location;
	}
	
	
	/**
	 * Get the path name for the named element. 
	 * @param context TransformationContext
	 * @param element NamedElement
	 * @return location String
	 */		
	public static String getTargetFileName(TransformationContext context, NamedElement element){
		if(context == null || element == null){
			return null;
		}
        // Ray 06/06/07 fix for Share Across Project
        //	String projectName = (String)context.get(TransformationSessionKeyConstants.PROJECT_NAME);		
        String projectName = ResourceMGR.getResourceManger().getProjectName(element);
		String projectPath = FileMGR.getProjectPath(projectName);	
		ResourceMGR resMgr = ResourceMGR.getResourceManger();		
		String bomRelURI = resMgr.getRelativeURI(element);    		
		bomRelURI = bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator));	// path until the bom element directory, e.g. RootExternalModel\External business items\MyBO\MyBO
		String absolutePath = projectPath + 
							File.separator + 
							bomRelURI.substring(0,bomRelURI.lastIndexOf(File.separator)); // path until the file directory, e.g. RootExternalModel\External business items\MyBO
		return absolutePath;
	}	
	
	
	/**
	 * Get the NamedElement representing file attachment for external schema. 
	 * @param externalSchema ExternalSchema
	 * @return element NamedElement
	 */		
	public static NamedElement getFileAttachmentForXSD(ExternalSchema externalSchema){
		if(externalSchema == null || externalSchema.getAspect() == null || !(externalSchema.getAspect().equals(EXTERNAL_SCHEMA_ASPECT))){
			return null;
		}

		List members = externalSchema.getOwnedMember();
		if(members != null && !members.isEmpty()){
			Iterator it = members.iterator();
			while(it.hasNext()){
				NamedElement mbr = (NamedElement)it.next();
				if(ExportOperation.FILE_ATTACHMENT.equals(mbr.getName()) && 
						"fileAttachment".equals(mbr.getAspect())){
					return mbr;
				}				
			}
		}    	
		return null;
	}

	
	/**
	 * Get the NamedElement representing file attachment for external wsdl. 
	 * @param externalSchema ExternalSchema
	 * @return element NamedElement
	 */		
	public static NamedElement getFileAttachmentForWSDL(ExternalService externalService){
		if(externalService == null){
			return null;
		}
		
		List  members = externalService.getOwnedMember();
		if(members != null && !members.isEmpty()){
			Iterator it = members.iterator();
			while(it.hasNext()){
				NamedElement mbr = (NamedElement)it.next();
				if(ExportOperation.FILE_ATTACHMENT.equals(mbr.getName()) && 
						"fileAttachment".equals(mbr.getAspect()) &&
						mbr instanceof ExternalSchema){
					List fileAttachments = ((ExternalSchema)mbr).getOwnedMember();
					if(!fileAttachments.isEmpty()){
						return (NamedElement)fileAttachments.get(0);
					}
				}				
			}
		}    	
		
		return null;
	}

	
	/**
	 * Get the file name representing file attachment. 
	 * @param directoryLocation String
	 * @return fileName String
	 */		
	public static String getFileNameForXSDAttachment(String directoryLocation) {
	    if (directoryLocation == null) {
	        return null;
	    }
	    
	    String fileName = null;
	    String filePath = directoryLocation.substring(0, directoryLocation.length() - (ExportOperation.FILE_ATTACHMENT.length() + 1));
	    
	    fileName = filePath.substring(filePath.lastIndexOf(File.separatorChar));
	    
	    return fileName;
	}
	
	/**
	 * Get the file name representing file attachment. 
	 * @param directoryLocation String
	 * @return fileName String
	 */		
	public static String getFileNameForWSDLAttachment(String directoryLocation) {
	    if (directoryLocation == null) {
	        return null;
	    }
	    
	    String fileName = null;
	    String filePath = directoryLocation.substring(0, directoryLocation.length() - ((ExportOperation.FILE_ATTACHMENT.length() + 1) * 2));
	    
	    fileName = filePath.substring(filePath.lastIndexOf(File.separatorChar));
	    
	    return fileName;
	}
	
	
	/**
	 * Find the xsd type contained in the top level schema element.
	 * @param xsdParent XSDComponent 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromParent(XSDComponent xsdParent, String convertedName){
		if (xsdParent == null || convertedName == null) {
			return null;
		}
		XSDTypeDefinition typeDefinition = null;
		if (xsdParent instanceof XSDComplexTypeDefinition) {
			typeDefinition = getTypeFromComplexType((XSDComplexTypeDefinition)xsdParent, convertedName, null);
		} else if (xsdParent instanceof XSDSimpleTypeDefinition) {
			typeDefinition = getTypeFromSimpleTypeDefinition((XSDSimpleTypeDefinition) xsdParent, convertedName);
		} else if (xsdParent instanceof XSDElementDeclaration) {
			typeDefinition = getTypeFromElementDeclaration((XSDElementDeclaration) xsdParent, convertedName, null);
		} else if (xsdParent instanceof XSDAttributeDeclaration) {
			typeDefinition = getTypeFromAttributeDeclaration((XSDAttributeDeclaration) xsdParent, convertedName);
		}
		
		return typeDefinition;
	}
	
	/**
	 * Find the xsd type contained in the complex type.
	 * @param xsdComplexType XSDComplexTypeDefinition 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromComplexType(XSDComplexTypeDefinition xsdComplexType, String convertedName, Set checkedParticles){
		if (xsdComplexType == null || xsdComplexType.getContent() == null || convertedName == null) {
			return null;
		}
		
		XSDTypeDefinition typeDefinition = null;
		if (xsdComplexType.getContent() instanceof XSDParticle) {
			typeDefinition = getTypeFromParticle((XSDParticle) xsdComplexType.getContent(), convertedName, checkedParticles);
		} else if (xsdComplexType.getContent() instanceof XSDSimpleTypeDefinition) {
			typeDefinition = getTypeFromSimpleTypeDefinition((XSDSimpleTypeDefinition) xsdComplexType.getContent(), convertedName);
		} 
		
		return typeDefinition;
	}

	/**
	 * Find the xsd type contained in particle.
	 * @param xsdContent XSDParticle 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromParticle (XSDParticle checkParticle, String propertyName, Set checkedParticles) {
		if (checkParticle == null || checkParticle.getContent() == null || (checkedParticles != null && checkedParticles.contains(checkParticle))) {
			return null;
		}
		
		if (checkedParticles == null) {
			checkedParticles = new HashSet();
		}
		
		checkedParticles.add(checkParticle);
		
		XSDParticleContent particleContent = checkParticle.getContent();
		XSDTypeDefinition xsdType = null;
		
		if (particleContent instanceof XSDElementDeclaration) {
			xsdType = getTypeFromElementDeclaration((XSDElementDeclaration) particleContent, propertyName, checkedParticles);
		} else if (particleContent instanceof XSDModelGroup) {
			XSDModelGroup modelGroup = (XSDModelGroup)particleContent;
			List particles = modelGroup.getContents();
			if (particles == null) {
			    return null;
			}
			XSDParticle particle = null;
			for (Iterator it = particles.iterator(); it.hasNext() && xsdType == null;) {
				xsdType = getTypeFromParticle((XSDParticle)it.next(), propertyName, checkedParticles);
			}
		} else if (particleContent instanceof XSDModelGroupDefinition) {
		    XSDModelGroupDefinition group = (XSDModelGroupDefinition)particleContent;
			XSDModelGroup modelGroup = group.getResolvedModelGroupDefinition().getModelGroup();
			if (modelGroup == null || modelGroup.getContents() == null) {
			    return null;
			}
			List particles = modelGroup.getContents();
			XSDParticle particle = null;
			for (Iterator it = particles.iterator(); it.hasNext() && xsdType == null;) {
				xsdType = getTypeFromParticle((XSDParticle)it.next(), propertyName, checkedParticles);
			}
		}
	    
		return xsdType;
	}

	/**
	 * Find the xsd type contained in simple type.
	 * @param xsdSimpleType XSDSimpleTypeDefinition 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromSimpleTypeDefinition (XSDSimpleTypeDefinition xsdSimpleType, String propertyName) {
		if (xsdSimpleType == null || propertyName == null) {
			return null;
		}
		
		if (xsdSimpleType.getName() != null && xsdSimpleType.getName().equals(propertyName)) {
			return xsdSimpleType;
		}
		
		List content = xsdSimpleType.getContents();
		if (content == null || content.isEmpty()) {
			return null;
		}
		
		Iterator it = content.iterator();
		XSDSimpleTypeDefinition xsdType = null;
		XSDSimpleTypeDefinition xsdSimpleTypeDefinition = null;
		
		while (it.hasNext() && xsdType == null) {
			xsdSimpleTypeDefinition = (XSDSimpleTypeDefinition)it.next();
			if (xsdSimpleTypeDefinition.getName() != null && xsdSimpleTypeDefinition.getName().equals(propertyName)) {
				xsdType = xsdSimpleTypeDefinition;
			}
		}
		
		return xsdType;
	}
	
	/**
	 * Find the xsd type contained in element declaration.
	 * @param xsdElement XSDElementDeclaration 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromElementDeclaration (XSDElementDeclaration xsdElement, String propertyName, Set checkedParticles) {
		if (xsdElement == null || propertyName == null) {
			return null;
		}
		
		if (xsdElement.getName() != null && xsdElement.getName().equals(propertyName)) {
			return xsdElement.getTypeDefinition();
		}
		
		XSDTypeDefinition xsdTypeDefinition = xsdElement.getTypeDefinition();
		XSDTypeDefinition xsdType = null;
		
		if (xsdTypeDefinition instanceof XSDComplexTypeDefinition) {
			xsdType = getTypeFromComplexType((XSDComplexTypeDefinition) xsdTypeDefinition, propertyName, checkedParticles);
		} else if (xsdTypeDefinition instanceof XSDSimpleTypeDefinition){
			xsdType = getTypeFromSimpleTypeDefinition((XSDSimpleTypeDefinition) xsdTypeDefinition, propertyName);
		}
		
		return xsdType;
	}

	/**
	 * Find the xsd type contained in attribute declaration.
	 * @param xsdAttribute XSDAttributeDeclaration 
	 * @param convertedName String
	 * @return typeDefinition XSDTypeDefinition
	 */		
	public static XSDTypeDefinition getTypeFromAttributeDeclaration (XSDAttributeDeclaration xsdAttribute, String propertyName) {
		if (xsdAttribute == null || propertyName == null) {
			return null;
		}
		
		if (xsdAttribute.getName() != null && xsdAttribute.getName().equals(propertyName)) {
			return xsdAttribute.getTypeDefinition();
		}
		
		XSDTypeDefinition xsdTypeDefinition = xsdAttribute.getTypeDefinition();
		XSDTypeDefinition xsdType = null;
		
		if (xsdTypeDefinition instanceof XSDSimpleTypeDefinition){
			xsdType = getTypeFromSimpleTypeDefinition((XSDSimpleTypeDefinition) xsdTypeDefinition, propertyName);
		}
		
		return xsdType;
	}
	
	/**
	 * Find the particle contained in another particle.
	 * @param checkParticle XSDParticle 
	 * @param propertyName String
	 * @return particle XSDParticle
	 */		
	public static XSDParticle getParticleFromParticle (XSDParticle checkParticle, String propertyName) {
		if (checkParticle == null || checkParticle.getContent() == null || propertyName == null) {
			return null;
		}
		
		XSDParticleContent particleContent = checkParticle.getContent();
		XSDParticle xsdParticle = null;
		
		if (particleContent instanceof XSDElementDeclaration) {
		    XSDElementDeclaration element = (XSDElementDeclaration) particleContent;
			if (element.getName() != null && element.getName().equals(propertyName)) {
				xsdParticle = checkParticle;
			}
		} else if (particleContent instanceof XSDModelGroup) {
			XSDModelGroup modelGroup = (XSDModelGroup)particleContent;
			List particles = modelGroup.getContents();
			if (particles == null) {
			    return null;
			}
			XSDParticle particle = null;
			for (Iterator it = particles.iterator(); it.hasNext() && xsdParticle == null;) {
			    xsdParticle = getParticleFromParticle((XSDParticle)it.next(), propertyName);
			}
		} else if (particleContent instanceof XSDModelGroupDefinition) {
		    XSDModelGroupDefinition group = (XSDModelGroupDefinition)particleContent;
			XSDModelGroup modelGroup = group.getResolvedModelGroupDefinition().getModelGroup();
			if (modelGroup == null || modelGroup.getContents() == null) {
			    return null;
			}
			List particles = modelGroup.getContents();
			XSDParticle particle = null;
			for (Iterator it = particles.iterator(); it.hasNext() && xsdParticle == null;) {
			    xsdParticle = getParticleFromParticle((XSDParticle)it.next(), propertyName);
			}
		}
	    
		return xsdParticle;
	}
	
	/**
	 * Loads the schema for the named element. 
	 * @param context TransformationContext
	 * @param externalSchema ExternalSchema
	 * @param ne NamedElement
	 * @return schema XSDSchema
	 */
	public static XSDSchema loadOriginalSchema(TransformationContext context, ExternalSchema externalSchema, NamedElement sourceElement) {
        String location = null;
        String directoryLocation = null;
        // Ray 06/06/07 fix for Share Across Project
        //String sourceProjectName = (String)context.get(TransformationSessionKeyConstants.PROJECT_NAME);
        String sourceProjectName = ResourceMGR.getResourceManger().getProjectName(sourceElement);
        String fileName = null;

        if (externalSchema != null && externalSchema.getAspect() != null) {
            if (externalSchema.getAspect().equals(EXTERNAL_SCHEMA_ASPECT)) {
                // get the BOM Class representing the XSD file
                NamedElement element = getFileAttachmentForXSD(externalSchema);
                directoryLocation = getLocationForElement(sourceProjectName, element);
                fileName = getFileNameForXSDAttachment(directoryLocation);
                if (fileName == null || directoryLocation == null) {
    				String params[] = {sourceElement.getName()};
    				LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
    				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
    				throw runtimeEx;
                }
                fileName = ExportOperationUtil.replaceConvertedSpaces(fileName);
		        location = directoryLocation + fileName + ExportOperation.XSD_FILE_EXT;				                    
            } else if (externalSchema.getAspect().equals(INLINE_SCHEMA_ASPECT)) {
                // get the BOM Class representing the XSD file
                NamedElement element = getFileAttachmentForWSDL((ExternalService)externalSchema.getOwningPackage());
                directoryLocation = getLocationForElement(sourceProjectName, element);
                fileName = getFileNameForWSDLAttachment(directoryLocation);
                if (fileName == null || directoryLocation == null) {
    				String params[] = {sourceElement.getName()};
    				LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
    				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
    				throw runtimeEx;
                }
                fileName = ExportOperationUtil.replaceConvertedSpaces(fileName);
		        location = directoryLocation + fileName + ExportOperation.WSDL_FILE_EXT;				                    
            }
        }
        
        XSDSchema origSchema = getXSD(location, sourceElement);
        if (origSchema != null) {
            ExportSession session = (ExportSession)context.get(TransformationSessionKeyConstants.EXPORT_SESSION);
            HashMap object2FileReg = (HashMap)session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY);
            registerXsdElement(context, externalSchema, origSchema);
//            registerXsdElement(context, origSchema.getSchemaLocation(), origSchema);     	// register the schema with its location for component retrieval
            registerXsdElementBasedOnLocation(context, origSchema.getSchemaLocation(), origSchema);	// register the schema with its location for component retrieval
            if (externalSchema.getAspect() != null && externalSchema.getAspect().equalsIgnoreCase(INLINE_SCHEMA_ASPECT)) {
		        if (((XSDSchemaImpl)origSchema).eContainer().eContainer() != null && ((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer() != null &&
		                ((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer() instanceof Definition) { //definition/types/xsdschemaextensibilityelement
		            Definition def = (Definition)((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer();
		            // register location+namespace for inline XSD in WSDL
		            if (!object2FileReg.containsKey(origSchema.getSchemaLocation() + origSchema.getTargetNamespace())) {
			            object2FileReg.put(origSchema.getSchemaLocation() + origSchema.getTargetNamespace(), def);
		            }
		        }
            }
        }	                

        return origSchema;
	}

	/**
	 * Loads the schema for the external schema element. 
	 * @param context TransformationContext
	 * @param externalSchema ExternalSchema
	 * @return schema XSDSchema
	 */
	public static XSDSchema loadOriginalSchema(TransformationContext context, ExternalSchema externalSchema) {
        String location = null;
        String directoryLocation = null;
        // Ray 06/06/07 fix for Share Across Project
        //        String sourceProjectName = (String)context.get(TransformationSessionKeyConstants.PROJECT_NAME);		
        String sourceProjectName = ResourceMGR.getResourceManger().getProjectName(externalSchema);
        String fileName = null;

        if (externalSchema != null && externalSchema.getAspect() != null) {
            if (externalSchema.getAspect().equals(EXTERNAL_SCHEMA_ASPECT)) {
                // get the BOM Class representing the XSD file
                NamedElement element = getFileAttachmentForXSD(externalSchema);
                directoryLocation = getLocationForElement(sourceProjectName, element);
                fileName = getFileNameForXSDAttachment(directoryLocation);
                if (fileName == null || directoryLocation == null) {
    				String params[] = {externalSchema.getName()};
    				LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
    				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
    				throw runtimeEx;
                }
                fileName = ExportOperationUtil.replaceConvertedSpaces(fileName);
		        location = directoryLocation + fileName + ExportOperation.XSD_FILE_EXT;				                    
            } else if (externalSchema.getAspect().equals(INLINE_SCHEMA_ASPECT)) {
                // get the BOM Class representing the XSD file
                NamedElement element = getFileAttachmentForWSDL((ExternalService)externalSchema.getOwningPackage());
                directoryLocation = getLocationForElement(sourceProjectName, element);
                fileName = getFileNameForWSDLAttachment(directoryLocation);
                if (fileName == null || directoryLocation == null) {
    				String params[] = {externalSchema.getName()};
    				LoggingUtil.logError(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION, params, null);
    				BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.ERROR_RETRIEVING_ORIGINAL_XSD_DEFINITION));
    				throw runtimeEx;
                }
                fileName = ExportOperationUtil.replaceConvertedSpaces(fileName);
		        location = directoryLocation + fileName + ExportOperation.WSDL_FILE_EXT;				                    
            }
        }
        
        XSDSchema origSchema = getXSD(location, externalSchema);
        if (origSchema != null) {
            ExportSession session = (ExportSession)context.get(TransformationSessionKeyConstants.EXPORT_SESSION);
            HashMap object2FileReg = (HashMap)session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY);
            registerXsdElement(context, externalSchema, origSchema);
//            registerXsdElement(context, origSchema.getSchemaLocation(), origSchema);     	// register the schema with its location for component retrieval
            registerXsdElementBasedOnLocation(context, origSchema.getSchemaLocation(), origSchema);     	// register the schema with its location for component retrieval
            if (externalSchema.getAspect() != null && externalSchema.getAspect().equalsIgnoreCase(INLINE_SCHEMA_ASPECT)) {
		        if (((XSDSchemaImpl)origSchema).eContainer().eContainer() != null && ((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer() != null &&
		                ((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer() instanceof Definition) { //definition/types/xsdschemaextensibilityelement
		            Definition def = (Definition)((XSDSchemaImpl)origSchema).eContainer().eContainer().eContainer();
		            // register location+namespace for inline XSD in WSDL
		            if (!object2FileReg.containsKey(origSchema.getSchemaLocation() + origSchema.getTargetNamespace())) {
			            object2FileReg.put(origSchema.getSchemaLocation() + origSchema.getTargetNamespace(), def);
		            }
		        }
            }
        }	                

        return origSchema;
	}


	/**
	 * Returns the schema registered with the namespace. 
	 * @param context TransformationContext 
	 * @param originalSchemaNamespace String
	 * @param baseType XSDTypeDefinition
	 * @return schema XSDSchema
	 */
	public static XSDSchema createSchemaForList(TransformationContext context, String originalSchemaNamespace, XSDTypeDefinition baseType) {
		XSDSchema xsd = XSDFactory.eINSTANCE.createXSDSchema();
		if (originalSchemaNamespace != null) {
			xsd.setTargetNamespace(originalSchemaNamespace + ExportOperation.COLLECTION_SUFFIX);
		}
		xsd.setSchemaForSchemaQNamePrefix("xsd");
		Map namespaces = xsd.getQNamePrefixToNamespaceMap();
		if (originalSchemaNamespace != null) {
			namespaces.put("tns", xsd.getTargetNamespace());
		}
		namespaces.put(xsd.getSchemaForSchemaQNamePrefix(), XSDConstants.SCHEMA_FOR_SCHEMA_URI_2001);
		String schemaLocation = baseType.getSchema().getSchemaLocation().substring(0, baseType.getSchema().getSchemaLocation().lastIndexOf(".")) +
		 									ExportOperation.COLLECTION_SUFFIX +
		 									ExportOperation.XSD_FILE_EXT;
		xsd.setSchemaLocation(schemaLocation);
	    
	 	if (NamingUtil.getRegistry(context, xsd) == null) {				
	  		NamingUtil.putRegistry(context, xsd, new XsdNamingRegistry());
	 	}
		
		registerXsdElement(context, originalSchemaNamespace, xsd);
		registerXsdElementBasedOnLocation(context, schemaLocation, xsd);
		
        ExportSession session = (ExportSession)context.get(TransformationSessionKeyConstants.EXPORT_SESSION);
        HashMap object2FileReg = (HashMap)session.getExportOptions().getAdditionalOption(TransformationSessionKeyConstants.OBJECT_TO_FILE_NAME_REGISTRY);
        if (!object2FileReg.containsKey(xsd)) {
            object2FileReg.put(xsd, baseType.getSchema().getSchemaLocation());
        }

        return xsd;
    }

	/**
	 * Checks if this type's target namespace is defined as meta schema. 
	 * @param baseType XSDTypeDefinition
	 * @return isMeta boolean
	 */
	public static boolean isFromMetaSchema(XSDTypeDefinition baseType) {
		if (baseType == null || baseType.getTargetNamespace() == null) {
			return false;
		}

		if (baseType.getTargetNamespace().equalsIgnoreCase(ExportOperation.PROCESS_SCHEMA_IMPORT) ||
				baseType.getTargetNamespace().equalsIgnoreCase(ExportOperation.PROCESS_SCHEMA_IMPORT_1998)) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Checks if this namespace is already imported in schema. 
	 * @param xsd XSDSchema
	 * @return isImported boolean
	 */
	public static boolean hasImportForNamespace(XSDSchema xsd, String ns) {
		if (xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty()|| ns == null) {
			return false;
		}
	
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDImport) {
				XSDImport xsdimport = (XSDImport) obj;
				if (xsdimport.getNamespace() == null) {
					continue;
				}
				if (xsdimport.getNamespace().equals(ns)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Checks if this file is already imported in schema. 
	 * @param xsd XSDSchema
	 * @return isImported boolean
	 */
	public static boolean hasImportForFile(XSDSchema xsd, String filePath) {
		if (xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty()|| filePath == null) {
			return false;
		}
	
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDImport) {
				XSDImport xsdimport = (XSDImport) obj;
				if (xsdimport.getSchemaLocation() == null) {
					continue;
				}
				if (xsdimport.getSchemaLocation().equals(filePath)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Checks if this file is already included in schema. 
	 * @param xsd XSDSchema
	 * @return isImported boolean
	 */
	public static boolean hasIncludeForFile(XSDSchema xsd, String filePath) {
		if (xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty()|| filePath == null) {
			return false;
		}
	
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDInclude) {
				XSDInclude xsdInclude = (XSDInclude) obj;
				if (xsdInclude.getSchemaLocation() == null) {
					continue;
				}
				if (xsdInclude.getSchemaLocation().equals(filePath)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Checks if this file is already imported in schema. 
	 * @param xsd XSDSchema
	 * @param filePath String
	 * @return isImported boolean
	 */
	public static boolean hasImportForFileName(XSDSchema xsd, String filePath) {
		if (xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty() || filePath == null) {
			return false;
		}
	
		String fileName = null;
		if (filePath.indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
			fileName = filePath.substring(filePath.lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);
		} else {
			fileName = filePath;
		}
		
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		String file = null;
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDImport) {
				XSDImport xsdimport = (XSDImport) obj;
				if (xsdimport.getSchemaLocation() == null) {
					continue;
				}
				if (xsdimport.getSchemaLocation().indexOf(ExportOperation.FILE_SEPARATOR_CHAR) != -1) {
					file = xsdimport.getSchemaLocation().substring(xsdimport.getSchemaLocation().lastIndexOf(ExportOperation.FILE_SEPARATOR_CHAR) + 1);
				} else {
					file = xsdimport.getSchemaLocation();
				}
				if (fileName.equals(file)) {
					return true;
				}
			}
		}
		return false;
	}


	/**
	 * Finds the schema from import for the element. 
	 * @param xsd XSDSchema
	 * @param targetNamespace String
	 * @param elementName String
	 * @return resolvedSchema XSDSchema
	 */
	public static XSDElementDeclaration getSchemaForImport(XSDSchema xsd, String targetNamespace, String elementName) {
		if (xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty() || 
				targetNamespace == null || elementName == null) {
			return null;
		}
	
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		XSDSchema xsdSchema = null;
		XSDElementDeclaration xsdElement = null;
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDImport) {
				XSDImport xsdimport = (XSDImport) obj;
				if (xsdimport.getSchemaLocation() == null) {
					continue;
				}
				if (xsdimport.getNamespace().equals(targetNamespace)) {
//					xsdimport.setSchemaLocation(ExportOperationUtil.replaceConvertedSpaces(xsdimport.getSchemaLocation()));
//					xsdSchema = xsdimport.getResolvedSchema();
					xsdSchema = forceLoad(ExportOperationUtil.replaceConvertedSpaces(xsdimport.getSchemaLocation()).replace('/', File.separatorChar ));
					xsdElement = getElementDeclarationFromSchema(xsdSchema, elementName);
					if (xsdElement != null) {
						return xsdElement;
					}
				}
			}
		}
		return null;
	}

	
	/**
	 * Finds the element from imported schema. 
 	 * @param context TransformationContext 
	 * @param xsd XSDSchema
	 * @param importedNamespace String
	 * @param elementName String
	 * @return resolvedElement XSDElementDeclaration
	 */
	public static XSDElementDeclaration getElementFromImported(TransformationContext context, XSDSchema xsd, String importedNamespace, String elementName) {
		if (context == null || xsd == null || xsd.getContents() == null || xsd.getContents().isEmpty() || 
				importedNamespace == null || elementName == null) {
			return null;
		}
	
		Iterator it = xsd.getContents().iterator();
		Object obj = null;
		XSDSchema xsdSchema = null;
		XSDElementDeclaration xsdElement = null;
		while (it.hasNext()) {
			obj = it.next();
			if (obj instanceof XSDImport) {
				XSDImport xsdImport = (XSDImport) obj;
				if (xsdImport.getSchemaLocation() == null) {
					continue;
				}
				if (xsdImport.getNamespace() != null && xsdImport.getNamespace().equals(importedNamespace)) {
					xsdSchema = (XSDSchema)OriginalXsdUtil.getSchema(context, xsdImport.getSchemaLocation());
					if (xsdSchema == null) {
						xsdSchema = forceLoad(ExportOperationUtil.replaceConvertedSpaces(xsdImport.getSchemaLocation()).replace('/', File.separatorChar ));
					}
					xsdElement = getElementDeclarationFromSchema(xsdSchema, elementName);
					if (xsdElement != null) {
						return xsdElement;
					}
				}
			}
		}
		return null;
	}

	
	// utility method to force load a schema from its URI
	private static XSDSchema forceLoad(String fileName){
		if (fileName == null) {
			return null;
		}
		if (fileName.startsWith("file:\\")) {
			fileName = fileName.substring(6);
		}
		File inputFile = new File(fileName);
		URI fileURI = URI.createFileURI(inputFile.getAbsolutePath());
		final ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.getResource(fileURI, true);
		if (!resourceSet.getResources().isEmpty()) {
			for (Iterator it = resourceSet.getResources().iterator(); it.hasNext(); ) {
				resource = (Resource)it.next();
				if (resource instanceof XSDResourceImpl){
					XSDResourceImpl xsdResource = (XSDResourceImpl)resource;
					if (!xsdResource.getContents().isEmpty()) {
						XSDSchema xsdSchema = (XSDSchema)xsdResource.getContents().get(0);
//	            		if (xsdSchema.getTargetNamespace() == null) {
//	            			String params[] = {xsdSchema.getSchemaLocation()};
//	            			LoggingUtil.logError(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION, params, null);
//	            			BTRuntimeException runtimeEx = new BTRuntimeException(TEResourceBundleSingleton.INSTANCE.getMessage(MessageKeys.NO_NAMESPACE_FOR_ORIGINAL_XSD_DEFINITION));
//	            			throw runtimeEx;
//	            		}

	                	return xsdSchema;
					}
				}
			}
		}
	
		return null;
	}	

	public static void cleanXSDRegistry(TransformationContext context){
		Map registry = (Map) context.get(ORIG_XSDS_REGISTRY);
		if(registry != null){
			registry.clear();
			context.remove(ORIG_XSDS_REGISTRY);
		}
	}

}
	
