package com.ibm.btools.te.ilm.heuristics.xsd.util;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry;


public class XsdNamingRegistry implements NamingRegistry{

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
	
	private  HashMap complexTypeNameRegistry = new HashMap();
	private  HashMap elementNameRegistry = new HashMap();	
	private  HashMap xsdSchemaNameSpaceRegistry = new HashMap();

	/**
	 * Returns a boolean whether the name is unique
	 * It searches the registry and and return false if name is not unique
	 * it returns true if name is unique and it stores the name in the registry.
	 * @param obj  the EObject of a construct
	 * @param name  the String name
	 * @return boolean the boolean value whether name is unique or not
	 */	
	public boolean isUniqueName(Object obj, String name)
	{
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}
		else if (map.values().contains(name)){
			// Ray: ensure uniqueness of XSD element only within the scope
			// of the complex type
			if (obj instanceof XSDElementDeclaration) {
				// Find the element currently in the registry that 
				// has the same name
				Iterator it = map.keySet().iterator();
				XSDElementDeclaration decl=null;
				while (it.hasNext()) {
					decl = (XSDElementDeclaration)it.next();
					if (map.get(decl).equals(name)) {
						break;
					}
				}
				// decl must never be null
				// Get the Complext Type that owns this element
				EObject declComplexType=decl.eContainer();
				while ((declComplexType != null) && !(declComplexType instanceof XSDComplexTypeDefinition)) {
					declComplexType=declComplexType.eContainer();
				}
				EObject newObjComplexType=((EObject)obj).eContainer();
				while ((newObjComplexType != null) && !(newObjComplexType instanceof XSDComplexTypeDefinition)) {
					newObjComplexType=newObjComplexType.eContainer();
				}
				if (declComplexType != null && newObjComplexType != null) {
					if (declComplexType != newObjComplexType) {
						return true;
					}
				}
			}
			return false;
		}
		else {
			//Store the name in the Registry
			map.put(obj, name);
			return true;
		}
	}
	
	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */		
	private HashMap getRegistry(Object obj)
	{
		if (obj instanceof XSDComplexTypeDefinition)
			return complexTypeNameRegistry;
		else if (obj instanceof XSDElementDeclaration)
			return elementNameRegistry;
		else if (obj instanceof XSDSchema)
			return xsdSchemaNameSpaceRegistry;
		else
			return null;
	}

	/**
	 * Returns a HashMap for the specific obj	 
	 * @param obj  the EObject of a type of construct	
	 * @return HashMap the corresponding hashmap for obj
	 */		
	private HashMap getRegistry(Class obj)
	{
		if (obj == XSDComplexTypeDefinition.class)
			return complexTypeNameRegistry;
		else if (obj == XSDElementDeclaration.class)
			return elementNameRegistry;
		else if (obj == XSDSchema.class)
			return xsdSchemaNameSpaceRegistry;
		else
			return null;
	}
	
	/**
	 * Returns a boolean whether the name is unique
	 * It searches the registry and and return false if name is not unique
	 * it returns true if name is unique and it stores the name in the registry.
	 * @param obj  the EObject of a construct
	 * @param name  the String name
	 * @return boolean the boolean value whether name is unique or not
	 */		
	public boolean isUniqueName(Object obj, Object parentObject, String name)
	{
		HashMap map = getRegistry(obj);
		List entries = null;
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return true;		 		
		}else if(map.get(parentObject) != null){
			entries = (List)map.get(parentObject);
			if(entries.contains(name)){
				return false;
			}else{
				entries.add(name);
				return true;
			}
		}else {
			entries = new LinkedList();
			entries.add(name);
			//Store the name in the Registry
			map.put(parentObject, entries);
			return true;
		}
	}
	public void deregister(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if (map == null){
			// name for this type of construct does not require name uniqueness
			return;		 		
		}
		else if (map.values().contains(name)){
			map.remove(obj);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#retrieveFromRegistry(java.lang.String)
	 */
	public Object retrieveFromRegistry(Class obj, String registryEntry) {
		
		HashMap registry = getRegistry(obj);
		if(registry.containsValue(registryEntry)){
			//get the object container that was registered for that entry.. 
			Iterator it = registry.keySet().iterator();
			while(it.hasNext()){
				Object variable = (EObject)it.next();
				if(registry.get(variable).equals(registryEntry)){
					return variable;
				}
			}
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.btools.te.ilm.heuristics.naming.NamingRegistry#register(java.lang.Object, java.lang.String)
	 */
	public void register(Object obj, String name) {
		HashMap map = getRegistry(obj);
		if(map == null){
			return;
		}
		map.put(obj, name);
	}

}
