package com.ibm.btools.te.ilm.rulehandler.mapping;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.wst.wsdl.ExtensibilityElement;
import org.eclipse.wst.wsdl.ExtensibleElement;
import org.eclipse.wst.wsdl.Message;
import org.eclipse.wst.wsdl.Output;
import org.eclipse.wst.wsdl.PortType;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.model.resourcemanager.ResourceMGR;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.wbit.bpelpp.impl.IdImpl;
import com.ibm.wbit.bpelpp.impl.VariableIdImpl;
import com.ibm.wsspi.sca.scdl.Component;
/**
 * Utility class that provides methods for generating URI.
 */
public class URIUtil {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$	
	public static String getObjectURI(EObject eObj, HashMap objectFilePathMap) {
		if (eObj instanceof NamedElement) {
			return getBOMElementURI((NamedElement)eObj); 
		} else {
			String className=eObj.getClass().getName();
			if (className.startsWith("com.ibm.wbit.bpel")) {
				return getBPELElementURI(eObj,objectFilePathMap);
			} else if (className.startsWith("org.eclipse.xsd")) {
				return getXsdElementURI(eObj,objectFilePathMap);
			} else if (className.startsWith("com.ibm.wsspi.sca.scdl")) {
				return getScdlElementURI(eObj,objectFilePathMap);
			} else if (className.startsWith("com.ibm.wbit.tel")) {
//				return getTelElementURI(eObj,objectFilePathMap);
			} else {
				return getWsdlElementURI(eObj,objectFilePathMap);
			}
			
		}
		return null;
	}
	/**
	 * Returns the BOM Element URI
	 * @param bomElement
	 * @return
	 */
	public static String getBOMElementURI(NamedElement bomElement) {
		String uri = bomElement.getUid();
		String resId = ResourceMGR.getResourceManger().getObjectResourceID(bomElement);
		return resId+".xmi#"+uri;
	}
	/**
	 * Returns the WID Element URI
	 */
	public static String getWIDElementURI(EObject object, String id, HashMap objectFilePathMap) {
		String uri = id;
		EObject topLevelElement = MappingUtil.getTopLevelElement(object);
		String resource=(String)objectFilePathMap.get(topLevelElement);
		return resource+"#"+id;
	}
	
	public static String getBPELElementURI(EObject object, HashMap objectFilePathMap) {
		String type = object.eClass().getName();
		String name = (String)MappingUtil.findAttribute(object,"name");
		String id = null;
		// Should be implemented using generated Switch util class??
		if (object instanceof ExtensibleElement) {
			ExtensibilityElement ee=MappingUtil.getExtensibilityElement((ExtensibleElement)object,IdImpl.class);
			if (ee!=null) {
				id = ((IdImpl)ee).getId().toString();
			} else {
				ee=MappingUtil.getExtensibilityElement((ExtensibleElement)object,VariableIdImpl.class);
				if (ee != null) {
					id = ((VariableIdImpl)ee).getVariableId().toString();					
				}
			}
		}
		if (id==null) {
			id=name;
		}
		return getWIDElementURI(object,id,objectFilePathMap);
	}
	/**
	 * Returns the XSD Element URI
	 */
	public static String getXsdElementURI(EObject object, HashMap objectFilePathMap) {
		String type = object.eClass().getName();
		return getWIDElementURI(object,getXsdQualifiedPath(object),objectFilePathMap);
	}

	public static String getWsdlElementURI(EObject object, HashMap objectFilePathMap) {
		String type = object.eClass().getName();
		if (object instanceof PortType) {
			return getWIDElementURI(object,((PortType)object).getQName().getLocalPart(),objectFilePathMap);
		} else if (object instanceof Message) {
			return getWIDElementURI(object,((Message)object).getQName().getLocalPart(),objectFilePathMap);
		} else {
			return getWIDElementURI(object,getWsdlQualifiedPath(object),objectFilePathMap);
		}
	}
	/**
	 * Returns the SCDL Element URI
	 */
	public static String getScdlElementURI(EObject object, HashMap objectFilePathMap) {
		String type = object.eClass().getName();
		if (object instanceof Component) {
			return getWIDElementURI(object,((Component)object).getName(),objectFilePathMap);	
		}
		return null;
	}
	
	/**
	 * Returns the XSD Element qualified path
	 */
	private static String getXsdQualifiedPath(EObject object) {
		String path = (String)findAttribute(object,"name");
		object = object.eContainer();
		while (!(object instanceof XSDSchema)) {
			String name = (String)findAttribute(object,"name");
			if (name!=null) {
				path = name+"."+path;				
			}
			object = object.eContainer();
		}
		// special handling for in-line schema in WSDL
		if (object instanceof XSDSchema && object.eContainer()!=null) {
			path=((XSDSchema)object).getTargetNamespace()+":"+path;
		}
		return path;
	}
	
	/**
	 * Returns the XSD Element qualified path
	 */
	private static String getWsdlQualifiedPath(EObject object) {
		String path = (String)findAttribute(object,"name");
		object = object.eContainer();
		while (!(object instanceof PortType || object instanceof Message)) {
			path = (String)findAttribute(object,"name")+"."+path;
			object = object.eContainer();
		}
		QName qname = (QName)findAttribute(object,"qName");
		if (qname!=null) {
			path=qname.getLocalPart()+"."+path;			
		}
		return path;
	}
	
	private static Object findAttribute (EObject object, String attribName) {
		EAttribute attribute=findAttribute(object.eClass(),attribName);
		if (attribute != null) {
			return object.eGet(attribute);			
		} else {
			return null;
		}
	}
	
	private static EAttribute findAttribute(EClass eClass, String attribName) {
		List attributes = eClass.getEAttributes();
		for (Iterator it=attributes.iterator();it.hasNext();) {
			EAttribute attribute = (EAttribute)it.next();
			if (attribute.getName().equalsIgnoreCase(attribName)) {
				return attribute;
			}
		}
		for (Iterator it=eClass.getESuperTypes().iterator();it.hasNext();) {
			EAttribute attribute = findAttribute((EClass)it.next(),attribName);
			if (attribute!=null) {
				return attribute;
			}
		}
		return null;
	}
	
	/**
	 * This method is responsible for generating type using eobject package uri plus
	 * its class name (last segment)
	 * @param object
	 * @return String packageuri + classname
	 */
	public static String createTypeForEobject(EObject object){	
		String uri = null;
		String className = null;
		if(object == null){
			return null;
		}
		EPackage epkg = object.eClass().getEPackage();
		if(epkg != null){
			uri = epkg.getNsURI();			
		}
		if(object.eClass().getInstanceClassName()!= null){
			if(object.eClass().getInstanceClassName().indexOf(".") != -1){
				className = object.eClass().getInstanceClassName().substring(object.eClass().getInstanceClassName().lastIndexOf(".")+1);
			}
		}
		//AZ 20070529 Apend the aspect to the type string
		if(object  instanceof NamedElement){
			NamedElement ne = (NamedElement)object;
			if(ne instanceof Activity){
				Activity activity = (Activity)ne;
				Action action = activity.getImplementation();
				return uri+"#"+className+"#"+action.getAspect();
			}
			if(ne.getAspect() != null && !(ne instanceof com.ibm.btools.bom.model.artifacts.Class)){
				return  uri+"#"+className+"#"+ne.getAspect();
			}
		}
		return uri+"#"+className;
	}

}
