package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.ValueSpecification;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRule;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleAction;
import com.ibm.btools.bom.model.processes.businessrules.BusinessRuleTemplate;
import com.ibm.btools.expression.bom.model.StructuredOpaqueExpression;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.br.core.model.IfThenRule;
import com.ibm.wbit.br.core.model.ParameterValue;
import com.ibm.wbit.br.core.model.Rule;
import com.ibm.wbit.br.core.model.RuleSet;
import com.ibm.wbit.br.core.model.RuleTemplate;
import com.ibm.wbit.br.core.model.TemplateInstanceRule;
/**
 This handler is responsbile for creating the ObjectDefinition mapping between
 *the source and target: 
 *SOURCE: 
 *TARGET:
 */
public class BRRuleHandler extends ConnectableRuleHandler {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	/**	
	 * @param rule The BRReusableTaskRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		MappingManager mappingManager = getMappingManager(rule);
		BusinessRule src = (BusinessRule)rule.getSource().get(0);		
		if(rule.getTarget().get(0)instanceof Rule){
			Rule target = (Rule)rule.getTarget().get(0);			
			if(src instanceof com.ibm.btools.bom.model.processes.businessrules.IfThenRule && target instanceof IfThenRule){
				com.ibm.btools.bom.model.processes.businessrules.IfThenRule  srcIfThen = (com.ibm.btools.bom.model.processes.businessrules.IfThenRule)src;
				mappingManager.addMapDefinition(srcIfThen.getRuleCondition(),((IfThenRule)target).getIf());
				mappingManager.addMapDefinition(srcIfThen.getRuleCondition(),((IfThenRule)target).getIf().getConditionExpression());
				
				//map consequence/action
				EList consequences = srcIfThen.getRuleConsequence();
				EList actions = ((IfThenRule)target).getThen().getAction();	
				if (!consequences.isEmpty() && !actions.isEmpty()) {
					mapListOfSrc2Target(consequences,actions,mappingManager);
				} else {
					for (int i=0; i < actions.size(); i++) {
						mappingManager.addMapDefinition(srcIfThen, (EObject)actions.get(i));
					}
				}				
//				mapListOfSrc2Target(srcIfThen.getRuleConsequence(),((IfThenRule)target).getThen().getAction(),mappingManager);
			}else if(src instanceof com.ibm.btools.bom.model.processes.businessrules.TemplateInstanceRule && target instanceof TemplateInstanceRule){
				com.ibm.btools.bom.model.processes.businessrules.TemplateInstanceRule srctir = (com.ibm.btools.bom.model.processes.businessrules.TemplateInstanceRule)src;
				TemplateInstanceRule tir = (TemplateInstanceRule)target;				
				mapListOfSrc2Target(srctir.getParameterValues(),tir.getParameterValue(),mappingManager);
				mappingManager.addMapDefinition(srctir.getTemplate(), tir.getTemplateRef());
			}
			mappingManager.addMapDefinition(src, target);
		}	
	}	
	
	private void mapListOfSrc2Target(EList srcs, EList targets,MappingManager mm){
		if(srcs != null && targets != null && targets.size()> 0 &&  
				targets.size() == srcs.size()){
			int size = srcs.size();
			//the source should correspond to the target in the collection
			for(int i =0;i<size;i++ ){
				mm.addMapDefinition((EObject)srcs.get(i), (EObject)targets.get(i));
				
				if (srcs.get(i) instanceof ValueSpecification && targets.get(i) instanceof ParameterValue) {
					ValueSpecification srcValueSpec = (ValueSpecification)srcs.get(i);
					ParameterValue tgtValue = (ParameterValue)targets.get(i);
					
					if (tgtValue.getValue() != null)
						mm.addMapDefinition(srcValueSpec, tgtValue.getValue());
				}
			}				
		}
	}
}
