package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.OpaqueExpression;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.LoopNode;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.DecisionRule;
import com.ibm.btools.te.ilm.heuristics.bpelp.impl.BpelpRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Activity;
import com.ibm.wbit.bpel.Condition;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Link;
import com.ibm.wbit.bpel.Process;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpel.Target;
import com.ibm.wbit.bpel.While;
import com.ibm.wbit.bpelpp.impl.AutonomyImpl;
import com.ibm.wbit.bpelpp.impl.ExecutionModeImpl;
import com.ibm.wbit.bpm.map.objectdefinition.EMFRef;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.map.objectdefinition.URIRef;

/**
 * This class is responsible for updating the ObjectDefinition mapping to reflect
 * the mapping of the BpelpRule.
 * 
 * Specifically, this involves updating the ObjectDefinitions to replace the BPEL
 * <switch> activities with the <empty> activities with transition conditions.
 */
public class BpelpRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

	private BpelpRuleImpl sourceRule;
	private Process bpelProcess;

	/** 
	 * @param rule the BpelpRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		/**
		 * Handles the BpelpRule
		 * Source:  AbstractBPEL <process> activity
		 * Target: BPEL+ <process> activity with all children conforming to the BPEL+ Specification,
		 * also satisfying the runtime limitations in effect
		 */

		sourceRule = (BpelpRuleImpl)rule;
		bpelProcess = (Process)sourceRule.getSource().get(0);
		MappingManager mapMgr = getMappingManager(rule);
		ObjectDefinition processOD = mapMgr.getObjectDefinition(bpelProcess);
		EObject bomProcess = (EObject)ProcessUtil.getProcessDefinitionRule(rule.getRoot().getContext()).getSource().get(0);
		if(processOD != null){
			List extElements = bpelProcess.getEExtensibilityElements();
			if(extElements != null && !extElements.isEmpty()){
				Iterator it = extElements.iterator();
				while(it.hasNext()){
					EObject extElement = (EObject)it.next();
					if(extElement instanceof ExecutionModeImpl || extElement instanceof AutonomyImpl){
						mapMgr.addMapDefinition(bomProcess, extElement);
					}
				}
			}
		}

		//Add to handle while condition
		List whileConditionList = this.sourceRule.getWhileWithConditionList();
		if(whileConditionList != null && !whileConditionList.isEmpty()){
			for(Iterator it = whileConditionList.iterator();it.hasNext();){
				Object eachWhile = it.next();
				if(eachWhile instanceof While){
					While bpelWhile = (While)eachWhile;
					ObjectDefinition whileOD = mapMgr.getObjectDefinition(bpelWhile);
					if(whileOD != null){
						EList referencedObjectDefintions = whileOD.getReferencedDefinitions();
						if(referencedObjectDefintions != null && !referencedObjectDefintions.isEmpty()){
							URIRef bomSourceDefinitionURI = (URIRef)referencedObjectDefintions.get(0);
							if(MappingManager.getUriMappings() != null && !MappingManager.getUriMappings().isEmpty()){
								ObjectDefinition sourceObjectDefinition = (ObjectDefinition)MappingManager.getUriMappings().get(bomSourceDefinitionURI.getUri());
								if(sourceObjectDefinition != null && sourceObjectDefinition.getObjectReference() != null){
									EObject bomLoopNode= ((EMFRef)sourceObjectDefinition.getObjectReference()).getEObject();
									if(bomLoopNode instanceof LoopNode){
										LoopNode ln = (LoopNode)bomLoopNode;
										OpaqueExpression oe = ln.getLoopCondition();
										Condition whileCondition = bpelWhile.getCondition();
										//add mapping between loop condition and while condition 
										if(oe != null && whileCondition != null){
											mapMgr.addMapDefinition(oe, whileCondition);
										}
									}
								}
							}
						}
					}
				}
			}
		}

		List switches = ((BpelpRuleImpl)rule).getTransformableProcessSwitch();
		if(switches.size() == 0){
			/* No decisions were mapped */
			return;
		}

		Map linksMap = ((BpelpRuleImpl)rule).getConsolidatedLinksMap();	    
		// MapRecord decChildRec = null;
		Empty empty = null;
		Switch bpelSwitch = null;
		for (Iterator it = switches.iterator(); it.hasNext(); ){
			// decChildRec = (MapRecord)it.next();
			Object each = it.next(); 
			if(!(each instanceof Switch)){
				continue;
			}
			bpelSwitch = (Switch)each;
			empty = ((BpelpRuleImpl)rule).getDecisionActivity(bpelSwitch);	  

			if (empty != null){
				//  Decision decision = getDecisionForEmpty(empty);
				Decision decision = getDecisionForEmpty(bpelSwitch);

				//MappingManager mapMgr = getMappingManager(rule);
				mapMgr.addMapDefinition(decision, empty);
				List decisionLinkList = extractDecisionLinks(linksMap, decision, empty);
				matchEdgesWithLinks(decision, decisionLinkList);
			}
		}	    
	}


	private List extractDecisionLinks(Map linksMap, Decision decision, Empty decisionEmpty){

		List result = new ArrayList();

		List caseActivities = new ArrayList();
		OutputPinSet ops = null;	    
		Map targetToLinksMap = (Map)linksMap.get(decisionEmpty);
		List links = null;
		Link link = null;
		if(targetToLinksMap == null){
			return result;
		}
		for (Iterator it = targetToLinksMap.values().iterator(); it.hasNext(); ){
			links = (List)it.next();
			for (Iterator it2 = links.iterator(); it2.hasNext(); ){
				link = (Link)it2.next();
				if (link.eContainer() != null){
					result.add(link);
				}
			}
		}

		return result;
	}

	/**
	 * Returns the BOM Decision node that corresponds to the BPEL Empty activity.
	 *  
	 * @param empty The Empty activity to return the decision for 
	 * @return The BOM Decision activity that is mapped to the given BPEL Empty, or
	 * null if the Decision node could not be found.
	 */
	private Decision getDecisionForEmpty(Switch bpelSwitch){		
		TransformationRule decisionRule = TransformationUtil.getRuleForTarget(ProcessUtil.getProcessDefinitionRule(sourceRule.getContext()), DecisionRule.class,bpelSwitch);
		if (decisionRule != null){
			Decision decision = (Decision)((PinSet)decisionRule.getSource().get(0)).eContainer();
			return decision;
		}
		return null;	    
	}

	/**
	 * For each BOM ActivityEdge outgoing from a decision, we need to represent it
	 * in the HashMap by mapping it to the BOM link that corresponds to it.  
	 * This method makes sure that each ActivityEdge is associated with the correct
	 * outgoing BOM edge, accounting for BOM link optimization which has occurred.
	 * 
	 * @param decision The BOM decision node
	 * @param linkList The list of BPEL links that need to be matched to bom outgoing 
	 * activity edges
	 */
	private void matchEdgesWithLinks(Decision decision, List linkList){

		/* There are two cases we need to handle:
		 * 1. The Case Activity is a single <empty>
		 * 		- Here, the BpelpRule takes the <link>s that are outgoing from the CaseActivity <empty>
		 *        and gives them a new target, which is the decision <empty>.  Therefore, the HashMap entries for the 
		 *        ActivityEdges orignally created are still relevant, and can be left as is.
		 *      - BUT, some of the links may not have a container any more, since they could have been consolidated.
		 *        In that case, we need to find the ActivityEdge that represents that link that was NOT consolidated 
		 *        (a.k.a. the 'master' link) and point the ActivityEdge mapRecord to that master link.
		 *      - We should be able to detect the master link as the with a <link> association that has a container, 
		 *        and whose ActivityEdge has the same source and target pin set.  If the target of the AE is not a 
		 *        pin set (but perhaps a StopNode), associate the AE with any master links in the same pin set. 
		 * 2. The Case Activity is a <sequence> or an AlternativeActivity
		 * 		- In this case, the BpelpRule creates a new <link> to connect the decision <empty>.
		 * 		- We need to figure out which OS each new link corresponds to by looking at the target of each
		 *     	  new link and trying to match the target activity with one of the case activities.
		 * 		- We use the OS --> Case Activity List that is populated in the DecisionHandler for this.
		 * 		- When we find the matching OS, associate all the outgoing AEs in the OS with this single link
		 */

//		OutputPinSet ops = null;
//		for (Iterator it = decision.getOutputPinSet().iterator(); it.hasNext(); ){
//		ops = (OutputPinSet)it.next();

//		String key = RuleHandlerUtils.getInternalKey(ops, RuleHandlerUtils.DECISION_OUTPUT_SET_ACTIVITY_LIST_KEY_TAG);
//		MapRecord opsActivitiesRec = getInternalMapRecord(key);
//		MapRecord childRec = (MapRecord)opsActivitiesRec.getChildrenRecords().get(0);

//		if (opsActivitiesRec.getChildrenRecords().size() == 1 && childRec.getRuntimeObject() instanceof Empty){
//		/* This is case 1 above */
//		consolidateDecisionLinks(ops);

//		} else {

//		/* This is case 2 above */
//		Link opsLink = findLinkForOutputPinSet(ops, linkList);

//		Pin pin = null;
//		for (Iterator it2 = BomUtils.getPinsInSet(ops).iterator(); it2.hasNext(); ){
//		pin = (Pin)it2.next();

//		if (pin.getOutgoing() == null){
//		continue;
//		}          

//		/* A MapRecord must exist for this link, since it will have been created
//		* the LinkRuleHandler */
//		List edgeRecordList = getLinkMapRecords(pin.getOutgoing());
//		for (Iterator it3 = edgeRecordList.iterator(); it3.hasNext(); ){
//		((MapRecord)it3.next()).setRuntimeObject(opsLink);
//		/* The OM Link details will be updated in the ProcessFlowPatternRuleHandler */
//		}

//		}
//		}
//		}
	}

	/**
	 * This method searches the list of BPEL links for the one that matches the 
	 * output pin set.  This is done by retrieving the list of activities that the
	 * output pin set was mapped to, and looking for a link that has one of those
	 * activities as its target.
	 * 
	 * @param ops The output pin set to find the link for
	 * @param linkList The list of BPEL links to search 
	 * @return The link corresponding to the output pin set
	 */
	private Link findLinkForOutputPinSet(OutputPinSet ops, List linkList){

		//Links links = ((Flow)bpelProcess.getActivity()).getLinks();

		Collection rules = TransformationUtil.getRulesForSource(ProcessUtil.getProcessDefinitionRule(sourceRule.getContext()),ops);
		Iterator rulesIterator = rules.iterator();
		List activities = new LinkedList();
		while(rulesIterator.hasNext()){
			TransformationRule rule = (TransformationRule)rulesIterator.next();
			activities.addAll(rule.getTarget());
		}
		Link link = null;
		for (Iterator it = linkList.iterator(); it.hasNext(); ){
			link = (Link)it.next();	        
			for (Iterator it2 = activities.iterator(); it2.hasNext(); ){	          
				Activity activity = (Activity)it2.next();
				if (activity == ((Target)link.getTargets().get(0)).getActivity()){
					return link;
				}
			}
		}	    
		return null;
	}



}
