package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.Definition;
import org.eclipse.wst.wsdl.PortType;

import com.ibm.btools.blm.ie.exprt.ExportSession;
import com.ibm.btools.blm.ie.exprt.option.BpelOptions;
import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.bom.model.services.Activity;
import com.ibm.btools.te.attributes.helper.AttributeNameConstants;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.attribute.AttributeValueProvider;
import com.ibm.btools.te.ilm.attribute.AttributeValueProviderFactory;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.helper.BpelOptionsUtil;
import com.ibm.btools.te.ilm.heuristics.helper.FabricIntegrationUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbiservers.businessrule.model.brgt.BusinessRuleGroupTable;
import com.ibm.wbit.ae.sacl.SACLRoot;
import com.ibm.wsspi.sca.scdl.DocumentRoot;
import com.ibm.wsspi.sca.scdl.Implementation;
import com.ibm.wsspi.sca.scdl.ImportBinding;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Action, BPEL PortType
 * TARGET: DocumentRoot 
 */
public class ComponentRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 

	/**
	 * The component rule will create target components based on export options
	 * the user chooses, some cases both Component and Import DocumentRoot will
	 * be created. e.g. best practice pattern option
	 * @param rule The ComponentRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {	
		MappingManager mapMgr = getMappingManager(rule);
		Action action = (Action)rule.getSource().get(0);		
		DocumentRoot component = (DocumentRoot)rule.getTarget().get(0);
		DocumentRoot component2 = null;
		
		if(rule.getSource().size()>= 3 && rule.getSource().get(rule.getSource().size()-1) instanceof CallBehaviorAction){
			CallBehaviorAction cba = (CallBehaviorAction)rule.getSource().get(rule.getSource().size()-1);
			if(FabricIntegrationUtil.isOverriden(cba)){
             // handleProcessImplementationArtifactsMap(rule, mapMgr, cba);
			//The package is handled in the processflowrule processScdlArtifacts(...)
              handleProcessImplementationArtifactsMap(rule, mapMgr, action);
			}
		}
		
		if(rule.getSource().size() == 2){
			handleProcessImplementationArtifactsMap(rule, mapMgr, action);
		}
		
		if(rule.getTarget().size() >= 2){
			Object target2 = rule.getTarget().get(1);
			if(target2 instanceof DocumentRoot){
				component2 = (DocumentRoot)target2;
			}else if(target2 instanceof SACLRoot ||
					target2 instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup){
				handleSACLBRGRegistry(mapMgr, action, target2);
			}
//			if(rule.getTarget().size() == 3 && rule.getTarget().get(2) instanceof SACLRoot){
			if(rule.getTarget().size() == 3 && (rule.getTarget().get(2) instanceof SACLRoot ||
					rule.getTarget().get(2) instanceof  com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup)){
				Object saclRootOrBRG = rule.getTarget().get(2);
				handleSACLBRGRegistry(mapMgr, action, saclRootOrBRG);
			}
		}	
		if(BpelOptions.isBestPracticePatternEnabed(BpelOptionsUtil.getExportSession())&& 
				//rule.getTarget().size()== 3){
				rule.getTarget().size()>= 3){
			//Object target3 = rule.getTarget().get(2);
			Object target3 = rule.getTarget().get((rule.getTarget().size())-1);
			if(target3 instanceof SACLRoot ||
					target3 instanceof  com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup){
				handleSACLBRGRegistry(mapMgr, action, target3);
			}
		}
		if(BomUtils.isTask(action) && action.getInStructuredNode() != null){
			handleTaskProcessImplRegistry(rule, mapMgr, action);
		}
		if (action instanceof StructuredActivityNode) {
			if (((StructuredActivityNode)action).getActivity()!=null) {
				mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component);	
				if(component.getComponent() != null){
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component.getComponent());
				}else if(component.getImport() != null){
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component.getImport());
				}
				mapComponentInterfaces(action, component, mapMgr);
				mapComponentImplementation(action, component, mapMgr);
				mapImportBinding(action, component, mapMgr);
				if(component2 != null){
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2);	
					if(component2.getComponent() != null){
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2.getComponent());		
					}else if(component2.getImport() != null){
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2.getImport());	
					}
					mapComponentInterfaces(action, component2, mapMgr);
					mapComponentImplementation(action, component2, mapMgr);
					mapImportBinding(action, component2, mapMgr);
				}
				return;
			}
		}
		mapMgr.addMapDefinition(action,component);
		if(component.getComponent() != null){
			mapMgr.addMapDefinition(action,component.getComponent());			
		}else if(component.getImport() != null){
			mapMgr.addMapDefinition(action,component.getImport());
		}
		mapComponentInterfaces(action, component, mapMgr);
		mapComponentImplementation(action, component, mapMgr);
		mapImportBinding(action, component, mapMgr);
		if(component2 != null){
			if (action instanceof StructuredActivityNode) {
				if (((StructuredActivityNode)action).getActivity()!=null) {
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2);	
					if(component2.getComponent() != null){
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2.getComponent());	
						if(component2.getComponent().getImplementation() != null){
							mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2.getComponent().getImplementation());
						}
					}else if(component2.getImport()!= null){
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component2.getImport());	
					}
					mapComponentInterfaces(action, component2, mapMgr);
					mapComponentImplementation(action, component2, mapMgr);
					mapImportBinding(action, component2, mapMgr);
					return;
				}
			}
			mapMgr.addMapDefinition(action,component2);
			if(component2.getComponent() != null){
				mapMgr.addMapDefinition(action,component2.getComponent());				
			}else if(component2.getImport() != null){
				mapMgr.addMapDefinition(action,component2.getImport());
			}
			mapComponentInterfaces(action, component2, mapMgr);
			mapComponentImplementation(action, component2, mapMgr);
			mapImportBinding(action, component2, mapMgr);
		}
	}
    
	/**
	 * Find the targets which are Process or Definition created as the default process implementation artifacts
	 * @param rule
	 * @param mapMgr
	 * @param action
	 */
	private void handleProcessImplementationArtifactsMap(TransformationRule rule, MappingManager mapMgr,Action action) {
		if(rule.getTarget() != null && !rule.getTarget().isEmpty()){
			for(Object eachTarget:rule.getTarget()){
				if(eachTarget instanceof com.ibm.wbit.bpel.Process){
					com.ibm.wbit.bpel.Process procTarget = (com.ibm.wbit.bpel.Process)eachTarget;
					CrossProjectReferenceUtil.registerTranformTargetWithProjectName(action, procTarget);
				}else if(eachTarget instanceof Definition){
					Definition artifactsDef = (Definition)eachTarget;
					CrossProjectReferenceUtil.registerTranformTargetWithProjectName(action, artifactsDef);
				}
			}
		}
	}
    
	/**
	 * Handle task imple type as process side artifacts 
	 * @param rule
	 * @param mapMgr
	 * @param action
	 */
	private void handleTaskProcessImplRegistry(TransformationRule rule, MappingManager mapMgr, Action action) {
		AttributeValueProvider avp= AttributeValueProviderFactory.createAttributeValueProvider(AttributeNameConstants.COMPONENT_NAME);
		Object implementation = avp.getAttributeValueForType(action,AttributeNameConstants.IMPLEMENTATION);		
		if(implementation instanceof com.ibm.btools.te.attributes.model.definition.implementation.wps.Process){
			Object defTarget = null;
			Object processTarget = null;
			if(rule.getTarget().size()== 4){
				defTarget = rule.getTarget().get(2);
				if(defTarget instanceof Definition){
					handleTaskArtifactsRegistry(mapMgr, action, defTarget);
				}
				processTarget = rule.getTarget().get(3);
				if(processTarget instanceof com.ibm.wbit.bpel.Process){
					handleTaskProcessRegistry(mapMgr, action, processTarget);
				}
			}else if(rule.getTarget().size()== 3){
				defTarget = rule.getTarget().get(1);
				if(defTarget instanceof Definition){
					handleTaskArtifactsRegistry(mapMgr, action, defTarget);
				}
				processTarget = rule.getTarget().get(2);
				if(processTarget instanceof com.ibm.wbit.bpel.Process){
					handleTaskProcessRegistry(mapMgr, action, processTarget);
				}
			}
		}
	}

	/**
	 * Local task implementation as Process
	 * @param mapMgr
	 * @param action
	 * @param object
	 */
	private void handleTaskProcessRegistry(MappingManager mapMgr, Action action, Object object) {
		if(BomUtils.isTask(action)){
			if(action.getInStructuredNode()!= null){
				mapMgr.addMapDefinition(action,(EObject)object);
			}
		}		
	}

	/**
	 * Local task implementation as Process
	 * @param mapMgr
	 * @param action
	 * @param object
	 */
	private void handleTaskArtifactsRegistry(MappingManager mapMgr, Action action, Object object) {
		if(BomUtils.isTask(action)){
			if(action.getInStructuredNode()!= null){
				mapMgr.addMapDefinition(action,(EObject)object);
			}
		}		

	}
	
	public void mapComponentImplementation(Action action,DocumentRoot docRoot, MappingManager mapMgr){
		if(docRoot.getComponent()!= null && docRoot.getComponent().getImplementation()!= null){
			Implementation target = docRoot.getComponent().getImplementation();
			if (action instanceof StructuredActivityNode){
				if (((StructuredActivityNode)action).getActivity()!=null) {
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),target);
				}else{
					mapMgr.addMapDefinition(action,target);
				}				
			}else{
				mapMgr.addMapDefinition(action,target);
			}
		}
	}
	
	public void mapImportBinding(Action action,DocumentRoot docRoot, MappingManager mapMgr){
		if(docRoot.getImport()!= null && docRoot.getImport().getBinding()!= null){
			ImportBinding target = docRoot.getImport().getBinding();
			if (action instanceof StructuredActivityNode){
				if (((StructuredActivityNode)action).getActivity()!=null) {
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),target);
				}else{
					mapMgr.addMapDefinition(action,target);
				}				
			}else{
				mapMgr.addMapDefinition(action,target);
			}
		}
	}

	/**
	 * Add mapping for SACLRoot and BusinessRuleGroup, including the default BusinessRuleGroupTable.
	 * @param mapMgr
	 * @param action
	 * @param target
	 */
	private void handleSACLBRGRegistry(MappingManager mapMgr, Action action, Object target) {
		if(action instanceof StructuredActivityNode){
			Activity activity = null;
			if(BomUtils.isTask(action) || BomUtils.isService(action)) {	//GJD 20080117 d167594
				if(action.getInStructuredNode()!= null){
					mapMgr.addMapDefinition(action,(EObject)target);
					if(target instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup){
						com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup rg = (com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup)target;
						BusinessRuleGroupTable rgt = rg.getBusinessRuleGroupTable();
						mapMgr.addMapDefinition(action,rgt);
					}
				} else if(BomUtils.isReusableTask(action) || BomUtils.isReusableService(action)){ //GJD 20080117 d167594
						mapMgr.addMapDefinition(action,(EObject)target);
						if(target instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup){
							com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup rg = (com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup)target;
							BusinessRuleGroupTable rgt = rg.getBusinessRuleGroupTable();
							mapMgr.addMapDefinition(action,rgt);
						}
				}else{
					if(target instanceof SACLRoot){
						mapMgr.addMapDefinition(action,(SACLRoot)target);
					}
				}
			}else{
				activity = ((StructuredActivityNode)action).getActivity();
			}
			if(activity != null){
				mapMgr.addMapDefinition(activity,(EObject)target);
				//Handle business service implementation override
				if(BomUtils.isServiceOperation(action)){
					if(target instanceof com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup){
						com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup rg = (com.ibm.wbiservers.businessrule.model.brg.BusinessRuleGroup)target;
						BusinessRuleGroupTable rgt = rg.getBusinessRuleGroupTable();
						mapMgr.addMapDefinition(action,rgt);
					}
				}
			}
		}else{
			mapMgr.addMapDefinition(action,(EObject)target);
		}
	}	

	/**
	 * This method is to add mapping for the export/import component
	 * Source: Action action
	 * Traget: Interface the export points to
	 * @param docRoot
	 * @param mapMgr
	 */
	public void mapComponentInterfaces(Action action,DocumentRoot docRoot, MappingManager mapMgr) {
		//AZ 20070318 create mapping for interfaces
		List interfacesList = null;
		if(docRoot.getImport() != null){
			interfacesList = docRoot.getImport().getInterfaceSet().getInterfaces();

		}else if(docRoot.getComponent() != null){
			interfacesList = docRoot.getComponent().getInterfaceSet().getInterfaces();
		}
		if(interfacesList != null && !interfacesList.isEmpty()){
			Iterator it = interfacesList.iterator();
			while(it.hasNext()){
				EObject target = (EObject)it.next();
				if (action instanceof StructuredActivityNode){
					if (((StructuredActivityNode)action).getActivity()!=null) {
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),target);
					}else{
						mapMgr.addMapDefinition(action,target);
					}
					continue;
				}
				mapMgr.addMapDefinition(action,target);
			}
		}			
	}		

}
