package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.models.InformationModel;
import com.ibm.btools.bom.model.organizationstructures.OrganizationUnitType;
import com.ibm.btools.bom.model.resources.BulkResourceType;
import com.ibm.btools.bom.model.resources.IndividualResourceType;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.ExportOperationUtil;
import com.ibm.btools.te.ilm.heuristics.helper.BomUtils;
import com.ibm.btools.te.ilm.heuristics.xsd.util.OriginalXsdUtil;
import com.ibm.btools.bom.model.artifacts.Class;
import com.ibm.btools.bom.model.artifacts.Classifier;
import com.ibm.btools.bom.model.externalmodels.ExternalDocument;
import com.ibm.btools.bom.model.externalmodels.ExternalSchema;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Type
 * TARGET: XSD Schema
 */
public class DataDefinitionRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * This method will be executed only when the source type is not PrimitiveType
	 */
	public void handleRule(TransformationRule rule) {
		if(!rule.getSource().isEmpty() && !(rule.getSource().get(0) instanceof com.ibm.btools.bom.model.artifacts.PrimitiveType)){
			//AZ 20070517 add mapping between the catalog and XSD schema, the catalogue is the immediate
			//owningpackge of the business item
			if(rule.getSource().get(0)  instanceof com.ibm.btools.bom.model.artifacts.Class){
				Class clazz = (Class)rule.getSource().get(0);
				if(clazz.getOwningPackage() instanceof InformationModel){
					InformationModel  catalog = (InformationModel)clazz.getOwningPackage();
					getMappingManager(rule).addMapDefinition(catalog,(EObject)rule.getTarget().get(0));					
				}else if(OriginalXsdUtil.isExternalType(clazz)){
					//map to the external schema, which is the container of the Class 
					if(clazz.getOwningPackage() instanceof ExternalSchema){
						ExportOperationUtil.addToExtDocumentWithArtCatList((ExternalDocument)clazz.getOwningPackage());
						getMappingManager(rule).addMapDefinition(clazz.getOwningPackage(), (EObject)rule.getTarget().get(0));
						return;
					}
				}
			}else if(rule.getSource().get(0) instanceof BulkResourceType ||rule.getSource().get(0) instanceof OrganizationUnitType ||
					rule.getSource().get(0) instanceof IndividualResourceType){
				EObject sourceType = (EObject)rule.getSource().get(0);
				if(sourceType instanceof Classifier){
					Classifier clazfier = (Classifier)sourceType;					
					String predefinedProjectName = null;
					if(clazfier.getIsAbstract()){
						predefinedProjectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(clazfier);					
					}else if(clazfier.getSuperClassifier()!= null && !clazfier.getSuperClassifier().isEmpty()){
						Classifier superClaz = (Classifier)clazfier.getSuperClassifier().get(0);
						if(superClaz.getIsAbstract()){							
							predefinedProjectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(superClaz);
						}
					}
					if(predefinedProjectName != null){						
						CrossProjectReferenceUtil.registerSharedLibProjectName(predefinedProjectName);
						CrossProjectReferenceUtil.registerTranformTargetWithProjectName((EObject)rule.getSource().get(0), (EObject)rule.getTarget().get(0));
					}
				}				
			}
			super.handleRule(rule);
		}
	}
}
