package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.processes.actions.Decision;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.OutputPinSet;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.DecisionRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.BomHelper;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpel.Case;
import com.ibm.wbit.bpel.Empty;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Switch;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;

/**
 *This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Decision
 * TARGET: BPEL Switch
 */
 
public class DecisionRuleHandler extends ConnectableRuleHandler {
	
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * Creates the ObjectDefinions for the BOM Decision and the BPEL Switch. A Bpel empty
	 * may be generated for this switch in special cases, then the empty will also be added as target. 
	 * If the Empty will be reduced during the Optimization process it will be eliminated from the ObjectDefinitions List
	 * due to no Container.
	 * @param rule the DecisionRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		
	    InputPinSet inputPinSet = (InputPinSet)rule.getSource().get(0);
	    Action san = (Action)inputPinSet.getAction();		
	    Decision decision = (Decision)inputPinSet.getAction();    
	    
		MappingManager mapMgr = getMappingManager(rule);

		Iterator it = rule.getTarget().iterator();
		while(it.hasNext()){
			EObject obj = (EObject)it.next();
			if(obj instanceof Switch || obj instanceof Empty){
				registerMergePoint(rule, inputPinSet, san);
				mapMgr.addMapDefinition(san, obj);
				if(obj instanceof Switch){
					Switch thisSwitch = (Switch)obj;
					List extElements = thisSwitch.getEExtensibilityElements();
					if(extElements != null && !extElements.isEmpty()){
						Iterator it2 = extElements.iterator();
						while(it2.hasNext()){
							EObject extElement = (EObject)it2.next();
							if(extElement instanceof Documentation || extElement instanceof Description){
								mapMgr.addMapDefinition(inputPinSet, extElement);
							}
						}
					}					
					
				}
			}
		}		  
		
	    for (it = decision.getOutputPinSet().iterator(); it.hasNext(); ){
	        OutputPinSet ops = (OutputPinSet)it.next();
	        Case decisionCase = ((DecisionRuleImpl)rule).getCaseForPinSet(ops);
	        mapMgr.addMapDefinition(ops, decisionCase);	       
	    }
		return;
	}
}
