package com.ibm.btools.te.ilm.rulehandler.mapping.wps;
/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.framework.util.TransformationUtil;
import com.ibm.btools.te.ilm.heuristics.scdl.ComponentRule;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wsspi.sca.scdl.Component;
import com.ibm.wsspi.sca.scdl.DocumentRoot;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Action
 * TARGET: Export DocumentRoot
 * The mapping between the BOM Action and interfaces the export points to 
 * will also be created
 */
public class ExportRuleHandler extends ComponentRuleHandler {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	/**
	 * 
	 * @param rule The ExportRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		//Component target = (Component)((DocumentRoot)rule.getSource().get(0)).getComponent();	
		DocumentRoot component = (DocumentRoot)rule.getTarget().get(0);
		TransformationRule componentRule  = TransformationUtil.getRuleForTarget(rule.getRoot(), ComponentRule.class, (DocumentRoot)rule.getSource().get(0));
		if(componentRule != null && component != null && component.getExport()!= null){
			Action action = (Action)componentRule.getSource().get(0);			
			MappingManager mapMgr = getMappingManager(rule);
			if (action instanceof StructuredActivityNode) {
				if (((StructuredActivityNode)action).getActivity()!=null) {
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component);	
					mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),component.getExport());	
					mapExportInterfaces(action,component, mapMgr);
					return;
				}
			}
			mapMgr.addMapDefinition(action,component);
			mapMgr.addMapDefinition(action,component.getExport());
			mapExportInterfaces(action,component, mapMgr);
		}	    
	}

	/**
	 * This method is to add mapping for the export component
	 * Source: Action action
	 * Traget: Interface the export points to
	 * @param component
	 * @param mapMgr
	 */
	private void mapExportInterfaces(Action action,DocumentRoot component, MappingManager mapMgr) {
		//AZ 20070318 create mapping for interfaces
		if(component.getExport() != null){
			List interfacesList = component.getExport().getInterfaceSet().getInterfaces();
			if(interfacesList != null && !interfacesList.isEmpty()){
				Iterator it = interfacesList.iterator();
				while(it.hasNext()){
					EObject target = (EObject)it.next();
					if (action instanceof StructuredActivityNode){
						if (((StructuredActivityNode)action).getActivity()!=null) {
							mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),target);
						}else{
							mapMgr.addMapDefinition(action,target);
						}
						continue;
					}
					mapMgr.addMapDefinition(action,target);
				}
			}			
		}
	}		

}
