package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsd.XSDComplexTypeContent;
import org.eclipse.xsd.XSDComplexTypeDefinition;
import org.eclipse.xsd.XSDElementDeclaration;
import org.eclipse.xsd.XSDModelGroup;
import org.eclipse.xsd.XSDParticle;
import org.eclipse.xsd.XSDParticleContent;
import org.eclipse.xsd.XSDSchema;

import com.ibm.btools.bom.model.artifacts.NamedElement;
import com.ibm.btools.bom.model.artifacts.PrimitiveType;
import com.ibm.btools.bom.model.artifacts.Type;
import com.ibm.btools.bom.model.artifacts.TypedElement;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.CrossProjectReferenceUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpm.map.base.BaseFactory;
import com.ibm.wbit.bpm.map.base.Descriptor;
import com.ibm.wbit.bpm.map.base.ObjectValue;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;
import com.ibm.wbit.bpm.trace.model.BPMConstants;

/**
 * This handler is responsbile for creating the ObjectDefinition mapping between
 * the source and target: 
 * SOURCE: BOM Type
 * TARGET: XSD Schema
 */
public class ListRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 

	/**
	 * This method will be executed only when the source type is not PrimitiveType
	 */
	public void handleRule(TransformationRule rule) {
		if(!rule.getSource().isEmpty() && rule.getSource().get(0) instanceof TypedElement){
			TypedElement typedElement = (TypedElement)rule.getSource().get(0);
			Type type = typedElement.getType();
			if(type != null){	
				if(rule.getParentRule() != null){
					if(rule.getParentRule().getTarget() != null && !rule.getParentRule().getTarget().isEmpty()){
						if(rule.getParentRule().getTarget().get(0) instanceof XSDSchema){
							XSDSchema target = null;
							if(type instanceof PrimitiveType){
								//we need to create lib for the predefined element project
								String projectName = CrossProjectReferenceUtil.getProjectNameForBOMElement(type);
								CrossProjectReferenceUtil.registerSharedLibProjectName(projectName);
								//CrossProjectReferenceUtil.registerTranformTargetWithProjectName(type, target);
								target = (XSDSchema)rule.getParentRule().getTarget().get(0);
								CrossProjectReferenceUtil.registerTranformTargetWithProjectName(type, target);								
								if(rule.getTarget().get(0) instanceof XSDComplexTypeDefinition){
									XSDComplexTypeDefinition complextTypeDef = (XSDComplexTypeDefinition)rule.getTarget().get(0);
									getMappingManager(rule).addMapDefinition(typedElement,complextTypeDef);
									addMultiplicityDescriptor(getMappingManager(rule),typedElement,complextTypeDef);
									mapComplexTypeDefinitionContent(getMappingManager(rule),typedElement,complextTypeDef);
								}
								//defect 166071
								ObjectDefinition typeElementOD = getMappingManager(rule).getObjectDefinition(typedElement);
								if(typeElementOD != null && projectName != null){
									CrossProjectReferenceUtil.registerBOMSrcWithSharedProjectName(typeElementOD, projectName);
								}
							}else if(type instanceof com.ibm.btools.bom.model.artifacts.Class){
								if(rule.getParentRule().getTarget().size()== 2 && rule.getParentRule().getTarget().get(1) instanceof XSDSchema ){
									target = (XSDSchema)rule.getParentRule().getTarget().get(1);
									getMappingManager(rule).addMapDefinition(type,target);									
									if(rule.getTarget().get(0) instanceof XSDComplexTypeDefinition){
										XSDComplexTypeDefinition complextTypeDef = (XSDComplexTypeDefinition)rule.getTarget().get(0);
										getMappingManager(rule).addMapDefinition(typedElement,complextTypeDef);
										addMultiplicityDescriptor(getMappingManager(rule),typedElement,complextTypeDef);
										mapComplexTypeDefinitionContent(getMappingManager(rule),typedElement,complextTypeDef);
									}         							
								}else if(rule.getTarget().get(0)instanceof XSDComplexTypeDefinition){
									EObject complexTypeDefList = (EObject)rule.getTarget().get(0);
									getMappingManager(rule).addMapDefinition(typedElement,complexTypeDefList);	
									addMultiplicityDescriptor(getMappingManager(rule),typedElement,complexTypeDefList);
									XSDComplexTypeDefinition complextTypeDef = (XSDComplexTypeDefinition)rule.getTarget().get(0);
									mapComplexTypeDefinitionContent(getMappingManager(rule),typedElement,complextTypeDef);
								}
								
							}							
						}
					}
				}				
			}
		}
	}

	private void addMultiplicityDescriptor(MappingManager mapMgr, EObject src,EObject target){
		if(mapMgr == null || src == null || target == null){
			return;
		}
		ObjectDefinition srcOD = mapMgr.getObjectDefinition(src);
		ObjectDefinition targetOD = mapMgr.getObjectDefinition(target);
		if(srcOD != null && targetOD != null){				
			if(!mapMgr.existsDescriptor(srcOD, "isMultiplicityMapping")){
				Descriptor srcDesc = mapMgr.createDescriptor("isMultiplicityMapping", "yes");
				srcOD.getDescriptor().add(srcDesc);
			}
			String originalProjectName = CrossProjectReferenceUtil.getProjectNameForBOMElement((NamedElement)src);
			if(!mapMgr.existsDescriptor(srcOD, BPMConstants.DESCRIPTOR_ORIGINAL_PROJECT_NAME)){
				Descriptor srcDesc = mapMgr.createDescriptor(BPMConstants.DESCRIPTOR_ORIGINAL_PROJECT_NAME, originalProjectName);
				srcOD.getDescriptor().add(srcDesc);
			}
			if(!mapMgr.existsDescriptor(targetOD, BPMConstants.DESCRIPTOR_ORIGINAL_PROJECT_NAME)){
				Descriptor targetDesc = mapMgr.createDescriptor( BPMConstants.DESCRIPTOR_ORIGINAL_PROJECT_NAME, originalProjectName);
				targetOD.getDescriptor().add(targetDesc);
			}
			if(!mapMgr.existsDescriptor(targetOD, "isMultiplicityMapping")){
				Descriptor targetDesc = mapMgr.createDescriptor("isMultiplicityMapping", "yes");
				targetOD.getDescriptor().add(targetDesc);
			}
		}	
		String projectName = CrossProjectReferenceUtil.getProjectNameForTargetObject(target);	
		if(projectName != null && srcOD != null ){
			CrossProjectReferenceUtil.registerBOMSrcWithSharedProjectName(srcOD, projectName);
		}
	}
    
	/**
	 * Map the content of the artificial complex type
	 * @param mapMgr
	 * @param src
	 * @param complexTypeDef
	 */
	private void mapComplexTypeDefinitionContent(MappingManager mapMgr,TypedElement src,XSDComplexTypeDefinition complexTypeDef){
		if(mapMgr == null ||complexTypeDef == null){
			return;
		}
		mapMgr.addMapDefinition(src,complexTypeDef);
		ObjectDefinition xTypeOD = mapMgr.getObjectDefinition(complexTypeDef);
		if(xTypeOD != null && !mapMgr.existsDescriptor(xTypeOD, BPMConstants.DESCRIPTOR_OBJECT_TYPE)){
			if(src.getType() instanceof PrimitiveType){
				Descriptor srcDesc = mapMgr.createDescriptor(BPMConstants.DESCRIPTOR_OBJECT_TYPE, src.getType().getName());
				xTypeOD.getDescriptor().add(srcDesc);
			}else{
				Descriptor srcDesc = mapMgr.createDescriptor("typeBomUid", src.getType().getUid());
				xTypeOD.getDescriptor().add(srcDesc);
			}
		}
		XSDComplexTypeContent content = complexTypeDef.getContent();
		if(content instanceof XSDParticle){
			XSDParticle particle = (XSDParticle)content;
			mapMgr.addMapDefinition(src,particle);
			XSDParticleContent particleContent = particle.getContent();
			if(particleContent instanceof XSDModelGroup){
				XSDModelGroup xsdModelGroup = (XSDModelGroup)particleContent;
				//This is the target to map										
				mapMgr.addMapDefinition(src,xsdModelGroup);			
				if(xsdModelGroup.getContents().get(0) instanceof XSDParticle){
					XSDParticle innerParticle = (XSDParticle)xsdModelGroup.getContents().get(0);
					mapMgr.addMapDefinition(src,innerParticle);	
					Descriptor maxDesc = mapMgr.createDescriptor(BPMConstants.MAX_OCCURS, String.valueOf(innerParticle.getMaxOccurs()));
					Descriptor minDesc = mapMgr.createDescriptor(BPMConstants.MIN_OCCURS, String.valueOf(innerParticle.getMinOccurs()));
					ObjectDefinition innerParticleOD = mapMgr.getObjectDefinition(innerParticle);
					if(innerParticleOD != null){
						innerParticleOD.getDescriptor().add(maxDesc);
						innerParticleOD.getDescriptor().add(minDesc);
					}
					if(innerParticle.getContent() instanceof XSDElementDeclaration){
						XSDElementDeclaration ed = (XSDElementDeclaration)innerParticle.getContent() ;						
						mapMgr.addMapDefinition(src,ed);					
					}					
				}
			}
		}
	}
}
