package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.actions.CallBehaviorAction;
import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.CallBehaviorActionRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.PartnerRuleImpl;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wsspi.sca.scdl.Reference;
import com.ibm.wsspi.sca.scdl.Wire;

/**
 *This handler is responsbile for creating the ObjectDefinition mapping between 
 * BOM <code>Activity</code> and WSDL <code>Part</code>
 */
public class PartnerRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**
	 * @param rule the PartnerRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {		
		/**
		 * Handles the PartnerRule
		 * Source: BOM Action
		 * Target: Reference
		 */
		Action action = (Action)rule.getSource().get(0);
		if (rule.eContainer() instanceof CallBehaviorActionRule) {
			//We are invoking a global action so we should map the references and wires to the CBA, and not the actual Activity
			for (EObject source : (List<EObject>)rule.getSource()) {
				if (source instanceof CallBehaviorAction) {
					action = (Action) source;
				}
			}
		}
		PartnerRuleImpl thisRule = (PartnerRuleImpl)rule;
		Reference target = thisRule.getSingleReference();		
		if(target == null){
			return;
		}	
		if(target.getWires() != null && !target.getWires().isEmpty()){
			Iterator wireIt = target.getWires().iterator();
			while(wireIt.hasNext()){
				Wire eachWire = (Wire)wireIt.next();
				MappingManager mapMgr = getMappingManager(rule);
				if (action instanceof StructuredActivityNode) {
					if (((StructuredActivityNode)action).getActivity()!=null) {
						mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),eachWire);	
						continue;
				   }
				}
				mapMgr.addMapDefinition(action,eachWire);	    
			}
		}
		MappingManager mapMgr = getMappingManager(rule);
		if (action instanceof StructuredActivityNode) {
			if (((StructuredActivityNode)action).getActivity()!=null) {
				mapMgr.addMapDefinition(((StructuredActivityNode)action).getActivity(),target);		
				return;
			}
		}
		mapMgr.addMapDefinition(action,target);	    
	}		
}
