package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.btools.bom.model.processes.activities.Activity;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.wbit.bpelpp.Description;
import com.ibm.wbit.bpelpp.Documentation;
import com.ibm.wbit.bpelpp.impl.AutonomyImpl;
import com.ibm.wbit.bpelpp.impl.ExecutionModeImpl;
import com.ibm.wbit.bpm.map.objectdefinition.ObjectDefinition;

/**
 * This handler is responsbile for creating the <code>ObjectDefinition</code> mapping 
 * between BOM <code> StructuredActivityNode </code> and BPEL <code>Process</code>
 */
public class ProcessDefinitionRuleHandler extends ConnectableRuleHandler {

	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 
	
	/**	 
	 * @param rule The ProcessDefinitionRule that has just executed.
	 */
	public void handleRule(TransformationRule rule) {
		/**
		 * Handles the ProcessDefinitionRule
		 * Source: BOM Activity
		 * Target: BPEL Process
		 */
		
		Activity activity = (Activity)rule.getSource().get(0);
		StructuredActivityNode san = activity.getImplementation();
		com.ibm.wbit.bpel.Process bpelProcess = (com.ibm.wbit.bpel.Process)rule.getTarget().get(0);
		
		MappingManager mapMgr = getMappingManager(rule);
		mapMgr.addMapDefinition(activity,bpelProcess);
		mapMgr.addInternalLinksKey(bpelProcess);
		
		ObjectDefinition processOD = mapMgr.getObjectDefinition(bpelProcess);
		if(processOD != null){
			List extElements = bpelProcess.getEExtensibilityElements();
			if(extElements != null && !extElements.isEmpty()){
				Iterator it = extElements.iterator();
				while(it.hasNext()){
					EObject extElement = (EObject)it.next();
					if(extElement instanceof Documentation || extElement instanceof Description){
						mapMgr.addMapDefinition(activity, extElement);
					}
				}
			}
		}
	}			
}
