package com.ibm.btools.te.ilm.rulehandler.mapping.wps;

/*
* Licensed Materials - Property of IBM
* 5724-I74, 5724-I75
*  Copyright IBM Corporation 2003, 2009. All rights reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.wst.wsdl.ExtensibleElement;

import com.ibm.btools.bom.model.processes.activities.Action;
import com.ibm.btools.bom.model.processes.activities.InputPinSet;
import com.ibm.btools.bom.model.processes.activities.PinSet;
import com.ibm.btools.bom.model.processes.activities.StructuredActivityNode;
import com.ibm.btools.te.framework.TransformationRule;
import com.ibm.btools.te.ilm.heuristics.abstractbpel.impl.SANReusableProcessRuleImpl;
import com.ibm.btools.te.ilm.heuristics.helper.AbstractbpelUtil;
import com.ibm.btools.te.ilm.heuristics.helper.CBPELMappingUtil;
import com.ibm.btools.te.ilm.heuristics.helper.ProcessUtil;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingManager;
import com.ibm.btools.te.ilm.rulehandler.mapping.MappingUtil;
import com.ibm.wbit.bpel.Flow;
import com.ibm.wbit.bpel.Pick;
import com.ibm.wbit.bpelpp.Task;
import com.ibm.wbit.bpelpp.impl.TaskImpl;
import com.ibm.wbit.tel.DocumentRoot;
import com.ibm.wbit.tel.ParameterType;
import com.ibm.wbit.tel.TPotentialInstanceCreator;
import com.ibm.wbit.tel.TPotentialOwner;
import com.ibm.wbit.tel.TPotentialStarter;
import com.ibm.wbit.tel.TTask;
import com.ibm.wbit.tel.impl.TTaskImpl;

/**
 *  This handler is responsbile for creating the ObjectDenifition mapping between 
 *  BOM <code> StructuredActivityNode </code> and BPEL <code>Flow </code>
 */
public class SANReusableProcessRuleHandler extends ConnectableRuleHandler {
	public static final String copyright = "Licensed Material - Property of IBM " + " 5724-I74, 5724-I75 " + "(C) Copyright IBM Corporation 2003, 2009. All Rights Reserved. " + "U.S. Government Users Restricted Rights - Use, duplication or disclosure " + "restricted by GSA ADP Schedule Contract with IBM Corp.".intern(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ 	
	
	private void mapInitiatingHumanTask(EObject src, TTask ttask, MappingManager mapMgr)
	{
		if (ttask.eContainer() instanceof DocumentRoot) {
			mapMgr.addMapDefinition(src, ttask.eContainer());
			mapMgr.addMapDefinition(src, ttask);
			
			//Initiating HT will have no calenders, no escalation chains, with the 
			//PotentialInstanceCreator and PotentialStarter being everybody 
			
			mapMgr.addMapDefinition(src, ttask.getStaffSettings());
			TPotentialInstanceCreator potentialInstanceCreator = 
				ttask.getStaffSettings().getPotentialInstanceCreator();
			
			//Map PotentialInstanceCreator
			if (potentialInstanceCreator != null) {
				mapMgr.addMapDefinition(src, potentialInstanceCreator);
				if (potentialInstanceCreator.getVerb() != null) {
					mapMgr.addMapDefinition(src, potentialInstanceCreator.getVerb());
					if (potentialInstanceCreator.getVerb().getParameter() != null) {
						for (ParameterType parameter : (List<ParameterType>)potentialInstanceCreator.getVerb().getParameter()) {
							mapMgr.addMapDefinition(src, parameter);
						}
					}
				}
			}
			
			//Map PotentialStarter
			TPotentialStarter potentialStarter = ttask.getStaffSettings().getPotentialStarter();
			if (potentialStarter != null) {
				mapMgr.addMapDefinition(src, potentialStarter);
				if (potentialStarter.getVerb() != null) {
					mapMgr.addMapDefinition(src, potentialStarter.getVerb());
					if (potentialStarter.getVerb().getParameter() != null) {
						for (ParameterType parameter : (List<ParameterType>)potentialStarter.getVerb().getParameter()) {
							mapMgr.addMapDefinition(src, parameter);
						}
					}
				}
			}
		}
	}
	/**
	 * @param rule the SANReusableProcessRule that has just executed
	 */
	public void handleRule(TransformationRule rule) {
		/**
		 * Handles the SANReusableProcessRule
		 * Source: BOM StructuredActivityNode
		 * Target: BPEL Flow
		 * Pick register in the rule: fStartableActivities.put(ipsNoCP<List>, createPick(ipsNoCP,true,initiateCorrelation));		
		 */		
		StructuredActivityNode san = (StructuredActivityNode)rule.getSource().get(0);
		SANReusableProcessRuleImpl sourceRule = (SANReusableProcessRuleImpl)rule;
		Flow bpelFlow = (Flow)rule.getTarget().get(0);
		EObject bpelProcess = ProcessUtil.getProcessDefinition(rule.getRoot().getContext());

		MappingManager mapMgr = getMappingManager(rule);
		
		CBPELMappingUtil.registerFlowMapping(san.getActivity(), bpelFlow, rule.getRoot().getContext());
		mapMgr.addMapDefinition(san.getActivity(),bpelFlow);

		Map startableActivities = sourceRule.getStartableActivities();
		Object key = null;
		Object value = null;
		for (Iterator it = startableActivities.keySet().iterator(); it.hasNext(); ){
			key = it.next();
			value = startableActivities.get(key);

			if (key instanceof List){
				if(value instanceof Pick){
					if(key instanceof List){
						List keyList = (List)key;
						if(keyList.get(0) instanceof InputPinSet){
							InputPinSet ips = (InputPinSet)keyList.get(0);
							Action action = ips.getAction();
							registerMergePoint(rule, ips, action);
						}
					}					
				}
				for (Iterator it2 = ((List)key).iterator(); it2.hasNext(); ){
					mapMgr.addMapDefinition((PinSet)it2.next(),(EObject)value);
				}				
			} else if (key instanceof PinSet){
				mapMgr.addMapDefinition((EObject)key,(EObject)value);
				Task task = (Task) MappingUtil.getExtensibilityElement((ExtensibleElement) value, TaskImpl.class);
				if(task != null){
					Object taskName = task.getName();
					TTask ttask = null;
					if (taskName instanceof TTask) {
						mapMgr.addMapDefinition((PinSet)key, task);
						ttask = (TTask) taskName;
						mapInitiatingHumanTask((EObject)key , ttask, mapMgr);
					}
				}
			} 			
		}	

		Map nonStartableActivities = sourceRule.getNonStartableActivities();
		for (Iterator it = nonStartableActivities.keySet().iterator(); it.hasNext(); ){
			key = it.next();
			value = nonStartableActivities.get(key);
			mapMgr.addMapDefinition((EObject)key,(EObject)value);
		}	

		Map responseActivities = sourceRule.getResponseActivities();
		for (Iterator it = responseActivities.keySet().iterator(); it.hasNext(); ){
			key = it.next();
			value = responseActivities.get(key);
			mapMgr.addMapDefinition((EObject)key,(EObject)value);

		}

		Map links = sourceRule.getLinks();
		for (Iterator it = links.keySet().iterator(); it.hasNext(); ){
			key = it.next();
			value = links.get(key);
			mapMgr.addInternalLinks((EObject)key,(EObject)value, bpelProcess );		
		}
	}	
}
